;    Copyright (C) 1997  Peter Vernon Lanspeary
;
;    This documentation is free; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This documentation is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this documentation; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



        page 58,132
arg_off = 6                       ; offset to first argument
code1   segment byte 'code'
        assume  cs:code1
public  sup250,supsim
;
;     SUBROUTINE SUP250( INDATA, OUTDATA, NSAMPLES, ODDEVEN )
;
;   Unpacks data from the DAS 250 A/D converter which has been packed by its
; data packing circuit.
;
; Arguments
;  logical *1 indata( nbytes )     ; packed data array                 INPUT
;  integer *2 outdata( nsamples ), ; unpacked data array               OUTPUT
;             nsamples,            ; number of data samples to unpack  INPUT
;             oddeven              ; odd/even flag for 1st sample   INPUT/OUTPUT
;
;  The data from the DAS 250A A/D has 12 significant bits . Pairs of samples
;  are packed into byte triples by a packing circuit .
;
;        Packed data
;    [ sample 1 , middle nibble ]       [ sample 1 , most sig. nibble ]
;    [ sample 2 , most sig. nibble ]    [ sample 1 , least sig. nibble ]
;    [ sample 2 , least sig. nibble ]   [ sample 2 , middle nibble ]
;
;       Unpacked data
; sample1 - 0000 [ most sig. nibble ] [ middle nibble ] [ least sig. nibble ]
; sample2 - "                                                               "
;
;               estimated execution time = 8.94 microseconds per sample
;
; Register use
;   ax - accumulator   - first byte of a sample
;   bx - base          - address of arguments ( ES: segment ) - mainly data
;   cx - count         - number of samples remaining
;   dx - data          - second byte of a sample
;
;   sp - stack pointer - readonly please
;   bp - base pointer  - base address for argument list
;   si - source index  - input array pointer
;   di - dest'n index  - output array pointer
;
;   cs - code segment  - don't touch
;   ds - data segment  - source array
;   ss - stack segment - don't touch
;   es - extra segment - destination array
;
sup250  proc    far
        push    bp
        mov     bp,sp
;
;               get arguments
;
        les     bx,[bp+8].arg_off
        mov     cx,es:[bx]      ; get number of samples
        test    cx,cx
        jz      end1            ; exit if no data
        les     bx,[bp+12].arg_off
        mov     ax,es:[bx]      ; get odd/even flag
        xor     es:[bx],cx      ; excl. or with no. of samples => new odd/even
        and     word ptr es:[bx],1  ; clear all but LSB from new flag
        and     ax,1            ; clear all but LSB from old flag
        lds     si,[bp].arg_off         ; get pointer to input array
        les     di,[bp+4].arg_off       ; get pointer to output array
        jnz     second1         ; start at second sample if odd
;
;                                                  most  mid  least
;               unpack 1st sample       sample 1    11    12   13
;               (7.875 micro sec)       sample 2    21    22   23
;
first1: mov     al,ds:[si]      ; get 1st byte     ?? ?? 12 11
        inc     si              ; incr I/P pointer
        mov     ah,al           ;                  12 11 12 11
        and     ax,0ff0h        ; 2 high nibbles   00 11 12 00
        mov     dl,ds:[si]      ; get second byte               ?? ?? 21 13
        and     dx,000fh        ; low nibble                    00 00 00 13
        or      ax,dx           ;                               00 11 12 13
        mov     es:[di],ax      ; store unpacked sample into O/P array
        inc     di              ; incr O/P pointer
        inc     di
        dec     cx              ; decr sample counter
        jz      end1            ; continue if not finished
;
;               unpack 2nd sample
;               (10.5 micro sec)
;
second1: mov    ah,ds:[si]      ; get 1st byte     21 ?? ?? ??
        inc     si              ; incr I/P pointer
        and     ax,0f000h       ;                  21 00 00 00
        mov     dl,ds:[si]      ;                               ?? ?? 23 22
        inc     si              ; incr I/P pointer
        mov     dh,dl           ;                               23 22 23 22
        and     dx,0ff0h        ;                               00 22 23 00
        or      ax,dx           ;                  21 22 23 00
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1            ;                  00 21 22 23
        mov     es:[di],ax      ; store unpacked sample into O/P array
        inc     di              ; incr O/P pointer
        inc     di
        loop    first1          ; continue if not finished
;
end1:   pop     bp
        ret
sup250  endp
;
;
;     SUBROUTINE SUPSIM( INDATA, OUTDATA, NSAMPLES, ODDEVEN )
;
;   Unpacks data from the simultaneous multichannel A/D .
;
; Arguments
;  logical *1 indata( nbytes )     ; packed data array                 INPUT
;  integer *2 outdata( nsamples ), ; unpacked data array               OUTPUT
;             nsamples,            ; number of data samples to unpack  INPUT
;             oddeven              ; odd/even flag for 1st sample   INPUT/OUTPUT
;
;  The data from the simultaneous multichannel A/D has 12 significant bits .
;  Pairs of samples are packed into byte triples by a built in packing
;  circuit .
;        Packed data
;    [ sample 1 , most sig. nibble ]    [ sample 1 , middle nibble ]
;    [ sample 1 , least sig. nibble ]   [ sample 2 , most sig. nibble ]
;    [ sample 2 , middle nibble ]       [ sample 2 , least sig. nibble ]
;
;       Unpacked data
; sample1 - 0000 [ most sig. nibble ] [ middle nibble ] [ least sig. nibble ]
; sample2 - "                                                               "
;
; Register use
;   ax - accumulator   - first byte of a sample
;   bx - base          - address of arguments ( ES: segment ) - mainly data
;   cx - count         - number of samples remaining
;   dx - data          -
;
;   sp - stack pointer - readonly please
;   bp - base pointer  - base address for argument list
;   si - source index  - input array pointer
;   di - dest'n index  - output array pointer
;
;   cs - code segment  - don't touch
;   ds - data segment  - source array
;   ss - stack segment - don't touch
;   es - extra segment - destination array
;
supsim  proc    far
        push    bp
        mov     bp,sp
;
;               get arguments
;
        les     bx,[bp+8].arg_off
        mov     cx,es:[bx]      ; get number of samples
        test    cx,cx
        jz      end2            ; exit if no data
        les     bx,[bp+12].arg_off
        mov     ax,es:[bx]      ; get odd/even flag
        xor     es:[bx],cx      ; excl. or with no. of samples => new odd/even
        and     word ptr es:[bx],1  ; clear all but LSB from new flag
        and     ax,1            ; clear all but LSB from old flag
        lds     si,[bp].arg_off         ; get pointer to input array
        les     di,[bp+4].arg_off       ; get pointer to output array
        jnz     second2         ; start at second sample if odd
;
;                                                  most  mid  least
;               unpack 1st sample       sample 1    11    12   13
;                                       sample 2    21    22   23
;
first2: mov     ah,ds:[si]      ; get 1st byte          11 12 ?? ??
        inc     si              ; incr I/P pointer
        mov     al,ds:[si]      ; get second byte       11 12 13 21
        shr     ax,1
        shr     ax,1
        shr     ax,1
        shr     ax,1            ; unpacked              00 11 12 13
        mov     es:[di],ax      ; store unpacked data into O/P array
        inc     di              ; incr O/P pointer
        inc     di
        dec     cx              ; decr sample counter
        jz      end2            ; continue if not finished
;
;       unpack 2nd sample
;
second2: mov    ah,ds:[si]      ; get first byte        13 21 00 00
        inc     si              ; incr I/P pointer
        mov     al,ds:[si]      ; get second byte       13 21 22 23
        inc     si              ;
        and     ax,0fffh        ; clear highest nibble  00 21 22 23
        mov     ds:[di],ax      ; store unpacked data in O/P array
        inc     di              ; incr O/P pointer
        inc     di
        loop    first2          ; continue if unfinished
;
end2:   pop     bp
        ret
supsim  endp
code1   ends
;
;
;       INTEGER *2 FUNCTION MXCRC( DATA, LENGTH )
;
;       MXCRC must be called as a FORTRAN function subprogram . MXCRC c
;  calculates the ( unsigned short ) cyclical redundancy checksum for the
;  array of LENGTH bytes , STRING .
;
; Arguments
;  logical *1 data(length)      # data array
;  integer *2 length            # number of bytes in data
;
;************************************************************
;       EQUIVALENT FORTRAN CODE :
;
;  integer *2 crc16(256)           # table of crc polynomial 0120001
;  mxcrc = 0
;  do i = 1, length
;    mxcrc = ieor( crc16( ieor( string(i) , mxcrc ) ) , mxcrc / 256 )
;************************************************************
;
; Register use
;   ax - accumulator   - al = data value
;   bx - base          - address of arguments ( ES: segment ) - mainly data
;   cx - count         - number of data bytes remaining
;   dx - data          - 16 bit CRC value
;
;   sp - stack pointer - readonly please
;   bp - base pointer  - base address for argument list
;   si - source index  - CRC16 array offset
;   di - dest'n index
;
;   cs - code segment  - don't touch
;   ds - data segment  - don't touch
;   ss - stack segment - don't touch
;   es - extra segment - argument segment
;
val_off =       6               ; offset to function value pointer
arg_off =       10              ; offset to the first argument pointer
;
;
code2   segment byte 'code'
        assume  cs:code2
        public  mxcrc
;
mxcrc   proc    far
        push    bp
        mov     bp,sp
;
;               get arguments ( or their pointers )
;
        les     bx,[bp+4].arg_off       ; get pointer to length of data array
        mov     cx,es:[bx]      ; put length of data array into counter
        les     bx,[bp].arg_off ; get pointer to data array
        xor     dx,dx           ; clear CRC
;
;               CRC calculation
;
$1:     mov     al,es:[bx]      ; get next data byte
        inc     bx              ;   & then point to the next item
        xor     ax,dx           ; exclusive or the data with the CRC
        xor     ah,ah           ; clear high byte
        sal     ax,1            ; double it to get data array offset
        mov     si,ax
        mov     ax,cs:crc16[si] ; get polynomial value from table
        mov     dl,dh           ; divide old CRC
        xor     dh,dh           ;   by 256
        xor     dx,ax           ; & exclusive or with the new value low byte
        loop    $1              ; decrement counter & loop until no more data
;end of loop
;
        les     bx,[bp].val_off ; get pointer to function value
        mov     es:[bx],dx      ; put function value on stack
        pop     bp
        ret
;
;       CRC-16 Look-up table for Polynomial 0120001
;
        .radix 8
crc16   dw 000000, 140301, 140601, 000500, 141401, 001700, 001200, 141101
        dw 143001, 003300, 003600, 143501, 002400, 142701, 142201, 002100
        dw 146001, 006300, 006600, 146501, 007400, 147701, 147201, 007100
        dw 005000, 145301, 145601, 005500, 144401, 004700, 004200, 144101
        dw 154001, 014300, 014600, 154501, 015400, 155701, 155201, 015100
        dw 017000, 157301, 157601, 017500, 156401, 016700, 016200, 156101
        dw 012000, 152301, 152601, 012500, 153401, 013700, 013200, 153101
        dw 151001, 011300, 011600, 151501, 010400, 150701, 150201, 010100
        dw 170001, 030300, 030600, 170501, 031400, 171701, 171201, 031100
        dw 033000, 173301, 173601, 033500, 172401, 032700, 032200, 172101
        dw 036000, 176301, 176601, 036500, 177401, 037700, 037200, 177101
        dw 175001, 035300, 035600, 175501, 034400, 174701, 174201, 034100
        dw 024000, 164301, 164601, 024500, 165401, 025700, 025200, 165101
        dw 167001, 027300, 027600, 167501, 026400, 166701, 166201, 026100
        dw 162001, 022300, 022600, 162501, 023400, 163701, 163201, 023100
        dw 021000, 161301, 161601, 021500, 160401, 020700, 020200, 160101
        dw 120001, 060300, 060600, 120501, 061400, 121701, 121201, 061100
        dw 063000, 123301, 123601, 063500, 122401, 062700, 062200, 122101
        dw 066000, 126301, 126601, 066500, 127401, 067700, 067200, 127101
        dw 125001, 065300, 065600, 125501, 064400, 124701, 124201, 064100
        dw 074000, 134301, 134601, 074500, 135401, 075700, 075200, 135101
        dw 137001, 077300, 077600, 137501, 076400, 136701, 136201, 076100
        dw 132001, 072300, 072600, 132501, 073400, 133701, 133201, 073100
        dw 071000, 131301, 131601, 071500, 130401, 070700, 070200, 130101
        dw 050000, 110301, 110601, 050500, 111401, 051700, 051200, 111101
        dw 113001, 053300, 053600, 113501, 052400, 112701, 112201, 052100
        dw 116001, 056300, 056600, 116501, 057400, 117701, 117201, 057100
        dw 055000, 115301, 115601, 055500, 114401, 054700, 054200, 114101
        dw 104001, 044300, 044600, 104501, 045400, 105701, 105201, 045100
        dw 047000, 107301, 107601, 047500, 106401, 046700, 046200, 106101
        dw 042000, 102301, 102601, 042500, 103401, 043700, 043200, 103101
        dw 101001, 041300, 041600, 101501, 040400, 100701, 100201, 040100
mxcrc   endp
code2   ends
;
        end
