        page    58,132
        title extended int 14 handler for communications ports
        name    excom
;
;
;           copied from Dr. Dobb's Journal of Software Tools,  June 1987
;
;       (C) 1987, Crystal Computer Consulting Inc.
;       This software may be used freely, at your own risk,
;       as long as this notice is not removed .
;
;       Basis for Tektronix 4??? terminal emulator .
;
_text   segment word    public 'code'
        assume  cs:_text, ss:_text, ds:_text, es:_text
;
        org     100h            ; start address for a .com file
                
start:  jmp     excom

        even
;
;               Miscellaneous constants
;               ***********************
;
BUFSZ   equ     0100h           ; buffer size
XOFFSZ  equ     00f0h           ; input flow control turnoff point
XONSZ   equ     00e0h           ; input flow control turnon point
DTR     equ     01h             ; select DTR input flow control
RTS     equ     02h             ; select RTS input flow control
XNXF    equ     04h             ; select XON/XOFF input flow control
OUT2    equ     08h             ; bit to set OUT2 on UART
NOCTS   equ     10h             ; CTS not required to transmit
NODSR   equ     20h             ; DSR not required to transmit
B19200  equ     40h             ; set baud rate to 19200
B38400  equ     80h             ; set baud rate to 38400
C1MASK  equ     10h             ; COMM1 mask for 8259
C2MASK  equ     08h             ; COMM2 mask for 8259
;
;
;                   PORT CONTROL BLOCKS
;                   *******************
;
;               Structure for port control blocks
;
pcbs    struc
    putptr  dw      ?           ; points to last char put
    getptr  dw      ?           ; points to next character to get
    bufcnt  dw      ?           ; number of characters in buffer
    bufend  dw      ?           ; address past buffer end
    pbase   dw      ?           ; port address
    timeout db      ?           ; timeout outer loop
    mask    db      ?           ; mask to enable port interrupt
    rxoff   db      ?           ; set if recv has been stopped
    opts    db      ?           ; holds extended options
    lchar   db      ?           ; holds chararacter attributes
    oldier  db      ?           ; old interrupt enable register
    oldlcr  db      ?           ; old line control register
    oldmcr  db      ?           ; old modem control register
    olddll  db      ?           ; old baud low divisor latch
    olddlm  db      ?           ; old baud high divisor latch
    buf     dw BUFSZ dup (?)    ; the buffer
pcbs    ends
;
;           allocate storage for port control blocks
;
pcb     pcbs    2   dup (<>)    ; array of port control blocks
;
old0B   dd      ?               ; old vector for INT 0B
old0C   dd      ?               ; old vector for INT 0C
old14   dd      ?               ; old vector for INT 14
oldmsk  db      ?               ; old mask for 8259
;
;           baud rate divisor table
;           ***********************
;
bauds   dw      0417h           ; divisor for 110 baud
        dw      0300h           ;             150
        dw      0180h           ;             300
        dw      00C0h           ;             600
        dw      0060h           ;            1200
        dw      0030h           ;            2400
        dw      0018h           ;            4800
        dw      000Ch           ;            9600
baud19  dw      0006h           ;           19200
baud38  dw      0003h           ;           38400

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                       ;
;  Interrupt handler for 0Bh and 0Ch  ( COMM1 and COMM2 respectively )  ;
;       put characters into the appropriate buffer                      ;
;       input flow control - stop input when buffer nears full          ;
;                                                                       ;
;  int0B and int0C are handlers used invoked by hardware interrupts     ;
;  generated by the UART when a data byte is received                   ;
;                                                                       ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;           point to COMM2 port control block with ds:si
;
int0B:
        push    ds
        push    si
        mov     si,cs
        mov     ds,si
        lea     si,pcb + size pcbs
        jmp     short a10
;
;           point to COMM1 port control block with ds:si
;
int0C:
        push    ds
        push    si
        mov     si,cs
        mov     ds,si
        lea     si,pcb
;
a10:
        push    di
        push    dx
        push    bx
        push    ax
;
;           read character. If this fills buffer to XOFFSZ, then send
;           XOFF or drop DTR and/or RTS as indicated by extended options
;
        mov     di,[si].putptr      ; buffer put pointer
        mov     dx,[si].pbase
        in      al,dx               ; read character and clear interrupt
        mov     ah,al
        cmp     [si].bufcnt,XOFFSZ
        jl      a40                 ; jump if buffer is below turnoff point
        cmp     [si].rxoff,0
        jne     a30                 ; jump if receive already disabled
        mov     [si].rxoff,1        ; indicate receiver now disabled.
        test    [si].opts,XNXF
        jz      a20                 ; jump if XON/XOFF is disabled
        mov     al,'S'-'@'
        out     dx,al               ; send ^S = XOFF
a20:
        mov     bl,[si].opts
        and     bl,DTR or RTS
        jz      a30
        add     dx,4
        in      al,dx
        not     bl
        and     al,bl
        out     dx,al               ; drop DTR and/or RTS
        sub     dx,4
;
;           if the buffer is full, set overflow status and exit
;
a30:
        cmp     [si].bufcnt,BUFSZ
        jl      a40
        or      word ptr [di],0200h
        jmp     short a99
;
;           read the status port
;
a40:
        add     dx,5
        in      al,dx
;
;           get new buffer pointer, adjust for wrap around if required
;
        inc     di
        inc     di
        cmp     di,[si].bufend
        jne     a50
        lea     di,[si].buf
;
;           store new buffer pointer and put status:data into the buffer
;
a50:
        mov     [si].putptr,di
        xchg    ah,al
        mov     [di],ax
        inc     [si].bufcnt
;
;           send interrupt complete command to interrupt controller
;
a99:
        mov     al,20h
        out     20h,al
        pop     ax
        pop     bx
        pop     dx
        pop     di
        pop     si
        pop     ds
        iret
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                           ;
;           14h SOFTWARE INTERRUPT HANDLER                  ;
;           ******************************                  ;
;                                                           ;
;       emulate PC BIOS functions plus extensions           ;
;                                                           ;
;       dx is used to select port for all calls             ;
;           dx=0 for COMM1,  dx=1 for COMM2                 ;
;                                                           ;
;       ah =0   initialize the port:                        ;
;         al bits 7-5 set baud rate                         ;
;              000=110,   001=150,   010=300,   011=600     ;
;              100=1200,  101=2400,  110=4800,  111=9600    ;
;         al bits 4-3 set parity:                           ;
;              00=none,   01=odd,    11=even                ;
;         al bit 2 is number of stop bits,                  ;
;              0=1 stop bit, 1=2 stop bits                  ;
;         al bits 1-0 set the word length                   ;
;              00=5,      01=6,      10=7,      11=8        ;
;           returns ah & al as in COMM STATUS (ah=3) below  ;
;           example:  ah = 10101110 is 2400 baud,           ;
;                odd parity, 2 stop bits, 7 bit characters  ;
;                                                           ;
;       ah = 1  transmit character in al                    ;
;           returns ah & al as in COMM STATUS (ah=3) below  ;
;           only bits 1-2-3-4-7 can be set in the returned  ;
;           ah .                                            ;
;                                                           ;
;       ah = 2  return received char in al                  ;
;           returns ah & al as in COMM STATUS (ah=3) below  ;
;           only bits 1-2-3-4-7 can be set in the returned  ;
;           ah . ah having any bits set is receive error or ;
;           timeout .                                       ;
;                                                           ;
;       ah = 3  return COMM port status in ah & al          ;
;         ah bits are:                                      ;
;           b0 = data ready,    b1 = overrun                ;
;           b2 = parity error,  b3 = framing error          ;
;           b4 = break detect,  b5 = xmit holding reg empty ;
;           b6 = xmit shift reg empty, b7 = timeout         ;
;         al bits are:                                      ;
;           b0 = delta CTS,     b1 = delta CSR              ;
;           b2 = trail edge ring, b3 = delta rec. detect    ;
;           b4 = CTS            b5 = DSR                    ;
;           b6 = ring indicator,b6 = rec. signal detect     ;
;                                                           ;
;       ah = 4  extended options,                           ;
;           returns 05a5ah in ax, used to identify excom    ;
;         al contains options as follows :                  ;
;           b0 = enable DTR input flow control              ;
;           b1 = enable RTS input flow control              ;
;           b2 = enable XON/XOFF input flow control         ;
;           b4 = don't require CTS to transmit              ;
;           b5 = don't require DSR to transmit              ;
;           b6 = set baud rate to 19200                     ;
;           b7 = set baud rate to 38400                     ;
;                                                           ;
;       ah = 5  remove excom, and restore vectors           ;
;                                                           ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
int14:
        sti
        push    ds
        push    si
        push    dx
        push    cx
        push    bx
;
;           point to selected COMM port control block with ds:si
;
        mov     bx,cs
        mov     ds,bx
        lea     si,pcb
        or      dx,dx
        jz      b00
        add     si,size pcbs
;
;           get port address in dx, return if port not installed
;
b00:
        mov     dx,[si].pbase
        or      dx,dx
        jz      b40
;
;           ah contains request type, jump to selected routine
;
        or      ah,ah
        jz      b000                ; initialize
        dec     ah
        jz      b100                ; transmit char
        dec     ah
        jnz     b10
        jmp     b200                ; receive char
b10:
        dec     ah
        jnz     b20
        jmp     b300                ; get status
b20:
        dec     ah
        jnz     b30
        jmp     b400                ; extended initialize
b30:
        dec     ah
        jnz     b40
        jmp     b500                ; remove excom
b40:
        pop     bx
        pop     cx
        pop     dx
        pop     si
        pop     ds
        iret
;
;           AH = 0     INITIALIZE PORT
;           **************************
;
b000:
        push    ax
;
;           initilize the port control block, this flushes the input buffers
;
        lea     ax,[si].buf
        mov     [si].putptr,ax
        inc     ax
        inc     ax
        mov     [si].getptr,ax
        add     ax,BUFSZ * 2
        mov     [si].bufend,ax
        sub     ax,ax
        mov     [si].bufcnt,ax
        mov     [si].rxoff,al
;
;           assert flow control signals and enable port interrupts
;
        add     dx,4
        mov     al,DTR or RTS or OUT2
        out     dx,al
        cli
        in      al,021h
        and     al,[si].mask
        out     021h,al         ; unmask com interrupt on 8259 int controller
        sti
        sub     dx,3
        mov     al,1
        out     dx,al               ; enable receive int. on UART
;
;           set baud rates and char. attributes.
;
        add     dx,2
        pop     ax
        mov     bl,al
        and     al,01fh
        mov     [si].lchar,al
        mov     al,[si].opts
        and     al,B19200 or B38400
        jz      b010
        call    b440                ; init only character attributes
        jmp     short b020
b010:
        mov     cl,4
        rol     bl,cl
        and     bx,0Eh              ; isolate baud rate selection.
        call    b430                ; init baud rate and character attributes
b020:
        sub     dx,3
        jmp     b300                ; return status
;
;           AH = 1      TRANSMIT THE CHARACTER IN AL
;           ****************************************
;
b100:
        push    ax
        add     dx,6
        mov     bh,[si].opts
        not     bh
        and     bh,NODSR or NOCTS
;
;           optionally wait for DSR and/or CTS
;
; ********bypass        call    b130
; ********bypass        jnz     b110
        dec     dx
        mov     bh,20h
;
;           wait for the transmitted char to be ready
;
        call    b130
        jnz    b110
        sub     dx,5
        pop     ax
        push    ax
;
;           actually send the character
;
        out     dx,al
        call    b310
        jmp     short b120
;
;           restore al and indicate a timeout ( bit 7 of status, ah )
;
b110:
        call    b310
        or      ah,080h
b120:
        pop     cx
        mov     al,cl
        jmp     b40
;
;           wait for selected status, bh is status, dx is port address
;
b130:
        mov     bl,[si].timeout         ; delay counter
b140:
        sub     cx,cx
b150:
        in      al,dx
        and     al,bh
        cmp     al,bh
        jz      b160                    ; status match
        loop    b150
        dec     bl
        jnz     b140
        or      bh,bh                   ; timeout set nonzero
b160:
        ret
;
;           AH = 2      RECEIVE CHARACTER, PUT IT IN AL
;           *******************************************
;
b200:
        cmp     [si].bufcnt,0
        jne     b230                ; chars in buffer, return one
;
;           wait for character to appear in the buffer, or return timeout
;
        mov     al,[si].timeout
        add     al,al
b210:
        sub     cx,cx
b220:
        cmp     [si].bufcnt,0
        jne     b230
        loop    b220
        dec     al
        jnz     b210
        mov     ax,08000h            ; timeout return value
        jmp     b40
;
;           if the receiver is disabled and room now exists, enable receiver
;
b230:
        cmp     [si].rxoff,0
        je      b250                ; receiver is already enabled
        cmp     [si].bufcnt,XONSZ
        jg      b250                ; buffer is full at that turn on point
        test    [si].opts,XNXF
        jz      b240
        mov     [si].rxoff,0
        mov     al,'Q'-'@'          ; send ^Q  = XON
        out     dx,al
;
;           it is easier just to assert DTR & RTS than to see if it is needed
;
b240:
        add     dx,4
        in      al,dx
        or      al,DTR or RTS
        out     dx,al               ; assert DTR and RTS
;
;           get the character from the buffer and update the buffer get pointer
;
b250:
        call    b310
        and     ah,01eh             ; status reported with a receive
        inc     bx
        inc     bx
        cmp     bx,[si].bufend
        jne     b260
        lea     bx,[si].buf
b260:
        mov     [si].getptr,bx      ; updated pointer
        dec     [si].bufcnt
        jmp     b40
;
;           AH = 3      GET PORT STATUS
;           ***************************
;
b300:
        call    b310
        inc     dx
        in      al,dx
        jmp     b40                 ; modem status
;
;           routine :
;           get line status  from input bufer and/or hardware into ah
;
b310:
        mov     dx,[si].pbase
        add     dx,5
        in      al,dx
        mov     ah,al               ; line status
        cmp     [si].bufcnt,0
        je      b320
;
;           when characters are in buffer, fix status as per status in buffer,
;           return as above plus bx=getptr, al = char.
;
        mov     cl,al
        mov     bx,[si].getptr
        mov     ax,[bx]             ; status:data (ah:al)
        mov     ch,01eh             ; these status bits from buffer
        and     ah,ch
        not     ch
        and     cl,ch
        or      ah,cl
        or      ah,1                ; char in buffer: show data ready
b320:
        ret
;
;           AH = 4   EXTENDED INITIALIZATION
;           ********************************
;
b400:
;           save options in the port control block
;
        mov     [si].opts,al
;
;           if extended baud rate, set up UART
;
        add     dx,3
        test    al,B19200
        jz      b410
        mov     bx,baud19-bauds
        call    b430
        jmp     short b420
b410:
        test    al,B38400
        jz      b420
        mov     bx,baud38-bauds
        call    b430
b420:
        mov     ax,05a5ah           ; excom signature
        jmp     b40
;
;           routine :
;           init the baud rate and character attributes
;
b430:
        mov     al,080h
        out     dx,al
        mov     ax,[bx+bauds]
        sub     dx,3
        out     dx,al               ; set low order divisor on UART
        inc     dx
        mov     al,ah
        out     dx,al               ; set high order divisor on UART
        inc     dx
        inc     dx
b440:
        mov     al,[si].lchar
        out     dx,al               ; set char. attributes
        ret
;
;           AH = 5      REMOVE EXCOM
;           ************************
;
b500:
        push    ds
        push    ds
;
;           restore status of interrupt controller and UARTS
;
        cli
        mov     ah,oldmsk
        and     ah,C1MASK or C2MASK
        in      al,021h
        and     al,not (C1MASK or C2MASK)
        or      al,ah
        out     021h,al
        sti
        lea     si,pcb              ; COMM1 port control block
        call    b510                ; restore COMM1 UART status
        lea     si,pcb+size pcbs    ; COMM2 port control block
        call    b510                ; restore COMM2 UART status
        jmp     short b530
;
;           routine to restore UART status
;
b510:
        mov     dx,[si].pbase
        or      dx,dx
        jz      b520                ; no port
        add     dx,3
        mov     al,080h
        out     dx,al               ; set access to baud divisors
        sub     dx,3
        mov     al,[si].olddll
        out     dx,al               ; baud divisor low
        inc     dx
        mov     al,[si].olddlm
        out     dx,al               ; baud divisor high
        add     dx,2
        mov     al,[si].oldlcr
        out     dx,al               ; line control register
        inc     dx
        mov     al,[si].oldmcr
        out     dx,al               ; modem control register
        sub     dx,3
        mov     al,[si].oldier
        out     dx,al               ; int. enable reg.
b520:
        ret
;
;           restore old int 0Bh vector
;
b530:
        lds     dx,old0B
        mov     ah,025h
        mov     al,0Bh
        int     021h
;
;           restore old int 0Ch vector
;
        pop     ds
        lds     dx,old0C
        mov     ah,025h
        mov     al,0Ch
        int     021h
;
;           restore old int 14h vector
;
        pop     ds
        lds     dx,old14
        mov     ah,025h
        mov     al,14h
        int     021h
;
;           free excom's memory, i.e. this memory
;
        push    es
        mov     ax,cs
        mov     es,ax
        mov     ah,049h
        int     021h
        pop     es
        jmp     b40
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                           ;
;       Installation entry point, must be at file end       ;
;           Initialize and stay resident.                   ;
;                                                           ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
excom:
;
;           initialize constant parts of each port control block
;
        mov     ax,040h
        mov     es,ax                   ; bios data segment
        sub     di,di                   ; offset to COMM1 base address
        mov     bx,07ch                 ; offset to COMM1 timeout
        mov     cl,not C1MASK           ; COMM1 interrupt mask
        lea     si,pcb                  ; COMM1 port control block address
        call    c00                     ; init COMM1
        inc     di
        inc     di                      ; offset to COMM2 base address
        inc     bx                      ; offset to COMM2 timeout
        mov     cl,not C2MASK           ; COMM2 interrupt mask
        add     si,size pcbs            ; COMM2 port control block address
        call    c00                     ; init COMM2
        jmp     short c10
;
;           subroutine to initialise parts of a pcb
;
c00:
        mov     dx,es:[di]              ; port base from BIOS
        mov     [si].pbase,dx
        mov     al,es:[bx]
        mov     [si].timeout,al         ; copy timeout from BIOS
        mov     [si].mask,cl            ; save interrupt mask
        ret
;
;           save old status of interrupt controller and UARTS
;
c10:
        in      al,021h
        mov     oldmsk,al
        lea     si,pcb                  ; COMM1 port control block
        call    c20                     ; save old COMM1 UART status
        lea     si,pcb + size pcbs      ; COMM2 port control plock
        call    c20                     ; save old COMM2 UART status
        jmp     short c40
;
;           subroutine to save UART status
;
c20:
        mov     dx,[si].pbase
        or      dx,dx
        jz      c30                     ; no port
        add     dx,3
        in      al,dx
        and     al,07fh
        mov     [si].oldlcr,al          ; line control register
        mov     al,080h
        out     dx,al                   ; set access to baud divisors
        sub     dx,3
        in      al,dx
        mov     [si].olddll,al          ; baud divisor low
        inc     dx
        in      al,dx
        mov     [si].olddlm,al          ; baud divisor high
        add     dx,2
        mov     al,[si].oldlcr
        out     dx,al                   ; restore register access
        inc     dx
        in      al,dx
        mov     [si].oldmcr,al          ; modem cotrol register
        sub     dx,3
        in      al,dx
        mov     [si].oldier,al          ; interrupt control register
c30:
        ret
;
;           release environnment memory, not used
;
c40:
        mov     bx,02ch
        mov     ax,[bx]
        mov     es,ax                   ; address of environment
        mov     ah,049h
        int     021h                    ; release memory
;
;           save old int0B vector and install new handler
;
        push    ds
        mov     ax,cs
        mov     ds,ax
        mov     ah,035h
        mov     al,0Bh
        int     021h
        mov     word ptr old0B,bx
        mov     ax,es
        mov     word ptr old0B + 2,ax
        lea     dx,int0B
        mov     ah,025h
        mov     al,0Bh
        int     021h
;
;           save old int0C vector and install new handler
;
        mov     ah,035h
        mov     al,0Ch
        int     021h
        mov     word ptr old0C,bx
        mov     ax,es
        mov     word ptr old0C + 2,ax
        lea     dx,int0C
        mov     ah,025h
        mov     al,0Ch
        int     021h
;
;           save old int14 vector and install new handler
;
        mov     ah,035h
        mov     al,14h
        int     021h
        mov     word ptr old14,bx
        mov     ax,es
        mov     word ptr old14 + 2,ax
        lea     dx,int14
        mov     ah,025h
        mov     al,14h
        int     021h
        pop     ds
;
;           exit and keep everything above the entry point 'excom'
;
        lea     dx,excom
       mov     cl,4
      shr     dx,cl
     inc     dx
        mov     ah,031h
        mov     al,0
        int     021h                    ; terminate and stay resident.
;		mov		dx,offset excom
;		int		27h
;
_text   ends
        end     start
        end
