;    Copyright (C) 1997  Peter Vernon Lanspeary
;
;    This documentation is free; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This documentation is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this documentation; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


TITLE   Hercules graphics module
        page 58,132
;
;       Michael Gordon - 8-Dec-86
;
; Certain routines were taken from the Hercules BIOS of Dave Tutelman - 8/86
; Others came from pcgraph.asm included in GNUPLOT by Colin Kelley
;
; modified slightly by Colin Kelley - 22-Dec-86
;       added header.mac, parameterized declarations
; added dgroup: in HVmodem to reach HCh_Parms and HGr_Parms - 30-Jan-87
;
; modified slightly for use in plot(3) routines back end.
; Gil Webster.
;
; modified for Lahey FORTRAN  by Peter Lanspeary
;   - modified public symbols ( i.e. subroutine names ) so that
;       they are legal FORTRAN names .
;   - argument passing convention is by reference - completely different
;       from C
;   - HEADER.MAC is removed because concept of different memory models is
;       not defined by the Lahey FORTRAN
;   - add the routine HG_Dump for dumping the screen onto a print file
;   - add the routine HG_xhair for generating & driving cross hairs
;
; Assemble with masm ver. 4.
;
if1
include lp_f77l.mac
endif
;
;       DEFINITIONS
;
GPg1_Base   equ     0B800h      ; Graphics page 1 base address ( segment )
Arg_Offset  equ     6           ; Argument offset for Lahey FORTRAN
;
_text   segment
        assume      cs:_text
        public HG_line, HG_colour, HG_mask, HG_on,    HG_off
        public HG_puts, HG_erase,  HG_dump, HG_xhair
;
hpixel  proc near
        ror word ptr cs:bmask,1
        jc cont
        ret
cont:
        push ax
        push bx
        push cx
        push dx
        push si
        push es
        mov cx,ax               ; x
        mov dx,bx               ; y
;
;           first compute the address of byte to be modified
;               = 90*[row/4] + [col/8] + 2^D*[row/4] + 2^F*page
;
        mov     bh,cl           ; col (low order) in BH
        mov     bl,dl           ; row (low order) in BL
        and     bx,0703H        ; mask the col & row remainders
IFDEF iAPX286
        shr     cx,3            ; col / 8
        shr     dx,2            ; row / 4
        mov     al,90
        mul     dx              ; AX = 90*[ row/4 ]
        add     ax,cx           ;  ... + col/8
        shl     bl,5            ; align row remainder
ELSE                    ; same as above, obscure but fast for 8086
        shr     cx,1            ; col / 2
        shr     cx,1            ; col / 4
        shr     cx,1            ; col / 8
        shr     dx,1            ; row / 2
        shr     dx,1            ; row / 4
;
;         begin fast multiply by 90 (1011010 B)
;
        shl     dx,1            ; ( row / 4 ) * 2
        mov     ax,dx           ; store ( row / 4 ) * 2
        shl     dx,1            ; ( row / 4 ) * 4
        shl     dx,1            ; ( row / 4 ) * 8
        add     ax,dx           ; ( row / 4 ) * 10
        shl     dx,1            ; ( row / 4 ) * 20
        add     ax,dx           ; ( row / 4 ) * 22
        shl     dx,1            ; ( row / 4 ) * 44
        shl     dx,1            ; ( row / 4 ) * 88
        add     ax,dx           ; ( row / 4 ) * 90  end fast multiply by 90
        add     ax,cx           ; add on the ( col / 8 )
        shl     bl,1            ; align row remainder
        shl     bl,1
        shl     bl,1
        shl     bl,1
        shl     bl,1
ENDIF
        add     ah,bl           ; use aligned row remainder
end_adr_calc:                   ; address of byte is now in AX
        mov     dx,GPg1_Base    ; base of pixel display to DX
        mov     es,dx           ; ...and thence to segment reg
        mov     si,ax           ; address of byte w/ pixel to index reg
        mov     cl,bh           ; bit addr in byte
        mov     al,80H          ; '1000 0000' in AL 
        shr     al,cl           ; shift mask to line up with bit to read/write
;
;           set the pixel
;              ( this will write `black' stuff on the screen as well )
set_pix:
        mov     ah,al
        and     ah, byte ptr cs:colour  ; mask .and. colour_flag
        not     al
        and     al,es:[si]              ; old_data_byte .and. .not. mask
        or      al,ah                   ; new_data_byte
        mov     es:[si],al              ; store new_data_byte on the screen
;
        pop es
        pop si
        pop dx
        pop cx
        pop bx
        pop ax
        ret
hpixel endp

;
;       SUBROUTINE HG_line( X1, Y1, X2, Y2 )
;   Draw a line on the the graphics screen from (X1,Y1) to (X2,Y2) .
;
lineproc HG_line, Hpixel
;
;       SUBROUTINE HG_erase
;   Clears the graphics screen
;
HG_erase proc far
        call near ptr clear             ; clear the graphics page
        ret
HG_erase endp
;
; clear - clear page 1 of the screen buffer to zero (effectively, blank
;       the screen)
;
clear   proc near
        push ax
        push cx
        push di
        push es
        mov ax, GPg1_Base
        mov es, ax          ; address segment of Hercules card
        xor di, di          ; first address offset of Hercules card = 0
        mov cx, 4000h       ; no. of words to clear
        xor ax, ax          ; source = 0
        cld                 ; autoincrement
        rep stosw           ; zero out screen page
        pop es
        pop di
        pop cx
        pop ax
        ret
clear   endp
;
;
;       SUBROUTINE HG_colour( ICOLOUR )
;   Changes the drawing colour to colour number ICOLOUR . For the
;   Hercules graphics card, there are in fact only two colours :-
;   ICOLOUR = 1 = `white',   ICOLOUR = 0 = `black'
;     ( any non-zero value of ICOLOUR is white )
;
HG_colour  proc far
        push bp
        mov     bp,sp
        xor     ah,ah           ; clear global colour flag - i.e. black
        les     bx,[bp].Arg_offset
        mov     al,es:[bx]      ; get colour argument
        test    al,al
        jz      cblack          ; if zero , keep black colour flag
        dec     ah              ; set global colour flag to white
cblack: mov     byte ptr cs:colour,ah
        pop bp
        ret
HG_colour endp
;
;
;       SUBROUTINE HG_mask( IMASK )
;   Set the value of the bit mask which controls the dashed line style
;   to IMASK . IMASK is a 16 bit integer .
;
HG_mask proc far
        push bp
        mov bp,sp
        les bx,[bp].Arg_offset
        mov ax,es:[bx]              ; get line drawing mask argument
        mov word ptr cs:bmask,ax    ;   & store it
        pop bp
        ret
HG_mask endp

HCtrl_Port      equ     03B8H   ; Hercules 6845 control port IO addr
HIndx_Port      equ     03B4H   ; Hercules 6845 index port IO addr
HScrn_Enable    equ     008h    ; Control port bit to enable video
HCh_Mode        equ     020h    ; Character output mode
HGr_Mode        equ     082h    ; Graphics output mode page 1
parm_count      equ     12
;
;       SUBROUTINE HG_Off
;   Switch graphics off . Go back to alpha-numeric screen .
;
HG_Off  proc far
        push    ax
        push    dx
        push    si
;
        mov     al, HCh_Mode            ; character mode
        mov     dx, HCtrl_Port
        mov     si, offset cs:HCh_Parms
        out     dx, al                  ; Set Hercules board to proper mode
        call    near ptr setParms       ; Set the 6845 parameters
        or      al, HScrn_Enable        ; Enable the video output
        out     dx, al
;
        pop si
        pop dx
        pop ax
        ret
HG_Off  endp
;
;
;       SUBROUTINE HG_On
;   Switch graphics on .
;
;  Values for Configuration switch
;
ADDR_config     equ     03BFH   ; Configuration switch
Config_gr_disable equ   00h     ; prevents setting of graphics mode (default)
Config_gr_enable  equ   01h     ; allows setting of graphics mode
Config_page1_out  equ   00h     ; mask Page 1 out of memory map 
                                ;    prevent its use
Config_page1_in   equ   02h     ; bring Page 1 into memory & allow its use 
;
;            
HG_On   proc far
        push ax
        push dx
        push si
;
        mov  al, Config_gr_enable + Config_page1_in
                                        ; send "HGC FULL"
        mov  dx, ADDR_config            ;   to the Herc configuration switch
        out  dx, al                     ;
;
        mov al, HGr_Mode                ; graphics mode
        mov dx, HCtrl_Port
        mov si, offset cs:HGr_Parms
        out dx, al                      ; Set Hercules board to proper mode
        call near ptr setParms          ; Set the 6845 parameters
        or al, HScrn_Enable             ; Enable the video output
        out dx, al
;
        pop si
        pop dx
        pop ax
        ret
HG_On   endp
;
;           Send 6845 parameters to Hercules board
; Register parameters :-
;  cs:di = pointer to list of 6845 parameters
;
setParms    proc near
        push    ax
        push    dx
        mov     dx, HIndx_Port      ; Index port addr -> DX
        mov     ah, 0               ; 0 -> parameter counter
sp_loop:
        mov     al, ah
        out     dx, al              ; output to 6845 addr register
        inc     dx                  ; next output to data register
        mov     al, cs:[si]         ; next control byte -> al
        inc     si
        out     dx, al              ; output control byte
        dec     dx                  ; 6845 index addr -> dx
        inc     ah                  ; bump addr
        cmp     ah, parm_count
        jnz     sp_loop
        pop     dx
        pop     ax
        ret
setParms    endp
;
;           SUBROUTINE HG_PUTS( CHAR_STRING, ROW, COLUMN )
;
;    - print text in graphics mode
; Arguments
;  character CHAR_STRING*(*)    # character string to be printed
;  integer *2 ROW, COLUMN       # screen row & starting column of the text
;
;       cx = row
;       bx = column
;       si = address of string (null terminated) to print
;
HG_puts proc far
        push bp
        mov bp, sp
;
;               get the arguments
;
        lds     bx,[bp].Arg_offset      ; get ptr to character string structure
        mov     dx,[bx+4]               ; get length of string
        mov     si,[bx]                 ; address segment of character string
        mov     ds,[bx+2]               ; address offset of character string
        les     bx,[bp+4].Arg_offset
        mov     cx,es:[bx]              ; get screen row number
        les     bx,[bp+8].Arg_offset
        mov     bx,es:[bx]              ; get screen column number
;
        call near ptr pr_char           ; display the character on the screen
        pop bp
        ret
HG_puts endp
;
; pr-char - output an 8x8 character from the IBM ROM to the Herc board
;
; Register parameters
;   bx = starting screen column number (0-89)
;   cx = screen row number (0-42)
;   dx = number of characters
;   ds:si = pointer to character string
;
IBMROM      equ     0F000h      ; address segment of IBM ROM
CHARTAB     equ     0FA6Eh      ; address offset of character table in IBM ROM
last_col    equ     89          ; number of graphics screen columns
last_row    equ     42          ; number of graphics screen rows
;
pr_char proc near
        push    ax
        push    di
        push    es
        test    cx,cx       ; if ( row < 0 .or.
        jl      pret
        cmp     cx,last_row ;      row > last_row )
        jg      pret        ;   return
;
;               loop for writing the characters one at a time
;
ploop:  lodsb                           ; get next char
        test    bx,bx                   ; if ( column < 0 .or.
        jl      pskip
        cmp     bx,last_col             ;      column > last_column )
        jg      pskip                   ;   do not display the character
;
; setup ds -> IBM ROM, and si -> index into IBM ROM character table located
;       at 0fa6eh in the ROM
;
        push    si                      ; save the character string pointer
        push    ds
        and     ax, 07fh
        mul     cs:CON8                 ; mult by 8 bytes of table per char
        mov     si, ax
        mov     ax, IBMROM
        mov     ds, ax
        assume  ds:nothing
        add     si, CHARTAB             ; add offset of character table
;
; compute index into Hercules screen memory for scan line 0.  The remaining
;       seven scan lines are all at fixed offsets from the first.
;
;       Since graphics mode treats the screen as sets of 16x4 "characters",
;       we need to map an 8x8 real character onto the front or back of
;       a pair of graphics "characters".  The first four scan lines of our
;       8x8 character will map to the top graphics "character", and the second
;       four scan lines map to the graphics character on the "line" (4 scan
;       lines high) below it.
;
;       For some exotic hardware reason (probably speed), all scan line 0
;       bits (i.e. every fourth scan line) are stored in memory locations
;       0-2000h in the screen buffer.  All scan line 1 bits are stored
;       2000h-4000h.  Within these banks, they are stored by rows.  The first
;       scan line on the screen (scan line 0 of graphics character row 0)
;       is the first 45 words of memory in the screen buffer.  The next 45
;       words are the first scan line graphics row 1, and since graphics
;       "characters" are 4 bits high, this second scan line is physically
;       the fifth scan line displayed on the screen.
;
;       SO, to display an 8x8 character, the 1st and 5th rows of dots are
;       both scan line 0 of the graphics "character", the 2nd and 6th are
;       scan line 1, and so on.
;
;       The column (0-89) tells which byte in a scan line we need to load.
;       Since it takes two rows of graphics characters to hold one row of
;       our characters, column+90 is a index to scan line 4 rows of pixels
;       higher (n+4).  Thus 180 bytes of screen memory in any bank (0h, 2000h,
;       4000h, 6000h) represent a row of 8x8 characters.
;       
;       The starting location in screen memory for the first scan line of
;       a character to be displayed will be:    (row*180)+column
;       The 5th scan line will be at:           (row*180)+column+90
;
;       The second and 6th scan lines will be at the above offsets plus
;       the bank offset of 2000h.  The third and 7th, add 4000h and finally
;       the 4th and 8th, add 6000h.
;
        mov     ax, GPg1_Base
        mov     es, ax                  ; es = hercules page 1
        mov     ax, cx                  ; get row
        mul     cs:CON180               ; mult by 180(10)
        mov     di, ax                  ; di = index reg
        cld                             ; insure right direction
;
;           output 8 segments of character to video ram
;
        lodsb                           ; line 0
        mov     es:[di+bx], al
        lodsb
        mov     es:[di+bx+2000h], al    ; line 1
        lodsb
        mov     es:[di+bx+4000h], al    ; line 2
        lodsb
        mov     es:[di+bx+6000h], al    ; line 3
        lodsb
        mov     es:[di+bx+90], al       ; line 4
        lodsb
        mov     es:[di+bx+2000h+90], al ; line 5
        lodsb
        mov     es:[di+bx+4000h+90], al ; line 6
        lodsb
        mov     es:[di+bx+6000h+90], al ; line 7
;
        pop     ds
        pop     si
pskip:
        inc     bx                      ; increment column number
        dec     dx                      ; decrement character counter
        jg      ploop                   ; loop back if not finished
;
pret:
        pop     es
        pop     di
        pop     ax
        ret
pr_char endp
;
;
;       SUBROUTINE HG_DUMP( PRINT_ROW, PRINT_COL, PRINT_BYTES )
;
;       Converts the area on the Hercules Graphics defined over the character
;  position (x,y) = ( print_row, print_col ) to a sequence of bytes which can
;  be sent directly to a dot matrix printer ( with each bit representing a 
;  dot of the printer matrix ) .
;
;
;   HG_Dump
;       Shell for Lahey FORTRAN
;   This shell provides a correct interface with Lahey FORTRAN . It gets
;   the routine arguments and puts them in the correct registers for the
;   work routine PRDUMP .
;
HG_Dump proc    far
        push    bp
        mov     bp,sp
        lds     si,[bp].Arg_offset      ; get pointer to PRINT_ROW
        mov     bx,ds:[si]              ; get value of PRINT_ROW => bx
        lds     si,[bp+4].Arg_offset    ; get pointer to PRINT_COL
        mov     dx,ds:[si]              ; get value of PRINT_COL => dx
        lds     di,[bp+8].Arg_offset    ; get pointer to PRINT_BYTES => ds:di
        call    near ptr prdump         ; call routine to do the work
        pop     bp
        ret
HG_Dump endp
;
;   PRDUMP
;       Work routine for reading an 8 x 8 group of pixels from the
;   character position (row,column)=(bx,dx)=( PRINT_ROW, PRINT_COL )
;   of the Hercules Graphics screen, and converting these to a set of
;   8 PRINT_BYTES which directly represent corresponding the dot matrix
;   patterns for a dot matrix printer .
;
;   Arguments are passed in the registers
; bx    = PRINT_ROW
; dx    = PRINT_COL
; ds:di = segment:offset pointer to PRINT_BYTES
;
;  All registers except cs, ss, ds, sp are destroyed
;
;                       Bit positions for each Screen_Byte
;                         128  64  32  16  8   4   2   1
;                          7   6   5   4   3   2   1   0
;                        +---+---+---+---+---+---+---+---+
;  Bit           128  7  |   |   |   |   |   |   |   |   | 1
;  positions      64  6  |   |   |   |   |   |   |   |   | 2  8 Bytes from
;  for            32  5  |   |   |   |   |   |   |   |   | 3  the Hercules
;  each           16  4  |   |   |   |   |   |   |   |   | 4    graphics
;  PRINT_BYTE      8  3  |   |   |   |   |   |   |   |   | 5     screen
;                  4  2  |   |   |   |   |   |   |   |   | 6    ( 1 byte
;                  2  1  |   |   |   |   |   |   |   |   | 7    per row )
;                  1  0  |   |   |   |   |   |   |   |   | 8
;                        +---+---+---+---+---+---+---+---+
;                          1   2   3   4   5   6   7   8
;       8 Bytes to the dot matrix printer ( 1 byte per column )
; 
prdump  proc near
;
;           clear the PRINT_BYTES
; register use
;  ax   = 0
;  cx   = word count
;  ds:di= pointer to destination PRINT_BYTE
;
        mov     word ptr cs:prt_ptr,di ; store offset of pointer to PRINT_BYTES
        cld                         ; set direction flag to auto increment
        mov     cx,4                ; no. of bytes to clear
        mov     ax,ds
        mov     es,ax               ; stosw uses extra segment
        xor     ax,ax               ; clear source
        rep stosw                   ; clear PRINT_BYTES
;
;           calculate the address offset of the first required Screen_byte
;             180 * row + column
; register use
;  ax   = temporary storage for 4*row
;  bx   =
;
        shl     bx,1                ; 2   * row
        shl     bx,1                ; 4   * row
        mov     ax,bx               ; temp store for 4 * row
        shl     bx,1                ; 8   * row
        shl     bx,1                ; 16  * row
        add     bx,ax               ; 20  * row
        shl     bx,1                ; 40  * row
        add     bx,ax               ; 44  * row
        shl     bx,1                ; 88  * row
        shl     bx,1                ; 176 * row
        add     bx,ax               ; 180 * row
        add     bx,dx               ; 180 * row + column
;
;           set up the first screen address
;
        mov     bp,bx               ; = address offset of first screen byte
        mov     ax,GPg1_Base
        mov     es,ax               ; = address segment of first screen byte
;
        mov     bx,16
        mov     dh,1                ; initialize print_mask
;
row:    mov     si,word ptr cs:[bx].Char_map-2 ; get offset from look-up table
        add     si,bp               ;  required screen address
        mov     ah,es:[si]          ;  get Screen_byte => ah
;
        mov     cx,8                ;  PRINT_BYTE counter
        mov     dl,128              ; initialize screen_mask
        mov     di,word ptr cs:prt_ptr    ;  pointer to first PRINT_BYTE
;
column: mov     al,ah               ; get Screen_byte => al
        and     al,dl               ; mask with Screen_mask
        jz      mtpix               ; if pixel is not empty
        or      ds:[di],dh          ;   include print_mask into PRINT_BYTE
mtpix:  shr     dl,1                ; move screen_mask to the right
        inc     di                  ; point to the next PRINT_BYTE
        loop    column              ; next column if not done all PRINT_BYTES
;
        shl     dh,1                ; move print_mask to the left
        dec     bx
        dec     bx                  ; point to previous Screen_byte
        jnz     row                 ; next row if not done all Screen_bytes
        ret
prdump  endp
;
;
;       SUBROUTINE HG_XHAIR( X, Y, CHAR )
;            Hercules Graphics X Hairs
;   This is a graphics cross-hairs shell for Lahey FORTRAN . The cross-hairs
; routine places cross-hairs on the graphics screen and allows the user to
; drive them around the screen using the arrow keys on the numeric keypad
; ( i.e.  keys 1, 2, 3, 4, 6, 7, 8, 9 ) . Cross-hairs movement is speeded up
; by holding the <CONTROL> key down while operating the arrow keys . By pressing
; a printable ASCII character key, the user stops the routine and passes the
; current cross-hairs position back to the calling routine through the arguments
; X and Y . The value of the printable ASCII character is returned through CHAR.
;
; Registers used :-
;  ax -   X = row number of cross hairs
;  bx -   Y = column number of cross hairs
;  dl -   exit character
;  di -   argument list pointer
;
;
HG_xhair proc far
        push    bp
        mov     bp,sp
;
        call    near ptr xhair          ; call the cross-hairs driver routine
;
        les     di,[bp].Arg_Offset      ; pointer to 1st argument
        mov     es:[di],ax              ;   store X position in arg list
        les     di,[bp+4].Arg_Offset    ; pointer to 2nd argument
        mov     es:[di],bx              ;   store Y position in arg list
        les     di,[bp+8].Arg_Offset    ; pointer to 3rd argument
        mov     bx,es:[di]              ;   address offset of char string
        mov     es,es:[di+2]            ;   address segment of char string
        mov     es:[bx],dl              ;   store exit character in string
;
        pop bp
        ret
HG_xhair endp
;
;       XHAIR
; This is the cross-hair driver routine . No benefit can be gained from
; optimizing the speed of this code because the speed is limited by the
; rate at which the keyboard sends repeated characters . Routine arguments
; are returned through registers
;  ax -   X = row number of cross hairs
;  bx -   Y = column number of cross hairs
;  dl -   exit character
;
; Other register use :-
;  cx -   Step size , 1= small step, 10h= big step
;  es -   Address offset for Hercules card
;
xhair   proc near
;
;           get initial cross-hair position and paint initial cross-hairs
;
        mov     ax,GPg1_Base            ; segment for Graphics base
        mov     es,ax
        mov     ax,word ptr cs:xhair_x  ; get initial X
        mov     bx,word ptr cs:xhair_y  ; get initial Y
        call    near ptr xor_row        ; draw initial horizontal hair
        call    near ptr xor_col        ; draw initial vertical hair
;
;           read a character from the keyboard
;
char1:  push    ax
        mov     ah,8                    ; function = read from keyboard
        int     21h                     ; DOS function request
        mov     dl,al                   ; put result into dl
        pop     ax
;
;           if ( printable ASCII character ) return to caller
;
        cmp     dl,' '                  ; if ( char < {first printable}
        jl      not_ascii
        cmp     dl,'}'                  ;      or char > {last printable}
        jg      not_ascii               ; then  goto not_ascii_character
;
        mov     word ptr cs:xhair_x,ax  ; store final X
        mov     word ptr cs:xhair_y,bx  ; store final Y
        call    near ptr xor_row        ; remove horizontal hair
        call    near ptr xor_col        ; remove vertical hair
        ret
;
;   If we get to this point, we do not have a printable ASCII character .
;   The arrow key codes are all preceded by the escape character <NUL> .
;   If the character received is not <NUL>, go back and read the keyboard again.
;
not_ascii:
        test    dl,dl                   ; if ( char <> <NULL> ) then
        jnz     char1                   ;   goto read_first_character
;
;           read the remaining character
;
char2:  push    ax
        mov     ah,8                    ; function = read from keyboard
        int     21h                     ; DOS function request
        mov     dl,al                   ; put result into dl
        pop     ax
;
;           ensure that the cursor is on the screen  ( 78 clock cycles )
;
check0: test    ax,ax       ; if ( X < 0 ) X = 0
        jge     check1
        xor     ax,ax
check1: cmp     ax,719      ; if ( X > 719 ) X = 719
        jle     check2
        mov     ax,719
check2: test    bx,bx       ; if ( Y < 0 ) Y = 0
        jge     check3
        xor     bx,bx
check3: cmp     bx,347      ; if ( Y > 347 ) Y = 347
        jle     vert
        mov     bx,347
;
; the keypad key codes are :-
;   Arrow      Numeric  other   slow    fast
;   up left       7     Home    00 47   00 77
;   up            8             00 48
;   up right      9     PgUp    00 49   00 84
;   right         6             00 4D   00 74
;   down right    3     PgDn    00 51   00 76
;   down          2             00 50
;   down left     1     End     00 4f   00 75
;   left          4             00 4b   00 73
;
;           test if a vertical movement is required
;
vert:   mov     cx,1    ; slow movement
        cmp     dl,47h  ; if ( up left
        jz      up
        cmp     dl,48h  ;   or up
        jz      up
        cmp     dl,49h  ;   or up right ) then
        jz      up      ; goto move_cross-hair_up
        cmp     dl,4fh  ; if ( down left
        jz      down
        cmp     dl,50h  ;   or down
        jz      down
        cmp     dl,51h  ;   or down right ) then
        jz      down    ; goto move_cross-hair_down
;
        mov     cx,10h  ; fast movement
        cmp     dl,75h  ; if ( down left
        jz      down
        cmp     dl,76h  ;   or down right ) then
        jz      down    ; goto move_cross-hair_down
        cmp     dl,77h  ; if ( up left
        jz      up
        cmp     dl,84h  ;   or up right ) then
        jz      up      ; goto move_cross-hair_up
;
;           test if a horizontal movement is required
;
horiz:  mov     cx,1    ; slow movement
        cmp     dl,47h  ; if ( up left
        jz      left
        cmp     dl,4bh  ;   or left
        jz      left
        cmp     dl,4fh  ;   or down left ) then
        jz      left    ; goto move_cross-hair_left
        cmp     dl,49h  ; if ( up right
        jz      right
        cmp     dl,4dh  ;   or right
        jz      right
        cmp     dl,51h  ;   or down right ) then
        jz      right   ; goto move_cross-hair_right
;
        mov     cx,10h  ; fast movement
        cmp     dl,75h  ; if ( down left
        jz      left
        cmp     dl,73h  ;   or left
        jz      left
        cmp     dl,77h  ;   or up left ) then
        jz      left    ; goto move_cross-hair_left
        cmp     dl,76h  ; if ( down right
        jz      right
        cmp     dl,74h  ;   or right
        jz      right
        cmp     dl,84h  ;   or up right ) then
        jz      right   ; goto move_cross-hair_right
;
;           If arrow key is not found, then read keyboard again
;
no_move:
        jmp     char1
;
;           move cross-hair up
;
up:     call    near ptr xor_row    ; remove existing horizontal line
        sub     bx,cx               ; decrement position Y
        jge     up_ok               ; if ( Y < )
        xor     bx,bx               ;   Y = 0
up_ok:  call    near ptr xor_row    ; draw new horizontal line
        jmp     horiz               ; goto test for horizontal movement
;
;           move cross-hair down
;
down:   call    near ptr xor_row    ; remove existing horizontal line
        add     bx,cx               ; increment position Y
        cmp     bx,347              ; if (
        jle     down_ok             ;     Y > 347 )
        mov     bx,347              ;   Y = 347
down_ok: call   near ptr xor_row    ; draw new horizontal line
        jmp     horiz               ; goto test for horizontal movement
;
;           move cross-hair left
;
left:   call    near ptr xor_col    ; remove existing vertical line
        sub     ax,cx               ; decrement position X
        jge     left_ok             ; if ( X < )
        xor     ax,ax               ;   X = 0
left_ok: call   near ptr xor_col    ; draw new vertical line
        jmp     char1               ; goto get next character from keyboard
;
;           move cross-hair right
;
right:  call    near ptr xor_col    ; remove existing vertical line
        add     ax,cx               ; increment position X
        cmp     ax,719              ; if (
        jle     right_ok            ;      X > 719 )
        mov     ax,719              ;   X = 719
right_ok: call  near ptr xor_col    ; draw new vertical line
        jmp     char1               ; goto get next character from keyboard
xhair   endp
;
;
; xor_row - change every bit on the screen in row [bx]
;
xor_row proc   near
        push    cx
        push    di
;
;       calculate address of first byte in row
;
        mov     di,bx           ; get row number
        shr     di,1            ; row / 2
        shr     di,1            ; row / 4
        shl     di,1            ; ( row / 4 ) * 2
        mov     cx,di           ; store ( row / 4 ) * 2
        shl     di,1            ; ( row / 4 ) * 4
        shl     di,1            ; ( row / 4 ) * 8
        add     di,cx           ; ( row / 4 ) * 10
        shl     di,1            ; ( row / 4 ) * 20
        add     di,cx           ; ( row / 4 ) * 22
        shl     di,1            ; ( row / 4 ) * 44
        shl     di,1            ; ( row / 4 ) * 88
        add     di,cx           ; ( row / 4 ) * 90  end fast multiply by 90
        mov     cx,bx           ; get row number
        and     cx,3            ; row & 3
        ror     cx,1
        ror     cx,1
        ror     cx,1            ; ( row & 3 ) * 2000h
        add     di,cx           ; address of first byte in row
;
;           change every bit in the row
;
        mov     cx,45           ; number of words in a row
xr0:    xor     word ptr es:[di],0ffffh     ; switch one word
        inc     di
        inc     di              ; point to next word
        loop    xr0
        pop     di
        pop     cx
        ret
xor_row endp
;
;
;   xor_col - change every bit in column [ax]
;
xor_col proc near
        push cx
        push dx
        push di
;
;           calculate address of first byte in column
;
        mov     di,ax           ; get column number
        shr     di,1            ; col / 2
        shr     di,1            ; col / 4
        shr     di,1            ; col / 8
        mov     cx,ax
        and     cx,7
        mov     dx,80h          ; put a bit mask
        shr     dx,cl           ;   on the right bit
;
;           change every bit in the column
        mov     cx,87           ; number of rows/4
xc0:    xor     es:[di],dl
        xor     es:[di+2000h],dl
        xor     es:[di+4000h],dl
        xor     es:[di+6000h],dl
        add     di,90
        loop    xc0
;
        pop di
        pop dx
        pop cx
        ret
xor_col endp
;
;       TABLES & CONSTANTS
;   Char_map  :- address displacements for successive lines of a character
;                on the Hercules Graphics screen
; Control sequences for Hercules card controller
;   HCh_Parms :- put it into alpha-numeric mode
;   HGr_Parms :- put it into graphics mode
;
Char_map    dw      0000h, 2000h, 4000h, 6000h, 005Ah, 205Ah, 405Ah, 605Ah
HCh_Parms   db      61H, 50H, 52H, 0FH, 19H, 06H, 19H, 19H, 02H, 0DH, 0BH, 0CH
HGr_Parms   db      35H, 2DH, 2EH, 07H, 5BH, 02H, 57H, 57H, 02H, 03H, 00H, 00H
CON8        db      8
CON180      db      180
;
;       VARIABLES
;
prt_ptr     dw      0       ; temp storage for offset ptr to printer dump array
bmask       dw      -1      ; bit pattern mask for dashed line styles
colour      dw      -1      ; colour : -1=white, 0=black
xhair_x     dw      359     ; stored cross hair coordinate
xhair_y     dw      173
_text   ends
        end
