;    Copyright (C) 1997  Peter Vernon Lanspeary
;
;    This documentation is free; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This documentation is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this documentation; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

        page    58,132
        title   Lahey FORTRAN interface for communications ports
        name    io_f77l
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                              ;;
;;                  AUXILARY PORT I/O ROUTINES                  ;;
;;                                                              ;;
;;         These routines are for use by Lahey FORTRAN .        ;;
;;                  written by Peter Lanspeary                  ;;
;;                                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
Arg_Offset  equ     6       ; Argument offset for subroutine
Arg_Off_f   equ     10      ; Argument offset for function subprogram
Fun_Value   equ     6       ; Stack offset for the function value
;
io_code segment
        assume      cs:io_code
        public      IO_close, IO_gets,   IO_paral,  IO_puts
        public      IO_read,  IO_serial, IO_status, IO_write
;
;
;           DEFINITIONS FOR SERIAL PORTS
;           ****************************
; status :-
Hser_5_bits     equ     0   ; word length = 5 bits
Hser_6_bits     equ     1   ;             = 6 bits
Hser_7_bits     equ     2   ;             = 7 bits
Hser_8_bits     equ     3   ;             = 8 bits
Hser_1_stop     equ     0   ; number of stop bits = 1
Hser_2_stop     equ     4   ; number of stop bits = 2
Hser_nopar      equ     0   ; set no parity
Hser_odd        equ     8   ; set odd parity
Hser_even       equ     18h ; set even parity
Hser_DTRrx      equ     1   ; enable DTR input flow control
Hser_RTSrx      equ     2   ; enable RTS input flow control
Hser_XONrx      equ     4   ; enable XON/XOFF input flow control
Hser_noCTS      equ     10h ; don't require CTS to transmit
Hser_noDSR      equ     20h ; don't require DSR to transmit
Hser_19200      equ     40h ; set baud rate to 19200
Hser_38400      equ     80h ; set baud rate to 38400
Htime_out       equ     80h ; time out status returned from int 14h read
;
bauds   dw      110, 150, 300, 600, 1200, 2400, 4800, 9600, 38400, 19200
Hnbauds         equ     ( $ - bauds ) / 2
;
;
;           DEFINITIONS FOR PARALLEL PORTS
;           ******************************
;
;       Each  parallel port is one of two INTEL 8255A PPI chips on an 8255A
;   expansion card.  One or two expansion cards can be installed, giving up
;   to 4 effective 16-bit ports.
;       The 8255 PPI is configured as a 16 bit parallel  communications port
;   to SBC microcomputer.  Both ports A and B are used in Mode 1 with Port A
;   being for the low data  byte and  Port B for the  high  byte.  Only the
;   handshakes for Port B are used.                  ___      ___   ___
;       The handshake lines from the output port are OBFb and ACKb.  STBb and
;   IBFb are the handshake lines for the input port.  The parallel ports are
;   required to talk to INTEL 8255A PPIs, but unfortunately, the handshaking
;   hardware is basically incompatible.  These port driving routines therefore
;   synthesize the output handshake through port C bit 4.  This means that
;   when  connected as an output port, OBFb is not connected and Port C bit 4
;   is used instead.  When  connected as an input port, IBFb is not connected
;   and Port C bit 4 is used instead.  The parallel port driving routines do
;   not require any monostable multivibrator to be installed on the handshake
;   lines.
;
;
;       mode control word : components
;
Hppi_mode   equ 80h     ; mode set flag - this must be included
;                        
; Group 11 - a control word must have just 1 switch from this group
;Name         address  Port Group  Port   Used                Not used
Hppi_a_0    equ 00h     ; A        ---- select mode 0           --
Hppi_a_1    equ 20h     ; A        ---- select mode 1           --
Hppi_a_2    equ 40h     ; A        ---- select mode 2           --
; Group 12 - any number of these switches can be added into a control word
Hppi_a_in   equ 10h     ; A         A   input                   output
Hppi_chi_in equ 08h     ; A      C high input                   output
Hppi_clo_in equ 01h     ; B      C low  input                   output
Hppi_b_in   equ 02h     ; B         B   input                   output
Hppi_b_1    equ 04h     ; B        ---- select mode 1           select mode 0
;
;       mode control words : READ and WRITE respectively
;
Hpar_in_ctrl    equ Hppi_mode + Hppi_b_1 + Hppi_a_in + Hppi_b_in + Hppi_clo_in
Hpar_out_ctrl   equ Hppi_mode + Hppi_b_1 + Hppi_chi_in
;
;       bit control definitions
Hppi_int_enB    equ 5   ; control word to set INTE on port B
Hppi_C4_lo      equ 8   ; control word to reset bit 4 on port C
Hppi_C4_hi      equ 9   ; control word to set bit 4 on port C
;
;       status word bit select masks
Hppi_intr_B     equ 1   ; select INTR port B
Hppi_IBF_B      equ 2   ; select IBF port B
Hppi_OBF_B      equ 2   ; select OBF port B
;
;       parallel port addresses
Haddr_par1  equ 01b0h   ; 8255A PPI no. 1 base address
Haddr_par2  equ 01b4h   ; 8255A PPI no. 2 base address
Haddr_ctr1  equ 01b8h   ; 8253 counter no. 1 base address
Haddr_par3  equ 01f0h   ; 8255A PPI no. 1 base address
Haddr_par4  equ 01f4h   ; 8255A PPI no. 2 base address
Haddr_ctr2  equ 01f8h   ; 8253 counter no. 1 base address
;
Hpar_portA  equ 0       ; address offset for Port A
Hpar_portB  equ 1       ; address offset for Port B
Hpar_portC  equ 2       ; address offset for Port C
Hpar_control    equ 3   ; address offset for the control register
;
Hcntr_0     equ 0       ; address offset for counter 0
Hcntr_1     equ 1       ; address offset for counter 1
Hcntr_2     equ 2       ; address offset for counter 2
Hcntr_ctrl  equ 3       ; address offset for the counter control register
;
ppi_addr:   dw  Haddr_par1, Haddr_par2, Haddr_par3, Haddr_par4
;
;
;   ADAT: auxillary device allocation table
;   ***************************************
;
;   The ADAT is a data structure for defining the status of any auxillary port
;   which is used.  A used auxillary port occupies one of the 8 slots in the
;   ADAT.  Each ADAT slot is 10 bytes long.  It contains the :-
;       1) logical unit assigned to the device
;       2) device ( serial or parallel port ) number
;       3) closed, opened or error status
;       4) new, read or write direction flag
;       5) odd_byte_buffer - input status, input data, output status and
;            output data registers
;   Items 4 and 5 are only used if the defined device is a parallel port.
;   Item 4 is required because the port must be reconfigured if the
;   direction of data transfer changes.  Item 5 is required because each
;   parallel port is treated as 16 bit hardware device and as an 8 bit
;   software device.  Hence, when IO_READ reads a byte for the first time, 2
;   bytes are read from the hardware.  The odd_byte_buffer is used to
;   temporarily store the second of these bytes.  Reading an odd byte causes
;   a 16 bit hardware read.  Even bytes are obtained from the
;   odd_byte_buffer.  When writing a byte to a parallel port, every odd byte
;   is temporarily stored in the odd_byte_buffer, and is only written out to
;   the hardware when IO_WRITE writes the even byte.
;
;       constants used by the ADAT
;
Hno_devs    equ 8          ; maximum number of devices which can be open
Hclosed     equ 0           ; device is closed
Hopened     equ 1           ; device is opened
Herror      equ -1          ; device has an error condition
Hnul_dev    equ 0           ; There is no device in this slot
Hmax_ports  equ 255         ; maximum number of ports of any one kind
Hserial     equ 000ffh      ; mask for selecting serial communications port
Hparallel   equ 0ff00h      ; mask for selecting parallel communications port
Hprv_new    equ 0           ; no previous port access
Hprv_read   equ 1           ; last port access was a READ
Hprv_write  equ 2           ; last port access was a WRITE
Hobb_full   equ -1          ; there is a byte waiting to be processed in the
                            ;   odd_byte_buffer i.e. is full
Hobb_mt     equ 0           ; the odd_byte_buffer is empty
;
;       structure for an entry in the table
;
ADAT_slot  struc
        log_un dw ?         ; logical unit assigned to the device
        dev_no dw Hnul_dev  ; device number : low byte  = serial port number
                            ;               : high byte = parallel port number
        status db Hclosed   ; status = closed, opened or error
        direct db Hprv_new  ; direction flag : new, read or write
        obb_is db Hobb_mt   ; odd_byte_buffer - input status register
        obb_id db ?         ;                 - input data register
        obb_os db Hobb_mt   ;                 - output status register
        obb_od db ?         ;                 - output data register
ADAT_slot   ends
;
        even
ADAT    ADAT_slot     Hno_devs dup(<>)
adat_end:
;
;       end of ADAT
;
;
;       LOGICAL *1 FUNCTION IO_SERIAL( IUNIT, IPORT, IBAUD )
;
;   Open the serial communications port IPORT and assign to it the logical unit
;   number IUNIT.  The data transfer rate is IBAUD.  The port is opened in
;   `raw' mode with 1 start bit, 1 stop bit, 8 data bits, and no parity bit.
;   If the port is successfully opened, the returned result IO_SERIAL is
;   .true., otherwise IO_SERIAL = .false.
; Arguments
;  integer *2 iunit,        # logical unit number specifying the output device
;             iport,        # serial communications port number
;             ibaud         # baud rate . The allowed values are :-
;                               110, 300, 600, 1200, 2400, 4800, 9600
;
;   IO_SERIAL shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = IO_SERIAL returned value = .true. or .false.
; bx = IUNIT
; cx = IPORT
; dx = IBAUD
;   other register use
; es:di = argument address - only assigned immediately before use
;
IO_serial   proc    far
        push    bp
        mov     bp,sp
        les     di,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,es:[di]              ;   get logical unit number
        les     di,[bp+4].Arg_Off_f     ; pointer to 2nd arg, IPORT
        mov     cx,es:[di]              ;   get device port number
        les     di,[bp+8].Arg_Off_f     ; pointer to 3rd arg, IBAUD
        mov     dx,es:[di]              ;   get the baud rate
        call    near ptr aux_serial     ; work routine
        les     di,[bp].Fun_value       ; pointer to function value, IO_SERIAL
        mov     es:[di],al              ;   IO_SERIAL = ax
        pop     bp
        ret
IO_serial   endp
;
;       aux_serial
;       **********
;
;           check that the port number is legal
;
aux_serial  proc near
        test     cx,cx              ; if ( IPORT < 1
        jle     aux_serr            ;       or
        cmp     cx,Hmax_ports       ;      IPORT > max_port_no )
        jg      aux_serr            ;   error return
;
;           search the ADAT for the port number,
;             and clear any slots where it is found
; register use                in                  out
;  bx                       IUNIT               IUNIT
;  cx = ADAT slot counter   IPORT               0000
;  dx                       IBAUD               IBAUD
;  di                                           IPORT
;  si = ADAT slot pointer   anything      point at first byte past end of ADAT
;
        mov     di,cs               ; avoid all those cs: prefixes
        mov     ds,di
        mov     ax,bx               ; save IUNIT in ax
        mov     di,cx               ; save IPORT in di
        mov     bx, cx              ; get the port number
        mov     cx, Hno_devs        ; set up the ADAT slot counter
        lea     si,adat             ; si is pointer to ADAT ( 1st slot )
aux_ssrch:
        cmp     word ptr [si].dev_no,bx     ; if ( IPORT is in this slot ) then
        jnz     aux_snpn
        mov     word ptr [si].dev_no,Hnul_dev   ; clear the device number
        mov     byte ptr [si].status,Hclosed    ; clear the status
        mov     byte ptr [si].direct,Hprv_new   ; clear the direction flag
                                                ; end if
aux_snpn:
        add     si,size adat_slot   ; point to next ADAT slot
        loop    aux_ssrch           ; goto test the next ADAT slot
        mov     bx,ax               ; get IUNIT from stack
;
;           test that the specified baud rate is available
;               and compile port control word
;
; register use                in                  out
;  ax =                     anything        port control word
;  bx =                     IUNIT           IUNIT
;  cx = baud rate down ctr  0000            baud rate id
;  dx = read only           IBAUD           IBAUD
;  di  =                    IPORT           IPORT
;  si = baud rate array ptr anything      point at first byte past end of array
;
;           port control word without baud rate
        mov     ax,Hser_8_bits + Hser_1_stop + Hser_nopar
        mov     cx,Hnbauds          ; number of available baud rates
        lea     si,bauds            ; pointer to list of legal baud rates
aux_sbrl:
        cmp     [si],dx             ; if ( given baud rate is in table )
        jz      aux_sbrf            ;   goto baud_rate_found
        add     ax,20h              ; increment control byte for next baud rate
        inc     si                  ; point to next item in baud rate list
        inc     si
        loop    aux_sbrl            ; goto check the next available baud rate
;
;           error return - unrecognized baud rate
;
aux_serr:
        xor     ax,ax               ; IO_SERIAL = .false.
        ret
;
;           baud rate found - find empty slot in ADAT
;
; register use                in                  out
;  ax =                     port ctrl word  port ctrl word
;  bx = read only           IUNIT           IUNIT
;  cx = ADAT slot counter   baud rate id    baud rate id
;  dx = empty slot pointer  IBAUD           anything
;  di = read only           IPORT           IPORT
;  si = ADAT slot pointer   anything        anything
;  es = temporary           anything        baud rate id
;
aux_sbrf:
        mov     es,cx               ; save baud rate id in es
        mov     cx,Hno_devs         ; set ADAT slot counter
        lea     si,adat             ; si is pointer to ADAT
        lea     dx,adat_end         ; empty slot past end of ADAT
aux_stch:
        cmp     word ptr [si].log_un,bx     ; if ( found logical unit ) then
        jz      aux_sgslt                   ;   goto got_slot
        cmp word ptr [si].dev_no,Hnul_dev   ; if ( no device in this slot
        jnz     aux_sskp                    ;        and
        cmp     dx,offset adat_end          ;      first empty slot ) then
        jnz     aux_sskp
        mov     dx,si                       ; assign empty slot, end if
aux_sskp:
        add     si,size ADAT_slot           ; point to next ADAT slot
        loop    aux_stch                    ; goto test the next ADAT slot
;
        cmp     dx,offset adat_end          ; if ( no empty slot )
        jz      aux_serr                    ;   error return
;
;           found ADAT empty slot, set up device allocation table entry
;
        mov     si,dx                       ; put empty slot pointer into si
aux_sgslt:
        mov  word ptr [si].log_un,bx        ; ADAT entry : logical unit = IUNIT
        mov  word ptr [si].dev_no,di        ;            : IPORT is the device
        mov  byte ptr [si].status,Hopened   ;            : device is open
        mov  byte ptr [si].direct,Hprv_new  ;            : port not used
                                            ; odd_byte_buffer status registers
        mov  byte ptr [si].obb_is,Hobb_mt   ;            : input  - empty
        mov  byte ptr [si].obb_os,Hobb_mt   ;            : output - empty
        mov     cx,es                       ; recover baud rate id from es
;
;           send control word ( which is already in AL )
;               ( set up BIOS call )
; register use                in                  out
;  ax =                     port ctrl word  anything
;  bx =                     IUNIT           IUNIT
;  cx = baud rate id        baud rate id    19200/38400 selector
;  dx = port select         IBAUD           port select
;  di = read only           IPORT           IPORT
;
        mov     dx,di               ; get IPORT from di
        xor     ah,ah               ; function = initialize port
        dec     dx
        int     14h                 ; BIOS call
;
;           for baud rate of 19200 or 38400, use extended options BIOS call
;
        mov     ah,4                ; function = extended options
        mov     al,Hser_noCTS + Hser_noDSR  ; no handshake or XON/XOFF etc
        cmp     cx,2                ; if ( < 19200 baud )
        ja      aux_snfb            ;   skip the higher baud rate
        ror     cl,1                ; turn cl = 1 or 2 into 40h or 80h
        ror     cl,1                ;   for 19200 or 3800 baud respectively
        or      al,cl               ; add baud rate to extended options ctl word
aux_snfb:
        int     14h                 ; BIOS call
;
;           normal exit
;
aux_snex:
        mov     al,1                ; IO_SERIAL = .true.
        ret
aux_serial  endp
;
;
;
;       LOGICAL *1 FUNCTION IO_PARAL( IUNIT, IPORT )
;
;   Open the parallel communications port IPORT and assign to it the logical
;   serial number IUNIT.  If the port is successfully opened, the returned
;   result IO_PARAL is .true. , otherwise IO_PARAL = .false.
; Arguments
;  integer *2 iunit,        # logical unit number specifying the output device
;             iport,        # parallel communications port number
;
;
;   IO_PARAL shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = IO_PARAL returned value = .true. or .false.
; bx = IUNIT
; cx = IPORT
;   other register use
; es:di = argument address - only assigned immediately before use
;
IO_paral    proc    far
        push    bp
        mov     bp,sp
        les     di,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,es:[di]              ;   get logical unit number
        les     di,[bp+4].Arg_Off_f     ; pointer to 2nd arg, IPORT
        mov     cx,es:[di]              ;   get device port number
        call    near ptr aux_parallel   ; work routine
        les     di,[bp].Fun_value       ; pointer to function value, IO_PARAL
        mov     es:[di],al              ;   IO_PARAL = al
        pop     bp
        ret
IO_paral    endp
;
;       aux_parallel
;       ************
;
;           check that the port number is legal
;
aux_parallel  proc near
        test     cx,cx              ; if ( IPORT < 1
        jle     aux_perr            ;       or
        cmp     cx,Hmax_ports       ;      IPORT > max_no._of_ports )
        jg      aux_perr            ;   error return
;
;           search the ADAT for the port number,
;             and clear any slots where it is found
; register use                in                  out
;  bx                       IUNIT               IUNIT
;  cx = ADAT slot counter   IPORT               0000
;  di                                           IPORT
;  si = ADAT slot pointer   anything      point at first byte past end of ADAT
;
        mov     di,cs               ; avoid all those cs: prefixes
        mov     ds,di
        mov     ax,bx               ; save IUNIT in ax
        mov     di,cx               ; save IPORT in di
        mov     bx, cx              ; get the port number
        xchg    bh,bl               ; convert to ADAT port entry
        mov     cx, Hno_devs        ; set up the ADAT slot counter
        lea     si,adat             ; si is pointer to ADAT ( 1st slot )
aux_psrch:
        cmp     word ptr [si].dev_no,bx     ; if ( IPORT is in this slot ) then
        jnz     aux_pnpn
        mov     word ptr [si].dev_no,Hnul_dev   ; clear the device number
        mov     byte ptr [si].status,Hclosed    ; clear the status
        mov     byte ptr [si].direct,Hprv_new   ; clear the direction flag
                                                ; end if
aux_pnpn:
        add     si,size adat_slot   ; point to next ADAT slot
        loop    aux_psrch           ; goto test the next ADAT slot
        mov     bx,ax               ; get IUNIT from ax
;
;           find empty slot in ADAT
;
; register use                in                  out
;  ax =                     port ctrl word  anything
;  bx = read only           IUNIT           IUNIT
;  cx = ADAT slot counter   anything        anything
;  dx = empty slot pointer  anything        anything
;  di = read only           IPORT           IPORT
;  si = ADAT slot pointer   anything        anything
;
        mov     cx,Hno_devs         ; set ADAT slot counter
        lea     si,adat             ; si is pointer to ADAT
        lea     dx,adat_end         ; empty slot past end of ADAT
aux_ptch:
        cmp     word ptr [si].log_un,bx     ; if ( found logical unit ) then
        jz      aux_pgslt                   ;   goto got_slot
        cmp word ptr [si].dev_no,Hnul_dev   ; if ( no device in this slot
        jnz     aux_pskp                    ;        and
        cmp     dx,offset adat_end          ;      first empty slot ) then
        jnz     aux_pskp
        mov     dx,si                       ; assign empty slot, end if
aux_pskp:
        add     si,size ADAT_slot           ; point to next ADAT slot
        loop    aux_ptch                    ; goto test the next ADAT slot
;
        cmp     dx,offset adat_end          ; if ( no empty slot )
        jz      aux_perr                    ;   error return
;
;           found ADAT empty slot, set up device allocation table entry
;
        mov     si,dx                       ; put empty slot pointer into si
aux_pgslt:
        mov     dx,di                       ; swap bytes to convert
        xchg    dh,dl                       ;   IPORT into table entry
        mov  word ptr [si].log_un,bx        ; ADAT entry : logical unit = IUNIT
        mov  word ptr [si].dev_no,dx        ;            : IPORT is the device
        mov  byte ptr [si].status,Hopened   ;            : device is open
        mov  byte ptr [si].direct,Hprv_new  ;            : device is not used
                                            ; odd_byte_buffer status registers
        mov  byte ptr [si].obb_is,Hobb_mt   ;            : input  - empty
        mov  byte ptr [si].obb_os,Hobb_mt   ;            : output - empty
;
;           normal exit
;
        mov     al,1                ; IO_PARAL = .true.
        ret
;
;           error return
;
aux_perr:
        xor     ax,ax
        ret
aux_parallel    endp
;
;
;       INTEGER *2 FUNCTION IO_STATUS( IUNIT, IPORT, IDIRECT )
;
;   The returned value of IO_STATUS is the status of the I/O device associated
;   with logical unit IUNIT.
;
;   IO_STATUS :- -1 = error
;                0  = closed
;                1  = opened
;
; Arguments
;   integer *2 iunit    # logical unit number of the I/O device         INPUT
;   integer *2 iport    # port number of the I/O device .               OUTPUT
;                       #   - no port = 0
;                       #   - serial port, 0 < iport < 256
;                       #   - parallel port, port = iport / 256
;   integer *2 idirect  # direction flag :- 0 = not used,               OUTPUT
;                       #                   1 = read
;                       #                   2 = write
;
;
;   IO_STATUS shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = IO_STATUS returned value
; bx = IUNIT
; cx = IPORT
; dx = IDIRECT
;   other register use
; ds:di = argument address - only assigned immediately before use
;
IO_status   proc    far
        push    bp
        mov     bp,sp
        lds     di,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,ds:[di]              ;   get logical unit number
        call    near ptr aux_status     ; work routine
        lds     di,[bp+4].Arg_Off_f     ; pointer to 2nd arg, IPORT
        mov     ds:[di],cx              ;   put device port number
        lds     di,[bp+8].Arg_off_f     ; pointer to 3rd arg, IDIRECT
        mov     ds:[di],dx              ;   put direction
        lds     di,[bp].Fun_value       ; pointer to function value, IO_STATUS
        mov     ds:[di],ax              ;   IO_STATUS = ax
        pop     bp
        ret
IO_status   endp
;
;       aux_status
;       **********
;
aux_status  proc near
        mov     cx,Hno_devs                 ; set ADAT slot counter
        lea     si,adat                     ; pointer to 1st slot in ADAT
aux_tsrch:
        cmp     word ptr cs:[si].log_un,bx  ; if ( IUNIT is in this slot )
        jz      aux_tflun                   ;   goto found_logical_unit_number
        add     si,size adat_slot           ; point to the next ADAT slot
        loop    aux_tsrch                   ; goto test next ADAT slot
;
;           device never opened - error return
;
aux_terr:
        xor     dx,dx                       ; IDIRECT = never used
        xor     cx,cx                       ; IPORT = none
        xor     ax,ax                       ; IO_CLOSE = .false.
        ret
;
;           found logical unit number - get the entry in the ADAT
;
aux_tflun:
        mov     al,byte ptr cs:[si].direct  ; direction
        cbw
        mov     dx,ax
        mov     al,byte ptr cs:[si].status  ; status
        cbw
        mov     cx,word ptr cs:[si].dev_no  ; device number
        ret
aux_status  endp
;
;
;       LOGICAL *1 FUNCTION IO_CLOSE( IUNIT )
;
;   Close the auxillary I/O device associated with logical unit number IUNIT.
;   This routine only removes the relevant entry from the ADAT (Auxillary
;   Device Allocation Table).  It does not talk to the hardware.  If logical
;   unit number IUNIT is not found in the ADAT, the returned value of IO_CLOSE
;   is .false.
; Arguments
;   integer *2 iunit        # logical unit number of the I/O device
;
;   IO_CLOSE shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = IO_CLOSE returned value = .true. or .false.
; bx = IUNIT
;
IO_close  proc    far
        push    bp
        mov     bp,sp
        lds     si,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,ds:[si]              ;   get logical unit number
        call    near ptr aux_close      ; work routine
        lds     si,[bp].Fun_value       ; pointer to function value, IO_CLOSE
        mov     ds:[si],al              ;   IO_CLOSE = ax
        pop     bp
        ret
IO_close  endp
;
;       aux_close
;  register use for arguments :-
; ax = returned value = number of bytes sent
; bx = IUNIT
;
aux_close   proc near
        mov     cx,Hno_devs                     ; set ADAT slot counter
        lea     si,adat                         ; pointer to 1st slot in ADAT
aux_csrch:
        cmp     word ptr cs:[si].log_un,bx  ; if ( IUNIT is in this slot )
        jz      aux_cflun                   ;   goto found_logical_unit_number
        add     si,size adat_slot           ; point to the next ADAT slot
        loop    aux_csrch                   ; goto test next ADAT slot
;
;           device never opened - error return
;
aux_cerr:
        xor     ax,ax                           ; IO_CLOSE = .false.
        ret
;
;           found logical unit number - is it a parallel port ?
;
;
;           is a parallel port -
;           is there anything in the odd_byte_buffer output data register ?
;
;
;           OBB o/p data register is full - flush it .
;
;
;           close the entry in the ADAT
;
aux_cflun:
        cmp     byte ptr cs:[si].status,Hclosed     ; if ( device is already
        jz      aux_cerr                            ;   closed ) error return
        mov     word ptr cs:[si].dev_no,Hnul_dev    ; clear device number
        mov     byte ptr cs:[si].status,Hclosed     ; status = clear
        mov     byte ptr cs:[si].direct,Hprv_new    ; device not used
        mov     al,1                                ; IO_CLOSE = .true.
        ret
aux_close   endp
;
;
;       INTEGER *2 FUNCTION IO_WRITE( IUNIT, DATA, NBYTES )
;
;   Write NBYTES bytes of data from the array DATA to the device with logical
;   unit number IUNIT.  The returned value of the routine, IO_WRITE is the
;   number of bytes actually written to the device.  If a write error occurs,
;   the value of IO_WRITE is negative :-
;       IO_WRITE = -1   - device not opened
;       IO_WRITE = -2   - hard write error
; Arguments
;  integer *2 iunit,        # logical unit number specifying the output device
;  logical *1 data(nbytes)  # array of data bytes to be written
;  integer *2 nbytes        # number of data bytes to be written
;
;
;   IO_WRITE shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = returned value = number of bytes sent
; bx = IUNIT
; cx = NBYTES
; di = address offset for data array
; ds = addrress segment for data array
;
IO_write  proc    far
        push    bp
        mov     bp,sp
        lds     si,[bp+8].Arg_Off_f     ; pointer to 3rd arg, NBYTES
        mov     cx,ds:[si]              ;   get number of bytes to send
        lds     si,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,ds:[si]              ;   get logical unit number
        lds     di,[bp+4].Arg_Off_f     ; pointer to data array => ds:di
        call    near ptr aux_write      ; work routine
        lds     si,[bp].Fun_value       ; pointer to function value, IO_WRITE
        mov     ds:[si],ax              ;   IO_WRITE = ax
        pop     bp
        ret
IO_write  endp
;
;
;       INTEGER *2 FUNCTION IO_PUTS( IUNIT, CHARS )
;
;   Write the character string CHARS to the device with logical
;   unit number IUNIT.  The returned value of the routine, IO_PUTS is the
;   number of bytes actually written to the device.  If a write error occurs,
;   the value of IO_PUTS is negative :-
;       IO_PUTS = -1   - device not opened
;       IO_PUTS = -2   - hard write error
; Arguments
;  integer *2 iunit,        # logical unit number specifying the output device
;  character *1 chars*(*)   # character string to be written
;
;
;   IO_PUTS shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = returned value = number of bytes sent
; bx = IUNIT
; cx = length of character string
; di = address offset for data array
; ds = addrress segment for data array
;
IO_puts  proc    far
        push    bp
        mov     bp,sp
        lds     si,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,ds:[si]              ;   get logical unit number
        lds     si,[bp+4].Arg_Off_f     ; pointer to char string structure ds:di
        mov     di,[si]                 ; addr offset of char string
        mov     cx,[si+4]               ; get length of string
        mov     ds,[si+2]               ; addr segment of char string
        call    near ptr aux_write      ; work routine
        lds     si,[bp].Fun_value       ; pointer to function value, IO_PUTS
        mov     ds:[si],ax              ;   IO_PUTS = ax
        pop     bp
        ret
IO_puts  endp
;
;       aux_write
;  register use for arguments :-
; ax = returned value = number of bytes sent
; bx = IUNIT
; cx = NBYTES
; di = address offset for data array
; ds = address segment for data array
;
;  other register use
; ah = write_function & returned error status
; cx = ADAT slot counter
; si = ADAT slot pointer
;
aux_write   proc near
        push    cx                  ; save NBYTES on stack
;
;           search the ADAT for the logical unit number
;
        mov     cx, Hno_devs        ; set up the ADAT slot counter
        lea     si,adat             ; di is pointer to ADAT ( 1st slot )
aux_wsrch:
        cmp     word ptr cs:[si].log_un,bx  ; if ( IUNIT is in this slot )
        jz      aux_wflun           ;    goto found_logical_unit_number
        add     si,size adat_slot   ; point to next ADAT slot
        loop    aux_wsrch           ; goto test the next ADAT slot
;
;           device not opened or logical unit not in ADAT - error return
;
aux_werr1:
        pop     cx                  ; restore stack
        mov     ax,-1               ; IO_WRITE = -1
        ret
;
;           found logical unit number
;
aux_wflun:
        cmp     byte ptr cs:[si].status,Hopened   ; if ( device is not open )
        jnz     aux_werr1           ;   error return
;
;           device is open - is it a serial port ?
;
        pop     cx                  ; get NBYTES from stack
        push    cx                  ; retore stack pointer
        cld                         ; direction flag = autoincrement
        mov     bx,word ptr cs:[si].dev_no  ; get the device number from ADAT
        test    bl,bl               ; if ( device is not a serial port )
        jz      aux_wptst           ;   skip to parallel port test
;
;           device is a serial port
;
        call    near ptr serial_write   ; write stuff to serial port
        jmp     aux_wret
;
;           is the device a parallel port ?
;
aux_wptst:
        test    bh,bh               ; if ( device is not a parallel port )
        jz      aux_wnul            ;   skip to unidentified device
;
;           device is a parallel port - get its base address
;
        call near ptr paral_write   ; write stuff to parallel port
;
;           normal return
;
aux_wret:
        mov     byte ptr cs:[si].direct,Hprv_write  ; have just written to port
        jl      aux_werr2                           ; jump if hardware error
        pop     ax                                  ; get NBYTES from stack
        ret
;
;           unidentified device - device not found error return
;
aux_wnul:
        jmp     aux_werr1
;
;           hardware error return
;
aux_werr2:
        pop     cx                  ; balance the stack
        mov     ax,-2               ;   IO_WRITE = -2
        ret
aux_write   endp
;
;
;       INTEGER *2 FUNCTION IO_READ( IUNIT, DATA, NBYTES )
;
;   Reads NBYTES bytes of data from the device with logical unit number IUNIT
;   into the array DATA.  The returned value of the routine, IO_READ is the
;   number of bytes actually read from the device.  If a read error occurs, the
;   value of IO_READ is negative :-
;       IO_READ = -1   - device not opened
;       IO_READ = -2   - hard read error
;
;
;   IO_READ shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = returned value = number of bytes to be read
; bx = IUNIT
; cx = NBYTES
; di = address offset for data array
; ds = address segment for data array
;
IO_read     proc    far
        push    bp
        mov     bp,sp
        lds     si,[bp+8].Arg_Off_f     ; pointer to 3rd arg, NBYTES
        mov     cx,ds:[si]              ;   get number of bytes to receive
        lds     si,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,ds:[si]              ;   get logical unit number
        lds     di,[bp+4].Arg_Off_f     ; pointer to data array => ds:si
        call    near ptr aux_read       ; work routine
        lds     si,[bp].Fun_value       ; pointer to function value, IO_READ
        mov     ds:[si],ax              ;   IO_READ = ax
        pop     bp
        ret
IO_read     endp
;
;
;       INTEGER *2 FUNCTION IO_GETS( IUNIT, CHARS )
;
;   Reads characters from the device with logical unit number IUNIT into the
;   character string CHARS.  The returned value of the routine, IO_GETS is the
;   number of bytes actually read from the device.  If a read error occurs, the
;   value of IO_GETS is negative :-
;       IO_GETS = -1   - device not opened
;       IO_GETS = -2   - hard read error
;
;
;   IO_GETS shell for Lahey FORTRAN
;     The arguments ( or their pointers ) are passed to the work routine in :-
; ax = returned value = number of bytes to be read
; bx = IUNIT
; cx = length of character string
; di = address offset for data array
; ds = addrress segment for data array
;
IO_gets  proc    far
        push    bp
        mov     bp,sp
        lds     si,[bp].Arg_Off_f       ; pointer to 1st arg, IUNIT
        mov     bx,ds:[si]              ;   get logical unit number
        lds     si,[bp+4].Arg_Off_f     ; pointer to char string structure ds:di
        mov     di,[si]                 ; addr offset of char string
        mov     cx,[si+4]               ; get length of string
        mov     ds,[si+2]               ; addr segment of char string
        call    near ptr aux_read       ; work routine
        lds     si,[bp].Fun_value       ; pointer to function value, IO_PUTS
        mov     ds:[si],ax              ;   IO_PUTS = ax
        pop     bp
        ret
IO_gets  endp
;
;       aux_read
;  register use for arguments :-
; ax = returned value = number of bytes received
; bx = IUNIT
; cx = NBYTES
; di = address offset for data array
; es = address segment for data array
;
;  other register use
; ah = read_function & returned error status
; cx = ADAT slot counter
; si = ADAT slot pointer
;
aux_read    proc near
        push    cx                  ; save NBYTES on stack
;
;           search the ADAT for the logical unit number
;
        mov     cx, Hno_devs        ; set up the ADAT slot counter
        lea     si,adat             ; si is pointer to ADAT ( 1st slot )
aux_rsrch:
        cmp     word ptr cs:[si].log_un,bx  ; if ( IUNIT is in this slot )
        jz      aux_rflun           ;    goto found_logical_unit_number
        add     si,size adat_slot   ; point to next ADAT slot
        loop    aux_rsrch           ; goto test the next ADAT slot
;
;           device not opened or logical unit not in ADAT - error return
;
aux_rerr1:
        pop     cx                  ; restore stack
        mov     ax,-1               ; IO_READ = -1
        ret
;
;           found logical unit number, Check that the port is open
;
aux_rflun:
        cmp     byte ptr cs:[si].status,Hopened   ; if ( device is not open )
        jnz     aux_rerr1           ;   error return
;
;           device is open, decide what sort of device
;
        pop     cx                  ; get NBYTES from stack
        push    cx
        cld                         ; direction flag = autoincrement
        mov     bx,word ptr cs:[si].dev_no ; get the device number from ADAT
        test    bl,bl               ; if ( device is not a serial port )
        jz      aux_rptst           ;   goto parallel port test
;
;           device is a serial port, read from it
;
        call    near ptr serial_read    ; read stuff from serial port
        jmp     aux_rret
;
;           is the device a parallel port ?
;
aux_rptst:
        mov     dx,Hparallel        ; mask for parallel port selection
        test    bh,bh               ; if ( device is not a parallel port )
        jz      aux_rnul            ;   skip to unidentified device
;
;           device is a parallel port
;
        call    near ptr paral_read ; read stuff from parallel port
;
;           normal return
;
aux_rret:
        mov     byte ptr cs:[si].direct,Hprv_read  ; have just read from port
        pop     ax                  ; get IO_READ = NBYTES from stack
        mov     cx,16               ; this is a 35 micro second delay
aux_rdel:                           ;  it is required for some unknown reason
        loop    aux_rdel            ;  to avoid spurious random errors
        ret
;
;           device not found error return
;
aux_rnul:
        jmp     aux_rerr1
;
;           hardware error return
;
aux_rerr2:
        pop     cx                  ; balance the stack
        mov     ax,-2               ;   IO_READ = -2
        ret
aux_read    endp
;
;
;           BASE IO ROUTINES
;           ****************
;
; Register use :-               in                 out
;   f       -                 anything          +ve=OK, -ve=error
;   ah,al   - status & data
;   bx      -               device number
;   cx      -                  NBYTES           anything
;   dx      - port address
;   ds:di   -               ptr to start of data array
;   cs:si   -               ADAT pointer        ADAT pointer
;
;           serial read routine
;           *******************
;
serial_read     proc near
        mov     dx,bx
        dec     dx                  ; port no. => dx
sr_loop:
        mov     ah,2                ; set read function
        int     14h                 ; receive from port - BIOS call
        cmp     ah,Htime_out        ; if timeout try again
        jz      sr_loop
        test    ah,ah               ; if ( error condition )
        jl      sr_ret              ;   goto hardware error exit
        mov     byte ptr ds:[di],al ; put data byte in array
        inc     di                  ; point to next (empty) array byte
        loop    sr_loop             ; goto receive the next data byte
        xor     ax,ax               ; clear sign flag
sr_ret: ret
serial_read     endp
;
;
;           serial write routine
;           ********************
;
serial_write    proc near
        mov     dx,bx
        dec     dx                  ; port no. => dx
sw_loop:
        mov     al,byte ptr ds:[di] ; get data byte
        mov     ah,1                ; set write function
        int     14h                 ; send to port, BIOS call
        cmp     ah,Htime_out        ; if time out then try again
        jz      sw_loop
        test    ah,ah               ; if ( error condition )
        jl      sw_ret              ;   goto hardware error exit
        inc     di                  ; point to next data byte
        loop    sw_loop             ; goto send the next data byte
        xor     ax,ax               ; clear sign flag
sw_ret: ret
serial_write    endp
;
;
;           parallel read routine
;           *********************
;
;           device is a parallel port - get its base address
;
paral_read      proc near
        xchg    bh,bl                               ; switch bytes of device no.
        dec     bx                                  ; => port no.
        shl     bx,1
        mov     dx,word ptr cs:[bx].ppi_addr        ; get base addr of port
        add     dx,Hpar_portC                       ; PPI port C address
;
;           set up values for toggling synthetic handshake ( synthetic ACK )
;               bh = select Port C bit 4 high,  bl = select Port C bit 4 low
;
        mov     bx,Hppi_C4_hi *256 + Hppi_C4_lo
;
;           if previous access was not READ, send control word
;
        cmp     byte ptr cs:[si].direct,Hprv_read   ; if in write mode,
        jz      pr_start                            ;   skip initialization
        inc     dx                              ; point to control port address
        mov     al,Hpar_in_ctrl                 ; port control word
        out     dx,al                           ; send it
        mov     al,bh                           ; raise the
        out     dx,al                           ;   synthetic ACK
        dec     dx                              ; point DX at 8255 Port C
;
;           data reading loop - 225 clks = 28uSec / word
;
pr_start:
        mov     al,byte ptr cs:[si].obb_is  ; get OBB output status register
pr_loop:
        cmp     al,Hobb_full                ; if ( o/p OBB is full ) then
        jnz     pr_even
; ODD DATA BYTE
;                               get data from buffer - 56 clks
        mov     al,byte ptr cs:[si].obb_id  ;   get data from OBB output data
        mov     byte ptr ds:[di],al         ;    register & put it in data array
        inc     di                          ;   point to next array byte
        mov     al,Hobb_mt                  ;   obb output status = empty
        loop    pr_loop                     ;   goto send the next data byte
        mov     byte ptr cs:[si].obb_is,al  ;   assign OBB output status = empty
        ret
; EVEN DATA BYTE                            ; else
;                wait for `ready to receive' handshake - 28 clks per loop (16 )
pr_even:
                                            ; DX points at port C
        in      al,dx                       ;   read Port C status
        and     al,Hppi_IBF_B               ;   mask all but Port B IBF flag
        jz      pr_even                     ;     wait until intrB is high
;                               read low byte - 26 clks
        dec     dx
        dec     dx                          ;   point DX to 8255 port A
        in      al,dx                       ;   read byte from port A
        mov     byte ptr ds:[di],al         ;     and put it in data array
        inc     di                          ;   point to next array byte
;                               read high byte - 30 clks
        inc     dx                          ;   point DX at 8255 port B
        in      al,dx                       ;   read byte from port B
        mov     byte ptr cs:[si].obb_id,al  ;     & put it in OBB i/p data reg.
;                               send handshake ACK - 28 clks
        inc     dx
        inc     dx                          ;   point DX at 8255 port D
        mov     al,bl                       ;   lower the   ___
        cli                                 ;   disable interrupts
        out     dx,al                       ;     synthetic ACK
        mov     al,bh                       ;   raise the   ___
        out     dx,al                       ;     synthetic ACK
        sti                                 ;   enable interrupts
        dec     dx                          ;   point DX at 8255 port C
;                               21 clks
        mov     al,Hobb_full                ;   OBB output status = full
        loop    pr_loop                     ;   goto send the next data byte
        mov     byte ptr cs:[si].obb_is,al  ;   assign OBB output status = full
        xor     ax,ax                       ; clear sign flag
        ret
paral_read      endp
;
;
;           parallel write routine
;           **********************
;
;           device is a parallel port - get its base address
;
paral_write     proc near
        xchg    bh,bl                               ; switch bytes of device no.
        dec     bx                                  ; => port no.
        shl     bx,1
        mov     dx,word ptr cs:[bx].ppi_addr        ; get base addr of port
        add     dx,Hpar_portC                       ; PPI port C address
;
;           set up values for toggling synthetic handshake ( synthetic ACK )
;               bh = select Port C bit 4 high,  bl = select Port C bit 4 low
;
        mov     bx,Hppi_C4_hi *256 + Hppi_C4_lo
;
;           if previous access was not WRITE, send control word
;
        cmp     byte ptr cs:[si].direct,Hprv_write  ; if in write mode,
        jz      pw_start                            ;   skip initialization
        inc     dx                              ; point to control port address
        mov     al,Hpar_out_ctrl                ; port control word
        out     dx,al                           ; send it
        mov     al,bh                           ; raise the
        out     dx,al                           ;   synthetic STB
        dec     dx                              ; point DX at 8255 Port C
;
;           data writing loop    218 clks - 28 uSec/word
;
pw_start:
        mov     al,byte ptr cs:[si].obb_os  ; get OBB output status register
pw_loop:
        cmp     al,Hobb_mt                  ; if ( o/p OBB is empty ) then
        jnz     pw_even
; ODD DATA BYTE
;                               put data in buffer - 56 clks
        mov     al,ds:[di]                  ;   get data byte and put it in
        mov     byte ptr cs:[si].obb_od,al  ;     the OBB output data register
        inc     di                          ;   point to next data byte
        mov     al,Hobb_full                ;   OBB output status = full
        loop    pw_loop                     ;   goto send the next data byte
        mov     byte ptr cs:[si].obb_os,al  ;   assign OBB output status = full
        ret
; EVEN DATA BYTE                            ; else
pw_even:
;                               send low byte - 29 clks
;                                           ; DX points at Port C
        dec     dx
        dec     dx                          ; point DX at 8255 Port A
        mov     al,byte ptr cs:[si].obb_od  ;   get odd byte from OBB output
        out     dx,al                       ;     data register & send it
;                               send high byte - 25 clks
        inc     dx                          ;   point DX at 8255 port B
        mov     al,ds:[di]                  ;   get even byte from data array
        out     dx,al                       ;     & send it
        inc     di                          ;   point to next data byte
;                               send handshake STB - 28 clks
        inc     dx
        inc     dx                          ;   point DX at 8255 port D
        mov     al,bl                       ;   lower the   ___
        cli                                 ;   disable interrupts
        out     dx,al                       ;     synthetic STB
        mov     al,bh                       ;   raise the   ___
        out     dx,al                       ;     synthetic STB
        sti                                 ;   enable interrupts
        dec     dx                          ;   point DX at 8255 port C
;                           wait for `got data' handshake - 28 clks (16)
pw_shake:
        in      al,dx                       ;   read Port C status
        and     al,Hppi_OBF_B               ;   mask all but Port B OBF flag
        jz      pw_shake                    ;     wait until OBF is high
;                                       21 clks
        mov     al,Hobb_mt                  ;   OBB output status = empty
        loop    pw_loop                     ;   goto send the next data byte
        mov     byte ptr cs:[si].obb_os,al  ;   assign OBB output status = empty
        xor     ax,ax               ; clear sign flag
        ret
paral_write     endp
;
;
io_code ends
        end
