c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      SUBROUTINE SINIT( FSBCGO )
c
c       This is the INITIALIZE module for GENACQ
c Argument
c  logical *1 fsbcgo            # communications with the SBC have been started
c
c initialize module
c   open data file
c   open scratch file
c   start the SBC
c   set the ambient parameters
c   store module header record
c   store block header record(s)
c
c  logical *1 ok,               # success / failure result
c             foldq,            # old value of fquiet
c             daflush,          # function : flush data file buffer
c             daopen,           #          : open data file
c             daput,            #          : write items to data file
c             daputc,           #          : write char string to data file
c             Emgwrt,           #          : write message
c             Equery,           #          : logical response
c             Existf            #          : file exists
c  integer *2 i,                # conversion loop index
c             ipdfn,            # iput device for data file name
c             lencmt,           # total length of comments
c             lenstr,           # length of a comment line
c             jljstr            # function : left justify string
c  real  Rapbar,                # barometric pressure ( pascals )
c        Raptem,                # ambient temperature ( Kelvin )
c        Rpcvt(4),              # pressure units conversion factors
c        Rpmax(4),              # max. pressure units change triggers
c        Qresp                  # function : accept & check real menu response
c  character Cmt*80             # comment line
c  subroutine datstr,           # get time and date
c             sbinit,           # start communications with SBC
c             sipf2d,           # disable access to secondary I/P file
c             smgwrt,           # write message to file
c             smmhdr,           # write module header record
c             sopwrt,           # write item to primary output file
c             sopwrtc,          # write character string to primary output file
c             ssipf2            # select entry point to secondary I/P file
c Messages
c  21000 - data file name ?
c  21001 - barometric pressure ?
c  21002 - ambient temperature ?
c  21003 - comments ?
c  21004 - data file already exists
c
      implicit none
      include 'genacq.kon'
c
      logical *1 ok, daflush, daopen, daput, daputc, Equery, Existf,
     1  emgwrt, fsbcgo, foldq
      integer *2 i, ipdfn, lencmt, lenstr, jljstr
      real  Rapbar, Raptem, Rpcvt(4), Rpmax(4), Qresp
      character Cmt*80
      include 'genacq.cmn'
c
c                  mBars  Torr     kPa     ins Hg
      data Rpmax / 850.0, 120.0,   35.0,   25.0 /
      data Rpcvt / 100.0, 133.322, 1000.0, 3386.379 /
c
      nmodul = 1
      do 10 i = 1, 255
10      dirrec(i) = 0
      fdef1 = .false.
      call ssipf2( '##init', ' ' )
c
c       open the data file
c
      ipdfn = iipf1
      foldq = fquiet
c     repeat                                                              1b
1        call smgwrt( 21000 )
        read( ipdfn, 2 ) Cdfnam
2       format( a )
        fquiet = .false.
        ipdfn = Hkbdlu
        if ( jljstr( Cdfnam ) .le. 0 ) goto 1
        if ( Existf( Cdfnam ) ) then                                      2b
          if ( .not. emgwrt( Hvdulu, 21004 + ipropt ) )
     1      call sfail( Hmsgerr )
          if ( .not. Equery( Hkbdlu, 'OK ?' ) )
     1      goto 1
          end if                                                          2e
c       until( New file )                                                 1e
      fquiet = foldq
      call sopwrtc( Charnb( Cdfnam ) )
      if ( fenxeq ) then                                                  1b
c
c           close the data file if it was opened
c
        if ( fdfopn ) then                                                2b
          close( UNIT=Hdatlu )
          fdfopn = .false.
          end if                                                          2e
        fdfopn = daopen( Hdatlu, Cdfnam, 512 )
        if ( .not. fdfopn ) goto 1
c       until( data file is opened )
c
c       start SBC
c
        if ( .not. fsbcgo ) then                                          2b
          call sbinit( .false., ' ' )
c
c only used when parallel port is defunct
c         call sppdis( .true. )
          fsbcgo = .true.
          end if                                                          2e
        end if                                                            1e
c
c       get ambient parameters
c
      dtnbyt = 0
      call datstr( Cdate )
c
c        barometric pressure
c
      call smgwrt( 21001 )
      Rapbar = Qresp( Iipf1, 25.0, 1100.0, Hntries )
      call sopwrt( Rapbar, Hreal )
      i = 1
3     if ( Rapbar .gt. Rpmax(i) ) then                                    1b
        Rapbar = Rapbar * Rpcvt(i)
      else                                                                1m
        i = i + 1
        goto 3
        end if                                                            1e
      write( Hvdulu, 4 ) Rapbar
4     format( ' = ', f7.0, ' Pascals' )
c
c       ambient temperature
c
      call smgwrt( 21002 )
      Raptem = Qresp( iipf1, 5.0, 106.0, Hntries )
      call sopwrt( Raptem, Hreal )
      if ( Raptem .gt. 41.0 )
     1  Raptem = ( Raptem - 32.0 ) * ( 5.0 / 9.0 )
      Raptem = Raptem + 273.15
      write( Hvdulu, 5 ) Raptem
5     format( ' = ', f6.2, ' Kelvin' )
c
c       Comments : written onto Hscrlu
c
      open( UNIT=Hscrlu, ACCESS='sequential', FORM='formatted',
     1  STATUS='unknown' )
      lencmt = 0
      call smgwrt( 21003 )
c     start loop                                                          1b
6       read( Iipf1, 2 ) Cmt
        lenstr = nblank( Cmt )
        call sopwrtc( Cmt(1:lenstr) )
        if ( lenstr .eq. 1 .and. Cmt(1:1) .eq. '.' )
     1    goto 7
        write( Hscrlu, 2 ) Cmt
        lencmt = lencmt + lenstr + 1
        goto 6
c       end loop                                                          1e
7     call sipf2d
c
c       write module header record
c
      if ( fenxeq ) then                                                  1b
        irec = 2
        call smmhdr( '#initial' )
c
c       write block header record
c
c   label
        ok = daputc( irec, '##init  ' )
c   number of bytes
        ok = ok .and. daput( irec, 74 + lencmt, 2 )
c   data file name
        ok = ok .and. daputc( irec, Cdfnam )
c   date
        ok = ok .and. daputc( irec, Cdate )
c   ambient pressure
        ok = ok .and. daput( irec, Rapbar, 4 )
c   ambient temperature
        ok = ok .and. daput( irec, Raptem, 4 )
c   number of comment characters - including terminating nulls
        ok = ok .and. daput( irec, lencmt, 2 )
c
c       write comment onto block header
c
        if ( lencmt .gt. 0 ) then                                         2b
          rewind( UNIT=Hscrlu )
c         start of loop                                                   3b
8           read( Hscrlu, 2, END=9, ERR=9 ) Cmt
            ok = ok .and. daputc( irec, Charnb( Cmt ) )
            ok = ok .and. daput( irec, 0, 1 )
            goto 8
c           end of loop                                                   3e
          end if                                                          2e
9       ok = ok .and. daflush(irec)
        if ( .not. ok )
     1    call sfail( Hblkerr )
        end if                                                            1e
      close( UNIT=Hscrlu, STATUS='delete' )
      return
      end


      SUBROUTINE SCALIB
c
c       This is the CALIBRATE module for genacq
c
c Variables
c  logical *1 fbar,             # calibrate baratron flag
c             eask              # function : logical response
c  integer *2 idev,             # device to be calibrated
c             Jresp             # function : integer response
c  subroutine scalbar,          # calibrate baratron
c             scalhw1,          # calibrate single hot wire
c             scaltvs,          # calibrate traverse
c             smgwrt,           # write message
c             sopwrt            # write item to primary output file
c Messages
c  22000 - what device to calibrate ?
c
      implicit none
      include 'genacq.kon'
c
      logical *1 fbar, eask
      integer *2 idev, Jresp
      include 'genacq.cmn'
c
c         Make the choice of device to be calibrated
c
      fbar = .false.
c     start of loop                                                       1b
1       call smgwrt( 22000 )
        idev = Jresp( Iipf1, Hexit, Hndev, Hntries )
        call sopwrt( idev, Hint2 )
        if ( idev .eq. Hexit ) then
          return
c
c           traverse
c
        else if ( idev .eq. Htraverse ) then                              2b
          call scaltvs
c
c           baratron
c
        else if ( idev .eq. Hbarocel ) then                               2m
          call scalbar
          fbar = .true.
c
c           single hot wire
c
        else if ( idev .eq. H1hwire ) then                                2m
          if ( .not. fbar ) then                                          3b
            fbar = eask( 'Calibrating baratron / barocel ?' )
            call sopwrt( fbar, Hlog1 )
            if ( fbar )
     1        call scalbar
            end if                                                        3e
          call scalhw1
        else if ( idev .eq. Hxhwire ) then
          call scalxhw
c
c           additional devices
c
c       else if ( ..........                                              2m
c         put any additional devices here
          end if                                                          2e
        goto 1
c       end of loop                                                       1e
      end


      SUBROUTINE SACQU
c
c       ***********************************
c       **                               **
c       **      ACQUIRE DATA MODULE      **
c       **                               **
c       ***********************************
c
c        Data acquisition module for GENACQ
c
c  acquire : set preliminary block parameters
c            set main block parameters
c            get preliminary data block
c            multiple do loop nest
c               get main data block
c
c set block parameters :              get data block
c   pause sub-block ?                   pause sub-block ?
c     A/D id                              no A/D ?
c       cycle time                          keyboard pause
c       no. of cycles                     else
c       A/D channel                         repeat
c       automatic monitoring ?                Hslo A/D or Hsan A/D
c         monitor voltage                       get data into array
c         monitor tolerance                     read & average the samples
c     Hseq,Hsan,Hsim : unpack data
c     Hslo,Hseq,Hsan : cycle time
c   traverse sub-block ?
c   traverse sub-block ?
c     traverse identifier
c     required coordinates                    display average
c   secondary data sub-block ?                automatic monitor ?
c     A/D id                                    test average => ok
c     cycle length                            else
c     Hseq : sampling period                    request => ok
c     Hseq,Hsan,Hsim : unpack data ?        until ok
c     Hslo,Hseq,Hsan : cycle time         store details on data file
c     number of cycles                  traverse sub-block ?
c     Hslo,Hsan : channel address list    drive traverse
c   primary data sub block ?              read traverse
c     A/D id                              store details on data file
c     cycle length                      preliminary data sub-block ?
c     Hseq : sampling period              get data onto data file
c     Hslo,Hseq,Hsan : cycle time         store details on data file
c     number of cycles                  main data sub-block ?
c     Hslo,Hsan : channel address list    get data onto data file
c     Hseq,Hsan,Hsim : unpack data        store details on data file
c
c
c
c  logical *1 fblk,             # this block is used
c             facq,             # data is acquired
c             foldq,            # old value of fquiet
c             fsetpar,          # the acquisition parameters must be set
c             eask              # function : logical response
c  character Cblk(3)*11         # block type
c  subroutine sdo,              # executes nested do loops
c             setdbk,           # set parameters for a data block
c             setofm,           # set array offsets for main data block
c             setofp,           # set array offsets for preliminary data block
c             sgtdat,           # get data
c             smmhdr,           # write module header record to data file
c             sopwrt,           # writes a value to primary output file
c
      implicit none
      include 'genacq.kon'
c
      logical *1 fblk, facq, foldq, fsetpar, eask, equery
      character Cblk(3)*11
      include 'genacq.cmn'
      include 'module.cmn'
      data Cblk / 'preliminary', 'main       ', 'extra      ' /
c
      fdef1 = .false.
      fsetpar = .true.
      if ( ( Imode .eq. Hmanual .or. Imode .eq. Hcopy .or. fmdxeq )
     1    .and. frepen ) then                                                            1b
        fsetpar = .not. Equery( Hkbdlu,
     1    'Repeat acquisition with same parameters ?' )
        end if                                                            1e
      if ( fsetpar ) then                                                 1b
c
c       get parameters for each data block
c
        facq = .false.
        do 1 iblock = 1, Hnblks                                           2b
          nvars(iblock) = 0
          fblk = eask( 'Are you using the ' // Charnb( Cblk(iblock) )
     1      // ' data block ?' )
          fblock(iblock) = fblk
          call sopwrt( fblk, Hlog1 )
          facq = facq .or. fblk
          if ( fblk ) then                                                3b
            call setdbk
            end if                                                        3e
1         continue                                                        2e
          end if                                                          1e
c
c       write module header record
c
      if ( facq .and. fenxeq ) then                                       1b
        nmodul = nmodul + 1
        foldq = fquiet
        fquiet = .false.
        call smmhdr( '#acquire' )
c
c       get data if required
c
        call sgtdat
        fquiet = foldq
        end if                                                            1e
      frepen = .true.
      return
      end


      SUBROUTINE SDISP
c
c    Display module for genacq
c
c constants
c   Hndat=16384                # maximum number of data points in a window
c   Hlabel='SDISP'             # routine label
c variables
c  logical *1 fcapt            # capture plot file used flag
c             fchange          # change sub-block header flag
c             fdata            # display data sub-block flag
c             fdump            # dumping data onto file
c             frdonly          # read only data file open
c             fsetup           # change setup
c             daopen           # function : open direct access file
c             daget            #          : read direct access file
c             Eask             #          : logical response
c             equery           #          : logical response
c             emgwrt           #          : write message
c             erdfacq          #          : read acquisition sub-block
c             existf           #          : file exists ?
c  integer *2 idirptr          # directory pointer = module number
c             ierr             # error status for opening dump file
c             irdfrec          # data file record pointer
c             irdfdmp          # data file record pointer used by dumper
c             iresrec          # record no.- reserved if data may follow
c             isetup           # set selection
c             Mdata(Hndat)     # data array
c             nwindow          # total data window length
c             Jextmdln         # function : record no. of ith module
c             jljstr           #          : left justify string
c             Jresp            # f77lib : integer response
c  integer *4 dnsamp           # dummy number of samples
c             itnbyt           # total no. of data bytes - close block
c  real rapbar                 # barometric pressure
c       raptem                 # ambient temperature
c       rchsiz                 # relative graphics lettering size
c       Qresp                  # function : real response
c  character Cmhdlab *8        # module header record label
c            Csublab *8        # sub-block header label
c            Cdump *32         # dump file name
c  subroutine srdftvs          # display traverse calibration
c             sextall          # display all Acquire sub-block header
c             smgwrt           # write message
c             sopwrt           # write item to primary O/P file
c             sopwrtc          # write char string to primary O/P file
c             ssdisp           # display data
c             srdfxxx          # edit sub-block header
c             srdfmhd          # get next module header
c             srdfinit         # display initialize block header
c             srdfclos         # display close block header
c             srdferr          # display error message
c
      implicit none
      integer *2 Hndat
      character Hlabel
      parameter( Hlabel='SDISP', Hndat=16384 )
      include 'genacq.kon'
c
      logical *1 fcapt, fchange, fdata, fdump, frdonly, fsetup,
     1  daopen, daget, Eask, equery, emgwrt, erdfacq, Existf
      integer *2 idirptr, ierr, irdfrec, irdfdmp, iresrec, isetup,
     1  Mdata(Hndat), nwindow, Jextmdln, jljstr, Jresp
      integer *4 dnsamp, itnbyt
      real rapbar, raptem, rchsiz, Qresp
      character Cmhdlab *8, Csublab *8, Cdump *32
      include 'genacq.cmn'
c
c Messages
c  24000 - Display module
c  24001 - Which module ?
c  24002 - Dump file name ? .
c  24003 - Lettering size ? .
c  24004 - samples/window ?
c  24005 - Data file name ?
c  24006 - Can't find data file
c
c           initialize
c
      call smgwrt( 24000 )
      frdonly = .false.
      fdata = .true.
      fcapt = .false.
      fdump = .false.
      Cdump = ' '
      rchsiz = 1.0
      nwindow = Hndat / 4
c
c           open a read only data file if one is not already open
c
      if ( .not. fdfopn .and. fenxeq ) then                                 1b
        if ( .not. emgwrt( Hvdulu, 24005 + ipropt ) )
     1    call sfail( Hmsgerr )
        read( Hkbdlu, 3 ) Cdfnam
        if ( existf( Cdfnam ) ) then                                        2b
          frdonly = daopen( Hdatlu, Cdfnam, 512 )
          irec = 1
          frdonly = daget( irec, nmodul, 2 )
          frdonly = frdonly .and. daget( irec, dirrec, 510 )
          irec = 32767
        else                                                                2m
          if ( .not. emgwrt( Hvdulu, 24006 + ipropt ) )
     1      call sfail( Hmsgerr )
          end if                                                            2e
        if ( .not. frdonly )
     1    return
        end if                                                              1e
c
c           ADJUST DISPLAY CHARACTERISTICS
c           ******************************
c
      fsetup = Eask( 'Do you want to change display setup ?' )
      call sopwrt( fsetup, Hlog1 )
      if ( fsetup ) then                                                   1b
c       start of loop                                                      2b
1         if ( .not. fquiet )
     1      write( Hvdulu, 2 ) fdata, fcapt, fdump, Cdump, rchsiz,
     2        nwindow
2         format( ' Select set up value' / '  0) Exit' /
     1            '  1) display data ?', t27, l1 /
     2            '  2) capture plots ?', t27, l1 /
     3            '  3) dump data ?', t27, l1, t35, a /
     4            '  4) lettering size =', t25, f4.2 /
     5            '  5) window length =', t24, i5, '   ' )
          isetup = Jresp( Iipf1, 0, 5, Hntries )
          call sopwrt( isetup, Hint2 )
          if ( isetup .eq. Hexit ) then                                    3b
            goto 4
          else if ( isetup .eq. 1 ) then                                   3m
c
c             display data ?
c
            fdata = Eask( 'Are you displaying data ?' )
            call sopwrt( fdata, Hlog1 )
          else if ( isetup .eq. 2 ) then                                   3m
c
c             capture plots ?
c
            fcapt = Eask( 'Will you capture any plots ?' )
            call sopwrt( fcapt, Hlog1 )
          else if ( isetup .eq. 3 ) then                                   3m
c
c             dump data ?
c
            call smgwrt( 24002 )
            read( Iipf1, 3 ) Cdump
3           format( a )
            call sopwrtc( Cdump )
            if ( fdump )
     1        close( UNIT=Hdmplu )
            fdump = .false.
            if ( jljstr( Cdump ) .gt. 0 ) then                             4b
              open( UNIT=Hdmplu, FILE=Cdump, ACCESS='append',
     1          FORM='formatted', STATUS='unknown', IOSTAT=ierr )
              fdump = ierr .eq. 0
              end if                                                       4e
          else if ( isetup .eq. 4 ) then                                   3m
c
c             change character size
c
            call smgwrt( 24003 )
            rchsiz = Qresp( Iipf1, 0.2, 5.0, Hntries )
            call sopwrt( rchsiz, Hreal )
          else if ( isetup .eq. 5 ) then                                   3m
c
c             change window length
c
            call smgwrt( 24004 )
            nwindow = Jresp( Iipf1, 32, Hndat, Hntries )
            call sopwrt( nwindow, Hint2 )
            end if                                                         3e
            goto 1
c         end of loop                                                      2e
        end if                                                             1e
c
c           specify what module headers to display
c
c     start of loop                                                        1b
4       call smgwrt( 24001 )
        idirptr = Jresp( Iipf1, -nmodul, nmodul, Hntries )
        call sopwrt( idirptr, Hint2 )
        fchange = idirptr .lt. 0
        idirptr = abs( idirptr )
        if ( idirptr .le. 0 ) then                                         2b
          if ( frdonly )
     1      close( UNIT=Hdatlu )
          return
          end if                                                           2e
        if ( .not. fenxeq ) goto 4
        write( Hvdulu, 5 ) idirptr
5       format( / ' Module Number ', i4 )
        if ( fdump )
     1    write( Hdmplu, 5 ) idirptr
c
c           display module header and then block headers
c
        irdfrec = Jextmdln( idirptr )
        irdfdmp = irdfrec
        call srdfmhd( Hvdulu, Cmhdlab, irdfrec )
        if ( fdump )
     1    call srdfmhd( Hdmplu, Cmhdlab, irdfdmp )
        irdfdmp = irdfrec
        if ( Cmhdlab .eq. '#initial' ) then                                2b
c
c           initialize block header
c
          call srdfinit( Hvdulu, Rapbar, Raptem, irdfrec )
          if ( fdump )
     1      call srdfinit( Hdmplu, Rapbar, Raptem, irdfdmp )
        else if ( Cmhdlab .eq. '#caltvs' ) then                            2m
c
c           traverse calibration block
c
          call srdftvs( Hvdulu, irdfrec )
          if ( fdump )
     1      call srdftvs( Hdmplu, irdfdmp )
        else if ( Cmhdlab .eq. '#acquire' .or.
     1            Cmhdlab .eq. '#calbar ' .or.
     2            Cmhdlab .eq. '#calhw1 ' .or.
     3            Cmhdlab .eq. '#calxhw ' )then                            2m
c
c           acquisition block
c
          iresrec = irdfrec
6         if ( irdfrec .ge. irec ) goto 4
            if ( erdfacq( Csublab, Dnsamp, Mdata, 0, irdfrec, .false. )
     1          ) then                                                     3b
            if ( fchange ) then                                            4b
              call srdfxxx( iresrec )
            else                                                           4m
              call sextall( Hvdulu )
              if ( fdump )
     1          call sextall( Hdmplu )
              if ( fdata .and. ( Csublab .eq. '###1dat1' .or.
     1             Csublab .eq. '###2dat2' ) ) then                        5b
                if ( Equery( Hkbdlu,
     1              'Do you want to display data ?' ) ) then               6b
                  irdfrec = iresrec
                  call ssdisp( fcapt, fdump, rchsiz, Cmhdlab, nwindow,
     1              irdfrec )
                  end if                                                   6e
                end if                                                     5e
              end if                                                       4e
            iresrec = irdfrec
            goto 6
            end if                                                         3e
c
c           close data block header
c
        else if ( Cmhdlab .eq. '#close  ' ) then                           2m
          call srdfclos( Hvdulu, itnbyt, irdfrec )
          if ( fdump )
     1      call srdfclos( Hdmplu, itnbyt, irdfdmp )
        else                                                               2m
          call srdferr( 'SDISP', Hrdfimod )
          end if                                                           2e
        goto 4
c       end of loop                                                        1e
      end


      SUBROUTINE SSDISP( FCAPT, FDUMP, RCHSIZ, CMHDLAB, NWINDOW,
     1  IRDFREC )
c
c
c           display data  block
c           *******************
c       All GENACQ graphics is done in this routine.
c
c Arguments
c  logical *1 fcapt            # capture plot file used flag
c             fdump            # dumping data onto file
c  real rchsiz                 # relative graphics lettering size
c  character Cmhdlab *8        # module header record label
c  integer *2 irdfrec          # data file record pointer
c
c constants
c   Hndat=8192                 # maximum number of data points in a window
c   Hlabel='SDISP'             # routine label
c   Hrow=42                    # row for message on graphics screen
c   Hcol=27                    # start column for message on graphics screen
c Variables
c  logical *1 fhead            # sub-block label dumped onto file
c  logical *1 equery           # logical response
c             erdfacq          # read acquisition sub-block
c  integer *2 i                # looping index
c             iplot            # 'iplot'th graph on page
c             ix               # data point number
c             Mdata(Hndat)     # data array
c             nplots           # number of graphs to be put on page
c             npts             # number of data points to plot
c             nwindow          # total data window length
c             nxplts           # page width in cells
c             nxaxis           # x axis type
c             nyplts           # page height in cells
c             nypos            # vertical position of graph on page
c             Jextchls         # chan addr list
c             Jextcyln         # no. of chans
c  integer *4 dnsamp           # dummy number of samples
c             dtnsamp          # total number of samples read from 1 sub-block
c             dnocy            # number of data acquisition cycles
c             dextnocy         # function : no of cycles
c             dsize            # max allowed no. of samples
c             dnskip           # number of data words to skip
c             dpresp           # f77lib : integer response
c  real rcytim                 # A/D cycle time
c       rsamtim                # A/D sampling period
c       rtime                  # cycle start time
c       rtimoff                # time offset for data
c       rx(Hndat),ry(Hndat)    # (x,y) data points to be plotted
c  real Qextcytm               # rdflib : get A/D cycle time
c       Qextsamt               #              A/D sampling period
c  character Csublab *8        # sub-block header label
c            Ctitle *23        # graph title
c            Gextadid*12       # rdflib : get A/D identity
c  subroutine ilgchar          # select graphics char size
c             ilgclos          # terminate graphics
c             ilgdata          # plot data
c             ilgopen          # ilglib : start graphics
c             ilgpage          # page & no. of cells
c             ilgraph          # cell no. & axis types
c             ilgtitl          # title on graph
c             ilgwait          # wait for user
c             ilgxlab          # x axis label
c             srdferr          # display error message
c             HG_puts          # put string on graphics screen
c
      implicit none
      integer *4 Hndat
      integer *2 Hrow, Hcol
      character Hlabel *5
      parameter( Hndat=8192, Hrow=42, Hcol=27, Hlabel='SDISP' )
      include 'genacq.kon'
c
      logical *1 fcapt, fdump, fhead, equery, erdfacq
      integer *2 i, iplot, irdfrec, ix, Mdata(Hndat),
     1  nplots, npts, nwindow, nxplts, nxaxis, nyplts, nypos,
     2  Jextchls, Jextcyln
      integer *4 dnsamp, dtnsamp, dnocy, dextnocy, dsize, dnskip, dpresp
      real rchsiz, rcytim, rsamtim, rtime, rtimoff, rx(Hndat),
     1  ry(Hndat), Qextcytm, Qextsamt
      character Cmhdlab *8, Csublab *8, Ctitle *23, Gextadid*12
c
c           select number of graphs on page & sample/cycle times
c
      fhead  = .false.
      nplots = Jextcyln()
      rcytim = Qextcytm()
      rsamtim = Qextsamt()
      rtimoff = 0.0
      dnocy = dextnocy()
      Dtnsamp = 0
      nxplts = ( nplots - 1 ) / 5 + 1
      nyplts = ( nplots + nxplts - 1 ) / nxplts
      dsize = nwindow / nplots * nplots
c
c           START OF LOOP TO DISPLAY ONE WINDOW ON THE DATA
c           ***********************************************
c
c           SKIP SOME OF THE DATA - IF REQUIRED
c
c     repeat                                                               1b
1       if ( dsize .gt. 0 ) then                                           2b
          if ( equery( Hkbdlu, 'Skip ?' ) ) then                           3b
            write( Hvdulu, 2 )
2           format( ' How many samples ? ' )
            dnskip = dpresp( Hkbdlu, 0, dnocy - Dtnsamp / nplots,
     1        Hntries ) * nplots
            if ( dnskip .gt. 0 ) then                                      4b
              Rtimoff = Rtimoff + dnskip / nplots * rcytim
c             repeat                                                       5b
3               if ( .not. erdfacq( Csublab, Dnsamp, Mdata,
     1              min( Hndat, dnskip ), irdfrec, .true. ) )
     2            call srdferr( Hlabel, Hrdfueof )
                Dtnsamp = Dtnsamp + Dnsamp
                dnskip = dnskip - Dnsamp
                if ( dnskip .gt. 0 )  goto 3
c               until( dnskip .le. 0 )                                     5e
              end if                                                       4e
            end if                                                         3e
          end if                                                           2e
        if ( .not. erdfacq( Csublab, Dnsamp, Mdata, dsize, irdfrec,
     1      .true. ) ) call srdferr( Hlabel, Hrdfueof )
        if ( Dnsamp .gt. 0 ) then                                          2b
c
c           PLOT DATA
c
          Dtnsamp = Dtnsamp + Dnsamp
          npts = Dnsamp / nplots
          if ( fcapt )
     1      call ilgopen( '?' )
          call ilgchar( rchsiz, 1.0 )
          call ilgpage( 'A4', 'wide', nxplts, nyplts )
          do 7 iplot = 1, nplots                                           3b
            nypos = mod( iplot - 1, nyplts ) + 1
            nxaxis = 1
            if ( nypos .eq. nyplts .or. iplot .ge. nplots)
     1        nxaxis = 2
            call ilgraph( ( iplot - 1 ) / nyplts + 1, nypos, nxaxis,
     1        2, 0 )
c
c          transfer data to real arrays
c
            ix = 1
            do 5 i = iplot, dnsamp, nplots                                 4b
              Rx(ix) = ix * rcytim + iplot * rsamtim + rtimoff
              Ry(ix) = Mdata(i)
              ix = ix + 1
5             continue                                                     4e
c
c          plot data & labels
c
            call ilgdata( Rx, Ry, 1, npts, 1 )
            if ( nxaxis .eq. 2 )
     1        call ilgxlab( 'time (seconds)' )
            write( Ctitle, 6 ) Gextadid(), Jextchls(iplot)
6           format( a12, ' Channel ', i2 )
            call ilgtitl( Ctitle )
7           continue                                                       3e
c
c          DUMP DATA ONTO FILE
c
          if ( fdump ) then                                                3b
            call HG_puts( '  Dump this onto file ?   ', Hrow, Hcol )
            if ( Equery( Hkbdlu, ' ' ) ) then                              4b
              if ( .not. fhead ) then                                      5b
                fhead = .true.
                write( Hdmplu, 8 ) Cmhdlab, Csublab, rcytim,
     1            rsamtim, dnocy, nplots
8      format( ' ;' / ' ; module    : ', a / ' ; sub-block : ', a /
     1         ' ; cycle time : ', g12.5, 'seconds' /
     2         ' ; chan to chan time : ', g12.5, 'seconds' /
     3         ' ; number of cycles : ', i8, ' , ', i2, ' channels' )
                end if                                                     5e
              rtime = rtimoff
              write( Hdmplu, 9 ) ( Jextchls(i), i = 1, nplots )
9             format( ' ; time (sec)  channel' / ' ;', 12x, 16i5 )
              do 11 i = 0, dnsamp - 1, nplots                                5b
                rtime = rtime + rcytim
                write( Hdmplu, 10 ) rtime,
     1            ( Mdata( i + ix ), ix = 1, nplots )
10              format( 1x, g13.6, 16i5 )
11              continue                                                   5e
              end if                                                       4e
            end if                                                         3e
          Rtimoff = Rtimoff + npts * Rcytim
c
          if ( Dnsamp .eq. dsize ) then                                    3b
            call HG_on
            call HG_puts( '          More ?          ', Hrow, Hcol )
            if ( .not. Equery( Hkbdlu, ' ' ) )
     1        dsize = 0
            call HG_off
            goto 1
          else                                                             3m
            call ilgwait( .true. )
            end if                                                         3e
          end if                                                           2e
c       until ( No more or end of data )                                   1e
      call ilgclos
      return
      end


      SUBROUTINE SCLOSE
c
c  This is the CLOSE module for GENACQ .
c
c  close module
c    update master file
c    write close module & block headers
c    close data file
c
c Variables
c  logical *1 ok,               # success / failure result
c             daflush,          # function : flush data file buffer
c             daput,            #          : write items to data file
c             daputc,           #          : write character string to file
c             emgwrt            #          : write message into file
c  integer *2 ilflsv(3),        # master file : library , series & volume
c             item,             # item chosen for alteration
c             lencmt,           # length of comment ( bytes )
c             Jresp             # Function : integer response
c  character Clfcmt*80          # comment for master file
c  subroutine sipf2d,           # disable access to secondary input file
c             smgwrt,           # write message
c             smmhdr,           # write module header record
c             sopwrt,           # write item to primary output file
c             sopwrtc,          # write character string to primary output file
c             ssipf2            # enable access to secondary input file
c
      implicit none
      include 'genacq.kon'
c
      logical *1 ok, daflush, daput, daputc, emgwrt
      integer *2 ilflsv(3), item, lencmt, Jresp
      character Clfcmt*80
      include 'genacq.cmn'
c
      nmodul = nmodul + 1
      fdef1 = .false.
c
c       determine default log file entries - read last record in log file
c
      call ssipf2( '##close', ' ' )
      Clfcmt = ' '
      if ( .not. fstlfr .and. fenxeq ) then                               1b
        open( UNIT=Hloglu, FILE=Clogfil, ACCESS='sequential',
     1    FORM='formatted', STATUS='old', ERR=9 )
        read( Hloglu, 1, ERR=10, END=10 )
1       format( //// )
c       start of loop                                                     2b
2         read ( Hloglu, 3, END=4, ERR=10 ) Ilflsv
3         format( i6, i9, i8 /// )
c         end of loop                                                     2e
4       close( UNIT=Hloglu )
        end if                                                            1e
c
c       select log file entries
c
c     repeat                                                              1b
5       call smgwrt( 25001 )
        item = Jresp( Iipf1, 0, 4, Hntries )
        call sopwrt( item, Hint2 )
        if ( item .eq. 4 ) then                                           2b
          call smgwrt( 25005 )
          read( Iipf1, 7 ) Clfcmt
          lencmt = nblank( Clfcmt ) + 1
          call sopwrtc( Clfcmt( 1: lencmt-1 ) )
          goto 5
        else if ( item .gt. 0 ) then                                      2m
          call smgwrt( 25001 + item )
          Ilflsv( item ) = Jresp( Iipf1, 0, 9999, Hntries )
          call sopwrt( Ilflsv(item), Hint2 )
          goto 5
          end if                                                          2e
c       until( item .eq. Hexit )                                          1e
      if ( fenxeq ) then                                                  1b
c
c       write new entries onto log file
c
        if ( fstlfr ) then                                                2b
          open( UNIT=Hloglu, FILE=Clogfil, ACCESS='sequential',
     1      FORM='formatted', STATUS='unknown', ERR=9 )
          if ( .not. emgwrt( Hloglu, 25006 ) )
     1      call sfail( Hmsgerr )
          fstlfr = .false.
        else                                                              2m
          open( UNIT=Hloglu, FILE=Clogfil, ACCESS='append',
     1      FORM='formatted', STATUS='old', ERR=9 )
          end if                                                          2e
        write( Hloglu, 6, ERR=9 ) Ilflsv, nmode, nmodul, dtnbyt
6       format( i6, i9, i8, 2i7, 8x, i12 )
        write( Hloglu, 7, ERR=11 ) Cdfnam
        write( Hloglu, 7, ERR=11 ) Cdate
        write( Hloglu, 7, ERR=11 ) Clfcmt
7       format( a )
        close( UNIT=Hloglu )
        end if                                                            1e
8     if ( fenxeq ) then                                                  1b
c
c           write data file headers
c
        call smmhdr( '#close  ' )
        ok = daputc( irec, '##close ' )
c no. of bytes
        ok = ok .and. daput( irec, 16 + lencmt, 2 )
c lib, ser, vol
        ok = ok .and. daput( irec, ilflsv, 6 )
c nmode, nmodul
        ok = ok .and. daput( irec, nmode, 4 )
c dtnbyt
        ok = ok .and. daput( irec, dtnbyt, 4 )
c no. of bytes in comment
        ok = ok .and. daput( irec, lencmt, 2 )
c comment
        ok = ok .and. daputc( irec, Clfcmt(1:lencmt-1) )
        ok = ok .and. daput( irec, 0, 1 )
        ok = ok .and. daflush( irec )
        if ( .not. ok )
     1    call sfail( Hblkerr )
        end if
      call sipf2d
      return
c
c     error messages
c
9     call smgwrt( 25007 )
      goto 8
10    call smgwrt( 25008 )
      goto 8
11    call smgwrt( 25009 )
      goto 8
      end


      SUBROUTINE SCALBAR
c
c       Calibrate a single baratron or barocel unit
c
c Variables
c  logical *1 fcadc,            # changing A/D chan flag
c             fdef,             # default calibration selected flag
c             foldq,            # old value of fquiet
c             eask,             # function : logical response
c             estdef            #          : select default calibration
c  integer *2 Jresp             # function : integer response
c  subroutine setdbk,           # set up parameter for a data block
c             sgtdat,           # get data for a data acquisition module
c             smmhdr,           # write module header record onto data file
c             sopwrt,           # write item to primary output file
c             srsdef            # disables access to default secondary I/P file
c Message
c  22100 - calibrate Baratron module
c  23030 - which A/D channel ?
c
      implicit none
      include 'genacq.kon'
c
      logical *1 fcadc, fdef, foldq, eask, estdef
      integer *2 Jresp
      include 'genacq.cmn'
      include 'module.cmn'
c
c       Documentation message
c
      frepen = .false.
      foldq = fquiet
      fquiet = .false.
      call smgwrt( 22100 )
      fquiet = foldq
      nmodul = nmodul + 1
c
c       get parameters
c
      fblock( Hprelim ) = .false.
      fblock( Hmain ) = .true.
      fblock( Hextra ) = .false.
      fdef = estdef( '#calbar', Cbarcmd )
      iblock = Hmain
      call setdbk
      call srsdef
c
c       default calibration is selected
c
      if ( fdef ) then                                                    1b
c
c       change A/D chan
c
        fcadc = Eask( 'Are you changing the A/D chan ?' )
        call sopwrt( fcadc, Hlog1 )
        if ( fcadc ) then                                                 2b
          call smgwrt( 23030 )
          iacqc( H1chls, Hmain ) = Jresp( Iipf1, 0, 15, Hntries )
          call sopwrt( iacqc( H1chls, Hmain ), Hint2 )
          end if                                                          2e
        end if                                                            1e
c
c       get data
c
      if ( fenxeq ) then                                                  1b
        fquiet = .false.
        call smmhdr( '#calbar ' )
        call sgtdat
        fquiet = foldq
        end if                                                            1e
      return
      end


      SUBROUTINE SCALHW1
c
c  Calibrate one or more normal hot wires .
c
c Variable
c  logical *1 fdef              # default calibration flag
c             fyesno,           # response to 'change params ?'
c             foldq,            # old value of fquiet
c             Eask,             # function : logical response
c             Estdef            #          : selects default calibration
c             Equery            #          : logical response
c  integer *2 i,                # looping index
c             ibchan,           # baratron A/D channel
c             ichan,            # hot wire A/D channel
c             icylen,           # no. of A/D channels used
c             item,             # change parameters item number
c             npts,             # number of calibration points
c             Jresp             # function : integer response
c  real rbmin, rbmax, rbmid     # min, max & junction baratron readings
c       rnsteps                 # no. steps in calibration with vortex generator
c  subroutine setdbk,           # set up parameters for a data block
c             setofm,           # set array offsets for main data block
c             setofp,           # set array offsets for preliminary data block
c             sgtdat,           # get prelim & main data blocks
c             smgwrt,           # write message
c             smmhdr,           # write module header record
c             sopwrt,           # write item to primary output file
c             srsdef            # disable access to secondary I/P file
c Message
c  22200 - Single hot wire calibration
c  22201 - set to min. W/T speed
c  22202 - set to max. W/T speed
c  22203 - Number of points ?
c  22204 - decrease W/T speed
c  22205 - set to junction point
c  22206 - install low speed anemometer
c  22207 - change parameters menu
c  22208 - assumed A/D channels
c  22209 - introduces wall proximity method
c  23030 - A/D channel ?
c  23061 - A/D converter id ?
c  23064 - No. of channels ?
c  23065 - Channel address list ?
c
      implicit none
      include 'genacq.kon'
c
      logical *1 fdef, fyesno, foldq, Eask, Estdef, Equery
      integer *2 i, ibchan, icylen, ichan, item, npts, Jresp
      real rbmin, rbmax, rbmid, rnsteps
      include 'genacq.cmn'
      include 'module.cmn'
c
c       documentation message
c
      frepen = .false.
      foldq = fquiet
      fquiet = .false.
      nmodul = nmodul + 1
      call smgwrt( 22200 )
      fquiet = foldq
c
c       set max & min baratron readings - manual pause
c
      fdef = estdef( '#calhw1a', Chw1cmd )
      iblock = Hprelim
      fblock(Hprelim) = .true.
      fblock(Hmain) = .false.
      fblock(Hextra) = .false.
      call setdbk
      call srsdef
c
c       Change baratron A/D chan ?
c
      if ( fdef ) then                                                    1b
        fyesno = eask( 'Change A/D chan for baratron ?' )
        call sopwrt( fyesno, Hlog1 )
        if ( fyesno ) then                                                2b
          call smgwrt( 23030 )
          iacqv( Hhchan, Hprelim ) = Jresp( Iipf1, 0, 15, Hntries )
          call sopwrt( iacqv( Hhchan, Hprelim ), Hint2 )
          end if                                                          2e
        end if                                                            1e
      ibchan = iacqv( Hhchan, Hprelim )
c
c        get min baratron reading
c
      if ( fenxeq ) then                                                  1b
        fquiet = .false.
        call smgwrt( 22201 )
        call smmhdr( '#calhw1 ' )
        call sgtdat
        rbmin = rmonav
c
c       get junction point
c
        call smgwrt( 22205 )
        call sgtdat
        rbmid = rmonav
c
c       get max baratron reading
c
        call smgwrt( 22202 )
        call sgtdat
        rbmax = rmonav
        fquiet = foldq
        end if                                                            1e
c
c       set up main data acquisition block - all 3 blocks are used
c
      fdef = estdef( '#calhw1b', Chw1cmd )
      do 3 iblock = 1, Hnblks
        fblock(iblock) = .true.
3       call setdbk
c
c           get number of calibration points
c
      call smgwrt( 22203 )
      npts = Jresp( Iipf1, 3, 30, Hntries )
      call sopwrt( npts, Hint2 )
      call srsdef
c
c           get baratron A/D chan
c
      iacqv( Hhchan, Hmain ) = ibchan
      iacqc( H2chls, Hmain ) = ibchan
c
c       CHANGE HOT WIRE A/D OR CHANS ?
c
      if ( fdef ) then                                                    1b
c       repeat                                                            2b
4         call smgwrt( 22207 )
          item = Jresp( Iipf1, 0, 6, Hntries )
          call sopwrt( item, Hint2 )
          icylen = iacqv( Hcylen + Hprimoff, Hmain )
c
c       get A/D id
c
          if ( item .eq. 1 ) then                                         3b
            call smgwrt( 23061 )
            iacqc( H1cid, Hmain ) = Jresp( Iipf1, 1, 4, Hntries )
            call sopwrt( iacqc( H1cid, Hmain ), Hint2 )
            iacqc( H1cid, Hextra ) = iacqc( H1cid, Hmain )
c
c       get sampling period
            if ( iacqc( H1cid, Hmain ) .eq. Hseq ) then                   4b
              racqv( Hsampd + Hprimoff, Hmain ) = 10.0e-6
              racqv( Hsampd + Hprimoff, Hextra ) = 10.0e-6
            else if ( iacqc( H1cid, Hmain ) .eq. Hslo .or.
     1          iacqc( H1cid, Hmain ) .eq. Hsan ) then                    4m
c
c       assume A/D chans
              do 5 i = 0, icylen - 1                                      5b
                iacqc( H1chls + i, Hmain ) = i
                iacqc( H1chls + i, Hextra ) = i
5               continue                                                  5e
              iacqc( H1chls + 1, Hextra ) = 15
              call smgwrt( 22208 )
              end if                                                      4e
c
c       get cycle length
c
          else if ( item .eq. 2 ) then                                    3m
            call smgwrt( 23064 )
            icylen = Jresp( Iipf1, 1, 16, Hntries )
            iacqv( Hcylen + Hprimoff, Hmain ) = icylen
            iacqv( Hcylen + Hprimoff, Hextra ) = icylen + 1
            call sopwrt( icylen, Hint2 )
            if ( iacqc( H1cid, Hmain ) .eq. Hslo .or.
     1          iacqc( H1cid, Hmain ) .eq. Hsan ) then                    4m
c
c       assume A/D chans
              do 6 i = 0, icylen - 1                                      5b
                iacqc( H1chls + i, Hmain ) = i
                iacqc( H1chls + i, Hextra ) = i
6               continue                                                  5e
              iacqc( H1chls + 1, Hextra ) = 15
              call smgwrt( 22208 )
              end if                                                      4e
c
c       get A/D chans
c
          else if ( item .eq. 3 ) then                                    3m
            if ( iacqc( H1cid, Hmain ) .eq. Hslo .or.
     1           iacqc( H1cid, Hmain ) .eq. Hsan ) then                   4b
            call smgwrt( 23065 )
            do 8 i = 1, icylen                                            5b
              if ( .not. fquiet )
     1          write( Hvdulu, 7 ) i
7               format( 1x, i2, ' ? ' )
              ichan = Jresp( Iipf1, 0, 15, Hntries )
              call sopwrt( ichan, Hint2 )
              iacqc( H1chls - 1 + i, Hmain ) = ichan
              iacqc( H1chls - 1 + i, Hextra ) = ichan
8             continue                                                    5e
            iacqc( H1chls + icylen, Hextra ) = 15
            end if                                                        4e
c
c       unpack data flag
c
          else if ( item .eq. 4 ) then                                    3m
            gacqc( H1unp, Hmain ) = .false.
            if ( iacqc( H1cid, Hmain ) .ne. Hslo ) then                   4b
              gacqc( H1unp, Hmain ) =
     1          eask( 'Are you unpacking data in the PC ?' )
              gacqc( H1unp, Hextra ) = gacqc( H1unp, Hmain )
              call sopwrt( gacqc( H1unp, Hmain ), Hlog1 )
              end if                                                      4e
c
c       number of calibration points
c
          else if ( item .eq. 5 ) then                                    3m
            call smgwrt( 22203 )
            npts = Jresp( Iipf1, 3, 30, Hntries )
            call sopwrt( npts, Hint2 )
            end if                                                        3e
          if ( item .ne. Hexit ) goto 4
c         until( item = exit )                                            2e
        end if                                                            1e
c
c       GET MAIN CALIBRATION DATA
c
      if ( fenxeq ) then                                                  1b
        fquiet = .false.
        rinc(1,Hmain) = ( rbmin / rbmax ) **
     1    ( 1.0 / real( npts - 1 ) )
c
c       use baratron as reference
c
        if ( rbmax - rbmid - 0.01 .gt. 0.0 ) then                         2b
          rstart(1, Hmain ) = rbmax
          rend( 1, Hmain ) = rbmid - 0.01
          call smgwrt( 22204 )
          fblock(Hextra) = .false.
          call sgtdat
          fblock(Hprelim) = .false.
          end if                                                          2e
c
c       use vortex shedding as reference
c
        if ( rbmid - rbmin - 0.01 .gt. 0.0 ) then                         2b
          rstart( 1, Hextra ) = rbmid / rinc( 1, Hmain ) + 0.01
          rend( 1, Hextra ) = rbmin
          rinc( 1, Hextra ) = rinc( 1, Hmain )
          rnsteps = log( rinc(1,Hextra) ) /
     1      log( rend(1,Hextra) / rstart(1,Hextra) )
          rinc(2,Hextra) = (rend(2,Hextra)/rstart(2,Hextra)) ** rnsteps
          call smgwrt( 22206 )
          fblock(Hmain) = .false.
          fblock(Hextra) = .true.
          call sgtdat
          end if                                                          2e
c
c       use zero-flow, wall proximity effect
c
        if ( Equery( Hkbdlu,
     1     'Calibration by zero-flow, wall proximity effect ? ' ) ) then  2b
          call smgwrt( 22209 )
c           switch off all blocks except MAIN
          fblock(Hprelim) = .false.
          fblock(Hmain)   = .true.
          fblock(Hextra)  = .false.
c           switch off all variables
          nvars(Hmain) = 0
c           switch off all sub-blocks except primary data
          Gacqc(Hhalt,  Hmain) = .false.
          Gacqc(Huttvs, Hmain) = .false.
          Gacqc(H2cdb,  Hmain) = .false.
          Gacqc(H1cdb,  Hmain) = .true.
c           change the primary data sub-block to use keyboard reference value
          Gacqc(H1kbdv, Hmain) = .true.
c         repeat                                                          3b
9           call sgtdat
            if ( .not. Equery( Hkbdlu, 'Finished ? ' ) ) goto 9
c           until( finished )                                             3e
            end if                                                        2e
        end if                                                            1e
      fquiet = foldq
      return
      end



      SUBROUTINE SCALXHW
c
c  Calibrate one or more single hot wires .
c
c Variable
c  logical *1 fdef              # default calibration flag
c             foldq,            # old value of fquiet
c             Eask,             # function : logical response
c             Estdef            #          : selects default calibration
c  integer *2 i,                # looping index
c             ichan,            # hot wire A/D channel
c             icylen,           # no. of A/D channels used
c             item,             # change parameters item number
c             npts,             # number of calibration points
c             Jresp             # function : integer response
c  subroutine setdbk,           # set up parameters for a data block
c             sgtdat,           # get prelim & main data blocks
c             smgwrt,           # write message
c             smmhdr,           # write module header record
c             sopwrt,           # write item to primary output file
c             srsdef            # disable access to secondary I/P file
c Message
c  22400 - inclined hot wire calibration
c  22203 - Number of points ?
c  22207 - change parameters menu
c  22208 - assumed A/D channels
c  23030 - A/D channel ?
c  23061 - A/D converter id ?
c  23064 - No. of channels ?
c  23065 - Channel address list ?
c
c                 assume cosine cooling law for effective angle of inclination
      implicit none
      include 'genacq.kon'
      logical *1 fdef, foldq, Eask, Estdef
      integer *2 i, ichan, icylen, item, npts, Jresp
      include 'genacq.cmn'
      include 'module.cmn'
c
c       Documentation message
c
      frepen = .false.
      foldq = fquiet
      fquiet = .false.
      call smgwrt( 22400 )
      fquiet = foldq
      nmodul = nmodul + 1
c
c       get parameters
c
      fblock( Hprelim ) = .false.
      fblock( Hmain ) = .true.
      fblock( Hextra ) = .false.
      fdef = estdef( '#calxhw', Cxhwcmd )
      iblock = Hmain
      call setdbk
      call srsdef
c
c       CHANGE HOT WIRE A/D OR CHANS ?
c
      if ( fdef ) then                                                    1b
c       repeat                                                            2b
4         call smgwrt( 22207 )
          item = Jresp( Iipf1, 0, 6, Hntries )
          call sopwrt( item, Hint2 )
          icylen = iacqv( Hcylen + Hprimoff, Hmain )
c
c       get A/D id
c
          if ( item .eq. 1 ) then                                         3b
            call smgwrt( 23061 )
            iacqc( H1cid, Hmain ) = Jresp( Iipf1, 1, 4, Hntries )
            call sopwrt( iacqc( H1cid, Hmain ), Hint2 )
c
c       get sampling period
            if ( iacqc( H1cid, Hmain ) .eq. Hseq ) then                   4b
              racqv( Hsampd + Hprimoff, Hmain ) = 10.0e-6
            else if ( iacqc( H1cid, Hmain ) .eq. Hslo .or.
     1          iacqc( H1cid, Hmain ) .eq. Hsan ) then                    4m
c
c       assume A/D chans
              do 5 i = 0, icylen - 1                                      5b
                iacqc( H1chls + i, Hmain ) = i
5               continue                                                  5e
              call smgwrt( 22208 )
              end if                                                      4e
c
c       get cycle length
c
          else if ( item .eq. 2 ) then                                    3m
            call smgwrt( 23064 )
            icylen = Jresp( Iipf1, 1, 16, Hntries )
            iacqv( Hcylen + Hprimoff, Hmain ) = icylen
            call sopwrt( icylen, Hint2 )
            if ( iacqc( H1cid, Hmain ) .eq. Hslo .or.
     1          iacqc( H1cid, Hmain ) .eq. Hsan ) then                    4m
c
c       assume A/D chans
              do 6 i = 0, icylen - 1                                      5b
                iacqc( H1chls + i, Hmain ) = i
6               continue                                                  5e
              call smgwrt( 22208 )
              end if                                                      4e
c
c       get A/D chans
c
          else if ( item .eq. 3 ) then                                    3m
            if ( iacqc( H1cid, Hmain ) .eq. Hslo .or.
     1           iacqc( H1cid, Hmain ) .eq. Hsan ) then                   4b
            call smgwrt( 23065 )
            do 8 i = 1, icylen                                            5b
              if ( .not. fquiet )
     1          write( Hvdulu, 7 ) i
7               format( 1x, i2, ' ? ' )
              ichan = Jresp( Iipf1, 0, 15, Hntries )
              call sopwrt( ichan, Hint2 )
              iacqc( H1chls - 1 + i, Hmain ) = ichan
8             continue                                                    5e
            end if                                                        4e
c
c       unpack data flag
c
          else if ( item .eq. 4 ) then                                    3m
            gacqc( H1unp, Hmain ) = .false.
            if ( iacqc( H1cid, Hmain ) .ne. Hslo ) then                   4b
              gacqc( H1unp, Hmain ) =
     1          eask( 'Are you unpacking data in the PC ?' )
              call sopwrt( gacqc( H1unp, Hmain ), Hlog1 )
              end if                                                      4e
c
c       number of calibration points
c
          else if ( item .eq. 5 ) then                                    3m
            call smgwrt( 22203 )
            npts = Jresp( Iipf1, 3, 30, Hntries )
            call sopwrt( npts, Hint2 )
            iend( 1, Hmain ) = npts
            end if                                                        3e
          if ( item .ne. Hexit ) goto 4
c         until( item = exit )                                            2e
        end if                                                            1e
c
c       get data
c
      if ( fenxeq ) then                                                  1b
        fquiet = .false.
        call smmhdr( '#calxhw ' )
        call sgtdat
        fquiet = foldq
        end if                                                            1e
      return
      end

