c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

c
c       TRAVERSE CALIBRATION ROUTINES
c
c       KNOWN BUGS
c       ##########
c  1) the displacement offset routines STVCDO and STVPOF
c       As written , these will only work for the simple traverse . This
c       OUGHT to work for any physical reference coordinates . This means
c       that STVCDO would be restricted to collecting data . The displace-
c       ment offsets would have to calculated by a separate routine .



      SUBROUTINE SCALTVS
c
c       CALibrate TraVerSe
c   Basic traverse calibration routine . The task is broken into four parts :
c                                                     movement  auto enabled
c     1) potentiometer <==> displacement calibration    yes         yes
c     2) determination of displacement offsets          yes         no
c     3) adjustment of the drift correction factors     yes         no
c     4) entering the reference probe offset            no          no
c
c       Calibration data is stored in the sequential access binary file
c  'tvscal.dat' .
c
c constants -- integer *2 for traverse calibration menu
c  Hdef_menu=1
c  Hpdt_menu=2
c  Hfpy_menu=3 
c  Hdoc_menu=4
c  Hdof_menu=5
c  Hrpo_menu=6
c  Htts_menu=7
c Variables
c  logical *1 fnew,             # new calibration file flag
c             foldq,            # olf value of fquiet
c             ok,               # success / failure result
c             eask,             # function : logical response
c             daput,            # write items to data file
c             daputc,           # write character strings to data file
c             daflush           # flush data file buffer
c  integer *2 item
c             jresp             # function : read integer
c  subroutine scheck,           #
c             seltch,           # select traverse channel
c             sfail,		# print message for fatal error message
c             smgwrt,           # write message
c             smmhdr,           # write module header record
c             smvdef,           # move traverseDOF to default position
c             sopwrt,           # write item to primary output file
c             stest_tvs,	# test traverse software without movement 
c             stvadj,           # do a drift correction
c             stvcdo,           # calibration : displacement offset
c             stvcpd,           # calibration : pot <==> displ  table
c             stvcrd,           # open & read traverse calibration file
c             stvpol,           # fit polynomials to potentiometer/displacement
c             stvpof,           # calibration : probe offset
c Messages
c  22300 - traverse type
c  22301 - safety => default position
c  22302 - can't write calibration file
c  22303 - can't open calibration file
c
      include 'genacq.kon'
      integer *2 Hdef_menu, Hdoc_menu, Hdof_menu, Hfpy_menu, Hpdt_menu,
     1           Hrpo_menu, Htts_menu 
      parameter( Hdef_menu=1, Hdoc_menu=4, Hdof_menu=5, Hfpy_menu=3, 
     1           Hpdt_menu=2, Hrpo_menu=6, Htts_menu=7 )
c
      logical *1 daflush, daput, daputc, fnew, foldq, ok, Eask
      integer *2 item, jresp
c
      include 'genacq.cmn'
      include 'tvstvs.cmn'
      external seltch, smvdef, stvcpd, stvpol, stvcdo, stvadj, stvpof
c
      foldq = fquiet
      fquiet = .false.
      fdef1 = .false.
      rgblock = 0.0
      rdisgn = 1.0
c
c       traverse type ?  etc
c
      call smgwrt( 22300 )
      itvsid = Jresp( iipf1, Hexit, Hauxil, Hntries )
      call sopwrt( itvsid, Hint2 )
      if ( itvsid .ne. Hexit ) then                                       1b
c
c       read calibration file
c
          fnew = Eask( 'Do you want a new calibration file ?' )
          call sopwrt( fnew, Hlog1 )
          if ( fenxeq ) then                                              2b
              if ( fnew ) then                                            3b
                  open ( UNIT=Htvslu,         FILE=Ctvsfil, 
     1                   ACCESS='sequential', FORM='unformatted',
     2                   STATUS='unknown',    ERR=4 )
              else                                                        3m
c                 read calibration file
                  call stvcrd
                  end if                                                  3e
              end if                                                      2e
c
c 	  start of menu loop                                              2b
1             call smgwrt( 22003 )
c Traverse calibration menu
c   0) Exit                  1) Move to def. pos'n
c   2) Pot. <=> displ table  3) Fit polynomial
c   4) Correct drift offset  5) Displacement offset
c   6) Ref. probe offset     7) Test software
c
              item = Jresp( iipf1, Hexit, Htts_menu, Hntries )
              call sopwrt( item, Hint2 )
              if ( item .eq. Hexit ) then                                 3b
c
c       EXIT
c
c       write traverse calibration file
c
                  if ( fenxeq ) then                                      4b
                      write( Htvslu, ERR=2 ) Mtvcal
                      close( UNIT=Htvslu )
                      fpass1 = .true.
                      goto 3
c
c       error writing calibration file
c
2                     call smgwrt( 22302 )
3                     continue
c
c       write data file module
c
                      nmodul = nmodul + 1
                      call smmhdr( '#caltvs ' )
                      ok = daputc( irec, '##caltvs' )
                      ok = ok .and. daput( irec, Hfsize *2, 2 )
                      ok = ok .and. daput( irec, Mtvcal, Hfsize *2 )
                      ok = ok .and. daflush( irec )
                      if ( .not. ok )
     1                    call sfail( Hblkerr )
                      end if                                              4e
                  fquiet = foldq
                  return
c
c       MOVE TO DEFAULT POSITION
c
              else if ( item .eq. Hdef_menu ) then                        3m
                  if ( .not. fnew .and. fenxeq ) then                     4b
                      call smgwrt( 22301 )
                      call seltch( smvdef, .false. )
                      end if                                              4e
c
c       POTENTIOMETER <=> DISPLACEMENT TABLE
c
              else if ( item .eq. Hpdt_menu ) then                        3m   
                  if ( fenxeq )
     1                call seltch( stvcpd, .true. )
c
c       FIT POLYNOMIAL TO POT/DISPL TABLE
c
              else if ( item .eq. Hfpy_menu ) then                        3m  
                  if ( fenxeq ) 
     1                call seltch( stvpol, .false. )
c
c       CORRECT DRIFT OFFSET
c
              else if ( item .eq. Hdoc_menu ) then                        3m  
                  if ( fenxeq )
     1                call seltch( stvadj, .false. )
c
c       DISPLACEMENT OFFSET
c
              else if ( item .eq. Hdof_menu ) then                        3m  
                  if ( fenxeq )
     1                call seltch( stvcdo, .false. )
c
c       REFERENCE PROBE OFFSET
c
              else if ( item .eq. Hrpo_menu ) then                        3m  
                  if ( fenxeq )
     1                call scheck( stvpof )
c
c       TEST SOFTWARE
c
              else if ( item .eq. Htts_menu ) then                        3m
                  call stest_tvs()
                  end if                                                  3e 
              goto 1
c             end of menu loop                                            2e
          end if                                                          1e

      return
c
c       error opening calibration file
c
4     call smgwrt( 22303 )
      fquiet = foldq
      return
      end



      SUBROUTINE STEST_TVS()
c
c test the traverse calibration by moving to a specified location and 
c reading the position
c
c constants - for menu items
c  Hreqd_tst=1    - required traverse position
c  Hincr_tst=2    - motion increment
c  Hnext_tst=3    - next traverse position
c  Hdriv_tst=4    - drive to required traverse position
c  Hread_tst=5    - read traverse position
c  Hdvrd_tst=6    - drive to required traverse position and read it
c local variables
c  integer *2
c   item           # selected menu item
c   Jresp          # function : read integer
c  real
c    act_posn(5)   # actual traverse position read from GETTVS
c    reqd_posn(5)  # required traverse position
c    reqd_inc      # increment in required traverse position
c    qresp         # function : real real
c  subroutine
c    gettvs, puttvs, smgwrt
c
      implicit none
      include 'genacq.kon'
      integer *2 Hdriv_tst, Hdvrd_tst, Hincr_tst, Hnext_tst, Hread_tst,
     1           Hreqd_tst
      parameter( Hdriv_tst=4, Hdvrd_tst=6, Hincr_tst=2, Hnext_tst=3, 
     1           Hread_tst=5, Hreqd_tst=1 )
c
      integer *2 item, Jresp
      real act_posn(5), reqd_posn(5), reqd_inc, qresp 
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
c     start of menu loop
1         call smgwrt( 22004 ) 
c Traverse test menu     0) Exit
c   1) Req'd position    2) Motion increm
c   3) Next probe pos'n  4) Drive to position
c   5) Read position     6) Drive & read position
          write( Hvdulu, 2 ) reqd_posn(2), reqd_inc, 
     1        reqd_posn(2) + reqd_inc
2         format( ' required position = ', f8.4 /
     1            ' motion increment  = ', f8.4 /
     2            ' next req''d pos''n  = ', f8.4, '   ' )
          item = Jresp( Hkbdlu, Hexit, Hdvrd_tst, Hntries )
c
c       EXIT
c
          if ( item .eq. Hexit ) then
              return
c
c       REQUIRED PROBE POSITION
c
          else if ( item .eq. Hreqd_tst ) then
              call smgwrt( 22005 )
              reqd_posn(2) = qresp( Hkbdlu, -100.0, 100.0, Hntries )
c
c       MOTION INCREMENT
c
          else if ( item .eq. Hincr_tst ) then
              call smgwrt( 22005 )
              reqd_inc  = qresp( Hkbdlu, -100.0, 100.0, Hntries )
c
c       NEXT PROBE POSITION
c
          else if ( item .eq. Hnext_tst ) then
              reqd_posn(2) = reqd_posn(2) + reqd_inc
c
c       DRIVE TO REQUIRED PROBE POSITION
c
          else if ( item .eq. Hdriv_tst ) then
              if ( fenxeq )
     1            call puttvs( itvsid, reqd_posn )
c
c       READ PROBE POSITION
c
          else if ( item .eq. Hread_tst ) then
              if ( fenxeq )
     1              call gettvs( itvsid, act_posn )
c
c       DRIVE TO REQUIRED PROBE POSITION AND READ IT
c
          else if ( item .eq. Hdvrd_tst ) then
              if ( fenxeq ) then
                  call puttvs( itvsid, reqd_posn )
                  call gettvs( itvsid, act_posn )
                  end if
              end if
          goto 1
c         end of menu loop    
c
      end
      


      SUBROUTINE SELTCH( SUBR, FAUTEN )
c
c       SELect Traverse CHannel
c     Allows the user to select traverse channels and then execute the
c sub-procedure SUBR on each channel .
c
c Argument
c  subroutine subr              # calibration sub-procedure for one channel
c  logical *1 fauten,           # automatic movement enable flag
c Variables
c  logical *1 fchsel,           # manual traverse channel selection flag
c             equery            # function : logical response
c  integer *2 i1,               # new idofmm
c             i2,               # prospective new value of 'calibrated DOFs
c             istchn,           # first traverse channel
c             ienchn,           # last traverse channel
c             Mstchn(3),        # start channel no. for each traverse
c             Mnochn(3),        # no. of channels for each traverse
c             Mstdof(3),        # start DOF no. for each traverse
c             jresp,            # function : integer response
c             jdofsw            # function : DOF bit switch generator
c  subroutine scheck,           # allow repeat execution of routine
c             smgwrt            # write messages
c Message
c 22304 - Traverse channel ?
c
      include 'genacq.kon'
c
      logical *1 fauten, fchsel, equery
      integer *2 i1, i2, istchn, ienchn, Mstchn(3), Mnochn(3),
     1  Mstdof(3), jresp, jdofsw
      include 'genacq.cmn'
      include 'tvstvs.cmn'
      external subr
c
      data Mstchn / 8, 1, 6 /
      data Mnochn / 1, 5, 2 /
      data Mstdof / 5, 1, 6 /
c
c       Channel select
c
      if ( itvsid .le. 0 .or. itvsid .gt. Hauxil )
     1  return
      fchsel = .false.
      if ( itvsid .ne. Hsimple )
     1  fchsel = Equery( Hkbdlu,
     2    'Are you selecting traverse channels ?' )
      idofmm = 0
c
c       automatic traverse movement
c
      fauto = .false.
      if ( fauten )
     1  fauto = Equery( Hkbdlu, 'Do you want automatic movement ?' )
c
c       set up channel select loop
c
      istchn = Mstchn( itvsid )
      ienchn = istchn + Mnochn( itvsid ) - 1
      do 5 i1 = ienchn, istchn, -1                                        1b
        if ( fchsel ) then                                                2b
c
c       select channel
c
c         start of loop                                                   3b
1           call smgwrt( 22304 )
            ichan = jresp( Hkbdlu, istchn, ienchn, Hntries )
            idof = Mstdof( itvsid ) + ichan - istchn
            i2 = ior( idofmm, jdofsw( idof ) )
            if ( i2 .ne. idofmm ) goto 3
            write( Hvdulu, 2 ) idof
2           format( ' DOF ', i2, ' already done' )
            if ( Equery( Hkbdlu, 'Are you finished' ) ) return
            goto 1
c           end of loop                                                   3e
        else                                                              2m
          ichan = i1
          idof = Mstdof( itvsid ) + ichan - istchn
          end if                                                          2e
3       idofmm = i2
        write ( Hvdulu, 4 ) ichan, idof
4       format( ' Traverse channel ', i2, ', DOF ', i2 )
c
c       execute sub-procedure
c
        call scheck( subr )
5       continue                                                          1e
      return
      end


      SUBROUTINE SMVDEF
c
c       Routine to drive DOF IDOF of the traverse to its default position -
c  if it exists .
c
c Arguments
c  real rdef            # default potentiometer reading
c Messages
c  22305 - Move to default manually
c
      include 'genacq.kon'
      real rdef
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
      if ( ftvpdt(ichan) ) then                                             1b
        rdef = Utvpot( Hdeftvs, ichan )
        if ( Utvpot( Hmintvs, ichan ) .le. rdef .and.
     1      rdef .lt. Utvpot( Hmaxtvs, ichan ) )
     2    call sdvpot( int( rdef ) )
      else                                                                  1m
        call smgwrt( 22305 )
        read( Hkbdlu, 1 )
1       format( )
        end if                                                              1e
      return
      end


      SUBROUTINE STVCPD
c
c     Subroutine TraVerse Calibration : Potentiometer <==> Displacement table
c  Creates the 'potentiometer <==> displacement' conversion tables .
c  There are two matching tables ( arrays ) , one for potentiometer readings
c Utvpot , and one for traverse carriage displacements Utvdis .
c The remaining details of these arrays are given under QTDIS & JTVPOT
c
c description of process
c   find maximum limit
c   find minimum limit
c   number of coordinate pairs ?
c   get the table internal points
c   set default drift corrections
c   set out of range default values
c   shift the carriage displacement datum
c   move to default position
c
c Variables
c  logical *1 equery            # function : logical response
c  integer *2 i1,               # temporary & looping variables
c             idelV,            # increment in pot. reading
c             irows,            # no. of rows used in P/D table, column ichan
c             iV,               # required / actual potentiometer reading
c             iVmin,            # minimum pot. reading
c             iVmax,            # maximum pot. reading
c             jresp             # function : integer response
c             jvpot,            # function : reads IDOFth traverse pot
c  real  rrdmax,                # temporary max displacement
c        rshift,                # shift in carriage displacement datum
c        Utvdis(Hnopts,2*Hntvch),   # traverse displacment table
c        Qdispl,                # function : moves traverse chan & reads pot
c        Qresp,                 #          : real response
c        Qtvdis,                #          : pot. => displ conversion
c  subroutine smgwrt,           # write message
c             smvdef,           # drive DOF to default position
c             spdtmm,           # completes max & min values in table
c             sortsh2,          # shell sort of an array
c             swap,             # swap 2 4 byte values
c             swapi             # swap 2 2 byte values
c Messages
c  22306 - to Max limit
c  22307 - to Min limit
c  22308 - How many calibration points ?
c  22309 - Out of range default
c  22330 - Move to new zero displ. datum
c
      include 'genacq.kon'
c
      include 'genacq.cmn'
      include 'tvstvs.cmn'
      logical *1 equery
      integer *2 i1, idelV, irows, iV, iVmin, iVmax, jresp, jvpot
      real rrdmax, rshift, Qdispl, Qtvdis
c
c       clear the fitted polynomial
c
      Mtvord(ichan) = 0
      do 3 i1 = 1, Hmaxpol
        Utvpol(i1,ichan) = 0.0
3       continue
c
c       maximum pot. voltage / displacement
c
      ftvpdt( ichan ) = .true.
      call smgwrt( 22306 )
      rdmax = 360.0
      rdmin = -180.0
      iVmax = 4095
      rrdmax = Qdispl( iVmax )
c
c       minimum pot. voltage / displacement
c
      call smgwrt( 22307 )
      iVmin = 0
      rdmin = Qdispl( iVmin )
      rdmax = rrdmax
      if ( iVmax .lt. iVmin ) then                                        1b
        call swap( rdmax, rdmin )
        call swapi( iVmax, iVmin )
        end if                                                            1e
c
c       Number of coordinate pairs ?
c
      call smgwrt( 22308 )
      irows = Jresp( Hkbdlu, 3, Hnopts - H1sttvs - 1, Hntries ) +
     1        H1sttvs - 1
      Mpdsiz( ichan ) = Irows
      Utvdis( irows, ichan ) = rdmax
      Utvpot( irows, ichan ) = real( iVmax )
c      WRITE(6,9999) ICHAN,irows,UTVPOT(irows,ICHAN),UTVDIS(irows,ICHAN)
      Utvdis( H1sttvs, ichan ) = rdmin
      Utvpot( H1sttvs, ichan ) = real( iVmin )
c      WRITE(6,9999) ICHAN,H1sttvs,UTVPOT(H1sttvs,ICHAN),
c     1    UTVDIS(H1sttvs,ICHAN)
      if ( rdmax .lt. rdmin )
     1  call swap( rdmax, rdmin )
c
c       get internal points
c    N.B. the potentiometer increment is multiplied by 8 to make use of
c    the available 15 bit precision of an integer
c
      idelV = ( iVmax - iVmin ) * 8 / ( irows - 4 )
      do 1 i1 = H1sttvs + 1, irows - 1                                              1b
        iV = iVmin + idelV * ( i1 - 4 ) / 8
        Utvdis( i1, ichan ) = qdispl( iV )
        Utvpot( i1, ichan ) = real( iV )
c      WRITE(6,9999) ICHAN,I1,UTVPOT(I1,ICHAN),UTVDIS(I1,ICHAN)
c9999  FORMAT( ' chan=', i2, ' row=', i2, ' (pot,displ)= ', 2g15.7 )
1       continue                                                          1e
c
c       sort displacements into ascending order
c                   table   last    first   no. of   sorted   no. of   column
c                           row      row     rows    column   columns  increm
      call sortsh2( Utvtab, irows, H1sttvs, Hnopts, Hntvch + ichan,
     1  2*Hntvch, Hntvch )
c      DO 9998 I1=1,IROWS
c          WRITE(6,9999) ICHAN,I1,UTVPOT(I1,ICHAN),UTVDIS(I1,ICHAN)
c9998      CONTINUE

c
c       default drift corrections
c
      Upddgc( ichan ) = 1.0
      Upddoc( ichan ) = 0.0
c
c       Put max & min values in arrays
c
      call Spdtmm( Utvdis( Hdeftvs, ichan ), irows )
      call Spdtmm( Utvpot( Hdeftvs, ichan ), irows )
c
c       out of range default values -- half way between limits
c
      call smgwrt( 22309 )
      iV = int( Utvpot(Hmaxtvs,ichan) + Utvpot(Hmintvs,ichan) ) / 2
      rrdmax = Utvpot( Hdeftvs, ichan )
      if ( Utvpot(Hmintvs,ichan) .le. rrdmax .and.
     1     rrdmax .le. Utvpot(Hmaxtvs,ichan) )
     2  iV = int( rrdmax )
      Utvdis( Hdeftvs, ichan ) = qdispl( iV )
      Utvpot( Hdeftvs, ichan ) = real( iV )
c      WRITE(6,9999) ICHAN,Hdeftvs,UTVPOT(I1,ICHAN),UTVDIS(I1,ICHAN)
c
c       shift carriage displacement datum
c
      if ( equery( Hkbdlu,
     1    'Do you want to shift the carriage displacement datum ?' ) )
     2    then                                                            1b
        call smgwrt( 22330 )
        call smgwrt( 22330 + ichan )
        iV = -1
        rshift = Qdispl( iV )
        rshift = rshift - Qtvdis( jvpot(), ichan )
c      WRITE(6,9997) RSHIFT
c9997  FORMAT( ' RSHIFT=', G15.7 )
        do 2 i1 = 1, irows                                                2b
          Utvdis( i1, ichan ) = Utvdis( i1,ichan ) + rshift
c          WRITE(6,9999) ICHAN,I1,UTVPOT(I1,ICHAN),UTVDIS(I1,ICHAN)
2         continue                                                        2e
        end if                                                            1e
c
c       move to default position
c
      call smvdef
      return
      end


      SUBROUTINE STVPOL
c
c Calculate the smooth fitting polynomial for converting potentiometer
c reading into traverse displacement .
c
      include 'genacq.kon'
      logical got_poly
      integer *2 i, item, last_row, npts, order, Jresp
      include 'genacq.cmn'
      include 'tvstvs.cmn'
      real err, Uerror(Hnopts), work(Hmaxpol*4+4), qpoly
c
      got_poly = Mtvord(ichan) .gt. 0 .and.
     1           Mtvord(ichan) .lt. Hmaxpol - 1
      last_row = Mpdsiz(ichan)
c     start of loop                                                       1b
c       write menu
1       call smgwrt( 22002 )
        npts = last_row - H1sttvs + 1
        item = Jresp( Hkbdlu, 0, 6, Hntries )
        if ( item .eq. 0 ) then                                           2b
          Mpdsiz(ichan) = last_row
          return
c
c           get polynomial order
c
        else if ( item .eq. 1 ) then                                      2m
          order = Jresp( Hkbdlu, 0, Hmaxpol-1, Hntries )
          if ( Mtvord(ichan) .ne. order ) got_poly = .false.
          Mtvord(ichan) = order
c
c           calculate polynomial
c
        else if ( item .eq. 2 ) then                                      2m
          call spyorth( Utvpot(H1sttvs,ichan),
     1      Utvdis(H1sttvs,ichan), npts, Utvpol(1,ichan),
     2      Mtvord(ichan), err, work )
          write( Hvdulu, 2 ) err
2         format( ' Mean square error of polynomial = ', g12.5 )
          got_poly = .true.
c
c           Cannot do anything else if polynomial is not calculated
c
        else if ( got_poly .and. Mtvord(ichan) .gt. 0 ) then              2m
          if ( item .eq. 3 .or. item .eq. 4 ) then                        3b
c
c           calculate error between polynomial and displacement
c
            do 3 i = H1sttvs, last_row                                    4b
              Uerror(i) = Utvdis(i,ichan) - qpoly( Utvpot(i,ichan),
     1          Utvpol(1,ichan), Mtvord(ichan) )
3             continue                                                    4e
c
c           Display data & error
c
            if ( item .eq. 3 ) then                                       4b
              write( Hvdulu, 4 )
4             format( '    Potent. Displac''t Polynom  Error' )
              do 6 i = H1sttvs, Mpdsiz(ichan)                             5b
                write( Hvdulu, 5 ) i - H1sttvs + 1, Utvpot(i,ichan),
     1            Utvdis(i,ichan), Utvdis(i,ichan) - Uerror(i),
     2            Uerror(i)
5               format( 1x, i2, 1x, f8.3, 3f9.4 )
6               continue                                                  5e
c
c           Plot error
c
            else                                                          4m
              call ilgline( 1, 1, 1 )
              call ilgdata( Utvpot(H1sttvs,ichan),
     1          Uerror(H1sttvs), 1, npts, 1 )
              call ilgwait()
              call ilgclos()
              end if                                                      4e
c
c           Delete data point -- move it to the end of the array
c
          else if ( item .eq. 5 ) then                                    3m
            write( Hvdulu, 7 )
7           format( ' Which one ? ' )
            order = Jresp( Hkbdlu, 1, npts, Hntries ) + H1sttvs - 1
            del_pot = Utvpot(order,ichan)
            del_dis = Utvdis(order,ichan)
            do 8 i = order, last_row-1                                    4b
              Utvpot(i,ichan) = Utvpot(i+1,ichan)
              Utvdis(i,ichan) = Utvdis(i+1,ichan)
8             continue                                                    4e
            Utvpot(last_row,ichan) = del_pot
            Utvdis(last_row,ichan) = del_dis
            last_row = last_row - 1
c
c           Restore deleted data
c
          else if ( item .eq. 6 ) then
            last_row = Mpdsiz(ichan)
c
c       sort displacements into ascending order
c                   table   last    first   no. of   sorted   no. of   column
c                           row      row     rows    column   columns  increm
            call sortsh2( Utvtab, last_row, H1sttvs, Hnopts,
     1                    Hntvch + ichan, 2*Hntvch, Hntvch )
            end if                                                        3e
          end if                                                          2e
        goto 1
c       end of loop                                                       1e
      end


      SUBROUTINE STVCDO
c
c       Displacement offset
c
c description of process
c   move to the reference point
c   read the potentiometer position
c   calculate displacement
c   enter physical position
c   calculate offset eg. = pos'n - displ'
c   move to default position
c
c N.B. This only works properly for the simple traverse - see BUGS at top
c
c  real car_pos                 # measured carriage position
c       Qdispl                  # function : measure carriage displacement
c       Qtvdis                  #          : convert pot => displacement
c  integer *2 iV                # potentiometer reading
c  subroutine smgwrt,           # write message
c             smvdef
c Messages
c  22339 - Move to zero displacement reference point
c
      implicit none
      include 'genacq.kon'
c
      real car_pos, Qdispl, Qtvdis
      integer *2 iV
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
      if ( ftvpdt( ichan ) ) then                                         1b
        rdmin = Utvdis( Hmintvs, ichan )
        rdmax = Utvdis( Hmaxtvs, ichan )
        call smgwrt( 22339 )
        iV = -1
        car_pos = Qdispl( iV )
        Utvoff( ichan ) = car_pos - Qtvdis( iV, ichan )
        call smvdef
        end if                                                            1e
      return
      end


      SUBROUTINE STVADJ
c
c       TraVerSe ADJust
c  Routine to evaluate and record the drift offset correction .
c
c description of process
c   GET ADJUSMENT POINT   select option
c   drive to max limit                   move to reference position
c   read pot (p)                         read pot (p) & get displ.
c   get max pot value from table (t)     displ => pot value from table (t)
c
c   get adjustment point 1
c   get adjustment point 2
c   gradient correction
c   offset correction
c   move to default position
c
c Variable
c  integer *2 iVpot1, iVpot2,   # new potentiometer readings
c             iVtab1, iVtab2,   # old pot. values from P/D table
c             Jadj              # function : gets 1 adjustment point
c  subroutine smgwrt            # write message
c Messages
c  22360 - Not moved. Abort drift correction.
c
      implicit none
      integer *2 iVpot1, iVpot2, iVtab1, iVtab2, Jadj
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
      if ( ftvpdt( ichan ) ) then                                         1b
c
c           set the default target positions
c
        iVpot1 = 4095
        iVpot2 = 0
        rdmin = Utvdis( Hmintvs, ichan )
        rdmax = Utvdis( Hmaxtvs, ichan )
c
c           set correction factors to null values
c
        Upddgc( ichan ) = 1.0
        Upddoc( ichan ) = 0.0
c
c           read the traverse reference positions ( or extrema )
c
        iVtab1 = Jadj( 'maximum', iVpot1, Hmaxtvs )
        iVtab2 = Jadj( 'minimum', iVpot2, Hmintvs )
c
c           calculate corrections
c
        if ( iVpot2 - iVpot1 .eq. 0 ) then                                2b
          call smgwrt( 22360 )
        else                                                              2m
          Upddgc( ichan ) = real( iVtab2 - iVtab1 ) /
     1                      real( iVpot2 - iVpot1 )
          Upddoc( ichan ) = real( iVtab1 ) -
     1                      real( iVpot1 ) * Upddgc(ichan)
          end if                                                          2e
        end if                                                            1e
      return
      end


      INTEGER FUNCTION JADJ( CMAXMIN, IVPOT, IINDEX )
c
c  Gets one adjustment point for drift correction . The returned result JADJ
c  is the potentiometer reading taken from the existing calibration table .
c  The other returned value is the parameter IVPOT which is the `drifted'or
c  new potentiometer reading for the reference or limit displacement .
c
c Arguments
c  character *(*) Cmaxmin   # 'maximum' or 'minimum'
c  integer *2 iVpot,        # required limit potentiometer reading
c             iindex,       # min/max = 2/3 : index for Utvdis
c Variables
c  integer *2 jVpot,        # function : read DOF pot
c             jtvpot        # function : convert pot => displ
c  real qdispl              # function : move & read a DOF
c  logical *1 equery        # function : logical response
c  subroutine sdvpot        # drive one traverse motor only
c Message
c  22001 - Move to drift adjust ref. <CR>
c
      implicit none
      include 'genacq.kon'
c
      character Cmaxmin*(*)
      integer *2 iVpot, iindex, jVpot, jtvpot
      real qdispl
      logical *1 equery
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
      if ( equery( Hkbdlu, 'Shall I drive to ' // Cmaxmin //
     1    ' displacement limit ?' ) ) then                                1b
        call sdvpot( iVpot )
        iVpot = jVpot()
        jadj  = int( Utvpot( iindex, ichan ) + 0.5 )
      else                                                                1m
        call smgwrt( 22001 )
        jadj = jtvpot( Qdispl( iVpot ), ichan )
        end if                                                            1e
      return
      end


      SUBROUTINE STVPOF
c
c       Reference probe offset
c
c description of process
c   select option
c     read each offset component in turn
c   or
c     read probe position
c     read potentiometers
c     calculate probe offsets
c
c Variables
c  logical *1 fproff,   # probe offset entered flag
c             equery    # function : logical response
c  integer *2 i1,       # looping index
c             ipposn(8) # potentiometer readings
c  real r,              # offset ordinate
c       Qresp,          # function : real response
c       Qtvdis          #          : Traverse DOF displacement
c  subroutine smgwrt,   # write message
c             sopwrt,   # write item to primary output file
c             stvsrd    # read traverse potentiometers
c Messsages
c  22361 - reference probe offset ?
c  22362 - probe physical position ?
c  23034 - x ?
c  23035 - y ?
c  23036 - z ?
c  23036 - Theta ?
c  23037 - Phi ?
c  23362 - Probe physical position ?
c
      implicit none
      include 'genacq.kon'
c
      logical *1 fproff, equery
      integer *2 i1, ipposn(8)
      real r, Qresp, Qtvdis
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
      fproff = Equery( Hkbdlu, 'Are you entering offset ?' )
      call sopwrt( fproff, Hlog1 )
      if ( fproff ) then                                                  1b
c
c       get probe offsets
c
        call smgwrt( 22335 )
        do  1 i1 = 1, 5                                                   2b
          call smgwrt( 23034 + i1 )
          r = Qresp( Hkbdlu, Hphysmin, Hphysmax, Hntries )
          Uproff( i1, itvsid ) = r
          call sopwrt( r, Hreal )
1         continue                                                        2e
      else                                                                1m
c
c       get probe physical position
c
        call smgwrt( 22362 )
        do 2 i1 = 1, 5                                                    2b
          call smgwrt( 23034 + i1 )
          Uproff( i1, itvsid ) = Qresp( Hkbdlu, Hphysmin, Hphysmax,
     1      Hntries )
2         continue                                                        2e
c
c       read potentiometers
c
        call stvsrd( ipposn )
c
c       calculate offset
c           simple traverse only
c
        if ( itvsid .ne. 1 ) then                                         2b
          write ( Hvdulu, 3 )
3         format( ' Can''t do this - simple traverse only' )
        else                                                              2m
          Uproff(2,1) = Uproff(2,1) - Qtvdis( ipposn(5), 8 ) - Utvoff(8)
          end if                                                          2e
        end if
      return
      end


      SUBROUTINE SPDTMM( UTV, ISIZE )
c
c       Put maximum and minimum values into the 2nd & 3rd rows of the
c  calibration table .
c
c Arguments
c  integer *2 isize     # index of last value in column ichan of P/D table
c  real  Utv(Hnopts)    # a column of a calibration table
c Variables
c  subroutine swap      # swap 2 4 byte values
c
      implicit none
      integer *2 isize
      real Utv(*)
      include 'tvstvs.cmn'
c
      Utv( Hmaxtvs ) = Utv( isize )
      Utv( Hmintvs ) = Utv( H1sttvs )
      if ( Utv( Hmintvs ) .gt. Utv( Hmaxtvs ) )
     1  call swap( Utv( Hmintvs ), Utv( Hmaxtvs ) )
      return
      end


      REAL FUNCTION QDISPL( IVPOT )
c
c       Move to a position , read the potentiometer and obtain the traverse
c  displacement of DOF IDOF
c
c Argument
c  integer *2 iVpot,            # potentiometer reading : desired & actual
c Variable
c  logical *1 eqabbr            # equivalent to an abbreviated form of ?
c             Equery            # function : read logical
c  integer *2 icntr,            # counter for number of attempts
c             jVpot             # function : reads a potentiometer
c             jljstr            #          : left justify string
c  real Qresp                   # function : real response
c  character Cstr*40            # response from user
c  subroutine sdvpot            # drive one traverse motor only
c             smgwrt            # write message
c             scase             # change case of text
c Messages
c  22363 - Manual move. <CR>
c  22364 - Displ. NOT inverted
c  22365 - Displ. inverted
c  22366 - Size of added gauge block ?
c  22367 - Size of removed gauge block ?
c  22368 - ALL gauge blocks removed
c  22369 - Available non-numerics :- ....
c  22331, 22335, 22337 - Angular displacement
c  22332, 22333, 22334, 22336, 22338 - linear displacement
c
      implicit none
      include 'genacq.kon'
c
      logical *1 eqabbr, Equery
      integer *2 iVpot, icntr, jVpot, jljstr
      real Qresp
      character Cstr*40
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
c           put the DOF in the correct place
c
      if ( fauto .and. 0 .le. iVpot .and. iVpot .lt. 4096 ) then          1b
        call sdvpot( iVpot )
      else                                                                1m
        call smgwrt( 22363 )
        read( Hkbdlu, 1 )
1       format()
        end if                                                            1e
      icntr = 0
      write( Hvdulu, 4 ) rgblock
c
c           GET DISPLACEMENT RESPONSE FROM USER
c
c     repeat                                                                1b
2       call smgwrt( 22330 + ichan )
c
c           read command or displacement
c
        read( Hkbdlu, 3, END=8 ) Cstr
3       format( a )
        if ( jljstr( Cstr ) .le. 0 ) goto 2
c       until( no. of chars > 0 )                                           1e
c
c           selected min or max
c
      call scase( Cstr, Cstr, 'l' )
      if ( eqabbr( Cstr, 'maximum', 3 ) ) then                              1b
        qdispl = rdmax
      else if ( eqabbr( Cstr, 'minimum', 3 ) ) then                         1m
        qdispl = rdmin
c
c           direction inversion
c
      else if ( eqabbr( Cstr, 'invert', 1 ) ) then                          1m
        rdisgn = - rdisgn
        if ( rdisgn .gt. 0 ) then                                           2b
          call smgwrt( 22364 )
        else                                                                2m
          call smgwrt( 22365 )
          end if                                                            2e
        goto 2
c
c           clear all gauge blocks
c
      else if ( eqabbr( Cstr, 'clear', 1 ) ) then                           1m
        rgblock = 0.0
        call smgwrt( 22368 )
        write( Hvdulu, 4 ) rgblock
        goto 2
c
c           add a gauge block
c
      else if ( eqabbr( Cstr, 'add', 1 ) ) then                             1m
        call smgwrt( 22366)
        rgblock = rgblock + Qresp( Hkbdlu, 0.0, Hmaxgblk, Hntries )
        write( Hvdulu, 4 ) rgblock
4       format( ' Total Gauge block length = ', f8.3, ' mm' )
        goto 2
c
c           remove a gauge block
c
      else if ( eqabbr( Cstr, 'remove', 1 ) ) then                          1m
        call smgwrt( 22367 )
        rgblock = rgblock - Qresp( Hkbdlu, 0.0, min(rgblock,Hmaxgblk),
     1    Hntries )
        write( Hvdulu, 4 ) rgblock
        goto 2
      else if ( eqabbr( Cstr, 'status', 1 ) ) then
        write( Hvdulu, 9 ) rdisgn, rdmin, rdmax
9       format( ' displacement sign    = ', f3.0 /
     1          ' minimum displacement = ', f8.3 /
     2          ' maximum displacement = ', f8.3 )
        write( Hvdulu, 4 ) rgblock
c
c           help
c
      else if ( eqabbr( Cstr, 'help', 1 ) ) then                            1m
        call smgwrt( 22369 )
        goto 2
      else                                                                  1m
        read( Cstr, *, ERR=7 ) qdispl
        qdispl = rdisgn * ( qdispl + rgblock )
        if ( rdmin - 3.0 .gt. qdispl .or. qdispl .gt. rdmax + 3.0 ) then    2b
          write( Hvdulu, 5 ) rdmin * rdisgn - rgblock,
     1      rdmax * rdisgn - rgblock
5         format( ' outside limits ', g12.5, ' & ', g12.5,
     1      ' . Try again' )
          icntr = icntr + 1
          if ( icntr .gt. Hntries ) then                                    3b
            write( Hvdulu, 6 )
6           format( ' Get help' )
            stop
            end if                                                          3e
          goto 2
          end if                                                            2e
        write( Hvdulu, 10 ) qdispl
10      format( ' Displacement = ', f10.4 )
        if ( Equery( Hkbdlu, 'Reject (ie. read again) ? ' ) )
     1      goto 2
        end if                                                              1e
      iVpot = jVpot()
      return
c
c       undecipherable response
c
7     call smgwrt( 22369 )
      goto 2
c
c       end of file
c
8     stop
      end


      INTEGER *2 FUNCTION JVPOT
c
c    Reads the IDOFth traverse potentiometer .
c
c  integer *2 ipposn(Hntvch)    # potentiometer readings
c
      implicit none
      include 'tvstvs.cmn'
      integer *2 ipposn(Hntvch)
c
      call stvsrd( ipposn )
      jvpot = ipposn( idof )
      return
      end


      INTEGER *2 FUNCTION JDOFSW( IDOF )
c
c   Converts the DOF number to a bit switch which selects that particular DOF
c Argument
c  integer *2 idof     # DOF number
c
      implicit none
      integer *2 idof
c
      Jdofsw = 2** ( idof -1 )
      return
      end


      SUBROUTINE SDVPOT( IVPOT )
c
c       Drive DOF IDOF of the traverse ITVSID to the position at which the
c  potentiometer reading is IVPOT .
c
c Argument
c  integer *2 iVpot             # required potentiometer reading for IDOF
c Variables
c  logical *1 ok,               # success / failure result
c             sdvtvs,           # function : drive traverse
c             ssttvs            #          : start traverse
c  integer *2 i1,               # looping index
c             ipposn(Hntvch),   # potentiometer positions array
c             Midsbc(3)         # traverse id required by sbctrl
c  subroutine sbwrtq,           # SBC - write queue
c             sbxeqq            # SBC - execute queue
c
      implicit none
      include 'genacq.kon'
      include 'genacq.cmn'
      include 'tvstvs.cmn'
      logical *1 ok, sdvtvs, ssttvs
      integer *2 iVpot, i1, ipposn(Hntvch), Midsbc(3)
c
      data Midsbc / 1, 5, 0 /
c
      do 1 i1 = 1, 8
1       ipposn(i1) = -1
      if ( itvsid .eq. 1 ) then                                           1b
        ipposn(1) = ivpot
      else                                                                1m
        ipposn(idof) = ivpot
        end if                                                            1e
      call sbwrtq
      ok = ssttvs( Midsbc( itvsid ) )
      ok = ok .and. sdvtvs( ipposn )
      if ( .not. ok )
     1  call sfail( Htvserr )
      call sbxeqq( Hnullu, -2 )
      return
      end


      SUBROUTINE PUTTVS( JTVSID, RTVXX )
c
c       Routine to drive the simple traverse only to the physical position
c  y = RTVXX(2)
c
c Arguments
c  integer *2 Jtvsid           # contains 2 ASCII characters describing
c                              # the traverse configuratio
c  real rtvxx(5)               # (x,y,z,theta,phi) traverse coordinate
c
c Variables
c  logical *1 ok,              # success / failure result
c             ssttvs,          # function : SBC - start traverse
c             sdvtvs           #          : SBC - drive traverse
c  integer *2 jtvpot,          # function : convert displacement to pot reading
c             reqd_pot         # required potentiometer reading
c  real ry                     # displacement
c  subroutine sbwrtq,          # SBC - write queue
c             sbxeqq,          # SBC - execute queue
c             stvcrd           # read traverse calibration file
c
      implicit none
      include 'genacq.kon'
c
      logical *1 ok, ssttvs, sdvtvs
      integer *2 reqd_pot, Jtvsid, jtvpot
      real rtvxx(5), ry
      include 'tvstvs.cmn'
c
c       for the first pass, read the calibration file
c
      if ( fpass1 )
     1  call stvcrd
      fpass1 = .false.
c
c       subtract displacement and probe offsets
c
      ry = rtvxx(2) - Utvoff(8) - Uproff(2,1)
      reqd_pot = Jtvpot( ry, 8 )
      write( Hvdulu, 1 )  rtvxx(2), ry, reqd_pot
1     format( ' PUTTVS : position     = ', f10.4 /
     1    10x, 'displacement = ', f10.4 /
     2    10x, 'potentiometer= ', i4 )
c
c       start the traverse
c
      call sbwrtq
      ok = ssttvs( 1 )
c
c       calculate pot required pot reading & drive to it
c
      ok = ok .and. sdvtvs( reqd_pot )
      call sbxeqq( Hnullu, -2 )
      return
      end


      SUBROUTINE GETTVS( JTVSID, RTVXX )
c
c       Routine to read the position of the simple traverse and produce the
c  result RTVXX = ( x, y, z, theta, phi )  - all but y are merely probe
c  offsets .
c
c Arguments
c  integer *2 Jtvsid           # contains 2 ASCII characters describing
c                              # the traverse configuratio
c  real rtvxx(5)               # (x,y,z,theta,phi) traverse coordinate
c
c Variables
c  integer *2 i,               # loop index
c             ipposn(8),       # potentiometer readings
c  real rtvxx(5),              # (x,y,z,theta,phi) traverse position
c       displace               # carriage displacement
c       qtvdis                 # function : pot readings to displacements
c  subroutine stvsrd,          # read all traverse potentiometers
c             stvcrd           # read traverse calibration file
c
      implicit none
      include 'genacq.kon'
c
      integer *2 Jtvsid, i, ipposn(8)
      real displace, rtvxx(5), qtvdis
      include 'tvstvs.cmn'
c
c       for first pass , read calibration file
c
      if ( fpass1 )
     1  call stvcrd
      fpass1 = .false.
c
c       read traverse potentiometers
c       put  position = probe offsets
c
      do 1 i = 1, 5
1       rtvxx(i) = Uproff( i, 1 )
c
c       read & calculate y position
c    = probe offset + displacement offset + displacement( pot reading )
c
      idof = 5
      ichan = Hntvch
      itvsid = 1
      call stvsrd( ipposn )
      displace = qtvdis( ipposn(5), 8 )
      rtvxx(2) = rtvxx(2) + Utvoff(8) + displace
      write( Hvdulu, 2 ) ipposn(5), displace, rtvxx(2)
2     format( ' GETTVS : potentiometer= ', i4 /
     1    10x, 'displacement = ', f10.4 /
     2    10x, 'position     = ', f10.4 )
      return
      end


      SUBROUTINE STVCRD
c
c       Routine to read the traverse calibration file .
c
      implicit none
      include 'genacq.kon'
      include 'genacq.cmn'
      include 'tvstvs.cmn'
c
      open( UNIT=Htvslu, FILE=Ctvsfil, ACCESS='sequential',
     1  FORM='unformatted', STATUS='old', ERR=1 )
      read( Htvslu, END=1, ERR=1 ) Mtvcal
      rewind( UNIT=Htvslu )
      return
c
1     write( Hvdulu, 2 )
2     format( ' STVCRD : FATAL : Can''t read CALTVS.DAT' )
      stop
      end


      SUBROUTINE STVSRD( IPPOSN )
c
c    Reads all traverse potentiometers .
c
c Argument
c  integer *2 ipposn(8),        # potentiometer readings
c
c Variable
c  logical *1 ok,               # success / failure result
c             ssttvs,           # SBC - start traverse
c             ssstvs            # SBC - read traverse ( serial line )
c  integer *2 Midsbc(3),        # traverse id required by sbctrl
c             idata(10),        # data received from SBC
c             i                 # looping index
c  integer *4 dnbytes           # number of data bytes received
c  subroutine sbxeqq,           # SBC - execute command queue
c             sbwrtq            # SBC - write command queue
c
      implicit none
      include 'genacq.kon'
      logical *1 ok, ssttvs, ssstvs
      integer *2 ipposn(8), Midsbc(3), idata(10), i
      integer *4 dnbytes
c
      include 'tvstvs.cmn'
      equivalence ( dnbytes, idata(1) )
      data Midsbc / 1, 5, 0 /
c
      call sbwrtq
      ok = ssttvs( Midsbc( itvsid ) )
      ok = ok .and. ssstvs()
      call sbxeqq( idata, -1 )
      if ( dnbytes .ne. 16 ) then
        write( Hvdulu, 1 )
1       format( 'STVSRD : Read wrong no. of bytes' )
        end if
      do 2 i = 1, 8
2       ipposn(i) = idata( i+2 )
      return
      end


c REAL FUNCTION QTVDIS( ITVPOT, JCHAN )
c INTEGER *2 FUNCTION JTVPOT( RTVDIS, JCHAN )
c
c Functions to convert :-
c       (1) potentiometer readings to traverse displacements
c       (2) traverse displacements to potentiometer readings
c
c Function parameters :-
c       Qtvdis :- output traverse displacement
c       Rtvdis :- input traverse displacement
c       Jtvpot :- output potentiometer reading
c       Itvpot :- input potentiometer reading
c       Ichno  :- traverse drive channel number :-
c  1            1 = plug yaw
c  2            2 = plug translation
c  3            3 = main tower outside normal displacement
c  4            4 = main tower inside normal displacement
c  5            5 = main tower yaw
c  6            6 = auxiliary tower normal displacement
c  7            7 = auxiliary tower yaw
c  5            8 = old (Kim Lim) tower
c
c Calibration Data :
c    The { potentiometer reading } <==> { traverse displacement }
c  calibration data for each traverse channel is contained in the arrays
c  Utvpot and Utvdis with the indices ( row, column ) where :
c      column number = traverse drive channel number
c      row number = coordinate pair number
c  i.e. a single coordinate pair is  Utvpot( I, Ichno ) , Utvdis( I, Ichno )
c    Utvpot is the array of traverse potentiometer readings .
c    Utcdis is the array of traverse displacements .
c    The first row of the array contains ordinate values to be used if the
c      abscissa value is out of range .
c    The second row contains the minimum value in each column .
c    The third row contains the maximum value in each column .
c
c    The smoothing polynomials are in the array Utvpol . The Ichno'th column
c    in the array contains the polynomial for channel 'Ichno' .
c
c Method :
c   If a smooth polynomial for calculating displacement from potentiometer
c   reading exists, it is used for the conversion . QTVDIS uses it directly;
c   JTVPOT inverts it by obtaining a first estimate by linear interpolation
c   from the table of coordinate pairs and then doing an iterative refinement.
c   If no polynomial exists, linear interpolation from the table of coordinate
c   pairs is done . A binary search of the input variable table is done until
c   a value which straddles the input variable value is found .
c
c  The table arrays Utvpot & Utvdis and the polynomials Utvpol are for
c  compensating for the potentiometer nonlinearities . The error arising from
c  amplifier and other electronic drift is corrected at initialization time .
c  For converting from potentiometer readings to traverse displacement :-
c       { corrected pot. voltage } = { basic pot. voltage } * _
c               {drift gradient correction } + { drift offset correction }


      REAL FUNCTION QTVDIS( ITVPOT, JCHAN )
c
c       Converts potentiometer readings into traverse displacement .
c
c Argument
c  integer itvpot,              # input ( uncorrected ) potentiometer reading
c          jchan                # traverse channel number
c Variable
c  real Rtvpot,                 # corrected potentiometer reading
c       Qlrpl1                  # function : linear interpolation
c
      implicit none
      include 'genacq.kon'
c
      integer *2 itvpot, jchan
      real Rtvpot, Qlrpl1, Qpoly
c
      include 'tvstvs.cmn'
c
c       apply drift offset correction
c
      Rtvpot = real( itvpot ) * Upddgc( jchan ) + Upddoc( jchan )
c
c       potentiometer reading is too large or too small
c
      if ( Rtvpot .lt. Utvpot( Hmintvs, jchan ) - 2.0 .or.
     1     Rtvpot .gt. Utvpot( Hmaxtvs, jchan ) + 2.0 ) then               1b
        Qtvdis = Utvdis( 1, jchan )
        write( Hvdulu, 1 ) jchan, itvpot, Qtvdis
1       format( ' QTVDIS : Pot. on channel ', i2, ' out of range' /
     1    6x, 'pot. = ', i6 / 6x, 'displ. = ', f9.3 )
c
c       use linear interpolation
c
      else if ( Mtvord(jchan) .le. 0 .or.
     1          Mtvord(jchan) .gt. Hmaxpol - 1 ) then                      1m
        Qtvdis = Qlrpl1( Utvpot( H1sttvs, jchan ),
     1    Utvdis( H1sttvs, jchan ), 1, Mpdsiz(jchan), Rtvpot )
c      WRITE(6,9999) RTVPOT, QTVDIS
c9999  FORMAT( ' QTVDIS(', G14.7, ')=  ', G14.7, ' LINEAR INTERPOLATION')
c
c       use fitted polynomial
c
      else
        Qtvdis = qpoly( Rtvpot, Utvpol(1,jchan), Mtvord(jchan) )
c      WRITE(6,9998) RTVPOT, QTVDIS
c9998  FORMAT( ' QTVDIS(', G14.7, ')=  ', G14.7, ' USING POLYNOMIAL' )
        end if                                                             1e
      write( Hvdulu, 2 ) itvpot, rtvpot, qtvdis
2     format( ' QTVDIS : potentiometer reading = ', i4 /
     1    10x, 'corrected pot reading = ', f9.3 /
     2    10x, 'traverse displacement = ', f8.4 )
      return
      end


      INTEGER *2 FUNCTION  JTVPOT( RTVDIS, JCHAN )
c
c       Converts traverse displacement into potentiometer reading .
c
c Arguments
c  real  Rtvdis        # traverse displacement
c  integer *2 jchan    # traverse channel number
c             straddle # indices of tabulated potentiometer values
c                      # straddling the target value
c             Jbsrch   # function : binary search
c Variables
c  real Qlrpl1          # function : linear interpolation
c
      implicit none
      include 'genacq.kon'
c
      integer *2 jchan, jcomch, straddle, Jbsrch
      real Rtvdis, rcomds, rtvpot, qzerli, Qlrpl1
      external qtvplz
      include 'tvstvs.cmn'
      common /tvsinv/ rcomds, jcomch
c
c         reject if displacement too large or too small
c
      if ( Rtvdis .lt. Utvdis( Hmintvs, jchan ) - 0.0005 .or.
     1    Rtvdis .gt. Utvdis( Hmaxtvs, jchan ) + 0.0005 ) then             1b
        rtvpot = Utvpot( 1, jchan )
        write( Hvdulu, 1 ) jchan, jtvpot, rtvdis
1       format( ' JTVPOT : Displ. on channel ', i2, ' out of range' /
     1    6x, 'pot. = ', i6 / 6x, 'displ. = ', f9.3 )
c
c       use linear interpolation
c
      else if ( Mtvord(jchan) .le. 0 .or.
     1          Mtvord(jchan) .gt. Hmaxpol - 1 ) then                      1m
        rtvpot = Qlrpl1( Utvdis(H1sttvs,jchan),
     1      Utvpot(H1sttvs,jchan), 1, Mpdsiz(jchan)-H1sttvs-1, Rtvdis )
c      WRITE(6,9999) RTVDIS, RTVPOT
c9999  FORMAT( ' JTVPOT( ', G14.7, ') = ', G14.7,
c     1        '  LINEAR INTERPOLATION' )
c
c       refine by inverting fitted polynomial
c
      else                                                                 1m
        straddle = max( H1sttvs, min( Mpdsiz(jchan),
     1        Jbsrch( Utvdis(H1sttvs,jchan), 1, Mpdsiz(jchan)-H1sttvs-1,
     2                Rtvdis ) + H1sttvs - 1 ) )
        jcomch = jchan
        rcomds = rtvdis
        rtvpot = Qzerli( Utvpot(straddle,jchan),
     1                   Utvpot(straddle+1,jchan), Qtvplz, 0.001 )
c      WRITE(6,9998) RTVDIS, RTVPOT, STRADDLE, UTVPOT(STRADDLE,JCHAN),
c     1              UTVPOT(STRADDLE+1,JCHAN)
c9998  FORMAT( ' JTVPOT( ', G14.7, ') = ', G14.7,
c     1        '  INVERTING POLYNOMIAL' /
c     2        ' STRADDLE,INITIAL_2 = ', I3, 3X, 2G15.7 )
        end if                                                             1e
      Jtvpot = ( rtvpot - Upddoc(jchan) ) / Upddgc(jchan) + 0.5
      write( Hvdulu, 2 ) rtvdis, rtvpot, jtvpot
2     format( ' JTVDIS : traverse displacement = ', i4 /
     1    10x, 'corrected pot reading = ', f9.3 /
     2    10x, 'potentiometer reading = ', f8.4 )
      return
      end


      REAL FUNCTION QTVPLZ( POT )
c
c  Evaluates the difference between the actual displacement 'rcomds' and the
c  displacement calculated from the fitted polynomial for traverse channel
c  'jcomch' at potentiometer reading POT .
c
      implicit none
      integer *2 jcomch
      real rcomds, pot, qpoly
      include 'tvstvs.cmn'
      common /tvsinv/ rcomds, jcomch
c
      if ( pot .lt. Utvpot(Hmintvs,jcomch) ) then
        qtvplz = Utvdis(Hmintvs,jcomch)
      else if ( pot .gt. Utvpot(Hmaxtvs,jcomch) ) then
        qtvplz = Utvdis(Hmaxtvs,jcomch)
      else
        qtvplz = rcomds - qpoly( pot, Utvpol(1,jcomch), Mtvord(jcomch) )
        end if
c      WRITE(6,9999) POT, QTVPLZ
c9999  FORMAT( ' QTVPLZ( ', G14.7, ') = ', G14.7 )
      return
      end


      BLOCK DATA TVSBLK
c
c           Initializes the TVSTVS ( traverse ) common block
c
      implicit none
      include 'tvstvs.cmn'
c
      data fpass1, ftvpdt / .true. , 8* .false. /
      data rdisgn, rgblock, Utvpot, Utvdis / 1.0, 0.0, 480*999999. /
      end
