c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      LOGICAL *1 FUNCTION DAOPEN( IFILELU, CFILNAM, IRECLEN )
c
c        Direct Access file OPEN
c Routine to perform open a direct access file with name IFILNAM and logical
c unit number IFILELU . The record length is fixed at IRECLEN bytes . There is
c no associated variable . If the file was opened successfully the returned
c value is DAOPEN=.true. ; otherwise DAOPEN=.false.
c N.B. 1) Maximum record length is 512 bytes
c
c    Arguments
c integer *2 ifilelu            # logical unit number of the file
c character*(*) cfilnam         # file name
c integer *2 ireclen            # length in  bytes of each file record
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c     Hbuflen                   # buffer length in bytes
c    Variables
c logical *1 gopbuf(Hbuflen),   # output buffer array
c            gipbuf(Hbuflen),   # input buffer array
c            fopen              # file is open
c integer *2 iopptr,iipptr,     # output and input buffer pointers
c            lenrec,            # length in bytes of each file record
c            dlrec              # length in words of each file record
c            iflu               # file logical unit number
c
      integer *2 Hvdulu, Hbuflen, Hbuflen2
      parameter( Hvdulu=6, Hbuflen=512, Hbuflen2=256 )
c
      integer *2 ifilelu, ireclen
      character *(*) cfilnam
c
c               COMMON BLOCK DECLARATION
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
c
      lenrec = max( min( ireclen / 2 * 2, Hbuflen ), 2 )
      iflu   = ifilelu
      iopptr = 1
      iipptr = lenrec + 1
      dlrec  = lenrec / 2
      open( UNIT=ifilelu, FILE=cfilnam, ACCESS='DIRECT', RECL=lenrec,
     1      STATUS='unknown', ERR=1 )
      daopen = .true.
      fopen  = .true.
      return
c
c           error message
c
1     write ( Hvdulu, 2 )
2     format( ' DAOPEN : Error' )
      daopen = .false.
      fopen  = .false.
      return
      end


      LOGICAL *1 FUNCTION DAPUT( NREC, DATA, LENGTH )
c
c       Appends the first LENGTH bytes of the data array DATA onto record NREC
c  of the direct access binary file opened by DAOPEN .
c       Pointers IOPPTR points to the next byte in GOPBUF to be written .
c
c    Arguments
c logical *1 data(*)            # data array
c integer *2 nrec,              # record number to be written
c            length,            # length ( in bytes ) of the DATA array
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c     Hbuflen                   # buffer length in bytes
c     Hbuflen2                  # buffer length in words
c integer *2 i                  # byte counter for loading the buffer
c            j                  # word counter for writing the buffer
c            dopbuf(Hbuflen2)   # output data buffer
c
      integer *2 Hvdulu, Hbuflen, Hbuflen2
      parameter( Hvdulu=6, Hbuflen=512, Hbuflen2=256 )
c
      logical *1 data(*)
      integer *2 i, j, nrec, length, dopbuf(Hbuflen2)
c
c               COMMON BLOCK DECLARATION
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
      equivalence ( gopbuf(1), dopbuf(1) )
c
      if ( fopen ) then
        do 1 i = 1, length
          gopbuf( iopptr ) = data(i)
          iopptr = iopptr + 1
          if ( iopptr .gt. lenrec ) then
            iopptr = 1
            write( iflu, REC=nrec, ERR=3 ) ( dopbuf(j), j = 1, dlrec )
            nrec = nrec + 1
            end if
1         continue
      else
        write( Hvdulu, 5 )
5       format( ' DAPUT : ABORT : File not open' )
        end if
      daput = fopen
      return
c
c           error message
c
3     write ( Hvdulu, 4 )
4     format( ' DAPUT : ABORT : Error writing to file' )
      daput = .false.
      return
      end


      LOGICAL *1 FUNCTION DAPUTC( NREC, CSTR )
c
c       Appends the character string CSTR onto record NREC
c  of the direct access binary file opened by DAOPEN .
c       Pointer IOPPTR points to the next byte in GOPBUF to be written .
c
c    Arguments
c integer *2 nrec,              # record number to be written
c character Cstr*(*)            # length ( in bytes ) of the DATA array
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c     Hbuflen                   # buffer length in bytes
c     Hbuflen2                  # buffer length in words
c logical *1 fbuffer(2)         # character buffer
c integer *2 i,                 # byte counter for loading the buffer
c            j,                 # word counter for writing the buffer
c            dopbuf(Hbuflen2),  # output data buffer
c            ibuffer            # character buffer
c
      integer *2 Hvdulu, Hbuflen, Hbuflen2
      parameter( Hvdulu=6, Hbuflen=512, Hbuflen2=256 )
c
      logical *1 fbuffer(2)
      integer *2 i, j, nrec, dopbuf(Hbuflen2), ibuffer
      character Cstr*(*)
      equivalence( fbuffer(1), ibuffer )
c
c               COMMON BLOCK DECLARATION
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
      equivalence ( gopbuf(1), dopbuf(1) )
c
      if ( fopen ) then
        do 1 i = 1, len( Cstr )
          ibuffer = ichar( Cstr(i:i) )
          gopbuf( iopptr ) = fbuffer(1)
          iopptr = iopptr + 1
          if ( iopptr .gt. lenrec ) then
            iopptr = 1
            write( iflu, REC=nrec, ERR=3 ) ( dopbuf(j), j = 1, dlrec )
            nrec = nrec + 1
            end if
1         continue
      else
        write( Hvdulu, 5 )
5       format( ' DAPUTC : ABORT : File not open' )
        end if
      daputc = fopen
      return
c
c           error message
c
3     write ( Hvdulu, 4 )
4     format( ' DAPUTC : ABORT : Error writing to file' )
      daputc = .false.
      return
      end


      LOGICAL *1 FUNCTION DAGET( NREC, DATA, LENGTH )
c
c  Reads the next LENGTH  bytes from record NREC of the direct access
c  binary file opened by DAOPEN into the array DATA .
c       Pointer IIPPTR points to the next byte to be read in GIPBUF .
c
c  N.B. If the read is successful the returned result is DAGET=.true.
c       If a read error occurs , the result is DAGET=.false. and an
c       error message is displayed .
c
c    Arguments
c logical *1 data(length)       # data array
c integer *2 nrec,              # record number to be read
c            length             # length ( in bytes ) of the data array
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c     Hbuflen                   # buffer length in bytes
c     Hbuflen2                  # buffer length in words
c integer *2 i                  # byte counter for unloading the buffer
c            j                  # word counter for reading the buffer
c            dopbuf(Hbuflen2)   # output data buffer
c
      logical *1 data(*)
      integer *2 Hvdulu, Hbuflen, Hbuflen2
      parameter( Hvdulu=6, Hbuflen=512, Hbuflen2=256 )
c
      integer *2 i, j, dipbuf(Hbuflen2), length, nrec
c
c               COMMON BLOCK DECLARATION
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
      equivalence ( gipbuf(1), dipbuf(1) )
c
      if ( fopen ) then
        do 1 i = 1,length
          if ( iipptr .gt. lenrec ) then
            iipptr = 1
            read( iflu, REC=nrec, ERR=3 ) ( dipbuf(j), j = 1, dlrec )
            nrec = nrec + 1
            end if
          data(i) = gipbuf( iipptr )
          iipptr = iipptr + 1
1         continue
      else
        write( Hvdulu, 5 )
5       format( ' DAGET : ABORT : File not open' )
        end if
      daget = fopen
      return
c
c           error message
c
3     write ( Hvdulu, 4 )
4     format( ' DAGET : ABORT : Error reading from file' )
      daget = .false.
      return
      end


      LOGICAL *1 FUNCTION DAFLUSH( NREC )
c
c       Flushes the data remaining in the output buffer GOPBUF onto record
c  NREC of the direct access file opened by DAOPEN .
c       Pointer IIPPTR points to the next byte in GOPBUF to be written
c
c    Argument
c integer *2 nrec               # record number to be written
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c     Hbuflen                   # buffer length in bytes
c     Hbuflen2                  # buffer length in words
c integer *2 i                  # word counter for writing the buffer
c            dopbuf(Hbuflen2)   # output data buffer
c
      integer *2 Hvdulu, Hbuflen, Hbuflen2
      parameter( Hvdulu=6, Hbuflen=512, Hbuflen2=256 )
c
      integer *2 j, dopbuf(Hbuflen2), nrec
c
c               COMMON BLOCK DECLARATION
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
      equivalence ( gopbuf(1), dopbuf(1) )
c
      if ( fopen ) then
        if ( iopptr .gt. 1 ) then
          do 1 j = iopptr, lenrec
1           gopbuf(j) = .false.
          write( iflu, REC=nrec, ERR=2 ) ( dopbuf(j), j = 1, dlrec )
          nrec = nrec + 1
          iopptr = 1
          end if
      else
        write( Hvdulu, 5 )
5       format( ' DAPUT : ABORT : File not open' )
        end if
      daflush = fopen
      return
c
c             error message
c
2     write ( Hvdulu, 3 )
3     format( ' DAFLUSH : ABORT : Error writing to file' )
      daflush = .false.
      return
      end


      SUBROUTINE DAINIT
c
c       Initializes so that the next call to DAGET reads from the beginning of
c  the record NREC from direct access binary file opened by DAOPEN .
c  If the associated variable is NREC which is used to sequentially
c  access the file , this causes NREC to be the next record .
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c     Hbuflen                   # buffer length in bytes
c     Hbuflen2                  # buffer length in words
c integer *2 i                  # byte counter for loading the buffer
c            dopbuf(Hbuflen2)   # output data buffer
c
      integer *2 Hvdulu, Hbuflen
      parameter( Hvdulu=6, Hbuflen=512 )
c
c               COMMON BLOCK DECLARATION
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
c
      iipptr = lenrec + 1
      return
      end


      CHARACTER *32 FUNCTION CBINARY( ITEM, NBITS )
c
c       Converts a 4 byte item into its its corresponding binary bit pattern
c
c Arguments
c  integer *2 item      # data item to be encoded
c             nbits     # number of bits to be encoded
c
c Constant
c  Hnbits=32            # max no of bits in data item
c Variables
c  integer *2 i,        # looping index
c  integer *4 imask     # bit mask
c
      implicit none
      integer *2 Hnbits
      parameter( Hnbits=32 )
c
      integer *4 item, imask
      integer *2 nbits, i
c
      do 1 i = 1, nbits
1       Cbinary(i:i) = '0'
      do 2 i = nbits + 1, Hnbits
2       Cbinary(i:i) = ' '
      imask = 1
      do 3 i = nbits, 1, -1
        if ( iand( item, imask ) .ne. 0 )
     1    Cbinary(i:i) = '1'
3       imask = ishft( imask, 1 )
      return
      end


      LOGICAL *1 FUNCTION EFACT( DPRDT, FACT, N )
c
c       Approximately factorizes the 4 byte integer value DPRD into N single
c  byte values as accurately as possible . The result EFACT is .true. if all
c  factors are less than 256 . The maximum value of N is 24 .
c
c Arguments
c  integer *4 Dprdt         # the number to be factorized
c  integer *2 fact(*)       # factors produced - single byte values
c             n             # number of factors required
c
c constants
c  Hnprimes=60              # number of primes < 256
c
c integer *4 Dprd,      # operand : ie the number to be factorized
c            Dprime,    # prime number
c integer *2 ifact(24),         # factors - 2 byte values
c         Iprime(Hnprimes),     # list of prime numbers up to 256
c         i, j,                 # looping index
c         iinc,              # increment for Dprdt after unsuccessful attempt
c         imin,                 # index of the best prime factor found so far
c         imax,                 # biggest factor found so far
c         max,                  # index of imax
c         n,                    # number of factors
c         nfact,                # number of factors found
c real rem,             # remainder
c      remmin           # smallest remainder so far found
c subroutine swapi      # swap 2 2 byte values
c
      implicit none
      integer *2 Hnprimes
      parameter( Hnprimes=60 )
c
      integer *4 Dprdt, Dprd, Dprime
      integer *2 fact(*), ifact(24), Iprime(Hnprimes), i, j, iinc,
     1           imin, imax, max, n, nfact
      real rem, remmin
      save Iprime
c
      data Iprime/ 256, 255, 254, 253, 252, 250,
     1   2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59,
     2   61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127,
     3   131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191,
     4   193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251 /
c
      do 1 i = 1, n
1       ifact(i) = 1
      efact = .true.
      if ( dprdt .ne. 1 ) then                                              1b
        iinc = 1
c       repeat                                                              2b
2         dprd = dprdt
          efact = .true.
          nfact = 0
c         repeat                                                            3b
3           nfact = nfact + 1
            remmin = 1.0
            do 4 j = 1, Hnprimes                                            4b
c
c               calculate fractional remainder
c
              Dprime = Iprime(j)
              if ( j .gt. 6 .and. Dprime .gt. dprd ) goto 5
              rem = real( dprd - dprd/dprime * dprime ) / real(Dprime)
c
c               minimize remainder
c
              if ( rem .lt. abs( remmin ) ) then                            5b
                remmin = rem
                imin = j
                end if                                                      5e
              if ( 1 - rem .lt. abs( remmin ) ) then                        5b
                remmin = rem -1
                imin = j
                end if                                                      5e
c
c               exit from loop on zero remainder
c
              if ( abs( remmin ) .lt. 0.002 ) goto 5
4             continue                                                      4e
c
c               select next factor & adjust operand
c
5           ifact( nfact ) = iprime( imin )
            Dprime = iprime(imin)
            Dprd = Dprd / Dprime
            if ( remmin .lt. 0.0 )
     1        dprd = dprd + 1
            if ( dprd .gt. 1 ) goto 3
c           until ( dprd .le. 1 )                                           3e
c
c               arrange factors in descending order
c
          Dprd = 1
          do 7 i = 1, nfact - 1                                             3b
            imax = ifact(i)
            max = i
            do 6 j = i + 1, nfact                                           4b
              if ( ifact(j) .gt. imax ) then                                5b
                imax = ifact(j)
                max = j
                end if                                                      5e
6             continue                                                      4e
            call swapi( ifact(max), ifact(i) )
7           Dprd = Dprd * ifact(i)                                          3e
          Dprd = Dprd * ifact(nfact)
c
c               distribute factors , starting with the biggest
c
          imax = 0
          if ( nfact .gt. n ) then                                          3b
            j = 1
            i = n + 1
c           start of loop                                                   4b
8             if ( ifact(i) * ifact(j) .le. 256 ) then                      5b
                ifact(j) = ifact(i) * ifact(j)
              else                                                          5m
c
c               if factor is too big shift to the next
c
                imax = imax + 1
                i = i - 1
                if ( imax .ge. n ) then                                     6b
                  efact = .false.
                  goto 9
                  end if                                                    6e
                end if                                                      5e
              j = j + 1
              if ( j .gt. n ) then                                          5b
                j = 1
                imax = 0
                end if                                                      5e
              i = i + 1
              if ( i .le. nfact ) goto 8
c             end of loop 4                                                 4e
9           end if                                                          3e
          if ( .not. efact ) then                                           3b
            dprdt = dprdt + iinc
            iinc = isign( iabs( iinc ) + 1, -iinc )
            end if                                                          3e
          if ( iinc .lt. -255 .or. .not. efact ) goto 2
c         until( iinc < -255 | efact )                                      2e
        end if                                                              1e
c
c               put factors into byte array
c
      do 10 i = 1, n
10      fact(i) = ifact(i)
      dprdt = dprd
      return
      end


      LOGICAL *1 FUNCTION EIN2( ITEM, ISET, NITEMS )
c
c  Test if the 2 byte integer ITEM  is in the list of integers ISET which
c  contains NITEMS items .
c
c Arguments
c  integer *2 item,     # test item
c             iset(*)   # test for membership of this list
c             nitems    # length of list
c Variables
c  integer *2 i,        # looping index
c
      implicit none
      integer *2 item, iset(*), nitems, i
c
      ein2 = .false.
      do 1 i = 1, nitems
1       ein2 = item .eq. iset(i) .or. ein2
      return
      end


      LOGICAL *1 FUNCTION EMGCLS
c
c       Close the currently open message file ( if any )
c  The returned value of EMGCLS is .true. if a message file was open ,
c                                  .false. if no message file was not open .
c Constant
c  Hdirlen=512                  # max. number of messages
c
      implicit none
      integer *2 Hdirlen
      parameter( Hdirlen=512 )
c
      logical*1 fopen
      integer *2 dirmsg(Hdirlen), dirrec(Hdirlen), msglu, nomsgs,
     1  norecs, recde, reclen
c
      common /Af77liba/ dirmsg, dirrec, msglu, nomsgs, norecs, recde,
     1  reclen, fopen
c
      emgcls = fopen
      if ( fopen ) then
        close( msglu )
        fopen = .false.
        end if
      return
      end


      BLOCK DATA EMGBLK
c
c       common block for message file handling
c
c Constant
c  Hdirlen=512                  # max. number of messages
c Variables
c  logical*1 fopen              # message file is open
c integer *2 dirmsg(Hdirlen),   # directory array of message numbers
c            dirrec(Hdirlen),   # directory array of message start record nos.
c            msglu,             # logical unit number of message file
c            nomsgs,            # number of messages in random access file
c            norecs,            # number of records in random access file
c            recde,             # record number of the last directory entry
c            reclen             # record length (bytes) in random access file
c
      implicit none
      integer *2 Hdirlen
      parameter( Hdirlen=512 )
c
      logical*1 fopen
      integer *2 dirmsg(Hdirlen), dirrec(Hdirlen), msglu, nomsgs,
     1  norecs, recde, reclen
c
      common /Af77liba/ dirmsg, dirrec, msglu, nomsgs, norecs, recde,
     1  reclen, fopen
c
      data fopen / .false. /
      end


      LOGICAL *1 FUNCTION EMGOPN( IMSGLU, CFILNAM )
c
c       MessaGe ( OPeN file )
c
c    The logical function EMGOPN explicitly opens the named message FILNAM
c  as logical unit number IMSGLU . The result is :--
c       EMGOPN = .true. if the open is successful
c                .false. if the open is unsuccessful
c Arguments
c  integer *2 imsglu            # logical unit number of opened message file
c  character cfilnam*(*)        # message file name
c
c Constants
c  Hvdulu=6                     # vdu logical unit number
c  Hdirlen=512                  # max number of messages
c Variables
c  integer *2 msgno,            # directory index
c             msgst,msgend,     # start & stop indices for reading directory
c             irec,             # record pointer variable
c
      implicit none
      integer *2 Hvdulu, Hdirlen
      parameter( Hvdulu=6, Hdirlen=512 )
c
      integer *2 imsglu, msgno, msgst, msgend, irec
      character cfilnam*(*)
c
c           COMMON BLOCK DECLARATION
c
      logical*1 fopen
      integer *2 dirmsg(Hdirlen), dirrec(Hdirlen), msglu, nomsgs,
     1  norecs, recde, reclen
c
      common /Af77liba/ dirmsg, dirrec, msglu, nomsgs, norecs, recde,
     1  reclen, fopen
c
      emgopn = .false.
      if ( fopen )
     1  close( msglu, ERR=98 )
      fopen = .false.
      msglu = imsglu
      open( msglu, FILE=cfilnam, ACCESS='direct', FORM='unformatted',
     1      STATUS='old', ERR=98 )
      read( msglu, REC=1, ERR=96 ) norecs, reclen, recde, nomsgs
      if ( norecs .lt. 3 .or. recde .ne. nomsgs / 21 + 2 .or.
     1     nomsgs .lt. 1 .or. nomsgs .gt. Hdirlen - 1 ) then
        write( Hvdulu, 1 ) norecs, reclen, recde, nomsgs
1       format ( ' EMGOPN : ABORT : File format error' /
     1    11x, 'Norecs = ', i7, '  reclen = ', i7,'  recde = ', i7,
     2    '  nomsgs = ', i7 )
      else
        msgst = 1
        do 2 irec = 2,recde
          msgend = min0( msgst+20 , nomsgs+1 )
          read( msglu, REC=irec, ERR=96 )
     1      ( dirmsg(msgno), dirrec(msgno), msgno = msgst, msgend )
2         msgst = msgst + 21
        emgopn = .true.
        fopen = .true.
        end if
      return
c
c           error messages
c
96    write( Hvdulu, 97 )
97    format( ' EMGOPN : ABORT : Can''t read message file' )
      return
98    write( Hvdulu, 99 )
99    format( ' EMGOPN : ABORT : Can''t open message file' )
      return
      end


      LOGICAL *1 FUNCTION EMGWRT( IOPFLU, IMSG )
c                  Message Write
c
c The logical function EMGWRT selects and displays the message number IMSG
c which resides on the opened message file . The
c random access message file must have been previously generated from an ASCII
c sequential access file by the program MSGCRT ( MeSsaGe CReaTe ) . The
c message is 'displayed' on the output file with logical unit number IOPFLU .
c
c  EMGWRT = .true.   if the message is correctly retrieved and displayed
c         = .false.  if their is an error in the message file or the message
c                         cannot be displayed
c arguments
c  integer *2 iopflu            # output file logical unit number
c             imsg              # message identifier
c constants
c   Hdirlen=512         # maximum number of messages allowed in a file
c   Hvdulu=6            # logical unit number of video display unit
c   Hlinlen=84          # maximum number of characters in a line
c variables
c integer *2 i,                 # message counter
c         IMSG,                 # number of the message to be displayed
c         nstart,nfinis,        # start & finish records for a message
c         irec,                 # record number to be written
c character Cmsg*(*)            # array containing a message line
c
      implicit none
      integer *2 Hvdulu, Hdirlen
      parameter( Hvdulu=6, Hdirlen=512 )
c
      integer *2 i, IMSG, nstart, nfinis, irec, iopflu
      character Cmsg*(84)
c
      logical*1 fopen
      integer *2 dirmsg(Hdirlen), dirrec(Hdirlen), msglu, nomsgs,
     1  norecs, recde, reclen
c
      common /Af77liba/ dirmsg, dirrec, msglu, nomsgs, norecs, recde,
     1  reclen, fopen
c
c           find first record of message
c
      emgwrt = .false.
      if ( fopen ) then
        i = 1
c       start of loop
5         if ( imsg .eq. dirmsg(i) ) then
            nstart = dirrec(i)
c           repeat
1             i = i +1
              nfinis = dirrec(i) - 1
              if ( nfinis .lt. nstart .and. i .lt. nomsgs ) goto 1
c             until ( nfinis >= nstart | i >= nomsgs )
            if ( nstart .gt. recde .and. nstart .le. nfinis .and.
     1          nfinis .le. norecs ) then
              emgwrt = .true.
c
c             display the message
c
              do 2 irec = nstart, nfinis
                read( msglu, REC=irec, ERR=98 ) Cmsg(1:reclen)
2               write( iopflu, 3 ) charnb( Cmsg( 1: reclen ) )
3               format( 1x, a )
            else if ( nstart .eq. nfinis + 1 ) then
              emgwrt = .true.
            else
              write( Hvdulu, 4 ) i, imsg, nstart, nfinis
4             format( ' EMGWRT : ABORT : File directory error' /
     1          6x, 'Entry no. ', i5, ' message id = ', i6,
     2          ' start record = ', i6, ' finish record = ', i6 )
              end if
            return
            end if
          i = i + 1
          if ( i .le. nomsgs ) goto 5
c         end of loop
        write( Hvdulu, 6 )
6       format( ' EMGWRT : WARNING : Message not found' )
      else
        write( Hvdulu, 7 )
7       format( ' EMGWRT : ABORT : Message file not open' )
        end if
      return
c
c           Error message
c
98    write( Hvdulu, 99 )
99    format( ' EMGWRT : ABORT : Error reading message file' )
      return
      end


      LOGICAL *1 FUNCTION EQABBR( CSTR, CTEST, MINLEN )
c
c       EQual ABBReviation
c     Tests if the character string CSTR is a legal abbreviation of the
c character string CTEST .
c
c Arguments
c  character cstr*(*),         # operand character string               INPUT
c            ctest*(*)         # test string                            INPUT
c  integer *2 minlen           # minimum number of characters allowed   INPUT
c                              # in the abbreviation
c Variables
c  integer *2 i,       # indexing counter
c             iend     # first letter in a word
c
      implicit none
      integer *2 i, iend, minlen
      character cstr*(*), ctest*(*)
c
c         find end of the operand string
c
      iend = len( cstr )
1     if ( iend .gt. 1 .and. cstr(iend:iend) .eq. ' ' ) then              1b
        iend = iend - 1
        goto 1
        end if                                                            1e
c
c         test
c
      if ( iend .gt. len( ctest ) ) then                                  1b
        eqabbr = .false.
      else                                                                1m
        eqabbr = .true.
        i = 1
2       if ( i .le. iend .and. eqabbr ) then                              2b
          eqabbr = eqabbr .and. cstr(i:i) .eq. ctest(i:i)
          i = i + 1
          goto 2
          end if                                                          2e
        eqabbr = eqabbr .and. i .gt. minlen
        end if                                                            1e
      return
      end


      LOGICAL *1 FUNCTION EQSIGN( X, Y )
c
c Returns .True. if x and y have the same sign
c
      implicit none
      real *4 x,y
c
      eqsign = x * y .ge. 0.0
      return
      end


      LOGICAL *1 FUNCTION EQUERY( IPLUN, MSG )
c
c       Displays a query on the vdu and receives a logical response with
c  logical unit number IPLUN .
c
c    Arguments
c integer *2 iplun              # input device logical unit number
c character msg*(*)             # message string
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c integer *2 i                  # looping index for searching for valid response
c character gquery*1            # received character
c           gok*12              # legal responses
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu = 6 )
c
      integer *2  iplun, i
      character gquery*1, gok*12, msg*(*)
      data gok / 'tT1yY. fF0nN' /
c
      write( Hvdulu, 1 ) msg
 1    format( 1x, a, ' ' )
c
c     do while( .true. )
 4      read( iplun, 2 ) Gquery
 2      format( a1 )
        do 5 i = 1, 12
          if ( gok(i:i) .eq. gquery ) then
            equery = i .le. 6
            return
            end if
 5        continue
        write( Hvdulu, 3 )
 3      format( ' ?' )
        goto 4
c       end do
c
      end


      LOGICAL *1 FUNCTION EXISTF( CFILE )
c
c         does the File CFILE    EXIST ?
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
      character cfile*(*)
c
      inquire( FILE = cfile, EXIST = existf, ERR = 1 )
      return
1     write( Hvdulu, 2 )
2     format( ' EXISTF : WARNING : Error in INQUIRE' )
      existf = .false.
      return
      end


      LOGICAL *1 FUNCTION E1BYTE( N )
c
c      Determines whether or not the integer N can be represented as a single
c  byte value .
c
      implicit none
      integer *2 n
c
      E1byte = 0 .le. n .and. n .lt. 256
      return
      end


      INTEGER *2 FUNCTION IARGC()
c
c         Returns the number of aqrguments to the main program .
c
c    constants
c Hlen                   # Maximum argument list length
c    Variables
c character Cmd*Hlen     # program arguments
c integer *2 i,          # count up index
c logical *1 fspace      # found an argument separator
c
      implicit none
      integer *2 Hlen
      parameter( Hlen=127)
c
      character  Cmd*Hlen
      integer *2 i
      logical *1 fspace
c
      call getcl( Cmd )
      iargc = 0
      fspace = .false.
      do 1 i = 1, Hlen
        if ( cmd(i:i) .eq. ' ' ) then
          fspace = .true.
        else if ( fspace ) then
          iargc = iargc + 1
          fspace = .false.
          end if
1       continue
      return
      end


      INTEGER *2 FUNCTION JBSRCH( X, NINCR, NPTS, X0 )
c
c       Binary SeaRCH
c  Function for locating the last element between the 1st and Nth elements
c  in the array X of N real numbers , which in the case of an ascending
c  array , is less than the value X0 . In the case of a descending array ,
c  it finds the last element which is greater than or equal to X0 . The
c  returned value of JBSRCH is the index of this element in the array X .
c    N.B. X must be an array of values arranged in STRICTLY ASCENDING or
c  STRICTLY DESCENDING order .
c
c     Arguments
c real  x(*),           # data array
c       x0              # search goal
c integer *2 nincr,     # data array indexing increment
c            npts       # number of data points used
c     Variables
c logical desc          # .true. if data array X is descending
c integer *2 n          # index of last point used
c           l,r,        # left & right hand indices of the search interval
c           c           # moving search index
c           JBSRCH      # index of last element in array less than X0
c
      real x(*), x0
      logical *1 desc
      integer *2 nincr, npts, n, l, r, c
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n = 1 + ( npts - 1 ) * nincr
      desc = x(1) .gt. x(n)
      if ( x0 .eq. x(1) ) then
        l = 1
      else if ( x0 .eq. x(n) ) then
        l = n - nincr
      else if ( x0 .lt. x(1) .neqv. desc ) then
        l = 0
      else if ( x0 .gt. x(n) .neqv. desc ) then
        l = n
      else
        l = 1
        r = n
1       if ( r - l .le. nincr ) goto 2
          c = l + ( r - l ) / 2 / nincr * nincr
          if ( x0 .gt. x(c) .neqv. desc ) then
            l = c
          else
            r = c
            end if
          goto 1
2       continue
        end if
      JBSRCH = l
      return
      end


      INTEGER *2 FUNCTION JINTUS( X )
c
c       J INTeger part ( UnSigned ) of X
c
c       Function for converting a (32 bit) real number into an unsigned
c  (16 bit) integer in the range 0 to 65535 . The fraction part is truncated .
c
c Arguments
c  real x           # source
c
c Variables
c  integer *4 i4    # intermediate integer - to be truncated
c  integer *2 i2(2) # 2 2 byte map of i4
c
      implicit none
      integer *4 i4
      integer *2 i2(2)
      real x
      equivalence( i4, i2(1) )
c
      i4 = x
      Jintus = i2(1)
      return
      end


      INTEGER *2 FUNCTION JLJSTN( CSTR, NCOLS )
c
c         Left justify the character string CSTR ; ie eliminate leading blanks
c and return the number of characters in it after trailing blanks have also
c been removed . NCOLS is the returned number of values in Cstr to the left of
c any ';' .
c
c    Argument
c character *(*) Cstr          # operand
c
c    Variables
c integer *2 ncols             # number of columns of data
c            length,           # length of operand
c            i,                # count up index
c            j,                # count down index
c            n,                # justified result index
c logical *1 fnocmt            # comment field not found
c            fspace            # found a numeric field separator
c
      implicit none
      character *(*) Cstr
      integer *2 ncols, length, i, j, n
      logical *1 fnocmt, fspace
c
      length = len( Cstr )
c
c       find first non blank char
c
      i = 1
1     if ( Cstr(i:i) .eq. ' ' .and. i .lt. length ) then                  1b
        i = i + 1
        goto 1
        end if                                                            1e
c
c       find last non blank char
c
      j = length
2     if ( Cstr(j:j) .eq. ' ' .and. j . gt. i ) then                      1b
        j = j - 1
        goto 2
        end if                                                            1e
      jljstn = j - i + 1
      if ( Cstr(j:j) .eq. ' ' )
     1  jljstn = 0
c
c       left justify
c
      ncols = 1
      fspace = .false.
      fnocmt = .true.
      do 3 n = 1, jljstn                                                  1b
        Cstr(n:n) = Cstr(i:i)
        fnocmt = fnocmt .and. cstr(i:i) .ne. ';'
        if ( cstr(i:i) .eq. ' ' .or. cstr(i:i) .eq. ',' ) then            2b
          fspace = .true.
        else if ( fspace .and. fnocmt ) then                              2m
          ncols = ncols + 1
          fspace = .false.
          end if                                                          2e
        i = i + 1
3       continue                                                          1e
c
c       fill up with blanks
c
      do 4 n = jljstn + 1, j                                              1b
        Cstr(n:n) = ' '
4       continue                                                          1e
      return
      end


      INTEGER *2 FUNCTION JLJSTR( CSTR )
c
c         Left justify the character string CSTR ; ie eliminate leading blanks
c and return the number of characters in it after trailing blanks have also
c been removed .
c
c    Argument
c character *(*) Cstr          # operand
c
c    Variables
c integer *2 length,           # length of operand
c            i,                # count up index
c            j,                # count down index
c            n,                # justified result index
c
      character *(*) Cstr
      integer *2 length, i, j, n
c
      length = len( Cstr )
c
c       find first non blank char
c
      i = 1
 1    if ( Cstr(i:i) .eq. ' ' .and. i .lt. length ) then
        i = i + 1
        goto 1
        end if
c
c       find last non blank char
c
      j = length
2     if ( Cstr(j:j) .eq. ' ' .and. j . gt. i ) then
        j = j - 1
        goto 2
        end if
      Jljstr = j - i + 1
      if ( Cstr(j:j) .eq. ' ' ) Jljstr = 0
c
c       left justify
c
      do 3 n = 1, jljstr
        Cstr(n:n) = Cstr(i:i)
3       i = i + 1
c
c       fill up with blanks
c
      do 4 n = jljstr + 1, j
4       Cstr(n:n) = ' '
      return
      end


      INTEGER *2 FUNCTION JRESP( IPLU, IMIN, IMAX, NOREQS )
c               Integer *2 RESPonse
c
c       The integer function JRESP returns two byte integer value between the
c  specified valid limits. This value is entered via the file with logical
c  unit number IPLU ( including possibly the keyboard ) , and is  checked
c  by JRESP, i.e. MIN < JRESP < MAX . The input file logical unit number
c  IPLU must be in the range 0 <= IPLU <= 99 , and must have been previously
c  defined if it is not a default device ( vdu, keyboard etc ). The maximum
c  number of input requests allowed NOREQS must be positive.
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer *2 Iplu              #  input file logical unit number
c            imax,             #  maximum valid response value
c            imin,             #  minimum valid response value
c            noreqs            #  maximum number of requests allowed
c
c constants
c     Hkbdlu                   # keyboard logical unit number
c     Hvdulu                   # v.d.u. logical unit number
c integer *2 cntr,             # counter for the number of input attempts
c            Iplu1             # input file logical unit number
c            nchars            # number of input characters
c            jljstr            # function : left justify string
c character Cstr*40            # input character string
c
      implicit none
      integer *2 Hkbdlu, Hvdulu
      parameter ( Hkbdlu=5, Hvdulu=6 )
c
      integer *2 imax, imin, iplu, noreqs, cntr, iplu1, nchars, jljstr
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. noreqs .le. 0 )
     1  write ( Hvdulu, 1 ) Iplu, Noreqs
1       format ( ' JRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = Hkbdlu
      do 7 cntr = 1, noreqs
c
c           read data
c
c       repeat
2         read( Iplu1, 3, END=9 ) Cstr
3         format( a )
          nchars = jljstr( Cstr )
          if ( nchars .le. 0 ) goto 2
c         until( nchars > 0 )
c
c           selected min or max
c
        if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
          jresp = imax
          return
        else if ( Cstr(1:3) .eq. 'min' .or. Cstr(1:3) .eq. 'MIN' ) then
          jresp = imin
          return
c
c           decode & check numeric value
c
        else
          read( Cstr, *, ERR=5 ) jresp
          if ( i2min0( imin, imax ) .le. jresp .and.
     1        jresp .le. i2max0( imin, imax ) )
     2      return
          write ( Hvdulu, 4 ) imin, imax
4         format(' JRESP : ERROR ; Integer outside limits ', i6, ' & ',
     1      i6 )
5         if ( Iplu1 .ne. Hkbdlu ) stop
          write ( Hvdulu, 6 )
6         format( ' Try again ' )
          end if
7       continue
c
c           give up
c
        write ( Hvdulu, 8 )
8       format( ' Get help' )
9       stop
      end


      REAL FUNCTION QARC0( RANGLE )
c
c    Real function which receives the value RANGLE as an angle of arc in
c radians and returns the equivalent value between Pi and -Pi
c
      implicit none
      real Hpi
      parameter ( Hpi = 3.1415926)
      real Rangle
c
      Qarc0 = mod( Rangle, 2.0 * Hpi )
      if ( Qarc0 .gt. Hpi ) then
        Qarc0 = Qarc0 - 2.0 * Hpi
      else if ( Qarc0 .lt. -Hpi ) then
        Qarc0 = Qarc0 + 2.0 * Hpi
        end if
      return
      end


      REAL FUNCTION QDIST( X, Y, A, B, C )
c
c       DISTance from a point to a line
c   Function to calculate the distance from the line  A.x + B.y + C = 0
c to the point (X,Y) .
c
c   Arguments
c real  a,b,c,          # parameters for defining the line
c       x,y             # coordinates of the point
c
      implicit none
      real a, b, c, x, y
c
      QDIST = 0.0
      if ( a .ne. 0.0 .or. b .ne. 0.0 )
     1  QDIST = abs( a * x + b * y + c ) / sqrt( a **2 + b **2 )
      return
      end


      REAL FUNCTION QLIMIT( R, RMIN, RMAX, ERRMSG )
c
c   Real function which ensures that the real variable R remains within the
c limits of Rmin and Rmax ; i.e. Rmin <= R <= Rmax
c
c    Arguments
c real  R,              # initial value
c       Rmin,           # minimum permissable value
c       Rmax            # maximum permissable value
c logical *1 errmsg     # .false. if error message is disabled
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      logical *1 Errmsg
      real r, rmax, rmin
c
      Qlimit = min( Rmax, max( Rmin, R ) )
      if ( R .ne. Qlimit .and. Errmsg )
     1  write( Hvdulu, 1 ) R, rmin, rmax
1       format( ' QLIMIT : ', g14.7, ' exceeds limits ', g14.7, ' and ',
     1    g14.7 )
      return
      end


       REAL FUNCTION QLRPL1( X, Y, NINCR, NPTS, X0 )
c
c       LineaR inTerPoLation from tabulated data .
c   Function to linearly interpolate ordinate y0, given a table of (x,y)
c coordinates in the arrays X & Y and an abicissa value x0 . Starting from the
c first elments of X & Y , and with an indexing  increment of NINCR , NPTS
c coordinate pairs are used from X & Y .
c The array X is searched until adjacent values x(l) & x(r) straddle x0 .
c   N.B. ELEMENTS OF X MUST BE IN STRICTLY ASCENDING ORDER OR STRICTLY
c        DESCENDING ORDER
c
c       Arguments
c real  x(*),y(*),      # arrays of tabulated (x,y) coordinates
c       x0              # target for which a y0 value is required
c integer *2 nincr,     # data array indexing increment
c            npts       # number of data points used
c
c constant
c   Hvdulu              # v.d.u. logical unit number
c  Variables
c integer *2 n          # size of arrays
c            l          # left hand indices of the search interval
c logical pass1         # .true. until an attempt to extrapolate is detected
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real x(*), y(*), x0
      integer *2 nincr, npts, n, l, Jbsrch
      logical *1 pass11,pass1k
      common /af77lib3/ pass11, pass1k
c
c       Find the array interval
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n = 1 + ( npts - 1 ) * nincr
      l = Jbsrch( x, nincr, npts, x0 )
      if ( l .lt. 1 .or. l .ge. n ) then
        if ( pass11 ) write ( Hvdulu, 1 )
1                     format( ' QLRPL1 : Extrapolating' )
        pass11 = .false.
        l = max( 1, min( l, n - nincr ) )
        end if
c
c       Interpolate
c
      qlrpl1 = ( y(l+nincr) - y(l) ) / ( x(l+nincr) - x(l) ) *
     1           ( x0 - x(l) ) + y(l)
      return
      end


      BLOCK DATA BF77LIB3
      logical *1 pass11,pass1k
      common /af77lib3/ pass11, pass1k
      data pass11, pass1k / .true., .true. /
      end


      REAL FUNCTION QMEAN( X, NINCR, NPTS )
c
c       arithmetic MEAN of a sequence
c     Function to find the arithmetic mean of X, a sequence of NPTS real numbers,
c selected from X  with increments of NINCR
c
c     Arguments
c real x(*)            # operand array
c integer *2 nincr,    # index increment
c            npts      # number of data points to be averaged
c     Variables
c integer *2 i         # a working array index
c double precision rtotal
c
      implicit none
      integer *2 nincr, npts, i
      double precision rtotal
      real x(*)
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      rtotal = 0.0d0
      do 1 i = 1, ( npts - 1 ) * nincr + 1, nincr
        rtotal = rtotal + x(i)
1       continue
      qmean = sngl( rtotal ) / float( npts )
      return
      end


      REAL FUNCTION QPOLY( X, B, K )
c
c       POLYnomial evaluation
c    Evaluates the Kth order polynomial :-
c P(x) = b(1) + b(2) * x + b(3) * x**2 + ... + b(k) * x**(k-1) + b(k+1) * x**k
c
c Arguments
c  real x,          # dependent variable
c       b(k+1)      # coefficients of the polynomial
c  integer *2 k     # order of the polynomial
c
c variable
c  integer *2 i     # looping index
c  double precision
c    double_x
c    sum
c
      implicit none
      integer *2 i, k
      real x, b(*)
      double precision double_x, sum
c
      Qpoly = 0.0
      if ( k .lt. 0 ) return
      sum = B(k+1)
      double_x = dble( x )
      do 1 i = k, 1, -1
        sum = sum * double_x + dble( b(i) )
1       continue
      Qpoly = real( sum )
      return
      end


      REAL FUNCTION QSRIND( X, Y, NINCR, NPTS, A, B, N )
c
c       Simpson's Rule INtegration ( Discrete )
c   Numerical integration of the function defined by the table of values given
c by the coordinate pairs in arrays X & Y over the interval a < x < b , by
c using Simpson's 4th order method with n subintervals .
c
c    Arguments
c real  x(m),y(m),      # (x,y) data arrays
c       a,              # lower limit of integration interval
c       b,              # upper limit of integration interval
c integer *2 nincr      # data array indexing increment
c            npts,      # number of data points
c            n          # number of subintervals - should be even
c
c    Variables
c real  h, h2,          # length & twice the length of a subinterval
c       tol,            # tolerance on incrementing
c       z,              # integration variable
c       QLRPL1          # linear interpolation function
c integer *2 n1         # number of subintervals - should be even
c
      implicit none
      integer *2 nincr, npts,  n, n1
      real a, b, h, h2, tol, x(*), y(*), z, qlrpl1
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n1 = max( n / 2 * 2, 2 )
      h = ( b - a ) / n1
      h2 = 2.0 * h
      tol = h * 1.2e-7 * n1
      z = a
      QSRIND = QLRPL1( x, y, nincr, npts, a ) + QLRPL1( x, y, nincr,
     1              npts, a+h )* 4.0 + QLRPL1( x, y, nincr, npts, b )
      tol = b - h2 - tol
1     if ( z .lt. tol ) then
        z = z + h2
        QSRIND = QSRIND + 2.0 * QLRPL1( x, y, nincr, npts, z ) + 4.0 *
     1             QLRPL1( x, y, nincr, npts, z+h )
        goto 1
        end if
      QSRIND = QSRIND * h / 3.0
      return
      end


      REAL FUNCTION QRESP( IPLU, RMIN, RMAX, NOREQS )
c               Real RESPonse
c       The real funtion QRESP returns a real value between the specified
c valid limits . This value is entered via the file with logical unit number
c IPLU ( default value 0 is the keyboard ) , and is checked by QRESP
c ie RMIN <= QRESP <= RMAX . The input file logical unit number must be in the
c range 0 <= IPLU <= 99 , and must have been previously defined if it is not
c a default device ( keyboard etc ) . The maximum number of input requests
c allowed NOREQS must be positive .
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer *2 Iplu       # input file logical unit number
c            Noreqs     # maximum number of requests allowed
c real  rMax,           # maximum valid response value
c       rMin            # minimum valid response value
c
c constants
c   Hkbdlu              # keyboard logical unit number
c   Hvdulu              # v.d.u. logical unit number
c integer *2 Cntr,      # counter for the number of input attempts
c            Iplu1      # input file logical unit number
c            nchars     # number of input characters
c            jljstr     # function : left justify string
c character Cstr*40     # input character string
c
      implicit none
      integer *2 Hvdulu, Hkbdlu
      parameter ( Hvdulu = 6, Hkbdlu = 5 )
c
      integer *2 iplu, noreqs, cntr, iplu1, nchars, jljstr
      real rMax, rMin
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. Noreqs .le. 0 )
     1  write( Hvdulu, 1 ) Iplu, Noreqs
1       format( ' QRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = Hkbdlu
      do 7 Cntr = 1, noreqs
c
c           read data
c
c     repeat
2         read( Iplu1, 3, END=9 ) Cstr
3         format( a )
          nchars = jljstr( Cstr )
          if ( nchars .le. 0 ) goto 2
c         until( nchars > 0 )
c           selected min or max
c
        if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
          qresp = rmax
          return
        else if ( Cstr(1:3) .eq. 'min' .or. Cstr(1:3) .eq. 'MIN' ) then
          qresp = rmin
          return
c
c           decode & check numeric value
c
        else
          read ( Cstr, *, ERR=5 ) Qresp
          if ( amin1( rMin, rMax ) .le. Qresp .and.
     1        Qresp .le. amax1( rMin, rMax ) )
     2      return
          write( Hvdulu, 4 ) rmin,rmax
4         format(' QRESP : ERROR ; Real outside limits ', g12.5, ' & ',
     1            g12.5 )
5         if ( Iplu1 .ne. Hkbdlu ) stop
          write ( Hvdulu, 6 )
6         format( ' Try again ' )
          end if
7       continue
c
c           give up
c
      write ( Hvdulu, 8 )
8     format( ' Get help' )
9     stop
      end


      REAL FUNCTION QZERLI( A, B, F, TOL )
c
c       ZEROS BY LINEAR Interpolation
c   Function which uses linear interpolation to iterate toward the zero of the
c function F(X) . The function QZERLI requires 2 starting values  X = A and
c X = B . QZERLI is not guarranteed to converge to a zero of the function F(X)
c . Care should be taken in selecting the starting values  X = A and  X = B .
c Knowing the shape of F(X) can help with this initial selection .
c
c real a,b,             # starting values of x (1st values of search interval)
c      xa,xb,           # calculated values of search interval boundary
c      fa,fb,           # =f(xa) & =f(xb)
c      tol1,tol         # tolerance on the zero value of   F( QZERLI )
c
      implicit none
      integer Hvdulu
      parameter( Hvdulu=6 )
c
      real a, b, xa, xb, f, fa, fb, tol1, tol, denom
      external f
c
      tol1 = abs( tol )
      xa = a
      xb = b
      fa = f( xa )
      fb = f( xb )
2     if ( abs( fa ) .gt. tol1 ) then
        denom = fb / fa - 1.0
        if ( denom .eq. 0.0 ) then
          write( Hvdulu, 1 )
1         format( ' QZERLI : ABORT : Zero slope.' )
          return
          end if
        QZERLI = xa + ( xa - xb ) / denom
        xb = xa
        fb = fa
        xa = QZERLI
        fa = f( xa )
        goto 2
        end if
      return
      end


      REAL FUNCTION RFLTUS( I )
c
c       R FLoaT ( UnSigned )
c
c Function to convert an unsigned 16 bit integer ( 0 to 65536 ) to a real
c  number .
c
c Argument
c  integer *2 i     # unsigned integer to be converted
c
c Variables
c  integer *4 i4    # signed 4 byte integer
c  integer *2 i2(2) # 2 2 byte overlays of i4
c
      implicit none
      integer *4 i4
      integer *2 i2(2), i
      equivalence( i4, i2(1) )
c
      i2(1) = i
      rfltus = real( i4 )
      return
      end


      SUBROUTINE DATSTR( CDATE )
c
c       DATe STRing
c
c           Put the date and time into the character string CDATE .
c           CDATE should be at least 24 characters long .
c Argument
c  character *(*) Cdate
c
c Variables
c  integer *2 imonth        # month number - extracted from cd
c  character cd*8           # numeric date string
c            ct*11          # numeric time string
c            cmonth(12)*3   # Month abbreviations
c

      implicit none
      integer *2 imonth
      character cdate*(*), cd*8, ct*11, cmonth(12)*3
      data cmonth / 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
     1              'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' /
c
      call date( cd )
      call time( ct )
      read( Cd(1:2), 1 ) imonth
1     format( i2 )
      cdate = Cmonth(imonth) // ' ' // Cd(4:5) // ' ' // Ct(1:8) //
     1  ' CST 19' // cd(7:8)
      return
      end


      SUBROUTINE DPEQ24( DPDATA, ICMD )
c
c     Subroutine to transfer the 3 low order bytes of a long ( 4 byte )
c  integer to a location in a SBC queue command array .
c
c Arguments
c  integer *4 Dpdata        # long integer
c  integer *2 Icmd(*)       # command array
c
      implicit none
      logical *1 dpdata(4), icmd(6)
c
      Icmd(1) = dpdata(1)
      Icmd(2) = .false.
      Icmd(3) = dpdata(2)
      Icmd(4) = .false.
      Icmd(5) = dpdata(3)
      Icmd(6) = .false.
      return
      end


      SUBROUTINE GETARG( IARG, CARG )
c
c         Returns the IARGth argument of the main program to the character
c   string CARG .
c
c Arguments
c  integer *2 iarg       # Argument to be selected
c  character Carg*(*)    # Argument string returned
c
c    constants
c Hlen                   # Maximum argument list length
c    Variables
c character Cmd*Hlen     # program arguments
c integer *2 i,          # count up index
c            igot        # argument number found
c            jbegin      # where the IARGth argument starts
c logical *1 fspace      # found an argument separator
c
      implicit none
      integer *2 Hlen
      parameter( Hlen=127)
c
      character  Cmd*Hlen, Carg*(*)
      integer *2 i, igot, iarg, jbegin
      logical *1 fspace
c
      call getcl( Cmd )
      igot = 0
      fspace = .false.
      do 1 i = 1, Hlen
        if ( cmd(i:i) .eq. ' ' ) then
          fspace = .true.
          if ( igot .eq. iarg ) then
            Carg = Cmd(jbegin:i-1)
            return
            end if
        else if ( fspace ) then
          igot = igot + 1
          jbegin = i
          fspace = .false.
          end if
1       continue
      Carg = ' '
      return
      end


      SUBROUTINE SCASE( CSTRIN, CSTROUT, CASE )
c
c       CASE CHange
c     Change the case of character string CSTR to upper, lower of edit case .
c
c Arguments
c  character cstrin*(*),    # operand character string               INPUT
c            cstrout*(*),   # result character string                INPUT
c            case*1         # 'l' or 'L' - change to lower case      INPUT
c                           # 'u' or 'U' - change to upper case
c                           # 'e' or 'E' - change to edit case
c                           # anything else - no change
c Constant
c  Hshift=32                # +ve or -ve displacement required to change case
c Variables
c  integer *2 i,            # indexing counter
c             nchars        # number of characters to process
c  logical *1 first         # first letter in a word
c
      implicit none
      integer *4 Hshift
      parameter( Hshift=32 )
c
      integer *2 i, nchars
      logical *1 first
      character cstrin*(*), cstrout*(*), case*1
c
      cstrout = cstrin
      nchars = min( len( cstrout ), len( cstrin ) )
c
c         change to lower case
c
      if ( case .eq. 'l' .or. case .eq. 'L' ) then                         1b
        do 1 i = 1, nchars                                                 2b
          if ( 'A' .le. cstrout(i:i) .and. cstrout(i:i) .le. 'Z' )
     1      cstrout(i:i) = char( ichar( cstrout(i:i) ) + Hshift )
1         continue                                                         2e
c
c         change to upper case
c
      else if ( case .eq. 'u' .or. case .eq. 'U' ) then                    1m
        do 2 i = 1, nchars                                                 2b
          if ( 'a' .le. cstrout(i:i) .and. cstrout(i:i) .le. 'z' )
     1      cstrout(i:i) = char( ichar( cstrout(i:i) ) - Hshift )
2         continue                                                         2e
c
c         change to edit case
c
      else if ( case .eq. 'e' .or. case .eq. 'E' ) then                    1m
        first = .true.
        do 3 i = 1, nchars                                                 2b
          if ( 'A' .le. cstrout(i:i) .and. cstrout(i:i) .le. 'Z' ) then    3b
            if ( .not. first )
     1        cstrout(i:i) = char( ichar( cstrout(i:i) ) + Hshift )
            first = .false.
          else if ( 'a'.le.cstrout(i:i) .and. cstrout(i:i).le.'z' ) then   3m
            if ( first )
     1        cstrout(i:i) = char( ichar( cstrout(i:i) ) - Hshift )
            first = .false.
          else                                                             3m
            first = .true.
            end if                                                         3e
3         continue                                                         2e
        end if                                                             1e
      return
      end


      SUBROUTINE SCHECK( SSBR )
c
c Subroutine to allow execution of subroutine Ssbr to be repeated if a
c   previous execution is unsatisfactory .
c
c constant Hkbdlu       # keyboard logical unit number
c logical Equery        # Function; # Displays request & returns reply
c logical *1 msg(5)
c data msg / 'OK ?' /
c
      implicit none
      integer *2 Hkbdlu
      parameter ( Hkbdlu=5 )
      logical Equery
c
1     call Ssbr
      if ( .not. Equery( Hkbdlu, 'OK ?' ) ) goto 1
      return
      end


      SUBROUTINE SIEQ16( IDATA, ICMD )
c
c     Subroutine to transfer the 2 bytes of a 16 bit integer to a location in
c  a SBC queue command array .
c
c Arguments
c  integer *2 idata         # 16 bit integer
c             Icmd(*)       # command array
c
      implicit none
      logical *1 idata(2), icmd(4)
c
      Icmd(1) = idata(1)
      Icmd(2) = .false.
      Icmd(3) = idata(2)
      Icmd(4) = .false.
      return
      end


      SUBROUTINE SLRREG( X, Y, NINCR, NPTS, A, B, CORR, MSERR )
c
c       LiNear REGression
c     Subroutine to do a linear regression ( least mean squares ) fit to NPTS
c  in the data arrays X & Y . Every NINCRth value is used . The line of best
c  fit is  y = A.x + B with correlation coefficient CORR and mean square error
c  MSERR .
c
c    Arguments                 input                           output
c real x(*),y(*),      # operand arrays
c      a,                                      # slope of line of best fit
c      b,                                      # y axis intercept of line
c      corr,                                   # correlation coefficient
c      mserr,                                   # mean square error
c integer *2 nincr,    # data array indexing increment
c            npts      # number of data points used
c
c    Variables
c real meanx,meany,             # mean of x and of y
c      varx,vary,               # variance of x and of y
c      covar                   # covariance of x & y
c
      implicit none
      real x(*), y(*), a, b, corr, mserr, meanx, meany, varx, vary,
     1  covar
      integer *2 nincr, npts
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      call SMEANS( x, y,  nincr, npts, meanx, meany, varx, vary, covar )
      if ( varx .ne. 0.0 ) then
        a = covar / varx
        b = meany - a * meanx
      else
        a = -9.999e29
        b = -9.999e29
        end if
      corr = varx * vary
      if ( corr .ne. 0.0 ) then
        corr = covar / sqrt( corr )
      else
        corr = -9.999e29
        end if
      mserr = vary * ( 1.0 - corr **2 )
      return
      end


      SUBROUTINE SMEANS( X, Y, NINCR, NPTS, MEANX, MEANY, VARX, VARY,
     1  COVAR )
c
c    Subroutine to find the means , variances and covariance of two sequences
c  X and Y of N real numbers .
c
c    Arguments
c
c real covar,                  # covariance of x & y
c      meanx,meany,            # means of x & y
c      varx,vary,              # variances of x & y
c      x(*),y(*)               # operand sequences
c integer *2 nincr,            # data array indexing increment
c            npts,             # number of data points used
c
c    Variables
c real sx,sy,sxx,syy,sxy,      # sums of x, y, x**2, y**2, x*y
c integer *2 i                 # working array index
c
      implicit none
      integer *2 nincr, npts, i
      real x(*), y(*), meanx, meany, varx, vary, covar, sx, sy,
     1  sxx, syy, sxy
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
c
c           calculate means
c
      sx = 0.0
      sy = 0.0
      do 1 i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = sx + x(i)
        sy = sy + y(i)
1       continue
      meanx = sx / npts
      meany = sy / npts
c
c           calculate variances & covariances
c
      sxx = 0.0
      syy = 0.0
      sxy = 0.0
c
c   evaluate sums
c
      do 2 i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = x(i) - meanx
        sy = y(i) - meany
        sxx = sxx + sx **2
        syy = syy + sy **2
        sxy = sxy + sx * sy
2       continue
c
c    calculate means
c
      varx = sxx / npts
      vary = syy / npts
      covar = sxy / npts
      return
      end


      SUBROUTINE SORTSH2( X, M1, M2, N, J1, M, INCCOL )
c
c               SORT ( SHell )
c       Subroutine for a shell sort into ascending or descending order .  The
c sorted array is an N x M matrix of N sets of Mtuples ( ie. N points in M
c dimensional space ) . The J1th column of the matrix is sorted into ascending
c or descending order , and all columns that are a multiple of INCCOL either
c side of the J1th column  undergo the same changes , while all other columns
c remain unchanged . Sorting occurs between rows M1 and M2 inclusive .
c If  M1 < M2 , the sort is descending . If M1 > M2 , the sort is ascending.
c  Eg. for   SORTSH2( X, 4, 1, 5, 1, 4 , 2 ) ,
c               2 4 3 1                 1 4 4 1
c               4 3 1 2                 2 3 3 2
c           X = 3 1 2 4     becomes X = 3 1 2 4
c               1 2 4 3                 4 2 1 3
c               3 1 2 4                 3 1 2 4
c   Arguments
c real x                    # array to be sorted
c integer *2 m1, m2         # range of rows to be sorted
c            n              # number of rows in matrix
c            j1             # column to be sorted
c            m              # number of columns in the matrix
c            inccol         # increment length for column index
c
c   Variables
c integer *2 i1, i2, i3, j, # working indices
c            id,            # sort segment size
c            m11, m12,      # starting row & -1
c            fstcol         # index of first column to be swapped
c            lastcol        # index of last column to be swapped
c
      implicit none
      integer *2 n, nn, m, m1, m2, i1, i2, i3, j, j1, id, m11, m12,
     1  fstcol, lastcol, inccol
      real x(n,m)
c
      if ( m1 .eq. m2 .or. m1 .lt. 1 .or. m1 .gt. n .or. m2 .lt. 1 .or.
     1  m2 .gt. n .or.j1 .lt. 1 .or. j1 .gt. m .or. inccol .lt. 0 .or.
     2  inccol .ge. m ) return
      m11 = min( m1, m2 )
      m12 = m11 - 1
      nn = abs( m2 - m1 ) + 1
      id = 2 ** int( alog( float( nn ) ) / alog( 2.0 ) + 0.0001 ) - 1
      fstcol = mod( J1 - 1, inccol ) + 1
      lastcol = fstcol + ( ( m - 1 ) / inccol ) * inccol
c     repeat
1       do 4 i1 = 1, nn - id
          do 3 i2 = i1 + m12, m11, -id
            i3 = i2 + id
            if ( x(i2,j1) .le. x(i3,j1) ) goto 4
            do 2 j = fstcol, lastcol, inccol
2             call swap( x(i2,j), x(i3,j) )
3           continue
4         continue
          id = id / 2
        if ( id .gt. 0 ) goto 1
c     until ( id <= 0 )
c
c           swap rows for descending sort
c
      if ( m1 .lt.  m2 ) then
        i1 = m1
        i2 = m2
c       repeat
5         do 6 j = fstcol, lastcol, inccol
6           call swap( x(i1,j), x(i2,j) )
          i1 = i1 + 1
          i2 = i2 - 1
          if ( i1 .lt. i2 ) goto 5
c       until ( i1 >= i2 )
        end if
      return
      end


      SUBROUTINE SPYORTH( X, Y, NPTS, POLY, ORDER, ERROR, WORK )
c
c     Calculates a polynomial of order `order' which approximates the weighted
c (x,y) data . The method uses the minimum mean squared fitting error to select
c a series of orthogonal polynomials . The orthogonal polynomials are then
c combined to produce the result polynomial `poly' .
c
c This method is chosen because
c  1) It is not ill-conditioned .
c  2) It uses a very small amount of `work' array space
c  3) Calculation of the error is trivial
c  4) Time is proportional to ORDER**2
c  5) It is possible to return the list of orthogonal polynomials . From
c     this list, all `best' polynomials of order up to 'order' can be
c     extracted with very little effort .
c
c Arguments
c  real x(npts),       # IN  # abscissa values
c       y(npts),       # IN  # ordinate values
c  integer *2 npts     # IN  # number of (x,y,weight) data points
c  real poly(order+1)  # OUT # coefficients of the approximating polynomial
c  integer *2 order    # IN  # order of the approximating polynomial
c  real error,         # OUT # mean square error of approximating polynomial
c  double precision
c       work(2,order+1)# IN  # working array (for coeffs of orthog polonomials)
c
c Local variables
c  integer *2
c   i                  # index over the data points
c   j,                 # index over 1 to order+1
c   k                  # index over 1 to j
c
c       N.B. It has been found that within this routine use of double
c            precision is necessary to avoid loss of accuracy through
c            round off error . However, there is no advantage to be
c            gained by supplying the data in double precision .
c  double precision
c   alpha, beta        # recursion coeffients
c   new_d, old_d       # norms (Pj,Pj) of jth & (j-1)th orgogonal polynomials
c   o_coeff            # coefficients defining the approximating polynomial
c                      # in terms of the orthogonal polynomials Pj(x)
c   temp               # temporary storage for most recent coeff of orthog poly
c   poly_x             # Pj(Xi)
c   poly2              # Pj(Xi).Pj(Xi)
c   int_error          # min mean square error -- internal use only
c
      integer *2 npts, order
      real x(*), y(*), poly(*), error
      double precision work(2,*)
c
      integer *2 i, j, k
      double precision alpha, beta, new_d, old_d, o_coeff, temp,
     1        poly_x, poly2, int_error
c
c             initialize coefficients of the recursion relationship
c
      alpha = 0.0
      beta  = 0.0
c
c             initialize the orthogonal polynomials and
c             result polynomial to zero
c
      do 1 j = 1, order + 1
          work(1,j) = 0.0
          work(2,j) = 0.0
          poly(j) = 0.0
1         continue
c
c             calculate the initial error
c
      int_error = 0.0
      do 2 i = 1, npts
          int_error = int_error + dble( y(i) ) **2
2         continue
c
c             start of outer loop -- to calculate the jth orthogonal polynomial
c             #################################################################
c
      old_d = 1.0
      do 7 j = 1, order + 1
c
c             calculate the coefficients of the jth orthogonal polynomial
c             ***********************************************************
c             using the recursion relationship ( i.e. from the previous 2
c             orthogonal polynomials
c
          work(2,j)=1.0
          if ( j .gt. 1) then
              do 3 k = 2, j
                  temp = work(1,k-1) - alpha * work(2,k-1) -
     1                   beta * work(1,k)
                  work(1,k) = work(2,k-1)
                  work(2,k-1) = temp
3                 continue
              end if
c
c             calculate new values for recursion coefficients
c             ***********************************************
c
          new_d = 0.0
          alpha = 0.0
          o_coeff = 0.0
c
          do 5 i = 1, npts
c
c             evaluate the jth orthogonal polynomial
c
              poly_x  = 0.0
              do 4 k = j, 1, -1
                  poly_x = poly_x * dble( x(i) ) + work(2,k)
4                 continue
c
c             calculate the various sums
c
              poly2   = poly_x **2
              new_d   = new_d + poly2
              alpha   = alpha + poly2 * dble( x(i) )
              o_coeff = o_coeff + poly_x * dble( y(i) )
5             continue
c
          alpha = alpha / new_d
          beta  = new_d / old_d
          o_coeff = o_coeff / new_d
          int_error = int_error - o_coeff **2 * new_d
          old_d = new_d
c
c             add the effect of the jth orthogonal polynomial to the result
c             *************************************************************
c             polynomial
c
          do 6 k = 1, j
              poly(k) = poly(k) + o_coeff * work(2,k)
6             continue
7         continue
c
c             end of outer loop
c             #################
c
      error = int_error
      return
      end


      SUBROUTINE SWAPI( I, J )
c
c   Subroutine for swapping 2 2 byte quantities - usually integer *2
c
      implicit none
      integer *2 i, j, k
c
      k = i
      i = j
      j = k
      return
      end


      SUBROUTINE SWAP( A, B )
c
c   Subroutine for swapping 2 4 byte quantities
c
      implicit none
      integer *4 a, b, c
c
      c = a
      a = b
      b = c
      return
      end


      INTEGER *2 FUNCTION XDECODE( CHEX, IDATA, NITEMS )
c
c       XDECODE
c
c    Routine to decode a hexadecimal ASCII string into an array of integers .
c The returned value of XDECODE is the number of integer values actually
c decoded . Any non-hexadecimal-digit character is understood as a separator .
c
c Arguments
c  character chex*(*)       # string of hexadecimal numbers to be decoded
c  integer *2 idata(*)      # decoded integer values
c  integer *2 nitems        # number of data bytes required
c
c Variables
c  integer *2 nodigs        # number of digits found
c             i             # character string index
c             ival          # value of hexadecimal digit
c             new2(2)       # word map of newdata
c  integer *4 newdata       # accumulated value of a new item being decoded
c  logical *1 fnew          # new item being decoded
c
      implicit none
      integer *2 idata(*), nitems, nodigs, i, ival, new2(2)
      integer *4 newdata
      logical *1 fnew
      character chex*(*)
      equivalence ( new2(1), newdata )
c
      newdata = 0
      nodigs = 0
      xdecode = 1
      fnew = .false.
      do 1 i = 1, len( Chex )
c
c           decode a single hexadecimal digit
c
        ival = ichar( Chex(i:i) )
        if ( 48 .le. ival .and. ival .le. 57 ) then
          ival = ival - 48
        else if ( 65 .le. ival .and. ival .le. 70 ) then
          ival = ival - 55
        else if ( 97 .le. ival .and. ival .le. 102) then
          ival = ival - 87
        else
          ival = -1
          end if
c
c           if a separator or more than 3 hex digits assign data item
c
        if ( ( ival .lt. 0 .or. nodigs .ge. 4 ) .and. fnew ) then
          idata( xdecode ) = new2(1)
          xdecode = xdecode + 1
          if ( xdecode .gt. nitems ) goto 2
          newdata = 0
          nodigs = 0
          fnew = .false.
          end if
c
c           if found hex digit, push it onto the new data item
c
        if ( ival .ge. 0 ) then
          nodigs = nodigs + 1
          newdata = 16 * newdata + ival
          fnew = .true.
          end if
1       continue
c
2       xdecode = xdecode - 1
        return
        end


      SUBROUTINE XENCODE( CHEX, IDATA, NITEMS )
c
c    Routine to code an array of bytes into a hexadecimal ASCII string .
c
c Arguments
c  character chex*(*)           # string of hex digits coded from 'data'
c  integer *2 idata(*)          # list of 16 bit integers to be encoded
c             nitems            # number of integers to be encoded
c
c  Each byte from DATA is converted into  <hex digit> <hex digit> <space>
c  The ASCII string is terminated with a null character .
c
c Variables
c  integer *2 length            # length of O/P character string - 1
c             j                 # character string index
c             i                 # data list index
c             lochar, hichar    # ASCII equivalent of low & high hex. digit
c  character *16 cdigit         # list of hex digits
c
      implicit none
      integer *2 length, j, i, lochar, hichar, nitems, idata(*)
      character cdigit*16, Chex*(*)
      data cdigit / '0123456789ABCDEF' /
c
      length = len( Chex ) - 1
      chex = ' '
      j = 1
c
      do 1 i = 1, nitems
        if ( j .gt. length ) return
        hichar = iand( idata(i), 240 ) / 16 + 1
        chex(j:j) = cdigit( hichar: hichar )
        j = j + 1
        lochar = iand( idata(i), 15 ) + 1
        chex(j:j) = cdigit( lochar: lochar )
        j = j + 2
1       continue
      return
      end


      INTEGER FUNCTION DPRESP( IPLU, IMIN, IMAX, NOREQS )
c               Integer *4 RESPonse
c
c       The integer function JRESP returns four byte integer value between the
c  specified valid limits. This value is entered via the file with logical
c  unit number IPLU ( including possibly the keyboard ) , and is  checked
c  by JRESP, i.e. MIN < JRESP < MAX . The input file logical unit number
c  IPLU must be in the range 0 <= IPLU <= 99 , and must have been previously
c  defined if it is not a default device ( vdu, keyboard etc ). The maximum
c  number of input requests allowed NOREQS must be positive.
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer *2 Iplu              #  input file logical unit number
c integer *4 imax,             #  maximum valid response value
c            imin,             #  minimum valid response value
c integer *2 noreqs            #  maximum number of requests allowed
c
c constants
c     Hkbdlu                   # keyboard logical unit number
c     Hvdulu                   # v.d.u. logical unit number
c integer *2 cntr,             # counter for the number of input attempts
c            Iplu1             # input file logical unit number
c            nchars            # number of input characters
c            jljstr            # function : left justify string
c character Cstr*40            # input character string
c
      implicit none
      integer *2 Hkbdlu, Hvdulu
      parameter ( Hkbdlu=5, Hvdulu=6 )
c
      integer *2 iplu, noreqs, cntr, iplu1, nchars, jljstr
      integer *4 imin, imax
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. noreqs .le. 0 )
     1  write ( Hvdulu, 1 ) Iplu, Noreqs
1       format ( ' DPRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = Hkbdlu
      do 7 cntr = 1, noreqs
c
c           read data
c
c       repeat
2         read( Iplu1, 3, END=9 ) Cstr
3         format( a )
          nchars = jljstr( Cstr )
          if ( nchars .le. 0 ) goto 2
c         until( nchars > 0 )
c
c           selected min or max
c
        if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
          dpresp = imax
          return
        else if ( Cstr(1:3) .eq. 'min' .or. Cstr(1:3) .eq. 'MIN' ) then
          dpresp = imin
          return
c
c           decode & check numeric value
c
        else
          read( Cstr, *, ERR=5 ) dpresp
          if ( min( imin, imax ) .le. dpresp .and.
     1        dpresp .le. max( imin, imax ) )
     2      return
          write ( Hvdulu, 4 ) imin, imax
4         format(' DPRESP : ERROR ; Integer outside limits ', i12,
     1      ' & ', i12 )
5         if ( Iplu1 .ne. Hkbdlu ) stop
          write ( Hvdulu, 6 )
6         format( ' Try again ' )
          end if
7       continue
c
c           give up
c
        write ( Hvdulu, 8 )
8       format( ' Get help' )
9       stop
      end
