c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


c            *****************************************************
c            **                                                 **
c            **  HPPREV - screen graphics simulation of HPISPP  **
c            **     Some IGL look-alike routines are included   **
c            **     for screen management                       **
c            **                                                 **
c            *****************************************************
c
c               simulated HPISPP routines
c N.B. BUFF, AXIS, LINE and SCALE are not implemented
c
c HPINIT( LIO, LMON, LCHARS, LUNIT, LDEV )
c NEWPEN(IPEN)
c NUMBER( X, Y, HEIGHT, FPN, ANGLE, ND )
c PLOT(X,Y,IPEN)
c PLOTS( IDUM1, IDUM2, LDEV1 )
c SYMBOL( X, Y, HEIGHT, LARAY, ANGLE, ICTL )
c WHERE( X, Y, FACT )
c
c               IGL lookalikes
c ANMODE
c ARC( RADIUS, START, FIN )
c ARCTOL(TOL)
c BELL
c CLOCAP
c DRAW(XX,YY)
c DRWREL(RX,RY)
c GREEK(LEN,STRING)
c GRSTOP
c GRSTRT( IDEV, IOPT )
c INUMBR( INUM, NCHARS )
c INVIS
c LINTYP( ITYPE )
c LOCATE( IMAXPT, PXARAY, PYARAY, IDAT, IGOT )
c MARKER( XX, YY, IMARK )
c ERRBAR( DELX, DELY, DELMIN )
c MOVE( XX, YY )
c MOVREL( RX, RY )
c NEWPAG
c NEWSCR
c OPNCAP( FILE )
c PSECAP
c RNUMBR( RNUM, NDEC, NCHARS )
c RSTCAP
c TEXT( LEN, STRING )
c TXTANG( PANGLE )
c TXTSIZ( PSIZE )
c VECMOD
c VISIBL
c VWPORT( LOWX, HIGHX, LOWY, HIGHY )
c WAIT
c WINDOW( LOWX, HIGHX, LOWY, HIGHY )


      BLOCK DATA HPPREV_BLK
c
c         Initialization of common block variables for HPPREV
c
      implicit none
      include 'hpprev.cmn'
c
      data capopn, capact, wnvpchg, gotfnt, onlne / 4*.false.,
     1 .true. /
      data tolarc, currwx, currwy, hporgx, hporgy, costh, sinth, vplx,
     1  vply, vphx, vphy, wnlx, wnly, wnhx, wnhy, currwx, currwy, xcell,
     2  ycell, symsize, symang / 10.0, 4*0.0, 1.0, 2*0.0, HG_Ymax,
     3  HG_Xmax, 3*0.0, 130.0, 100.0, 2*0.0, 0.9, 1.0, 1.7, 0.0 /
      data stdout / 6 /
      end


      SUBROUTINE HPINIT( LIO, LMON, LCHARS, LUNIT, LDEV )
c
c HPPREV - subroutine HPINIT
c
c Initialize the software and the plotter
c
c Parameters :
c   LIO = handshake method desired : 0 = enquire / acknowledge
c                                    1 = xon / xoff
c                                    2 = hp-ib
c
c         NOTE : The xon / xoff method is not supported by the 7225 plotter
c                with the 17603 ( non eavesdrop ) personality module
c         NOTE : If the value of LIO is negative, the initialization of system
c                configuration parameters will be suppressed.
c
c   LMON = Monitor mode control : 0 = no monitor mode
c                                 1 = monitor mode requested
c   LCHARS = The standard character set desired
c   LUNIT  = Unit system desired : 0 = inches
c                                  1 = centimetres
c   LDEV   = Logical unit of the plotter
c
      implicit none
      integer *2 lio, lmon, lchars, lunit, ldev
      integer *2 idum1, idum2, ldev1
      include 'hpprev.cmn'
c
c     open capture section
c
      if ( capopn .and. capact ) then
        write( caplu ) hpicap, 0.0, 0.0, lio, lmon
        write( caplu ) lchars, lunit, ldev
        end if
      stdout = ldev
      go to 10


      ENTRY PLOTS( IDUM1, IDUM2, LDEV1 )
c
c  HPPREV - entry PLOTS
c
c Turn on the plotter
c Parameters :
c  LDUM1 = Dummy parameter retained for compatibility
c  LDUM2 = Dummy parameter retained for compatibility
c  LDEV1 = Logical unit of the plotter
c
c     open capture section
c
      if ( capopn .and. capact ) then
        write( caplu ) ptscap, 0.0, 0.0, idum1, idum2
        write( caplu ) ldev1
        end if
      stdout = ldev1
10    hporgx = 0.0
      hporgy = 0.0
      currwx = 0.0
      currwy = 0.0
      return
      end


      SUBROUTINE NEWPEN( IPEN )
c
c Library : HPPREV
c Function : simulate the use of newpen  :
c Argument
c  integer *2 ipen     # pen number or dashed line type
c     0 < ipen < 8 pen number on plotter
c     0 = ipen     no pens
c     -7 < i < 0   dashed line type
c
c   -1 = solid,     -2 = dotted,     -3 = short dash,  -4 = long dash
c   -5 = dot dash,  -6 = short-dash long-dash, -7 = 2-short-dashes long-dash
c
      implicit none
      integer *2 ipen
      logical *1 mycap
      include 'hpprev.cmn'
c
c     open capture section
c
      mycap = .false.
      if ( capopn .and. capact ) then
        capact = .false.
        mycap  = .true.
        write( caplu ) nwpcap, 0.0, 0.0, ipen, Hzero
        end if
c
c         select pen number or line type
c
      if ( ipen .ne. 0 .and. iabs( ipen ) .lt. 8 ) then
        call lintyp( iabs( ipen ) - 1 )
      else
        call lintyp( 0 )
        end if
c
c         close capture flags
c
      if ( mycap )
     1  capact = .true.
      return
      end


      SUBROUTINE NUMBER( X, Y, HEIGHT, FPN, ANGLE, ND )
c
c HPPREV - subroutine NUMBER
c
c Draws a floating point number a specified location, size and angle.
c
c Parameters :
c  real *4    X,Y    = The coordinates of the first character to be produced
c             HEIGHT = The height of the character to be plotted
c             FPN    = Floating point number to be written
c             ANGLE  = The angle at which the characters are to be plotted
c                       ( degrees counterclockwise from the horizontal )
c  integer *2 ND     = Number of decimal places .
c
c      ND = 0   Draw as an integer with a decimal point
c         = -1  Supress the decimal point and draw only the integer portion
c         <= -1 Supress the decimal point and truncate ABS(ND)-1 digits from
c               the least significant potion of the value .
c         >= 0  Draw ND digits to the right of the decimal point
c
c constants
c   Hndec=7         # maximum no. of dec places
c   Hfieldlen=18    # format field length
c variables
c  character Cstr*18,          # character string to be plotted
c            Cformat*8         # format of conversion
c  real cx, cy                 # updated character position
c  integer *2 i,               # loop index
c             iend,            # index of last character
c             n                # bounded value of nd
c
      implicit none
      integer *2 Hndec, Hfieldlen
      parameter( Hndec=7, Hfieldlen=18)
c
      logical *1 mycap
      character Cstr*Hfieldlen, Cformat*8
      real x, y, height, fpn, angle, cx, cy
      integer *2 nd, i, iend, n
c
      include 'hpprev.cmn'
c
c         open capture section
c
      mycap = .false.
      if ( capopn .and. capact ) then
        capact = .false.
        mycap  = .true.
        write( caplu ) numcap, x, y, nd, Hzero
        write( caplu ) height, fpn, angle
        end if
c
c         set size and angle
c
      call txtsiz( height )
      call txtang( angle )
      call sincos( symang )
c
c         convert to character string
c
      write( Cformat, 1 ) Hfieldlen,Hndec
1     format( '(f', i2, '.', i2, ')' )
      write( Cstr, Cformat ) fpn
      n = max( -7, min( 7, nd ) )
      if ( n .ge. -1 ) then
        iend = n + Hfieldlen - Hndec
      else
        iend = Hfieldlen + 1 + n
        end if
c
c         plot required characters
c
      call move( x, y )
      cx = x
      cy = y
      do 2 i = 1, iend
        if ( Cstr(i:i) .ne. ' ' )
     1    call prvpchr( Cstr(i:i), cx, cy )
2       continue
c
c         close capture flags
c
      if ( mycap )
     1  capact = .true.
      return
      end


      SUBROUTINE PLOT( X, Y, IPEN )
c
c HPPREV - subroutine PLOT on  VIS500
c
c Moves with the pen up or down to a new position
c Parameters :
c  real x,y = The coordinates of the position to which the pen is to be moved
c  integer *2 ipen = 2    draw
c                    3    move
c                    999  same as negative 3, but terminate I/O
c  NOTE : If ipen 2 or 3 negative, then reset origin after command execution
c         and flush buffer.
c
      implicit none
      logical *1 mycap
      integer *2 ipen, jpen
      real *4 x, y, xnew, ynew
c
      include 'hpprev.cmn'
c
c         sort out pen control
c
      jpen = iabs( ipen )
      if ( jpen .ne. 2 .and. jpen .ne. 3 .and. ipen .ne. 999 )
     1  return
c
c     open capture section
c
      mycap = .false.
      if ( capopn .and. capact ) then
        capact = .false.
        mycap  = .true.
        write( caplu ) pltcap, x, y, ipen, Hzero
        end if
c
c         plot coordinate
c
      xnew = x + hporgx
      ynew = y + hporgy
      if ( jpen .eq. pendown ) then
        call draw( xnew, ynew )
      else
        call move( xnew, ynew )
        end if
c
c         redefine origin relative to last point
c
      if ( ipen .lt. 0 ) then
        hporgx = xnew
        hporgy = ynew
        end if
c
c         close capture flags
c
      if ( mycap )
     1  capact = .true.
      return
      end


      SUBROUTINE SYMBOL( X, Y, HEIGHT, LARAY, ANGLE, ICTL )
c
c HPPREV - subroutine SYMBOL
c
c Draws a label character string or special centered symbol at a specified
c location, size and angle.
c
c Parameters :
c  real *4    X,Y    = The coordinates of the first character to be produced
c             HEIGHT = The height of the character to be plotted
c  logical *1 LARAY(ICTL) The array containing characters to be produced
c  integer *2 LARAY  = single ASCII character or special centred symbol integer
c  real *4    ANGLE  = The angle at which the characters are to be plotted
c                       ( degrees counterclockwise from the horizontal )
c  integer *2 ICTL   = Code specifying interpretation of LARAY
c    ICTL > 0  The number of characters in LARAY to be drawn .
c         = 0  Draw only the single character which is right-justified in
c              LARAY(1). This is the only way to reference some of the
c              characters when the host computer stores characters internally
c              in fewer than eight bits.
c         < 0  Draw the special symbol represented by the integer in LARAY(1).
c              If ICTL = -1, the x,y point is plotted with the pen up.  The
c              special symbols are drawn with no connecting lines.
c              If ICTL < -1, the x,y point is plotted with the pen down. The
c              special symbols are drawn with connecting lines.
c
      implicit none
      integer *2 ictl, ilen, i, imark
      logical *1 laray(*), temp(2), mycap
      real x, y, cx, cy, height, angle
c
      include 'hpprev.cmn'
      equivalence ( imark, temp(1) )
c
c     open capture section
c
      if ( ictl .gt .0 ) then
c           normal text
        ilen = ictl
      else if ( ictl .eq. 0 ) then
c           single character
        ilen = 2
      else
c           centered symbol
        ilen = 1
        end if
      mycap = .false.
      if ( capopn .and. capact ) then
        capact = .false.
        mycap  = .true.
        write( caplu ) symcap, x, y, ilen, ictl
        write( caplu ) height, angle, ( laray(i), i = 1, ilen )
        end if
c
      call txtsiz( height )
      call txtang( angle )
      call sincos( symang )
      if ( ictl .gt. 0 ) then
c
c          normal text
c
        cx = x
        cy = y
        do 1 i = 1, ictl
          temp(1) = laray(i)
1         call prvpchr( char(imark), cx, cy )
      else if ( ictl .eq. 0 ) then
c
c          single character in high order byte
c
        temp(1) = laray(i)
        call prvpchr( char(imark), cx, cy )
      else
c
c          special symbols
c    Use equiv of imark & temp to get LARAY(1) . Also use neg imark to signal
c    marker to ignore X,Y & use current position of screen X,Y
c
        imark = 0
        temp(1) = laray(1)
        if ( 0 .le. imark .and. imark .le. 14 ) then
          if ( ictl .ne. -1 )
     1      call draw( x, y )
          call prvdsm( sym, imark+1, x, y, 2 )
          call move( x, y )
          end if
        end if
c
c         close capture flags
c
      if ( mycap )
     1  capact = .true.
      return
      end


      SUBROUTINE WHERE( X, Y, FACT )
c
c HPPREV - subroutine WHERE
c
c Obtains the current x,y pen location and scale factor
c Parameters :
c  real X    = current x location
c       Y    = current y location
c       FCT  = current scale factor  = 1.0
c
      implicit none
      real x, y, fact
      include 'hpprev.cmn'
c
c     open capture section
c
      if ( capopn .and. capact )
     1  write( caplu ) whrcap, 0.0, 0.0, Hzero, Hzero
c
      x = currwx
      y = currwy
      fact = 1.0
      return
      end


      SUBROUTINE ARC( RADIUS, START, FIN )
c
c subroutine to draw arc of RADIUS from START angle to FIN angle (deg
c anticlock ) using algorithm for parametric circle
c
      implicit none
      real radius, start, fin, x, y, thstrt, thfin, tmptht, arclen,
     1  tmptol, xrel, yrel, tol, psize, pangle
      integer *2 narcs, i
      include 'hpprev.cmn'
c
      x = currwx
      y = currwy
      thstrt = start
      thfin = fin
c
c set angles modulo 360.
c
      if ( abs(thstrt) .gt. 360.0 )
     1  thstrt = amod( start, 360.0 )
      if ( abs(thfin) .gt. 360.0 )
     1  thfin = amod( fin, 360.0 )
c
c make both positive angles & thfin bigger than thstart for correct length
c
      if ( thstrt .lt. 0.0 )
     1  thstrt = 360.0 + thstrt
      if ( thfin .lt. 0.0 )
     1  thfin = 360.0 + thfin
      tmptht = thfin
      if ( thfin .lt. thstrt )
     1  tmptht = thfin + 360.0
      arclen = tmptht - thstrt
c
c chord tol set to give tol closest to tolarc but with integral
c number of chords
c
      if ( arclen .le. tolarc ) then
       narcs = 1
       tmptol = arclen
      else
        narcs = int( arclen / tolarc + 0.5 )
        tmptol = arclen / narcs
        end if
c
c         move to start position
c
      call sincos( thstrt )
      xrel = radius
      yrel = 0
      call rotate( xrel, yrel )
      call move( x + xrel, y + yrel )
c
c         draw arc segments
c
      call sincos( tmptol )
      do 1 i = 1,narcs
        call rotate(xrel, yrel)
        call draw( x+xrel, y+yrel )
1       continue
      return


      ENTRY ARCTOL( TOL )
c
c   set arc tolerance ( ie. resolution ) to TOL degrees
c
      if ( capopn .and. capact )
     1  write( caplu ) actcap, tol, 0.0, Hzero, Hzero
      tolarc = tol
      return


      ENTRY TXTSIZ( PSIZE )
c
c set text character height to PSIZE in window coordinates
c
      if ( capopn .and. capact )
     1  write( caplu ) tszcap, psize, 0.0, Hzero, Hzero
      symsize = abs( psize ) * 0.1
      return


      ENTRY TXTANG( PANGLE )
c
c code for capture section in each routine
c
      if ( capopn .and. capact )
     1  write( caplu ) tancap, pangle, 0.0, 0, 0
      symang = pangle
      return
      end


      SUBROUTINE GRSTRT( IDEV, IOPT )
c
c HPPREV - subroutine GRSTRT - IGL lookalike
c    Start screen graphics .
c Parameters :
c  integer *2 idev     # Hercules graphics card only
c             iopt     # ??????
c  N.B. These arguments are not actually used
c
      implicit none
      character chr*1
      integer *2 idev, iopt, i, iptr, symnum, cpairs
c
      include 'hpprev.cmn'
c
c         DATA STRUCTURE FOR CHARACTER FONTS
c  There are two  character fonts,
c    1) ASCII characters and
c    2) special characters .
c  The number of symbols and the TOTAL number of data points in each fonts
c is known and specified in HPPREV.CMN .
c  The font data a concatenation of two arrays ;
c    1) a 2 by NUMBER_OF_SYMBOLS array of integer*2; i.e. an array of pairs
c       with one pair for each character . The first number of a pair is the
c       index of the first point, and the second number of a pair is the index
c       of the last point in the font data .
c    2) a string of NUMBER_OF_POINTS*2 . Each pair of points is a point in a
c       character .
c
c real ascfnt(nascpnt*2)         # fonts of all ASCII characters
c      symfnt(nspcpnt*2)         # fonts of all special centred symbols
c integer *2 ascdir(2,nascsym)   # directory of ASCII char fonts in ASCFNT
c            symdir(2,nspcsym)   # directory of special char fonts in SYMFNT
c
      integer *2 ascdir(2,nascsym), symdir(2,nspcsym)
      real ascfnt(nascpnt*2), symfnt(nspcpnt*2)
      equivalence ( ascdir(1,1), asc(1) ),
     1            ( ascfnt(1), asc(nascsym+1) ),
     2            ( symdir(1,1), sym(1) ),
     3            ( symfnt(1), sym(nspcsym+1) )
c
      if ( .not. gotfnt ) then
c
c get  ascii fonts
c
        iptr = 1 + nascsym
        do 1 i = 1, nascsym
1         ascdir(1,i) = -1
        open( UNIT=FNTLU, FILE='\ilg\ascfnt.ilg', STATUS='old',
     1    FORM='formatted', ACCESS='sequential' )
2       read( FNTLU, *, END=3 ) chr, cpairs,
     1      ( asc(i), i = iptr, iptr + cpairs * 2 - 1 )
          symnum = max( 1, ichar(chr) - 31 )
          ascdir( 1, symnum ) = iptr
          iptr = iptr + cpairs * 2
          ascdir( 2, symnum ) = iptr - 2
          goto 2
3       close( UNIT=FNTLU )
c
c get special symbol fonts
c
        iptr = nspcsym + 1
        do 4 i = 1, nspcsym
4         symdir(1,i) = -1
        open( UNIT=FNTLU, FILE='\ilg\symfnt.ilg', STATUS='old',
     1    FORM='formatted', ACCESS='sequential' )
5       read( FNTLU, *, END=6 ) chr, cpairs,
     1      ( sym(i), i = iptr, iptr + cpairs * 2 - 1 )
          symnum = max( 1, ichar(chr) - 47 )
          symdir( 1, symnum ) = iptr
          iptr = iptr + cpairs * 2
          symdir( 2, symnum ) = iptr - 2
          goto 5
6       close ( UNIT=FNTLU )
        gotfnt = .true.
        end if
c
c text cell aspects, text size, angle params
c
      xcell = 9.0
      ycell = 10.0
      symsize = 0.17
      symang  = 0.0
c
c scaling params
c
      vplx = 0.0
      vply = HG_Ymax
      vphx = HG_Xmax
      vphy = 0.0
      wnlx = 0.0
      wnly = 0.0
      wnhx = 129.0
      wnhy = 99.0
      call v5scale
      currwx = 0.0
      currwy = 0.0
      icurrvx = 0
      icurrvy = 0
c
c arc tolerance
c
      tolarc = 10.0
c
c clear both screens
c
      call newscr
      call newpag
      return


      ENTRY GRSTOP
c
c HPPREV - entry GRSTOP - ILG lookalike
c    Terminate graphics
c stop plotting - clear screen , reset scrolling & set to anmode
c
      if (capopn) call clocap
      call newpag
      call anmode
      call newscr
      return
      end


      SUBROUTINE INUMBR( INUM, NCHARS )
c
c       Plot the integer INUM with NCHARS characters
c    Size and angle previously set by calls to TXTSIZ & TXTANG
c
      implicit none
      integer *2 inum, nchars
      character Cstr*12
c
      write( Cstr, 1 ) inum
1     format( i6 )
      call text( nchars, cstr( max( 1, min( 6, 7 - nchars ) ): ) )
      return
      end


      SUBROUTINE MARKER( X, Y, IMARK )
c
c       draw the special symbol specified IMARK at location X, Y
c    Size and angle previously set by calls to TXTSIZ & TXTANG
c
      implicit none
      real x, y
      integer *2 imark
      include 'hpprev.cmn'
c
      if ( 0 .le. imark .and. imark .le. 14 ) then
        call sincos( symang )
        call prvdsm( sym, imark+1, x, y, 2 )
        call move( x, y )
        end if
      return
      end


      SUBROUTINE ERRBAR( DELX, DELY, DELMIN )
c
c       ERRor BAR
c   Draws an error bar at the current pen position .
c   The error bar is rotated by the angle specified by TXTANG .
c If DELX >= DELMIN and DELY >= DELMIN, the error bar is drawn as a rectangle .
c If DELX = 0, the error bar is drawn as an `I' . If DELY = 0, the error bar is
c drawn as an `I' lying on its side . With DELX < DELMIN or DELY < DELMIN, the
c error bar is in transition between a rectangle and an `I' .
c
c Arguments
c  real delx,           # width of the error bar
c       dely,           # height of the error bar
c       delmin,         # minimum width of the bar terminator
c
c Variables
c  real delHx, delHy,   # unrotated 1st quadrant position of width bar
c       delVx, delVy,   # unrotated 1st quadrant position of height bar
c       rx,ry,          # centre of the error bar
c       Ax,Ay, Bx,By,   # rotated end points of the height bar
c       Cx,Cy, Dx,Dy    # rotated end points of the width bar
c  logical * rectangle  # the error bar forms a rectangle
c
      implicit none
      real delX, delY, delmin, delHx, delHy, delVx, delVy,
     1  rx, ry, Ax, Ay, Bx, By, Cx, Cy, Dx, Dy
      logical *1 rectangle
      include 'hpprev.cmn'
c
c           decide unrotated positions and lengths of the height & width bars
c
      call sincos( symang )
      rx    = currwx
      ry    = currwy
      delHx = abs( delX ) / 2.0
      delVy = abs( delY ) / 2.0
      delVx = max( delHx, delMin / 2.0 )
      delHy = max( delVy, delMin / 2.0 )
      rectangle = delHx .eq. delVx .and. delHy .eq. delVy
c
c           left end of top height bar
c
      Ax = -delVX
      Ay = delVy
      call rotate( Ax, Ay )
      call move( rx + Ax, ry + Ay )
c
c           right end of top height bar
c
      Bx = delVx
      By = delVy
      call rotate( Bx, By )
      call draw( rx + Bx, ry + By )
c
c           top end of right width bar
c

      if ( .not. rectangle ) then
        Cx = delHx
        Cy = delHy
        call rotate( Cx, Cy )
        call draw( rx + Cx, ry + Cy )
        end if
c
c           bottom end of right width bar
c
      Dx = delHx
      Dy = - delHy
      call rotate( Dx, Dy )
      call draw( rx + Dx, ry + Dy )
c
c           right & left ends of bottom height bar
c
      if ( delVy .gt. 0.0 ) then
        if ( .not. rectangle )
     1    call draw( rx - Ax, ry - Ay )
        call draw( rx - Bx, ry - By )
c
c           bottom and top ends of left width height bar
c
        if ( delVx .gt. 0.0 ) then
          if ( .not. rectangle )
     1      call draw( rx - Cx, ry - Cy )
          call draw( rx - Dx, ry - Dy )
          end if
      else if ( delVx .gt. 0 ) then
        if ( .not. rectangle )
     1    call move( rx - Cx, ry - Cy )
        call draw( rx - Dx, ry - Dy )
        end if
      return
      end


      SUBROUTINE MVDRAW
c
c handles move,draw & relative move,draw commnads
c
      implicit none
      integer *2 left, right, bottom, top
      parameter( left=1, right=2, bottom=4, top=8 )
c
      real xx, yy, rx, ry, x, y, x1, y1, x2, y2, x3, y3
      logical *1 drawfl, relfl
      integer *2 code, c1, c2, c, ivx1, ivx2, ivy1, ivy2, ipen
c
      include 'hpprev.cmn'
c
c various entry points
c
      ENTRY DRWREL(RX,RY)
      drawfl = .true.
      go to 5

      ENTRY MOVREL(RX,RY)
      drawfl = .false.
5     relfl = .true.
      go to 20

      ENTRY DRAW(XX,YY)
      drawfl = .true.
      go to 10

      ENTRY MOVE (XX,YY)
      drawfl = .false.
10    relfl =.false.
20    continue
c
c add on relative co-ords or set to params
c
      if (relfl) then
        x = currwx + rx
        y = currwy + ry
      else
        x = xx
        y = yy
        end if
c
c now clip - cohen - sutherland algorithm ref newman&sproull pg 66
c
      x1 = currwx
      y1 = currwy
      x2 = x
      y2 = y
      c1 = code( x1, y1 )
      c2 = code( x2, y2 )
15    if ( c1 .ne. 0 .or. c2 .ne. 0 ) then
c
c           totally invisible
        if ( iand( c1, c2 ) .ne. 0 ) go to 99
        c = c1
        if ( c .eq. 0 ) c = c2
c
c           crosses left edge
        if ( iand( c, left ) .ne. 0 )  then
          y3 = y1 + ( wnlx - x1 ) * ( y2 - y1 ) / (x2 - x1 )
          x3 = wnlx
        else if ( iand( c, right ) .ne. 0 ) then
c
c           crosses right edge
          y3 = y1 + ( wnhx - x1 ) * ( y2 - y1 ) / (x2 - x1 )
          x3 = wnhx
c
c           crosses left edge
        else if ( iand( c, bottom ) .ne. 0 ) then
          x3 = x1 + ( wnly - y1 ) * (x2 - x1 ) / ( y2 - y1 )
          y3 = wnly
c
c           crosses top edge
        else if ( iand( c, top ) .ne. 0 ) then
          x3 = x1 + ( wnhy - y1 ) * (x2 - x1 ) / ( y2 - y1 )
          y3 = wnhy
          end if
        if ( c .eq. c1 ) then
          x1 = x3
          y1 = y3
          c1 = code( x1, y1 )
        else
          x2 = x3
          y2 = y3
          c2 = code( x2, y2 )
          end if
        goto 15
        end if
c
c     open capture section
c
      if ( capopn .and. capact ) then
        if ( drawfl ) then
          if ( x1 .ne. prx .or. y1 .ne. pry )
     1      write( caplu ) pltcap, x1, y1, 3, Hzero
c             draw
          ipen = 2
        else
c             move
          ipen = 3
          end if
        write( caplu ) pltcap, x2, y2, ipen, Hzero
        end if
c
c x1,y1 - x2,y2 is visible portion of line
c
c viewport or window changed ??
c
        if ( wnvpchg )
     1    call v5scale
c
c calculate screen co-ords
c
        if ( x1 .eq. prx ) then
          ivx1 = icurrvx
        else
          ivx1 = int( xscale * ( x1 - wnlx ) + vpxl05 )
          end if
        if (y1 .eq. pry) then
          ivy1 = icurrvy
        else
          ivy1 = int( yscale * ( y1 - wnly ) + vpyl05 )
          end if
        ivx2 = int( xscale * ( x2 - wnlx ) + vpxl05 )
        ivy2 = int( yscale * ( y2 - wnly ) + vpyl05 )
        prx = x2
        pry = y2
        if ( drawfl .and. onlne )
     1    call HG_line( ivx1, ivy1, ivx2, ivy2 )
c
c update new screen position
c
      icurrvx = ivx2
      icurrvy = ivy2
c
c set new current window position
c
99    currwx = x
      currwy = y
      return
      end




      INTEGER *2 FUNCTION CODE(X,Y)
c
c code function for sector definition of cohen-sutherland clipping alg
c
      implicit none
      integer *2 left, right, bottom, top
      parameter( left=1, right=2, bottom=4, top=8 )
c
      real x,y
      include 'hpprev.cmn'
c
      code = 0
      if ( x .lt. wnlx )  then
        code = left
      else if( x .gt. wnhx ) then
        code = right
        end if
      if ( y .lt. wnly ) then
        code = ior( code, bottom )
      else if( y .gt. wnhy ) then
        code = ior( code, top )
        end if
      return
      end


c
c HPPREV - CAPTURE routines - CAPTURE, CLOCAP, PSECAP, RSTCAP
c


      SUBROUTINE OPNCAP( FILE )
c
c open capture file
c
      implicit none
      character*(*) file
      include 'hpprev.cmn'
c
      if ( capopn .or. len(file) .le. 0 )
     1  return
      open( UNIT=caplu, FILE=file, ACCESS='sequential',
     1  FORM='unformatted', STATUS='unknown' )
      capopn = .true.
      capact = .true.
      return


      ENTRY CLOCAP
c
c close capture file
c
      if ( capopn )
     1  close( UNIT=caplu )
      capopn = .false.
      capact = .false.
      return


      ENTRY PSECAP
c
c pause capture - temporary halt
c
      capact = .false.
      return


      ENTRY RSTCAP
c
c restart capture - temporary halt
c
      capact = .true.
      return
      end


      SUBROUTINE RNUMBR( RNUM, NDEC, NCHARS )
c
c       Plot the real number RNUM with NCHARS characters and NDEC decimal
c  places
c    Size and angle previously set by calls to TXTSIZ & TXTANG
c
      implicit none
      real rnum
      character *18 Cstr
      integer *2 ndec, nchars, ndec1, nchars1
c
      ndec1 = max( 0, min( 7, ndec ) )
      nchars1 = max( 2, min( 18, nchars ) )
      write( Cstr, 1 ) rnum
1     format( f18.7 )
      call text( nchars1, Cstr( 12 + ndec1 - nchars1 : ) )
      return
      end


      SUBROUTINE TEXT( LEN, STRING )
c
c           Plot string at current window x & y
c    Size and angle previously set by calls to TXTSIZ & TXTANG
c
      implicit none
      character string*(*)
      real cx, cy
      integer *2 len, i
      include 'hpprev.cmn'
c
c         place the pen at the start point
c
      call sincos( symang )
      cx = currwx
      cy = currwy
      call move( cx, cy )
c
c         plot characters, one at a time
c
      do 1 i = 1, len
1       call prvpchr( string(i:i), cx, cy )
      return
      end


      SUBROUTINE GREEK( LENG, STRING )
c
c           Plot string of greek and other special characters at current
c  window x & y . Size and angle previously set by calls to TXTSIZ & TXTANG
c
      implicit none
      character string*(*)
      integer *2 leng, i, chind, iind(2)
      logical *1 ok
      real *4 rind, rx, ry, cx, cy
      equivalence ( rind, iind(1) )
      include 'hpprev.cmn'
c
c         place the pen at the start point
c
      call sincos( symang )
      cx = currwx
      cy = currwy
      call move( cx, cy )
c
c         plot characters, one at a time
c
      do 1 i = 1, leng
        chind = ichar( string(i:i) ) - 47
        ok = chind .ge. 0 .and. chind .le. nspcsym
        if ( ok ) then
          rind = sym( chind )
          ok = iind(1) .ne. -1
          end if
        if ( .not. ok )
     1    chind = ichar('@') - 47
c
c draw character  - not centered
c
        call prvdsm( sym, chind, cx, cy, 3 )
c
c move pen to bottom right of char cell
c
        rx =  symsize * xcell
        ry = 0.0
        call rotate(rx,ry)
        cx = cx + rx
        cy = cy + ry
1       continue
      call move( cx, cy )
      return
      end


      SUBROUTINE VWPORT( LOWX, HIGHX, LOWY, HIGHY )
c
c HPPREV - subroutine VWPORT - IGL lookalike
c    Set viewport limits
c       Note that the sign of the y-scale factor has been reversed for the
c Hercules graphics card and the y-offset is shifted to the other limit .
c The scaling is changed to simulate the Visual 500 terminal .
c
      implicit none
      real lowx, highx, lowy, highy
      include 'hpprev.cmn'
c
c     open capture section
c
      if ( capopn .and. capact ) then
        write( caplu ) vwpcap, lowx, highx, Hzero, Hzero
        write( caplu ) lowy, highy
        end if
c
      vplx = max( lowx * HG_Xmax / V500_Xmax, 0.0 )
      vphx = min( highx * HG_Xmax / V500_Xmax, HG_Xmax )
      vply = max( HG_Ymax - lowy * HG_Ymax / V500_Ymax, 0.0 )
      vphy = min( HG_Ymax - highy * HG_Ymax / V500_Ymax, HG_Ymax )
      wnvpchg = .true.
      return
      end


      SUBROUTINE WINDOW( LOWX, HIGHX, LOWY, HIGHY )
c
c set user space limits
c
      implicit none
      real lowx, highx, lowy, highy
      include 'hpprev.cmn'
c
c     open capture section
c
      if ( capopn .and. capact ) then
        write( caplu ) wincap, lowx, highx, Hzero, Hzero
        write( caplu ) lowy, highy
        end if
c
      wnlx = lowx
      wnhx = highx
      wnly = lowy
      wnhy = highy
      wnvpchg = .true.
      return
      end


      SUBROUTINE PRVDSM( FNT, IND, X, Y, CENT )
c
c HPPREV - subroutine PRVDSM
c
c              Draw symbol
c Parameters
c  real fnt(*)    # array of fonts data ( including directory )
c       x,y       # position of character ( window coordinates )
c integer *2 ind  # index of required character in fonts array
c            cent # 1 = standard ASCII character
c                 # 2 = centred special symbol
c                 # 3 = non-centred special symbol i.e. Greek etc
c
c variables
c  integer *2 i,         # index for fonts data
c             ipen,      # penup or pendown for move or draw
c             idirfnt(2) # start & finish indices of font data
c  real *4 rx, ry,       # position of a point wrt character position
c
      implicit none
      integer *2 i, ind, cent, idirfnt(2)
      logical *1 fpenup
      real *4 fnt(*), rx, ry, x, y, rdirfnt
      include 'hpprev.cmn'
      equivalence( rdirfnt, idirfnt(1) )
c
c now go through font co-ords
c
      rdirfnt = fnt(ind)
      do 1 i = idirfnt(1), idirfnt(2), 2
        rx = fnt(i)
        ry = fnt(i+1)
        fpenup = i .eq. idirfnt(1)
        if ( rx .lt. 0.0 ) then
          fpenup = .true.
          rx = -rx
          end if
        if ( ry .lt. 0.0 ) then
          fpenup = .true.
          ry = -ry
          end if
        if ( cent .eq. 1 ) then
          if ( ind .eq. 72 .or. ind .eq. 75 .or. ind .eq. 81 .or.
     1        ind .eq. 82 .or. ind .eq. 90 )
     2      ry = ry - 4.0
        else if ( cent .eq. 2 ) then
          rx = rx - 4.5
          ry = ry - 5.0
        else if ( cent .eq. 3 ) then
          if ( ind .gt. 48 )
     1      ry = ry - 3.0
          end if
        rx = rx * symsize
        ry = ry * symsize
c
c rotate coords
c
        call rotate(rx,ry)
        rx = x + rx
        ry = y + ry
        if ( fpenup ) then
          call move( rx, ry )
        else
          call draw( rx, ry )
          end if
1       continue
      return
      end


      SUBROUTINE PRVPCHR( CHAR, X, Y )
c
c Draw the ASCII character CHAR at X,Y using params in symbol common block
c NOTE : x,y are updated to position at bottom right of rotated character cell
c
      implicit none
      character char*1
      logical *1 ok
      integer *2 chind, iind(2)
      real *4 x, y, rx, ry, rind
      include 'hpprev.cmn'
      equivalence ( rind, iind(1) )
c
c check valid char
c
      chind = ichar(char) - 31
      ok = chind .ge. 0 .and. chind .le. nascsym
      if ( ok ) then
        rind = asc( chind )
        ok = iind(1) .ne. -1
        end if
      if ( .not. ok )
     1  chind = ichar('#') - 31
c
c draw character  - not centered
c
      call prvdsm( asc, chind, x, y, 1 )
c
c move pen to bottom right of char cell
c
      rx = symsize * xcell
      ry = 0.0
      call rotate(rx,ry)
      x = x + rx
      y = y + ry
      call move( x, y )
      return
      end


      SUBROUTINE ROTATE( RX, RY )
c
c  rotate the coordinate (RX,RY) by the angle specified by a call to
c  SINCOS
c
      implicit none
      real rx, ry, tx, ty
      include 'hpprev.cmn'
c
      tx = rx
      ty = ry
      rx = tx * costh - ty * sinth
      ry = tx * sinth + ty * costh
      return
      end


      SUBROUTINE SINCOS( ANG )
c
c Set angle ANG to modulo 360
c
      implicit none
      real ang, angl
      include 'hpprev.cmn'
c
      angl = ang - ( aint( ang / 360.0 ) * 360.0 )
c
c do 0, 90, 180 and 270 as separate angles to avoid errors
c
      if (angl .eq. 0.0) then
        costh = 1.0
        sinth = 0.0
      else if (angl .eq. 90.0) then
        costh = 0.0
        sinth = 1.0
      else if (angl .eq. 180.0) then
        costh = -1.0
        sinth = 0.0
      else if (angl .eq. 270.0) then
        costh = 0.0
        sinth = -1.0
      else
        angl = angl*radcon
        costh = cos( angl )
        sinth = sin( angl )
        end if
      return
      end


      SUBROUTINE V5SCALE
c
c Calculates window to viewport mapping params .
c
      include 'hpprev.cmn'
      xscale = ( vphx - vplx ) / ( wnhx - wnlx )
      yscale = ( vphy - vply ) / ( wnhy - wnly )
      vpxl05 = vplx + 0.5
      vpyl05 = vply + 0.5
      wnvpchg = .false.
      return
      end


c???????????????????????????????????????????????????????????????????????
c???????????????????????????????????????????????????????????????????????
c
c     ALL THE CODE BELOW THIS LINE IS HARDWARE DEPENDENT .
c

      SUBROUTINE LINTYP( ITYPE )
c
c sets line style for subsequent plotting
c        ( for the Hercules graphics card )
c argument
c  integer *2 itype          # dashed line type
c Variables
c  integer *2 ipattern(8)    # bit pattern for line type
c             imask          # bit mask
c
c   bit pattern     Hex    dec itype  Description
c 1111111111111111  FFFF     -1  0   solid
c 0011001100110011  3333  13107  1   dotted
c 0011111100111111  3F3F  16191  2   short dash
c 0001111111111111  1FFF   8191  3   long dash
c 0011001111111111  33FF  13311  4   dot, dash
c 0011110011111111  3CFF  15615  5   short dash, long dash
c 0011001100111111  333F  13119  6   2 short dashes, long dash
c 0000000000000000  0000      0  7   nothing
c
      implicit none
      integer *2 itype, imask, ipattern(8)
      data ipattern / -1, 13107, 16191, 8191, 13311, 15615, 13119, 0 /
      include 'hpprev.cmn'
c
c     open capture section
c
      if ( capopn .and. capact )
     1  write( caplu ) lincap, 0.0, 0.0, itype, Hzero
c
      imask = ipattern( mod( abs( itype ), 8 ) + 1 )
      call HG_mask( imask )
      return
      end


      SUBROUTINE WAIT
c
c waits for user c/r
c
c Constants
c  integer *2 Hblack,          # switch dots off
c             Hwhite,          # switch dots on
c             Hoff,            # switch graphics off
c             Hon              # switch graphics on
c
      implicit none
      integer *2 Hblack, Hwhite
      parameter( Hblack=0, Hwhite=1 )
c
      include 'hpprev.cmn'
c
      read( stdin, 1 )
1     format()
      return


      ENTRY NEWPAG
c
c clear graphics screen
c
      if ( capopn .and. capact )
     1  write( caplu ) npgcap, 0.0, 0.0, Hzero, Hzero
      if ( onlne )
     1  call HG_erase
      return


      ENTRY NEWSCR
c
c clear alphanumeric screen
c
      if ( capopn .and. capact )
     1  write( caplu ) nsccap, 0.0, 0.0, Hzero, Hzero
      if ( onlne )
     1  write( stdout, 2 )
2     format( ' ', 24(/) )
      return


      ENTRY INVIS
c
c sets screen into "dots off" mode
c
      if ( capopn .and. capact )
     1  write( caplu ) invcap, 0.0, 0.0, Hzero, Hzero
      if ( onlne )
     1  call HG_colour( Hblack )
      return


      ENTRY VISIBL
c
c resets screen back to "dots on" mode
c
      if ( capopn .and. capact )
     1  write( caplu ) viscap, 0.0, 0.0, Hzero, Hzero
      if ( onlne )
     1  call HG_colour( Hwhite )
      return


      ENTRY ANMODE
c
c set term to vis500 alpha numeric mode
c
      if ( onlne )
     1  call HG_Off
      return


      ENTRY VECMOD
c
c set term to vis500 vector mode
c
      if ( onlne )
     1  call HG_On
      return


      ENTRY ONLINE
c
c routines to toggle online/offline mode via switch onlne
c
      onlne = .true.
      return


      ENTRY OFLINE
      onlne = .false.
      return
      end


      SUBROUTINE SCR_DUMP( CPRTFILE )
c
c       Dumps the Hercules Graphics screen onto the printable file CPRTFILE
c Arguments
c  character Cprtfile*(*)   # name of printable dump file
c
c constants
c  Character Clspace*3      # line spacing   <esc>, 'A', 8
c            Clstart*5      # start of line  <esc>, '*', 6, 720
c            Clend*1        # end of line    <line feed>
c            Cmode4*8       # mode 4 etc.    <esc>, 'J', 4, <esc>, '*', 4, 0, 4
c            Cbelleof*6     # <line feed>, <esc>, 2, <bell>, <bell>, <control-Z>
c            Cdashes*2      # dashes         128, 128
c variables
c  integer *2 iprtrow,      # printer row number
c             iprtcol,      # printer column number
c             PrtBytes(4)   # bytes sent to printer for one character position
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
      include 'hpprev.cmn'
c
      character Cprtfile*(*), Clspace*3, Clstart*5, Clend*1, Cmode4*8,
     1  Cbelleof*6, Cdashes*2
      integer *2 iprtrow, iprtcol
      logical *1 PrtBytes(8)
      data Clspace  / z'1B4108' /
      data Clstart  / z'1B2A06D002' /
      data Clend    / z'0A' /
      data Cmode4   / z'1B4A041B2A040004' /
      data Cbelleof / z'0A1B0207071A' /
      data Cdashes  / z'8080' /
c
      open( UNIT=dumplu, FILE=Cprtfile, ACCESS='transparent',
     1  FORM='unformatted', STATUS='unknown', ERR=3 )
      write( dumplu ) Clspace
      do 2 iprtrow = 0, 43
        write( dumplu ) Clstart
        do 1 iprtcol = 0, 89
          call HG_dump( iprtrow, iprtcol, PrtBytes )
          write( dumplu ) PrtBytes
1         continue
        write( dumplu ) Clend
2       continue
      write( dumplu ) Cmode4, ( Cdashes, iprtcol = 1, 512 ), Cbelleof
      close( UNIT=dumplu )
      return
c
3     write( Hvdulu, 4 ) Cprtfile
4     format( ' SCR_DUMP : ABORT : Can''t open file ', a )
      return
      end


      SUBROUTINE LOCATE( IMAXPT, RXARAY, RYARAY, CDAT, IGOT )
c
c Returns array of coordinates in window units of cross-hair positions
c Arguments :
c  integer *2 IMAXPT   = maximum number of data points that can be read
c  real *4  RXARAY(*)  = array of x values
c           RYARAY(*)  = array of corresponding y values
c  character*1 CDAT(*) = array of characters used to transmit x-hair positions
c  integer *2 IGOT     = actual number positions returned
c sequence is terminated when 'x' is typed or IGOT is equal to IMAXPT .
c
      implicit none
      integer *2 ix, iy, imaxpt, igot
      character cdat(imaxpt)*1
      real rxaray(*), ryaray(*)
      include 'hpprev.cmn'
c
      if ( wnvpchg )
     1  call v5scale
      igot = 0
c     repeat
1       igot = igot + 1
        call HG_xhair( ix, iy, cdat( igot ) )
        rxaray(igot) = ( real( ix ) - vplx ) / xscale + wnlx
        ryaray(igot) = ( real( iy ) - vply ) / yscale + wnly
        if ( cdat(igot) .ne. 'X' .and. cdat(igot) .ne. 'x' .and.
     1      igot .lt. imaxpt ) go to 1
c       until( not x and igot < imaxpt )
      return
      end
