c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


c
c       Direct calls to external routines
c       *********************************
c  library      integer *2      real            logical *1      subroutine
c               function        function        function
c --------------------------------------------------------------------------
c  fortran      mod int index abs iand
c  intrinsic
c
c  f77lib       jintus xdecode  rfltus          emgopn emgcls   sieq16 dpeq24
c               io_read io_write                emgwrt ein2     daput daflush
c                                               efact  e1byte
c                                               existf io_serial
c                                               io_paral
c  asslib       mxcrc                                           sup250 supsim
c
c
c               ##############################################
c               ###                                        ###
c               ###     Basic Data Acquisition Routines    ###
c               ###                                        ###
c               ##############################################
c
c These are SBC pseudo-directives :
c       SLOACQ( RCYTIM, INOCY, ICYLEN, ICHLST, IPFLU, JREC )
c       SEQACQ( RCYTIM, RSAMPD, DNOCY, ICYLEN, IPFLU, JREC )
c       SANACQ( RCYTIM, DNOCY, ICYLEN, ICHLST, IPFLU, JREC )
c       SIMACQ( DNOCY, ICYLEN, IPFLU, JREC )
c

      SUBROUTINE SLOACQ( RCYTIM, INOCY, ICYLEN, ICHLST, IPFLU, IREC )
c
c       ACQuire data from the SLOw a/d converter .
c          Ie. the DAS 1128 .
c     INOCY frames of data , each of ICYLEN samples , are acquired into the
c  SBC data array . The frame period is RCYTIM seconds . The channel to
c  channel sampling period within each frame is 24.25 microseconds for
c  the Z80 A  (4 MHz) CPU . The sequence of ICYLEN A/D channel addresses for
c  each acquisition cycle is in ICHLST .
c     The data is transmitted up the parallel communications port and is
c  deposited in the random access file with logical unit number IPFLU ,
c  starting at record IREC .
c
c Arguments
c  real Rcytim              # acquisition cycle period
c  integer Inocy,           # Number of data acquisition cycles
c          Icylen,          # number of samples per acquisition cycle
c          Ichlst(Icylen),  # A/D channel address list
c          Ipflu,           # logical unit number of data file
c          Irec             # data file record number
c
c constant
c  Hstaddr=17408            # start address of data array in SBC memory
c variables
c  integer *2 Inbyte(2),    # Total number of data bytes - unpacked
c  integer *4 Dcylen        # acquisition cycle length - no. of channels used
c             Dnbyte        # Total number of data bytes - unpacked
c  logical *1 sbcada,       # use the DAS 1128 A/D on the Z80 A
c             sppsbc        # parallel transmit block data from SBC
c             ssssbc        # serial transmit block data from quasi disk
c             ok            # result of writing data transimission command
c  subroutine swrtq         # send `write queue' directive
c             sxeqq         # send `execute queue' directive and receive data
c
c
      implicit none
      integer *2 Hstaddr
      parameter( Hstaddr=17408)
c
      logical *1 sbcada, sppsbc, ssssbc, ok
      integer *2 Icylen, Inocy, Ichlst(*), Ipflu, Irec, Inbyte(2)
      integer *4 Dcylen, Dnbyte
      real rcytim
      equivalence ( Dnbyte, Inbyte(1) )
      include 'sbctrl.cmn'
c
      Dcylen = Icylen
      Dnbyte = Dcylen * 2 * Inocy
      call sbwrtq
      if ( sbcada( Rcytim, Inocy, Icylen, Ichlst ) ) then
        if ( fppdis ) then
          ok = ssssbc( 17408, Inbyte(1) )
        else
          ok = sppsbc( 17408, Inbyte(1) )
          end if
        if ( ok )
     1    call sbxeqq( Ipflu, Irec )
        end if
      return
      end



      SUBROUTINE SEQACQ( RCYTIM, RSAMPD, DNOCY, ICYLEN, IPFLU, IREC )
c
c       ACQuire data from the fast DAS 250A A/D converter - in sequential
c          channel addressing mode .
c     DNOCY frames of data , each of ICYLEN samples , are acquired from the
c  DAS 250A A/D converter . The frame period is RCYTIM seconds . The channel
c  to channel sampling period within each frame is RSAMPD seconds . The A/D
c  is in sequential channel addressing mode .
c     The data is acquired onto Quasi Disk , is then transmitted up the
c  parallel communications port and is deposited in the random access file
c  with logical unit number IPFLU , starting at record IREC .
c
c Arguments
c  real Rcytim,             # acquisition cycle period
c       Rsampd              # A/D sampling interval
c  integer *4 Dnocy,        # Number of data acquisition cycles
c  integer Icylen,          # number of samples per acquisition cycle
c  integer Ipflu,           # logical unit number of data file
c          Irec             # data file record number
c
c constants
c  Hqdid=0                  # base address of the quasi disk
c  Hstaddr=0                # quasi-disk start address
c variables
c  integer *4 Dnbyte,       # Total number of packed data bytes
c             Dnopak        # function : calculate number of packed bytes
c  logical *1 smpseq,       # use the DAS 250A A/D in sequential mode
c             smpxeq,       # execute MicroProfessor queue
c             sqddma,       # initialize DMA card for filling quasi disk
c             sppqd,        # parallel transmit block data from quasi disk
c             sssqd         # serial transmit block data from quasi disk
c             ok            # result of writing data transimission command
c  subroutine sambyt        # calculate number of packed bytes
c             swrtq         # send `write queue' directive
c             sxeqq         # send `execute queue' directive and receive data
c
      implicit none
      integer *2 Hqdid
      integer *4 Hstaddr
      parameter( Hqdid=0, Hstaddr=0 )
c
      integer *4 Dnocy, Dnbyte, Dnopak
      integer *2 Icylen, Ipflu, Irec
      real Rcytim, Rsampd
      logical *1 smpseq, sqddma, smpxeq, sppqd, sssqd, ok
      include 'sbctrl.cmn'
c
      Dnbyte = Dnopak( Dnocy * Icylen )
      call sbwrtq
      if ( smpseq( Rcytim, Rsampd, Dnocy, Icylen ) ) then
        if ( sqddma( Hstaddr, Dnbyte, Hqdid ) ) then
          if ( smpxeq() ) then
            if ( fppdis ) then
              ok = sssqd( Hstaddr, Dnbyte, Hqdid )
            else
              ok = sppqd( Hstaddr, Dnbyte, Hqdid )
              end if
            if ( ok )
     1        call sbxeqq( Ipflu, Irec )
            end if
          end if
        end if
      return
      end


      SUBROUTINE SANACQ( RCYTIM, DNOCY, ICYLEN, ICHLST, IPFLU, IREC )
c
c       ACQuire data from the fast DAS 250A A/D converter - in random
c          channel addressing mode .
c     DNOCY frames of data , each of ICYLEN samples , are acquired from the
c  DAS 250A A/D converter . The frame period is RCYTIM seconds . The channel
c  to channel sampling period within each frame is 20.67 microseconds . The
c  A/D converter is in random channel addressing mode .
c     The data is acquired onto Quasi Disk , is then transmitted up the
c  parallel communications port and is deposited in the random access file
c  with logical unit number IPFLU , starting at record IREC .
c
c Arguments
c  real Rcytim              # acquisition cycle period
c  integer *4 Dnocy,        # Number of data acquisition cycles
c  integer *2 Icylen,       # number of samples per acquisition cycle
c             Ichlst(Icylen),# A/D channel address list
c  integer *2 Ipflu,        # logical unit number of data file
c             Irec          # data file record number
c
c constant
c  Hqdid=0                  # base address of the quasi disk
c  Hstaddr=0                # quasi-disk start address
c variables
c  integer *4 Dnbyte        # Total number of packed data bytes
c             Dnopak        # function : calculate number of packed bytes
c logical *1 smpran,        # use the DAS 250A A/D in sequential mode
c            smpxeq,        # execute MicroProfessor queue
c            sqddma,        # initialize DMA card for filling quasi disk
c            sppqd          # parallel transmit block data from quasi disk
c             sssqd         # serial transmit block data from quasi disk
c             ok            # result of writing data transimission command
c  subroutine sambyt        # calculate number of packed bytes
c             swrtq         # send `write queue' directive
c             sxeqq         # send `execute queue' directive and receive data
c
      implicit none
      integer *2 Hqdid
      integer *4 Hstaddr
      parameter( Hqdid=0, Hstaddr=0 )
c
      real Rcytim
      integer *4 Dnocy, Dnbyte, Dnopak
      integer *2 Icylen, Ichlst(*), Ipflu, Irec
      logical *1 smpran, smpxeq, sqddma, sppqd, sssqd, ok
      include 'sbctrl.cmn'
c
      Dnbyte = Dnopak( Dnocy * Icylen )
      call sbwrtq
      if ( smpran( Rcytim, Dnocy, Icylen, Ichlst ) ) then
        if ( sqddma( Hstaddr, Dnbyte, Hqdid ) ) then
          if ( smpxeq() ) then
            if ( fppdis ) then
              ok = sssqd( Hstaddr, Dnbyte, Hqdid )
            else
              ok = sppqd( Hstaddr, Dnbyte, Hqdid )
              end if
            if ( ok )
     1        call sbxeqq( Ipflu, Irec )
            end if
          end if
        end if
      return
      end


      SUBROUTINE SIMACQ( DNOCY, ICYLEN, IPFLU, IREC )
c
c       ACQuire data from the fast SIMultaneous multichannel A/D converter .
c  The channels used must be selected at the A/D converter front panel . The
c  data acquisition rate is determined by the internal or an external square
c  wave generator .
c     DNOCY frames of data , each of ICYLEN samples , are acquired from the
c  DAS 250A A/D converter .
c     The data is acquired onto Quasi Disk , is then transmitted up the
c  parallel communications port and is deposited in the random access file
c  with logical unit number IPFLU , starting at record IREC .
c
c Arguments
c  integer *4 Dnocy,        # Number of data acquisition cycles
c  integer *2 Icylen,       # number of samples per acquisition cycle
c  integer *2 Ipflu,        # logical unit number of data file
c             Irec          # data file record number
c constant
c  Hqdid=0                  # base address of the quasi disk
c  Hstaddr=0                # quasi-disk start address
c variables
c  integer *4 Dnbyte,       # Total number of packed data bytes
c             Dnopak        # function : calculate number of packed bytes
c logical *1 simfad,        # start the A/D & the DMA card
c            sqddma,        # initialize DMA card for filling quasi disk
c            sppqd          # parallel transmit block data from quasi disk
c             sssqd         # serial transmit block data from quasi disk
c             ok            # result of writing data transimission command
c  subroutine sambyt        # calculate number of packed bytes
c             swrtq         # send `write queue' directive
c             sxeqq         # send `execute queue' directive and receive data
c
      implicit none
      integer *2 Hqdid
      integer *4 Hstaddr
      parameter( Hqdid=0, Hstaddr=0 )
c
      integer *4 Dnocy, Dnbyte, Dnopak
      integer *2 Icylen, Ipflu, Irec
      logical *1 simfad, sqddma, sppqd, sssqd, ok
      include 'sbctrl.cmn'
c
      Dnbyte = Dnopak( Dnocy * Icylen )
      call sbwrtq
      if ( sqddma( Hstaddr, Dnbyte, Hqdid ) ) then
        if ( simfad() ) then
          if ( fppdis ) then
            ok = sssqd( Hstaddr, Dnbyte, Hqdid )
          else
            ok = sppqd( Hstaddr, Dnbyte, Hqdid )
            end if
          if ( ok )
     1      call sbxeqq( Ipflu, Irec )
          end if
        end if
      return
      end


      SUBROUTINE SBUNPK( UNPACK )
c
c       Set or reset the unpack data flag .
c Arguments
c  logical *1 unpack    # any packed data which is received will be unpacked
c
      implicit none
      logical *1 unpack
      include 'sbctrl.cmn'
c
      funpak = unpack
      return
      end


      SUBROUTINE SPPDIS( PPDIS )
c
c       Disable or enable use of the parallel line for data transfers to
c  this machine . Note that the parallel can still be used, but commands
c  which use the parallel line cannot be written onto the command queue .
c Arguments
c  logical *1 ppdis     # writing parallel line using commands is disabled
c
      implicit none
      logical *1 ppdis
      include 'sbctrl.cmn'
c
      fppdis = ppdis
      return
      end


      INTEGER *4 FUNCTION DNOPAK( DNSAMP )
c
c       Calculates the number of packed data bytes DNBYTE from the number of
c  data samples acquired .
c
c Arguments
c  integer *4 Dnsamp,   # Total number of data samples
c
      implicit none
      integer *4 Dnsamp
c
      Dnopak = mod( Dnsamp, 2 ) + Dnsamp * 3 / 2
      return
      end


      SUBROUTINE SBCLOD( CFILNAM )
c
c       Down loads an command queue from ASCII file FILNAM to the SBC
c  through the auxilliary serial line . The ASCII data file consists of
c  a series of hexadecimal coded commands . Each of these commands is :
c    1) a header line  - containing at least one colon ':' - and anything else
c    2) one or more lines of hexadecimal digit pairs . Characters which are
c       not hexadecimal digits ( except : which marks the header of the next
c       command , and so will cause an error ) are separators .
c  The command queue must be complete . If an EXIT (00) command is not the last
c  command in the file, then one is automatically appended to terminate the
c  queue .
c
c variables
c  character cfilnam*(*),   # name of data file
c            casc*Hlinlen   # characters read from file
c  logical *1 fdum,         # dummy result from Emchk
c             emchk,        # function : checks consistency of globals
c             existf        #          : file exists
c             wrtcmd,       #          : writes command to SBC
c  integer *2 n,            # cumulative length ( bytes ) of a command
c             inc,          # no of bytes decoded from a line
c             m,            # length of parameter list of a command +2
c             icmd(Hcmdlen),# command array
c             xdecode       # function : decodes hex ASCII string
c subroutine serror,        # handles error messages
c            sexit,         # exit command
c            sbwrtq         # send 'write queue' directive
c
      implicit none
      include 'sbctrl.kon'
c
      character cfilnam*(*), casc*Hlinlen
      logical *1 fdum, emchk, existf, wrtcmd
      integer *2 n, inc, m, icmd(Hcmdlen), xdecode
      include 'sbctrl.cmn'
c
      call sbwrtq
      if ( .not. existf( cfilnam ) ) then                                   1b
         call serror( 'SBCLOD', Habort + Hnofile )
      else if ( Emchk( fdum, 'SBCLOD  ' ) ) then                          1m
        open( Hlodlu, FILE=Cfilnam, ACCESS='sequential',
     1        FORM='formatted', STATUS='old', ERR=6 )
c       start loop                                                          2b
c         repeat                                                            3b
1           read( Hlodlu, 2, END=4, ERR=7 ) Casc
2           format( a )
            if ( 0 .eq. index( Casc, ':' ) ) goto 1
c           until ( the line has a : in it )                                3e
8         n = 0
c         repeat                                                            3b
3           read( Hlodlu, 2, END=6, ERR=7 ) Casc
            if ( 0 .ne. index( Casc, ':' ) ) then                           4b
              call serror( 'SBCLOD', Hwarn + Hucmd )
              goto 8
            else                                                            4m
              inc = xdecode( Casc, icmd( n+1 ), Hlinlen / 2 )
              if ( 0 .eq. icmd(1) )
     1          goto 4
              n = n + inc
              m = icmd(2) + 2
              end if                                                        4e
            if ( m .gt. n .or. n .lt. 2 ) goto 3
c           until ( m .le. n .and. n .ge. 2 )                                       3e
          fdum = wrtcmd( m, icmd )
          goto 1
c         end loop                                                          2e
        end if                                                              1e
4     call sexit
      close( Hlodlu )
      return
c
6     call serror( 'SBCLOD', Habort + Hueof )
      goto 4
c
7     call serror( 'SBCLOD', Habort + Hnofile )
      goto 4
      end


      BLOCK DATA SBCBLK
c
c           common block initialization
c
      implicit none
      include 'sbctrl.cmn'
      data fdirfg, finit, fsavq, funpak, fppopn, fspopn /
     1  .true. , 5*.false. /
      data csmgfil / 'c:\genacq\bin\sbcmsg.gsm' /
      end


      SUBROUTINE SBCMSG( CFILE )
c
c       Set the SBC message file name to CFILE
c Argument
c   character Cfile*(*)        # name of SBC message file
c                                 ( including MS-DOS path name )
      implicit none
      character Cfile*(*)
      include 'sbctrl.cmn'
c
      Csmgfil = Cfile
      return
      end


      SUBROUTINE SBINIT( DEBUG, CSAVFIL )
c
c       Opens the serial line and initializes global variables .
c
c Arguments
c  logical *1 debug         # .true. if in debug mode
c  character Csavfil*(*)    # name of save command queue file
c
c Variables
c  integer *2 io_write,     # function ; write bytes to port
c  logical *1 io_serial,       # function : opens serial port
c  subroutine serror        # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 debug, existf
      character Csavfil*(*)
      integer *2 io_write
      logical *1 io_serial
      include 'sbctrl.cmn'
c
c               open serial port
c
      if ( .not. fspopn .and. .not. debug ) then                            1b
        if ( .not. io_serial( Hserlu, Hsport, Hbaud ) )
     1    call serror( 'SBINIT', Hfatal + Hssfd )
        end if                                                              1e
c
c               open save command queue file if required
c
      if ( fsavq )
     1  close( Hsavlu )
      fsavq = .false.
      if ( Csavfil .ne. ' ' ) then                                          1b
        if ( .not. Existf( Csavfil ) ) then                                 2b
          open( Hsavlu, FILE=Csavfil, ACCESS='sequential',
     1          FORM='formatted', STATUS='unknown', ERR=2 )
          fsavq = .true.
        else                                                                2m
          call serror( 'SBINIT', Hwarn + Hnofile )
          end if                                                            2e
        end if                                                              1e
c
c               send initial ctrl-Q prompt
c
1     fdebug = debug
      if ( .not. ( finit .or. fdebug ) ) then                               1b
        if ( io_write( Hserlu, Hctrlq, 1 ) .lt. 0 )
     1    call serror( 'SBINIT', Hfatal + Hssfd )
        call sctrlq
        end if                                                              1e
c
c               initialize global flags
c
      fdirfg = .true.
      imode = Hexit
      finit = .true.
      ftvinz = .false.
      iunpen = 0
      funpak = .false.
      return
c
2     call serror( 'SBINIT', Hwarn + Hnofile )
      goto 1
      end


      SUBROUTINE SBSTOP
c
c       Subroutine to issue a 'stop' directive to the SBC . The SBC program
c  is stopped . SBC must then be restarted from the SBC console .
c
c subroutine serror,            # handles error messages
c            sendir,            # send directive to SBC
c            sexit              # exit command
c
      implicit none
      include 'sbctrl.kon'
c
      include 'sbctrl.cmn'
c
      if ( finit ) then
        if ( .not. fdirfg )
     1    call sexit
        if ( fdebug ) then
          call serror( 'SBSTOP', Hdebug )
        else
          imode = Hexit
          call sendir( 'SBSTOP  ', imode )
          end if
      else
        call serror( 'SBSTOP', Habort + Hinit )
        end if
      finit = .false.
      return
      end


      SUBROUTINE SBWRTQ
c
c        Subroutine  to issue a 'write command queue' directive to the SBC .
c  The 'write command queue' directive must be issued before any command can
c  be written onto the queue .
c
c subroutine serror             # handles error messages
c            sexit              # exit command
c            sendir             # send directive
c
      implicit none
      include 'sbctrl.kon'
      include 'sbctrl.cmn'
c
      if ( finit ) then
        if ( .not. fdirfg )
     1    call SEXIT
        imode = Hwrtq
        imode = ( imode + imode ) / 2
        if ( .not. fdebug ) then
          call sendir( 'SBWRTQ', imode )
        else
          call serror( 'SBWRTQ', Hdebug )
          end if
        fdirfg = .false.
      else
        call serror( 'SBWRTQ', Habort + Hinit )
        end if
      return
      end


      SUBROUTINE SBXEQQ( OPFLU, JREC )
c
c       Subroutine to isue an 'execute command queue' directive to the SBC ,
c  monitor queue execution , and enable receiving of block data from the
c  serial and parallel lines .
c
c Arguments
c  integer *2 opflu,    # output file logical unit number
c             jrec,     # output file record number / type
c
c Variables
c  integer *2 icmdid    # command identifier
c             mcmdid(Hnocmds)   # complete list of command identifiers
c             io_write, # system function : see 'C write'
c  logical *1 ein2      # function ; integer membership of an array string
c
c subroutine sbkdat,            # get block data from serial or parallel port
c            serror,            # handles error messages
c            sivbyt,            # receives byte from SBC
c            sendir,            # sends directive to SBC
c            sexit              # terminates writing command queue
c            sctrlq             # wait for ctrl-q from SBC
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 icmdid, mcmdid(Hnocmds), io_write, opflu, jrec
      logical *1 ein2
      include 'sbctrl.cmn'
      data mcmdid / Hsbcdma, Hbmdma, Hqddma, Hrdsbc, Hsssbc,
     1  Hppsbc, Hbmsbc, Hqdsbc, Hstsbc, Hrdbm, Hssbm, Hppbm, Hsbcbm,
     2  Hstbm, Hrdqd, Hssqd, Hppqd, Hsbcqd, Hstqd, Hmpxeq, Hmpseq,
     3  Hmpran, Hmpsw, Hmpdel, Hmpjmp, Hmpwat, Hmpdum, Hsbcada,
     4  Hsbcadb, Hsimfad, Hrdtvs, Hsstvs, Hpptvs, Hsbctvs, Hsttvs,
     5  Hdvtvs, Hbnksw, Hsbc6dm, Hbm6dm, Hqd6dm, Hwait, Hskip, Hdelay,
     6  Hdead, Hdummy, Hexit /
c
      if ( finit ) then                                                     1b
        if ( .not. fdirfg )
     1    call Sexit
        fdirfg = .true.
        imode = Hxeqq
        if ( fdebug ) then                                                  2b
          call Serror( 'SBXEQQ', Hdebug + HXEQdiserr )
        else                                                                2m
          call sendir( 'SBXEQQ', imode )
c
c       the name of every command executed by the SBC is sent back along
c       the serial line so that this routine knows what to do
c
c         start of loop                                                     3b
1           fdirfg = .true.
            call sivbyt( icmdid )
            if ( io_write( Hserlu, Hctrlq, 1 ) .lt. 0 )
     1        call serror( 'SBXEQQ', Hfatal + Hssfd )
            if ( icmdid .eq. Hexit ) goto 2
            if ( ein2( icmdid, mcmdid, Hnocmds - 1 ) ) then                 4b
c
c       data coming back through serial line
c
              if ( icmdid .eq. Hsssbc .or. icmdid .eq. Hssbm .or.
     1            icmdid .eq. Hssqd  .or. icmdid .eq. Hsstvs ) then         5b
                call Sbkdat( Hserlu, Hserlu, jrec, opflu )
                iunpen = 0
c
c       data coming back through parallel port
c
              else if ( icmdid .eq. Hppsbc .or. icmdid .eq. Hppbm .or.
     1            icmdid .eq. Hppqd  .or. icmdid .eq. Hpptvs ) then         5m
                call Sbkdat( Hplllu, Hserlu, jrec, opflu )
                iunpen = 0
c
c       data acquired : set the unpacking permission
c
              else if ( icmdid .eq. Hsbcada .or. icmdid .eq. Hsbcadb )
     1            then
                iunpen = 0
              else if ( icmdid .eq. Hmpseq .or. icmdid .eq. Hmpran )
     1            then
                iunpen = 1
              else if ( icmdid .eq. Hsimfad ) then
                iunpen = 2
c
c       dead command executed
c
              else if ( icmdid .eq. Hdead ) then                            5m
                call Serror( 'SBXEQQ', Hnote + HXdead )
                end if                                                      5e
            else                                                            4m
c
c       unknown command executed
c
              call serror( 'SBXEQQ', Hwarn + Hcmdiderr )
              end if                                                        4e
            goto 1
c           end of loop                                                     3e
2         call sctrlq
          end if                                                            2e
      else                                                                  1m
        call serror( 'SBXEQQ', Habort + Hinit )
        end if                                                              1e
      fdirfg = .true.
      return
      end


      SUBROUTINE SBRDQ( OPFLU, STATUS )
c
c       Subroutine to issue a 'read command queue' directive to the SBC , and
c  then read the queue onto device or file with logical unit number IPFLU .
c
c Arguments
c integer *2 opflu,     # command queue file logical unit number
c            status     # 0 = print hex form of command
c                       # 1 = & name of command
c                       # 2 = & functional description
c                       # 3 = & list of parameters
c                       # 4 = & other rules & comments
c Variables
c  logical *1 emgopn,           # opens a named message file
c             emgwrt,           # reads a message from file
c             rdcmd,            # function ; reads a command from SBC
c             getmsg            # .true. if messages from file are required
c  integer *2 icmd(Hcmdlen),    # command array
c             i,                # looping index
c             i1,               # command identifier
c             n,                # number of bytes in command
c subroutine sexit              # exit command
c            sendir             # sends directive to SBC
c            emgcls             # close message file
c            serror             # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 emgopn, emgwrt, rdcmd, getmsg
      integer *2 opflu, status, icmd(Hcmdlen), i, i1, n
      include 'sbctrl.cmn'
c
      if ( finit ) then                                                     1b
        if ( .not. fdirfg )
     1    call sexit
        fdirfg = .true.
        imode = Hrdq
        if ( fdebug ) then                                                  2b
          call Serror( 'SBRDQ', Hdebug )
        else                                                                2m
          getmsg = status .gt. 0 .and. status .lt. 5
          call sendir( 'SBRDQ', imode )
          if ( getmsg ) then                                                3b
            getmsg = emgopn( Hsmglu, Csmgfil )
            if ( .not. getmsg )
     1        call serror( 'SBRDQ', Hwarn + Hmgopn )
            end if                                                          3e
c         repeat                                                            3b
1           if ( .not. Rdcmd( n, icmd ) ) then                              4b
              call Serror( 'SBRDQ', Hwarn + Hrecerr )
            else if ( getmsg ) then                                         4m
              i1 = iand( icmd(1), 255 )
              do 2 i = i1, ( status -1 ) *1000 + i1, 1000                   5b
                if ( .not. emgwrt( opflu, i ) )
     1            stop
2               continue                                                    5e
              end if                                                        4e
            call prtcmd( opflu, n, icmd )
            if ( icmd(1) .ne. Hexit ) goto  1
c         until( icmd(1) .eq. Hexit )                                       3e
          if ( getmsg )
     1      call emgcls
          call sctrlq
          end if                                                            2e
      else                                                                  1m
        call serror( 'SBRDQ', Habort + Hinit )
        end if                                                              1e
      return
      end


      SUBROUTINE SBSWAP
c
c       Subroutine to hand control of the SBC over to the SBC console .
c
c subroutine serror,            # handles error messages
c            sexit,             # exit command
c            sendir             # sends directive to SBC
c
      implicit none
      include 'sbctrl.kon'
      include 'sbctrl.cmn'
c
      if ( finit ) then
        if ( .not. fdirfg ) call sexit
        imode = Hswap
        if ( fdebug ) then
          call serror( 'SBSWAP', Hdebug )
        else
          call sendir( 'SBSWAP', imode )
          end if
      else
        call serror( 'SBSWAP', Habort + Hinit )
        end if
      return
      end


      LOGICAL *1 FUNCTION SBCDMA( ISTADR, INOBYT )
c
c       Set up the S100 DMA card to receive data and place it in the SBC data
c array .
c
c Arguments
c  integer istadr,      # start address
c          inobyt       # number of bytes to transfer
c variable
c  logical *1 sbcwrt    # function ; writes SBC transfer command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 istadr, inobyt
c
      sbcdma = .false.
      call serror( 'SBCDMA', Habort + Hnone )
      return
      end


      LOGICAL *1 FUNCTION SQDDMA( DSTADR, DNOBYT, IQDID )
c
c       Set up the S100 DMA card to receive data and place it on the quasi
c  disk .
c
c Arguments
c  integer *2 iqdid     # base address of the quasi disk
c  integer *4 Dstadr,   # quasi disk start address
c             Dnobyt    # number of bytes to be transferred
c variable
c  logical *1 sqdwrt    # function ; writes quasi disk command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 iqdid
      integer *4 Dstadr, Dnobyt
      logical *1 sqdwrt
c
      sqddma = sqdwrt( Hqddma, Dstadr, Dnobyt, Iqdid, 'SQDDMA' )
      return
      end



      LOGICAL *1 FUNCTION SRDSBC( ISTADR, INOBYT )
c
c       Data transfer from SBC data array to console
c
c Arguments
c  integer *2 istadr,   # start address
c             inobyt    # number of bytes to be transferred
c variable
c  logical *1 sbcwrt    # function ; writes SBC transfer command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 istadr, inobyt
      logical *1 sbcwrt
c
      srdsbc = sbcwrt( Hrdsbc, istadr, inobyt, 'SRDSBC' )
      return
      end


      LOGICAL *1 FUNCTION SSSSBC( ISTADR, INOBYT )
c
c       Data transfer from SBC data array to binary serial line .
c
c Arguments
c  integer *2 istadr,   # start address
c             inobyt    # number of bytes to be transferred
c variable
c  logical *1 sbcwrt    # function ; writes SBC transfer command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 istadr, inobyt
      logical *1 sbcwrt
c
      ssssbc = sbcwrt( Hsssbc, istadr, inobyt, 'SSSSBC' )
      return
      end



      LOGICAL *1 FUNCTION SPPSBC( ISTADR, INOBYT )
c
c       Data transfer from SBC data array to parallel port
c
c Arguments
c  integer *2 istadr,   # start address
c             inobyt    # number of bytes to be transferred
c variable
c  logical *1 sbcwrt    # function ; writes SBC transfer command
c             io_paral  # function ; opens parallel port
c  subroutine serror    # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 istadr, inobyt
      logical *1 sbcwrt, io_paral
      include 'sbctrl.cmn'
c
      if ( fppdis ) then
        call serror( 'SPPSBC', Habort + Hppdis )
      else
        if ( .not. fppopn .and. .not. fdebug ) then
          if ( .not. io_paral( Hplllu, Hpport ) )
     1      call serror( 'SPPSBC', Hfatal + Hppfd )
          fppopn = .true.
          end if
        sppsbc = sbcwrt( Hppsbc, istadr, inobyt, 'SPPSBC' )
        end if
      return
      end


      LOGICAL *1 FUNCTION SBMSBC
c
c       Transfers the entire contents of the SBC data array onto static RAM .
c
c subroutine serror     # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      call serror( 'SBMSBC', Habort + Hnone )
      sbmsbc = .false.
      return
      end



      LOGICAL *1 FUNCTION SQDSBC( DSTADR, DNOBYT, IQDID )
c
c       Transfers contents of entire SBC data array onto quasi disk .
c
c Arguments
c  integer *4 dstadr,       # quasi disk start address
c             dnobyt        # number of bytes to be transferred - ignored
c  integer iqdid            # quasi disk base address
c
c Variable
c  logical *1 sqdwrt        # function ; writes quasi disk command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 Iqdid
      integer *4 dstadr, dnobyt
      logical *1 sqdwrt
c
      sqdsbc = sqdwrt( Hqdsbc, Dstadr, Dnobyt, Iqdid, 'SQDSBC' )
      return
      end



      LOGICAL *1 FUNCTION SSTSBC
c
c       Reset the data array pointer for the SBC data array .
c
c Variable
c logical *1 szero      # function : send command with no parameters
c
      implicit none
      include 'sbctrl.kon'
      logical *1 szero
c
      Sstsbc = Szero( Hstsbc, 'SSTSBC' )
      return
      end



      LOGICAL *1 FUNCTION SBCWRT( ICMDID, ISTADR, INOBYT, CLABEL )
c
c      Writes a ' SBC data array ' command to the SBC .
c
c Arguments
c  integer *2 icmdid,       # command identifier
c             istadr,       # SBC start address
c             inobyt,       # number of bytes to be transferred
c  character Clabel*(*)     # name of the calling routine
c
c variable
c  integer *2 icmd(7)       # command array
c  logical *1 emchk,        # function ; consistency check on globals
c             wrtcmd        # function ; writes a command to the SBC
c subroutine serror         # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 icmdid, istadr, inobyt, icmd(7), jstadr, jnobyt
      integer *4 dstadr, dnobyt
      character Clabel*(*)
      logical *1 emchk, wrtcmd
      include 'sbctrl.cmn'
      equivalence ( jstadr, dstadr ), ( jnobyt, dnobyt )
c
      if ( fdebug ) call serror( Clabel, Hdebug )
      if ( Emchk( sbcwrt, Clabel ) ) then
        dstadr = 0
        dnobyt = 0
        jstadr = istadr
        jnobyt = inobyt
        if ( 0 .lt. dnobyt .and. dnobyt + dstadr .lt. 65536 ) then
          icmd(1) = icmdid
          icmd(2) = 5
          call sieq16( istadr, icmd(3) )
          call sieq16( inobyt, icmd(5) )
          icmd(7) = 0
          sbcwrt = wrtcmd( 7, icmd )
        else
          call serror( Clabel, Habort + Hparerr )
          sbcwrt = .false.
          end if
        end if
      return
      end


      LOGICAL *1 FUNCTION SRDBM
c
c       Transfer data from static RAM to console .
c
c Variable
c subroutine serror     # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      call serror( 'SRDBM', Habort + Hnone )
      srdbm = .false.
      return
      end


      LOGICAL *1 FUNCTION SSSBM
c
c       Block transfer of data from static RAM to binary serial line .
c
c Variables
c  subroutine serror        # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      call serror( 'SSSBM', Habort + Hnone )
      sssbm = .false.
      return
      end


      LOGICAL *1 FUNCTION SPPBM
c
c       Block transfer of data from static RAM to parallel port
c
c subroutine serror        # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      call serror( 'SPPBM', Habort + Hnone )
      sppbm = .false.
      return
      end


      LOGICAL *1 FUNCTION SSBCBM
c
c       Transfers data from static RAM to SBC data array .
c
c subroutine serror     # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      call serror( 'SSBCBM', Habort + Hnone )
      ssbcbm = .false.
      return
      end


      LOGICAL *1 FUNCTION SSTBM
c
c       Reset the data array pointers for the static RAM ( bulk memory )
c
c subroutine serror     # send command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      call serror( 'SSTBM', Habort + Hnone )
      Sstbm = .false.
      return
      end


      LOGICAL *1 FUNCTION SRDQD( DSTADR, DNOBYT, IQDID )
c
c           Block data transfer from quasi disk to the binary serial line .
c
c Arguments
c  integer *4 Dstadr,       # quasi disk start address
c             Dnobyt        # number of bytes to be transferred
c  integer *2 iqdid         # quasi-disk base address
c Variable
c  logical *1 sqdwrt        # function : writes quasi-disk command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *4 Dstadr, Dnobyt
      integer *2 iqdid
      logical *1 sqdwrt
c
      srdqd = sqdwrt( Hrdqd, Dstadr, Dnobyt, Iqdid, 'SRDQD' )
      return
      end


      LOGICAL *1 FUNCTION SSSQD( DSTADR, DNOBYT, IQDID )
c
c       Block data transfer from quasi disk to the binary serial line .
c
c Arguments
c  integer *4 Dstadr,       # quasi disk start address
c             Dnobyt        # number of bytes to be transferred
c  integer *2 iqdid         # quasi-disk base address
c Variable
c  logical *1 sqdwrt        # function : writes quasi-disk command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *4 Dstadr, Dnobyt
      integer *2 iqdid
      logical *1 sqdwrt
c
      sssqd = sqdwrt( Hssqd, Dstadr, Dnobyt, Iqdid, 'SSSQD' )
      return
      end


      LOGICAL *1 FUNCTION SPPQD( DSTADR, DNOBYT, IQDID )
c
c       Block data transfer from quasi disk to parallel port
c
c Arguments
c  integer *4 Dstadr,   # quasi disk start address
c             Dnobyt    # number of bytes to be transferred
c  integer *2 iqdid     # quasi-disk base address
c variable
c  logical *1 sqdwrt    # function ; writes quasi disk command
c             io_paral  # function ; opens parallel port
c subroutine serror     # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *4 Dstadr, Dnobyt
      integer *2 iqdid
      logical *1 sqdwrt, io_paral
      include 'sbctrl.cmn'
c
      if ( fppdis ) then
        call serror( 'SPPSBC', Habort + Hppdis )
      else
        if ( .not. fppopn. and. .not. fdebug ) then
          if ( .not. io_paral( Hplllu, Hpport ) )
     1      call serror( 'SPPQD', Hfatal + Hppfd )
          fppopn = .true.
          end if
        sppqd = sqdwrt( Hppqd, Dstadr, Dnobyt, Iqdid, 'SPPQD' )
        end if
      return
      end


      LOGICAL *1 FUNCTION SSBCQD( DSTADR, DNOBYT, IQDID )
c
c       Data transfer from quasi disk to SBC data array
c
c Arguments
c  integer *4 Dstadr,       # quasi disk start address
c             Dnobyt        # number of bytes to be transferred
c  integer *2 iqdid         # quasi-disk base address
c Variable
c  logical *1 sqdwrt        # function : writes quasi-disk command
c
      implicit none
      include 'sbctrl.kon'
c
      integer *4 Dstadr, Dnobyt
      integer *2 iqdid
      logical *1 sqdwrt
c
      ssbcqd = sqdwrt( Hsbcqd, Dstadr, Dnobyt, Iqdid, 'SSBCQD' )
      return
      end


      LOGICAL *1 FUNCTION SSTQD
c
c       Resets the quasi disk pointers
c
c Variables
c  logical *1 szero     # function ; writes command with no parameters
c
      implicit none
      include 'sbctrl.kon'
      logical *1 szero
c
      Sstqd = Szero( Hstqd, 'SSTQD' )
      return
      end


      LOGICAL *1 FUNCTION SQDWRT( ICMDID, DSTADR, DNOBYT, IQDID,
     1  CLABEL )
c
c       Writes a quasi disk command to the SBC
c
c Arguments
c  integer *2 icmdid        # command identifier
c  integer *4 dstadr,       # quasi disk start address
c             dnobyt        # number of bytes to be transferred
c  integer *2 iqdid,        # quasi-disk identifier ( base address )
c  character Clabel*(*)     # name of calling routine
c Variables
c  logical *1 emchk,        # function ; consistency checks on globals
c             wrtcmd        # function ; writes a command to the SBC
c             icmd(10)      # command array
c  subroutine serror        # handles error messages
c             dpeq24        # transfers 3 byte integer
c
c
      implicit none
      include 'sbctrl.kon'
c
      integer *4 dstadr, dnobyt
      integer *2 icmdid, iqdid, icmd(9)
      character Clabel*(*)
      logical *1 emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( Clabel, Hdebug )
      if ( Emchk( Sqdwrt, Clabel ) ) then
        if ( 0 .le. dstadr .and. dstadr .le. Hqdsize .and.
     1      0 .lt. dnobyt .and. dstadr + dnobyt .lt. Hqdsize + 1 ) then
          icmd(1) = icmdid
          icmd(2) = 8
          call dpeq24( dstadr, icmd(3) )
          call dpeq24( dnobyt, icmd(6) )
          icmd(9) = iqdid
          sqdwrt = Wrtcmd( 10, icmd )
        else
          call serror( Clabel, Habort + Hparerr )
          end if
        end if
      return
      end


      LOGICAL *1 FUNCTION SBCADA( RCYTIM, INOCY, ICYLEN, ICHLST )
c
c       Sends a 'Use the DAS 1128 slow A/D converter - Z80 A 4MHz' command to
c  the SBC .
c
c  N.B. the value of RCYTIM may be adjusted slightly by this routine to
c       the value actually used
c Arguments
c  real  Rcytim,            # Cycle time in seconds
c  integer *2 Inocy,        # Number of acquisition cycles
c             Icylen,       # Number of samples in each acquisition cycle
c             Ichlst(Icylen)# List of A/D converter channel addresses
c Variables
c  logical *1 sbcsad,       # use DAS 1128 A/D
c  integer *4 Dclks         # Cycle time in system clock cycles
c  subroutine serror        # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 sbcsad
      integer *4 Dclks
      real  Rcytim
      integer *2 Inocy, Icylen, Ichlst(*)
c
      Dclks = Rcytim * Hphi4 + 0.5
      if ( int( 130 + 97 * Icylen ) .le. Dclks ) then
        sbcada = sbcsad( Hsbcada, 'SBCADA', Dclks, Inocy, Icylen,
     1    Ichlst )
        rcytim = real( Dclks ) / Hphi4
      else
        call serror( 'SBCADA', Habort + Hparerr )
        sbcada = .false.
        end if
      return
      end


      LOGICAL *1 FUNCTION SBCADB( RCYTIM, INOCY, ICYLEN, ICHLST )
c
c       Sends a 'Use DAS 1128 slow A/D converter - Z80 B 6MHz' command to the
c  SBC .
c  N.B. the value of RCYTIM may be adjusted slightly by this routine to
c       the value actually used
c
c Arguments
c  real  Rcytim,            # Cycle time in seconds
c  integer *2 Inocy,        # Number of acquisition cycles
c             Icylen,       # Number of samples in each acquisition cycle
c             Ichlst(Icylen)# List of A/D converter channel addresses
c
c Variables
c  logical *1 sbcsad,       # use DAS 1128 A/D
c  integer *4 Dclks         # Cycle time in system clock cycles
c  subroutine serror        # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 sbcsad
      integer *4 Dclks
      real  Rcytim
      integer *2 Inocy, Icylen, Ichlst(*)
c
      Dclks = Rcytim * Hphi6 + 0.5
      if ( int( 134 + 136 * Icylen ) .le. Dclks ) then
        sbcadb = sbcsad( Hsbcadb, 'SBCADB', Dclks, Inocy, Icylen,
     1    Ichlst )
        Rcytim = real( Dclks ) / Hphi6
      else
        call serror( 'SBCADB', Habort + Hparerr )
        sbcadb = .false.
        end if
      return
      end


      LOGICAL *1 FUNCTION SBCSAD( ICMDID, CLABEL, DCLKS, NOCY, ICYLEN,
     1  ICHLST )
c
c       Sends a 'Use the DAS 1128 slow A/D converter' command to the SBC .
c Arguments
c  integer *2 Icmdid,       # command identifier
c  character clabel*(*)     # name of calling routine
c  integer *4 Dclks         # Cycle time in system clock pulses
c  integer *2 nocy,         # number of acquisition cycles
c             icylen,       # number of samples in each acquisition cycle
c             ichlst(icylen)# list of A/D converter channel addresses
c
c Variables
c  integer *2 icmd(Hcmdlen),# command array
c             i             # channel select index
c  logical *1 efact,        # function : factorizes
c             emchk,        # function : checks consistency of globals
c             wrtcmd        # function : writes a command to the SBC
c  subroutine serror,       # handles error messages
c             sieq16,       # put 2 byte integer into command array
c
      implicit none
      include 'sbctrl.kon'
c
      character clabel*(*)
      integer *4 Dclks
      integer *2 Icmdid, nocy, icylen, ichlst(*), icmd(Hcmdlen), i
      logical *1 efact, emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( Clabel, Hdebug )
      if ( Emchk( sbcsad, Clabel ) ) then                                   1b
        sbcsad = Dclks .gt. 1 .and. Dclks .le. 256 **3 .and.
     1    Nocy .gt. 0 .and. Nocy * Icylen .le. 20480 .and.
     2    Icylen .gt. 0 .and. Icylen .le. Hcmdlen -14
        if ( sbcsad ) then                                                  2b
          icmd(1) = Icmdid
          icmd(2) = 12 + Icylen
          if ( Dclks .gt. 1048576 ) then                                    3b
            Dclks = Dclks / 256
            Icmd(3) = Hctcx256
          else if ( Dclks .gt. 131072 ) then                                3m
            Dclks = Dclks / 16
            Icmd(3) = Hctcx16
          else                                                              3m
            Dclks = Dclks / 2
            Icmd(3) = Hctcx2
            end if                                                          3e
          if ( Efact( Dclks, icmd(4), 2 ) ) then                            3b
            Dclks = int( icmd(4) ) * icmd(5) * 2
            if ( Icmd(3) .eq. Hctcx16 ) then                                4b
              Dclks = Dclks * 8
            else if ( Icmd(3) .eq. Hctcx256 ) then                          4m
              Dclks = Dclks * 128
              end if                                                        4e
            call sieq16( nocy, icmd(10) )
            icmd(12) = Icylen
            icmd(13) = 0
            icmd(14) = Icylen
            do 1 i = 1,Icylen                                               4b
              icmd( 14 + i ) = ichlst(i)
              if ( ichlst(i) .lt. 0 .or. ichlst(i) .gt. 15 ) then           5b
                sbcsad = .false.
                call serror( Clabel, Habort + Hparerr )
                end if                                                      5e
1             continue                                                      4e
            if ( sbcsad )
     1        sbcsad = wrtcmd( 14 + icylen, icmd )
          else                                                              3m
            sbcsad = .false.
            call serror( Clabel, Habort + Hfctzerr )
            end if                                                          3e
        else                                                                2m
          call serror( Clabel, Habort + Hparerr )
          end if                                                            2e
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION SIMFAD
c
c       Sends a 'Use simultaneous multichannel fast A/D' command to the SBC
c
c Variable
c  logical *1 szero # function : send command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      Simfad = Szero( Hsimfad, 'SIMFAD' )
      return
      end


      LOGICAL *1 FUNCTION SMPXEQ
c
c       Sends execute MicroProfessor command queue to SBC .
c
c Variable
c  logical *1 szero # function ; send command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      Smpxeq = Szero( Hmpxeq, 'SMPXEQ' )
      return
      end


      LOGICAL *1 FUNCTION SMPSEQ( RCYTIM, RSAMPD, DNOCY, ICYLEN )
c
c       Sends a 'use DAS 250A fast A/D in sequential channel addressing mode'
c  command to the SBC .
c  N.B. the value of RCYTIM, RSAMPD and DNOCY, may be adjusted slightly by
c       this routine to the value actually used
c
c Arguments
c  real Rcytim,         # Acquisition cycle period ( in seconds )
c       Rsampd,         # Sampling period ( in seconds )
c  integer *4 Dnocy,    # Number of acquisition cycles
c  integer *2 icylen,   # acquisition cycle length - no. of samples
c constant
c  Hcymax=16777215      # maximum number of countable acquisition cycles
c Variables
c  integer *4 Dnopak    # function : calculate number of packed bytes
c  integer *2 icmd(11)  # command array
c             Jintus,   # function : converts real to unsigned integer
c  logical *1 emchk,    # function ; checks consistency of globals
c             efact,    # function ; factorizes
c             wrtcmd,   # function ; writes a command to SBC
c  real Rfltus          # function : unsigned integer to real convert
c  subroutine serror    # handles error messages
c             sieq16,   # puts a 2 byte integer into the command array
c
      implicit none
      integer *4 Hcymax
      parameter( Hcymax=16777215 )
      include 'sbctrl.kon'
c
      real Rcytim, Rsampd, Rfltus
      integer *4 Dnocy, Dnopak
      integer *2 icylen, icmd(11), Jintus
      logical *1 emchk, efact, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( 'SMPSEQ', Hdebug )
      if ( Emchk( Smpseq, 'SMPSEQ' ) ) then                                 1b
        if ( 65536.0 / Hmpphi .le. Rsampd .or. Rsampd .le. 7.0 / Hmpphi
     1      .or. Rcytim .lt. Icylen * Rsampd + 1.0 / Hmpphi .or.
     2      65536.0 / Hmpphi .le. Rcytim .or. Icylen .le. 0 .or.
     3      Icylen .gt. 16 .or. Hcymax .lt. Dnocy .or. 0 .gt. Dnocy .or.
     4      Dnopak( Dnocy * Icylen ) .gt. Hqdsize ) then                                                            2b
          smpseq = .false.
          call serror( 'SMPSEQ', Habort + Hparerr )
        else                                                                2m
          icmd(1) = Hmpseq
          icmd(2) = 9
c
c           set timing values & no. of channels
c
          call sieq16( Jintus( Rcytim * Hmpphi + 0.5 ), Icmd(3) )
          call sieq16( Jintus( Rsampd * Hmpphi + 0.5 ), Icmd(5) )
          Icmd(10) = Icylen
c
c           if no. of cycles = 0 , go forever
c
          if ( 0 .eq. Dnocy ) then                                          3b
            icmd(11) = 0
          else                                                              3m
            icmd(11) = 1
            end if                                                          3e
          if ( Efact( Dnocy, Icmd(7), 3 ) ) then                            3b
            smpseq = Wrtcmd( 11, icmd )
            rcytim = ( real(Icmd(3)) + real(Icmd(4)) * 256.0 ) / Hmpphi
            rsampd = ( real(Icmd(5)) + real(Icmd(6)) * 256.0 ) / Hmpphi
            Dnocy = int( Icmd(7) ) * Icmd(8) * Icmd(9)
          else                                                              3m
            smpseq = .false.
            call serror( 'SMPSEQ', Habort + Hfctzerr )
            end if                                                          3e
          end if                                                            2e
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION SMPRAN( RCYTIM, DNOCY, ICYLEN, ICHLST )
c
c         Sends a 'Use DAS 250A fast A/D converter in random channel
c  addressing mode' command to the SBC .
c  N.B. the value of RCYTIM may be adjusted slightly by this routine to
c       the value actually used
c
c Argument
c  real Rcytim              # acquisition cycle period
c  integer *4 Dnocy,        # number of acquistion cycles
c          icylen,          # no. of samples per acquisition cycle
c          ichlst(icylen),  # A/D channel address list
c constant
c  Hcymax=16777215      # maximum number of countable acquisition cycles
c Variables
c  integer *4 Dupnob        # function : calculate number of packed bytes
c  integer *2 icmd(Hcmdlen) # command array
c             i,            # looping index
c  logical *1 emchk,        # function ; checks globals for consistency
c             wrtcmd        # function ; writes command to SBC
c  subroutine dpeq24        # 3 byte integer transfer
c             serror,       # handles error messages
c
      implicit none
      integer *4 Hcymax
      parameter( Hcymax=16777215 )
      include 'sbctrl.kon'
c
      real Rcytim
      integer *4 Dnocy, Dnopak
      integer *2 Icylen, ichlst(*), icmd(Hcmdlen), i
      logical *1 emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( 'SMPRAN', Hdebug )
      if ( Emchk( Smpran, 'SMPRAN' ) ) then                                1b
c
c           check the arguments
c
        if ( ( 107 + 37* Icylen ) / Hmpphi .gt. Rcytim .or.
     1      Rcytim .gt. 256.0 ** 2 / Hmpphi .or. Hcymax .lt. Dnocy .or.
     2      Dnocy .le. 0 .or. Icylen .le. 0 .or.
     3      Icylen .gt. Hcmdlen - 10 .or.
     4      Dnopak( Dnocy * Icylen ) .gt. Hqdsize ) then                                                            2b
          smpran = .false.
          call serror( 'SMPRAN', Habort + Hparerr )
        else                                                                2m
          do 1 i = 1, icylen                                                3b
            icmd(i+8) = ichlst(i)
            if ( ichlst(i) .lt. 0 .or. ichlst(i) .gt. 15 ) then             4b
              smpran = .false.
              call serror( 'SMPRAN', Habort + Hparerr )
              end if                                                        4e
1           continue                                                        3e
          end if                                                            2e
        if ( smpran ) then                                                  2b
          icmd(1) = Hmpran
          icmd(2) = 6 + icylen
c
c               set the clock frame rate - fast or slow
c
          if ( Rcytim .lt. 16.0 * 257.0 / Hmpphi ) then                     3b
            icmd(3) = 149
            icmd(4) = int( Rcytim * Hmpphi / 16.0 + 0.5 )
            Rcytim = icmd(4) / Hmpphi * 16.0
          else                                                              3m
            icmd(3) = 181
            icmd(4) = int( Rcytim * Hmpphi / 256.0 + 0.5 )
            Rcytim = real( icmd(4) ) / Hmpphi * 256.0
            end if                                                          3e
          call dpeq24( Dnocy, icmd(5) )
          icmd(8) = Icylen
          smpran = Wrtcmd( 8+Icylen, icmd )
          end if                                                            2e
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION SMPSW( ISW )
c
c       Change state of or pulse the specified device switch
c Argument
c  integer *2 isw       # device to be switched
c     0  no action
c     1  switch pipe flow valve on or off
c     2  switch traverse on or off
c     3  send an active low pulse to the Hewlett/Packard spectrum analyser .
c
c  integer *2 icmd(3),  # command array
c             msw(4)    # table of switch bytes
c  logical *1 Emchk,    # checks globals for consistency
c             wrtcmd    # writes command to SBC
c  subroutine serror,   # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 isw, icmd(3), msw(4)
      logical *1 Emchk, wrtcmd
      include 'sbctrl.cmn'
c
      data msw / 0, 2, 4, 9 /
c
      if ( fdebug )
     1  call serror( 'SMPSW', Hdebug )
      if ( Emchk( Smpsw, 'SMPSW' ) ) then
        if ( 0 .gt. isw .or. isw .ge. 4 ) then
          smpsw = .false.
          call serror( 'SMPSW', Habort + Hparerr )
        else
          icmd(1) = Hmpsw
          icmd(2) = 1
          icmd(3) = msw( isw + 1 )
          smpsw = wrtcmd( 3, icmd )
          end if
        end if
      return
      end


      LOGICAL *1 FUNCTION SMPDEL( TIME )
c
c       Send a 'delay MicroProfessor' command to SBC
c
c Argument
c  real time            # time delay in seconds
c Variables
c  integer *2 icmd(4)   # command array
c             jintus,   # function ; real =.gt. unsigned integer
c  logical *1 emchk,    # function ; checks consistency of globals
c             wrtcmd    # function ; sends a command to the SBC
c  subroutine serror,   # handles error messages
c             sieq16,   # puts a 2 byte integer into the command array
c
      implicit none
      include 'sbctrl.kon'
c
      real time
      integer *2 icmd(4), jintus
      logical *1 emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call Serror( 'SMPDEL', Hdebug )
      if ( Emchk( Smpdel, 'SMPDEL' ) ) then
        if ( 0.0005 .gt. time .or. time .gt. 65.53550 ) then
          smpdel = .false.
          call serror( 'SMPDEL', Habort + Hparerr )
        else
          icmd(1) = Hmpdel
          icmd(2) = 2
          call sieq16( Jintus( time * 998.9912 + 0.5 ), icmd(3) )
          smpdel = wrtcmd( 4, icmd )
          end if
        end if
      return
      end



      LOGICAL *1 FUNCTION SMPJMP( JADDR, NLOOP )
c
c       Sends a MicroProfessor jump command to the SBC
c
c Arguments
c  integer *2 jaddr,    # jump address in MicroProfessor command queue
c             nloop,    # number of loops or passes through SMPJMP
c Variable
c  integer *2 icmd(5)   # command array
c  logical *1 emchk,    # function ; checks consistency of globals
c             e1byte,   # function ; checks for single byte value
c             serror,   # handles error messages
c             wrtcmd    # function ; writes a command to the SBC
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 jaddr, nloop, icmd(5)
      logical *1 emchk, e1byte, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( 'SMPJMP', Hdebug )
      if ( Emchk( Smpjmp, 'SMPJMP' ) ) then
        if ( E1byte( Jaddr ) .and. E1byte( Nloop ) ) then
          icmd(1) = Hmpjmp
          icmd(2) = 3
          icmd(3) = jaddr
          icmd(4) = nloop
          icmd(5) = 0
          smpjmp = wrtcmd( 5, icmd )
        else
          smpjmp = .false.
          call serror( 'SMPJMP', Habort+ Hparerr )
          end if
        end if
      return
      end



      LOGICAL *1 FUNCTION SMPWAT
c
c       Sends a 'MicroProfessor wait for keyboard' command to the SBC
c
c variable
c  logical *1 szero     # function ; writes a command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      Smpwat = Szero( Hmpwat, 'SMPWAT' )
      return
      end



      LOGICAL *1 FUNCTION SMPDUM
c
c       Sends a 'MicroProfessor dummy'command to the SBC .
c
c Variable
c  logical *1 szero # function ; writes a command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      Smpdum = Szero( Hmpdum, 'SMPDUM' )
      return
      end


      LOGICAL *1 FUNCTION SRDTVS
c
c       Reads the potentiometer positions into the console
c
c variable
c  logical *1 srdpot    # function: writes 'read traverse pots' command
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 srdpot
c
      srdtvs = srdpot( Hrdtvs, 'SRDTVS' )
      return
      end


      LOGICAL *1 FUNCTION SSSTVS
c
c       Reads the potentiometer positions to the serial line
c
c variable
c  logical *1 srdpot    # function: writes 'read traverse pots' command
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 srdpot
c
      ssstvs = srdpot( Hsstvs, 'SSSTVS' )
      return
      end


      LOGICAL *1 FUNCTION SPPTVS
c
c       Reads the potentiometer positions to the parallel port
c
c Variable
c  logical *1 srdpot    # function: writes 'read traverse pots' command
c             io_paral  #         : open parallel port
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 srdpot, io_paral
      include 'sbctrl.cmn'
c
      if ( fppdis ) then
        call serror( 'SPPSBC', Habort + Hppdis )
      else
        if ( .not. fppopn .and. .not. fdebug ) then
          if ( .not. io_paral( Hplllu, Hpport) )
     1      call serror( 'SPPQD', Hfatal + Hppfd )
          fppopn = .true.
          end if
        spptvs = srdpot( Hpptvs, 'SPPTVS' )
        end if
      return
      end


      LOGICAL *1 FUNCTION SBCTVS
c
c       Reads the potentiometer positions into the SBC data array
c
c variable
c  logical *1 srdpot    # function: writes 'read traverse pots' command
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 srdpot
c
      sbctvs = srdpot( Hsbctvs, 'SBCTVS' )
      return
      end


      LOGICAL *1 FUNCTION SRDPOT( ICMDID, CLABEL )
c
c       Writes a command to read traverse potentiometers .
c
c Arguments
c  integer *2 icmdid,   # command identifier
c  character Clabel*(*) # name of calling routine
c Variables
c  integer *2 icmd(4)   # command array
c  logical *1 emchk,    # function ; checks consistency of globals
c             wrtcmd,   # function ; writes a command to SBC
c  subroutine serror    # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 icmd(4), icmdid
      character Clabel*(*)
      logical *1 emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( Clabel, Hdebug )
      if ( Emchk( srdpot, Clabel ) ) then                                   1b
        if ( ftvinz ) then                                                  2b
          icmd(1) = icmdid
          icmd(2) = 1
          if ( itvsid .eq. 1 ) then                                         3b
            icmd(3) = 16
          else if ( itvsid .eq. 5 ) then                                    3m
            icmd(3) = 31
          else                                                              3m
            icmd(3) = 0
            end if                                                          3e
          if ( icmd(3) .ne. 0 ) then                                        3b
            srdpot = wrtcmd( 3, icmd )
          else                                                              3m
            srdpot = .false.
            call serror( Clabel, Habort + Hparerr )
            end if                                                          3e
        else                                                                2m
          srdpot = .false.
          call serror( Clabel, Habort + Hinit )
          end if                                                            2e
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION SSTTVS( TVSID )
c
c       Initializes the specified traverse .
c  The values of the parameters are given below :-
c
c  Traverse     simple traverse                 multidirectional traverse
c  Itvsid       1                               5
c----------------------------------------------------------------------------
c  Dcmdid       ea                              ea
c  Dplen        2c                              2c
c  Derror       00                              00
c  Dused        10                              1f
c  Dbklsh       10                              1f
c  Dbkdir       00                              1a
c  Dchlst       00 00 00 00 0a 00 00 00         0b 0c 0d 0e 0f 00 00 00
c  Dptol        00 00 00 00 01 00 00 00         03 02 02 03 03 00 00 00
c  Dsmbnd       00 00 00 00 01 00 00 00         0c 08 0c 10 0c 00 00 00
c  Dmfbnd       00 00 00 00 01 00 00 00         0c 08 0c 10 0c 00 00 00
c  Dmpid        00 00 00 00 10 00 00 00         10 20 10 08 18 00 00 00
c
c Argument
c  integer *2 tvsid,     # traverse identifier
c variables
c  integer *2 icmd1(46), # command array : for simple traverse
c             icmd5(46)  # command array : for multidirectional traverse
c  logical *1 emchk,     # function ; checks consistency of globals
c             wrtcmd,    # function ; writes a command to SBC
c  subroutine serror     # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 tvsid, icmd1(46), icmd5(46)
      logical *1 emchk, wrtcmd
      save icmd1, icmd5
      include 'sbctrl.cmn'
c
      data icmd1 / z'ea', z'2c', z'00', z'10', z'10', z'00',
     1  4*z'00', z'0a', 7*z'00', z'01', 7*z'00', z'01', 7*z'00',
     2  z'01', 7*z'00', z'10', 3*z'00' /
      data icmd5 / z'ea', z'2c', z'00', z'1f', z'1f', z'1a',
     1     z'0b', z'0c', z'0d', z'0e', z'0f', 3*z'00',
     2     z'03', z'02', z'02', z'03', z'03', 3*z'00',
     3     z'0c', z'08', z'0c', z'10', z'0c', 3*z'00',
     4     z'0c', z'08', z'0c', z'10', z'0c', 3*z'00',
     5     z'10', z'20', z'10', z'08', z'18', 3*z'00' /
c
      if ( fdebug )
     1  call serror( 'SSTTVS', Hdebug )
      if ( Emchk( ssttvs, 'SSTTVS' ) ) then
        itvsid = tvsid
        if ( itvsid .eq. 1 ) then
          ssttvs = wrtcmd( 46, icmd1 )
        else if ( itvsid .eq. 5 ) then
          ssttvs = wrtcmd( 46, icmd5 )
        else
          ssttvs = .false.
          call serror( 'SSTTVS', Habort + Hparerr )
          end if
        end if
      ftvinz = ssttvs
      return
      end


      LOGICAL *1 FUNCTION SDVTVS( IPPOSN )
c
c       Drives the traverse previously specified in a call to SSTTVS to the
c  position defined by the potentiometer readings IPPOSN
c    The values of the parameters are given below :-
c
c Parameter Itvsid  Value(s)
c----------------------------
c  Dcmdid   1 & 5    aa
c  Dplen    1 & 5    27
c  Dhilim   1 & 5    00
c  Dlolim   1 & 5    00
c  Dfinis   1 & 5    00
c  Dovrun   1 & 5    00
c  Dmalfn   1 & 5    00
c  Derror   1 & 5    00
c  Dincr    1 & 5    00
c  Ddest      1      0000 0000 0000 0000 xxxx 0000 0000 0000
c  Ddest      5      xxxx xxxx xxxx xxxx xxxx 0000 0000 0000
c  Dposn    1 & 5    0000 0000 0000 0000 0000 0000 0000 0000
c
c For the simple traverse IPPOSN is an single integer - which is the required
c  potentiometer reading .
c For the multidirectional traverse IPPOSN is an array of 5 integers, with the
c  ith value being the required potentiometer reading for the ith DOF .
c The traverse is driven only in absolute position mode ( Dincr=0 ).
c
c argument
c  integer *2 ipposn(5),    # required potentiometer positions
c variable
c  integer *2 icmd(42)      # command array
c             i1,i2,        # indexing variables
c  logical emchk,           # function ; checks consistency of globals
c          wrtcmd,          # function ; writes a command to SBC
c  subroutine serror        # handles error messages
c             sieq16        # places a 2 byte integer in the command array
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 ipposn(5), icmd(42), i1, i2
      logical emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( 'SDVTVS', Hdebug )
      if ( Emchk( sdvtvs, 'SDVTVS' ) ) then                                 1b
        if ( ftvinz ) then                                                  2b
          icmd(1) = Hdvtvs
          icmd(2) = 39
          do 1 i1 = 3, 41
1           icmd(i1) = 0
          if ( itvsid .eq. 1 ) then                                         3b
            if ( -1 .le. ipposn(1) .and. ipposn(1) .lt. 4096 ) then         4b
              call sieq16( ipposn(1), icmd(18) )
              sdvtvs = wrtcmd( 41, icmd )
            else                                                            4m
              sdvtvs = .false.
              call serror( 'SDVTVS', Habort + Hparerr )
              end if                                                        4e
          else if ( itvsid .eq. 5 ) then                                    3m
            i2 = 10
            sdvtvs = .true.
            do 2 i1 = 1, 5                                                  4b
              sdvtvs = sdvtvs .and. -1 .le. ipposn(i1) .and.
     1          ipposn(i1) .lt. 4096
              call sieq16( ipposn(i1), icmd(i2) )
2             i2 = i2 + 2                                                   4e
            if ( sdvtvs ) then                                              4b
              sdvtvs = wrtcmd( 41, icmd )
            else                                                            4m
              call serror( 'SDVTVS', Habort + Hparerr )
              end if                                                        4e
          else                                                              3m
            sdvtvs = .false.
            call serror( 'SDVTVS', Habort + Hinit )
            end if                                                          3e
        else                                                                2m
          call serror( 'SDVTVS', Habort + Hinit )
          end if                                                            2e
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION SWAIT
c
c       SBC 'wait for keyboard entry'
c
c variable
c  logical *1 szero  # function : writes a command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      Swait = Szero( Hwait, 'SWAIT' )
      return
      end


      LOGICAL *1 FUNCTION SSKIP( JADDR, NLOOP )
c
c       Sends a SKIP command to the SBC
c
c Arguments
c  integer *2  jaddr,   # jump relative queue address
c              nloop,   # number of branched passes
c variables
c  integer *2 icmd(5)   # command array
c  logical Emchk,       # function ; checks consistency of globals
c          E1byte,      # function ; checks for single byte value
c          wrtcmd       # function ; writes a command to the SBC
c  subroutine serror,   # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2  jaddr, nloop, icmd(5)
      logical Emchk, E1byte, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call Serror( 'SSKIP', Hdebug )
      if ( Emchk( sskip, 'SSKIP' ) ) then
        if ( -128 .le. Jaddr .and. Jaddr .lt. 128 .and.
     1      E1byte( Nloop ) ) then
          icmd(1) = Hskip
          icmd(2) = 3
          icmd(3) = jaddr
          icmd(4) = nloop
          icmd(5) = 0
          sskip = wrtcmd( 5, icmd )
        else
          sskip = .false.
          call serror( 'SSKIP', Hwarn + Hparerr )
          end if
        end if
      return
      end


      LOGICAL *1 FUNCTION SDELAY( MINS, SECS, MILSEC )
c
c       Sends a DELAY command to the SBC .
c
c arguments
c  integer *2 mins,     # minutes +    #
c             secs,     # seconds +    #  delay time
c             milsec,   # milliseconds #
c variables
c  integer *2 icmd(8)   # command array
c  logical *1 Emchk,    # function ; checks consistency of globals
c             wrtcmd    # function ; writes a command to the SBC
c  subroutine serror    # handles error messages
c             sieq16,   # puts a 2 byte integer into the command array
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 mins, secs, milsec, icmd(8)
      logical *1 Emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call Serror( 'SDELAY', Hdebug )
      if ( Emchk( sdelay, 'SDELAY' ) ) then
        icmd(1) = Hdelay
        icmd(2) = 6
        call sieq16( mins, icmd(3) )
        call sieq16( secs, icmd(5) )
        call sieq16( milsec, icmd(7) )
        sdelay = wrtcmd( 8, icmd )
        end if
      return
      end


      LOGICAL *1 FUNCTION SDEAD
c
c       Dead command ; is normally substituted for unavailable commands .
c
c variables
c  logical *1 szero     # function ; writes a command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      sdead = szero( Hdead, 'SDEAD' )
      return
      end


      LOGICAL *1 FUNCTION SDUMMY
c
c       Dummy command - does nothing .
c
c variable
c  logical szero    # function ; writes a command with no parameters
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 szero
c
      Sdummy = Szero( Hdummy, 'SDUMMY' )
      return
      end


      LOGICAL *1 FUNCTION SEXIT
c
c       Sends an end of queue command to the SBC .
c
c variables
c  integer *2 j         # no. of tries counter
c  logical *1 emchk,    # function ; checks consistency of globals
c             senbyt    # function ; writes command to SBC
c  subroutine serror    # handles error messages
c             sctrlq    # waits for ctrl-q from SBC
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 j
      logical *1 emchk, senbyt
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call Serror( 'SEXIT', Hdebug )
      if ( Emchk( Sexit, 'SEXIT' ) ) then                                   1b
        fdirfg = .true.
        sexit = .true.
        if ( fsavq )
     1    write( Hsavlu, 1 )
1         format( ' Command :' / 10x, '00' )
        if ( .not. fdebug ) then                                            2b
          do 2 j = 1, Hmaxtries                                             3b
            if ( senbyt( Hexit ) ) then                                     4b
              call sctrlq
              return
              end if                                                        4e
            call serror( 'SEXIT', Hwarn + Htrnerr )
2           continue                                                        3e
          call serror( 'SEXIT', Hfatal + Htrnerr )
          end if                                                            2e
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION SZERO( ICMDID, CLABEL )
c
c       Sends a command with no parameters to the SBC .
c
c arguments
c  integer *2 icmdid        # command identifier
c  character Clabel*(*)     # name of calling routine
c variables
c  integer *2 icmd(2)       # command array
c  logical *1 emchk,        # function ; checks consistency of globals
c             wrtcmd        # function ; writes command to SBC
c subroutine serror         # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 icmdid, icmd(2)
      character Clabel*(*)
      logical *1 emchk, wrtcmd
      include 'sbctrl.cmn'
c
      if ( fdebug )
     1  call serror( Clabel, Hdebug )
      Icmd(1) = Icmdid
      Icmd(2) = 0
      if ( Emchk( Szero, Clabel ) )
     1  Szero = Wrtcmd( 2, Icmd )
      return
      end


      LOGICAL *1 FUNCTION EMCHK( COK, CLABEL )
c
c       Globals consistency check ; must be initialized and in write command
c  queue mode .
c
c argument
c  logical *1 Cok           # duplicate of result
c  character  Clabel*(*)    # name of calling routine
c variable
c  subroutine serror        # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 Cok
      character Clabel*(*)
      include 'sbctrl.cmn'
c
      Cok = imode .eq. Hwrtq .and. finit .and. .not. fdirfg
      Emchk = Cok
      if ( imode .ne. Hwrtq )
     1  call Serror( Clabel, Habort + Hmoderr )
      if ( fdirfg )
     1  call Serror( Clabel, Habort + HXdirv )
      if ( .not. finit )
     1  call serror( Clabel, Habort + Hinit )
      return
      end


      LOGICAL *1 FUNCTION RDCMD( N, ICMD )
c
c       Reads N bytes of command into the command array ICMD from the SBC .
c
c arguments
c  integer *2 n,        # number of bytes in command line
c             icmd(n)   # command array
c variables
c  integer *2 verify,   # = control Q if transmission is OK
c             io_read,  # system function ; see C 'read'
c             itries,   # number of attempts to read command line
c             j,        # looping index
c  subroutine sivebyt   # receives and echoes a byte from the SBC
c             serror    # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 n, icmd(*), verify, io_read, itries, j
      include 'sbctrl.cmn'
c
      if ( fdebug ) then                                                    1b
        rdcmd = .false.
        call Serror( 'RDCMD', Hdebug )
      else                                                                  1m
        do 2 itries = 1, Hmaxtries                                          2b
          rdcmd = .true.
          call sivbyt( icmd(1) )
          call sivbyt( icmd(2) )
          n = icmd(2) + 2
          if ( icmd(2) .gt. 0 ) then                                        3b
            do 1 j = 3, n                                                   4b
1             call sivbyt( icmd(j) )                                        4e
            end if                                                          3e
          verify = 0
          j = io_read( Hserlu, verify, 1 )
          if ( j .eq. 1 .and. verify .eq. Hack )
     1      return
          call serror( 'RDCMD', Hwarn + Hrecerr )
2         continue                                                          2e
        call serror( 'RDCMD', Hfatal + Hrecerr )
        rdcmd = .false.
        end if                                                              1e
      return
      end


      LOGICAL *1 FUNCTION WRTCMD( N, ICMD )
c
c       Writes the first N bytes of the array ICMD to the SBC serial port as
c a queue command .
c
c arguments
c  integer *2 n,        # number of bytes in command
c             icmd(n),  # command array
c variables
c  integer *2 verify    # ACK or NACK to verify line sent and received
c             itries,   # number of write command attempts
c             i0_write, # function ; writes a byte to a port
c             j         # command array index
c  logical *1 senbyt    # sends a byte to SBC and receives the echo
c  subroutine serror    # handles error messages
c             prtcmd    # writes command to ASCII file
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 n, icmd(*)
      integer *2 verify, itries, io_write, j
      logical *1 senbyt
      include 'sbctrl.cmn'
c
      wrtcmd = .false.
      if( fsavq )
     1  call prtcmd( Hsavlu, n, icmd )
      if ( fdebug ) then                                                    1b
        call serror( 'WRTCMD', Hdebug )
        call prtcmd( Hvdulu, n, icmd )
        wrtcmd = .true.
      else if ( n .gt. 0 ) then                                             1e
        do 1 itries = 1, Hmaxtries                                          2b
          wrtcmd = .true.
          do 2 j = 1, n                                                     3b
2           wrtcmd = wrtcmd .and. senbyt( icmd(j) )                         3e
          verify = Hnack
          if ( wrtcmd )
     1      verify = Hack
          j = io_write( Hserlu, verify, 1 )
          if ( j .eq. 1 .and. wrtcmd )
     1      return
          call serror( 'WRTCMD', Hwarn + Htrnerr )
1         continue                                                          2e
        call Serror( 'WRTCMD', Hfatal + Htrnerr )
        end if                                                              1e
      return
      end


      SUBROUTINE PRTCMD( OPFLU, N, ICMD )
c
c       Writes N bytes of command GCMD onto the formatted output file with
c  logical unit number OPFLU .
c
c Arguments
c  integer *2 opflu,    # logical unit number of output file
c             n,        # number of bytes in the command
c             icmd(n),  # command array
c             i         # byte index
c
      implicit none
      include 'sbctrl.kon'
      integer *2 opflu, n, icmd(*), i
c
      if ( n .gt. 0 ) then
        write( Opflu, 1 ) ( iand( icmd(i), 255 ), i = 1, n )
1       format( ' Command :' / ( 11x, z2, 1x, z2, 1x, z2, 1x, z2,
     1  1x, z2, 1x, z2, 1x, z2, 1x, z2, 1x, z2, 1x, z2, 1x, z2, 1x, z2,
     2  1x, z2, 1x, z2, 1x, z2, 1x, z2 ) )
        end if
      return
      end


      SUBROUTINE SENDIR( CLABEL, IDIRV )
c
c       Sends a directive to the SBC , after receiving a control-Q
c
c arguments
c  character Clabel*(*) # Name of calling routine
c  integer *2 idirv     # directive to be sent
c variables
c  integer *2 i         # attempts counter
c  logical *1 senbyt    # function ; sends & checks a byte to SBC
c  subroutine serror    # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      character Clabel*(*)
      integer *2 i, idirv
      logical *1 senbyt
      include 'sbctrl.cmn'
c
      do 1 i = 1, Hmaxtries
        if ( Senbyt( idirv ) )
     1    return
1       call Serror( Clabel, Hwarn + Htrnderr )
      call Serror( Clabel, Hfatal + Htrnderr )
      return
      end


      SUBROUTINE SCTRLQ
c
c       Waits for a control-Q character from the SBC
c
c variables
c  integer *2 i         # attempts counter
c             ictrl     # character read
c             io_read,  # function : reads from a port
c subroutine serror     # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 i, ictrlq, io_read
      include 'sbctrl.cmn'
c
      do 2 i = 1, Hmaxtries
1       if ( io_read( Hserlu, ictrlq, 1 ) .le. 0 ) then
          call serror( 'SCTRLQ', Hwarn + Hrecerr )
        else
          if ( ictrlq .ne. Hctrlq ) goto 1
          return
          end if
2       continue
      call serror( 'SCTRLQ', Hfatal + Hrecerr )
      return
      end


      LOGICAL *1 FUNCTION SENBYT( OUT )
c
c       Sends a byte to the SBC and checks that it is echoed correctly
c Arguments
c  integer *2  out      # word containing data byte sent
c Variables
c  integer *2  iout     # trucated data byte sent
c              iin,     # echoed data byte
c             io_read,  # read a byte stream from a port
c             io_write, # write a byte stream to a port
c  subroutine serror    # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 iin, iout, out, io_read, io_write
      include 'sbctrl.cmn'
c
      iin = 0
      iout = iand( out, 255 )
      if ( io_write( Hserlu, iout, 1 ) .ne. 1 .or.
     1    io_read( Hserlu, iin, 1 ) .ne. 1 )
     2  call serror( 'SENBYT', Hfatal + Htrnerr )
      senbyt = iout .eq. iin
      return
      end


      SUBROUTINE SIVBYT( IIN )
c
c       Receives and echoes  a byte from the SBC .
c Argument
c  logical *1 iin       # data byte
c variables
c  integer *2 io_read   # function : read single byte from a port
c             io_write  #            write a single byte to a port
c subroutine serror     # handles error messages
c
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 iin
      integer *2 io_read, io_write
      include 'sbctrl.cmn'
c
      if ( io_read( Hserlu, iin, 1 ) .ne. 1 .or.
     1     io_write( Hserlu, iin, 1 ) .ne. 1 )
     2  call serror( 'SIVEBYT', Hfatal + Hrecerr )
      return
      end


      SUBROUTINE SERROR( CSOURCE, MSGNO )
c
c       Subroutine to display error message
c Arguments
c  character Csource*(*)    # calling source identifier
c  integer *2 msgno,        # error message number
c
c  constants
c Hshift=1024   # bit shift to get error status
c
c  constants - error types
c Hmsg=0        # a regular message
c Hdebug=1024   # debug messages
c Hnote=2048    # some unusual but 'harmless' error
c Hwarn=3072    # might have dangerous side effects
c Habort=4096   # serious enough to abort execution of the routine
c Hfatal=5120   # fatal error - stops the program
c
c  constants - error codes
c Htrnderr=1    # Serial Line - Directive Transmission Error
c Hxdead=2      # Dead command Executed
c Hcmdiderr=3   # Unknown command executed
c HXEQdiserr=4  # Debug Mode : Queue execution disabled
c Hmoderr=5     # Wrong mode - must be writing command queue
c Hparerr=6     # Write command : invalid parameter
c Htrnerr=7     # Serial Transmission error
c Hrecerr=8     # Serial Receiving error
c Hnone=9       # Command not available
c Hfctzerr=10   # Can't factorize to bytes
c Hnoisy=11     # Noisy data Transmission
c Hppfd=12      # Can't open parallel port
c Hssfd=13      # Can't open serial line
c Hinit=14      # Not initialized
c Hueof=15      # Unexpected end of file
c Hucmd=16      # Ignore unfinished command
c HXdirv=17     # Need a directive now
c Hmgopn=18     # Can't open message file
c Hnofile=19    # Can't find file
c Hdaterr=20    # Can't write to data file
c Hppdis=21     # Parallel port is disabled
c
c  variables
c character Cstat(7)*7  # error status array
c integer *2 imsg,      # message number in "text"
c            istat,     # message status
c
c
      implicit none
      include 'sbctrl.kon'
c
      character Cstat(7)*7, Csource*(*)
      integer *2 imsg, msgno, istat
c
      data Cstat / 'Message', ' Debug ', '  Note ', 'Warning',
     1             ' Abort ', ' Fatal ', 'Unknown' /
c
      istat = min( abs( msgno ) / Hshift, 6 )
      write ( Hvdulu, 1 ) Cstat( istat+1 ), Csource
1     format( 2x, a, ' : Error in ', a )
      imsg = msgno - istat * Hshift
      if ( istat .ne. Hdebug / Hshift ) then
        if ( imsg .eq. Htrnderr ) then
          write( Hvdulu, 2 )
2         format( ' Serial Line - Directive Transmission Error' )
        else if ( imsg .eq. Hxdead ) then
          write( Hvdulu, 3 )
3         format( ' Dead command Executed' )
        else if ( imsg .eq. Hcmdiderr ) then
          write( Hvdulu, 4 )
4         format( ' Unknown command executed' )
        else if ( imsg .eq. HXEQdiserr ) then
          write( Hvdulu, 5 )
5         format( ' Debug Mode : Queue execution disabled' )
        else if ( imsg .eq. Hmoderr ) then
          write( Hvdulu, 6 )
6         format( ' Wrong mode - must be writing command queue' )
        else if ( imsg .eq. Hparerr ) then
          write( Hvdulu, 7 )
7         format( ' Write command : invalid parameter' )
        else if ( imsg .eq. Htrnerr ) then
          write( Hvdulu, 8 )
8         format( ' Serial Transmission error' )
        else if ( imsg .eq. Hrecerr ) then
          write( Hvdulu, 9 )
9         format( ' Serial Receiving error' )
        else if ( imsg .eq. Hnone ) then
          write( Hvdulu, 10 )
10        format( ' Command not available' )
        else if ( imsg .eq. Hfctzerr ) then
          write( Hvdulu, 11 )
11        format( ' Can''t factorize to bytes' )
        else if ( imsg .eq. Hppfd ) then
          write( Hvdulu, 12 )
12        format( ' Can''t open parallel port' )
        else if ( imsg .eq. Hssfd ) then
          write( Hvdulu, 13 )
13        format( ' Can''t open serial line' )
        else if ( imsg .eq. Hinit ) then
          write( Hvdulu, 14 )
14        format( ' Not initialized' )
        else if ( imsg .eq. Hueof ) then
          write( Hvdulu, 15 )
15        format( ' Unexpected end of file' )
        else if ( imsg .eq. Hucmd ) then
          write( Hvdulu, 16 )
16        format( ' Ignore unfinished command' )
        else if ( imsg .eq. HXdirv ) then
          write( Hvdulu, 17 )
17        format( ' Need a directive now' )
        else if ( imsg .eq. Hmgopn ) then
          write( Hvdulu, 18 )
18        format( ' Can''t open message file' )
        else if ( imsg .eq. Hnofile ) then
          write( Hvdulu, 19 )
19        format( ' Can''t find file' )
        else if ( imsg .eq. Hnoisy ) then
          write( Hvdulu, 20 )
20        format( ' Noisy data Transmission' )
        else if ( imsg .eq. Hdaterr ) then
          write( Hvdulu, 21 )
21        format( ' Can''t write to data file' )
        else if ( imsg .eq. Hppdis ) then
          write( Hvdulu, 24 )
24        format( ' Parallel port is disabled' )
        else
          write( Hvdulu, 22 )
22        format( ' Unknown Error' )
          end if
        end if
      if ( istat .eq. Hfatal / Hshift ) then
        write( Hvdulu, 23 )
23      format( ' Crash' )
        stop
        end if
      return
      end




      SUBROUTINE SBKDAT( KDATLU, KACKLU, JREC, OUTPUT )
c
c       Subroutine to receive and store data sent by block data transmission
c  routines for the SBC control library . The argument list is the same as for
c  QBKDAT . Data can be received either from a serial line or from the
c  parallel port . OUTPUT is the logical unit number of the output file, and
c  JREC is the first record to be written or a file type flag . The result
c  QBKDAT is the ( integer *4 ) number of data bytes received .
c
c Arguments
c  integer *2 kdatlu,      # logical unit of data receive port
c             kacklu,      # logical unit of acknowledge send port
c             jrec,        # record number of output data file
c             output(*)    # logical unit number of output data file or
c                          #  output data array
c       Values of JREC
c               >0  output file is random access binary
c               =0  output file is sequential access ASCII
c               =-1 data is put in the array OUTPUT
c               <-1 there is no output file or array
c
c       If the data goes into a file, the sign of OUTPUT has special meaning.
c       Values of OUTPUT : >0 verbose :- block no. every 100 blocks
c                        : <0 suppress display of block number
c       If the data goes into an array, display of block number is suppressed
c
c  logical *1 daflush          # function : flush data file buffer
c             daput            #          : put data into d.a. file
c  integer *2 i,               # output byte counter
c             irec,            # record number of output data file
c             opflu,           # logical unit number of output data file
c             ipnbyt(2)        # number of data bytes read
c  integer *4 dpnbyte,         # number of data bytes read - from DPBKDAT
c             dipbuf,          # number of data bytes read -> file
c             dpbkdat          # function - gets data & returns how much
c  subroutine serror           # handles error messages
c
      implicit none
      integer Hbuflen
      parameter( Hbuflen=512 )
      include 'sbctrl.kon'
c
      integer *2 kdatlu, kacklu, jrec, opflu, output(*), i, irec,
     1  ipnbyte(2)
      logical *1 daflush, daput
      integer *4 dpnbyte, dipbuf, dpbkdat
c
c           COMMON BLOCK DECLARATION
c
c  logical *1 gopbuf(Hbuflen),   # output buffer array
c             gipbuf(Hbuflen),   # input buffer array
c             fopen              # file is open
c  integer *2 iopptr,iipptr,   # output and input buffer pointers
c             lenrec,          # length in bytes of each file record
c             dlrec,           # length in words of each file record
c             iflu             # logical unit number of output file
c
      logical *1 gopbuf(Hbuflen), gipbuf(Hbuflen), fopen
      integer *2 iopptr, iipptr, lenrec, dlrec, iflu
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  iflu, fopen
      equivalence ( dipbuf, gipbuf(1) ), ( dpnbyte, ipnbyte(1) )
c
      opflu = abs( output(1) )
c
c           direct access binary file, first check that OPFLU is OK
c
      if ( jrec .gt. 0 ) then
        if ( opflu .ne. iflu .or. lenrec .lt. 4 )
     1    call serror( 'SBKDAT', Hfatal + Hdaterr )
        irec = jrec
        if ( .not. daput( jrec, dpnbyte, 4 ) )
     1    call serror( 'SBKDAT', Hfatal + Hdaterr )
        dpnbyte = dpbkdat( kdatlu, kacklu, jrec, output )
        if ( .not. daflush( jrec ) )
     1    call serror( 'SBKDAT', Hfatal + Hdaterr )
        read( opflu, REC=irec, ERR=2 ) ( gipbuf(i), i = 1, lenrec )
        dipbuf = dpnbyte
        write( opflu, REC=irec, ERR=2 ) ( gipbuf(i), i = 1, lenrec )
c
c           sequential access ASCII file
c
      else if ( jrec .eq. 0 ) then
        dpnbyte = dpbkdat( kdatlu, kacklu, jrec, output )
        write ( opflu, 1 ) dpnbyte
1       format( ' SBKDAT: ', i12, ' data bytes' )
c
c           data array
c
      else if ( jrec .eq. -1 ) then
        dpnbyte = dpbkdat( kdatlu, kacklu, jrec, output(3) )
        output(1) = ipnbyte(1)
        output(2) = ipnbyte(2)
      else
        dpnbyte = dpbkdat( kdatlu, kacklu, jrec, output )
        end if
      return
c
2     call serror( 'SBKDAT', Hfatal + Hdaterr )
      return
      end



      INTEGER *4 FUNCTION DPBKDAT( KDATLU, KACKLU, JREC, OUTPUT )
c
c       Routine to receive and store data sent by a block data transmit
c  routine in the SBC . Data can be received either from a serial line or from
c  the parallel port . OUTPUT is the logical unit number of the output file,
c  and JREC is the first record to be written or a file type flag . The result
c  DPBKDAT is the ( integer *4 ) number of data bytes received .
c
c Arguments
c  integer *2 kdatlu,      # logical unit of data receive port
c             kacklu,      # logical unit of acknowledge send port
c             jrec,        # record number of output data file
c             output(*)    # logical unit number of output data file or
c                          #  output data array
c       Values of JREC
c               >0  output file is random access binary
c               =0  output file is sequential access ASCII
c               =-1 data is put in the array OUTPUT
c               <-1 there is no output file or array
c
c       If the data goes into a file, the sign of OUTPUT has special meaning.
c       Values of OUTPUT : >0 verbose :- block no. every 100 blocks
c                        : <0 suppress display of block number
c       If the data goes into an array, display of block number is suppressed
c
c Variables
c  logical *1 datin(256)       # data input array  - before unpacking
c             datout(390)      # data output array - after unpacking
c             blk100           # block /100 to be indicated
c             daput            # function : writes to d.a. file
c             fout(2)          # logical unit of output file - from arg list
c             output(*)        # logical unit number of data file or -
c                              #  data array
c  integer *4 icount           # byte count for data array
c  integer *2 j,               # block counter
c             k,               # byte output counter
c             nblks,           # number of data blocks to receive
c             opflu,           # logical unit number of output data file
c             nbytes,          # number of data bytes in a block
c             insamp,          # number of samples to be unpacked
c             nspares,         # no. of bytes at least partly unused by unpacker
c             iodevn           # odd or even byte marker for unpacking
c             iout             # logical unit of output file - from arg list
c  subroutine supsim,          # unpack data from fast simultaneous A/D
c             sup250,          # unpack data from DAS250 A/D
c             scrcrd,          # read a data block with CRC
c             serror           # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      integer *4 icount
      integer *2 jrec, opflu, j, k, nblks, nbytes, insamp, nspares,
     1  iodevn, kdatlu, kacklu, iout
      logical *1 datin(256), datout(390), blk100, daput, output(*),
     1  fout(2)
      include 'sbctrl.cmn'
      equivalence ( fout(1), iout )
c
      jdatlu = kdatlu
      jacklu = kacklu
      if ( jrec .ge. 0 ) then
        fout(1) = output(1)
        fout(2) = output(2)
        opflu = abs( iout )
        blk100 = iout .gt. 0 .and. iout .ne. Hvdulu
      else
        iout = 0
        blk100 = .false.
        end if
c
c       No. of blocks block
c
      call scrcrd( nblks, nbytes )
      if ( nblks .gt. 0 .and. nbytes .eq. 2 ) then                         1b
        if ( iout .gt. 0 )
     1    write( Hvdulu, 1 ) nblks
1         format( ' DPBKDAT: ', i6, ' Blocks.' / )
        if ( jrec .eq. 0 )
     1    write( opflu, 2 )
2         format( / ' DPBKDAT: data' )
        nspares = 0
        iodevn = 0
        dpbkdat = 0
        icount = 1
c
c       loop for getting a data transmission block
c
        do 5 j = 1, nblks                                                  2b
          call scrcrd( datin( 1 + nspares ), nbytes )
          nbytes = nbytes + nspares
c
c       unpack the data if necessary
c
          if ( funpak .and. iunpen .eq. 1 ) then                           3b
            insamp = nbytes * 2 / 3
            nspares = min( 1, mod( nbytes + iodevn, 3 ) )
            call sup250( datin, datout, insamp, iodevn )
            datin(1) = datin(nbytes)
            nbytes = 2 * insamp
          else if ( funpak .and. iunpen .eq. 2 ) then                      3m
            insamp = nbytes * 2 / 3
            nspares = min( 1, mod( nbytes + iodevn, 3 ) )
            call supsim( datin, datout, insamp, iodevn )
            datin(1) = datin(nbytes)
            nbytes = 2 * insamp
          else                                                             3m
            do 6 k = 1, nbytes                                             4b
6             datout(k) = datin(k)                                         4e
            end if                                                         3e
c
c       put the data on file
c
          if ( nbytes .gt. 0 ) then                                        3b
            Dpbkdat = Dpbkdat + nbytes
            if (jrec .gt. 0 ) then                                         4b
              if ( .not. daput( jrec, datout, nbytes ) )
     1          call serror( 'DPBKDAT', Hfatal + Hdaterr )
            else if ( jrec .eq. 0 ) then                                   4m
              write( opflu, 3 ) ( datout(k), k = 1, nbytes )
3             format( 1x,z2,1x,z2,1x,z2,1x,z2, 1x,z2,1x,z2,1x,z2,1x,z2,
     1          1x,z2,1x,z2, 1x,z2,1x,z2, 1x,z2,1x,z2, 1x,z2,1x,z2 )
            else if ( jrec .eq. -1 ) then                                  4m
              do 8 k = 1, nbytes
                output(icount) = datout(k)
8               icount = icount + 1
              end if                                                       4e
            end if                                                         3e
          if ( blk100 ) then                                               3b
            if ( mod( j, 100 ) .eq. 0 )
     1        write( Hvdulu, 4 ) j
4             format( '+', i5 )
            end if                                                         3e
5         continue                                                         2e
      else                                                                 1m
        write( Hvdulu, 7 )
7       format( ' DPBKDAT : No data' )
        end if                                                             1e
      return
      end


      SUBROUTINE SCRCRD( DATA, NBYTES )
c
c       Reads one block of data from the file or device with file descriptor
c  JDATFD into the array DATA which is at least NBYTES long . The data block
c  is read as binary with the following format :
c   <nul,nul> <soh,soh> <nbytes> <nbytes of data> <crc>
c       N.B. The <crc> and <nbytes> are 16 bit quantities . NUL and SOH are
c  8 bit ASCCII characters.
c       Ack / Nack signals are sent down the serial line with file descriptor
c  JACKFD
c
c arguments
c  logical *1 data(nbytes) # data array
c  integer *2 nbytes,      # number of data bytes received
c
c variables
c  integer *2 crc,         # received crc
c             itries,      # receive attempts down counter
c             anack(2)     # acknowledge string
c             mxcrc        # function : calculate crc of a data string
c             io_read,     #          : read bytes from a port
c             io_write,    #          : write bytes to a port
c             jrdhdr       #          : read a header byte
c  logical *1 badcrc       # calculated crc does not match received crc
c  subroutine serror       # handles error messages
c
      implicit none
      include 'sbctrl.kon'
c
      logical *1 data(*), badcrc
      integer *2 nbytes, crc, itries, anack(2), mxcrc, io_read,
     1  io_write, jrdhdr
      include 'sbctrl.cmn'
      data anack / 256, 0 /
      save anack
c
      do 13 itries = 1, Hmaxtries                                          1b
c
c           ensure that a <nul>, <nul>, <soh>, <soh> is received before
c           proceeding
c
10      if ( jrdhdr() .ne. 0 ) goto 10
11      if ( jrdhdr() .ne. 0 ) goto 10
12      if ( jrdhdr() .eq. 0 ) goto 12
        if ( header .ne. 1 ) goto 10
        if ( jrdhdr() .eq. 0 ) goto 11
        if ( header .ne. 1 ) goto 10
c
c           read number of data bytes and data array
c
        if ( io_read( jdatlu, nbytes, 2 ) .eq. 2 ) then                    2b
          if ( io_read( jdatlu, data, nbytes ) .ne. nbytes )
     1      call serror( 'SCRCRD', Hwarn + Hrecerr )
        else                                                               2m
          call serror( 'SCRCRD', Hwarn + Hrecerr )
          end if                                                           2e
c
c           read & calculate checksum - then acknowledge
c
        if ( io_read( jdatlu, crc, 2 ) .eq. 2 ) then                       2b
          anack(2) = Hack
          badcrc = crc .ne. mxcrc( data, nbytes )
          if ( badcrc ) then                                               3b
            anack(2) = Hnack
            call serror( 'SCRCRD', Hwarn + Hnoisy )
            end if                                                         3e
          if ( io_write( jacklu, anack, 3 ) .ne. 3 )
     1      call serror( 'SCRCRD', Hwarn + Htrnerr )
          if ( .not. badcrc )
     1      return
        else                                                               2m
          call serror( 'SCRCRD', Hwarn + Hrecerr )
          end if                                                           2e
13      continue
      call serror( 'SCRCRD', Hfatal + Hnoisy )
      return
      end



      INTEGER *2 FUNCTION JRDHDR
c
c Auxilliary function to read 1 header byte of a data block
c
c Variable
c  integer *2 io_read, # function - read bytes from a port
c
      implicit none
      include 'sbctrl.kon'
c
      integer *2 io_read
      include 'sbctrl.cmn'
c
      header = 0
      if ( io_read( jdatlu, header, 1 ) .eq. 1 ) then
        jrdhdr = header
      else
        call serror( 'JRDHDR', Hwarn + Hrecerr )
        jrdhdr = -1
        end if
      return
      end


