c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

      PROGRAM CAL_1128
c
c       For calibration of the DAS1128 A/D converter. The procedure is
c  to drive channel 0 of the A/D converter from the voltage calibrator,
c  with a preset starting value, final value and increment.
c
c   Usage :-
c       CAL_1128 [data-file-name]
c
c data-file-name is the name of a formatted sequential access file into which
c the calibration data is placed .
c
c constants
c  CALIB_LU  = 10     # logical unit number of output calibration disk file
c  MAX_BINS  = 9      # maximum histogram size
c  MAX_DATA  = 16384  # maximum number of data samples collected
c  MAX_TRIES = 4      # maximum number of input attempts
c  TEDIOUS   = 100    # it gets awfully tedious if more voltage levels than this
c  N_CHANS   = 1      # number of A/D converter channels used
c  CHAN_LIST = 0      # A/D channel used
c  INTO_ARRAY=-1      # specifies that data from A/D goes into array
c  SHORT_FALSE =.FALSE.
c  SHORT_TRUE  =.TRUE.
c
c local variables
c  integer
c    cal_lu           # logical unit number of output calibration file
c    histogram(MAX_BINS)  # data histogram
c    i                # index for histogram array
c    n_bins           # number of distinct data values
c    no_samples       # number of data samples from A/D converter
c    no_volt          # number of voltage settings
c    iargc            # function : number of program arguments
c    Jresp            #          : read integer
c  integer *2
c    AD_data(MAX_DATA)# data samples from A/D converter
c    min_data         # minimum data value from A/D converter
c    short_no_samples # integer *2 of "no_samples"
c  logical
c    do_histogram     # calculate histogram
c    first_data       # first data sample
c  real
c    cycle_time       # time between data samples
c    first_volt       # first voltage value
c    step_volt        # voltage step
c    last_volt        # last voltage value
c    mean             # mean value of the data
c    req_volt         # current calibrator voltage
c    std_dev          # standard deviation of the data
c    milli_round      # function : round to 3 decimal places
c    Qresp            #          : read_real
c  character
c    cal_file*(64)    # calibration file name
c    Cdate*8          # date string
c    crosses*70       # array of Xs for printing histogram
c    Ctime            # time string
c    item*1           # selected menu item
c    Cyes*3           # function : converts logical to character
c  subroutine
c    time, date       # system : time & data
c    bell, newscr     # hpprev
c    sbinit, sbstop, sbunpk, sppdis, sloacq    # sbctrl
c    getarg           # get program argument
c    shisti           # histogram of A/D data
c    smeani           # mean & variance of A/D data
c
      include 'stdio.def'
      integer    CALIB_LU, MAX_BINS, MAX_DATA, MAX_TRIES, TEDIOUS
      integer *2 N_CHANS, CHAN_LIST, INTO_ARRAY
      logical *1 SHORT_FALSE, SHORT_TRUE
      parameter( CALIB_LU=10, MAX_BINS=9,  MAX_DATA=16384,
     1           MAX_TRIES=4, TEDIOUS=100,
     2           N_CHANS=1,   CHAN_LIST=0, INTO_ARRAY=-1,
     3           SHORT_FALSE=.FALSE., SHORT_TRUE=.TRUE. )
c
      integer cal_lu, histogram(MAX_BINS), i, n_bins, no_samples,
     1        no_volt, iargc, Jresp
      integer *2 AD_data(MAX_DATA), min_data, short_no_samples
      logical do_histogram, first_data
      real    cycle_time, first_volt, step_volt, last_volt, mean,
     1        req_volt, std_dev, milli_round, Qresp
      character cal_file*(64), Cdate*8, crosses*70, Ctime*11,
     1          item*1, Cyes*3
c
      data cal_lu, no_samples / 0, 1024 /
      data do_histogram, first_data / .TRUE., .TRUE. /
      data cycle_time, first_volt, step_volt, last_volt /
     1     0.001, 0.000, 0.100, 10.000 /
      data cal_file / ' ' /
c
      do 19 i = 1, 70
          crosses(i:i) = 'X'
19        continue
c
c           start the data acquisition system
c
      write( STD_OUT, 1 )
1     format( ' Start the data acquisition system ( i.e. SBC etc. )' )
      write( STD_OUT, 2 )
2     format( ' Press <return> to continue' )
      read( STD_IN, 3 )
3     format( )
      call sbinit( SHORT_FALSE, ' ' )
      call sbunpk( SHORT_TRUE )
      call sppdis( SHORT_FALSE )
c
c           open data file
c
      if ( iargc() .gt. 0 ) then                                          1b
          call getarg( 1, cal_file )
          end if                                                          1e
c     start of main loop                                                  1b
c
c           fix up the voltage levels -- round all to nearest millivolt
c
4         last_volt = milli_round( last_volt )
          first_volt = milli_round( first_volt )
          step_volt = sign( milli_round( step_volt ),
     1                        last_volt - first_volt )
          no_volt = int( ( last_volt - first_volt ) / step_volt ) + 1
c
c           print menu & get response
c
          call newscr()
          write( STD_OUT, 5 ) charnb(cal_file), cycle_time, no_samples,
     1        first_volt, last_volt, step_volt, Cyes( do_histogram )
5         format( ' Select' /
     1            '   E= Exit' /
     2            '   O= Output file    ', a /
     3            '   T= Time between samples ', f8.6, ' seconds' /
     4            '   N= Number of samples    ', i5 /
     5            '   F= First voltage value  ', f7.3 /
     6            '   L= Last voltage value   ', f7.3 /
     7            '   S= Step in voltage      ', f7.3 /
     8            '   H= Histogram of data -- ', a /
     9            '   C= Calibrate            ' )
          if ( no_volt .gt. TEDIOUS )
     1        write( STD_OUT, 6 ) no_volt, TEDIOUS
6             format( ' WARNING : ', i7, ' is more than ', i4,
     1                ' voltage steps    ' )
          read( STD_IN, 7, END=98, ERR=96 ) item
7         format( a )
          if ( 'a' .le. item .and. item .le. 'z' )
     1        item = char( ichar( item ) - ichar( 'a' ) + ichar( 'A' ) )
c
c           EXIT
c           ****
c
          if ( item .eq. 'E' ) then                                       2b
              if ( cal_lu .gt. 0 )
     1            close( UNIT=cal_lu )
              call sbstop
              stop
c
c           OUTPUT FILE
c           ***********
c
          else if ( item .eq. 'O' ) then                                  2m
              write( STD_OUT, 8 )
8             format( ' Name of output file ?  ' )
              read( STD_IN, 7, END=98, ERR=96 ) cal_file
c
c           TIME BETWEEN SAMPLES
c           ********************
c
          else if ( item .eq. 'T' ) then                                  2m
              write( STD_OUT, 9 )
9             format( ' Enter the time between samples  ' )
              cycle_time = Qresp( STD_IN, 57.0e-6, 4.1943, MAX_TRIES )
c
c           NUMBER OF SAMPLES
c           *****************
c
          else if ( item .eq. 'N' ) then                                  2m
              write( STD_OUT, 10 )
10            format( ' Enter the number of samples  ' )
              no_samples = Jresp( STD_IN, 1, MAX_DATA-2, MAX_TRIES )
c
c           FIRST VOLTAGE VALUE
c           *******************
c
          else if ( item .eq. 'F' ) then                                  2m
              write( STD_OUT, 11 )
11            format( ' Enter the first voltage value  ' )
              first_volt = Qresp( STD_IN, -0.100, 11.000, MAX_TRIES )
c
c           LAST VOLTAGE VALUE
c           ******************
c
          else if ( item .eq. 'L' ) then                                  2m
              write( STD_OUT, 12 )
12            format( ' Enter the last voltage value  ' )
              last_volt = Qresp( STD_IN, -0.100, 11.000, MAX_TRIES )
c
c           STEP IN VOLTAGE
c           ***************
          else if ( item .eq. 'S' ) then                                  2m
              write( STD_OUT, 13 )
13            format( ' Enter the voltage step (ignore sign)  ' )
              step_volt = Qresp( STD_IN, 0.001, 11.000, MAX_TRIES )
c
c           DO HISTOGRAM
c           ************
c
          else if ( item .eq. 'H' ) then                                  2m
              do_histogram = .not. do_histogram
c
c           PERFORM CALIBRATION
c           *******************
c
          else if ( item .eq. 'C' ) then                                  2m
c
c           open the data file
c
              if ( cal_lu .ne. 0 .and. cal_lu .ne. STD_OUT ) then         3b
                  close( UNIT=cal_lu )
                  cal_lu = 0
                  end if                                                  3e
              if ( cal_file .ne. ' ' ) then                               3b
                  cal_lu = CALIB_LU
                  open( UNIT=cal_lu, FILE=cal_file, STATUS='unknown',
     1                  ACCESS='append', ERR=94 )
              else                                                        3m
                  cal_lu = STD_OUT
                  end if                                                  3e
              first_data = .TRUE.
C        WRITE(6,*) ' CAL_LU = ', CAL_LU
c
c           get actual A/D parameters by calling the A/D for 2 samples
c           and write a parameter summary to the output file
c
              call date( Cdate )
              call time( Ctime )
              short_no_samples = 2
              call sloacq( cycle_time, short_no_samples, N_CHANS,
     1                     CHAN_LIST, AD_data, INTO_ARRAY )
              if ( cal_lu .ne. 0 .and. cal_lu .ne. STD_OUT )
     1            write( cal_lu, 20 ) Cdate, Ctime, cycle_time,
     2                 no_samples, first_volt, last_volt, step_volt
              write( STD_OUT, 20 ) Cdate, Ctime, cycle_time, no_samples,
     1                            first_volt, last_volt, step_volt
20            format( ' ; Date                  ', a /
     1                ' ; Time                  ', a /
     2                ' ; time between samples  ', f9.6 /
     3                ' ; number of samples     ', i5 /
     4                ' ; A/D converter channel    0' /
     5                ' ; voltage -- start      ', f7.3 /
     6                ' ;         -- finish     ', f7.3 /
     7                ' ;         -- increment  ', f7.3 )
              write( STD_OUT, 2 )
              read( STD_IN, 3 )
              req_volt = first_volt
c
c             start calibration loop
c
21            call newscr()
c
c         collect data
c
              short_no_samples = no_samples
              call sloacq( cycle_time, short_no_samples, N_CHANS,
     1                     CHAN_LIST, AD_data, INTO_ARRAY )
              write( STD_OUT, 29 ) 7
29            format( a1 )
c
c         calculate & print mean and standard deviation statistics
c
              call smeani( AD_data(3), 1, no_samples, mean, std_dev )
              std_dev = sqrt( std_dev )
              if ( cal_lu .ne. 0 .and. cal_lu .ne. STD_OUT ) then         3b
                  if ( first_data )
     1                write( cal_lu, 22 )
                  write( cal_lu, 23 ) req_volt, mean, std_dev,
     1                                mean / 400.0, std_dev / 400.0
                  end if                                                  3e
              write( STD_OUT, 22 )
              write( STD_OUT, 23 ) req_volt, mean, std_dev,
     1                             mean / 400.0, std_dev / 400.0
22            format( ' Calibrator  A/D-reading       A/D-"voltage"' /
     1                '  voltage    mean  std_dev     mean  std_dev' )
23            format( 2x, f6.3, 3x, f7.2, 1x, f7.2, 3x, f7.4, 1x, f7.4 )
c
c         calculate & print histogram + warning if required
c
              if ( do_histogram ) then                                    3b
                  call shisti( AD_data(3), 1, no_samples, 70,
     1                         histogram, min_data, n_bins, MAX_BINS )
                  if ( n_bins .gt. MAX_BINS ) then                        4b
                      if ( cal_lu .ne. 0 .and. cal_lu .ne. STD_OUT )
     1                    write( cal_lu, 24 )
                      write( STD_OUT, 24 )
24                    format( ' Data too noisy for histogram' )
                  else                                                    4m
C        WRITE(6,*) 'N_BINS, HISTOGRAM= ',
C     1              N_BINS, (HISTOGRAM(I),I=1,N_BINS)
                      do 26 i = 1, n_bins                                 5b
                          if ( cal_lu .ne. 0 .and. cal_lu .ne. STD_OUT )
     1                        write( cal_lu, 25 ) min_data - 1 + i,
     2                            crosses(1:histogram(i))
                          write( STD_OUT, 25 ) min_data - 1 + i,
     1                        crosses(1:histogram(i))
25                        format( ' ;@', i4, 1x, a )
26                        continue                                        5e
                      end if                                              4e
                  end if                                                  3e
c
c         print required voltage
c         wait for <return> or exit/back/repeat/continue
c
                first_data = .FALSE.
c               start of input loop                                       3b
27                  write( STD_OUT, 28 ) step_volt, step_volt
28                  format( ' Select' /
     2                      '  E= Exit   -- return to main menu' /
     3                      '  R= Repeat -- same voltage' /
     4                      '  B= Back   -- subtract ', f6.3, ' volts' /
     5                      '  C= Continue -- add ', f6.3, ' volts' /
     6                      '  <Enter> = Continue         ' )
                    read( STD_IN, 7, END=98, ERR=96 ) item
                    if ( 'a' .le. item .and. item .le. 'z' )
     1                  item = char( ichar( item ) -
     2                         ichar( 'a' ) + ichar( 'A' ) )
                    if ( item .eq. 'E' ) then                             4b
                        goto 4
                    else if ( item .eq. 'R' ) then                        4m
                        goto 21
                    else if ( item .eq. 'B' ) then                        4m
                        req_volt = milli_round( req_volt - step_volt )
                        goto 21
                    else if ( item .eq. 'C' .or. item .eq. ' ' ) then     4m
                        req_volt = milli_round( req_volt + step_volt )
                        if ( step_volt .gt. 0.0 ) then                    5b
                            if ( req_volt .gt. last_volt )
     1                          goto 4
                        else                                              5m
                            if ( req_volt .lt. last_volt )
     1                          goto 4
                            end if                                        5e
                        goto 21
                        end if                                            4e
                    goto 21
c                   end of input loop                                     3e
              end if                                                      2e
          goto 4
c         end of main loop                                                1e
c
c           ERROR TRAPS
c           ***********
c
94    write( STD_OUT, 95 )
95    format( ' Cannot open file ', a )
      write( STD_OUT, 2 )
      read( STD_IN, 3 )
      goto 4
c
96    write( STD_OUT, 97 )
97    format( ' Cannot read keyboard' )
      goto 4
c
98    call sbstop
      stop
      end



      SUBROUTINE SHISTI( AD_DATA, INCREMENT, N_POINTS, MAX_HEIGHT,
     1                   HISTOGRAM, MIN_DATA, N_BINS, MAX_BINS )
c
c  Construct a histogram of the integer data AD_DATA .
c
c    Arguments
c  integer *2 AD_data(*)  # INPUT  # data array
c  integer increment      # INPUT  # data array indexing increment
c          n_points       # INPUT  # number of data points used
c          max_height     # INPUT  # maximum allowed scaled height of histogram
c          histogram(*)   # OUTPUT # histogram data
c  integer *2 min_data    # OUTPUT # minimum data value
c  integer n_bins         # OUTPUT # number of distinct data values
c          max_bins       # INPUT  # maximum allowed number of histogram values
c
c local variables
c  integer *2
c    max_data   # maximum data value
c  integer
c    index      # data array index
c    i_bin      # histogram bin to be incremented
c    max_hist   # actual maximum unscaled height of histogram
c    n_last     # index of last available data array element
c
      include 'stdio.def'
      integer *2 min_data, max_data, AD_data(*)
      integer increment, n_points, max_height, histogram(*), n_bins,
     1        max_bins, index, i_bin, max_hist, n_last
c
      n_last = ( n_points - 1 ) * increment + 1
c
c       find the limits of the data and test for ability to continue
c
C        WRITE(6,9999) ( AD_DATA(INDEX), INDEX=1,N_LAST,INCREMENT)
C9999    FORMAT( 16I5 )
      min_data = ad_data(1)
      max_data = ad_data(1)
      do 1 index = 1 + increment, n_last, increment                       1b
          min_data = min( min_data, ad_data(index) )
          max_data = max( max_data, ad_data(index) )
1         continue                                                        1e
      n_bins = max_data - min_data + 1
      if ( n_bins .eq. 1 ) then                                           1b
          histogram(1) = min( max_height, n_points )
      else if ( n_bins .le. max_bins ) then                               1m
c
c       clear the histogram values
c
          do 2 index = 1, n_bins
              histogram(index) = 0
2             continue
c
c       calculate the unscaled histogram
c
          max_hist = 0
          do 3 index = 1, n_last, increment                               2b
              i_bin = ad_data(index) - min_data + 1
              histogram( i_bin ) = histogram( i_bin ) + 1
              max_hist = max( max_hist, histogram( i_bin ) )
3             continue                                                    2e
c
c       scale the histogram if the peak exceeds the permitted maximum
c
          if ( max_hist .gt. max_height ) then                            2b
              do 4 index = 1, n_bins
                  histogram(index) =
     1                ( histogram(index) * max_height ) / max_hist
4                 continue
              end if                                                      2e
          end if                                                          1e
      return
      end



      SUBROUTINE SMEANI( AD_DATA, INCREMENT, N_POINTS, MEAN, VARIANCE )
c
c       real MEAN of Integers
c   Calculates the mean and variance of an array of 2-byte integers .
c
c    Arguments
c  integer *2 AD_data(*)  # data array
c  integer increment,     # data array indexing increment
c          n_points       # number of data points used
c  real    Mean           # mean of data
c          Variance       # variance of data
c
c local variables
c  integer
c    n_last           # index of last available data array element
c    index            # data array index
c  double precision
c    sum              # sum for mean and variance
c    double_mean      # mean of data
c
      integer *2 AD_data(*)
      integer increment, n_points, n_last, index
      real    Mean, Variance
      double precision sum, double_mean
c
      n_last = ( n_points - 1 ) * increment + 1
C        WRITE(6,9999) ( AD_DATA(INDEX), INDEX=1,N_LAST,INCREMENT)
C9999    FORMAT( 16I5 )
      sum = 0.0d0
      do 1 index = 1, n_last, increment
          sum = sum + dble( AD_data(index) )
1         continue
      double_mean = sum / dble( n_points )
c
      sum = 0.0d0
      do 2 index = 1, n_last, increment
          sum = sum + ( dble( AD_data(index) ) - double_mean ) **2
2         continue
c
      mean = sngl( double_mean )
      variance = sngl( sum ) / real( n_points )
      return
      end



      CHARACTER *3 FUNCTION CYES( LYES )
c
c  Converts logical YES to character representation .
c
      include 'stdio.def'
      logical Lyes
      if ( Lyes ) then
          cyes = 'yes'
      else
          cyes = 'no'
          end if
      return
      end


      REAL FUNCTION MILLI_ROUND( X )
c
c Rounds X to 3 decimal places (i.e. voltages to the nearest millivolt) .
c
      include 'stdio.def'
      real x
      milli_round = sign( real( int( 1000.0 * abs( x ) + 0.5 ) ) /
     1                    1000.0, x )
      return
      end


      INTEGER *4 FUNCTION JRESP( IPLU, IMIN, IMAX, NOREQS )
c               Integer RESPonse
c
c       The integer function JRESP returns an integer value between the
c  specified valid limits. This value is entered via the file with logical
c  unit number IPLU ( including possibly the keyboard ) , and is  checked
c  by JRESP, i.e. MIN < JRESP < MAX . The input file logical unit number
c  IPLU must be in the range 0 <= IPLU <= 99 , and must have been previously
c  defined if it is not a default device ( vdu, keyboard etc ). The maximum
c  number of input requests allowed NOREQS must be positive.
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer Iplu    #  input file logical unit number
c         imax,   #  maximum valid response value
c         imin,   #  minimum valid response value
c         noreqs  #  maximum number of requests allowed
c
c local variables
c  integer
c    cntr,        #  counter for the number of input attempts
c    Iplu1        #  input file logical unit number
c    nchars       # number of input characters
c    jljstr       # function : left justify string
c character
c    Cstr*40      # input character string
c
      include 'stdio.def'
c
      integer iplu, imax, imin, noreqs, cntr, iplu1, nchars, jljstr
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. noreqs .le. 0 )
     1  write ( STD_OUT, 1 ) Iplu, Noreqs
1       format ( ' JRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = STD_IN
      do 20 cntr = 1, noreqs
c
c           read data
c
          nchars = 0
21        if ( nchars .le. 0 ) then
              read( Iplu1, 2, END=7 ) Cstr
2             format( a )
              nchars = jljstr( Cstr )
              goto 21
              end if
c
c           selected min or max
c
          if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
              jresp = imax
              return
          else if ( Cstr(1:3) .eq. 'min' .or.
     1              Cstr(1:3) .eq. 'MIN' ) then
              jresp = imin
              return
c
c           decode & check numeric value
c
          else
              read( Cstr, *, ERR=8 ) jresp
              if ( min( imin, imax ) .le. jresp .and.
     1             jresp .le. max( imin, imax ) )
     2            return
              write ( STD_OUT, 3 ) imin, imax
3             format( ' JRESP : ERROR ; Integer outside limits ',
     1            i11, ' & ', i11 )
4             if ( Iplu1 .ne. STD_IN ) stop
              write ( STD_OUT, 5 )
5             format( ' Try again ' )
              goto 20
c
8             write( STD_OUT, 9 )
9             format( ' JRESP : ERROR : Can''t decode that ' )
              goto 4
              end if
20        continue
c
c           give up
c
      write ( STD_OUT, 6 )
6     format( ' Get help' )
7     stop
      end



      INTEGER FUNCTION JLJSTR( CSTR )
c
c         Left justify the character string CSTR ; ie eliminate leading blanks
c and return the number of characters in it after trailing blanks have also
c been removed .
c
c    Argument
c character *(*) Cstr   # operand
c
c    Variables
c integer length,       # length of operand
c         i,            # count up index
c         j,            # count down index
c         n,            # justified result index
c
      include 'stdio.def'
      character *(*) Cstr
      integer length, i, j, n
c
      length = len( Cstr )
c
c       find first non blank char
c
      i = 1
1     if ( Cstr(i:i) .eq. ' ' .and. i .lt. length ) then
          i = i + 1
          goto 1
          end if
c
c       find last non blank char
c
      j = length
2     if ( Cstr(j:j) .eq. ' ' .and. j . gt. i ) then
          j = j - 1
          goto 2
          end if
      Jljstr = j - i + 1
      if ( Cstr(j:j) .eq. ' ' ) Jljstr = 0
c
c       left justify
c
      do 3 n = 1, jljstr
          Cstr(n:n) = Cstr(i:i)
          i = i + 1
3         continue
c
c       fill up with blanks
c
      do 4 n = jljstr + 1, j
          Cstr(n:n) = ' '
4         continue
      return
      end


      INTEGER FUNCTION IARGC()
c
c         Returns the number of aqrguments to the main program .
c
c    constants
c MAX_LEN=127         # Maximum argument list length
c
c    Variables
c character Cmd*Hlen  # program arguments
c integer i           # count up index
c logical fspace      # found an argument separator
c
      implicit none
      integer MAX_LEN
      parameter( MAX_LEN=127)
c
      character  Cmd*MAX_LEN
      integer i
      logical fspace
c
      call getcl( Cmd )
      iargc = 0
      fspace = .false.
      do 1 i = 1, MAX_LEN
        if ( cmd(i:i) .eq. ' ' ) then
          fspace = .true.
        else if ( fspace ) then
          iargc = iargc + 1
          fspace = .false.
          end if
1       continue
      return
      end



      REAL FUNCTION QRESP( IPLU, RMIN, RMAX, NOREQS )
c               Real RESPonse
c       The real funtion QRESP returns a real value between the specified
c valid limits . This value is entered via the file with logical unit number
c IPLU ( default value 0 is the keyboard ) , and is checked by QRESP
c ie RMIN <= QRESP <= RMAX . The input file logical unit number must be in the
c range 0 <= IPLU <= 99 , and must have been previously defined if it is not
c a default device ( keyboard etc ) . The maximum number of input requests
c allowed NOREQS must be positive .
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer Iplu    # input file logical unit number
c         Noreqs  # maximum number of requests allowed
c real  rMax,     # maximum valid response value
c       rMin      # minimum valid response value
c
c local variables
c  integer
c    Cntr         # counter for the number of input attempts
c    Iplu1        # input file logical unit number
c    nchars       # number of input characters
c    jljstr       # function : left justify string
c  character
c    Cstr*40      # input character string
c
      include 'stdio.def'
c
      integer iplu, noreqs, cntr, iplu1, nchars, jljstr
      real rMax, rMin
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. Noreqs .le. 0 )
     1  write( STD_OUT, 1 ) Iplu, Noreqs
1       format( ' QRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = STD_IN
      do 7 Cntr = 1, noreqs
c
c           read data
c
c         repeat
2             read( Iplu1, 3, END=9 ) Cstr
3             format( a )
              nchars = jljstr( Cstr )
              if ( nchars .le. 0 ) goto 2
c             until( nchars > 0 )
c
c           selected min or max
c
          if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
              qresp = rmax
              return
          else if ( Cstr(1:3) .eq. 'min' .or. Cstr(1:3) .eq. 'MIN' )
     1            then
              qresp = rmin
              return
c
c           decode & check numeric value
c
          else
              read ( Cstr, *, ERR=5 ) Qresp
              if ( amin1( rMin, rMax ) .le. Qresp .and.
     1                Qresp .le. amax1( rMin, rMax ) )
     2            return
              write( STD_OUT, 4 ) rmin,rmax
4             format(' QRESP : ERROR ; Real outside limits ',
     1            g12.5, ' & ', g12.5 )
5             if ( Iplu1 .ne. STD_IN ) stop
              write ( STD_OUT, 6 )
6             format( ' Try again ' )
              end if
7         continue
c
c           give up
c
      write ( STD_OUT, 8 )
8     format( ' Get help' )
9     stop
      end


      SUBROUTINE GETARG( IARG, CARG )
c
c         Returns the IARGth argument of the main program to the character
c   string CARG .
c
c Arguments
c  integer   iarg       # Argument to be selected
c  character Carg*(*)   # Argument string returned
c
c    constants
c MAX_LEN               # Maximum argument list length
c    Variables
c character Cmd*MAX_LEN # program arguments
c integer i,            # count up index
c         igot          # argument number found
c         jbegin        # where the IARGth argument starts
c logical fspace        # found an argument separator
c
      include 'stdio.def'
      integer MAX_LEN
      parameter( MAX_LEN=127 )
c
      character  Cmd*MAX_LEN, Carg*(*)
      integer i, igot, iarg, jbegin
      logical fspace
c
      call getcl( Cmd )
      igot = 0
      fspace = .false.
      do 1 i = 1, MAX_LEN
        if ( cmd(i:i) .eq. ' ' ) then
          fspace = .true.
          if ( igot .eq. iarg ) then
            Carg = Cmd(jbegin:i-1)
            return
            end if
        else if ( fspace ) then
          igot = igot + 1
          jbegin = i
          fspace = .false.
          end if
1       continue
      Carg = ' '
      return
      end

