c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM MSGCRT
c
c       ( MeSsaGe CReaTe )
c
c     The program MSGCRT takes an ASCII sequential access file , and generates
c a random access message file {program name}.gsm . The new message file can
c then be used by the logical function EMGWRT to display a numbered message .
c The logical function EMGOPN opens an explicitly named message file .
c
c      Form of invocation :     msgcrt [ infile [ outfile ] ]
c               infile  = input file
c               outfile = output file
c If the output file name has no .* extension , .gsm is assumed .
c If no output file is named , the .* extension on infile is replaced with
c .gsm , which then becomes the name of the output file .
c If infile has no .* extension , the default extension .msg is assumed .
c If neither infile nor outfile are given , the input file is  messag.msg
c and the output file is  messag.gsm .
c
c SEQUENTIAL ACCESS FILE - STRUCTURE
c      The first line of a message begins with the character # followed
c  immediately by a legal integer (2 byte) . A message may consist of any
c  number  of lines of text  .
c      N.B. The character # cannot occur within a message as the first
c          character in a line . <Blank> is a recognized character .
c
c DIRECT ( RANDOM ) ACCESS FILE STRUCTURE
c       Number of records : Variable , maximum is 'maxnorecs'
c       Number of words per record : 42 ( 84 characters )
c       First record contains : No. of records, No. of words per record,
c               Number of the last directory record, No. of messages in file .
c       2nd to Recde record : contains a string of directory entries
c               Each directory entry consists of 2 integers which are
c                       ( Message number , Message start record number )
c               The last directory entry points to the first (hypothetical)
c               record beyond the last message .
c       Recde +1 to End Of File records : contain messages
c               Each message is terminated by a line which begins with a #
c
c Other limitations are :
c       (1) <maximum number of characters per text line> = 83
c       (2) <maximum number of messages allowed in a file> = 511
c
c Hkbdlu=5          # logical unit number of the keyboard
c Hlinlen=84        # max number of characters in a message line
c Hdirlen=512       # maximum number of messages allowed in a file
c Hmsglu1=10        # logical unit number of the source file
c Hmsglu2=11        # logical unit number of the object file
c Hvdulu=6          # logical unit number of video display unit
c
c integer *2 nargs,             # number of program arguments
c         irec,                 # record number being written
c         dirmsg(Hdirlen),      # directory array of message numbers
c         dirrec(Hdirlen),      # directory array of message start record nos.
c         imsg,                 # number of the message to be displayed
c         msgst, msgend,        # extrema for writing directory records
c         msgno,                # directory array index
c         nomsgs,               # number of messages in random access file
c         norecs,               # number of records in random access file
c         recde                 # record number of the last directory record
c character Cmsg*Hlinlen        # array containing a message line
c
      integer *2 Hkbdlu, Hlinlen, Hdirlen, Hmsglu1, Hmsglu2, Hvdulu
      parameter( Hkbdlu=5, Hlinlen=84, Hdirlen=512, Hmsglu1=10,
     1           Hmsglu2=11, Hvdulu=6 )
c
      integer *2 nargs, irec, dirmsg(Hdirlen), dirrec(Hdirlen), imsg,
     1  msgst, msgend, msgno, nomsgs, norecs, recde
      character Cmsg*Hlinlen
c
      nargs = iargc()
c
c       open input file
c
      if ( nargs .ge. 1 ) then
        call getarg( 1, cmsg )
        if ( 0 .eq. index( cmsg, '.' ) )
     1   cmsg = charnb( cmsg ) // '.msg'
      else
        cmsg = 'messag.msg'
        end if
      open( UNIT=Hmsglu1, FILE=cmsg, ACCESS='sequential',
     1      FORM='formatted', STATUS='old' )
c
c       open output file
c
      if ( nargs .le. 1 ) then
        cmsg =  cmsg( 1: index( cmsg, '.' ) ) // 'gsm'
      else
        call getarg( 2, cmsg )
        if ( 0 .eq. index( cmsg, '.' ) )
     1    cmsg = charnb( cmsg ) // '.gsm'
        end if
      open( UNIT=Hmsglu2, FILE=Cmsg, ACCESS='direct',
     1      FORM='unformatted', RECL=Hlinlen, STATUS='new', ERR=96 )
c
c ***** determine size of object file
c
      nomsgs = 0
      norecs = 0
1     if ( nomsgs .le. Hdirlen-1 ) then
        read( Hmsglu1, 10, END=2, ERR=98 ) Cmsg
10      format( a )
        if( cmsg(1:1) .eq. '#' ) then
          nomsgs = nomsgs + 1
        else
          norecs = norecs + 1
          end if
        goto 1
        end if
2     recde = nomsgs / 21 + 2
      norecs = norecs + recde
      if ( nomsgs .gt. Hdirlen - 1 ) then
        write( Hvdulu, 3 ) Hdirlen - 1
3       format( ' MSGCRT : WARNING : Too many messages in file.',
     1          ' Max = ', i7 )
        end if
c
      rewind( Hmsglu1, ERR=98 )
      irec = recde + 1
c
c  **** transfer messages from source to object files
c
      msgno = 1
c     start of loop
c
c       convert a single text line from the ASCII sequential access file
c        to a single record on the direct access object file
c
4       read( Hmsglu1, 10, END=7, ERR=98 ) Cmsg
c
c               new message detected
c
        if( cmsg(1:1) .eq. '#' ) then
          read( Cmsg(2:), *, ERR=5 ) imsg
c
c               generate directory entry
c
          dirmsg(msgno) = imsg
          dirrec(msgno) = irec
          msgno = msgno + 1
        else
          write( Hmsglu2, REC=irec, ERR=96 ) cmsg
          irec = irec + 1
          end if
        goto 4
c
5       write( Hvdulu, 6 ) msgno
6       format( ' MSGCRT : WARNING : Can''t decode message number' )
        goto 4
c
c           make last directory entry
c
7     dirrec(msgno) = irec
      dirmsg(msgno) = -32767
c
c       write directory records onto object file
c
      msgst = 1
      do 8 irec = 2,recde
        msgend = min0( msgst+20 , nomsgs+1 )
        write( Hmsglu2, REC=irec, ERR=96 )
     1    ( dirmsg(msgno), dirrec(msgno), msgno = msgst, msgend )
        msgst = msgst + 21
8       continue
      write( Hmsglu2, REC=1, ERR=96 ) norecs, Hlinlen, recde, nomsgs
      write( Hvdulu, 9 ) Nomsgs, Norecs
9     format( ' MSGCRT : OK : No. of messages : ', i5 /
     1        15x, 'Number of records : ', i5 )
      close( Hmsglu1 )
      close( Hmsglu2 )
      stop
c
c           error writing object file
c
96    write( Hvdulu, 97 )
97    format( ' MSGCRT : FATAL : Error writing object file' )
      stop
c
c           error reading source file
c
98    write( Hvdulu, 99 )
99    format( ' MSGCRT : FATAL : Error reading source file' )
      stop
      end
