c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM SBCACQ
c
c       Acquires data from any of the available A/D converters and puts
c       it in a direct access ( unformatted file ) . This program is
c       for testing the A/D converters .
c
c   Usage :-
c       SBCACQ [data-file-name]
c   data-file-name is the name of a direct  access  file into which  data is
c   placed . The file record length is 512 bytes . The file must not already
c   exist .
c      The user can put the program into `debug' mode where the instructions
c  for the SBC are  displayed on the  terminal  rather than sent to the SBC.
c  If not in `debug' mode, the data acquisition  system (i.e. SBC etc.) must
c  be started before selecting the A/D converter .
c      The selectable A/D converters are :-
c    1) DAS 1128 - random
c    2) DAS 250A - sequential mode
c    3) DAS 250A - random mode
c    4) Una-Voce - hard clock
c  The user can also select whether or not to unpack the data, and whether
c  to use the parallel line or the serial line for data transmission .
c  A simple display of the data can also be obtained .
c  The user can test the limits of  parameters can by specifying  negative
c  ( i.e. stupid ) values .
c
c Constants
c  Hkbdlu=5     # keyboard logical unit number
c  Hvdulu=6     # vdu logical unit number
c  Hdata=7      # data file logical unit number
c  Hmaxtries=3  # maximum number of input attempts
c menu items
c  Hexit=0      # exit
c  H1128=1      # DAS 1128 A/D converter
c  H250s=2      # DAS 250A A/D converter - sequential channel addressing mode
c  H250r=3      # DAS 250A A/D converter - random channel addressing mode
c  Hunav=4      # una - voca A/D converter
c  Hswitch=5    # access to parallel port and unpack on/off switches
c Variables
c  logical *1 fdebug,   # debugging switch
c             fdata(512),# data array
c             Equery,   # function : logical response
c             daopen    #          : open direct access file
c  integer *2 i, j,     # looping index
c             istart,   # start of data - record number
c             irec,     # data file record number
c             ichlst(32),# list of channel addresses
c             iatod,    # item selected from A/D menu
c             inocy,    # number of acquisition cyles
c             icylen,   # number of channels unsed
c             iargc,    # function : number of program arguments
c             jresp,    #          : integer response
c  integer *4 Dnocy,    # number of acquisition cycles
c             Dpresp    # function : long integer response
c  real Rcytim,         # acquisition cycle period
c       Rsampd,         # channel to channel time
c       Qresp           # function : real response
c  character cfile*32   # data file name
c            cprtcmd*32 # name of command file
c
      implicit none
      integer *2 Hkbdlu, Hvdulu, Hdata, Hmaxtries, Hexit, H1128,
     1  H250s, H250r, Hunav, Hswitch, Hexam
      parameter( Hkbdlu=5, Hvdulu=6, Hdata=7, Hmaxtries=3, Hexit=0,
     1  H1128=1, H250s=2, H250r=3, Hunav=4, Hswitch=5, Hexam=6 )
c
      logical *1 fdebug, fdata(512), Equery, daopen
      integer *2 i, j, irec, ichlst(32), iatod, inocy, icylen, iargc,
     1  istart, jresp
      integer *4 Dnocy, Dpresp
      real Rcytim, Rsampd, Qresp
      character cfile*32, cprtcmd*32
c
      irec = 1
c
c           start the data acquisition system
c
      fdebug = Equery( Hkbdlu, 'Are you using debug mode ?' )
      if ( .not. fdebug ) then
        write( Hvdulu, 1 )
1       format( ' Start the data acquisition system ( i.e. SBC etc. )'
     1      / '   Then press <return>' )
        read( Hkbdlu, 2 )
2       format( )
c
c           open data file
c
        if ( iargc() .le. 0 ) then
          cfile = 'sbcacq.dat'
        else
          call getarg( 1, cfile )
          end if
        Cprtcmd = ' '
        if ( .not. daopen( Hdata, Cfile, 512 ) ) goto 13
      else
        write( Hvdulu, 18 )
18      format( ' Command file name ? ' )
        read( Hkbdlu, 19 ) Cprtcmd
19      format( a )
        end if
      call sbinit( fdebug, Cprtcmd )
c
c           select A/D converter
c
c     start of loop
3       write( Hvdulu, 4 )
4         format( ' Which A/D converter ?  0) Exit' /
     1      24x, '1) DAS 1128 - random' /
     2      24x, '2) DAS 250A - sequential mode' /
     3      24x, '3) DAS 250A - random mode' /
     4      24x, '4) Una-Voce - hard clock' /
     5      24x, '5) Adjust switches' /
     6      24x, '6) Examine data              ' )
        iatod = jresp( Hkbdlu, Hexit, Hexam, Hmaxtries )
c
c           Exit
c
        if ( iatod .eq. Hexit ) then
          call sbstop
          stop
c
c           DAS 1128 A/D converter
c
        else if ( iatod .eq. H1128 ) then
          write( Hvdulu, 5 )
5         format( ' Acquisition cycle period (seconds) ?' )
          Rcytim = Qresp( Hkbdlu, 57.0e-6, 4.1943, Hmaxtries )
          write( Hvdulu, 6 )
6         format( '+Number of acquistion cycles ? ' )
          Inocy = Jresp( Hkbdlu, 1, 20480, Hmaxtries )
          write( Hvdulu, 7 )
7         format( '+Number of channels ? ' )
          Icylen = Jresp( Hkbdlu, 1, 32, Hmaxtries )
          write( Hvdulu, 8 )
8         format( '+A/D channels' / )
          do 10 i = 1, icylen
            write( Hvdulu, 9 ) i
9           format( '+', i2, ' ? ' )
10          ichlst(i) = Jresp( Hkbdlu, 0, 15, Hmaxtries )
          istart = irec
          call sloacq( Rcytim, Inocy, Icylen, Ichlst, Hdata, irec )
          write( Hvdulu, 21 ) Rcytim, Inocy, Icylen
21        format( ' cycle time ', g13.6 / ' no. of cycles ', i9 /
     1      1x, i2, ' channels' )
c
c           DAS 250 A - sequential channel addressing mode
c
        else if ( iatod .eq. H250s ) then
          write( Hvdulu, 5 )
          Rcytim = Qresp( Hkbdlu, 4.5e-6, 36.6117e-3, Hmaxtries )
          write( Hvdulu, 11 )
11        format( '+Channel-to-channel period (seconds) ? ' )
          Rsampd = Qresp( Hkbdlu, 3.91062e-6, 36.6117e-3, Hmaxtries )
          write( Hvdulu, 6 )
          Dnocy = Dpresp( Hkbdlu, 1, 1398100, Hmaxtries )
          write( Hvdulu, 7 )
          Icylen = Jresp( Hkbdlu, 1, 16, Hmaxtries )
          istart = irec
          call seqacq( Rcytim, Rsampd, Dnocy, Icylen, Hdata, irec )
          write( Hvdulu, 20 ) Rcytim, Rsampd, Dnocy, Icylen
20        format( ' cycle time ', g13.6 / ' sampling time ', g13.6 /
     1      ' no. of cycles ', i9 / 1x, i2, ' channels' )
c
c           DAS 250 A - random channel addressing mode
c
        else if ( iatod .eq. H250r ) then
          write( Hvdulu, 5 )
          Rcytim = Qresp( Hkbdlu, 80.5e-6, 36.611e-3, Hmaxtries )
          write( Hvdulu, 6 )
          Dnocy = Dpresp( Hkbdlu, 1, 1398100, Hmaxtries )
          write( Hvdulu, 7 )
          Icylen = Jresp( Hkbdlu, 1, 32, Hmaxtries )
          write( Hvdulu, 8 )
          do 12 i = 1, icylen
            write( Hvdulu, 9 ) i
12          ichlst(i) = Jresp( Hkbdlu, 0, 15, Hmaxtries )
          istart = irec
          call sanacq( Rcytim, Dnocy, Icylen, Ichlst, Hdata, irec )
          write( Hvdulu, 21 ) Rcytim, Dnocy, Icylen
c
c           Una - voce
c
        else if ( iatod .eq. Hunav ) then
          write( Hvdulu, 6 )
          Dnocy = Jresp( Hkbdlu, 1, 1398100, Hmaxtries )
          write( Hvdulu, 7 )
          Icylen = Jresp( Hkbdlu, 1, 32, Hmaxtries )
          istart = irec
          call simacq( Dnocy, Icylen, Hdata, irec )
          write( Hvdulu, 22 ) Dnocy, Icylen
22        format( ' no. of cycles ', i9 / 1x, i2, ' channels' )
        else if ( iatod .eq. Hswitch ) then
          call sbunpk( Equery( Hkbdlu, 'Is data to be unpacked ?' ) )
          call sppdis( Equery( Hkbdlu,
     1      'Is the parallel port to be disabled ?' ) )
        else if ( iatod .eq. Hexam ) then
          do 17 i = istart, irec - 1
            read( Hdata, REC=i ) fdata
            write( Hvdulu, 15 )  ( fdata(j), j = 1, 256 )
15          format( 16( 1x,z2, 1x,z2, 1x,z2, 1x,z2, 1x,z2, 1x,z2,
     1                  1x,z2, 1x,z2, 1x,z2, 1x,z2, 1x,z2, 1x,z2,
     2                  1x,z2, 1x,z2, 1x,z2, 1x,z2 / ),
     3              '   Press <RETURN> to continue' )
            read( Hkbdlu, 16 )
16          format()
            write( Hvdulu, 15 )  ( fdata(j), j = 257, 512 )
            read( Hkbdlu, 16 )
17          continue
          end if
        goto 3
c           end of loop
c
13    write( Hvdulu, 14 ) cfile
14    format( ' Can''t open data file ', a )
      stop
      end
