c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM SBCDAT
c
c       TEST SBC DATA TRANSMISSION
c    Program to receive data  transmitted from the SBC and store the data in
c  a file or display it on the terminal .
c    The program must have one or two arguments . The first argument must be
c  a switch  consisting of a `-' followed  by one or more  letters .  One of
c  these letters must be `p' or `s', denoting which line (parallel or serial
c  respectively ) is used  for transmitting  the data . The second  argument
c  is the name of a file into which the data is put .  If there is no second
c  argument,  the  data  file  is  called  `sbcdat.dat' .  The data  file is
c  sequential access formatted .
c
c       Form of invocation :
c               SBCDAT <switch> <output file>
c       where <switch> = -[npqst]       n = no output to file
c                                       p = receive from parallel line
c                                       q = quick - do not store or type data
c                                       s = receive from serial line
c                                       t = type results
c
c
c constants
c  Hnullu=-1            # Nul file logical unit number
c  Hvdulu=6             # vdu logical unit number
c  Hfillu=12            # data file logical unit number
c  Hparlu=13            # logical unit no. for parallel port
c  Hserlu=14            # logical unit no. for serial port
c
c Variables
c  integer *4 dnbyte    # Total number of data bytes read
c             dpbkdat   # reads block formatted data onto file
c  integer *2 iargc,    # library function
c             idatlu    # logical unit number for data input port
c  character Copfile*64 # output data file name
c            switch*16  # switches
c  logical *1 quick,    # data is not to be stored or displayed
c             type,     # data is to be displayed
c             noop,     # data is not written to file
c             existf    # function : file exists
c             io_serial #          : opens a serial line - raw mode
c             io_paral  #          : opens the parallel port ( I/P )
c  subroutine getarg    # get a program argument
c
      implicit none
      integer *2 Hvdulu, Hfillu, Hnullu, Hserlu, Hparlu
      parameter( Hvdulu=6, Hfillu=12, Hnullu=0, Hserlu=13, Hparlu=14 )
c
      integer *4 dnbyte, dpbkdat
      integer *2 idatlu, iargc
      character Copfile*64, switch*16
      logical *1 quick, type, noop, existf, io_serial, io_paral
c
c               Get Switches
c
      if ( iargc() .le. 0 ) then
        write( Hvdulu, 1 )
1       format( ' SBCDAT : FATAL : No arguments' )
        stop
        end if
      call getarg( 1, switch )
      if ( switch(1:1) .ne. '-' ) then
        write( Hvdulu, 2 )
2       format( ' SBCDAT: FATAL : 1st Arg is not a switch' )
        stop
        end if
      quick = index( switch, 'q' ) .ne. 0
      type = index( switch, 't' ) .ne. 0 .and. .not. quick
      noop = index( switch, 'n' ) .ne. 0 .or. quick
      if ( noop )
     1  write( Hvdulu, 3 )
3       format ( ' SBCDAT : NOTE : No output' )
c
c               Open O/P file
c
      if ( .not. noop ) then
        if ( iargc() .lt. 2 ) then
          Copfile = 'sbcdat.dat'
        else
          call getarg( 2, Copfile )
          end if
        if ( existf( Copfile ) ) then
          write( Hvdulu, 4 ) Copfile
4         format( ' SBCDAT : WARNING : Output file ', a,
     1      ' already exists' )
          end if
        open( UNIT=Hfillu, FILE=Copfile, STATUS='unknown', ERR=9 )
        end if
c
c               Open I/P port
c
      if ( .not. io_serial( Hserlu, 2, 9600 ) ) then
        write( Hvdulu, 11 )
11      format( ' SBCDAT : FATAL : Cannot open serial port' )
        stop
        end if
      if ( index( switch, 's' ) .ne. 0 ) then
        if ( index( switch, 'p' ) .ne. 0 ) then
          write( Hvdulu, 5 )
5         format(  ' SBCDAT : FATAL : Data from only ONE port please' )
          stop
          end if
        idatlu = Hserlu
      else if ( index( switch, 'p' ) .ne. 0 ) then
        idatlu = Hparlu
        if ( .not. io_paral( Hparlu, 1 ) ) then
          write( Hvdulu, 12 )
12        format( ' SBCDAT : FATAL : Cannot open parallel port' )
          stop
          end if
      else
        write( Hvdulu, 6 )
6       format( ' SBCDAT : FATAL : No I/P device' )
        stop
        end if
c
c               Get Data
c
      write( Hvdulu, 7 )
7     format( ' SBCDAT : Send from SBC' )
      if ( type ) then
        Dnbyte = Dpbkdat( idatlu, Hserlu, 0, Hvdulu )
      else if ( .not. noop ) then
        Dnbyte = Dpbkdat( idatlu, Hserlu, 0, Hfillu )
      else
        Dnbyte = Dpbkdat( idatlu, Hserlu, -2, Hnullu )
        end if
      write( Hvdulu, 8 ) Dnbyte
8     format( ' SBCDAT : Total of ', i12, ' Data bytes' )
      stop
c
9     write( Hvdulu, 10 )
10    format( ' SBCDAT : FATAL : Cannot open data file' )
      stop
      end
