;
;    Copyright (C) 1997  Rod G. Curtin
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;

;
; down line load program for intel hex files
; assumes unix talk is at other end of console line
; Transmission is initiated by sending xon, 1 line
; at a time is received (assume terminated with cr only)
; line cr/lf is appended to end of console buffer and
; chars are transfered to disk buffer which is written
; when full. Each new line is requested with xon and
; end of tranmission is designated with ctl-z.
;
; CPM equates
;
boot    equ     0000h
bdos    equ     0005h
fctbl   equ     005ch
buffer  equ     0080h
tpa     equ     0100h
;
; BDOS calls
;
conout  equ     2
printf  equ     9
readcn  equ     10
close   equ     16
deletf  equ     19
writef  equ     21
makef   equ     22
;
; ctlchar and other equates
;
dc1     equ     17
eof     equ     26      ;ctl-z
cr      equ     13
lf      equ     10
;
; start of prog
;
        org     tpa
;
        lxi     sp,stack
        lxi     d,fctbl         ;delete filke file first
        call    delete
        lxi     d,fctbl
        call    make
        lxi     d,nodir
        inr     a
        cz      finis           ;error
;
; set disk buffer pointers
;
        lxi     d,buffer
        mvi     b,128
        push    d
        push    b
;
; solicit transmission
;
snddc1: mvi     e,dc1
        mvi     c,conout
        call    bdos
;
; read console buffer
;
        mvi     c,readcn
        lxi     d,conbuf
        call    bdos
        jmp     save
;
; save redirects to here on ctl-z
;
done:   pop     b               ;buffer pointers
        pop     d
        mvi     a,128
        cmp     b
        jz      done2           ;empty
        mvi     a,eof           ;fill with ctl-z = eof in cpm
done1:  stax    d
        inx     d
        dcr     b
        jnz     done1
        call    wrtbuf
;
; close file
;
done2:  mvi     c,close
        lxi     d,fctbl
        call    bdos
        lxi     d,clserr
        inr     a
        cz      finis
        lxi     d,okmsg
;
; finis prints message and returns tO cpm
;
finis:  mvi     c,printf
        call    bdos
        jmp     boot
;
; Subroutines
;
; save moves chars to disk buffer and writes when full
;
; REG Usage hl - conbuf ptr
;           c    conbuf count
;           de - dskbuf ptr
;               b   dskbuf count
;
save:   lda     conbuf+1        ;char count
        mov     c,a
        lxi     h,conbuf+1
        mvi     b,0
        dad     b               ;hl -> to last char
;
; check for EOF
;
        mvi     a,eof
        sub     m
        jz      done
;
; put in cr(if not there) and lf
;
        mvi     a,cr
        sub     m
        jz      putlf
        inx     h
        inr     c
        mvi     a,cr
        mov     m,a
putlf:  inx     h
        inr     c
        mvi     a,lf
        mov     m,a
;
; transfer buffer
;
        mov     a,c     ;save count
        pop     b       ;restore disk buffer params
        mov     c,a
        pop     d
        lxi     h,conbuf+2      ; 1st char
nxtchr: mov     a,m
        stax    d
        inx     d
        inx     h
;
; disk buffer full ?
;
        dcr     b
        cz      wrtbuf          ;yes write
;
; all chars moved ?
;
        dcr     c
        jnz     nxtchr
;
; save disk buffer params for next time
;
        push    d
        push    b
        jmp     snddc1          ;next line please !!
;
; write disk buffer
;
wrtbuf: push    h               ;save conbuf params
        push    b
        lxi     d,fctbl
        call    write
        lxi     d,space
        ora     a
        cnz     finis
        lxi     d,buffer
        pop     b
        pop     h
        MVI     B,128
        RET
DELETE: MVI     C,DELETF
        JMP     BDOS
;
MAKE:   MVI     C,MAKEF
        JMP     BDOS
;
WRITE:  MVI     C,WRITEF
        JMP     BDOS
;
; MESSAGES
;
NODIR:  DB      'NO DIRECTORY SPACE $'
SPACE:  DB      'OUT OF DISK SPACE$'
CLSERR: DB      'CANNOT CLOSE FILE$'
OKMSG:  DB      'TRANSMISSION COMPLETE$'
;
; BUFFER SPACE
;
CONBUF: DB      255
        DS      256
;
; STACK
;
        DS      32
STACK:
        END

