/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 *          Binding functions for using clipping functions
 *                        Sun FORTRAN
 *
 *  The names of the functions called from the FORTRAN are the same as
 *  the names of the original C source functions . However the arguments
 *  may be different, according to the following rules .
 *
 *      +-------------------------------+----------------------------+
 *      |  C argument type              | FORTRAN argument type      |
 *      +-------------------------------+----------------------------+
 *      | coord_pair point;             | real x, y                  |
 *      | coord_triple point;           | real x, y, z               |
 *      | clip_area *viewport           | integer viewport           |
 *      +-------------------------------+----------------------------+
 *
 *  LOGICAL FUNCTIONS return values of
 *    .TRUE.  -- for success
 *    .FALSE. -- for failure
 *
 *  If the returned value of the C source is a pointer, the binding function
 *  presents the returned value as an INTEGER . Because this essentially
 *  a kludge, the user must not change the returned value if he wishes to
 *  make use of it, and should not attempt to use it as an integer .
 *
 *  N.B. These binding functions depend upon the nature of the FORTRAN
 *       implementation of parameter passing . This is different for
 *       nearly every `brand' of FORTRAN, and so these bindings are only
 *       supposed to work with SUN FORTRAN .
*/

#include "system.h"     /* system dependent stuff */
#include CLIP_HEADER
#include <stdio.h>

/*  LOGICAL FUNCTION CLIP( X, Y, Z, CLIP_DESCRIPTOR )
 *  REAL X, Y, Z
 *  INTEGER CLIP_DESCRIPTOR
*/
int clip__( x, y, z, clip_ptr )
float *x, *y, *z;
int * clip_ptr;
  {
    coord_triple point;
    point._x = *x;
    point._y = *y;
    point._z = *z;
    return (int) clip( &point, (clip_area *) clip_ptr );
  }


/*  LOGICAL FUNCTION MOVE_CLIP( X, Y, Z, CLIP_DESCRIPTOR )
 *  REAL X, Y, Z
 *  INTEGER CLIP_DESCRIPTOR
*/
int move_clip__( x, y, z, clip_ptr )
float *x, *y, *z;
int * clip_ptr;
  {
    coord_triple point;
    point._x = *x;
    point._y = *y;
    point._z = *z;
    return (int) move_clip( &point, (clip_area *) clip_ptr );
  }


/*  INTEGER FUNCTION DEF_2RCLIP( X_MIN, Y_MIN, X_MAX, Y_MAX, X_FIRST, Y_FIRST )
 *  REAL X_MIN, Y_MIN, X_MAX, Y_MAX, X_FIRST, Y_FIRST
*/
int def_2rclip__( x_min, y_min, x_max, y_max, x_first, y_first )
float *x_min, *y_min, *x_max, *y_max, *x_first, *y_first;
  {
    coord_pair min, max, first;
    min._x = *x_min;
    min._y = *y_min;
    max._x = *x_max;
    max._y = *y_max;
    first._x = *x_first;
    first._y = *y_first;
    return (int) def_2rclip( &min, &max, &first );
  }


/*  INTEGER FUNCTION DEF_3RCLIP( X_MIN, Y_MIN, Z_MIN, X_MAX, Y_MAX, Z_MAX,
 *                               X_FIRST, Y_FIRST, Z_FIRST )
 *  REAL X_MIN, Y_MIN, Z_MIN, X_MAX, Y_MAX, Z_MAX, X_FIRST, Y_FIRST, Z_FIRST
*/
int def_3rclip__( x_min, y_min, z_min, x_max, y_max, z_max,
                 x_first, y_first, z_first )
float *x_min, *y_min, *z_min, *x_max, *y_max, *z_max,
      *x_first, *y_first, *z_first;
  {
    coord_triple min, max, first;
    min._x = *x_min;
    min._y = *y_min;
    min._z = *z_min;
    max._x = *x_max;
    max._y = *y_max;
    max._z = *z_max;
    first._x = *x_first;
    first._y = *y_first;
    first._z = *z_first;
    return (int) def_3rclip( &min, &max, &first );
  }


/*  LOGICAL FUNCTION CLEAR_CLIP( CLIP_DESCRIPTOR )
 *  INTEGER CLIP_DESCRIPTOR
*/
int clear_clip__( clip_ptr )
int * clip_ptr;
  {
    return (int) clear_clip( (clip_area *) clip_ptr );
  }


/*  LOGICAL FUNCTION OFF_CLIP( CLIP_DESCRIPTOR )
 *  INTEGER CLIP_DESCRIPTOR
*/
int off_clip__( clip_ptr )
int * clip_ptr;
  {
    return (int) off_clip( (clip_area *) clip_ptr );
  }


/*  LOGICAL FUNCTION ON_CLIP( CLIP_DESCRIPTOR )
 *  INTEGER CLIP_DESCRIPTOR
*/
int on_clip__( clip_ptr )
int * clip_ptr;
  {
    return (int) on_clip( (clip_area *) clip_ptr );
  }
