/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *          Binding functions for using error handler functions from
 *                        Sun FORTRAN
 *
 *  The names of the functions called from the FORTRAN are the same as
 *  the names of the original C source functions . However the arguments
 *  may be different, according to the following rules .
 *
 *      +-------------------------------+----------------------------+
 *      |  C argument type              | FORTRAN argument type      |
 *      +-------------------------------+----------------------------+
 *      | char *string;                 | character string*(*)       |
 *      | int status;                   | integer status             |
 *      +-------------------------------+----------------------------+
 *
 *  LOGICAL FUNCTIONS return values of
 *    .TRUE.  -- for success
 *    .FALSE. -- for failure
 *
 *  If the returned value of the C source is a pointer, the binding function
 *  presents the returned value as an INTEGER . Because this essentially
 *  a kludge, the user must not change the returned value if he wishes to
 *  make use of it, and should not attempt to use it as an integer .
 *
 *  N.B. These binding functions depend upon the nature of the FORTRAN
 *       implementation of parameter passing . This is different for
 *       nearly every `brand' of FORTRAN, and so these bindings are only
 *       supposed to work with SUN FORTRAN .
*/

#include "system.h"
#include <stdio.h>
#include <stdlib.h>
#include ERROR_HEADER

char *malloc();

/*  SUBROUTINE MAKE_ERROR( STATUS, WHERE, TEXT )
 *  INTEGER STATUS
 *  CHARACTER WHERE*(*), TEXT*(*)
*/
void make_error__( status, where, text, len_where, len_text )
int *status, len_where, len_text;
char *where, *text;
  {
    int counter;
    char *ptr;
    char *new_where = (char *)
        malloc( sizeof( char ) * (unsigned) ( len_where + 1 ) );
    char *new_text = (char *)
        malloc( sizeof( char ) * (unsigned) ( len_text + 1 ) );
    for( ptr = new_where, counter = 0; counter <= len_where; counter++ )
        *ptr++ = *where++;
    *ptr = '\000';
    for( ptr = new_text, counter = 0; counter <= len_text; counter++ )
        *ptr++ = *text++;
    *ptr = '\000';
    make_error( *status, new_where, new_text );
    (void) free( new_where );
    (void) free( new_text );
  }


/*  LOGICAL FUNCTION IS_ERROR( STATUS )
 *  INTEGER STATUS
*/
int is_error__( status )
int *status;
  {
    return (int) is_error( *status );
  }


/*  SUBROUTINE CLEAR_ERROR()
*/
void clear_error__()
  {
    clear_error();
  }


/* INTEGER FUNCTION DUMP_ERROR( STATUS )
 * INTEGER STATUS
*/
int dump_error__( status )
int *status;
  {
    return dump_error( stdout, *status );
  }

