/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *          Binding functions for using device drivers from
 *                        Sun FORTRAN
 *
 *  The names of the functions called from the FORTRAN are the same as
 *  the names of the original C source functions . However the arguments
 *  may be different, according to the following rules .
 *
 *      +-------------------------------+----------------------------+
 *      |  C argument type              | FORTRAN argument type      |
 *      +-------------------------------+----------------------------+
 *      | int i;                        | integer i                  |
 *      | coord_pair point;             | real x, y                  |
 *      | char *string;                 | character string*(*)       |
 *      | double *value;                | real value                 |
 *      | clip_area *viewport           | integer viewport           |
 *      | double * OR char * value;     | real real_value            |
 *      |                               | character string_value*(*) |
 *      | coord_pair * OR char * value; | real x, y                  |
 *      |                               | character string_value*(*) |
 *      +-------------------------------+----------------------------+
 *
 *  Sometimes a pointer argument may be able to point to more than one kind
 *  of object. In these cases, one argument in the C source becomes 2 or more
 *  arguments in the binding function.
 *
 *  LOGICAL FUNCTIONS return values of
 *    .TRUE.  -- for success
 *    .FALSE. -- for failure
 *
 *  If the returned value of the C source is a pointer, the binding function
 *  presents the returned value as an INTEGER . Because this essentially
 *  a kludge, the user must not change the returned value if he wishes to
 *  make use of it, and should not attempt to use it as an integer .
 *
 *  N.B. These binding functions depend upon the nature of the FORTRAN
 *       implementation of parameter passing . This is different for
 *       nearly every `brand' of FORTRAN, and so these bindings are only
 *       supposed to work with SUN FORTRAN .
*/

#include "system.h"             /* system dependent stuff */
#include <stdio.h>
#include DEVICE_G_HEADER

char *malloc();
static FILE * file_ptr = (FILE *) 0;

/*  INTEGER FUNCTION TF_DEVICE( DEVICE_TYPE, FILE_NAME, OPTION_STRING )
*/
int tf_device_( device_type, file_name, options, file_len, opt_len )
int *device_type;
char *file_name, *options;
int  file_len, opt_len;
  {
    int counter,
        ret_value;
    char *ptr,
         *opt_str  = malloc( sizeof( char ) * (unsigned) ( opt_len + 1 ) ),
         *file_str = malloc( sizeof( char ) * (unsigned) ( file_len + 1 ) );
    for( ptr = opt_str, counter = 1; counter <= opt_len; counter++ )
        *ptr++ = *options++;
    *ptr = '\000';
    if ( *file_name != ' ' && file_len > 0 )
      {
        for( ptr = file_str, counter = 1; counter <= file_len; counter++ )
            *ptr++ = *file_name++;
        *ptr = '\000';
        file_ptr = fopen( file_str, "w" );
        ret_value = (int) t_device( *device_type, file_ptr, opt_str );
        (void) free( file_str );
      }
    else
        ret_value = (int) t_device( *device_type, file_ptr = stdout, opt_str );
    (void) free( opt_str );
    return ret_value;
  }


/*  LOGICAL FUNCTION TF_DUMP( FILE_NAME )
*/
int tf_dump_( file_name, len )
char *file_name;
int len;
  {
    char *file_str = malloc( sizeof( char ) (unsigned) ( len + 1 ) ),
         *ptr = file_str;
    int counter, ret_value;
    for( counter=1; counter <= len; counter++ )
        *ptr++ = *file_name++;
    *ptr = '\000';
    ret_value = (int) t_dump( file_str );
    (void) free( file_str );
    return ret_value;
  }


/*  INTEGER FUNCTION TF_BEGIN( FILE_NAME, OPTION_STRING )
 *  CHARACTER FILE_NAME*(*), OPTION_STRING*(*)
 *
 *   returns the clip_area descriptor
*/
int tf_begin_( file_name, options, file_len, opt_len )
char *file_name, *options;
int  file_len, opt_len;
  {
    int counter,
        ret_value;
    char *ptr,
         *opt_str  = malloc( sizeof( char ) * (unsigned) ( opt_len + 1 ) ),
         *file_str = malloc( sizeof( char ) * (unsigned) ( file_len + 1 ) );
    for( ptr = opt_str, counter = 1; counter <= opt_len; counter++ )
        *ptr++ = *options++;
    *ptr = '\000';
    if ( *file_name != ' ' && file_len > 0 )
      {
        for( ptr = file_str, counter = 1; counter <= file_len; counter++ )
            *ptr++ = *file_name++;
        *ptr = '\000';
        file_ptr = fopen( file_str, "w" );
        ret_value = (int) t_begin( file_ptr, opt_str );
        (void) free( file_str );
      }
    else
        ret_value = (int) t_begin( stdout, file_ptr = opt_str );
    (void) free( opt_str );
    return ret_value;
  }


/*  LOGICAL FUNCTION TF_END()
*/
int tf_end_()
  {
    int ret_value;
    ret_value = (int) t_end();
    if ( file_ptr && file_ptr != stdout && file_ptr != stderr )
      {
        (void) fclose( file_ptr );
        file_ptr = (FILE *) 0;
      }
    return ret_value;
  }


/*  LOGICAL FUNCTION TF_VWPORT( CLIP_DESCRIPTOR )
 *  INTEGER CLIP_DESCRIPTOR
*/
int tf_vwport_( clip_ptr )
int * clip_ptr;
  {
    return (int) t_vwport( (clip_area *) clip_ptr );
  }


/*  LOGICAL FUNCTION TF_COLOUR( PEN_NUMBER )
 *  INTEGER PEN_NEUMBER
*/
int tf_colour_( pen_number )
int *pen_number;
  {
    return (int) t_colour( *pen_number );
  }


/*  LOGICAL FUNCTION TF_DASHED( STYLE )
 *  INTEGER STYLE
*/
int tf_dashed_( style )
int *style;
  {
    return (int) t_dashed( *style );
  }


/*  LOGICAL FUNCTION TF_THICK( WIDTH )
 *  INTEGER WIDTH
*/
int tf_thick_( width )
int *width;
  {
    return (int) t_thick( *width );
  }


/*  LOGICAL FUNCTION TF_GR_OFF()
*/
int tf_gr_off_()
  {
    return (int) t_gr_off();
  }


/*  LOGICAL FUNCTION TF_GR_ON()
*/
int tf_gr_on_()
  {
    return (int) t_gr_on();
  }


/*  LOGICAL FUNCTION TF_VISIB()
*/
int tf_visib_()
  {
    return (int) t_visib();
  }


/*  LOGICAL FUNCTION TF_INVIS()
*/
int tf_invis_()
  {
    return (int) t_invis();
  }


/*  LOGICAL FUNCTION TF_REVERS()
*/
int tf_revers_()
  {
    return (int) t_revers();
  }


/*  LOGICAL FUNCTION TF_REPLAC()
*/
int tf_replac_()
  {
    return (int) t_replac();
  }


/*  LOGICAL FUNCTION TF_NEWPAG()
*/
int tf_newpag_()
  {
    return (int) t_newpag();
  }


/*  LOGICAL FUNCTION TF_NEWSCR()
*/
int tf_newscr_()
  {
    return (int) t_newscr();
  }


/*  LOGICAL FUNCTION TF_BELL()
*/
int tf_bell_()
  {
    return (int) t_bell();
  }


/*  LOGICAL FUNCTION TF_WAIT()
*/
int tf_wait_()
  {
    return (int) t_wait();
  }


/*  LOGICAL FUNCTION TF_TXTMOV( X, Y )
 *  REAL X, Y
*/
int tf_txtmov_( x, y )
float *x, *y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return (int) t_txtmov( &point );
  }


/*  LOGICAL FUNCTION TF_TXTCUR( ROW, COLUMN )
 *  INTEGER ROW, COLUMN
*/
int tf_txtcur_( row, column )
int *row, *column;
  {
    return (int) t_txtcur( *row, *column );
  }


/*  LOGICAL FUNCTION TF_TXTSET( ATTRIBUTE, REAL_VALUE, STRING_VALUE )
 *  INTEGER ATTRIBUTE
 *  REAL    REAL_VALUE
 *  CHARACTER STRING_VALUE*(*)
*/
int tf_txtset_( attribute, real_value, string_value, str_len )
int *attribute, str_len;
float *real_value;
char  *string_value;
  {
    if ( *attribute == T_CH_ALIGN )
      {
        int result;
        char *new_str = malloc( sizeof( char ) * (unsigned) 3 );
        char *ptr = new_str;
        *ptr++ = *string_value++;
        *ptr++ = *string_value;
        *ptr = '\000';
        result = (int) t_txtset( *attribute, new_str );
        (void) free( new_str );
        return result;
      }
    else if ( *attribute == -T_CH_ALIGN )
        return (int) t_txtset( *attribute, string_value );
    else
      {
        double value = *real_value;
        return (int) t_txtset( *attribute, (char *) &value );
      }
  }


/*
 *  LOGICAL FUNCTION TF_SYMDRW( X, Y, SYMBOL )
 *  REAL X, Y
 *  CHARACTER SYMBOL *1
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_symdrw_( x, y, symbol, symbol_len )
float *x, *y;
char  *symbol;
int   symbol_len;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return (int) t_symdrw( &point, *symbol );
  }


/*
 *  LOGICAL FUNCTION TF_TXTDRW( STRING )
 *  CHARACTER STRING *(*)
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_txtdrw_( string, str_len )
/*
 *      it is necessary to copy the string to a new place and then append
 *      a NUL character to it.
*/
char  *string;
int   str_len;
  {
    int counter, result;
    char *new_str = (char *)
        malloc( sizeof( char ) * (unsigned) ( str_len + 1 ) );
    char *ptr = new_str;
    for( counter = 1; counter <= str_len; counter++ )
        *ptr++ = *string++;
    *ptr = '\000';
    result = (int) t_txtdrw( new_str );
    (void) free( new_str );
    return result;
  }

/*
 *  INTEGER FUNCTION TF_CH_DEF( WHAT, CHAR, INDEX, X, Y )
 *  INTEGER WHAT
 *  CHARACTER CHAR*1
 *  INTEGER INDEX
 *  REAL X, Y
*/
int tf_ch_def_( what, character, index, x, y, str_len )
/*
 * This function allows the user to obtain the shape of any character defined
 * in the ASCII and SYMBOL fonts. With the exception of string alignment, the
 * transformations implied by t_txtset are applied to the returned coordinates.
 *
 * The returned values are
 *  T_FT_SIZE       X & Y   ( width & height of characters)
 *  T_FT_EXIST      T_CH_DEF
 *  T_FT_LENGTH     T_CH_DEF
 *  T_FT_FIRST      T_CH_DEF
 *  T_FT_LAST       T_CH_DEF
 *  T_FT_MIDDLE     T_CH_DEF
 *  T_FT_POINT      X & Y   ( coordinate of a point )
 */
int  *what;         /* specifies what to do (T_FT_* defined values) */
char *character;    /* character to fetch from the font */
int  *index;        /* coordinate number   */
float *x, *y;       /* returned coordinate */
int  str_len;
  {
    coord_pair point;
    int result = (int) t_ch_def( *what, *character, *index, &point );
    if ( *what == T_FT_SIZE || *what == T_FT_POINT )
      {
        *x = point._x;
        *y = point._y;
      }
    return result;
  }


/*
 *  LOGICAL FUNCTION TF_TXTGET( STRING, MAX_CHARS )
 *  CHARACTER STRING *(*)
 *  INTEGER MAX_CHARS
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_txtget_( string, max_chars, str_len )
char  *string;
int   *max_chars, str_len;
  {
    char *dest, *src, *end_dest, *c_str;
    int  result;
    unsigned int max_len = *max_chars <= str_len ? *max_chars : str_len;

    c_str = malloc( sizeof( char ) * ( max_len + 1 ) );
    result = (int) t_txtget( c_str, max_chars );
    end_dest = string + str_len;
    for ( dest = string, src = c_str; *src && dest < end_dest; )
        *dest++ = *src++;
    while ( dest < end_dest )
        *dest++ = ' ';
    (void) free( c_str );
    return result;
  }


/*
 *  LOGICAL FUNCTION TF_TXTPUT( STRING )
 *  CHARACTER STRING *(*)
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_txtput_( string, str_len )
/*
 *      it is necessary to copy the string to a new place and then append
 *      a NUL character to it.
*/
char  *string;
int   str_len;
  {
    int counter, result;
    char *new_str =
        (char *) malloc( sizeof( char ) * (unsigned) ( str_len + 1 ) ),
         *ptr = new_str;
    for( counter = 1; counter <= str_len; counter++ )
        *ptr++ = *string++;
    *ptr = '\000';
    result = (int) t_txtput( new_str );
    (void) free( new_str );
    return result;
  }


/*
 *  LOGICAL FUNCTION TF_BOXFIL( LEFT, BOTTOM, RIGHT, TOP, STYLE )
 *  REAL LEFT, BOTTOM, RIGHT, TOP
 *  INTEGER STYLE
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_boxfil_( left, bottom, right, top, style )
float *left, *bottom, *right, *top;
int *style;
  {
    coord_pair lower_left, upper_right;
    lower_left._x = *left;
    lower_left._y = *bottom;
    upper_right._x = *right;
    upper_right._y = *top;
    return (int) t_boxfil( &lower_left, &upper_right, *style );
  }


/*
 *  LOGICAL FUNCTION TF_XHRGET( X, Y, CHAR_PRESSED )
 *  REAL X, Y
 *  CHARACTER CHAR_PRESSED *1
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_xhrget_( x, y, char_pressed, n_chars )
float *x, *y;
char *char_pressed;
int n_chars;
  {
    int result;
    coord_pair point;
    char *dest;
    result = (int) t_xhrget( &point, char_pressed );
    *x = point._x;
    *y = point._y;

    dest = char_pressed + 1;
    while ( dest < char_pressed + n_chars )
        *dest++ = ' ';
    return result;
  }


/*
 *  LOGICAL FUNCTION TF_XHRPUT( X, Y )
 *  REAL X, Y
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_xhrput_( x, y )
float *x, *y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return (int) t_xhrput( &point );
  }


/*
 *  LOGICAL FUNCTION TF_STATUS( ATTRIBUTE, X, Y, STRING_VALUE )
 *  INTEGER ATTRIBUTE
 *  REAL    X, Y
 *  CHARACTER STRING_VALUE*(*)
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_status_( attribute, x, y, string, str_len )
int *attribute, str_len;
float *x, *y;
char  *string;
  {
    if ( *attribute == T_ST_DEVICE )
      {
        int result;
        char *dest, *src, *end_dest;
        char *c_str = malloc( sizeof( char ) * ( str_len + 1 ) );
        result = (int) t_status( *attribute, string );
        end_dest = string + str_len;
        for ( dest = string, src = c_str; *src && dest < end_dest; )
            *dest++ = *src++;
        while ( dest < end_dest )
            *dest++ = ' ';
        (void) free( c_str );
        return result;
      }
    else if ( *attribute == T_ST_SCREEN || *attribute == T_ST_CATEG ||
              *attribute == T_ST_MONO  )
      {
        int i_result, j_result;
        j_result = (int) t_status( *attribute, &i_result );
        *y = *x = (float) i_result;
        return j_result;
      }
    else
      {
        coord_pair point;
        int result;
        result = (int) t_status( *attribute, (char *) &point );
        *x = point._x;
        *y = point._y;
        return result;
      }
  }


/*
 *  LOGICAL FUNCTION TF_PR_TER()
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_pr_ter_()
  {
    return (int) t_pr_ter( stdout );
  }


/*
 *  LOGICAL FUNCTION TF_VECMOV( X, Y )
 *  REAL X, Y
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_vecmov_( x, y )
float *x, *y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return (int) t_vecmov( &point );
  }


/*
 *  LOGICAL FUNCTION TF_VECDRW( X, Y )
 *  REAL X, Y
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_vecdrw_( x, y )
float *x, *y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return (int) t_vecdrw( &point );
  }


int iand_( a, b )
unsigned *a,*b;
  {
    return *a & *b;
  }

