/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *          Binding functions for using device drivers from
 *                        VAX FORTRAN
 *
 *  The names of the functions called from the FORTRAN are the same as
 *  the names of the original C source functions . However the arguments
 *  may be different, according to the following rules .
 *
 *      +-------------------------------+----------------------------+
 *      |  C argument type              | FORTRAN argument type      |
 *      +-------------------------------+----------------------------+
 *      | int i;                        | integer i                  |
 *      | coord_pair point;             | real x, y                  |
 *      | char *string;                 | character string*(*)       |
 *      | double *value;                | real value                 |
 *      | clip_area *viewport           | integer viewport           |
 *      | double * OR char * value;     | real real_value            |
 *      |                               | character string_value*(*) |
 *      | coord_pair * OR char * value; | real x, y                  |
 *      |                               | character string_value*(*) |
 *      +-------------------------------+----------------------------+
 *
 *  Sometimes a pointer argument may be able to point to more than one kind
 *  of object. In these cases, one argument in the C source becomes 2 or more
 *  arguments in the binding function.
 *
 *  LOGICAL FUNCTIONS return values of
 *    .TRUE.  -- for success
 *    .FALSE. -- for failure
 *
 *  If the returned value of the C source is a pointer, the binding function
 *  presents the returned value as an INTEGER . Because this essentially
 *  a kludge, the user must not change the returned value if he wishes to
 *  make use of it, and should not attempt to use it as an integer .
 *
 *  N.B. These binding functions depend upon the nature of the FORTRAN
 *       implementation of parameter passing . This is different for
 *       nearly every `brand' of FORTRAN, and so these bindings are only
 *       supposed to work with VAX FORTRAN .
*/

#include "system.h"             /* system dependent stuff */
#include <stdio.h>
#include <descrip.h>
#include DEVICE_G_HEADER
#include "dev_udef.h"

#define VMS_LOGICAL(result)     ( - ( result != 0 ) )

char *malloc();
static int current_device = NONEXISTENT_DEVICE;

/*  INTEGER FUNCTION TF_DEVICE( DEVICE_TYPE, FILE_NAME, OPTION_STRING )
*/
int tf_device( device_type, file_name, options )
int VAR_PTR device_type;
struct dsc$descriptor_s *file_name, *options;
  {
    int counter,
        ret_value;
    char *dest, *src,
         *opt_str  = malloc( sizeof( char ) * ( options->dsc$w_length + 1 ) ),
         *file_str = malloc( sizeof( char ) * ( file_name->dsc$w_length + 1) );
    for( dest = opt_str, src = options->dsc$a_pointer, counter = 1;
         counter <= options->dsc$w_length; counter++ )
        *dest++ = *src++;
    *dest = '\000';
    if ( *(file_name->dsc$a_pointer) != ' ' && file_name->dsc$w_length > 0 )
      {
        for( dest = file_str, src = file_name->dsc$a_pointer, counter = 1;
             counter <= file_name->dsc$w_length; counter++ )
            *dest++ = *src++;
        *dest = '\000';
        ret_value = (int) t_device( *device_type, fopen( file_str, "w" ),
            opt_str );
      }
    else
        ret_value = (int) t_device( *device_type, stdout, opt_str );
    (void) free( file_str );
    (void) free( opt_str );
    if ( ret_value )
        current_device = *device_type;
    return ret_value;
  }


/*  LOGICAL FUNCTION TF_DUMP( FILE_NAME )
*/
int tf_dump( file_name )
struct dsc$descriptor_s * file_name;
  {
    char * src, * dest;
    char * file_str = malloc( sizeof( char ) * ( file_name->dsc$w_length + 1 ) );
    int counter, ret_value;
    for( src = file_name->dsc$a_pointer, dest = file_str, counter = 1;
         counter <= file_name->dsc$w_length; counter++ )
        *dest++ = *src++;
    *dest = '\000';
    ret_value = (int) (* device_ptr->t_DUMP)( file_str );
    (void) free( file_str );
    return VMS_LOGICAL( ret_value );
  }


/*  INTEGER FUNCTION TF_BEGIN( FILE_NAME, OPTION_STRING )
 *  CHARACTER FILE_NAME*(*), OPTION_STRING*(*)
 *
 *   returns the clip_area descriptor
*/
int tf_begin( file_name, options )
struct dsc$descriptor_s * file_name, * options;
  {
    int counter,
        ret_value;
    char * dest, * src,
         * opt_str = malloc( sizeof( char ) * ( options->dsc$w_length + 1 ) ),
         * file_str = malloc( sizeof( char ) * ( file_name->dsc$w_length + 1 ) );
    for( dest = opt_str, src = options->dsc$a_pointer, counter = 1;
         counter <= options->dsc$w_length; counter++ )
        *dest++ = *src++;
    *dest = '\000';
    if ( *(file_name->dsc$a_pointer) != ' ' && file_name->dsc$w_length > 0 )
      {
        for( dest = file_str, src = file_name->dsc$a_pointer, counter = 1;
             counter <= file_name->dsc$w_length; counter++ )
            *dest++ = *src++;
        *dest = '\000';
        ret_value = (int)
            (* device_ptr->t_BEGIN)( fopen( file_str, "w" ), opt_str );
      }
    else
        ret_value = (int) (* device_ptr->t_BEGIN)( stdout, opt_str );
    (void) free( opt_str );
    (void) free( file_str );
    return ret_value;
  }


/*  LOGICAL FUNCTION TF_END()
*/
int tf_end()
  {
    int ret_value;
    FILE * file_ptr;
    if ( current_device )
        file_ptr = t_register( T_REG_FILE, (FILE *) 0,
                               current_device, (char *) 0 );
    ret_value = (int) t_end();
    if ( file_ptr && file_ptr != stdout && file_ptr != stderr )
        (void) fclose( file_ptr );
    return VMS_LOGICAL( ret_value );
  }


/*  LOGICAL FUNCTION TF_VWPORT( CLIP_DESCRIPTOR )
 *  INTEGER CLIP_DESCRIPTOR
*/
int tf_vwport( clip_ptr )
int * clip_ptr;
  {
    return VMS_LOGICAL( (* device_ptr->t_VWPORT)( (clip_area *) clip_ptr ) );
  }


/*  LOGICAL FUNCTION TF_COLOUR( PEN_NUMBER )
 *  INTEGER PEN_NEUMBER
*/
int tf_colour( pen_number )
int *pen_number;
  {
    return VMS_LOGICAL( (* device_ptr->t_COLOUR)( *pen_number ) );
  }


/*  LOGICAL FUNCTION TF_DASHED( STYLE )
 *  INTEGER STYLE
*/
int tf_dashed( style )
int *style;
  {
    return VMS_LOGICAL( (* device_ptr->t_DASHED)( *style ) );
  }


/*  LOGICAL FUNCTION TF_THICK( WIDTH )
 *  INTEGER WIDTH
*/
int tf_thick( width )
int *width;
  {
    return VMS_LOGICAL( (* device_ptr->t_THICK)( *width ) );
  }


/*  LOGICAL FUNCTION TF_GR_OFF()
*/
int tf_gr_off()
  {
    return VMS_LOGICAL( (* device_ptr->t_GR_OFF)() );
  }


/*  LOGICAL FUNCTION TF_GR_ON()
*/
int tf_gr_on()
  {
    return VMS_LOGICAL( (* device_ptr->t_GR_ON)() );
  }


/*  LOGICAL FUNCTION TF_VISIB()
*/
int tf_visib()
  {
    return VMS_LOGICAL( (* device_ptr->t_VISIB)() );
  }


/*  LOGICAL FUNCTION TF_INVIS()
*/
int tf_invis()
  {
    return VMS_LOGICAL( (* device_ptr->t_INVIS)() );
  }


/*  LOGICAL FUNCTION TF_REVERS()
*/
int tf_revers()
  {
    return VMS_LOGICAL( (* device_ptr->t_REVERS)() );
  }


/*  LOGICAL FUNCTION TF_REPLAC()
*/
int tf_replac()
  {
    return VMS_LOGICAL( (* device_ptr->t_REPLAC)() );
  }


/*  LOGICAL FUNCTION TF_NEWPAG()
*/
int tf_newpag()
  {
    return VMS_LOGICAL( (* device_ptr->t_NEWPAG)() );
  }


/*  LOGICAL FUNCTION TF_NEWSCR()
*/
int tf_newscr()
  {
    return VMS_LOGICAL( (* device_ptr->t_NEWSCR)() );
  }


/*  LOGICAL FUNCTION TF_BELL()
*/
int tf_bell()
  {
    return VMS_LOGICAL( (* device_ptr->t_BELL)() );
  }


/*  LOGICAL FUNCTION TF_WAIT()
*/
int tf_wait()
  {
    return VMS_LOGICAL( (* device_ptr->t_WAIT)() );
  }


/*  LOGICAL FUNCTION TF_TXTMOV( X, Y )
 *  REAL X, Y
*/
int tf_txtmov( x, y )
float *x, *y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return VMS_LOGICAL( (* device_ptr->t_TXTMOV)( &point ) );
  }


/*  LOGICAL FUNCTION TF_TXTCUR( ROW, COLUMN )
 *  INTEGER ROW, COLUMN
*/
int tf_txtcur( row, column )
int *row, *column;
  {
    return VMS_LOGICAL( (* device_ptr->t_TXTCUR)( *row, *column ) );
  }


/*  LOGICAL FUNCTION TF_TXTSET( ATTRIBUTE, REAL_VALUE, STRING_VALUE )
 *  INTEGER ATTRIBUTE
 *  REAL    REAL_VALUE
 *  CHARACTER STRING_VALUE*(*)
*/
int tf_txtset( attribute, real_value, string_value )
int * attribute;
float * real_value;
struct dsc$descriptor_s * string_value;
  {
    if ( *attribute == T_CH_ALIGN )
      {
        int result;
        char new_str[3];
        char * dest = new_str;
        new_str[0] = *(string_value->dsc$a_pointer);
        new_str[1] = *((string_value->dsc$a_pointer)+1);
        new_str[2] = '\000';
        result = (int) (* device_ptr->t_TXTSET)( *attribute, new_str );
        (void) free( new_str );
        return VMS_LOGICAL( result );
      }
    else if ( *attribute == -T_CH_ALIGN )
        return VMS_LOGICAL(
          (* device_ptr->t_TXTSET)( *attribute, string_value->dsc$a_pointer ) );
    else
      {
        double value = *real_value;
        return VMS_LOGICAL(
            (* device_ptr->t_TXTSET)( *attribute, (char *) &value ) );
      }
  }


/*
 *  LOGICAL FUNCTION TF_SYMDRW( X, Y, SYMBOL )
 *  REAL X, Y
 *  CHARACTER SYMBOL *1
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_symdrw( x, y, symbol )
float * x, * y;
struct dsc$descriptor_s * symbol;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return VMS_LOGICAL(
        (* device_ptr->t_SYMDRW)( &point, *(symbol->dsc$a_pointer) ) );
  }


/*
 *  LOGICAL FUNCTION TF_TXTDRW( STRING )
 *  CHARACTER STRING *(*)
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_txtdrw( string )
/*
 *      it is necessary to copy the string to a new place and then append
 *      a NUL character to it.
*/
struct dsc$descriptor_s * string;
  {
    int counter, result;
    char * new_str = malloc( sizeof( char ) * ( string->dsc$w_length + 1 ) );
    char * dest, * src;
    for( dest = new_str, src = string->dsc$a_pointer, counter = 1;
         counter <= string->dsc$w_length; counter++ )
        *dest++ = *src++;
    *dest = '\000';
    result = (int) (* device_ptr->t_TXTDRW)( new_str );
    (void) free( new_str );
    return VMS_LOGICAL( result );
  }

/*
 *  INTEGER FUNCTION TF_CH_DEF( WHAT, CHAR, INDEX, X, Y )
 *  INTEGER WHAT
 *  CHARACTER CHAR*1
 *  INTEGER INDEX
 *  REAL X, Y
*/
int tf_ch_def( what, charact, index, x, y )
/*
 * This function allows the user to obtain the shape of any character defined
 * in the ASCII and SYMBOL fonts. With the exception of string alignment, the
 * transformations implied by t_txtset are applied to the returned coordinates.
 *
 * The returned values are
 *  T_FT_SIZE       X & Y   ( width & height of characters)
 *  T_FT_EXIST      T_CH_DEF
 *  T_FT_LENGTH     T_CH_DEF
 *  T_FT_FIRST      T_CH_DEF
 *  T_FT_LAST       T_CH_DEF
 *  T_FT_MIDDLE     T_CH_DEF
 *  T_FT_POINT      X & Y   ( coordinate of a point )
 */
int  * what;         /* specifies what to do (T_FT_* defined values) */
struct dsc$descriptor_s * charact;    /* character to fetch from the font */
int  * index;        /* coordinate number   */
float * x, * y;            /* returned coordinate */
  {
    coord_pair point;
    int result = (int) (* device_ptr->t_CH_DEF)( *what,
            *(charact->dsc$a_pointer), *index, &point );
    if ( *what == T_FT_SIZE || *what == T_FT_POINT )
      {
        *x = point._x;
        *y = point._y;
      }
    return result;
  }


/*
 *  LOGICAL FUNCTION TF_TXTGET( STRING, MAX_CHARS )
 *  CHARACTER STRING *(*)
 *  INTEGER MAX_CHARS
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_txtget( string, max_chars, str_len )
struct dsc$descriptor_s * string;
int    * max_chars;
  {
    char *dest, *src, *end_dest, *c_str;
    int  result;
    unsigned int max_len = *max_chars <= string->dsc$w_length ?
                               *max_chars : string->dsc$w_length;

    c_str = malloc( sizeof( char ) * ( max_len + 1 ) );
    result = (int) (* device_ptr->t_TXTGET)( c_str, max_chars );
    end_dest = string->dsc$a_pointer + string->dsc$w_length;
    for ( dest = string->dsc$a_pointer, src = c_str; *src && dest < end_dest; )
        *dest++ = *src++;
    while ( dest < end_dest )
        *dest++ = ' ';
    (void) free( c_str );
    return VMS_LOGICAL( result );
  }


/*
 *  LOGICAL FUNCTION TF_TXTPUT( STRING )
 *  CHARACTER STRING *(*)
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_txtput( string )
/*
 *      it is necessary to copy the string to a new place and then append
 *      a NUL character to it.
*/
struct dsc$descriptor_s * string;
  {
    int counter, result;
    char * dest, * src;
    char * new_str = malloc( sizeof( char ) * ( string->dsc$w_length + 1 ) );
    for( dest = new_str, src = string->dsc$a_pointer, counter = 1;
         counter <= string->dsc$w_length; counter++ )
        *dest++ = *src++;
    *dest = '\000';
    result = (int) (* device_ptr->t_TXTPUT)( new_str );
    (void) free( new_str );
    return VMS_LOGICAL( result );
  }


/*
 *  LOGICAL FUNCTION TF_BOXFIL( LEFT, BOTTOM, RIGHT, TOP, STYLE )
 *  REAL LEFT, BOTTOM, RIGHT, TOP
 *  INTEGER STYLE
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_boxfil( left, bottom, right, top, style )
float * left, * bottom, * right, * top;
int *style;
  {
    coord_pair lower_left, upper_right;
    lower_left._x = *left;
    lower_left._y = *bottom;
    upper_right._x = *right;
    upper_right._y = *top;
    return VMS_LOGICAL(
        (* device_ptr->t_BOXFIL)( &lower_left, &upper_right, *style ) );
  }


/*
 *  LOGICAL FUNCTION TF_XHRGET( X, Y, CHAR_PRESSED )
 *  REAL X, Y
 *  CHARACTER CHAR_PRESSED *1
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_xhrget( x, y, char_pressed )
float * x, * y;
struct dsc$descriptor_s * char_pressed;
  {
    int result;
    coord_pair point;
    char *dest;
    result = (int)
        (* device_ptr->t_XHRGET)( &point, char_pressed->dsc$a_pointer );
    *x = point._x;
    *y = point._y;

    dest = char_pressed->dsc$a_pointer + 1;
    while ( dest < char_pressed->dsc$a_pointer + char_pressed->dsc$w_length )
        *dest++ = ' ';
    return VMS_LOGICAL( result );
  }


/*
 *  LOGICAL FUNCTION TF_XHRPUT( X, Y )
 *  REAL X, Y
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_xhrput( x, y )
float * x, * y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return VMS_LOGICAL( (* device_ptr->t_XHRPUT)( &point ) );
  }


/*
 *  LOGICAL FUNCTION TF_STATUS( ATTRIBUTE, X, Y, STRING )
 *  INTEGER ATTRIBUTE
 *  REAL    X, Y
 *  CHARACTER string*(*)
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_status( attribute, x, y, string )
int *attribute;
float *x, *y;
struct dsc$descriptor_s *string;
  {
    if ( *attribute == T_ST_DEVICE )
      {
        int result;
        char *dest, *src, *end_dest;
        char *c_str = malloc( sizeof( char ) * ( string->dsc$w_length + 1 ) );
        result = (int) (* device_ptr->t_STATUS)( *attribute,
               string->dsc$a_pointer );
        end_dest = string->dsc$a_pointer + string->dsc$w_length;
        for ( dest = string->dsc$a_pointer, src = c_str;
              *src && dest < end_dest; )
            *dest++ = *src++;
        while ( dest < end_dest )
            *dest++ = ' ';
        (void) free( c_str );
        return VMS_LOGICAL( result );
      }
    else if ( *attribute == T_ST_SCREEN || *attribute == T_ST_CATEG ||
              *attribute == T_ST_MONO  )
      {
        int i_result, j_result;
        j_result = (int) (* device_ptr->t_STATUS)( *attribute, &i_result );
        *y = *x = (float) i_result;
        return VMS_LOGICAL( j_result );
      }
    else
      {
        coord_pair point;
        int result;
        result = (int) (* device_ptr->t_STATUS)( *attribute, (char *) &point );
        *x = point._x;
        *y = point._y;
        return VMS_LOGICAL( result );
      }
  }


/*
 *  LOGICAL FUNCTION TF_PR_TER()
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_pr_ter()
  {
    return VMS_LOGICAL( (* device_ptr->t_PR_TER)( stdout ) );
  }


/*
 *  LOGICAL FUNCTION TF_VECMOV( X, Y )
 *  REAL X, Y
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_vecmov( x, y )
float *x, *y;
  {
    coord_pair point;
    point._x = * x;
    point._y = * y;
    return VMS_LOGICAL( (* device_ptr->t_VECMOV)( &point ) );
  }


/*
 *  LOGICAL FUNCTION TF_VECDRW( X, Y )
 *  REAL X, Y
 *
 *   returns    .TRUE.  -- for success
 *              .FALSE. -- for failure
*/
int tf_vecdrw( x, y )
float * x, * y;
  {
    coord_pair point;
    point._x = *x;
    point._y = *y;
    return VMS_LOGICAL( (* device_ptr->t_VECDRW)( &point ) );
  }


