/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 *          Binding functions for using device drivers from
 *                        Microsoft FORTRANv4
 *
 *  The names of the functions called from the FORTRAN are the same as
 *  the names of the original C source functions . However the arguments
 *  may be different, according to the following rules .
 *
 *      +-------------------------------+----------------------------+
 *      |  C argument type              | FORTRAN argument type      |
 *      +-------------------------------+----------------------------+
 *      | int i;                        | integer i                  |
 *      | coord_pair point;             | real x, y                  |
 *      | char *string;                 | character string*(*)       |
 *      | double *value;                | real value                 |
 *      | clip_area *viewport           | integer viewport           |
 *      | double * OR char * value;     | real real_value            |
 *      |                               | character string_value*(*) |
 *      | coord_pair * OR char * value; | real x, y                  |
 *      |                               | character string_value*(*) |
 *      +-------------------------------+----------------------------+
 *
 *  Sometimes a pointer argument may be able to point to more than one kind
 *  of object. In these cases, one argument in the C source becomes 2 or more
 *  arguments in the binding function.
 *
 *  LOGICAL FUNCTIONS return values of
 *    .TRUE.  -- for success
 *    .FALSE. -- for failure
 *
 *  If the returned value of the C source is a pointer, the binding function
 *  presents the returned value as an INTEGER . Because this essentially
 *  a kludge, the user must not change the returned value if he wishes to
 *  make use of it, and should not attempt to use it as an integer .
 *
*/

#include "system.h"             /* system dependent stuff */
#include <stdio.h>
#include DEVICE_G_HEADER

int strlen();


/*  INTEGER *2 FUNCTION TFF_DEVICE( DEVICE_TYPE, FILE_NAME, OPTION_STRING )
 *  INTEGER *2 DEVICE_TYPE [value]
 *  CHARACTER  FILE_NAME*64 [reference],
 *             OPTION_STRING*128 [reference]
*/
int tff_device( device_type, file_name, options )
int device_type;
char VAR_PTR file_name, VAR_PTR options;
  {
    FILE * file_ptr;
    file_ptr = *file_name ? (FILE *) fopen( file_name, "w" ) :
                            (FILE *) stdout;
    (void) t_device( device_type, file_ptr, T_NOT_BEGIN );
    return (int) t_begin( file_ptr, options );
  }


/*  INTEGER *2 FUNCTION TFF_DUMP( FILE_NAME )
 *  CHARACTER FILE_NAME*64
*/

int tff_dump( file_name )
char VAR_PTR file_name;
  {
    return (int) (* device_ptr->t_DUMP)( file_name );
  }


/*  INTEGER *2 FUNCTION TFF_BEGIN( FILE_NAME, OPTION_STRING )
 *  CHARACTER FILE_NAME*64, OPTION_STRING*128
 *
 *   returns the clip_area descriptor
*/
int tff_begin( file_name, options )
char VAR_PTR file_name, VAR_PTR options;
  {
    return (int) (* device_ptr->t_BEGIN)(
        ( *file_name ? (FILE *) fopen( file_name, "w" ) : (FILE *) stdout ),
        options );
  }


/*  INTEGER *2 FUNCTION TFF_END()
*/
int tff_end()
  {
    return (* device_ptr->t_END)();
  }


/*  INTEGER *2 FUNCTION TFF_VWPORT( CLIP_DESCRIPTOR )
 *  INTEGER CLIP_DESCRIPTOR
*/
int tff_vwport( clip_ptr )
clip_area VAR_PTR clip_ptr;
  {
    return (* device_ptr->t_VWPORT)( clip_ptr );
  }


/*  INTEGER *2 FUNCTION TFF_COLOUR( PEN_NUMBER )
 *  INTEGER *2 PEN_NUMBER
*/
int tff_colour( pen_number )
int pen_number;
  {
    return (* device_ptr->t_COLOUR)( pen_number );
  }


/*  INTEGER *2 FUNCTION TFF_DASHED( STYLE )
 *  INTEGER *2 STYLE
*/
int tff_dashed( style )
int style;
  {
    return (* device_ptr->t_DASHED)( style );
  }


/*  INTEGER *2 FUNCTION TFF_THICK( WIDTH )
 *  INTEGER *2 WIDTH
*/
int tff_thick( width )
int width;
  {
    return (* device_ptr->t_THICK)( width );
  }


/*  INTEGER *2 FUNCTION TFF_GR_OFF()
*/
int tff_gr_off()
  {
    return (* device_ptr->t_GR_OFF)();
  }


/*  INTEGER *2 FUNCTION TFF_GR_ON()
*/
int tff_gr_on()
  {
    return (* device_ptr->t_GR_ON)();
  }


/*  INTEGER *2 FUNCTION TFF_VISIB()
*/
int tff_visib()
  {
    return (* device_ptr->t_VISIB)();
  }


/*  INTEGER *2 FUNCTION TFF_INVIS()
*/
int tff_invis()
  {
    return (* device_ptr->t_INVIS)();
  }


/*  INTEGER *2 FUNCTION TFF_REVERS()
*/
int tff_revers()
  {
    return (* device_ptr->t_REVERS)();
  }


/*  INTEGER *2 FUNCTION TFF_REPLAC()
*/
int tff_replac()
  {
    return (* device_ptr->t_REPLAC)();
  }


/*  INTEGER *2 FUNCTION TFF_NEWPAG()
*/
int tff_newpag()
  {
    return (* device_ptr->t_NEWPAG)();
  }


/*  INTEGER *2 FUNCTION TFF_NEWSCR()
*/
int tff_newscr()
  {
    return (* device_ptr->t_NEWSCR)();
  }


/*  INTEGER *2 FUNCTION TFF_BELL()
*/
int tff_bell()
  {
    return (* device_ptr->t_BELL)();
  }


/*  INTEGER *2 FUNCTION TFF_WAIT()
*/
int tff_wait()
  {
    return (* device_ptr->t_WAIT)();
  }


/*  INTEGER *2 FUNCTION TFF_TXTMOV( POINT )
 *  DOUBLE PRECISION POINT(2) [reference]
*/
int tff_txtmov( point )
coord_pair *point;
  {
    return (* device_ptr->t_TXTMOV)( point );
  }


/*  INTEGER *2 FUNCTION TFF_TXTCUR( ROW, COLUMN )
 *  integer *2 row, column [value]
*/
int tff_txtcur( row, column )
int row, column;
  {
    return (* device_ptr->t_TXTCUR)( row, column );
  }


/*  INTEGER *2 FUNCTION TFF_TXTSET( ATTRIBUTE, VALUE, STRING )
 *  integer *2 attribute [value]
 *  double precision value [reference]
 *  character  string*3 [reference]
*/
int tff_txtset( attribute, value, string )
int attribute;
double VAR_PTR value;
char VAR_PTR string;
  {
    return (int) (* device_ptr->t_TXTSET)( attribute, 
        ( ( attribute == T_CH_ALIGN || attribute == -T_CH_ALIGN ) ?
          string : (char VAR_PTR) value ) );
  }


/*  INTEGER *2 FUNCTION TFF_SYMDRW( POINT, SYMBOL )
 *  double precision point(2) [reference]
 *  integer *2 symbol [value]
*/
int tff_symdrw( point, symbol )
coord_pair VAR_PTR point;
int symbol;
  {
    return (int) (* device_ptr->t_SYMDRW)( point, symbol );
  }


/*
 *  INTEGER *2 FUNCTION TFF_TXTDRW( STRING )
 *  character string*128 [reference]
*/
int tff_txtdrw( string )
char VAR_PTR string;
  {
    return (int) (* device_ptr->t_TXTDRW)( string );
  }


/*  INTEGER *2 FUNCTION TFF_CH_DEF( WHAT, CHARACT, IINDEX, POINT )
 *  integer *2 what [value], charact [value], iindex [value]
 *  double precision point(2) [reference]
*/
int tff_ch_def( what, charact, index, point )
int what, charact, index;
coord_pair VAR_PTR point;
  {
    return (int) (* device_ptr->t_CH_DEF)( what, charact, index, point );
  }


/*
 *  INTEGER *2 FUNCTION TFF_TXTGET( STRING, MAX_CHARS )
 *  character string*128
 *  integer *2 max_chars
*/
int tff_txtget( string, max_chars )
char VAR_PTR string;
int VAR_PTR max_chars;
  {
    int result = (int) t_txtget( string, *max_chars );
    *max_chars = strlen( string );
    return result;
  }


/*  INTEGER *2 FUNCTION TF_TXTPUT( STRING )
 *  CHARACTER STRING*128 [reference]
*/
int tff_txtput( string )
char VAR_PTR string;
  {
    return (int) (* device_ptr->t_TXTPUT)( string );
  }


/*  INTEGER *2 FUNCTION TFF_BOXFIL( LOWER_LEFT, UPPER_RIGHT, STYLE )
 *  double precision lower_left(2) [reference], upper_right(2) [reference]
 *  integer *2 style [value]
*/
int tff_boxfil( lower_left, upper_right, style )
coord_pair  VAR_PTR lower_left, VAR_PTR upper_right;
int style;
  {
    return (int) (* device_ptr->t_BOXFIL)( lower_left, upper_right, style );
  }


/*  INTEGER *2 FUNCTION TFF_XHRGET( POINT, CHAR_PRESSED )
 *  double precision point(2) [reference]
 *  character char_pressed*1
*/
int tff_xhrget( point, char_pressed )
coord_pair VAR_PTR point;
char VAR_PTR char_pressed;
  {
    return (int) (* device_ptr->t_XHRGET)( point, char_pressed );
  }


/*  INTEGER *2 FUNCTION TFF_XHRPUT( POINT )
 *  double precision point(2) [reference]
*/
int tff_xhrput( point )
coord_pair VAR_PTR point;
  {
    return (* device_ptr->t_XHRPUT)( point );
  }


/*  INTEGER *2 FUNCTION TFF_STATUS( ATTRIBUTE, STATUS, POINT, STRING, STR_LEN )
 *  integer *2 attribute [value]
 *  integer *2 status    [reference]
 *  double precision point(2) [reference]
 *  character string*128 [reference]
 *  integer *2 str_len   [reference]
*/
int tff_status( attribute, status, point, string, str_len )
int attribute,
    VAR_PTR status;
coord_pair VAR_PTR point;
char  VAR_PTR string;
int   VAR_PTR str_len;
  {
    int result;
    if ( attribute == T_ST_DEVICE )
      {
        result = (int) (* device_ptr->t_STATUS)( attribute, string );
        *str_len = strlen( string );
      }
    else if ( attribute == T_ST_SCREEN || attribute == T_ST_CATEG ||
              attribute == T_ST_MONO   || attribute == T_ST_CLIP )
        result = (int) (* device_ptr->t_STATUS)( attribute,
            (char VAR_PTR) status );
    else
        result = (int) (* device_ptr->t_STATUS)( attribute, (char *) point );
    return result;
  }


/*  INTEGER *2 FUNCTION TF_PR_TER()
 *
*/
int tff_pr_ter()
  {
    return (int) (* device_ptr->t_PR_TER)( stdout );
  }


/*  INTEGER *2 FUNCTION TFF_VECMOV( POINT )
 *  double precision point(2) [reference]
*/
int tff_vecmov( point )
coord_pair VAR_PTR point;
  {
    return (int) (* device_ptr->t_VECMOV)( point );
  }


/*  INTEGER *2 FUNCTION TFF_VECDRW( POINT )
 *  double precision point(2) [reference]
*/
int tff_vecdrw( point )
coord_pair VAR_PTR point;
  {
    return (int) (* device_ptr->t_VECDRW)( point );
  }


