c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

c
c          Binding functions for using device drivers from
c                        Microsoft FORTRANv4
c
c  The names of the functions called from the FORTRAN are the same as
c  the names of the original C source functions . However the arguments
c  may be different, according to the following rules .
c
c      +-------------------------------+----------------------------+
c      |  C argument type              | FORTRAN argument type      |
c      +-------------------------------+----------------------------+
c      | int i;                        | integer i                  |
c      | coord_pair point;             | real x, y                  |
c      | char *string;                 | character string*(*)       |
c      | double *value;                | real value                 |
c      | clip_area *viewport           | integer viewport           |
c      | double * OR char * value;     | real real_value            |
c      |                               | character string_value*(*) |
c      | coord_pair * OR char * value; | real x, y                  |
c      |                               | character string_value*(*) |
c     +-------------------------------+----------------------------+
c
c  Sometimes a pointer argument may be able to point to more than one kind
c  of object. In these cases, one argument in the C source becomes 2 or more
c  arguments in the binding function.
c
c  LOGICAL FUNCTIONS return values of
c    .TRUE.  -- for success
c    .FALSE. -- for failure
c
c  If the returned value of the C source is a pointer, the binding function
c  presents the returned value as an INTEGER . Because this essentially
c  a kludge, the user must not change the returned value if he wishes to
c  make use of it, and should not attempt to use it as an integer .
c
c  N.B. These binding functions depend upon the nature of the FORTRAN
c       implementation of parameter passing . This is different for
c       nearly every `brand' of FORTRAN, and so these bindings are only
c       supposed to work with SUN FORTRAN . For IBM PC compatibles, the
c       Microsoft C and Fortran compilers are used and two sets of binding
c       routines are used :
c        1) top set in Fortran (called tf_* ) set up the arguments and 
c           retrieve the return values as required by C .
c        2) the bottom set in C ( called tff_* ) open files and call 
c           the driver function from the driver functions array
c


$notruncate
      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_DEVICE [c,alias:'_tff_device']
     2        ( C_DEVICE, C_FILE, C_OPTION )
      integer *2 c_device [value]
      character  c_file   [reference] *64,
     1           c_option [reference] *128
      end


      INTEGER FUNCTION TF_DEVICE( DEVICE_TYPE, FILE_NAME,
     1                            OPTION_STRING )
$include:'stdio.def'
      integer device_type
      character file_name*(*), option_string*(*)
      character c_file*64, c_option*128
      integer last_char
      integer *2 tff_device
c
      c_option = option_string
      if ( option_string .eq. ' ' ) then
          last_char = 1
      else
          last_char = min( len( option_string ) + 1, len( c_option ) )
          end if
      c_option(last_char:last_char) = char( 0 )
      if ( file_name .ne. ' ' .and. len( file_name ) .gt. 0 ) then
          c_file = file_name
          last_char = min( len( file_name ) + 1, len( c_file ) )
          c_file(last_char:last_char) = char( 0 )
      else
          c_file(1:1) = char(0)
          end if
      tf_device = tff_device( int2( device_type ), c_file, c_option )
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_DUMP [c,alias:'_tff_dump']( C_FILE )
      character c_file [reference]*64
      end


      LOGICAL FUNCTION TF_DUMP( FILE_NAME )
$include:'stdio.def'
      character file_name*(*)
      character c_file*64
      integer last_char
      integer *2 tff_dump
c
      c_file = file_name
      if ( file_name .eq. ' ' ) then
          last_char = 1
      else
          last_char = min( len( file_name ) + 1, len( c_file ) )
          end if
      c_file(last_char:last_char) = char( 0 )
      tf_dump = tff_dump( c_file ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_BEGIN[c,alias:'_tff_begin'] ( C_FILE, C_OPTION )
      character c_file   [reference]*64,
     1          c_option [reference]*128
      end


      INTEGER FUNCTION TF_BEGIN( FILE_NAME, OPTION_STRING )
$include:'stdio.def'
      character file_name*(*), option_string*(*)
      character c_file*64, c_option*64
      integer last_char
      integer *2 tff_begin
c
      c_option = option_string
      if ( option_string .eq. ' ' ) then
          last_char = 1
      else
          last_char = min( len( option_string ) + 1, len( c_option ) )
          end if
      c_option(last_char:last_char) = char( 0 )
      if ( file_name .ne. ' ' .and. len( file_name ) .gt. 0 ) then
          c_file = file_name
          last_char = min( len( file_name ) + 1, len( c_file ) )
          c_file(last_char:last_char) = char( 0 )
      else
          c_file(1:1) = char(0)
          end if
      tf_begin = tff_begin( c_file, c_option )
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_END[c,alias:'_tff_end'] ()
      end


      LOGICAL FUNCTION TF_END()
$include:'stdio.def'
      integer *2 tff_end
      tf_end = tff_end() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_VWPORT[c,alias:'_tff_vwport'] ( CLIP_DESCRIPTOR )
      integer clip_descriptor [value]
      end


      LOGICAL FUNCTION TF_VWPORT( CLIP_DESCRIPTOR )
$include:'stdio.def'
      integer clip_descriptor
      integer *2 tff_vwport
      tf_vwport = tff_vwport( clip_descriptor ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_COLOUR[c,alias:'_tff_colour'] ( COLOUR )
      integer *2 colour [value]
      end


      LOGICAL FUNCTION TF_COLOUR( PEN_NUMBER )
$include:'stdio.def'
      integer pen_number
      integer *2 tff_colour
      tf_colour = tff_colour( int2( pen_number ) ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_DASHED[c,alias:'_tff_dashed'] ( STYLE )
      integer *2 style [value]
      end


      LOGICAL FUNCTION TF_DASHED( STYLE )
$include:'stdio.def'
      integer style
      integer *2 tff_dashed
      tf_dashed = tff_dashed( int2( style ) ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_THICK[c,alias:'_tff_thick'] ( WIDTH )
      integer *2 width [value]
      end


      LOGICAL FUNCTION TF_THICK( WIDTH )
$include:'stdio.def'
      integer width
      integer *2 tff_thick
      tf_thick = tff_thick( int2( width ) ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_GR_OFF[c,alias:'_tff_gr_off'] ()
      end


      LOGICAL FUNCTION TF_GR_OFF()
$include:'stdio.def'
      integer *2 tff_gr_off
      tf_gr_off = tff_gr_off() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_GR_ON[c,alias:'_tff_gr_on'] ()
      end


      LOGICAL FUNCTION TF_GR_ON()
$include:'stdio.def'
      integer *2 tff_gr_on
      tf_gr_on = tff_gr_on() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_VISIB[c,alias:'_tff_visib'] ()
      end


      LOGICAL FUNCTION TF_VISIB()
$include:'stdio.def'
      integer *2 tff_visib
      tf_visib = tff_visib() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_INVIS[c,alias:'_tff_invis'] ()
      end


      LOGICAL FUNCTION TF_INVIS()
$include:'stdio.def'
      integer *2 tff_invis
      tf_invis = tff_invis() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_REVERS[c,alias:'_tff_revers'] ()
      end


      LOGICAL FUNCTION TF_REVERS()
$include:'stdio.def'
      integer *2 tff_revers
      tf_revers = tff_revers() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_REPLAC[c,alias:'_tff_replac'] ()
      end


      LOGICAL FUNCTION TF_REPLAC()
$include:'stdio.def'
      integer *2 tff_replac
      tf_replac = tff_replac() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_NEWPAG[c,alias:'_tff_newpag'] ()
      end


      LOGICAL FUNCTION TF_NEWPAG()
$include:'stdio.def'
      integer *2 tff_newpag
      tf_newpag = tff_newpag() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_NEWSCR[c,alias:'_tff_newscr'] ()
      end


      LOGICAL FUNCTION TF_NEWSCR()
$include:'stdio.def'
      integer *2 tff_newscr
      tf_newscr = tff_newscr() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_BELL[c,alias:'_tff_bell'] ()
      end


      LOGICAL FUNCTION TF_BELL()
$include:'stdio.def'
      integer *2 tff_bell
      tf_bell = tff_bell() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_WAIT[c,alias:'_tff_wait'] ()
      end


      LOGICAL FUNCTION TF_WAIT()
$include:'stdio.def'
      integer *2 tff_wait
      tf_wait = tff_wait() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_TXTMOV[c,alias:'_tff_txtmov'] ( POINT )
      double precision point[reference](2)
      end


      LOGICAL FUNCTION TF_TXTMOV( X, Y )
$include:'stdio.def'
      integer *2 tff_txtmov
      real x, y
      double precision point(2)
      point(1) = x
      point(2) = y
      tf_txtmov = tff_txtmov( point ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_TXTCUR[c,alias:'_tff_txtcur'] ( ROW, COLUMN )
      integer *2 row    [value],
     1           column [value]
      end


      LOGICAL FUNCTION TF_TXTCUR( ROW, COLUMN )
$include:'stdio.def'
      integer row, column
      integer *2 tff_txtcur
      tf_txtcur = tff_txtcur( int2( row ), int2( column ) ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_TXTSET[c,alias:'_tff_txtset'] ( ATTRIBUTE, VALUE, STRING )
      integer *2       attribute [value]
      double precision value     [reference]
      character        string    [reference] *3
      end


      LOGICAL FUNCTION TF_TXTSET( ATTRIBUTE, VALUE, STRING )
$include:'stdio.def'
$include:'f77_g.con'
      integer attribute
      real    value
      double precision c_value
      character string*(*), c_string*3
      integer *2 tff_txtset

      c_string(1:2) = string
      c_string(3:3) = char( 0 )
      c_value = value
      tf_txtset = tff_txtset( attribute, c_value, c_string ) .gt. 0
      if ( attribute .eq. -T_CH_ALIGN ) then
          string = c_string(1:2)
      else
          value = c_value
          end if
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_SYMDRW[c,alias:'_tff_symdrw'] ( POINT, SYMBOL )
      double precision point  [reference] (2)
      integer *2       symbol [value]
      end


      LOGICAL FUNCTION TF_SYMDRW( X, Y, SYMBOL )
$include:'stdio.def'
      real x, y
      character symbol*1
      double precision point(2)
      integer *2 tff_symdrw
      point(1) = x
      point(2) = y
      tf_symdrw = tff_symdrw( point, int2( ichar( symbol ) ) ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_TXTDRW[c,alias:'_tff_txtdrw'] ( STRING )
      character string[reference]*128
      end


      LOGICAL FUNCTION TF_TXTDRW( STRING )
$include:'stdio.def'
      character string *(*), c_string*128
      integer last_char
      integer *2 tff_txtdrw
c
      c_string = string
      last_char = min( len( string ) + 1, len( c_string ) )
      c_string(last_char:last_char) = char( 0 )
      tf_txtdrw = tff_txtdrw( c_string ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_CH_DEF[c,alias:'_tff_ch_def']
     2        ( WHAT, CHARACT, IINDEX, POINT )
      integer *2       what    [value],
     1                 charact [value],
     2                 iindex  [value]
      double precision point   [reference] (2)
      end


      INTEGER FUNCTION TF_CH_DEF( WHAT, CHARACT, IINDEX, X, Y )
$include:'stdio.def'
$include:'f77_g.con'
      integer what
      character charact*1
      integer iindex
      real x, y
      double precision point(2)
      integer *2 tff_ch_def
c
      tf_ch_def = tff_ch_def( int2( what ), int2( ichar( charact ) ),
     1                        int2( iindex ), point )
      if ( what .eq. T_FT_SIZE .or. what .eq. T_FT_POINT ) then
          x = point(1)
          y = point(2)
          end if
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_TXTGET[c,alias:'_tff_txtget'] ( STRING, MAX_CHARS )
      character  string    [reference] *128
      integer *2 max_chars [reference]
      end


      LOGICAL FUNCTION TF_TXTGET( STRING, MAX_CHARS )
$include:'stdio.def'
      character string*(*), c_string*128
      integer max_chars
      integer *2 n_chars, tff_txtget
c
      n_chars = min( max_chars, len( string ), len( c_string ) - 1 )
      tf_txtget = tff_txtget( c_string, n_chars ) .gt. 0
      if ( n_chars .lt. 0 ) n_chars = 0
      string = c_string
      if ( n_chars .lt. len( string ) ) string(n_chars+1:) = ' '
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_TXTPUT[c,alias:'_tff_txtput'] ( STRING )
      character string[reference]*128
      end


      LOGICAL FUNCTION TF_TXTPUT( STRING )
$include:'stdio.def'
      character string*(*), c_string*128
      integer last_char
      integer *2 tff_txtput
c
      c_string = string
      last_char = min( len( string ) + 1, len( c_string ) )
      c_string(last_char:last_char) = char( 0 )
      tf_txtput = tff_txtput( c_string ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_BOXFIL[c,alias:'_tff_boxfil']
     2        ( LOWER_LEFT, UPPER_RIGHT, STYLE )
      double precision lower_left  [reference] (2),
     1                 upper_right [reference] (2)
      integer *2       style       [value]
      end


      LOGICAL FUNCTION TF_BOXFIL( LEFT, BOTTOM, RIGHT, TOP, STYLE )
$include:'stdio.def'
      real left, bottom, right, top
      integer style
      double precision lower_left(2), upper_right(2)
      integer *2 tff_boxfil
c
      lower_left(1) = left
      lower_left(2) = bottom
      upper_right(1) = right
      upper_right(2) = top
      tf_boxfil = tff_boxfil( lower_left, upper_right, int2( style ) )
     1            .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_XHRGET[c,alias:'_tff_xhrget'] ( POINT, CHAR_PRESSED )
      double precision point        [reference] (2)
      character        char_pressed [reference] *1
      end


      LOGICAL FUNCTION TF_XHRGET( X, Y, CHAR_PRESSED )
$include:'stdio.def'
      real x, y
      character char_pressed*(*)
      double precision point(2)
      integer *2 tff_xhrget
c
      tf_xhrget = tff_xhrget( point, char_pressed ) .gt. 0
      x = point(1)
      y = point(2)
      if ( len( char_pressed ) .gt. 1 ) char_pressed(2:) = ' '
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_XHRPUT[c,alias:'_tff_xhrput'] ( POINT )
      double precision point[reference](2)
      end


      LOGICAL FUNCTION TF_XHRPUT( X, Y )
$include:'stdio.def'
      real x, y
      integer *2 tff_xhrput
      double precision point(2)

      point(1) = x
      point(2) = y
      tf_xhrput = tff_xhrput( point ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_STATUS[c,alias:'_tff_status']
     2        ( ATTRIBUTE, STATUS, POINT, STRING, STR_LEN )
      integer *2       attribute [value],
     1                 status    [reference]
      double precision point     [reference] (2)
      character        string    [reference] *128
      integer *2       str_len   [reference]
      end


      LOGICAL FUNCTION TF_STATUS( ATTRIBUTE, STATUS, X, Y, STRING )
$include:'stdio.def'
$include:'f77_g.con'
      integer attribute, status
      real    x, y
      character string*(*), c_string*128
      double precision point(2)
      integer *2 str_len, stat2, tff_status

      tf_status = tff_status( attribute, stat2, point,
     1                        c_string, str_len ) .ne. 0
      if ( attribute .eq. T_ST_DEVICE ) then
          string = c_string
          if ( str_len .lt. len( string ) ) string(str_len+1:) = ' '
      else if ( attribute .eq. T_ST_SCREEN .or.
     1          attribute .eq. T_ST_CATEG  .or.
     2          attribute .eq. T_ST_MONO   .or.
     3          attribute .eq. T_ST_CLIP   ) then
          status = stat2
      else
          x = point(1)
          y = point(2)
          end if
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_PR_TER[c,alias:'_tff_pr_ter'] ()
      end


      LOGICAL FUNCTION TF_PR_TER()
$include:'stdio.def'
      integer *2 tff_pr_ter
      tf_pr_ter = tff_pr_ter() .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_VECMOV[c,alias:'_tff_vecmov'] ( POINT )
      double precision point[reference](2)
      end


      LOGICAL FUNCTION TF_VECMOV( X, Y )
$include:'stdio.def'
      real x, y
      double precision point(2)
      integer *2 tff_vecmov

      point(1) = x
      point(2) = y
      tf_vecmov = tff_vecmov( point ) .gt. 0
      return
      end


      INTERFACE TO INTEGER *2 FUNCTION
     1    TFF_VECDRW[c,alias:'_tff_vecdrw'] ( POINT )
      double precision point[reference](2)
      end


      LOGICAL FUNCTION TF_VECDRW( X, Y )
$include:'stdio.def'
      real x, y
      double precision point(2)
      integer *2 tff_vecdrw

      point(1) = x
      point(2) = y
      tf_vecdrw = tff_vecdrw( point ) .gt. 0
      return
      end

