/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ************************************************************************ */
/*                                                                          */
/*                      Device Selection function                           */
/*                                                                          */
/* ************************************************************************ */
#include "system.h"
#include <stdio.h>
#define  T_DEVICE_SOURCE_CODE
#include DEVICE_G_HEADER

char * malloc();
T_dev_fns VAR_PTR device_ptr;

FILE * t_device( device_type, device, options )
/*
 * This function is used to select the device for graphics output and start the
 * graphics by calling ``t_begin()''. ``device_type'' is one of
 *    VISUAL500_TERMINAL, VISUAL550_TERMINAL, XTERM_VT100TEK4014,
 *    HEWLETT_GL_PLOTTER, POSTSCRIPT_PRINTER, DUMMY_GRAPHICS_DEV,
 *    DOT_MATRIX_PRINTER                                        -- unavailable
 *    HERCULES_GRAPHICS,                  --- #define IBM_PC
 *    ENHANCED_GRAPHICS,                  --- #define IBM_PC    -- unavailable
 *    SUN_3_WORKSTATION,                  --- #define SUN3_WORKSTATION
 *    X11_WINDOWS,                        --- #define X_WINDOWS -- unavailable
 * Some of these drivers are restricted to particular hardware and are
 * included by defining one (or more) of the symbols IBM_PC, SUN3_WORKSTATION,
 * X_WINDOWS etc .
 * ``device'' is the pointer to the graphics device or output file, or in the
 * case of a window environment is a unique identifying handle for the graphics
 * window . The returned value of ``t_device()' is usually ``device'', but if
 * ``device'' is a null pointer and
 *   1) the graphics device is a window in a windowing environment or the
 *      graphics hardware is driven DIRECTLY by the driver ``t_device()''
 *      creates and returns a unique value of ``device'',
 *   2) the graphics device generates output to a file, graphics is sent to
 *     ``stdout'' and the value ``stdout'' is returned,
 * to be used as a device identifier in future calls to ``t_device()'' .
 * ``options'' is a character string of device dependent options . If
 * ``options'' is the value T_SELECT_ONLY or T_NOT_BEGIN, ``t_device()'' only
 * selects the device and does not start the graphics (i.e. does not call
 * ``t_begin()'') . For further details on the arguments, refer to the device
 * specific documentation .
*/
int device_type;        /* Specifies device type */
FILE *device;           /* Pointer to output device ( default is stdout ) */
char VAR_PTR options;   /* Text string of options ( default is "" ) */
  {
    int category = T_ST_FILE;
    switch ( device_type )
      {
#ifndef IBM_PC
        case VISUAL500_TERMINAL : device_ptr = & V500_dev_ptrs; break;
        case VISUAL550_TERMINAL : device_ptr = & V550_dev_ptrs; break;
        case XTERM_VT100TEK4014 : device_ptr = & Xterm_dev_ptrs; break;
#endif
        case HEWLETT_GL_PLOTTER : device_ptr = & HPgl_dev_ptrs; break;
        case POSTSCRIPT_PRINTER : device_ptr = & Post_dev_ptrs; break;
        case DUMMY_GRAPHICS_DEV : device_ptr = & Dum_dev_ptrs;  break;
/*      case DOT_MATRIX_PRINTER : device_ptr = & DMXP_dev_ptrs; break; */
#ifdef IBM_PC
        case HERCULES_GRAPHICS  : device_ptr = & Herc_dev_ptrs; break;
/*      case ENHANCED_GRAPHICS  : device_ptr = & EGA_dev_ptrs;  break; */
#endif
#ifdef SUN3_WORKSTATION
        case SUN_3_WORKSTATION  : device_ptr = & Sun3_dev_ptrs; break;
#endif
#ifdef X_WINDOWS
/*      case X11_WINDOWS        : device_ptr = & X11_dev_ptrs;  break; */
#endif
        default :
            return (FILE *) make_error( ERR_ABORT, (char VAR_PTR) "t_device",
                (char VAR_PTR) "Unknown device" );
      }
    (void) t_status( T_ST_CATEG, (char VAR_PTR) &category );
    if ( ! device )
        if ( category & T_ST_FILE )
            device = stdout;
        else if ( category & ( T_ST_HARD + T_ST_WIND ) )
            device = (FILE *) malloc( (unsigned) sizeof( FILE ) );
        else
            return (FILE *) make_error( ERR_ABORT, (char VAR_PTR) "t_device",
                (char VAR_PTR) "Unknown device category" );
    if ( (char VAR_PTR) options != T_NOT_BEGIN )
        (void) t_begin( device, options );
    return device;
  }

