TITLE   Hercules graphics module
        page 58,132
;
;    Copyright (C) 1997  Peter Vernon Lanspeary
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;


;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; This is the bottom end of a graphics driver for the Hercules Graphics card .
;
; Some of this stuff came via Michael Gordon, Dave Tutelman, Colin Kelley and
; Gil Webster . About half of the code is new; the rest has been modified .
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       HERCULES GRAPHICS CARD MEMORY MAP .
;   Each byte of screen memory maps to 8 horizontally adjacent pixels . A
;   block of 90 bytes maps to a horizontal line over the full screen width .
;   Adjacent 90-byte blocks map to every 4th line on the screen . The start
;   addresses of the top 4 rows on the screen are
;       Herc_Base + Page_No * 8000h + ( 0000h, 2000h, 4000h, 6000h )
;   where  Herc_Base = 0B0000h is the base address of the graphics card .
;   32k bytes of memory are reserved for each graphics page ( Page_No = 0, 1 )
;
;       +--- 0000h  0001h  ....  005Eh  005Fh
;       |    2000h  2001h  ....  205Eh  205Fh
;     +5Ah   4000h  4001h  ....  405Eh  405Fh
;       |    6000h  6001h  ....  605Eh  605Fh
;       +--- 005Ah  005Bh  ....  00B2h  00B3h
;       |    205Ah  205Bh  ....  20B2h  20B3h
;     +5Ah   405Ah  405Bh  ....  40B2h  40B3h
;       |    605Ah  605Bh  ....  60B2h  60B3h
;       +--- 00B4h  00B5h  ....  010Ch  010Dh
;       20B4h  20B5h  ....  210Ch  210Dh
;         .      .            .      .
;         .      .            .      .
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Assemble with MASM version 4 or higher, or TASM Version 1 or higher .
;
;
;               DEFINE THE CODE AND DATA MEMORY MODELS HERE
;       Defaults are LARGE_CODE and LARGE_DATA .
;  SMALL_CODE   equ     1
;  SMALL_DATA   equ     1
LARGE_CODE      equ     1
LARGE_DATA      equ     1
;
;               DEFINE YOUR COMPILER HERE
;  LATTICE_C    equ     1
  MICROSOFT_C  equ     1
;  TURBO_C         equ     1
;
;               DEFINE THE CPU CHIP HERE
;  intel8086    equ     1
;  intel80286   equ     1
  intel80386   equ     1
;
IFDEF intel80386
    intel80286  equ     1
ENDIF
P8086
;
;       DEFINITIONS
;
;       Hardware & Firmware constants
ROM_seg         equ     0F000h  ; address segment of IBM ROM
Char_Table      equ     0FA6Eh  ; address offset of character table in IBM ROM
Herc_Base       equ     0B000h  ; Base address segment for Hercules screen map
Page_Size       equ     800h    ; Graphics page size / 10h

HERC_X          equ     720     ; width of the screen ( no. of pixels )
HERC_Y          equ     348     ; height of the screen ( no. of pixels )
ALPH_LINES      equ     43      ; number of complete text rows on screen
ALPH_COLUMNS    equ     90      ; number of text columns on screen
ALPH_HEIGHT     equ     8       ; height of firmware text
ALPH_WIDTH      equ     8       ; width of firmware text

;
;       Global, External constants
;   ( equivalent definitions are used by the calling routines )
;
TRUE            equ     1
FALSE           equ     0
SOLID_LINE      equ     0FFFFh  ; line style bit pattern mask for solid lines
;
;   permitted `dot_level's  -- similar to Visual 500 terminal
DOT_ON          equ     0       ; switch pixels on
DOT_OFF         equ     1       ; switch pixels off
DOT_REV         equ     2       ; change (complement) pixels
DOT_NEW         equ     3       ; replace pixels
;
;       character definitions
CR              equ     0Dh     ; carriage return
LF              equ     0Ah     ; line feed
;
;
;       Local constants
ifdef SMALL_CODE
    Arg_Offset  equ     4       ; Argument offset for near procedures
else
    Arg_Offset  equ     6       ; Argument offset for far procedures
endif
;
;
;       MACRO
;
;  Begin_proc is a macro which ensures that the PUBLIC procedures have addresses
;  of the correct size; i.e. `near' for SMALL_CODE and `far' for LARGE_CODE .
;
Begin_proc macro proc_name
ifdef   LARGE_CODE
     proc_name proc far
else
     proc_name proc near
endif
endm
;
;       define the code section for the specified C compiler and code/data size
;
ifdef   LATTICE_C
  PGROUP  GROUP   PROG
  PROG    SEGMENT BYTE PUBLIC 'PROG'
          ASSUME  CS:PROG
else
  ifdef   MICROSOFT_C
    ifdef   SMALL_CODE
      _TEXT   SEGMENT WORD PUBLIC 'CODE'
      ASSUME  CS:_TEXT
    else
      ifdef   LARGE_CODE
        HG_C_TEXT   SEGMENT WORD PUBLIC 'CODE'
        ASSUME  CS:HG_C_TEXT
      endif
    endif
  else
    ifdef   TURBO_C
      ifdef   SMALL_CODE
        _TEXT   SEGMENT BYTE PUBLIC 'CODE'
                ASSUME  CS:_TEXT
      else
        ifdef   LARGE_CODE
          HG_C_TEXT   SEGMENT BYTE PUBLIC 'CODE'
                      ASSUME  CS:HG_C_TEXT
        endif
      endif
    endif
  endif
endif
;
;               List of Routines
;
        public _Hg_box,  _Hg_clear, _Hg_dots, _Hg_draw, _Hg_dump,  _Hg_getc,
        public _Hg_getx, _Hg_move,  _Hg_off,  _Hg_on,   _Hg_point, _Hg_putc,
        public _Hg_putx, _Hg_style
;
;
;       Herc_addr - calculate the address of the screen pixel for point (x,y)
;****************
; Register arguments :
;            input                  output
;  ax    = abscissa   x
;  bx    = ordinate   y
;  cl                           = pixel bit address
;  es:di                        = address of screen pixel
;
; Registers used : ax, bx, cx, dx, di, es
; Registers destroyed : cl, di, es
;
Herc_addr   proc near
            push ax
            push bx
            push dx
            push cx
;
;                       copy the coordinate
            mov  cx,ax          ; x
            mov  dx,HERC_Y-1
            sub  dx,bx          ; y = HERC_Y - 1 - y
;
;           first compute the address of byte to be modified
;               = 90*(row/4) + (col/8) + 2^D*[row/4] + 2^F*page
;
            mov  bh,cl          ; col (low order) in BH
            mov  bl,dl          ; row (low order) in BL
            and  bx,0703H       ; mask the col & row remainders
IFDEF intel80286
            P286N
            shr  cx,3           ; col / 8
            shr  dx,2           ; row / 4
            mov  ax,90
            mul  dx             ; AX = 90*[ row/4 ]
            add  ax,cx          ;  ... + col/8
            shl  bl,5           ; align row remainder
            P8086
ELSE                    ; same as above, obscure but fast for 8086
            shr  cx,1           ; col / 2
            shr  cx,1           ; col / 4
            shr  cx,1           ; col / 8
            shr  dx,1           ; row / 2
            shr  dx,1           ; row / 4
;
;         begin fast multiply by 90 (1011010 B)
;
            shl  dx,1           ; ( row / 4 ) * 2
            mov  ax,dx          ; store ( row / 4 ) * 2
            shl  dx,1           ; ( row / 4 ) * 4
            shl  dx,1           ; ( row / 4 ) * 8
            add  ax,dx          ; ( row / 4 ) * 10
            shl  dx,1           ; ( row / 4 ) * 20
            add  ax,dx          ; ( row / 4 ) * 22
            shl  dx,1           ; ( row / 4 ) * 44
            shl  dx,1           ; ( row / 4 ) * 88
            add  ax,dx          ; ( row / 4 ) * 90  end fast multiply by 90
            add  ax,cx          ; add on the ( col / 8 )
            shl  bl,1           ; align row remainder
            shl  bl,1
            shl  bl,1
            shl  bl,1
            shl  bl,1
ENDIF
            add  ah,bl          ; use aligned row remainder
; end of address calculation ; address of pixel is now in AX
;
;
;               get screen address & pixel bit mask ready for use
;                  es:[si] = screen address,   al = pixel mask
            mov  dx,Herc_Base+Page_Size   ; base of pixel display to DX
            mov  es,dx          ; ...and thence to segment reg
            mov  di,ax          ; address of byte w/ pixel to index reg
            pop  cx
            mov  cl,bh          ; bit address of the pixel in the screen byte
            pop  dx
            pop  bx
            pop  ax
            ret
Herc_addr   endp
;
;
;   Herc_byte
;***************
;   Write to the graphics screen pixels addressed by
; segment:offset:bits  es:di:al in the manner indicated by the dot_level .
;
; Register arguments
;   cl - flags which bits can be used in the screen byte - i.e. pixel mask
;   ch - line style mask
;   es:di - address of screen byte
; Registers used : ax, cl, ch, di, es
;   al - final screen mask
;   ah - dot_level
; Registers destroyed : ax
;
Herc_byte   proc near
            mov  al,ch
            and  al,cl                  ;     mask = style_mask & pixel_mask;
            mov  ah, byte ptr cs:dot_level   ; get dot_level
            cmp  ah,DOT_NEW             ; if ( dot_level != DOT_NEW )
            jz   short y_dot_new        ;   {
            cmp  ah,DOT_ON
            jnz  short y_dot_off        ;     if ( dot_level == DOT_ON ) {
            or   es:[di],al             ;         screen |= mask;
            ret                         ;         return; } /* turn pixel on */
y_dot_off:  cmp  ah,DOT_OFF
            jnz  short y_dot_rev        ;     else if ( dot_level == DOT_OFF )
            not  al                     ;       {     /* turn pixel off */
            and  es:[di],al             ;         screen &= ! mask;
            ret                         ;         return; }
y_dot_rev:  xor  es:[di],al             ;     else if ( dot_level == DOT_REV )
            ret                         ;       { screen ~= mask; return; } }
;                                       ; else  if ( dot_level == DOT_NEW )
y_dot_new:  mov  ah,cl                  ;  { /*keep bits not within pixel_mask*/
            not  ah                     ;     ! pixel_mask
            and  ah,es:[di]             ;     screen & ! pixel_mask
            or   al,ah                  ;     mask | screen & ! pixel_mask
            mov  es:[di],al             ;     ===>>> screen
            ret                         ;     return; }
Herc_byte   endp
;
;
;   Herc_pix    - writes to the pixel at coordinate (x,y) on the screen
;***************
;  Register arguments :
;     ax  -  x = abscissa
;     bx  -  y = ordinate
;  Register use :
;     ax - abscissa,  al - pixel bit mask, ah - dot_level
;     bx - ordinate
;     es:di - screen_byte address
;     cs
;  Registers destroyed : none
;
Herc_pix    proc    near
;
;               save registers used in this routine
;
            push ax
            push cx
            push di
            push es
;
;               test for invalid coordinate ( i.e. not on screen )
;
            cmp  ax,HERC_X                  ; if ( x < 0 || x >= HERC_X ||
            jae  short p_cleanup
            cmp  bx,HERC_Y                  ;      y < 0 || y >= HERC_Y )
            jae  short p_cleanup            ;   return
;
;               get the screen pixel byte-bit address

            call near ptr Herc_addr         ; calculate screen pixel address
            mov  al,80H                     ; '1000 0000' in AL  and shift mask
            shr  al,cl                      ; to line up with bit to read/write
;
;           set the pixel
;
            mov  ah, byte ptr cs:dot_level  ; get dot_level
            ror  word ptr cs:style_mask,1   ; rotate style_mask thru CARRY
            jc   short p_style_on           ; if ( ! style_mask_CARRY )
            cmp  ah,DOT_NEW                 ;   {
            jnz  short p_cleanup            ;     if ( dot_level == DOT_NEW )
            not  al                         ;        turn pixel off
            and  es:[di],al                 ;   }
            jmp  short p_cleanup            ; else if ( style_mask_CARRY )
p_style_on: cmp  ah,DOT_ON                  ;   { if ( dot_level == DOT_ON )
            jz   short p_on_pix             ;        turn pixel on
            cmp  ah,DOT_OFF                 ;     else if
            jnz  short p_dot_rev            ;        ( dot_level == DOT_OFF )
            not  al                         ;
            and  es:[di],al                 ;        turn pixel off
            jmp  short p_cleanup            ;     else if
p_dot_rev:  cmp  ah,DOT_REV                 ;        ( dot_level == DOT_REV )
            jnz  short p_on_pix             ;
            xor  es:[di],al                 ;        change pixel
            jmp  short p_cleanup            ;     else
p_on_pix:   or   es:[di],al                 ;        turn pixel on
;
;               finished, restore registers
;
p_cleanup:  pop  es
            pop  di
            pop  cx
            pop  ax
            ret
Herc_pix    endp
;
;
;******************************************************************************
;
; int Hg_move( X1, Y1 );
; int X1, Y1;
;   Place the start of a line segment at (X1,Y1) . The returned value is TRUE
;   if (X1,Y1) is on the screen, otherwise the returned value is FALSE and the
;   pen location is not changed .
;
;******************************************************************************
; Register use :
;   ax - returned value
;   cx - start point - x[1]
;   dx -               y[1]
;   ss:bp - Argument pointer
;   sp, cs
Begin_proc  _Hg_move
            push bp
            mov  bp,sp
            xor  ax,ax                  ; returned value = FALSE
            mov  cx,[bp].Arg_Offset     ; start point - x[1]
            mov  dx,[bp+2].Arg_Offset   ;             - y[1]
            cmp  cx,HERC_X
            jae  short m_cleanup        ; if ( 0 <= x[1] < HERC_X
            cmp  dx,HERC_Y              ;      0 <= y[1] < HERC_Y )
            jae  short m_cleanup        ;   {
            mov  word ptr cs:move_x,cx  ;     save x[1];
            mov  word ptr cs:move_y,dx  ;     save y[1];
            inc  ax                     ;     returned value = TRUE
m_cleanup:  pop  bp                     ;   }
            ret
_Hg_move    endp
;
;
;******************************************************************************
;
; int Hg_point( X, Y );
; int X, Y;
;   Place a single dot at pixel location (X,Y) . The returned value is TRUE
;   if (X,Y) is on the screen, otherwise the returned value is FALSE and the
;   point is not drawn .
;
;******************************************************************************
; Register use :
;   ax - start point - x
;   bx -               y
;   cx - returned value
;   ss:bp - Argument pointer
;   sp, cs
Begin_proc  _Hg_point
            push bp
            mov  bp,sp
            push word ptr cs:style_mask
            xor  cx,cx                  ; returned value = FALSE
            mov  ax,[bp].Arg_Offset     ; start point - x
            mov  bx,[bp+2].Arg_Offset   ;             - y
            cmp  ax,HERC_X
            jae  short o_cleanup        ; if ( 0 <= x < HERC_X &&
            cmp  bx,HERC_Y              ;      0 <= y < HERC_Y )
            jae  short o_cleanup        ;   {
            call near ptr del_xhair     ; remove any existing cross hairs
            mov  word ptr cs:style_mask,SOLID_LINE  ;ensure a bright point
            mov  word ptr cs:move_x,ax              ;    save x;
            mov  word ptr cs:move_y,bx              ;    save y;
            call near ptr Herc_pix                  ;    pixel( x, y );
            inc  cx                                 ;    returned value = TRUE;
o_cleanup:  mov  ax,cx                              ;   }
            pop  word ptr cs:style_mask
            pop  bp
            ret
_Hg_point   endp
;
;******************************************************************************
;
; int Hg_draw( X2, Y2 );
; int X2, Y2;
;
;   Draw a line on the the graphics screen from (X1,Y1) to (X2,Y2) .
;   (X1,Y1) is the point last drawn to by _Hg_draw, moved to by _Hg_move, or
;   is at the end of the last text string drawn by _Hg_putc . The returned
;   value is TRUE if (X1,Y1) and (X2,Y2) are both on the screen, otherwise the
;   returned value is FALSE and the line is not drawn .
;
;******************************************************************************
;
; Register use :
;   ax = start point - x[1]
;   bx = start point - y[1]
;   cx = finish point - x[2]
;   dx = absolute difference between first and last x values
;   si = finish point - y[2]
;   di = absolute difference between first and last y values, returned value
;   bp = dependent value tracker
;   ss:bp = argument pointer, dependent value tracker
;   sp, cs
; Registers destroyed : ax, bx, cx, dx
;
;
Begin_proc  _Hg_draw
            push bp
            mov  bp,sp
            push di
            push si
;
;               get arguments
;
            mov  ax,word ptr cs:move_x        ; start point - x[1]
            mov  bx,word ptr cs:move_y        ;               y[1]
            mov  cx,[bp].Arg_Offset           ; finish point - x[2]
            mov  si,[bp+2].Arg_Offset         ;                y[2]
;
;               check for legal end points
;
            xor  di,di                  ; returned value = FALSE
            cmp  ax,HERC_X              ; if ( x[1] < 0 || x[1] > HERC_X
            jae  short l_abort
            cmp  bx,HERC_Y              ;   || y[1] < 0 || y[1] > HERC_Y
            jae  short l_abort
            cmp  cx,HERC_X              ;   || x[2] < 0 || x[2] > HERC_X g
            jae  short l_abort
            cmp  si,HERC_Y              ;   || y[2] < 0 || y[2] > HERC_Y )
            jb   short l_first          ;       return
l_abort:    jmp  l_cleanup
;
;               save the end point
;
l_first:    call near ptr del_xhair     ; remove any existing cross hairs
            mov  word ptr cs:move_x,cx  ; save the new pen location
            mov  word ptr cs:move_y,si
;
;               calculate the x & y components of the step size
;               from start point to finish point
;
            mov  dx,ax
            sub  dx,cx                  ; x_step = x[1] - x[2]
            jnc  short l_del_x
            neg  dx                     ; x_step = abs( x[1] - x[2] )
l_del_x:    mov  di,bx
            sub  di,si                  ; y_step = y[1] - y[2]
            jnc  short l_quick
            neg  di                     ; y_step = abs( y[1] - y[2] )
;
;               if the line is horizontal and the line style is SOLID_LINE,
;               then the rapid-fill (for filling boxes) can be used
;
l_quick:    jnz  short l_sel_ctrl       ; if ( y_step == 0 &&
            cmp  dx,8
            jl   short l_sel_ctrl       ;      x_step >= 8 &&
            cmp  word ptr cs:style_mask,0ffffh
            jnz  short l_sel_ctrl       ;      style_mask == SOLID_LINE )
            mov  bp,bx                  ;   {  y
            mov  si,ax                  ;      x_min
            mov  di,cx                  ;      x_max
            mov  al,0ffh                ;      pixel mask
            call near ptr box_line      ;      rapid horizontal line
l_end_q:    jmp  short l_ok_end         ;   }
;
;               select the controlling value to increment
;
;               when drawing a vector, either the x or y value is changed
;               by one for each filled pixel . This controlling value is
;               y if the y_step is larger than the x_step ; otherwise the
;               controlling value is x . The other value (y or x) which we
;               call the dependent value is adjusted to follow the direction
;               of the vector.
;
l_sel_ctrl: cmp  dx,di
            jb   short l_y_ctrl           ; if ( x_step >= y_step )
;
;                   Select increment or decrement for y
;                                           {
l_x_ctrl:   cmp  bx,si                    ;   if ( y[1] >= y[2] )
            jb   short l_y_inc1
            mov  si,-1                    ;       set decrement y;
            jmp  short l_y_dec1           ;   else
l_y_inc1:   mov  si,1                     ;       set increment y;
l_y_dec1:   call near ptr Herc_pix        ;   pixel( x[1], y[1] ); - start point
;
;               loop for drawing the line from (x[1],y[1]) to (x[2],y[2])
;
            mov  bp,dx                  ;     initialize dependent value tracker
            shr  bp,1                   ;       = x_step/2;
l_x_loop:   cmp  ax,cx                  ;     while( x[1] != x[2] )
            je   short l_ok_end         ;       {
            jb   xinc1                  ;         if ( x[1] < x[2] )
            dec  ax                     ;           x[1]--;
            dec  ax                     ;         else
xinc1:      inc  ax                     ;           x[1]++;
            add  bp,di                  ;         tracker += y_step;
            cmp  bp,dx                  ;         if ( tracker >= x_step )
            jb   short l_noystep        ;           {
            sub  bp,dx                  ;             tracker -= x_step;
            add  bx,si                  ;             y[1]++  or y[1]--;
l_noystep:  call near ptr Herc_pix      ;             pixel( x[1], y[1] );
            jmp  short l_x_loop         ;           }
;                                               }
;                                            }
;                                          else
;               Select increment or decrement for x
;                                            {
l_y_ctrl:   cmp  ax,cx                  ;     if ( x[1] < x[2] )
            jb   l_x_inc2               ;
            mov  cx,-1                  ;         set decrement x;
            jmp  short l_x_dec2         ;     else
l_x_inc2:   mov  cx,1                   ;         set increment x;
l_x_dec2:   call near ptr Herc_pix      ;     pixel( x[1], y[1] );
;
;               loop for drawing the line from (x[1],y[1]) to (x[2],y[2])
;
            mov  bp,di                  ;     initialize dependent value tracker
            shr  bp,1                   ;       = y_step / 2
l_y_loop:   cmp  bx,si                  ;     while( y[1] != y[2] )
            je   short l_ok_end         ;       {
            jb   short yinc2            ;         if ( y[1] < y[1] )
            dec  bx                     ;           y[1]--;
            dec  bx                     ;         else
yinc2:      inc  bx                     ;           y[1]++;
            add  bp,dx                  ;         tracker +- x_step;
            cmp  bp,di                  ;         if ( tracker >= y_step )
            jb   short l_noxstep        ;           {
            sub  bp,di                  ;             tracker =- y_step;
            add  ax,cx                  ;             x[1]++ or x[1]--;
l_noxstep:  call near ptr Herc_pix      ;             pixel( x[1], y[1] );
            jmp  short l_y_loop         ;           }
;                                       ;       }
; clean up and exit                     ;    }
;
l_ok_end:   mov  di,TRUE                ; returned value = TRUE
l_cleanup:  mov  ax,di                  ; ax = returned value
            pop  si
            pop  di
            pop  bp
            ret
_Hg_draw    endp
;
;******************************************************************************
;
;  void Hg_clear( Page );
;  int Page;
;                Clears screen page `Page'
;  Page 0 is the alphnumeric screen .
;  Page 1 is the graphics screen .
;
;******************************************************************************
;
;       Register use :
;  ax  - Hercules card parge segment, zero ( for clearing screen )
;  cx  - down-counter
;  di  - address offset for the screen
;  ss:bp  - argument pointer
;  es  - address segment for the screen
;  sp  -
;
;  Registers destroyed : ax, cx
;
Begin_proc  _Hg_clear
            push bp
            mov  bp,sp
            push di
            push es
            mov  ah,[bp].Arg_Offset     ; get page number
            and  ax,0100h               ;   = 0 or 100h
            shl  ah,1                   ;     0    200h
            shl  ah,1                   ;     0    400h
            shl  ah,1                   ;     0    800h = Page size / 16.
            add  ax, Herc_Base
            mov  es, ax                 ; address segment of Hercules card
            xor  di, di                 ; 1st address offset of Hercules card= 0
            mov  cx, 4000h              ; no. of words to clear
            xor  ax, ax                 ; source = 0
            cld                         ; autoincrement
            rep  stosw                  ; zero out screen page
            mov  byte ptr cs:xhair_on,FALSE     ; no more cross hairs
            pop  es
            pop  di
            pop  bp
            ret
_Hg_clear   endp
;
;******************************************************************************
;
;  void Hg_dots( dot_level );
;  int dot_level;
;
;    Changes the dots level  i.e. the manner in which data is put onto the
;    screen . The valid `dot_level's are
;
;                 Vector drawing                Text
;   0  DOT_ON   add bright pixels *     overwrite text without erasing
;   1  DOT_OFF  switch pixels off *     write dark text
;   2  DOT_REV  change pixels     *     dark text, bright background
;   3  DOT_NEW  force vector to the     erase underlying screen contents
;               dashed line pattern
;
;******************************************************************************
;
; Register use : ax, bp, sp, cs
; Registers destroyed : ax
;
Begin_proc  _Hg_dots
            push bp
            mov  bp,sp
            mov  al,[bp].Arg_offset     ; get `dot_level' argument
            and  ax,03h                 ;   restrict to 0 - 3
            mov  byte ptr cs:dot_level,al
            pop  bp
            ret
_Hg_dots    endp
;
;******************************************************************************
;
;  void Hg_style( style );
;  int style;
;
;   Set the value of the bit mask which controls the dashed line style
;   to `style' . `style' is a 16 bit integer .
;
;******************************************************************************
;
;       Register use
; ax - style_mask value (argument)
; ss:bp - argument pointer
; sp, cs
;
Begin_proc  _Hg_style
            push bp
            mov  bp,sp
            mov  ax,[bp].Arg_offset     ; get line drawing mask argument
            mov  word ptr cs:style_mask,ax      ;   & store it
            pop  bp
            ret
_Hg_style   endp
;
;  Port addresses
;
ADDR_6845_index equ     03B4H   ; 6845 CRT controller index register
ADDR_6845_data  equ     03B5H   ; 6845 CRT controller data register
ADDR_mode_ctrl  equ     03B8H   ; display mode control port
ADDR_status     equ     03BAH   ; display status port
ADDR_config     equ     03BFH   ; Configuration switch
;
;  Values for display mode control port
;
Mode_ctl_text  equ     00H     ; text mode (power on default)
Mode_ctl_graph equ     02H     ; graphics mode
Mode_ctl_disable equ   00h     ; disable (blank) the screen (power on default)
Mode_ctl_enable  equ   08h     ; enable (activate) the screen
Mode_ctl_steady  equ   00h     ; disable text blinking (power on default)
Mode_ctl_blink   equ   20h     ; enable text blinking
Mode_ctl_page0   equ   00h     ; Graphics page 0 @B0000 (power on default)
Mode_ctl_page1   equ   80h     ; Graphics page 1 @B8000
Mode_text equ Mode_ctl_text + Mode_ctl_disable + Mode_ctl_blink + Mode_ctl_page0
                               ; Text mode control byte
Mode_graphics equ Mode_ctl_graph + Mode_ctl_disable + Mode_ctl_blink + Mode_ctl_page1
                               ; Graphics mode control byte
;
;  Values for Configuration switch
;
Config_gr_disable equ   00h     ; prevents setting of graphics mode (default)
Config_gr_enable  equ   01h     ; allows setting of graphics mode
Config_page1_out  equ   00h     ; mask Page 1 out of memory map 
                                ;    prevent its use
Config_page1_in   equ   02h     ; bring Page 1 into memory & allow its use 
;
parm_count      equ     12     ; number of 6845 control parameters
;
;******************************************************************************
;
;   void Hg_Off();
;   Switch graphics off . Go back to alpha-numeric screen .
;
;******************************************************************************
;
;    Register use
; al
; dx - address of Hercules control port
; cs:si - pointer to control sequence array
;    Registers destroyed : al, dx
;
Begin_proc  _Hg_Off
            push si
;
            mov  si, offset cs:HCh_Params
            mov  al, Mode_text          ; send text mode control byte
            mov  dx, ADDR_mode_ctrl     ;   to the Herc control port
            out  dx, al                 ;
            call near ptr CRT6845_params  ; Set the 6845 parameters
            or   al, Mode_ctl_enable    ; Enable the video output
            out  dx, al
;
            pop  si
            ret
_Hg_Off     endp
;
;******************************************************************************
;
;   void Hg_On();
;   Switch graphics on . Graphics card page 1 is used for graphics
;
;******************************************************************************
;
;    Register use
; al
; dx - address of Hercules control port
; cs:si - pointer to table of control bytes for 6845 CRT controller
;    Registers destroyed : al, dx
;
Begin_proc  _Hg_On
            push si
;
            mov  si, offset cs:HGr_Params
;            
            mov  al, Config_gr_enable + Config_page1_in
                                        ; send "HGC FULL"
            mov  dx, ADDR_config        ;   to the Herc configuration switch
            out  dx, al                 ;   
;
            mov  al, Mode_graphics      ; send graphics mode control byte
            mov  dx, ADDR_mode_ctrl     ;   to the Herc control port
            out  dx, al                 ;
            call near ptr CRT6845_params  ; Set the 6845 parameters
            or   al, Mode_ctl_enable    ; Enable the video output
            out  dx, al
;
            pop  si
            ret
_Hg_On      endp
;
;       CRT6845_params
;************************
;           The CRT (cathode ray tube) controller is the 6845 chip
;           Send 6845 parameters to Hercules board
;
; Register parameters :-
;  cs:di = pointer to list of 6845 parameters
;
;   Register use
; ah - parameter counter and index for 6845 address register
; al - control byte for video controller
; dx - address of video controller index port
; cs:di - pointer to list of 6845 parameter table
;
CRT6845_params proc near
            push ax
            push dx
            mov  dx,ADDR_6845_index          ; point DX at 6845 address register
            xor  ah,ah                  ; parameter counter = index = 0
sp_loop:    mov  al,ah                  ; send index 
            out  dx,al                  ;   to 6845 address register
            inc  dx                     ; point DX at 6845 data register
            mov  al,cs:[si]             ; get the control byte from the table
            inc  si                     ; increment pointer to parameter table
            out  dx,al                  ; send parameter to 6845 data register
            dec  dx                     ; point DX at 6845 addr register again
            inc  ah                     ; increment param counter & index
            cmp  ah,parm_count          ; if ( not sent all parameters )
            jnz  sp_loop                ;     goto start of loop
            pop  dx
            pop  ax
            ret
CRT6845_params endp
;
;******************************************************************************
;
;   int Hg_putc( charact );
;   char charact;
;
;    - Put a character on the graphics screen at the current pen position
;      The returned value is TRUE if the pen is at a valid screen location
;      and the character is printable ( i.e. ' ' to '~' ) ; otherwise the
;      returned value is FALSE .
;
;******************************************************************************
;
Begin_proc  _Hg_putc
            push bp
            mov  bp, sp
            push di
            push si
            push ds
            push es
;                                           a temporary variable on the stack
;   Register use
; ax - abscissa of pen location
; bx - ordinate of pen location
; cl - bit address of pixel on screen
; dx - `charact'
; es:di - address of screen byte
;               get the arguments
;
            mov  ax,word ptr cs:move_x  ; abscissa x  => ax
            mov  bx,word ptr cs:move_y  ; abscissa y  => bx
            mov  dx,[bp].Arg_Offset     ; `charact' => dx
            and  dx,007fh               ; remove parity bit from `charact'
;
;               check that the character is on the screen
;
            xor  cx,cx                  ; returned value = FALSE;
            cmp  ax,HERC_X-ALPH_WIDTH   ; if ( x < 0 || x >= HERC_X-ALPH_WIDTH
            ja   c_skip
            cmp  bx,HERC_Y-ALPH_HEIGHT  ;   || y < 0 || y >= HERC_Y-ALPH_HEIGHT
            ja   c_skip
            cmp  dl,' '                 ;   || `charact' < {first printable}
            jl   c_skip
            cmp  dl,'~'                 ;   || `charact' > {last printable} )
            jb   c_ROM_font
c_skip:     jmp  c_cleanup              ;     return
;
;   Register use
; dx    - address offset for font of character in ROM
; ds:si - address of font of character in ROM
;
;               calculate the address ds:si of the character font in ROM
;
c_ROM_font: call near ptr del_xhair     ; remove any existing cross hairs
            shl  dx,1
            shl  dx,1                   ; 8 * charact   ( 8 bytes of
            shl  dx,1                   ;  font per char in ROM )
            add  dx,Char_Table
            mov  si,dx                  ; = offset of font address
            mov  dx,ROM_seg
            mov  ds,dx                  ; = segment of font address
;
;   Register use
; ax - row of pixels for the character
; bx - index for the array of screen row_map addresses, Char_map
; cl - dot_level
; ch - bit address of pixel on screen
; dx - number of rows downcounter, screen character mask
; ds:si - address of font of character in ROM
; es:[di+bp] - address of screen byte
;
;               initialize for loop
;
            cld                         ; direction flag = autoincrement
            mov  dx,ALPH_HEIGHT         ; number of rows downcounter
            mov  bp,ax                  ; save coordinate abscissa
            add  bx,ALPH_HEIGHT-1       ; point to the top row
            mov  ch,byte ptr cs:dot_level   ; get dot_level  => ch
;
;               Calculate the screen address of the top row of pixels
;
;  start loop
c_loop:     mov  ax,bp
            call near ptr Herc_addr     ; screen address => es:di,cl
            dec  bx                     ; decrement screen row
            lodsb                       ; get row of font pixels from [ds:si]
            xor  ah,ah
            ror  ax,cl                  ; align the pixels
;
            cmp  ch,DOT_ON              ; if ( dot_level == DOT_ON )
            jnz  short c_dotrev
            or   es:[di],ax             ;   screen_word |= aligned_pixel_row
            jmp  short c_index
;
c_dotrev:   cmp  ch,DOT_REV             ; else if ( dot_level == DOT_REV )
            jnz  short c_dotoff
            push dx
            mov  dx,0ffh                ;   generate word mask to isolate the
            ror  dx,cl                  ;    character pixels and align it
            not  ax                     ;   reverse video pixels
            and  ax,dx                  ;    & switch on those inside the char
            not  dx                     ;   complement mask & use it to get the
            and  dx,es:[di]             ;    screen pixels on each side of char
            or   ax,dx                  ;   include them into the char word
            mov  es:[di],ax             ;    & put it back on the screen
            pop  dx
            jmp  short c_index
;
c_dotoff:   cmp  ch,DOT_OFF             ; else if ( dot_level == DOT_OFF )
            jnz  short c_dotnew
            not  ax
            and  es:[di],ax             ;    screen_word &= ! aligned_pixel_row
            jmp  short c_index
;                                       ; else
c_dotnew:   push dx
            mov  dx,0ff00h                ;    generate word mask to protect
            ror  dx,cl                  ;     pixels to L & R of the character
            and  dx,es:[di]             ;    get screen pixels on each side
            or   ax,dx                  ;    & include these into the char word
            mov  es:[di],ax             ;    & put it back on the screen
            pop  dx
;
c_index:    dec  dx                     ; decrement row_down_counter
            jg   c_loop                 ;   row of pixels
;end of loop
;               adjust the current pen position
;
            add  bp,ALPH_WIDTH          ; move pen to right
            cmp  bp,HERC_X              ; if ( x_pen >= HERC_X )
            jl   c_x_ok
            mov  bp,HERC_X-1            ;     x_pen = HERC_X-1
c_x_ok:     mov  word ptr cs:move_x,bp  ; save the pen location
            mov  cx,TRUE                ; returned value = TRUE
;
c_cleanup:  mov  ax,cx
            pop  es
            pop  ds
            pop  si
            pop  di
            pop  bp
            ret
_Hg_putc    endp
;
;******************************************************************************
;
;   char Hg_getc();
;
;    - get an ASCII from the keyboard without echoing it to the screen
;
;******************************************************************************
;
;    Register use :
; al =  character returned from DOS call - to the calling program
; ah =  function specifier for DOS call
; ds:bx = pointer to returned value
; ss:bp = argument pointer
; sp
;
Begin_proc  _Hg_getc
;
g_dos:      mov  ah,8                   ; function = read from keyboard
            int  21h                    ; DOS function request : char => al
            test al,al                  ; if ( char == NUL )
            jnz  g_cleanup              ;   {
            mov  ah,8                   ;     we have a non-ASCII special char
            int  21h                    ;     so read the rest of it
            jmp  short g_dos            ;     goto read_another_character
;                                       ;   }
g_cleanup:  and  ax,7fh                 ; ensure ASCII character
            ret
_Hg_getc    endp
;
;
;******************************************************************************
;
;  void Hg_dump( row, column, bytes );
;  int row, column;
;  char *bytes;
;       Converts the area on the Hercules Graphics defined over the character
;  position (`row',`column') to a sequence of `bytes' which can be sent directly
;  to a dot matrix printer ( with each bit representing a dot of the printer
;  matrix ) .
;
;******************************************************************************
;
;
;       Shell for C
;  This shell provides an interface with C . It gets the routine arguments
;  and puts them in the correct registers for the work routine PRDUMP .
;
;   Register use :
; bx - character row position
; dx - character column position
; ds:di - pointer to result array `bytes'
; ss:bp - argument pointer
; sp
;
Begin_proc  _Hg_Dump
            push bp
            mov  bp,sp
            push di
            push ds
            mov  bx,[bp].Arg_offset     ; get value of `row' => bx
            mov  dx,[bp+2].Arg_Offset   ; get value of `column' => dx
IF LARGE_DATA
            lds  di,[bp+4].Arg_Offset   ; get pointer to `bytes' => ds:di
ELSE
            mov  di,[bp+4].Arg_Offset   ; get pointer to `bytes' => ds:di
ENDIF
            call near ptr Herc_dump     ; call routine to do the work
            pop  ds
            pop  di
            pop  bp
            ret
_Hg_Dump    endp
;
;   PRDUMP
;       Work routine for reading an 8 x 8 group of pixels from the
;   character position (row,column)=(bx,dx)=( `row', `column' )
;   of the Hercules Graphics screen, and converting these to a set of
;   8 `bytes' which directly represent corresponding the dot matrix
;   patterns for a dot matrix printer .
;
;   Arguments are passed in the registers
; bx    = `row'
; dx    = `column'
; ds:di = segment:offset pointer to PRINT_BYTES
;
;                       Bit positions for each Screen_Byte
;                         128  64  32  16  8   4   2   1
;                          7   6   5   4   3   2   1   0
;                        +---+---+---+---+---+---+---+---+
;  Bit           128  7  |   |   |   |   |   |   |   |   | 1
;  positions      64  6  |   |   |   |   |   |   |   |   | 2  8 Bytes from
;  for            32  5  |   |   |   |   |   |   |   |   | 3  the Hercules
;  each           16  4  |   |   |   |   |   |   |   |   | 4    graphics
;  PRINT_BYTE      8  3  |   |   |   |   |   |   |   |   | 5     screen
;                  4  2  |   |   |   |   |   |   |   |   | 6    ( 1 byte
;                  2  1  |   |   |   |   |   |   |   |   | 7    per row )
;                  1  0  |   |   |   |   |   |   |   |   | 8
;                        +---+---+---+---+---+---+---+---+
;                          1   2   3   4   5   6   7   8
;       8 Bytes to the dot matrix printer ( 1 byte per column )
;
;
;    Register use
; ax - zero, temporary storage for 4*row
; bx - address offset of first screen byte, index for Char_map array
; cx - word count, byte counter
; dl - screen_mask - for selecting a pixel from the screen
; dh - print_mask  - bit to be included into `byte'
; ds:di - pointer to destination `byte'
; es:si - pointer to screen byte
; bp - address offset of first screen byte
;
;    Registers destroyed : ax, bx, cx, dx
;
Herc_dump   proc near
            push si
            push es
            xor  ax,ax                  ; returned value = FALSE
            cmp  bx,ALPH_LINES          ; if ( row < 0 || row > ALPH_LINES ||
            ja   d_cleanup
            cmp  dx,ALPH_COLUMNS        ;     col < 0 || col > ALPH_COLUMNS )
            jae  d_cleanup              ;        return FALSE;
            call near ptr del_xhair     ; remove any existing cross hairs
;
;           initialize array index to screen address offset and print_mask
;       SI is used temporarily for the array index and CH is used for the
;       print_mask until BX and DH become available .
;
            mov  si,8                   ; if ( row == ALPH_LINES )
            mov  ch,16                  ;   { print_mask = 16;
            cmp  bx,ALPH_LINES          ;     array_index = 8; }
            jz   last_row               ; else
            mov  ch,1                   ;   { print_mask = 1;
            add  si,si                  ;     array_index = 16; }
;
;           clear the `bytes'
;
last_row:   mov  word ptr cs:prt_ptr,di ; store offset of pointer to `bytes'
            mov  ds:[di],ax             ; clear 1st 2 bytes
            mov  ds:[di+2],ax           ; clear bytes 3 & 4
            mov  ds:[di+4],ax           ; clear bytes 5 & 6
            mov  ds:[di+6],ax           ; clear bytes 7 & 8
;
;           calculate the address offset of the first required Screen_byte
;             180 * row + column
;
            shl  bx,1                   ; 2   * row
            shl  bx,1                   ; 4   * row
            mov  ax,bx                  ; temp store for 4 * row
            shl  bx,1                   ; 8   * row
            shl  bx,1                   ; 16  * row
            add  bx,ax                  ; 20  * row
            shl  bx,1                   ; 40  * row
            add  bx,ax                  ; 44  * row
            shl  bx,1                   ; 88  * row
            shl  bx,1                   ; 176 * row
            add  bx,ax                  ; 180 * row
            add  bx,dx                  ; 180 * row + column
;
;           set up the first screen address
;
            mov  bp,bx                  ; = address offset of first screen byte
            mov  ax,Herc_Base+Page_Size
            mov  es,ax                  ; = address segment of first screen byte
;
            mov  bx,si                  ; transfer index of screen pointer
            mov  dh,ch                  ; transfer print_mask
;
row:        mov  si,word ptr cs:[bx].Char_map-2 ; get offset from look-up table
            add  si,bp                  ; required screen address
            mov  ah,es:[si]             ; get Screen_byte => ah
;
            mov  cx,8                   ; `byte' counter
            mov  dl,128                 ; initialize screen_mask
            mov  di,word ptr cs:prt_ptr ; pointer to first `byte'
;
column:     mov  al,ah                  ; get Screen_byte => al
            and  al,dl                  ; mask with Screen_mask
            jz   empty_pix              ; if pixel is not empty
            or   ds:[di],dh             ;   include print_mask into `byte'
empty_pix:  shr  dl,1                   ; move screen_mask to the right
            inc  di                     ; point to the next `byte'
            loop column                 ; next column if not done all `byte's
;
            shl  dh,1                   ; move print_mask to the left
            dec  bx
            dec  bx                     ; point to previous Screen_byte
            jnz  row                    ; next row if not done all Screen_bytes
            mov  ax,1                   ; returned value = TRUE;
;
d_cleanup:  pop  es
            pop  si
            ret
Herc_dump   endp

;
;******************************************************************************
;
;   int Hg_putx( X1, Y1 );
;   int X1, Y1;
;
;   Puts the Cross hairs at position (X1,Y1) on the screen, but does not show
;   them on the screens . The returned value is TRUE if (X1,Y1) is on the
;   screen, otherwise the returned value is FALSE and the pen location is not
;   changed .
;
;******************************************************************************
;
; Register use :
;   ax - returned value
;   cx - point - X1
;   dx -         Y1
;   ss:bp - Argument pointer
;   sp, cs
Begin_proc  _Hg_putx
            push bp
            mov  bp,sp
            xor  cx,cx                  ; returned value = FALSE
            mov  ax,[bp].Arg_Offset     ; abscissa - X1
            mov  bx,[bp+2].Arg_Offset   ;          - Y1
            cmp  ax,HERC_X
            jae  t_cleanup              ; if ( 0 <= x[1] < HERC_X
            cmp  bx,HERC_Y              ;      0 <= y[1] < HERC_Y )
            jae  t_cleanup              ;   {
            call near ptr del_xhair     ;     remove cross hairs
            mov  word ptr cs:xhair_x,ax ;     save X1
            mov  word ptr cs:xhair_y,bx ;     save Y1
            call near ptr Herc_row      ;     draw row
            call near ptr Herc_col      ;     draw column
            mov  byte ptr cs:xhair_on,TRUE ;  xhair_on = TRUE
            inc  cx                     ;     returned value = TRUE
t_cleanup:  mov  ax,cx                  ;   }
            pop  bp
            ret
_Hg_putx    endp
;
;
; del_xhair
;***********
;       If cross hairs are on the screen, remove them
;
del_xhair   proc near
            push ax
            push bx
            push di
            push es
            mov  al,byte ptr cs:xhair_on
            test al,al                  ; if ( xhair_on )
            jz   z_cleanup              ;   {
            mov  ax,word ptr cs:xhair_x ;     get cross-hair abscissa
            mov  bx,word ptr cs:xhair_y ;     get cross-hair ordinate
            call near ptr Herc_row      ;     remove row
            call near ptr Herc_col      ;     remove column
            mov  byte ptr cs:xhair_on,FALSE ; xhair_on = FALSE
z_cleanup:  pop  es                     ;   }
            pop  di
            pop  bx
            pop  ax
            ret
del_xhair   endp
;
;
;******************************************************************************
;
;   char Hg_getx( x, y );
;   int *x, *y;
;   char char_pressed;
;
;            Hercules Graphics X Hairs
;   This is a graphics cross-hairs shell for C . The cross-hairs
; routine places cross-hairs on the graphics screen and allows the user to
; drive them around the screen using the arrow keys on the numeric keypad
; ( i.e.  keys 1, 2, 3, 4, 6, 7, 8, 9 ) . Cross-hairs movement is quadrupled
; by pressing the `Ins'  key, and is reduced fourfold pressing the `Delete'
; key . By pressing a printable ASCII character key, the user stops the routine
; and passes the current cross-hairs position back to the calling routine
; through the arguments X and Y . The value of the printable ASCII character
; is returned as the value of Hg_getx() .
;
;******************************************************************************
;
; Registers used :-
;  ax -   X = row number of cross hairs
;  bx -   Y = column number of cross hairs
;  dx -   exit character
;  ds:di - pointers in argument list ( for `x', `y' & `char_pressed' )
;  ss:bp - argument list pointer
;
Begin_proc  _Hg_getx
            push bp
            mov  bp,sp
            push di
            push es
;
            call near ptr xhair         ; call the cross-hairs driver routine
;
IF LARGE_DATA
            les  di,[bp].Arg_Offset     ; ds:di = pointer `x' to 1st argument
            mov  es:[di],ax             ;   *x = abscissa of cross hair
            les  di,[bp+4].Arg_Offset   ; ds:di = pointer `y' to 2nd argument
            mov  es:[di],bx             ;   *y = ordinate of cross hair
ELSE
            mov  di,[bp].Arg_Offset     ; di = pointer `x' to 1st argument
            mov  [di],ax                ;   *x = abscissa of cross hair
            mov  di,[bp+2].Arg_Offset   ; di = pointer `y' to 2nd argument
            mov  [di],bx                ;   *y = ordinate of cross hair
ENDIF
            and  dx,007fh               ;   ensure the returned char is ASCII
            mov  ax,dx                  ;   *char_pressed = key pressed
;
            pop  es
            pop  di
            pop bp
            ret
_Hg_getx    endp
;
;       XHAIR
; This is the cross-hair driver routine . No benefit can be gained from
; optimizing the speed of this code because the speed is limited by the
; rate at which the keyboard sends repeated characters . Routine arguments
; are returned through registers
;
;    Register arguments :
;  ax -   X = abscissa of cross hairs
;  bx -   Y = ordinate of cross hairs
;  dl -   exit character
;
;    Register use :
;  ax -   X = abscissa of cross hairs, used in DOS call,
;  bx -   Y = ordinate of cross hairs
;  cx -   Step size , 1= smallest step, 4, 16, 64, 256 = biggest step
;  dl -   exit character, key_pressed
;  di -   used by Herc_row & Herc_col
;  es -   Address segment for Hercules card
;    Registers destroyed:
;  ax, bc, cx, dx, di, es
;
xhair       proc near
;
;           get initial cross-hair position and paint initial cross-hairs
;
            mov  ax,word ptr cs:xhair_x ; get initial X
            mov  bx,word ptr cs:xhair_y ; get initial Y
            mov  cx,word ptr cs:xh_step ; initial step size
            mov  dl,byte ptr cs:xhair_on
            test dl,dl                  ; if ( ! xhair_on )
            jnz  char1                  ;   {
            call near ptr Herc_row      ;     draw initial horizontal hair
            call near ptr Herc_col      ;     draw initial vertical hair
;                                           }
;           read a character from the keyboard
;
char1:      push ax
            mov  ah,8                   ; function = read from keyboard
            int  21h                    ; DOS function request
            mov  dl,al                  ; put result into dl
            pop  ax
;
;           if ( printable ASCII character ) return to caller
;
            cmp  dl,CR                  ; if ( char == {carriage return}
            jz   is_ascii
            cmp  dl,LF                  ;      or char == {line feed}
            jz   is_ascii
            cmp  dl,' '                 ;      or ( char > {first printable}
            jl   not_ascii
            cmp  dl,'~'                 ;      and char < {last printable} ) )
            jg   not_ascii              ; then  return with char
;                                       ; else  goto not_ascii_character
is_ascii:   mov  word ptr cs:xhair_x,ax ; store final X
            mov  word ptr cs:xhair_y,bx ; store final Y
            mov  word ptr cs:xh_step,cx ; store step size
            call near ptr Herc_row      ; remove horizontal hair
            call near ptr Herc_col      ; remove vertical hair
            mov  byte ptr cs:xhair_on,FALSE  ; cross hairs are off
            ret
;
;   If we get to this point, we do not have a printable ASCII character .
;   The arrow key codes are all preceded by the escape character <NUL> .
;   If the character received is not <NUL>, go back and read the keyboard again.
;
not_ascii:  test dl,dl                  ; if ( char <> <NULL> ) then
            jnz  char1                  ;   goto read_first_character
;
; the keypad key codes are :-
;   Arrow      Numeric  other   slow    fast
;   up left       7     Home    00 47   00 77
;   up            8             00 48
;   up right      9     PgUp    00 49   00 84
;   right         6             00 4D   00 74
;   down right    3     PgDn    00 51   00 76
;   down          2             00 50
;   down left     1     End     00 4f   00 75
;   left          4             00 4b   00 73
;
; increment
;   increase      0     Insert  00 52
;   decrease      .     Delete  00 53
;
;           read the remaining character
;
char2:      push ax
            mov  ah,8                   ; function = read from keyboard
            int  21h                    ; DOS function request
            mov  dl,al                  ; put result into dl
            pop  ax
;
;           check for increase in step size
;
x_faster:   cmp  dl,52h                 ; if ( 2nd_char == INSERT )
            jnz  x_slower               ;   {
            shl  cx,1                   ;     step_size *= 4;
            shl  cx,1
            cmp  cx,256                 ;     if ( step_size > 256 )
            jle  char1                  ;         step_size = 256;
            mov  cx,256                 ;     goto read_first_character;
            jmp  near ptr char1         ;   }
x_slower:   cmp  dl,53h                 ; else if ( 2nd_char == DELETE )
            jnz  check0                 ;   {
            shr  cx,1                   ;     step_size /= 4;
            shr  cx,1
            jnz  char1                  ;     if ( step_size == 0 )
            inc  cx                     ;         step_size = 1;
            jmp  near ptr char1         ;     goto read_first_character;
;                                            }
;           ensure that the cursor is on the screen  ( 78 clock cycles )
;
check0:     test ax,ax                  ; if ( X < 0 ) X = 0
            jge  check1
            xor  ax,ax
check1:     cmp  ax,HERC_X              ; if ( X >= HERC_X ) X = HERC_X - 1
            jl   check2
            mov  ax,HERC_X-1
check2:     test bx,bx                  ; if ( Y < 0 ) Y = 0
            jge  check3
            xor  bx,bx
check3:     cmp  bx,HERC_Y              ; if ( Y >= HERC_Y ) Y = HERC_Y - 1
            jle  vert
            mov  bx,HERC_Y-1
;
;           test if a vertical movement is required
;
vert:       cmp  dl,47h                 ; if ( up left
            jz   up
            cmp  dl,48h                 ;   or up
            jz   up
            cmp  dl,49h                 ;   or up right ) then
            jz   up                     ; goto move_cross-hair_up
            cmp  dl,4fh                 ; if ( down left
            jz   down
            cmp  dl,50h                 ;   or down
            jz   down
            cmp  dl,51h                 ;   or down right ) then
            jz   down                   ; goto move_cross-hair_down
;
;           test if a horizontal movement is required
;
horiz:      cmp  dl,47h                 ; if ( up left
            jz   left
            cmp  dl,4bh                 ;   or left
            jz   left
            cmp  dl,4fh                 ;   or down left ) then
            jz   left                   ; goto move_cross-hair_left
            cmp  dl,49h                 ; if ( up right
            jz   right
            cmp  dl,4dh                 ;   or right
            jz   right
            cmp  dl,51h                 ;   or down right ) then
            jz   right                  ; goto move_cross-hair_right
;
;           If arrow key is not found, then read keyboard again
;
no_move:    jmp  char1
;
;           move cross-hair up
;
up:         call near ptr Herc_row      ; remove existing horizontal line
            add  bx,cx                  ; increment position Y
            cmp  bx,HERC_Y
            jl   up_ok                  ; if ( Y >= HERC_Y )
            mov  bx,HERC_Y-1            ;     Y = HERC_Y - 1;
up_ok:      call near ptr Herc_row      ; draw new horizontal line
            jmp  horiz                  ; goto test for horizontal movement
;
;           move cross-hair down
;
down:       call near ptr Herc_row      ; remove existing horizontal line
            sub  bx,cx                  ; decrement position Y
            jge  down_ok                ; if ( Y < 0 )
            xor  bx,bx                  ;     Y = 0;
down_ok:    call near ptr Herc_row      ; draw new horizontal line
            jmp  horiz                  ; goto test for horizontal movement
;
;           move cross-hair left
;
left:       call near ptr Herc_col      ; remove existing vertical line
            sub  ax,cx                  ; decrement position X
            jge  left_ok                ; if ( X < 0 )
            xor  ax,ax                  ;     X = 0;
left_ok:    call near ptr Herc_col      ; draw new vertical line
            jmp  char1                  ; goto get next character from keyboard
;
;           move cross-hair right
;
right:      call near ptr Herc_col      ; remove existing vertical line
            add  ax,cx                  ; increment position X
            cmp  ax,HERC_X
            jl   right_ok               ; if ( X >= HERC_X )
            mov  ax,HERC_X-1            ;   X = HERC_X - 1;
right_ok:   call near ptr Herc_col      ; draw new vertical line
            jmp  char1                  ; goto get next character from keyboard
xhair       endp
;
;
;       Herc_row
;**************
;       - change every bit on the screen in row [bx]
;
;    Register argument
; bx - ordinate of the row to be changed
;
;    Registers used :
; ax - abscissa of first column
; cx - down counter
; es:di - screen address
;    Registers destroyed : di, es ( by Herc_addr )
;
Herc_row    proc near
            push ax
            push cx
;
;       calculate address of first byte in row
;
            xor  ax,ax                  ; abscissa = 0
            call Herc_addr
;
;           change every bit in the row
;
            mov  cx,45                  ; number of words in a row
xr0:        xor  word ptr es:[di],0ffffh ; switch one word
            inc  di
            inc  di                     ; point to next word
            loop xr0
;
            pop  cx
            pop  ax
            ret
Herc_row    endp
;
;
;       Herc_col
;**************
;       - change every bit in column [ax]
;
;    Register argument
; ax - abscissa of column to be changed
;
;    Registers used :
; bx - ordinate of the top row
; bl - pixel mask
; cx - down counter
; es:di - screen address
;    Registers destroyed : di, es ( by Herc_addr )
;
Herc_col    proc near
            push bx
            push cx
;
;           calculate address of first byte in column
;
            mov  bx,347                 ; ordinate = 347 = top row
            call Herc_addr
            mov  bl,80h                 ; put a bit mask
            shr  bl,cl                  ;   on the correct bit
;
;           change every bit in the column
;
            mov  cx,87                  ; number of rows/4
xc0:        xor  es:[di],bl
            xor  es:[di+2000h],bl
            xor  es:[di+4000h],bl
            xor  es:[di+6000h],bl
            add  di,90
            loop xc0
;
            pop  cx
            pop  bx
            ret
Herc_col    endp
;
;
VERT_HATCH  equ 6
HORIZ_HATCH equ VERT_HATCH
DIAG_HATCH  equ VERT_HATCH*141/200*2
;
;
;******************************************************************************
;
;   int Hg_box( x_min, y_min, x_max, y_max, style )
;   int x_min, y_min, x_max, y_max, style;
;
;   Draws a box with lower left corner (x_min,y_min) and upper right corner
;   (x_max,y_max) and fills it with a hatching pattern determined by `style' .
;   The available styles are
;      0 = empty, box outline only      5 = horizontal lines
;      1 = fill the box completely      6 = vertical lines
;      2 = 50% `grey' fill              7 = diagonal cross hatch
;      3 = NW-SE diagonal lines         8 = horizontal-vertical cross
;      4 = NE-SW diagonal lines                 hatch
;   The returned value of Hg_box is TRUE only if both the coordinates
;   (x_min,y_min) and (x_max,y_max) are on the screen and 0 <= style <= 8 .
;
;******************************************************************************
;
; Register use :
;   ax - returned value - hatching pixel mask
;   si - lower left corner of box - abscissa
;   cx -                          - ordinate
;   di - upper right corner of box - abscissa
;   dx -                           - ordinate
;   bp - style of box fill
;
Begin_proc  _Hg_box
            push bp
            mov  bp,sp
            push di
            push si
            push word ptr cs:style_mask ; must save line style
;
;               get arguments
;
            mov  si,[bp].Arg_Offset     ; lower left corner - abscissa - x_min
            mov  cx,[bp+2].Arg_Offset   ;                   - ordinate - y_min
            mov  di,[bp+4].Arg_Offset   ; upper right corner - abscissa - x_max
            mov  dx,[bp+6].Arg_Offset   ;                    - ordinate - y_max
            mov  bp,[bp+8].Arg_Offset   ; style of box fill
;
;               ensure lower_left < upper_right
;
            cmp  si,di
            jle  b_swap_y               ; if ( x_min > x_max )
            xchg si,di                  ;     swap( x_min, x_max )
b_swap_y:   cmp  cx,dx                  ; if ( y_min > y_min )
            jle  b_limits               ;     swap( y_min, y_min )
            xchg cx,dx
;
;               check that the box corners are on the screen
;                   and that the syle is legal
;
b_limits:   xor  ax,ax                  ; returned value = FALSE
            cmp  si,HERC_X              ; if ( x_min < 0 || x_min >= HERC_X ||
            jae  short b_skip
            cmp  cx,HERC_Y              ;   || y_min < 0 || y_min >= HERC_Y ||
            jae  short b_skip
            cmp  di,HERC_X              ;   || x_max < 0 || x_max >= HERC_X ||
            jae  short b_skip
            cmp  dx,HERC_Y              ;   || y_max < 0 || y_max >= HERC_Y ||
            jae  short b_skip
            cmp  bp,9                   ;   || style < 0 || style > 9 )
            jb   short b_xhair
b_skip:     jmp  short b_cleanup        ;      return FALSE;
;
;               remove crosshairs
;
b_xhair:    call near ptr del_xhair     ;     remove cross hairs
;
;               draw the box outline
;
            mov  cs:style_mask,SOLID_LINE   ; force solid line style
            call near ptr box_outline   ; draw box outline
;
;               adjust limits to within the box
;
            inc  si                     ; x_min++
            inc  cx                     ; y_min++
            dec  di                     ; x_max--
            dec  dx                     ; y_max--
            cmp  si,di
            jg   b_true                 ; if ( x_min > x_max ||
            cmp  cx,dx                  ;      y_min > y_max )
            jg   b_true                 ;   return TRUE;
;
;               fill the box with the required hatching style
;
            dec  bp
            jnz  b_grey                 ; if ( style == 1 )
            mov  bx,0ffffh              ;     pixel bit mask
            call near ptr box_check     ;     fill every pixel in box
b_grey:     dec  bp
            jnz  b_315                  ; else if ( style == 2 )
            mov  bx,0055h               ;     pixel bit mask
            call near ptr box_cross     ;     fill box with 50% grey
b_315:      dec  bp
            jnz  b_45                   ; else if ( style == 3 )
            mov  bx,0002h               ;     pixel bit mask
            call near ptr box_cross     ;     -45 degree hatching
b_45:       dec  bp
            jnz  b_horiz                ; else if ( style == 4 )
            mov  bx,8000h               ;     pixel bit mask
            call near ptr box_cross     ;     45 degree hatching
b_horiz:    dec  bp
            jnz  b_vert                 ; else if ( style == 5 )
            mov  bx,00ffh               ;     pixel bit mask
            call near ptr box_check     ;     horizontal hatching
b_vert:     dec  bp
            jnz  b_cross                ; else if ( style == 6 )
            mov  bx,1111h               ;     pixel bit mask
            call near ptr box_check     ;     vertical hatching
b_cross:    dec  bp
            jnz  b_check                ; else if ( style == 7 )
            mov  bx,8002h               ;     pixel bit mask
            call near ptr box_cross     ;     diagonal cross hatching
b_check:    dec  bp
            jnz  b_true                 ; else if ( style == 8 )
            mov  bx,11ffh               ;     pixel bit mask
            call near ptr box_check     ;   vertical & horizontal cross hatching
b_true:     mov  ax,TRUE                ; returned value = TRUE
;
b_cleanup:  pop  word ptr cs:style_mask
            pop  si
            pop  di
            pop  bp
            ret
_Hg_box     endp
;
;   box_outline
;*****************
;   draw the outline of the box
;
; Register arguments
;   si - lower left corner of box - abscissa
;   cx -                          - ordinate
;   di - upper right corner of box - abscissa
;   dx -                           - ordinate
; Register use :
;   ax - abscissa, hatching pixel mask
;   bx - ordinate
;   bp - ordinate for horizontal line drawing routine
; Registers destroyed : ax, bx
;
box_outline proc near
            push bp
            mov  al,0ffh                ; pixel mask - select all pixels in byte
            mov  bp,cx                  ; y = y_min
            call near ptr box_line      ; draw line from (x_min,y) to (x_min,y)
            mov  bp,dx                  ; y = y_max
            call near ptr box_line      ; draw line from (x_max,y) to (x_max,y)
            mov  ax,si                  ; x = x_min;
            mov  bx,cx                  ; y = y_min;
            call near ptr box_90line    ; vertical_line( x_min );
            mov  ax,di                  ; x = x_max;
            call near ptr box_90line    ; vertical_line( x_max );
            pop  bp
            ret
box_outline endp
;
;   box_check
;****************
;   fill the box with a horizontally and/or vertically aligned hatching
;   pattern ( fill, horizontal, vertical or check ) . The values for the
;   pixel bit mask are :
;     bx = FFFFh = solid fill
;        = 00FFh = horizontal hatching
;        = 1111h = vertical hatching
;        = 11FFh = checked (cross) hatching
; Register arguments
;   al - composite pixel bit mask
;   ah - line_counter
;   bl -  rare hatching pattern mask :  for every fourth line
;   bh -  common hatching pattern mask :  for every 3 adjacent lines
;   si - lower left corner of box - abscissa
;   cx -                          - ordinate
;   di - upper right corner of box - abscissa
;   dx -                           - ordinate
; Register use :
;   bp - current coordinate - ordinate
; Registers destroyed : ax, bx
;
box_check   proc near
            push bp
            xor  ah,ah                  ; line_counter = 0;
            mov  bp,cx                  ; for( y = y_min;
bc_loop:    cmp  bp,dx                  ;      y < y_max; )
            jg   bc_end                 ;   {
            mov  al,bl                  ;     use common pixel bit mask
            inc  ah                     ;     line_counter++;
            and  ah,3                   ;     line_counter %= 4;
            jz   bc_solid               ;     if ( line_counter == 0 )
            mov  al,bh                  ;         use common pixel bit mask
bc_solid:   call near ptr box_line      ;     horizontal_line( y++ );
            jmp  short bc_loop          ;   }
bc_end:     pop bp
            ret
box_check   endp
;
;
;   box_cross
;****************
;   fill the box with a diagonally aligned hatching pattern ( SW-NE hatch,
;   SE-NW hatch, 50% grey, or cross hatched ) . The values for the pixel
;   bit mask are :
;     bx = 0055h = 50% grey fill
;        = 8000h = SE-NW hatching
;        = 0002h = SW-NE hatching
;        = 8002h = diagonal cross hatching
; Register arguments
;   al -  composite pixel bit mask
;   bl -  SE-NW hatching pattern mask
;   bh -  SW-NE hatching pattern mask
;   si - lower left corner of box - abscissa
;   cx -                          - ordinate
;   di - upper right corner of box - abscissa
;   dx -                           - ordinate
; Register use :
;   bp - current coordinate - ordinate
; Registers destroyed : ax, bx
;
box_cross   proc near
            push bp
            mov  bp,cx                  ; for( y = y_min;
bx_loop:    cmp  bp,dx                  ;      y < y_max; )
            jg   bx_end                 ;   {
            mov  al,bh                  ;     get SW-NE hatching mask => al
            or   al,bl                  ;     & or it with SE-NW hatching mask
            call near ptr box_line      ;     horizontal_line( y++ );
            ror  bh,1                   ;     rotate SW-NE mask to right
            rol  bl,1                   ;     rotate SE-NW mask to left
            jmp  short bx_loop          ;   }
bx_end:     pop bp
            ret
box_cross   endp
;
;   box_90line
;****************
;   draw a line vertically upward from the lower edge of the box (ax,cx+1) to
;   the top edge of the box (ax,dx-1)
;
; Register arguments
;   ax -  abscissa
;   cx - lower left corner of box - ordinate
;   dx - upper right corner of box - ordinate
; Register use :
;   bp - current coordinate - ordinate
; Registers destroyed : none
;
box_90line  proc near
            push bx
            mov  bx,cx
            inc  bx                     ; for( y = y_min + 1;
b90l_loop:  cmp  bx,dx                  ;      y < y_max; )
            jge  b90l_end               ;   {
            call Herc_pix               ;     draw_pixel_at_(x,y);
            inc  bx                     ;     y++;
            jmp  short b90l_loop        ;   }
b90l_end:   pop  bx
            ret
box_90line  endp
;
;
;   box_line
;**************
;   draws a horizontal line on the screen from (x_min,y)=(si,bp) to
;       (x_max,y)=(di,bp) .
;
;  Register arguments
; al - pixel bit hatching mask - determines style of hatching or filling
; si - x_min
; di - x_max
; bp - y
;
; Register use
;   ax - x for address calculation,
;   bx - y for address calculation,
;   cl - pixel bit mask
;   ch - hatching style mask
;   dl - pixel mask for the left hand end point
;   dh - pixel mask for the right hand end point
;   si - address offset for the right hand end of the line
;   di - address offset for the left hand end of the line
;
box_line    proc near
            push ax
            push bx
            push cx
            push dx
            push si
            push di
            push bp
            push es
            mov  ch,al
;
;               ensure  x_min <= x_max
;
            cmp  si,di
            jle  bl_x_max               ; if ( x_min > x_max )
            xchg si,di                  ;    swap( x_min, x_max )
;
;               calculate screen address of right hand line endpoint
;
bl_x_max:   mov  ax,di                  ; x = x_max
            mov  bx,bp                  ; y
            call near ptr Herc_addr     ; screen_address(x,y) => es:di:cl
            mov  dh,80h
            shr  dh,cl                  ; bit address => pixel mask
            xchg si,di                  ; park address offset in a safe place si
;                                       ;  x_min is now in di
;
;               calculate screen address of left hand line endpoint
;
            mov  ax,di                  ; x = x_min
            call near ptr Herc_addr     ; screen_address(x,y) => es:di:cl
            mov  dl,80h
            shr  dl,cl                  ; bit address => pixel mask
;
;               deal with the special case where both line endpoints have
;               the same byte address
;
            cmp  si,di                  ; if ( right_byte == left_byte )
            jl   bl_cleanup             ;   /* insurance */
            jnz  bl_1st_col
;
            dec  dl                     ;   {
            dec  dh                     ;     make pixel bit mask to
;            sub  dh,dl                  ;     fill every pixel between
;            inc  dh                     ;     and including those in dh
;            shl  dh,1                   ;     and dl .
            xor  dh,dl
            inc  dl
            or   dh,dl
            jmp  short bl_put_pix       ;   }
;                                       ; else
;               `put' the pixels addressed by es:di - the first byte
;                                       ;   {
bl_1st_col: dec  dl                     ;     set all bits in the pixel mask
            shl  dl,1                   ;     which are to the right of the
            inc  dl                     ;     leftmost pixel in the line .
            mov  cl,dl                  ;     pixel mask argument for Herc_byte
            call near ptr Herc_byte     ;     write to pixels es:di:al
;
;               `put' all the `full' bytes between the endpoints
;
            inc  di                     ; for ( left_byte++;
bl_loop:    cmp  si,di                  ;       left_byte < right_byte; )
            jle  bl_lastcol             ;   {
            mov  cl,0ffh                ;     use all pixels in hatch pattern
            call near ptr Herc_byte     ;     write to pixels es:di:al
            inc  di                     ;     left_byte++
            jmp  bl_loop                ;   }
;
;               `put' the last byte onto the screen
;
bl_lastcol: dec  dh                     ; set all bits to the left of the right-
            not  dh                     ; most pixel in the endpoint pixel mask
bl_put_pix: mov  cl,dh                  ; pixel bit mask argument for Herc_byte
            call near ptr Herc_byte     ; write to pixels es:di:al
;
bl_cleanup: pop  es
            pop  bp
            inc  bp                     ; y++
            pop  di
            pop  si
            pop  dx
            pop  cx
            pop  bx
            pop  ax
            ret
box_line    endp
;
;
;       TABLES
;
;   Char_map  :- address displacements for successive lines of a character
;                on the Hercules Graphics screen
; Control sequence tables for 6845 CRT controller
;   HCh_Params :- table for text mode
;   HGr_Params :- table for graphics mode
;
; description of 6845 registers
; Index         Description
;  0    Total characters per row including SYNC less 1
;  1    Number of visible characters per row
;  2    Position of the first character during SYNC, less 1
;  3    Number of characters during SYNC per row, less 1
;  4    Number of rows less 1, including the row during vertical retrace
;  5    Number of scans (row fractions) in addition to total number of rows
;  6    Number of visible rows
;  7    Row number to begin the trace, less 1 (This will always last 16 scans)
;  8    Always output 2 ( consult 6845 data sheet )
;  9    Number of scans per row, less 1
; 10    First scan where the cursor will overlay a character
; 11    Last scan where the cursor will overlay a character
; 12    Always output 0 ( consult 6845 data sheet )
; 13    Always output 0 ( consult 6845 data sheet )
; 14    Offset of the cursor position in the display buffer (H)
; 15    Offset of the cursor position in the display buffer (L)
; 16    Offset into the buffer when lightpen tripped (H)
; 17    Offset into the buffer when lightpen tripped (L)
;
Char_map    dw      0000h, 2000h, 4000h, 6000h, 005Ah, 205Ah, 405Ah, 605Ah
HCh_Params  db      61H, 50H, 52H, 0FH, 19H, 06H, 19H, 19H, 02H, 0DH, 0BH, 0CH
HGr_Params  db      35H, 2DH, 2EH, 07H, 5BH, 02H, 57H, 57H, 02H, 03H, 00H, 00H
;
;       VARIABLES
;
prt_ptr     dw      0       ; temp storage for offset ptr to printer dump array
style_mask  dw  SOLID_LINE  ; bit pattern mask for dashed line styles
xhair_x     dw      0       ; stored cross hair coordinate
xhair_y     dw      0
xh_step     dw      1       ; crosshair movement step
move_x      dw      0       ; current pen location
move_y      dw      0
dot_level   db      DOT_ON  ; dot level for writing pixels
xhair_on    db      FALSE   ; TRUE only if cross hairs are `on'
;
;
ifdef LATTICE_C
  PROG    ends
else
  ifdef   MICROSOFT_C
    ifdef   SMALL_CODE
      _TEXT   ends
    else
      ifdef   LARGE_CODE
        HG_C_TEXT   ends
      endif
    endif
  else
    ifdef   TURBO_C
      ifdef   SMALL_CODE
        _TEXT   ends
      else
        ifdef   LARGE_CODE
          HG_C_TEXT   ends
        endif
      endif
    endif
  endif
endif

                  end
