c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM ILG_CONT
c
c ILG_CONT reads data required to do a contour plot and prepares an
c ILGPLOT script which can draw the contours within a context determined
c by the user of ILGPLOT.
c
c  The data is primarily a set of altitude values (Z) defined on a
c  rectangular (X,Y) Cartesian grid. The input mechanism is similar to that
c  of ILGPLOT with initial input from the argument list, then from standard
c  input and then from file. Output is to STANDARD out unless an output file
c  is specified.
c
c   COMMAND LIST -- CONTOUR INPUT
c  Name        Arguments       Description
c
c  grid     input_file_name  Two dimensional array of altitude values with N_x
c                            columns and N_y rows specified row by row (i.e.
c                            column index varies most rapidly). No defaults.
c  abscissa input_file_name  List of N_x abscissa values, one for each column
c                            of the grid array, in strictly ascending order.
c                            No defaults.
c  ordinate input_file_name  List of N_y ordinate values, one for each row of
c                            the grid array, in strictly ascending order.
c                            No defaults.
c  level    input_file_name  List of N_level altitude (Z) values for which
c                            contours are drawn. By default 10 contours are
c                            distributed uniformly over the range of
c                            altitudes in the grid array.
c  x_clip   min_x max_x      minimum and maximum grid array column numbers for
c                            for which contours are generated.
c                            Default: min_x=1, max_x=N_x.
c  y_clip   min_y max_y      minimum and maximum grid array row numbers for
c                            which contours are generated.
c                            Default: min_y=1, max_y=N_y.
c  z_clip   min_z max_z      minimum and maximum altitude values for which
c                            contours are prepared. Defaults: altitude limits.
c  build
c  make                      make/build the contours from the data
c  clear    abscissa         clear the abscissa data and clip limits
c           ordinate         clear the ordinate data and clip limits
c           grid             clear the altitude data and clip limits
c           level            clear the contour levels
c           none             clear nothing
c           all              clear all of the above
c  show                      print the required data to the output file
c           abscissa ordinate
c           grid level
c           x_clip y_clip z_clip
c           none all
c
c
c   COMMAND LIST -- PROGRAM CONTROL
c  Name        Arguments       Description
c
c  stop                      stop the program
c  exit                      stop the program
c  quit                      stop the program
c  [no]prompts               switch prompts on [off]
c  [no]wait                  wait for user to press <RETURN> after each command
c  output  output-file-name  name of output file. Default is standard output.
c  format  f77-format-spec   fortran format specifier used for output
c  execute command-file-name take commands from file
c  interactive               take commands from user (standard input)
c  return                    return to previous input file
c  help                      give help
c
      include 'stdio.def'
      integer arg_len, end_string, i, start_string, iargc, str_length
      character arg*32, prog_args*256
c     subroutine contour_switch
      data prog_args / ' ' /
c
c       get program arguments
c       pile all command line arguments into a single string
c
      start_string = 1
      do 2 i = 1, iargc()
          call getarg( i, arg )
          arg_len = str_length( arg )
          end_string = start_string + arg_len - 1
          prog_args( start_string : end_string ) = arg(1:arg_len)
          start_string = end_string + 2
2         continue
c
c       Identification message
c
      write( STD_OUT, 3 )
3     format( ' This is ILG_CONT v1.0 contour preparation for ILGPLOT' /
     1        ' Copyright (C) 1997  Peter Vernon Lanspeary' /
     2        ' Type "help" for help.' /
     3        ' This is free software; type "warranty" and' /
     4        ' "licence"to see the warranty and licence' )
c
c       now can begin work
c
      call contour_switch( prog_args )
      stop
      end



      BLOCK DATA CONTOUR_BLOCK
c
c  N.B. These initialisations may not be portable.
c       Those which do not work are not important and can be commented out.
c
      include 'stdio.def'
      include 'ilg_cont.def'
      include 'ilg_cont.cmn'
c
      data out_lu, absc_ptr, ord_ptr, grid_ptr, level_ptr,
     1     n_absc, n_ord, n_grid, n_level,
     2     min_x, max_x, min_y, max_y /
     3     STD_OUT, 4*1, 4*0, 4*0 /
      data min_z, max_z / NEG_R_INFTY, R_INFTY /
c     data done / MAX_DATA*NO_EDGE /
      data output_fmt / '(1x,g12.5,1x,g12.5)' /
      end



      SUBROUTINE CONTOUR_SWITCH( PROG_ARGS )
c
c Contour generation program controller
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      real    r1, r2
      integer command_lu, new_lu, i, j, empty_ptr, start_ptr,
     1    n_pts, lun_stack, open_file, read_data, str_length
      logical every, cr_wait, echo, execute, first_arg, prompt_enable,
     1        eqabbr, get_int, get_real, next_token
      character command_file*32, output_file*32, prog_args*(*),
     1          token*32
c     subroutine casech, wait_for_CR
c
      include 'ilg_cont.cmn'
c
      data every, cr_wait, execute, first_arg, prompt_enable /
     1     .FALSE., .FALSE., .FALSE., .TRUE.,    .TRUE. /
      data command_lu, empty_ptr /
     4     STD_IN, 1 /
      data command_file, output_file, token / 3 * ' ' /
c
      selection_criterion = MINIMUM_ANGLE
      min_z = -R_INFTY
      max_z =  R_INFTY
c
      first_arg  = prog_args .ne.  ' '
      command_lu = STD_IN
      cr_wait  = .FALSE.
c
c
c       start of main loop
c
1     continue                                                            1b
          echo = prompt_enable .and. command_lu .ne. STD_IN
c
c       exhaust the program arguments before reading the command stream
c
          if ( first_arg ) then                                           2b
              call get_token( prog_args, token )
              first_arg  = .FALSE.
          else                                                            2m
              if ( prompt_enable ) write( STD_OUT, 2 )
2                 format( ' ilg_cont>> ' )
              if ( .not. next_token( command_lu, token, TOK_EMPTYOK,
     1                               echo ) ) goto 16
              end if                                                      2e
          call casech( token, 'l' )
c
c data storage rules:
c  grid, abscissa, ordinate and grid data are stored in the array
c  "data_array". The pointers to the beginning of these data are
c  "grid_ptr", "absc_ptr", "ord_ptr" and "level_ptr". The number of
c  values is "n_grid", "n_absc", "n_ord" and "n_level". "empty_ptr"
c  is the index of the first unused place in "data_array".
c
c       read grid of altitude values
c
          if ( eqabbr( token, 'grid', 1 ) ) then                          2b
              start_ptr = empty_ptr
              n_pts = read_data( command_lu, data_array, empty_ptr,
     1                    prompt_enable, echo, 'altitude' )
              if ( n_pts .gt. 0 ) then                                    3b
                  call clear_data( data_array, grid_ptr, n_grid,
     1                             empty_ptr )
                  grid_ptr = start_ptr - n_grid
                  n_grid   = n_pts
                  end if                                                  3e
c
c           read abscissa values
c
          else if ( eqabbr( token, 'abscissa', 1 ) ) then                 2m
              start_ptr = empty_ptr
              n_pts = read_data( command_lu, data_array, empty_ptr,
     1                    prompt_enable, echo, 'abscissa' )
              if ( n_pts .gt. 0 ) then                                    3b
                  call clear_data( data_array, absc_ptr, n_absc,
     1                             empty_ptr )
                  absc_ptr = start_ptr - n_absc
                  n_absc   = n_pts
                  end if                                                  3e
c
c           read ordinate values
c
          else if ( eqabbr( token, 'ordinate', 2 ) ) then                 2m
              start_ptr = empty_ptr
              n_pts = read_data( command_lu, data_array, empty_ptr,
     1                    prompt_enable, echo, 'ordinate' )
              if ( n_pts .gt. 0 ) then                                    3b
                  call clear_data( data_array, ord_ptr, n_ord,
     1                             empty_ptr )
                  ord_ptr = start_ptr - n_ord
                  n_ord   = n_pts
                  end if                                                  3e
c
c           read contour levels
c
          else if ( eqabbr( token, 'level', 1 ) ) then                    2m
              start_ptr = empty_ptr
              n_pts = read_data( command_lu, data_array, empty_ptr,
     1                    prompt_enable, echo, 'contour level' )
              if ( n_pts .gt. 0 ) then                                    3b
                  call clear_data( data_array, level_ptr, n_level,
     1                             empty_ptr )
                  level_ptr = start_ptr - n_level
                  n_level   = n_pts
                  end if                                                  3e
c
c           x_clip   min_x max_x
c
          else if ( eqabbr( token, 'x_clip', 1 ) ) then                   2m
              if ( prompt_enable )
     1            write( STD_OUT, 3 ) 'minimum abscissa'
3             format( ' Clip limit : ', a, ' ? ' )
              if ( get_int( command_lu, i, 1, I_INFTY, echo ) ) then      3b
                  if ( prompt_enable )
     1                write( STD_OUT, 3 ) 'maximum abscissa'
                  if ( get_int( command_lu, j, 1, I_INFTY, echo))then     4b
                      min_x = min( i, j )
                      max_x = max( i, j )
                      end if                                              4e
                  end if                                                  3e
c
c           y_clip   min_y max_y
c
          else if ( eqabbr( token, 'y_clip', 1 ) ) then                   2m
              if ( prompt_enable )
     1            write( STD_OUT, 3 ) 'minimum ordinate'
              if ( get_int( command_lu, i, 1, I_INFTY, echo ) ) then      3b
                  if ( prompt_enable )
     1                write( STD_OUT, 3 ) 'maximum ordinate'
                  if ( get_int( command_lu, j, 1, I_INFTY, echo )
     1                    ) then                                          4b
                      min_y = min( i, j )
                      max_y = max( i, j )
                      end if                                              4e
                  end if                                                  3e
c
c           z_clip   min_z max_z
c
          else if ( eqabbr( token, 'z_clip', 1 ) ) then                   2m
              if ( prompt_enable )
     1            write( STD_OUT, 3 ) 'minimum altitude'
              if ( get_real( command_lu, r1, -R_INFTY, R_INFTY, echo )
     1                ) then                                              3b
                  if ( prompt_enable )
     1                write( STD_OUT, 3 ) 'maximum altitude'
                  if ( get_real( command_lu, r2, -R_INFTY, R_INFTY,
     1                    echo ) ) then                                   4b
                      min_z = min( r1, r2 )
                      max_z = max( r1, r2 )
                      end if                                              4e
                  end if                                                  3e
c
c
c             make/build the contours from the data
c
          else if ( eqabbr( token, 'make', 1 ) .or.
     1              eqabbr( token, 'build', 1 ) ) then                   2m
              if ( n_absc .lt. MIN_ABSC ) then                           3b
                  write( STD_ERR, 4 ) 'abscissa', n_absc, MIN_ABSC
4                 format( ' MAKE : ABORT : Insufficient ', a,
     1                ' data. Got ', i6, '. Need >= ', i6 )
              else if ( n_ord .lt. MIN_ORD ) then                        3m
                  write( STD_ERR, 4 ) 'ordinate', n_ord, MIN_ORD
              else if ( n_level .lt. MIN_LEVEL ) then                    3m
                  write( STD_ERR, 4 )
     1                'contour level', n_level, MIN_LEVEL
              else if ( n_grid .lt. n_absc * n_ord ) then                3m
                  write( STD_ERR, 4 )
     1                'altitude', n_grid, n_absc * n_ord
              else                                                       3m
                  if ( min_x .eq. 0 .or. max_x .eq. 0 ) then             4b
                      min_x = 1
                      max_x = n_absc
                      end if                                             4e
                  min_x = max( 1, min( min_x, n_absc - 1 ) )
                  max_x = min( max_x, n_absc )
                  if ( min_y .eq. 0 .or. max_y .eq. 0 ) then             4b
                      min_y = 1
                      max_y = n_ord
                      end if                                             4e
                  min_y = max( 1, min( min_y, n_ord - 1 ) )
                  max_y = min( max_y, n_ord )
c
c             sort abscissa and ordinate values into ascending order
c               sortsh( data, 1st_sorted_row, last_sorted_row, declared_rows,
c                       sorted_col, declared_cols)
                  call sortsh( data_array(absc_ptr),
     1                         n_absc, 1, n_absc, 1, 1 )
                  call sortsh( data_array(ord_ptr),
     1                         n_ord, 1, n_ord, 1, 1 )
c
c                 all data is passed to MAKE_CONTOURS through common block
c
                  call make_contours()
                  end if                                                 3e
c
c           clear
c            abscissa ordinate grid level x_clip y_clip z_clip none all help
c
          else if ( eqabbr( token, 'clear', 1 ) ) then                   2m
              if ( prompt_enable ) write( STD_OUT, 5 ) ' Clear'
5             format( a, ' ? (all...none/help) ' )
              if ( .not. next_token( command_lu, token, TOK_NOBLANK,
     1                               echo ) ) goto 16
              every = eqabbr( token, 'all', 2 )
c
              if ( eqabbr( token, 'help', 1 ) )
     1            write( STD_ERR, 6 )
6                 format( ' abscissa ordinate grid level ',
     1                    'x_clip y_clip z_clip none all' )
c
              if ( every .or. eqabbr( token, 'abscissa', 1 ) ) then      3b
                  call clear_data( data_array, absc_ptr, n_absc,
     1                             empty_ptr )
                  n_absc = 0
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'ordinate', 1 ) ) then      3b
                  call clear_data( data_array, ord_ptr, n_ord,
     1                             empty_ptr )
                  n_ord = 0
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'grid', 1 ) ) then          3b
                  call clear_data( data_array, grid_ptr, n_grid,
     1                             empty_ptr )
                  n_grid = 0
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'level', 1 ) ) then         3b
                  call clear_data( data_array, level_ptr, n_level,
     1                             empty_ptr )
                  n_level = 0
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'x_clip', 1 ) ) then        3b
                  min_x = 0
                  max_x = 0
                  end if                                                 3e
              if ( every .or. eqabbr( token, 'y_clip', 1 ) ) then        3b
                  min_y = 0
                  max_x = 0
                  end if                                                 3e
              if ( every .or. eqabbr( token, 'z_clip', 1 ) ) then        3b
                  min_z = -R_INFTY
                  max_z =  R_INFTY
                  end if                                                 3e
c
c           show
c            abscissa ordinate grid level x_clip y_clip z_clip none all help
c
          else if ( eqabbr( token, 'show', 2 ) ) then                    2m
              if ( prompt_enable ) write( STD_OUT, 5 ) ' Show'
              if ( .not. next_token( command_lu, token, TOK_NOBLANK,
     1                               echo ) ) goto 16
              every = eqabbr( token, 'all', 2 )
              if ( eqabbr( token, 'help', 1 ) )
     1            write( STD_ERR, 6 )
c
              if ( every .or. eqabbr( token, 'abscissa', 1 ) ) then      3b
                  write( STD_OUT, 7 ) n_absc, ' abscissa',
     1                ( data_array(i),
     2                  i = absc_ptr, absc_ptr + n_absc - 1 )
7                 format( / 1x, i5, a, ' values' / ( 1x, 5g13.5 ) )
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'ordinate', 1 ) ) then      3b
                  write( STD_OUT, 7 ) n_ord, ' ordinate',
     1                ( data_array(i),
     2                  i = ord_ptr, ord_ptr + n_ord - 1 )
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'grid', 1 ) ) then          3b
                  write( STD_OUT, 7 ) n_grid, ' altitude',
     1                ( data_array(i),
     2                  i = grid_ptr, grid_ptr + n_grid - 1 )
                  end if                                                 3e
c
              if ( every .or. eqabbr( token, 'level', 1 ) ) then         3b
                  write( STD_OUT, 7 ) n_level, ' contour level',
     1                ( data_array(i),
     2                  i = level_ptr, level_ptr + n_level - 1 )
                  end if                                                 3e
              if ( every .or. eqabbr( token, 'x_clip', 1 ) ) then        3b
                  write( STD_OUT, 8 ) min_x, max_x
 8                format( ' x_clip (minimum,maximum) = ', 2i6 )
                  end if                                                 3e
              if ( every .or. eqabbr( token, 'y_clip', 1 ) ) then        3b
                  write( STD_OUT, 9 ) min_y, max_y
 9                format( ' y_clip (minimum,maximum) = ', 2i6 )
                  end if                                                 3e
              if ( every .or. eqabbr( token, 'z_clip', 1 ) ) then        3b
                  write( STD_OUT, 10 ) min_z, max_z
 10               format( ' z_clip (minimum,maximum) = ', 2g13.5 )
                  end if                                                 3e
c
c       STOP / EXIT / QUIT
c
          else if ( eqabbr( token, 'stop', 2 ) .or.
     1              eqabbr( token, 'exit', 3 ) .or.
     2              eqabbr( token, 'quit', 1 ) ) then                     2m
              if ( out_lu .ne. STD_OUT .and.
     1             out_lu .ne. STD_ERR .and. out_lu .gt. 0 )
     2            close( UNIT=out_lu )
              return
c
c       PROMPTS / NOPROMPTS / INTERACTIVE
c
          else if ( eqabbr( token, 'prompts', 1 ) ) then                  2m
              prompt_enable = .TRUE.
          else if ( eqabbr( token, 'noprompts', 3 ) ) then                2m
              prompt_enable = .FALSE.
          else if ( eqabbr( token, 'interactive', 1 ) ) then              2m
              prompt_enable = .TRUE.
              execute = .FALSE.
              if ( command_lu .ne. STD_IN ) then                          3b
                  if ( lun_stack( LUN_PUSH, command_lu ) .gt. 0 )
     1                command_lu = STD_IN
                  end if                                                  3e
              command_file = ' '
c
c       HELP
c
          else if ( eqabbr( token, 'help', 1 ) ) then                     2m
              write( STD_OUT, 11 )
11            format( / ' Available commands (optional chars in []):' /
     1 '   exi[t]/q[uit]/st[op]     -- stop the program' /
     2 '   p[rompts]/nop[rompts]    -- (not) prompt for input' /
     3 '   wa[it]/now[ait]          -- (not) wait for CR' /
     4 '   ou[tput]  <output-file>  -- destination for output' /
     5 '   exe[cute] <command-file> -- take commands from file' /
     6 '   f[ormat]  <format-spec>  -- fortran output format spec' /
     7 '   i[nteractive]            -- take commands from user' /
     8 '   r[eturn]                 -- to previous input file' /
     9 '   h[elp] gu[ide] li[cence] w[arranty] -- user''s guide' /
     a '   g[rid]     <input_file>  -- read altitude data' /
     b '   a[bscissa] <input_file>  -- read abscissa data' /
     c '   or[dinate] <input_file>  -- read ordinate data' /
     d '   le[vel]    <input_file>  -- read contour levels' )
              write( STD_OUT, 12 )
 12           format(
     1 '   x[_clip] <min_x> <max_x> -- min & max grid columns used' /
     2 '   y[_clip] <min_y> <max_y> -- min & max grid row used' /
     3 '   z[_clip] <min_z> <max_z> -- min & max contour levels' /
     4 '   b[uild]/m[ake]           -- build contours from data' /
     5 '   c[lear]  <what>          -- clear data' /
     6 '   sh[ow]   <what>          -- print data to output file' /
     7 '   st[raightest] sm[allest] -- path selection criterion'/
     8 '   <what> = [n]one [a]bscissa [o]rdinate [g]rid [l]evel' /
     9 '            [x]_clip [y]_clip [z]_clip [all] [h]elp' /
     a ' Need new line after <input_file>. Can abbreviate commands.' )
c
c       WAIT FOR <RETURN>
c
          else if ( eqabbr( token, 'wait', 2 ) ) then                     2m
              cr_wait = .TRUE.
          else if ( eqabbr( token, 'nowait', 3 ) ) then                   2m
              cr_wait = .FALSE.
c
c       EXECUTE new command stream
c
          else if ( eqabbr( token, 'execute', 3 ) ) then                  2m
              if ( prompt_enable ) write( STD_OUT, 13 )
13                format( ' Command file name ? ' )
              if ( .not. next_token( command_lu, command_file,
     1                TOK_EMPTYOK, echo ) ) goto 16
              if ( str_length( command_file ) .gt. 0 ) then               3b
                  new_lu = lun_stack( LUN_PUSH, command_lu )
                  if ( new_lu .gt. 0 ) new_lu =
     1                    open_file( new_lu, command_file, 'old' )
                  if ( new_lu .gt. 0 ) then                               4b
                      command_lu = new_lu
                      execute = .TRUE.
                      prompt_enable = .FALSE.
                  else                                                    4m
                      call wait_for_CR()
                      end if                                              4e
                  end if                                                  3e
c
c           open output file
c
          else if ( eqabbr( token, 'output', 2 ) ) then                   2m
              if ( prompt_enable ) write( STD_OUT, 14 )
14                format( ' Output file name ? ' )
              if ( .not. next_token( command_lu, output_file,
     1                TOK_EMPTYOK, echo ) ) goto 16
              if ( output_file .eq. 'vdu' ) output_file = ' '
              if ( out_lu .ne. STD_OUT .and.
     1             out_lu .ne. STD_ERR .and. out_lu .gt. 0 )
     2            close( UNIT=out_lu )
              out_lu = open_file( RES_OUT_LU, output_file, 'unknown' )
              if ( out_lu .le. 0 ) then                                   3b
                  out_lu = STD_OUT
                  call wait_for_CR()
                  end if                                                  3e
c             initialise output file
              call write_contour( -I_INFTY, R_INFTY, R_INFTY, R_INFTY,
     1                 ' ' )
c
c           return to previous command file
c
          else if ( eqabbr( token, 'return', 1 ) ) then                   2m
              goto 16
c
c           user's guide
c
          else if ( eqabbr( token, 'guide', 2 ) ) then                    2m
              write( STD_OUT, 20 ) MAX_DATA
 20           format( / '         User''s guide for ILG_CONT' / /
     1  ' ILG_CONT is a pre-processor for ILGPLOT. ILG_CONT generates' /
     2  ' a script which ILG_PLOT uses to draw contour maps. The' /
     3  ' command languages for ILGPLOT and ILG_CONT are similar.' /
     4  ' ILG_CONT requires 4 data arrays which can be read from file' /
     5  ' or keyboard:' /
     6  '   1) height or altitude values in a rectangular GRID' /
     7  '   2) an ABSCISSA value for each point over the grid width' /
     8  '   3) an ORDINATE value for each point over the grid height' /
     9  '   4) a set of height LEVELS at which contours are drawn' /
     a  ' Total array space is ', i7, ' values. Arrays read from' /
     b  ' keyboard or command file are terminated by "end". Use' /
     c  ' "show" to look at your data. Use "output" to open an output' /
     d  ' file. Use "grid", "abscissa", "ordinate" and "level" to' /
     e  ' define the data arrays. Use "x_clip" and "y_clip" to' /
     f  ' restrict the region of grid actually used to make contours' //
     g  ' If you have problems, use "help".' )
c
c       PRINT WARRANTY
c
          else if ( eqabbr( token, 'warranty', 1 ) ) then                 2m
              write( STD_OUT, 21 )
21            format(
     1 'This program is distributed in the hope that it will be' /
     2 'useful, but WITHOUT ANY WARRANTY; without even the implied' /
     3 'warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR' /
     4 'PURPOSE.  See the GNU General Public License for more details.')
c
c       PRINT LICENSE
c
          else if ( eqabbr( token, 'license', 2 ) ) then                  2m
              write( STD_OUT, 22 )
22            format(
     1 'This program is free software; you can redistribute it and/or' /
     2 'modify it under the terms of the GNU General Public License' /
     3 'as published by the Free Software Foundation; either' /
     4 'version 1, or (at your option) any later version.' //
     5 'You should have received a copy of the GNU General Public' /
     6 'License along with this program; if not, write to the Free' /
     7 'Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,',
     8 'USA.' )
c
c           output format specifier
c
          else if ( eqabbr( token, 'format', 1 ) ) then                   2m
              if ( prompt_enable ) write( STD_OUT, 19 )
 19           format( ' FORTRAN output specifier ?' /
     1          ' Warning - use this feature at your own risk.' /
     2          ' Must use single Fw.d, Gw.d or Ew.d type specifier' )
              if ( .not. next_token( command_lu, token,
     1                TOK_EMPTYOK, echo ) ) goto 16
              if ( token .ne. ' ' .and.
     1             index( 'fgeFGE', token(1:1) ) .ne. 0 ) then            3b
                  i = str_length( token )
                  write( output_fmt, 18, ERR=1 ) token(:i), token(:i)
 18               format( '(1x,', a, ',1x,', a, ')' )
                  end if
c
c           contour path selection
c
          else if ( eqabbr( token, 'straightest', 2 ) ) then              2m
              selection_criterion = MINIMUM_ANGLE
          else if ( eqabbr( token, 'smallest', 2 ) ) then                 2m
              selection_criterion = MINIMUM_LENGTH
          else if ( token .ne. ';' .and. token .ne. ' ' ) then            2m
              write( STD_OUT, 15 )
     1            token( 1 : max( 1, str_length( token ) ) )
 15           format( ' ILG_CONT : illegal command ', a /
     1            ' Type "h <RETURN>" for help' )
              end if                                                      2e
          if ( CR_wait ) call wait_for_CR()
          goto 1
c
c             END OF FILE TRAP
c             ****************
c
 16   write( STD_OUT, 17 )
 17   format( ' ILG_CONT : Return / End of file' )
c
c     If no I/P files left on the input stack then goodbye
c
      if ( command_lu .eq. STD_IN .and.
     1        lun_stack( LUN_INDEX, 0 ) .le. 0 ) then                     1b
          return
      else                                                                1m
c
c             At least one file is still on the stack, so close the
c             previous file, pop the file from the stack & set flags
c
          if ( command_lu .ne. STD_IN )
     1        close( UNIT=command_lu )
          command_lu = lun_stack( LUN_POP, 0 )
          if ( command_lu .eq. STD_IN ) then                              2b
              prompt_enable = .TRUE.
              execute = .FALSE.
          else                                                            2m
              prompt_enable = .FALSE.
              execute = .TRUE.
              end if                                                      2e
          end if                                                          1e
      goto 1
      end



      INTEGER FUNCTION READ_DATA( COMMAND_LU, DATA_ARRAY, EMPTY_PTR,
     1                            PROMPT_ENABLE, ECHO, LABEL )
c
c Arguments
c  integer command_lu    # logical unit no. of command stream
c  real    data_array(*) # memory allocated for data
c  integer empty_ptr     # start of space available for new data
c  logical prompt_enable # enable prompts
c  logical echo          # echo inputs
c  character label*(*)   # type of data to be read e.g. ordinate
c
      include 'stdio.def'
      include 'ilg_cont.def'
      logical   echo, prompt_enable, next_token
      integer   command_lu, data_lu, empty_ptr,
     1          open_file, read_array, str_length
      real      data_array(*)
      character data_file*64, label*(*)
c     subroutine flush_token, wait_for_CR

c
c                Open data file
c
      read_data = 0
      if ( prompt_enable )
     1    write( STD_OUT, 1 ) label( : max( 1, str_length(label) ) )
1         format( ' File name for ', a, ' data ? ' )
      if ( next_token( command_lu, data_file, TOK_EMPTYOK, echo )
     1        ) then                                                      1b
          if ( data_file .ne. ' ' .and. data_file .ne. 'vdu' ) then       2b
              data_lu = open_file( RES_DATA_LU, data_file, 'old' )
              if ( data_lu .gt. 0 ) then                                  3b
c
c                 read formatted data file
c
                  call flush_token()
                  read_data = read_array( data_lu, data_array,
     1                                    empty_ptr, MAX_DATA, echo )
                  close( UNIT=data_lu )
              else                                                        3m
                  call wait_for_CR()
                  end if                                                  3e
c
c                 read data from command stream
c
          else                                                            2m
              if ( prompt_enable ) write( STD_OUT, 2 )
2             format( ' Enter the (x,y) coordinates ',
     1                '- terminate with ''end''' )
              call flush_token()
              read_data = read_array( command_lu, data_array,
     1                                empty_ptr, MAX_DATA, echo )
              end if                                                      2e
          end if                                                          1e
      return
      end



      SUBROUTINE PRINT_POINTERS()
      include 'stdio.def'
      include 'ilg_cont.def'
      include 'ilg_cont.cmn'
      write( STD_OUT, 1 ) absc_ptr, n_absc, ord_ptr, n_ord,
     1                    grid_ptr, n_grid, level_ptr, n_level
 1    format( ' Pointers size :' /
     1        '  abscissa ', 2i5 /
     2        '  ordinate ', 2i5 /
     3        '  grid     ', 2i5 /
     4        '  levels   ', 2i5 )
      return
      end



      INTEGER FUNCTION READ_ARRAY( INPUT_LU, ARRAY, EMPTY_PTR,
     1                             ARRAY_SIZE, ECHO )
c
c  Reads data from the file with logical unit number INPUT_LU file into
c  the array ARRAY starting at location EMPTY_PTR. ARRAY_SIZE marks the end
c  of the array. A ';' in the data file begins a comment field. Comment
c  fields are terminated by an end of line. Input format is list directed
c  ( ie. free ).
c
c   Arguments
c integer input_lu      # data file logical unit number
c real    array(*)      # data array
c integer empty_ptr     # start of useful array space
c         array_size    # maximum array index
c logical echo          # echo lines which generate an error
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer array_size, empty_ptr, index, input_lu, str_length
      logical echo, next_token
      real    array(*)
      character token*32
c     subroutine flush_token
c
c         read a token
c
      index = empty_ptr
1     if ( index .le. array_size ) then                                   1b
          if ( next_token( input_lu, token, TOK_NOBLANK, echo ) ) then    2b
              if ( token(1:1) .eq. ';' ) then                             3b
                  call flush_token()
              else if ( token(1:3) .eq. 'end' ) then                      3m
c
c                 have read the word 'end': legal finish to reading data
                  call flush_token()
                  read_array = index - empty_ptr
                  empty_ptr  = index
                  return
              else                                                        3m
c
c       decode the token
                  read( token, 2, ERR=3 ) array(index)
2                 format( bn, f32.0 )
                  index = index + 1
                  end if                                                  3e
c
c       token not available because of EOF: legal finish to reading data
          else                                                            2m
              call flush_token()
              read_array = index - empty_ptr
              empty_ptr  = index
              return
              end if                                                      2e
          goto 1
 3        write( STD_ERR, 4 ) token( : max( 1, str_length( token ) ) )
 4        format( ' READ_ARRAY : WARNING : Cannot decode ', a,
     1            ' . Value skipped.' )
          goto 1
          end if                                                          1e
c
c       data array full: Not enough space for data
      write( STD_ERR, 5 )
 5    format( ' READ_ARRAY : ABORT : Not enough space for data.',
     1        ' Suggestion: use "clear"' )
      call flush_token()
      read_array = 0
      return
      end




      SUBROUTINE CLEAR_DATA( ARRAY, START_CLEAR, N_CLEAR, START_EMPTY )
c
c Clears N_CLEAR elements of data in ARRAY from index START_CLEAR to
c START_EMPTY-1 .
c Arguments
c  real    array(*)      # data array
c  integer start_clear   # index : start of data to clear
c          n_clear       # number of items to clear
c          start_empty   # index : end of useful data + 1
c
c  Clears the data in ARRAY from
c  ARRAY(START_CLEAR) to ARRAY(START_CLEAR+N_CLEAR-1) by copying
c  ARRAY(START_CLEAR+N_CLEAR) ... ARRAY(START_EMPTY-1) to
c  ARRAY(START_CLEAR) ... ARRAY(START_EMPTY-N_CLEAR-1) .
c
      include 'stdio.def'
      include 'ilg_cont.def'
      integer dest, source, start_clear, n_clear, start_empty
      real    array(*)
      include 'ilg_cont.cmn'
c
      if ( n_clear .gt. 0 ) then
          dest = start_clear
          do 1 source = start_clear + n_clear, start_empty - 1
              array(dest) = array(source)
              dest = dest + 1
 1            continue
          start_empty = dest

c
c     adjust any data pointers which point to any of the "source" data
c
          source = start_clear + n_clear
          if ( absc_ptr .ge. source )
     1        absc_ptr = absc_ptr - n_clear
          if ( ord_ptr .ge. source )
     1        ord_ptr = ord_ptr - n_clear
          if ( grid_ptr .ge. source )
     1        grid_ptr = grid_ptr - n_clear
          if ( level_ptr .ge. source )
     1        level_ptr = level_ptr - n_clear
          write( STD_ERR, 2 ) n_clear
 2        format( ' CLEAR_DATA : ', i6, ' data values cleared' )
          end if
      return
      end



      SUBROUTINE MAKE_CONTOURS()
c
c  Makes the contours.  For each contour level, search through the
c  grid for starting points.  This is tricky. The order and direction
c  of search from each grid node is important.  The search is broken
c  into segments, each defined by a horizontal or vertical line of
c  nodes and the direction of a ray from each node.  At each node
c  along a segment START_CONTOUR searches for the start of a contour
c  and then finds the rest of the contour.
c

      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer i_grid, i_level, x1, x2, y1, y2
      logical quit, start_contour
c
      include 'ilg_cont.cmn'
c
c           This is the outer loop which is executed once for each
c           contour level. Contour levels are selected sequentially
c           from LEVEL
c
      do 3 i_level = level_ptr, level_ptr + n_level - 1                  1b
          if ( min_z .le. data_array(i_level) .and.
     1                    data_array(i_level) .le. max_z ) then          2b
c
c             For each contour level, start with a clean "done" list.
c
              do 1 i_grid = grid_ptr, grid_ptr + n_absc * n_ord - 1      3b
                  done(i_grid) = char( NO_EDGE )
1                 continue                                               3e
c
c           Need to step through all the altitude data (GRID) looking
c           for starting points for the contour level LEVEL(I_LEVEL)
c           The order of the search is important.  We go around the
c           boundary of the grid, working our way towards the middle.
c
c           define the boundaries of the search region
c           (x1,y1) = lower left corner,  (x2,y2) = upper right corner
              x1 = min_y
              y1 = min_x
              x2 = max_y
              y2 = max_x
c         start of main search loop
              quit = .FALSE.
2             if ( .not. quit ) then                                     3b
c
c             first we look at the 4 external edges of the grid
c             this is really only necessary on the first pass through the loop
c               -- bottom edge
                  quit = start_contour( x1, x2-1, y1, y1,
     1                       BOTTOM_EDGE, i_level )
c               -- right edge
                  quit = start_contour( x2, x2, y1, y2-1,
     1                       LEFT_EDGE, i_level ) .or. quit
c               -- top edge
                  quit = start_contour( x1, x2-1, y2, y2,
     1                       BOTTOM_EDGE, i_level ) .or. quit
c               -- left edge
                  quit = start_contour( x1, x1, y1, y2-1,
     1                       LEFT_EDGE, i_level ) .or. quit
c
c             Now we look at the internal rays from the grid nodes along
c             each edge.
c               -- bottom edge
                  quit = start_contour( x1+1, x2-1, y1, y1,
     1                       LEFT_EDGE, i_level ) .or. quit
c               -- right edge
                  quit = start_contour( x2-1, x2-1, y1+1, y2-1,
     1                       BOTTOM_EDGE, i_level ) .or. quit
c               -- top edge
                  quit = start_contour( x1+1, x2-1, y2-1, y2-1,
     1                       LEFT_EDGE, i_level ) .or. quit
c               -- left edge
                  quit = start_contour( x1, x1, y1+1, y2-1,
     1                       BOTTOM_EDGE, i_level ) .or. quit
c
c             bring the edges of the search-box in by one grid node
c             and repeat the search until one of the calls to
c             START_CONTOUR says "quit"
c
                  x1 = x1 + 1
                  y1 = y1 + 1
                  x2 = x2 - 1
                  y2 = y2 - 1
                  goto 2
                  end if                                                 3e
              end if                                                     2e
3         continue                                                       1e
      return
      end



      LOGICAL FUNCTION START_CONTOUR( FIRST_X, LAST_X, FIRST_Y, LAST_Y,
     1                                DIRECTION, I_LEVEL )
c
c  At each node along a segment START_CONTOUR searches for the start of a
c  contour and then finds the rest of the contour.
c
c Arguments
c  integer
c   first_x, last_x   # abscissa indices indicating search range in grid data
c   first_y, last_y   # ordinate indices indicating search range in grid data
c   direction         # LEFT_EDGE or BOTTOM_EDGE, direction of search
c   i_level           # index for contour level
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer direction, first_grid, first_x, first_y, grid_step,
     1        int_step, i_grid, i_level, last_grid, last_x, last_y,
     2        x_step, y_step
      real    x, y
      include 'ilg_cont.cmn'
c
      start_contour = .TRUE.
      x_step = last_x - first_x
      y_step = last_y - first_y
      if ( x_step .lt. 0 .or. y_step .lt. 0 ) then                       1b
c         Negative search direction is not an error condition but
c         can occur the size of the search box decreases to less
c         than zero and requires a QUIT signal
          continue
      else if ( 1 .gt. first_x .or. last_x .gt. n_absc ) then            1m
          write( STD_ERR, 1 ) first_x, last_x, n_absc
1         format( ' START_CONTOUR : ABORT : Illegal abscissa range. ',
     1            'Need 1 < ', i5, ' < ', i5, ' < ', i5 )
      else if ( 1 .gt. first_y .or. last_y .gt. n_ord ) then             1m
          write( STD_ERR, 2 ) first_y, last_y, n_ord
2         format( ' START_CONTOUR : ABORT : Illegal ordinate range. ',
     1            'Need 1 < ', i5, ' < ', i5, ' < ', i5 )
      else if ( x_step .ne. 0 .and. y_step .ne. 0 ) then                 1m
          write( STD_ERR, 3 ) x_step, ystep
3         format( ' START_CONTOUR : ABORT : Only one of X_STEP=',i5,
     1            ' and Y_STEP=', i5, ' can be non-zero' )
      else if ( direction .ne. LEFT_EDGE .and.
     1          direction .ne. BOTTOM_EDGE ) then                        1m
          write( STD_ERR, 4 ) direction
4         format( ' START_CONTOUR : ABORT : Illegal DIRECTION = ', i5 )
      else if ( i_level .lt. level_ptr .or.
     1          i_level .ge. level_ptr + n_level ) then                  1m
          write( STD_ERR, 5 ) i_level
5         format( ' START_CONTOUR : ABORT : Illegal I_LEVEL = ', i5 )
      else                                                               1m
c
c     generate QUIT signal if search range is zero
          start_contour = x_step .eq. 0 .and. y_step .eq. 0
c     determine grid increment for interpolation
          if ( direction .eq. LEFT_EDGE ) then                           2b
              int_step = n_absc
          else if ( direction .eq. BOTTOM_EDGE ) then                    2m
              int_step = 1
              end if                                                     2e
c     generate array index limits for the loop which steps along the
c     grid segment
          if ( y_step .gt. 0 ) then                                      2b
              grid_step = n_absc
          else                                                           2m
              grid_step = 1
              end if                                                     2e
          first_grid = grid_ptr + first_x - 1 + ( first_y - 1 ) * n_absc
          last_grid  = grid_ptr + last_x - 1 + ( last_y - 1 ) * n_absc
c
          do 10 i_grid = first_grid, last_grid, grid_step                2b
c
c             look at the edge indicated by DIRECTION
c             if a contour does not already cut this edge 
c
              if ( ichar( done(i_grid) ) .ne. direction .and. 
     1             ichar( done(i_grid) ) .ne. BOTTOM_LEFT ) then         3b
                  if ( data_array(i_grid) .le. data_array(i_level)
     1                 .eqv. data_array(i_level) .lt.
     2                       data_array(i_grid+int_step) ) then          4b
c
c             We have found two grid points along an edge which straddle
c             the required contour value.  Now write the first point of
c             the contour and then track the path of the contour.
c
                      init_grid = i_grid
                      init_edge = direction
                      call interpolate_grid( i_grid, i_grid + int_step,
     1                                       i_level, x, y )
                      call write_contour( out_lu, x, y,
     1                         data_array(i_level), output_fmt )
                      call track_contour( i_level, i_grid,
     1                                    direction, x, y )
c
c             List the starting edge of the contour as "occupied"
c
                      if ( ichar( done(i_grid) ) .eq. BOTTOM_EDGE .or.
     1                     ichar( done(i_grid) ) .eq. LEFT_EDGE   .or.
     2                     ichar( done(i_grid) ) .eq. BOTTOM_LEFT) then  5b
                          done(i_grid) = char( BOTTOM_LEFT )
                      else                                               5m
                          done(i_grid) = char( direction )
                          end if                                         5e
                      init_grid = 0
                      init_edge = NO_EDGE
                      end if                                             4e
                  end if                                                 3e
10            continue                                                   2e
          end if                                                         1e
      return
      end



      SUBROUTINE PRINT_DONE()
c
c Prints an ASCII map of the "occupied" grid edges.  Grid nodes are
c indicated with '.'.  Occupied edges are marked with '|' or '_'.
c
      include 'stdio.def'
      include 'ilg_cont.def'
      integer i, j
      character*2 left(144), bottom(144)
      include 'ilg_cont.cmn'
c
      do 1 i = 0, n_absc * n_ord - 1
          if ( ichar( done(i+grid_ptr) ) .eq. LEFT_EDGE .or.
     1         ichar( done(i+grid_ptr) ) .eq. BOTTOM_LEFT ) then
              left(i+1) = '| '
          else
              left(i+1) = '  '
              end if
          if ( ichar( done(i+grid_ptr) ) .eq. BOTTOM_EDGE .or.
     1         ichar( done(i+grid_ptr) ) .eq. BOTTOM_LEFT ) then
              bottom(i+1) = '._'
          else
              bottom(i+1) = '. '
              end if
 1        continue
      do 2 j = n_ord-1, 0, -1
          write( STD_OUT, 3 ) ( left(i), i= 1+j*n_absc, (1+j)*n_absc )
 3        format( 5x, 35a2 )
          write( STD_OUT, 4 ) j,
     1        ( bottom(i), i= 1+j*n_absc, (1+j)*n_absc )
 4        format( 1x, i3, 1x, 35a2 )
 2        continue
      return
      end



      SUBROUTINE TRACK_CONTOUR( I_LEVEL, START_GRID, START_EDGE,
     1                          FIRST_X, FIRST_Y )
c
c Track the contour level I_LEVEL across the grid from the edge defined
c by the grid location START_GRID and the direction START_EDGE.  The
c first point in the contour line is (FIRST_X,FIRST_Y).
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer edge, i, i_grid, i_level, n_list, prev_dirn,
     1        prev_grid, start_edge, start_grid, list_sel,
     2        search_edge, select_edge
      logical terminate_tracking(MAX_EDGE), quit
      real    x_hist(TRAK_MEM), y_hist(TRAK_MEM), first_x, first_y
      integer left_grid_step(MAX_EDGE), bottom_grid_step(MAX_EDGE),
     1        left_new_dirn(MAX_EDGE),  bottom_new_dirn(MAX_EDGE),
     2        dirn_list(MAX_EDGE),      grid_list(MAX_EDGE)
c
      include 'ilg_cont.cmn'
      data left_grid_step   / 0, -1, I_INFTY, I_INFTY, -1, 1 /
      data bottom_grid_step / 0, 1, I_INFTY, I_INFTY, I_INFTY, I_INFTY /
      data left_new_dirn    / BOTTOM_EDGE, BOTTOM_EDGE, BOTTOM_EDGE,
     1                        BOTTOM_EDGE, LEFT_EDGE,   LEFT_EDGE /
      data bottom_new_dirn  / LEFT_EDGE,   LEFT_EDGE,   LEFT_EDGE,
     1                        LEFT_EDGE,   BOTTOM_EDGE, BOTTOM_EDGE /
c
c N.B. test the mechanism in MAKE_CONTOURS for avoiding duplication of contours
c
c       prev_grid and prev_dirn are required in order to remember where
c       we have come from, so that we don't choose to go backwards along
c       the contour.  Copy initial values of working variables from 
c       subroutine arguments.
c
      prev_grid = 0
      prev_dirn = NO_EDGE
      edge = start_edge
      i_grid = start_grid
      do 1 i = 1, TRAK_MEM                                               1b
          x_hist(i) = first_x
          y_hist(i) = first_y
1         continue                                                       1e

      quit = .FALSE.
c
c           (re)initialise the BOTTOM_GRID_STEP which depend on grid
c           size N_ABSC (3, 4, 5 & 6) -- all the others are constants
c           N_ABSC should not change unless the abscissa data are changed
c
      bottom_grid_step(3) = -n_absc
      bottom_grid_step(4) = -n_absc + 1
      bottom_grid_step(5) =  n_absc
      bottom_grid_step(6) = -n_absc
c
c           (re)initialise the LEFT_GRID_STEP which depend on
c           grid size N_ABSC (3 & 4) -- all the others are constants
c           N_ABSC should not change unless the abscissa data are changed
c
      left_grid_step(3) = n_absc
      left_grid_step(4) = n_absc - 1
c
c     start of main tracking loop
c
2     if ( .not. quit ) then                                             1b
c
c    Clear the list of available contour intersections
c
          do 3 i = 1, MAX_EDGE                                           2b
              dirn_list(i) = NO_EDGE
              grid_list(i) = 0
              terminate_tracking(i) = .FALSE.
3             continue                                                   2e
          n_list = 1
c
c    Searching from a defined edge = BOTTOM_EDGE, find all possible
c    contour paths and store them in GRID_LIST and DIRN_LIST 
c      +---.    *   Current location
c      |   |   ===  Current edge direction
c      *===+   --- horizontal searches required
c      |   |    |  vertical searches required
c      +---+   + * possible new locations
c
          if ( edge .eq. BOTTOM_EDGE ) then                              2b
              do 4 i = 1, 6                                              3b
                  grid_list(n_list) = i_grid + bottom_grid_step(i)
                  if ( n_list .le. MAX_EDGE .and.
     1                 ( grid_list(n_list) .ne. prev_grid .or.
     2                   bottom_new_dirn(i) .ne. prev_dirn ) ) then      4b
                      dirn_list(n_list) = search_edge(
     1                    bottom_new_dirn(i), grid_list(n_list),
     2                    i_level, terminate_tracking(n_list) )
                      if ( dirn_list(n_list) .ne. NO_EDGE ) then         5b
                          n_list = n_list + 1
                          end if                                         5e
                      end if                                             4e
4                 continue                                               3e
c
c    Searching from a defined edge = LEFT_EDGE, find all possible
c    contour paths and store them in GRID_LIST and DIRN_LIST
c                   *   Current location
c      +---+---.    H   Current edge direction
c      |   H   |   --- horizontal searches required
c      +---*---+    |  vertical searches required
c                  + * possible new locations
c
          else if ( edge .eq. LEFT_EDGE ) then                           2m
              do 5 i = 1, 6                                              3b
                  grid_list(n_list) = i_grid + left_grid_step(i)
                  if ( n_list .le. MAX_EDGE .and.
     1                 ( grid_list(n_list) .ne. prev_grid .or.
     2                   left_new_dirn(i) .ne. prev_dirn ) ) then        4b
                      dirn_list(n_list) = search_edge(
     1                   left_new_dirn(i), grid_list(n_list),
     2                   i_level, terminate_tracking(n_list) )
                      if ( dirn_list(n_list) .ne. NO_EDGE ) then         5b
                          n_list = n_list + 1
                          end if                                         5e
                      end if                                             4e
5                 continue                                               3e
          else                                                           2m
              write( STD_ERR, 6 ) 'Undefined edge type',
     1            i_grid - grid_ptr + 1, i_level - level_ptr + 1
6             format( ' TRACK_CONTOUR : ABORT : ', a /
     1            11x, 'grid index    ', i6 /
     2            11x, 'contour level ', i6 /
     3            11x, 'terminate tracking' )
              quit = .TRUE.
              end if                                                     2e
          if ( n_list .gt. 1 ) then                                      3b
              list_sel = select_edge( grid_list, dirn_list, n_list-1,
     1                                x_hist, y_hist, i_level )
              call write_contour( out_lu, x_hist(3), y_hist(3),
     1                            data_array(i_level), output_fmt )
              prev_grid = i_grid
              prev_dirn = edge
              i_grid = grid_list(list_sel)
              edge   = dirn_list(list_sel)
c
c         Mark the newly discovered contour intersection, or "edge" as
c         as "occupied".
c
              if ( ichar( done(i_grid) ) .eq. BOTTOM_EDGE .or.
     1             ichar( done(i_grid) ) .eq. LEFT_EDGE ) then           5b
                  done(i_grid) = char( BOTTOM_LEFT )
              else                                                       5m
                  done(i_grid) = char( edge )
                  end if                                                 5e
              quit   = terminate_tracking(list_sel)
          else                                                           2m
              write( STD_ERR, 6 ) 'Unexpected end of contour',
     1            i_grid - grid_ptr + 1, i_level - level_ptr + 1
              quit = .TRUE.
              end if                                                     2e
          goto 2
          end if                                                         1e
c         end-of-main-tracking-loop
c
      call write_contour( out_lu, -R_INFTY, -R_INFTY, -R_INFTY, ' ' )
      return
      end




      REAL FUNCTION VECTOR_LENGTH( VECTOR )
c
c Length of a two-dimensional vector. Take precautions against overflow.
c
      include 'stdio.def'
      real vector(2), big, small
c
      if ( abs(vector(1)) .ge. abs(vector(2)) ) then
          big   = abs(vector(1))
          small = vector(2)
      else
          big   = abs(vector(2))
          small = vector(1)
          end if
      if ( big .eq. 0.0 ) then
          vector_length = 0.0
      else
          vector_length = big * sqrt( 1.0 + ( small / big ) **2 )
          end if
      return
      end



      REAL FUNCTION DOT_PRODUCT( A, B )
c
c Dot (or inner) product of two two-dimensional vectors.
c
      include 'stdio.def'
      real a(2), b(2)
c
      dot_product = a(1) * b(1) + a(2) * b(2)
      return
      end




      INTEGER FUNCTION SELECT_EDGE( GRID_LIST, DIRN_LIST, LIST_LEN,
     1                              X_HIST, Y_HIST, I_LEVEL )
c
c Selects the "best" from the list of feasible contour path increments
c given in GRID_LIST, DIRN_LIST.  The length of this list is LIST_LEN.
c (X_HIST,Y_HIST) is a history the TRAK_MEM most recent coordinates
c in the contour path.  For the present "best" means "select"
c  1) the least curved contour path i.e. the one with the least change in
c     direction between line segments,
c  2) the shortest new line segment.
c
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer actual_criterion, grid_list(MAX_EDGE),
     1        dirn_list(MAX_EDGE), list_len, i, i_level, grid_step
      real    x_hist(TRAK_MEM), y_hist(TRAK_MEM),
     1        x_list(MAX_EDGE), y_list(MAX_EDGE),
     2        vector_1(2), vector_2(2), length_1, length_2,
     3        cos_angle, max_cos_angle, min_length,
     4        vector_length, dot_product
      include 'ilg_cont.cmn'
c
      select_edge = 1
      if ( list_len .le. 0 ) then                                        1b
          write( STD_ERR, 1 )
     1        x_hist(3), y_hist(3), i_level - level_ptr + 1
1         format( ' SELECT_EDGE : BUG: no feasible contour path. ' /
     1            12x, 'LIST_LEN=0 at (X,Y)=(', 2g13.5,
     2            ') at contour number ', i5 )
      else                                                               1m
c
c       shuffle the remembered previous (x,y) values of the contour
c       back by one to make room for the new value
c
          do 2 i = 1, TRAK_MEM-1                                         2b
              x_hist(i) = x_hist(i+1)
              y_hist(i) = y_hist(i+1)
2             continue                                                   2e
c
c       if there is only one feasible path, we do not to optimise it
c
          if ( list_len .eq. 1 ) then                                    2b
c       select the correct interpolation direction
              if ( dirn_list(1) .eq. LEFT_EDGE ) then                    3b
                  grid_step = n_absc
              else                                                       3m
                  grid_step = 1
                  end if                                                 3e
c       calculate new path segment
              call interpolate_grid( grid_list(1),
     1                 grid_list(1)+grid_step, i_level,
     2                 x_list(1), y_list(1) )
          else                                                           2m
c
c       there at least two feasible contour paths,
c       calculate the first vector
c
              actual_criterion = selection_criterion
              if ( selection_criterion .eq. MINIMUM_ANGLE ) then         3b
                  vector_1(1) = x_hist(2) - x_hist(1)
                  vector_1(2) = y_hist(2) - y_hist(1)
                  length_1    = vector_length( vector_1 )
c
c       if the first vector is of zero length, we cannot use the minimum
c       angle criterion for selecting the contour path
c
                  if ( length_1 .le. 0.0 ) then                          4b
                      actual_criterion = MINIMUM_LENGTH
                      end if                                             4e
                  end if                                                 3e
              max_cos_angle = -2.0
              min_length = R_INFTY
c
c       Calculate all the possible new contour path segments, the vector
c       formed by the new segment and the angle between the new vector
c       and the old vector.  Choose the new vector which produces the
c       smallest change in direction.
c
              do 3 i = 1, list_len                                       3b
c       select the correct interpolation direction
                  if ( dirn_list(1) .eq. LEFT_EDGE ) then                4b
                      grid_step = n_absc
                  else                                                   4m
                      grid_step = 1
                      end if                                             4e
c       calculate new path segment
                  call interpolate_grid( grid_list(i),
     1                     grid_list(i)+grid_step,
     2                     i_level, x_list(i), y_list(i) )
c       calculate new vector and its length
                  vector_2(1) = x_list(i) - x_hist(2)
                  vector_2(2) = y_list(i) - y_hist(2)
                  length_2    = vector_length( vector_2 )
c    
c       For the minimum angle criterion
c       calculate "angle" between vectors
                  if ( actual_criterion .eq. MINIMUM_ANGLE ) then        4b
                      if ( length_2 .le. 0.0 ) then                      5b
                          cos_angle = -2.0
                      else                                               5m
                          cos_angle = dot_product( vector_1, vector_2 )
     1                          / length_1 / length_2
                          end if                                         5e
c       select minimum angle
                      if ( cos_angle .gt. max_cos_angle ) then           5b
                          max_cos_angle = cos_angle
                          select_edge = i
                          end if                                         5e
c
c       For the minimum length criterion
                  else if ( actual_criterion .eq. MINIMUM_LENGTH ) then  4m
                      if ( length_2 .lt. min_length ) then               5b
                          min_length = length_2
                          select_edge = i
                          end if                                         5e
                  else                                                   4m
                      select_edge = 1
                      end if                                             4e
3                 continue                                               3e
              end if                                                     2e
          end if                                                         1e
c       select returned contour coordinates
      x_hist(3) = x_list( select_edge )
      y_hist(3) = y_list( select_edge )
      return
      end




      INTEGER FUNCTION SEARCH_EDGE( DIRECTION, I_GRID,
     1                              I_LEVEL, TERMINATE_TRACKING )
c
c Search along the edge of the grid defined by DIRECTION and I_GRID for the
c contour level DATA_ARRAY(I_LEVEL).  The returned value of SEARCH_EDGE is
c LEFT_EDGE, BOTTOM_EDGE (indicating that the contour intersects a left or
c bottom edge) or NO_EDGE (indicating no intersection).
c
c Arguments
c  integer direction          # defines direction as LEFT_EDGE (vertical) or
c                             #   BOTTOM_EDGE (horizontal)
c          i_grid             # grid index defining lower left end of edge
c          i_level            # defines the required contour level
c  logical terminate_tracking # returned value; found end of contour --
c                             #   terminate tracking
c
c This algorithm is best expressed as a series of macros:
c search_edge = no_edge_found
c if ( direction == LEFT_EDGE )
c     if ( left_edge_in_grid )
c         if ( left_edge_intersection_found )
c             terminate_tracking_at_left_edge?
c             update_discovered_left_edges
c             search_edge = left_edge_found
c else ( direction == BOTTOM_EDGE )
c     if ( bottom_edge_in_grid )
c         if ( bottom_edge_intersection_found )
c             terminate_tracking_at_bottom_edge?
c             update_discovered_bottom_edges
c             search_edge = bottom_edge_found
c     
c
      include 'stdio.def'
      include 'ilg_cont.def'
      integer direction, first_col, first_row, i_grid, i_level,
     1        last_col, last_row
      logical terminate_tracking
c
      include 'ilg_cont.cmn'
c
c       calculate the DATA_ARRAY index of the first and last useful rows
c       in the same column as I_GRID
c
      first_row = mod( i_grid - grid_ptr, n_absc ) +
     1                ( min_y - 1 ) * n_absc + grid_ptr
      last_row  = first_row + ( max_y - min_y ) * n_absc
c
c       calculate the DATA_ARRAY index of the first and last useful columns
c       in the same row as I_GRID
c
      first_col = ( i_grid - grid_ptr ) / n_absc * n_absc
     1                + min_x - 1 + grid_ptr
      last_col  = first_col - min_x + max_x
c
c       SEARCH ALONG LEFT EDGE
c       Look for a contour level intersection vertically upward from
c       I_GRID -- but only if the EDGE is within the clipped grid and
c       is not already "occupied" by an intersection.
c
      terminate_tracking = .FALSE.
      search_edge = NO_EDGE
      if ( direction .eq. LEFT_EDGE ) then                               1b
          if ( first_row .le. i_grid .and.
     1         i_grid + n_absc .le. last_row .and.
     2         first_col .le. i_grid .and. i_grid .le. last_col .and.
     3         ichar( done(i_grid) ) .ne. LEFT_EDGE .and.
     4         ichar( done(i_grid) ) .ne. BOTTOM_LEFT ) then             2b
              if ( data_array(i_grid) .le. data_array(i_level) .eqv.
     1             data_array(i_level) .lt. data_array(i_grid+n_absc)
     2                ) then                                             3b
c
c       Terminate tracking because contour is at the left or right boundary
c       of the grid or it joined the end of the same contour.
c
                  terminate_tracking =
     1        i_grid .le. first_col .or. i_grid .ge. last_col .or.
     2        ( i_grid .eq. init_grid .and. init_edge .eq. direction )
                  search_edge = LEFT_EDGE
                  end if                                                 3e
              end if                                                     2e
c
c       SEARCH ALONG BOTTOM EDGE
c       Look for a contour level intersection horizontally to the right
c       of I_GRID -- but only if the EDGE is within the clipped grid
c       and is not already "occupied" by an intersection.
c
      else if ( direction .eq. BOTTOM_EDGE ) then                        1b
          if ( first_col .le. i_grid .and.
     1         i_grid + 1 .le. last_col .and.
     2         first_row .le. i_grid .and. i_grid .le. last_row .and.
     3         ichar( done(i_grid) ) .ne. BOTTOM_EDGE .and.
     4         ichar( done(i_grid) ) .ne. BOTTOM_LEFT ) then             2b
              if ( data_array(i_grid) .le. data_array(i_level) .eqv.
     1             data_array(i_level) .lt. data_array(i_grid+1) ) then  3b
c
c       Terminate tracking because contour is at the top or bottom boundary
c       of the grid or it joined the start of the same contour.
c
                  terminate_tracking =
     1        i_grid .le. first_row .or. i_grid .ge. last_row .or.
     2        ( i_grid .eq. init_grid .and. init_edge .eq. direction )
              search_edge = BOTTOM_EDGE
              end if                                                     3e
          end if                                                         2e
      else                                                               1m
          write( STD_ERR, 1 ) i_grid - grid_ptr + 1,
     1                        i_level - level_ptr + 1
 1        format( ' SEARCH_EDGE : ABORT : Undefined edge type at grid ',
     1        'index ', i6 / 23x, 'contour level ', i6 /
     2         23x, 'terminate tracking' )
          terminate_tracking = .TRUE.
          end if                                                         1e
      return
      end



      SUBROUTINE INTERPOLATE_GRID( I_GRID1, I_GRID2, I_LEVEL, X, Y )
c
c Interpolates the location of a point (X,Y) on the contour from grid locations
c I_GRID1 and I_GRID2, and the countour level I_LEVEL.
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer i_grid1, i_grid2, i_level
      real x, y, x1, y1, x2, y2, z_ratio
      include 'ilg_cont.cmn'
c
c       start of the interpolation segment
c
      x1 = data_array( absc_ptr + mod( i_grid1 - grid_ptr, n_absc ) )
      y1 = data_array( ord_ptr + ( i_grid1 - grid_ptr ) / n_absc )
c
c       end of the interpolation segment
c
      x2 = data_array( absc_ptr + mod( i_grid2 - grid_ptr, n_absc ) )
      y2 = data_array( ord_ptr + ( i_grid2 - grid_ptr ) / n_absc )
c
c       Calculate the ratio   z = ( z - z1 ) / ( z2 - z1 )
c       which is used for both X & Y. Watch out for divide-by-zero
c
      z_ratio = data_array(i_grid2) - data_array(i_grid1)
      if ( z_ratio .ne. 0.0 ) then
          z_ratio = ( data_array(i_level) - data_array(i_grid1) ) /
     1              z_ratio
c
c       finish the interpolation
c
          x = ( x2 - x1 ) * z_ratio + x1
          y = ( y2 - y1 ) * z_ratio + y1
      else
          x = ( x1 + x2 ) / 2.0
          y = ( y1 + y2 ) / 2.0
          end if
      return
      end


      SUBROUTINE WRITE_CONTOUR( OUT_LU, X, Y, Z, OUTPUT_FMT )
c
c Write the (X,Y) points defining the contour as an ILGPLOT script. Only one
c coordinate pair is written with each call to WRITE_CONTOUR.
c There are two special calls to WRITE_CONTOUR which do not write coordinates.
c     Header information is written to output file by calling WRITE_CONTOUR
c         with a negative OUT_LU.
c     Use X = Y = Z = -R_INFTY to terminate a contour line.
c
      include 'stdio.def'
      include 'ilg_cont.def'
      integer out_lu
      real    x, y, z, first_x, first_y, first_z
      logical first_call, first_point, first_stored
      character output_fmt*(*)
c
      save first_call, first_point, first_stored,
     1     first_x, first_y, first_z
      data first_call, first_point, first_stored /
     1     .TRUE., .TRUE., .FALSE. /
      data first_x, first_y, first_z /
     1     R_INFTY, R_INFTY, R_INFTY /
c
c Write header information
c
      if ( out_lu .lt. 0 ) then
          first_point = .TRUE.
          first_stored = .FALSE.
          first_call  = .TRUE.
c
c Terminate a contour line
c
      else if ( x .eq. -R_INFTY .and. y .eq. -R_INFTY .and.
     1     z .eq. -R_INFTY ) then
          if ( .not. first_point .and. .not. first_stored ) then
              write( out_lu, 1, ERR=5 )
 1            format( ' end'/ ' 1 1 max no' / ' ;' )
              end if
          first_point = .TRUE.
          first_stored = .FALSE.
c
c Don't write the first point; it might be the last; wait for more.
c
      else if ( first_point ) then
          first_point = .FALSE.
          first_x = x
          first_y = y
          first_z = z
          first_stored = .TRUE.
c
c OK, this must be at least the second point in a line.
c
      else
c         write the first coordinate of the line (+ header if required)
          if ( first_stored ) then
              if ( first_call ) then
                  write( out_lu, 2, ERR=5 )
 2                format( ' ; ' /
     1    ' ; This is an ILGPLOT script for producing a contour plot' /
     2    ' ; Written by ILG_CONT, author Peter V. Lanspeary' / ' ;' )
                  first_call = .FALSE.
                  end if
              write( out_lu, 3, ERR=5 ) first_z
 3            format( / ' ;' /
     1            ' ; contour level = ', g12.5 /
     2            ' ; drawn with a solid line' /
     3            ' line 0 1 1' /
     4            ' data vdu' )
              write( out_lu, output_fmt, ERR=5 ) first_x, first_y
              first_stored = .FALSE.
              end if
c         write the second and following points
          write( out_lu, output_fmt, ERR=5 ) x, y
c         format( 1x, g12.5, 1x, g12.5 )
          end if
      return
c
 5    write( STD_ERR, 6 ) out_lu
 6    format( ' WRITE_CONTOUR : ABORT : Cannot write to unit ', i6 )
      return
      end



ccccccccccccccccccccccccc start of general utilities ccccccccccccccccccccccc



      SUBROUTINE SORTSH( X, M1, M2, N, J1, M )
c                sortsh( data, 1st_sorted_row, last_sorted_row, declared_rows,
c                        sorted_col, declared_cols)
c
c               SORT ( SHell )
c       Subroutine for a shell sort into ascending or descending order .  The
c sorted array is an N x M matrix of N sets of Mtuples ( ie. N points in M
c dimensional space ) . Only the J1th column of the matrix is sorted into
c ascending or descending order , with the elements of the other columns
c undergoing the same exchanges . Sorting occurs between rows M1 and M2
c inclusive . If  M1 < M2 , the sort is descending . If M1 > M2 , the sort is
c ascending . Eg. for   SORTSH( X, 1, 4, 5, 1, 4 ) ,
c               2 4 3 1                 1 2 4 3
c               4 3 1 2                 2 4 3 1
c           X = 3 1 2 4     becomes X = 3 1 2 4
c               1 2 4 3                 4 3 1 2
c               3 1 2 4                 3 1 2 4
c
c real  x(n,m)          # operand array
c integer n,m,          # number of rows and columns in array
c         m1,m2,        # starting and finishing row indices
c         i1,i2,i3,j,   # working indices
c         j1,           # column to be sorted
c         id,           # sort segment size
c         m11,m12       # starting row & -1
c
      include 'stdio.def'
      integer n, nn, m, m1, m2, i1, i2, i3, j, j1, id, m11, m12
      real x(n,m)
c
      if ( m1 .eq. m2 .or. m1 .lt. 1 .or. m1 .gt. n .or. m2 .lt. 1 .or.
     1     m2 .gt. n .or.j1 .lt. 1 .or. j1 .gt. m ) then
          write( STD_ERR, 1 )
1         format( ' SORTSH : ABORT : Invalid Argument' )
          return
          end if
      m11 = min( m1, m2 )
      m12 = m11 - 1
      nn = abs( m2 - m1 ) + 1
      id = 2 ** int( alog( real( nn ) ) / alog( 2.0 ) + 0.0001 ) - 1
c     repeat
2         do 5 i1 = 1, nn-id
              do 4 i2 = i1+m12, m11, -id
                  i3 = i2 + id
                  if ( x(i2,j1) .le. x(i3,j1) ) goto 5
                  do 3 j = 1, m
                      call swap( x(i2,j), x(i3,j) )
3                     continue
4                 continue
5             continue
          id = id / 2
          if ( id .gt. 0 ) goto 2
c         until ( id <= 0 )
      if ( m1 .lt.  m2 ) then
          i1 = m1
          i2 = m2
c         repeat
6             do 7 j = 1, m
                  call swap( x(i1,j), x(i2,j) )
7                 continue
              i1 = i1 + 1
              i2 = i2 - 1
              if ( i1 .lt. i2 ) goto 6
c             until ( i1 >= i2 )
          end if
      return
      end



      SUBROUTINE SWAP( A, B )
      include 'stdio.def'
      real a, b, temp
c
      temp = a
      a = b
      b = temp
      return
      end



      LOGICAL FUNCTION GET_REAL( STREAM_LU, VALUE,
     1                           MINIMUM, MAXIMUM, ECHO )
c
c       reads a real number from the command stream
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer stream_lu, ntries
      logical echo, eqabbr, next_token
      real    value, maximum, minimum
      character string*32
c
c           read from command stream
c
      get_real = .TRUE.
      do 3 ntries = 1, MAX_TRIES                                          1b
          if ( .not. next_token( stream_lu,
     1                           string, TOK_NOBLANK, echo ) ) then       2b
c             end of file detected
              goto 7
          else                                                            2m
              call casech( string, 'l' )
c             user abort
              if ( string .eq. 'end' ) then                               3b
                  get_real = .FALSE.
c           selected  maximum
              else if ( eqabbr( string, 'maximum', 3 ) ) then             3m
                  value = maximum
c           selected  minimum
              else if ( eqabbr( string, 'minimum', 3 ) ) then             3m
                  value = minimum
c
c           decode & check numeric value
              else                                                        3m
                  read( string, 10, ERR=3 ) value
10                format( bn, f32.0 )
                  if ( value .lt. min( minimum, maximum ) .or.
     1                 value .gt. max( minimum, maximum ) ) then          4b
                      write ( STD_ERR, 1 ) minimum, maximum
1                     format(' GET_REAL : ERROR ; real outside limits ',
     1                    g15.7, ' & ', g15.7 )
                      if ( stream_lu .ne. STD_IN ) goto 7
                      write ( STD_OUT, 2 )
2                     format( ' Try again' )
c                     skip to end of loop
                      goto 3                                              4e
                      end if                                              3e
                  end if                                                  2e
              end if
          return
c
c           error detected
c
3         write( STD_OUT, 4 )
4         format( ' GET_REAL : Can''t decode that . Try again .' )
5         continue                                                        1e
      write( STD_OUT, 6 )
6     format( 8x, 'Get Help' )
c
c     end of file
7     get_real = .FALSE.
      return
      end



      LOGICAL FUNCTION GET_INT( STREAM_LU, VALUE,
     1                          MINIMUM, MAXIMUM, ECHO )
c
c       reads integer from the command stream
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer stream_lu, ntries, value, maximum, minimum
      logical echo, eqabbr, next_token
      character string*32
c
c           read from command stream
c
      get_int = .TRUE.
      do 3 ntries = 1, MAX_TRIES                                         1b
          if ( .not. next_token( stream_lu,
     1                           string, TOK_NOBLANK, echo ) ) then       2b
c             end of file detected
              goto 7
          else                                                            2m
              call casech( string, 'l' )
c             user abort
              if ( string .eq. 'end' ) then                               3b
                  get_int = .FALSE.
c           selected  maximum
              else if ( eqabbr( string, 'maximum', 3 ) ) then             3m
                  value = maximum
c           selected  minimum
              else if ( eqabbr( string, 'minimum', 3 ) ) then             3m
                  value = minimum
c
c           decode & check numeric value
              else                                                        3m
                  read( string, 10, ERR=3 ) value
10                format( bn, i32 )
                  if ( value .lt. min( minimum, maximum ) .or.
     1                 value .gt. max( minimum, maximum ) ) then          4b
                      write ( STD_ERR, 1 ) minimum, maximum
1                     format(' GET_INT : ERROR ; integer outside ',
     1                    'limits ', i12, ' & ', i12 )
                      if ( stream_lu .ne. STD_IN ) goto 7
                      write ( STD_OUT, 2 )
2                     format( ' Try again ' )
c                     skip to end of loop
                      goto 3                                              4e
                      end if                                              3e
                  end if                                                  2e
              end if
          return
c
c           error detected
c
3         write( STD_OUT, 4 )
4         format( ' GET_INT : Can''t decode that . Try again .' )
5         continue                                                        1e
      write( STD_OUT, 6 )
6     format( 8x, 'Get Help' )
c
c
c     end of file
7     get_int = .FALSE.
      return
      end



      INTEGER FUNCTION LUN_STACK( ACTION, NEW_LU )
c
c    logical unit number stack control
c
c Arguments
c  integer action     # LUN_PUSH   push NEW_LU onto stack and return new value
c                     # LUN_INDEX  return location of top of stack
c                     # LUN_POP    pop value from stack and return it
c          new_lun    # new logical unit pushed onto stack
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer stack(STACK_LEN), top, action, new_lu
      save stack, top
      data stack, top / STACK_LEN*0, 1 /
c
      lun_stack = 0
      top = max( 1, min( top, STACK_LEN + 1 ) )
      if ( action .eq. LUN_PUSH ) then                                    1b
          if ( top .le. STACK_LEN ) then                                  2b
              stack(top) = new_lu
              lun_stack = top + LUN_BASE
              top = min( top, STACK_LEN ) + 1
          else
              write( STD_ERR, 1 )
1             format( ' LUN_STACK : Stack is full' )
              end if                                                      2e
      else if ( action .eq. LUN_INDEX ) then                              1m
          lun_stack = top - 1
      else if ( action .eq. LUN_POP ) then                                1m
          top = max( 1, top - 1 )
          lun_stack = stack(top)                                          1e
      else
          write( STD_ERR, 2 )
2         format( ' LUN_STACK : Unknown ACTION' )
          end if
      return
      end



      INTEGER FUNCTION OPEN_FILE( LUN, FILE_NAME, NEW_OLD )
c
c     Opens a text file for reading ( NEW_OLD = 'old' ) or writing
c ( NEW_OLD = 'new' ). If FILE_NAME is '?', the name of the file is obtained
c from the user. If the file is for reading, the file name cannot be blank.
c If the file is for writing a blank filename is interpreted as the vdu screen.
c The logical unit used for the file is LUN. If the file is successfully opened,
c LUN is the returned result . If a file could not be opened, the returned value
c is OPEN_FILE = 0 .
c
c Arguments
c  integer   lun          # logical unit no. of file to be opened
c  character file_name*(*)# file name
c            new_old*(*)  # file status : 'old'=read, 'new'=write, or 'unknown'
c
c Local Variables
c  integer length         # length of file name string
c          str_length     # function : length of string
c character user_resp*64  # name of file - obtained from user
c
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer lun, length, str_length
      logical next_token
      character file_name*(*), new_old*(*), user_resp*64
c
c             check argument
c
      if ( new_old .ne. 'old' .and. new_old .ne. 'new' .and.
     1     new_old .ne. 'unknown' ) then
          write( STD_OUT, 1 ) new_old
1         format( ' OPEN_FILE : ABORT : Illegal status ', a /
     1            21x, 'Need ''old'', ''new'' or ''unknown''' )
          open_file = 0
      else if ( lun .lt. 0 .or. lun .ge. 100 ) then
          write( STD_OUT, 2 ) lun
2         format( ' OPEN_FILE : ABORT : Illegal logical unit number ',
     1            i12, '. Must be 0 to 99.' )
          open_file = 0
      else
          open_file = 0
c
c             get file name
c
3         if ( file_name .eq. '?' ) then
              length = 1
              write( STD_OUT, 4 )
4             format( ' Name of file ? ' )
              if ( .not. next_token( STD_IN, user_resp, TOK_EMPTYOK,
     1                               .FALSE. ) ) goto 6
          else
              user_resp = file_name
              end if
          length = str_length( user_resp )
          if ( length .gt. 0 ) then
              open( UNIT=lun, FILE=user_resp, STATUS=new_old,
     1              ACCESS='sequential', FORM='formatted', ERR=6 )
              open_file = lun
          else if ( new_old .ne. 'old' ) then
              open_file = STD_OUT
              end if
          end if
      return
c
c             TRAPS
c
6     write( STD_OUT, 7 ) user_resp(1:length)
7     format( ' OPEN_FILE : ABORT : Cannot open file ', a )
      open_file = 0
      return
      end



      INTEGER FUNCTION STR_LENGTH( STRING )
c
c    Find the location of the last non-blank character in the character
c string STRING . If the whole string is blank, the returned value is 0 .
c
      include 'stdio.def'
c
      character string*(*)
c
      str_length = len( string )
1     if ( str_length .ge. 1 ) then
          if ( string(str_length:str_length) .eq. ' ' ) then
              str_length = str_length - 1
          else
              return
              end if
          goto 1
          end if
      return
      end



      LOGICAL FUNCTION EQABBR( CSTR, CTEST, MINLEN )
c
c       EQual ABBReviation
c     Tests if the character string CSTR is a legal abbreviation of the
c character string CTEST .
c
c Arguments
c  character cstr*(*),         # operand character string               INPUT
c            ctest*(*)         # test string                            INPUT
c  integer minlen              # minimum number of characters allowed   INPUT
c                              # in the abbreviation
c Variables
c  integer i,       # indexing counter
c          iend     # first letter in a word
c
      include 'stdio.def'
c
      integer i, iend, minlen
      character cstr*(*), ctest*(*)
c
c         find end of the operand string
c
      iend = len( cstr )
1     if ( iend .gt. 1 .and. cstr(iend:iend) .eq. ' ' ) then
          iend = iend - 1
          goto 1
          end if
c
c         test
c
      if ( iend .gt. len( ctest ) ) then
          eqabbr = .false.
      else
          eqabbr = .true.
          i = 1
2         if ( i .le. iend .and. eqabbr ) then
              eqabbr = eqabbr .and. cstr(i:i) .eq. ctest(i:i)
              i = i + 1
              goto 2
              end if
          eqabbr = eqabbr .and. i .gt. minlen
          end if
      return
      end


      LOGICAL FUNCTION NEXT_TOKEN( STREAM_LU, TOKEN, TYPE, ECHO )
c
c     Get next token from command stream
c
c Arguments
c  integer   stream_lu
c  character token*(*)
c  integer   type
c  logical   echo
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer stream_lu, previous_lu, type, str_length
      logical echo
      character token*(*), line*128
c
      save previous_lu
      data previous_lu / STD_IN /
c
      if ( stream_lu .ne. previous_lu ) call flush_token()
      previous_lu = stream_lu
      next_token = .FALSE.
c
      if ( type .eq. TOK_TO_EOL ) then
          call get_token( char( TOK_TO_EOL ), token )
          if ( token .eq. ' ' ) read( stream_lu, 2, END=3 ) token
      else  if ( type .eq. TOK_NOBLANK .or.
     1           type .eq. TOK_EMPTYOK ) then
          call get_token( ' ', token )
1         if ( token .eq. ' ' .or. token(1:1) .eq. ';' ) then
              read( stream_lu, 2, END=3 ) line
2             format( a )
              call get_token( line, token )
              if ( type .eq. TOK_NOBLANK .or. token(1:1) .eq. ';' )
     1            goto 1
              end if
          end if
      next_token = .TRUE.
      if ( echo ) write( STD_OUT, 4 )
     1    token( 1 : max( str_length(token), 1 ) )
4     format( 1x, a )
3     return
      end



      SUBROUTINE GET_TOKEN( STRING, TOKEN )
c
c Extract the next token from STRING and place it in TOKEN . On the
c any call where STRING is not blank and only one character in length,
c STRING is saved and the search for a token starts at the beginning of
c the string . In a call where STRING = ' ' (and is one character long),
c the next token is extracted from the previously saved STRING . The
c maximum length of a saved string is 128 characters . If there are no more
c tokens in the string the returned value of TOKEN is ' '.
c
c Arguments
c  character string*(*)   # INPUT  # string containing tokens
c            token*(*)    # OUTPUT # returned token
c
c local variables
c  integer
c   len_saved         # non blank length of the saved string
c   start, finish     # start & finish indices in the saved string of the token
c   str_length        # function : non-blank length of string
c  character
c   saved*128         # saved string
c
      include 'stdio.def'
      include 'ilg_cont.def'
c
      integer len_saved, start, finish, str_length
      character saved*128, string*(*), token*(*)
c
      save len_saved, saved, start
      data saved / ' ' /
      data len_saved, start / 0, 1 /
c
      if ( string .eq. char( TOK_TO_EOL ) ) then
          if ( start .lt. len_saved .and. len_saved .gt. 0 ) then
              token = saved(start+1:len_saved)
              start = len_saved + 1
          else
              token = ' '
              end if
      else
          if ( len(string) .ne. 1 .or. string(1:1) .ne. ' ' ) then
              saved = string
              len_saved = str_length( string )
              start = 1
              end if
          if ( len_saved .le. 0 ) then
              token = ' '
          else
c
1             if ( start .le. len_saved ) then
                  if ( saved(start:start) .eq. ' ' ) then
                      start = start + 1
                      goto 1
                      end if
                  finish = start
2                 if ( finish .lt. len_saved ) then
                      if ( saved(finish+1:finish+1) .ne. ' ' ) then
                          finish = finish + 1
                          goto 2
                          end if
                      end if
                  token = saved(start:finish)
                  start = finish + 1
              else
                  token = ' '
                  end if
              end if
          end if
      return
      end



      SUBROUTINE FLUSH_TOKEN()
c
      include 'stdio.def'
c
      character flushed_token*1
c
1     call get_token( ' ', flushed_token )
      if ( flushed_token .ne. ' ' ) goto 1
      return
      end



      SUBROUTINE CASECH( STRING, CASE )
c
c       CASE CHange
c     Change the case of character string CSTR to upper, lower of edit case .
c
c Arguments
c  character string*(*),  # IN/OUT # operand character string
c            case*1       # INPUT  # 'l' or 'L' - change to lower case
c                                  # 'u' or 'U' - change to upper case
c                                  # 'e' or 'E' - change to edit case
c                                  # anyting else - no change
c local variables
c  integer i        # indexing counter
c  logical first    # first letter in a word
c
      include 'stdio.def'
c
      integer i, str_length
      logical first
      character string*(*), case*1
c
c
c         change to lower case
c
      if ( case .eq. 'l' .or. case .eq. 'L' ) then                        1b
          do 1 i = 1, str_length( string )                                2b
              if ( 'A' .le. string(i:i) .and. string(i:i) .le. 'Z' )
     1            string(i:i) = char(
     2                ichar( string(i:i) ) + ichar('a') - ichar('A') )
1             continue                                                    2e
c
c         change to upper case
c
      else if ( case .eq. 'u' .or. case .eq. 'U' ) then                   1m
          do 2 i = 1, str_length( string )                                2b
              if ( 'a' .le. string(i:i) .and. string(i:i) .le. 'z' )
     1            string(i:i) = char(
     2                ichar( string(i:i) ) + ichar('A') - ichar('a') )
2             continue                                                    2e
c
c         change to edit case
c
      else if ( case .eq. 'e' .or. case .eq. 'E' ) then                   1m
          first = .TRUE.
          do 3 i = 1, str_length( string )                                2b
              if ( 'A' .le. string(i:i) .and.
     1                      string(i:i) .le. 'Z' ) then                   3b
                  if ( .not. first )
     1                string(i:i) = char( ichar( string(i:i) ) +
     2                    ichar('a') - ichar('A') )
                  first = .FALSE.
              else if ( 'a'.le. string(i:i) .and.
     1                          string(i:i) .le.'z' ) then                3m
                  if ( first )
     1                string(i:i) = char( ichar( string(i:i) ) +
     2                    ichar('A') - ichar('a') )
                  first = .FALSE.
              else                                                        3m
                  first = .TRUE.
                  end if                                                  3e
3             continue                                                    2e
          end if                                                          1e
      return
      end


      SUBROUTINE WAIT_FOR_CR()
      include 'stdio.def'
c
      write( STD_OUT, 1 )
 1    format( ' Press <RETURN> to continue' )
      read( STD_IN, 2 )
 2    format()
      return
      end
