/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* ************************************************************************ */
/*                                                                          */
/*                              dev_font.h                                  */
/*                                                                          */
/*      Data type definitions for stroke-character fonts                    */
/*                                                                          */
/* ************************************************************************ */

#ifndef POINTER_DEFINED
#include POINTER_HEADER
#endif

#ifndef STD_TYPE_DEFINED
#include STD_TYPE_HEADER
#endif

#ifndef ERROR_DEFINED
#include ERROR_HEADER
#endif



/*
 *      STROKE-FONT CHARACTER DESCRIPTION
 *      *********************************
 *
 *  description of a single stroke-character
*/
typedef struct
  {
    char font_char;     /* font character                               */
    int  font_npts;     /* number of data points defining the character */
    float font_shift;   /* vertical coordinate displacement             */
    float font_midpt;   /* vertical midpoint of the character           */
    float VAR_PTR font_coord;   /* pointer to array of (x,y) data points*/
  }
    StrokeChar;

/*
 *  character font description
*/
typedef struct
  {
    StrokeChar VAR_PTR font_dir;    /* directory of character descriptions */
    boolean font_got;               /* character font file has been read   */
    int font_siz;                   /* number of characters */
    double font_hei;                /* height of characters */
    double font_wid;                /* width of characters  */
    double font_gap;                /* gap between characters */
  }
    StrokeFont;

/*
 *  stroke text attributes
*/
typedef struct
  {
    double height;      /* height of character in pixels                    */
    double aspect;      /* aspect ratio = width / height                    */
    double slant;       /* slant - expressed as shear factor                */
    double angle;       /* direction - radians clockwise from horizontal    */
    double gap;         /* space between chars - expressed in char widths   */
    struct              /* code letters for position coordinate of the      */
      {                 /*   character string written by `t_txtdrw'         */
        char horiz;     /*     horizontal position  'l', 'L' = left;        */
        char vert;      /*        'c', 'C' = centre;  'r', 'R' = right .    */
      }                 /*     vertical position  't', 'T' = top;           */
        align;          /*        'c', 'C' = centre;  'b', 'B' = bottom .   */
    boolean bold;       /* writing in bold-face font                        */
    boolean changed;    /* an attribute was changed since text was drawn    */
    StrokeFont VAR_PTR font_ptr;    /* pointer to the current char font     */
    double map[2][2];   /* transformation matrix for scaling etc            */
    double b_radius;    /* wobble radius for bold characters                */
    int b_passes;       /* number of passes for bold characters             */
  }
    Text_Attribute;


/*
 *      OPTIONS LISTS
 *      *************
 *
 *  Allowed data types of variables in an options list of type Option_Elem .
 *  The ``opt_vartyp'' value must be one of these . These values are used
 *  as an index into an array of format strings, so the values must not be
 *  changed.
*/
#define OPT_CHAR    0   /* character            */
#define OPT_SHORT   1   /* short signed integer */
#define OPT_INT     2   /* signed integer       */
#define OPT_LONG    3   /* long signed integer  */
#define OPT_FLOAT   4   /* single precision floating point */
#define OPT_DOUBLE  5   /* double precision floating point */
#define OPT_STRING  6   /* character string */
#define OPT_NUM_ARG -1.0e37 /* indicates the the value is to be obtained by */
                            /* decoding the value of the next token */
/*
 * element of the options list
*/
typedef struct
  {
    char VAR_PTR opt_string; /* word                           */
    short int opt_minlen;    /* minimum length of abbreviation */
    short int opt_vartyp;    /* data type of variable          */
    char VAR_PTR opt_varptr; /* pointer to static variable to be set */
    float opt_value;         /* required value of variable     */
  }
    Option_Elem;


/*
 *      DEVICE REGISTRATION
 *      *******************
 *
 *  Available actions for ``t_register()'' -- These are not for public use .
 * They are to be used only by ``t_begin()'' .
*/
#define T_REGISTER  -4  /* register the device   */
#define T_DEREGIST  -5  /* deregister the device */
#define T_REG_DATA  -6  /* return a pointer to the graphics status data  */
                        /* -- which is private and is not useful to user */
/*
 * special value returned by ``find_regist()'' if the required device was not
 * found
*/
#define T_NOT_REGIST ( (Register_Elem VAR_PTR) -8 )
/*
 * This is the data structure defines a single device in the device register .
 * Character pointers are assumed to be able to point to anything .
*/
typedef struct
  {
    FILE *       dev_file;  /* output file or device handle */
    int          dev_type;  /* one of device types defined in device_g.h */
    char VAR_PTR dev_data;  /* pointer to graphics status data structure */
  }
    Register_Elem;


/*
 *      functions in ``dev_comm.c''
*/
boolean clear_font(), map_font(), reset_font();
int  read_font(), bold_font(), t_strlen();
char VAR_PTR t_register(), VAR_PTR file_font(), VAR_PTR define_font(),
     VAR_PTR t_strcpy();

