/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef ERROR_DEFINED

#ifndef STD_TYPE_DEFINED
#include STD_TYPE_HEADER
#endif
/*
 *                      error.h
 *
 *      Permissible values of `status' - first argument of `make_error',
 *      `is_error' and `dump_error' .
 *   ERR_CALL  is used only with `make_error'
 *   ERR_ALL   is used with `is_error' or `dump_error'
 *   ERR_PRINT is a flag used only with `make_error'
*/
#define ERR_CALL   0    /* signals that a called function has returned with  */
                        /*   error status                                    */
#define ERR_ALL    0    /* detect or print any message                       */
#define ERR_NOTE   1    /* advice for user, not an error ; results are OK    */
#define ERR_WARN   2    /* execution continues but result may be wrong       */
#define ERR_ABORT  3    /* terminate execution of function without finishing */
#define ERR_FATAL  4    /* serious error ; print message and stop the program*/
#define ERR_PRINT  8    /* print the error on the error file ( which is set  */
                        /* by `file_error()' ) as well as queueing it        */

/*
 *      These values can be added to the `status' to indicate a math' error .
 *      The presence of one of these additional information to be queued .
*/
#define ERR_DOMAIN 16   /* Argument(s) not in domain of function  */
#define ERR_OVER   32   /* Result is greater than maximum allowed */
#define ERR_UNDER  48   /* Result is indistinguishable from 0.0   */
#define ERR_SING   64   /* Argument produces singularity          */
#define ERR_TLOSS  80   /* Argument produces total loss of significant digits */
#define ERR_PLOSS  96   /* Arg. produces partial loss of significant digits   */

void clear_error();
boolean make_error(), is_error(), file_error();
int dump_error();

#define ERROR_DEFINED 1
#endif
