/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


/* ************************************************************************* */
/*                                                                           */
/*                              std_type.h                                   */
/*                                                                           */
/*                        STANDARD TYPE DEFINITIONS                          */
/*                                                                           */
/* ************************************************************************* */

#ifndef STD_TYPE_DEFINED

/*
 *              BOOLEAN
 *            ***********
 * for legibility of code
*/
#define TRUE   1
#define FALSE  0
typedef int boolean;

#define BITS_PER_BYTE   8
#define LOG_2_10        3.32192809488
/*
 *              CHARACTERS
 *            **************
 * limiting values
*/
#define MAX_CHAR        0x7f
#define MIN_CHAR        0x00

/*
 *              INTEGERS
 *            ************
 * limiting values
*/
#define MAX_INT         ( (int) ( (unsigned int) -1 >> 1 ) )
#define MIN_INT         ( - MAX_INT )
#define MAX_INT_FIGS    \
    ( (int) ( (double) ( ( sizeof(int) * BITS_PER_BYTE - 1 ) / LOG_2_10 ) ) )

#define MAX_LONG_INT    ( (long int) ( (long unsigned int) -1L >> 1 ) )
#define MIN_LONG_INT    ( - MAX_LONG_INT )
#define MAX_LONG_FIGS   \
( (int) ( (double) ( ( sizeof(long int) * BITS_PER_BYTE - 1 ) / LOG_2_10 ) ) )

#define MAX_UNSIGNED        ( (unsigned int) -1 )
#define MIN_UNSIGNED        0
#define MAX_LONG_UNSIGNED   ( (unsigned int) -1L )
#define MIN_LONG_UNSIGNED   0
/*
 *              DOUBLE
 *            **********
 *      maximum and minimum values for floating point numbers
 *  Sorry folks . If you can think of a system-independent way of working these
 *  out from first principles, please let me know .
*/
#ifdef IEEE_FLOATING
#define MAX_DOUBLE      0.179769313486231E+309
#define MIN_DOUBLE      0.556268464629E-308
#define MAX_DEXPON      309     /* mantissa normalized to between 1.0 & 0.1 */
#define MIN_DEXPON     -308     /* mantissa normalized to between 1.0 & 0.1 */
#define DOUBLE_MANT_BITS 52     /* number of mantissa bits */
#define DOUBLE_SIG_FIGS  15

#define MAX_FLOAT       0.3402823E+39
#define MIN_FLOAT       0.2938736E-38
#define MAX_FEXPON      39      /* mantissa normalized to between 1.0 & 0.1 */
#define MIN_FEXPON     -38      /* mantissa normalized to between 1.0 & 0.1 */
#define FLOAT_MANT_BITS 23      /* number of mantissa bits */
#define FLOAT_SIG_FIGS   7
#else

#ifdef VAX_FLOATING
/*
 *              DEC just have to be different of course
*/
#define MAX_DOUBLE      0.89884656743115E+308
#define MIN_DOUBLE      0.556268464629E-308
#define MAX_DEXPON      308     /* mantissa normalized to between 1.0 & 0.1 */
#define MIN_DEXPON     -308     /* mantissa normalized to between 1.0 & 0.1 */
#define DOUBLE_MANT_BITS 52     /* number of mantissa bits */
#define DOUBLE_SIG_FIGS  15

#define MAX_FLOAT       0.1701411E+39
#define MIN_FLOAT       0.2938736E-38
#define MAX_FEXPON      39      /* mantissa normalized to between 1.0 & 0.1 */
#define MIN_FEXPON     -38      /* mantissa normalized to between 1.0 & 0.1 */
#define FLOAT_MANT_BITS 23      /* number of mantissa bits */
#define FLOAT_SIG_FIGS   7
#endif
#endif
/*
 *        FLOATING POINT PRECISION SWITCH
 *        ===============================
 *    The floating point data type is redefined as `floating' . If the value
 *    LESS_PRECISION is defined before the line including this file ( #include
 *    "stdlib.h" ), `floating' values are defined as `float' - single precision.
 *    If the value LESS_PRECISION is not defined, `floating' values are defined
 *    as `double' precision values .
*/
#ifdef LESS_PRECISION
typedef float  floating;
#define MAX_FLOATING MAX_FLOAT
#define MIN_FLOATING MIN_FLOAT
#define MAX_EXPON    MAX_FEXPON
#define MIN_EXPON    MIN_FEXPON
#define MANT_BITS    FLOAT_MANT_BITS
#define SIG_FIGS     FLOAT_SIG_FIGS
#else
typedef double floating;
#define MAX_FLOATING MAX_DOUBLE
#define MIN_FLOATING MIN_DOUBLE
#define MAX_EXPON    MAX_DEXPON
#define MIN_EXPON    MIN_DEXPON
#define MANT_BITS    DOUBLE_MANT_BITS
#define SIG_FIGS     DOUBLE_SIG_FIGS
#endif
#define INFINITY      MAX_FLOATING


typedef struct                  /*      COORDINATE PAIR DATA TYPE            */
  {                             /*      =========================            */
    floating _x;                /* x, abscissa                               */
    floating _y;                /* y, ordinate                               */
  }
    coord_pair;

typedef struct                  /*      COORDINATE TRIPLE DATA TYPE          */
  {                             /*      ===========================          */
    floating _x;                /* x                                         */
    floating _y;                /* y                                         */
    floating _z;                /* z                                         */
  }
    coord_triple;


#define STD_TYPE_DEFINED
#endif
