c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM CENTROID
c
c       Calculate the location of the centroid of a finite element from
c  ANSYS, given ,
c    1) a file containing NODE_NO, JUNK, X, Y, Z
c    2) a file containing ELEMENT_NO, JUNK, JUNK, JUNK, NODE_NO1, .... NODE_NO8
c    3) a file containing ELEMENT_NO, VELOCITY_X, VELOCITY_Y, VELOCITY_Z, SPEED
c  and rewrite the velocity file (3) with three extra columns containing
c  CENTROID_X, CENTROID_Y, CENTROID_Z
c
c       A line in which the first non-blank character is a ';' is a comment
c  is copied to the output file and then ignored . Each profile is terminated
c  by a line whic is not a comment line and contains only the word 'end' .
c
c constants
c  Hkbdlu               # keyboard logical unit no.
c  Hvdulu               # vdu logical unit no.
c  Hnodelu              # logical unit no. of node locations file
c  Helemlu              # logical unit no. of element definition file
c  Hvelilu              # logical unit no. of velocity definition file
c  Hvelolu              # logical unit no. of velocity locations file
c  Hmaxnodes            # maximum number of nodes
c  Hmaxelems            # maximum number of elements
c
c variables
c  character Cfile*64,          # file name
c            Cline*80           # line read from a file
c  integer *2 ielem,            # element no. from velocity description file
c             iline,            # input line counter
c             inode,            # node counter for element description file
c             ishape,           # node shape 1=oblong, 2=prism, 3=tetrahedron
c             ncentr,           # no. of values to be averaged to get centriod
c             elemptr,          # pointer to row in element description file
c             nelems,           # no. of elements in element description file
c             nnodes,           # number of nodes in node location file
c             nnode,            # number of a node belonging to an element
c             nodeptr,          # pointer to row in node location file
c             nodeno(Hmaxnodes),# node numbers from node location file
c             nodecross(Hmaxnodes),# node cross reference table, ( pointers )
c             npts,             # velocity value counter
c             elemno(Hmaxelems),# element numbers from element description file
c             elemcross(Hmaxelems),# element cross reference table, ( pointers )
c             enode(8,Hmaxelems),# nodes belongings to each element - from
c                               #  element description file
c  integer *4 jljstr            # function - left justify string
c             jresp             #          - integer response
c  real rjunk,                  # junk
c       xveloc, yveloc, zveloc, # velocity from velocity description file
c       speed,                  # air speed from velocity description file
c       xcentr, ycentr, zcentr, # calculated position of element centroid
c       xnode(Hmaxnodes),       ###
c       ynode(Hmaxnodes),       ### location of nodes - from node location file
c       znode(Hmaxnodes)        ###
c
c
      implicit none
      integer *2 Hkbdlu, Hvdulu, Hnodelu, Helemlu, Hvelilu, Hvelolu,
     1  Hmaxnodes, Hmaxelems
      parameter( Hkbdlu=5, Hvdulu=6, Hnodelu=11, Helemlu=12,
     1   Hvelilu=13, Hvelolu=14, Hmaxnodes=8000, Hmaxelems=8000 )
c
      character Cfile*64, Cline*80
      integer *2 iline, nnodes, nelems, npts, ielem, elemptr, ncentr,
     1  inode, nnode, nodeptr, nodeno(Hmaxnodes), nodecross(Hmaxnodes),
     2  elemno(Hmaxelems), elemcross(Hmaxelems), enode(8,Hmaxelems),
     3  ishape
      integer *4 jljstr, jresp
      real rjunk, xveloc, yveloc, zveloc, speed, xcentr, ycentr, zcentr,
     1  xnode(Hmaxnodes), ynode(Hmaxnodes), znode(Hmaxnodes)
      data nodecross, elemcross / Hmaxnodes*0, Hmaxelems*0 /
c
c               ELEMENT SHAPE
c
      write( Hvdulu, 7 )
7     format( ' What is the element shape ?  1) oblong' /
     1  30x, '2) triangular prism' / 30x, '3) tetrahedral       ', $ )
      ishape = Jresp( Hkbdlu, 1, 3, 3 )
c
c               NODE LOCATIONS
c
c       Open node location file
c
      write( Hvdulu, 1 )
1     format( ' Name of Node locations file ? ', $ )
      read( Hkbdlu, 2 ) Cfile
2     format( a )
      open( UNIT=Hnodelu, FILE=Cfile, STATUS='old' )
c
c       read node locations
c
      iline = 0
      nnodes = 1
c     start loop                                                         1b
3       iline = iline + 1
        read( Hnodelu, 2, END=6 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then     2b
          read( Cline, *, ERR=4 ) nodeno(nnodes), rjunk,
     1      xnode(nnodes), ynode(nnodes), znode(nnodes)
          nodecross( nodeno(nnodes) ) = nnodes
          nnodes = nnodes + 1
          end if                                                         2e
        goto 3
4        write( Hvdulu, 5 ) iline, Cline
5        format( ' ABORT : Cannot read node locations file at line ',
     1     i8 / 1x, a )
        goto 3
c       end of loop                                                      1e
6     nnodes = nnodes - 1
c
c             ELEMENT DEFINITIONS
c
c       Open element definition file
c
      write( Hvdulu, 11 )
11    format( ' Name of Element definition file ? ', $ )
      read( Hkbdlu, 2 ) Cfile
      open( UNIT=Helemlu, FILE=Cfile, STATUS='old' )
c
c       Read element definition file
c
      iline = 0
      nelems = 1
c     start loop                                                         1b
12      iline = iline + 1
        read( Helemlu, 2, END=15 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then     2b
          read( Cline, *, ERR=13 ) elemno(nelems), rjunk, rjunk, rjunk,
     1      ( enode(inode,nelems), inode = 1, 8 )
          elemcross( elemno(nelems) ) = nelems
          nelems = nelems + 1
          end if                                                         2e
        goto 12
13       write( Hvdulu, 14 ) iline, Cline
14       format( ' ABORT : Cannot read node locations file at line ',
     1     i8 / 1x, a )
        goto 12
c       end of loop                                                      1e
15    nelems = nelems - 1
c
c           CONVERTING VELOCITY DEFINITIONS FILE TO VELOCITY LOCATIONS FILE
c
c       Open velocity definition file and locations file
c
      write( Hvdulu, 20 )
20     format( ' Name of Velocity definition file ? ', $ )
      read( Hkbdlu, 2 ) Cfile
      open( UNIT=Hvelilu, FILE=Cfile, STATUS='old' )
      open( UNIT=Hvelolu, FILE=Cfile, STATUS='new' )
      write( Hvdulu, 21 ) Cfile
21    format( ' Velocity location file : ', a )
c
      iline = 0
      npts = 1
c     start loop                                                         1b
22      iline = iline + 1
c
c           read velocity data
c
        read( Hvelilu, 2, END=28 ) Cline
        if ( jljstr( Cline ) .gt. 0 ) then                               2b
          if ( Cline(1:1) .eq. ';' ) then                                3b
            write( Hvelolu, 2 ) Cline
          else                                                           3m
            read( Cline, *, ERR=26 ) ielem, xveloc, yveloc, zveloc,
     1        speed
            npts = npts + 1
c
c           find element in element description table
c
            elemptr = elemcross( ielem )
            if ( 0 .lt. elemptr .and. elemptr .le. nelems ) then         4b
c
c           find nodes for the element and calculated centroid
c
              ncentr = 0
              xcentr = 0.0
              ycentr = 0.0
              zcentr = 0.0
              do inode = 1, 8                                            5b
                if ( ishape .eq. 1 .or. ishape .eq. 2 .and.
     1              inode .ne. 4 .and. inode .ne. 8 .or. ishape .eq. 3
     2              .and. inode .ne. 4 .and. inode .lt. 6 ) then         6b
                  nnode = enode( inode, elemptr )
                  nodeptr = nodecross( nnode )
                  if ( 0 .lt. nodeptr .and. nodeptr .le. nnodes ) then   7b
                    xcentr = xcentr + xnode(nodeptr)
                    ycentr = ycentr + ynode(nodeptr)
                    zcentr = zcentr + znode(nodeptr)
                    ncentr = ncentr + 1
                  else                                                   7m
                    write( Hvdulu, 24 ) nnode
24                  format( ' Cannot find node ', i4,
     1                ' in node location table' )
                    end if                                               7e
                  end if                                                 6e
                end do                                                   5e
              xcentr = xcentr / ncentr
              ycentr = ycentr / ncentr
              zcentr = zcentr / ncentr
c
c           write velocity location file
c
              write( Hvelolu, 25 ) ielem, xveloc, yveloc, zveloc,
     1          speed, speed **2 / -2.0, xcentr, ycentr, zcentr
25            format( 1x, i4, 8g13.5 )
            else                                                         4m
              write( Hvdulu, 23 ) ielem
23            format( ' Cannot find element ', i8,
     1          ' in description table' )
              end if                                                     4e
            end if                                                       3e
          end if                                                         2e
        goto 22
c
c
26        write( Hvdulu, 27 ) iline, Cline
27        format( ' Cannot read velocity definition file at line ', i8 /
     1      1x, a )
        goto 22
c       end of loop                                                      1e
28    stop
      end

