c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM CHEERS_PROFILE
c
c       Calculate the profile of the contraction of the boundary layer
c  wind tunnel according to the method of Cheers .
c
c  Reference : Cheers F.,  Note on Wind Tunnel Contractions,
c                 Aeronautical Research Council Reports and Memoranda,
c                 No. 2137, March 1945 .
c
      include 'stdio.def'
      integer   MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      integer   calc_enable, iargc, item, par_lu, prf_lu, out_lu,
     1          Jresp, new_lun
      logical   read_profile, Equery
      character out_file*32, profile_file*32
      data      out_file, profile_file / ' ', 'contraction.prf' /
c
      calc_enable = YES
      par_lu = new_lun()
      prf_lu = new_lun()
      out_lu = new_lun()
      if ( iargc() .gt. 0 ) then                                          1b
          call getarg( 1, profile_file )
          if ( iargc() .gt. 1 ) then                                      2b
              call getarg( 1, out_file )
              end if                                                      2e
          end if                                                          1e
      if ( .not. read_profile( prf_lu, profile_file ) ) stop
      call set_default_params()
      call calc_results()
c
c     start of loop                                                       1b
1         call new_scr()
          write( STD_OUT, 2 ) out_file
2         format( ' Cheers wind tunnel contraction' /
     Z        ' 0) Exit' /
     1        ' 1) Set parameters' /
     2        ' 2) Output file    ', a /
     3        ' 3) Write results' /
     4        ' 4) Display results' /
     5        ' 5) Plot results' /
     6        ' 6) Help' )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
          if ( item .eq. 0 ) then                                         2b
              if ( Equery( STD_IN, 'Are you sure ?' ) ) stop
          else if ( item .eq. 1 ) then                                    2m
              call set_parameters( par_lu, calc_enable )
              if ( calc_enable .eq. YES ) call calc_results()
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Output file ? ' )
              call cresp( STD_IN, out_file, MAX_TRIES )
          else if ( item .eq. 3 ) then                                    2m
              call write_results( out_lu, out_file )
          else if ( item .eq. 4 ) then                                    2m
              call write_results( STD_OUT, ' ' )
          else if ( item .eq. 5 ) then                                    2m
              call plot_results()
          else if ( item .eq. 6 ) then                                    2m
              call give_help()
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE GIVE_HELP()
      include 'stdio.def'
      write( STD_OUT, 1 )
1     format( ' Calculation of a wind tunnel contraction profile' /
     1  ' according to the formula of ' /
     2  '   F. Cheers (1945) : Note on Wind Tunnel Contractions,' /
     3  '     Aeronautical Research Council Reports and Memoranda,'
     4  '          No. 2137, March 1945.' / /
     5  ' Synopsis :   cheers [profile_file [output_file]]' /
     6  '   profile_file = file containing actual contraction profile'/
     7  18x, 'Each line may be a' /
     8  20x, 'comment -- beginning with a ";"' /
     9  20x, 'units specifier -- one of "inches", "feet", "mm" or ',
     A       '"metres" (default is mm)' /
     B  20x, 'x y data  consisting of two numbers x and y' /
     C  22x, 'x = distance from start of contraction' /
     D  22x, 'y = distance from centreline' /
     E  18x, 'N.B. units specifier precedes data' /
     F  '   output_file  = name of output file' )
      return
      end


      LOGICAL FUNCTION READ_PROFILE( RESERV_LU, PROFILE_FILE )
c
c   Read contraction profile file into global arrays.
c
c parameters
c  reserv_lu           # logical unit reserved for input file
c  profile_file        # name of input file containing contraction profile
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer   reserv_lu
      character profile_file*(*)
c
      integer   iline, prf_lu, open_file, str_length
      real      x, y, units_factor
      character line*256, token*32
      include   'cheers.cmn'
c
      data      units_factor / 1.0 /
      data      line, token / ' ', ' ' /
c
c           open profile file for input
c
      read_profile = .FALSE.
      if ( profile_file .eq. ' ' .or. reserv_lu .le. 0 ) then             1b
          write( STD_ERR, 1 )
1         format( 'READ_PROFILE : FATAL : Undefined profile file or ',
     1            'logical unit number' )
      else                                                                1m
          prf_lu = open_file( reserv_lu, profile_file, 'old' )
          if ( prf_lu .le. 0 ) then                                       2b
              write( STD_ERR, 2 ) profile_file
2             format( 'READ_PROFILE : FATAL : Cannot open profile ',
     1                'file ', a )
          else                                                            2m
c
c           read contraction profile
c
              n_profile  = 0
              iline = 0
3             if ( n_profile .lt. MAX_PROFILE ) then                      3b
                  iline = iline + 1
                  read( prf_lu, 4, END=7, ERR=8 ) line
4                 format( a )
                  if ( str_length( line ) .gt. 0 ) then                   4b
                      call get_token( line, token )
                      if ( token .eq. 'metres' ) then                     5b
                          units_factor = 1.0
                      else if ( token .eq. 'mm' ) then                    5m
                          units_factor = 0.001
                      else if ( token .eq. 'inches' ) then                5m
                          units_factor = 0.0254
                      else if ( token .eq. 'feet' ) then                  5m
                          units_factor = 0.3048
                      else if ( token(1:1) .ne. ';' ) then                5m
                          read( token, *, ERR=5 ) x
                          call get_token( ' ', token )
                          if ( token .ne. ' ' ) then                      6b
                              read( token, *, ERR=5 ) y
                              n_profile = n_profile + 1
                              profile(n_profile,PRF_X) = x*units_factor
                              profile(n_profile,PRF_Y) = y*units_factor
                              end if                                      6e
                          end if                                          5e
                      end if                                              4e
                  goto 3
c
c             Trap for decoding error
c
5                 write( STD_ERR, 6 ) token(:str_length(token)), iline
6                 format( ' READ_PROFILE : WARNING : Cannot decode ',
     1                a, ' at line ', i5, ' Skip line' )
                  goto 3
                  end if                                                  3e
c
c             got to end of file
c
7             close( UNIT=prf_lu )
              read_profile = .TRUE.
c       ascending sortsh( array, last_row, 1st_row, n_rows, sort_col, n_cols )
              call sortsh( profile, n_profile, 1, MAX_PROFILE, 1, 2 )
              return
8             write( STD_ERR, 9 ) iline
9             format( ' READ_PROFILE : ABORT : Cannot read line ', i5 )
              end if                                                      2e
          end if                                                          1e
      return
      end



      REAL FUNCTION GET_PROFILE( X )
c
c       Evaluate a point on the wind tunnel contraction by linear interpolation
c  from the profile data.
c
c Argument
c  real x              # distance from contraction inlet
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer index, jdhunt
      real x
      include 'cheers.cmn'
c
c
      if ( x .le. profile(1,PRF_X) ) then                                 1b
          get_profile = profile(1,PRF_Y)
      else if ( x .ge. profile(n_profile,PRF_X) ) then                    1m
          get_profile = profile(n_profile,PRF_Y)
      else                                                                1m
          index = jdhunt( profile(1,PRF_X), 1, n_profile, -999, x )
          get_profile =
     1        ( profile(index+1,PRF_Y) - profile(index,PRF_Y) ) /
     2        ( profile(index+1,PRF_X) - profile(index,PRF_X) ) *
     3        ( x - profile(index,PRF_X) ) + profile(index,PRF_Y)
          end if                                                          1e
      return
      end


      SUBROUTINE SET_PARAMETERS( RESERV_LU, CALC_ENABLE )
c
c Set all parameters required for calculation of Cheers contraction profile.
c
c Arguments
c  integer reserv_lu     # logical unit number reserved for parameters file
c          calc_enable   # NO/YES - enable calculation of Cheers profile
c
      include 'stdio.def'
      include 'cheers.def'
      integer MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      integer reserv_lu, calc_enable, item, par_lu, Jresp, open_file
      real Qresp
      character no_yes(0:1)*5, param_file*32
      include 'cheers.cmn'
c
      data no_yes, param_file / 'no   ', 'yes  ', ' ' /
c
c
c     start of loop                                                       1b
10        call new_scr()
          c_ratio = a / b
          write( STD_OUT, 1 ) a, b, c_ratio, n_cheers, x_first, x_last,
     1            y_tolerance, no_Ls, theta_ref, no_yes(calc_enable),
     2            param_file
1         format( ' Calculation parameters.' / '  0) exit' /
     1      '   1) inlet height            a = ', f8.4, ' m' /
     2      '   2) outlet height           b = ', f8.4, ' m' /
     3      '      contraction ratio         =  ', f7.3 /
     4      '   3) number of profile points  =  ', i3 /
     5      '   4) inlet location          x = ', f8.3, ' m' /
     6      '   5) outlet location         x = ', f8.3, ' m' /
     7      '   6) ordinate tolerance ratio  = ', f7.4, ' %' /
     8      '   7) number of L coefficients  =  ', i3 /
     9      '   8) profile shift ref angle   = ', f7.4, ' radians' /
     a      '   9) enable calculations       = ', a /
     b      '  10) set sensible defaults' /
     c      '  11) parameters file           = ', a /
     d      '  12) read parameters from file' /
     e      '  13) write parameters to file' )
          item = Jresp( STD_IN, 0, 13, MAX_TRIES )
          if ( 1 .le. item .and. item .le. 8 ) then                       2b
              write( STD_OUT, 2 )
2             format( ' Value ? ' )
              end if                                                      2e
          if ( item .eq. 0 ) then                                         2b
              if ( b .gt. a ) call swap( a, b )
              if ( x_first .gt. x_last ) call swap( x_first, x_last )
              c_ratio = a / b
              new_params = .TRUE.
              return
c
c             inlet height
c
          else if ( item .eq. 1 ) then                                    2m
              a = Qresp( STD_IN, 0.001, 10.0, MAX_TRIES )
c
c             outlet height
c
          else if ( item .eq. 2 ) then                                    2m
              b = Qresp( STD_IN, 0.001, 10.0, MAX_TRIES )
c
c             number of profile points
c
          else if ( item .eq. 3 ) then                                    2m
              n_cheers = Jresp( STD_IN, 3, MAX_CHEERS, MAX_TRIES )
c
c             inlet location
c
          else if ( item .eq. 4 ) then                                    2m
              x_first = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES )
c
c             outlet location
c
          else if ( item .eq. 5 ) then                                    2m
              x_last = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES )
c
c             ordinate tolerance ratio
c
          else if ( item .eq. 6 ) then                                    2m
              y_tolerance = Qresp( STD_IN, 0.001, 100.0, MAX_TRIES )
c
c             number of L coefficients
c
          else if ( item .eq. 7 ) then                                    2m
              no_Ls = Jresp( STD_IN, 2, MAX_L, MAX_TRIES )
c
c             profile shift ref angle
c
          else if ( item .eq. 8 ) then                                    2m
              theta_ref = Qresp( STD_IN, 0.0, PI, MAX_TRIES )
c
c             enable calculations
c
          else if ( item .eq. 9 ) then                                    2m
              calc_enable = mod( calc_enable + 1, 2 )
c
c             set sensible defaults
c
          else if ( item .eq. 10 ) then                                   2m
              call set_default_params()
c
c             parameter file
c
          else if ( item .eq. 11 ) then                                   2m
              write( STD_OUT, 3 )
3             format( ' Name of parameter file ? ' )
              call Cresp( STD_IN, param_file, MAX_TRIES )
c
c             read parameters from file
c
          else if ( item .eq. 12 ) then                                   2m
              par_lu = open_file( reserv_lu, param_file, 'old' )
              if ( par_lu .gt. 0 ) then                                   3b
                  read( par_lu, *, ERR=5, END=5 ) a
                  read( par_lu, *, ERR=5, END=5 ) b
                  read( par_lu, *, ERR=5, END=5 ) n_cheers
                  read( par_lu, *, ERR=5, END=5 ) x_first
                  read( par_lu, *, ERR=5, END=5 ) x_last
                  read( par_lu, *, ERR=5, END=5 ) y_tolerance
                  read( par_lu, *, ERR=5, END=5 ) no_Ls
                  read( par_lu, *, ERR=5, END=5 ) theta_ref
                  close( UNIT=par_lu )
                  end if                                                  3e
c
c             write parameters to file
c
          else if ( item .eq. 13 ) then                                   2m
              par_lu = open_file( reserv_lu, param_file, 'unknown' )
              if ( par_lu .gt. 0 ) then                                   3b
                  write( par_lu, 4, ERR=7 ) a, b, n_cheers, x_first,
     1                x_last, y_tolerance, no_Ls, theta_ref
4                 format( 1x, f8.4, ' ; inlet height' /
     1                    1x, f8.4, ' ; outlet height' /
     2                    1x, i3, 6x, '; number of profile points' /
     3                    1x, f8.3, ' ; inlet location' /
     4                    1x, f8.3, ' ; outlet location' /
     5                    1x, f7.4, '  ; ordinate tolerance ratio' /
     6                    1x, i3, 6x, ' ; number of L coefficients' /
     7                    1x, f7.4, '  ; profile shift ref angle' )
                  if ( par_lu .ne. STD_OUT .and. par_lu .ne. STD_ERR )
     1                close( UNIT=par_lu )
                  end if                                                  3e
              end if                                                      2e
          goto 10
c
5         close( UNIT=par_lu)
          write( STD_ERR, 6 ) 'read', param_file
6         format( ' SET_PARAMETERS : WARNING : Cannot ', a,
     1            ' parameters file ', a )
          goto 10
c
7         if ( par_lu .ne. STD_OUT .and. par_lu .ne. STD_ERR )
     1        close( UNIT=par_lu)
          write( STD_ERR, 6 ) 'write', param_file
          goto 10
c         end of loop                                                     1e
      end



      SUBROUTINE SET_DEFAULT_PARAMS()
c
c  These are the parameter values required to produce the correct results
c
      include 'stdio.def'
      include 'cheers.def'
c
      include 'cheers.cmn'
c
      a        = 1.125
      b        = 0.2286
      c_ratio  = a / b
      n_cheers = 50
      x_first  = profile(1,PRF_X)
      x_last   = profile(n_profile,PRF_X)
      y_tolerance = 0.1
      no_Ls    = 10
      theta_ref= PI / 2.0
      new_params = .TRUE.
      return
      end



      BLOCK DATA CHEERS_BLOCK
      include 'stdio.def'
      include 'cheers.def'
      include 'cheers.cmn'
      data  n_cheers, no_Ls, n_profile / 50, 10, 0 /
      data  new_params / .TRUE. /
      data  a, b, c_ratio, theta_ref, x_first, x_last, y_tolerance /
     1      1.125, 0.2286, 4.9213, 1.570796, 0.0, 1.6, 0.1 /
      end


      REAL FUNCTION CHEERS_X( THETA )
c
c    Calculate abscissa of one point on the surface of a Cheer's
c    contraction profile for a given value of THETA.
c    Symbols have same meaning as in Cheers' paper -- except that the
c    subscript of L starts from 1 instead of 0.
c    The returned value is the abscissa of the profile.
c    CH_X_OFFSET is added to the result to match the Cheers and actual profiles
c
c Arguments
c  real theta     # location parameter, =PI at x=-\infty, =0 at x=\infty
c                 # =PI/2 at inflection point
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer i
      real    cos_sum, fact, K, term, theta, tsign
      include 'cheers.cmn'
c
      if ( new_params ) then
          K = 0.5 * log( c_ratio )
          L(1) = cosh(K) - 1
          L(2) = sinh(K) - K
          term = K
          do 1 i = 3, no_Ls
              term = term * K / real( i - 1 )
              L(i) = L(i-2) - term
1             continue
          new_params = .FALSE.
          end if
c
      cos_sum = L(1) * cos( theta )
      tsign = -1.0
      do 2 i = 2, no_Ls
          fact = i
          cos_sum = cos_sum + tsign * L(i) * cos( fact * theta ) / fact
          tsign = - tsign
2         continue
      cheers_x = a / PI * log( 2.0 * cos( theta / 2.0 ) ) -
     1           b / PI * log( 2.0 * sin( theta / 2.0 ) ) -
     2           2.0 * sqrt( a * b ) / PI * cos_sum +
     3           ch_x_offset
      return
      end



      REAL FUNCTION CHEERS_Y( THETA )
c
c    Calculate ordinate of one point on the surface of a Cheer's contraction
c    profile  for a given value of THETA.
c    Symbols have same meaning as in Cheers' paper -- except that the
c    subscript of L starts from 1 instead of 0.
c    The returned value is the ordinate of the profile.
c
c Arguments
c  real theta     # location parameter, =PI at x=-\infty, =0 at x=\infty
c                 # =PI/2 at inflection point
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer i
      real    sin_sum, fact, term, K, theta, tsign
      include 'cheers.cmn'
c
      if ( new_params ) then
          K = 0.5 * log( c_ratio )
          L(1) = cosh(K) - 1
          L(2) = sinh(K) - K
          term = K
          do 1 i = 3, no_Ls
              term = term * K / real( i - 1 )
              L(i) = L(i-2) - term
1             continue
          new_params = .FALSE.
          end if

      sin_sum = L(1) * sin( theta )
      tsign = -1.0
      do 2 i = 2, no_Ls
          fact = i
          sin_sum = sin_sum + tsign * L(i) * sin( fact * theta ) / fact
          tsign = - tsign
2         continue
      cheers_y = a / PI * theta / 2.0 + b / PI * ( PI - theta ) / 2.0
     1           - 2.0 * sqrt( a * b ) / PI * sin_sum
      return
      end



      REAL FUNCTION CHEERS_OFFSET( THETA )
c
c    Points on the Cheers contraction have an origin determined by the
c mathematics of the Cheers formula. This function returns a value which,
c when added to values from CHEERS_X(THETA), maps the Cheers profile
c point onto the actual contraction profile.
c
c Argument
c  real theta
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer index, jdhunt
      real    cheers_ref, profile_ref, theta, cheers_x, cheers_y
c
      include 'cheers.cmn'
c
c         calculate the X value of the reference point on the Cheers profile
c         -- automatically calculates cheers_y
c
      cheers_ref = cheers_y( theta )
c
c        linearly interpolate the value of x on the actual profile which
c        has a y equal to ch_y_ref. check feasibilty first
c
      if ( cheers_ref .gt. profile(1,PRF_Y) ) then                        1b
          cheers_offset = 0.0
          write( STD_OUT, 1 ) theta
1         format( ' CHEERS_OFFSET : WARNING : Not feasible for THETA=',
     1            g12.5 )
      else if ( cheers_ref .lt. profile(n_profile,PRF_Y) ) then           1m
          cheers_offset = 0.0
          write( STD_OUT, 1 ) theta
      else                                                                1m
          index = jdhunt( profile(1,PRF_Y), 1, n_profile, -999,
     1                    cheers_ref)
          profile_ref =
     1        ( profile(index+1,PRF_X) - profile(index,PRF_X) ) /
     2        ( profile(index+1,PRF_Y) - profile(index,PRF_Y) ) *
     3      ( cheers_ref - profile(index,PRF_Y) ) + profile(index,PRF_X)
          cheers_offset = profile_ref - cheers_x(theta)
          end if                                                          1e
      return
      end



      REAL FUNCTION CHEERS_TARGET( THETA )
c
c  This routine is used to find the value of X (x_target) required for a
c  particular THETA.
c
      include 'stdio.def'
      include 'cheers.def'
c
      real theta, cheers_x
      include 'cheers.cmn'
c
      cheers_target = cheers_x( theta ) - x_target
      return
      end



      SUBROUTINE CALC_RESULTS()
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer i
      real theta, theta_1, theta_2, x_step, cheers_offset, tolerance,
     1     cheers_x, cheers_y, cheers_offset, cheers_target,
     2     get_profile, qzerli
      double precision ms_sum
      include 'cheers.cmn'
      external cheers_target
c
c         generate new horizontal offset
c
      ch_x_offset = 0.0
      ch_x_offset = cheers_offset( theta_ref )
      ms_sum   = 0.0
      tolerance = y_tolerance / abs( b - a ) / 200.0
c
c         initialize iteration of abscissa value
c
      theta_1 = PI * 0.99
      theta_2 = PI * 0.50
      x_target = x_first
      x_step   = ( x_last - x_first ) / real( n_cheers - 1 )
      do 1 i = 1, n_cheers                                                1b
c
c         calculate abscissa value and point on Cheers profile
c
          theta = qzerli( theta_1, theta_2, cheers_target, tolerance )
          cheers(i,PRF_X) = cheers_x( theta )
          cheers(i,PRF_Y) = cheers_y( theta )
          cheers(i,PRF_A) = get_profile( cheers(i,PRF_X) )
c
c         calculate rms error
c
          ms_sum = ms_sum + ( cheers(i,PRF_Y) - cheers(i,PRF_A) ) **2
c
c         prepare for next iteration
c
          x_target = x_target + x_step
          if ( i .gt. 1 ) theta_1 = theta_2
          theta_2  = theta
1         continue                                                        1e
      rms_error = sqrt( ms_sum / dble( n_cheers) )
      return
      end



      SUBROUTINE WRITE_RESULTS( RESERV_LU, OUT_FILE )
c
c  Writes the calculated Cheers contraction profile and the actual
c  contraction profiles to file OUT_FILE.
c
c Argument
c  integer reserv_lu   # logical unit number reserved for output
c          out_file    # name of output file
c
      include 'stdio.def'
      include 'cheers.def'
c
      integer i, out_lu, open_file, reserv_lu
      real    cheers_x, cheers_y
      character out_file*(*)
      include 'cheers.cmn'
c
      out_lu = open_file( reserv_lu, out_file, 'unknown' )
      if ( out_lu .gt. 0 ) then                                           1b
          write( out_lu, 1 ) cheers_x(theta_ref), cheers_y(theta_ref),
     1                       ch_x_offset, rms_error
1         format( ' ; Comparison of Cheers and actual' /
     1            ' ;   contraction profiles.' /
     2            ' ; co-reference point (', f8.4, ',', f8.4, ')' /
     3            ' ; Cheers horizontal offset = ', f8.4 /
     4            ' ; R.M.S. error             = ', f9.5 /
     5            ' ;' /
     6            ' ; dimensions are in metres' /
     7            ' ;  x      Cheers   actual' )
          do 5 i = 1, n_cheers                                            2b
              write( out_lu, 2 ) cheers(i,PRF_X), cheers(i,PRF_Y),
     1                           cheers(i,PRF_A)
2             format( 3( 1x, f8.4 ) )
              if ( out_lu .eq. STD_OUT .or. out_lu .eq. STD_ERR ) then    3b
                  if ( mod( i, 20 ) .eq. 0 .or. i .eq. n_cheers ) then    4b
                      write( STD_OUT, 3 )
3                     format( ' Press <RETURN> to continue' )
                      read( STD_IN, 4 )
4                     format()
                      end if                                              4e
                  end if                                                  3e
5             continue                                                    2e
          if ( out_lu .ne. STD_OUT .and. out_lu .ne. STD_ERR )
     1        close( UNIT=out_lu )
          end if                                                          1e
      return
      end



      SUBROUTINE PLOT_RESULTS
c
      include 'stdio.def'
      integer VISUAL_500_TERMINAL, MAX_TRIES
      logical NO_ZOOM, ZOOM
      parameter( VISUAL_500_TERMINAL=1, MAX_TRIES=4,
     1           NO_ZOOM=.FALSE.,       ZOOM=.TRUE. )
c
      integer item, Jresp
      logical ok, ilgread
      real    x_min, x_max, y_min, y_max, x, y
      character chr*1
c
      call calc_data_limits( x_min, y_min, x_max, y_max )
      call ilgdevi( VISUAL_500_TERMINAL, ' ' )
      call ilgopen( ' ' )
      call make_graph( NO_ZOOM, x_min, y_min, x_max, y_max )
c
c     start of loop
1         call new_scr()
          write( STD_OUT, 2 )
2         format( ' Plot results' / '  0) Exit' /
     1      '  1) Zoom using crosshairs' /
     2      '  2) Redraw full size ' /
     3      '  3) Read crosshairs' )
          item = Jresp( STD_IN, 0, 3, MAX_TRIES )
          if ( item .eq. 0 ) then
              call ilgclos()
              return
          else if ( item .eq. 1 ) then
              call make_graph( ZOOM, x_min, y_min, x_max, y_max )
          else if ( item .eq. 2 ) then
              call make_graph( NO_ZOOM, x_min, y_min, x_max, y_max )
          else if ( item .eq. 3 ) then
              ok = ilgread( x, y, chr )
              write( STD_OUT, 3 ) x, y
3             format( ' X = ', f9.4, ' m' /
     1                ' Y = ', f9.4, ' m' /
     2                ' Press <RETURN> to continue' )
              read( STD_IN, 4 )
4             format()
              end if
          goto 1
c         end of loop
      end



      SUBROUTINE MAKE_GRAPH( ZOOM, X_MIN, Y_MIN, X_MAX, Y_MAX )
      include 'stdio.def'
      include 'cheers.def'
      logical zoom
      real    x_min, x_max, y_min, y_max
      include 'cheers.cmn'
c
      if ( zoom ) then
          call ilgwild( 'xx', x_min, y_min, x_max, y_max )
          end if
      call ilgimit( x_min, y_min, x_max, y_max )
      call ilgline( 0, 1, 1 )
      call ilgdata( cheers(1,PRF_X), cheers(1,PRF_Y), 1, n_cheers, 0 )
      call ilgline( 0, 3, 1 )
      call ilgdata( cheers(1,PRF_X), cheers(1,PRF_A), 1, n_cheers, 0 )
      call ilgxlab( 'x (m)' )
      call ilgylab( 'y (m)' )
c      call ilgxlab( 'Distance from contraction inlet (m)' )
c      call ilgylab( 'Distance from centreline (m)' )
c      call ilgtitl( 'Comparison of Cheers and Actual Contraction' //
c     1              '  Profile' )
      call ilgnote( '|10r  |5-1 Cheers|r  |5-3 Actual' )
      return
      end



      SUBROUTINE CALC_DATA_LIMITS( X_MIN, Y_MIN, X_MAX, Y_MAX )
      include 'stdio.def'
      include 'cheers.def'
c
      integer i
      real x_min, x_max, y_min, y_max
      include 'cheers.cmn'
c
      x_max = cheers(1,PRF_X)
      x_min = x_max
      y_max = max( cheers(1,PRF_Y), cheers(1,PRF_A) )
      y_min = y_max
      do 1 i = 2, n_cheers                                                1b
          x_max = max( x_max, cheers(i,PRF_X) )
          x_min = min( x_min, cheers(i,PRF_X) )
          y_max = max( y_max, cheers(i,PRF_Y), cheers(i,PRF_A) )
          y_min = min( y_min, cheers(i,PRF_Y), cheers(i,PRF_A) )
1         continue                                                        1e
c
      return
      end



      REAL FUNCTION QZERLI( A, B, F, TOL )
c
c       ZEROS BY LINEAR Interpolation
c   Function which uses linear interpolation to iterate toward the zero of the
c function F(X). The function QZERLI requires 2 starting values X=A and X=B.
c QZERLI is not guarranteed to converge to a zero of the function F(X). Care
c should be taken in selecting the starting values X=A and X=B. Knowing the
c shape of F(X) can help with this initial selection .
c
c real a,b,             # starting values of x (1st values of search interval)
c      xa,xb,           # calculated values of search interval boundary
c      fa,fb,           # =f(xa) & =f(xb)
c      tol1,tol         # tolerance on the zero value of   F( QZERLI )
c
      include 'stdio.def'
c
      real a, b, xa, xb, f, fa, fb, tol1, tol, denom
c
      tol1 = abs( tol )
      xa = a
      xb = b
      QZERLI = b
      fa = f( xa )
      fb = f( xb )
1     if ( abs( fa ) .gt. tol1 ) then
          denom = fb / fa - 1.0
          if ( denom .eq. 0.0 ) then
              write( STD_OUT, 2 )
2             format( ' QZERLI : ABORT : Zero slope.' )
              return
              end if
          QZERLI = xa + ( xa - xb ) / denom
          xb = xa
          fb = fa
          xa = QZERLI
          fa = f( xa )
          goto 1
          end if
      return
      end



      SUBROUTINE NEW_SCR()
      include 'stdio.def'
c
      integer    ESCAPE
      parameter( ESCAPE=27 )
      logical  got_term, vis500, equery
      save got_term, vis500
      data got_term, vis500 / .FALSE., .FALSE. /
c
      if ( .not. got_term  ) then
          vis500 = Equery( STD_IN, 'Visual 500 terminal ?' )
          got_term = .TRUE.
          end if
      if ( vis500 ) then
          write( STD_OUT, 1 ) char( ESCAPE )
1         format( 1x, a1, 'w' )
      else
          write( STD_OUT, 2 ) char( ESCAPE ), char( ESCAPE )
2         format( 1x, a1, '[2J', a1, '[H' )
          end if
      return
      end



      SUBROUTINE GET_TOKEN( STRING, TOKEN )
c
c Extract the next token from STRING and place it in TOKEN . On the
c any call where STRING is not blank and only one character in length,
c STRING is saved and the search for a token starts at the beginning of
c the string . In a call where STRING = ' ' (and is one character long),
c the next token is extracted from the previously saved STRING . The
c maximum length of a saved string is 128 characters . If there are no more
c tokens in the string the returned value of TOKEN is ' '.
c
c Arguments
c  character string*(*)   # INPUT  # string containing tokens
c            token*(*)    # OUTPUT # returned token
c
c local variables
c  integer
c   len_saved         # non blamk length of the saved string
c   start, finish     # start & finish indices in the saved string of the token
c   str_length        # function : non-blank length of string
c  character
c   saved*128         # saved string
c
      include 'stdio.def'
c
      integer len_saved, start, finish, str_length
      character saved*128, string*(*), token*(*)
c
      save saved, start
      data saved / ' ' /
      data start / 1 /
c
      if ( len(string) .ne. 1 .or. string(1:1) .ne. ' ' ) then
          saved = string
          len_saved = str_length( string )
          start = 1
          end if
      if ( len_saved .le. 0 ) then
          token = ' '
      else
c
1         if ( start .le. len_saved ) then
              if ( saved(start:start) .eq. ' ' ) then
                  start = start + 1
                  goto 1
                  end if
              finish = start
2             if ( finish .lt. len_saved ) then
                  if ( saved(finish+1:finish+1) .ne. ' ' ) then
                      finish = finish + 1
                      goto 2
                      end if
                  end if
              token = saved(start:finish)
              start = finish + 1
          else
              token = ' '
              end if
          end if
      return
      end



      INTEGER FUNCTION OPEN_FILE( LUN, FILE_NAME, NEW_OLD )
c
c     Opens a text file for reading ( NEW_OLD = 'old' ) or writing
c (NEW_OLD='new'). If the file is for reading, the file name cannot be blank.
c If the file is for writing a blank filename is interpreted as the vdu screen.
c The logical unit used for the file is LUN. If the file is successfully opened,
c LUN is the returned result . If a file could not be opened, the returned value
c is OPEN_FILE = 0 .
c
c Arguments
c  integer   lun          # logical unit no. of file to be opened
c  character file_name*(*)# file name
c            new_old*(*)  # file status : 'old'=read, 'new'=write, or 'unknown'
c
c Local Variables
c  integer length         # length of file name string
c          str_length     # function : length of string
c
      include 'stdio.def'
c
      integer lun, length, str_length
      character file_name*(*), new_old*(*)
c
c             check argument
c
      if ( new_old .ne. 'old' .and. new_old .ne. 'new' .and.
     1     new_old .ne. 'unknown' ) then
          write( STD_OUT, 1 ) new_old
1         format( ' OPEN_FILE : ABORT : Illegal status ', a /
     1            21x, 'Need ''old'', ''new'' or ''unknown''' )
          open_file = 0
      else if ( lun .lt. 0 .or. lun .ge. 100 ) then
          write( STD_OUT, 2 ) lun
2         format( ' OPEN_FILE : ABORT : Illegal logical unit number ',
     1            i12, '. Must be 0 to 99.' )
          open_file = 0
      else
          open_file = 0
          length = str_length( file_name )
          if ( length .gt. 0 ) then
              open( UNIT=lun, FILE=file_name, STATUS=new_old,
     1              ACCESS='sequential', FORM='formatted', ERR=6 )
              open_file = lun
          else if ( new_old .ne. 'old' ) then
              open_file = STD_OUT
              end if
          end if
      return
c
c             TRAPS
c
6     write( STD_OUT, 7 ) file_name(1:length)
7     format( ' OPEN_FILE : ABORT : Cannot open file ', a )
      open_file = 0
      return
      end


      INTEGER FUNCTION STR_LENGTH( STRING )
c
c    Find the location of the last non-blank character in the character
c string STRING . If the whole string is blank, the returned value is 0 .
c
      include 'stdio.def'
c
      character string*(*)
c
      str_length = len( string )
1     if ( str_length .ge. 1 ) then
          if ( string(str_length:str_length) .eq. ' ' ) then
              str_length = str_length - 1
          else
              return
              end if
          goto 1
          end if
      return
      end


      INTEGER FUNCTION JRESP( IPLU, IMIN, IMAX, NOREQS )
c               Integer RESPonse
c
c       The integer function JRESP returns an integer value between the
c  specified valid limits. This value is entered via the file with logical
c  unit number IPLU ( including possibly the keyboard ) , and is  checked
c  by JRESP, i.e. MIN < JRESP < MAX . The input file logical unit number
c  IPLU must be in the range 0 <= IPLU <= 99 , and must have been previously
c  defined if it is not a default device ( vdu, keyboard etc ). The maximum
c  number of input requests allowed NOREQS must be positive.
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer Iplu      #  input file logical unit number
c         imax,     #  maximum valid response value
c         imin,     #  minimum valid response value
c         noreqs    #  maximum number of requests allowed
c
c integer cntr,     # counter for the number of input attempts
c         Iplu1     # input file logical unit number
c         nchars    # number of input characters
c         str_length# function : length of string
c character Cstr*40 # input character string
c
      include 'stdio.def'
c
      integer iplu, imax, imin, noreqs, cntr, iplu1, nchars, str_length
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 ) then
          write ( STD_ERR, 1 ) Iplu
1         format ( ' JRESP : WARNING : Illegal logical unit ', i12 /
     1             20x, ' Changed to keyboard' )
          Iplu1 = Iplu
      else if ( noreqs .le. 0 ) then
          write ( STD_ERR, 2 ) Noreqs
2         format ( ' JRESP : WARNING : Less than one request allowed ',
     1             i12, ' Use 1' )
          end if
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = STD_IN
      do 9 cntr = 1, max( noreqs, 1 )
c
c           read data
c
          nchars = 0
3         if ( nchars .le. 0 ) then
              read( Iplu1, 4, END=11 ) Cstr
4             format( a )
              nchars = str_length( Cstr )
              goto 3
              end if
c
c           selected min or max
c
          if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
              jresp = imax
              return
          else if ( Cstr(1:3) .eq.'min' .or. Cstr(1:3) .eq. 'MIN' ) then
              jresp = imin
              return
c
c           decode & check numeric value
c
          else
              read( Cstr, *, ERR=7 ) jresp
              if ( min( imin, imax ) .le. jresp .and.
     1             jresp .le. max( imin, imax ) ) return
              write ( STD_ERR, 5 ) imin, imax
5             format(' JRESP : ERROR ; Integer outside limits ', i11,
     1               ' & ', i11 )
              if ( Iplu1 .ne. STD_IN ) stop
              write ( STD_ERR, 6 )
6             format( ' Try again ' )
              end if
          goto 9
c
c           decoding trap
c
7             write( STD_ERR, 8 )
8             format( ' JRESP : ERROR : Cannot decode that ' )
              if ( Iplu1 .ne. STD_IN ) stop
              write ( STD_ERR, 6 )
9         continue
c
c           give up
c
      write ( STD_ERR, 10 )
10    format( ' I give up . Get help' )
11    stop
      end



      INTEGER FUNCTION JDHUNT( X_DATA, INCREM, N_POINTS, INIT_GUESS,
     1                         X0 )
c
c       inteval Doubling HUNT
c Function for locating the last element between the 1st and Nth elements in
c the array X of N real numbers , which in the case of an ascending array, is
c less than the value X0 . In the case of a descending array, it finds the
c last element which is greater than or equal to X0 . The returned value of
c JDHUNT is the index of this element in the array X . The search begins with
c an interval-doubling hunt until the target value X0 is straddled, and then
c the search interval is reduced by bisection search . INITIAL_GUESS is an
c initial guess for the returned result supplied by the calling program . If
c INITIAL_GUESS=-999, and JDHUNT has been used before, the previous returned
c result is used as the starting value . If INITIAL_GUESS is not within the
c bounds of the data array (e.g. 0), the search is by bisection only .
c    N.B. X must be an array of values arranged in STRICTLY ASCENDING or
c  STRICTLY DESCENDING order .
c
c     Arguments
c real x_data(*)        # data array
c integer increm,       # data array indexing increment
c         n_points      # number of data points used
c         init_guess    # starting point for search
c real x0               # search target value
c
c     Variables
c logical desc          # .true. if data array X is descending
c         not_pass1     # second or later call to this function
c integer nlast         # index of last point used
c         left, right,  # left & right hand indices of the search interval
c         centre        # moving search index
c         saved_result  # result of the previous call
c         step          # hunt interval size
c
      include 'stdio.def'
      real x_data(*), x0
      logical desc, not_pass1
      integer increm, n_points, init_guess,
     1        nlast, left, right, centre, saved_result, step
c
      save not_pass1, saved_result
      data not_pass1 / .FALSE. /
      data saved_result / 0 /
c
      if( n_points .le. 0 .or. increm .le. 0 ) return
      nlast = 1 + ( n_points - 1 ) * increm
      desc = x_data(1) .gt. x_data(nlast)
c
c             deal with the boundary conditions
c ( the method of left=0, right=nlast+1 from Numerical Recipes does not
c   work for nincr>1 . )
c
      if ( x0 .lt. x_data(1) .neqv. desc ) then                          1b
          left = 1
      else if ( x0 .gt. x_data(nlast) .neqv. desc ) then                 1m
          left = nlast
      else if ( x0 .eq. x_data(1) ) then                                 1m
          left = 1
      else if ( x0 .eq. x_data(nlast) ) then                             1m
          left = nlast - increm
      else                                                               1m
c
c             decide if a hunt is required
c
          if ( not_pass1 .and. init_guess .eq. -999 .or.
     1            init_guess .ge. 1 .and. init_guess .le. nlast ) then   2b
              if ( init_guess .eq. -999 ) then                           3b
                  left = saved_result
              else                                                       3m
                  left = init_guess
                  end if                                                 3e
              left = max( 1, min( nlast,
     1            ( ( left - 1 ) /  increm ) * increm + 1 ) )
c
c             doubling hunt procedure
c
              step = increm
              if ( x0 .ge. x_data(left) .eqv. desc ) then                3b
c                 start loop                                             4b
1                     right = left
                      left = right - step
                      step = 2 * step
                      if ( left .lt. 1 ) then                            5b
                          left = 1
                      else if ( x0 .lt. x_data(left) .neqv. desc ) then  5m
                          goto 1
                          end if                                         5e
c                     end loop                                           4e
              else                                                       3m
                  right = left
c                 start of loop                                          4b
2                     left = right
                      right = left + step
                      step = 2 * step
                      if ( right .gt. nlast ) then                       5b
                          right = nlast
                      else if ( x0 .ge. x_data(right) .neqv. desc ) then 5m
                          goto 2
                          end if                                         5e
c                     end of loop                                        4e
                  end if                                                 3e
          else                                                           2m
              left = 1
              right = nlast
              end if                                                     2e
c
c             bisection procedure
c ( the simple averaging of left & right does not find a valid array index
c   for centre )
c
3         if ( right - left .gt. increm ) then                           2b
              centre = left + ( right - left ) / 2 / increm * increm
              if ( x0 .gt. x_data(centre) .neqv. desc ) then             3b
                  left = centre
              else                                                       3m
                  right = centre
                  end if                                                 3e
              goto 3
              end if                                                     2e
          end if                                                         1e
      not_pass1 = .TRUE.
      saved_result = left
      JDHUNT = left
      return
      end



      SUBROUTINE CRESP( LUN, STRING, N_TRIES )
c
c Get a character string from file with logical unit number LUN into STRING .
c
c Arguments
c  integer   lun          # IN  # logical unit number of input file
c  character string*(*)   # OUT # destination string
c  integer   n_tries      # IN  # maximum number of attempts
c
c Local variables
c  integer count          # attempts counter
c
      include 'stdio.def'
      integer lun, n_tries, count
      character string*(*)
c
      do 4 count = 1, n_tries
          read( lun, 1, ERR=2, END=5 ) string
1         format( a )
          return
2         write( STD_ERR, 3 )
3         format( ' CRESP : WARNING : Cannot read input' )
4         continue
c
5     write( STD_ERR, 6 )
6     format( ' Abort' )
      stop
      end



      SUBROUTINE SWAP( A, B )
      include 'stdio.def'
      real a, b, temp
c
      temp = a
      a = b
      b = temp
      return
      end



      INTEGER FUNCTION NEW_LUN()
c
c  Selects a new (unused) logical unit for a file .
c  The first logical unit is 10 .
c
      include 'stdio.def'
      integer next_lun
      save    next_lun
      data next_lun / 10 /
c
      new_lun  = next_lun
      next_lun = next_lun + 1
      return
      end



      LOGICAL FUNCTION EQUERY( IPLUN, MSG )
c
c       Displays a query on the vdu and receives a logical response with
c  logical unit number IPLUN .
c
c    Arguments
c integer   iplun     # input device logical unit number
c character msg*(*)   # message string
c
c constant
c     Hvdulu          # v.d.u. logical unit number
c integer iok,        # index of response in legal response list
c         index,      # lexical function
c character gquery*1  # received character
c           gok*8     # legal responses
c
      include 'stdio.def'
c
      integer iok, index
      integer iplun
      character gquery*1, gok*10, msg*(*)
      data gok / 't1yY. f0nN' /
c
      write( STD_OUT, 1 ) msg
1     format( 1x, a )
c
c     start of loop
2         read( iplun, 3 ) Gquery
3         format( a1 )
          iok = index( gok, Gquery )
          if ( iok .gt. 0 ) then
              equery = iok .le. 5
              return
              end if
          write( STD_OUT, 4 ) gok
4         format( ' Expecting one of ', a )
          goto 2
c         end of loop
c
      end



      REAL FUNCTION QRESP( IPLU, RMIN, RMAX, NOREQS )
c               Real RESPonse
c       The real funtion QRESP returns a real value between the specified
c valid limits . This value is entered via the file with logical unit number
c IPLU ( default value 0 is the keyboard ) , and is checked by QRESP
c ie RMIN <= QRESP <= RMAX . The input file logical unit number must be in the
c range 0 <= IPLU <= 99 , and must have been previously defined if it is not
c a default device ( keyboard etc ) . The maximum number of input requests
c allowed NOREQS must be positive .
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer Iplu      #  input file logical unit number
c         noreqs    #  maximum number of requests allowed
c real  rMax,           # maximum valid response value
c       rMin            # minimum valid response value
c
c integer cntr,     # counter for the number of input attempts
c         Iplu1     # input file logical unit number
c         nchars    # number of input characters
c         str_length# function : length of string
c character Cstr*40 # input character string
c
      include 'stdio.def'
c
      integer iplu, noreqs, cntr, iplu1, nchars, str_length
      real rMax, rMin
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 ) then
          write ( STD_ERR, 1 ) Iplu
1         format ( ' QRESP : WARNING : Illegal logical unit ', i12 /
     1             20x, ' Changed to keyboard' )
          Iplu1 = Iplu
      else if ( noreqs .le. 0 ) then
          write ( STD_ERR, 2 ) Noreqs
2         format ( ' QRESP : WARNING : Less than one request allowed ',
     1             i12, ' Use 1' )
          end if
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = STD_IN
      do 9 cntr = 1, max( noreqs, 1 )
c
c           read data
c
          nchars = 0
3         if ( nchars .le. 0 ) then
              read( Iplu1, 4, END=11 ) Cstr
4             format( a )
              nchars = str_length( Cstr )
              goto 3
              end if
c
c           selected min or max
c
          if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
              qresp = rmax
              return
          else if ( Cstr(1:3) .eq.'min' .or. Cstr(1:3) .eq. 'MIN' ) then
              qresp = rmin
              return
c
c           decode & check numeric value
c
          else
              read( Cstr, *, ERR=7 ) qresp
              if ( min( rmin, rmax ) .le. qresp .and.
     1             qresp .le. max( rmin, rmax ) ) return
              write ( STD_ERR, 5 ) rmin, rmax
5             format( ' QRESP : ERROR ; Real outside limits ', g12.5,
     1                ' & ', g12.5 )
              if ( Iplu1 .ne. STD_IN ) stop
              write ( STD_ERR, 6 )
6             format( ' Try again ' )
              end if
          goto 9
c
c           decoding trap
c
7             write( STD_ERR, 8 )
8             format( ' QRESP : ERROR : Cannot decode that ' )
              if ( Iplu1 .ne. STD_IN ) stop
              write ( STD_ERR, 6 )
9         continue
c
c           give up
c
      write ( STD_ERR, 10 )
10    format( ' I give up . Get help' )
11    stop
      end



      SUBROUTINE SORTSH( X, M1, M2, N, J1, M )
c
c               SORT ( SHell )
c       Subroutine for a shell sort into ascending or descending order .  The
c sorted array is an N x M matrix of N sets of Mtuples ( ie. N points in M
c dimensional space ) . Only the J1th column of the matrix is sorted into
c ascending or descending order , with the elements of the other columns
c undergoing the same exchanges . Sorting occurs between rows M1 and M2
c inclusive . If  M1 < M2 , the sort is descending . If M1 > M2 , the sort is
c ascending . Eg. for   SORTSH( X, 1, 4, 5, 1, 4 ) ,
c               2 4 3 1                 1 2 4 3
c               4 3 1 2                 2 4 3 1
c           X = 3 1 2 4     becomes X = 3 1 2 4
c               1 2 4 3                 4 3 1 2
c               3 1 2 4                 3 1 2 4
c
c real  x(n,m)          # operand array
c integer n,m,          # number of rows and columns in array
c         m1,m2,        # starting and finishing row indices
c         i1,i2,i3,j,   # working indices
c         j1,           # column to be sorted
c         id,           # sort segment size
c         m11,m12       # starting row & -1
c
      include 'stdio.def'
      integer n, nn, m, m1, m2, i1, i2, i3, j, j1, id, m11, m12
      real x(n,m)
c
      if ( m1 .eq. m2 .or. m1 .lt. 1 .or. m1 .gt. n .or. m2 .lt. 1 .or.
     1     m2 .gt. n .or.j1 .lt. 1 .or. j1 .gt. m ) then
          write( STD_ERR, 1 )
1         format( ' SORTSH : ABORT : Invalid Argument' )
          return
          end if
      m11 = min( m1, m2 )
      m12 = m11 - 1
      nn = abs( m2 - m1 ) + 1
      id = 2 ** int( alog( real( nn ) ) / alog( 2.0 ) + 0.0001 ) - 1
c     repeat
2         do 5 i1 = 1, nn-id
              do 4 i2 = i1+m12, m11, -id
                  i3 = i2 + id
                  if ( x(i2,j1) .le. x(i3,j1) ) goto 5
                  do 3 j = 1, m
                      call swap( x(i2,j), x(i3,j) )
3                     continue
4                 continue
5             continue
          id = id / 2
          if ( id .gt. 0 ) goto 2
c         until ( id <= 0 )
      if ( m1 .lt.  m2 ) then
          i1 = m1
          i2 = m2
c         repeat
6             do 7 j = 1, m
                  call swap( x(i1,j), x(i2,j) )
7                 continue
              i1 = i1 + 1
              i2 = i2 - 1
              if ( i1 .lt. i2 ) goto 6
c             until ( i1 >= i2 )
          end if
      return
      end



