
c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM FILLET
c
c   Program to calculate the first derivative, second derivative and 
c   curvature of the wind-tunnel-contraction profile.
c
      call contour
      call parameters
      call work
      stop
      end


      SUBROUTINE WORK
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hopflu = 4          # logical unit of output file
c  Hmm    = 1000.0     # 1000 mm in a metre
c variables
c  integer i           # profile point number
c  real curvature,     # curvature of the contraction profile
c       h,             # half height of contraction
c       second,        # slope of the slope of the contraction profile
c       slope,         # slope of the contraction profile
c       z,             # distance downstream from contraction inlet
c       qcont          # contraction profile function
c  character Cfile*64  # data file name
c
      implicit none
      integer Hvdulu, Hkbdlu, Hopflu
      real Hmm
      parameter( Hvdulu=6, Hkbdlu=5, Hopflu=4, Hmm = 1000.0 )
c
      character Cfile*64
      integer i
      real curvature, h, second, slope, z, qcont
      include 'deriv.cmn/list'
c
c         open output file
c
      write( Hvdulu, 2 )
2     format( ' Output file name ? ', $ )
      read( Hkbdlu, 3 ) Cfile
3     format( a )
      open( UNIT=Hopflu, FILE=Cfile, STATUS='new',
     1    CARRIAGECONTROL='list' )
      write( Hvdulu, 4 )
      write( Hopflu, 4 )
4     format(
     1  '    distance   cont-    slope        second       radius' /
     2  '      from    raction              derivative       of' /
     3  '     inlet    height                            curvature' )
      i = 1
      do z = zmin, zmax, zinc                                             1b       1b
        h = qcont( z )
        slope = ( qcont(z+zinc) - qcont(z-zinc) ) / 2.0 / zinc
        second = ( - qcont(z+zinc+zinc) + 16.0 * qcont(z+zinc) -
     1    30.0 * qcont(z) + 16.0 * qcont(z-zinc) - qcont(z-zinc-zinc) )
     2    / 12.0 / zinc **2
        curvature = second / sqrt( 1 + slope **2 ) **3
        write( Hvdulu, 1 ) i, z * Hmm, h * Hmm, slope, second, curvature
        write( Hopflu, 1 ) i, z * Hmm, h * Hmm, slope, second, curvature
1       format( 1x, i4, f8.1, f8.3, 3g13.5 )
        i = i + 1
        end do                                                            1e
      return
      end


      SUBROUTINE CONTOUR
c
c           READ CONTRACTION PROFILE FILE & MAKE CUBIC SPLINE FUNCTION
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hprflu = 8          # contraction profile file
c variables
c  integer iline,      # line number in file
c          ierr,       # error flag from ICSCCU
c          jljstr      # function : left justify string
c  character Cfile*64  # name of file
c            Cline*80  # line read from data file
c  logical existf      # function : file exists ?
c
      implicit none
      integer Hkbdlu, Hvdulu, Hprflu
      real Hmm
      parameter( Hkbdlu=5, Hvdulu=6, Hprflu=8, Hmm=1000.0 )
c
      integer ierr, ijunk, iline, jljstr
      character Cfile*64, Cline*80
      logical existf
      include 'deriv.cmn/list'
c
c           open input file
c
c     repeat                                                                1b
1       write( Hvdulu, 2 )
2       format( ' Name of contraction profile file ? ', $ )
        read( Hkbdlu, 3 ) Cfile
3       format( a )
        if ( .not. Existf( Cfile ) ) goto 1
c       until( file exists )                                                1e
      open( UNIT=Hprflu, FILE=Cfile, STATUS='old' )
c
c           read contraction profile
c
      ncont = 1
      iline = 1
c     start loop                                                            1b
4       iline = iline + 1
        read( Hprflu, 3, END=7 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then        2b
          read( Cline, *, ERR=5 ) ijunk, zcont(ncont), xcont(ncont)
          zcont(ncont) = zcont(ncont) * 25.4 / Hmm
          xcont(ncont) = xcont(ncont) * 25.4 / Hmm
          ncont = ncont + 1
          end if                                                            2e
        goto 4
5       write( Hvdulu, 6 ) iline, Cline
6       format( ' Cannot read file at line ', i8 / 1x, a )
        goto 4
c       end of loop                                                         1e
c
7     ncont = ncont - 1
      close( UNIT=Hprflu )
c
c           calculate cubic spline fit
c
      call sortsh( tcont, ncont, 1, Hncont, 1, 5 )
      call icsccu( zcont, xcont, ncont, spline, Hncont, ierr )
      if ( ierr .ne. 0 ) then                                               1b
        write( Hvdulu, 8 ) ierr
8       format( ' FATAL : Cannot calculate spline, ierr = ', i7 )
        stop
        end if                                                              1e
      return
      end


      SUBROUTINE PARAMETERS
c
c         SET USERS PARAMETERS
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hmm=1000.0          # 1000 mm in a metre
c variables
c  integer item,       # item selected from the menu
c          Jresp,      # function : integer response
c  real Qresp          # function : real response
c  character Ctaper(0:6)*8     # available taper types
c
      implicit none
      integer Hkbdlu, Hvdulu
      real Hmm
      parameter( Hkbdlu=5, Hvdulu=6, Hmm=1000.0 )
c
      integer item, Jresp
      real Qresp
      character Ctaper(0:6)*8
      include 'deriv.cmn/list'
      zmin     = zcont(1)
      zmax     = zcont(ncont)
      do while( .true. )                                                  1b
        write( Hvdulu, 10 ) f_thick * Hmm, s_length * Hmm, zinc * Hmm
10      format( ' Change initial parameters.  0) exit' /
     1    29x, '1) fillet material thickness = ', f7.3, 'mm' /
     2    29x, '2) settling chamber length   = ', f7.1, 'mm' /
     3    29x, '3) streamwise step size      = ', f7.3, 'mm  ', $ )
        item = Jresp( Hkbdlu, 0, 3, 3 )
        if ( item .eq. 0 ) then                                           2b
          return
        else if ( item .eq. 1 ) then                                      2m
          write( Hvdulu, 11 )
11        format( ' What is the fillet material thickness ? (mm) ', $ )
          f_thick = Qresp( Hkbdlu, 0.0, 25.0, 3 ) / Hmm
        else if ( item .eq. 2 ) then                                      2m
          write( Hvdulu, 12 )
12        format( ' What is the settling chamber length ? (mm) ', $ )
          s_length = Qresp( Hkbdlu, 0.0, 2000.0, 3 ) / Hmm
        else if ( item .eq. 3 ) then                                      2m
          write( Hvdulu, 13 )
13        format( ' What is the streamwise step size ? (mm) ', $ )
          zinc = Qresp( Hkbdlu, 0.0, 200.0, 3 ) / Hmm
          end if                                                          2e
        end do
      end


      REAL FUNCTION QCONT( Z )
c
c       Evaluate a point on the wind tunnel contraction by interpolation with
c  the cubic spline fitted to the data .
c
c Argument
c  real z              # distance from contraction inlet
c
c constant
c  Hvdulu=6            # vdu logical unit no.
c variable
c  integer ierr,       # error flag from ICSCCU
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real d, z
      integer *4 ind, ierr, jbsrch
      include 'deriv.cmn/list'
c
c
      if ( z .le. zcont(1) ) then                                           1b
        qcont = xcont(1)
      else if ( z .ge. zcont(ncont) ) then                                  1m
        qcont = xcont(ncont)
      else                                                                  1m
        ind = jbsrch( zcont, 1, ncont, z )
        d   = z - zcont(ind)
        qcont = ( ( spline(ind,3) * d + spline(ind,2) ) * d +
     1          spline(ind,1) ) * d + xcont(ind)
        end if                                                              1e
      return
      end
