c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM FILLET
c
c   Program to generate the profiles for the fillet pieces to be installed
c   in the wind tunnel contraction . The initial contraction will be a
c   regular octagon at the inlet, becoming square at the exit . The output
c   is a file which can be used to draw the shape of the fillet piece .
c
      call contour
      call parameters
      call work
      stop
      end


      SUBROUTINE WORK
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hopflu = 4          # logical unit of output file
c  Hmm    = 1000.0     # 1000 mm in a metre
c variables
c  integer i           # profile point number
c  real a, a1, a2      # height of a fillet
c       area,          # cross section area of the contraction
c       between,       # distance between two adjacent fillets
c       c,             # distance of fillet centreline from contraction centre
c       c_old,         # previous value of c
c       h,             # half height of contraction
c       h_final,       # half height of contraction at exit
c       h_init,        # half height of contraction at inlet
c       h_octag,       # initial half length of octagon side
c       s,             # distance along the fillet centreline
c       shrink,        # proportion by which height has shrunk
c       width,         # (diagonal) width of fillet
c       z,             # distance downstream from contraction inlet
c       z_old,         # previous value of z
c       qtaper,        # fillet taper function
c       qcont          # contraction profile function
c  character Cfile*64  # data file name
c
      implicit none
      integer Hvdulu, Hkbdlu, Hopflu
      real Hmm
      parameter( Hvdulu=6, Hkbdlu=5, Hopflu=4, Hmm = 1000.0 )
c
      integer i, itemp
      real a, a1, a2, area, between, c, c_old, h, h_final, h_init,
     1  h_octag, s, shrink, width, z, z_old, qtaper, qcont
      character Cfile*64
      include 'fillet.cmn/list'
c
c         open output file
c
      write( Hvdulu, 2 )
2     format( ' Output file name ? ', $ )
      read( Hkbdlu, 3 ) Cfile
3     format( a )
      open( UNIT=Hopflu, FILE=Cfile, STATUS='new',
     1    CARRIAGECONTROL='list' )
      write( Hvdulu, 4 )
      write( Hopflu, 4 )
4     format( '    distance   cont-   fillet  fillet  distance   area' /
     1        '      from    raction  length   half   between' /
     2        '     inlet    height           width   fillets' )
c
c         end conditions
c
      h_init = qcont( zmin )
      h_final = qcont( zmax )
      h_octag = h_init / ( 1.0 + sqrt(2.0) )
c      WRITE(6,9998) H_INIT, H_FINAL, H_OCTAG
c      WRITE(HOPFLU,9998) H_INIT, H_FINAL, H_OCTAG
c9998  FORMAT( ' H_INIT, H_FINAL, H_OCTAG ', 3G13.5 )
c
c         start off at the upstream end of the settling chamber
c
      i = 1
      z = - s_length
      h = qcont( z )
      a = h / ( 1.0 + 1.0 / sqrt(2.0) )
      width = a / sqrt(2.0) - f_thick
      between = ( h - a )
      area =  h **2 - a **2 / 2.0
      c_old = h - a / 2.0
      z_old = z
      s = 0
      write( Hvdulu, 1 ) i, z * Hmm, h * Hmm, s * Hmm, width * Hmm,
     1    between * Hmm, area
      write( Hopflu, 1 ) i, z * Hmm, h * Hmm, s * Hmm, width * Hmm,
     1    between * Hmm, area
c
c         calculate fillet profile
c
      i = i + 1
      z = zmin
c     do z = zmin, zmax, zinc                                             1b       1b
5     if ( z .le. zmax ) then
        h = qcont( z )
c
        shrink = ( h - h_final ) / ( h_init - h_final )
        itemp = ( itaper +1 ) / 2
        if ( itemp .le. 1 .or. itemp .eq. 3 ) then                        2b
          a1 = qtaper( shrink, z, itaper ) * h /
     1           ( 1.0 + 1.0 / sqrt(2.0) )
          end if                                                          2e
        if ( itemp .eq. 2 .or. itemp .eq. 3 ) then                        2b
          a2 = h - ( h_final + ( h_octag - h_final ) *
     1        qtaper( shrink, z, itaper ) )
          end if                                                          2e
        if ( itemp .eq. 2 ) then                                          2b
          a = a2
        else if ( itemp .eq. 3 ) then                                     2m
          a = a1 * shrink + a2 * ( 1.0 - shrink )
        else                                                              2m
          a = a1
          end if                                                          2e
c
        width = a / sqrt(2.0) - f_thick
        between = ( h - a )
        area =  h **2 - a **2 / 2.0
        c = h - a / 2.0
        s = s + sqrt( 2.0 * ( c - c_old ) **2 + ( z - z_old) **2 )
        write( Hvdulu, 1 ) i, z * Hmm, h * Hmm, s * Hmm, width * Hmm,
     1    between * Hmm, area
        write( Hopflu, 1 ) i, z * Hmm, h * Hmm, s * Hmm, width * Hmm,
     1    between * Hmm, area
1       format( i4, f9.1, f8.2, f9.1, f8.2, f9.2, f9.6 )
c
        c_old = c
        z_old = z
        i = i + 1
c
        z = z + zinc
        if ( z .ne. zmax + zinc .and. z .gt. zmax ) then                  2b
          z = zmax
          end if                                                          2e
        if ( width .gt. 0 )
     1    goto 5
        end if
c       end do                                                            1e
      return
      end


      REAL FUNCTION QTAPER( SHRINK, Z, ITAPER )
c
c       Specifies the fraction by which the fillet has shrunk because of
c  reduced cross-section area .
c     N.B.  QTAPER(0) = 1.0
c           QTAPER(1) = 0.0  if ITAPER != 0
c
c  Arguments
c    real shrink               # fraction of maximum contraction height
c         z                    # distance from contraction inlet
c    integer itaper            # type of taper   0 = none
c                              #                 1 = linear
c                              #                 2 = sinusoidal
c
      implicit none
      real Hpi
      parameter( Hpi=3.1415926 )
c
      real shrink, z
      integer itaper
c
      if ( itaper .eq. 0 ) then                                           1b
c
c           no taper
c
        qtaper = 1.0
      else if ( mod( itaper, 2 ) .eq. 1 ) then                            1m
c
c           linear taper
c
        qtaper = shrink
      else if ( mod( itaper, 2 ) .eq. 0 ) then                            1m
c
c           sinusoidal taper
c
        qtaper = 0.5 * ( 1.0 - cos( shrink * Hpi ) )
        end if                                                            1e
      return
      end



      SUBROUTINE CONTOUR
c
c           READ CONTRACTION PROFILE FILE & MAKE CUBIC SPLINE FUNCTION
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hprflu = 8          # contraction profile file
c variables
c  integer iline,      # line number in file
c          ierr,       # error flag from ICSCCU
c          jljstr      # function : left justify string
c  character Cfile*64  # name of file
c            Cline*80  # line read from data file
c  logical existf      # function : file exists ?
c
      implicit none
      integer Hkbdlu, Hvdulu, Hprflu
      real Hmm
      parameter( Hkbdlu=5, Hvdulu=6, Hprflu=8, Hmm=1000.0 )
c
      integer ierr, ijunk, iline, jljstr
      character Cfile*64, Cline*80
      logical existf
      include 'fillet.cmn/list'
c
c           open input file
c
c     repeat                                                                1b
1       write( Hvdulu, 2 )
2       format( ' Name of contraction profile file ? ', $ )
        read( Hkbdlu, 3 ) Cfile
3       format( a )
        if ( .not. Existf( Cfile ) ) goto 1
c       until( file exists )                                                1e
      open( UNIT=Hprflu, FILE=Cfile, STATUS='old' )
c
c           read contraction profile
c
      ncont = 1
      iline = 1
c     start loop                                                            1b
4       iline = iline + 1
        read( Hprflu, 3, END=7 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then        2b
          read( Cline, *, ERR=5 ) ijunk, zcont(ncont), xcont(ncont)
          zcont(ncont) = zcont(ncont) * 25.4 / Hmm
          xcont(ncont) = xcont(ncont) * 25.4 / Hmm
          ncont = ncont + 1
          end if                                                            2e
        goto 4
5       write( Hvdulu, 6 ) iline, Cline
6       format( ' Cannot read file at line ', i8 / 1x, a )
        goto 4
c       end of loop                                                         1e
c
7     ncont = ncont - 1
      close( UNIT=Hprflu )
c
c           calculate cubic spline fit
c
      call sortsh( tcont, ncont, 1, Hncont, 1, 5 )
      call icsccu( zcont, xcont, ncont, spline, Hncont, ierr )
      if ( ierr .ne. 0 ) then                                               1b
        write( Hvdulu, 8 ) ierr
8       format( ' FATAL : Cannot calculate spline, ierr = ', i7 )
        stop
        end if                                                              1e
      return
      end


      SUBROUTINE PARAMETERS
c
c         SET USERS PARAMETERS
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hmm=1000.0          # 1000 mm in a metre
c variables
c  integer item,       # item selected from the menu
c          Jresp,      # function : integer response
c  real Qresp          # function : real response
c  character Ctaper(0:6)*8     # available taper types
c
      implicit none
      integer Hkbdlu, Hvdulu
      real Hmm
      parameter( Hkbdlu=5, Hvdulu=6, Hmm=1000.0 )
c
      integer item, Jresp
      real Qresp
      character Ctaper(0:6)*8
      include 'fillet.cmn/list'
      data Ctaper / 'none    ', 'lin fill', 'sin fill', 'lin gap ',
     1  'sin gap ', 'lin merg', 'sin merg' /
c
      zmin     = zcont(1)
      zmax     = zcont(ncont)
      do while( .true. )                                                  1b
        write( Hvdulu, 10 ) f_thick * Hmm, s_length * Hmm, zinc * Hmm,
     1                      Ctaper(itaper)
10      format( ' Change initial parameters.  0) exit' /
     1    29x, '1) fillet material thickness = ', f7.3, 'mm' /
     2    29x, '2) settling chamber length   = ', f7.1, 'mm' /
     3    29x, '3) streamwise step size      = ', f7.3, 'mm' /
     4    29x, '4) taper type                = ', a, '   ', $ )
        item = Jresp( Hkbdlu, 0, 4, 3 )
        if ( item .eq. 0 ) then                                           2b
          return
        else if ( item .eq. 1 ) then                                      2m
          write( Hvdulu, 11 )
11        format( ' What is the fillet material thickness ? (mm) ', $ )
          f_thick = Qresp( Hkbdlu, 0.0, 25.0, 3 ) / Hmm
        else if ( item .eq. 2 ) then                                      2m
          write( Hvdulu, 12 )
12        format( ' What is the settling chamber length ? (mm) ', $ )
          s_length = Qresp( Hkbdlu, 0.0, 2000.0, 3 ) / Hmm
        else if ( item .eq. 3 ) then                                      2m
          write( Hvdulu, 13 )
13        format( ' What is the streamwise step size ? (mm) ', $ )
          zinc = Qresp( Hkbdlu, 0.0, 200.0, 3 ) / Hmm
        else if ( item .eq. 4 ) then                                      2m
          write( Hvdulu, 14 )
14        format( ' Which taper type  0) none' /
     1      19x, '1) linear fillet' /  19x, '2) sinusoidal fillet' /
     2      19x, '3) linear gap' / 19x, '4) sinusoid gap' /
     3      19x, '5) linear merge' / 19x, '6) sinusoid merge   ',  $ )
          itaper = Jresp( Hkbdlu, 0, 6, 3 )
          end if                                                          2e
        end do
      end


      REAL FUNCTION QCONT( Z )
c
c       Evaluate a point on the wind tunnel contraction by interpolation with
c  the cubic spline fitted to the data .
c
c Argument
c  real z              # distance from contraction inlet
c
c constant
c  Hvdulu=6            # vdu logical unit no.
c variable
c  integer ierr,       # error flag from ICSCCU
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real d, z
      integer *4 ind, ierr, jbsrch
      include 'fillet.cmn/list'
c
c
      if ( z .le. zcont(1) ) then                                           1b
        qcont = xcont(1)
      else if ( z .ge. zcont(ncont) ) then                                  1m
        qcont = xcont(ncont)
      else                                                                  1m
        ind = jbsrch( zcont, 1, ncont, z )
        d   = z - zcont(ind)
        qcont = ( ( spline(ind,3) * d + spline(ind,2) ) * d +
     1          spline(ind,1) ) * d + xcont(ind)
        end if                                                              1e
      return
      end
