c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM MODIFY
c
c    This is a program for modifying the upstream end of the wind tunnel
c  contraction . The basis of the modification is to change the profile over
c  the section X_I to X_F ( where X_I is the distance from the inlet to the
c  start of the modified section and X_F is the distance from the inlet to the
c  end of the modified section ) and have continuous displacement, slope and
c  second derivative at these points . In practice, formulation of the problem
c  with unknown starting point X_I results in a pair of rational equations with
c  three unknowns X_I, X_F and Kappa  ( which  is underdetermined ) . If we put
c  X_I = 0, the problem is much easier to solve . The second derivative function
c  for the  modified profile is arbitrarily specified as a quadratic . This
c  second derivative function is
c                 f''(x) = Kappa * ( X_I _ x) * ( x - x1 )             -(1)
c  where x1 is a constant determined the displacement end condition at X_F .
c  If the original profile is y(x), then
c                          x1 = X_F - y(X_F) / Kappa / X_F  .          -(2)
c  The slope of the modified profile must satisfy the end condition at X_F is
c                                  2
c                error1 = Kappa X_F ( X_F/3 - x1 /2 ) - y'(X_F)        -(3)
c                error1 = 0                                            -(4)
c  and the displacement end condition at X_F is
c                             3
c           error2 = Kappa X_F ( X_F/2 - x1 ) / 6 + h - y(X_F)         -(5)
c           error2 = 0                                                 -(6)
c  The technique is to get a rough solution by plotting the errors (3) & (4) as
c  functions of X_F for specified values of Kappa . Equations (3)=(4) and
c  (5)=(6) are then solved for X_F by a linear interpolation method . Kappa
c  is chosen so that the difference between the solution for(3)=(4) and the
c  solution for (5)=(6) is zero . This gives values for the parameters X_F
c  and Kappa which are constants for the modified profile .
c
      implicit none
      integer Hkbdlu, Hvdulu
      parameter( Hkbdlu=5, Hvdulu=6 )
c
      integer item, Jresp
c
c             get the old contraction contour
c
      call contour
c     start of loop
c
c             set the global parameters
c
1       call parameters
        call newscr
        write( Hvdulu, 2 )
2       format( ' Select process 0) exit' /
     1          '                1) rough estimate' /
     2          '                2) fine tune' /
     3          '                3) plot profile   ', $ )
        item = Jresp( Hkbdlu, 0, 3, 3 )
        if ( item .eq. 0 ) then
          call ilgclos
          stop
        else if ( item .eq. 1 ) then
c
c             get approximate solution
c
          call rough
        else if ( item .eq. 2 ) then
c
c             get accurate solution
c
          call fine
        else if ( item .eq. 3 ) then
c
c             plot and write the result to file
c
          call profile
          end if
        goto 1
c       end of loop
      end


      SUBROUTINE ROUGH
c
c       get a graphical approximate solution
c
      implicit none
      integer Hnpts
      parameter( Hnpts=400 )
c
      integer npts
      real abscissa(Hnpts), err_s(Hnpts), err_d(Hnpts), x,
     1  err_slope, err_displ
      include 'modify.cmn/list'
c
c         calculate displacement and slope error arrays
c
      npts = 0
      do x = xmin, xmax, xstep
          npts = npts + 1
          abscissa(npts) = x
          err_d(npts) = err_displ( x )
          err_s(npts) = err_slope( x )
          end do
c
c           plot errors
c
      call ilgline( 0, 1, 1 )
      call ilgdata( abscissa, err_s, 1, npts, 1 )
      call ilgline( 0, 2, 1 )
      call ilgdata( abscissa, err_d, 1, npts, 0 )
      return
      end


      SUBROUTINE FINE
c
c         get accurate solution
c
      implicit none
      integer Hkbdlu, Hvdulu
      parameter( Hkbdlu=5, Hvdulu=6 )
c
      real diff, err_slope, err_displ, qzerli, qresp
      logical equery
      external err_slope, err_displ
      include 'modify.cmn/list'
c
c           solve for the current value of Kappa
c
      x_join = qzerli( xmin, xmax, err_slope, tolerance )
      diff = qzerli( xmin, xmax, err_displ, tolerance ) - x_join
      write( Hvdulu, 1 ) diff, x_join
1     format( ' Difference in zero crossings = ', g13.5, ' at ', g13.5 )
c     repeat
c
c           get a new value of Kappa and solve for it
c
2       write( Hvdulu, 3 )
3       format( '   What is the new second derivative amplitude ',
     1    'factor ? ', $ )
        kappa = Qresp( Hkbdlu, 0.1, 10.0, 3 )
        x_join = qzerli( xmin, xmax, err_slope, tolerance )
        diff = qzerli( xmin, xmax, err_displ, tolerance ) - x_join
        write( Hvdulu, 1 ) diff, x_join
c
        if ( .not. Equery( Hkbdlu, 'Finished fine tuning ?' ) ) goto 2
c       until ( finished )
      return
      end


      SUBROUTINE PROFILE
c
c       Plot the modified profile and write it to file
c
      implicit none
      integer Hnpts, Hmm, Hvdulu, Hkbdlu, Hopflu
      parameter( Hnpts=400, Hmm=1000.0, Hvdulu=6, Hkbdlu=5, Hopflu=4 )
c
      integer npts
      real abscissa(Hnpts), new(Hnpts), old(Hnpts), x, ydd, x1, qcont
      character Cnote*48, Cfile*64
      include 'modify.cmn/list'
c
      write( Hvdulu, 1 )
1     format( ' Name of output file ? ', $ )
      read( Hkbdlu, 2 ) Cfile
2     format( a )
      open( UNIT=Hopflu, FILE=Cfile, STATUS='new' )
      write( Hopflu, 5 )
5     format( ' ; dist.     old      new    difference' /
     1  ' ; from    profile  profile' / ' ; inlet' /
     2  ' ; ( dimensions in mm )' )
c
c       calculate parameter x1 for the second derivative function
c
      npts = 0
      ydd = ( - qcont( x_join + xinc + xinc ) +
     1  16.0 * qcont( x_join + xinc ) - 30.0 * qcont( x_join ) +
     2  16.0 * qcont( x_join - xinc ) - qcont( x_join - xinc - xinc ) )
     3  / 12.0 / xinc **2
      x1 = x_join - ydd / kappa / x_join
c
c         calculate the new and old contraction profiles
c
      do x = xmin, xmax, xstep
          npts = npts + 1
          ydd = ( - qcont( x + xinc + xinc ) +
     1        16.0 * qcont( x + xinc ) - 30.0 * qcont( x) +
     2        16.0 * qcont( x - xinc ) - qcont( x - xinc - xinc ) ) /
     3        12.0 / xinc **2
          abscissa(npts) = x
          old(npts) = qcont( x )
          if ( x .gt. 0.0 .and. x .lt. x_join ) then
            new(npts) = kappa * x **3 / 6.0 * ( x / 2.0 - x1 ) + h
          else
            new(npts) = old(npts)
            end if
          write( Hopflu, 3 ) x * Hmm, old(npts) * Hmm, new(npts) * Hmm,
     1      old(npts) * Hmm - new(npts) * Hmm
3         format( 1x, 4f9.3 )
          end do
c
c         plot old and new profiles
c
      call ilgline( 0, 1, 1 )
      call ilgdata( abscissa, old, 1, npts, 1 )
      call ilgline( 0, 2, 1 )
      call ilgdata( abscissa, new, 1, npts, 0 )
      write( Cnote, 4 ) x_join, h, x_join
4     format( '|m(', f9.3, ',', f9.3, ')|d(', f9.3, ',0.0)' )
      call ilgnote( Cnote )
      return
      end


      SUBROUTINE CONTOUR
c
c           READ CONTRACTION PROFILE FILE & MAKE CUBIC SPLINE FUNCTION
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hprflu = 8          # contraction profile file
c variables
c  integer iline,      # line number in file
c          ierr,       # error flag from ICSCCU
c          jljstr      # function : left justify string
c  character Cfile*64  # name of file
c            Cline*80  # line read from data file
c  logical existf      # function : file exists ?
c
      implicit none
      integer Hkbdlu, Hvdulu, Hprflu
      real Hmm
      parameter( Hkbdlu=5, Hvdulu=6, Hprflu=8, Hmm=1000.0 )
c
      integer ierr, ijunk, iline, jljstr
      character Cfile*64, Cline*80
      logical existf
      include 'modify.cmn/list'
c
c           open input file
c
c     repeat                                                                1b
1       write( Hvdulu, 2 )
2       format( ' Name of contraction profile file ? ', $ )
        read( Hkbdlu, 3 ) Cfile
3       format( a )
        if ( .not. Existf( Cfile ) ) goto 1
c       until( file exists )                                                1e
      open( UNIT=Hprflu, FILE=Cfile, STATUS='old' )
c
c           read contraction profile
c
      ncont = 1
      iline = 1
c     start loop                                                            1b
4       iline = iline + 1
        read( Hprflu, 3, END=7 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then        2b
          read( Cline, *, ERR=5 ) ijunk, zcont(ncont), xcont(ncont)
          zcont(ncont) = zcont(ncont) * 25.4 / Hmm
          xcont(ncont) = xcont(ncont) * 25.4 / Hmm
          ncont = ncont + 1
          end if                                                            2e
        goto 4
5       write( Hvdulu, 6 ) iline, Cline
6       format( ' Cannot read file at line ', i8 / 1x, a )
        goto 4
c       end of loop                                                         1e
c
7     ncont = ncont - 1
      close( UNIT=Hprflu )
c
c           calculate cubic spline fit
c
      call sortsh( tcont, ncont, 1, Hncont, 1, 5 )
      call icsccu( zcont, xcont, ncont, spline, Hncont, ierr )
      if ( ierr .ne. 0 ) then                                               1b
        write( Hvdulu, 8 ) ierr
8       format( ' FATAL : Cannot calculate spline, ierr = ', i7 )
        stop
        end if                                                              1e
      return
      end


      SUBROUTINE PARAMETERS
c
c         SET USERS PARAMETERS
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hmm=1000.0          # 1000 mm in a metre
c variables
c  integer item,       # item selected from the menu
c          Jresp,      # function : integer response
c  real Qresp          # function : real response
c  character Ctaper(0:6)*8     # available taper types
c
      implicit none
      integer Hkbdlu, Hvdulu
      real Hmm
      parameter( Hkbdlu=5, Hvdulu=6, Hmm=1000.0 )
c
      integer item, Jresp
      real Qresp, qcont
      character Ctaper(0:6)*8
      include 'modify.cmn/list'
c
      h = qcont( 0.0 )
      do while( .true. )                                                  1b
        call newscr
        write( Hvdulu, 10 ) kappa, xinc * Hmm, xmin * Hmm, xmax * Hmm,
     1    xstep * Hmm, tolerance
10      format( ' Change parameters.  0) exit' /
     1    9x, '1) second derivative amplitude factor = ', f7.3 /
     2    9x, '2) finite differencing increment      = ', f7.2, 'mm' /
     3    9x, '3) minimum distance downstream        = ', f7.2, 'mm' /
     4    9x, '4) maximum distance downstream        = ', f7.2, 'mm' /
     5    9x, '5) plotting step size                 = ', f7.2, 'mm' /
     5    9x, '6) zero detection tolerance           = ', g13.6,
     6    '   ', $ )
        item = Jresp( Hkbdlu, 0, 6, 3 )
        if ( item .eq. 0 ) then                                           2b
          return
        else if ( item .eq. 1 ) then                                      2m
          write( Hvdulu, 11 )
11        format( ' What is the 2nd derivative amplitude factor ? ', $ )
          kappa = Qresp( Hkbdlu, 0.01, 100.0, 3 )
        else if ( item .eq. 2 ) then                                      2m
          write( Hvdulu, 12 )
12        format( ' What is finite differencing increment ? (mm) ', $ )
          xinc = Qresp( Hkbdlu, 0.01, 200.0, 3 ) / Hmm
        else if ( item .eq. 3 ) then                                      2m
          write( Hvdulu, 13 )
13        format( ' What is the minimum distance downstream ? (mm) ', $)
          xmin = Qresp( Hkbdlu, 0.0, 2000.0 - xinc, 3 ) / Hmm
        else if ( item .eq. 4 ) then                                      2m
          write( Hvdulu, 14 )
14        format( ' What is the maximum distance downstream ? (mm) ', $)
          xmax = Qresp( Hkbdlu, xmin + xinc, 2000.0, 3 ) / Hmm
        else if ( item .eq. 5 ) then                                      2m
          write( Hvdulu, 15 )
15        format( ' What is the plotting step size ? (mm) ', $ )
          xstep = Qresp( Hkbdlu, 0.002, 0.200, 3 ) / Hmm
        else if ( item .eq. 6 ) then                                      2m
          write( Hvdulu, 16 )
16        format( ' What is the zero detection tolerance ? ', $ )
          tolerance = qresp( Hkbdlu, 1.0e-9, 1.0, 3 )
          end if                                                          2e
        end do
      end


      REAL FUNCTION QCONT( Z )
c
c       Evaluate a point on the wind tunnel contraction by interpolation with
c  the cubic spline fitted to the data .
c
c Argument
c  real z              # distance from contraction inlet
c
c constant
c  Hvdulu=6            # vdu logical unit no.
c variable
c  integer ierr,       # error flag from ICSCCU
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real d, z
      integer *4 ind, ierr, jbsrch
      include 'modify.cmn/list'
c
c
      if ( z .le. zcont(1) ) then                                           1b
        qcont = xcont(1)
      else if ( z .ge. zcont(ncont) ) then                                  1m
        qcont = xcont(ncont)
      else                                                                  1m
        ind = jbsrch( zcont, 1, ncont, z )
        d   = z - zcont(ind)
        qcont = ( ( spline(ind,3) * d + spline(ind,2) ) * d +
     1          spline(ind,1) ) * d + xcont(ind)
        end if                                                              1e
      return
      end


      REAL FUNCTION ERR_DISPL( X_F )
c
c         Error function for the modified profile displacement
c Argument
c  real x_f            # location of end of modified profile
c Variables
c  real ydd,           # second derivative of the old profile
c       x1,            # constant in the second deriv of modified profile fn
c       qcont          # function : evaluate point on contraction contour
c
      implicit none
      real ydd, x_f, x1, qcont
      include 'modify.cmn/list'
c
      ydd = ( - qcont( x_f + xinc + xinc ) + 16.0 * qcont( x_f + xinc)
     1    - 30.0 * qcont( x_f) + 16.0 * qcont( x_f- xinc ) -
     2    qcont( x_f- xinc - xinc ) ) / 12.0 / xinc **2
      x1 = x_f - ydd / kappa / x_f
      err_displ = kappa * x_f **3 / 6.0 * ( x_f / 2.0 - x1 ) + h -
     1    qcont( x_f )
      return
      end


      REAL FUNCTION ERR_SLOPE( X_F )
c
c         Error function for the modified profile slope
c Argument
c  real x_f            # location of end of modified profile
c Variables
c  real yd,            # slope of the old profile
c       ydd,           # second derivative of the old profile
c       x1,            # constant in the second deriv of modified profile fn
c       qcont          # function : evaluate point on contraction contour
c
      implicit none
      real yd, ydd, x_f, x1, qcont
      include 'modify.cmn/list'
c
      yd =  ( qcont( x_f + xinc ) - qcont( x_f - xinc ) ) / 2.0 / xinc
      ydd = ( - qcont( x_f + xinc + xinc ) + 16.0 * qcont( x_f + xinc)
     1    - 30.0 * qcont( x_f) + 16.0 * qcont( x_f- xinc ) -
     2    qcont( x_f- xinc - xinc ) ) / 12.0 / xinc **2
      x1 = x_f - ydd / kappa / x_f
      err_slope = kappa * x_f **2 * ( x_f / 3.0 - x1 / 2.0 ) - yd
      return
      end

