c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM PREP7
c
c   Program to generate a file for ANSYS PREP7 preprocessor . The problem to
c   be solved is the potential flow through a wind tunnel contraction using
c   the `heat conduction through a homogeneous solid' analogy .
c
c  The wind-tunnel contraction has a regular octagonal cross-section geometry.
c

      implicit none
      integer *2 Hkbdlu, Hvdulu, Hprep7, Hprflu
      integer *4 Hncont
      parameter( Hkbdlu=5, Hvdulu=6, Hprep7=7, Hprflu=8, Hncont=50 )
c
      call contour
      call parameters
      call header
      call nodes
      call elements
      call trailer
      stop
      end


      SUBROUTINE CONTOUR
c
c           READ CONTRACTION PROFILE FILE & MAKE CUBIC SPLINE FUNCTION
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hprflu = 8          # contraction profile file
c variables
c  integer iline,      # line number in file
c          ierr,       # error flag from ICSCCU
c          jljstr      # function : left justify string
c  character Cfile*64  # name of file
c            Cline*80  # line read from data file
c  logical existf      # function : file exists ?
c
      implicit none
      integer Hkbdlu, Hvdulu, Hprflu
      parameter( Hkbdlu=5, Hvdulu=6, Hprflu=8 )
c
      integer ierr, ijunk, iline, jljstr
      character Cfile*64, Cline*80
      logical existf
      include 'octagon.cmn/list'
c
c           open input file
c
c     repeat                                                                1b
1       write( Hvdulu, 2 )
2       format( ' Name of contraction profile file ? ', $ )
        read( Hkbdlu, 3 ) Cfile
3       format( a )
        if ( .not. Existf( Cfile ) ) goto 1
c       until( file exists )                                                1e
      open( UNIT=Hprflu, FILE=Cfile, STATUS='old' )
c
c           read contraction profile
c
      ncont = 1
      iline = 1
c     start loop                                                            1b
4       iline = iline + 1
        read( Hprflu, 3, END=7 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then        2b
          read( Cline, *, ERR=5 ) ijunk, zcont(ncont), xcont(ncont)
          zcont(ncont) = zcont(ncont) * 25.4 / 1000.0
          xcont(ncont) = xcont(ncont) * 25.4 / 1000.0
          ncont = ncont + 1
          end if                                                            2e
        goto 4
5       write( Hvdulu, 6 ) iline, Cline
6       format( ' Cannot read file at line ', i8 / 1x, a )
        goto 4
c       end of loop                                                         1e
c
7     ncont = ncont - 1
      close( UNIT=Hprflu )
c
c           calculate cubic spline fit
c
      call sortsh( tcont, ncont, 1, Hncont, 1, 5 )
      call icsccu( zcont, xcont, ncont, spline, Hncont, ierr )
      if ( ierr .ne. 0 ) then                                               1b
        write( Hvdulu, 8 ) ierr
8       format( ' FATAL : Cannot calculate spline, ierr = ', i7 )
        stop
        end if                                                              1e
      return
      end


      SUBROUTINE PARAMETERS
c
c           GET PARAMETERS FROM USER
c     N.B. Metric units are used .
c
c constants
c  Hkbdlu = 5                  # keyboard logical unit no.
c  Hvdulu = 6                  # vdu logical unit no.
c variables
c  integer item,               # item selected for change
c          Jresp               # function : integer response
c  real zlap_in, zlap_out,     # inlet and outlet overlap
c       Qresp                  # function : real response
c
      implicit none
      integer Hvdulu, Hkbdlu
      parameter( Hvdulu=6, Hkbdlu=5 )
c
      integer item, Jresp
      real zlap_in, zlap_out, Qresp
      include 'octagon.cmn/list'
      data zlap_in, zlap_out / 0.4, 0.1 /
c
      do while ( .true. )                                                 1b
        write( Hvdulu, 11 ) zlap_in, zlap_out, nsegments,
     1    high_temp, low_temp
11      format( ' Change which item ?  0) Exit ' /
     1    22x, '1) Inlet overlap          = ', g12.5 /
     2    22x, '2) Exit overlap           = ', g12.5 /
     3    22x, '3) No. of radial segments = ', i4 /
     5    22x, '4) Inlet temperature      = ', g12.5 /
     6    22x, '5) Exit temperature       = ', g12.5, '   ', $ )
        item = Jresp( Hkbdlu, 0, 5, 3 )
        if ( item .eq. 0 ) then                                           2b
          goto 12
c
c           limits on streamwise distance
c
        else if ( item .eq. 1 ) then                                      2m
          write( Hvdulu, 1 )
1         format( ' Streamwise inlet overlap ? (m) ', $ )
          zlap_in = qresp( Hkbdlu, 0.0, 0.7, 3 )
        else if ( item .eq. 2 ) then                                      2m
          write( Hvdulu, 2 )
2         format( ' Streamwise exit overlap ? (m) ', $ )
          zlap_out = qresp( Hkbdlu, 0.0, 0.4, 3 )
c
c           number of line segments on a radius
c
        else if ( item .eq. 3 ) then                                      2m
          write( Hvdulu, 3 )
3         format( ' Number of radial segments ? ', $ )
          nsegments = jresp( Hkbdlu, 4, 14, 3 )
c
c           contraction inlet and exit temperatures
c
        else if ( item .eq. 4 ) then                                      2m
          write( Hvdulu, 4 )
4         format( ' Inlet temperature ? ', $ )
          high_temp = qresp( Hkbdlu, 0.0, 1.0e6, 3 )
        else if ( item .eq. 5 ) then                                      2m
          write( Hvdulu, 5 )
5         format( ' Exit temperature ? ', $ )
          low_temp = qresp( Hkbdlu, -1.0e6, 0.0, 3 )
          end if                                                          2e
        end do                                                            1e
c
12    zmin = zcont(1) - zlap_in
      zmax = zcont(ncont) + zlap_out
      n_xsect = ( nsegments + 1 ) **2  ! this is wrong !!!!!!!!!!!!!
      end


      SUBROUTINE HEADER
c
c         WRITE THE PREP7 HEADER
c
c constant
c  Hprep7 = 7          # logical unit no. for PREP7 file
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c variables
c  character Cfile*64  # name of file
c
      implicit none
      integer Hprep7, Hkbdlu, Hvdulu, Hprflu, Hncont
      parameter( Hprep7=7, Hkbdlu=5, Hvdulu=6, Hncont=50 )
c
      character Cfile*64
c
c           open the PREP7 file
c
      write( Hvdulu, 10 )
10    format( ' Name of the PREP7 file ? ', $ )
      read( Hkbdlu, 11 ) Cfile
11    format( a )
      open( UNIT=Hprep7, FILE=Cfile, ACCESS='sequential',
     1  STATUS='new', CARRIAGECONTROL='list', FORM='formatted' )
c
c           write the PREP7 file header
c
      write( Hprep7, 12 )
12    format( '/PREP7' /
     1        '/TITLE wind tunnel contraction potential flow' /
     2        'KAN,-1'  /
     3        'ET,1,70' /
     4        'KXX,1,1' / )
      return
      end


      SUBROUTINE NODES
c
c       WRITE THE NODES TO THE PREP7 FILE.
c
c   Also counts nodes.
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c  Hvdulu = 6                  # logical unit no. of vdu
c  Hr2m1  = 0.4142135624
c variables
c  integer icol,               # node array column number
c          irow,               # node array row number
c          node                # node number
c real
c  real edge,                  # distance of contraction profile from c/line
c       edge_last,             # previous value for edge
c       slope,                 # slope of contraction over previous interval
c       x,                     # distance from vertical centre-plane
c       y,                     # distance from horizontal centre-plane
c       ystep,                 # y increment
c       z,                     # distance from contraction entrance
c       z_last,                # previous value for z
c       qcont                  # function :  calculate point on profile
c
      implicit none
      integer Hprep7, Hvdulu
      real Hr2m1
      parameter( Hprep7=7, Hvdulu=6, Hr2m1=0.4142135624 )
c
      integer icol, irow, node
      real edge, edge_last, slope, x, y, ystep, z, z_last, qcont
      include 'octagon.cmn/list'
c
      n_first = 1
      n_last  = 0
c
c         initialize loop
c
      z = zmin
      nsections = 0
      slope = 0.0
      edge = qcont( z )
      node = 1
c
c         body of loop
c
      do while( z .le. zmax )                                             1b
        x = 0.0
        y = 0.0
        ystep = edge * Hr2m1 / nsegments
        do icol = 0, nsegments
          y = 0.0
          do irow = 1, ( icol + 3 ) / 2
            write( Hprep7, 1 ) node, x, y, z
1           format( ' N,',i4, ',', g12.5, ',', g12.5, ',', g12.5 )
            if ( irow .eq. 1 .and. mod( icol, 2 ) .eq. 1 ) then
              y = y + ystep
            else
              y = y + ystep  * 2.0
              end if
            node = node + 1
            end do
          x = x + edge / nsegments
          end do
c
c         increments
c
        z_last = z
        z = z + 2.0 * edge / ( real( 2 * nsegments ) - slope )
        edge_last = edge
        edge = qcont( z )
        slope = ( edge - edge_last ) / ( z - z_last )
c
        if ( nsections .le. 0 )
     1    n_xsect = node - 1
        nsections = nsections + 1
        end do                                                            1e
      n_last = node - 1
      write( Hvdulu, 2 ) nsections, n_xsect, n_last
2     format( ' number of cross sections  = ', i5 /
     1        ' no. of nodes in a section = ', i5 /
     1        ' total number of nodes     = ', i5 )
      return
      end


      REAL FUNCTION QCONT( Z )
c
c       Evaluate a point on the wind tunnel contraction by interpolation with
c  the cubic spline fitted to the data .
c
c Argument
c  real z              # distance from contraction inlet
c
c constant
c  Hvdulu=6            # vdu logical unit no.
c variable
c  integer ierr,       # error flag from ICSCCU
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real d, z
      integer *4 ind, ierr, jbsrch
      include 'octagon.cmn/list'
c
      if ( z .le. zcont(1) ) then                                           1b
        qcont = xcont(1)
      else if ( z .ge. zcont(ncont) ) then                                  1m
        qcont = xcont(ncont)
      else                                                                  1m
        ind = jbsrch( zcont, 1, ncont, z )
        d   = z - zcont(ind)
        qcont = ( ( spline(ind,3) * d + spline(ind,2) ) * d +
     1          spline(ind,1) ) * d + xcont(ind)
        end if                                                              1e
      return
      end


      SUBROUTINE ELEMENTS
c
c         DEFINE THE ELEMENTS - in terms of nodes
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c variable
c  integer icntr,              # number of elements done in this column
c          icol, irow,         # column and row number
c          isect,              # cross section number in a cross section
c          k,                  # increment required to get to the last node
c          nelem,              # number of elements
c          next,               # first node in next section for current element
c          node,               # first node in this section for current element
c
      implicit none
      integer Hprep7, Hvdulu
      parameter( Hprep7=7, Hvdulu=6 )
c
      integer icntr, icol, irow, isect, k, nelem, next, node
      include 'octagon.cmn/list'
c
      nelem = 0
      do isect = 1, nsections - 1                                         1b
        node = ( isect - 1 ) * n_xsect
        next  = node + n_xsect
        do icol = 1, nsegments                                            2b
          icntr = 0
          k = ( icol + 3 ) / 2
          if ( mod( icol, 2 ) .eq. 0 ) then                               3b
c
c             for even numbered columns , element types
c               ALPHA, BETA, ALPHA, BETA ....
c
            node  = node  + 1
            next  = next  + 1
            do irow = 1, ( icol + 1 ) / 2                                 4b
              if ( icntr .lt. icol ) then                                 5b
c
c               Type alpha element
c
                icntr = icntr + 1
                nelem = nelem + 1
                write( Hprep7, 1 ) node, node+1, node+k, node+k,
     1                             next, next+1, next+k, next+k
1               format( 'E,', i4, 7( ',', i4 ) )
                if ( icntr .lt. icol ) then                               6b
c
c               Type beta element
c
                  icntr = icntr + 1
                  node  = node  + 1
                  next  = next  + 1
                  nelem = nelem + 1
                  write( Hprep7, 1 ) node, node-1+k, node+k, node+k,
     1                               next, next-1+k, next+k, next+k
                  end if                                                  6e
                end if                                                    5e
              end do                                                      4e
          else                                                          3m
c
c             for odd numbered columns , element types
c                BETA, ALPHA, BETA, ALPHA ....
c
            do irow = 1, ( icol + 1 ) / 2                                 4b
              if ( icntr .lt. icol ) then                                 5b
c
c               Type beta element
c
                icntr = icntr + 1
                node  = node  + 1
                next  = next  + 1
                nelem = nelem + 1
                write( Hprep7, 1 ) node, node-1+k, node+k, node+k,
     1                             next, next-1+k, next+k, next+k
c
c               Type alpha element
c
                if ( icntr .lt. icol ) then                               6b
                  icntr = icntr + 1
                  nelem = nelem + 1
                  write( Hprep7, 1 ) node, node+1, node+k, node+k,
     1                               next,  next+1,  next+k,  next+k
                  end if                                                  6e
                end if                                                    5e
              end do                                                      4e
            end if                                                        3e
          end do                                                          2e
        end do                                                            1e
      write( Hvdulu, 2 ) nelem
2     format( ' number of elements  = ', i4 )
      return
      end


      SUBROUTINE TRAILER
c
c           WRITE THE PREP7 FILE TRAILER
c constant
c  Hprep7 = 7                  # logical unit no. for PREP7 file
c
      implicit none
      integer Hprep7
      parameter( Hprep7=7 )
c
      include 'octagon.cmn/list'
c
      write( Hprep7, 4 ) n_first, n_first + n_xsect - 1, low_temp,
     1    n_last - n_xsect + 1, n_last, high_temp
4     format( '/SHOW,9600,4014' /
     1        '/VIEW,1,1,1,1' /
     2        'ELSIZE,,1' /                       !out
     3        'VMESH,ALL' /                       !out
     4        'NSEL,NODE,', i4, ',', i4 /
     5        'NT,ALL,TEMP,',g12.5 /
     6        'NALL' /
     7        'NSEL,NODE,', i4, ',', i4 /
     8        'NT,ALL,TEMP,',g12.5 /
     9        'NALL' /
     a        'ITER,1,1,1' /
     b        'WSORT,Z' / )
      close( UNIT=Hprep7 )
      return
      end
