c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM PREP7
c
c   Program to generate a file for ANSYS PREP7 preprocessor . The problem to
c   be solved is the potential flow through a wind tunnel contraction using
c   the `heat conduction through a homogeneous solid' analogy
c
c  The wind-tunnel contraction has a square octagonal cross-section geometry.
c 
      implicit none
      integer *2 Hkbdlu, Hvdulu, Hprep7, Hprflu
      integer *4 Hncont
      parameter( Hkbdlu=5, Hvdulu=6, Hprep7=7, Hprflu=8, Hncont=50 )
c
      call contour
      call parameters
      call header
      call keypoints
      call lines
      call volumes
      call trailer
      stop
      end


      SUBROUTINE CONTOUR
c
c           READ CONTRACTION PROFILE FILE & MAKE CUBIC SPLINE FUNCTION
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hprflu = 8          # contraction profile file
c variables
c  integer iline,      # line number in file
c          ierr,       # error flag from ICSCCU
c          jljstr      # function : left justify string
c  character Cfile*64  # name of file
c            Cline*80  # line read from data file
c  logical existf      # function : file exists ?
c
      implicit none
      integer Hkbdlu, Hvdulu, Hprflu
      parameter( Hkbdlu=5, Hvdulu=6, Hprflu=8 )
c
      integer ierr, ijunk, iline, jljstr
      character Cfile*64, Cline*80
      logical existf
      include 'square.cmn/list'
c
c           open input file
c
c     repeat                                                                1b
1       write( Hvdulu, 2 )
2       format( ' Name of contraction profile file ? ', $ )
        read( Hkbdlu, 3 ) Cfile
3       format( a )
        if ( .not. Existf( Cfile ) ) goto 1
c       until( file exists )                                                1e
      open( UNIT=Hprflu, FILE=Cfile, STATUS='old' )
c
c           read contraction profile
c
      ncont = 1
      iline = 1
c     start loop                                                            1b
4       iline = iline + 1
        read( Hprflu, 3, END=7 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then        2b
          read( Cline, *, ERR=5 ) ijunk, zcont(ncont), xcont(ncont)
          zcont(ncont) = zcont(ncont) * 25.4 / 1000.0
          xcont(ncont) = xcont(ncont) * 25.4 / 1000.0
          ncont = ncont + 1
          end if                                                            2e
        goto 4
5       write( Hvdulu, 6 ) iline, Cline
6       format( ' Cannot read file at line ', i8 / 1x, a )
        goto 4
c       end of loop                                                         1e
c
7     ncont = ncont - 1
      close( UNIT=Hprflu )
c
c           calculate cubic spline fit
c
      call sortsh( tcont, ncont, 1, Hncont, 1, 5 )
      call icsccu( zcont, xcont, ncont, spline, Hncont, ierr )
      if ( ierr .ne. 0 ) then                                               1b
        write( Hvdulu, 8 ) ierr
8       format( ' FATAL : Cannot calculate spline, ierr = ', i7 )
        stop
        end if                                                              1e
      return
      end


      SUBROUTINE PARAMETERS
c
c           GET PARAMETERS FROM USER
c     N.B. Metric units are used .
c
c constants
c  Hkbdlu = 5                  # keyboard logical unit no.
c  Hvdulu = 6                  # vdu logical unit no.
c variables
c  integer item,               # item selected for change
c          Jresp               # function : integer response
c  real zlap_in, zlap_out,     # inlet and outlet overlap
c       Qresp                  # function : real response
c
      implicit none
      integer Hvdulu, Hkbdlu
      parameter( Hvdulu=6, Hkbdlu=5 )
c
      integer item, Jresp
      real zlap_in, zlap_out, Qresp
      include 'square.cmn/list'
      data zlap_in, zlap_out / 0.2, 0.05 /
c
      do while ( .true. )                                                 1b
        write( Hvdulu, 11 ) zlap_in, zlap_out, nsegments, key_inc,
     1    high_temp, low_temp
11      format( ' Change which item ?  0) Exit ' /
     1    22x, '1) Inlet overlap          = ', g12.5 /
     2    22x, '2) Exit overlap           = ', g12.5 /
     3    22x, '3) No. of radial segments = ', i4 /
     4    22x, '4) Keypoints increment    = ', i4 /
     5    22x, '5) Inlet temperature      = ', g12.5 /
     6    22x, '6) Exit temperature       = ', g12.5, '   ', $ )
        item = Jresp( Hkbdlu, 0, 6, 3 )
        if ( item .eq. 0 ) then                                           2b
          goto 12
c
c           limits on streamwise distance
c
        else if ( item .eq. 1 ) then                                      2m
          write( Hvdulu, 1 )
1         format( ' Streamwise inlet overlap ? (m) ', $ )
          zlap_in = qresp( Hkbdlu, 0.0, 0.7, 3 )
        else if ( item .eq. 2 ) then                                      2m
          write( Hvdulu, 2 )
2         format( ' Streamwise exit overlap ? (m) ', $ )
          zlap_out = qresp( Hkbdlu, 0.0, 0.4, 3 )
c
c           number of line segments on a radius
c
        else if ( item .eq. 3 ) then                                      2m
          write( Hvdulu, 3 )
3         format( ' Number of radial segments ? ', $ )
          nsegments = jresp( Hkbdlu, 4, 14, 3 )
c
c           keypoint increment from section to section
c
        else if ( item .eq. 4 ) then                                      2m
          write( Hvdulu, 4 )
4         format( ' Key points increment ? ', $ )
          key_inc = Jresp( Hkbdlu, ncorners, 1000, 3 )
c
c           contraction inlet and exit temperatures
c
        else if ( item .eq. 5 ) then                                      2m
          write( Hvdulu, 5 )
5         format( ' Inlet temperature ? ', $ )
          high_temp = qresp( Hkbdlu, 0.0, 1.0e6, 3 )
        else if ( item .eq. 6 ) then                                      2m
          write( Hvdulu, 6 )
6         format( ' Exit temperature ? ', $ )
          low_temp = qresp( Hkbdlu, -1.0e6, 0.0, 3 )
          end if                                                          2e
        end do                                                            1e
c
12    zmin = zcont(1) - zlap_in
      zmax = zcont(ncont) + zlap_out
      n_xsect = ( nsegments + 1 ) **2
      return
      end


      SUBROUTINE HEADER
c
c         WRITE THE PREP7 HEADER
c
c constant
c  Hprep7 = 7          # logical unit no. for PREP7 file
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c variables
c  character Cfile*64  # name of file
c
      implicit none
      integer Hprep7, Hkbdlu, Hvdulu, Hprflu, Hncont
      parameter( Hprep7=7, Hkbdlu=5, Hvdulu=6, Hncont=50 )
c
      character Cfile*64
c
c           open the PREP7 file
c
      write( Hvdulu, 10 )
10    format( ' Name of the PREP7 file ? ', $ )
      read( Hkbdlu, 11 ) Cfile
11    format( a )
      open( UNIT=Hprep7, FILE=Cfile, ACCESS='sequential',
     1  STATUS='new', CARRIAGECONTROL='list', FORM='formatted' )
c
c           write the PREP7 file header
c
      write( Hprep7, 12 )
12    format( '/PREP7' /
     1        '/TITLE wind tunnel contraction potential flow' /
     2        'KAN,-1'  /
     3        'ET,1,70' /
     4        'KXX,1,1' / )
      return
      end


      SUBROUTINE KEYPOINTS
c
c           Write keypoints to the PREP7 file
c   Also counts nodes.
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c variables
c  integer key_pt              # identifier for 1st keypoint in a x-section
c real
c  real edge,                  # distance of contraction profile from c/line
c       z,                     # distance from contraction entrance
c       qcont                  # function :  calculate point on profile
c
      implicit none
      integer Hprep7
      real Hr2m1, Hsqrt2
      parameter( Hprep7=7, Hr2m1=0.4142135624, Hsqrt2=1.4142135624 )
c
      integer key_pt
      real edge, edge_last, z, z_last, slope, qcont
      include 'square.cmn/list'
c
      n_first = 1
      n_last  = 0
      k_first = 1
c
c         initialize loop
c
      z = zmin
      key_pt = 1
      nsections = 0
      slope = 0.0
      edge = qcont( z )
c
c         body of loop
c
      do while( z .le. zmax )                                             1b
        write( Hprep7, 1 ) key_pt,     0.0,  0.0,  z
        write( Hprep7, 1 ) key_pt + 1, 0.0,  edge, z
        write( Hprep7, 1 ) key_pt + 2, edge, edge, z
        write( Hprep7, 1 ) key_pt + 3, edge, 0.0,  z
1       format( 'k,', i4, ',', g12.5, ',', g12.5, ',', g12.5 )
c
c         increments
c
        z_last = z
        z = z + 2.0 * edge / ( real( 2 * nsegments ) - slope )
        edge_last = edge
        edge = qcont( z )
        slope = ( edge - edge_last ) / ( z - z_last )
c
        k_last = key_pt + 3
        key_pt = key_pt + key_inc
        n_last = n_last + n_xsect
        nsections = nsections + 1
        end do                                                            1e
      return
      end


      REAL FUNCTION QCONT( Z )
c
c       Evaluate a point on the wind tunnel contraction by interpolation with
c  the cubic spline fitted to the data .
c
c Argument
c  real z              # distance from contraction inlet
c
c constant
c  Hvdulu=6            # vdu logical unit no.
c variable
c  integer ierr,       # error flag from ICSCCU
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real d, z
      integer *4 ind, ierr, jbsrch
      include 'square.cmn/list'
c
      if ( z .le. zcont(1) ) then                                           1b
        qcont = xcont(1)
      else if ( z .ge. zcont(ncont) ) then                                  1m
        qcont = xcont(ncont)
      else                                                                  1m
        ind = jbsrch( zcont, 1, ncont, z )
        d   = z - zcont(ind)
        qcont = ( ( spline(ind,3) * d + spline(ind,2) ) * d +
     1          spline(ind,1) ) * d + xcont(ind)
        end if                                                              1e
      return
      end


      SUBROUTINE LINES
c
c         DEFINES THE LINE SEGMENTS JOINING KEYPOINTS
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c variables
c  integer key_pt              # identifier for 1st keypoint in a x-section
c          key_nxt             # 1st keypoint in next x-section
c
      implicit none
      integer Hprep7
      parameter( Hprep7=7 )
c
      integer key_pt, key_nxt
      include 'square.cmn/list'
c
c         initialize loop
c
      key_pt  = k_first
      key_nxt = k_first + key_inc
c
c         body of loop
c
      do while( key_nxt .le. k_last )
        write( Hprep7, 1 ) key_pt,     key_pt + 1, nsegments
        write( Hprep7, 1 ) key_pt + 1, key_pt + 2, nsegments
        write( Hprep7, 1 ) key_pt + 2, key_pt + 3, nsegments
        write( Hprep7, 1 ) key_pt + 3, key_pt,     nsegments
        write( Hprep7, 1 ) key_pt,     key_nxt,     1
        write( Hprep7, 1 ) key_pt + 1, key_nxt + 1, 1
        write( Hprep7, 1 ) key_pt + 2, key_nxt + 2, 1
        write( Hprep7, 1 ) key_pt + 3, key_nxt + 3, 1
c
c         increments
c
        key_pt  = key_pt  + key_inc
        key_nxt = key_nxt + key_inc
        end do                                                            1e
      write( Hprep7, 1 ) key_pt,     key_pt + 1, nsegments
      write( Hprep7, 1 ) key_pt + 1, key_pt + 2, nsegments
      write( Hprep7, 1 ) key_pt + 2, key_pt + 3, nsegments
      write( Hprep7, 1 ) key_pt + 3, key_pt,     nsegments
c
1     format( 'L,', i4, ',', i4, ',', i4 )
      return
      end


      SUBROUTINE VOLUMES
c
c         DEFINES THE VOLUMES
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c variables
c  integer key_pt              # identifier for 1st keypoint in a x-section
c          key_nxt             # 1st keypoint in next x-section
c
      implicit none
      integer Hprep7
      parameter( Hprep7=7 )
c
      integer key_pt, key_nxt
      include 'square.cmn/list'
c
c         initialize loop
c
      key_pt  = k_first
      key_nxt = k_first + key_inc
c
c         body of loop
c
      do while( key_nxt .le. k_last )
        write( Hprep7, 1 ) key_pt, key_pt + 1, key_pt + 2, key_pt + 3,
     1      key_nxt, key_nxt + 1, key_nxt + 2, key_nxt + 3
c
c         increments
c
        key_pt  = key_pt  + key_inc
        key_nxt = key_nxt + key_inc
        end do                                                            1e
1     format( 'V,', i4, ',', i4, ',', i4, ',', i4,
     1         ',', i4, ',', i4, ',', i4, ',', i4 )
      return
      end


      SUBROUTINE TRAILER
c
c           WRITE THE PREP7 FILE TRAILER
c constant
c  Hprep7 = 7                  # logical unit no. for PREP7 file
c
      implicit none
      integer Hprep7
      parameter( Hprep7=7 )
c
      include 'square.cmn/list'
c
      write( Hprep7, 4 ) n_first, n_first + n_xsect - 1, low_temp,
     1    n_last - n_xsect + 1, n_last, high_temp
4     format( '/SHOW,9600,4014' /
     1        '/VIEW,1,1,1,1' /
     2        'ELSIZE,,1' /
     3        'VMESH,ALL' /
     4        'NSEL,NODE,', i4, ',', i4 /
     5        'NT,ALL,TEMP,',g12.5 /
     6        'NALL' /
     7        'NSEL,NODE,', i4, ',', i4 /
     8        'NT,ALL,TEMP,',g12.5 /
     9        'NALL' /
     a        'ITER,1,1,1' /
     b        'WSORT,Z' / )
      close( UNIT=Hprep7 )
      return
      end
