c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM TAPER
c
c   Program to generate a file for ANSYS PREP7 preprocessor . The problem to
c   be solved is the potential flow through a wind tunnel contraction using
c   the `heat conduction through a homogeneous solid' analogy .
c     The wind tunnel contraction begins with a regular octagonal cross-section
c   at the inlet . The `corner fillets' are tapered to a point at the exit
c   so that the exit cross-section is square .
c
c N.B **** the algorithm used for transformation from octagonal to ****
c     **** square cross-section geometry with tetrahedral elements ****
c     **** does not work.  The node locations are OK, but you need ****
c     **** to redo the definitions of the tetrahedral elements by  ****
c     **** hand                                                    ****
c
      implicit none
      integer *2 Hkbdlu, Hvdulu, Hprep7, Hprflu
      integer *4 Hncont
      parameter( Hkbdlu=5, Hvdulu=6, Hprep7=7, Hprflu=8, Hncont=50 )
c
      call contour
      call parameters
      call header
      call nodes
      call elements
      call trailer
      stop
      end


      SUBROUTINE CONTOUR
c
c           READ CONTRACTION PROFILE FILE & MAKE CUBIC SPLINE FUNCTION
c
c constants
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c  Hprflu = 8          # contraction profile file
c variables
c  integer iline,      # line number in file
c          ierr,       # error flag from ICSCCU
c          jljstr      # function : left justify string
c  character Cfile*64  # name of file
c            Cline*80  # line read from data file
c  logical existf      # function : file exists ?
c
      implicit none
      integer Hkbdlu, Hvdulu, Hprflu
      parameter( Hkbdlu=5, Hvdulu=6, Hprflu=8 )
c
      integer ierr, ijunk, iline, jljstr
      character Cfile*64, Cline*80
      logical existf
      include 'taper.cmn/list'
c
c           open input file
c
c     repeat                                                                1b
1       write( Hvdulu, 2 )
2       format( ' Name of contraction profile file ? ', $ )
        read( Hkbdlu, 3 ) Cfile
3       format( a )
        if ( .not. Existf( Cfile ) ) goto 1
c       until( file exists )                                                1e
      open( UNIT=Hprflu, FILE=Cfile, STATUS='old' )
c
c           read contraction profile
c
      ncont = 1
      iline = 1
c     start loop                                                            1b
4       iline = iline + 1
        read( Hprflu, 3, END=7 ) Cline
        if ( jljstr( Cline ) .gt. 0 .and. Cline(1:1) .ne. ';' ) then        2b
          read( Cline, *, ERR=5 ) ijunk, zcont(ncont), xcont(ncont)
          zcont(ncont) = zcont(ncont) * 25.4 / 1000.0
          xcont(ncont) = xcont(ncont) * 25.4 / 1000.0
          ncont = ncont + 1
          end if                                                            2e
        goto 4
5       write( Hvdulu, 6 ) iline, Cline
6       format( ' Cannot read file at line ', i8 / 1x, a )
        goto 4
c       end of loop                                                         1e
c
7     ncont = ncont - 1
      close( UNIT=Hprflu )
c
c           calculate cubic spline fit
c
      call sortsh( tcont, ncont, 1, Hncont, 1, 5 )
      call icsccu( zcont, xcont, ncont, spline, Hncont, ierr )
      if ( ierr .ne. 0 ) then                                               1b
        write( Hvdulu, 8 ) ierr
8       format( ' FATAL : Cannot calculate spline, ierr = ', i7 )
        stop
        end if                                                              1e
      return
      end


      SUBROUTINE PARAMETERS
c
c           GET PARAMETERS FROM USER
c     N.B. Metric units are used .
c
c constants
c  Hkbdlu = 5                  # keyboard logical unit no.
c  Hvdulu = 6                  # vdu logical unit no.
c variables
c  integer item,               # item selected for change
c          Jresp               # function : integer response
c  real zlap_in, zlap_out,     # inlet and outlet overlap
c       Qresp                  # function : real response
c  character Ctaper(0:6)*8     # available taper types
c
      implicit none
      integer Hvdulu, Hkbdlu
      parameter( Hvdulu=6, Hkbdlu=5 )
c
      integer item, Jresp
      real zlap_in, zlap_out, Qresp
      character Ctaper(0:6)*8
      include 'taper.cmn/list'
      data Ctaper / 'none    ', 'lin fill', 'sin fill', 'lin gap ',
     1  'sin gap ', 'lin merg', 'sin merg' /
      data zlap_in, zlap_out / 0.4, 0.1 /
c
      do while ( .true. )                                                 1b
        write( Hvdulu, 11 ) zlap_in, zlap_out, nsegments,
     1    high_temp, low_temp, Ctaper(itaper)
11      format( ' Change which item ?  0) Exit ' /
     1    22x, '1) Inlet overlap          = ', g12.5 /
     2    22x, '2) Exit overlap           = ', g12.5 /
     3    22x, '3) No. of radial segments = ', i4 /
     5    22x, '4) Inlet temperature      = ', g12.5 /
     6    22x, '5) Exit temperature       = ', g12.5 /
     7    22x, '6) taper type             = ', a, '   ', $ )
        item = Jresp( Hkbdlu, 0, 6, 3 )
        if ( item .eq. 0 ) then                                           2b
          goto 12
c
c           limits on streamwise distance
c
        else if ( item .eq. 1 ) then                                      2m
          write( Hvdulu, 1 )
1         format( ' Streamwise inlet overlap ? (m) ', $ )
          zlap_in = qresp( Hkbdlu, 0.0, 0.7, 3 )
        else if ( item .eq. 2 ) then                                      2m
          write( Hvdulu, 2 )
2         format( ' Streamwise exit overlap ? (m) ', $ )
          zlap_out = qresp( Hkbdlu, 0.0, 0.4, 3 )
c
c           number of line segments on a radius
c
        else if ( item .eq. 3 ) then                                      2m
          write( Hvdulu, 3 )
3         format( ' Number of radial segments ? ', $ )
          nsegments = jresp( Hkbdlu, 4, 14, 3 )
c
c           contraction inlet and exit temperatures
c
        else if ( item .eq. 4 ) then                                      2m
          write( Hvdulu, 4 )
4         format( ' Inlet temperature ? ', $ )
          high_temp = qresp( Hkbdlu, 0.0, 1.0e6, 3 )
        else if ( item .eq. 5 ) then                                      2m
          write( Hvdulu, 5 )
5         format( ' Exit temperature ? ', $ )
          low_temp = qresp( Hkbdlu, -1.0e6, 0.0, 3 )
        else if ( item .eq. 6 ) then                                      2m
          write( Hvdulu, 14 )
14        format( ' Which taper type  0) none' /
     1      19x, '1) linear fillet' /  19x, '2) sinusoidal fillet' /
     2      19x, '3) linear gap'    / 19x, '4) sinusoid gap' /
     3      19x, '5) linear merge'  / 19x, '6) sinusoid merge   ',  $ )
          itaper = Jresp( Hkbdlu, 0, 6, 3 )
          end if                                                          2e
        end do                                                            1e
c
12    zmin = zcont(1) - zlap_in
      zmax = zcont(ncont) + zlap_out
      end


      SUBROUTINE HEADER
c
c         WRITE THE PREP7 HEADER
c
c constant
c  Hprep7 = 7          # logical unit no. for PREP7 file
c  Hkbdlu = 5          # keyboard logical unit no.
c  Hvdulu = 6          # vdu logical unit no.
c variables
c  character Cfile*64  # name of file
c
      implicit none
      integer Hprep7, Hkbdlu, Hvdulu, Hprflu, Hncont
      parameter( Hprep7=7, Hkbdlu=5, Hvdulu=6, Hncont=50 )
c
      character Cfile*64
c
c           open the PREP7 file
c
      write( Hvdulu, 10 )
10    format( ' Name of the PREP7 file ? ', $ )
      read( Hkbdlu, 11 ) Cfile
11    format( a )
      open( UNIT=Hprep7, FILE=Cfile, ACCESS='sequential',
     1  STATUS='new', CARRIAGECONTROL='list', FORM='formatted' )
c
c           write the PREP7 file header
c
      write( Hprep7, 12 )
12    format( '/PREP7' /
     1        '/TITLE wind tunnel contraction potential flow' /
     2        'KAN,-1'  /
     3        'ET,1,70' /
     4        'KXX,1,1' / )
      return
      end


      SUBROUTINE NODES
c
c       WRITE THE NODES TO THE PREP7 FILE.
c
c   Also counts nodes.
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c  Hvdulu = 6                  # logical unit no. of vdu
c variables
c  integer icol,               # node array column number
c          irow,               # node array row number
c          node                # node number
c  real a, a1, a2,             # half height of vertical side wall
c       b,                     # half width of diagonal fillet
c       d,                     # length of upper diagonal
c       h,                     # half height of profile
c       h_final,               # half height of profile at exit
c       h_init,                # half height of profile at inlet
c       h_last,                # previous half height of profile
c       h_octag,               # initial half length of octagon side
c       shrink,                # proportion by which height has shrunk
c       slope,                 # slope of contraction over previous interval
c       x,                     # distance from vertical centre-plane
c       xdiag,                 # x for nodes in diagonal layer
c       y,                     # distance from horizontal centre-plane
c       ydiag,                 # y for nodes in diagonal layer
c       ystep,                 # y increment
c       z,                     # distance from contraction entrance
c       z_last,                # previous value for z
c       qcont,                 # function :  calculate point on profile
c       qtaper                 #          :  fillet taper function
c
      implicit none
      integer Hprep7, Hvdulu
      parameter( Hprep7=7, Hvdulu=6 )
c
      integer itemp, icol, irow, node
      real a, a1, a2, b, d, h, h_final, h_init, h_last, h_octag,
     1     shrink, slope, x, xdiag, y, ydiag, ystep, z, z_last,
     2     qcont, qtaper
      include 'taper.cmn/list'
c
c         initialize
c
      z = zmin
      nsections = 0
      slope = 0.0
      h = qcont( z )
      h_final = qcont( zmax )
      h_init  = qcont( zmin )
      h_octag = h_init / ( 1.0 +  sqrt(2.0) )
      node = 1
      n_first = 1
      n_last  = 0
      ni_xsect = 0
      nf_xsect = 0
      nprism = 0
c
c         body of loop
c
      do while( z .le. zmax )                                             1b
        nf_xsect = node
c
c         overall dimensions
c
c
        shrink = ( h - h_final ) / ( h_init - h_final )
        itemp = ( itaper + 1 ) / 2
        if ( itemp .eq. 1 .or. itemp .eq. 3 ) then                        2b
          a1 = h - qtaper( shrink, z, itaper ) * h /
     1           ( 1.0 + 1.0 / sqrt(2.0) )
          end if                                                          2e
        if ( itemp .eq. 2 .or. itemp .eq. 3 ) then                        2b
          a2 = ( h_final + ( h_octag - h_final ) *
     1        qtaper( shrink, z, itaper ) )
          end if                                                          2e
        if ( itemp .eq. 2 ) then                                          2b
          a = a2
        else if ( itemp .eq. 3 ) then                                     2m
          a = a1 * shrink + a2 * ( 1.0 - shrink )
        else                                                              2m
          a = a1
          end if                                                          2e
c
        b = ( h - a ) / sqrt(2.0)
        d = ( h + a ) / sqrt(2.0)
c
        x = 0.0
        do icol = 0, nsegments                                            2b
c
c         vertical strip in horizontal layer
c
          y = 0.0
          if ( b / a .gt. 0.05 ) then                                     3b
            ystep = a / nsegments
          else                                                            3m
            ystep = ( a + b / 2.0 ) / nsegments
            end if                                                        3e
          do irow = 1, ( icol + 3 ) / 2                                   3b
            if ( irow .eq. 1 ) then                                       4b
              y = 0
            else if ( irow .eq. 2 .and. mod( icol, 2 ) .eq. 1 ) then      4m
              y = y + ystep
            else
              y = y + ystep * 2.0
              end if                                                      4e
            write( Hprep7, 1 ) node, x, y, z
1           format( ' N,',i4, ',', g12.5, ',', g12.5, ',', g12.5 )
            node = node + 1
            end do                                                        3e
c
c         `vertical' strip in diagonal layer
c
          if ( b / a .gt. 0.05 ) then                                     3b
            xdiag = x
            ydiag = y
            ystep = b / nsegments
            do irow = ( icol + 1 ) / 2, 1, -1                             4b
              if ( irow .eq. 1 .and. mod( icol, 2 ) .eq. 1 ) then         5b
                xdiag = xdiag - ystep  / sqrt(2.0)
                ydiag = ydiag + ystep  / sqrt(2.0)
              else                                                        5m
                xdiag = xdiag - ystep  * sqrt(2.0)
                ydiag = ydiag + ystep  * sqrt(2.0)
                end if                                                    5e
              write( Hprep7, 1 ) node, xdiag, ydiag, z
              node = node + 1
              end do                                                      4e
            end if                                                        3e
          x = x + h / nsegments
          end do                                                          2e
c
c         increments
c
        z_last = z
        z = z + 2.0 * h / ( real( 2 * nsegments ) - slope )
        h_last = h
        h = qcont( z )
        slope = ( h - h_last ) / ( z - z_last )
c
        if ( b / a .gt. 0.05 )
     1    nprism = nprism + 1
        if ( nsections .le. 0 )
     1    ni_xsect = node - 1
        nsections = nsections + 1
        end do                                                            1e
      n_last = node - 1
      nf_xsect = node - nf_xsect
      write( Hvdulu, 2 ) nsections, nprism, ni_xsect, nf_xsect, n_last
2     format( ' number of cross sections        = ', i5 /
     1        ' no. of double layer sections    = ', i5 /
     2        ' no. of nodes in initial section = ', i5 /
     3        ' no. of nodes in final section   = ', i5 /
     4        ' total number of nodes           = ', i5 )
      return
      end


      REAL FUNCTION QCONT( Z )
c
c       Evaluate a point on the wind tunnel contraction by interpolation with
c  the cubic spline fitted to the data .
c
c Argument
c  real z              # distance from contraction inlet
c
c constant
c  Hvdulu=6            # vdu logical unit no.
c variable
c  integer ierr,       # error flag from ICSCCU
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real d, z
      integer *4 ind, ierr, jbsrch
      include 'taper.cmn/list'
c
      if ( z .le. zcont(1) ) then                                           1b
        qcont = xcont(1)
      else if ( z .ge. zcont(ncont) ) then                                  1m
        qcont = xcont(ncont)
      else                                                                  1m
        ind = jbsrch( zcont, 1, ncont, z )
        d   = z - zcont(ind)
        qcont = ( ( spline(ind,3) * d + spline(ind,2) ) * d +
     1          spline(ind,1) ) * d + xcont(ind)
        end if                                                              1e
      return
      end


      REAL FUNCTION QTAPER( SHRINK, Z, ITAPER )
c
c       Specifies the fraction by which the fillet has shrunk because of
c  reduced cross-section area .
c     N.B.  QTAPER(0) = 1.0
c           QTAPER(1) = 0.0  if ITAPER != 0
c
c  Arguments
c    real shrink               # fraction of maximum contraction height
c         z                    # distance from contraction inlet
c    integer itaper            # type of taper   0 = none
c                              #                 1 = linear
c                              #                 2 = sinusoidal
c
      implicit none
      real Hpi
      parameter( Hpi=3.1415926 )
c
      real shrink, z
      integer itaper
c
      if ( itaper .eq. 0 ) then                                           1b
c
c           no taper
c
        qtaper = 1.0
      else if ( mod( itaper, 2 ) .eq. 1 ) then                            1m
c
c           linear taper
c
        qtaper = shrink
      else if ( mod( itaper, 2 ) .eq. 0 ) then                            1m
c
c           sinusoidal taper
c
        qtaper = 0.5 * ( 1.0 - cos( shrink * Hpi ) )
        end if                                                            1e
      return
      end


      SUBROUTINE ELEMENTS
c
c         DEFINE THE ELEMENTS - in terms of nodes
c
c constant
c  Hprep7 = 7                  # logical unit no. of PREP7 file
c variable
c  integer icntr,              # number of elements done in this column
c          icol, irow,         # column and row number
c          isect,              # cross section number in a cross section
c          k,                  # increment required to get to the last node
c          k_nxt,              # k for next section
c          nelem,              # number of elements
c          next,               # first node in next section for current element
c          node,               # first node in this section for current element
c
      implicit none
      integer Hprep7, Hvdulu
      parameter( Hprep7=7, Hvdulu=6 )
c
      integer icntr, icol, irow, isect, k, k_nxt, nelem, next, node
      include 'taper.cmn/list'
c
      nelem = 0
      do isect = 1, nsections - 1                                         1b
        if ( isect .le. nprism ) then                                     2b
          node = ( isect - 1 ) * ni_xsect
          next  = node + ni_xsect
        else if ( isect .eq. nprism + 1 ) then                            2m
          node = ( isect - 1 ) * ni_xsect
          next  = node + nf_xsect
        else                                                              2m
          node = ( nprism ) * ni_xsect +
     1           ( isect - nprism - 1 ) * nf_xsect
          next  = node + nf_xsect
          end if                                                          2e
c      WRITE(7,9987) ISECT, NPRISM, NODE, NEXT, NF_XSECT, NI_XSECT
c9987  FORMAT( ' ISECT, NPRISM, NODE, NEXT, NF/I_XSECT ', 6I7 )

        do icol = 1, nsegments                                            2b
          icntr = 0
c
c         calculate last node increment for current and next sections
c
          if ( isect .lt. nprism ) then                                   3b
            k = icol + 1
            k_nxt = k
          else if ( isect .eq. nprism ) then                              3m
            k = icol + 1
            k_nxt = ( icol + 3 ) / 2
          else                                                            3m
            k = ( icol + 3 ) / 2
            k_nxt = k
            end if                                                        3e
c
c             START do the bottom, horizontal layer
c             =====================================
c
          if ( mod( icol, 2 ) .eq. 0 ) then                               3b
c
c             for even numbered columns , element types
c               ALPHA, BETA, ALPHA, BETA ....
c
            node  = node  + 1
            next  = next  + 1
            do irow = 1, ( icol + 1 ) / 2                                 4b
              if ( icntr .lt. icol ) then                                 5b
c
c               Type alpha element
c
                icntr = icntr + 1
                nelem = nelem + 1
c      WRITE(7,9997)
c9997  FORMAT( ' HORIZONTAL LAYER, EVEN COLUMN NO., ELEM TYPE ALPHA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
c9998  FORMAT( ' ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT ' / 8I5 )
                write( Hprep7, 1 ) node, node+1, node+k, node+k,
     1                     next, next+1, next+k_nxt, next+k_nxt
1               format( 'E,', i4, 7( ',', i4 ) )
                if ( icntr .lt. icol ) then                               6b
c
c               Type beta element
c
                  icntr = icntr + 1
                  node  = node  + 1
                  next  = next  + 1
                  nelem = nelem + 1
c      WRITE(7,9996)
c9996  FORMAT( ' HORIZONTAL LAYER, EVEN COLUMN NO., ELEM TYPE BETA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                  write( Hprep7, 1 ) node, node-1+k, node+k, node+k,
     1                   next, next-1+k_nxt, next+k_nxt, next+k_nxt
                  end if                                                  6e
                end if                                                    5e
              end do                                                      4e
          else                                                            3m
c
c             for odd numbered columns , element types
c                BETA, ALPHA, BETA, ALPHA ....
c
            do irow = 1, ( icol + 1 ) / 2                                 4b
              if ( icntr .lt. icol ) then                                 5b
c
c               Type beta element
c
                icntr = icntr + 1
                node  = node  + 1
                next  = next  + 1
                nelem = nelem + 1
c      WRITE(7,9995)
c9995  FORMAT( ' HORIZONTAL LAYER, ODD COLUMN NO., ELEM TYPE BETA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                write( Hprep7, 1 ) node, node-1+k, node+k, node+k,
     1                 next, next-1+k_nxt, next+k_nxt, next+k_nxt
c
c               Type alpha element
c
                if ( icntr .lt. icol ) then                               6b
                  icntr = icntr + 1
                  nelem = nelem + 1
c      WRITE(7,9994)
c9994  FORMAT( ' HORIZONTAL LAYER, ODD COLUMN NO., ELEM TYPE ALPHA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                  write( Hprep7, 1 ) node, node+1, node+k, node+k,
     1                    next,  next+1,  next+k_nxt,  next+k_nxt
                  end if                                                  6e
                end if                                                    5e
              end do                                                      4e
            end if                                                        3e
c
c             END do the bottom, horizontal layer
c
c             START do the top, diagonal layer
c             ================================
c
          if ( isect .lt. nprism ) then                                   3b
            node  = node  - 1
            next  = next  - 1
c
c             element types BETA, ALPHA, BETA, ALPHA ....
c
            do irow = 1, ( icol + 1 ) / 2                                 4b
              if ( icntr .lt. 2 * icol ) then                             5b
c
c               Type beta element
c
                icntr = icntr + 1
                node  = node  + 1
                next  = next  + 1
                nelem = nelem + 1
c      WRITE(7,9993)
c9993  FORMAT( ' DIAGONAL LAYER, ELEM TYPE BETA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                write( Hprep7, 1 ) node, node+k, node+k+1, node+k+1,
     1                 next, next+k_nxt, next+k_nxt+1, next+k_nxt+1
c
c               Type alpha element
c
                if ( icntr .lt. 2 * icol ) then                           6b
                  icntr = icntr + 1
                  nelem = nelem + 1
c      WRITE(7,9992)
c9992  FORMAT( ' DIAGONAL LAYER, ELEM TYPE ALPHA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                  write( Hprep7, 1 ) node, node+1, node+k+1, node+k+1,
     1                      next,  next+1, next+k_nxt+1, next+k_nxt+1
                  end if                                                  6e
                end if                                                    5e
              end do                                                      4e
            if ( mod( icol, 2 ) .eq. 0 ) then                             4b
              node = node + 1
              next = next + 1
              end if                                                      4e
c
c             END do the top, diagonal layer
c
c             START  the last section with upper layer
c                    make a row of tetrahedral elements
c
          else if ( isect .eq. nprism ) then                              3m
            node = node  - 1
c            next = next - 1
c
c             element types BETA, ALPHA, BETA, ALPHA ....
c
            do irow = 1, ( icol + 1 ) / 2                                 4b
              if ( icntr .lt. 2 * icol ) then                             5b
c
c               Type beta element
c
                icntr = icntr + 1
                node  = node  + 1
                nelem = nelem + 2
c      WRITE(7,9991)
c9991  FORMAT( ' TETRAHEDRAL, ELEM TYPE BETA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                write( Hprep7, 1 ) node, node+k, node+k+1, node+k+1,
     1            next+k_nxt, next+k_nxt, next+k_nxt, next+k_nxt
                write( Hprep7, 1 ) node, node+k+1, next, next,
     1            next+k_nxt, next+k_nxt, next+k_nxt, next+k_nxt
c
c               Type alpha element
c
                if ( icntr .lt. 2 * icol ) then                           6b
                  icntr = icntr + 1
                  nelem = nelem + 2
c      WRITE(7,9990)
c9990  FORMAT( ' TETRAHEDRAL, ELEM TYPE ALPHA' )
c      WRITE(7,9998) ISECT, ICOL, IROW, ICNTR, NODE, NEXT, K, K_NXT
                  write( Hprep7, 1 ) node, node+1, node+k+1, node+k+1,
     1                      next,  next, next, next
                  write( Hprep7, 1 ) node+1, node+1+k, next, next,
     1            next+k_nxt, next+k_nxt, next+k_nxt, next+k_nxt
                  end if                                                  6e
                end if                                                    5e
              end do                                                      4e
            if ( mod( icol, 2 ) .eq. 0 ) then                             4b
              node = node + 1
              end if                                                      4e
c
c             END tetragonal elements
c
            end if                                                        3e
          end do                                                          2e
c      WRITE(7,9989)
c9989  FORMAT( // )
        end do                                                            1e
      write( Hvdulu, 2 ) nelem
2     format( ' number of elements  = ', i4 )
      return
      end


      SUBROUTINE TRAILER
c
c           WRITE THE PREP7 FILE TRAILER
c constant
c  Hprep7 = 7                  # logical unit no. for PREP7 file
c
      implicit none
      integer Hprep7
      parameter( Hprep7=7 )
c
      include 'taper.cmn/list'
c
      write( Hprep7, 4 ) n_first, n_first + ni_xsect - 1, high_temp,
     1    n_last - nf_xsect + 1, n_last, low_temp
4     format( '/SHOW,9600,4014' /
     1        '/VIEW,1,1,1,1' /
     4        'NSEL,NODE,', i4, ',', i4 /
     5        'NT,ALL,TEMP,',g12.5 /
     6        'NALL' /
     7        'NSEL,NODE,', i4, ',', i4 /
     8        'NT,ALL,TEMP,',g12.5 /
     9        'NALL' /
     a        'ITER,1,1,1' /
     b        'WSORT,Z' / )
      close( UNIT=Hprep7 )
      return
      end


