c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c



      PROGRAM VISCOSITY
c
c  Calculate viscosity and density of air over a range of temperatures
c  -- uses Sutherland's formula and the ideal gas equation for air.
c
c  Usage:  VISCOSITY <initial_temperature> <final_temperature> <temperature_step>
c
      include 'stdio.def'
      real       ATMOS_PRESS,
     1           CELS_to_KELV,
     2           GAS_CONSTANT
      parameter( ATMOS_PRESS  = 101325.0,
     1           CELS_to_KELV = 273.15,
     2           GAS_CONSTANT = 287.0 )
c
      integer    n_args, out_lu, iargc, new_lun, open_file
      real       density, dyn_visc, kin_visc, start, stop, step, temp,
     1           Qmuair
      character  number*64, output_file*64
c     subroutine getarg
c
      data output_file / ' ' /
      data start, stop, step / 20.0, 300.0, 10.0 /
c
      n_args = iargc()
      if ( n_args .gt. 0 ) then
          call getarg( 1, number )
          read( number, * ) start
          if ( n_args .gt. 1 ) then
              call getarg( 2, number )
              read( number, * ) stop
              if ( n_args .gt. 2 ) then
                  call getarg( 3, number )
                  read( number, * ) step
                  if ( n_args .gt. 3 ) then
                      call getarg( 4, output_file )
                      end if
                  end if
              end if
          end if
c
      if ( output_file .eq. ' ' ) then
          out_lu = STD_OUT
      else
          out_lu = open_file( new_lun(), output_file, 'unknown' )
          end if
      if ( out_lu .gt. 0 ) then
          write( out_lu, 1 )
1         format( ';  Temperature  density 1.0e6*viscosity' /
     1            ';  (C)    (K)   (kg/m3) dynamic kinematic' )
          do 3 temp = start + CELS_to_KELV, stop + CELS_to_KELV , step
              density= ATMOS_PRESS / temp / GAS_CONSTANT
              dyn_visc = Qmuair( temp ) * 1.0e6
              kin_visc = dyn_visc / density
              write( out_lu, 2 ) temp - CELS_to_KELV, temp, density,
     1            dyn_visc, kin_visc, 1.0 / kin_visc
2             format( 1x, f6.2, 1x, f7.2, 1x, f7.5, 1x, f7.3, 1x, f7.3,
     1            1x, f7.5 )
3             continue
          close( UNIT=out_lu )
          end if
      stop
      end



      INTEGER FUNCTION OPEN_FILE( LUN, FILE_NAME, NEW_OLD )
c
c     Opens a text file for reading ( NEW_OLD = 'old' ) or writing
c ( NEW_OLD = 'new' ). If FILE_NAME is '?', the name of the file is obtained
c from the user. If the file is for reading, the file name cannot be blank.
c If the file is for writing a blank filename is interpreted as the vdu screen.
c The logical unit used for the file is LUN. If the file is successfully opened,
c LUN is the returned result . If a file could not be opened, the returned value
c is OPEN_FILE = 0 .
c
c Arguments
c  integer   lun          # logical unit no. of file to be opened
c  character file_name*(*)# file name
c            new_old*(*)  # file status : 'old'=read, 'new'=write, or 'unknown'
c
c Local Variables
c  integer length         # length of file name string
c          str_length     # function : length of string
c
c
      include 'stdio.def'
      include 'token.def'
c
      integer lun, length, str_length
      logical next_token
      character file_name*(*), new_old*(*)
c
c             check argument
c
      if ( new_old .ne. 'old' .and. new_old .ne. 'new' .and.
     1     new_old .ne. 'unknown' ) then
          write( STD_OUT, 1 ) new_old
1         format( ' OPEN_FILE : ABORT : Illegal status ', a /
     1            21x, 'Need ''old'', ''new'' or ''unknown''' )
          open_file = 0
      else if ( lun .lt. 0 .or. lun .ge. 100 ) then
          write( STD_OUT, 2 ) lun
2         format( ' OPEN_FILE : ABORT : Illegal logical unit number ',
     1            i12, '. Must be 0 to 99.' )
          open_file = 0
      else
          open_file = 0
          length = str_length( file_name )
          if ( length .gt. 0 ) then
              open( UNIT=lun, FILE=file_name, STATUS=new_old,
     1              ACCESS='sequential', FORM='formatted', ERR=6 )
              open_file = lun
          else if ( new_old .ne. 'old' ) then
              open_file = STD_OUT
              end if
          end if
      return
c
c             TRAPS
c
6     write( STD_OUT, 7 ) file_name(1:length)
7     format( ' OPEN_FILE : ABORT : Cannot open file ', a )
      open_file = 0
      return
      end
