/*
    Copyright (C) 1997  Peter Vernon Lanspeary

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * This program prepares a script file for ftp (or kermit) for transferring
 * a file-system (files + directories) from one machine to another . The form
 * of command line is
 *
 * ftpscript [-help]              help message only
 *           [-put]               put files from local machine to remote machine
 *           [-get]               get files from local machine to remote machine
 *           [-ascii]             transfer of ASCII files
 *           [-binary]            transfer of binary files
 *           [-kermit]            generate commands for kermit
 *           [-ftp]               generate commands for ftp
 *           [-local {system}]    local file system
 *           [-remote {system}]   remote file system
 *           [-dir {system}]      format of file-system listing
 *           [-in {file_list}]    name of input (file-system listing) file
 *           [-out {ftp_script}]  name of output ftp script file
 *
 * The default arguments are :
 *   -ftp -local DOS -remote UNIX -put -ascii
 * If the file-system listing is not specified, stdin is assumed .
 * If the ftp script file is not specified, stdout is assumed .
 *
 * {system} = VMS, unix or DOS\n\
 * directory files must NOT be included in the file-system listing
 * Using the ftp script file
 * VMS : replace "username" and "password" in the script file with the correct
 *       values and run the script file as a DCL command file;
 *       i.e. @{ftp_script}
 *       The script file can be prepared using
 *       DIR/EXCLUDE=*.DIR;* /NOHEADING/NOTRAIL/OUT={FTP_SCRIPT} [...]
 * UNIX: replace "username" in the script file with the correct value
 *       ftp {remote_host} < {ftp_script}
 *       then type the remote host password
 *       The script file can be prepared using
 *         find . -name "*" -type f -print
 * DOS : Start ftp and obtain a connection . Then use the "take" command --
 *       take {ftp_script}
 * KERMIT :
 *       Start kermit on local and remote machines, put the remote machine
 *       in server mode and get back to the local kermit prompt . Then type
 *       take {ftp_script}
 *
 *
 *
 *          INCLUDED FILES
 ***************************************
*/
#include <stdio.h>
#include "system.h"
#include POINTER_HEADER
#include STD_TYPE_HEADER

/*
 *
 *          CONSTANT DEFINITIONS
 ***************************************
 *
 *      system types
*/
#define VMS    0
#define UNIX   1
#define DOS    2

#define PUT          0      /* send files from local to remote machine    */
#define GET          1      /* receive files from remote to local machine */
#define MAX_WORD_LEN 15     /* maximum length of word in the options list */
#define NOT_SET      -1     /* variable has not been set     */
#define MAX_PATH_LEN 64     /* maximum directory path length */
#define MAX_STR      128    /* maximum string length         */
#define FILE_TYPE    1      /* the token is a file name      */
#define DIRECT_TYPE  2      /* the token is a directory name */
#define DOT_DOT  (&dot_dot) /* tells change_direct to cd ..  */
#define NULL_STR     (char VAR_PTR) 0
/*
 *      OPTIONS LISTS
 *      *************
 *
 *  Allowed data types of variables in an options list of type Option_Elem .
 *  The ``opt_vartyp'' value must be one of these . These values are used
 *  as an index into an array of format strings, so the values must not be
 *  changed.
*/
#define OPT_CHAR    0   /* character            */
#define OPT_SHORT   1   /* short signed integer */
#define OPT_INT     2   /* signed integer       */
#define OPT_LONG    3   /* long signed integer  */
#define OPT_FLOAT   4   /* single precision floating point */
#define OPT_DOUBLE  5   /* double precision floating point */
#define OPT_STRING  6   /* character string */
#define OPT_NUM_ARG -1.0e37 /* indicates the the value is to be obtained by */
                            /* decoding the value of the next token */
/*
 *  CATCH_NULL(pointer,caller,err_type)
 ****************************************
 *      If "pointer" has a NULL value, an "Arg is NULL pointer" error message
 * is generated and the function is aborted . Otherwise, nothing is done .
 *      "pointer"   is the pointer to be tested,
 *      "caller"    is a character string naming the function
 *      "err_type"  is the type of the returned zero value which indicates an
 *                    error condition
*/
#define CATCH_NULL(pointer,caller,err_type) \
  if ( ! (pointer) )    \
      return (err_type) make_error( ERR_ABORT, caller, err_null )


/*
 *      masks for character typing macros
*/
#define NUL            '\000'    /* Nul    */
#define DEL            '\177'    /* Delete */
#define CH_UPPER_CASE   1
#define CH_LOWER_CASE   2
#define CH_DECIMAL      4
#define CH_SPACE        8
#define CH_PUNCTUATION  16
#define CH_CONTROL      32
#define CH_HEXADECIMAL  64
#define CH_BACKSPACE    128
#define CH_LAST         DEL
#define CH_MASK         DEL

/*
 *  toascii(c)
 **************
 * converts the argument to an ASCII character by applying a mask
*/
#define toascii(c)      ( (c) & CH_MASK )

/*
 *  isspace(c)
 **************
 * is a space character.  ( space, carriage return, newline, (horizontal) tab,
 * vertical tab, formfeed )
*/
#define isspace(c)      ( 0 != ( ch_type[ toascii( c ) ] & CH_SPACE ) )

/*
 * Permissible values of error `status' - first argument of `make_error',
 *      `is_error' and `dump_error' .
 *   ERR_CALL  is used only with `make_error'
 *   ERR_ALL   is used with `is_error' or `dump_error'
 *   ERR_PRINT is a flag used only with `make_error'
*/
#define ERR_CALL   0    /* signals that a called function has returned with  */
                        /*   error status                                    */
#define ERR_ALL    0    /* detect or print any message                       */
#define ERR_NOTE   1    /* advice for user, not an error ; results are OK    */
#define ERR_WARN   2    /* execution continues but result may be wrong       */
#define ERR_ABORT  3    /* terminate execution of function without finishing */
#define ERR_FATAL  4    /* serious error ; print message and stop the program*/
#define ERR_PRINT  8    /* print the error on the error file ( which is set  */
                        /* by `file_error()' ) as well as queueing it        */

/*
 *      These values can be added to the `status' to indicate a math' error .
 *      The presence of one of these additional information to be queued .
*/
#define ERR_DOMAIN 16   /* Argument(s) not in domain of function  */
#define ERR_OVER   32   /* Result is greater than maximum allowed */
#define ERR_UNDER  48   /* Result is indistinguishable from 0.0   */
#define ERR_SING   64   /* Argument produces singularity          */
#define ERR_TLOSS  80   /* Argument produces total loss of significant digits */
#define ERR_PLOSS  96   /* Arg. produces partial loss of significant digits   */


/*
 *           TYPE DEFINITIONS
 ***************************************
 *
 *      Machine description
*/
typedef struct mach_struct
  {
    int mc_id;                /* VMS, UNIX, DOS */
    char mc_cd_str[MAX_STR];  /* cd or lcd command string for ftp */
    int mc_cd_len;            /* length of cd or lcd command string */
  }
    VAR_PTR Machine;

/*
 *      Directory description
*/
typedef struct dir_struct
  {
    struct dir_struct VAR_PTR dir_up;   /* points to directory above       */
    struct dir_struct VAR_PTR dir_down; /* points to first directory below */
    struct dir_struct VAR_PTR dir_next; /* points to adjacent directory    */
    int                dir_level;       /* directory level from the top    */
    char VAR_PTR       dir_name;        /* name of directory               */
  }
    VAR_PTR Directory;

/*
 * element of the options list
*/
typedef struct
  {
    char VAR_PTR opt_string; /* word                           */
    short int opt_minlen;    /* minimum length of abbreviation */
    short int opt_vartyp;    /* data type of variable          */
    char VAR_PTR opt_varptr; /* pointer to static variable to be set */
    float opt_value;         /* required value of variable     */
  }
    Option_Elem;



/*
 *           GLOBAL VARIABLES
 ***************************************
*/
static struct mach_struct
    local,                  /* description of local machine            */
    remote;                 /* description of remote machine           */
static struct dir_struct    /* top directory                           */
    root = { (Directory) 0, (Directory) 0, (Directory) 0, 0,
             (char VAR_PTR) "root" };
static Directory
    pwd = & root,           /* where ftp is pointed at                 */
    path[MAX_PATH_LEN] = { &root },     /* directory path              */
    VAR_PTR end_path = path;/* points to bottom end of directory path  */
static boolean
    kermit = NOT_SET,       /* TRUE = kermit, FALSE = ftp              */
    binary = NOT_SET,       /* transfer mode, TRUE=binary, FALSE=ASCII */
    help   = NOT_SET;       /* TRUE = print help message, then stop    */
static int
    list_mach_id = NOT_SET, /* file listing format = VMS, UNIX, or DOS */
    direction = NOT_SET;    /* direction of file transfer = PUT or GET */
static FILE
    *out_file = (FILE *) 0, /* output file pointer -- script for ftp   */
    *in_file = (FILE *) 0;  /* I/P file pointer, source directory list */
static char
    dot_dot = '.',          /* tells change_direct to cd .. */
/*
 *      Error messages
*/
    VAR_PTR err_null        = "Null output file pointer",
    VAR_PTR err_local       = "Unknown local machine",
    VAR_PTR err_remote      = "Unknown remote machine",
    VAR_PTR err_write       = "Cannot write",
    VAR_PTR err_machine     = "Unknown mchine type",
    VAR_PTR err_nneg        = "Max. string length <= 0",
    VAR_PTR err_mtsep       = "Empty separator string";

/*
 *
 *      EXTERNAL FUNCTIONS
 ********************************
 *
 * macros
 *   ferror()
 *
 * external file declarations
 *
 * int fclose(), fprintf(), fputc(), fputs();
*/
FILE *fopen();
char *fgets(), *malloc();
double fabs();
int free(), exit();


boolean ftp_cd_down(), ftp_cd_up(), ftp_cd_write(), ftp_mkdir(), ftp_put_get(),
        is_in_path(), match_direct(),
        parse_options(), process_line(), script_header(), script_trailer();
int     extract_name();
Directory change_direct(), find_direct(), make_direct();

boolean make_error(), is_error();
int dump_error();
boolean isabbr();
int     strcmp(), strlen(), strnlen();
char ch_type[];          /* character classification  array */
char VAR_PTR strcat(), VAR_PTR strchr(), VAR_PTR strdup(), VAR_PTR strndup(),
     VAR_PTR strupr(), VAR_PTR strntok();


main( argc, argv )
/*
 * ftpscript [-help]              help message only
  *          [-put]               put files from local machine to remote machine
 *           [-get]               get files from local machine to remote machine
 *           [-kermit]            generate commands for kermit
 *           [-ftp]               generate commands for ftp
 *           [-ascii]             transfer of ASCII files
 *           [-binary]            transfer of binary files
 *           [-local <system>]    local file system
 *           [-remote <system>]   remote file system
 *           [-dir <system>]      file system of file-system listing
 *           [-in <file_list>]    name of input (file-system) file
 *           [-out <ftp_script>]  name of output ftp script file
*/
int  argc;
char *argv[];
  {
    int arg_rem;              /* remaining space in all_args */
    char VAR_PTR name = "ftpscrpt";
    char all_args[MAX_STR];
    static char
         dir_str[MAX_STR],    /* format of file system listing VMS, UNIX, DOS */
         local_str[MAX_STR],  /* specifies local system VMS, UNIX or DOS  */
         remote_str[MAX_STR], /* specifies remote system VMS, UNIX or DOS */
         in_name[MAX_STR],    /* name of file-system listing input file   */
         out_name[MAX_STR];   /* name of ftp-script file */
/*
 *******************************************************************************
 *
 *      PARSE PROGRAM ARGUMENTS
 *      this table defines all the permitted options
*/
    static Option_Elem option_list[] = {
/*         option   min  variable   address of     value assigned to variable
 *                  len  type       variable       for this option */
	{ "-HELP",   1, OPT_INT,    (char VAR_PTR) &help,      TRUE },
	{ "-PUT",    1, OPT_INT,    (char VAR_PTR) &direction, PUT },
        { "-GET",    1, OPT_INT,    (char VAR_PTR) &direction, GET },
        { "-KERMIT", 1, OPT_INT,    (char VAR_PTR) &kermit,    TRUE },
        { "-FTP",    1, OPT_INT,    (char VAR_PTR) &kermit,    FALSE },
        { "-BINARY", 1, OPT_INT,    (char VAR_PTR) &binary,    TRUE },
        { "-ASCII",  1, OPT_INT,    (char VAR_PTR) &binary,    FALSE },
        { "-LOCAL",  1, OPT_STRING, local_str,  OPT_NUM_ARG },
        { "-REMOTE", 1, OPT_STRING, remote_str, OPT_NUM_ARG },
        { "-DIR",    1, OPT_STRING, dir_str,    OPT_NUM_ARG },
        { "-IN",     1, OPT_STRING, in_name,    OPT_NUM_ARG },
        { "-OUT",    1, OPT_STRING, out_name,   OPT_NUM_ARG },
        { NULL_STR,  0, 0,          NULL_STR,   0.0 } };
/*
 *      Concatenate all command line arguments into a single string
*/
    *all_args = '\000';
    while( --argc > 0 && ( arg_rem = MAX_STR - strlen( all_args ) - 1 ) > 0 )
        (void) strcat( strncat( all_args, *++argv, arg_rem ), " " );
    if ( parse_options( all_args, option_list, name ) &&
	 ! is_error( ERR_WARN) && ! ( help == TRUE ) )
      {
/*
 *      use default values for absent arguments
 *      -ftp -local DOS -remote UNIX -put -ascii
*/
        if ( direction == NOT_SET )
            direction = PUT;
        if ( kermit == NOT_SET )
            kermit = FALSE;
        if ( binary == NOT_SET )
            binary = FALSE;
/*  default local machine is DOS */
        if ( isabbr( strupr( local_str ), "VMS", 1 ) )
            local.mc_id = VMS;
        else if ( isabbr( strupr( local_str ), "UNIX", 1 ) )
            local.mc_id = UNIX;
        else if ( isabbr( strupr( local_str ), "DOS", 1 ) )
            local.mc_id = DOS;
        else
            local.mc_id = DOS;

/*  default remote machine is UNIX */
        if ( isabbr( strupr( remote_str ), "VMS", 1 ) )
            remote.mc_id = VMS;
        else if ( isabbr( remote_str, "UNIX", 1 ) )
            remote.mc_id = UNIX;
        else if ( isabbr( remote_str, "DOS", 1 ) )
            remote.mc_id = DOS;
        else
            remote.mc_id = UNIX;

/*  default file-system listing is UNIX */
        if ( isabbr( strupr( dir_str ), "VMS", 1 ) )
            list_mach_id = VMS;
        else if ( isabbr( dir_str, "UNIX", 1 ) )
            list_mach_id = UNIX;
        else if ( isabbr( dir_str, "DOS", 1 ) )
            list_mach_id = VMS;
        else
            list_mach_id = UNIX;
/*
 *        END of setting defaults
 *******************************************************************************
 *
 *        open file-system list (input) file
*/
        if ( *in_name == '\000' )
            in_file = stdin;
        else
            in_file = fopen( in_name, "r" );

/*
 *        open ftp script (output) file
*/
        if ( in_file )
          {
            if ( *out_name == '\000' )
                out_file = stdout;
            else
                out_file = fopen( out_name, "w" );
            if ( out_file )
              {
/*
 *******************************************************************************
 *        NOW DO THE WORK
 *      First write the script header, then process each line in the
 *      file listing, write the terminating commands to the script file.
 *      The program constructs a picture of the directory structure as
 *      it parses the contents of the file system listing
*/
                if ( script_header() )
                  {
                    while ( process_line() );
                    if ( ! script_trailer() )
                        (void) make_error( ERR_CALL, name, NULL_STR );
                  }
                else
                    (void) make_error( ERR_CALL, name, NULL_STR );
/*
 *        close files
*/
                if ( out_file != stdout )
                    (void) fclose( out_file );
              }
            else
                (void) make_error( ERR_ABORT, name,
                    (char VAR_PTR) "Cannot open output file" );
            if ( in_file != stdin )
                (void) fclose( in_file );
          }
        else
            (void) make_error( ERR_ABORT, name,
            (char VAR_PTR) "Cannot open input file" );
        (void) dump_error( ERR_ALL );
      }
    else
      {
        (void) dump_error( ERR_ALL );
    (void) fputs(
"ftpscript : prepares ftp or kermit script file from a file system listing\n\n\
  command line arguments :\n\
   [-put]               put files from local machine to remote machine\n\
   [-get]               get files from local machine to remote machine\n\
   [-kermit]            generate commands for kermit\n\
   [-ftp]               generate commands for ftp\n\
   [-ascii]             transfer of ASCII files\n\
   [-binary]            transfer of binary files\n\
   [-local {system}]    local file system\n\
   [-remote {system}]   remote file system\n\
   [-dir {system}]      syntax of file-system listing\n\
   [-in {file_list}]    name of file-system listing file\n\
   [-out {ftp_script}]  name of output ftp script file\n\
   {system} = VMS, unix or DOS\n\
   directory files must NOT be included in the file-system listing\n\n\
   Press <RETURN> to continue", stderr );
        while( getchar() != '\n' );
        (void) fputs(
"\nThe default arguments are :\n\
  -ftp -local DOS -remote UNIX -put -ascii\n\
If the file-system listing is not specified, stdin is assumed .\n\
If the ftp script file is not specified, stdout is assumed .\n\
{system} = VMS, unix or DOS\n\
directory files must NOT be included in the file-system listing\n\n\
Using the ftp script file ;\n\
VMS : replace \"username\" and \"password\" in the script file with the correct\n\
      values and run the script file as a DCL command file; i.e. @{ftp_script}\n\
      The script file can be prepared using\n\
      DIR/EXCLUDE=*.DIR;* /NOHEADING/NOTRAIL/OUT={FTP_SCRIPT} [...]\n\
UNIX: replace \"username\" in the script file with the correct value\n\
      ftp {remote_host} < {ftp_script}\n\
      then type the remote host password\n\
      The script file can be prepared using\n\
        find . -name \"*\" -type f -print\n\
DOS : Start ftp and obtain a connection . Then use the \"take\" command --\n\
      take {ftp_script}\n\
KERMIT :\n\
      Start kermit on local and remote machines and get back to the local\n\
      in server mode and get back to the local kermit prompt . Then type\n\
      take {ftp_script}\n", stderr );
      }
  }



boolean script_header()
/*
 * write a header to the script file
*/
  {
    char VAR_PTR name = "script_header";
    if ( kermit )
      {
/*
 *          put the remote kermit in server mode and then
 *          set the file type for ascii or binary teransfers
*/
        fputs( "comment : PVL (1992)\n", out_file );
	fputs( "set file type ", out_file );
        if ( binary )
            fputs( "binary\n", out_file );
        else if ( local.mc_id == VMS )
            fputs( "ascii\n", out_file );
        else
            fputs( "text\n", out_file );
      }
    else /* ftp */
      {
/*
 *          for ftp, the startup sequence is different for each
 *          operating system
*/
        if ( local.mc_id == VMS )
            fputs( "$ ftp remote_machine\nlogin username\npassword\n",
               out_file );
        else if ( local.mc_id == UNIX )
            fputs( "username\n", out_file );
        else if ( local.mc_id != DOS )
            return make_error( ERR_ABORT, name, err_local );
        fputs( ( binary ? "binary\n" : "ascii\n" ), out_file );
      }
    if ( ferror( stdout ) )
        return make_error( ERR_ABORT, name, err_write );
    return TRUE;
  }



boolean script_trailer()
/*
 *  Write the last part of the script file .
*/
  {
    if ( kermit )
        fputs( "finish\n", out_file );
    else if ( local.mc_id != DOS )
        fputs( "quit\n", out_file );
    if ( ferror( out_file ) )
        return make_error( ERR_ABORT, (char VAR_PTR) "script_trailer",
             err_write );
    return TRUE;
  }



int extract_name( token, line )
/*
 * Extract a name ``token'' from the ``line'' read from file .
 * If successful, a value of FILE_TYPE, or DIRECT_TYPE is returned ( to
 * indicate that the token is either a directory name (which is part of
 * the path) or a file name), otherwise the returned value is FALSE;
*/
char VAR_PTR VAR_PTR token,     /* extracted token */
     VAR_PTR line;              /* line of text read from file */
  {
    static boolean
        open_paren = FALSE,     /* found [ for VMS path+file name    */
        start_of_name = TRUE;   /* now starting a new path+file name */
    static char
        VAR_PTR token_list[MAX_PATH_LEN],
        VAR_PTR VAR_PTR end_list = (char VAR_PTR VAR_PTR) 0,
                                /* points to last token in in token_list */
        VAR_PTR VAR_PTR curr_token; /* points to current token in list  */
                                    /* this is returned */
    int name_type;              /* returned value FILE_TYPE or DIRECT_TYPE */
    char
        VAR_PTR semi_colon,     /* points to semicolon in VMS file name */
        VAR_PTR first_char,     /* points to first non-space character  */
        VAR_PTR name = "extract_name",
        VAR_PTR chr_ptr;
    CATCH_NULL( token && line, name, int );
/*
 *    if at the start of the line
 *    -- the whole of the preious line has been processed. Construct a list
 *    of all the tokens in the line.
*/
    if ( start_of_name )
      {
/*
 *    skip white-space
*/
        for ( first_char = line; *first_char && isspace( *first_char );
            first_char++ );
        if ( ! *first_char )
            return FALSE;
/*
 *    skip initial directory marker [ . / or \ ;
 *    in the case of VMS the first is skipped if it is not preceded by a .
*/
        if ( list_mach_id == VMS )
          {
            if ( *first_char == 'D' )
              {
                while( *++first_char != ':' );
                first_char++;
              }
            if ( *first_char == '[' )
              {
                open_paren = TRUE;
                while( *++first_char != '.' );
                first_char++;
              }
            else
                open_paren = FALSE;
          }
        else if ( list_mach_id == UNIX )
          {
            while( *first_char == '/' || *first_char == '.' )
                first_char++;
          }
        else if ( list_mach_id == DOS )
          {
            if ( *(first_char+1) == ':' )
                first_char += 2;
            while( *first_char == '\\' )
                first_char++;
          }
/*
 *    clear all tokens from the list
*/
        if ( end_list )
            for ( curr_token = token_list; curr_token <= end_list;
                (void) free( *curr_token++ ) );
/*
 *    convert VMS or DOS file-spec syntax to UNIX
*/
        if ( list_mach_id == VMS )
          {
/*          if a [ is found, change all .s to /s up to the ] .
            then change the ] to / */
            if ( open_paren )
              {
                for( chr_ptr = first_char; *chr_ptr && *chr_ptr != ']';
                        chr_ptr++ )
                    if ( *chr_ptr == '.' )
                        *chr_ptr = '/';
                if ( *chr_ptr == ']' )
                    *chr_ptr = '/';
                else
                    return make_error( ERR_ABORT, name,
                        (char VAR_PTR) "Invalid VMS file syntax - missing ]" );
              }
          }
        else if ( list_mach_id == DOS )
/*          change all \s to /s */
            for( chr_ptr = first_char; *chr_ptr; chr_ptr++ )
                if ( *chr_ptr == '\\' )
                    *chr_ptr = '/';
/*
 *    extract the first token on the line
*/
        if ( *token = strntok( first_char, "/ \t\r\n\v\f", MAX_STR ) )
          {
            *(end_list = token_list ) = strdup( *token );
            start_of_name = FALSE;
/*
 *    Now extract the remaining tokens from the line
 *    If a token has been found place it on the list
*/
            do
                if ( *token = strntok( NULL_STR, "/ \t\r\n\v\f", MAX_STR ) )
                    *++end_list = strdup( *token );
            while( *token && ( end_list - token_list < MAX_PATH_LEN - 1 ) );
            if ( *token && ( end_list - token_list >= MAX_PATH_LEN - 1 ) )
                return make_error( ERR_ABORT, name,
                    (char VAR_PTR) "Directory structure too deep" );
          }
        else
            return make_error( ERR_ABORT, name,
                (char VAR_PTR) "Token not found" );
/*
 *    Now we have all the tokens on the line, send the first one out
*/
        *token = *( curr_token = token_list );
      }
/*
 *    we are not at the start of a name, so pick the next token off the list
*/
    else
      {
        if ( curr_token >= end_list )
            return make_error( ERR_ALL, name,
                (char VAR_PTR) "Tried to read beyond token list" );
        *token = *++curr_token;
      }
/*
 *    decide if the token is a directory name or file name,
 *    for VMS file system, truncate ';' and anything following it from the token
*/
    if ( curr_token >= end_list )
      {
        name_type = FILE_TYPE;
        if ( semi_colon = strchr( *token, ';' ) )
            *semi_colon = '\000';
        start_of_name = TRUE;
      }
    else
        name_type = DIRECT_TYPE;
    return name_type;
  }



boolean process_line()
/*
 *  Process a file name from the input file and write the necessary ftp
 *  commands . The ftp commands are "cd", "lcd", "put", "get", "mkdir".
 *
 *  As each line is processed, a picture of the directory structure is
 *  constructed. This is used to control the sequence of "cd" and "mkdir"
 *  commands.
*/
  {
    char VAR_PTR name = "process_line",
         VAR_PTR token,   /* file/directory name parsed from I/P file */
         line[MAX_STR];   /* line from input file */
    int name_type;
    Directory tmp_dir,
              track = &root;
/*
 *    read a line of text from the input file
*/
    if ( ! fgets( line, MAX_STR, in_file ) )
        return FALSE;
/*
 *      read the next directory name or the file name from the file list line
*/
    while ( ( name_type = extract_name( &token, line ) ) == DIRECT_TYPE )
      {
/*
 *        if the name is a directory,
 *        keep track of where the file list line is
 *        if the directory does not exist, create it
 *        ignore any directory name "."
*/
        if ( strcmp( token, "." ) )
          {
            if ( ! ( tmp_dir = find_direct( token, track ) ) )
                if ( ! match_direct( track ) ||
                     ! ( tmp_dir = make_direct( token, track ) ) ||
                     ! ftp_mkdir( token ) )
                    return make_error( ERR_CALL, name, NULL_STR );
            track = tmp_dir;
/*
 *        if we have departed from the current path change the directory
 *        (``pwd'') to the one just above ``track'' .
*/
            if ( ! is_in_path( track ) )
/*
 *              In this case, track has diverged from the path
*/
                if ( track->dir_level - pwd->dir_level <= 0 )
                  {
                    while ( ( tmp_dir = change_direct( DOT_DOT ) ) &&
                            tmp_dir != track->dir_up )
                    if ( ! tmp_dir )
                        return make_error( ERR_CALL, name, NULL_STR );
                  }
/*
 *              In this case, track is below pwd and has just moved further
 *              down, so search in ``pwd'' for the name of the directory
 *              above ``track'' and hop into it .
*/
                else if ( track->dir_level - pwd->dir_level > 1 )
                    if ( ! change_direct( track->dir_up->dir_name ) )
                        return make_error( ERR_ABORT, name,
                            (char VAR_PTR) "Bug : Cannot cd toward track" );
          }
      }
/*
 *        We have a file name, and first make the ``track'' and ``pwd''
 *        directories the same .
*/
    if ( name_type == FILE_TYPE )
      {
        if ( ! strcmp( token, "." ) )
            (void) make_error( ERR_WARN, name,
                (char VAR_PTR) "Trivial file name \".\"" );
        else if ( ! match_direct( track ) || ! ftp_put_get( token ) )
            return make_error( ERR_CALL, name, NULL_STR );
      }
    else
        return make_error( ERR_ABORT, name,
            (char VAR_PTR) "Unknown return value from extract_name()" );
    return TRUE;
  }



boolean is_in_path( directory )
/*
 * Returns TRUE if ``directory'' is in the current ``path'', otherwise FALSE .
*/
Directory directory;
  {
    Directory VAR_PTR dir_ptr;
    CATCH_NULL( directory, (char VAR_PTR) "is_in_path", boolean );
    for( dir_ptr = path; dir_ptr <= end_path; dir_ptr++ )
        if ( directory == *dir_ptr )
            return TRUE;
    return FALSE;
  }



boolean match_direct( directory )
/*
 * Change_direct to bring the location of ftp to ``directory'' .
*/
Directory directory;
  {
    char VAR_PTR name = "match_direct";
    Directory tmp_dir;
    CATCH_NULL( directory, name, boolean );
/*
 *        ``pwd'' cannot be more than one level above ``directory''
*/
    if ( pwd->dir_level < directory->dir_level - 1 )
        return make_error( ERR_ABORT, name,
            (char VAR_PTR) "Bug : pwd too far above directory" );
/*
 *        move down one level
*/
    else if ( pwd->dir_level == directory->dir_level - 1 )
      {
        if ( ! change_direct( directory->dir_name ) )
            return make_error( ERR_ABORT, name, (char VAR_PTR)
                "Bug : pwd && directory in different branches" );
      }
/*
 *        repeat cd .. until ``directory'' and ``pwd'' are the same
*/
    else if ( pwd->dir_level > directory->dir_level )
      {
        while ( ( tmp_dir = change_direct( DOT_DOT ) ) &&
                tmp_dir != directory )
        if ( ! tmp_dir )
            return make_error( ERR_CALL, name, NULL_STR );
      }
/*
 *        ``directory'' must be the same as ``pwd'' if at the same level
*/
    else if ( pwd != directory )
        return make_error( ERR_ABORT, name, (char VAR_PTR)
         "Bug : pwd && directory at same level but in different branches" );
    if ( ! ftp_cd_write( &local ) || ! ftp_cd_write( &remote ) )
        return make_error( ERR_CALL, name, NULL_STR );
    return TRUE;
  }



boolean ftp_mkdir( dir_name )
/*
 * Writes an ftp or kermit command to create a directory ``dir_name'' on the
 * remote machine .
*/
char VAR_PTR dir_name;
  {
    static boolean first_call = TRUE;
    static char VAR_PTR format_str = "unknown machine type %s\n",
                VAR_PTR name = "ftp_mkdir";
    CATCH_NULL( dir_name, name, boolean );
/*
 *        on the first call, decide the text for the mkdir command
*/
    if ( first_call )
      {
        if ( kermit )
          {
            if ( direction == PUT )
              {
                /* "remote host" is available on all kermits */
                if ( remote.mc_id == VMS )
                    format_str = "remote host create/directory [.%s]\n";
                else if ( remote.mc_id == UNIX )
                    format_str = "remote host mkdir %s\n";
                else if ( remote.mc_id == DOS )
                    format_str = "remote host md %s\n";
              }
            else if ( direction == GET )
        /* each kermit has a different command for making a local directory */
              {
                if ( local.mc_id == VMS )
                    format_str = "local host create/directory [.%s]\n";
                else if ( local.mc_id == UNIX )
                    format_str = "! mkdir %s\n";
                else if ( local.mc_id == DOS )
                    format_str = "run md %s\n";
              }
            else
                return make_error( ERR_ABORT, name, err_remote );
          }
/*
 *        for ftp, mkdir dir_name
*/
        else /* ftp */
          {
          /* "mkdir" is available on all FTPs for making remote directories */
            if ( direction == PUT )
              {
                if ( remote.mc_id == VMS )
                    format_str = "mkdir [.%s]\n";
                else if ( remote.mc_id == UNIX )
                    format_str = "mkdir %s\n";
                else if ( remote.mc_id == DOS )
                    format_str = "mkdir %s\n";
              }
            else if ( direction == GET )
           /* each FTP has a different command for making a local directory */
              {
                if ( local.mc_id == VMS )
                    format_str = "spawn\n create/directory [.%s]\neoj\n";
                else if ( local.mc_id == UNIX )
                    format_str = "! mkdir %s\n";
                else if ( local.mc_id == DOS )
                    format_str = "run md %s\n";
              }
            else
                return make_error( ERR_ABORT, name, err_remote );
          }
        first_call = FALSE;
      }
/*
 *      write the mkdir command
*/
    (void) fprintf( out_file, format_str, (char *) dir_name );
    if ( ferror( out_file ) )
        return make_error( ERR_ABORT, name, err_write );
    return TRUE;
  }



boolean ftp_put_get( file_name )
/*
 *  writes a "put file_name" or "get file_name" to the output file .
*/
char VAR_PTR file_name;
  {
    static boolean first_call = TRUE;
    static char VAR_PTR name = "ftp_put_get",
                VAR_PTR format_str;
    CATCH_NULL( file_name, name, boolean );
/*
 *    on the first call, decide the form of file transfer commsnd
*/
    if ( first_call )
      {
        if ( direction == PUT )
          {
            if ( kermit )
                format_str = "send %s\n";
            else
                if ( local.mc_id == VMS )
                    format_str = "put %s %s\n";
                else
                    format_str = "put %s\n";
          }
        else if ( direction == GET )
/*         for getting files, kermit is the same as ftp */
          {
            if ( ! kermit && local.mc_id == VMS )
                format_str = "get %s %s\n";
            else
                format_str = "get %s\n";
          }
        else
            return make_error( ERR_ABORT, name,
                (char VAR_PTR) "Unknown direction" );
        first_call = FALSE;
      }
/*
 *    write the file transfer command
*/
    (void) fprintf( out_file, format_str, (char *) file_name,
                                          (char *) file_name );
    if ( ferror( out_file ) )
        return make_error( ERR_ABORT, name,
            (char VAR_PTR) "Cannot write output file" );
    return TRUE;
  }



Directory change_direct( dir_name )
/*
 * Change the present working directory -- which is where ftp is pointed at --
 * down one level to the directory ``dir_name'' (i.e. cd dir_name ) or, if
 * ``dir_name'' is DOT_DOT move up one level (i.e. cd .. ) .
*/
char VAR_PTR dir_name;
  {
    char VAR_PTR name = "change_direct";
    Directory tmp_dir;
    CATCH_NULL( dir_name, name, Directory );
/*
 *    cd ..
*/
    if ( dir_name == DOT_DOT )
      {
        if ( tmp_dir = pwd->dir_up )
          {
            if ( ! ftp_cd_up( &local ) || ! ftp_cd_up( &remote ) )
                return (Directory) make_error( ERR_CALL, name, NULL_STR );
            end_path--;
            pwd = tmp_dir;
          }
      }
/*
 *    cd dir_name
*/
    else
      {
        if ( tmp_dir = find_direct( dir_name, pwd ) )
          {
            if ( ! ftp_cd_down( &local, tmp_dir->dir_name ) ||
                 ! ftp_cd_down( &remote, tmp_dir->dir_name ) )
                return (Directory) make_error( ERR_CALL, name, NULL_STR );
            *++end_path = pwd = tmp_dir;
          }
        else
            return (Directory) make_error( ERR_ABORT, name,
                (char VAR_PTR) "Cannot find directory" );
      }
    return pwd;
  }



boolean ftp_cd_up( machine )
/*
 * Add the directory name .. or - to either the local or remote change_directory
 * string . The argument ``machine'' points either to the local or remote
 * machine descriptor .
*/
Machine machine;
  {
    static char VAR_PTR name = "ftp_cd_up";
    CATCH_NULL( machine, name, boolean );
/*
 *    dump the change directory string if it is full
*/
    if ( machine->mc_cd_len + 3 >= MAX_STR )
        if ( ! ftp_cd_write( machine ) )
            return FALSE;
/*
 *  add .-, /.. or \.. to change directory string
*/
    if ( machine->mc_cd_len > 0 )
      {
        if ( machine->mc_id == VMS )
            (void) strcat( machine->mc_cd_str, "." );
        else if ( machine->mc_id == UNIX )
            (void) strcat( machine->mc_cd_str, "/" );
        else if ( machine->mc_id == DOS )
            (void) strcat( machine->mc_cd_str, "\\" );
        else
            return make_error( ERR_ABORT, name, err_machine );
        machine->mc_cd_len++;
      }
    if ( machine->mc_id == VMS )
      {
        (void) strcat( machine->mc_cd_str, "-" );
        machine->mc_cd_len++;
      }
    else if ( machine->mc_id == UNIX || machine->mc_id == DOS )
      {
        (void) strcat( machine->mc_cd_str, ".." );
        machine->mc_cd_len += 2;
      }
    else
        return make_error( ERR_ABORT, name, err_machine );
    return TRUE;
  }



boolean ftp_cd_down( machine, dir_name )
/*
 * Add the directory name ``dir_name'' to either the local or remote
 * change_directory string . The argument ``machine'' points either to the
 * local or remote machine descriptor .
*/
Machine machine;        /* machine specifier */
char VAR_PTR dir_name;  /* name of directory */
  {
    static char VAR_PTR name = "ftp_cd_up";
    int dir_name_len;
    CATCH_NULL( machine && dir_name, name, boolean );
/*
 *    dump the change directory string if it is full
*/
    dir_name_len = strlen( dir_name );
    if ( machine->mc_cd_len + dir_name_len + 1 >= MAX_STR )
        if ( ! ftp_cd_write( machine ) )
            return FALSE;
/*
 *  add . / or \ to the change_directory string
*/
      {
        if ( machine->mc_id == VMS )
          {
            (void) strcat( machine->mc_cd_str, "." );
            machine->mc_cd_len++;
          }
        else if ( machine->mc_cd_len > 0 )
          {
            if ( machine->mc_id == UNIX )
                (void) strcat( machine->mc_cd_str, "/" );
            else if ( machine->mc_id == DOS )
                (void) strcat( machine->mc_cd_str, "\\" );
            else
                return make_error( ERR_ABORT, name, err_machine );
            machine->mc_cd_len++;
          }
      }
/*
 *  add the directory name to the change_directory string
*/
    (void) strcat( machine->mc_cd_str, dir_name );
    machine->mc_cd_len += dir_name_len;
    return TRUE;
  }


boolean ftp_cd_write( machine )
/*
 * Write ftp ``cd'' and ``lcd'' commands to match up either the local or
 * remote required directory with the actual directory . The argument
 * ``machine'' points either to the local or remote machine descriptor .
*/
Machine machine;
  {
    static char VAR_PTR name = "ftp_cd_write",
                *loc_format, *rem_format,
                *loc_command, *rem_command;
    static boolean loc_first = TRUE,
                   rem_first = TRUE;
    CATCH_NULL( out_file, name, boolean );
/*
 *      early return if the path is empty
*/
    if ( machine->mc_cd_len <= 0 || machine->mc_cd_str[0] == (char) 0 )
        return TRUE;
/*
 *    FOR THE LOCAL MACHINE
*/
    if ( machine == &local )
      {
/*
 *    on the first call, decide the form of file transfer commsnd
*/
        if ( loc_first )
          {
            if ( machine->mc_id == VMS )
              loc_format = "%s [%s]\n";
            else /* UNIX or DOS */
              loc_format = "%s %s\n";
            if ( kermit )
                loc_command = ( ( local.mc_id == VMS ) ?
                    "local cwd" : "cwd" );
            else /* ftp */
                loc_command = "lcd";
            loc_first = FALSE;
          }
/*
 *    write the cd command, with extra \n for local VMS kermit "remote cd"
*/
        (void) fprintf( out_file, loc_format, loc_command,
                        local.mc_cd_str );
      }
/*
 *    FOR THE REMOTE MACHINE
*/
    else if ( machine == &remote )
      {
        if ( rem_first )
          {
            if ( machine->mc_id == VMS )
              rem_format = "%s [%s]\n";
            else /* UNIX or DOS */
              rem_format = "%s %s\n";
            if ( kermit )
                rem_command = "remote cwd";
            else /* ftp */
                rem_command = "cd";
            rem_first = FALSE;
          }
/*
 *    write the cd command, with extra \n for local VMS kermit "remote cd"
*/
        (void) fprintf( out_file, rem_format, rem_command, remote.mc_cd_str );
        if ( local.mc_id == VMS && kermit )
            fputs( "\n", out_file );
      }
    else
        return make_error( ERR_ABORT, name,
            (char VAR_PTR) "Machine is neither local nor remote" );
    if ( ferror( out_file ) )
        return make_error( ERR_ABORT, name, err_write );
/*
 *    clear the change_directory string
*/
    machine->mc_cd_str[0] = (char) 0;
    machine->mc_cd_len = 0;
    return TRUE;
  }



Directory find_direct( name, parent )
char VAR_PTR name;
Directory parent;
  {
    Directory dir;
    CATCH_NULL( name && parent, name, Directory );
    for( dir = parent->dir_down; dir; dir = dir->dir_next )
        if ( ! strcmp( dir->dir_name, name ) )
            return dir;
    return (Directory) 0;
  }



Directory make_direct( dir_name, parent )
/*
 * Make a directory with name ``dir_name'' and attach it underneath
 * ``parent'' .
*/
char VAR_PTR dir_name;
Directory parent;
  {
    Directory dir, prev_dir = (Directory) 0;
    char VAR_PTR name = "make_direct";
    CATCH_NULL( dir_name && parent, name, Directory );
    for( dir = parent->dir_down; dir; dir = dir->dir_next )
        prev_dir = dir;
    if ( dir = (Directory) malloc( (unsigned) sizeof(struct dir_struct) ) )
      {
        dir->dir_up = parent;
        if ( prev_dir )
            prev_dir->dir_next = dir;
        else
            parent->dir_down = dir;
        dir->dir_down = dir->dir_next = (Directory) 0;
        dir->dir_name = strdup( dir_name );
        dir->dir_level = parent->dir_level + 1;
        return dir;
      }
    return (Directory) make_error( ERR_ABORT, name,
        (char VAR_PTR) "Cannot malloc" );
  }



boolean parse_options( string, option_table, caller )
/*
 * This function parses ``string'' and assigns values to variables in
 * accordance with the rules specified in the ``option_table'' . Each word
 * in ``string'' is examined to see if it is an abbreviation of an element
 * in the options table. The value of the option is set to the required value .
 * If the caller attempts to use a word not in the ``option_table'', a WARNING
 * error message is generated .
 *   ``option_table'' is an array of data type ``option_elem''. The last element
 * of the array must be filled with NULL pointers and zeros.
*/
char VAR_PTR string;
Option_Elem VAR_PTR option_table;
char VAR_PTR caller;
  {
    int space_left;
    char VAR_PTR next_ptr,
         VAR_PTR str_ptr,
         VAR_PTR name = "parse_options",
         word_str[MAX_WORD_LEN+1];
    Option_Elem VAR_PTR table_ptr;
    static char VAR_PTR format[] =
        { "%c", /* OPT_CHAR  */     "%h",  /* OPT_SHORT */
          "%d", /* OPT_INT   */     "%ld", /* OPT_LONG  */
          "%f", /* OPT_FLOAT */     "%lf", /* OPT_DOUBLE*/
          "%s", /* OPT_STRING */ };
    CATCH_NULL( string && option_table && caller, name, boolean );
/*
 *          skip initial white space
*/
    while( *string && isspace( *string ) )
        string++;
    for( next_ptr = string; *next_ptr; )
      {
/*
 *          copy next word to word_str,
*/
        for( str_ptr = word_str, space_left = MAX_WORD_LEN;
                *next_ptr && ! isspace( *next_ptr );
                next_ptr++ )
            if ( space_left )
              {
                *str_ptr++ = *next_ptr;
                space_left--;
              }
        *str_ptr = '\000';
/*
 *          ensure that ``next_ptr'' points to start of next word
*/
        while( *next_ptr && isspace( *next_ptr ) )
            next_ptr++;
/*
 *          change the selected word to upper case
*/
        for( str_ptr = word_str; *str_ptr; str_ptr++ )
            if ( 'a' <= *str_ptr && *str_ptr <= 'z' )
                *str_ptr += 'A' - 'a';
/*
 *          parse the selected word
*/
        for( table_ptr = option_table; table_ptr->opt_string; table_ptr++ )
          {
            if ( isabbr( (char VAR_PTR) word_str, table_ptr->opt_string,
                         table_ptr->opt_minlen ) )
              {
/*
 *          At this point, `word_str' is an abbreviation of an element
 *          in the options table. The value of the option is set to the
 *          required value. Then go on to parse the next word in the option
 *          string.
 *
 *          If the required value is OPT_NUM_ARG, the next word in the options
 *          string is decoded and assigned to the variable . Be careful with
 *          the floating point comparison .
*/
                if ( fabs( table_ptr->opt_value / OPT_NUM_ARG - 1.0 ) < 2.0e-6 )
                  {
/*
 *          next_ptr should point to a number which can be decoded and
 *          assigned to the variable . first, decide the format string
*/
                    if ( 1 == sscanf( (char *) next_ptr,
                            (char *) format[ table_ptr->opt_vartyp ],
                            table_ptr->opt_varptr ) )
                      {
/*
 *          move next_ptr to the start of the next word
*/
                        while( *next_ptr && ! isspace( *next_ptr ) )
                            next_ptr++;
                        while( *next_ptr && isspace( *next_ptr ) )
                            next_ptr++;
                      }
                  }
                else
                  {
/*                  Assign preset value to the variable . Check for floating
 *                  point values first . If the data is an integer type,
 *                  prevent possible type-conversion rounding error . By
 *                  isolating the ``int_value'' from the floating-point stuff,
 *                  we avoid the possibility of an integer overflow on a
 *                  redundant calculation . It is the users responsibility to
 *                  ensure that the specified option value will actually fit
 *                  into where it is supposed to go .
*/
                    if ( (int) table_ptr->opt_vartyp == OPT_FLOAT )
                        *( (float VAR_PTR) table_ptr->opt_varptr) =
                            table_ptr->opt_value;
                    else if ( (int) table_ptr->opt_vartyp == OPT_DOUBLE )
                        *( (double VAR_PTR) table_ptr->opt_varptr) =
                            table_ptr->opt_value;
                    else
                      {
                        long int int_value = table_ptr->opt_value +
                            ( table_ptr->opt_value > 0.0 ? 0.5 : -0.5 );
                        if (      (int) table_ptr->opt_vartyp == OPT_CHAR )
                            *( (char VAR_PTR) table_ptr->opt_varptr) =
                                int_value;
                        else if ( (int) table_ptr->opt_vartyp == OPT_SHORT )
                            *( (short int VAR_PTR) table_ptr->opt_varptr) =
                                int_value;
                        else if ( (int) table_ptr->opt_vartyp == OPT_INT )
                            *( (int VAR_PTR) table_ptr->opt_varptr) =
                                int_value;
                        else if ( (int) table_ptr->opt_vartyp == OPT_LONG )
                            *( (long int VAR_PTR) table_ptr->opt_varptr) =
                                int_value;
                        else
                            return make_error( ERR_ABORT, name,
                                (char VAR_PTR) "Illegal data type" );
                      }
                  }
                break;
              }
          }
/*
 *          Report an error if the end of the options table is reached
*/
        if ( ! table_ptr->opt_string )
          {
            (void) make_error( ERR_WARN, caller,
                (char VAR_PTR) "Illegal option" );
            (void)  make_error( ERR_WARN, caller,
                (char VAR_PTR) strdup( word_str ) );
          }
      }
    return TRUE;
  }



/* *********************** string handlers ***************************** */


/*
 *  NULL character pointer
*/
#define Z_PTR   (char VAR_PTR) 0


/*
 *  CATCH_NEG(max_chars,caller)
 ********************************
 *      If the integer "max_chars" is zero or negative, an
 * "Max. string length <= 0" error message is generated and the value of
 * "max_chars" is set to zero . Otherwise, nothing is done .
 *      "max_chars" maximum character string length,
 *      "caller"    is a character string naming the function
*/
#define CATCH_NEG(max_chars,caller) \
  if ( max_chars <= 0 )    \
    { max_chars = 0; \
      (void) make_error( ERR_WARN, caller, err_nneg ); }


/*
 *      Array which defines the classification of ASCII characters
*/
char ch_type[128] = {
  CH_CONTROL,     /* NUL */
  CH_CONTROL,     /* SOH */
  CH_CONTROL,     /* STX */
  CH_CONTROL,     /* ETX */
  CH_CONTROL,     /* EOT */
  CH_CONTROL,     /* ENQ */
  CH_CONTROL,     /* ACK */
  CH_CONTROL,     /* BEL */
  CH_CONTROL,     /* BS */
  CH_SPACE | CH_CONTROL, /* HT */
  CH_SPACE | CH_CONTROL, /* LF */
  CH_SPACE | CH_CONTROL, /* VT */
  CH_SPACE | CH_CONTROL, /* FF */
  CH_SPACE | CH_CONTROL, /* CR */
  CH_CONTROL,     /* SO */
  CH_CONTROL,     /* SI */
  CH_CONTROL,     /* DLE */
  CH_CONTROL,     /* DC1 */
  CH_CONTROL,     /* DC2 */
  CH_CONTROL,     /* DC3 */
  CH_CONTROL,     /* DC4 */
  CH_CONTROL,     /* NAK */
  CH_CONTROL,     /* SYN */
  CH_CONTROL,     /* ETB */
  CH_CONTROL,     /* CAN */
  CH_CONTROL,     /* EM */
  CH_CONTROL,     /* SUB */
  CH_CONTROL,     /* ESC */
  CH_CONTROL,     /* FS */
  CH_CONTROL,     /* GS */
  CH_CONTROL,     /* RS */
  CH_CONTROL,     /* US */
  CH_SPACE,       /* SP */
  CH_PUNCTUATION, /* ! */
  CH_PUNCTUATION, /* " */
  CH_PUNCTUATION, /* # */
  CH_PUNCTUATION, /* $ */
  CH_PUNCTUATION, /* % */
  CH_PUNCTUATION, /* & */
  CH_PUNCTUATION, /* ' */
  CH_PUNCTUATION, /* ( */
  CH_PUNCTUATION, /* ) */
  CH_PUNCTUATION, /* * */
  CH_PUNCTUATION, /* + */
  CH_PUNCTUATION, /* , */
  CH_PUNCTUATION, /* - */
  CH_PUNCTUATION, /* . */
  CH_PUNCTUATION, /* / */
  CH_HEXADECIMAL | CH_DECIMAL, /* 0 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 1 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 2 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 3 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 4 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 5 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 6 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 7 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 8 */
  CH_HEXADECIMAL | CH_DECIMAL, /* 9 */
  CH_PUNCTUATION, /* : */
  CH_PUNCTUATION, /* ; */
  CH_PUNCTUATION, /* < */
  CH_PUNCTUATION, /* = */
  CH_PUNCTUATION, /* > */
  CH_PUNCTUATION, /* ? */
  CH_PUNCTUATION, /* @ */
  CH_HEXADECIMAL | CH_UPPER_CASE, /* A */
  CH_HEXADECIMAL | CH_UPPER_CASE, /* B */
  CH_HEXADECIMAL | CH_UPPER_CASE, /* C */
  CH_HEXADECIMAL | CH_UPPER_CASE, /* D */
  CH_HEXADECIMAL | CH_UPPER_CASE, /* E */
  CH_HEXADECIMAL | CH_UPPER_CASE, /* F */
  CH_UPPER_CASE,  /* G */
  CH_UPPER_CASE,  /* H */
  CH_UPPER_CASE,  /* I */
  CH_UPPER_CASE,  /* J */
  CH_UPPER_CASE,  /* K */
  CH_UPPER_CASE,  /* L */
  CH_UPPER_CASE,  /* M */
  CH_UPPER_CASE,  /* N */
  CH_UPPER_CASE,  /* O */
  CH_UPPER_CASE,  /* P */
  CH_UPPER_CASE,  /* Q */
  CH_UPPER_CASE,  /* R */
  CH_UPPER_CASE,  /* S */
  CH_UPPER_CASE,  /* T */
  CH_UPPER_CASE,  /* U */
  CH_UPPER_CASE,  /* V */
  CH_UPPER_CASE,  /* W */
  CH_UPPER_CASE,  /* X */
  CH_UPPER_CASE,  /* Y */
  CH_UPPER_CASE,  /* Z */
  CH_PUNCTUATION, /* [ */
  CH_PUNCTUATION, /* \ */
  CH_PUNCTUATION, /* ] */
  CH_PUNCTUATION, /* ^ */
  CH_PUNCTUATION, /* _ */
  CH_PUNCTUATION, /* ` */
  CH_HEXADECIMAL | CH_LOWER_CASE,  /* a */
  CH_HEXADECIMAL | CH_LOWER_CASE,  /* b */
  CH_HEXADECIMAL | CH_LOWER_CASE,  /* c */
  CH_HEXADECIMAL | CH_LOWER_CASE,  /* d */
  CH_HEXADECIMAL | CH_LOWER_CASE,  /* e */
  CH_HEXADECIMAL | CH_LOWER_CASE,  /* f */
  CH_LOWER_CASE,  /* g */
  CH_LOWER_CASE,  /* h */
  CH_LOWER_CASE,  /* i */
  CH_LOWER_CASE,  /* j */
  CH_LOWER_CASE,  /* k */
  CH_LOWER_CASE,  /* l */
  CH_LOWER_CASE,  /* m */
  CH_LOWER_CASE,  /* n */
  CH_LOWER_CASE,  /* o */
  CH_LOWER_CASE,  /* p */
  CH_LOWER_CASE,  /* q */
  CH_LOWER_CASE,  /* r */
  CH_LOWER_CASE,  /* s */
  CH_LOWER_CASE,  /* t */
  CH_LOWER_CASE,  /* u */
  CH_LOWER_CASE,  /* v */
  CH_LOWER_CASE,  /* w */
  CH_LOWER_CASE,  /* x */
  CH_LOWER_CASE,  /* y */
  CH_LOWER_CASE,  /* z */
  CH_PUNCTUATION, /* { */
  CH_PUNCTUATION, /* | */
  CH_PUNCTUATION, /* } */
  CH_PUNCTUATION, /* ~ */
  CH_CONTROL };   /* DEL */

/* ************************************************************************* */
/*                                                                           */
/*                STRING OPERATORS - boolean returned                        */
/*                                                                           */
/* ************************************************************************* */


boolean isabbr( string, abbrev, minlen )
/*
 *    The function `is_abbr' tests if the character string `string' is a legal
 * abbreviation of the character string `abbrev' . The test is case sensitive .
*/
  char VAR_PTR string,  /* string containing the abbreviation */
       VAR_PTR abbrev;  /* test string */
  int minlen;   /* minimum number of characters allowed in the abbreviation */
  {
    static char VAR_PTR name = "isabbr";
    boolean same;       /* strings are the same so far  */
    CATCH_NULL( string && abbrev, name, boolean );
    if ( strlen( string ) <= strlen( abbrev ) )
      {
        for( same = TRUE;
             *string && ( same &= *string++ == *abbrev++ );
             minlen-- );
        return same && minlen <= 0;
      }
    else
       return FALSE;
  }


int strcmp( str_1, str_2 )
/*
 *      String Compare
 *   The function `strcmp' lexicographically compares string `str_1' with
 * string `str_2', starting with the first character and continuing until a
 * difference is found or the end of a string is reached. The returned value
 * of `strcmp' is
 *   < 0  if `str_1' <  `str_2'
 *   = 0  if `str_1' == `str_2'
 *   > 0  if `str_1' >  `str_2'
*/
char VAR_PTR str_1, VAR_PTR str_2;  /* character strings to be compared */
  {
    static char VAR_PTR name = "strcmp";
    CATCH_NULL( str_1 && str_2, name, int );
    for( ; *str_1 == *str_2; str_1++, str_2++ )
        if ( ! *str_1 )   /* only one string need be tested here */
          return 0;
    return *str_1 - *str_2;
  }


char VAR_PTR strcat( str_dest, str_added )
/*
 *      Concatenate Strings .
 *   Concatenates string `str_added' onto the end of the character string
 * `str_dest' .
*/
char VAR_PTR str_dest, VAR_PTR str_added;
  {
    static char VAR_PTR name = "strcat";
    char VAR_PTR destin;
    CATCH_NULL( str_dest && str_added, name, char VAR_PTR );
    for( destin = str_dest; *destin; destin++ );
    while( *str_added )
      *destin++ = *str_added++;
    *destin = NUL;
    return str_dest;
  }


char VAR_PTR strchr( str, c )
/*
 *        STRing CHaRacter point
 *   Returns a pointer to the first character `c' in the string `str' . If the
 * character `c' is not found in string `str', the NULL pointer is returned .
 * The terminating NUL is considered part of the string .
*/
char VAR_PTR str, c;
  {
    static char VAR_PTR name = "strchr";
    CATCH_NULL( str, name, char VAR_PTR );
    for( ; *str != c && *str; str++ );
    return *str == c ? str : Z_PTR;
  }


char VAR_PTR strdup( string )
/*
 *    Save a string on the heap and return a pointer to that place . If memory
 * cannot be allocated, a NULL pointer is returned .
*/
char VAR_PTR string;
  {
    static char VAR_PTR name = "strdup";
    char VAR_PTR pointer;
    CATCH_NULL( string, name, char VAR_PTR );
    if ( pointer = (char VAR_PTR)
            malloc( (unsigned int) ( strlen( string ) + 1 ) * sizeof(char) ) )
        (void ) strcpy( pointer, string );
    return pointer;
  }


char VAR_PTR strndup( string, n )
/*
 *    Save a string on the heap and return a pointer to that place . Space for
 * up to `n' characters (and the terminating NUL) is allocated, and a maximum
 * of `n' characters are copied to the new string . If memory cannot be
 * allocated, a NULL pointer is returned .
*/
char VAR_PTR string;
  {
    static char VAR_PTR name = "strndup";
    char VAR_PTR pointer;
    int alloc_len;          /* allocated length of saved string */
    CATCH_NULL( string, name, char VAR_PTR );
    CATCH_NEG( n, name );
    if ( n <= 0 )
        return Z_PTR;
    alloc_len = strnlen( string, n );
    if ( pointer = (char VAR_PTR) malloc(
            (unsigned int) ( alloc_len + 1 ) * sizeof(char) ) )
        (void) strncpy( pointer, string, alloc_len );
    return pointer;
  }


int strlen( string )
/*
 *      Measures the length of a `string' .
*/
char VAR_PTR string;
  {
    static char VAR_PTR name = "strlen";
    char VAR_PTR ptr;
    CATCH_NULL( string, name, int );
    for( ptr = string; *ptr; ptr++ );
    return ptr - string;
  }


int strnlen( string, n )
/*
 *      Measures the length of a `string' . A maximum of `n' characters are
 * examined and if this maximum is reached, the returned value is `n' .
*/
char VAR_PTR string;
int n;
  {
    static char VAR_PTR name = "strnlen";
    char VAR_PTR ptr;
    CATCH_NULL( string, name, int );
    CATCH_NEG( n, name );
    for( ptr = string; *ptr && n--; ptr++ );
    return ptr - string;
  }


char VAR_PTR strupr( string )
/*
 * Converts lower case characters in `string' to upper case . Other characters
 * are not changed . The returned value is `string' .
*/
char VAR_PTR string;
  {
    static char VAR_PTR name = "strupr";
    char VAR_PTR ptr;
    CATCH_NULL( string, name, char VAR_PTR );
    for( ptr = string; *ptr; ptr++ )
        if ( islower( *ptr ) )
            *ptr += 'A' - 'a';
    return string;
  }


char VAR_PTR strntok( string, separators, n )
/*
 * strntok() considers the `string' to consist of a sequence of zero or more
 * text tokens separated by spans of one or more characters from `separator'.
 * The first call (with pointer `string' specified) returns a pointer to the
 * first character of the first token, and will have written a null character
 * into `string' immediately following the returned token. The function keeps
 * track of its position in the string between separate calls, so that
 * subsequent calls (which must be made with the first argument a NULL pointer)
 * will work through the `string' immediately following that token. In this way
 * subsequent calls will work through the `string' until no tokens remain.
 * `separators' may be different from call to call. When no token remains in
 * `string', a NULL pointer is returned . A maximum of `n' characters in
 * `string' are examined .
*/
char VAR_PTR string, VAR_PTR separators;
int n;
  {
    static char VAR_PTR name = "strntok";
/*
 *      These variables must be preserved between function calls
*/
    static char
      end_char = NUL,               /* to remember the char after a token */
      VAR_PTR token_str = Z_PTR,    /* saved token string */
      VAR_PTR start_token = Z_PTR,  /* pointer to start of token */
      VAR_PTR end_token = Z_PTR;    /* pointer to end of token */
    static int n_token = 0;         /* number of remaining chars in `string' */
    CATCH_NULL( separators, name, char VAR_PTR );
    if ( strlen( separators ) <= 0 )
        return (char VAR_PTR) make_error( ERR_ABORT, name, err_mtsep );
/*
 *      if a new string of tokens is provided, save it on the heap, but
 *      first free any memory already used .
*/
    if ( string != Z_PTR )
      {
        CATCH_NEG( n, name );
        n_token = n;
        if ( token_str )
            (void) free( (char *) token_str );
        start_token = end_token = token_str = strndup( string, n_token );
        if ( end_token )
            end_char = *end_token;
        else
            end_char = NUL;
      }
/*
 *      return with NULL pointer if memory allocation failed or if a previous
 *      call reached the end of `string' .
*/
    if ( ! start_token || n_token <= 0 )
        return Z_PTR;
/*
 *      find the start of a token
*/
    *end_token = end_char;
    for ( start_token = end_token;
          *start_token && n_token && strchr( separators, *start_token );
          start_token++, n_token-- );
/*
 *      return with NULL pointer if a token is not found
*/
    if ( ! *start_token || n_token <= 0 )
        return start_token = Z_PTR;
/*
 *      find the end of the token and then append a NULL before returning
*/
    for ( end_token = start_token;
          *end_token && n_token && ! strchr( separators, *end_token );
          end_token++, n_token-- );
    end_char = *end_token;
    *end_token = NUL;
    return start_token;
  }



/* ************************************************************************ */
/*                                                                          */
/*                      ERROR CONDITION HANDLERS                            */
/*                                                                          */
/* ************************************************************************ */
/*                                                                          */
/*      These functions allow the user to record errors detected by other   */
/*  functions . Pointers to the error message strings are stored on a       */
/*  queue .The error messages can be printed or written to a file . The     */
/*  user can test whether or not errors have occurred, and can clear any    */
/*  recorded errors .                                                       */
/*                                                                          */
/*      Any recorded error has a `status' which indicates the severity of   */
/*  the error . Permissible values of `status' -- first argument of         */
/*  `make_error', `is_error' and `dump_error' -- are,                       */
/*                                                                          */
/*       ERR_CALL  is used only with `make_error' .                         */
/*       ERR_ALL   is used with `is_error' or `dump_error' .                */
/*       ERR_FATAL is the highest message status .                          */
/*       ERR_NOTE  is the lowest message status .                           */
/*       ERR_PRINT is a flag used only with `make_error'                    */
/*                                                                          */
/*  The following values can be added to the `status' to indicate a `math'  */
/*  error . The presence of one of these additional information to be       */
/*  queued .                                                                */
/*       ERR_DOMAIN     Argument(s) not in domain of function               */
/*       ERR_OVER       Result is greater than maximum allowed              */
/*       ERR_UNDER      Result is indistinguishable from 0.0                */
/*       ERR_SING       Argument produces singularity                       */
/*       ERR_TLOSS      Argument produces total loss of significant digits  */
/*       ERR_PLOSS      Arg. produces partial loss of significant digits    */
/*                                                                          */
/* External functions used : fprintf(), free(), malloc()                    */
/*                                                                          */
/* ************************************************************************ */

#define SEVERITY_MASK  ( ERR_NOTE | ERR_WARN | ERR_ABORT | ERR_FATAL )
#define MATH_MASK      ( ERR_DOMAIN | ERR_OVER | ERR_UNDER | \
                         ERR_SING | ERR_TLOSS | ERR_PLOSS )
#define MIN_MATH       ERR_DOMAIN
/*
 *                              TYPE DEFINITIONS
 *                     ********************************
*/
typedef struct error_st         /* error message queue structure */
  {
    int  type;                  /* category of error             */
    char VAR_PTR location;      /* name of routine where error occurred */
    char VAR_PTR message;       /* error message                 */
    struct error_st VAR_PTR next_err;  /* pointer to next error message */
  }
    VAR_PTR Error;

/*
 *                             GLOBAL VARIABLES
 *                     ********************************
*/
static Error
  first_error = 0,          /* pointer to first logged error message    */
  curr_error = 0;           /* pointer to last logged error message     */
FILE *error_file = (FILE *) 0; /* output file for writing error messages */

static char
    VAR_PTR err_math[] = { "", "Domain", "Overflow", "Underflow", "Singularity",
            "Total loss of significance", "Partial loss of significance", "", },
                            /* Printed text for math errors */
    VAR_PTR err_sev[] = { "", "Note", "Warning", "Abort", "Fatal" },
                            /* printed text for error status/severity   */
    VAR_PTR err_unknown = "unknown function";

/*
 *                                FUNCTIONS
 *                    ********************************
*/

boolean make_error( status, where, text )
/*
 *    Records an error message of severity `status' for dumping at a
 *  later time . `where' and `text' are pointers to static character
 *  strings . `where' is the name of the function where the error has
 *  been detected . `text' is the text of the error message .
 *  If the ERR_PRINT bit in `status' is set, the error message is written
 *  to the error file ( set by `file_error()', default is `stderr' ) .
 *  If status contains a `math' error value, then `text' is interpreted as
 *  a pointer to `double' and the text of the error message reports the
 *  nature of the error and , if `text' is not a NULL pointer, the value of
 *  *( (double VAR_PTR) text) . If the error is not a `math' error and `text'
 *  in not a NULL pointer, `text' should be a static character string so
 *  that it does not disappear when the calling function returns .
*/
int status;
char VAR_PTR where, VAR_PTR text;
  {
    Error new_ptr;
    if ( ( status & SEVERITY_MASK ) == ERR_CALL && curr_error )
        text = curr_error->location;
/*
 *          allocate memory for the queue entry
*/
    if ( ( new_ptr = (Error) malloc( sizeof( struct error_st ) ) ) )
      {
/*
 *          fix pointer to current error
*/
        if ( curr_error )
            curr_error->next_err = new_ptr;
        else
            first_error = new_ptr;
/*
 *          fill in the queue entries
 *  in the case of a math error with non-NULL `text', the numeric value should
 *  be saved in allocated memory first .
*/
        curr_error = new_ptr;
        curr_error->type = status;
        curr_error->location = where ? where : err_unknown;
        if ( ( status & SEVERITY_MASK ) == ERR_CALL &&
             ( status & MATH_MASK ) && text )
          {
            curr_error->message = malloc( (unsigned) sizeof(double) );
            *((double VAR_PTR) curr_error->message) = *((double VAR_PTR) text);
          }
        else
            curr_error->message  = text;
        curr_error->next_err = (Error) 0;
/*
 *          write the error ( if required )
*/
        if ( status & ERR_PRINT )
          {
            if ( ! error_file )
                error_file = stderr;
            if ( ( status & SEVERITY_MASK ) == ERR_CALL )
                (void) fprintf( error_file,
                    "Error in value returned to %s from %s", where, text );
            else
              {
                (void) fprintf( error_file, "%s error in %s: ",
                    err_sev[ status & SEVERITY_MASK ], where );
                if ( status & MATH_MASK )
                  {
                    (void) fputs( (char *)
                        err_math[ ( status & MATH_MASK ) / MIN_MATH ],
                        error_file );
                    if ( text )
                        (void) fprintf( error_file, " from argument %f",
                            *( (double VAR_PTR) text ) );
                  }
                else if ( text )
                    (void) fputs( (char *) text, error_file );
              }
            (void) fputc( '\n', error_file );
          }
      }
    else
      {
        (void) fputs( "\nCannot allocate memory in make_error\n", stderr );
        exit( 1 );
      }
    return FALSE;
  }



int dump_error( status )
/*
 *    Dumps all logged errors with a status `status' or higher onto the
 * error file ( set by `file_error()', default is `stderr' ) .
 *    Returns the number of errors printed .
 *
 *  Synopsis :    int no_of_errors = dump_error( status );
*/
int status;
  {
    Error ptr;
    int err_count = 0;
    if ( ! error_file )
        error_file = stderr;
    for( ptr = first_error; ptr; ptr = ptr->next_err )
        if ( ( ptr->type & SEVERITY_MASK ) >= status )
          {
            if ( ( ptr->type & SEVERITY_MASK ) == ERR_CALL )
                (void) fprintf( error_file,
                    "Error in value returned to %s from %s",
                    ptr->location, ptr->message );
            else
              {
                (void) fprintf( error_file, "%s error in %s: ",
                    err_sev[ ptr->type & SEVERITY_MASK ], ptr->location );
                if ( ptr->type & MATH_MASK )
                  {
                    (void) fputs( (char *)
                        err_math[ ( ptr->type & MATH_MASK ) / MIN_MATH ],
                        error_file );
                    if ( ptr->message )
                        (void) fprintf( error_file, " from argument %f",
                            *( (double VAR_PTR) ptr->message ) );
                  }
                else if ( ptr->message )
                    (void) fputs( (char *) ptr->message, error_file );
              }
            (void) fputc( '\n', error_file );
            err_count++;
          }
    if ( err_count )
        (void) fprintf( error_file, "%d errors printed\n", err_count );
    return err_count;
  }


boolean is_error( status )
/*
 *    Has an error of status `status' or higher been recorded ?
 * Returns TRUE if an error of the required status has been recorded,
 * and FALSE if no error has been logged .
 *
 *  Synopsis :    boolean errors_exist = is_error( status );
*/
int status;
  {
    Error ptr;
    for( ptr = first_error; ptr; ptr = ptr->next_err )
        if ( ( ptr->type & SEVERITY_MASK ) >= status )
            return TRUE;
    return FALSE;
  }

