c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      BLOCK DATA DTBLK
c
c           Initialize common block variables
c
      implicit none
      include 'digtab.cmn'
c
      data port, baud, reject, slx, sly / 1, 2400, 4, 1, 1 /
      data ax, ay, bx, by, h, phi, resol, w, theta, x0, y0 /
     1  2*1.0, 2*0.0, 350.0, 0.0, 0.127, 350.0, 3*0.0 /
      data fixed0, metric, lingrf, scaled, graphic, dbug /
     1  2*.true., 4*.false. /
      end


      SUBROUTINE DTCAL( DEFLT )
c
c       Digitizing Tablet CALibrate
c    Performs a complete calibration of the digitizing tablet .
c Argument
c  logical *1 deflt    # .true. = default hardware settings are used
c                      # .false. = default hardware settings are not used
c Variable
c  logical *1 equery   # function : logical response
c
      implicit none
      logical *1 deflt, def1, equery
      include 'digtab.cmn'
c
      def1 = deflt
c     repeat
1       call DTHARD( def1 )
        def1 = .false.
        call DTMSSG( 'OK ? ' )
        if ( .not. EQUERY( Hkbdlu, ' ' ) ) goto 1
c       until( ok )
c     repeat
2       call DTPAGE
        call DTMSSG( 'OK ? ' )
        if ( .not. EQUERY( Hkbdlu, ' ' ) ) goto 2
c       until( ok )
      call DTMSSG( 'Are you defining axes ?' )
      if ( EQUERY( Hkbdlu, ' ' ) ) then
c       repeat
3         call DTAXES
          call DTMSSG( 'OK ? ' )
          if ( .not. EQUERY( Hkbdlu, ' ' ) ) goto 3
c         until( ok )
      else
        call DTPGRF
        end if
      return
      end


      SUBROUTINE DTDBUG( DEBUG )
c
c       Set debug mode .
c
      implicit none
      logical *1 debug
      include 'digtab.cmn'
c
      dbug = debug
      return
      end


      SUBROUTINE DTWAIT
c
c       wait for carriage return - in graphics mode
c
      implicit none
      include 'digtab.cmn'
c
      call vecmod
      graphic = .true.
      call DTMSSG( 'Press <return> to continue' )
      read( Hkbdlu, 1 )
1     format( )
      graphic = .false.
      call anmode
      return
      end


      SUBROUTINE DTPGRF
c
c       Initialize graphics and plot the page
c
c integer *2 i          # corner indexing variable
c real x,y,             # plotted page corners ( initial & final )
c      ih,iw            # graph height & width in screen units
c
      implicit none
      real Hv500h, Hv500w
      parameter( Hv500h=779.0, Hv500w=1023.0 )
c
      integer *2 i
      real x, y, ih, iw
      include 'digtab.cmn'
c
      call GRSTRT( 0, 0 )
      ih = min( Hv500h, Hv500h * 1.3132 * h / w )
      iw = min( Hv500w, Hv500w * 0.7615 * w / h )
      call VWPORT( ( Hv500w - iw ) / 2.0 , ( Hv500w + iw ) / 2.0 ,
     1             ( Hv500h - ih ) / 2.0 , ( Hv500h + ih ) / 2.0 )
      call WINDOW( -0.03 * w , 1.03 * w , -0.03 * h , 1.03 * h )
      call MOVE( 0.0, 0.0 )
      call DRAW( 0.0,  h  )
      call DRAW(  w,   h  )
      call DRAW(  w,  0.0 )
      call DRAW( 0.0, 0.0 )
      call SINCOS( -theta )
      do 1 i = 1, 4
        x = xn(i) - xll
        y = yn(i) - yll
        call ROTATE( x, y )
        call MOVE( x, y )
        x = xc(i) - xll
        y = yc(i) - yll
        call ROTATE( x, y )
        call DRAW( x, y )
1       continue
      graphic = .not. dbug
      if ( graphic )
     1  call vecmod
      return
      end


      SUBROUTINE DTEND
c
c       Terminate screen graphics
c
      implicit none
      include 'digtab.cmn'
c
      if ( graphic ) then
        call anmode
        graphic = .false.
        end if
      return
      end


      SUBROUTINE HIPAD( X, Y, MODE , BUTTON )
c
c  Get a coordinate pair from the HIPAD digitizing tablet and check that the
c tablet is operating in the correct mode .
c
c Arguments
c  real X, Y          = coordinate pair in 'tablet coordinates'
c  integer *2 MODE    = required mode of the tablet
c    = 0 if the 1st coordinate of a switched stream
c    = 1 if a successive coordinate of a switched stream
c    = 2 if a coordinate in the point mode
c    = 3 if a coordinate of a stream with cursor released
c    = 4 if a coordinate of a stream with cursor depressed
c    ( = 5 if a coordinate in the switched stream or point mode )
c  integer *2 BUTTON  = identifies the cursor button used
c    = 0 if none     The button is only correctly identified if the origin
c    = 2 if red      is floating . If the origin is fixed, the digitizer
c    = 4 if yellow   is colour-blind ; it thinks that the red button is
c    = 5 if green    yellow , and the white button is green . Ie. it can
c    = 9 if white    only distinguish between LHS and RHS .
c
c integer *2 colour(3:6),  # list button colours ( order is important )
c            i,            # skipped point counter
c            ix, iy,       # integer form of coordinates from tablet
c            m,            # actual mode of the tablet
c            IO_gets       # function : get character string from serial port
c logical *1 err           # mode error
c            f1err,        # first occurrence of mode error
c character Cdigstr*15     # characters from digitizer
c           Cmoderr(0:4)*23# required responses to mode error
c           Chr1,          # first char received in a line from digitizer
c
      implicit none
      include 'digtab.cmn'
      real x, y
      integer *2 colour(3:6), i, ix, iy, mode, button, m, IO_gets
      logical *1 err, f1err
      character Cdigstr*Hnchars, Cmoderr(0:5)*23, Chr1*1
c
      data colour / 9, 2, 5, 4 /
      data Cmoderr(0) / 'switched stream        ' /
      data Cmoderr(1) / 'switched stream        ' /
      data Cmoderr(2) / 'point                  ' /
      data Cmoderr(3) / 'stream (release button)' /
      data Cmoderr(4) / 'stream (press button)  ' /
      data Cmoderr(5) / 'switch stream or point ' /
c
      f1err = .true.
c     repeat
c
c           get data coordinate from aux. port
c
1       if ( IO_gets( Hdiglu, Cdigstr ) .ne. Hnchars )
     1    goto 8
c
c           remove parity bit
c
        do 5 i = 1, 15
          Cdigstr(i:i) = char( iand( ichar( Cdigstr(i:i) ), 127 ) )
5         continue
        if ( dbug )
     1    write( Hvdulu, 3 ) Cdigstr(1:13)
3         format( 1x, a )
        read( Cdigstr, 4, ERR=8 ) chr1, ix, iy
4       format( a1, 2i6 )
        button = colour( ichar( chr1 ) / 16 )
        m = mod( ichar( chr1 ), 16 )
        if ( m .eq. 3 ) button = 0
c
c           check for correct mode
c
        err = ( mode .ne. 5 .or. m .gt. 2 ) .and.
     1        ( m .gt. 1 .or. mode .gt. 1 ) .and. m .ne. mode
        if ( err .and. f1err ) then
          write ( Hvdulu, 2 ) charnb( Cmoderr(mode) )
2         format( ' HIPAD : MODE ERROR : Use ', a, ' mode' )
          f1err = .false.
        else if ( err ) then
          write( Hvdulu, 6 ) Hbell
6         format( '+', a1 )
          end if
        if ( err ) goto 1
c     until ( ! err )
c
c     Convert from integer to floating point
c
      if ( metric ) then
        x = real( ix ) / 10.0
        y = real( iy ) / 10.0
      else
        x = real( ix ) / 1000.0
        y = real( iy ) / 1000.0
        end if
      if ( m .ne. 2 ) then
        do 7 i = 1, reject
          if ( IO_gets( Hdiglu, Cdigstr ) .ne. Hnchars )
     1      goto 8
7         continue
        end if
      return
c
c         error message
c
8     call DTMSSG( 'HIPAD : Cannot read data' )
      call DTEND
      stop
      end


      SUBROUTINE DTHARD( DEF )
c
c    Define the HIPAD digitizing tablet hardware settings
c  and set up the default transformation laws .
c
c  Argument
c   logical *1 def          # default hardware settings are used
c
c The data transmission rate depends on the digitizer baud rate .
c 4800 baud = 32 points per second, 2400 = 16, 1200 = 8, 300 = 2 .
c
c  character Corigin*5,     # 'fixed' or 'float 'ing digitizer origin
c            Cunits*6,      # 'metric' or 'inches' digitizer units
c            Cresol*4,      # 'high' or 'low ' digitizer resolution
c            Cset*80        # setup command string
c  integer *2 baud1,        # provisional baud rate
c             inum,         # result from string search
c             isetlu,       # logical unit of setup source device
c             jljstr        # function : left justify string
c  logical *1 IO_serial     # function : open serial port
c
      implicit none
      character Corigin*5, Cunits*6, Cresol*4, Cset*80
      integer *2 baud1, inum, isetlu, jljstr
      logical *1 IO_serial, def
      include 'digtab.cmn'
c
      if ( graphic ) then
        call anmode
        graphic = .false.
        end if
c
c           default hardware settings
c
      reject = 4
      port   = 2
      baud   = 2400
      fixed0 = .true.
      metric = .true.
      resol  = 0.127
      Corigin= 'fixed'
      Cunits = 'metric'
      Cresol = 'high'
      errfac = 5.0
c
c           set up default transformation laws
c             ( N.B. there is no 'tablet' to 'page' default . )
c
      x0 = 0.0
      y0 = 0.0
      phi = 0.0
      scaled = .false.
      bx = 0.0
      by = 0.0
      by = 0.0
      ax = 1.0
      ay = 1.0
      slx = 1
      sly = 1
c
c           set up file or keyboard for input
c
      if ( def ) then
        isetlu = Hdeflu
        open( UNIT=Hdeflu, FILE='\ilg\digtab.fmt', STATUS='old',
     1    ACCESS='sequential', FORM='formatted', ERR=10 )
      else
        isetlu = Hkbdlu
        write( Hvdulu, 1 )
1       format( ' Enter hardware settings ( finish with `end'')' / )
        end if
c
c           loop for reading and interpreting a setup line
c
c     repeat
c
c           read, convert to lower case & left justify
c
2       read( isetlu, 3, END=9 ) Cset
3       format( a )
        if ( jljstr( Cset ) .le. 0 .or. Cset(1:1) .eq. ';' ) goto 2
        call scase( Cset, Cset, 'l' )
        inum = index( Cset, 'reject' )
c
c           no. of rejected points per accepted point
c
        if ( inum .gt. 0 ) then
          read( Cset(inum+6:), *, ERR=4 ) reject
          reject = min( max( reject, 0 ), 31 )
          end if
c
c           serial port number
c
4       inum = index( Cset, 'port' )
        if ( inum .gt. 0 ) then
          read( Cset(inum+4:), *, ERR=5 ) port
          port = min( max( port, 1 ), 2 )
          end if
c
c           baud rate
c
5       inum = index( Cset, 'baud' )
        if ( inum .gt. 0 ) then
          read( Cset(inum+4:), *, ERR=6 ) baud1
          if ( baud1 .ne. 300 .and. baud1 .ne. 1200 .and.
     1         baud1 .ne. 2400 .and. baud1 .ne. 4800 ) goto 6
          baud = baud1
          end if
c
c           fixed or floating digitizer origin
c
6       if ( index( Cset, 'fixed' ) .gt. 0 ) then
          Corigin = 'fixed'
          fixed0 = .true.
        else if ( index( Cset, 'float' ) .gt. 0 ) then
          Corigin = 'float'
          fixed0 = .false.
          end if
c
c           millimeter or inch digitizer units
c
        if ( index( Cset, 'metric' ) .gt. 0 ) then
          Cunits = 'metric'
          metric = .true.
        else if ( index( Cset, 'inches' ) .gt. 0 ) then
          Cunits = 'inches'
          metric = .false.
          end if
c
c           high (200 lpi) or low (100 lpi) digitizer resolution
c
        if ( index( Cset, 'high' ) .gt. 0 ) then
          Cresol = 'high'
          resol = 0.005
        else if ( index( Cset, 'low' ) .gt. 0 ) then
          Cresol = 'low'
          resol = 0.01
          end if
        if ( metric .and. resol .lt. 0.05 ) then
          resol = resol * 25.4
        else if ( .not. metric .and. resol .gt. 0.05 ) then
          resol = resol / 25.4
          end if
c
c           error factor
c
        inum = index( Cset, 'error' )
        if ( inum .gt. 0 ) then
          read( Cset(inum+5:), *, ERR=14 ) errfac
          errfac = min( max( errfac, 2.0 ), 50.0 )
          end if
c
c           display settings
c
14      if ( index( Cset, 'display' ) .gt. 0 ) then
          write( Hvdulu, 7 ) reject, reject + 1, port, baud, Corigin,
     1      Cunits, Cresol, errfac
7         format( ' Hardware settings :' /
     1            '   reject ', i2, ' out of ', i2, ' points received' /
     2            '   serial port number = ', i2 /
     3            '   baud rate = ', i5 /
     4            '   ', a, ' digitizer origin' /
     5            '   ', a, ' digitizer units' /
     6            '   ', a, ' digitizer resolution' /
     7            '   error factor = ', f5.1 / )
          end if
c
c           help
c
        if ( index( Cset, 'help' ) .gt. 0 ) then
          write( Hvdulu, 8 )
8         format( '    Keywords for specifying hardware settings are',
     1        ' in upper case.' /
     2      '     1) Number of rejected points per accepted point in',
     3        ' stream mode : REJECT 4' /
     4      '     2) Serial port to be used : PORT 2' /
     5      '     3) Baud rate of serial port : BAUD 2400' /
     6      '     4) FIXED of FLOATing digitizer coordinate origin' /
     7      '     5) METRIC (mm) or INCHES digitizer units' /
     8      '     6) HIGH (200 lpi) or LOW (100 lpi) digitizer',
     9      ' resolution' /
     a      '     7) allowed ERROR factor on axes & scaling' /
     b      '     8) DISPAY hardware settings' )
          end if
        if ( index( Cset, 'end' ) .eq. 0 ) goto 2
c       until( end )
9     if ( .not. IO_serial( Hdiglu, port, baud ) )
     1  goto 12
      if ( def )
     1  close( Hdeflu )
      return
c
c           fatal error mesages
c
10    write( Hvdulu, 11 )
11    format( ' FATAL : Can''t read setup file' )
      stop
c
12    write( Hvdulu, 13 )
13    format( ' FATAL : Can''t open serial port' )
      stop
      end


      SUBROUTINE DTPAGE
c
c    Specify the corners of the page according to the rules in 1)a)
c
c integer *2 button,    # identifies the cursor button used
c            i,         # index
c            j, l,      # ends of pair of sides
c            k,         # vertex to be optimized
c            m          # index of maximum value for sorting
c
c real  a(4),           # arg (angle) of a corner wrt centroid of data points
c       cx, cy,         # centre of the vertex locus
c       d(4),           # distance between corners of the rectangle
c       err(4),         # error or dist the corner is moved in one iteration
c       r,              # length of semi diagonal for 90 degree corner
c       qarc0           # function : normalizes angle to +/- pi
c
c logical *1 ok,        # validity flag
c            equery     # function for prompting and receiving logical input
c
c subroutine hipad,     # get coord pair from HIPAD & check mode
c            swap       # swap 2 real values
c
      implicit none
      integer *2 button, i, j, l, k, m
      real  a(4), cx, cy, d(4), err(4), r, xn1, yn1, qarc0
      logical *1 ok, equery
      include 'digtab.cmn'
      equivalence ( a(1), d(1), err(1) )
c
c     repeat
1       call DTMSSG( 'Define the corners of the page' )
c       repeat
2         do 3 i = 1, 4
            call HIPAD( xn( i ), yn( i ), 2 , button, .false. )
3           continue
          call DTMSSG( 'Page corners OK ?' )
          if ( .not. EQUERY( Hkbdlu, ' ' ) ) goto 2
c       until ( Page corners are OK ? )
c
c         sort the corners into proper order ul,ur,lr & ll
c
        ok = .true.
        cx = ( xn(1) + xn(2) + xn(3) + xn(4) ) / 4.0
        cy = ( yn(1) + yn(2) + yn(3) + yn(4) ) / 4.0
        do 4 i = 1, 4
          a(i) = Hpi - atan2( yn(i) - cy, xn(i) - cx )
4         continue
        do 6 i = 1, 3
          m = i
          do 5 j = i+1, 4
            if ( a(m) .gt. a(j) ) m = j
5           continue
          call SWAP( xn(i), xn(m) )
          call SWAP( yn(i), yn(m) )
          call SWAP( a(i), a(m) )
6         continue
c
c     iterate vertex coordinates so that sides become perpendicular
c
        cx = 4.0 * resol **2
        do 7 i = 1, 4
          err(i) = cx
          xc(i) = xn(i)
          yc(i) = yn(i)
7         continue
        j = 1
        k = 2
        l = 3
c       repeat
8         cx = ( xn(j) + xn(l) ) / 2.0
          cy = ( yn(j) + yn(l) ) / 2.0
          r = 0.5 * sqrt( ( xn(j) - xn(l) ) **2 +
     1                    ( yn(j) - yn(l) ) **2 )
          theta = atan2( yn(k) - cy , xn(k) - cx )
          xn1 = ( r * cos( theta ) + cx + xn(k) ) / 2.0
          yn1 = ( r * sin( theta ) + cy + yn(k) ) / 2.0
          err(k) = ( xn1 - xn(k) ) **2 + ( yn1 - yn(k) ) **2
          xn(k) = xn1
          yn(k) = yn1
          j = mod( j, 4 ) + 1
          k = mod( k, 4 ) + 1
          l = mod( l, 4 ) + 1
          if ( err(1) + err(2) + err(3) + err(4) .ge. resol ** 2 )
     1      goto 8
c         until ( err(1) + err(2) + err(3) + err(4) .lt. resol ** 2 )
c
c      check that the inclination of the top edge is less than 30 degrees
c
        xll = xn(4) - resol
        yll = yn(4) - resol
        theta = 0.0
        do 9 i = 1, 4
          l = mod( i, 4 ) + 1
          theta = theta + atan2( yn(l) - yn(i), xn(l) - xn(i) )
          d(i) = ( ( yn(l) - yn(i) ) **2 + ( xn(l) - xn(i) ) **2 ) **0.5
9         continue
        theta = QARC0( theta + Hpi ) * 45.0 / Hpi
        if ( abs( theta ) .gt. 30.0 ) then
          call DTMSSG( 'Inclination > 30 degrees . Realign it.' )
          ok = .false.
          end if
        h = ( d(2) + d(4) ) / 2.0 + resol * errfac / 2.0
        w = ( d(1) + d(3) ) / 2.0 + resol * errfac / 2.0
c
c           Initialize graphics & plot the page
c
        call DTPGRF
        if ( .not. ok ) goto 1
c       until ( ok )
      if ( dbug )
     1  call dtwait
      return
      end


      SUBROUTINE DTAXES
c
c   Subroutine to calculate the position of the axes on the page, to calculate
c the 'page' to 'axes' transformation and , if necessary scale the axes and
c calculate the 'axes' to 'graph' transformation .
c
c
c real  a,                      # slope of x axis
c       cx,                     # y intercept of x axis ('page' coords)
c       cy,                     # x intercept of y axis ('page' coords)
c       covarh,                 # covariance of x & y, or x & z for x axis
c       covarv,                 # covariance of x & y, or y & z for y axis
c       d,                      # denominator = 1 + ax **2
c       err,                    # sum of mean square errors for axes
c       errthr,                 # threshold rms error squared
c       errx,erry,              # mean square errors for scaling of x & y axes
c       meanxh,meanxv,          # mean value of x for x & y axes respectively
c       meanyh,meanyv,          # mean value of y for x & y axes respectively
c       meanzh,meanzv,          # mean value of x for x & y axes respectively
c       t,                      # for calculating limits of axes
c       varxh,varxv,            # variance of x for x & y axes respectively
c       varyh,varyv,            # variance of y for x & y axes respectively
c       varzh,varzv,            # variance of z for x & y axes respectively
c       xi(Hnaxpts),yi(Hnaxpts),# (x,y) coords defining x axis
c       zi(Hnaxpts),            # scale values for x axis
c       xj(Hnaxpts),yj(Hnaxpts),# (x,y) coords defining y axis
c       zj(Hnaxpts),            # scale values for y axis
c
c integer *2 i,                 # looping index
c            mi, mj,            # number of data points for x & y axes
c            jresp              # function ; receives and checks integer input
c
c logical *1 equery,       # function for prompting and receiving logical input
c            ok                 # .true. scaling corr. coeffs are ok
c
c subroutine saxdat,           # get data for defining & ascaling an axis
c            srotat,           # rotation linear transformation
c            smeans            # means, variances & covariance of x & y
c
      implicit none
      include 'digtab.cmn'
      real a, cx, cy, covarh, covarv, d, err, errthr,
     1  errx, erry, meanxh, meanxv, meanyh, meanyv, meanzh, meanzv,
     2  t, varxh, varxv, varyh, varyv, varzh, varzv,
     3  xi(Hnaxpts), yi(Hnaxpts), zi(Hnaxpts),
     4  xj(Hnaxpts), yj(Hnaxpts), zj(Hnaxpts)
      integer *2 i, mi, mj, jresp
      logical *1 equery, ok
c
      if ( .not. graphic .and. .not. dbug ) then
        call vecmod
        graphic = .true.
        end if
      errthr =  ( errfac * resol ) **2
c     repeat                                                              1b
c
c    set up default transformation laws
c
1       lingrf = .false.
        x0 = 0.0
        y0 = 0.0
        phi = 0.0
        scaled = .false.
        bx = 0.0
        by = 0.0
        ax = 1.0
        ay = 1.0
        slx = 1
        sly = 1
c
        call DTMSSG( 'Are axes scaled ? ' )
        scaled = Equery( Hkbdlu, ' ' )
c
c   get data & evaluate means for x axis
c
        call DTMSSG( 'HORIZONTAL AXIS - Press <return> continue' )
        call wait
        if ( scaled ) then                                                2b
          call DTMSSG( 'Define the scaling law ?  1=Linear, 2=Log ' )
          slx = JRESP( Hkbdlu, 1, 2, Hnotries )
          end if                                                          2e
        call SAXDAT( xi, yi, zi, mi, slx )
        call SMEANS( xi, yi, 1, mi, meanxh, meanyh, varxh, varyh,
     1    covarh )
c
c   get data & evaluate means for y axis
c
        call DTMSSG( 'VERTICAL AXIS - Press <return> to continue' )
        call wait
        if ( scaled ) then                                                2b
          call DTMSSG( 'Define the scaling law ?  1=Linear, 2=Log ' )
          sly = JRESP( Hkbdlu, 1, 2, Hnotries )
          end if                                                          2e
        call SAXDAT( xj, yj, zj, mj, sly )
        call SMEANS( xj, yj, 1, mj, meanxv, meanyv, varxv, varyv,
     1    covarv )
c
c   calculate 'page' to 'axes' transformation
c
        err = max( 0.0, varyh - covarh **2 / varxh ) +
     1        max( 0.0, varxv - covarv **2 / varyv )
        ok = .true.
        if ( err .lt. 2.0 * errthr ) then                                 2b
          a = ( ( mi -1 ) * covarh - ( mj - 1 ) * covarv ) /
     1        ( ( mi - 1 ) * varxh + ( mj - 1 ) * varyv )
          cx = meanyh - a * meanxh
          cy = meanxv + a * meanyv
          d = 1 + a **2
          x0 = ( cy - a * cx ) / d
          y0 = ( cx + a * cy ) / d
          phi = atan( a ) * 180.0 / Hpi
c
c       PLOT AXES
c               left hand end of x axis
c
          if ( cx .lt. 0.0 ) then                                         3b
            call MOVE( -cx / a, 0.0 )
          else if ( cx .gt. h ) then                                      3m
            call MOVE( ( h - cx ) / a, h )
          else                                                            3m
            call MOVE( 0.0, cx )
            end if                                                        3e
c
c               right hand end of x axis
c
          t = w * a + cx
          if ( t .lt. 0.0 ) then                                          3b
            call DRAW( -cx / a, 0.0 )
          else if ( t .gt. h ) then                                       3m
            call DRAW( ( h - cx ) / a, h )
          else                                                            3m
            call DRAW( w, t )
            end if                                                        3e
c
c               bottom end of y axis
c
          if ( cy .lt. 0.0 ) then                                         3b
            call MOVE( 0.0, cy / a )
          else if ( cy .gt. w ) then                                      3m
            call MOVE( w, ( cy - w ) / a )
          else                                                            3m
            call MOVE( cy, 0.0 )
            end if                                                        3e
c
c               top end of y axis
c
          t = cy - h * a
          if ( t .lt. 0.0 ) then                                          3b
            call DRAW( 0.0, cy / a )
          else if ( t .gt. w ) then                                       3m
            call DRAW( w, ( cy - w ) / a )
          else                                                            3m
            call DRAW( t, h )
            end if                                                        3e
c
          if ( scaled ) then                                              3b
c                    X AXIS
c    convert data to 'axis' coords , bring it onto the x axis and then
c    calculate means for 'axes' to 'graph' transformation
c
            call SINCOS( -phi )
            do 2 i = 1, mi                                                4b
              xi(i) = xi(i) - x0
              yi(i) = yi(i) - y0
              call ROTATE( xi(i), yi(i) )
2             continue                                                    4e
            call SMEANS( xi, zi, 1, mi, meanxh, meanzh, varxh, varzh,
     1        covarh )
c
c                    Y AXIS
c    convert data to 'axis' coords , bring it onto the y axis and then
c    calculate means for 'axes' to 'graph' transformation
c
            do 3 i = 1, mj                                                4b
              xj(i) = xj(i) - x0
              yj(i) = yj(i) - y0
              call ROTATE( xj(i), yj(i) )
3             continue                                                    4e
            call SMEANS( yj, zj, 1, mj, meanyv, meanzv, varyv, varzv,
     1        covarv )
c
c        calculate the 'axes' to 'graph' transformation
c
            ax = covarh / varzh
            bx = meanxh - ax * meanzh
            errx = varxh - covarh **2 / varzh
            ay = covarv / varzv
            by = meanyv - ay * meanzv
            erry = varyv - covarv **2 / varzv
            ok = errx .lt. errthr / 2.0 .and. erry .lt. errthr / 2.0
c
c       if axes are unacceptable; initialize graphics & plot the page
c
            if ( .not. ok ) then                                          4b
              call DTMSSG( 'Large error in scaling of axes' )
              call DTPGRF
              end if                                                      4e
            end if                                                        3e
        else                                                              2m
          call DTMSSG( 'Error in drawing of axes' )
          call DTPGRF
          end if                                                          2e
        if ( err .ge. 2.0 * errthr .or. .not. ok ) goto 1                 1e
c       until ( err .lt. 2.0 * errthr .and. ok )
      if ( dbug )
     1  call DTWAIT
      return
      end


      SUBROUTINE SAXDAT( X, Y, Z, M, SCLAW )
c
c  Subroutine to get the data required for defining the position of an axis
c on the page and, if necessary, scaling that axis
c
c Arguments
c  real x(Hnaxpts), y(Hnaxpts), # set of (x,y) points defining the axis
c       z(Hnaxpts),             # scale values on the axis defined by (x,y)
c       qresp                   # function; receives & checks real input
c  integer *2 m,                # number of points on line
c             sclaw             # =1 for linear, =2 for log scaling
c Variable
c  integer *2 i,                # working index
c  logical *1 equery,           # function : logical input
c             digzpt            #          : to get single point from digitizer
c  subroutine transf            # transform from low to high coord system
c
c
      implicit none
      include 'digtab.cmn'
      integer *2 m, i, sclaw
      logical *1 DIGZPT, equery
      real x(Hnaxpts), y(Hnaxpts), z(Hnaxpts), qresp
c
      m = Hnaxpts
c     repeat
c       repeat
1         call DTMSSG( 'Define the axis :- point mode.' )
          do 2 i = 1, Hnaxpts
            if ( .not. DIGZPT( x(i), y(i), 2, 10 ) ) then
              m = i - 1
              goto 3
            else
              x(i) = x(i)-xll
              y(i) = y(i)-yll
              call SINCOS( -theta )
              call ROTATE( x(i), y(i) )
              if ( scaled ) then
                call DTMSSG( 'Scaled value ? ' )
                z(i) = QRESP( Hkbdlu, -1.0e37, 1.0e37, Hnotries )
                call DTMSSG( ' ' )
                if ( sclaw .eq. 2 )
     1            z(i) = log( z(i) )
                end if
              end if
2           continue
3         if ( m .lt. 3 ) then
            call DTMSSG( 'You must specify at least 3 points on ' //
     1        'an axis. Start again. Press <return> to continue.' )
            goto 1
            end if
c         until ( no. of points > 3 )
        call DTMSSG( 'Verify axis, OK ?' )
        if ( .not. EQUERY( Hkbdlu, ' ' ) ) goto 1
c       until ( axis is OK )
      return
      end


      SUBROUTINE DTMSSG( CSTR )
c
c       Put a message on the alpha-numeric or graphics screen
c
      implicit none
      integer *2 istart
      character Cstr*(*), Cblank*90
      include 'digtab.cmn'
      data Cblank / ' ' /
      save Cblank
c
      if ( graphic ) then
        call HG_puts( Cblank, HLastRow, 0 )
        if ( Cstr .ne. ' ' ) then
          istart = max( 1, ( HLastCol - len( Cstr ) ) / 2 )
          call HG_puts( Cstr, HLastRow, istart )
          end if
      else
        write( Hvdulu, 1 ) Charnb( Cstr )
1       format( 1x, a )
        end if
      return
      end


      SUBROUTINE DIGZLN( X, Y, M, N, MODE )
c
c  Subroutine to get a series of data points ( up to N points ) from the
c digitizing tablet .
c
c Arguments
c  real x(n),y(n)       # (x,y) coordinate array
c  integer m,           # number of data points in line
c          n,           # size of coordinate array
c          mode         # required tablet mode
c
c Variables
c  real x1,y1           # surplus coordinate for array overflows
c  logical *1 equery,   # function for prompting and receiving logical input
c             digzpt    # function to get single data point from HIPAD tablet
c  subroutine transf    # transform from low to high coord system
c
      implicit none
      logical *1 equery, digzpt
      integer *2 i, m, n, mode
      real x(n), y(n), x1, y1
      include 'digtab.cmn'
c
      lingrf = .false.
      m = n
      if ( .not. graphic .and. .not. dbug ) then
        call vecmod
        graphic = .true.
        end if
c     repeat
1       call DTMSSG( 'Digitize line' )
        do 2 i = 1, n
          if ( .not. DIGZPT( x(i), y(i), mode , 2 * Hnotries ) ) then
            m = i -1
            goto 3
            end if
          lingrf = .true.
          call TRANSF( x(i), y(i) )
2         continue
3       if ( i .gt. n ) then
          call DTMSSG( ' DIGZLN : Too many data points .' )
4         if ( DIGZPT( x1, y1, mode, 2 * Hnotries ) )
     1      goto 4
          end if
        call DTMSSG( 'Verify line, OK ?' )
        if ( .not. EQUERY( Hkbdlu, ' ' ) ) then
          lingrf = .true.
          call invis
          do 5 i = 1, m
            call transr( x(i), y(i), lingrf )
            lingrf = .false.
5           continue
          call visibl
          goto 1
          end if
c     until( EQUERY( 'Verify line, OK ?' ) )
      lingrf = .false.
      if ( dbug )
     1  call DTWAIT
      return
      end


      LOGICAL *1 FUNCTION DIGZPT( X, Y, MODE, NOTRY )
c
c   Function to get a single data point (X,Y) from the digitizing tablet .
c The result is in the tablet coordinate system .
c   DIGZPT = .true. if the point is OK and is not the end of line marker
c            .false. if the acquisition is unsuccessful or the point is an
c                       end of line marker
c Argument
c  real x, y            # (x,y) coordinate
c  integer *2 mode      # required tablet mode
c             notry     # max no. of attempts at entering a point
c
c Variables
c  real xo,yo           # plotted points
c  integer *2 i         # attempt counter
c             button    # identifies the cursor button used
c  logical *1 equery    # function : prompting and receiving logical input
c             valid     #          : test if a point is on the page
c
      implicit none
      real x, y, xo, yo
      integer *2 i, mode, button, notry
      logical *1 equery, valid
      include 'digtab.cmn'
c
      digzpt = .false.
      if ( .not. graphic .and. .not. dbug ) then
        call vecmod
        graphic = .true.
        end if
      call SINCOS( -theta )
      do 1 i = 1, notry
        call HIPAD( x, y, mode, button, .false. )
        if ( VALID( x, y ) ) then
          digzpt = .true.
          xo = x - xll
          yo = y - yll
          call ROTATE( xo, yo )
          if ( lingrf ) then
            call DRAW( xo, yo )
          else
            call MOVE( xo, yo )
            call DRAW( xo, yo )
            end if
          return
          end if
        call DTMSSG( 'End of line ?' )
        if ( EQUERY( Hkbdlu, ' ' ) ) then
          call DTMSSG( ' ' )
          x = -9.999e29
          y = x
          return
          end if
        call DTMSSG( 'WARNING : Point off page . Re-enter' )
1       continue
      return
      end



      SUBROUTINE TRANSF( X, Y )
c
c  Subroutine to convert (x,y) from 'tablet' coordinate system
c                                to 'page'   coordinate system
c                                to 'axis'   coordinate system
c                                to 'graph'  coordinate system
c Argument
c  real x, y,           # coordinate
c
      implicit none
      real x, y
      include 'digtab.cmn'
c
      x = x - xll
      y = y - yll
      call SINCOS( -theta )
      call ROTATE( x, y )
      x = x - x0
      y = y - y0
      call SINCOS( -phi )
      call ROTATE( x, y )
      x = ( x - bx ) / ax
      y = ( y - by ) / ay
      if ( slx .eq. 2 )
     1  x = exp( x )
      if ( sly .eq. 2 )
     1  y = exp( y )
      return
      end


      SUBROUTINE TRANSR( X, Y, FIRST )
c
c  Subroutine to convert (x,y) from 'graph'  coordinate system
c                                to 'axis'   coordinate system
c                                to 'page'   coordinate system
c                                to 'tablet' coordinate system
c
c Arguments
c  real x, y,           # coordinate
c  logical *1 first     # first point in a line
c
      implicit none
      logical *1 first
      real x, y
      include 'digtab.cmn'
c
      if ( slx .eq. 2 )
     1  x = alog( x )
      if ( sly .eq. 2 )
     1  y = alog( y )
      x = ax * x + bx
      y = ay * y + by
      call SINCOS( phi )
      call ROTATE( x, y )
      x = x + x0
      y = y + y0
      if ( .not. graphic .and. .not. dbug ) then
        call vecmod
        graphic = .true.
        end if
      if ( first ) then
        call MOVE( x, y )
      else
        call DRAW( x, y )
        end if
      call SINCOS( theta )
      call ROTATE( x, y )
      x = x + xll
      y = y + yll
      if ( dbug )
     1  call DTWAIT
      return
      end


      LOGICAL *1 FUNCTION VALID( X, Y )
c
c  Function to test if the coordinate (x,y) [ expressed in the 'tablet
c coordinate' system ] is on the defined page .
c
c Arguments
c  real x, y             # coordinate
c Variables
c  real x1,y1            # coordinates in 'paper coordinates'
c
c subroutine rotate     # rotation linear transformation
c
      implicit none
      real x, y, x1, y1
      include 'digtab.cmn'
c
      x1 = x - xll
      y1 = y - yll
      call ROTATE( x1, y1 )
      valid = 0 .le. x1 .and. x1 .le. w .and. 0 .le. y1 .and. y1 .le. h
      return
      end
