c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM MOMENT_STATISTICS
c
c Calculate statistics of columns of data in a file.  You might
c use a spreadsheet program for this, but some data files can
c be awfully long ... and there is no spreadsheet program on this VAX.
c
c constants
c   Hkbdlu=5        # keyboard logical unit no.
c   Hvdulu=6        # screen logical unit no.
c   Hmax_data=1000  # available data space
c variables
c  integer
c   column          # data column number
c   col_index       # column index into data array
c   line_cols       # number of columns of data to be written to file
c   max_moment      # maximum moment to be calculated
c   moment          # moment counter
c   m_index         # index into array of moment results
c   ip_lu           # input file logical unit number
c   op_lu           # output file logical unit number
c   n_args          # number of command line arguments
c   n_rows          # number of data rows
c   n_columns       # number of data columns
c   iargcs          # function : number of command line arguments
c   open_file       # function : open file
c   new_lun         # function : get new logical unit number
c   jresp           # function : read integer
c  real
c   array(Hmax_data)# data array
c   means(Hmax_mom) # array for statistical moments
c   denomin         # denominator for calculating higher moments
c  character
c   Cip_file*64     # name of input file
c   Cop_file*64     # name of output file
c   Cmoment(8)*8    # list of moment names
c
      implicit none
      integer Hkbdlu, Hvdulu, Hmax_data, Hmax_mom
      parameter( Hkbdlu=5, Hvdulu=6, Hmax_data=1000, Hmax_mom=200 )
c
      integer column, col_index, iargc, ip_lu, line_cols,
     1        max_moment, moment, m_index, n_args, n_rows, n_columns,
     1        op_lu, open_file, new_lun, Jresp
      real array(Hmax_data), means(Hmax_mom), denomin
      character Cip_file*64, Cop_file*64, Cmoment(8)*8
      data Cmoment / 'mean', 'r.m.s.', 'skewness', 'flatness',
     1    'sup skew', 'sup flat', 'hyp skew', 'hyp flat' /
c
c           complain about command line arguments if there are fewer than 2
c
      n_args = iargc( ',' )
      if ( n_args .lt. 2 ) then
          write( Hvdulu, 1 )
1         format( ' usage' /
     1            ' MEANS <max_moment> <input_file> [<output_file>]' )
          end if
c
c           determine maximum moment order
c
      if ( n_args .lt. 1 ) then
          write( Hvdulu, 2 )
2         format( ' Maximum moment = ? ', $ )
          max_moment = Jresp( Hkbdlu, 1, 8, 4 )
          Cip_file = '?'
      else
          call getarg( 1, Cip_file, ',' )
          read( Cip_file, *, ERR=5 ) max_moment
          if ( max_moment .ge. 1 .and. max_moment .le. 8 ) goto 5
          write( Hvdulu, 3 ) max_moment
3         format( ' MEANS : Illegal maximum moment ', i12 )
4         write( Hvdulu, 2 )
          max_moment = Jresp( Hkbdlu, 1, 8, 4 )
5         end if
c
c           open input file -- get name from command line if possible
c
      if ( n_args .lt. 2 ) then
          Cip_file = '?'
      else
          call getarg( 2, Cip_file, ',' )
          end if
      ip_lu = open_file( new_lun(), Cip_file, 'old', 0 )
      if ( ip_lu .le. 0 ) then
          write( Hvdulu, 6 ) Cip_file
6         format( ' MEANS : Cannot open', a )
          stop
          end if
c
c           open output file -- get name from command line if possible
c
      if ( n_args .lt. 3 ) then
          Cop_file = ' '
      else
          call getarg( 3, Cop_file, ',' )
          end if
      op_lu = open_file( new_lun(), Cop_file, 'new', 0 )
      if ( op_lu .le. 0 ) then
          write( Hvdulu, 6 ) Cop_file
          stop
          end if
c
c           read the data file
c
      call read_data( ip_lu, array, Hmax_data, n_rows, n_columns )
c
c           calculate the means and moments
c
      col_index = 1
      m_index = 1
      do 9 column = 1, n_columns
          call smeang( array(column), n_columns, n_rows,
     1                 means(m_index), max_moment )
c           convert mean-square to rms
          if ( max_moment .gt. 1 ) then
              means( m_index + 1 ) =
     1            sqrt( means( m_index + 1 ) )
              end if
c           normalize higher order moments with rms
          if ( max_moment .gt. 2 ) then
              denomin = means( m_index + 1 ) ** 2
              do 8 moment = 2, max_moment - 1
                  denomin = denomin * means( m_index + 1 )
                  means( m_index + moment ) = means( m_index + moment )
     1                / denomin
8                 continue
              end if
          col_index = col_index + n_columns
          m_index = m_index + max_moment
9         continue
c
c           write results
c
      if ( op_lu .eq. 6 ) then
          line_cols = 5
      else
          line_cols = 8
          end if
      do 13 column = 0, max_moment * ( n_columns - 1 ),
     1              line_cols * max_moment
          do 11 moment = 1, max_moment
              write( op_lu, 10 ) Cmoment(moment),
     1            ( means(m_index), m_index = moment + column, moment
     2            + min( column + ( line_cols - 1 ) * max_moment,
     3            ( n_columns - 1 ) * max_moment ), max_moment )
10            format( 1x, a8, 1x, 8g14.6 )
11            continue
          write( op_lu, 12 )
12        format()
13        continue
      stop
      end

