c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

      PROGRAM CALBAR
c
c       CALibrate BARocel or barotron
c Program to examine a raw data file for '#calbar' modules , and compile a
c sequential access calibration file containing the information required to
c convert the A/D readings into pressure differences .
c
c Local Variables
c  integer
c    item           # selected item number
c    len_cf_name    # length of calibration file name
c    len_df_name    # length of data file name
c    iargc          # function : number of program arguments
c    jljstr         #          : left justify string
c    Jresp          #          : integer response
c  character
c    Calib_file*64  # name of calibration file
c    Cdata_file*64  # name of data file
c  subroutine
c    average_calibrations # average the calibrations in a calibration file
c    calib_param_menu     # menu for calibration parameters
c    parse_file_structure # formulate calibrations from data file
c    setup_menu           # menu for setup parameters
c    calc_linearisation   # calculate linearisation relation
c  library routines
c    Cresp          # character string response
c    getarg         # get program argument
c    ilgwait        # ilglib
c    newscr         # hpprev
c
      include 'stdio.def/list'
      include 'calbar.def/list'
      integer item, len_cf_name, len_df_name, iargc, jljstr, Jresp
      character Calib_file*64, Cdata_file*64
      data Calib_file, Cdata_file / ' ', ' ' /
      data len_cf_name, len_df_name / 1, 1 /
c
c
c               Print Label
c
      write( STD_OUT, 1 )
1     format( 21x, 'CALBAR : Baratron pressure sensor calibrator' /
     1        21x, 43( '=' ) / )
c
c
c         Get program arguments -- if any
c             first argument  = data file name
c             second argument = calibration file name
c
      if ( iargc() .gt. 0 ) then                                          1b
          call getarg( 1, Cdata_file )
          if ( iargc() .gt. 1 )
     1        call getarg( 2, Calib_file )
          end if                                                          1e
      len_df_name = max( jljstr( Cdata_file ), 1 )
      len_cf_name = max( jljstr( Calib_file ), 1 )
c
c     start of loop                                                       1b
2         call newscr()
          write( STD_OUT, 3 ) Cdata_file(1:len_df_name),
     1        Calib_file(1:len_cf_name)
3         format( ' Main menu' / '  0) Exit' /
     1        '  1) Name of data file', 12x, a /
     2        '  2) Name of calibration file     ', a /
     3        '  3) Setup parameters menu' /
     4        '  4) Calibration parameters menu' /
     5        '  5) Do calibrations' /
     6        '  6) Average calibrations' /
     7        '  7) Calculate linearisation relation ', $ )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c           EXIT
c           close files first
c
          if ( item .eq. 0 ) then                                         2b
              stop
c
c           NAME OF DATA FILE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Enter name of data file ', $ )
              call Cresp( STD_IN, Cdata_file, MAX_TRIES )
              len_df_name = max( jljstr( Cdata_file ), 1 )
c
c           NAME OF CALIBRATION FILE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Enter name of calibration file ', $ )
              call Cresp( STD_IN, Calib_file, MAX_TRIES )
              len_cf_name = max( jljstr( Calib_file ), 1 )
c
c           SETUP PARAMETERS MENU
c
          else if ( item .eq. 3 ) then                                    2m
              call setup_menu()
c
c           CALIBRATION PARAMETERS MENU
c
          else if ( item .eq. 4 ) then                                    2m
              call calib_param_menu()
c
c           DO CALIBRATIONS
c
          else if ( item .eq. 5 ) then                                    2m
              if ( Cdata_file .ne. ' ' ) then                             3m
                  call parse_file_structure( Cdata_file, Calib_file )
              else                                                        3m
                  write( STD_OUT, 6 )
6                 format( ' CALBAR : Must specify data file' )
                  call ilgwait()
                  end if                                                  3e
c
c           AVERAGE CALIBRATIONS
c
          else if ( item .eq. 6 ) then                                    2m
              if( Calib_file .ne. ' ' ) then                              3b
                  call average_calibrations( Calib_file )
              else                                                        3m
                  write( STD_OUT, 7 )
7                 format( ' CALBAR : Must specify calibration file' )
                  call ilgwait()
                  end if                                                  3e
c
c           CALCULATE LINEARISATION RELATION
c
          else if ( item .eq. 7 ) then                                    2m
              call calc_linearisation()
              end if                                                      2e
          goto 2
c         end of loop                                                     1e
      end




      SUBROUTINE SETUP_MENU()
c
c       Menu for setup parameters
c Setup parameters menu
c   0) exit
c   1) verbosity                        1
c   2) set host machine                 IBM P/C
c   3) auxillary file name              calbar.aux
c   4) type of pressure measurement     pre-centrebody dynamic
c   5) input reference pressure units   Torr
c   6) output reference pressure units  Pa
c   7) wind tunnel plug number          3
c   8) linearisation file name          [planspear.com]press_trans.lin
c
c local variables
c  integer
c   aux_len     # length of auxillary file name
c   item        # selected menu item
c   ip_units    # specifies the input pressure units
c   op_units    # specifies the output pressure units
c   jljstr      # function : left justify string
c   Jresp       #          : integer response
c   new_lun     #          : new logical unit number
c   open_file   #          : open file
c  real
c   Pcvt_list(8)  # conversion factors for units into Pascals
c  character
c   Caux_file*32  # name of auxillary file
c   Chost(0:2)*8  # host machine description
c   Cold_aux*32   # previous name of auxillary file
c   Cpunits(8)*4  # available units of pressure
c  subroutine
c   Cresp         # read character string
c
c global variables
c  aux_lu, Ctype, host_type, plug_no, press_conv, verbosity
c
      include 'stdio.def'
      include 'calbar.def'
c
      integer aux_len, item, ip_units, op_units,
     1    jljstr, Jresp, new_lun, open_file
      real Pcvt_list(8)
      character Caux_file*32, Chost(0:2)*8, Cold_aux*32, Cpunits(8)*4
c
      include 'calbar.cmn/list'
      save aux_len, ip_units, op_units,
     1     Caux_file, Chost, Cold_aux, Cpunits
      data aux_len, ip_units, op_units / 1, 5, 1 /
      data Caux_file, Cold_aux, Chost, Cpunits / ' ', ' ',
     1  'VAX-VMS ', 'PDP11/34', 'IBM XT  ',
     2  'Pa  ', 'kPa ', 'mBar', 'psi ', 'Torr', 'inHg', 'mm W', 'in W' /
      data Pcvt_list / 1.0, 1000.0, 100.0, 6894.76, 133.3224, 338689.0,
     1  9.804895, 249.0443 /
c
c
c     start of loop
1         call newscr()
          write( STD_OUT, 2 ) verbosity, Chost(host_type),
     1        Caux_file(1:aux_len), Ctype, Cpunits(ip_units),
     2        Cpunits(op_units), plug_no
2         format( ' Setup parameters menu' / '   0) exit' /
     1        '   1) verbosity                        ', i2 /
     2        '   2) set host machine                 ', a /
     3        '   3) auxillary file name              ', a /
     4        '   4) type of pressure measurement     ', a /
     5        '   5) input reference pressure units   ', a /
     6        '   6) output reference pressure units  ', a /
     7        '   7) wind tunnel plug number          ', i3, '  ', $ )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c             EXIT
c
          if ( item .le. 0 ) then                                         2b
              if ( aux_lu .ne. 0 .and. Caux_file .ne. Cold_aux ) then     3b
                  close( UNIT=aux_lu )
                  aux_lu = 0
                  end if                                                  3e
              if ( Caux_file .ne. ' ' .and. aux_lu .eq. 0 )
     1            aux_lu = open_file( new_lun(), Caux_file, 'new', 0 )
              if ( aux_lu .ne. 0 )
     1            Cold_aux = Caux_file
              press_conv = Pcvt_list(ip_units) / Pcvt_list(op_units)
              return
          else if ( item .eq. 1 ) then                                    2m
c
c             VERBOSITY
c
              write( STD_OUT, 3 )
3             format( ' Enter verbosity (0-2) ', $ )
              verbosity = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c             HOST MACHINE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' What is the host machine ? 0) VAX - VMS' /
     1            28x, '1) PDP11/34 - unix' /
     2            28x, '2) IBM XT/AT - MS-DOS' )
              host_type = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c             AUXILLARY FILE NAME
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Name of auxillary file ?', $ )
              call Cresp( STD_IN, Caux_file, MAX_TRIES )
              aux_len = max( 1, jljstr( Caux_file ) )
c
c
c             TYPE OF PRESSURE MEASUREMENT
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Enter type of pressure measurement ', $ )
              call Cresp( STD_IN, Ctype, MAX_TRIES )
c
c             INPUT PRESSURE UNITS
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 7 ) 'input'
7             format( ' Enter the units of ', a, ' reference pressures'/
     1            t10, '1) Pascals',     t30, '5) Torr' /
     2            t10, '2) KiloPascals', t30, '6) Inches of Mercury' /
     3            t10, '3) Millibars ',  t30, '7) mm of Water' /
     4            t10, '4) Psi',         t30, '8) Inches of Water ', $ )
              ip_units = Jresp( STD_IN, 1, 8, MAX_TRIES )
c
c             OUTPUT REFERENCE PRESSURE UNITS
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 7 ) 'output'
              op_units = Jresp( STD_IN, 1, 8, MAX_TRIES )
c
c             WIND TUNNEL PLUG NUMBER
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 8 )
8             format( ' Enter the wind tunnel plug number ', $ )
              plug_no = Jresp( STD_IN, -5, 15, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE CALIB_PARAM_MENU()
c
c Calibration parameters menu
c 0) Exit
c 1) Linearisation file name
c 2) Correct non-linearity
c 3) Order of calibration polynomial
c 4) Process all modules in the data file
c
c local variables
c  integer
c   item        # selected menu item
c   linear_len  # length of linearisation file name
c   jljstr      # function : left justify string
c   Jresp       #          : read integer
c  character
c   Cyes        # function : 'no', 'yes', 'query'
c  subroutine
c   cresp       # read string
c
c  global variables
c   all_modules, calib_order, lin_correct, linear_file
c
c
      include 'stdio.def'
      include 'calbar.def'
c
      integer item, linear_len, jljstr, Jresp
      character Cyes*5
      include 'calbar.cmn'
c
c
c     start of loop
1         call newscr()
          linear_len = max( 1, jljstr( linear_file ) )
          write( STD_OUT, 2 ) linear_file(:linear_len),
     1        Cyes(lin_correct), calib_order, Cyes(all_modules)
2         format( ' Calibration parameters menu' / '   0) Exit' /
     1        '   1) Linearisation file name          ', a /
     2        '   2) Default linearisation file name  ' /
     3        '   3) Correct non-linearity            ', a /
     4        '   4) Order of calibration polynomial  ', i2 /
     5        '   5) Process all modules in data file ', a, '  ', $ )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         1b
              return
c
c       LINEARISATION FILE NAME
c
          else if ( item .eq. 1 ) then                                    1m
              write( STD_OUT, 3 )
3             format( ' What value ', $ )
              call cresp( STD_IN, linear_file, MAX_TRIES )
c
c       DEFAULT LINEARISATION FILE NAME
c
          else if ( item .eq. 2 ) then                                    1m
              linear_file = DEF_LINEAR_FILE
c
c       CORRECT NON-LINEARITY
c
          else if ( item .eq. 3 ) then                                    1m
              lin_correct = mod( lin_correct + 1, 3 )
c
c       ORDER OF CALIBRATION POLYNOMIAL
c
          else if ( item .eq. 4 ) then                                    1m
              write( STD_OUT, 3 )
              calib_order = Jresp( STD_IN, 1, MAX_POLY-1, MAX_TRIES )
c
c       PROCESS ALL MODULES IN DATA FILE
c
          else if ( item .eq. 5 ) then                                    1m
              all_modules = mod( all_modules + 1, 2 )
              end if                                                      1e
          goto 1
c         end of loop
      end



      SUBROUTINE CALC_LINEARISATION()
c
c Calculate a linearising relation for linearising the output of a pressure
c transducer . The linearising relation is a "least squared error" polynomial .
c
c constants
c  MAX_EXPT=256          # maximum number of data points
c  MAX_FITT=MAX_EXPT/2
c  TRANSDUCER=0          # start of transducer index in experimental data array
c  REFERENCE=MAX_FITT    # start of reference index in experimental data array
c
c
c local variables
c  integer
c   capt_len      # length of capture file name
c   dest, src     # destination and source array indices
c   expt_len      # length of experimental data file name
c   expt_lu       # logical unit number of experimental data file
c   i             # array index
c   increm        # number of columns in experimental data file
c   item          # selected menu item
c   n_expt        # number of data points
c   res_expt      # reserved logical unit number of experimental data file
c   jljstr        # function : left justify string
c   Jresp         #          : read integer
c   io_linearisation         : read/write linearisation file
c   new_lun       #          : get new logical unit number
c   open_file     #          : open file
c  real
c   fitting_err(MAX_FITT)     # difference between reference data & polynomial
c   expt_data(MAX_EXPT)       # experimental data
c   qpoly                     # function : calculate polynomial value
c  double precision
c   work(MAX_WORK)            # work array for spyorth
c  character
c   capt_file*32  # capture file name
c   expt_file*32  # name of experimental data file
c   Cdone*8      # function : 'not done', 'done'
c  subroutine
c   cresp         # read string
c   ilgclos, ilgdata, ilgline, ilgopen, ilgwait   # ilglib : plotting
c   newscr        # hpprev : clear screen
c   read_data     # read data table
c   spyorths      # calculate best fit polynomial
c
c
c global variables
c  aux_lu, linear_file, lin_maxim, lin_minim, lin_offset,
c  lin_order, lin_poly, lin_slope, res_linear
c
c
      include 'stdio.def'
      include 'calbar.def'
      integer MAX_EXPT, MAX_FITT, TRANSDUCER, REFERENCE
      parameter( MAX_EXPT=256, MAX_FITT=MAX_EXPT/2,
     1    TRANSDUCER=0, REFERENCE=MAX_FITT )
c
      integer capt_len, dest, src, expt_len, expt_lu, i, increm, item,
     1    n_expt, res_expt,
     2    io_linearisation, jljstr, Jresp, new_lun, open_file
      real fitting_err(MAX_FITT), expt_data(MAX_EXPT), qpoly
      double precision work(MAX_WORK)
      character capt_file*32, expt_file*32, Cdone*8
c
      include 'calbar.cmn'
      save capt_len, expt_len, expt_lu, n_expt, res_expt, capt_file
      data capt_len, expt_len, expt_lu, n_expt, res_expt /
     1     1, 1, 0, 0, NO /
      data capt_file / ' ' /
c
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) expt_file(:expt_len), n_expt,
     1        lin_order, Cdone(got_linear), capt_file(:capt_len)
2         format( ' Pressure sensor linearisation' / '   0) exit' /
     1        '   1) Experimental data file   ', a /
     2        '   2) Read experimental data  ', i4, ' points' /
     3        '   3) Edit experimental data' /
     4        '   4) Polynomial order         ', i2 /
     5        '   5) Calculate linearising polynomial ', a /
     6        '   6) Set zero (0 in -> 0 out)' /
     7        '   7) Display polynomial' /
     8        '   8) Display data & errors' /
     9        '   9) Capture file name        ', a /
     a        '  10) Plot error' /
     b        '  11) Write linearisation file' /
     c        '  12) Read linearisation file ', $ )
          item = Jresp( STD_IN, 0, 12, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           EXPERIMENTAL DATA FILE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ', $ )
              call cresp( STD_IN, expt_file, MAX_TRIES )
              expt_len = max( 1, jljstr( expt_file ) )
              n_expt   = 0
              got_linear = NO
c
c           READ EXPERIMENTAL DATA
c
          else if ( item .eq. 2 ) then                                    2m
              if ( res_expt .eq. 0 )
     1            res_expt = new_lun()
              expt_lu = open_file( res_expt, expt_file, 'old', 0 )
              if ( expt_lu .ne. 0 ) then                                  3b
                  call read_data( expt_lu, expt_data, MAX_EXPT, n_expt,
     1                increm )
                  close( UNIT=expt_lu )
                  expt_lu = 0
                  if ( n_expt .ge. 4 ) then                               4b
c
c           At this stage, the experimental data is ordered just as it was read
c           from file . When arranged in an array of `increm' rows and `n_expt'
c           columns, the first row contains transducer data and the second row
c           contains reference data . For processing, the first 2 rows of this
c           array must be transposed . The first step is to transfer the second
c           row to a temporary array (`fitting_error') .
c
                      src = 2
                      do 4 dest = 1, n_expt                               5b
                          fitting_err(dest) = expt_data(src)
                          src = src + increm
4                         continue                                        5e
c
c           Now copy the first row into the first column .
c
                      src = 1 + increm
                      do 5 dest = 2, n_expt                               5b
                          expt_data(dest) = expt_data(src) * press_conv
                          src = src + increm
5                         continue                                        5e
c
c           Copy the contents of the temporary array into the second column .
c           Also find the minimum and maximum transducer readings .
c
                      lin_minim = expt_data(src)
                      lin_maxim = expt_data(src)
                      do 6 dest = 1, n_expt                               5b
                          lin_minim = min( lin_minim, expt_data(dest) )
                          lin_maxim = max( lin_maxim, expt_data(dest) )
                          expt_data(dest+REFERENCE) = fitting_err(dest)
     1                       * press_conv
6                         continue                                        5e
                      got_linear = NO
                  else                                                    4m
                      write( STD_ERR, 7 ) n_expt
7                     format( ' CALABR : ABORT : Insuffient data (',i1,
     1                    ') for linearisation' )
                      call ilgwait()
                      end if                                              4e
              else                                                        3m
                  write( STD_ERR, 8 )
8                 format( ' CALBAR : ABORT : Cannot read ',
     1                'linearisation data' )
                  call ilgwait()
                  end if                                                  3e
c
c           EDIT EXPERIMENTAL DATA
c
          else if ( item .eq. 3 ) then                                    2m
              if ( n_expt .gt. 1 ) then                                   3b
                  call ilgopen( ' ' )
                  call ilgxlab( 'Transducer' )
                  call ilgylab( 'Reference' )
                  call ilgtitl( 'Editing experimental data' )
                  if ( got_linear .eq. YES ) then                         4b
                      call ilgraph( 1, 1, 0, 0, 1 )
                      call ilgline( 0, 3, 1 )
                      call ilgdata( expt_data(TRANSDUCER+1),
     1                              fitting_err, 1, n_expt, 1 )
                      call ilgnote( '|m(0.0,0.0)|d(10000.0,0.0)' )
                      end if                                              4e
                  call ilgraph( 1, 1, 3, 3, 0 )
                  call ilgline( 0, 5, 1 )
                  call sfudg2( expt_data, 1, 2, MAX_FITT, 2,
     1                         1, n_expt, n_expt )
                  call ilgclos()
                  end if                                                  3e
c
c           POLYNOMIAL ORDER
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              lin_order = Jresp( STD_IN, 1, MAX_POLY-1, MAX_TRIES )
              got_linear = NO
c
c           CALCULATE LINEARISING POLYNOMIAL
c
          else if ( item .eq. 5 ) then                                    2m
              if ( n_expt .gt. 1 ) then                                   3b
c
c           calculate polynomial
                  call spyorths( expt_data(TRANSDUCER+1),
     1                expt_data(REFERENCE+1), n_expt,
     1                lin_poly, lin_order, lin_error, work )
c
c           calculate fitting error
                  do 9 dest = 1, n_expt                                   4b
                      fitting_err(dest) =
     1                    qpoly( expt_data(TRANSDUCER+dest),
     2                           lin_poly, lin_order ) -
     3                    expt_data(REFERENCE+dest)
9                     continue                                            4e
c
c           calculate slope & offset for values outside linearisation range
                  lin_slope = ( qpoly( lin_maxim, lin_poly, lin_order )
     1                - qpoly( lin_minim, lin_poly, lin_order ) ) /
     2                ( lin_maxim - lin_minim )
                  lin_offset = qpoly( lin_minim, lin_poly, lin_order ) -
     1                lin_slope * lin_minim
                  got_linear = YES
              else                                                        3m
                  write( STD_ERR, 10 )
10                format( ' WARNING : No data available' )
                  call ilgwait()
                  end if                                                  3e
c
c           SET ZERO (0 IN -> 0 OUT)
c           This means only that the first polynomial coefficient is
c           subtracted from the reference data and the is set to zero
c
          else if ( item .eq. 6 ) then                                    2m
              do 11 dest = 1, n_expt                                       3b
                  expt_data(REFERENCE+dest) =
     1                expt_data(REFERENCE+dest) - lin_poly(1)
11                continue                                                3e
              lin_offset = lin_offset - lin_poly(1)
              lin_poly(1) = 0.0
c
c           DISPLAY POLYNOMIALS
c
          else if ( item .eq. 7 ) then                                    2m
              if ( got_linear .eq. YES ) then                             3b
                  write( STD_OUT, 12 ) lin_error, lin_minim, lin_maxim,
     1                lin_slope, lin_offset, lin_order,
     2                ( lin_poly(src), src = 1, lin_order + 1 )
12                format( ' RMS fitting error          ', g12.5 /
     1                    ' Minimum transducer reading ', g12.5 /
     2                    ' Maximum transducer reading ', g12.5 /
     3                    ' Slope for extended range   ', g13.6 /
     4                    ' Offset for extended range  ', g13.6 /
     5                    ' Order of fitted polynomial  ', i3 /
     6                    ' Fitted polynomial ' / ( 1x, 5g15.7 ) )
                  if ( aux_lu .gt. 0 )
     1                write( aux_lu, 12 ) lin_error, lin_minim,
     2                    lin_maxim, lin_slope, lin_offset, lin_order,
     3                    ( lin_poly(src), src = 1, lin_order + 1 )
              else                                                        3m
                  write( STD_ERR, 13 )
13                format( ' WARNING : Polynomial not available' )
                  end if                                                  3e
              call ilgwait()
c
c           DISPLAY DATA & ERRORS
c
          else if ( item .eq. 8 ) then                                    2m
              if ( n_expt .le. 1 ) then                                   3b
                  write( STD_ERR, 10 )
              else if ( got_linear .ne. YES ) then                        3m
                  write( STD_ERR, 13 )
              else                                                        3m
                  write( STD_OUT, 14 ) ( src,
     1                expt_data(TRANSDUCER+src),
     2                expt_data(REFERENCE+src),
     3                qpoly( expt_data(TRANSDUCER+src),
     4                       lin_poly, lin_order ),
     5                fitting_err(src), src = 1, n_expt )
14                format( '     transducer reference linearised fit',
     1                'ting-error' / ( 1x, i3, 1x, f9.4, 1x, f9.4, 1x,
     2                f9.4, 1x, g13.6 ) )
                  if ( aux_lu .ne. 0 )
     1                write( STD_OUT, 14 ) ( src,
     2                    expt_data(TRANSDUCER+src),
     3                    expt_data(REFERENCE+src),
     4                    qpoly( expt_data(TRANSDUCER+src),
     5                           lin_poly, lin_order ),
     6                    fitting_err(src), src = 1, n_expt )
                  end if                                                  3e
              call ilgwait()
c
c           CAPTURE FILE NAME
c
          else if ( item .eq. 9 ) then                                    2m
              write( STD_OUT, 3 )
              call cresp( STD_IN, capt_file, MAX_TRIES )
              capt_len = max( 1, jljstr( capt_file ) )
c
c           PLOT ERROR
c
          else if ( item .eq. 10 ) then                                   2m
              if ( n_expt .le. 1 ) then                                   3b
                  write( STD_ERR, 10 )
                  call ilgwait()
              else if ( got_linear .ne. YES ) then                        3m
                  write( STD_ERR, 13 )
                  call ilgwait()
              else                                                        3m
                  call ilgopen( capt_file )
                  call ilgline( 1, 0, 1 )
                  call ilgdata( expt_data(TRANSDUCER+1), fitting_err,
     1                          1, n_expt, 1 )
                  call ilgwait()
                  call ilgclos()
                  end if                                                  3e
c
c           WRITE LINEARISATION FILE
c             ensure that the file is closed, then reopen for writing
c
          else if ( item .eq. 11 ) then                                   2m
              if ( got_linear .ne. YES ) then                             3b
                  write( STD_ERR, 13 )
                  call ilgwait()
              else                                                        3m
                  src = IO_linearisation( LIN_WRITE )
                  end if                                                  3e
c
c           READ LINEARISATION FILE
c
          else if ( item .eq. 12 ) then                                   2m
              got_linear = IO_linearisation( LIN_READ )
              n_expt = 0
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      LOGICAL FUNCTION IO_LINEARISATION( ACTION )
c
c Read (if `action' = LIN_READ) or write (if `action' = LIN_WRITE) the
c linearization file .
c
c Argument
c  integer action   # LIN_READ = read, or LIN_WRITE = write linearisation file
c
c local variables
c  integer
c   i           # polynomial array index
c   linear_lu   # logical unit number of linearisation file (0=closed)
c   res_linear  # reserved value of linear_lu
c   new_lun     # function : get new logical unit number
c   open_file   #          : open file
c
c global variables
c  lin_order,  lin_error, lin_maxim, lin_minim,
c  lin_offset, lin_poly,  lin_slope, linear_file
c
      include 'stdio.def'
      include 'calbar.def'
c
      integer action, i, linear_lu, res_linear, new_lun, open_file
      include 'calbar.cmn'
      save linear_lu, res_linear
      data linear_lu, res_linear / 0, 0 /
c
      if ( res_linear .eq. 0 )
     1    res_linear = new_lun()
c
      io_linearisation = NO
      if ( action .eq. LIN_READ ) then                                    1b
          linear_lu = open_file( res_linear, linear_file, 'old', -1 )
          if ( linear_lu .ne. 0 ) then                                    2b
              read( linear_lu, ERR=2 ) lin_error, lin_minim, lin_maxim,
     1            lin_slope, lin_offset, lin_order,
     2            ( lin_poly(i), i = 1, lin_order + 1 )
              close( UNIT=linear_lu )
              io_linearisation = YES
              end if                                                      2e
      else if ( action .eq. LIN_WRITE ) then                              1m
          linear_lu = open_file( res_linear, linear_file, 'unknown', -1)
          if ( linear_lu .ne. 0 ) then                                    2b
              write( linear_lu, ERR=2 ) lin_error, lin_minim, lin_maxim,
     1            lin_slope, lin_offset, lin_order,
     2            ( lin_poly(i), i= 1, lin_order + 1 )
              close( UNIT=linear_lu )
              io_linearisation = YES
              end if                                                      2e
      else
          write( STD_ERR, 1 ) action
1         format( ' IO_LINEARISATION : ABORT : Illegal action ', i11 )
          call ilgwait()
          end if                                                          1e
      return
c
2     write( STD_ERR, 3 )
3     format( ' IO_LINEARISATION : ABORT : Cannot read or write file' )
      call ilgwait()
      return
      end



      LOGICAL FUNCTION CHECK_STRUCT( CSUBLAB, N_SAMP )
c
c Checks the structure of the data file and returns a value of .TRUE. if it is
c acceptable or .FALSE. if it is not .
c
c Arguments
c  character Csublab*(*)  # sub-block label
c  integer n_samp         # number of data samples (for data sub-blocks)
c
c local variables
c  logical
c   eextkbdv      # function : is keyboard reference value available
c  integer
c   Jextcyln()    # function : number of A/D channels used
c  character
c   Gextblk()*8   # function : block label
c  subroutine
c   sextall       # display details of sub-block
c
c global variables
c  verbosity
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical eextkbdv
      integer n_samp, Jextcyln
      character Csublab*(*), Gextblk*8
      include 'calbar.cmn'
c
      if ( verbosity .gt. 0 )
     1        call sextall()
c
c             This first test cannot cause failure .
c
      if ( Gextblk() .ne. '##main  ' ) then                               1b
          write( STD_OUT, 1 )
1         format( ' CALBAR : NOTE : Not a ##main data block' )
          end if                                                          1e
c
c             Remaining tests can cause failure
c
      check_struct = .FALSE.
      if ( Csublab .ne. '###1dat1' .and. Csublab .ne. '###2dat2' ) then   1b
          write( STD_OUT, 2 ) 'Non-data sub-block'
2         format( ' CALBAR : WARNING : ', a, ' . Sub-block skipped' )
      else if ( Jextcyln() .gt. 1 ) then                                  1m
          write( STD_OUT, 2 ) 'Too many A/D channels'
      else if ( n_samp .lt. MIN_DATA ) then                               1m
          write( STD_OUT, 2 ) 'Not enough data'
      else if ( n_samp .gt. MAX_DATA ) then                               1m
          write( STD_OUT, 2 ) 'Too much data'
      else if ( .not. eextkbdv() ) then
          write( STD_OUT, 2 ) 'No keyboard reference value'
      else                                                                1m
          check_struct = .TRUE.
          end if                                                          1e
      return
      end



      REAL FUNCTION AD_TO_VOLTS()
c
c Returns the change in A/D converted value per volt .
c
      include 'stdio.def'
      integer Jextadid
c
c       for the DAS 1128
      if ( Jextadid() .eq. 1 ) then
          AD_to_volts = 400.0
c
c       for the DAS 250A
      else
          AD_to_volts = -409.6
          end if
      return
      end



      SUBROUTINE ADD_TO_CALIB_TABLE( RAW_DATA, INDEX, SCALE_FACTOR )
c
c Calculate the mean & rms voltage from the baratron from the raw data array .
c The scale factor SCALE_FACTOR is used to convert the A/D converter readings
c to voltage .
c
c Arguments
c  integer *2 raw_data(*)   # raw data array
c  integer index            # index into the calibration data table
c  real scale_factor        # converts A/D reading into volts
c
c local variables
c  integer
c   Jextnocy    # function : number of A/D cycles
c  real
c   Qextkbdv    # function : extract keyboard reference value
c  subroutine
c   smeani      # mean & mean square of raw data array
c
c
      include 'stdio.def'
      include 'calbar.def'
c
      integer *2 raw_data(*)
      integer index, Jextnocy
      real scale_factor, Qextkbdv
      include 'calbar.cmn'
c
      call smeani( raw_data, 1, Jextnocy(), calib(index,BAR_MEAN),
     1    calib(index,BAR_RMS) )
      calib(index,BAR_MEAN) = calib(index,BAR_MEAN) / scale_factor
      calib(index,BAR_RMS)  = sqrt( calib(index,BAR_RMS) /
     1    abs( scale_factor ) )
c
      calib(index,BAR_PRESS) = Qextkbdv() * press_conv
      if ( abs( calib(index,BAR_PRESS) ) .le. 1.0e-20 )
     1    calib(index,BAR_PRESS) = 0.0
      return
      end



      SUBROUTINE PARSE_FILE_STRUCTURE( DATA_FILE, CALIB_FILE )
c
c Reads each baratron calibration module in turn and parses the sequence
c of blocks and sub-blocks in each module, and compiles a table of calibration
c data for each calibration .
c Formulation of the calibration relations is performed by PROCESS_DATA  .
c All raw data file handling is confined to PARSE_FILE_STRUCTURE .
c
c Arguments
c  character data_file*(*)    # name of data file
c            calib_file*(*)   # name of calibration file
c
c conatant
c  MAX_DATA=4096    # maximum number of data samples from A/D converter
c
c local variables
c  logical
c   check_struct    # function : tests for data file structure
c   equery          #          : read logical value
c   erdfacq         #          : read data sub-block
c   erdfmhd         # rdflib   : read data module header
c   erdfopn         # rdflib   : open raw data file
c  integer *2
c   Mdata(MAX_DATA) # raw data array
c  integer
c   module_no       # data file module counter
c   n_samp          # number data samples read by erdfacq
c   rdf_rec         # raw data file record number
c   Jextchls        # function : extract A/D channel number
c  real
c   air_press       # air pressure (unused)
c   air_temp        # air temperature (unused)
c   to_volts        # conversion factor for A/D value to voltage
c   AD_to_volts     # function : conversion factor for A/D values to volts
c   qmuair          # function : dynamic viscosity of air
c   qresp           #          : read real
c  character
c   Csublab*8       # sub-block header label
c   Gextadid*12     #          : identify A/D converter
c  subroutine
c    srdfcls        # rdflib : close data file
c    srdfmute       #        : set verbosity
c    srdfhost       #        : set source machine
c    srdfinit       #        : read initialize module data
c    process_data   # formulate calibration relations
c    add_to_calib_table # add one row to the calibration data table
c
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical check_struct, equery, erdfacq, erdfmhd, erdfopn
      integer *2 Mdata(MAX_DATA)
      integer module_no, n_samp, rdf_rec, Jextchls
      real air_press, air_temp, to_volts, AD_to_volts, qmuair, qresp
      character Csublab*8, Gextadid*12, data_file*(*), calib_file*(*)
c
      include 'calbar.cmn'
c
c
      if ( .not. erdfopn( data_file, rdf_rec, aux_lu ) ) then             1b
         write( STD_OUT, 1 )
1        format( ' CALBAR : Cannot open data file ' )
         return
         end if                                                           1e
      call srdfmute( verbosity )
      call srdfhost( host_type )
c
c             get ambient conditions
c
      if ( erdfmhd( '#initial', rdf_rec ) )
     1    call srdfinit( air_press, air_temp, rdf_rec )
c
c             main loop : one per calibration/module
c
      module_no = 0
2     if ( erdfmhd( '#calbar ', rdf_rec ) ) then                          1b
c
c             skip calibration module if it is not required
c
          module_no = module_no + 1
          write( STD_OUT, 3 ) module_no
3         format( ' Found baratron calibration module number ', i3 )
          if ( all_modules .ne. YES ) then                                2b
              if ( .not. Equery( STD_IN,
     1                'Continue with this calibration ? ' ) )
     2            goto 2
                  end if                                                  2e
          n_points = 0
c
c           one row is added to the calibration data table each time
c           around this loop
c
c         start of loop
4         if ( erdfacq( Csublab, n_samp, Mdata, MAX_DATA, rdf_rec,
     1            .FALSE. ) .and. n_points .lt. MAX_POINTS ) then         2b
              if ( check_struct( Csublab, n_samp ) ) then                 3b
                  n_points = n_points + 1
                  if ( n_points .le. 1 ) then                             4b
                      to_volts = AD_to_volts()
                      Cbar_ad = Gextadid()
                      bar_chan = Jextchls(1)
                      end if                                              4e
                  call add_to_calib_table( Mdata, n_points, to_volts )
                  end if                                                  3e
              goto 4
              end if                                                      2e
c             end of loop
c
c           now we have the data for one calibration, check that there
c           is enough data
c
          if ( n_points .lt. 3 ) then                                     2b
            write( STD_OUT, 5 )
5           format( ' CALBAR : WARNING : Too few calibration points' )
          else                                                            2m
              call process_data( calib_file )
              end if                                                      2e
          goto 2
          end if                                                          1e
c         end of main loop
      call write_calibr( '##_CLOSE_##' )
      call srdfcls()
      return
      end



      SUBROUTINE EDIT_CALIBR( ERROR )
c
c Edit the calibration data for hot-wire number IWIRE . If a fitting error
c is available in the array ERROR, as indicated by the value of DONE_CALIB,
c the fitting error is plotted as a not-to-scale dashed line background .
c
c Arguments
c  real error(*)    # fitting errors
c
c subroutines
c  ilgclos, ilgdata, ilgline, ilgnote, ilgopen, ilgraph, ilgtitl,
c  ilgxlab, ilgylab, sfudg2
c
c global variables
c   calib, calib_done, n_points
c
      include 'stdio.def'
      include 'calbar.def'
c
      real error(*)
      include 'calbar.cmn'
c
      call ilgopen( ' ' )
      if ( calib_done ) then
          call ilgraph( 1, 1, 0, 0, 1 )
          call ilgline( 0, 3, 1 )
          call ilgdata( calib(1,BAR_MEAN), error, 1, n_points, 0 )
          call ilgnote( '|m(0.01,0.0)|d(100.0,0.0)' )
          end if
      call ilgtitl( 'Editing calibration data' )
      call ilgxlab( 'voltage' )
      call ilgylab( 'pressure' )
      call ilgraph( 1, 1, 3, 3, 1 )
      call ilgline( 0, 5, 1 )
      call sfudg2( calib, BAR_MEAN, BAR_PRESS, MAX_POINTS, BAR_RMS, 1,
     1             n_points, n_points )
      call ilgclos()
      return
      end



      SUBROUTINE CALC_CALIBR( ERROR )
c
c       Calibrate the pressure sensor calibration . The pressure is expressed
c as a polynomial of sensor output voltage .
c
c Arguments
c   real error(*)   # error of the fitting polynomial
c   integer iwire   # specifies the hot-wire to calibrate
c
c  integer
c    aux_enable     # NO/YES enable writing to auxillary file
c    capt_len       # length of capture file name
c    i              # looping index
c    item           # seleted menu item
c    npts           # number of data points
c    Jresp          # function : read integer
c    Jljstr         #          : left justify string
c  logical
c    first_loop     # have not printed display menu yet
c    Eyes           # function : NO/YES/QUERY to logical
c  real
c    fitted_poly(MAX_POINTS)  # fitted polynomial data
c    Qpoly          # function : evaluate polynomial
c  double precision
c    work(MAX_WORK) # work area for SPYORTHS
c  character
c    Capture_file*64# name of capture file
c    Cyes*5         # function : NO/YES/QUERY
c  subroutine
c    Cresp
c    newscr
c    spyorths
c    ilgclos, ilgdata, ilgraph, ilgline, ilgnote, ilgopen,
c    ilgpage, ilgslaw, ilgtitl, ilgwait, ilgxlab, ilgylab
c
c
      include 'stdio.def'
      include 'calbar.def'
c
      integer aux_enable, capt_len, i, item, iwire, npts, Jresp, Jljstr
      logical first_loop, Eyes
      real error(*), fitted_poly(MAX_POINTS), Qpoly
      double precision work(MAX_WORK)
      character Capture_file*64, Cyes*5
c
      include 'calbar.cmn'
      save capture_file, capt_len
      data capture_file / ' ' /
      data capt_len / 1 /
c
c       do polynomial regression and then calculate values of the fitting
c       polynomial and the fitting error
c
1     call spyorths( calib(1,BAR_MEAN), calib(1,BAR_PRESS), n_points,
     1        calib_poly, calib_order, calib_error, work )
      do 2 i = 1, n_points                                                1b
          fitted_poly(i) = Qpoly( calib(i,BAR_MEAN), calib_poly,
     1        calib_order )
          error(i) = calib(i,BAR_PRESS) - fitted_poly(i)
2         continue                                                        1e
      calib_error = sqrt( calib_error )
      write( STD_OUT, 3 ) calib_error
3     format( ' Rms fitting error ', g12.5 )
      calib_done = .TRUE.
c
c       menu for displaying results
c
      first_loop = .TRUE.
c     start of loop                                                       1b
4         if ( .not. first_loop )
     1        call newscr()
          first_loop = .FALSE.
          write( STD_OUT, 5 ) Capture_file(:capt_len)
5         format( ' Display menu' / '   0) Exit' /
     1        '   1) Display calibration' /
     2        '   2) Display table of data' /
     3        '   3) Plot data & fitted polynomial' /
     4        '   4) Capture file  ', a /
     5        '   5) Change parameters & recalculate' )
          if ( aux_lu .gt. 0 )
     1        write( STD_OUT, 6 ) Cyes( aux_enable )
6             format( '   6) Output to auxillary file ', a )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c       DISPLAY CALIBRATION
c
          else if ( item .eq. 1 ) then                                    2m
              call display_calibr(
     1            Eyes( aux_enable, 'Write to auxillary file ?' ) )
              call ilgwait()
c
c       DISPLAY DATA TABLE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 7 )   ( i, calib(i,BAR_PRESS),
     1            calib(i,BAR_MEAN), fitted_poly(i), error(i),
     2            i = 1, n_points )
7             format( /
     1            ' ************* Calibration data **************' /
     2            ' No. Air-speed Mean-volts polynomial fit-error' /
     3            ( 1x, i3, 4f10.4 ) )
              if ( aux_lu .ne. 0 ) then                                   3b
                  if ( Eyes( aux_enable, 'Write to auxillary file ?' ) )
     1                write( aux_lu, 7 )   ( i, calib(i,BAR_PRESS),
     2                    calib(i,BAR_MEAN), fitted_poly(i), error(i),
     3                    i = 1, n_points )
                  end if                                                  3e
              call ilgwait()
c
c       PLOT DATA & FITTED POLYNOMIAL
c
          else if ( item .eq. 3 ) then                                    2m
c
c           plot data and fitted polynomial
              call ilgopen( Capture_file )
              call ilgpage( 'a4', 'wide', 2, 1 )
              call ilgslaw( 'lin', 'lin' )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgline( 1, 0, 1 )
              call ilgdata( calib(1,BAR_MEAN), calib(1,BAR_PRESS),
     1                      1, n_points, 1 )
              call ilgline( 0, 1, 1 )
              call ilgdata( calib(1,BAR_MEAN), fitted_poly,
     1                      1, n_points, 0 )
              call ilgxlab( 'Sensor output (volts)' )
              call ilgylab( 'Pressure' )
              call ilgtitl( 'Calibration polynomial' )
              call ilgnote( '|2l |5-1 fitted curve|r   |s1 = data' )
c
c           plot RMS signal fluctuation
              call ilgpage( 'a4', 'WIDE', 2, 2 )
              call ilgraph( 2, 1, 11, 11, 1 )
              call ilgslaw( 'lin', 'lin' )
              call ilgline( 4, 1, 1 )
              call ilgdata( calib(1,BAR_MEAN), calib(1,BAR_RMS),
     1                      1, n_points, 1 )
              call ilgxlab( 'Sensor output (volts)' )
              call ilgylab( 'Volts' )
              call ilgtitl( 'RMS signal fluctuation' )
c
c           plot fitting error
              call ilgraph( 2, 2, 11, 11, 1 )
              call ilgline( 5, 1, 1 )
              call ilgdata( calib(1,BAR_MEAN), error, 1, n_points, 1 )
              call ilgxlab( 'Sensor output (volts)' )
              call ilgylab( 'Fitting Error' )
              call ilgwait
              call ilgclos
c
c       CAPTURE FILE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 8 )
8             format( ' Capture file name ? ', $ )
              call Cresp( STD_IN, Capture_file, MAX_TRIES )
              capt_len = max( 1, jljstr( Capture_file ) )
c
c       CHANGE PARAMETERS & RECALCULATE
c
          else if ( item .eq. 5 ) then
              call calib_param_menu()
              goto 1
c
c       AUXILLARY FILE ENABLE
c
          else if ( item .eq. 6 ) then                                    2m
              aux_enable = mod( aux_enable + 1, 3 )
              end if                                                      2e
          goto 4
c         end of loop                                                     1e
      end



      SUBROUTINE DISPLAY_CALIBR( OK_AUX )
c
c Display the calibration information for hot-wire IWIRE specified by WHAT .
c
c Arguments
c  logical ok_aux # write to auxillary file
c
c local variables
c  integer
c   i      # looping index
c  subroutine
c   ilgwait
c
c global variables
c  aux_lu, bar_chan, calib_no, calib_order, n_points, plug_no, calib_error,
c  calib_poly, Ctype, Cbar_ad
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical ok_aux
      integer i
      include 'calbar.cmn'
c
      write( STD_OUT, 1, ERR=3 ) calib_no, Ctype, plug_no, Cbar_ad,
     1    bar_chan, n_points
      if ( ok_aux .and. aux_lu .ne. 0 )
     1    write( aux_lu, 1, ERR=4 ) calib_no, Ctype, plug_no, Cbar_ad,
     2    bar_chan, n_points
1     format( '******** CALIBRATION INCIDENTALS ********' /
     1    6x, 'Calibration number         ', i3 /
     2    6x, 'Type of hot wire used      ', a32 /
     3    6x, 'Wind tunnel plug number    ', i4 /
     4    6x, 'A/D converter used         ', a12 /
     5    6x, 'A/D converter channel      ', i3 /
     6    6x, 'No. of calibration points  ', i3 )
c
      if ( calib_done ) then                                              1b
          write( STD_OUT, 2, ERR=3 ) calib_order, calib_error,
     1            ( i, calib_poly(i+1), i = 0, calib_order )
2         format( / '    ***** Polynomial calibration *****' /
     1        6x, 'Order of fitted polynomial ', i2 /
     2        6x, 'RMS error of regression    ', g13.6 /
     3        6x, 'Calibration polynomial ' /
     4        ( 11x, i2, g14.6 ) )
          if ( ok_aux .and. aux_lu .ne. 0 )
     1        write( aux_lu, 2, ERR=4 ) calib_order, calib_error,
     2            ( i, calib_poly(i+1), i = 0, calib_order )
          end if                                                          1e
      return
c
3     write( STD_ERR, 5 ) 'STD_OUT'
      call ilgwait()
      return
4     write( STD_ERR, 5 ) 'auxillary file'
5     format( ' DISPLAY_CALIBR : Cannot write to ', a )
      return
      end



      LOGICAL FUNCTION WRITE_CALIBR( CALIB_FILE )
c
c Writes the pressure sensor calibration to file CALIB_FILE .
c
c Argument
c  character calib_file*(*) # logical unit number of the calibration file
c
c
c local variables
c  logical Existf     # function : file exists ?
c          Eyes       #          : NO, YES, QUERY -> logical
c  integer i          # looping array index
c          calib_lu   # logical unit number for calibration file (0 if closed)
c          name_len   # non-blank length of calibration file name
c          res_calib  # reserved logical unit number for calibration file
c          jljstr     # function : non-blank string length
c          io_linearisation      : for all linearisation file IO
c          new_lun()  #          : get new logical unit number
c  character old_file*64  # previous file name
c
c global variables
c  bar_chan, calib_no, calib_order, lin_correct, lin_order, n_points, plug_no,
c  calib, calib_error, calib_poly, lin_error, lin_maxim, lin_minim, lin_offset,
c  lin_slope, lin_poly, Cbar_ad, Ctype
c
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical Existf, Eyes
      integer i, calib_lu, name_len, res_calib,
     1    io_linearisation, jljstr, new_lun
      character calib_file*(*), old_file*64
c
      include 'calbar.cmn'
      save res_calib, calib_lu, old_file
      data res_calib, calib_lu / 0, 0 /
      data old_file / ' ' /
c
c
c       WRITE_CLOSE -- close the calibration file and return successfully
c
      write_calibr = .FALSE.
      if ( calib_file .eq. '##_CLOSE_##' ) then                                  1b
          write_calibr = .TRUE.
          if ( calib_lu .ne. 0 )
     1        close( UNIT=calib_lu )
          calib_lu = 0
c
c       check arguments
c
      else if ( calib_file .eq. ' ' ) then                                1m
          write( STD_ERR, 1 )
1         format( ' WRITE_CALIBR : ABORT : O/P file not specified' )
c
c       open calibration file
c           If the file name has changed, close the old one first .
c           Then if no file is open, open it . If the file already
c           exists, it is opened for appending .
c
      else                                                                1m
          name_len = max( 1, jljstr( Calib_file ) )
          if ( res_calib .eq. 0 )
     1        res_calib = new_lun()
          if ( calib_lu .ne. 0 .and. calib_file .ne. old_file ) then      2b
              close( UNIT=calib_lu )
              calib_lu = 0
              end if                                                      2e
          if ( calib_lu .eq. 0 ) then                                     2b
              if ( Existf( Calib_file ) ) then                            3b
                  open( UNIT=res_calib, FILE=Calib_file, STATUS='old',
     1                ACCESS='append', FORM='unformatted', ERR=3,
     2                RECORDTYPE='variable', BLOCKSIZE=8192 )
              else                                                        3m
                  open( UNIT=res_calib, FILE=Calib_file, STATUS='new',
     1                ACCESS='sequential', FORM='unformatted', ERR=3,
     2                RECORDTYPE='variable', BLOCKSIZE=8192 )
                  end if                                                  3e
              calib_lu = res_calib
              old_file = calib_file
              calib_no = 1
              end if                                                      2e
c
c           Now write to the file .
c           Include the pressure sensor linearisation correction.
c
          if ( Eyes( lin_correct,
     1               'Correct for transducer nonlinearity ?') ) then      2b
c
c           Read the linearisation correction from file if it is not available
c
              if ( .not. got_linear ) then                                3b
                  got_linear = io_linearisation( LIN_READ )
                  end if                                                  3e
              if ( got_linear ) then                                      3b
                  write( calib_lu, ERR=3 ) MAGIC,
     1               calib_no, Ctype, plug_no, Cbar_ad, bar_chan,
     2               n_points, calib_order, calib_error,
     3               ( calib_poly(i), i = 1, calib_order + 1 ),
     4               ( calib(i,BAR_PRESS), calib(i,BAR_MEAN),
     5                 calib(i,BAR_RMS), i = 1, n_points ),
     6               lin_order, lin_error, lin_maxim, lin_minim,
     7               lin_offset, lin_slope,
     8               ( lin_poly(i), i=1,lin_order + 1 )
              else                                                        3m
                  write( STD_ERR, 2 )
2                 format( ' CALBAR : ABORT : Cannot write calibration' /
     1                '   Linearisation relation not available .' )
                  call ilgwait()
                  end if                                                  3e
          else                                                            2m
c
c           write a dummy pressure sensor linearisation correction.
c           ( order=1, error=0.0, maximum=0.0, minimum=0.0,
c             offset=0.0, slope=1.0, polynomial=0.0,1.0 )
c
              write( calib_lu, ERR=3 ) MAGIC,
     1            calib_no, Ctype, plug_no, Cbar_ad, bar_chan, n_points,
     2            calib_order, calib_error,
     3            ( calib_poly(i), i = 1, calib_order+1 ),
     4            ( calib(i,BAR_PRESS), calib(i,BAR_MEAN),
     5              calib(i,BAR_RMS), i = 1, n_points ),
     6            1, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0
              end if                                                      2e
          calib_no = calib_no + 1
          write_calibr = .TRUE.
          end if                                                          1e
      return
c
c           Error traps
c
3     write( STD_ERR, 4 ) Calib_file(1:name_len)
4     format( ' WRITE_CALIBR : ABORT : Cannot write to file ', a )
      return
c
5     write( STD_ERR, 6 ) Calib_file(1:name_len)
6     format( ' WRITE_CALIBR : ABORT : Cannot open file ', a )
      return
      end



      SUBROUTINE PROCESS_DATA( CALIB_FILE )
c
c  Control the process of producing and writing the calibration relations .
c
c Arguments
c   character calib_file*(*)  # name of file containing calibrations
c
c local variables
c  logical
c    written        # relations for the current hot-wire have been written
c    Equery         # function : read logical
c    write_calibr   #          : write calibration relation
c  integer
c    i              # looping index
c    item           # selected menu item
c    name_len       # non-blank length of calibration file name
c    jljstr         # function : non-blank string length
c    Jresp          #          : read integer
c  real
c    fit_error(MAX_POINTS)  # polynomial fitting error at each data point
c  character
c    Cdone*8        # function : 'done' / 'not done'
c  subroutine
c    calc_calibr    # find polynomial of voltage calibration relation
c    calib_param_menu # calibration parameters menu
c    Cresp          # read string
c    display_calibr # display calibration
c    edit_calibr    # edit calibration data
c    ilgwait        # wait for <CR>
c    newscr         # clear screen
c    sortsh         # sorts the calibration data
c
c global variables
c   aux_lu, calib_done, n_points, calib
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical written, Equery, write_calibr
      integer i, item, name_len, jljstr, Jresp
      real fit_error(MAX_POINTS)
      character calib_file*(*), Cdone*8
      include 'calbar.cmn'
c
c       initialize
c
      calib_done = .FALSE.
      written    = .FALSE.
      name_len = max( 1, jljstr( Calib_file ) )
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 )  Cdone(calib_done),
     1        calib_file(1:name_len)
2         format( ' Processing calibration data for pressure sensor' /
     N        '  -1) Abandon' /
     Z        '   0) Exit (+write)' /
     1        '   1) Set calibration parameters' /
     2        '   2) Sort calibration data' /
     3        '   3) Edit calibration data' /
     4        '   4) Display calibration data' /
     5        '   5) Calculate calibration relation  ', a /
     6        '   6) Display calibration' /
     7        '   7) Write calibration' /
     8        '   8) Calibration file ', a )
          item = Jresp( STD_IN, -1, 8, MAX_TRIES )
c
c       ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, 'Abandon. Are you sure ? ' ) )
     1            return
c
c       EXIT (+WRITE)
c       If calibration has been calculated but not written, write it to file .
c
          else if ( item .eq. 0 ) then                                    2m
              if ( calib_done .and. .not. written )
     1            written = write_calibr( calib_file )
              call write_calibr( '##_CLOSE_##' )
              return
c
c       SET CALIBRATION PARAMETERS
c
          else if ( item .eq. 1 ) then                                    2m
              call calib_param_menu()
c
c       SORT CALIBRATION DATA
c
          else if ( item .eq. 2 ) then                                    2m
              call sortsh( calib, n_points, 1, MAX_POINTS, BAR_PRESS,
     1            BAR_RMS )
c
c       EDIT CALIBRATION DATA
c
          else if ( item .eq. 3 ) then                                    2m
              call edit_calibr( fit_error )
c
c       DISPLAY CALIBRATION DATA
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 ) ( i, calib(i,BAR_PRESS),
     1            calib(i,BAR_MEAN), calib(i,BAR_RMS), i = 1, n_points )
3             format( / ' ********* Calibration data *********' /
     1                  ' No.  Pressure  Mean-volts  RMS-volts' /
     2                  ( 1x, i3, 3f10.4 ) )
              if ( aux_lu .ne. 0 )
     1            write( aux_lu, 3 ) ( i, calib(i,BAR_PRESS),
     2                calib(i,BAR_MEAN), calib(i,BAR_RMS),
     3                i = 1, n_points )
              call ilgwait()
c
c       CALCULATE CALIBRATION RELATION
c
          else if ( item .eq. 5 ) then                                    2m
              call calc_calibr( fit_error )
              calib_done = .TRUE.
c
c       DISPLAY CALIBRATION
c
          else if ( item .eq. 6 ) then
              call display_calibr( .TRUE. )
              call ilgwait()
c
c       WRITE CALIBRATION
c
          else if ( item .eq. 7 ) then                                    2m
              if ( calib_done ) then                                      3b
                  if ( written ) then                                     4b
                      if ( .not. Equery( STD_IN,
     1                             'Already done. Are you sure ? ' ) )
     2                    goto 1
                      end if                                              4e
                  written = write_calibr( calib_file )
              else                                                        3m
                  write( STD_OUT, 4 )
4                 format( ' Calibration not available for writing .' )
                  call ilgwait()
                  end if                                                  3e
c
c       SPECIFY CALIBRATION FILE
c
        else if ( item .eq. 8 ) then                                      2m
              write( STD_OUT, 5 )
5             format( ' Calibration file name ? ', $ )
              call Cresp( STD_IN, Calib_file, MAX_TRIES )
              name_len = max( 1, jljstr( Calib_file ) )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE AVERAGE_CALIBRATIONS( CALIB_FILE )
c
c Averaging of calibrations already stored in file CALIB_FILE .
c
c Arguments
c   character calib_file*(*)  # name of calibration file
c
c local variables
c  logical
c    got_data       # calibration data has been read
c    got_list       # list of calibrations to average is available
c    written        # calibration is written to file
c    Equery         # function : read logical
c    read_avg_data  #          : read all calibration data for averaging
c    write_calibr   #          : write a calibration to file
c  integer
c    avg_list(MAX_AVERAGE)  # list of calibration records to be averaged
c    i              # looping index
c    item           # selected menu item
c    name_len       # length of calibration file name
c    n_average      # number of calibrations to be averaged
c    Jresp          # function : read integer
c    jljstr         #          : left justify string
c    read_list      #          : read list of integers
c  real
c    fit_error(MAX_POINTS)  # error of the fitted polynomial
c  character
c    Cdone*8        # function : 'done' / 'not done'
c  subroutine
c    calc_calibr    # calculate calibration relation
c    calib_param_menu set calibration parameters
c    Cresp          # read string
c    display_calibr # display calibration
c    edit_calibr    # edit calibration data
c    ilgwait        # wait for <CR>
c    isortsh        # sort list if integers
c    newscr         # clear screen
c    sortsh         # sort data array
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical got_data, got_list, written, Equery, read_avg_data,
     1    write_calibr
      integer avg_list(MAX_AVERAGE), i, item, name_len, n_average,
     1    Jresp, jljstr, read_list
      real fit_error(MAX_POINTS)
      character calib_file*(*), Cdone*8
c
      include 'calbar.cmn'
      save got_list, n_average
      data got_list / .FALSE. /
      data n_average / 1 /
c
c       initialize
c
      calib_done = .FALSE.
      written    = .FALSE.
      got_data   = .FALSE.
      name_len = max( 1, jljstr( Calib_file ) )
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) calib_file(1:name_len),
     1        ( avg_list(i), i=1,n_average )
2         format( ' Averaging menu ' / '  -1) Abandon' / '   0) Exit' /
     1        '   1) Calibration file', 15x, a /
     2        '   2) Set calibration parameters' /
     3        '   3) Select calibrations to average ', 8i3 )
          if ( got_list )
     1        write( STD_OUT, 3 ) Cdone(calib_done)
3             format( '   4) Sort calibration data' /
     1            '   5) Edit calibration data' /
     2            '   6) Display calibration data' /
     3            '   7) Calculate calibration relation  ', a /
     5            '   8) Display calibration' /
     6            '   9) Write calibration' )
          item = Jresp( STD_IN, -1, 9, MAX_TRIES )
c
c       ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, 'Abandon. Are you sure ? ' ) )
     1            return
c
c       EXIT (+WRITE)
c       If calibration has been calculated but not written, write it to file .
c
          else if ( item .eq. 0 ) then                                    2m
              if ( calib_done .and. .not. written )
     1            written = write_calibr( calib_file )
              call write_calibr( '##_CLOSE_##' )
              return
c
c       SPECIFY CALIBRATION FILE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Calibration file name ? ', $ )
              call Cresp( STD_IN, Calib_file, MAX_TRIES )
              name_len = max( 1, jljstr( Calib_file ) )
              got_data = .FALSE.
c
c       SET CALIBRATION PARAMETERS
c
          else if ( item .eq. 2 ) then                                    2m
              call calib_param_menu()
c
c       SELECT CALIBRATIONS TO AVERAGE
c
          else if ( item .eq. 3 ) then                                    2m
              n_average = read_list( avg_list, MAX_AVERAGE )
              call isortsh( avg_list, n_average )
              got_list =
     1            0 .lt. n_average .and. n_average .le. MAX_AVERAGE
              got_data = .FALSE.
c
c       SORT CALIBRATION DATA
c
          else if ( got_list ) then                                       2m
c
c       read the calibration file
c
              if ( .not. got_data ) then                                  3b
                  got_data = read_avg_data( calib_file, avg_list,
     1                n_average )
                  if ( got_data )
     1                calib_done = .FALSE.
                  end if                                                  3e
              if ( .not. got_data ) then                                  3b
                  write( STD_OUT, 5 )
5                 format( ' Cannot read data from calibration file .' )
                  call ilgwait()
              else if ( item .eq. 4 ) then                                3m
                  call sortsh( calib, n_points, 1,
     1                MAX_POINTS, BAR_PRESS, BAR_RMS )
c
c       EDIT CALIBRATION DATA
c
              else if ( item .eq. 5 ) then                                3m
                  call edit_calibr( fit_error )
c
c       DISPLAY CALIBRATION DATA
c
              else if ( item .eq. 6 ) then                                3m
                  write( STD_OUT, 6 ) ( i, calib(i,BAR_PRESS),
     1                calib(i,BAR_MEAN), calib(i,BAR_RMS),
     2                i = 1, n_points )
6                 format( / ' ********* Calibration data *********' /
     1                      ' No.  Pressure  Mean-volts  RMS-volts' /
     2                      ( 1x, i3, 3f10.4 ) )
                  if ( aux_lu .ne. 0 )
     1                write( aux_lu, 6 ) ( i, calib(i,BAR_PRESS),
     2                    calib(i,BAR_MEAN), calib(i,BAR_RMS),
     3                    i = 1, n_points )
                  call ilgwait()
c
c       CALCULATE CALIBRATION RELATION
c
              else if ( item .eq. 7 ) then                                3m
                  call calc_calibr( fit_error )
                  calib_done = .TRUE.
c
c       DISPLAY CALIBRATION
c
              else if ( item .eq. 8 ) then                                3m
                  call display_calibr( .TRUE.)
                  call ilgwait()
c
c       WRITE CALIBRATION
c
              else if ( item .eq. 9 ) then                                3m
                  if ( calib_done ) then                                  4b
                      written = write_calibr( calib_file )
                  else                                                    4m
                      write( STD_OUT, 7 )
7                 format( ' Calibration not available for writing.' )
                      call ilgwait()
                      end if                                              4e
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of loop
      end



      INTEGER FUNCTION READ_LIST( LIST, MAX_SIZE )
c
c   Read a list of up to MAX_SIZE integers from the keyboard into the
c   array LIST . The returned value of READ_LIST is the number of values
c   actually read . The values read must be between 0 and 10000
c   inclusive .
c
c Arguments
c  integer list(*)      # OUT # list of values
c          max_size     # IN  # maximum number of values
c Local variables
c  integer count        # number of values read
c          i            # looping index
c          rec_no       # record number
c  character Cline*64   # one line of input
c  subroutine scase     # change case
c
      include 'stdio.def'
      integer count, i, list(*), max_size, rec_no
      character Cline*64
c
      write( STD_OUT, 1 ) max_size
1     format( ' Enter list of calibration records (in the calibration' /
     1        '    file) to read for averaging calibration data .' /
     2        '    Enter one record number on each line .' /
     3        '    Do not not repeat record numbers .' /
     4        '    Maximum list length is ', i4 /
     5        '    Terminate the list by typing "end" .' // )
      count = 0
c
c           check if the array is full
c
c     start of loop                                                       1b
2         if ( count .ge. max_size ) then                                 2b
              write( STD_OUT, 3 )
3             format( ' list is full' )
              read_list = count
              return
              end if                                                      2e
c
c           read a line
c
          write( STD_OUT, 4 )
4         format( ' Record number ?  ', $ )
          call Cresp( STD_IN, Cline, 4 )
          call scase( cline, cline, 'L' )
          if ( Cline .eq. 'end' ) then                                    2b
              read_list = count
              return
              end if                                                      2e
c
c           decode integer value
c
          read( Cline, *, ERR=7 ) rec_no
c
c           check that the value is not already in the list
c
          do 6 i = 1, count                                               2b
              if ( rec_no .eq. list(i) ) then                             3b
                  write( STD_OUT, 5 ) rec_no, i
5                 format( ' Sorry,', i3, ' is already the', i3,
     1                'th list element .' )
                  goto 2
                  end if                                                  3e
6             continue                                                    2e
c
c           if we get to here, the value is ok; so put it in the list
c
          count = count + 1
          list(count) = rec_no
          goto 2
c         end of loop                                                     1e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c           error traps
c
7         write( STD_OUT, 8 )
8         format( ' Cannot decode value' )
          goto 2
      end



      SUBROUTINE ISORTSH( IARRAY, SIZE )
c
c               SORT ( SHell )
c (Shell) sort the contents of the array IARRAY into ascending order .
c
c Arguments
c   integer iarray(*) # operand array
c           size      # length of the array
c local variables
c   integer i1,i2,i3  # looping indices
c           id        # sort segment size
c
      include 'stdio.def'
      integer iarray(*), size, i1, i2, i3, id, itmp
c
      if ( size .lt. 2 ) return
      id = 2 ** int( log( real( size ) ) / log( 2.0 ) + 0.0001 ) - 1
c     repeat                                                              1b
1         do 3 i1 = 1, size - id                                          2b
              do 2 i2 = i1, 1, -id                                        3b
                  i3 = i2 + id
                  if ( iarray(i2) .le. iarray(i3) ) goto 3
                  itmp = iarray(i3)
                  iarray(i3) = iarray(i2)
                  iarray(i2) = itmp
2                 continue                                                3e
3             continue                                                    2e
          id = id / 2
          if ( id .gt. 0 ) goto 1
c     until ( id <= 0 )                                                   1e
      return
      end



      LOGICAL FUNCTION READ_AVG_DATA( CALIB_FILE, LIST, LEN_LIST )
c
c Read the calibration data from the LEN_LIST calibrations specified in LIST
c stored in the calibration file CALIB_FILE .
c
c Arguments
c   character Calib_file*(*)  # name of calibration file
c   integer list(*)           # list of calibration records required
c           len_list          # number of calibration records
c
c local variables
c  logical
c    Existf     # function : does a file exist ?
c  integer
c    calib_lu   # logical unit number of calibration file
c    count      # counter for calibration records in file
c    i          # implicit looping index
c    list_index # pointer to an item in the LIST of calibration record numbers
c    name_len   # non-blank length of CALIB_FILE
c    npts       # number of data points appended to the list
c    jljstr     # function : left justify string
c    new_lun    #          : get new logical unit number
c  character
c    Cmagic*2   # Magic (ID) number for a calibration file record
c  subroutine
c    ilgwait    # ilglib :
c
c global variables
c   bar_chan, calib_no, calib_order, calib_no, lin_correct, lin_order,
c   n_points, plug_no, calib, calib_error, calib_poly, lin_error,
c   lin_maxim, lin_minim, lin_offset, lin_poly, lin_slope, Cbar_ad,
c   Ctype
c
      include 'stdio.def'
      include 'calbar.def'
c
      logical Existf
      integer calib_lu, count, i, list(*), len_list, list_index,
     1    name_len, npts, jljstr, new_lun
      character Calib_file*(*), Cmagic*2
c
      include 'calbar.cmn'
      save calib_lu
      data calib_lu / 0 /
c
c
      read_avg_data = .FALSE.
c
c       open file
c
      name_len = max( 1, jljstr( calib_file ) )
      if ( .not. Existf( Calib_file ) ) then                              1b
          write( STD_ERR, 1 ) Calib_file(1:name_len)
1         format( ' Cannot find calibration file ', a )
          call ilgwait()
      else                                                                1m
          if ( calib_lu .eq. 0 ) then
              calib_lu = new_lun()
              end if
          if ( calib_lu .eq. 0 )
     1        goto 7
          open( UNIT=calib_lu, FILE=Calib_file, STATUS='old',
     1        ACCESS='sequential', FORM='unformatted', ERR=7,
     2        RECORDTYPE='variable', BLOCKSIZE=8192 )
c
c       Read the file . Note that it is not necessary to read the contents
c       of a file record which is to be skipped . It is not necessary to
c       read the linearisation information (or even desirable as this would
c       overwrite valid data) .
c       Note that the list is ascending order, and so the last record number
c       is the biggest .
c
          n_points = 0
          list_index = 1
          do 2 count = 1, list(len_list)                                        2b
              if ( count .eq. list(list_index) ) then                     3b
                  read( calib_lu, ERR=3, END=5 ) Cmagic,
     1                calib_no, Ctype, plug_no, Cbar_ad, bar_chan,
     2                npts, calib_order, calib_error,
     3                ( calib_poly(i), i = 1, calib_order+1 ),
     4                ( calib(i,BAR_PRESS), calib(i,BAR_MEAN),
     5                  calib(i,BAR_RMS),
     6                  i = n_points + 1, n_points + npts )
                  if ( Cmagic .ne. MAGIC )
     1                goto 9
                  n_points = n_points + npts
                  list_index = list_index + 1
             else                                                       3m
                 read( calib_lu, ERR=3, END=5 )
                 end if                                                 3e
2            continue                                                   2e
          close( UNIT=calib_lu )
c
c       check the A/D converter channels
c
          read_avg_data = .TRUE.
          calib_no = calib_no + 1
          end if                                                        1e
      return
c
c       error traps
c
3     write( STD_ERR, 4 ) calib_file(1:name_len)
4     format( ' Cannot read calibration file ', a )
      return
5     write( STD_ERR, 6 ) calib_file(1:name_len)
6     format( ' Unexpected EOF in calibration file ', a,
     1    ' Cannot complete reading file' )
      close( UNIT=calib_lu )
      return
7     write( STD_ERR, 8 ) calib_file(1:name_len)
8     format( ' Cannot open calibration file ', a )
      close( UNIT=calib_lu )
      return
9     write( STD_ERR, 10 ) calib_file(1:name_len)
10     format( 1x, a, ' is not a calibration file .' /
     1    ' Illegal magic number .' )
      end



      CHARACTER *8 FUNCTION CDONE( BOOLEAN )
c
c Returns 'done' if the argument is .TRUE., otherwise returns 'not done'
c
      include 'stdio.def'
      logical boolean
c
      if ( boolean ) then
          Cdone = 'done'
      else
          Cdone = 'not done'
          end if
      return
      end


      BLOCK DATA BAR_BLOCK
      include 'stdio.def'
      include 'calbar.def'
c
      include 'calbar.cmn'
c
c  MISCELLANEOUS PARAMETERS
      data all_modules, aux_lu, verbosity / YES, 0, 1 /
c
c  INCIDENTAL CALIBRATION INFORMATION
      data calib_no, host_type, plug_no / 1, 2, 3 /
      data Cbar_ad, Ctype / ' ', ' ' /
c
c  CALIBRATION PARAMETERS
      data calib_done / .FALSE. /
      data calib_order / 1 /
c
c  CALIBRATION DATA
      data n_points, bar_chan / 0, 9 /
      data press_conv / 133.3224 /
c
c
c  LINEARISATION INFORMATION
      data lin_correct, lin_order / NO, 1 /
      data lin_maxim, lin_minim, lin_offset, lin_slope,
     1    lin_poly, lin_error / 3*0.0, 1.0, 1.0, MAX_POLY*0.0 /
      data linear_file / DEF_LINEAR_FILE /
      end

