c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM CALHW1
c
c       CALibrate single Hot Wire 1
c
c Program to examine a raw data file for '#calhw1' modules , and compile a
c sequential access binary calibration file which contains the information
c required to convert the A/D readings from hot wires into air speeds .
c
c Local Variables
c  integer
c    got_baratron   # baratron calibration is available
c    got_amplif     # amplifier output offset voltage is available
c    get_baratron   # function : get baratron calibration
c    get_amplif     #          : get amplifier output offset voltage
c  integer
c    item           # selected item number
c    len_cf_name    # length of calibration file name
c    len_df_name    # length of data file name
c    iargc          # function : number of program arguments
c    jljstr         #          : left justify string
c    Jresp          #          : integer response
c  character
c    Calib_file*64  # name of calibration file
c    Cdata_file*64  # name of data file
c    Cyes*5         # function : 'no' / 'yes' / 'query'
c  subroutine
c    average_calibrations # average the calibrations in a calibration file
c    calib_param_menu     # menu for calibration parameters
c    parse_file_structure # formulate calibrations from data file
c    setup_menu           # menu for setup parameters
c  library routines
c    Cresp          # character string response
c    getarg         # get program argument
c    ilgwait        # ilglib
c    newscr         # hpprev
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer got_baratron, got_amplif, get_baratron, get_amplif
      integer item, len_cf_name, len_df_name, iargc, jljstr, Jresp
      character Calib_file*64, Cdata_file*64, Cyes*5
      data got_baratron, got_amplif / NO, NO /
      data len_cf_name, len_df_name / 1, 1 /
      data Calib_file, Cdata_file / ' ', ' ' /
c
c               Print Label
c
      write( STD_OUT, 1 )
1     format( 21x, 'CALHW1 : Single Hot wire calibrator' /
     1        21x, 35( '=' ) / )
c
c
c         Get program arguments -- if any
c             first argument  = data file name
c             second argument = calibration file name
c
      if ( iargc() .gt. 0 ) then                                          1b
          call getarg( 1, Cdata_file )
          if ( iargc() .gt. 1 )
     1        call getarg( 2, Calib_file )
          end if                                                          1e
      len_df_name = max( jljstr( Cdata_file ), 1 )
      len_cf_name = max( jljstr( Calib_file ), 1 )
c
c     start of loop                                                       1b
2         call newscr()
          write( STD_OUT, 3 ) Cdata_file(1:len_df_name),
     1        Calib_file(1:len_cf_name), Cyes(got_baratron),
     2        Cyes(got_amplif)
3         format( ' Main menu' /
     n        ' -1) Convert calibration file to latest version' /
     z        '  0) Exit' /
     1        '  1) Name of data file', 12x, a /
     2        '  2) Name of calibration file     ', a /
     3        '  3) Specify baratron calibration ', a /
     4        '  4) Amplifier gain & offset      ', a /
     5        '  5) Setup parameters menu' /
     6        '  6) Calibration parameters menu' /
     7        '  7) Find hot-wire calibration relations' /
     8        '  8) Average calibrations' /
     9        '  9) Help                  ', $ )
          item = Jresp( STD_IN, -1, 9, MAX_TRIES )
c
c           CONVERT CALIBRATION FILE TO LATEST VERSION
c
          if ( item .eq. -1 ) then                                        2b
              call Latest_file_version( Calib_file )
c
c           EXIT
c
          else if ( item .eq. 0 ) then                                    2m
              stop
c
c           NAME OF DATA FILE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Enter name of data file ', $ )
              call Cresp( STD_IN, Cdata_file, MAX_TRIES )
              len_df_name = max( jljstr( Cdata_file ), 1 )
c
c           NAME OF CALIBRATION FILE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Enter name of hot-wire calibration file ', $ )
              call Cresp( STD_IN, Calib_file, MAX_TRIES )
              len_cf_name = max( jljstr( Calib_file ), 1 )
c
c           SPECIFY BARATRON CALIBRATION
c
          else if ( item .eq. 3 ) then                                    2m
              got_baratron = Get_baratron()
c
c           CALCULATE AMPLIFIER OFFSET VOLTAGE
c
          else if ( item .eq. 4 ) then                                    2m
              got_amplif = get_amplif()
c
c           SETUP PARAMETERS MENU
c
          else if ( item .eq. 5 ) then                                    2m
              call setup_menu()
c
c           CALIBRATION PARAMETERS MENU
c
          else if ( item .eq. 6 ) then                                    2m
              call calib_param_menu()
c
c           DO CALIBRATIONS
c
          else if ( item .eq. 7 ) then                                    2m
              if ( got_baratron .eq. NO )
     1            got_baratron = Get_baratron()
              if ( got_amplif .eq. NO )
     1            got_amplif = get_amplif()
              if ( got_baratron .eq. NO ) then                            3b
                  write( STD_OUT, 6 )
6                 format( ' CALHW1 : Do not have baratron calibration' )
                  call ilgwait()
              else if ( got_amplif .eq. NO ) then                         3e
                  write( STD_OUT, 7 )
7                 format( ' CALHW1 : Do not have amplifier offset(s)' )
                  call ilgwait()
              else if ( Cdata_file .eq. ' ' ) then                        3m
                  write( STD_OUT, 8 )
8                 format( ' CALHW1 : Must specify data file' )
                  call ilgwait()
              else                                                        3m
                  call parse_file_structure( Cdata_file, Calib_file )
                  end if                                                  3e
c
c           AVERAGE CALIBRATIONS
c
          else if ( item .eq. 8 ) then                                    2m
              if( Calib_file .ne. ' ' ) then                              3b
                  call average_calibrations( Calib_file )
              else                                                        3m
                  write( STD_OUT, 9 )
9                 format( ' CALHW1 : Must specify calibration file' )
                  call ilgwait()
                  end if                                                  3e
c
c           HELP
c
          else if ( item .eq. 9 ) then                                    2m
              write( STD_OUT, 10 )
10            format( ' Select the required menu item by typing and ',
     1              'integer, then <RETURN> .' /
     2            ' The names of the data and calibration files, a ',
     3              'baratron calibration and' /
     4            ' matching amplier gains & offsets must be ',
     5              'specified before evaluating' /
     6            ' the calibration relations (menu item 7) . Menu ',
     7              'item 8 can be used to' /
     8            ' average previously calculated calibrations .' )
              call ilgwait()
              end if                                                      2e
          goto 2
c         end of loop                                                     1e
      end




      SUBROUTINE SETUP_MENU
c
c       Menu for setup / environment parameters .
c
c local variables
c  integer
c    aux_len          # length of auxillary file name
c    item             # selected menu item
c    iunits           # pressure units specifier index
c    jljstr           # function : left justify string
c    Jresp            #          : read integer
c    new_lun          #          : get new logical unit number
c    open_file        #          : open file
c  real
c    Pcvt_list(8)     # conversion factors for units into Pascals
c    Qresp            #          : read real
c  character
c    Caux_file*64     # current name of auxillary file
c    Cold_aux*64      # previous name of auxillary file
c    Chost(0:2)*8     # host machine descriptions
c    Cpunits(8)*4     # list of pressure units
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer aux_len, item, iunits, jljstr, Jresp, new_lun, open_file
      real Pcvt_list(8), Qresp
      character Caux_file*64, Cold_aux*64, Chost(0:2)*8, Cpunits(8)*4
c
      include 'calhw1.cmn'
      save aux_len, iunits, Pcvt_list,
     1    Caux_file, Cold_aux, CPunits, Chost
      data Pcvt_list / 1.0, 1000.0, 100.0, 6894.76, 133.3224, 338689.0,
     1  9.804895, 249.0443 /
      data CPunits, Chost, Caux_file, Cold_aux /
     1  'Pa  ', 'kPa ', 'mBar', 'psi ', 'Torr', 'inHg', 'mm W', 'in W',
     2  'VAX-VMS ', 'PDP11/34', 'IBM XT  ', ' ', ' ' /
      data aux_len, iunits / 1, 1 /
c
c     start of loop
1         call newscr()
          write( STD_OUT, 2 ) verbosity, Chost(host_type),
     1        Caux_file(1:aux_len), Ctype, Cpunits(iunits),
     2        iplug, trip_diam
2         format( ' Select item' / '   0) exit' /
     1        '   1) verbosity                ', i3 /
     2        '   2) set host machine         ', a /
     3        '   3) auxillary file name      ', a /
     4        '   4) hot wire description     ', a /
     5        '   5) reference pressure units ', a /
     6        '   6) wind tunnel plug number  ', i3 /
     7        '   7) boundary layer trip diam ', f7.3, ' mm ', $ )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c             EXIT
c
          if ( item .le. 0 ) then                                         2b
              if ( aux_lu .ne. 0 .and. Caux_file .ne. Cold_aux ) then     3b
                  close( UNIT=aux_lu )
                  aux_lu = 0
                  end if                                                  3e
              if ( Caux_file .ne. ' ' .and. aux_lu .eq. 0 )
     1            aux_lu = open_file( new_lun(), Caux_file, 'new', 0 )
              if ( aux_lu .ne. 0 )
     1            Cold_aux = Caux_file
              return
          else if ( item .eq. 1 ) then                                    2m
c
c             VERBOSITY
c
              write( STD_OUT, 3 )
3             format( ' Enter verbosity (0-2) ', $ )
              verbosity = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c             HOST MACHINE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' What is the host machine ? 0) VAX - VMS' /
     1            28x, '1) PDP11/34 - unix' /
     2            28x, '2) IBM XT/AT - MS-DOS' )
              host_type = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c             AUXILLARY FILE NAME
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Name of auxillary file ?', $ )
              call Cresp( STD_IN, Caux_file, MAX_TRIES )
              aux_len = max( 1, jljstr( Caux_file ) )
c
c
c             DESCRIPTION OF HOT WIRE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Enter description of hot wire ', $ )
              call Cresp( STD_IN, Ctype, MAX_TRIES )
c
c             INPUT PRESSURE UNITS
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 7 )
7             format( ' Enter the units of input reference pressures' /
     1            t10, '1) Pascals',     t30, '5) Torr' /
     2            t10, '2) KiloPascals', t30, '6) Inches of Mercury' /
     3            t10, '3) Millibars ',  t30, '7) mm of Water' /
     4            t10, '4) Psi',         t30, '8) Inches of Water ', $ )
              iunits = Jresp( STD_IN, 1, 8, MAX_TRIES )
              press_conv = Pcvt_list( iunits )
c
c             WIND TUNNEL PLUG NUMBER
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 8 )
8             format( ' Enter the wind tunnel plug number ', $ )
              iplug = Jresp( STD_IN, -5, 15, MAX_TRIES )
c
c             BOUNDARY LAYER TRIP DIAMETER
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 9 )
9             format( ' Enter the boundary layer trip diameter (mm) ',
     1            $ )
              trip_diam = Qresp( STD_IN, 1.0, 10.0, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      INTEGER FUNCTION GET_BARATRON()
c
c open the baratrom calibration file or change the baratron calibration
c record .
c
c local variables
c  logical
c    EBAR_OPEN      # function : open pressure sensor calibration file
c  integer
c    name_len       # length of file name
c    record         # record number of baratron calibration file
c    item           # selected menu item
c    Jresp          # function : integer response
c    jljstr         #          : left justify string
c  character
c    Cbar_file*64   # name of baratron calibration file
c  subroutine
c    Cresp          # read character string
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical EBAR_OPEN
      integer name_len, record, item, Jresp, jljstr
      character Cbar_file*64
      include 'calhw1.cmn'
      save Cbar_file, name_len, record
      data Cbar_file / ' ' /
      data name_len, record / 1, 0 /
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) Cbar_file(1:name_len), record
2         format( ' Baratron calibration menu' / '   0) Exit' /
     1        '   1) file name     ', a /
     2        '   2) record number ', i3, ' ', $ )
          item = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              get_baratron = NO
              if ( Cbar_file .ne. ' ' ) then                              3b
                  if ( EBAR_OPEN( record, Cbar_file, aux_lu ) )
     1                get_baratron = YES
                  end if                                                  3e
              return
c
c           BARATRON FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Baratron calibration file name ? ', $ )
              call Cresp( STD_IN, Cbar_file, MAX_TRIES )
              name_len = max( 1, jljstr( Cbar_file ) )
c
c           RECORD NUMBER
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Baratron calibration record number ? ' /
     1            '  ( 0 = view calibrations before selecting ) ', $ )
              record = Jresp( STD_IN, 0, 50, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE CALIB_PARAM_MENU()
c
c menu for calibration parameters
c
c local variables
c  integer
c    item       # selected menu item
c    Jresp      # function : integer response
c  character
c    Cyes*5     # function : 'no', 'yes', 'query'
c  subroutine
c    newscr     # HPPREV
c    menu_siddal_davies # menu for selecting siddal
c    menu_poly_volts
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer item, Jresp
      character Cyes*5
      include 'calhw1.cmn'
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes(siddal_davies), Cyes(poly_volts),
     1        Cyes(plot_spect), Cyes(all_modules), ref_wire
2         format( ' Calibration parameters menu' / '   0) Exit' /
     1      '   1) Siddal-Davies law', 22x, a5 /
     2      '   2) Polynomial of voltage', 18x, a5 /
     3      '   3) Force plot of vortex shedding spectrum ', a5 /
     4      '   4) Process all modules in the data file   ', a5 /
     5      '   5) Reference wire (wall-proximity method) ', i2 )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
c
c           RETURN
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           SELECT SIDDAL-DAVIES LAW -- no / yes / query
c
          else if ( item .eq. 1 ) then                                    2m
              siddal_davies = mod( siddal_davies + 1, 3 )
              if ( siddal_davies .ne. NO )
     1            call menu_siddal_davies()
c
c           SELECT POLYNOMIAL OF VOLTAGE -- no / yes / query
c
          else if ( item .eq. 2 ) then                                    2m
              poly_volts = mod( poly_volts + 1, 3 )
              if ( poly_volts .ne. NO )
     1            call menu_poly_volts()
c
c           FORCE PLOT OF SHEDDING SPECTRA -- no / yes / query
c
          else if ( item .eq. 3 ) then                                    2m
              plot_spect = mod( plot_spect + 1, 3 )
c
c           PROCESS ALL MODULES IN THE DATA FILE -- no / yes / query
c
          else if ( item .eq. 4 ) then                                    2m
              all_modules = mod( all_modules + 1, 3 )
c
c           REFERENCE WIRE (WALL-PROXIMITY METHOD)
c
          else if ( item .eq. 5 ) then
              write( STD_OUT, 3 )
3             format( ' Reference wire ? ', $ )
              ref_wire = Jresp( STD_IN, 1, 16, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE MENU_SIDDAL_DAVIES()
c
c  Menu for selecting parameters for Siddal-Davies calibration law
c
c local variables
c  integer
c    item     # selected menu item
c    Jresp    # function : read integer
c  real
c    Qresp    # function : read real
c  character
c    Cyes*5   # function : 'no', 'yes', 'query'
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer item, Jresp
      real Qresp
      character Cyes*5
      include 'calhw1.cmn'
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes(siddal_optim), siddal_expon,
     1        siddal_order, siddal_wexp
2         format( ' Siddal-Davies method parameters' / '   0) Exit' /
     1        '   1) optimized exponent  '  a5 /
     2        '   2) velocity exponent   ', f7.4 /
     3        '   3) Polynomial order    ', i2 /
     4        '   4) Regression weight exponent ', f7.4 /
     5        '   5) Help  ', $ )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
          if ( item .eq. 0 ) then                                         2b
              return
          else if ( item .eq. 1 ) then                                    2m
              siddal_optim = mod( siddal_optim + 1, 3 )
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              siddal_expon = Qresp( STD_IN, 0.25, 1.5, MAX_TRIES )
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
              siddal_order = Jresp( STD_IN, 1, MAX_ORDER, MAX_TRIES )
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              siddal_wexp = Qresp( STD_IN, -2.0, 0.0, MAX_TRIES )
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 4 )
4             format(
     1  ' Extended Siddal-Davies calibration relationship :' /
     2  '    Voltage_squared = Polynomial_of( Air_speed ** exponent )' /
     3  ' For the original Siddal-Davies relation, the velocity ',
     4     'exponent is fixed' /
     5  ' at 0.5 and the polynomial order is 2 . The regression ',
     6     'weight for each' /
     7  ' data point is  Air_speed**exponent (default=0) .' )
              call ilgwait()
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE MENU_POLY_VOLTS()
c
c  Menu for selecting parameters for polynomial calibration law
c
c local variables
c  integer
c    item     # selected menu item
c    Jresp    # function : read integer
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer item, Jresp
      real Qresp
      include 'calhw1.cmn'
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) poly_order, poly_wexp, poly_offset
2         format( ' Polynomial-of-voltage method parameters' /
     Z        '   0) Exit' /
     1        '   1) Polynomial order   ', i2 /
     2        '   2) Regression weight exponent ', f7.4 /
     3        '   3) Abscissa offset    ', f7.4 /
     4        '   4) Help  ', $ )
          item = Jresp( STD_IN, 0, 4, MAX_TRIES )
          if ( item .eq. 0 ) then                                         2b
              return
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              poly_order = Jresp( STD_IN, 1, MAX_ORDER, MAX_TRIES )
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
              poly_wexp = Qresp( STD_IN, -2.0, 0.0, MAX_TRIES )
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
              poly_offset = Qresp( STD_IN, 0.0, 7.5, MAX_TRIES )
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Polynomial calibration relationship :' /
     1            '    Air_speed = Polynomial_of( Voltage - offset )' /
     2            ' The regression weight for each data point is ',
     3                'Air_speed**exponent' / ' (default=-0.5) .' )
              call ilgwait()
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      INTEGER FUNCTION GET_AMPLIF()
c
c       GET amplifier output OFFset voltage
c The returned result is YES if gain and offset values have been calculated
c for all the hot-wires ( otherwise NO ) .
c N.B. You cannot have two input or two output voltages the same .
c
c constants
c  FLAG_ALL=15  # Flags showing which voltages are available
c  FLAG_IN1=1
c  FLAG_IN2=2
c  FLAG_OUT1=4
c  FLAG_OUT2=8
c
c local variables
c  logical
c    Equery             # function : read logical
c    done_all           # gain & offset all amplifiers is known
c  integer
c    got_off(MAX_WIRES)
c    i, j               # looping index
c    item               # selected menu item
c    iwire              # hot-wire number
c    name_len           # length of setup file name
c    nchars             # number of characters in line read from setup file
c    setup_lu           # logical number of setup file
c    jljstr             # function : left justify string
c    Jresp              #          : read integer
c    new_lun            #          : get new logical unit number
c    open_file          #          : open file
c  real
c    Vin_1(MAX_WIRES)   # 1st amp. input voltage
c    Vin_2(MAX_WIRES)   # 2nd amp. input voltage
c    Vout_1(MAX_WIRES)  # 1st amp. output voltage
c    Vout_2(MAX_WIRES)  # 2nd amp. output voltage
c    Qresp              # function : read real
c  character
c    Cline*80           # line of text read from setup file
c    setup_file*32      # name of setup file
c  subroutine
c    Cresp
c    newscr
c    ilgwait
c
      include 'stdio.def'
      include 'calhw1.def'
      integer FLAG_ALL, FLAG_IN1, FLAG_IN2, FLAG_OUT1, FLAG_OUT2
      parameter( FLAG_ALL=15, FLAG_IN1=1, FLAG_IN2=2, FLAG_OUT1=4,
     1    FLAG_OUT2=8 )
c
      logical done_all, Equery
      integer got_off(MAX_WIRES), i, j, item, iwire, name_len, nchars,
     1    setup_lu, jljstr, Jresp, new_lun, open_file
      real Vin_1(MAX_WIRES),  Vin_2(MAX_WIRES),
     1     Vout_1(MAX_WIRES), Vout_2(MAX_WIRES), Qresp
      character Cline*80, setup_file*32
      include 'calhw1.cmn'
      save got_off, iwire, Vin_1, Vin_2, Vout_1, Vout_2, setup_file
      data name_len, iwire, got_off / 1, 1, MAX_WIRES*0 /
      data setup_file / ' ' /
c
      iwire = max( 1, min( iwire, n_offsets ) )
c
c     start of loop                                                       1b
1         call newscr()
          if ( got_off(iwire) .eq. FLAG_ALL ) then                        2b
              Amp_gain(iwire) = ( Vout_2(iwire) - Vout_1(iwire) ) /
     1            ( Vin_2(iwire)  - Vin_1(iwire) )
              Amp_offset(iwire) = Vout_1(iwire) - Amp_gain(iwire) *
     1            Vin_1(iwire)
              end if                                                      2e
          write( STD_OUT, 2 ) n_offsets, iwire, Vin_1(iwire),
     1        Vout_1(iwire), Vin_2(iwire), Vout_2(iwire),
     2        setup_file(:name_len)
2         format( ' Amplifier output offset menu ' / '   0) exit' /
     1        '   1) number of hot-wires     ', i2 /
     2        '   2) selected hot-wire       ', i2 /
     3        '   3) first input voltage    ', f8.4 /
     4        '   4) first output voltage   ', f8.4 /
     5        '   5) second input voltage   ', f8.4 /
     6        '   6) second output voltage  ', f8.4 /
     7        '   7) clear all' /
     8        '   8) Set up file', 12x, a /
     9        '   9) Read setup file' /
     a        '  10) Write setup file' /
     b        '  11) Help  ', $ )
          if ( got_off(iwire) .eq. FLAG_ALL ) then                        3b
              write( STD_OUT, 3 ) Amp_gain(iwire), Amp_offset(iwire)
3             format( '      amplifier gain ', f9.4 /
     1                '      output offset  ', f9.4 )
          else                                                            3m
              write( STD_OUT, 4 )
4             format( ' Cannot calculate gain or offset ' )
              end if                                                      3e
          item = Jresp( STD_IN, 0, 11, MAX_TRIES )
c
c               RETURN
c
          if ( item .eq. 0 ) then                                         2b
              done_all = .TRUE.
              do 5 item = 1, n_offsets                                    3b
                  done_all = done_all .and.
     1                got_off(item) .eq. FLAG_ALL
5                 continue                                                3e
              get_amplif = YES
              if ( .not. done_all ) then                                  3b
                  if ( .not. Equery( STD_IN,
     1                  'Amplifier(s) not specified . Really quit ?' ) )
     2                goto 1
                  get_amplif = NO
                  end if                                                  3e
              return
c
c               NUMBER OF HOT-WIRES
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Value ? ', $ )
              n_offsets = Jresp( STD_IN, 1, 16, MAX_TRIES )
              iwire = max( 1, min( iwire, n_offsets ) )
c
c               SELECTED HOT-WIRE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 6 )
              iwire = Jresp( STD_IN, 1, n_offsets, MAX_TRIES )
c
c               FIRST INPUT VOLTAGE
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 6 )
              Vin_1(iwire) = Qresp( STD_IN, -15.0, 15.0, MAX_TRIES )
              if ( abs( Vin_1(iwire) - Vin_2(iwire) ) .gt. 0.01 )
     1            got_off(iwire) = ior( got_off(iwire), FLAG_IN1 )
c
c               FIRST OUTPUT VOLTAGE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 6 )
              Vout_1(iwire) = Qresp( STD_IN, -15.0, 15.0, MAX_TRIES )
              if ( abs( Vout_1(iwire) - Vout_2(iwire) ) .gt. 0.01 )
     1            got_off(iwire) = ior( got_off(iwire), FLAG_OUT1 )
c
c               SECOND INPUT VOLTAGE
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 6 )
              Vin_2(iwire) = Qresp( STD_IN, -15.0, 15.0, MAX_TRIES )
              if ( abs( Vin_1(iwire) - Vin_2(iwire) ) .gt. 0.01 )
     1            got_off(iwire) = ior( got_off(iwire), FLAG_IN2 )
c
c               SECOND OUTPUT VOLTAGE
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 6 )
              Vout_2(iwire) = Qresp( STD_IN, -15.0, 15.0, MAX_TRIES )
              if ( abs( Vout_1(iwire) - Vout_2(iwire) ) .gt. 0.01 )
     1            got_off(iwire) = ior( got_off(iwire), FLAG_OUT2 )
c
c               CLEAR AMPLIFIER SETTINGS - RESPECIFY
c
          else if ( item .eq. 7 ) then                                    2m
              do 7 i = 1, MAX_WIRES                                       3b
                  got_off(i) = 0
7                 continue                                                3e
c
c               SPECIFY SET UP FILE
c
          else if ( item .eq. 8 ) then                                    2m
              write( STD_OUT, 6 )
              call Cresp( STD_IN, setup_file, MAX_TRIES )
              name_len = max( 1, jljstr( setup_file ) )
c
c               READ SETUP FILE
c
          else if ( item .eq. 9 ) then                                    2m
c               open the setup file
              setup_lu = open_file( new_lun(), setup_file, 'old', 0 )
              if ( setup_lu .gt. 0 ) then                                 3b
                  do 10 j = 1, n_offsets                                  4b
c
c               read a line from the setup file
8                     read( setup_lu, 9, END=11, ERR=20 ) Cline
9                     format( a )
c               skip and read the next line if it is too short or is a comment
                      nchars = jljstr( cline )
                      if ( nchars .lt. 9 .or. cline(1:1) .eq. ';' )
     1                    goto 8
c               decode the data
                      read( cline, *, ERR=22 ) i,
     1                    Vin_1(i), Vout_1(i), Vin_2(i), Vout_2(i)
                      got_off(i) = FLAG_ALL
10                    continue                                            4e
11                close( UNIT=setup_lu )
              else                                                        3m
                  call ilgwait()
                  end if                                                  3e
c
c               WRITE SETUP FILE
c
          else if ( item .eq. 10 ) then                                   2m
c
c               open setup file
              setup_lu = open_file( new_lun(), setup_file, 'new', 0 )
              if ( setup_lu .gt. 0 ) then                                 3b
                  write( setup_lu, 12, ERR=24 ) ( i, Vin_1(i),
     1                Vout_1(i), Vin_2(i), Vout_2(i), i = 1, n_offsets )
12                format( ' ;   AMPLIFIER SPECIFICATION' /
     1                ' ; wire  Vin_1     Vout_1    Vin_2     Vout_2' /
     2                ( 1x, i4, 4f9.4 ) )
                  if ( setup_lu .eq. STD_OUT ) then                       4b
                      call ilgwait()
                  else                                                    4m
                      close( UNIT=setup_lu )
                      end if                                              4e
              else                                                        3m
                  call ilgwait()
                  end if                                                  3e
c
c               HELP
c
          else if ( item .eq. 11 ) then                                   2m
              write( STD_OUT, 13 )
13            format( ' The gain and offset of the amplifier matching ',
     1              'the CTA output to the A/D' /
     2            ' converter are specified by measuring two ',
     3              'different (input,output) voltage' /
     4            ' pairs. For convenience, these amplifier ',
     5              'properties can be written to and' /
     6            ' then read from a file using menu items 8, 9 & 10.' )
              call ilgwait()
              end if                                                      2e
          goto 1
c
c               ERROR TRAPS
c
20        write( STD_ERR, 21 ) setup_file(:name_len)
21        format( ' Cannot read setup file ', a )
          close( UNIT=setup_lu )
          call ilgwait()
          goto 1
c
22        write( STD_ERR, 23 ) setup_file(:name_len)
23        format( ' Cannot decode setup file ', a )
          close( UNIT=setup_lu )
          call ilgwait()
          goto 1
c
24        write( STD_ERR, 25 ) setup_file(:name_len)
25        format( ' Cannot write to setup file ', a )
          close( UNIT=setup_lu )
          call ilgwait()
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE PARSE_FILE_STRUCTURE( DATA_FILE, CALIB_FILE )
c
c Reads each hot-wire calibration module in turn and parses the sequence
c of blocks and sub-blocks until the sub-blocks containing the calibration
c data are encountered . This routine does not actually read the calibration
c -- that job is done by READ_CALIB_DATA which is called by this routine .
c Formulation of the calibration relations is performed by PROCESS DATA  .
c
c All raw data file handling is confined to PARSE_FILE_STRUCTURE and
c READ_CALIB_DATA .
c
c Arguments
c  character data_file*(*)    # name of data file
c            calib_file*(*)   # name of calibration file
c
c local variables
c  integer
c    module_no      # module number containing a calibration
c    unused         # unused address for calling ERDFACQ
c  logical
c    check_struct   # function : tests for checking structure of data file
c    erdfacq        # rdflib   : read data sub-block
c    erdfmhd        # rdflib   : read data module header
c    erdfopn        # rdflib   : open raw data file
c    Equery         #          : read boolean
c    read_calib_data#          : read calibration data into velocity &
c                   #            voltage arrays
c  real
c    air_press      # ambient air condition
c    air_temp       # ""
c    qmuair         # function : dynamic viscosity of air
c    qresp          #          : read real
c  character
c    Csublab*8      # sub-block header label
c  subroutine
c    srdfcls        # rdflib : close data file
c    srdfmute       #        : set verbosity
c    srdfhost       #        : set source machine
c    srdfinit       #        : read initialize module data
c    process_data   # formulate calibration relations
c
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer module_no, unused
      logical check_struct, erdfacq, erdfmhd, erdfopn, Equery,
     1    read_calib_data
      character data_file*(*), calib_file*(*), Csublab*8
      real air_press, air_temp, qmuair, qresp
c
      include 'calhw1.cmn'
c
      if ( .not. erdfopn( data_file, rdf_rec, aux_lu ) ) then             1b
         write( STD_OUT, 1 )
1        format( ' CALHW1 : Cannot open data file ' )
         return
         end if                                                           1e
      call srdfmute( verbosity )
      call srdfhost( host_type )
c
c             get ambient conditions
c
      if ( erdfmhd( '#initial', rdf_rec ) ) then                          1b
          call srdfinit( air_press, air_temp, rdf_rec )
      else                                                                1m
          write( STD_OUT, 10 )
10        format( ' Initialize module not found' /
     1        ' What is the barometric pressure ? (Pascals) ', $ )
          air_press = Qresp( STD_IN, 90.0e3, 110.0e3, 4 )
          write( STD_OUT, 11 )
11        format( ' What is the ambient temperature ? (Celsius) ', $ )
          air_temp = 273.15 + Qresp( STD_IN, 0.0, 40.0, 4 )
          end if                                                          1e
      air_density = air_press / air_temp / AIR_GAS_CONST
      air_viscosity = qmuair( air_temp ) / air_density
c
c             main loop : one per calibration
c
      module_no = 0
2     if ( erdfmhd( '#calhw1 ', rdf_rec ) ) then                          1b
c
c             skip calibration module if it is not required
c
         module_no = module_no + 1
         write( STD_OUT, 3 ) module_no
3        format( ' Found hot-wire calibration module number ', i3 )
         if ( all_modules .ne. YES ) then                                 2b
            if ( .not. Equery( STD_IN,
     1                         'Continue with this calibration ? ' ) )
     2         goto 2
               end if                                                     2e
c
c             reset the structure checking routine
         call check_struct( ' ', ' ', CHECK_RESET )
c
c             read preliminary block ; pause sub-block
c                 ( establishes upper limit )
c
         if ( .not. erdfacq( Csublab, unused, unused, 0, rdf_rec,
     1                       .FALSE. ) ) then                             2b
            write( STD_OUT, 4 )
4           format( ' CALHW1 : No blocks in module ' )
         else if ( check_struct( '##prelim', '###pause', 1 ) ) then       2m
c
c             read preliminary block ; pause sub-block
c                 ( establishes method junction point )
c
            if ( .not. erdfacq( Csublab, unused, unused, 0,
     1                          rdf_rec, .FALSE. ) ) then                 3b
                  write( STD_OUT, 5 )
5                 format( ' CALHW1 : No junction point sub-block' )
            else if ( check_struct( '##prelim', '###pause', 2 ) ) then    3m
c
c             read preliminary block ; pause sub-block
c                 ( establishes lower limit )
c
               if ( .not. erdfacq( Csublab, unused, unused, 0,
     1                             rdf_rec, .FALSE. ) ) then              4b
                     write( STD_OUT, 6 )
6                    format( ' CALHW1 : No lower limit sub-block' )
               else if ( check_struct( '##prelim', '###pause', 3 ) )then  4m
c
c               read preliminary block ; pause sub-block
c                   ( wait for operator )
c
                  if ( .not. erdfacq( Csublab, unused, unused, 0,
     1                                rdf_rec, .FALSE. ) ) then           5b
                     write( STD_OUT, 7 )
7                   format( ' CALHW1 : No wait for operator sub-block' )
                  else if ( check_struct( '##prelim', '###pause', 4 )
     1                    ) then                                          5m
c
c               read traverse ; put probe into free stream
c
                     if ( .not. erdfacq( Csublab, unused, unused,
     1                        0, rdf_rec, .FALSE. ) ) then                6b
                        write( STD_OUT, 8 )
8                       format( ' CALHW1 : no traverse sub-block' )
                     else if ( check_struct( '##prelim', '###trav ', 4 )
     1                       ) then                                       6m
c
c               read data from the main calibration blocks
c               and then process the data to produce the calibrations
c
                        if ( read_calib_data() ) then                     7b
                            call process_data( calib_file )
                            end if                                        7e
                        end if                                            6e
                     end if                                               5e
                  end if                                                  4e
               end if                                                     3e
            end if                                                        2e
         goto 2
         end if                                                           1e
c        end of module loop
      call srdfcls()
      call write_calibr( MAGIC, calib_file, WRITE_CLOSE )
      return
      end



      LOGICAL FUNCTION CHECK_STRUCT( REQ_BLOCK, REQ_SUBBLK, BLOCK_NO )
c
c             acceptance tests for 1st preliminary block
c
c  Details
c REQ_BLOCK REQ_SUBBLK BLOCK_NO  ICHAN   CADID      COMMENTS
c   prelim     pause      1       any     any    upper limit established
c   prelim     pause      2       any     any    junction point established
c   prelim     pause      3       same    same   lower limit established
c   prelim     pause      4       ---     ---    wait for operator
c   prelim     trav       5       ---     ---    put probe in free stream
c In these tests REQ_BLOCK must match the actual block type -- except if
c REQ_BLOCK is '##main  ' and the actual block is '##extra ' .
c REQ_SUBBLK must be the same as CHECK_SUBLAB unless in the 4th or succeeding
c block, REQ_SUBBLK is '###2dat2' and CHECK_SUBLAB is '###1dat1' .
c If primary sub-block check number of chans and channel address list
c
c Arguments
c  character req_block*8   # required block label
c            req_subblk*8  # required sub-block label
c  integer block_no        # block number
c
c local variables
c  integer
c    prelim_adid  # A/D converter used for setup pause
c    prelim_chan  # A/D channel used for setup pause
c    Jextadid     # function : identify A/D converter
c    Jextchls     #          : get A/D chan addr list
c  logical
c    first_1dat1  # first hot-wire data is still to be read
c    first_2dat2  # first baratron data is still to be read
c    first_pause  # first monitoring pause is still to be read
c    check_baratron#function : check baratron sub-block header
c    check_monitor#          : check monitor sub-block header
c    check_wire   #          : check hot-wire sub-block header
c    eextamon     #          : automatic monitor
c  real
c    qextamon     # function : get averaged monitor value
c  character
c    message*30   # error message string
c    Gextblk*8    #          : get block label
c    Gextsubl*8   #          : get sub-block label
c  subroutine
c    sextall      # display sub-block header
c
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer block_no, prelim_adid, prelim_chan, Jextadid, Jextchls
      logical first_1dat1, first_2dat2, first_pause,
     1    check_baratron, check_monitor, check_wire, eextamon
      real qextamon
      character req_block*(*), req_subblk*(*), message*32,
     1    Gextblk*8, Gextsubl*8
      include 'calhw1.cmn'
      save prelim_adid, prelim_chan,
     1    first_1dat1, first_2dat2, first_pause
      data first_1dat1, first_2dat2, first_pause / 3*.TRUE. /
c
c
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       reset - clear memory of reading data
c
      if ( block_no .eq. CHECK_RESET ) then                               1b
          first_pause = .TRUE.
          first_2dat2 = .TRUE.
          first_1dat1 = .TRUE.
          check_struct = .TRUE.
          return
          end if                                                          1e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       check block & sub-block labels
c
      write( message, 1 ) req_block, min( max( block_no, -99 ), 999 )
1     format( ' CALHW1 : ', a8, ' number ', i3, '  ' )
      check_struct = .FALSE.
      if ( Gextblk() .ne. req_block ) then                                1b
          write( STD_OUT, 2 ) message, Gextblk(), 'required'
2         format( 5a )
      else if ( Gextsubl() .ne. req_subblk ) then                         1m
              write( STD_OUT, 2 ) message, req_subblk, ' : ',
     1            Gextsubl(), 'required'
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       first pause is for maximum speed
c        -- manual monitoring pause is required, record the A/D & channel
c
      else if ( block_no .eq. 1 ) then                                    1m
          if ( qextamon() .eq. 999999.9e29 .or. eextamon() ) then         2b
              write( STD_OUT, 2 ) message,
     1            ' Need manual monitoring pause'
          else                                                            2m
              prelim_adid = Jextadid()
              prelim_chan = Jextchls( 1 )
              check_struct = .TRUE.
              end if                                                      2e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     second or third pause for junction point or minimum speed
c        -- manual monitoring pause is required, check the A/D & channel
c
      else if ( block_no .eq. 2 .or. block_no .eq. 3 ) then               1m
          if ( qextamon() .eq. 999999.9e29 ) then                         2b
              write( STD_OUT, 2 ) message,
     1            ' Need manual monitoring pause'
          else if ( prelim_adid .ne. Jextadid() ) then                    2m
              write( STD_OUT, 2 ) message, 'A/D converter changed'
          else if ( prelim_chan .ne. Jextchls(1) ) then                   2m
              write( STD_OUT, 2 ) message,
     1               'Baratron channel changed'
          else                                                            2m
              check_struct = .TRUE.
              end if                                                      2e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     the fourth pause or the traverse sub-block
c
      else if ( block_no .eq. 4 ) then                                    1m
          if ( Gextsubl() .eq. '###pause' ) then                          2b
              if ( qextamon() .ne. 999999.9e29 ) then                     3b
                  write( STD_OUT, 2 ) message, 'Pause must be simple'
              else                                                        3m
                  check_struct = .TRUE.
                  end if                                                  3e
          else                                                            2m
              check_struct = .TRUE.
              end if                                                      2e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       check block containing pitot-tube/hot-wire data
c
      else if ( block_no .eq. PITOT_STATIC ) then                         1m
c
c       automatic monitored pause of the baratrom or hot-wire
          if ( Gextsubl() .eq. '###pause' ) then                          2b
              check_struct = check_monitor( first_pause, message )
c
c       baratron signal
          else if ( Gextsubl() .eq. '###2dat2' ) then                     2m
              check_struct = check_baratron( first_2dat2, message )
c
c       hot-wire signal
          else if ( Gextsubl() .eq. '###1dat1' ) then                     2m
              check_struct = check_wire( first_1dat1, block_no,
     1            message )
              end if                                                      2e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       check block containing vortex-shedding/hot-wire data
c
      else if ( block_no .eq. VORTEX_SHEDDING ) then                      1m
c
c       automatic monitored pause of the baratrom or hot-wire
          if ( Gextsubl() .eq. '###pause' ) then                          2b
              check_struct = check_monitor( first_pause, message )
c
c       hot-wire signal
          else if ( Gextsubl() .eq. '###1dat1' ) then                     2m
              check_struct = check_wire( first_1dat1, block_no,
     1            message )
              end if                                                      2e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       check block containing wall-proximity/hot-wire data
c
      else if ( block_no .eq. WALL_PROXIMITY ) then                       1m
c
c       hot-wire signal
          if ( Gextsubl() .eq. '###1dat1' ) then                          2m
              check_struct = check_wire( first_1dat1, block_no,
     1            message )
              end if                                                      2e
      else                                                                1m
          write( STD_OUT, 2 ) message,
     1        'CALHW1 : BUG : Incorrect call to CHECK_STRUCT'
          end if                                                          1e
      if ( verbosity .gt. 0 )
     1    call sextall
      return
      end



      LOGICAL FUNCTION CHECK_MONITOR( FIRST_PAUSE, ERR_MSG )
c
c Check that the pause sub-block header is ok .
c
c Arguments
c  logical   first_pause  # this pause is the first encountered
c  character err_msg*(*)  # error message
c
c local variable
c  logical
c    eextamon     # function : is automatic monitor pause
c  integer
c    pause_adid   # identity of A/D converter
c    pause_chan   # A/D channel
c    Jextadid     # function : identifies A/D converter
c    Jextchls     #          : lists A/D channels
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical first_pause, eextamon
      integer pause_adid, pause_chan, Jextadid, Jextchls
      character err_msg*(*)
      save pause_adid, pause_chan
c
      check_monitor = .FALSE.
      if ( .not. eextamon() ) then
          write( STD_OUT, 1 ) err_msg, 'Need automatic monitor pause'
1         format( a, 1x, a )
      else if ( first_pause ) then
          pause_adid = Jextadid()
          pause_chan = Jextchls( 1 )
          first_pause = .FALSE.
          check_monitor = .TRUE.
      else if ( pause_adid .ne. Jextadid() ) then
          write( STD_OUT, 1 ) err_msg,
     1        'Monitoring A/D converter changed'
      else if ( pause_chan .ne. Jextchls( 1 ) ) then
          write( STD_OUT, 1 ) err_msg, 'Monitoring A/D channel changed'
      else
          check_monitor = .TRUE.
          end if
      return
      end



      LOGICAL FUNCTION CHECK_BARATRON( FIRST_2DAT2, ERR_MSG )
c
c Check that the baratron (secondary) sub-block header is ok .
c
c Arguments
c  logical   first_2dat2  # this secondary data sub-block is first encountered
c  character err_msg*(*)  # error message
c
c local variables
c  integer
c    dat22_adid   # identity of A/D converter
c    dat22_chan   # A/D channel
c    Jextadid     # function : identifies A/D converter
c    Jextchls     #          : lists A/D channels
c    Jextcyln     #          : A/D cycle length
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical first_2dat2
      integer dat22_adid, dat22_chan, Jextadid, Jextchls, Jextcyln
      character err_msg*(*)
      save dat22_adid, dat22_chan
c
      check_baratron = .FALSE.
      if ( Jextcyln() .ne. 1 ) then
          write( STD_OUT, 1 ) err_msg,
     1        'Can only have one Baratron channel'
1         format( a, 1x, a )
      else if ( first_2dat2 ) then
          dat22_adid = Jextadid()
          dat22_chan = Jextchls( 1 )
          first_2dat2 = .FALSE.
          check_baratron = .TRUE.
      else if ( dat22_adid .ne. Jextadid() ) then
          write( STD_OUT, 1 ) err_msg, 'Baratron A/D converter changed'
      else if ( dat22_chan .ne. Jextchls(1) ) then
          write( STD_OUT, 1 ) err_msg, 'Baratron A/D channel changed'
      else
          check_baratron = .TRUE.
          end if
      return
      end



      LOGICAL FUNCTION CHECK_WIRE( FIRST_1DAT1, BLOCK_ID, ERR_MSG )
c
c Checks that the hot-wire (primary) sub-block header is ok .
c
c Arguments
c  logical   first_1dat1  # this secondary data sub-block is first encountered
c  integer   block_id     # PITOT_STATIC, VORTEX_SHEDDING, WALL_PROXIMITY
c  character err_msg*(*)  # error message
c
c local variables
c  integer
c    cycle_len    # actual number of hot-wires
c    dat11_adid   # identity of A/D converter
c    dat11_chan(MAX_WIRES)  # A/D channel
c    dat11_cyln   # required number of hot-wires
c    i            # looping index
c    Jextadid     # function : identifies A/D converter
c    Jextchls     #          : lists A/D channels
c    Jextcyln     #          : A/D cycle length
c  logical
c    eextkbdv     # function : is there a keyboard reference value
c
      include 'stdio.def'
      include 'calhw1.def'
      character err_msg*(*)
      integer block_id, cycle_len, dat11_adid, dat11_chan(MAX_WIRES),
     1    dat11_cyln, i, Jextadid, Jextchls, Jextcyln
      logical first_1dat1, eextkbdv
      save dat11_adid, dat11_chan, dat11_cyln
c
      check_wire = .FALSE.
c
c       keyboard reference value is used for vortex shedding or wall
c       proximity method
c
      if ( block_id .eq. VORTEX_SHEDDING .or.
     1        block_id .eq. WALL_PROXIMITY ) then
          if ( .not. eextkbdv() ) then
              write( STD_OUT, 1 ) err_msg,
     1            'Keyboard reference value required'
1             format( a, 1x, a )
              return
              end if
          end if
c
c       an extra A/D channel is used for the vortex shedding method
c
      if ( block_id .eq. VORTEX_SHEDDING ) then
          cycle_len = Jextcyln() - 1
      else
          cycle_len = Jextcyln()
          end if
c
c       check the A/D converter, number of channels, and list of channels
c
      if ( first_1dat1 ) then
          dat11_adid = Jextadid()
          dat11_cyln = cycle_len
          do 2 i = 1, dat11_cyln
              dat11_chan( i ) = Jextchls( i )
2             continue
          first_1dat1 = .FALSE.
          check_wire = .TRUE.
c       assume Jextadid returns 1, 2 or 3
      else if ( dat11_adid .ne. Jextadid() .and. .not.
     1        ( dat11_adid .gt. 1 .and. Jextadid() .gt. 1 ) ) then
          write( STD_OUT, 1 ) err_msg, 'Hot-wire A/D converter changed'
      else if ( dat11_cyln .ne. cycle_len ) then
          write( STD_OUT, 1 ) err_msg, 'Number of hot-wires changed'
      else
          check_wire = .TRUE.
          do 3 i = 1, dat11_cyln
              check_wire = check_wire .and.
     1            dat11_chan(i) .eq. Jextchls(i)
3             continue
          if ( .not. check_wire )
     1        write( STD_OUT, 1 ) err_msg,
     2            'Hot-wire A/D channel changed'
          end if
      return
      end



      LOGICAL FUNCTION READ_CALIB_DATA()
c
c               read data from the main calibration blocks
c
c constants
c   MAX_DATA = 4000             # maximum number of A/D samples
c   MAX_WORK = 10000            # maximum size of work array
c   TEST_SECTION_WIDTH = 228.73 # test section width in mm
c
c local variables
c  logical
c    first_wire       # have not read the first hot-wire data
c    offset_ok        # amplifier offsets and gains are OK
c    check_struct     # function : tests for data file structure
c    EBAR_AD_toP      #          : convert A/D readings to pressure
c    equery           #          : read logical value
c    erdfacq          #          : read data sub-block
c    eyes             #          : NO/YES/QUERY to logical
c  integer *2
c    Mdata(MAX_DATA)  # raw data array
c  integer
c    first_prox       # index of 1st air-speed/hot-wire values containing
c    last_proxx       # index of last air-speed/hot-wire values containing
c                     #     wall proximity calibration data
c    i                # looping indices
c    npts             # number of calibration (air-speed,voltage) points
c    n_samp           # number data samples read by erdfacq
c    n_signal         # number of vortex shedding signal data samples
c    source           # source index for copying vortex shedding signal data
c    step             # increment for source
c    get_amplif       # function : get amplifier gains & offsets
c    Jextcyln         #          : A/D cycle length
c    Jextnocy         #          : no. of cycles
c  real
c    dyn_press(MAX_DATA)  # Baratron pressure readings
c    max_signal       # maximum value of vortex shedding signal
c    min_signal       # minimum value of vortex shedding signal
c    to_volts         # conversion factor for A/D value to voltage
c    ts_dyn_press     # free stream dynamic pressure in  the test section
c    vortex_wire(MAX_DATA),   # vortex shedding signal
c    work(MAX_WORK)   # work array
c    AD_to_volts      # function : conversion factor for A/D values to volts
c    Qextcytm         #          : cycle period
c    Qextkbdv         #          : get keyboard reference value
c    qmean            #          : mean of a real array
c    qtts_dyn         #          : calculate T/S dynamic pressure
c    qvkvs_raw        #          : air speed from vortex street freq
c  character
c    Csublab*8        # sub-block header label
c    Gextblk*8        # function : get block header label
c    Gextadid*12      #          : identify A/D converter
c  subroutine
c    smeani           # mean of integers
c
c
      include 'stdio.def'
      include 'calhw1.def'
      integer MAX_DATA, MAX_WORK
      real TEST_SECTION_WIDTH
      character NAME*19
      parameter( MAX_DATA=4000, MAX_WORK=10000,
     1    TEST_SECTION_WIDTH=228.73, NAME=' READ_CALIB_DATA : ' )
c
      logical first_wire, offset_ok,
     1    check_struct, EBAR_ADtoP, erdfacq, equery, eyes
      integer *2 Mdata(MAX_DATA)
      integer first_prox, i, last_prox, npts, n_samp, n_signal, source,
     1    step, get_amplif, Jextcyln, Jextnocy
      real dyn_press(MAX_DATA), max_signal, min_signal, to_volts,
     1    ts_dyn_press, vortex_wire(MAX_DATA), work(MAX_DATA),
     2    AD_to_volts, Qextcytm, Qextkbdv, qmean, qtts_dyn, qvkvs_raw
      character Csublab*8, Gextblk*8, Gextadid*12
c
      include 'calhw1.cmn'
      equivalence ( dyn_press(1), vortex_wire(1) ),
     1            ( Mdata(1), work(1) )
c
c           read main block ; pause sub-block
c           ( wait for correct W/T speed )
c
      read_calib_data = .FALSE.
      npts = 1
      first_wire = .TRUE.
      first_prox = 0
      last_prox  = 0
c     start of loop
1     if ( erdfacq( Csublab, n_samp, Mdata, MAX_DATA, rdf_rec, .FALSE. )
     1        ) then                                                      1b
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           For a ##main block and ###pause sub-block, the air speed is
c           measured with a pitot-static tube . The ###pause sub-block
c           is used to set the air speed . This is followed by a ###2dat2
c           sub-block for measuring the air speed and then a ###1dat1
c           sub-block for recording the hot-wire  voltages .
c
c           read the ###pause
c
          if ( Gextblk() .eq. '##main  ' .and.
     1            Csublab .eq. '###pause' ) then                          2b
              if ( .not. check_struct( '##main  ', '###pause',
     1                PITOT_STATIC ) ) then                               3b
                  write( STD_OUT, 10 ) name
10                format( a, 'Defect in ###pause sub-block for pitot-',
     1                'tube/hot-wire calibration' )
c
c           read baratron data -- check first
c
              else if ( .not. erdfacq( Csublab, n_samp, Mdata, MAX_DATA,
     1                rdf_rec, .FALSE. ) ) then                           3m
                  write( STD_OUT, 11 ) name
11                format( a,
     1                'Pitot-tube data ###2dat2 sub-block not found' )
              else if ( .not. check_struct( '##main  ', '###2dat2',
     1                PITOT_STATIC ) ) then                               3m
                  write( STD_OUT, 12 ) name
12                format( a,
     1                'Defect in ###2dat2 pitot-tube data sub-block' )
c
c           now we have the baratron reading calculate
c           1) pre-centrebody pressures (convert A/D reading to pressure)
c           2) mean pre-centrebody pressure
c           3) free stream dynamic pressure
c           4) free stream air speed
c
              else if ( EBAR_ADtoP( Mdata, dyn_press, 1, n_samp ) ) then  3m

                  ts_dyn_press = qtts_dyn(
     1                press_conv * qmean( dyn_press, 1, n_samp),
     2                iplug, trip_diam )
                  if ( ts_dyn_press .gt. 0.0 ) then                       4b
                      calib(npts,AIR_SPEED,1) =
     1                    sqrt( 2.0 / air_density * ts_dyn_press )
                  else                                                    4m
                      calib(npts,AIR_SPEED,1) = 0.0
                      end if                                              4e
c
c             read main block ; primary data sub-block
c                 ( get hot wire samples )
c
                  if ( .not. erdfacq( Csublab, n_samp, Mdata, MAX_DATA,
     1                    rdf_rec, .FALSE. ) ) then                       4b
                     write( STD_OUT, 13 ) name
13                   format( a,
     1                   'Hot-wire data ###1dat1 sub-block not found' )
                  else if ( .not. check_struct( '##main  ', '###1dat1',
     1                    PITOT_STATIC ) ) then                           4m
                      write( STD_OUT, 14 ) name
14                    format( a,
     1                    'Defect in ###1dat1 hot-wire data sub-block' )
                  else                                                    4b
c
c             for the first hot-wire data, determine the number of
c             wires and the conversion-to-volts factor
c
                      if ( first_wire ) then                              5b
                          n_wires = Jextcyln()
                          to_volts = AD_to_volts()
                          Cwire_ad = Gextadid()
                          first_wire = .FALSE.
                          end if                                          5e
c
c             calculate mean voltage
c
                      call mean_hw_volts( Mdata, npts, to_volts )
                      npts = npts + 1
                      end if                                              4e     4e
                  end if                                                  3e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c             For a ##extra block and ##pause sub-block, the air speed
c             is measured by frequency of vortex shedding from a cylinder .
c             The ###pause sub-block is used to set the air speed . This
c             is followed by a ###1dat1 sub-block containing all hot-wire
c             data -- the vortex shedding signal is recorded on an extra
c             A/D channel . The diameter of the cylinder is entered by
c             the user .
c
          else if ( Gextblk() .eq. '##extra ' .and.
     1              Csublab   .eq. '###pause' ) then                      2m
              if ( .not. check_struct( '##extra ', '###pause',
     1                VORTEX_SHEDDING ) ) then                            3b
                  write( STD_OUT, 20 ) name
20                format( a, 'Defect in ###pause sub-block for ',
     1                'vortex-shedding/hot-wire calibration' )
c
c             read main block ; primary data sub-block
c                 ( get hot wire samples )
c
              else if ( .not. erdfacq( Csublab, n_samp, Mdata, MAX_DATA,
     1                rdf_rec, .FALSE. ) ) then                           3m
                  write( STD_OUT, 13 ) name
              else if ( .not. check_struct( '##extra ', '###1dat1',
     1                VORTEX_SHEDDING ) ) then                            3m
                  write( STD_OUT, 14 )
              else                                                        3m
c
c             for the first hot-wire data, determine the number of
c             wires and the conversion-to-volts factor
c
                  if ( first_wire ) then                                  4b
                      n_wires = Jextcyln() - 1
                      to_volts = AD_to_volts()
                      Cwire_ad = Gextadid()
                      first_wire = .FALSE.
                      end if                                              4e
c
c             extract signal from vortex-detecting wire
c             record the maximum and minimum signal levels
c
                  source = Jextcyln()
                  step = source
                  max_signal = Mdata(source)
                  min_signal = Mdata(source)
                  n_signal = Jextnocy()
                  do 21 i = 1, n_signal                                   4b
                      vortex_wire(i) = Mdata(source)
                      source = source + step
21                    continue                                            4e
                  call range_data( vortex_wire, 1, n_signal,
     1                min_signal, max_signal )
c
c             check for a constant or clipped vortex shedding signal
c
                  if ( max_signal .le. min_signal + 2.0 ) then            4b
                      if ( max_signal .le. 0.1 .or.
     1                     min_signal .ge. 4094.9 ) then                  5b
                          write( STD_OUT, 22 ) name,
     1                        min_signal / to_volts
22                        format( a, 'WARNING : Vortex shedding signal',
     1                        ' clipped at ', f7.3, ' Volts' )
                      else                                                5m
                          write( STD_OUT, 23 ) name,
     1                        min_signal / to_volts
23                        format( a, 'WARNING : Vortex shedding signal',
     1                        ' constant at ', f7.3, ' Volts' )
                          end if                                          5e
                  else                                                    4m
c
c             calculate mean voltage
c             This must be done before calculating free stream speed because
c             QVKVS_RAW corrupts Mdata which occupies same address space as
c             WORK
c
                      call mean_hw_volts( Mdata, npts, to_volts )
c
c             calculate free stream air speed
c
                      calib(npts,AIR_SPEED,1) =
     1                    qvkvs_raw( vortex_wire, 1, n_signal,
     2                        Qextcytm(), Qextkbdv() / 1000.0,
     3                        air_viscosity, work, Eyes( plot_spect,
     4                        'Plot vortex shedding spectrum ?' ) )
                      npts = npts + 1
                      end if                                              4e
                  end if                                                  3e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c             For a ##main block and a ###1dat1 sub-block, the air speed
c             is not measured . This part of the calibration depends on
c             the relationship between distance of the hot-wire(s) from
c             the wall and the hot-wire voltage due to wall proximity
c             effect . This is a method for extending the calibration to
c             extremely low speeds .
c
          else if ( Gextblk() .eq. '##main  ' .and.
     1              Csublab   .eq. '###1dat1' ) then                      2m
              if ( .not. check_struct( '##main', '###1dat1',
     1                WALL_PROXIMITY ) ) then                             3b
                  write( STD_OUT, 30 ) name
30                format( a, 'Defect in ###1dat1 sub-block for wall-',
     1                'proximity/hot-wire calibration' )
              else                                                        3m
                  if ( first_wire ) then                                  4b
                      n_wires = Jextcyln()
                      to_volts = AD_to_volts()
                      Cwire_ad = Gextadid()
                      first_wire = .FALSE.
                      end if                                              4e
                  if ( first_prox .eq. 0 )
     1                first_prox = npts
                  last_prox = npts
                  calib(npts,AIR_SPEED,1) = 1.0 / Qextkbdv()
c
c             calculate mean voltage
c
                  call mean_hw_volts( Mdata, npts, to_volts )
                  npts = npts + 1
                  end if                                                  3e
              end if                                                      2e
          goto 1
          end if                                                          1e
c         end of loop
      n_points(1) = npts - 1
c
c             At this point we should have some air-speed/hot-wire-voltage
c             coordinates . Proceed only if more than 3 data points were read .
c
      if ( n_points(1) .lt. 3 ) then                                         1b
          write( STD_OUT, 2 ) n_points(1)
2         format( 1x, i1, ' data points is insufficient for a ',
     1        'calibration .' )
          call ilgwait()
      else                                                                1m
c
c             Check that the number of amplifiers for which there are
c             known gains is the same as the calibrated wires .
c
          offset_ok = .TRUE.
c         start of loop
40        if ( n_offsets .ne. n_wires ) then                              2b
              write( STD_OUT, 41 ) n_wires, n_offsets
41            format( ' I have calibration data for ', i2, ' hot-' /
     1            'wires but amplifier offsets and gains for ', i2 )
              if ( .not. Equery( STD_IN,
     1                'Do wish to fix this up ? (else abort) .' ) )
     2            return
              n_offsets = n_wires
              offset_ok = get_amplif() .eq. YES
              goto 40
          else if ( .not. offset_ok ) then                                2m
              write( STD_OUT, 42 )
42            format( ' WARNING : Amplifier gain(s) and offset(s) ',
     1            'still not properly specified .' )
              call ilgwait()
              offset_ok = get_amplif() .eq. YES
              goto 40
              end if                                                      2e
c
c             Now we have the amplifier gains and offsets, convert hot
c             wire voltages from A/D coverter values to CTA output values
c             Also ensure that the correct number of data points and the
c             air speed values are recorded for each wire .
c
          do 44 i = 1, n_wires                                            2b
              n_points(i) = n_points(1)
              do 43 npts = 1, n_points(i)                                 3b
                  calib(npts,AIR_SPEED,i) = calib(npts,AIR_SPEED,1)
                  calib(npts,WIRE_MEAN,i) = ( calib(npts,WIRE_MEAN,i) -
     1                Amp_offset(i) ) / Amp_gain(i)
                  calib(npts,WIRE_RMS,i) = calib(npts,WIRE_RMS,i) /
     1                abs( Amp_gain(i) )
43                continue                                                3e
44            continue                                                    2e
c
c             select the constant of proportionality for wall-distance
c             vs. air speed for wall-proximity/hot-wire calibration
c
          if ( first_prox .gt. 0 ) then                                   2b
              if ( first_prox .le. 1 .and. last_prox .ge. npts ) then     3b
                  write( STD_OUT, 45 ) name
45                format( a, 'Air speed has not been measured . Cannot',
     1                ' complete wall-proximity calibration .' )
                  return
                  end if                                                  3e
              call proximity_speed( first_prox, last_prox )
              end if                                                      2e
c
c             if we get to this point, we have successfully obtained
c             a set of calibration coordinates
c
          read_calib_data = .TRUE.
          end if                                                          1e
      return
      end



      REAL FUNCTION AD_TO_VOLTS()
c
c Returns the change in A/D converted value per volt .
c
      include 'stdio.def'
      integer Jextadid
c
c       for the DAS 1128
      if ( Jextadid() .eq. 1 ) then
          AD_to_volts = 400.0
c
c       for the DAS 250A
      else
          AD_to_volts = -409.6
          end if
      return
      end



      SUBROUTINE MEAN_HW_VOLTS( RAW_DATA, INDEX, SCALE_FACTOR )
c
c Calculate the mean & rms voltage from each hot-wire from the raw data array .
c The scale factor SCALE_FACTOR is used to convert the A/D converter readings
c to voltage .
c
c Arguments
c  integer *2 raw_data(*)   # raw data array
c  integer index            # index into the array of air-speed and mean & rms
c                           # voltage
c  real scale_factor        # converts A/D reading into volts
c
c local variables
c  integer i        # looping index
c          Jextcyln # function : A/D cycle length
c          Jextnocy #          : number of A/D cycles
c
      include 'stdio.def'
      include 'calhw1.def'
      integer *2 raw_data(*)
      integer index, i, Jextcyln, Jextnocy
      real scale_factor
      include 'calhw1.cmn'
c
      do 1 i = 1, n_wires
          call Smeani( raw_data(i), Jextcyln(), Jextnocy(),
     1        calib(index,WIRE_MEAN,i), calib(index,WIRE_RMS,i) )
          calib(index,WIRE_MEAN,i) = calib(index,WIRE_MEAN,i) /
     1        scale_factor
          calib(index,WIRE_RMS,i) = sqrt( calib(index,WIRE_RMS,i) ) /
     1        abs( scale_factor )
1         continue
      return
      end



      SUBROUTINE PROXIMITY_SPEED( FIRST_PROX, LAST_PROX )
c
c Select the constant of proportionality for wall-distance vs. air speed
c for wall-proximity/hot-wire calibration . Of the total number
c N_POINTS(REF_WIRE) of calibration coordinates available, FIRST_PROX to
c LAST_PROX use the wall proximity method, and so require appropriate scaling .
c
c NOTE THAT THIS ROUTINE ONLY WORKS FOR THE SELECTED REFERENCE WIRE
c (DEFAULT=1) . I.E. FOR CALIBRATION OF A SINGLE WIRE .
c
c Arguments
c  integer first_prox,  # row index of first wall-proximity calibration point
c          last_prox    # row index of last wall-proximity calibration point
c
c constant
c  DISTANCE_FRACTION=0.03**2
c Local variables
c  logical
c    found              # selected point has been found
c    got_hair           # crosshairs read successfully
c    Equery             # function : read logical value
c    ilgread            #          : read crosshairs
c  integer
c    i                  # looping index
c    isel               # index of selected wall-proximity calibration point
c    n_prox             # number of wall-proximity calibration points
c    n_ref              # number of reference calibration points
c    ref_end            # row index of last reference calibration point
c    ref_start          # row index of first reference calibration point
c    Jresp              # function : read integer
c  real
c    dist               # distance from data point to crosshairs
c    min_dist           # distance from crosshairs to closest data point
c    max_prox_speed     # maximum unscaled air speed value for wall-prox method
c    max_ref_speed      # maximum air speed value from reference data
c    min_absciss        # minimum abscissa for plotting all data
c    max_absciss        # maximum abscissa for plotting all data
c    old_factor         # previous vaue of scale_factor
c    scale_factor       # air speed scale factor for wall proximity data
c    x_min, x_max       # minimum & maximum abscissa limits on graph
c    y_min, y_max       # minimum ordinate value on graph
c    x_select, y_select # coordinate value from crosshairs
c    x_toler, y_toler   # +/- tolerance for searching for a data point
c    Qresp              # function : read real
c  character
c    press*1            # key pressed for crosshairs
c  subroutine
c    ilgdata, ilgimit, ilginvs, ilgline
c    sfudg2             # graphical editing of data
c    range_data         # calculate range of data in an array
c
c
      include 'stdio.def'
      include 'calhw1.def'
      real DISTANCE_FRACTION
      parameter( DISTANCE_FRACTION=0.009 )
c
      logical found, got_hair, Equery, ilgread
      integer first_prox, last_prox, i, isel, n_prox, n_ref, ref_end,
     1    ref_start, Jresp
      real dist, min_dist, max_prox_speed, max_ref_speed, min_absciss,
     1    max_absciss, old_factor, scale_factor, x_min, x_max,
     2    y_min, y_max, x_select, y_select, x_toler, y_toler,
     3    Qresp
      character press*1, unused_char*1
      include 'calhw1.cmn'
c
c       First check the reference wire
c
      if ( 1 .gt. ref_wire .or. ref_wire .gt. n_wires ) then              1b
          write( STD_OUT, 1 ) ref_wire
1         format( ' Illegal reference wire ', i3, '. Respecify ', $ )
          ref_wire = Jresp( STD_IN, 1, n_wires, MAX_TRIES )
          end if                                                          1e
c
c       Determine where the wall proximity data is, and where the
c       reference data (measured air speed) is .
c
c     reference data is first, then wall proximity data
      if ( first_prox .gt. 1 ) then                                       1b
          ref_start = 1
          ref_end = first_prox - 1
          n_ref = ref_end
      else                                                                1m
c     wall proximity data is first, then reference data
          ref_start = last_prox + 1
          ref_end = n_points(ref_wire)
          n_ref = n_points(ref_wire) - last_prox
          end if                                                          1e
      n_prox = last_prox - first_prox + 1
c
c       Plot the air-speed as a function of hot-wire voltage .
c       The lower edge is at zero air speed because the wall proximity
c       data is at lower speed than the reference data .
c
      call range_data( calib(ref_start,AIR_SPEED,ref_wire), 1, n_ref,
     1    y_min, max_ref_speed )
      call range_data( calib(1,WIRE_MEAN,ref_wire), 1,
     1    n_points(ref_wire), min_absciss, max_absciss )
      call ilgimit( min_absciss, 0.0, max_absciss, max_ref_speed )
      x_min = min_absciss
      y_min = 0.0
      x_max = max_absciss
      y_max = max_ref_speed
      x_toler = ( max_absciss - min_absciss )
      y_toler = max_ref_speed
      call ilgline( 1, 0, 1 )
      call ilgdata( calib(ref_start,WIRE_MEAN,ref_wire),
     1    calib(ref_start,AIR_SPEED,ref_wire), 1, n_ref, 0 )
      call ilgxlab( 'CTA output (Volts)' )
      call ilgylab( 'Air speed (m/s)' )
      call ilgtitl( 'Calibration by wall proximity' )
c
c       find the maximum "air speed" in the wall-proximity data
c
      call range_data( calib(first_prox,AIR_SPEED,ref_wire), 1, n_prox,
     1    y_min, max_prox_speed )
c
c       Select scale_factor so that the maximum speed of the wall-proximity
c       data is the same as the reference data, and apply the scale factor
c       to the wall-proximity data
c
      if ( scale_factor .lt. 1.0e-6 .or. scale_factor .gt. 1.0e-6 )
     1    scale_factor = max_ref_speed / max_prox_speed
      do 2 i = first_prox, last_prox                                      1b
          calib(i,AIR_SPEED,ref_wire) = calib(i,AIR_SPEED,ref_wire)
     1        * scale_factor
2         continue                                                        1e
c
c       plot the wall-proximity data as Xs
c
      call ilgline( 5, 0, 1 )
      call ilgdata( calib(first_prox,WIRE_MEAN,ref_wire),
     1    calib(first_prox,AIR_SPEED,ref_wire), 1, n_prox, 0 )

c     start of loop                                                       1b
3         call newscr()
c
c       Print message, invite the user to grab a wall-proximity data point
c       with the cross-hairs . Special characters are
c         'C' 'c' -- capture graph
c         'E' 'e' -- edit the wall-proximity data,
c         'H' 'h' -- help .
c         'R' 'r' -- replot with original scaling
c         'S' 's' -- enter the scale_factor directly,
c         'X' 'x' -- exit,
c         'Z' 'z' -- zoom in
c
          write( STD_OUT, 4 ) scale_factor
4         format( ' Scale factor is ', g14.7 /
     1      ' Select a wall-proximity point (X) to move vertically' /
     2      '   ( Capture, Edit, Help, Replot, Type-scale-factor, ',
     3          'eXit, Zoom-in )' /)
c
c       EXIT
c
          got_hair = ilgread( x_select, y_select, press )
          if ( 'a' .le. press .and. press .le. 'z' )
     1        press = char( ichar(press) + ichar('A') - ichar('a') )
          if ( .not. got_hair .or.
     1            press .eq. 'X' .or. press .eq. 'Q' ) then               2b
              if ( Equery( STD_IN, 'Finished . Are you sure ?' ) ) then   3b
                  call ilgclos()
                  return
                  end if                                                  3e
c
c       HELP
c
          else if ( press .eq. 'H' ) then                                 2m
              write( STD_OUT, 5 )
5             format( ' Help for scaling the wall proximity data .' /
     1  ' You must line up the wall-proximity data (crosses) with the' /
     2  ' reference data for which air speed was measured (squares)' /
     3  ' by choosing a vertical scale factor. Select a wall proxim-' /
     4  ' ity point with the crosshairs and move it vertically. All' /
     5  ' other wall-proximity data will be scaled proportionally. By' /
     6  ' pressing E you can edit the wall proximity data . R replots' /
     7  ' with the original scaling. C replots and captures with the' /
     8  ' same scaling. S allows you to enter the scale factor' /
     9  ' directly. Z uses rectangle corners to zoom in.' /
     a  ' Press X to finish.' )
              call ilgwait()
c
c       EDIT WALL-PROXIMITY DATA
c
          else if ( press .eq. 'E' ) then                                 2m
              call sfudg2( calib(1,1,ref_wire), WIRE_MEAN, AIR_SPEED,
     1            MAX_POINTS, 3, 1,
     2            n_points(ref_wire), n_points(ref_wire) )
c
c       REPLOT WITH ORIGINAL SCALING
c
          else if ( press .eq. 'C' .or. press .eq. 'R' .or.
     2              press .eq. 'Z' ) then                                   2m
              if ( press .eq. 'C' ) then                                    3b
                  call ilgopen( '?' )
              else if ( press .eq. 'R' ) then                               3m
                  x_min = min_absciss
                  y_min = 0.0
                  x_max = max_absciss
                  y_max = max_ref_speed
c
c       ZOOM IN
c
              else                                                        3m
c
c       get first corner
c
c                 start of loop                                           4b
6                     write( STD_OUT, 7 )
7                     format( ' Specify first corner of zoom window' )
                      if ( .not. ilgread( x_min, y_min, unused_char ) )
     1                    goto 6
c                     end of loop                                         4e
c
c       get opposite corner
c
c                 start of loop                                           4b
8                     write( STD_OUT, 9 )
9                     format( ' Specify opposite corner of zoom window')
                      if ( .not. ilgread( x_max, y_max, unused_char ) )
     1                    goto 8
c                     end of loop                                         4e
                  if ( x_min .gt. x_max )
     1                call swap4( x_min, x_max )
                  if ( y_min .gt. y_max )
     1                call swap4( y_min, y_max )
                  end if                                                  3e
c
c       plot both reference and proximity data
c
              x_toler = x_max - x_min
              y_toler = y_max - y_min
              call ilgimit( x_min, y_min, x_max, y_max )
              call ilgline( 1, 0, 1 )
              call ilgdata( calib(ref_start,WIRE_MEAN,ref_wire),
     1            calib(ref_start,AIR_SPEED,ref_wire), 1, n_ref, 0 )
              call ilgline( 5, 0, 1 )
              call ilgdata( calib(first_prox,WIRE_MEAN,ref_wire),
     1            calib(first_prox,AIR_SPEED,ref_wire), 1, n_prox, 0 )
              call ilgxlab( 'CTA output (Volts)' )
              call ilgylab( 'Air speed (m/s)' )
              call ilgtitl( 'Calibration by wall proximity' )
c
c       ENTER SCALE-FACTOR DIRECTLY
c
          else                                                            2m
              if ( press .eq. 'T' ) then                                  3b
                  old_factor = scale_factor
                  write( STD_OUT, 10 )
10                format( ' What is the scale-factor ? ', $ )
                  scale_factor = Qresp( STD_IN, 0.1 * scale_factor,
     1                10.0 * scale_factor, MAX_TRIES ) / scale_factor
                  found = .TRUE.
c
c       NOT A SPECIAL CASE -- find the selected point
c       select the wall proximity datum which is closest to the
c       crosshairs and the check if is close enough
c
              else                                                        3m
                  isel = first_prox
                  min_dist = 1.0e36
                  do 11 i = first_prox, last_prox
                      dist =
     1    ( ( calib(i,WIRE_MEAN,ref_wire) - x_select ) / x_toler ) **2 +
     2    ( ( calib(i,AIR_SPEED,ref_wire) - y_select ) / y_toler ) **2
                      if ( dist .lt. min_dist ) then
                          min_dist = dist
                          isel = i
                          end if
11                    continue
                  found = DISTANCE_FRACTION .gt. min_dist
                  if ( found ) then                                       4b
c
c       specify another point with the crosshairs .
c
12                    write( STD_OUT, 13 )
13                    format( ' Move to where ?' )
                      if ( .not. ilgread( x_select, y_select,
     1                    unused_char ) ) goto 14
                      old_factor = scale_factor
                      scale_factor = y_select /
     1                    calib( isel, AIR_SPEED, ref_wire )
                      end if                                              4e
                  end if                                                  3e
c
c       Rub out the wall-proximity data from the graph
c
              if ( found ) then                                           3b
                  call ilginvs()
                  call ilgdata( calib(first_prox,WIRE_MEAN,ref_wire),
     1                calib(first_prox,AIR_SPEED,ref_wire), 1, n_prox,
     2                0 )
c
c       Use the ordinate as the new ordinate of the grabbed point, and scale
c       all the other wall-proximity points proportionally .
c
                  do 14 i = first_prox, last_prox                         4b
                      calib(i,AIR_SPEED,ref_wire) =
     1                    calib(i,AIR_SPEED,ref_wire) * scale_factor
14                    continue                                            4e
                  scale_factor = scale_factor * old_factor
c
c Re-draw the wall-proximity data at the new scale
c
                  call ilgdata( calib(first_prox,WIRE_MEAN,ref_wire),
     1                calib(first_prox,AIR_SPEED,ref_wire), 1, n_prox,
     2                0 )
                  end if                                                  3e
              end if                                                      2e
          goto 3
c         end of loop                                                     1e
      end



      SUBROUTINE RANGE_DATA( DATA, INCREM, N_PTS, MIN_DATA, MAX_DATA )
c
c Find the minimum MIN_DATA and maximum MAX_DATA values contained in the
c array DATA . N_PTS values are sampled at intervals of INCREM .
c
c Arguments
c  real    data(*)  # data array
c  integer increm   # sampling increment
c          n_pts    # number of data points
c  real   min_data  # minimum value
c         max_data  # maximum value
c
c Local variables
c  integer i        # looping index
c
      include 'stdio.def'
      integer increm, n_pts, i
      real data(*), min_data, max_data
c
      min_data = data(1)
      max_data = data(1)
      if ( n_pts .gt. 1 ) then
          do 1 i = 2, n_pts, increm
              min_data = min( min_data, data(i) )
              max_data = max( max_data, data(i) )
1             continue
          end if
      return
      end



      SUBROUTINE PROCESS_DATA( CALIB_FILE )
c
c  Control the process of producing and writing the calibration relations .
c
c Arguments
c   character calib_file*(*)  # name of file containing calibrations
c
c local variables
c  integer
c    abscissa       # specifies graph abscissa for editing
c    i              # looping index
c    item           # selected menu item
c    iwire          # current hot-wire number
c    Jresp          # function : read integer
c  logical
c    name_len       # non-blank length of calibration file name
c    ok             # used to check that all wires have been done
c    wires_done(MAX_WIRES)  # list of which wires have been done
c    written        # relations for the current hot-wire have been written
c    Equery         # function : read logical
c    eyes           #          : convert NO/YES/QUERY to logical
c    write_calibr   #          : write calibration relation
c    str_length     #          : non-blank string length
c  real
c    fit_error(MAX_MEAN)  # polynomial fitting error at each data point
c  character
c    Cdone*8        # function : 'done' / 'not done'
c  subroutine
c    calib_param_menu # calibration parameters menu
c    sortsh         # sorts the calibration data
c    edit_calibr    # edit calibration data
c    poly_calibr    # find polynomial of voltage calibration relation
c    siddal_calibr  # find Siddal-Davies calibration relation
c    wait           # wait for <CR>
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer abscissa, i, item, iwire, name_len, Jresp, str_length
      logical ok, wires_done(MAX_WIRES), written,
     1    Equery, eyes, write_calibr
      real fit_error(MAX_MEAN)
      character calib_file*(*), Cdone*8
      include 'calhw1.cmn'
c
c       initialize
c
      iwire = 1
      abscissa = WIRE_MEAN
      done_siddal = .FALSE.
      done_poly = .FALSE.
      written   = .FALSE.
      do 1 i = 1, n_wires                                                 1b
          wires_done(i) = .FALSE.
1        continue                                                         1e
      name_len = max( 1, str_length( Calib_file ) )
c
c     start of loop                                                       1b
2         call newscr()
          write( STD_OUT, 3 ) iwire, Cdone(done_siddal),
     1        Cdone(done_poly), calib_file(1:name_len)
3         format( ' Processing calibration data for hot-wire ', i2 /
     N        '  -1) Abandon' /
     Z        '   0) Exit (+write)' /
     1        '   1) Set calibration parameters' /
     2        '   2) Sort calibration data' /
     3        '   3) Edit calibration data' /
     4        '   4) Display calibration data' /
     5        '   5) Do Siddal-Davies law      ', a /
     6        '   6) Do polynomial of voltage  ', a /
     7        '   7) Display calibration' /
     8        '   8) Write calibration' /
     9        '   9) Proceed as set-up for all wires' /
     a        '  10) Next hot-wire' /
     b        '  11) Select hot-wire' /
     c        '  12) Calibration file', a /
     d        '  13) Help  ', $ )
          item = Jresp( STD_IN, -1, 12, MAX_TRIES )
c
c       ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, 'Abandon. Are you sure ? ' ) )
     1            return
c
c       EXIT (+WRITE)
c       If calibration has been calculated but not written, write it to file .
c       If some wires remain uncalibrated, query before exit
c
          else if ( item .eq. 0 ) then                                    2m
              if ( ( done_siddal .or. done_poly ) .and. .not. written )
     1                then                                                3b
                  written = write_calibr( MAGIC, calib_file, iwire )
                  if ( written )
     1                wires_done(iwire) = .TRUE.
                  end if                                                  3e
              ok = .TRUE.
              do 4 i = 1, n_wires                                         3b
                  ok = ok .and. wires_done(i)
4                 continue                                                3e
              if ( .not. ok ) then                                        3b
                  if ( .not. Equery( STD_IN,
     1                    'Not all wires calibrated. Are you sure ?' ) )
     2                goto 2
                  end if                                                  3e
              return
c
c       SET CALIBRATION PARAMETERS
c
          else if ( item .eq. 1 ) then                                    2m
              call calib_param_menu()
c
c       SORT CALIBRATION DATA
c
          else if ( item .eq. 2 ) then                                    2m
              call sortsh( calib(1,1,iwire), n_points(iwire), 1,
     1            MAX_POINTS, WIRE_MEAN, 3 )
c
c       EDIT CALIBRATION DATA
c
          else if ( item .eq. 3 ) then                                    2m
              call edit_calibr( fit_error, abscissa, iwire )
c
c       DISPLAY CALIBRATION DATA
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 5 )   ( i, calib(i,AIR_SPEED,iwire),
     1            calib(i,WIRE_MEAN,iwire), calib(i,WIRE_RMS,iwire),
     2            i = 1, n_points(iwire) )
5             format( /
     1            16('*'), ' Calibration data ', 16('*') /
     2            ' No. Air-speed Mean-volts RMS-volts ' /
     3            ( 1x, i3, 3f10.4 ) )
              if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1            write( aux_lu, 5 )   ( i, calib(i,AIR_SPEED,iwire),
     2                calib(i,WIRE_MEAN,iwire), calib(i,WIRE_RMS,iwire),
     3                i = 1, n_points(iwire) )
              call ilgwait()
c
c       DO SIDDAL-DAVIES LAW
c
          else if ( item .eq. 5 ) then                                    2m
              call siddal_calibr( fit_error, iwire )
              abscissa = AIR_SPEED
c
c       DO POLYNOMIAL OF VOLTAGE
c
          else if ( item .eq. 6 ) then                                    2m
              call poly_calibr( fit_error, iwire )
              abscissa = WIRE_MEAN
c
c       DISPLAY CALIBRATION
c
          else if ( item .eq. 7 ) then
              call display_calibr( CALIBR_HEADER, iwire, .TRUE. )
              if ( done_siddal )
     1            call display_calibr( CALIBR_SIDDAL, iwire, .TRUE. )
              if ( done_poly )
     1            call display_calibr( CALIBR_POLY, iwire, .TRUE. )
              call ilgwait()
c
c       WRITE CALIBRATION
c
          else if ( item .eq. 8 ) then                                    2m
              if ( done_siddal .or. done_poly ) then                      3b
                  if ( wires_done(iwire) ) then                           4b
                      if ( Equery( STD_IN,
     1                             'Already done. Are you sure ? ' ) )
     2                    goto 2
                      end if                                              4e
                  written = write_calibr( MAGIC, calib_file, iwire )
                  if ( written ) then                                     4b
                      wires_done(iwire) = .TRUE.
c                     increment to next wire if possible
                      iwire = mod( iwire, n_wires ) + 1
                      done_siddal = .FALSE.
                      done_poly = .FALSE.
                      written = .FALSE.
                      end if                                              4e
              else                                                        3m
                  write( STD_OUT, 6 ) iwire
6                 format( ' Calibration of hot-wire ', i2,
     1                ' not available for writing .' )
                  call ilgwait()
                  end if                                                  3e
c
c       PROCEED AS SET-UP
c
          else if ( item .eq. 9 ) then                                    2m
              do 8 i = 1, n_wires                                         3b
                  write( STD_OUT, 7 ) i
7                 format( ' Evaluating calibration relation for hot-',
     1               'wire ', i2 )
                  if ( wires_done(i) ) then                               4b
                      if ( .not. Equery( STD_IN,
     1                        'Already done . Again ? ' ) ) goto 8
                      end if                                              4e
                  done_siddal = .FALSE.
                  done_poly = .FALSE.
                  written = .FALSE.
                  if ( eyes( siddal_davies,
     1                    'Do Siddal-Davies calibration ? ' ) )
     2                call siddal_calibr( fit_error, i )
                  if ( eyes( poly_volts,
     1                  'Do polynomial of volts calibration ? ' ) )
     2                call poly_calibr( fit_error, i )
                  if ( done_poly .or. done_siddal ) then                  4b
                      written = write_calibr( MAGIC, calib_file, i )
                      if ( written )
     1                    wires_done(i) = .TRUE.
                  else                                                    4m
                      write( STD_OUT, 6 ) i
                      end if                                              4e
8                 continue                                                3e
c
c       NEXT HOT-WIRE
c
          else if ( item .eq. 10 ) then                                   2m
              i = 1
c             repeat                                                      3b
9                 iwire = mod( iwire, n_wires ) + 1
                  i = i + 1
                  if ( wires_done(iwire) .and. i .le. n_wires ) goto 9
c                 until( found not-done hot-wire )                        3e
              if ( wires_done(iwire) ) then                               3b
                  write( STD_OUT, 10 )
10                format( ' All wires calibrated ' )
                  call ilgwait()
              else                                                        3m
                  done_siddal = .FALSE.
                  done_poly = .FALSE.
                  written = .FALSE.
                  end if                                                  3e
c
c       SELECT HOT-WIRE
c
          else if ( item .eq. 11 ) then                                   2m
              write( STD_OUT, 11 )
11            format( ' What is the hot-wire number ? ', $ )
              iwire = Jresp( STD_IN, 1, n_wires, MAX_TRIES )
              if ( wires_done(iwire) ) then                               3b
                  write( STD_OUT, 12 )
12                format( ' Calibration already done .' )
                  call ilgwait()
              else                                                        3m
                  done_siddal = .FALSE.
                  done_poly = .FALSE.
                  written = .FALSE.
                  end if                                                  3e
c
c       SPECIFY CALIBRATION FILE
c
          else if ( item .eq. 11 ) then                                   2m
              write( STD_OUT, 13 )
13            format( ' Calibration file name ? ', $ )
              call Cresp( STD_IN, Calib_file, MAX_TRIES )
              name_len = max( 1, str_length( Calib_file ) )
c
c       HELP
c
          else if ( item .eq. 12 ) then                                   2e
              write( STD_OUT, 14 )
14            format( ' You now have air-speed vs. CTA-output-voltage ',
     1              ' data for the calibrated wires .' /
     2            ' Use this menu to calculate the hot-wire ',
     3              'calibration relations . Sorting the' /
     4            ' data effects only the graphical display, not the ',
     5              'numerical result . The data' /
     6            ' can be edited or displayed . Each hot-wire can ',
     7              'selected as required . Menu' /
     8            ' item 9 processes each hot-wire in turn . By ',
     9              'default a calibration is written' /
     a            ' when EXIT is selected . To avoid writing a ',
     b              'calibration, select ABANDON . The' /
     c            ' calibration file is changed with menu item 12 .' )
              call ilgwait()
              end if                                                      2e
          goto 2
c         end of loop                                                     1e
      end



      LOGICAL FUNCTION WRITE_CALIBR( IDENTIFICATION, CALIB_FILE, IWIRE )
c
c Writes the hot-wire calibration for wire number IWIRE to file with logical
c unit number IWIRE .
c
c Argument
c  character identification*2   # 2 character record identification
c            calib_file*(*) # logical unit number of the calibration file
c  integer   iwire          # hot-wire number
c
c local variables
c  logical Existf     # function : file exists ?
c  integer i          # looping array index
c          calib_lu   # logical unit number for calibration file (0 if closed)
c          name_len   # non-blank length of calibration file name
c          res_cal_lu # reserved logical unit number for calibration file
c          new_lun()  # function : get new logical unit number
c          str_length #          : non-blank string length
c  character old_file*64  # previous file name
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical Existf
      integer calib_lu, i, iwire, name_len, res_cal_lu,
     1    new_lun, str_length
      character identification*2, calib_file*(*), old_file*64
      include 'calhw1.cmn'
      save res_cal_lu, calib_lu, old_file
      data res_cal_lu / 0 /
      data old_file / ' ' /
c
c
c       WRITE_CLOSE -- close the calibration file and return successfully
c
      write_calibr = .FALSE.
      name_len = max( 1, str_length( Calib_file ) )
      if ( iwire .eq. WRITE_CLOSE ) then                                  1b
          write_calibr = .TRUE.
          if ( calib_lu .ne. 0 )
     1        close( UNIT=calib_lu )
          calib_lu = 0
c
c       check arguments
c
      else if ( calib_file .eq. ' ' ) then                                1m
          write( STD_ERR, 1 )
1         format( ' WRITE_CALIBR : ABORT : O/P file not specified' )
      else if ( iwire .ne. WIRE_AVERAGE .and.
     1          ( 0 .gt. iwire .or. iwire .gt. n_wires ) ) then           1m
          write( STD_ERR, 2 ) iwire
2         format( ' WRITE_CALIBR : ABORT : Illegal wire number ', i12 )
c
c       open calibration file
c           If the file name has changed, close the old one first .
c           Then if no file is open, open it . If the file already
c           exists, it is opened for appending .
c
      else                                                                1m
          if ( res_cal_lu .eq. 0 ) then
              res_cal_lu = new_lun()
              end if
          if ( calib_lu .ne. 0 .and. calib_file .ne. old_file ) then      2b
              close( UNIT=calib_lu )
              calib_lu = 0
              end if                                                      2e
          if ( calib_lu .eq. 0 ) then                                     2b
              if ( Existf( Calib_file ) ) then                            3b
                  open( UNIT=res_cal_lu, FILE=Calib_file, STATUS='old',
     1                ACCESS='append', FORM='unformatted', ERR=5,
     2                RECORDTYPE='variable', BLOCKSIZE=8192 )
              else                                                        3m
                  open( UNIT=res_cal_lu, FILE=Calib_file, STATUS='new',
     1                ACCESS='sequential', FORM='unformatted', ERR=5,
     2                RECORDTYPE='variable', BLOCKSIZE=8192 )
                  end if                                                  3e
              calib_lu = res_cal_lu
              old_file = calib_file
              calib_no = 1
              end if                                                      2e
c
c           Now write to the file .
c
          if ( iwire .eq. WIRE_AVERAGE ) then                             2b
              write( calib_lu, ERR=3 ) identification,
     1            calib_no, Ctype, iplug, trip_diam, Cwire_ad,
     2            wire_chan(1), n_points(1),
     3            Amp_gain(1), Amp_offset(1),
     4            done_siddal, siddal_expon, siddal_order, siddal_error,
     5                ( siddal_poly(i), i = 1, siddal_order+1 ),
     6            done_poly, poly_order, poly_error, poly_offset,
     7                ( poly_poly(i+1), i = 0, poly_order ),
     8            ( mean(i,AIR_SPEED), mean(i,WIRE_MEAN),
     9                mean(i,WIRE_RMS), i = 1, n_points(1) )
          else                                                            2m
              write( calib_lu, ERR=3 ) identification,
     1            calib_no, Ctype, iplug, trip_diam, Cwire_ad,
     2            wire_chan(iwire), n_points(iwire),
     3            Amp_gain(iwire), Amp_offset(iwire),
     4            done_siddal, siddal_expon, siddal_order, siddal_error,
     5                ( siddal_poly(i), i = 1, siddal_order+1 ),
     6            done_poly, poly_order, poly_error, poly_offset,
     7                ( poly_poly(i+1), i = 0, poly_order ),
     8            ( calib(i,AIR_SPEED,iwire), calib(i,WIRE_MEAN,iwire),
     9                calib(i,WIRE_RMS,iwire), i = 1, n_points(iwire) )
              end if                                                      2e
          calib_no = calib_no + 1
          write_calibr = .TRUE.
          end if                                                          1e
      return
c
c           Error traps
c
3     write( STD_ERR, 4 ) Calib_file(1:name_len)
4     format( ' WRITE_CALIBR : ABORT : Cannot write to file ', a )
      return
c
5     write( STD_ERR, 6 ) Calib_file(1:name_len)
6     format( ' WRITE_CALIBR : ABORT : Cannot open file ', a )
      return
      end



      SUBROUTINE DISPLAY_CALIBR( WHAT, IWIRE, OK_AUX )
c
c Display the calibration information for hot-wire IWIRE specified by WHAT .
c
c Arguments
c  integer what   # CALIBR_HEADER, CALIBR_SIDDAL, CALIBR_POLY
c          iwire  # hot-wire number
c  logical ok_aux # write to auxillary file
c
c local variables
c  integer i      # looping index
c
      include 'stdio.def'
      include 'calhw1.def'
      logical ok_aux
      integer what, iwire, i
      include 'calhw1.cmn'
c
      if ( iand( what, CALIBR_HEADER ) .eq. CALIBR_HEADER ) then
          if ( iwire .eq. WIRE_AVERAGE ) then
              i = 1
          else
              i = iwire
              end if
          write( STD_OUT, 1, ERR=4 )
     1        calib_no, Ctype, iplug, trip_diam, Cwire_ad,
     2        wire_chan(i), n_points(i),
     3        Amp_gain(i), Amp_offset(i)
          if ( ok_aux .and. ( 0 .gt. aux_lu .and. aux_lu .lt. 100 ) )
     1        write( aux_lu, 1, ERR=5 )
     2            calib_no, Ctype, iplug, trip_diam, Cwire_ad,
     3            wire_chan(i), n_points(i),
     4            Amp_gain(i), Amp_offset(i)
1         format( '******** CALIBRATION INCIDENTALS ********' /
     1        6x, 'Calibration number', 9x, i3 /
     2        6x, 'Type of hot wire used  ', a32 /
     3        6x, 'Wind tunnel plug number    ', i4 /
     4        6x, 'Boundary layer trip diam   ', f7.3 /
     5        6x, 'A/D converter used', 9x, a12 /
     6        6x, 'A/D converter channel', 6x, i3 /
     7        6x, 'No. of calibration points  ', i3 /
     8        6x, 'Amplifier gain', 13x, f9.5 /
     9        16x, 'offset', 11x, f9.5 )
          end if
c
c
      if ( iand( what, CALIBR_SIDDAL ) .eq. CALIBR_SIDDAL ) then
          write( STD_OUT, 2, ERR=4 )
     1        siddal_expon, siddal_order, siddal_error,
     2        ( i, siddal_poly(i+1), i = 0, siddal_order )
2         format( / '   ***** Siddal-Davies calibration *****' /
     1        6x, 'Air speed exponent', 9x, f10.7 /
     2        6x, 'Order of fitted polynomial ', i2 /
     3        6x, 'RMS error of regression    ', g13.6 /
     4        6x, 'Calibration polynomial ' /
     5        ( 11x, i2, g13.6 ) )
          if ( ok_aux .and. ( aux_lu .gt. 0 .and. aux_lu .lt. 100 ) )
     1        write( aux_lu, 2, ERR=5 )
     2            siddal_expon, siddal_order, siddal_error,
     3            ( i, siddal_poly(i+1), i = 0, siddal_order )
          end if
c
c
      if ( iand( what, CALIBR_POLY ) .eq. CALIBR_POLY ) then
          write( STD_OUT, 3, ERR=4 ) poly_order, poly_error,
     1        ( i, poly_poly(i+1), i = 0, poly_order )
3         format( / '    ***** Polynomial calibration *****' /
     1        6x, 'Order of fitted polynomial ', i2 /
     2        6x, 'RMS error of regression    ', g13.6 /
     3        6x, 'Calibration polynomial ' /
     4        ( 11x, i2, g13.6 ) )
          if ( ok_aux .and. ( aux_lu .gt. 0 .and. aux_lu .lt. 100 ) )
     1        write( aux_lu, 3, ERR=5 ) poly_order, poly_error,
     2            ( i, poly_poly(i+1), i = 0, poly_order )
          end if
      return
c
4     write( STD_ERR, 6 ) 'STD_OUT'
      return
5     write( STD_ERR, 6 ) 'auxillary file'
6     format( ' DISPLAY_CALIBR : Cannot write to ', a )
      return
      end



      SUBROUTINE EDIT_CALIBR( ERROR, ABSCISSA, IWIRE )
c
c Edit the calibration data for hot-wire number IWIRE . The ABSCISSA for the
c graph is either AIR_SPEED or WIRE_MEAN ( the ordinate is the other one )
c If a fitting error is available in the array ERROR, as indicated by the
c value of DONE_SIDDAL and DONE_POLY, the fitting error is plotted as a
c not-to-scale dashed line background .
c
c Arguments
c  real error(*)        # fitting errors
c  integer iwire        # hot-wire number
c          abscissa     # AIR_SPEED or WIRE_MEAN
c
c local variables
c  integer ordinate     # plotted ordinate ( AIR_SPEED or WIRE_MEAN )
c  subroutines
c    ilgopen, ilgline, ilgdata, ilgraph, ilgline, sfudg2
c
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer ordinate, iwire, abscissa
      real error(*)
      include 'calhw1.cmn'
c
      if ( abscissa .eq. AIR_SPEED ) then
          ordinate = WIRE_MEAN
      else if ( abscissa .eq. WIRE_MEAN ) then
          ordinate = AIR_SPEED
      else
          write( STD_ERR, 1 )
1         format( ' ABORT : BUG in EDIT_CALIBR : Illegal argument ',
     1        'abscissa = ', i12 )
          return
          end if
      call ilgopen( ' ' )
      if ( done_siddal .or. done_poly ) then
          if ( abscissa .eq. AIR_SPEED )
     1        call ilgslaw( 'log', 'lin' )
          call ilgraph( 1, 1, 0, 0, 1 )
          call ilgline( 0, 3, 1 )
          if ( iwire .eq. WIRE_AVERAGE ) then
              call ilgdata( mean(1,abscissa), error, 1, n_points(1), 0 )
          else
              call ilgdata( calib(1,abscissa,iwire), error, 1,
     1            n_points(iwire), 0 )
              end if
          call ilgnote( '|m(0.01,0.0)|d(100.0,0.0)' )
          end if
      call ilgtitl( 'Editing calibration data' )
      if ( abscissa .eq. AIR_SPEED ) then
          call ilgslaw( 'log', 'lin' )
          call ilgxlab( 'air speed (m/s)' )
          call ilgylab( 'voltage' )
      else
          call ilgslaw( 'lin', 'log' )
          call ilgxlab( 'voltage' )
          call ilgylab( 'air speed (m/s)' )
          end if
      call ilgraph( 1, 1, 3, 3, 1 )
      call ilgline( 0, 5, 1 )
      if ( iwire .eq. WIRE_AVERAGE ) then
          call sfudg2( mean(1,1), abscissa, ordinate, MAX_MEAN,
     1                 3, 1, n_points(1), n_points(1) )
      else
          call sfudg2( calib(1,1,iwire), abscissa, ordinate, MAX_POINTS,
     1                 3, 1, n_points(iwire), n_points(iwire) )
          end if
      call ilgclos()
      return
      end



      REAL FUNCTION MEAN_SQ_ERROR( EXPONENT )
c
c       Evaluates the mean square error of the polynomial curve fit to the
c  calibration data
c
c Argument
c  real exponent    # velocity exponent
c
c local variables
c  integer i                   # looping index
c          npts                # number of data points
c  real
c    speed_expon(MAX_MEAN)     # array of air-speed data ** exponent
c    squared_volts(MAX_MEAN)   # array of squared volatge
c  complex
c    work(MAX_ORDER)           # work array for SPYORTH
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer i, npts
      real exponent, speed_expon(MAX_MEAN), squared_volts(MAX_MEAN),
     1     weight(MAX_MEAN)
      complex *16 work(MAX_ORDER)
      include 'calhw1.cmn'
c
c           calculate U**n
c
      if ( siddal_wire .eq. WIRE_AVERAGE ) then                           1b
          npts = n_points(1)
          do 1 i = 1, npts                                                2b
              speed_expon(i)   = mean(i,AIR_SPEED) ** exponent
              squared_volts(i) = mean(i,WIRE_MEAN) **2
              weight(i)  = max( 0.001, mean(i,AIR_SPEED) ) ** poly_wexp
1             continue
      else                                                                1m
          npts = n_points(siddal_wire)
          do 2 i = 1, npts                                                2b
              speed_expon(i) = calib(i,AIR_SPEED,siddal_wire) **exponent
              squared_volts(i) = calib(i,WIRE_MEAN,siddal_wire) **2
              weight(i)  = max( 0.001, calib(i,AIR_SPEED,siddal_wire) )
     1            ** poly_wexp
2             continue                                                    2e
          end if                                                          1e
c
c           do polynomial regression
c
      call spyorth( speed_expon, squared_volts, weight, npts,
     1              siddal_poly, siddal_order, mean_sq_error, work )
      return
      end



      SUBROUTINE SIDDAL_CALIBR( ERROR, IWIRE )
c
c       Calibrate the hot wire according to the Sidall & Davies calibration law
c
c Arguments
c   real error(*)   # error of the fitting polynomial
c   integer iwire   # specifies the hot-wire to calibrate
c
c  real
c    INITIAL_STEP    = 0.02
c    OPTIM_TOLERANCE = 0.0001
c  integer
c    aux_enable     # NO/YES enable writing to auxillary file
c    cap_len        # length of capture file name
c    i              # looping index
c    item           # seleted menu item
c    npts           # number of calibration points
c    Jresp          # function : read integer
c    Jljstr         #          : left justify string
c  logical
c    first_loop     # have not printed display menu yet
c    Eyes           # function : NO/YES/QUERY to logical
c  real
c    fitted_poly(MAX_DATA)    # fitted polynomial data
c    squared_volts(MAX_DATA)  # array of mean hot-wire voltage squared
c    speed_expon(MAX_DATA)    # air speed ** siddal exponent
c    mean_sq_error  # function : calculate fitted polynomial, mean squared error
c    qdscop         #          : DSC optimization
c    Qpoly          #          : evaluate polynomial
c  character
c    Capture_file*64# name of capture file
c    Clabel*14      # X axis label
c    Cyes*5         # function : NO/YES/QUERY
c  subroutine
c    Cresp
c    newscr
c    ilgclos, ilgdata, ilgraph, ilgline, ilgnote, ilgopen,
c    ilgpage, ilgslaw, ilgtitl, ilgwait, ilgxlab, ilgylab
c
c
      include 'stdio.def'
      include 'calhw1.def'
      real INITIAL_STEP, OPTIM_TOLERANCE
      parameter( INITIAL_STEP=0.02, OPTIM_TOLERANCE=0.0001 )
c
      integer aux_enable, cap_len, i, item, iwire, npts, Jresp, Jljstr
      logical first_loop, Eyes
      real error(*), fitted_poly(MAX_MEAN), squared_volts(MAX_MEAN),
     1    speed_expon(MAX_MEAN), mean_sq_error, qdscop, Qpoly
      character Capture_file*64, Clabel*14, Cyes*5
      include 'calhw1.cmn'
      external mean_sq_error
      save capture_file, cap_len
      data capture_file / ' ' /
      data cap_len / 1 /
c
c       Heat transfer law fit
c
      siddal_wire = iwire
c
c       optimize velocity exponent and do regression
c       ( skip this if the velocity exponent is not optimized)
c
1     if ( Eyes( siddal_optim, 'Optimize velocity exponent ? ' ) ) then   1b
          siddal_expon = 0.5
          siddal_expon = qdscop( siddal_expon, mean_sq_error,
     1        INITIAL_STEP, OPTIM_TOLERANCE, .FALSE. )
          write( STD_OUT, 2 ) 'Best', siddal_expon
2         format( 1x, a, ' value of velocity exponent = ', f9.4 )
      else                                                                1m
          write( STD_OUT, 2 ) 'Fixed', siddal_expon
          end if                                                          1e
c
c       calculate the rms fitting error (and fitting polynomial)
c       for fixed value of velocity exponent
c
      siddal_error = sqrt( mean_sq_error( siddal_expon ) )
      write( STD_OUT, 3 ) siddal_error
3     format( ' Rms fitting error ', g12.5 )
c
c       calculate values of the fitting polynomial and the fitting error
c
      if ( iwire .eq. WIRE_AVERAGE ) then                                 1b
          do 4 i = 1, n_points(1)                                         2b
              speed_expon(i) = mean(i,AIR_SPEED) ** siddal_expon
              squared_volts(i) = mean(i,WIRE_MEAN) **2
              fitted_poly(i)   = Qpoly( speed_expon(i), siddal_poly,
     1            siddal_order )
              error(i) = squared_volts(i) - fitted_poly(i)
4             continue                                                    2e
      else                                                                1m
          do 5 i = 1, n_points(iwire)                                     2b
              speed_expon(i) = calib(i,AIR_SPEED,iwire) ** siddal_expon
              squared_volts(i) = calib(i,WIRE_MEAN,iwire) **2
              fitted_poly(i)   = Qpoly( speed_expon(i), siddal_poly,
     1            siddal_order )
              error(i) = squared_volts(i) - fitted_poly(i)
5             continue                                                    2e
          end if                                                          1e
      done_siddal = .TRUE.
c
c       menu for displaying results
c
      first_loop = .TRUE.
c     start of loop                                                       1b
6         if ( .not. first_loop ) call newscr()
          first_loop = .FALSE.
          write( STD_OUT, 7 ) Capture_file(1:cap_len)
7         format( ' Display menu' / '   0) Exit' /
     1        '   1) Display calibration' /
     2        '   2) Display table of data' /
     3        '   3) Plot data & fitted polynomial' /
     4        '   4) Capture file  ', a /
     5        '   5) Change parameters & recalculate' /
     6        '   6) Help' )
          if ( aux_lu .gt. 0 )
     1        write( STD_OUT, 8 ) Cyes( aux_enable )
8             format( '   7) Output to auxillary file ', a )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c       DISPLAY CALIBRATION
c
          else if ( item .eq. 1 ) then                                    2m
              call display_calibr( CALIBR_HEADER + CALIBR_SIDDAL,
     1          iwire, Eyes( aux_enable, 'Write to auxillary file ?' ) )
              call ilgwait()
c
c       DISPLAY DATA TABLE
c
          else if ( item .eq. 2 ) then                                    2m
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  write( STD_OUT, 9 ) ( i, mean(i,AIR_SPEED),
     1                mean(i,WIRE_MEAN), speed_expon(i),
     2                squared_volts(i), fitted_poly(i), error(i),
     3                i = 1, n_points(1) )
                  if ( Eyes( aux_enable, 'Write to auxillary file ?' )
     1                     .and. aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     2                write( aux_lu, 9 ) ( i, mean(i,AIR_SPEED),
     3                    mean(i,WIRE_MEAN), speed_expon(i),
     4                    squared_volts(i), fitted_poly(i), error(i),
     5                    i = 1, n_points(1) )
              else                                                        3m
                  write( STD_OUT, 9 )   ( i, calib(i,AIR_SPEED,iwire),
     1                calib(i,WIRE_MEAN,iwire), speed_expon(i),
     2                squared_volts(i), fitted_poly(i), error(i),
     3                i = 1, n_points(iwire) )
                  if ( Eyes( aux_enable, 'Write to auxillary file ?' )
     1                     .and. aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     2                write( aux_lu, 9 ) ( i, calib(i,AIR_SPEED,iwire),
     3                    calib(i,WIRE_MEAN,iwire), speed_expon(i),
     4                    squared_volts(i), fitted_poly(i), error(i),
     5                    i = 1, n_points(iwire) )
                  end if                                                  3e
              call ilgwait()
9             format( /
     1            16('*'), ' Siddal-Davies calibration data ', 16('*') /
     2            ' No. Air-speed Mean-volts speed**n volts**2 ',
     3                'polynomial fit-error' / ( 1x, i3, 6f10.4 ) )
c
c       PLOT DATA & FITTED POLYNOMIAL
c
          else if ( item .eq. 3 ) then                                    2m
c
c           plot data and fitted polynomial
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  npts = n_points(1)
              else                                                        3m
                  npts = n_points(iwire)
                  end if                                                  3e
              call ilgopen( Capture_file )
              call ilgpage( 'a4', 'wide', 2, 1 )
              call ilgslaw( 'lin', 'lin' )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgline( 1, 0, 1 )
              call ilgdata( speed_expon, squared_volts, 1, npts, 1 )
              call ilgline( 0, 1, 1 )
              call ilgdata( speed_expon, fitted_poly, 1, npts, 0 )
              write( Clabel, 10 ) siddal_expon
10            format( 'Speed|5t', f5.3 )
              call ilgxlab( Clabel )
              call ilgylab( 'Voltage Squared' )
              call ilgtitl( 'Siddal-Davies calibration' )
              call ilgnote( '|2l |5-1 fitted curve|r   |s1 = data' )
c
c           plot RMS signal fluctuation
              call ilgpage( 'a4', 'WIDE', 2, 2 )
              call ilgraph( 2, 1, 11, 11, 1 )
              call ilgline( 4, 1, 1 )
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  call ilgdata( speed_expon, mean(1,WIRE_RMS),
     1                1, npts, 1 )
              else                                                        3m
                  call ilgdata( speed_expon, calib(1,WIRE_RMS,iwire),
     1                1, npts, 1 )
                  end if                                                  3e
              call ilgxlab( Clabel )
              call ilgylab( 'Volts' )
              call ilgtitl( 'RMS signal fluctuation' )
c
c           plot fitting error
              call ilgraph( 2, 2, 11, 11, 1 )
              call ilgline( 5, 1, 1 )
              call ilgdata( speed_expon, error, 1, npts, 1 )
              call ilgxlab( Clabel )
              call ilgylab( 'Fitting Error' )
              call ilgwait
              call ilgclos
c
c       CAPTURE FILE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 11 )
11            format( ' Capture file name ? ', $ )
              call Cresp( STD_IN, Capture_file, MAX_TRIES )
              cap_len = max( 1, jljstr( Capture_file ) )
c
c       CHANGE PARAMETERS AND RECALCULATE
c
          else if ( item .eq. 5 ) then
              call menu_siddal_davies()
              goto 1
c
c       HELP
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 12 )
12            format( ' A calibration relation has been calculated ',
     1                'and is available for examination' /
     2            ' of (1) calibration polynomial coeffients, (2) ',
     3                'tabular comparison of data' /
     4            ' and fitted polynomial and (3) graphical ',
     5                'comparison of data and fitted' /
     6            ' polynomial . Change the calibration parameters ',
     7                'using menu item (5) .' )
              call ilgwait()
c
c       AUXILLARY FILE ENABLE
c
          else if ( item .eq. 7 .and. aux_lu .gt. 0 ) then                2m
              aux_enable = YES - aux_enable
              end if                                                      2e
          goto 6
c         end of loop                                                     1e
      end



      SUBROUTINE POLY_CALIBR( ERROR, IWIRE )
c
c       Calibrate the hot wire according to the Sidall & Davies calibration law
c
c Arguments
c   real error(*)   # error of the fitting polynomial
c   integer iwire   # specifies the hot-wire to calibrate
c
c  integer
c    aux_enable     # NO/YES enable writing to auxillary file
c    cap_len        # length of capture file name
c    i              # looping index
c    item           # seleted menu item
c    npts           # number of data points
c    Jresp          # function : read integer
c    Jljstr         #          : left justify string
c  logical
c    first_loop     # have not printed display menu yet
c    Eyes           # function : NO/YES/QUERY to logical
c  real
c    fitted_poly(MAX_MEAN)  # fitted polynomial data
c    Qpoly          #          : evaluate polynomial
c  complex
c    work(MAX_ORDER)# work area for SPYORTH
c  character
c    Capture_file*64# name of capture file
c    Cyes*5         # function : NO/YES/QUERY
c  subroutine
c    Cresp
c    newscr
c    spyorth
c    ilgclos, ilgdata, ilgraph, ilgline, ilgnote, ilgopen,
c    ilgpage, ilgslaw, ilgtitl, ilgwait, ilgxlab, ilgylab
c
c
      include 'stdio.def'
      include 'calhw1.def'
c
      integer aux_enable, cap_len, i, item, iwire, npts, Jresp, Jljstr
      logical first_loop, Eyes
      real error(*), fitted_poly(MAX_MEAN), voltage(MAX_MEAN),
     1     weight(MAX_MEAN), Qpoly
      complex *16 work(MAX_ORDER)
      character Capture_file*64, Cyes*5
c
      include 'calhw1.cmn'
      save capture_file, cap_len
      data capture_file / ' ' /
      data cap_len / 1 /
c
c       do polynomial regression and then calculate values of the fitting
c       polynomial and the fitting error
c
c       ... for averaged calibration
c
1     if ( iwire .eq. WIRE_AVERAGE ) then                                 1b
          do 2 i = 1, n_points(1)
              voltage(i) = mean(i,WIRE_MEAN) - poly_offset
              weight(i)  = max( 0.001, mean(i,AIR_SPEED) ) ** poly_wexp
2             continue
          call spyorth( voltage(1), mean(1,AIR_SPEED), weight(1),
     1        n_points(1), poly_poly, poly_order, poly_error, work )
          do 3 i = 1, n_points(1)                                         2b
              fitted_poly(i)   = Qpoly( mean(i,WIRE_MEAN) - poly_offset,
     1            poly_poly, poly_order )
              error(i) = mean(i,AIR_SPEED) - fitted_poly(i)
3             continue                                                    2e
c
c       ... for individual calibration
c
      else
          do 4 i = 1, n_points(1)
              voltage(i) = calib(i,WIRE_MEAN,iwire) - poly_offset
              weight(i)  = max( 0.001, calib(i,AIR_SPEED,iwire) )
     1            ** poly_wexp
4             continue
          call spyorth( voltage(1), calib(1,AIR_SPEED,iwire), weight(1),
     1        n_points(iwire), poly_poly, poly_order, poly_error, work )
          do 5 i = 1, n_points(iwire)                                     2b
              fitted_poly(i)   = Qpoly( calib(i,WIRE_MEAN,iwire) -
     1            poly_offset, poly_poly, poly_order )
              error(i) = calib(i,AIR_SPEED,iwire) - fitted_poly(i)
5             continue                                                    2e
          end if                                                          1e
      poly_error = sqrt( poly_error )
      write( STD_OUT, 6 ) poly_error
6     format( ' Rms fitting error ', g12.5 )
      done_poly = .TRUE.
c
c       menu for displaying results
c
      first_loop = .TRUE.
c     start of loop                                                       1b
7         if ( .not. first_loop ) call newscr()
          first_loop = .FALSE.
          write( STD_OUT, 8 ) Capture_file(1:cap_len)
8         format( ' Display menu' / '   0) Exit' /
     1        '   1) Display calibration' /
     2        '   2) Display table of data' /
     3        '   3) Plot data & fitted polynomial' /
     4        '   4) Capture file  ', a /
     5        '   5) Change parameters & recalculate' /
     6        '   6) Help' )
          if ( aux_lu .gt. 0 )
     1        write( STD_OUT, 9 ) Cyes( aux_enable )
9             format( '   7) Output to auxillary file ', a )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c       DISPLAY CALIBRATION
c
          else if ( item .eq. 1 ) then                                    2m
              call display_calibr( CALIBR_HEADER + CALIBR_POLY,
     1          iwire, Eyes( aux_enable, 'Write to auxillary file ?' ) )
              call ilgwait()
c
c       DISPLAY DATA TABLE
c
          else if ( item .eq. 2 ) then                                    2m
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  write( STD_OUT, 10 )   ( i, mean(i,AIR_SPEED),
     1                mean(i,WIRE_MEAN), fitted_poly(i), error(i),
     2                i = 1, n_points(1) )
10                format( /
     1              16('*'), ' Polynomial calibration data ', 16('*') /
     2              ' No. Air-speed Mean-volts polynomial fit-error' /
     3              ( 1x, i3, 4f10.4 ) )
                  if ( Eyes( aux_enable, 'Write to auxillary file ?' )
     1                 .and. aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     2                write( aux_lu, 10 ) ( i, mean(i,AIR_SPEED),
     3                    mean(i,WIRE_MEAN), fitted_poly(i), error(i),
     4                    i = 1, n_points(1) )
              else                                                        3m
                  write( STD_OUT, 10 )   ( i, calib(i,AIR_SPEED,iwire),
     1                calib(i,WIRE_MEAN,iwire), fitted_poly(i),
     2                error(i), i = 1, n_points(iwire) )
                  if ( Eyes( aux_enable, 'Write to auxillary file ?' )
     1                 .and. aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     2                write( aux_lu, 10 ) ( i, calib(i,AIR_SPEED,iwire),
     3                    calib(i,WIRE_MEAN,iwire), fitted_poly(i),
     4                    error(i), i = 1, n_points(iwire) )
                  end if                                                  3e
              call ilgwait()
c
c       PLOT DATA & FITTED POLYNOMIAL
c
          else if ( item .eq. 3 ) then                                    2m
c
c           plot data and fitted polynomial
              call ilgopen( Capture_file )
              call ilgpage( 'a4', 'wide', 2, 1 )
              call ilgslaw( 'lin', 'log' )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgline( 1, 0, 1 )
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  npts = n_points(1)
                  call ilgdata( mean(1,WIRE_MEAN),
     1                mean(1,AIR_SPEED), 1, npts, 1 )
                  call ilgline( 0, 1, 1 )
                  call ilgdata( mean(1,WIRE_MEAN),
     1                fitted_poly, 1, npts, 0 )
              else                                                        3m
                  npts = n_points(iwire)
                  call ilgdata( calib(1,WIRE_MEAN,iwire),
     1                calib(1,AIR_SPEED,iwire), 1, npts, 1 )
                  call ilgline( 0, 1, 1 )
                  call ilgdata( calib(1,WIRE_MEAN,iwire),
     1                fitted_poly, 1, npts, 0 )
                  end if                                                  3e
              call ilgxlab( 'CTA output (volts)' )
              call ilgylab( 'Air speed (m/s)' )
              call ilgtitl( 'Polynomial calibration' )
              call ilgnote( '|2l |5-1 fitted curve|r   |s1 = data' )
c
c           plot RMS signal fluctuation
              call ilgpage( 'a4', 'WIDE', 2, 2 )
              call ilgraph( 2, 1, 11, 11, 1 )
              call ilgslaw( 'lin', 'lin' )
              call ilgline( 4, 1, 1 )
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  call ilgdata( mean(1,WIRE_MEAN),
     1                mean(1,WIRE_RMS), 1, npts, 1 )
              else                                                        3m
                  call ilgdata( calib(1,WIRE_MEAN,iwire),
     1                calib(1,WIRE_RMS,iwire), 1, npts, 1 )
                  end if                                                  3e
              call ilgxlab( 'CTA output (volts)' )
              call ilgylab( 'Volts' )
              call ilgtitl( 'RMS signal fluctuation' )
c
c           plot fitting error
              call ilgraph( 2, 2, 11, 11, 1 )
              call ilgline( 5, 1, 1 )
              if ( iwire .eq. WIRE_AVERAGE ) then                         3b
                  call ilgdata( mean(1,WIRE_MEAN), error, 1,
     1                npts, 1 )
              else                                                        3m
                  call ilgdata( calib(1,WIRE_MEAN,iwire),
     1                error, 1, npts, 1 )
                  end if                                                  3e
              call ilgxlab( 'CTA output (volts)' )
              call ilgylab( 'Fitting Error' )
              call ilgwait
              call ilgclos
c
c       CAPTURE FILE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 11 )
11            format( ' Capture file name ? ', $ )
              call Cresp( STD_IN, Capture_file, MAX_TRIES )
              cap_len = max( 1, jljstr( Capture_file ) )
c
c       CHANGE PARAMETERS & RECALCULATE
c
          else if ( item .eq. 5 ) then
              call menu_poly_volts()
              goto 1
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 12 )
12            format( ' A calibration relation has been calculated ',
     1                'and is available for examination' /
     2            ' of (1) calibration polynomial coeffients, (2) ',
     3                'tabular comparison of data' /
     4            ' and fitted polynomial and (3) graphical ',
     5                'comparison of data and fitted' /
     6            ' polynomial . Change the calibration parameters ',
     7                'using menu item (5) .' )
              call ilgwait()
c
c       AUXILLARY FILE ENABLE
c
          else if ( item .eq. 7 .and. aux_lu .gt. 0 ) then                2m
              aux_enable = YES - aux_enable
              end if                                                      2e
          goto 7
c         end of loop                                                     1e
      end



      SUBROUTINE AVERAGE_CALIBRATIONS( CALIB_FILE )
c
c Averaging of calibrations already stored in file CALIB_FILE .
c
c Arguments
c   character calib_file*(*)  # name of calibration file
c
c local variables
c  logical
c    got_data       # calibration data has been read
c    got_list       # list of calibrations to average is available
c    written        # calibration is written to file
c    Equery         # function : read logical
c    read_avg_data  #          : read all calibration data for averaging
c    write_calibr   #          : write a calibration to file
c  integer
c    abscissa       # AIR_SPEED or WIRE_MEAN for abscissa
c    avg_list(MAX_WIRES)  # list of calibration records to be averaged
c    i              # looping index
c    item           # selected menu item
c    name_len       # length of calibration file name
c    n_average      # number of calibrations to be averaged
c    Jresp          # function : read integer
c    jljstr         #          : left justify string
c    read_list      #          : read list of integers
c  real
c    fit_error(MAX_MEAN)  # error of the fitted polynomial
c  character
c    Cdone*8        # function : 'done' / 'not done'
c
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical got_data, got_list, written,
     1    Equery, read_avg_data, write_calibr
      integer abscissa, avg_list(MAX_WIRES), i, item, name_len,
     1    n_average, Jresp, jljstr, read_list
      real fit_error(MAX_MEAN)
      character calib_file*(*), Cdone*8
c
      include 'calhw1.cmn'
      save got_list, n_average
      data done_poly, done_siddal, got_data, got_list, written /
     1  5* .FALSE. /
      data abscissa, n_average / 1, 0 /
c
c       initialize
c
      done_poly = .FALSE.
      done_siddal = .FALSE.
      written = .FALSE.
      got_data = .FALSE.
      name_len = max( 1, jljstr( Calib_file ) )
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) calib_file(1:name_len),
     1        ( avg_list(i), i=1,n_average )
2         format( ' Averaging menu ' / '  -1) Abandon' / '   0) Exit' /
     1        '   1) Calibration file', 15x, a /
     2        '   2) Set calibration parameters' /
     3        '   3) Select calibrations to average ', 8i3 )
          write( STD_OUT, 12 )
12        format( '   4) Help' )
          if ( got_list )
     1        write( STD_OUT, 3 ) Cdone(done_siddal), Cdone(done_poly)
3             format( '   5) Sort calibration data' /
     1            '   6) Edit calibration data' /
     2            '   7) Display calibration data' /
     3            '   8) Calculate Siddal-Davies law      ', a /
     4            '   9) Calculate polynomial of voltage  ', a /
     5            '  10) Display calibration' /
     6            '  11) Write calibration' )
          item = Jresp( STD_IN, -1, 11, MAX_TRIES )
c
c       ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, 'Abandon. Are you sure ? ' ) )
     1            return
c
c       EXIT (+WRITE)
c       If calibration has been calculated but not written, write it to file .
c       If some wires remain uncalibrated, query before exit
c
          else if ( item .eq. 0 ) then                                    2m
              if ( ( done_siddal .or. done_poly ) .and. .not. written )
     1                then                                                3b
                  written = write_calibr( MAGIC, calib_file,
     1                                    WIRE_AVERAGE )
                  end if                                                  3e
              call write_calibr( MAGIC, calib_file, WRITE_CLOSE )
              return
c
c       SPECIFY CALIBRATION FILE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Calibration file name ? ', $ )
              call Cresp( STD_IN, Calib_file, MAX_TRIES )
              name_len = max( 1, jljstr( Calib_file ) )
              got_data = .FALSE.
c
c       SET CALIBRATION PARAMETERS
c
          else if ( item .eq. 2 ) then                                    2m
              call calib_param_menu()
c
c       SELECT CALIBRATIONS TO AVERAGE
c
          else if ( item .eq. 3 ) then                                    2m
              n_average = read_list( avg_list, MAX_WIRES )
              call isortsh( avg_list, n_average )
              got_list = 0 .lt. n_average .and. n_average .le. MAX_WIRES
              got_data = .FALSE.
c
c       HELP
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' This menu is used for calculating ',
     1                'calibration relations from the data of' /
     2            ' one or more calibrations already written to ',
     3                'calibration file . Only the' /
     4            ' calibration file name and calibration parameters ',
     5                'can be changed before' /
     6            ' selecting the calibrations from which the data ',
     7                'is to be taken . After' /
     8            ' selecting the source calibrations, additional ',
     9                'menu items for processing' /
     a            ' the data become available .' )
              call ilgwait()
c
c       SORT CALIBRATION DATA
c
          else if ( got_list ) then                                       2m
c
c       read the calibration file
c
              if ( .not. got_data ) then                                  3b
                  got_data = read_avg_data( MAGIC, calib_file, avg_list,
     1                n_average )
                  if ( got_data ) then                                    4b       4m
                      done_siddal = .FALSE.
                      done_poly = .FALSE.
                      end if                                              4e
                  end if                                                  3e
              if ( .not. got_data ) then                                  3b
                  write( STD_OUT, 6 )
6                 format( ' Cannot read data from calibration file .' )
                  call ilgwait()
              else if ( item .eq. 5 ) then                                3m
                  call sortsh( mean(1,1), n_points(1), 1, MAX_MEAN,
     1                WIRE_MEAN, 3 )
c
c       EDIT CALIBRATION DATA
c
              else if ( item .eq. 6 ) then                                3m
                  call edit_calibr( fit_error, abscissa, WIRE_AVERAGE )
c
c       DISPLAY CALIBRATION DATA
c
              else if ( item .eq. 7 ) then                                3m
                  write( STD_OUT, 7 )   ( i, mean(i,AIR_SPEED),
     1                mean(i,WIRE_MEAN), mean(i,WIRE_RMS),
     2                i = 1, n_points(1) )
7                 format( /
     1                16('*'), ' Calibration data ', 16('*') /
     2                ' No. Air-speed Mean-volts RMS-volts ' /
     3                ( 1x, i3, 3f10.4 ) )
                  if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1                write( aux_lu, 7 )   ( i, mean(i,AIR_SPEED),
     2                    mean(i,WIRE_MEAN), mean(i,WIRE_RMS),
     3                    i = 1, n_points(1) )
                  call ilgwait()
c
c       DO SIDDAL-DAVIES LAW
c
              else if ( item .eq. 8 ) then                                3m
                  call siddal_calibr( fit_error, WIRE_AVERAGE )
                  abscissa = AIR_SPEED
c
c       DO POLYNOMIAL OF VOLTAGE
c
              else if ( item .eq. 9 ) then                                3m
                  call poly_calibr( fit_error, WIRE_AVERAGE )
                  abscissa = WIRE_MEAN
c
c       DISPLAY CALIBRATION
c
              else if ( item .eq. 10 ) then                               3m
                  call display_calibr( CALIBR_HEADER, WIRE_AVERAGE,
     1                .TRUE.)
                  if ( done_siddal )
     1                call display_calibr( CALIBR_SIDDAL, WIRE_AVERAGE,
     2                    .TRUE. )
                  if ( done_poly )
     1                call display_calibr( CALIBR_POLY, WIRE_AVERAGE,
     2                    .TRUE. )
                  call ilgwait()
c
c       WRITE CALIBRATION
c
              else if ( item .eq. 11 ) then                               3m
                  if ( done_siddal .or. done_poly ) then                  4b
                      written = write_calibr( MAGIC, calib_file,
     1                                        WIRE_AVERAGE )
                  else                                                    4m
                      write( STD_OUT, 8 )
8                     format( ' Average calibration of hot-wire .' )
                      call ilgwait()
                      end if                                              4e
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of loop
      end



      INTEGER FUNCTION READ_LIST( LIST, MAX_SIZE )
c
c   Read a list of up to MAX_SIZE integers from the keyboard into the
c   array LIST . The returned value of RAW_READ_LIST is the number of
c   values actually read . The values read must be between 0 and 10000
c   inclusive .
c
c Arguments
c  integer list(*)      # OUT # list of values
c          max_size     # IN  # maximum number of values
c Local variables
c  integer count        # number of values read
c          i            # looping index
c          rec_no       # record number
c  character Cline*64   # one line of input
c  subroutine scase     # change case
c
      include 'stdio.def'
      integer count, i, list(*), max_size, rec_no
      character Cline*64
c
      write( STD_OUT, 1 ) max_size
1     format( ' Enter list of calibration records (in the calibration' /
     1        '    file) to read for averaging calibration data .' /
     2        '    Enter one record number on each line .' /
     3        '    Do not not repeat record numbers .' /
     4        '    Maximum list length is ', i4 /
     5        '    Terminate the list by typing "end" .' // )
      count = 0
c
c           check if the array is full
c
c     start of loop                                                       1b
2         if ( count .ge. max_size ) then                                 2b
              write( STD_OUT, 3 )
3             format( ' list is full' )
              read_list = count
              return
              end if                                                      2e
c
c           read a line
c
          write( STD_OUT, 4 )
4         format( ' Record number ?  ', $ )
          call Cresp( STD_IN, Cline, 4 )
          call scase( cline, cline, 'L' )
          if ( Cline .eq. 'end' ) then                                    2b
              read_list = count
              return
              end if                                                      2e
c
c           decode integer value
c
          read( Cline, *, ERR=7 ) rec_no
c
c           check that the value is not already in the list
c
          do 6 i = 1, count                                               2b
              if ( rec_no .eq. list(i) ) then                             3b
                  write( STD_OUT, 5 ) rec_no, i
5                 format( ' Sorry,', i3, ' is already the', i3,
     1                'th list element .' )
                  goto 2
                  end if                                                  3e
6             continue                                                    2e
c
c           if we get to here, the value is ok; so put it in the list
c
          count = count + 1
          list(count) = rec_no
          goto 2
c         end of loop                                                     1e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c           error traps
c
7         write( STD_OUT, 8 )
8         format( ' Cannot decode value' )
          goto 2
      end



      SUBROUTINE ISORTSH( IARRAY, SIZE )
c
c               SORT ( SHell )
c (Shell) sort the contents of the array IARRAY into ascending order .
c
c Arguments
c   integer iarray(*) # operand array
c           size      # length of the array
c local variables
c   integer i1,i2,i3  # looping indices
c           id        # sort segment size
c
      include 'stdio.def'
      integer iarray(*), size, i1, i2, i3, id, itmp
c
      if ( size .lt. 2 ) return
      id = 2 ** int( log( real( size ) ) / log( 2.0 ) + 0.0001 ) - 1
c     repeat                                                              1b
1         do 3 i1 = 1, size - id                                          2b
              do 2 i2 = i1, 1, -id                                        3b
                  i3 = i2 + id
                  if ( iarray(i2) .le. iarray(i3) ) goto 3
                  itmp = iarray(i3)
                  iarray(i3) = iarray(i2)
                  iarray(i2) = itmp
2                 continue                                                3e
3             continue                                                    2e
          id = id / 2
          if ( id .gt. 0 ) goto 1
c     until ( id <= 0 )                                                   1e
      return
      end



      LOGICAL FUNCTION READ_AVG_DATA( IDENTIFICATION, CALIB_FILE,
     1                                LIST, LEN_LIST )
c
c Read the calibration data from the LEN_LIST calibrations specified in LIST
c stored in the calibration file CALIB_FILE .
c
c Arguments
c   character identification*2   # 2 character record identification
c             Calib_file*(*)  # name of calibration file
c   integer list(*)           # list of calibration records required
c           len_list          # number of calibration records
c
c local variables
c  logical
c    check_avg_amplif # function check amplifier settings
c    Existf     # function : does a file exist ?
c  integer
c    calib_lu   # logical unit number of calibration file
c    count      # counter for calibration records in file
c    i          # implicit looping index
c    list_index # pointer to an item in the LIST of calibration record numbers
c    name_len   # non-blank length of CALIB_FILE
c    npts       # number of data points appended to the list
c    new_lun    # function : get new logical unit number
c    str_length #          : non-blank length of string
c  character
c    Cmagic*2   # Magic (ID) number for a calibration file record
c
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical check_avg_amplif, Existf
      integer count, list(*), len_list, calib_lu, i, list_index,
     1    name_len, npts, new_lun, str_length
      character identification*2, Cmagic*2, Calib_file*(*)
      include 'calhw1.cmn'
c
c
      read_avg_data = .FALSE.
c
c       open file
c
      name_len = max( 1, str_length( calib_file ) )
      if ( .not. Existf( Calib_file ) ) then                              1b
          write( STD_ERR, 1 ) Calib_file(1:name_len)
1         format( ' Cannot find calibration file ', a )
          call ilgwait()
      else                                                                1m
          if ( calib_lu .eq. 0 ) then
              calib_lu = new_lun()
              end if
          if ( calib_lu .eq. 0 )
     1        goto 7
          open( UNIT=calib_lu, FILE=Calib_file, STATUS='old',
     1        ACCESS='sequential', FORM='unformatted', ERR=7,
     2        RECORDTYPE='variable', BLOCKSIZE=8192 )
c
c       write to the file .
c
          n_points(1) = 0
          list_index = 1
          do 2 count = 1, list(len_list)                                  2b
              if ( count .eq. list(list_index) ) then                     3b
                  read( calib_lu, ERR=3, END=5 ) Cmagic,
     1                calib_no, Ctype, iplug, trip_diam, Cwire_ad,
     2                wire_chan(list_index), npts,
     3                Amp_gain(list_index), Amp_offset(list_index),
     4            done_siddal, siddal_expon, siddal_order, siddal_error,
     5                    ( siddal_poly(i), i = 1, siddal_order+1 ),
     6                done_poly, poly_order, poly_error, poly_offset,
     7                    ( poly_poly(i+1), i = 0, poly_order ),
     8                ( mean(i,AIR_SPEED), mean(i,WIRE_MEAN),
     9                    mean(i,WIRE_RMS),
     a                    i = n_points(1) + 1, n_points(1) + npts )
                  if ( Cmagic .ne. identification )
     1                goto 9
                  n_points(1) = n_points(1) + npts
                  list_index = list_index + 1
              else                                                        3m
                  read( calib_lu, ERR=3, END=5 )
                  end if                                                  3e
2            continue                                                     2e
          close( UNIT=calib_lu)
c
c       check the A/D converter channels
c
          read_avg_data = check_avg_amplif( list_index - 1 )
          calib_no = calib_no + 1
          end if                                                        1e
      return
c
c       error traps
c
3     write( STD_ERR, 4 ) calib_file(1:name_len)
4     format( ' Cannot read calibration file ', a )
      return
5     write( STD_ERR, 6 ) calib_file(1:name_len)
6     format( ' Unexpected EOF in calibration file ', a,
     1    ' Cannot complete reading file' )
      close( UNIT=calib_lu )
      return
7     write( STD_ERR, 8 ) calib_file(1:name_len)
8     format( ' Cannot open calibration file ', a )
      close( UNIT=calib_lu )
      return
9     write( STD_ERR, 10 ) calib_file(1:name_len)
10     format( 1x, a, ' is not a calibration file .' /
     1    ' Illegal magic number .' )
      end



      LOGICAL FUNCTION CHECK_AVG_AMPLIF( N_CALIBS )
c
c Check the amplifier gain and offset (and also the A/D converter channel)
c to be used for the averaged calibrations .
c
c Argument
c  integer n_calibs   # number of calibrations read for averaging
c
c local variables
c  logical
c    same_chan  # all A/D channels of the averaged calibrations are the same
c  integer
c    del_index  # index of amplifier gain/offset to be deleted
c    i          # looping index
c    item       # selected menu item
c    sel_amplif # NO/YES - select amplifier from list
c    sel_chan   # selected A/D converter channel
c    sel_mean   # NO/YES - selecting mean amplifier gain & offset
c    Jresp      # function : read integer
c  real
c    avg_gain   # average amplifier gain
c    avg_offset # average amplifier offset
c    sel_gain   # selected amplifier offset
c    sel_offset # selected amplifier offset
c    qmean      # function : mean
c    Qresp      #          : read real
c  character
c    Cyes*5     # function : 'no', 'yes', 'query'
c  subroutine
c    ilgwait
c    newscr
c
      include 'stdio.def'
      include 'calhw1.def'
c
      logical same_chan
      integer del_index, i, item, n_calibs, sel_amplif, sel_chan,
     1    sel_mean, Jresp
      real avg_gain, avg_offset, sel_gain, sel_offset, qmean, Qresp
      character Cyes*5
      include 'calhw1.cmn'
c
      same_chan = .TRUE.
      sel_chan = wire_chan(1)
      do 1 i = 2, n_calibs
          same_chan = same_chan .and. sel_chan .eq. wire_chan(i)
1         continue
      if ( .not. same_chan ) then
          write( STD_OUT, 2 ) sel_chan
2         format( ' A/D channel changed. Using channel ', i2 )
          call ilgwait()
          end if
      avg_gain = qmean( amp_gain, 1, n_calibs )
      sel_gain = avg_gain
      avg_offset = qmean( amp_offset, 1, n_calibs )
      sel_offset = avg_offset
      sel_amplif = 0
      sel_mean = YES
c
c     start of loop                                                       1b
3         call newscr()
          write( STD_OUT, 4 ) avg_gain, avg_offset, ( i, amp_gain(i),
     1        amp_offset(i), wire_chan(i), i = 1, n_calibs )
4         format( '  -----amplifier-----' /
     1            '  No.   gain   offset  A/D chan' /
     2            ' mean ', 2f8.3 / ( 2x, i2, 2x, 2f8.3, 2x, i2 ) )
          write( STD_OUT, 5 ) sel_amplif, Cyes(sel_mean), sel_chan,
     1        sel_gain, sel_offset
5         format( / ' Check amplifier gain and offset' /
     Z        '  -1) Abandon' / '   0) Exit (ok)' /
     1        '   1) Selected amplifier ', i2 /
     2        '   2) Selected mean      ', a /
     3        '   3) Selected A/D chan  ', i2 /
     4        '   4) Amplifier gain     ', f8.3 /
     5        '   5) Amplifier offset   ', f8.3 /
     6        '   6) Delete amplifier' /
     7        '   7) Help   ', $ )
          item = Jresp( STD_IN, -1, 7, MAX_TRIES )
c
c           ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              check_avg_amplif = .FALSE.
              return
c
c           EXIT (OK)
c
          else if ( item .eq. 0 ) then                                    2m
              amp_gain(1)   = sel_gain
              amp_offset(1) = sel_offset
              wire_chan(1)  = sel_chan
              check_avg_amplif = .TRUE.
              return
c
c           SELECTED AMPLIFIER
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Which amplifier ? ', $ )
              sel_amplif = Jresp( STD_IN, 1, n_calibs, MAX_TRIES )
              sel_mean = NO
              sel_gain = amp_gain(sel_amplif)
              sel_offset = amp_offset(sel_amplif)
c
c           SELECTED MEAN
c
          else if ( item .eq. 2 ) then                                    2m
              sel_mean = YES
              sel_gain = avg_gain
              sel_offset = avg_offset
              sel_amplif = 0
c
c           SELECTED A/D CHAN
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 7 )
7             format( ' A/D converter channel ', $ )
              sel_chan = Jresp( STD_IN, 0, 15, MAX_TRIES )
c
c           AMPLIFIER GAIN
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 8 )
8             format( ' Amplifier gain ? ', $ )
              sel_gain = Qresp( STD_IN, -125.0, 125.0, MAX_TRIES )
              sel_amplif = 0
              sel_mean = NO
c
c           AMPLIFIER OFFSET
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 9 )
9             format( ' Amplifier offset ? ', $ )
              sel_offset = Qresp( STD_IN, -12.0, 12.0, MAX_TRIES )
              sel_amplif = 0
              sel_mean = NO
c
c           DELETE AMPLIFIER
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 10 )
10            format( ' Delete which amplifier ? ', $ )
              del_index = Jresp( STD_IN, 1, n_calibs, MAX_TRIES )
              do 11 i = del_index + 1, n_calibs
                  amp_gain(i-1) = amp_gain(i)
                  amp_offset(i-1) = amp_offset(i)
11                continue
              n_calibs = n_calibs - 1
              avg_gain = qmean( amp_gain, 1, n_calibs )
              avg_offset = qmean( amp_offset, 1, n_calibs )
c
c           HELP
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 12 )
12            format( ' The specified calibration data has been read ',
     1                'from the calibration file .' /
     2             ' Using this menu, you can now edit the matching ',
     3                'amplifier gain & offset' /
     4             ' before continuing with the analysis .' )
              call ilgwait()
              end if                                                      2e
          goto 3
c         end of loop                                                     1e
      end



      CHARACTER *5 FUNCTION CYES( BOOLEAN )
c
c Returns a character string depending on the value of BOOLEAN :
c   'no   '  for  0,
c   'yes  '       1,
c   'query'       2,
c   '????? '      anything else
c
      include 'stdio.def'
      integer boolean
c
      if ( boolean .eq. 0 ) then                                          1b
          Cyes = 'no '
      else if ( boolean .eq. 1 ) then                                     1m
          Cyes = 'yes'
      else if ( boolean .eq. 2 ) then                                     1m
          Cyes = 'query'
      else                                                                1m
          Cyes = '?????'
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION EYES( BOOLEAN, QUERY_MESSAGE )
c
c Converts the integer value BOOLEAN which must be NO, YES or QUERY to
c a logical value . If BOOLEAN is QUERY, the user is requested to make
c YES/NO decision .
c
c
      include 'stdio.def'
      include 'calhw1.def'
      integer boolean
      logical Equery
      character query_message*(*)
c
      if ( boolean .eq. NO ) then
          eyes = .FALSE.
      else if ( boolean .eq. YES ) then
          eyes = .TRUE.
      else if ( boolean .eq. QUERY ) then
          eyes = Equery( STD_IN, query_message )
      else
          write( STD_ERR, 1 ) boolean
1         format( ' EYES : WARNING : Illegal boolean ', i12 )
          end if
      return
      end



      CHARACTER *8 FUNCTION CDONE( BOOLEAN )
c
c Returns 'done' if the argument is .TRUE., otherwise returns 'not done'
c
      include 'stdio.def'
      logical boolean
c
      if ( boolean ) then
          Cdone = 'done'
      else
          Cdone = 'not done'
          end if
      return
      end



      BLOCK DATA CALHW1_BLOCK
c
c Initialize common block data .
c
      include 'stdio.def'
      include 'calhw1.def'
      include 'calhw1.cmn'
c
c       miscellaneous
      data all_modules, aux_lu, plot_spect, rdf_rec, verbosity /
     1     YES, 0, NO, 0, 1 /
c
c       calibration data
      data n_points, n_offsets, n_wires, ref_wire, wire_chan /
     1    MAX_WIRES*0, 1, 1, 1, MAX_WIRES*0 /
      data press_conv / 1.0 /
c
c       incidental calibration information
      data calib_no, host_type, iplug / 0, 2, 3 /
      data trip_diam / -1.0 /
      data Ctype / 'single normal hot-wire' /
c
c       Siddal-Davies method
      data done_siddal / .FALSE. /
      data siddal_davies, siddal_optim, siddal_order, siddal_wire /
     1    YES, NO, 2, 1 /
      data siddal_expon, siddal_wexp / 0.5, -0.0 /
c
c       Polynomial-of-voltage method
      data done_poly / .FALSE. /
      data poly_volts, poly_order / YES, 4 /
      data poly_offset, poly_wexp / 5.0, -1.0 /
      end



      SUBROUTINE LATEST_FILE_VERSION( CALIB_FILE )
c
c Argument
c  character calib_file*(*) # logical unit number of the calibration file
c
c constant
c  NOMINAL_TRIP=1.0
c
c  integer
c   i         # looping array index
c   in_lu     # logical unit number for old calibration file
c   name_len  # non-blank length of calibration file name
c   out_lu    # logical unit number for new calibration file
c   new_lun() # function : get new logical unit number
c   str_length#          : non-blank string length
c  logical
c   Existf    # function : file exists ?
c  character
c   abort_label*18 # ' CALHW1 : ABORT : '
c   Cmagic*2  # magic number of file record
c   identification*2
c  subroutine
c   ilgwait
c
c
      include 'stdio.def'
      include 'calhw1.def'
      real NOMINAL_TRIP
      character abort_label*18
      parameter( NOMINAL_TRIP=1.0, abort_label=' CALHW1 : ABORT : ' )
c
      integer i, in_lu, name_len, out_lu, new_lun, str_length
      logical Existf
      character calib_file*(*), Cmagic*2, identification*2
c
      include 'calhw1.cmn'
c
c       open file
c
      name_len = max( 1, str_length( calib_file ) )
      if ( .not. Existf( Calib_file ) ) then                              1b
          write( STD_ERR, 1 ) Calib_file(1:name_len)
1         format( ' CALHW1 : ABORT : Cannot find calibration file ', a )
          call ilgwait()
      else                                                                1m
          if ( in_lu .eq. 0 ) then
              in_lu = new_lun()
              end if
          if ( in_lu .eq. 0 )
     1        goto 5
          open( UNIT=in_lu, FILE=Calib_file, STATUS='old',
     1        ACCESS='sequential', FORM='unformatted', ERR=5,
     2        RECORDTYPE='variable', BLOCKSIZE=8192 )
c
c       read the magic number and identify the file version number so that
c       the file can be read correctly
c
          read( in_lu, ERR=9, END=9 ) identification
          if ( identification .eq. MAGIC ) then                           2b
              write( STD_OUT, 2 ) Calib_file
2             format( ' CALHW1 : NOTE : File ', a, 'is already the ',
     1            'latest version; no conversion required. ' )
              call ilgwait()
              close( UNIT=in_lu )
              return
              end if                                                      2e
          rewind( UNIT=in_lu )
c
c       open the NEW converted calibration file
c
          if ( out_lu .eq. 0 ) then                                       2b
              out_lu = new_lun()
              end if                                                      2e
          if ( out_lu .eq. 0 )
     1        goto 5
          open( UNIT=out_lu, FILE=Calib_file, STATUS='new',
     1          ACCESS='sequential', FORM='unformatted', ERR=5,
     2          RECORDTYPE='variable', BLOCKSIZE=8192 )
c
c         start of loop -- one loop for each file record
c
c       read Version 1 file record
c
3             continue                                                    2b
              if ( identification .eq. MAGIC_V1 ) then                    3b
                  read( in_lu, ERR=7, END=4 )
     1                Cmagic, calib_no, Ctype, iplug, Cwire_ad,
     2            wire_chan(1), n_points(1), Amp_gain(1), Amp_offset(1),
     3            done_siddal, siddal_expon, siddal_order, siddal_error,
     4                    ( siddal_poly(i), i = 1, siddal_order + 1 ),
     5                done_poly, poly_order, poly_error, poly_offset,
     6                    ( poly_poly(i), i = 1, poly_order + 1 ),
     7                ( mean(i,AIR_SPEED), mean(i,WIRE_MEAN),
     8                    mean(i,WIRE_RMS), i = 1, n_points(1) )
                  trip_diam = NOMINAL_TRIP
                  if ( Cmagic .ne. identification )
     1                goto 11
c
c       Illegal magic number
c
              else                                                        3m
                  goto 11
                  end if                                                  3e
c
c       write lastest version record
c
              write( out_lu, ERR=14) identification,
     1            calib_no, Ctype, iplug, trip_diam, Cwire_ad,
     2            wire_chan(1), n_points(1),
     3            Amp_gain(1), Amp_offset(1),
     4            done_siddal, siddal_expon, siddal_order, siddal_error,
     5                ( siddal_poly(i), i = 1, siddal_order+1 ),
     6            done_poly, poly_order, poly_error, poly_offset,
     7                ( poly_poly(i+1), i = 0, poly_order ),
     8            ( mean(i,AIR_SPEED), mean(i,WIRE_MEAN),
     9                mean(i,WIRE_RMS), i = 1, n_points(1) )
              goto 3
c             end of loop                                                 2e
c
c       normal end of input file
4         close( UNIT=in_lu )
          close( UNIT=out_lu )
          end if                                                          1e
      return
c
c       ABNORMAL RETURN TRAPS
c       *********************
c
c       cannot open input file
c
5     write( STD_ERR, 6 ) abort_label, calib_file(1:name_len), 'input'
6     format( a, 'Cannot open file ', a, ' for', a )
      close( UNIT=in_lu )
      return
c
c       cannot read input file
c
7     write( STD_ERR, 8 ) abort_label, 'read', calib_file(1:name_len)
8     format( a, 'Cannot ', a, ' file ', a )
      goto 15
c
c       empty input file
c
9     write( STD_ERR, 10 ) abort_label, calib_file(1:name_len)
10    format( a, 'Empty input file ', a )
      close( UNIT=in_lu )
      call ilgwait()
      return
c
c       dodgy magic number
c
11    write( STD_ERR, 12 ) abort_label
12    format( a, 'Magic number (record identification) changed or '
     1        'illegal' )
      goto 15
c
c       cannot open output file
c
13    write( STD_ERR, 6 ) abort_label, calib_file(1:name_len), 'output'
      goto 15
c
c       cannot write output file
c
14    write( STD_ERR, 8 ) abort_label, 'write', calib_file(1:name_len)
15    close( UNIT=in_lu )
      close( UNIT=out_lu )
      call ilgwait()
      return
      end


