c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM DRIFT
c
c This program processes the data from experiments which measure the
c drift in hot-wire anemometer output voltage as a function of time and
c temperature.
c
c Read an edited auxillary file from RAWLOOK, convert the date-time stamp 
c to time in minutes, convert the temperature readings to Celsius, and add
c the amplifier OUTPUT offset voltage to the hot-wire readings .
c
c drift [-n <number_of_wires>] [-i <input_file>] [-o <output_file>]
c       [-s <setup_file>]
c
c
c constant
c  MAX_TRIES=4
c
c local variables
c  integer
c    append       # 1 (0) = (not) append results without resetting timer
c    iadjust      # 1 (0) = (not) adjust for voltage jump between I/P files
c    in_lu        # logical unit number of input file
c    item         # selected menu item
c    len_in       # length of input file name
c    len_out      # length of output file name
c    nwires       # number of hot-wires
c    out_lu       # logical unit number of output file
c    res_in_lu    # reserved logical unit number of input file
c    res_out_lu   # reserved logical unit number of output file
c    iargc        # function : count program arguments
c    jljstr       #          : left justify string
c    Jresp        #          : read integer
c    new_lun      #          : get new logical unit number
c    open_file    #          : open file
c  character
c    Cin_file*64  # name of input file
c    Cout_file*64 # name of output file
c    Csetup_file*64 # name of setup file
c    Cmode(0:1)*12  # 'reset timer', 'append timer'
c    Cyes(0:1)*3    # 'no ', 'yes'
c
      include 'stdio.def'
      integer MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      integer append, iadjust, in_lu, item, len_in, len_out, nwires,
     1    out_lu, res_in_lu, res_out_lu,
     2    iargc, jljstr, Jresp, new_lun, open_file
      character Cin_file*64, Cout_file*64, Csetup_file*64, Cold_out*64,
     1    Cmode(0:1)*12, Cyes(0:1)*3
      data Cin_file, Cout_file, Csetup_file, Cold_out, Cmode, Cyes /
     1    ' ', ' ', ' ', ' ', 'reset timer', 'append timer',
     2    'no ', 'yes' /
      data len_in, len_out, nwires / 1, 1, 4 /
c
      call parse_args( Cin_file, Cout_file, Csetup_file, nwires )
      len_in = max( 1, jljstr( Cin_file ) )
      res_in_lu = new_lun()
      len_out = max( 1, jljstr( Cout_file ) )
      res_out_lu = new_lun()
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) Cin_file(1:len_in), Cout_file(1:len_out),
     1        Cmode(append), Cyes(iadjust)
2         format( ' Select' / '   0) exit' /
     1      '   1) input file    ', a /
     2      '   2) output file   ', a /
     3      '   3) set analysis parameters' /
     4      '   4) process data' /
     5      '   5) ', a /
     6      '   6) inter-file adjust ', a /
     7      '   7) help  ', $ )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
          if ( item .eq. 0 ) then
              stop
c
c           INPUT FILE
c
          else if ( item .eq. 1 ) then
              write( STD_OUT, 3 )
3             format( ' File name ? ', $ )
              call cresp( STD_IN, Cin_file, MAX_TRIES )
              len_in = max( 1, jljstr( Cin_file ) )
c
c           OUTPUT FILE
c
          else if ( item .eq. 2 ) then
              write( STD_OUT, 3 )
              call cresp( STD_IN, Cout_file, MAX_TRIES )
              len_out = max( 1, jljstr( Cout_file ) )
c
c           SET ANALYSIS PARAMETERS
c
          else if ( item .eq. 3 ) then
              call set_param( Csetup_file, nwires )
c
c           PROCESS DATA
c
          else if ( item .eq. 4 ) then
c
c           open input and output files
c
              in_lu = open_file( res_in_lu, Cin_file, 'old', 0 )
              if ( out_lu .ne. 0 .and. Cout_file .ne. Cold_out ) then
                  if ( out_lu .ne. STD_OUT )
     1                close( UNIT=out_lu )
                  out_lu = 0
                  end if
              if ( out_lu .eq. 0 .and. Cout_file .ne. Cold_out ) then
                  out_lu = open_file( res_out_lu, Cout_file, 'new', 0 )
                  Cold_out = Cout_file
                  end if
              if ( in_lu .ne. 0 .and. out_lu .ne. 0 ) then
                  call process_data( in_lu, out_lu, nwires, append,
     1                iadjust )
                  close( UNIT=in_lu )
                  end if
c
c           APPEND_MODE
c
          else if ( item .eq. 5 ) then
              append = 1 - append
c
c           INTER-FILE ADJUST
c
          else if ( item .eq. 6 ) then
              iadjust = 1 - iadjust
c
c           HELP
c
          else if ( item .eq. 7 ) then
              write( STD_OUT, 4 )
4             format(
     1    ' DRIFT is for processing data from hot-wire drift' /
     2    ' experiments . It reads an edited auxillary file produced' /
     3    ' by rawlook, converts the date-time stamp to time in' /
     4    ' minutes, converts the temperature readings to Celsius,' /
     5    ' and converts the hot-wire amplifier output voltage to CTA' /
     6    ' output voltage .' / / ' Form of invocation :' /
     7    '  drift [-n <number_of_wires>] [-i <input_file>]' /
     8    '        [-o <output_file>]     [-s <setup_file>]' //
     9    ' Input file format : Blank lines are ignored . Comment' /
     a    ' lines begin with '';'' and are copied directly to the', /
     b    ' output file . All other lines begin with a time/date' /
     c    ' stamp, followed by a thermometer reading and' /
     d    ' <number_of_wires> hot-wire mean output readings .' /
     e    ' Press <CR>' )
              call wait
              end if
          goto 1
c         end of loop
      end



      SUBROUTINE PARSE_ARGS( IN_FILE, OUT_FILE, SETUP_FILE, N_WIRES )
c
c drift [-n <number_of_wires>] [-i <input_file>] [-o <output_file>]
c       [-s <setup_file>]
c
c Arguments
c  character in_file*(*)      # name of input data file
c            out_file*(*)     # name of output data file
c            setup_file*(*)   # name of setup file
c  integer n_wires            # number of hot-wires
c
c local variables
c  integer
c    arg_no   # argument number
c    n_args   # number of arguments
c    iargc    # function : number of arguments
c  character
c    Carg*64  # argument string
c
      include 'stdio.def'
      integer MAX_WIRES
      parameter( MAX_WIRES=4 )
c
      integer arg_no, n_args, n_wires, iargc
      character Carg*64, in_file*(*), out_file*(*), setup_file*(*)
c
      n_args = iargc()
      arg_no = 1
c     start of loop
1     if ( arg_no .le. n_args ) then
          call getarg( arg_no, Carg )
          call scase( Carg, Carg, 'l' )
          if ( Carg(1:2) .eq. '-n' ) then
              arg_no = arg_no + 1
              if ( arg_no .le. n_args ) then
                  call getarg( arg_no, Carg )
                  read( Carg, *, ERR=1 ) n_wires
                  n_wires = max( 1, min( n_wires, MAX_WIRES ) )
                  end if
          else if ( Carg(1:2) .eq. '-i' ) then
              arg_no = arg_no + 1
              if ( arg_no .le. n_args )
     1            call getarg( arg_no, in_file )
          else if ( Carg(1:2) .eq. '-o' ) then
              arg_no = arg_no + 1
              if ( arg_no .le. n_args )
     1            call getarg( arg_no, out_file )
          else if ( Carg(1:2) .eq. '-s' ) then
              arg_no = arg_no + 1
              if ( arg_no .le. n_args )
     1            call getarg( arg_no, setup_file )
              end if
          arg_no = arg_no + 1
          goto 1
          end if
c         end of loop
      return
      end



      SUBROUTINE SET_PARAM( SETUP_FILE, N_WIRES )
c
c Set analysis parameters .
c
c Arguments
c  character setup_file*(*)   # name of setup file
c  integer n_wires            # number of hot_wires
c
c constant
c   MAX_TRIES=4
c   MAX_WIRES=4
c
c local variable
c  integer
c    file_len   # length of setup file name
c    i          # loop index
c    item       # selected menu item
c    iwire      # selected wire number
c    setup_lu   # logical unit number of setup file
c    jljstr     # function : left justify string
c    Jresp      #          : read integer
c    new_lun    #          : new logical number
c    open_file  #          : open file
c    str_length #          : non-blank string length
c  real Qresp   # function : read real
c  character
c    setup_file*64 # setup file name
c
c global variables
c  real
c    Therm_gain   # thermometer amplifier gain
c    Therm_off    # thermometer amplifier output offset
c    wire_gain(MAX_WIRES) # hot-wire amplifier gain
c    wire_off(MAX_WIRES)  # hot-wire amplifier output offset
c
c
      include 'stdio.def'
      integer MAX_TRIES, MAX_WIRES
      parameter( MAX_TRIES=4, MAX_WIRES=4 )
c
      integer file_len, i, item, iwire, n_wires, setup_lu,
     1    jljstr, Jresp, new_lun, open_file, str_length
      real Qresp
      character setup_file*(*)
c
c global variables
      real Therm_gain, Therm_off, wire_gain(MAX_WIRES),
     1    wire_off(MAX_WIRES)
      common /drift_cmn/ Therm_gain, Therm_off, wire_gain, wire_off
      data file_len, iwire / 1, 1 /
c
c
      file_len = max( 1, jljstr( setup_file ) )
c
c     start of loop
1         call newscr()
          write( STD_OUT, 2 ) Therm_gain, Therm_off, n_wires, iwire,
     1        wire_gain(iwire), wire_off(iwire), setup_file(:file_len)
2         format( ' Select ' / '   0) Exit' /
     1        '   1) Thermometer gain        ', f8.4 /
     2        '   2) Thermometer O/P offset  ', f8.4, ' volts' /
     3        '   3) Number of hot-wires     ', i4 /
     4        '   4) Selected hot-wire       ', i4 /
     5        '   5) Hot-wire amp gain       ', f8.4 /
     6        '   6) Hot-wire amp O/P offset ', f8.4, ' volts' /
     7        '   7) Set up file             ', a /
     8        '   8) Read setup file' /
     9        '   9) Write setup file  ', $ )
          item = Jresp( STD_IN, 0, 9, MAX_TRIES )
          if ( item .eq. 0 ) then
              return
          else if ( item .eq. 1 ) then
c
c           THERMOMETER GAIN
c
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              Therm_gain = Qresp( STD_IN, 1.0, 200.0, MAX_TRIES )
c
c           THERMOMETER OFFSET
c
          else if ( item .eq. 2 ) then
              write( STD_OUT, 3 )
              Therm_off = Qresp( STD_IN, -200.0, 200.0, MAX_TRIES )
c
c           NUMBER OF HOT-WIRES
c
          else if ( item .eq. 3 ) then
              write( STD_OUT, 3 )
              n_wires = Jresp( STD_IN, 1, MAX_WIRES, MAX_TRIES )
c
c           SELECTED HOT-WIRE
c
          else if ( item .eq. 4 ) then
              write( STD_OUT, 3 )
              iwire = Jresp( STD_IN, 1, n_wires, MAX_TRIES )
c
c           HOT-WIRE AMP GAIN
c
          else if ( item .eq. 5 ) then
              write( STD_OUT, 3 )
              wire_gain(iwire) = Qresp( STD_IN, 1.0, 10.0, MAX_TRIES )
c
c           HOT-WIRE AMP OFFSET
c
          else if ( item .eq. 6 ) then
              write( STD_OUT, 3 )
              wire_off(iwire) = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES)
c
c           SET UP FILE
c
          else if ( item .eq. 7 ) then
              write( STD_OUT, 3 )
              call Cresp( STD_IN, setup_file, MAX_TRIES )
              file_len = max( 1, jljstr( setup_file ) )
c
c           READ SETUP FILE
c
          else if ( item .eq. 8 ) then
              setup_lu = open_file( new_lun(), setup_file, 'old', 0 )
              call read_setup( setup_lu, Therm_gain, Therm_off )
              do 4 i = 1, n_wires
                  call read_setup( setup_lu, wire_gain(i), wire_off(i) )
4                 continue
              close( UNIT=setup_lu )
c
c           WRITE SETUP FILE
c
          else if ( item .eq. 9 ) then
              setup_lu = open_file( new_lun(), setup_file, 'new', 0 )
              write( setup_lu, 5 ) Therm_gain, Therm_off,
     1            ( wire_gain(i), wire_off(i), i, i = 1, n_wires )
5             format( ' ;   DRIFT SETUP FILE' /
     1            2f9.4, ' Thermometer gain and output offset' /
     2            4( 2f9.4, ' Gain and output offset for hot-wire ',
     3               i2 / ) )
              if ( setup_lu .eq. STD_OUT ) then
                  call wait()
              else
                  close( UNIT=setup_lu )
                  end if
              end if
          goto 1
c         end of loop
      end



      SUBROUTINE READ_SETUP( LUN, GAIN, OFFSET )
c
c Read 2 real numbers from a line of the file with logical unit number LUN .
c The first is an amplifier GAIN and the second is the output OFFSET of the
c same amplifier .
c
c Arguments
c  integer lun  # logical unit number of setup file
c  real gain    # ampliier gain
c       offset  # amplifier output offset
c
c Local variables
c  integer
c    nchars     # number of characters in line
c    second     # location of second token
c    jljstr     # function : left justify string
c  character
c    Cline*80   # line read from setup file
c
      include 'stdio.def'
      integer lun, nchars, second, jljstr
      real gain, offset
      character Cline*80
c
c       read a line from the file
c
1     read( lun, 2, END=4, ERR=6 ) Cline
2     format( a )
      nchars = jljstr( cline )
c
c       skip and read the next line if it is too short or is a comment
c
      if ( nchars .lt. 3 .or. cline(1:1) .eq. ';' )
     1    goto 1
c
c       find the end of the first value (first space)
c
      second = 2
3     if ( cline(second:second) .ne. ' ' ) then
          second = second + 1
          goto 3
          end if
c
c       decode the gain and offset, read the next line if cannot decode
c
      read( Cline(:second), *, ERR=1 ) gain
      read( Cline(second:), *, ERR=1 ) offset
      return
c
ccccccccccccccccccccccccccccccccccccccccccccccccccc
c       error traps
c
4     write( STD_ERR, 5 )
5     format( 'End of setup file' )
      return
c
6     write( STD_ERR, 7 )
7     format( 'Cannot read setup file' )
      return
      end



      SUBROUTINE PROCESS_DATA( IN_LU, OUT_LU, N_WIRES, APPEND, IADJUST )
c
c    Process the drift data . The files have already been opened .
c
c Arguments
c  integer in_lu    # logical unit number of input file
c          out_lu   # logical unit number of output file
c          n_wires  # number of hot-wires
c          append   # 1 (0) = (not) append results without resetting timer
c    iadjust        # 1 (0) = (not) adjust for voltage jump between I/P files
c
c constants
c  HOUR=1
c  MINUTE=2
c  SECOND=3
c  DAY=4
c  MONTH=5
c  YEAR=6
c
c local variables
c  logical
c    first_data     # the first line of data has not been read
c    operating(MAX_WIRES) # specifies which wires are still operating
c    saved_state    # saved hot-wire voltage & temperature from previous file
c    start_file     # at the start of the input file
c  integer
c    day_elapsed    # day number
c    i              # looping index
c    iline          # number of lines read
c    td_prev(6)     # previous time and date stamp
c    td_stamp(6)    # time and date stamp
c    str_length     # function : number of non-blank characters in string
c  real
c    elapsed_hours  # elapsed time
c    temp           # temperature
c    wire_volts(MAX_WIRES)  # mean voltage
c    saved_temp             # saved temperature
c    saved(MAX_WIRES)   # saved hot-wire voltage
c    adjust(MAX_WIRES)  # hot-wire voltage adjustment to match ends of files
c  character
c    Cline*80       # line read from input file
c    Cfirst_char*1  # function  : get the first non-blank character
c
c global variables
c  real
c    Therm_gain   # thermometer amplifier gain
c    Therm_off    # thermometer amplifier output offset
c    wire_gain(MAX_WIRES) # hot-wire amplifier gain
c    wire_off(MAX_WIRES)  # hot-wire amplifier output offset
c
c
      include 'stdio.def'
      integer DAY, MONTH, YEAR, HOUR, MINUTE, SECOND, MAX_WIRES
      parameter( DAY=4, MONTH=5, YEAR=6, HOUR=1, MINUTE=2, SECOND=3,
     1    MAX_WIRES=4 )
c
      logical first_data, operating(MAX_WIRES), saved_state, start_file
      integer append, days_elapsed, i, iadjust, iline, in_lu, out_lu,
     1    n_wires, td_prev(6), td_stamp(6), str_length
      real elapsed_hours, temp, wire_volts(MAX_WIRES), saved_temp,
     1    saved(MAX_WIRES), adjust(MAX_WIRES)
      character Cline*80, char1*1, Cfirst_char*1
c
c global variables
      real Therm_gain, Therm_off, wire_gain(MAX_WIRES),
     1     wire_off(MAX_WIRES)
      common /drift_cmn/ Therm_gain, Therm_off, wire_gain, wire_off
      save days_elapsed, td_prev, td_stamp, elapsed_hours, saved_temp,
     1    saved, adjust
      data first_data / .TRUE. /
      data saved, adjust / MAX_WIRES*0.0, MAX_WIRES*0.0 /
c
      start_file = .TRUE.
      if ( append .eq. 0 )
     1     first_data = .TRUE.
      iline = 0
      do 1 i = 1, n_wires
          operating(i) = .TRUE.
1         continue
c     start of loop                                                       1b
2         read( in_lu, 3, END=9, ERR=98 ) Cline
3         format( a )
          iline = iline + 1
          char1 = Cfirst_char(Cline)
c
c       copy comments to output
c
          if ( char1 .eq. ';' ) then                                      2b
              write( out_lu, 4, ERR=96 ) Cline( 1: str_length( Cline ) )
4             format( 1x, a )
          else if ( char1 .ne. ' ' ) then                                 2m
c
c       read time & data, and convert to elapsed time in hours
c
              if ( first_data ) then                                      3b
                  read( Cline(1:17), 5, ERR=94 ) td_stamp
5                 format( i2, 1x, i2, 1x, i2, 1x, i2, 1x, i2, 1x, i2 )
                  td_prev(HOUR)   = td_stamp(HOUR)
                  td_prev(MINUTE) = td_stamp(MINUTE)
                  td_prev(SECOND) = td_stamp(SECOND)
                  elapsed_hours = 0.0
                  days_elapsed = 0
                  first_data = .FALSE.
              else                                                        3m
                  td_prev(DAY)   = td_stamp(DAY)
                  td_prev(MONTH) = td_stamp(MONTH)
                  td_prev(YEAR ) = td_stamp(YEAR)
                  read( Cline, 5, ERR=94 ) td_stamp
                  if ( td_prev(DAY)   .ne. td_stamp(DAY) .or.
     1                 td_prev(MONTH) .ne. td_stamp(MONTH) .or.
     2                 td_prev(YEAR)  .ne. td_stamp(YEAR) )
     3                    days_elapsed = days_elapsed + 1
                  td_stamp(HOUR)   = td_stamp(HOUR)   - td_prev(HOUR)
                  td_stamp(MINUTE) = td_stamp(MINUTE) - td_prev(MINUTE)
                  td_stamp(SECOND) = td_stamp(SECOND) - td_prev(SECOND)
                  elapsed_hours = real( days_elapsed * 24 +
     1                td_stamp(HOUR) ) + td_stamp(MINUTE) / 60.0 +
     2                td_stamp(SECOND) / 3600.0
                  end if                                                  3e
c
c         convert to voltage & subtract hot-wire amplifier output offset
c
              read( Cline(18:), *, ERR=92 ) temp,
     1            ( wire_volts(i), i = 1, n_wires )
              if ( start_file .and. saved_state ) then                    3b
                  if ( iadjust .gt. 0 ) then                              4b
                      do 11 i = 1, n_wires
                          adjust(i) = adjust(i) + saved(i) -
     1                    wire_volts(i) - ( saved_temp - temp ) * 0.013
11                        continue
      WRITE(6,9999) ( ADJUST(I), I=1,N_WIRES )
9999  FORMAT( ' voltage adjustment ', 4g12.5 )
                  else                                                    4m
                      do 12 i = 1, n_wires
                          adjust(i) = 0.0
12                        continue
                      end if                                              4e
                  start_file = .FALSE.
                  end if                                                  3e
              do 7 i = 1, n_wires
                  if ( operating(i) .and. wire_volts(i) .le. 0.0 ) then   3b
                      operating(i) = .FALSE.
                      write( STD_OUT, 6 ) i, Cline(1:17), elapsed_hours
6                     format( ' Hot-wire ', i2, ' not operating at ',
     1                    a, ' -- ', f8.4, ' hours' )
                      end if
7                 continue
              write( out_lu, 8 ) elapsed_hours,
     1            ( temp - Therm_off ) / Therm_gain,
     2            ( ( wire_volts(i) + adjust(i) - wire_off(i) ) /
     3            wire_gain(i), i = 1, n_wires )
8             format( 1x, 6f8.4 )
              end if                                                      2e
          goto 2
c         end of loop                                                     1e
c
c           normal termination
9     continue
      saved_temp = temp
      do 10 i = 1, n_wires
          saved(i) = wire_volts(i)
10        continue
      saved_state = .TRUE.
      return
c
cccccccccccccccccccccccccccccccccccccccccc
c       error traps
c
92    write( STD_ERR, 93 )
93    format( ' Cannot read temperature or hot-wire voltage . '
     1    'Press <CR>' )
      call wait
      return
94    write( STD_ERR, 95 ) iline
95    format( ' Cannot read time/date on line ', i5, ' Press <CR>' )
      call wait
      return
96    write( STD_ERR, 97 )
97    format( ' Cannot write to output file . Press <CR>' )
      call wait
      return
98    write( STD_ERR, 99 )
99    format( ' Cannot read input file . Press <CR>' )
      call wait
      return
      end



      CHARACTER *1 FUNCTION CFIRST_CHAR( STRING )
c
c Returns the first non-blank character in the string STRING .
c If the string is entirely blank, a blank is returned .
c
      include 'stdio.def'
      character string*(*)
      integer i
c
      do 1 i = 1, len(string)
          if ( string(i:i) .ne. ' ' ) then
              cfirst_char = string(i:i)
              return
              end if
1         continue
      cfirst_char = ' '
      return
      end
