c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM PATCH
c
c         Program for patching segments of one or more `genacq' data files
c  form a new data file .
c
c  DATA FILE
c    form          = 'UNFORMATTED'
c    organization  = 'SEQUENTIAL'
c    recordtype    = 'FIXED'
c    record length = 512 bytes
c    access        = 'DIRECT'
c
c       The data file is organized into modules , with each module consisting
c of a MODULE HEADER record followed by one or more data blocks . Each data
c block begins with a BLOCK HEADER record and can be followed by one or more
c data sub-blocks . Each data sub-block begins with a SUB-BLOCK header record
c and can be followed by data records . The first 32-bits of data records
c contain a 4 byte integer equal to the number of bytes of data .
c       Typically,
c   1) Modules - correspond with major experimental funtions ; eg
c         initialization, calibration of probes, data acquisition, closing
c   2) Blocks  - contain a complete data acquisition cycle ; eg
c         a pause, move probe, and acquire in turn from all A/D converters
c   3) Sub-blocks - consist of a single function ; eg move the probe
c
c Each data structure header record begins with an 8 byte ASCII character label
c followed by a 2-byte integer equal to the number of bytes after it .
c   Module header label begins with '#'
c   Block header label begins with '##'
c   Sub-block header label begins with '###'
c
c       The module header directory is written onto the first record of the
c file . It consists of simply :-
c   a) nmodul - integer*2 - which is the number of module headers on the file
c   b) dirrec - integer*2 (255) - which is a list of the record numbers of each
c                    module header record - in sequential order of occurrence
c
c   open input files
c   open output file
c   set operating parameters
c   do the data transfer
c   modify the directory record
c
c   USING PATCH
c 1) Input files :
c   a) Terminate selection of input files with an empty file name
c   b) Maximum length of file name is 48 characters
c   c) Up to 8 input files can be specified
c   d) Input files must be from `genacq', and must have been read from
c      magnetic tape using READTAPE , or must have been passed through
c      tapcvt after arriving via KERMIT .
c 2) Output File :
c   a) Only one output file is produced
c   b) The default output file name is PATCH.OUT .
c   c) Selected whole records are copied from the input files to the
c      output file .
c 3) Record Transfer :
c  Transfer of data is specified by defining a `transfer set' . The
c  things defined by a transfer set are :
c   a) input file
c   b) what records are to be taken from the input file ( ie. first to last )
c   c) first output file record number to be written
c  up to 32 transfer sets can be defined .
c 4) Directory record :
c  If a record which is referred to in the input file directory is copied,
c  a directory entry is created in the output file .
c      The output file directory can be edited after the data transfer has
c  been completed .
c 5) N.B.
c  You must decide exactly what records you want transferred from the input
c  file(s) , and exactly where you want them transferred to BEFORE you run
c  this program . You can probably get the necessary information from the
c  appropriate `.scr' file in the [.SCR] directory generated by DIRCVT.
c  These scratch files contain the record numbers of all header records .
c  If you cannot get the necessary information from the scratch files,
c  then you must DUMP/ASC/WORD/HEX the data files .
c 6) N.B.
c  The output file can easily be scrambled ; so be careful
c
c constants
c  Hnipf          # maximum number of input files
c  Hipflu         # logical unit number of the first input file - 1
c  Hopflu         # logical unit number of the output file
c  Hkbdlu         # logical unit number of keyboard
c  Hmaxrec        # max no of records in a file
c  Hnset          # max no of transfer sets
c  Hvdulu         # logical unit number of vdu
c
c variables
c  logical *2 fexist           # input file exists
c  integer *4 i                # looping index
c             nipf             # number of input files
c  integer *2 ifirst           # = iset if first pass on selecting transfer
c                              #  parameters
c             iipdir(256)      # input file directory
c             iipdptr          # input file directory pointer
c             iobuff(256)      # I/O buffer for record transfer
c             iopdir(256)      # output file directory
c             iopdptr          # output file directory pointer
c             ioprec           # output file record number
c             ipflu            # list of input file logical unit numbers
c             iset             # current transfer set
c             iiprec1(Hnset)   # first record of input file to be transferred
c             iiprecn(Hnset)   # last record of input file to be transferred
c             itripf(Hnset)    # input file number - for record transfer
c             ioprec1(Hnset)   # first record of output file
c             ioprecn(Hnset)   # last record of output file
c             nchars           # number of characters in a filename
c             nset             # number of record transfer sets
c             jljstr           # function : left justify string
c             Jresp            #          : integer response
c  logical *4 Equery           # function : logical response
c  character Cipfnam(Hnipf)*48 # input file names
c            Copfnam*48        # output file name
c            Cok*19            # query string
c
      implicit none
      integer *4 Hnipf, Hipflu, Hopflu, Hkbdlu, Hmaxrec, Hnset, Hvdulu
      parameter( Hnipf=8, Hipflu=9, Hopflu=4, Hkbdlu=5, Hmaxrec=65536,
     1  Hnset=32, Hvdulu=6 )
      logical *2 fexist
      logical *4 Equery
      integer i, nipf, Jljstr, Jresp, str_length
      integer *2 ifirst, iipdir(256), iipdptr, iobuff(256), iopdir(256),
     1  iopdptr, ioprec, ipflu, iset, item, iiprec1(Hnset),
     2  iiprecn(Hnset), itripf(Hnset), ioprec1(Hnset), ioprecn(Hnset),
     3  nchars, nset
      character Cipfnam(Hnipf)*48, Copfnam*48, Cok*19
c
      data itripf, iiprec1, iiprecn, ioprec1, ioprecn / 32*1, 128*2 /
c
      write( Hvdulu, 97 )
97    format( 31x, 'PATCH' / 30x, 7( '=' ) //
     1  ' Program for patching segments of one or more `genacq'' data',
     2  ' files to' / ' form a new data file' )
c
c         open I/P files
c
      nipf = 1
      do while( nipf .le. Hnipf )
1       write( Hvdulu, 2 ) nipf
2       format( ' What is the name of the ', i2, 'th input file ? ', $ )
        read( Hkbdlu, 3 ) Cipfnam(nipf)
3       format( a )
        nchars = jljstr( Cipfnam(nipf) )
        if ( nchars .le. 0 ) goto 5
        inquire( FILE=Cipfnam(nipf)(:nchars), ERR=99, EXIST=fexist )
        if ( fexist ) then
          ipflu = nipf + Hipflu
          open( UNIT=ipflu, FILE=Cipfnam(nipf)(:nchars), STATUS='old',
     1      ACCESS='direct', FORM='unformatted', RECL=128, SHARED,
     2      RECORDTYPE='fixed' )
          nipf = nipf + 1
        else
          write( Hvdulu, 4 )
4         format( ' PATCH : File not found' )
          end if
        end do
5     nipf = nipf - 1
      if ( nipf .le. 0 ) stop
c
c         open O/P file
c
      write( Hvdulu, 6 )
6     format( ' What is the name of the output file ? ', $ )
      read( Hkbdlu, 3 ) Copfnam
      nchars = jljstr( Copfnam )
      if ( nchars .le. 0 )
     1  Copfnam = 'patch.out'
      open( UNIT=Hopflu, FILE=Copfnam, STATUS='NEW', ACCESS='direct',
     1      FORM='unformatted', RECL=128, RECORDTYPE='fixed' )
c
c         DETERMINE OPERATING PARAMETERS
c
      ifirst = 1
      iset = 1
      nset = 1
      do while( .true. )
        nset = max( iset, nset )
        write( Hvdulu, 11 ) iset, iiprec1(iset), iiprecn(iset),
     1    ioprec1(iset), itripf(iset), ioprecn(iset),
     2    Cipfnam(itripf(iset))
11      format( ' Select transfer parameters for set ', i2 /
     1    9x, '0) finished      4) first input file record number ', i7/
     2    9x, '1) previous set  5) last input file record number ', i7/
     3    9x, '2) next set      6) first output file record number ',i7/
     4    9x, '3) input file ', i2, '    last output file record ',
     5    'number ',i7 / 10x, a, ' ', $ )
        item = Jresp( Hkbdlu, 0, 6, 3 )
        if ( item .gt. 2 )
     1    write( Hvdulu, 12 )
12        format( 30x, ' and what is its value ? ', $ )
        if ( item .eq. 0 ) then
          goto 13
        else if ( item .eq. 1 ) then
          iset = max( iset - 1, 1 )
        else if ( item .eq. 2 ) then
          iset = min( iset + 1, Hnset )
          if ( ifirst .lt. iset ) ifirst = iset
        else if ( item .eq. 3 ) then
          itripf(iset) = Jresp( Hkbdlu, 1, nipf, 4 )
        else if ( item .eq. 4 ) then
          iiprec1(iset) = Jresp( Hkbdlu, 1, Hmaxrec, 4 )
          iiprecn(iset) = max( iiprec1(iset), iiprecn(iset) )
        else if ( item .eq. 5 ) then
          i = iiprec1(iset)
          iiprecn(iset) = Jresp( Hkbdlu, i, Hmaxrec, 4 )
        else if ( item .eq. 6 ) then
          ioprec1(iset) = Jresp( Hkbdlu, 1, Hmaxrec, 4 )
          ifirst = ifirst + 1
          end if
        ioprecn(iset) = iiprecn(iset) - iiprec1(iset) + ioprec1(iset)
        if ( iset .lt. Hnset .and. ifirst .eq. iset )
     1    ioprec1(iset+1) = ioprecn(iset) + 1
        end do
13    write( Cok, 17 ) nset
17    format( i2, ' data sets : OK ?' )
      if ( .not. Equery( Hkbdlu, Cok ) ) then
        write( Hvdulu, 16 )
16      format( ' How many ? ', $ )
        nset = Jresp( Hkbdlu, 0, nset, 4 )
        end if
c
c         TRANSFER THE DATA & COMPILE DIRECTORY
c  read records iiprec1(iset) to iiprecn(iset) from file unit ipflu(iset) and
c  copy them into o/p file starting at ioprec1(iset)
c
      write( Hvdulu, 14 )
14    format( ' Start data transfer' )
      iopdptr = 2
      do iset = 1, nset
c
c           read I/P file directory & position the directory pointer
c
        ipflu = Hipflu + itripf(iset)
        read( ipflu, REC=1 ) iipdir
        iipdptr = 2
        do while( iipdir(iipdptr) .lt. iiprec1(iset) )
          iipdptr = iipdptr + 1
          end do
c
c           initialize transfer
c
        ioprec = ioprec1(iset)
        write( Hvdulu, 15 ) iiprec1(iset), iiprecn(iset), Cipfnam(iset),
     1    ioprec1(iset), ioprecn(iset), Copfnam
15      format( ' Transfer records ',i7, ' to ', i7, ' in file ', a /
     1    '   to records ', i7, ' to ', i7, ' in file ', a )
c
c           transfer loop
c
        do i = iiprec1(iset), iiprecn(iset)
c
c           transfer one record
c
          read( ipflu, REC=i, ERR=30 ) iobuff
          write( Hopflu, REC=ioprec, ERR=33 ) iobuff
          if ( mod( i, 10 ) .eq. 0 )
     1        write( Hvdulu, 18 ) i, ipflu, ioprec, Hopflu
18            format( 1x, i6, '(', i2, ') => ', i6, '(', i2, ')' )
c
c           update O/P file directory
c
          if ( iipdir(iipdptr) .eq. i ) then
            iopdir(iopdptr) = ioprec
            iipdptr = iipdptr + 1
            iopdptr = iopdptr + 1
            end if
          ioprec = ioprec + 1
          goto 32
30        write( Hvdulu, 31 ) ipflu, i, Cipfnam(iset)
31        format( ' PATCH : Cannot read unit ', i2, ' record number ',
     1            i8 / '       : file name ', a )
32        end do
        end do
      iopdir(1) = iopdptr - 2
c
c         modify directory record
c
      do while( .true. )
        write( Hvdulu, 20 ) ( iopdir(i), i = 1, iopdir(1)+1 )
20      format( ' Output file directory record' / 1x, 10i7 )
        write( Hvdulu, 21 )
21      format( ' Which entry do you wish to change ? ', $ )
        i = iopdir(1) + 2
        item = Jresp( Hkbdlu, 0, i, 4 )
        if ( item .le. 0 ) goto 22
        write( Hvdulu, 12 )
        iopdir(item) = Jresp( Hkbdlu, 0, Hmaxrec, 4 )
        end do
22    write( Hopflu, REC=1 ) iopdir
c
c       close data files
c
      do i = Hipflu, Hipflu + nipf
        close( UNIT = i )
        end do
      close( Hopflu )
      stop
c
c
33    write( Hvdulu, 34 ) Copfnam(:str_length(Copfnam)), ioprec
34    format( ' PATCH : ABORT : Cannot write file ', a, ' at record ',
     1        i7 )
      stop
c
99    write( Hvdulu, 98 )
98    format( ' PATCH : Invalid file name' )
      goto 1
      end
