c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


       PROGRAM PVGRAD
c
c       Evaluate the static pressure gradient in the test section of the
c  wind tunnel from data in a file .
c    The data file contains a number of measurement sequences . Each sequence
c  is enclosed by zero reference readings which are used to eliminate drift
c  by linear interpolation .
c
c
c constants
c  MAX_TRIES=4    # maximum number of input attempts
c
c variables
c  real
c   amp_gain      # pressure sensor amplifier gain in Volts/Torr
c   trip_diam     # boundary layer trip diameter (mm)
c   qresp         # function : read real
c  integer
c   in_lu         # logical unit number of input file, 0=closed
c   out_lu        # logical unit number of output file, 0=closed
c   item          # item selected from menu
c   len_ipfile    # length of input file name
c   len_opfile    # length of output file name
c   request_fs    # YES/NO -- request free stream speed from user
c   res_in_lu     # logical unit number of input file
c   res_out_lu    # logical unit number of output file
c   iargc         # function : number of command line arguments
c   Jresp         #          : read integer
c   new_lun       #          : get new logical unit number
c   open_file     #          : open file
c   str_length    #          : non-blank length of string
c  characters
c   Cgain*64      # 3rd program argument = amplifier gain
c   Cipfile*64    # input file name
c   Copfile*64    # output file name
c  subroutine
c   cresp         # read string
c   do_analysis
c   getarg        # get program argument
c   ilgwait       # wait for <CR>
c   newscr        # clear alphanumeric screen
c
c
      include 'stdio.def'
      integer MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      real amp_gain, trip_diam, qresp
      integer in_lu, item, len_ipfile, len_opfile, out_lu, res_in_lu,
     1    res_out_lu, request_fs,
     2    iargc, Jresp, new_lun, open_file, str_length
      character Cgain*64, Cipfile*64, Copfile*64, Crequest(0:1)*31
c
      data in_lu, out_lu, len_ipfile, len_opfile, request_fs /
     1     0, 0, 1, 1, NO /
      data Cgain, Cipfile, Copfile / ' ', ' ', ' ' /
      data Crequest / 'pre-centrebody dynamic pressure', 'keyboard' /
c
c
c       Read the command line arguments.
c         pvgrad <input_file> <out_file> <amplifier_gain>
c
      res_in_lu = new_lun()
      res_out_lu = new_lun()
      if ( iargc() .ge. 1 ) then                                          1b
          call getarg( 1, Cipfile )
          if ( iargc() .ge. 2 ) then                                      2b
              call getarg( 2, Copfile )
              if ( iargc() .ge. 3 ) then                                  3b
                  call getarg( 3, Cgain )
                  read( Cgain, *, ERR=13 ) amp_gain
                  end if                                                  3e
              end if                                                      2e
          end if                                                          1e
      len_ipfile = max( 1, str_length( Cipfile ) )
      len_opfile = max( 1, str_length( Copfile ) )
c
c       main menu loop
c
1         call newscr()                                                   1b
          request_fs = mod( request_fs, 2 )
          write( STD_OUT, 2 ) Cipfile(:len_ipfile),
     1        Copfile(:len_opfile), amp_gain, trip_diam,
     2        Crequest(request_fs)
2         format( ' Main menu' /
     1        '   0) Exit' /
     2        '   1) Input file name    ', a  /
     3        '   2) Output file name   ', a  /
     4        '   3) Amplifier gain     ', g14.7 /
     5        '   4) Boundary layer trip diameter  ', f7.3, ' mm' /
     6        '   5) Obtain free stream speed from ', a /
     7        '   6) Do analysis' /
     8        '   7) Help - file format ', $ )
c
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( out_lu .ne. 0 )
     2            close( UNIT=out_lu )
              stop
c
c       INPUT FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Input data file name ? ', $ )
              call cresp( STD_IN, Cipfile, MAX_TRIES )
              len_ipfile = max( 1, str_length( Cipfile ) )
c
c       OUTPUT FILE NAME
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Output data file name ? ', $ )
              call cresp( STD_IN, Copfile, MAX_TRIES )
              len_opfile = max( 1, str_length( Copfile ) )
c
c       AMPLIFIER GAIN
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Gain factor for static pressure & zero ',
     1            'readings ? ', $ )
              amp_gain = qresp( STD_IN, 0.5, 5000.0, MAX_TRIES )
c
c       BOUNDARY LAYER TRIP DIAMETER
c
          else if ( item .eq. 4 ) then
              write( STD_OUT, 6 )
6             format( ' What is the boundary layer trip diameter ',
     1            '(mm) ? ', $ )
              trip_diam = Qresp( STD_IN, 1.0, 10.0, MAX_TRIES )
c
c       REQUEST FREE STREAM SPEED ?
c
          else if ( item .eq. 5 ) then                                    2m
              request_fs = mod( request_fs + 1, 2 )
c
c       ANALYSIS
c
          else if ( item .eq. 6 ) then                                    2m
              if ( Cipfile .eq. ' ' ) then                                3b
                  write( STD_ERR, 7 )
7                 format( ' PVGRAD : Please supply input file name' )
                  call ilgwait()
              else                                                        3m
c
c       open input and output files
c
                  if ( in_lu .ne. 0 ) then                                4b
                      close( UNIT=in_lu )
                      in_lu = 0
                      end if                                              4e
                  in_lu = open_file( res_in_lu, Cipfile, 'old', 0 )
                  if ( in_lu .eq. 0 ) then                                4b
                      write( STD_ERR, 8 ) Cipfile(:len_ipfile)
8                     format( ' PVGRAD : Cannot open input file ', a )
                      call ilgwait()
                  else                                                    4m
                      call read_press_data( in_lu, Copfile, amp_gain,
     1                                      trip_diam, request_fs )
                      close( UNIT=in_lu )
                      end if                                              4e
                  end if                                                  3e
c
c       HELP
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 9 )
9             format( ' Data file format :' /
     1            '     Each line in the data file corresponds to a ',
     2                'single measurement from a' /
     3            '     manometer, and the data is assumed to be in ',
     4                'chronological order. Each line' /
     5            '     in the data file has three fields occupying ',
     6                'up to a total of 80 characters.' )
              write( STD_OUT, 10 )
10            format( '     The fields are :-' /
     1            6x, '<descripion> <average> <standard_deviation>' //
     2            ' <description>     is a two character label which ',
     3                ' describes the purpose of the' /
     4            '    line . The first character can be ''p'', ''P''',
     5                ' ''z'', ''Z'', ''s'', ''S'' or '';''. If' /
     6            '    it is a ''p'' or ''P'', the next character ',
     7                'must be a hexadecimal digit denoting' /
     8            '    test section plug number . A line beginning ',
     9                'with ''P'' or ''p'' contains a' /
     a            '    static pressure measurement at a particular ',
     b                'plug .' )
              call ilgwait()
              write( STD_OUT, 11 )
11            format( 7x, 'If the first character is a ''z'' or ''Z''',
     1                ' the measurement is a zero' /
     2            '    reference, and the next character must be one ',
     3                'of :--' /
     4            9x, '''n'' or ''N'' -- initial zero for a new ',
     5                'measurement sequence' /
     6            9x, '''r'' or ''R'' -- final zero for a measurement ',
     7                'sequence . If the previous' /
     8            10x, 'zero reference was the same type, it becomes ',
     9                'the initial reference .' /
     a            '     If the first character is an ''s'' or ''S'', a ',
     b                'new run is started. The next' /
     c            '     character can be anything, but the <average> ',
     d                'field contains the measured' /
     e            '     pre-centrebody dynamic pressure . If the first ',
     f                'character is a '';'', the' /
     g            '     line is a comment .' )
              write( STD_OUT, 12 )
12            format( ' <average>     is a average static pressure, ',
     1                'dynamic pressure or zero reading .' /
     2            '    The static pressure and zero readings are ',
     3                'assumed to be factored by a' /
     4            '    specified amplifier gain .' /
     5            ' <standard_deviation> is a sample standard ',
     6                'deviation on the <average> . This' /
     7            '    is an optional field and is not read .' )
              call ilgwait()
              end if                                                      2e
          goto 1
c         end of main loop
c
c
c       ERROR TRAPS
c
13    write( STD_OUT, 14 )
14    format( ' PVGRAD : WARNING : Cannot read argument 1 : ',
     1        'amplifier gain' )
      call ilgwait()
      goto 1
c
      end



      SUBROUTINE READ_PRESS_DATA( IN_LU, OUT_FILE, AMP_GAIN, TRIP_DIAM,
     1                            REQUEST_FS )
c
c parameters
c  integer   in_lu          # logical unit number of input file
c  character out_file*(*)   # logical unit number of output file
c  real      amp_gain       # amplifier gain
c            trip_diam      # diameter of the boundary layer trip
c  integer   request_fs     # YES/NO request free stream speed from user
c
c    HOW IT WORKS
c
c    first_zero = run_started = zero_found = false
c    start_index = 1
c    start loop:
c      read I/P file record
c      count the columns
c      if 1st col = ;
c         print comment
c      else if 1st col = p and zero_found and run_started
c         assign plug number
c         calculate distance from boundary layer trip
c         read data
c      else if 1st col = z and ( 2nd col = n or 2nd col = r and not zero_found)
c           and run_started
c         first_zero = zero_found = true
c         read first zero
c      else if 1st col = z and 2nd col = r and run_started
c         read average into buffer
c         if not first_zero
c           transfer second zero into first zero
c         transfer buffered zero into second zero
c         first_zero = false
c         calculate offset & gradient for zero reference line
c      else if 1st col = s
c         if run_started and index > 1
c           do analysis
c         zero_found = false
c         read average into pre-centrebody dynamic pressure
c      end loop
c end_of_input_file:
c    if run_started and index > 1
c      do analysis
c
c
c logical
c   fgotz               # have a first zero reference value
c   firstz              # previous zero reference was a `first'
c   frungo              # a run has been started
c  integer
c   i                   # looping indices
c   iplug               # wind tunnel plug number -- temporary storage
c   n_chars             # no. of characters in current I/P file record
c   n_cols              # number of columns of data
c   n_points            # number of data points in RESULT array
c   record_no           # input file record number
c   seq_counter         # sequence number - source
c   seq_index           # index into the current sequence
c   seq_length          # number of items in the current sequence
c   seq_start           # starting index of current sequence
c   jljstn              # function : left justifies & counts columns
c  real
c   amp_offset          # amplifier offset
c   dynam_press         # dynamic pressure in Torr
c   results(MAX_DATA,MAX_VARS) # array of results & data
c     The column indices are
c       PLUG_NO    = 1    # wind tunnel plug number
c       SEQUENCE   = 2    # data sequence number
c       STREAM_X   = 3    # distance from leading edge
c       MEAN_VOLTS = 4    # mean voltage
c       CBODY_DYN  = 5    # pre-centrebody dynamic pressure
c       STAT_PRESS = 6    # static pressure
c       PRES_RATIO = 7    # static/dynamic pressure
c   zbuff,              # temporarily buffered value of zero2
c   zero1, zero2        # endpoints of zeroreference line
c   zgrad,              # rate of change of zero reference
c  character
c   Cline*80            # input file data record
c  subroutine
c   do_analysis
c   ilgwait
c
      include 'stdio.def'
      include 'pvgrad.def'
      real NOT_SET
      parameter( NOT_SET=-9.999e-29 )
c
      logical fgotz, firstz, frungo
      integer i, iplug, n_chars, n_cols, n_points, in_lu,
     1    record_no, seq_counter, seq_index, seq_length,
     2    seq_start, request_fs, jljstn
      real amp_gain, amp_offset, trip_diam, dynam_press,
     1    results(MAX_DATA,MAX_VARS), zbuff, zero1, zero2, zgrad
      character Cline*80, out_file*(*)
c
c         initialize flags & indices
c
      fgotz =  .false.
      frungo = .false.
      firstz = .false.
      seq_length  = 1
      seq_counter = 1
      seq_start   = 1
      n_points    = 0
      record_no   = 0
      amp_offset  = NOT_SET
c
c         start loop for reading a data file record
c
c         read I/P file record
c
1         read( in_lu, 2, END=11, ERR=12 ) Cline                          1b
2         format( a )
          record_no = record_no + 1
c
c         count the columns
c
          n_chars = jljstn( Cline, n_cols )
c
c    **** found comment line, print it out
c
          if ( Cline(1:1) .eq. ';' ) then                                 2b
              write( STD_OUT, 3 ) Cline(1:n_chars)
3             format( 1x, a )
c
c    **** static pressure data record
c
          else if ( n_chars .gt. 4 .and. n_cols .gt. 1 ) then             2m
              if ( ( Cline(1:1) .eq. 'p' .or. Cline(1:1) .eq. 'P' )
     1             .and. fgotz .and. frungo ) then                        3b
c
c         decode 2nd col and assign plug number
c
                  iplug = 0
                  if ( '0' .le. Cline(2:2) .and. Cline(2:2) .le. '9' )
     1                    then                                            4b
                      iplug = ichar( Cline(2:2) ) - ichar( '0' )
                  else if ( 'A' .le. Cline(2:2) .and.
     1                    Cline(2:2) .le. 'F' ) then                       4m
                      iplug = ichar( Cline(2:2) ) - ichar( 'A' ) + 10
                  else if ( 'a' .le. Cline(2:2) .and.
     1                    Cline(2:2) .le. 'f' ) then                       4m
                      iplug = ichar( Cline(2:2) ) - ichar( 'a' ) + 10
                      end if                                              4e
c
c
                  if ( 0 .lt. iplug .and. iplug .lt. 16 ) then            4b
                      if ( n_points .lt. MAX_DATA ) then                  5b
c
c         read average into data array
c
                          read( Cline(3:), *, ERR=14 )
     1                        results(n_points+1,MEAN_VOLTS)
c
c         increment data index and sequence length counter
c
                          n_points = n_points + 1
                          seq_length  = seq_length + 1
                          results(n_points,PLUG_NO) = iplug
                          results(n_points,SEQUENCE) = seq_counter
                      else                                                5m
                          write( STD_ERR, 4 ) record_no
4                         format( ' WARNING : Array space full at ',
     1                        'line number ', i4 )
                          end if                                          5e
                  else                                                    4m
                      write( STD_OUT, 5 ) record_no
5                     format( ' WARNING : Unknown plug number in ',
     1                    'line ', i4 )
                      end if                                              4e
c
c    ***** first zero reference
c
              else if ( ( Cline(1:1) .eq. 'z' .or.
     1                Cline(1:1) .eq. 'Z' ) .and. frungo ) then           3m
                  if ( Cline(2:2) .eq. 'n' .or. Cline(2:2) .eq. 'N'
     1                    .or. ( Cline(2:2) .eq. 'r' .or.
     2                    Cline(2:2) .eq. 'R' ) .and. .not. fgotz ) then  4b
c
c         read average into first zero
c
                      read( Cline(3:), *, ERR=14 ) zero1
                      if ( .not. fgotz .or. amp_offset .eq. NOT_SET )
     1                    amp_offset = zero1
                      fgotz = .true.
                      firstz = .true.
c
c    ***** second reference zero
c
                  else if ( ( Cline(2:2) .eq. 'r' .or.
     1                    Cline(2:2) .eq. 'R' ) .and. frungo ) then       4m
c
c         read average zero reference into buffer
c
                      read( Cline(3:), *, ERR=14 ) zbuff
c
c         if the last zero was not the first, roll the second into the first
c
                      if ( .not. firstz )
     1                    zero1 = zero2
                      zero2 = zbuff
                      firstz = .false.
c
c         calculate gradient for zero reference line
c
                      zgrad = ( zero2 - zero1 ) /
     1                    real( seq_length + 1 )
c
c           calculate static pressure ratio for the existing sequence
c
                      seq_index = 0
                      do 6 i = seq_start, n_points                        5b
                          seq_index = seq_index + 1
                          results(i,MEAN_VOLTS) = results(i,MEAN_VOLTS)
     1                        - zero1 - zgrad * seq_index + amp_offset
                          results(i,CBODY_DYN) = dynam_press
6                         continue                                        5e
c
c           update pointers
c
                      seq_counter = seq_counter + 1
                      seq_start   = n_points + 1
                      seq_length  = 0
                  else                                                    4m
                      write( STD_OUT, 7 ) record_no
7                     format( ' WARNING : Unknown zero reference ',
     1                    'type in line ', i4 )
                      end if                                              4e
c
c     **** start run
c
              else if ( Cline(1:1) .eq. 's' .or. Cline(1:1) .eq. 'S' )
     1                then                                                3m
c
c           write results to file
c
                  if ( n_points .gt. 1 .and. frungo ) then                4b
                      if ( amp_offset .eq. NOT_SET ) then                 5b
                          write( STD_ERR, 16 )
                          call ilgwait()
                      else                                                5m
                          call do_analysis( results, n_points,
     1                        amp_gain, amp_offset, request_fs,
     2                        out_file )
                          firstz = .false.
                          end if                                          5e
                      end if                                              4e
c
c         read average into pre-centrebody dynamic pressure
c
                  read( Cline(3:), *, ERR=14 ) dynam_press
                  seq_counter = 1
                  frungo = .true.
                  fgotz = .false.
                  amp_offset = NOT_SET
                  seq_start = 1
              else                                                        3m
c
c         I/P data file record not understood
c
                  write( STD_ERR, 8 ) record_no
8                 format( ' WARNING : Invalid 1st character in line ',
     1                i4 )
                  end if                                                  3e
          else                                                            2m
              write( STD_ERR, 9 ) record_no
9             format( ' WARNING : Line ', i4, 'not long enough .' )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
c
c           end of input file found : write results to file
c
11    if ( n_points .gt. 1 .and. frungo ) then                            1b
          if ( amp_offset .eq. NOT_SET ) then                             2b
              write( STD_ERR, 16 )
              call ilgwait()
          else                                                            2m
              call do_analysis( results, n_points, amp_gain,
     1            amp_offset, trip_diam, request_fs, out_file )
              end if                                                      2e
          end if                                                          1e
      close( UNIT=in_lu )
      in_lu = 0
      return
c
12    write( STD_ERR, 13 )
13    format( ' WARNING : Cannot read data file' )
      call ilgwait()
      return
c
14    write( STD_ERR, 15 ) record_no
15    format( ' WARNING : Cannot decode number at line ', i4 )
      call ilgwait()
      goto 1
c
16    format( ' WARNING : Amplifier offset unknown. Cannot do ',
     1        'analysis' )
      end


      INTEGER FUNCTION PV_OPEN_FILE( OUT_FILE )
c
c Opens output file of name OUT_FILE .
c
c Argument
c  character out_file*(*)         # name of output file
c
c local variables
c  integer
c   out_lu      # logical unit number of output file
c   res_out_lu  # reserved logical unit number of output file
c   new_lun     # function : get new logical unit number
c   open_file   #          : open file
c  character
c   prev_file   # name of previous output file
c
c
      integer out_lu, res_out_lu, new_lun, open_file
      character out_file*(*), prev_file*64
c
      save out_lu, res_out_lu, prev_file
      data out_lu, res_out_lu / 0, 0 /
      data prev_file / ' ' /
c
      if ( res_out_lu .eq. 0 )
     1    res_out_lu = new_lun()
      if ( ( prev_file .eq. ' ' .or. out_file .ne. prev_file ) .and.
     1     out_lu .ne. STD_OUT .and. out_lu .ne. 0 ) then                 1b
          close( UNIT=out_lu )
          out_lu = 0
          end if                                                          1e
      if ( out_lu .eq. 0 )
     1    out_lu = open_file( res_out_lu, out_file, 'unknown', 0 )
      pv_open_file = out_lu
      if ( out_lu .gt. 0 ) then                                           1b
          prev_file = out_file
      else                                                                1m
          write( STD_ERR, 1 )
1         format( ' PVGRAD : WARNING : Cannot open output file' )
          call ilgwait()
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION PV_TABULATE( RESULTS, N_POINTS, OUT_FILE )
c
c Tabulate the data and results array .
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points               # number of data points
c  character out_file*(*)         # name of output file, 0=closed
c
c local variables
c  integer
c   index         # data array row index
c   pv_open_file  # function : open output file
c  subroutine
c   ilgwait
c
      include 'stdio.def'
      include 'pvgrad.def'
c
      real results(MAX_DATA,MAX_VARS)
      integer index, n_points, out_lu, pv_open_file
      character out_file*(*)
c
      out_lu = pv_open_file( out_file )
      pv_tabulate = out_lu .gt. 0
      if ( pv_tabulate ) then                                             1b
          write( out_lu, 1, ERR=2 )  ( index, results(index,CBODY_DYN),
     1        int( results(index,PLUG_NO) + 0.1 ),
     2        int( results(index,SEQUENCE) + 0.1 ),
     3        results(index,STREAM_X),   results(index,MEAN_VOLTS),
     4        results(index,STAT_PRESS), results(index,PRES_RATIO),
     5        index = 1, n_points )
1         format( ' ;' /
     1      ' ; static pressure distribution data and results'/
     2      ' ; ' /
     3      ' ;  dynam plug sequ- dist.   mean     static    static/' /
     4      ' ;  Torr  no.  ence  x(m)  voltage  press (Pa)  dynamic' /
     5      ( i4, f7.3, i3, i4, f7.3, f9.4, f11.5, g13.5 ) )
          if ( out_lu .eq. STD_OUT )
     1        call ilgwait()
          end if                                                          1e
      return
c
2     write( STD_ERR, 3 )
3     format( ' WARNING : Cannot write to output file' )
      pv_tabulate = .FALSE.
      call ilgwait()
      return
      end



      CHARACTER *16 FUNCTION NAME_RES( WHAT )
c
c Returns a character string description of the variable in the WHATth
c column of the results array .
c
      include 'stdio.def'
      include 'pvgrad.def'
c
      integer what
      character names(MAX_VARS)*16
      save names
      data names /           'plug number    ', 'sequence number',
     1    'stream distance', 'mean voltage   ', 'dynamic press. ',
     2    'static pressure', 'pressure ratio ' /
c
      if ( 1 .le. what .and. what .le. MAX_VARS ) then                    1b
          name_res = names(what)
      else                                                                1m
          name_res = 'unknown'
          end if                                                          1e
      return
      end



      SUBROUTINE PV_PLOT( RESULTS, N_POINTS )
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points               # number of data points
c
c local variables
c  logical
c   plotted     # graph is plotted & visible
c   scaled      # scaling has been performed for the graph
c   ilgread     # function : read crosshairs
c  integer
c   abscissa    # results column index for abscissa
c   ordinate    # results column index for ordinate
c   capt_len    # length of capture file name
c   i           # looping index
c   item        # selected menu item
c   seq_len     # number of data items in a sequence
c   seq_start   # start index of sequence
c   symbol      # plotting symbol used for the sequence
c   Jresp       # function : read integer
c   str_length  #          : non-blank string length
c  real
c   xmin, xmax  # abscissa limits
c   ymin, ymax  # ordinate limits
c  character
c   capt_file*64# name of capture file
c   cjunk*1     # character pressed for crosshairs -- not used
c   name_res*16 # function : result name
c  subroutine
c   cresp       # read string
c   ilgclos, ilgdata, ilgimit, ilgline, ilgopen, ilgxlab, ilgylab
c   newscr      # clear page
c   swap4       # swap values
c
c
      include 'stdio.def'
      include 'pvgrad.def'
c
      logical plotted, scaled, ilgread
      integer n_points, abscissa, ordinate, capt_len, i, item,
     1        seq_len, seq_start, symbol, Jresp, str_length
      real results(MAX_DATA,MAX_VARS), xmin, xmax, ymin, ymax
      character capt_file*64, cjunk*1, name_res*16
      save abscissa, ordinate, capt_len, capt_file
      data abscissa, ordinate, capt_len / PLUG_NO, MEAN_VOLTS, 1 /
      data capt_file / ' ' /
c
      scaled = .FALSE.
      plotted = .FALSE.
      capt_len = max( 1, str_length( capt_file ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(abscissa), name_res(ordinate),
     1            capt_file(:capt_len)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Abscissa      ', a /
     2        '   2) Ordinate      ', a /
     3        '   3) Capture file  ', a /
     4        '   4) Plot' /
     5        '   5) Zoom' /
     6        '   6) Rescale  ', $ )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              call ilgclos()
              return
c
c           Abscissa
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) 'abscissa',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a / ( 2x, i2, ') ', a ) )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              scaled = .FALSE.
c
c           Ordinate
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 ) 'ordinate',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              scaled = .FALSE.
c
c           Capture file
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Name of capture file ? ', $ )
              call cresp( STD_IN, capt_file, MAX_TRIES )
              capt_len = max( 1, str_length( capt_file ) )
c
c           Plot, Zoom or Rescale
c
          else if ( 4 .le. item .and. item .le. 6 ) then                  2m
c
c             find limits of data if not previously scaled or `rescaling'
c
              if ( .not. scaled .or. item .eq. 6 ) then                   3b
                  xmin = results(1,abscissa)
                  xmax = xmin
                  ymin = results(1,ordinate)
                  ymax = ymin
                  do 5 i = 1, n_points                                    4b
                      xmin = min( xmin, results(i,abscissa) )
                      xmax = max( xmax, results(i,abscissa) )
                      ymin = min( ymin, results(i,ordinate) )
                      ymax = max( ymax, results(i,ordinate) )
5                     continue                                            4e
                  scaled = .TRUE.
                  end if                                                  3e
c
c             If zooming, get the corners of the zoom window
c
              if ( item .eq. 5 ) then                                     3b
                  if ( plotted ) then                                     4b
                      call newscr()
                      write( STD_OUT, 6 )  'first'
6                     format( ' Locate the ', a, ' corner of the ',
     1                    'zoom window' )
7                     if ( ilgread( xmin, ymin, cjunk ) ) then            5b
                          write( STD_OUT, 6 )  'second'
                          if ( ilgread( xmax, ymax, cjunk ) ) then        6b
                              if ( xmin .gt. xmax )
     1                            call swap4( xmin, xmax )
                              if ( ymin .gt. ymax )
     1                            call swap4( ymin, ymax )
                              xmax = max( xmax, 1.01 * xmin )
                              ymax = max( ymax, 1.01 * ymin )
                          else                                            6m
                              scaled = .FALSE.
                              goto 1
                              end if                                      6e
                      else                                                5m
                          scaled = .FALSE.
                          goto 1
                          end if                                          5e
                  else                                                    4m
                      write( STD_ERR, 8 )
8                     format( ' WARNING : Must plot before zooming' )
                      call ilgwait()
                      end if                                              4e
                  end if                                                  3e
c
c             start graphics
c
              call ilgopen( capt_file )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgimit( xmin, ymin, xmax, ymax )
c
c             plot one sequence at a time, with a different symbol for each
c
              seq_start = 1
c             start of loop
9             if ( seq_start .le. n_points ) then                         3b
                  symbol = results(seq_start,SEQUENCE) + 0.1
                  seq_len = 1
10                if ( seq_start + seq_len .le. n_points ) then           4b
                      if ( symbol .eq. int( results( seq_start+seq_len,
     1                        SEQUENCE) + 0.1 ) ) then                    5b
                          seq_len = seq_len + 1
                          goto 10
                          end if                                          5e
                      end if                                              4e
                  call ilgline( mod( symbol - 1, 14 ) + 1, 0, 1 )
                  call ilgdata( results(seq_start,abscissa),
     1                results(seq_start,ordinate), 1, seq_len, 0 )
                  seq_start = seq_start + seq_len
                  goto 9
                  end if                                                  3e
c                 end of loop
              call ilgylab( name_res(ordinate) )
              call ilgxlab( name_res(abscissa) )
              plotted = .true.
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE PV_EDIT( RESULTS, N_POINTS )
c
c Edit the results and data array .
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points                # number of data points
c
c local variables
c  integer
c   abscissa    # results column index for abscissa
c   ordinate    # results column index for ordinate
c   i           # looping index
c   item        # selected menu item
c   Jresp       # function : read integer
c  character
c   name_res*16 # function : result name
c  subroutine
c   ilgxlab, ilgylab
c   newscr      # clear page
c   sfudg2      # edit data
c
c
      include 'stdio.def'
      include 'pvgrad.def'
c
      real results(MAX_DATA,MAX_VARS)
      integer n_points, abscissa, ordinate, i, item, Jresp
      character name_res*16
      save abscissa, ordinate
      data abscissa, ordinate / PLUG_NO, MEAN_VOLTS /
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(abscissa), name_res(ordinate)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Abscissa      ', a /
     2        '   2) Ordinate      ', a /
     3        '   3) Edit     ', $ )
          item = Jresp( STD_IN, 0, 3, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              call ilgclos()
              return
c
c           Abscissa
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) 'abscissa',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a / ( 2x, i2, ') ', a ) )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
c
c           Ordinate
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 ) 'ordinate',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
c
c           Edit
c
          else if ( item .eq. 3 ) then                                    2m
              call ilgopen( ' ' )
              call ilgylab( name_res(ordinate) )
              call ilgxlab( name_res(abscissa) )
              call sfudg2( results, abscissa, ordinate, MAX_DATA,
     1                MAX_VARS, 1, n_points, n_points )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE PV_AVERAGE( RESULTS, N_POINTS, OUT_FILE )
c
c Calculate the (ensemble) average of a result column for each wind tunnel
c test section port .
c
c Arguments
c  real      results(MAX_DATA,MAX_VARS) # data & results array
c  integer   n_points                   # number of data points
c  character out_file*(*)               # name of output file
c
c constants
c   FEET_TO_METRES=0.3048
c
c local variables
c  logical
c   averaged    # average has been calculated
c  integer
c   counts(MAX_PLUGS)   # number of values averaged at each port
c   i           # looping index
c   item        # selected menu item
c   ordinate    # results column index for ordinate
c   out_len     # length of output file name
c   out_lu      # logical unit number of output file, 0=closed
c   port_no     # test section port number
c   res_out_lu  # logical unit number of output file
c   Jresp       # function : read integer
c   new_lun     #          : get new logical unit number
c   pv_open_file#          : open output file
c   str_length  #          : non-blank length of string
c  real
c   mean(MAX_PLUGS)     # mean value at each plug
c   std_dev(MAX_PLUGS)  # standard deviation of the mean at each plug
c  character
c   prev_file*64# name of previous output file
c   name_res*16 # function : result name
c  subroutine
c   cresp       # read string
c   newscr      # clear screen
c
c
      include 'stdio.def'
      include 'pvgrad.def'
      real FEET_TO_METRES
      parameter( FEET_TO_METRES=0.3048 )
c
      logical averaged
      integer n_points, counts(MAX_PLUGS), i, item, ordinate, out_len,
     1        out_lu, port_no, res_out_lu,
     2        Jresp, new_lun, pv_open_file, str_length
      real    results(MAX_DATA,MAX_VARS), mean(MAX_PLUGS),
     1        std_dev(MAX_PLUGS)
      character out_file*(*), prev_file*64, name_res*16
c
      save ordinate, prev_file
      data ordinate / MEAN_VOLTS /
      data prev_file / ' ' /
      averaged = .FALSE.
c
c     start of menu loop                                                  1b
1         out_len = max( 1, str_length( out_file ) )
          call newscr()
          write( STD_OUT, 2 ) name_res(ordinate), out_file(:out_len)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Averaged variable ', a /
     2        '   2) Tabulation file   ', a /
     3        '   3) Calculate ensemble average' /
     4        '   4) Tabulate ensemble average  ', $ )
          item = Jresp( STD_IN, 0, 4, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           Averaged variable
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )  ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select averaged variable' /
     1            ( 2x, i2, ') ', a ) )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              averaged = .FALSE.
c
c           Tabulation file
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Name of tabulation file ? ', $ )
              call cresp( STD_IN, out_file, MAX_TRIES )
c
c           calculate ensemble average
c
          else if ( item .eq. 3 ) then                                    2m
c
c           initialize
              do 5 port_no = 1, MAX_PLUGS                                 3b
                  counts(port_no)  = 0
                  mean(port_no)    = 0.0
                  std_dev(port_no) = 0.0
5                 continue                                                3e
c
c           accumulate totals
              do 6 i = 1, n_points                                        3b
                  port_no = results(i,PLUG_NO) + 0.1
                  if ( 1 .le. port_no .and.
     1                    port_no .le. MAX_PLUGS ) then                   4b
                      counts(port_no) = counts(port_no) + 1
                      mean(port_no) = mean(port_no) +
     1                    results(i,ordinate)
                      std_dev(port_no) = std_dev(port_no) +
     1                    results(i,ordinate) **2
                      end if                                              4e
6                 continue                                                3e
c
c           calculate average & standard deviation of the average
              do 7 port_no = 1, MAX_PLUGS                                 3b
                  if ( counts(port_no) .gt. 1 ) then                      4b
                      mean(port_no) = mean(port_no) /
     1                    real( counts(port_no) )
                      std_dev(port_no) = sqrt( max( 0.0,
     1                    ( std_dev(port_no) / counts(port_no) -
     2                      mean(port_no) **2 ) /
     3                    real( counts(port_no) - 1 ) ) )
                  else if ( counts(port_no) .gt. 0 ) then                 4m
                      std_dev(port_no) = 0.0
                      end if                                              4e
7                 continue                                                3e
              averaged = .TRUE.
c
c               tabulate ensemble average
c
          else if ( item .eq. 4 ) then                                    2m
              if ( averaged ) then                                        3b
                  out_lu = pv_open_file( out_file )
                  if ( out_lu .gt. 0 ) then                               4b
                      write( out_lu, 8 ) name_res(ordinate)
8                     format( ' ;' /
     1                    ' ; ensemble averaged ', a / ' ; ' /
     2                    ' ; plug dist.   mean      standard' /
     3                    ' ; no.  x(m)              deviation' )
                      do 10 port_no = 1, MAX_PLUGS                        5b
                          if ( counts(port_no) .gt. 0 ) then              6b
                              write( out_lu, 9 ) port_no,
     1                            FEET_TO_METRES * port_no - 0.164,
     2                            mean(port_no), std_dev(port_no)
9                             format( 3x,i2, 2x, f6.3, 2g13.6e1 )
                              end if                                      6e
10                        continue                                        5e
                      if ( out_lu .eq. STD_OUT )
     1                    call ilgwait()
                      end if                                              4e
              else                                                        3m
                  write( STD_ERR, 11 )
11                format( ' PVGRAD : WARNING : Calculate average ',
     1                'before tabulation' )
                  call ilgwait()
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE DO_ANALYSIS( RESULTS, TOTAL_POINTS,
     1                        AMP_GAIN, AMP_OFFSET, TRIP_DIAM,
     2                        REQUEST_FS, OUT_FILE )
c
c       Analyse the data collected from input file .
c
c Arguments
c  real      results(MAX_DATA,MAX_VARS)  # data & results array
c  integer   total_points   # number of data points
c  real      amp_gain,      # amplifier gain
c            amp_offset     # amplifier offset
c            trip_diam      # diameter of the boundary layer trip
c  integer   request_fs     # YES/NO request free stream speed from user
c  character out_file*(*)   # name of output file
c
c constants
c  FEET_TO_METRES  = 0.3048
c  TORR_TO_PASCALS = 133.322
c    instructions to fs_table
c  GET_FREE_STREAM = 1      # get free stream speed from table
c  APPEND_TO_TABLE = 2      # append data to table
c
c
c local variables
c  logical
c   deduce        # deduce dynamic pressure from pre-centrebody dynamic pressure
c   ok            # unused returned result from fs_table
c   written_table # table of results and data has been written
c   Equery        # function : read boolean
c   eyes          #          : convert YES/NO to logical
c   fs_table      #          : store/retrieve data in free stream speed table
c   pv_tabulate   #          : tabulate data and results
c  integer
c   index         # row index in results array
c   item          # selected menu item
c   n_points      # number of data points in results array
c   out_len       # length of output file name
c   Jresp         # function : read integer
c   str_length    #          : non-blank string length
c  real
c   fs_speed      # free stream air speed
c   ts_dyn_press  # free stream dynamic pressure in the test section
c   Qresp         # function : read real
c   qtts_dyn      # function : convert pre-centrebody dynamic press to air speed
c  subroutine
c   cresp         # read string
c   newscr        # clear screen
c   pv_average    # ensemble average of results
c   pv_edit       # edit data
c   pv_plot       # plot data
c   pv_poly_fit   # fit polynomial to data
c   sortsh        # sort array of data
c
      include 'stdio.def'
      include 'pvgrad.def'
      integer GET_FREE_STREAM, APPEND_TO_TABLE
      real FEET_TO_METRES, TORR_TO_PASCALS, AIR_DENSITY
      parameter( GET_FREE_STREAM=1, APPEND_TO_TABLE=2 )
      parameter( FEET_TO_METRES=0.3048, TORR_TO_PASCALS=133.322,
     1           AIR_DENSITY=1.204 )
c
      logical written_table, deduce, ok,
     1        Equery, eyes, fs_table, pv_tabulate
      integer total_points, index, item, n_points, out_len, request_fs,
     1        Jresp, str_length
      real results(MAX_DATA,MAX_VARS), amp_gain, amp_offset, fs_speed,
     1     trip_diam, ts_dyn_press, Qresp, qtts_dyn
      character out_file*(*)
c
c
c       Compile the data table
c
      deduce = .not. eyes( request_fs,
     1                     'Request free stream speed from user ? ' )
      do 3 index = 1, total_points                                        1b
          results(index,STREAM_X) = FEET_TO_METRES *
     1        results(index,PLUG_NO) - 0.164
          results(index,STAT_PRESS) = ( results(index,MEAN_VOLTS) -
     1        amp_offset ) / amp_gain * TORR_TO_PASCALS
c
c           calculate the free stream dynamic pressure from the
c           pre-centrebody dynamic pressure
c
          if ( deduce ) then                                              2b
              ts_dyn_press = qtts_dyn(
     1            results(index,CBODY_DYN) * TORR_TO_PASCALS,
     2            int( results(index,PLUG_NO) + 0.001 ),
     3            trip_diam )
              if ( ts_dyn_press .gt. 0.0 ) then                           3b
                  results(index,PRES_RATIO) =
     1                results(index,STAT_PRESS) / ts_dyn_press
              else                                                        3m
                  results(index,PRES_RATIO) = 0.0
                  end if                                                  3e
c
c           retrieve the free stream speed from the accumulated table
c           if possible .
c
          else                                                            2m
              if ( .not. fs_table( GET_FREE_STREAM,
     1              results(index,CBODY_DYN), fs_speed ) ) then           3b
c
c           If not successful, ask the user and append the result to the
c           table.
c
                  write( STD_OUT, 2 ) results(index,CBODY_DYN)
2                 format( ' What is the free stream speed for a' /
     1               '  pre-centrebody dynamic pressure of ',
     2               g12.5, ' ? ', $ )
                  fs_speed = Qresp( STD_IN, 0.5, 15.0, MAX_TRIES )
                  ok = fs_table( APPEND_TO_TABLE,
     1                results(index,CBODY_DYN), fs_speed )
                  end if                                                  3e
              results(index,PRES_RATIO) = results(index,STAT_PRESS) /
     1            ( 0.5 * AIR_DENSITY * fs_speed **2 )
              end if                                                      2e
3         continue
c
c       Now we have done the data compilation, we can start the analysis
c
      n_points = total_points
      written_table = .false.
c
c     start of menu loop                                                  1b
4         out_len = max( 1, str_length( out_file ) )
          call newscr()
          write( STD_OUT, 5 ) out_file(:out_len)
5         format( ' Select' / '   0) Exit' /
     1        '   1) Tabulation file  ', a /
     2        '   2) Tabulate results' /
     3        '   3) Plot results' /
     4        '   4) Edit data' /
     5        '   5) Ensemble average' /
     6        '   6) Fit polynomial' /
     7        '   7) Shift (offset) data' /
     8        '   8) Restore data   ', $ )
          item = Jresp( STD_IN, 0, 8, MAX_TRIES )
c
c       ***** exit
c
          if ( item .eq. 0 ) then                                         2b
c
c           write run results onto file
c
              if ( .not. written_table ) then                             3b
                  if ( Equery( STD_OUT, 'Tabulate data & results ? ' )
     1                    ) then                                          4b
                      written_table =
     1                    pv_tabulate( results, n_points, out_file )
                      end if                                              4e
                  end if                                                  3e
              return
c
c           Tabulation file
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Name of tabulation file ? ', $ )
              call cresp( STD_IN, out_file, MAX_TRIES )
c
c           Tabulate results
c
          else if ( item .eq. 2 ) then                                    2m
              written_table = pv_tabulate( results, n_points, out_file)
c
c           Plot results
c
          else if ( item .eq. 3 ) then                                    2m
              call pv_plot( results, n_points )
c
c           Edit data
c
          else if ( item .eq. 4 ) then                                    2m
              call pv_edit( results, n_points )
c
c           Ensemble average
c
          else if ( item .eq. 5 ) then                                    2m
              call pv_average( results, n_points, out_file )
c
c           Fit polynomial to data
c
          else if ( item .eq. 6 ) then                                    2m
              call pv_poly_fit( results, n_points, out_file )
c
c           Shift (offset) data
c           recompile the data table after doing this
c
          else if ( item .eq. 7 ) then                                    2m
              call pv_offset( results, n_points, trip_diam, deduce )
              goto 4
c
c           Restore data
c
          else if ( item .eq. 8 ) then                                    2m
              n_points = total_points
              call sortsh( results, total_points, 1,
     1            MAX_DATA, SEQUENCE, MAX_VARS )
              end if                                                      2e
          goto 4
c         end of menu loop                                                1e
      end



      SUBROUTINE PV_OFFSET( RESULTS, N_POINTS, TRIP_DIAM, DEDUCE )
c
c Offset one of the data columns -- typically static pressure .
c
c Arguments
c  real results(MAX_DATA,MAX_VARS)  # data & results array
c  integer n_points                 # number of data points
c  real    trip_diam                # diameter of boundary layer trip (mmm)
c  logical deduce                   # deduce dynamic pressure from
c                                   # pre-centrebody measurement
c
c constants
c  GET_FREE_STREAM = 1
c  APPEND_TO_TABLE = 2
c  TORR_TO_PASCALS = 133.322
c  AIR_DENSITY     = 1.204 )
c
c local variables
c  integer
c   column              # results column index for abscissa
c   i                   # looping index
c   item                # selected menu item
c   Jresp               # function : read integer
c  logical
c   ok                  # unused returned result from fs_table
c   fs_table            # function : store/retrieve data in free stream table
c  real
c   fs_speed            # free stream speed in test section
c   off_accum(MAX_VARS) # accumulated offset for each variable
c   off_increm          # incremental offset
c   off_total           # required total offset
c   ts_dyn_press        # free stream dynamic pressure in the test section
c   Qresp               # function : read real
c   qtts_dyn            #          : convert pre-centrebody to test section q
c  character
c   name_res*16         # function : result name
c  subroutine
c   newscr              # clear page
c
c
      include 'stdio.def'
      include 'pvgrad.def'
      integer GET_FREE_STREAM, APPEND_TO_TABLE
      real TORR_TO_PASCALS, AIR_DENSITY
      parameter( GET_FREE_STREAM=1, APPEND_TO_TABLE=2 )
      parameter( TORR_TO_PASCALS=133.322, AIR_DENSITY=1.204 )
c
      integer n_points, column, i, item, Jresp
      logical deduce, ok, fs_table
      real results(MAX_DATA,MAX_VARS), fs_speed, off_accum(MAX_VARS),
     1     off_increm, off_total, trip_diam, ts_dyn_press,
     2     Qresp, Qtts_dyn
      character name_res*16
      save column, off_increm, off_total, off_accum
      data column / STAT_PRESS /
      data off_increm, off_total, off_accum /
     1     0.0,        0.0,       MAX_VARS*0.0 /
c
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(column), off_increm, off_total,
     1        name_res(column), off_accum(column)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Offset variable         ', a /
     2        '   2) Incremental offset      ', g12.5 /
     3        '   3) Total required offset   ', g12.5 /
     4        '   4) Clear accumulated offset' /
     5        '   5) Apply incremental offset' /
     6        '   6) Apply total offset' /
     7        '      Accumulated offset for ', a, ' : ', g12.5, ' ', $ )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           Offset variable
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select offset variable' / ( 2x, i2, ') ', a ) )
              column = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
c
c           Incremental offset
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' What is the incremental offset ? ', $ )
              off_increm = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES )
c
c           Total offset
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' What is the required total offset ? ', $ )
              off_total = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES )
c
c           Clear accumulated offset
c
          else if ( item .eq. 4 ) then                                    2m
              off_accum(column) = 0.0
c
c           Apply incremental offset
c
          else if ( item .eq. 5 .or. item .eq. 6 ) then                   2m
              if ( item .eq. 5 ) then                                     3b
                  do 6 i = 1, n_points                                    4b
                      results(i,column) = results(i,column) + off_increm
6                     continue                                            4e
                  off_accum(column) = off_accum(column) + off_increm
c
c           Apply total offset
c
              else if ( item .eq. 6 ) then                                3m
                  do 7 i = 1, n_points                                    4b
                      results(i,column) = results(i,column) +
     1                    off_total - off_accum(column)
7                     continue                                            4e
                  off_accum(column) = off_total
                  end if                                                  3e
c
c       Recalculate PRES_RATIO column of the data table if the adjusted
c       variable is CBODY_DYN or STAT_PRESS .
c
              if ( column .eq. CBODY_DYN .or.
     1             column .eq. STAT_PRESS ) then                          3b
                  do 9 i = 1, n_points                                    4b
c
c           calculate the free stream dynamic pressure from the
c           pre-centrebody dynamic pressure
c
                      if ( deduce ) then                                  5b       2b
                          ts_dyn_press = qtts_dyn(
     1                        results(i,CBODY_DYN) * TORR_TO_PASCALS,
     3                        int( results(i,PLUG_NO) + 0.001 ),
     4                        trip_diam )
                          if ( ts_dyn_press .gt. 0.0 ) then               3b
                              results(i,PRES_RATIO) =
     1                            results(i,STAT_PRESS) / ts_dyn_press
                          else                                            3m
                              results(i,PRES_RATIO) = 0.0
                              end if                                      3e
c
c           retrieve the free stream speed from the accumulated table
c           if possible .
c
                      else                                                5m    2m
                          if ( .not. fs_table( GET_FREE_STREAM,
     1                          results(i,CBODY_DYN), fs_speed ) ) then   6b
c
c           If not successful, ask the user and append the result to the
c           table.
c
                              write( STD_OUT, 8 ) results(i,CBODY_DYN)
8                             format( ' What is the free stream speed ',
     1                            'for a pre-centrebody dynamic ',
     2                            'pressure of ', g12.5, ' ? ', $ )
                              fs_speed = Qresp( STD_IN, 0.5, 15.0,
     1                            MAX_TRIES )
                              ok = fs_table( APPEND_TO_TABLE,
     1                                 results(i,CBODY_DYN), fs_speed )
                              end if                                      6e
                          results(i,PRES_RATIO) =
     1                        results(i,STAT_PRESS) /
     2                        ( 0.5 * AIR_DENSITY * fs_speed **2 )
                          end if                                          5e
9                     continue                                            4e
                  end if                                                  3e
c
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end




      LOGICAL FUNCTION FS_TABLE( ACTION, PRE_CBODY_DYN, FS_SPEED )
c
c Arguments
c  integer action           # GET_FREE_STREAM - speed from table
c                           # APPEND_TO_TABLE - (pre_cbody_dyn,fs_speed)
c  real    pre_cbody_dyn    # pre-centrebody dynamic pressure (Torr)
c          fs_speed         # free stream speed in test section (m/s)
c
      include 'stdio.def'
      integer GET_FREE_STREAM, APPEND_TO_TABLE, MAX_TABLE
      parameter( GET_FREE_STREAM=1, APPEND_TO_TABLE=2, MAX_TABLE=16 )
c
      integer action, table_len, i
      real pre_cbody_dyn, fs_speed,
     1     table_dyn(MAX_TABLE), table_fs(MAX_TABLE)
      save table_len, table_dyn, table_fs
      data table_len / 0 /
c
c       Search for value in the table
c
      fs_table = .FALSE.
      if ( action .eq. GET_FREE_STREAM ) then
          do 1 i = 1, table_len
              if ( abs( pre_cbody_dyn - table_dyn(i) ) /
     1             ( abs( pre_cbody_dyn ) + abs( table_dyn(i) ) )
     2             .lt. 1.0e-4 ) then
                  fs_table = .TRUE.
                  fs_speed = table_fs(i)
                  return
                  end if
1             continue
c
c       Append data pair to table
c
      else if ( action .eq. APPEND_TO_TABLE ) then
          if ( table_len .lt. MAX_TABLE ) then
              table_len = table_len + 1
              table_dyn(table_len) = pre_cbody_dyn
              table_fs(table_len) = fs_speed
              fs_table = .TRUE.
          else
              write( STD_ERR, 2 )
2             format( ' FS_TABLE : ABORT : Table is full. ',
     1                'New value not appended.' )
              end if
c
c       Illegal first argument
c
      else
          write( STD_ERR, 3 )
3         format( ' FS_TABLE : ABORT : Unknown action' )
          end if
      return
      end



      SUBROUTINE PV_POLY_FIT( RESULTS, N_POINTS, OUT_FILE )
c
c Fit a polynomial to the selected abscissa-ordinate columns in the results
c array.
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points                # number of data points
c
c local variables
c  logical
c   fitted      # fitted polynomial has been calculated
c  integer
c   abscissa    # results column index for abscissa
c   ordinate    # results column index for ordinate
c   i           # looping index
c   item        # selected menu item
c   capt_len    # length of capture file name
c   out_len     # length of output file name
c   out_lu      # logical unit number
c   poly_order  # order of fitted polynomial
c   Jresp       # function : read integer
c   pv_open_file#          : open output file
c   str_length  #          : non-blank string length
c  real
c   fit_error   # rms. fitting error
c   poly_coeffs(MAX_ORDER)  # polynomial coefficients
c   poly_values(MAX_DATA)   # evaluated polynomial at abscissa values
c   qpoly                   # function : evaluate polynomial
c  double precision
c   work(MAX_ORDER,2)       # work array for SPYORTH
c  character
c   capt_file*64# name of capture file
c   out_file*(*)# name of output file
c   name_res*16 # function : result name
c  subroutine
c   cresp       # read string
c   ilgclos, ilgdata, ilgline, ilgopen, ilgwait, ilgxlab, ilgylab
c   newscr      # clear page
c   spyorths    # calculate curve fitting polynomial
c
c
      include 'stdio.def'
      include 'pvgrad.def'
c
c
      logical fitted
      integer n_points, abscissa, ordinate, i, item, capt_len, out_len,
     1        out_lu, poly_order, Jresp, pv_open_file, str_length
      real results(MAX_DATA,MAX_VARS), fit_error,
     1     poly_coeffs(MAX_ORDER), poly_values(MAX_DATA), qpoly
      double precision work(MAX_ORDER,2)
      character capt_file*64, out_file*(*), name_res*16
c
      save abscissa, ordinate, out_len, capt_len, poly_order
      data abscissa, ordinate, out_len, capt_len, poly_order /
     1     PLUG_NO, MEAN_VOLTS, 1, 1, 3 /
      data capt_file / ' ' /
c
c
      fitted = .FALSE.
      out_len = max( 1, str_length( out_file ) )
      capt_len = max( 1, str_length( capt_file ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(abscissa), name_res(ordinate),
     1        out_file(:out_len), capt_file(:capt_len), poly_order
2         format( ' Select' / '   0) Exit' /
     1        '   1) Abscissa', 12x, a /
     2        '   2) Ordinate', 12x, a /
     3        '   3) Tabulation file     ', a /
     4        '   4) Capture file        ', a /
     5        '   5) Polynomial order    ', i2 /
     6        '   6) Calculate polynomial' /
     7        '   7) Print polynomial' /
     8        '   8) Tabulate polynomial' /
     9        '   9) Plot polynomial      ', $ )
          item = Jresp( STD_IN, 0, 9, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           Abscissa
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) 'abscissa',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a / ( 2x, i2, ') ', a ) )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              fitted = .FALSE.
c
c           Ordinate
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 ) 'ordinate',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              fitted = .FALSE.
c
c           Tabulation file
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Name of tabulation file ? ', $ )
              call cresp( STD_IN, out_file, MAX_TRIES )
              out_len = max( 1, str_length( out_file ) )
c
c           Capture file
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Name of capture file ? ', $ )
              call cresp( STD_IN, capt_file, MAX_TRIES )
              capt_len = max( 1, str_length( capt_file ) )
c
c           Polynomial order
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' What is the polynomial order ', $ )
              poly_order = Jresp( STD_IN, 1, 12, MAX_TRIES )
              fitted = .FALSE.
c
c           Calculate polynomial
c
          else if ( item .eq. 6 ) then                                    2m
              call spyorths( results(1,abscissa), results(1,ordinate),
     1            n_points, poly_coeffs, poly_order, fit_error, work )
              fit_error = sqrt( abs( fit_error ) )
              do 7 i = 1, n_points                                        3b
                  poly_values(i) = qpoly( results(i,abscissa),
     1                poly_coeffs, poly_order )
7                 continue                                                3e
              fitted = .TRUE.
c
c           Any illegal values of item
c
          else if ( item .lt. 7 .or. item .gt. 9 ) then                   2m
              continue
c
c           Anything which requires prior calculation of polynomial
c
          else if ( fitted ) then                                         2m
c
c           Print polynomial
c
              if ( item .eq. 7 ) then                                     3b
                  out_lu = pv_open_file( out_file )
                  if ( out_lu .gt. 0 ) then                               4b
                      write( out_lu, 8 ) name_res(ordinate),
     1                    poly_order, name_res(abscissa), fit_error,
     2                    ( i, poly_coeffs(i), i = 1, poly_order + 1 )
8                     format( 1x, a, ' as a ', i2,
     1                    ' order polynomial of ', a /
     2                    ' rms. fitting error      : ', g13.6 /
     2                    ' polynomial coefficients :' /
     3                    ( 1x, i2, 3x, g15.8 ) )
                      if ( out_lu .eq. STD_OUT )
     1                    call ilgwait()
                  else                                                    4m
                      write( STD_OUT, 9 )
9                 format( 'WARNING : Calculate the polynomial first' )
                      call ilgwait()
                      end if                                              4e
c
c           Tabulate polynomial
c
              else if ( item .eq. 8 ) then                                3m
                  out_lu = pv_open_file( out_file )
                  if ( out_lu .gt. 0 ) then                               4b
                      write( out_lu, 10 )
     1                    name_res(abscissa), name_res(ordinate),
     2                    ( i, results(i,abscissa), results(i,ordinate),
     3                    poly_values(i), results(i,ordinate) -
     4                    poly_values(i), i = 1, n_points )
10                    format( 7x, 'abscissa        ordinate', 6x,
     1                    'fitting     fitting' /
     2                    3x, 2a16, ' polynomial    error' /
     3                    ( 1x, i3, g15.6, g16.5, 2g13.5 ) )
                      if ( out_lu .eq. STD_OUT )
     1                    call ilgwait()
                  else                                                    4m
                      write( STD_OUT, 9 )
                      call ilgwait()
                      end if                                              4e
c
c           Plot polynomial
c
              else if ( item .eq. 9 ) then                                3m
                  call ilgopen( capt_file )
                  call ilgraph( 1, 1, 11, 11, 1 )
                  call ilgline( 1, 0, 1 )
                  call ilgdata( results(1,abscissa),
     1                results(1,ordinate), 1, n_points, 1 )
                  call ilgline( 0, 1, 1 )
                  call ilgdata( results(1,abscissa),
     1                poly_values, 1, n_points, 0 )
                  call ilgylab( name_res(ordinate) )
                  call ilgxlab( name_res(abscissa) )
                  call ilgwait()
                  call ilgclos()
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end

