c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


       PROGRAM TTSQQ3
c
c       Read data from experiments which measures the free-stream dynamic
c pressure in the test section of the wind tunnel as a function of
c pre-centrebody dynamic pressure, test section plug number and trip diameter.
c
c    The data file contains a number of measurement sequences . Each sequence
c  is enclosed by zero reference readings which are used to eliminate drift
c  by linear interpolation .
c
c
c constants
c  MAX_TRIES=4    # maximum number of input attempts
c
c variables
c  real
c   amp_gain      # pressure sensor amplifier gain in Volts/Torr
c   qresp         # function : read real
c  integer
c   in_lu         # logical unit number of input file, 0=closed
c   out_lu        # logical unit number of output file, 0=closed
c   item          # item selected from menu
c   len_ipfile    # length of input file name
c   len_opfile    # length of output file name
c   res_in_lu     # logical unit number of input file
c   res_out_lu    # logical unit number of output file
c   iargc         # function : number of command line arguments
c   Jresp         #          : read integer
c   new_lun       #          : get new logical unit number
c   open_file     #          : open file
c   str_length    #          : non-blank length of string
c  characters
c   Cgain*64      # 3rd program argument = amplifier gain
c   Cipfile*64    # input file name
c   Copfile*64    # output file name
c   Csegment(0:1)*11 # analyse "one segment" or "all data"
c  subroutine
c   cresp         # read string
c   do_analysis
c   getarg        # get program argument
c   ilgwait       # wait for <CR>
c   newscr        # clear alphanumeric screen
c
c
      include 'stdio.def'
      integer MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      real amp_gain, qresp
      integer in_lu, item, len_ipfile, len_opfile, out_lu, res_in_lu,
     1    res_out_lu, all_data,
     2    iargc, Jresp, new_lun, open_file, str_length
      character Cgain*64, Cipfile*64, Copfile*64, Csegment(0:1)*11
c
      data in_lu, out_lu, len_ipfile, len_opfile /
     1     0, 0, 1, 1 /
      data Cgain, Cipfile, Copfile / ' ', ' ', ' ' /
      data Csegment / 'one segment', 'all data' /
c
c
c       Read the command line arguments.
c         pvgrad <input_file> <out_file> <amplifier_gain>
c
      res_in_lu = new_lun()
      res_out_lu = new_lun()
      if ( iargc() .ge. 1 ) then                                          1b
          call getarg( 1, Cipfile )
          if ( iargc() .ge. 2 ) then                                      2b
              call getarg( 2, Copfile )
              if ( iargc() .ge. 3 ) then                                  3b
                  call getarg( 3, Cgain )
                  read( Cgain, *, ERR=12 ) amp_gain
                  end if                                                  3e
              end if                                                      2e
          end if                                                          1e
c
c       main menu loop
c
1         call newscr()                                                   1b
          len_ipfile = max( 1, str_length( Cipfile ) )
          len_opfile = max( 1, str_length( Copfile ) )
          write( STD_OUT, 2 ) Cipfile(:len_ipfile),
     1        Copfile(:len_opfile), amp_gain,
     2        Csegment(all_data), Csegment( mod( all_data + 1, 2 ) )
2         format( ' Main menu' /
     1        '   0) Exit' /
     2        '   1) Input file name    ', a  /
     3        '   2) Output file name   ', a  /
     4        '   3) Amplifier gain     ', g14.7 /
     5        '   4) Analyse ', a, ' ( otherwise ', a, ' )' /
     6        '   5) Do analysis' /
     7        '   6) Help - file format ', $ )
c
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( out_lu .ne. 0 )
     2            close( UNIT=out_lu )
              stop
c
c       INPUT FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Input data file name ? ', $ )
              call cresp( STD_IN, Cipfile, MAX_TRIES )
c
c       OUTPUT FILE NAME
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Output data file name ? ', $ )
              call cresp( STD_IN, Copfile, MAX_TRIES )
c
c       AMPLIFIER GAIN
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Gain factor for static pressure & zero ',
     1            'readings ? ', $ )
              amp_gain = qresp( STD_IN, 0.5, 5000.0, MAX_TRIES )
c
c       ANALYSE ALL DATA IN ONE SEGMENT
c
          else if ( item .eq. 4 ) then                                    2m
              all_data = mod( all_data + 1, 2 )
c
c       ANALYSIS
c
          else if ( item .eq. 5 ) then                                    2m
              if ( Cipfile .eq. ' ' ) then                                3b
                  write( STD_ERR, 6 )
6                 format( ' TSQQ3 : Please supply input file name' )
                  call ilgwait()
              else                                                        3m
c
c       open input and output files
c
                  if ( in_lu .ne. 0 ) then                                4b
                      close( UNIT=in_lu )
                      in_lu = 0
                      end if                                              4e
                  in_lu = open_file( res_in_lu, Cipfile, 'old', 0 )
                  if ( in_lu .eq. 0 ) then                                4b
                      write( STD_ERR, 7 ) Cipfile(:len_ipfile)
7                     format( ' TSQQ3 : Cannot open input file ', a )
                      call ilgwait()
                  else                                                    4m
                      call read_press_data( in_lu, Copfile, amp_gain,
     1                                      all_data )
                      close( UNIT=in_lu )
                      end if                                              4e
                  end if                                                  3e
c
c       HELP
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 8 )
8             format( ' Data file format :' /
     1            '     Each line in the data file corresponds to a ',
     2                'single measurement from a' /
     3            '     manometer, and the data is assumed to be in ',
     4                'chronological order. Each line' /
     5            '     in the data file has three fields occupying ',
     6                'up to a total of 80 characters.' )
              write( STD_OUT, 9 )
9             format( '     The fields are :-' /
     1            6x, '<descripion> <average> <standard_deviation>' //
     2            ' <description>     is a two character label which ',
     3                ' describes the purpose of the' /
     4            '    line . The first character can be ''p'', ''P''',
     5                ' ''z'', ''Z'', ''s'', ''S'' or '';''. If' /
     6            '    it is a ''p'' or ''P'', the next character ',
     7                'must be a hexadecimal digit denoting' /
     8            '    test section plug number . A line beginning ',
     9                'with ''P'' or ''p'' contains a' /
     a            '    static pressure measurement at a particular ',
     b                'plug .' )
              call ilgwait()
              write( STD_OUT, 10 )
10            format( 7x, 'If the first character is a ''z'' or ''Z''',
     1                ' the measurement is a zero' /
     2            '    reference, and the next character must be one ',
     3                'of :--' /
     4            9x, '''n'' or ''N'' -- initial zero for a new ',
     5                'measurement sequence' /
     6            9x, '''r'' or ''R'' -- final zero for a measurement ',
     7                'sequence . If the previous' /
     8            10x, 'zero reference was the same type, it becomes ',
     9                'the initial reference .' /
     a            '     If the first character is an ''s'' or ''S'', a ',
     b                'new run is started. The next' /
     c            '     character can be anything, but the <average> ',
     d                'field contains the measured' /
     e            '     pre-centrebody dynamic pressure . If the first ',
     f                'character is a '';'', the' /
     g            '     line is a comment .' )
              write( STD_OUT, 11 )
11            format( ' <average>     is a average static pressure, ',
     1                'dynamic pressure or zero reading .' /
     2            '    The static pressure and zero readings are ',
     3                'assumed to be factored by a' /
     4            '    specified amplifier gain .' /
     5            ' <standard_deviation> is a sample standard ',
     6                'deviation on the <average> . This' /
     7            '    is an optional field and is not read .' )
              call ilgwait()
              end if                                                      2e
          goto 1
c         end of main loop
c
c
c       ERROR TRAPS
c
12    write( STD_OUT, 13 )
13    format( ' TSQQ3 : WARNING : Cannot read argument 1 : ',
     1        'amplifier gain' )
      call ilgwait()
      goto 1
c
      end



      SUBROUTINE READ_PRESS_DATA( IN_LU, OUT_FILE, AMP_GAIN, ALL_DATA )
c
c parameters
c  integer   in_lu          # logical unit number of input file
c  character out_file*(*)   # logical unit number of output file
c  real      amp_gain       # amplifier gain
c  integer   all_data       # YES/NO analyse all data in one lump i.e.
c                             do not split data at "s"
c
c    HOW IT WORKS
c
c    first_zero = run_started = zero_found = false
c    start_index = 1
c    start loop:
c      read I/P file record
c      count the columns
c      if 1st col = ;
c         print comment
c      else if 1st col = p and zero_found and run_started
c         assign plug number
c         calculate distance from boundary layer trip
c         read data
c      else if 1st col = z and ( 2nd col = n or 2nd col = r and not zero_found)
c           and run_started
c         first_zero = zero_found = true
c         read first zero
c      else if 1st col = z and 2nd col = r and run_started
c         read average into buffer
c         if not first_zero
c           transfer second zero into first zero
c         transfer buffered zero into second zero
c         first_zero = false
c         calculate offset & gradient for zero reference line
c      else if 1st col = s
c         if run_started and index > 1
c           do analysis
c           zero_found = false
c         read average into pre-centrebody dynamic pressure
c      end loop
c end_of_input_file:
c    if run_started and index > 1
c      do analysis
c
c
c logical
c   got_warning         # warning error has occurred without waiting for <CR>
c   fgotz               # have a first zero reference value
c   firstz              # previous zero reference was a `first'
c   frungo              # a run has been started
c   eyes                # function : convert integer to logical
c  integer
c   i                   # looping indices
c   iplug               # wind tunnel plug number -- temporary storage
c   n_chars             # no. of characters in current I/P file record
c   n_cols              # number of columns of data
c   n_points            # number of data points in RESULT array
c   record_no           # input file record number
c   seq_counter         # sequence number - source
c   seq_index           # index into the current sequence
c   seq_length          # number of items in the current sequence
c   seq_start           # starting index of current sequence
c   jljstn              # function : left justifies & counts columns
c  real
c   amp_offset          # amplifier offset
c   dynam_press         # dynamic pressure in Torr
c   results(MAX_DATA,MAX_VARS) # array of results & data
c     The column indices are
c       PLUG_NO    = 1    # wind tunnel plug number
c       SEQUENCE   = 2    # data sequence number
c       STREAM_X   = 3    # distance from leading edge
c       MEAN_VOLTS = 4    # mean voltage
c       CBODY_DYN  = 5    # pre-centrebody dynamic pressure
c       STAT_PRESS = 6    # static pressure
c       PRES_RATIO = 7    # static/dynamic pressure
c       AMPL_GAIN  = 8    # result index of amplifier gain
c   zbuff,              # temporarily buffered value of zero2
c   zero1, zero2        # endpoints of zeroreference line
c   zgrad,              # rate of change of zero reference
c  character
c   Cline*80            # input file data record
c  subroutine
c   do_analysis
c   ilgwait
c
      include 'stdio.def'
      include 'ttsqq3.def'
      real NOT_SET
      parameter( NOT_SET=-9.999e-29 )
c
      logical fgotz, firstz, frungo, got_warning, eyes
      integer i, iplug, n_chars, n_cols, n_points, in_lu,
     1    record_no, seq_counter, seq_index, seq_length,
     2    seq_start, all_data, jljstn
      real adjust, amp_gain, amp_offset, dynam_press,
     1    results(MAX_DATA,MAX_VARS), zbuff, zero1, zero2, zgrad
      character Cline*80, out_file*(*)
c
c         initialize flags & indices
c
      got_warning = .FALSE.
      fgotz =  .false.
      frungo = .false.
      firstz = .false.
      seq_length  = 0
      seq_counter = 1
      seq_start   = 1
      n_points    = 0
      record_no   = 0
      amp_offset  = NOT_SET
c
c         start loop for reading a data file record
c
c         read I/P file record
c
1         read( in_lu, 2, END=11, ERR=12 ) Cline                          1b
2         format( a )
          record_no = record_no + 1
c
c         count the columns
c
          n_chars = jljstn( Cline, n_cols )
c
c    **** found comment line, print it out
c
          if ( Cline(1:1) .eq. ';' ) then                                 2b
              write( STD_OUT, 3 ) Cline(1:n_chars)
3             format( 1x, a )
c
c    **** static pressure data record
c
          else if ( n_chars .gt. 4 .and. n_cols .gt. 1 ) then             2m
              if ( ( Cline(1:1) .eq. 'p' .or. Cline(1:1) .eq. 'P' )
     1             .and. fgotz .and. frungo ) then                        3b
c
c         decode 2nd col and assign plug number
c
                  iplug = 0
                  if ( '0' .le. Cline(2:2) .and. Cline(2:2) .le. '9' )
     1                    then                                            4b
                      iplug = ichar( Cline(2:2) ) - ichar( '0' )
                  else if ( 'A' .le. Cline(2:2) .and.
     1                    Cline(2:2) .le. 'F' ) then                       4m
                      iplug = ichar( Cline(2:2) ) - ichar( 'A' ) + 10
                  else if ( 'a' .le. Cline(2:2) .and.
     1                    Cline(2:2) .le. 'f' ) then                       4m
                      iplug = ichar( Cline(2:2) ) - ichar( 'a' ) + 10
                      end if                                              4e
c
c
                  if ( 0 .lt. iplug .and. iplug .lt. 16 ) then            4b
                      if ( n_points .lt. MAX_DATA ) then                  5b
c
c         read average into data array
c
                          read( Cline(3:), *, ERR=14 )
     1                        results(n_points+1,MEAN_VOLTS)
c
c         increment data index and sequence length counter
c
                          n_points = n_points + 1
                          seq_length  = seq_length + 1
                          results(n_points,PLUG_NO) = iplug
                          results(n_points,SEQUENCE) = seq_counter
                          results(n_points,CBODY_DYN) = dynam_press
                          results(n_points,AMPL_GAIN) = amp_gain
                      else                                                5m
                          write( STD_ERR, 4 ) record_no
4                         format( ' WARNING : Array space full at ',
     1                        'line number ', i4 )
                          got_warning = .TRUE.
                          end if                                          5e
                  else                                                    4m
                      write( STD_OUT, 5 ) record_no
5                     format( ' WARNING : Unknown plug number in ',
     1                    'line ', i4 )
                      got_warning = .TRUE.
                      end if                                              4e
c
c    ***** first zero reference
c
              else if ( ( Cline(1:1) .eq. 'z' .or.
     1                Cline(1:1) .eq. 'Z' ) .and. frungo ) then           3m
                  if ( Cline(2:2) .eq. 'n' .or. Cline(2:2) .eq. 'N'
     1                    .or. ( Cline(2:2) .eq. 'r' .or.
     2                    Cline(2:2) .eq. 'R' ) .and. .not. fgotz ) then  4b
c
c         read average into first zero
c
                      read( Cline(3:), *, ERR=14 ) zero1
                      if ( .not. fgotz .or. amp_offset.eq.NOT_SET ) then  5b
                          amp_offset = zero1
                          end if                                          5e
                      fgotz = .true.
                      firstz = .true.
c
c    ***** second reference zero
c
                  else if ( ( Cline(2:2) .eq. 'r' .or.
     1                    Cline(2:2) .eq. 'R' ) .and. frungo ) then       4m
c
c         read average zero reference into buffer
c
                      read( Cline(3:), *, ERR=14 ) zbuff
c
c         if the last zero was not the first, roll the second into the first
c
                      if ( .not. firstz )
     1                    zero1 = zero2
                      zero2 = zbuff
                      firstz = .false.
c
c         calculate gradient for zero reference line
c
                      zgrad = ( zero2 - zero1 ) /
     1                    real( seq_length + 2 )
c
c           calculate static pressure ratio for the existing sequence
c
                      adjust = amp_offset - zero1 - zgrad
                      seq_index = 0
c                      adjust = - zero1 - zgrad
                      do 6 i = seq_start, n_points                        5b
                          seq_index = seq_index + 1
                          results(i,MEAN_VOLTS) = results(i,MEAN_VOLTS)
     1                        + adjust
                          adjust = adjust - zgrad
6                         continue                                        5e
c
c           update pointers
c
                      seq_counter = seq_counter + 1
                      seq_start   = n_points + 1
                      seq_length  = 0
                  else                                                    4m
                      write( STD_OUT, 7 ) record_no
7                     format( ' WARNING : Unknown zero reference ',
     1                    'type in line ', i4 )
                      got_warning = .TRUE.
                      end if                                              4e
c
c     **** change amplifier gain
c
              else if ( Cline(1:1) .eq. 'g' .or. Cline(1:1) .eq. 'G' )
     1                then                                                3m
c                 read amplifier gain
                  read( Cline(3:), *, ERR=14 ) amp_gain
c
c     **** start run
c
              else if ( Cline(1:1) .eq. 's' .or. Cline(1:1) .eq. 'S' )
     1                then                                                3m
                  if ( .not. Eyes( all_data, 'Analyse segment ?' ) )
     1                    then                                            4b
c
c           write results to file
c
                      if ( n_points .gt. 1 .and. frungo ) then            5b
                          if ( amp_offset .eq. NOT_SET ) then             6b
                              write( STD_ERR, 16 )
                              call ilgwait()
                          else                                            6m
                              if ( got_warning )
     1                            call ilgwait()
                              call do_analysis( results, n_points,
     1                            amp_offset, out_file )
                              got_warning = .FALSE.
                              firstz = .false.
                              end if                                      6e
                          end if                                          5e
                      fgotz = .false.
                      amp_offset = NOT_SET
                      seq_counter = 1
                      seq_start = 1
                      end if                                              4e
c
c         read average into pre-centrebody dynamic pressure
c
                  read( Cline(3:), *, ERR=14 ) dynam_press
                  frungo = .true.
              else                                                        3m
c
c         I/P data file record not understood
c
                  write( STD_ERR, 8 ) record_no
8                 format( ' WARNING : Invalid 1st character in line ',
     1                i4 )
                  got_warning = .TRUE.
                  end if                                                  3e
          else                                                            2m
              write( STD_ERR, 9 ) record_no
9             format( ' WARNING : Line ', i4, 'not long enough .' )
              got_warning = .TRUE.
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
c
c           end of input file found : write results to file
c
11    if ( n_points .gt. 1 .and. frungo ) then                            1b
          if ( amp_offset .eq. NOT_SET ) then                             2b
              write( STD_ERR, 16 )
16            format( ' WARNING : Amplifier offset unknown. Cannot do ',
     1                'analysis' )
              call ilgwait()
          else                                                            2m
              if ( got_warning )
     1            call ilgwait()
              call do_analysis( results, n_points, amp_offset, out_file)
              got_warning = .FALSE.
              end if                                                      2e
          end if                                                          1e
      close( UNIT=in_lu )
      in_lu = 0
      if ( got_warning )
     1    call ilgwait()
      return
c
12    write( STD_ERR, 13 )
13    format( ' WARNING : Cannot read data file' )
      call ilgwait()
      return
c
14    write( STD_ERR, 15 ) record_no
15    format( ' WARNING : Cannot decode number at line ', i4 )
      call ilgwait()
      goto 1
      end



      INTEGER FUNCTION TT_OPEN_FILE( OUT_FILE )
c
c Opens output file of name OUT_FILE .
c
c Argument
c  character out_file*(*)         # name of output file
c
c local variables
c  integer
c   out_lu      # logical unit number of output file
c   res_out_lu  # reserved logical unit number of output file
c   new_lun     # function : get new logical unit number
c   open_file   #          : open file
c  character
c   prev_file   # name of previous output file
c
c
      include 'stdio.def'
c
      integer out_lu, res_out_lu, new_lun, open_file
      character out_file*(*), prev_file*64, old_file*64
c
      save out_lu, res_out_lu, prev_file, old_file
      data out_lu, res_out_lu / 0, 0 /
      data prev_file, old_file / 2 * ' ' /
c
      if ( res_out_lu .eq. 0 )
     1    res_out_lu = new_lun()
      if ( out_file .eq. '-' ) then                                       1b
          out_file = old_file
          end if                                                          1e
c
c       close an open file if it is new or is different from the previous
c
      if ( prev_file .ne. ' ' .and. out_file .ne. prev_file ) then        1b
          if ( out_lu .ne. STD_OUT .and. out_lu .ne. 0 ) then             2b
              close( UNIT=out_lu )
              out_lu = 0
              end if                                                      2e
          end if                                                          1e
c
c       open the file if ( it is new or is different from the previous )
c          and ( it has been changed to the screen )
c
      if ( ( ( out_file .ne. prev_file .or. prev_file .eq. ' ' ) .and.
     1       ( prev_file .ne. ' ' .or. out_file .ne. old_file .or.
     2         out_file .eq. ' ' ) ) .or. out_lu .eq. 0 ) then            1b
          out_lu = open_file( res_out_lu, out_file, 'unknown', 0 )
          end if                                                          1e
      tt_open_file = out_lu
c
c       remember the file name
c
      if ( out_lu .gt. 0 ) then                                           1b
          if ( out_file .ne. prev_file ) then                             2b
              old_file = prev_file
              prev_file = out_file
              end if                                                      2e
      else                                                                1m
          write( STD_ERR, 1 )
1         format( ' TSQQ3 : WARNING : Cannot open output file' )
          call ilgwait()
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION TT_TABULATE( RESULTS, N_POINTS, OUT_FILE )
c
c Tabulate the data and results array .
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points               # number of data points
c  character out_file*(*)         # name of output file, 0=closed
c
c local variables
c  integer
c   index         # data array row index
c   tt_open_file  # function : open output file
c  subroutine
c   ilgwait
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      real results(MAX_DATA,MAX_VARS)
      integer index, n_points, out_lu, tt_open_file
      character out_file*(*)
c
      out_lu = tt_open_file( out_file )
      tt_tabulate = out_lu .gt. 0
      if ( tt_tabulate ) then                                             1b
          write( out_lu, 1 )
1         format( ' ;' /
     1      ' ; static pressure distribution data and results'/
     2      ' ; ' /
     3      ' ;    Cbody plug sequ- dist.  mean    ', 2( ' t-section' )
     4          '    amplifier' /
     5      ' ;    dynam no.  ence   x    voltage    dynamic  C-body'
     6          '     gain' /
     7      ' ;    Torr             (m)     (V)       (Pa)     ratio' )
          do 3 index = 1, n_points                                        2b
              write( out_lu, 2, ERR=4 )
     1            index, results(index,CBODY_DYN),
     2            int( results(index,PLUG_NO) + 0.1 ),
     3            int( results(index,SEQUENCE) + 0.1 ),
     4            results(index,STREAM_X),  results(index,MEAN_VOLTS),
     5            results(index,TS_DYN_PR), results(index,PRES_RATIO),
     6            results(index,AMPL_GAIN)
2             format( i4, f9.3, i3, i4, f7.3, f9.4, f11.5, 2g13.5 )
              if ( out_lu .eq. STD_OUT .and. ( mod( index, 16 ) .eq. 0
     1                .or. index .eq. n_points ) ) then                   3b
                  call ilgwait()
                  if ( index .ne. n_points )
     1                write( out_lu, 1 )
                  end if                                                  3e
3             continue                                                    2e
          end if                                                          1e
      return
c
4     write( STD_ERR, 5 )
5     format( ' WARNING : Cannot write to output file' )
      tt_tabulate = .FALSE.
      call ilgwait()
      return
      end



      CHARACTER *16 FUNCTION NAME_RES( WHAT )
c
c Returns a character string description of the variable in the WHATth
c column of the results array .
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      integer what
      character names(MAX_VARS)*16
      save names
      data names /
     1    'plug number    ',
     2    'sequence number',
     3    'stream distance',
     4    'mean voltage   ',
     5    'c-body dyn pres',
     6    't-sect dyn pres',
     7    't/c press ratio',
     8    'amplifier gain ' /
c
      if ( 1 .le. what .and. what .le. MAX_VARS ) then                    1b
          name_res = names(what)
      else                                                                1m
          name_res = 'unknown'
          end if                                                          1e
      return
      end



      SUBROUTINE TT_PLOT( RESULTS, N_POINTS )
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points               # number of data points
c
c local variables
c  logical
c   plotted     # graph is plotted & visible
c   scaled      # scaling has been performed for the graph
c   ilgread     # function : read crosshairs
c  integer
c   abscissa    # results column index for abscissa
c   ordinate    # results column index for ordinate
c   abs_scale   # abscissa scaling  --  1 = 'lin', 2 = 'log'
c   ord_scale   # ordinate scaling  --  1 = 'lin', 2 = 'log'
c   capt_len    # length of capture file name
c   i           # looping index
c   item        # selected menu item
c   seq_len     # number of data items in a sequence
c   seq_start   # start index of sequence
c   symbol      # plotting symbol used for the sequence
c   Jresp       # function : read integer
c   str_length  #          : non-blank string length
c  real
c   xmin, xmax  # abscissa limits
c   ymin, ymax  # ordinate limits
c  character
c   capt_file*64# name of capture file
c   cjunk*1     # character pressed for crosshairs -- not used
c   cscale(2)*3 # 'lin', 'log'
c   name_res*16 # function : result name
c  subroutine
c   cresp       # read string
c   ilgclos, ilgdata, ilgimit, ilgline, ilgopen, ilgxlab, ilgylab
c   newscr      # clear page
c   swap4       # swap values
c
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      logical plotted, scaled, ilgread
      integer n_points, abscissa, ordinate, abs_scale, ord_scale,
     1        capt_len, i, item, seq_len, seq_start, symbol,
     2        Jresp, str_length
      real results(MAX_DATA,MAX_VARS), xmin, xmax, ymin, ymax
      character capt_file*64, cjunk*1, Cscale(2)*3, name_res*16
      save abscissa, ordinate, abs_scale, ord_scale, capt_len,
     1     capt_file, Cscale
      data abscissa, ordinate, abs_scale, ord_scale, capt_len /
     1     PLUG_NO, MEAN_VOLTS, 1, 1, 1 /
      data capt_file, Cscale / ' ', 'lin', 'log' /
c
      scaled = .FALSE.
      plotted = .FALSE.
      capt_len = max( 1, str_length( capt_file ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(abscissa), name_res(ordinate),
     1        capt_file(:capt_len), Cscale(abs_scale), Cscale(ord_scale)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Abscissa      ', a /
     2        '   2) Ordinate      ', a /
     3        '   3) Capture file  ', a /
     4        '   4) Abscissa scale ', a /
     5        '   5) Ordinate scale ', a /
     6        '   6) Plot' /
     7        '   7) Zoom' /
     8        '   8) Rescale  ', $ )
          item = Jresp( STD_IN, 0, 8, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              call ilgclos()
              return
c
c           Abscissa
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) 'abscissa',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a / ( 2x, i2, ') ', a ) )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              scaled = .FALSE.
c
c           Ordinate
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 ) 'ordinate',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              scaled = .FALSE.
c
c           Capture file
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Name of capture file ? ', $ )
              call cresp( STD_IN, capt_file, MAX_TRIES )
              capt_len = max( 1, str_length( capt_file ) )
c
c           Abscissa scale
c
          else if ( item .eq. 4 ) then                                    2m
              abs_scale = mod( abs_scale, 2 ) + 1
c
c           Ordinate scale
c
          else if ( item .eq. 5 ) then                                    2m
              ord_scale = mod( ord_scale, 2 ) + 1
c
c           Plot, Zoom or Rescale
c
          else if ( 6 .le. item .and. item .le. 8 ) then                  2m
c
c             find limits of data if not previously scaled or `rescaling'
c
              if ( .not. scaled .or. item .eq. 8 ) then                   3b
                  xmin = results(1,abscissa)
                  xmax = xmin
                  ymin = results(1,ordinate)
                  ymax = ymin
                  do 5 i = 1, n_points                                    4b
                      xmin = min( xmin, results(i,abscissa) )
                      xmax = max( xmax, results(i,abscissa) )
                      ymin = min( ymin, results(i,ordinate) )
                      ymax = max( ymax, results(i,ordinate) )
5                     continue                                            4e
                  scaled = .TRUE.
                  end if                                                  3e
c
c             If zooming, get the corners of the zoom window
c
              if ( item .eq. 7 ) then                                     3b
                  if ( plotted ) then                                     4b
                      call newscr()
                      write( STD_OUT, 6 )  'first'
6                     format( ' Locate the ', a, ' corner of the ',
     1                    'zoom window' )
7                     if ( ilgread( xmin, ymin, cjunk ) ) then            5b
                          write( STD_OUT, 6 )  'second'
                          if ( ilgread( xmax, ymax, cjunk ) ) then        6b
                              if ( xmin .gt. xmax )
     1                            call swap4( xmin, xmax )
                              if ( ymin .gt. ymax )
     1                            call swap4( ymin, ymax )
                              xmax = max( xmax, 1.01 * xmin )
                              ymax = max( ymax, 1.01 * ymin )
                          else                                            6m
                              scaled = .FALSE.
                              goto 1
                              end if                                      6e
                      else                                                5m
                          scaled = .FALSE.
                          goto 1
                          end if                                          5e
                  else                                                    4m
                      write( STD_ERR, 8 )
8                     format( ' WARNING : Must plot before zooming' )
                      call ilgwait()
                      end if                                              4e
                  end if                                                  3e
c
c             start graphics
c
              call ilgopen( capt_file )
              call ilgraph( 1, 1, 27, 27, 1 )
              call ilgslaw( Cscale(abs_scale), Cscale(ord_scale) )
              call ilgimit( xmin, ymin, xmax, ymax )
c
c             plot one sequence at a time, with a different symbol for each
c
              seq_start = 1
c             start of loop
9             if ( seq_start .le. n_points ) then                         3b
                  symbol = results(seq_start,SEQUENCE) + 0.1
                  seq_len = 1
10                if ( seq_start + seq_len .le. n_points ) then           4b
                      if ( symbol .eq. int( results( seq_start+seq_len,
     1                        SEQUENCE) + 0.1 ) ) then                    5b
                          seq_len = seq_len + 1
                          goto 10
                          end if                                          5e
                      end if                                              4e
                  call ilgline( mod( symbol - 1, 14 ) + 1, 0, 1 )
                  call ilgdata( results(seq_start,abscissa),
     1                results(seq_start,ordinate), 1, seq_len, 0 )
                  seq_start = seq_start + seq_len
                  goto 9
                  end if                                                  3e
c                 end of loop
              call ilgylab( name_res(ordinate) )
              call ilgxlab( name_res(abscissa) )
              plotted = .true.
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE TT_EDIT( RESULTS, N_POINTS )
c
c Edit the results and data array .
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points                # number of data points
c
c local variables
c  integer
c   abscissa    # results column index for abscissa
c   ordinate    # results column index for ordinate
c   abs_scale   # abscissa scaling  --  1 = 'lin', 2 = 'log'
c   ord_scale   # ordinate scaling  --  1 = 'lin', 2 = 'log'
c   i           # looping index
c   item        # selected menu item
c   Jresp       # function : read integer
c  character
c   cscale(2)*3 # 'lin', 'log'
c   name_res*16 # function : result name
c  subroutine
c   ilgxlab, ilgylab
c   newscr      # clear page
c   sfudg2      # edit data
c
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      real    results(MAX_DATA,MAX_VARS)
      integer n_points, abscissa, ordinate, abs_scale, ord_scale,
     1        i, item, Jresp
      character Cscale(2)*3, name_res*16
      save abscissa, ordinate, abs_scale, ord_scale
      data abscissa, ordinate, abs_scale, ord_scale /
     1     PLUG_NO, MEAN_VOLTS, 1, 1 /
      data Cscale / 'lin', 'log' /
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(abscissa), name_res(ordinate),
     1                        Cscale(abs_scale), Cscale(ord_scale)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Abscissa', 7x, a /
     2        '   2) Ordinate', 7x, a /
     3        '   3) Abscissa scale ', a /
     4        '   4) Ordinate scale ', a /
     5        '   5) Edit  ', $ )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              call ilgclos()
              return
c
c           Abscissa
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) 'abscissa',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a / ( 2x, i2, ') ', a ) )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
c
c           Ordinate
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 ) 'ordinate',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
c
c           Abscissa scale
c
          else if ( item .eq. 3 ) then                                    2m
              abs_scale = mod( abs_scale, 2 ) + 1
c
c           Ordinate scale
c
          else if ( item .eq. 4 ) then                                    2m
              ord_scale = mod( ord_scale, 2 ) + 1
c
c           Edit
c
          else if ( item .eq. 5 .or. item .eq. 6 ) then                   2m
              call ilgopen( ' ' )
              call ilgslaw( Cscale(abs_scale), Cscale(ord_scale) )
              call ilgylab( name_res(ordinate) )
              call ilgxlab( name_res(abscissa) )
              call sfudg2( results, abscissa, ordinate, MAX_DATA,
     1                     MAX_VARS, 1, n_points, n_points )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE TT_AVERAGE( RESULTS, N_POINTS, OUT_FILE,
     1    TRIP_DIAM, MARGIN_CBD )
c
c Calculate the (ensemble) average of a result column for each wind tunnel
c test section port .
c
c Arguments
c  real      results(MAX_DATA,MAX_VARS) # data & results array
c  integer   n_points                   # number of data points
c  character out_file*(*)               # name of output file
c  real      trip_diam                  # diameter of trip balls
c            margin_cbd                 # margin in permitted pre-centrebody
c                                       # dynamic pressure expressed as a
c                                       # factor greater than one
c local variables
c  logical
c   averaged    # average has been calculated
c  integer
c   abscissa    # averaging base variable
c   abs_scale   # abscissa scaling  --  1 = 'lin', 2 = 'log'
c   bin_ptr     # index into array of ensemble averages
c   capt_len    # length of capture file name
c   counts(MAX_BINS)   # number of values averaged at each port
c   i           # looping index
c   item        # selected menu item
c   i_point     # row index to results array
c   n_bins      # number of bins used for ensemble averaging
c   ordinate    # results column index for ordinate
c   ord_scale   # ordinate scaling  --  1 = 'lin', 2 = 'log'
c   out_len     # length of output file name
c   out_lu      # logical unit number of output file, 0=closed
c   Jresp       # function : read integer
c   str_length  #          : non-blank length of string
c   tt_open_file#          : open output file
c  real
c   base(MAX_BINS)    # averaging base values
c   base_jitter       # bin width -- proportion of base value
c   max_cbd(MAX_BINS) # maximum pre-centrbody dynamic pressures
c   min_cbd(MAX_BINS) # minimum pre-centrbody dynamic pressures
c   mean(MAX_BINS)    # mean value at each plug
c   std_dev(MAX_BINS) # standard deviation of the mean at each plug
c   Qresp             # function : read real
c  character
c   abscis_str*16 # temporary storage for name_res(abscissa)
c   capt_file*64  # capture file name
c   cscale(2)*3   # 'lin', 'log'
c   out_file*64   # name of output file
c   name_res*16   # function : result name
c  subroutine
c   cresp       # read string
c   ilgdata, ilgline, ilgopen, ilgtitl, ilgwait, ilgxlab, ilgylab # ilglib
c   newscr      # clear screen
c
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      logical averaged
      integer n_points,
     1        abscissa, abs_scale, ord_scale, bin_ptr, capt_len,
     2        counts(MAX_BINS), i, item, i_point, n_bins, ordinate,
     3        out_len, out_lu, Jresp, str_length, tt_open_file
      real    results(MAX_DATA,MAX_VARS), trip_diam, margin_cbd,
     1        base(MAX_BINS), base_jitter, max_cbd(MAX_BINS),
     2        min_cbd(MAX_BINS), mean(MAX_BINS), std_dev(MAX_BINS),
     3        Qresp
      character abscis_str*16, out_file*(*), capt_file*64,
     1        Cscale(2)*3, name_res*16
c
      save abscissa, ordinate, abs_scale, ord_scale, out_lu,
     1     base_jitter, Cscale, capt_file
      data abscissa, ordinate, abs_scale, ord_scale, out_lu /
     1     PLUG_NO, PRES_RATIO, 1, 1, 0 /
      data base_jitter / 0.01 /
      data capt_file, Cscale / ' ', 'lin', 'log' /
      averaged = .FALSE.
c
c     start of menu loop                                                  1b
1         out_len = max( 1, str_length( out_file ) )
          capt_len = max( 1, str_length( capt_file ) )
          call newscr()
          write( STD_OUT, 2 ) name_res(ordinate), name_res(abscissa),
     1        out_file(:out_len), capt_file(:capt_len),
     2        base_jitter * 100.0, Cscale(abs_scale), Cscale(ord_scale)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Averaged variable ', a /
     2        '   2) Base variable     ', a /
     3        '   3) Tabulation file   ', a /
     4        '   4) Capture file      ', a /
     5        '   5) Max % base jitter ', g10.3 /
     6        '   6) Abscissa scale ', a /
     7        '   7) Ordinate scale ', a /
     8        '   8) Tabulate ensemble average' /
     9        '   9) Plot ensemble average' /
     9        '  10) Write calibration file records' /
     a        '      (Without fitted polynomials)   ', $ )
          item = Jresp( STD_IN, 0, 10, MAX_TRIES )
c
c           0) Exit
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           1) Averaged variable
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )  'averaged',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a, ' variable' /
     1            ( 2x, i2, ') ', a ) )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              averaged = .FALSE.
c
c           2) Base variable
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )  'base',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              averaged = .FALSE.
c
c           3) Tabulation file
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 ) 'tabulation'
4             format( ' Name of ', a, ' ? ', $ )
              call cresp( STD_IN, out_file, MAX_TRIES )
c
c           4) Capture file
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 4 ) 'capture'
              call cresp( STD_IN, capt_file, MAX_TRIES )
c
c           5) Maximum base jitter
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 5 ) name_res(abscissa)
5             format( ' Maximum percentage base jitter ?' /
     1            '  ( i.e. width of ', a, ' bin )      ', $ )
              base_jitter = Qresp( STD_IN, 0.0, 30.0, MAX_TRIES ) / 100.
              averaged = .FALSE.
c
c           6) abscissa scale
c
          else if ( item .eq. 6 ) then                                    2m
              abs_scale = mod( abs_scale, 2 ) + 1
c
c           7) Ordinate scale
c
          else if ( item .eq. 7 ) then                                    2m
              ord_scale = mod( ord_scale, 2 ) + 1
c
c           8-10 calculate ensemble average if not done and output is required
c           ******************************************************************
c
          else if ( 8 .le. item .and. item .le. 10 ) then                 2m
              if ( .not. averaged ) then                                  3b
                write( STD_OUT, 6 )
6               format( ' Calculating ensemble average' )
c
c           initialize the sums
                  n_bins = 0
                  do 10 bin_ptr = 1, MAX_BINS                             4b
                      base(bin_ptr)    = 0.0
                      counts(bin_ptr)  = 0
                      mean(bin_ptr)    = 0.0
                      std_dev(bin_ptr) = 0.0
                      min_cbd(bin_ptr) =  1.0e36
                      max_cbd(bin_ptr) = -1.0e36
10                    continue                                            4e
c
c           deal with every item in the results list
c
                  do 13 i_point = 1, n_points                             4b
c
c           search for the base value in results in "base()" => bin_ptr
c
                      do 11 bin_ptr = 1, n_bins                           5b
                          if ( abs( results(i_point,abscissa) /
     1                         base(bin_ptr) - 1 ) .lt. base_jitter )
     2                        goto 12
11                        continue                                        5e
c
c           if it is not found, increment the number of bins "n_bins", add
c           the base value to "base" and "bin_ptr=n_bins"
c
                      n_bins = n_bins + 1
c                     bin_ptr = n_bins
                      base(n_bins) = results(i_point,abscissa)
c
c           accumulate totals
c           this is where we land if the base value is "found"
c
12                    counts(bin_ptr) = counts(bin_ptr) + 1
                      mean(bin_ptr) = mean(bin_ptr) +
     1                    results(i_point,ordinate)
                      std_dev(bin_ptr) = std_dev(bin_ptr) +
     1                    results(i_point,ordinate) **2
                      min_cbd(bin_ptr) =  min( min_cbd(bin_ptr),
     1                    results(i_point,CBODY_DYN) )
                      max_cbd(bin_ptr) =  max( max_cbd(bin_ptr),
     1                    results(i_point,CBODY_DYN) )
13                    continue                                            4e
c
c           calculate average & standard deviation of the average
c
                  do 14 bin_ptr = 1, n_bins                               4b
                      if ( counts(bin_ptr) .gt. 1 ) then                  5b
                          mean(bin_ptr) = mean(bin_ptr) /
     1                        real( counts(bin_ptr) )
                          std_dev(bin_ptr) = sqrt( max( 0.0,
     1                        ( std_dev(bin_ptr) - counts(bin_ptr) *
     2                          mean(bin_ptr) **2 ) /
     3                        real( counts(bin_ptr) - 1 ) ) )
                      else                                                5m
                          std_dev(bin_ptr) = 0.0
                          end if                                          5e
14                    continue                                            4e
                  averaged = .TRUE.
                  end if                                                  3e
c
c               8) tabulate ensemble average
c               ****************************
c
              if ( item .eq. 8 ) then                                     3b
                  out_lu = tt_open_file( out_file )
                  if ( out_lu .gt. 0 ) then                               4b
                      abscis_str = name_res(abscissa)
                      write( out_lu, 20 ) name_res(ordinate),
     1                    abscis_str(:8), abscis_str(9:)
20                    format( ' ;' /
     1                    ' ; ensemble averaged ', a / ' ; ' /
     2    ' ;   ', a8, '-    count  mean      standard   %std. dev/' /
     3    ' ;   ', a8, '                      deviation    of mean' )
                      do 22 bin_ptr = 1, n_bins                           5b
                          if ( counts(bin_ptr) .gt. 0 ) then              6b
                              write( out_lu, 21 ) bin_ptr,
     1                            base(bin_ptr),
     2                            int( counts(bin_ptr) + 0.5 ),
     3                            mean(bin_ptr), std_dev(bin_ptr),
     4                            100.0 * std_dev(bin_ptr) /
     5              mean(bin_ptr) / sqrt( real( counts(bin_ptr) ) )
21                            format( i4, g13.6e1, i3, 3g13.6e1 )
                              end if                                      6e
                          if ( out_lu .eq. STD_OUT .and.
     1                            ( mod( bin_ptr, 16 ) .eq. 0 .or.
     2                              bin_ptr .eq. n_bins ) ) then          7b
                              call ilgwait()
                              if ( bin_ptr .ne. n_bins )
     1                            write( out_lu, 20 )
                              end if                                      7e
22                        continue                                        5e
                      end if                                              4e
c
c               9) plot ensemble average
c               ************************
c
              else if ( item .eq. 9 ) then                                3m
                  call ilgopen( capt_file )
                  call ilgraph( 1, 1, 27, 27, 1 )
                  call ilgslaw( Cscale(abs_scale), Cscale(ord_scale) )
                  call ilgline( 1, 0, 1 )
                  call ilgdata( base, mean, 1, n_bins, 1 )
                  call ilgxlab( name_res(abscissa) )
                  call ilgylab( name_res(ordinate) )
                  call ilgtitl( 'Ensemble averaged results' )
                  call ilgwait()
                  call ilgclos()
c
c               10) Write calibration file records
c                  (Without fitted polynomials)
c               **********************************
c $ <plug_number> <trip_diameter> <min_cb_dyn_press> <max_cb_dyn_press> <n>
c <polynomial_coeff(0)> ... <polynomial_coeff(n)>
c
              else if ( item .eq. 10 ) then                               3b
                  if ( abscissa .ne. PLUG_NO ) then                       4m
                      write( STD_ERR, 30 ) 'base', name_res(PLUG_NO)
30                    format( ' TSQQ3 : ABORT : ', a,
     1                        ' variable must be ', a )
                      call ilgwait()
                  else if ( ordinate .ne. PRES_RATIO ) then               4m
                     write( STD_ERR, 30 ) 'averaged',
     1                   name_res(PRES_RATIO)
                     call ilgwait()
                  else                                                    4m
                      out_lu = tt_open_file( out_file )
                      write( out_lu, 31 )
31                    format(
     1                ' ;plug trip  min c/body  max c/body  polynomial'/
     2                ' ; no. diam  dyn. press  dyn. press  order' /
     3                ' ;--------polynomial-coefficients--------' )
                      do 33 bin_ptr = 1, n_bins                           5b
                          write( out_lu, 32 ) int( base(bin_ptr) + 0.5),
     1                        trip_diam, min_cbd(bin_ptr) / margin_cbd,
     2                        max_cbd(bin_ptr)*margin_cbd, mean(bin_ptr)
32                        format( ' $  ', i2, f6.2, 2g12.5e1, '   0' /
     1                        g16.7 )
33                        continue                                        5e
                      if ( out_lu .eq. STD_OUT )
     1                    call ilgwait()
                      end if                                              4e
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE DO_ANALYSIS( RESULTS, TOTAL_POINTS, AMP_OFFSET,
     1                        OUT_FILE )
c
c       Analyse the data collected from input file .
c
c Arguments
c  real      results(MAX_DATA,MAX_VARS)  # data & results array
c  integer   total_points   # number of data points
c  real      amp_offset     # amplifier offset
c  character out_file*(*)   # name of output file
c
c constants
c  FEET_TO_METRES  = 0.3048
c  TORR_TO_PASCALS = 133.322
c
c local variables
c  logical
c   written_table # table of results and data has been written
c   Equery        # function : read boolean
c   tt_tabulate   #          : tabulate data and results
c  integer
c   index         # row index in results array
c   item          # selected menu item
c   n_points      # number of data points in results array
c   out_len       # length of output file name
c   Jresp         # function : read integer
c   str_length    #          : non-blank string length
c  real
c   margin_cbd    # margin in permitted pre-centrebody dynamic pressure
c   trip_diam     # diameter of boundary layer trip (mm)
c   QBAR_DISPLtoP # function : convert pressure sensor output to Torr
c   Qresp         #          : read real
c  subroutine
c   cresp         # read string
c   newscr        # clear screen
c   sortsh        # sort array of data
c   tt_average    # ensemble average of results
c   tt_edit       # edit data
c   tt_plot       # plot data
c   tt_poly_fit   # fit polynomial to data
c
      include 'stdio.def'
      include 'ttsqq3.def'
      real FEET_TO_METRES, TORR_TO_PASCALS
      parameter( FEET_TO_METRES=0.3048, TORR_TO_PASCALS=133.322 )
c
      logical written_table, got_diam, Equery, tt_tabulate
      integer total_points, index, item, n_points, out_len,
     1        Jresp, str_length
      real    results(MAX_DATA,MAX_VARS), amp_offset,
     1        margin_cbd, trip_diam, QBAR_DISPLtoP, Qresp
      character out_file*(*)
      save got_diam, margin_cbd, trip_diam
      data got_diam / .FALSE. /
      data margin_cbd, trip_diam / 1.2, 0.0 /
c
c
c       Compile the data table
c
1     do 2 index = 1, total_points                                        1b
          results(index,STREAM_X) = FEET_TO_METRES *
     1        results(index,PLUG_NO) - 0.164
c         free stream dynamic pressure in test section
          results(index,TS_DYN_PR) = ( results(index,MEAN_VOLTS) -
     1        amp_offset ) / results(index,AMPL_GAIN) * TORR_TO_PASCALS
c         pre-centrebody dynamic pressure
          results(index,CBODY_DYN) = QBAR_DISPLtoP(
     1        results(index,CBODY_DYN) ) * TORR_TO_PASCALS
          results(index,PRES_RATIO) = results(index,TS_DYN_PR) /
     1        results(index,CBODY_DYN)
2         continue
c
c       Now we have done the data compilation, we can start the analysis
c
      n_points = total_points
      written_table = .false.
c
c     start of menu loop                                                  1b
3         out_len = max( 1, str_length( out_file ) )
          call newscr()
          write( STD_OUT, 4 ) out_file(:out_len), trip_diam, margin_cbd
4         format( ' Select' / '   0) Exit' /
     1        '   1) Tabulation file  ', a /
     2        '   2) Trip diameter    ', f6.3, ' mm' /
     3        '   3) C''b dyn. margin ', f6.3 /
     4        '   4) Tabulate results' /
     5        '   5) Plot results' /
     6        '   6) Edit data' /
     7        '   7) Ensemble average **' /
     8        '   8) Fit polynomial   **' /
     9        '   9) Shift (offset) data' /
     a        '  10) Restore data' /
     b        '  ** = write calibration file ', $ )
          item = Jresp( STD_IN, 0, 10, MAX_TRIES )
c
c           0) exit
c           *******
c
          if ( item .eq. 0 ) then                                         2b
c
c           write run results onto file
c
              if ( .not. written_table ) then                             3b
                  if ( Equery( STD_OUT, 'Tabulate data & results ? ' )
     1                    ) then                                          4b
                      written_table =
     1                    tt_tabulate( results, n_points, out_file )
                      end if                                              4e
                  end if                                                  3e
              return
c
c           1) Tabulation file
c           ******************
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Name of tabulation file ? ', $ )
              call cresp( STD_IN, out_file, MAX_TRIES )
c
c           2) Trip diameter (mm)
c           *********************
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' What is the diameter of the boundary layer ',
     1            'trip ? (in mm) ', $ )
              trip_diam = Qresp( STD_IN, 0.5, 10.0, MAX_TRIES )
              got_diam = .TRUE.
c
c           3) Precentrebody dynamic pressure margin
c           ****************************************
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 7 )
7             format( ' What is margin in permitted pre-centrebody ',
     1            'dynamic pressure ? (percent) ', $ )
              margin_cbd = Qresp( STD_IN, 0.0, 1000.0, MAX_TRIES )
c
c           4) Tabulate results
c           *******************
c
          else if ( item .eq. 4 ) then                                    2m
              written_table = tt_tabulate( results, n_points, out_file)
c
c           5) Plot results
c           ***************
c
          else if ( item .eq. 5 ) then                                    2m
              call tt_plot( results, n_points )
c
c           6) Edit data
c           ************
c
          else if ( item .eq. 6 ) then                                    2m
              call tt_edit( results, n_points )
c
c           7) Ensemble average
c           *******************
c
          else if ( item .eq. 7 ) then                                    2m
              if ( got_diam ) then
                  call tt_average( results, n_points, out_file,
     1                             trip_diam, margin_cbd )
              else                                                        3m
                  write( STD_OUT, 8 )
8                 format(' TTSQQ3 : ABORT : Must specify trip diameter')
                  call ilgwait()
                  end if                                                  3e
c
c           8) Fit polynomial to data
c           *************************
c
          else if ( item .eq. 8 ) then                                    2m
              if ( got_diam ) then                                        3b
                  call tt_poly_fit( results, n_points, out_file,
     1                              trip_diam, margin_cbd )
              else                                                        3m
                  write( STD_OUT, 8 )
                  call ilgwait()
                  end if                                                  3e
c
c           9) Shift (offset) data
c              recompile the data table after doing this
c           ********************************************
c
          else if ( item .eq. 9 ) then                                    2m
              call tt_offset( results, n_points )
              goto 1
c
c           10) Restore data
c           ****************
c
          else if ( item .eq. 10 ) then                                   2m
              n_points = total_points
              call sortsh( results, total_points, 1,
     1            MAX_DATA, SEQUENCE, MAX_VARS )
              end if                                                      2e
          goto 3
c         end of menu loop                                                1e
      end



      SUBROUTINE TT_OFFSET( RESULTS, N_POINTS )
c
c Offset one of the data columns -- typically static pressure .
c
c Arguments
c  real results(MAX_DATA,MAX_VARS)  # data & results array
c  integer n_points                 # number of data points
c
c local variables
c  integer
c   column              # results column index for abscissa
c   i                   # looping index
c   item                # selected menu item
c   Jresp               # function : read integer
c  real
c   off_accum(MAX_VARS) # accumulated offset for each variable
c   off_increm          # incremental offset
c   off_total           # required total offset
c   Qresp               # function : read real
c  character
c   name_res*16         # function : result name
c  subroutine
c   newscr              # clear page
c
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      integer n_points, column, i, item, Jresp
      real    results(MAX_DATA,MAX_VARS), off_accum(MAX_VARS),
     1        off_increm, off_total, Qresp
      character name_res*16
c
      save column, off_increm, off_total, off_accum
      data column / TS_DYN_PR /
      data off_increm, off_total, off_accum /
     1     0.0,        0.0,       MAX_VARS*0.0 /
c
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(column), off_increm, off_total,
     1        name_res(column), off_accum(column)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Offset variable         ', a /
     2        '   2) Incremental offset      ', g12.5 /
     3        '   3) Total required offset   ', g12.5 /
     4        '   4) Clear accumulated offset' /
     5        '   5) Apply incremental offset' /
     6        '   6) Apply total offset' /
     7        '      Accumulated offset for ', a, ' : ', g12.5, ' ', $ )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c           Exit
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           Offset variable
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select offset variable' / ( 2x, i2, ') ', a ) )
              column = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
c
c           Incremental offset
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' What is the incremental offset ? ', $ )
              off_increm = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES )
c
c           Total offset
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' What is the required total offset ? ', $ )
              off_total = Qresp( STD_IN, -100.0, 100.0, MAX_TRIES )
c
c           Clear accumulated offset
c
          else if ( item .eq. 4 ) then                                    2m
              off_accum(column) = 0.0
c
c           Apply incremental offset
c
          else if ( item .eq. 5 .or. item .eq. 6 ) then                   2m
              if ( item .eq. 5 ) then                                     3b
                  do 6 i = 1, n_points                                    4b
                      results(i,column) = results(i,column) + off_increm
6                     continue                                            4e
                  off_accum(column) = off_accum(column) + off_increm
c
c           Apply total offset
c
              else if ( item .eq. 6 ) then                                3m
                  do 7 i = 1, n_points                                    4b
                      results(i,column) = results(i,column) +
     1                    off_total - off_accum(column)
7                     continue                                            4e
                  off_accum(column) = off_total
                  end if                                                  3e
c
c       Recalculate PRES_RATIO column of the data table if the adjusted
c       variable is CBODY_DYN or TS_DYN_PR .
c
              if ( column .eq. CBODY_DYN .or.
     1             column .eq. TS_DYN_PR ) then                           3b
                  do 9 i = 1, n_points                                    4b
                      results(i,PRES_RATIO) =
     1                    results(i,TS_DYN_PR) / results(i,CBODY_DYN)
9                     continue                                            4e
                  end if                                                  3e
c
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE TT_POLY_FIT( RESULTS, N_POINTS, OUT_FILE,
     1                        TRIP_DIAM, MARGIN_CBD )
c
c Fit a polynomial to the selected abscissa-ordinate columns in the results
c array.
c
c Arguments
c  real results(MAX_DATA,MAX_VARS) # data & results array
c  integer n_points                # number of data points
c  real      trip_diam             # diameter of trip balls
c            margin_cbd            # margin in permitted pre-centrebody
c                                  # dynamic pressure expressed as a
c                                  # factor greater than one
c
c constants
c   TINY = 1.0e-5
c local variables
c  logical
c   fitted      # fitted polynomial has been calculated
c   apply_transform # function : apply transformation to data array
c  integer
c   abscissa    # results column index for abscissa
c   ordinate    # results column index for ordinate
c   abs_scale   # abscissa scaling  --  1 = 'lin', 2 = 'log'
c   ord_scale   # ordinate scaling  --  1 = 'lin', 2 = 'log'
c   capt_len    # length of capture file name
c   i           # looping index
c   i_data      # data array index
c   item        # selected menu item
c   n_lines     # number of table lines printed since the last <RETURN>
c   out_len     # length of output file name
c   out_lu      # logical unit number
c   plug_number # wind tunel plug number written to calibration file
c   poly_order  # order of fitted polynomial
c   table_len   # number of tabulated or plotted points
c   Jresp       # function : read integer
c   tt_open_file#          : open output file
c   str_length  #          : non-blank string length
c  real
c   abs_xform(3)# abscissa transformation
c   fit_error   # rms. fitting error
c   max_cbd     # maximum pre-centrebody dynamic pressure
c   min_cbd     # minimum pre-centrebody dynamic pressure
c   ord_xform(3)# ordinate transformation
c   poly_coeffs(MAX_ORDER)  # polynomial coefficients
c   x(MAX_DATA) # transformed abscissa data
c   y(MAX_DATA) # transformed ordinate data
c   x_table(MAX_TABLE)      # transformed abscissa values for table & plotting
c   y_table(MAX_TABLE)      # transformed ordinate values for table & plotting
c   x_mark      # threshold value for tabulating a data value
c   y_poly      # temporary variable -- evaluated polynomial
c   qpoly       # function : evaluate polynomial
c   x_step      # abscissa step for tabe and graph
c   rev_transform           # function : invert data transformation
c  double precision
c   work(MAX_ORDER,2)       # work array for SPYORTH
c  character
c   capt_file*64# name of capture file
c   cscale(2)*3 # 'lin', 'log'
c   out_file*(*)# name of output file
c   name_res*16 # function : result name
c  subroutine
c   cresp       # read string
c   define_transform # define transformation for data
c   ilgclos, ilgdata, ilgline, ilgopen, ilgwait, ilgxlab, ilgylab
c   newscr      # clear page
c   spyorths    # calculate curve fitting polynomial
c   sortsh      # sort data array
c
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
c
      real TINY
      parameter( TINY=1.0e-5 )
c
      logical fitted, apply_transform
      integer n_points, abscissa, ordinate, abs_scale, ord_scale,
     1        capt_len, i, i_data, item, n_lines, out_len, out_lu,
     2        plug_number, poly_order, table_len,
     3        Jresp, tt_open_file, str_length
      real results(MAX_DATA,MAX_VARS), trip_diam, margin_cbd,
     1     abs_xform(3), fit_error, max_cbd, min_cbd, ord_xform(3),
     2     poly_coeffs(MAX_ORDER), x(MAX_DATA), y(MAX_DATA),
     3     x_table(MAX_TABLE), y_table(MAX_TABLE), x_mark, x_step,
     4     y_poly, qpoly, rev_transform
      double precision work(MAX_ORDER,2)
      character capt_file*64, Cscale(2)*3, out_file*(*), name_res*16
c
      save abscissa, ordinate, abs_scale, ord_scale, out_len, capt_len,
     1     poly_order, table_len,
     2     abs_xform, ord_xform, capt_file, Cscale
      data abscissa, ordinate, abs_scale, ord_scale, out_len, capt_len,
     1     poly_order, table_len /
     1     PLUG_NO, MEAN_VOLTS, 1, 1, 1, 1, 3, MAX_TABLE /
      data abs_xform, ord_xform /
     1     TRANS_LOG_10, 1.0, 0.0, TRANS_IDENTITY, 1.0, 0.0 /
      data capt_file, Cscale / ' ', 'lin', 'log' /
c
c
      fitted = .FALSE.
      out_len = max( 1, str_length( out_file ) )
      capt_len = max( 1, str_length( capt_file ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) name_res(abscissa), name_res(ordinate),
     1        out_file(:out_len), capt_file(:capt_len),
     2        Cscale(abs_scale), Cscale(ord_scale),
     3        table_len, poly_order
2         format( ' Select' / '   0) Exit' /
     1        '   1) Abscissa', 12x, a /
     2        '   2) Ordinate', 12x, a /
     3        '   3) Tabulation file     ', a /
     4        '   4) Capture file        ', a /
     5        '   5) Abscissa scale      ', a /
     6        '   6) Ordinate scale      ', a /
     7        '   7) Plotted points / table length ', i3 /
     8        '   8) Polynomial order               ', i2 /
     9        '   9) Specify abscissa transformation' /
     a        '  10) Specify ordinate transformation' /
     b        '  11) set to defaults' /
     c        '  12) Print polynomial' /
     d        '  13) Tabulate polynomial' /
     e        '  14) Plot polynomial' /
     f        '  15) Write calibration file record' /
     g        '      (Including fitted polynomial) ', $ )
          item = Jresp( STD_IN, 0, 15, MAX_TRIES )
c
c           0) Exit
c           *******
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           1) Abscissa
c           ***********
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) 'abscissa',
     1            ( i, name_res(i), i = 1, MAX_VARS )
3             format( ' Select ', a / ( 2x, i2, ') ', a ) )
              abscissa = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              fitted = .FALSE.
c
c           2) Ordinate
c           ***********
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 ) 'ordinate',
     1            ( i, name_res(i), i = 1, MAX_VARS )
              ordinate = Jresp( STD_IN, 1, MAX_VARS, MAX_TRIES )
              fitted = .FALSE.
c
c           3) Tabulation file
c           ******************
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Name of tabulation file ? ', $ )
              call cresp( STD_IN, out_file, MAX_TRIES )
              out_len = max( 1, str_length( out_file ) )
c
c           4) Capture file
c           ***************
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Name of capture file ? ', $ )
              call cresp( STD_IN, capt_file, MAX_TRIES )
              capt_len = max( 1, str_length( capt_file ) )
c
c           5) Abscissa scale
c           *****************
c
          else if ( item .eq. 5 ) then                                    2m
              abs_scale = mod( abs_scale, 2 ) + 1
c
c           6) Ordinate scale
c           *****************
c
          else if ( item .eq. 6 ) then                                    2m
              ord_scale = mod( ord_scale, 2 ) + 1
c
c           7) Table length
c           ***************
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Number of plotted points / table length ? ', $ )
              table_len = Jresp( STD_IN, 0, MAX_TABLE, MAX_TRIES )
c
c           8) Polynomial order
c           *******************
c
          else if ( item .eq. 8 ) then                                    2m
              write( STD_OUT, 7 )
7             format( ' What is the polynomial order ', $ )
              poly_order = Jresp( STD_IN, 1, 12, MAX_TRIES )
              fitted = .FALSE.
c
c           9) Specify abscissa transformation
c           **********************************
c
          else if ( item .eq. 9 ) then                                    2m
              call define_transform( abs_xform )
              fitted = .FALSE.
c
c           10) Specify ordinate transformation
c           ***********************************
c
          else if ( item .eq. 10 ) then                                   2m
              call define_transform( ord_xform )
              fitted = .FALSE.
c
c           11) set to defaults
c           *******************
c
          else if ( item .eq. 11 ) then                                   2m
              abs_xform(TRANS_TRAN) = TRANS_LOG_10
              abs_xform(TRANS_GAIN) = 1.0
              abs_xform(TRANS_OFFS) = 0.0
              ord_xform(TRANS_TRAN) = TRANS_IDENTITY
              ord_xform(TRANS_GAIN) = 1.0
              ord_xform(TRANS_OFFS) = 0.0
              abscissa = CBODY_DYN
              ordinate = PRES_RATIO
              abs_scale = 2
              ord_scale = 1
              fitted = .FALSE.
c
c           12 -- 15 Calculate polynomial
c           *****************************
c
          else if ( 12 .le. item .and. item .le. 15 ) then                2m
              if ( .not. fitted ) then                                    3b
                  write( STD_OUT, 11 )
11                format( ' Calculating polynomial' )
c                 first sort the abscissa data into ascending order
                  call sortsh( results, n_points, 1, MAX_DATA,
     1                         abscissa, MAX_VARS )
                  if ( .not. apply_transform( results(1,abscissa), x,
     1                    n_points, abs_xform ) ) then                    4b
                      write( STD_ERR, 12 ) name_res(abscissa)
12                    format( ' TSQQ3 : ABORT : Could not apply ',
     1                    'transformation to ', a )
                      call ilgwait()
                  else if ( .not. apply_transform( results(1,ordinate),
     1                    y, n_points, ord_xform ) ) then                 4m
                      write( STD_ERR, 12 ) name_res(ordinate)
                      call ilgwait()
                  else                                                    4m
c                     first sort the abscissa data into ascending order
                      call sortsh( results, n_points, 1, MAX_DATA,
     1                             abscissa, MAX_VARS )
                      call spyorths( x, y, n_points, poly_coeffs,
     1                               poly_order, fit_error, work )
                      fit_error = sqrt( abs( fit_error ) )
                      min_cbd = results(1,CBODY_DYN)
                      max_cbd = results(1,CBODY_DYN)
                      do 13 i = 1, n_points                               5b
                          min_cbd = min( min_cbd, results(i,CBODY_DYN) )
                          max_cbd = max( max_cbd, results(i,CBODY_DYN) )
13                        continue                                        5e
                      i = 1
                      x_step =  ( x(n_points) - x(1) ) /
     1                          real( table_len - 1 )
                      do 14 x_mark = x(1), x(n_points), x_step
                          x_table(i) = x_mark
                          y_table(i) = qpoly( x_mark,
     1                        poly_coeffs, poly_order )
                          i = i + 1
14                        continue
                      fitted = .TRUE.
                      end if                                              4e
                  end if                                                  3e
c
c           12) Print polynomial
c           ********************
c
              if ( item .eq. 12 ) then                                    3b
                  out_lu = tt_open_file( out_file )
                  if ( out_lu .gt. 0 ) then                               4b
                      write( out_lu, 21 ) name_res(ordinate),
     1                    poly_order, name_res(abscissa), fit_error,
     2                    ( i, poly_coeffs(i), i = 1, poly_order + 1 )
21                    format( 1x, a, ' as a ', i2,
     1                    ' order polynomial of ', a /
     2                    ' rms. fitting error      : ', g13.6 /
     2                    ' polynomial coefficients :' /
     3                    ( 1x, i2, 3x, g15.8 ) )
                      if ( out_lu .eq. STD_OUT )
     1                    call ilgwait()
                  else                                                    4m
                      call ilgwait()
                      end if                                              4e
c
c           13) Tabulate polynomial
c           ***********************
c
              else if ( item .eq. 13 ) then                               3m
c
c                 open output file
c                 print table header
c
                  out_lu = tt_open_file( out_file )
                  if ( out_lu .gt. 0 ) then                               4b
                      write( out_lu, 31 )
     1                    name_res(abscissa), name_res(ordinate)
31                    format( '       -------abscissa-------  -------',
     1                    'ordinate-------     fitting     fitting' /
     2                    10x, a16, 8x, a16, 7x, 'polynomial    error' /
     3                    6x, 2( '    raw      transformed' ) )
c
c                     print the table, once round the loop for each row
c                     in addition (only unique) values of the data are printed
c                     -- one value between each table row
c
                      x_mark = x_table(1) * ( 1.0 - TINY )
                      i_data = 1
                      n_lines = 1
                      do 35 i = 1, table_len                              5b
c                         print data value if the table has gone past it
                          if ( i_data .le. n_points ) then                6b
                              if ( x(i_data) .le. x_mark ) then           7b
                                  y_poly = qpoly( x(i_data),
     1                                poly_coeffs, poly_order )
                                  write( out_lu, 32 ) i_data,
     1                              results(i_data,abscissa), x(i_data),
     2                              results(i_data,ordinate), y(i_data),
     3                              y_poly, y(i_data) - y_poly
32                                format( ' (', i3, ')', 6g12.5e1 )
                                  n_lines = n_lines + 1
c                                 skip past repeated data values
33                                if ( i_data .le. n_points ) then        8b
                                      if ( x(i_data) .le. x_mark ) then   9b
                                          i_data = i_data + 1
                                          goto 33
                                          end if                          9e
                                      end if                              8e
                                  end if                                  7e
                              end if                                      6e
c                         print the table row
                          write( out_lu, 34 ) i,
     1                        rev_transform( x_table(i), abs_xform ),
     2                        x_table(i),
     3                        rev_transform( y_table(i), ord_xform ),
     4                        y_table(i)
34                        format( 2x, i3, 1x, 3g12.5e1, 12x, g12.5e1 )
                          n_lines = n_lines + 1
                          x_mark = x_table(i) + x_step
c                         wait for user if screen is full or finished
                          if ( out_lu .eq. STD_OUT .and.
     1                         ( n_lines .ge. 20 .or.
     2                           i .eq. n_points ) ) then                 6b
                              call ilgwait()
                              n_lines = 1
                              if ( i .lt. table_len )
     1                            write( out_lu, 31 )
     2                            name_res(abscissa), name_res(ordinate)
                              end if                                      6e
35                        continue                                        5e
                  else                                                    4m
                      call ilgwait()
                      end if                                              4e
c
c           14) Plot polynomial
c           *******************
c
              else if ( item .eq. 14 ) then                               3m
                  call ilgopen( capt_file )
                  call ilgraph( 1, 1, 11, 11, 1 )
                  call ilgslaw( Cscale(abs_scale), Cscale(ord_scale) )
                  call ilgline( 1, 0, 1 )
                  call ilgdata( x, y, 1, n_points, 1 )
                  call ilgline( 0, 1, 1 )
                  call ilgdata( x_table, y_table, 1, table_len, 0 )
                  call ilgylab( 'transformed ' // name_res(ordinate) )
                  call ilgxlab( 'transformed ' // name_res(abscissa) )
                  call ilgwait()
                  call ilgclos()
c
c           15) Write calibration file record
c              (Including fitted polynomial)
c           *********************************
c
              else if ( item .eq. 15 ) then                               3m
c
c           check that the plug number is always the same
c
                  plug_number = results(1,PLUG_NO)
                  i = 2
41                if ( i .le. n_points ) then                             4b
                      if ( abs( plug_number - results(i,PLUG_NO) )
     1                        .lt. 0.5 ) then                             5b
                          i = i + 1
                          goto 41
                          end if                                          5e
                      end if                                              4e
                  if ( i .le. n_points ) then                             4b
                      write( STD_ERR, 42 )
42                    format( ' TSQQ3 : ABORT : Test section plug ',
     1                    'number has changed' )
                      call ilgwait()
c
c           abscissa must be Pre-centrebody dynamic pressure, and ordinate
c           must be test-section / pre-centrebody dynamic pressure ratio
c
                  else if ( abscissa .ne. CBODY_DYN ) then                4m
                      write( STD_ERR, 43 ) 'abscissa', name_res(PLUG_NO)
43                    format( ' TSQQ3 : ABORT : ', a,
     1                    ' variable must be ', a )
                      call ilgwait()
                  else if ( ordinate .ne. PRES_RATIO ) then               4m
                      write( STD_ERR, 43 ) 'ordinate',
     1                    name_res(PRES_RATIO)
                      call ilgwait()
c
c           must use log transformation on abscissa and identity
c           transformation on the ordinate
c
                  else if ( int( abs_xform(TRANS_TRAN) + 0.5 ) .ne.
     1                    TRANS_LOG_10 .or. abs(
     2                    abs_xform(TRANS_GAIN) - 1.0 ) .gt. TINY .or.
     3                    abs( abs_xform(TRANS_OFFS) ) .gt. TINY ) then   4m
                      write( STD_ERR, 44 ) 'abscissa'
44                    format( ' TSQQ3 : ABORT : Illegal ', a,
     1                    ' transformation. Use default' )
                      call ilgwait()
                  else if ( int( ord_xform(TRANS_TRAN) + 0.5 ) .ne.
     1                    TRANS_IDENTITY .or. abs(
     2                    ord_xform(TRANS_GAIN) - 1.0 ) .gt. TINY .or.
     3                    abs( ord_xform(TRANS_OFFS) ) .gt. TINY ) then   4m
                      write( STD_ERR, 44 ) 'ordinate'
                      call ilgwait()
                  else                                                    4m
c
c           Write the calibration record to file
c $ <plug_number> <trip_diameter> <min_cb_dyn_press> <max_cb_dyn_press> <n>
c <polynomial_coeff(0)> ... <polynomial_coeff(n)>
c
                      out_lu = tt_open_file( out_file )
                      if ( out_lu .gt. 0 ) then                           5b
                          write( out_lu, 45 )
45                        format(
     1                ' ;plug trip  min c/body  max c/body  polynomial'/
     2                ' ; no. diam  dyn. press  dyn. press  order' /
     3                ' ;--------polynomial-coefficients--------' )
                          write( out_lu, 46 ) plug_number,
     1                        trip_diam, min_cbd / margin_cbd,
     2                        max_cbd * margin_cbd, poly_order,
     3                        ( poly_coeffs(i), i = 1, poly_order + 1 )
46                        format( ' $  ', i2, f6.2, 2g12.5e1, i4 /
     1                        ( 5g16.7 ) )
                          if ( out_lu .eq. STD_OUT )
     1                        call ilgwait()
                          end if                                          5e
                      end if                                              4e
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE DEFINE_TRANSFORM( TRANSFORM )
c
c Defines a transformation to be applied to a data array . The transformation
c is of the form  X => Y .
c
c     Y = TRANSCENDENTAL( GAIN * X + OFFSET )
c
c Argument
c  real transform(3)  # array defining the transform
c
c  transform(1) = 1.0  -- TRANSCENDENTAL = IDENTITY
c               = 2.0  --                = RECIPROCAL
c               = 3.0  --                = LOG_base_10
c  transform(2) = GAIN
c  transform(3) = OFFSET
c
c local variables
c  integer
c   item          # selected menu item
c   trans_ptr     # specifies trancendental function
c   Jresp         # function : read integer
c  real
c   Qresp         # function : read real
c  character
c   transcend
c  subroutine
c   newscr
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      integer item, trans_ptr, Jresp
      real transform(3), Qresp
      character transcend(3)*11
c
      save transcend
      data transcend / 'identity   ', 'reciprocal ', 'log base 10' /
c
      trans_ptr = transform(TRANS_TRAN) + 0.5
      if ( trans_ptr .lt. TRANS_IDENTITY .or.
     1        trans_ptr .gt. TRANS_LOG_10 )
     2    trans_ptr = TRANS_IDENTITY
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) transcend( trans_ptr ),
     1        transform(TRANS_GAIN), transform(TRANS_OFFS)
2         format( ' Select' / '   0) Exit' /
     1        '   1) Clear' /
     2        '   2) Transcendental ', a /
     3        '   3) Gain           ', g12.6e1 /
     4        '   4) Offset         ', g12.6e1 /
     5        ' Y = TRANSCENDENTAL( GAIN * X + OFFSET )  ', $ )
          item = Jresp( STD_IN, 0, 4, MAX_TRIES )
c
c           0) Exit
c           *******
c
          if ( item .eq. 0 ) then                                         2b
              transform(TRANS_TRAN) = trans_ptr
              return
c
c           1) Clear
c           ********
c
          else if ( item .eq. 1 ) then                                    2m
              trans_ptr = TRANS_IDENTITY
              transform(TRANS_GAIN) = 1.0
              transform(TRANS_OFFS) = 0.0
c
c           2) Transcendental
c           *****************
c
          else if ( item .eq. 2 ) then                                    2m
              trans_ptr = mod( trans_ptr, 3 ) + 1
c
c           3) Gain
c           *******
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Gain ? ', $ )
              transform(TRANS_GAIN) =
     1            Qresp( STD_IN, -1.0e9, 1.0e9, MAX_TRIES )
c
c           4) Offset
c           *********
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Offset ? ', $ )
              transform(TRANS_OFFS) =
     1            Qresp( STD_IN, -1.0e9, 1.0e9, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      LOGICAL FUNCTION APPLY_TRANSFORM( IN_DATA, OUT_DATA, N_DATA,
     1                                  TRANSFORM )
c
c Applies a transformation to each of the N_DATA elements of the data array
c IN_DATA and places the result in the array OUT_DATA . The transformation
c is of the form  IN_DATA => OUT_DATA .
c
c     OUT_DATA(i) = TRANSCENDENTAL( GAIN * IN_DATA(i) + OFFSET )
c
c Argument
c  real    in_data(*)     # input array
c          out_data(*)    # output array
c  integer n_data         # number of data points
c  real    transform(3)   # defines the transformation from DEFINE_TRANSFORM
c
c  transform(1) = 1.0  -- TRANSCENDENTAL = IDENTITY
c               = 2.0  --                = RECIPROCAL
c               = 3.0  --                = LOG_base_10
c  transform(2) = GAIN
c  transform(3) = OFFSET
c
c local variables
c  integer
c    i            # data array index
c    trans_ptr    # specifies type of transcendental function
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      integer n_data, i, trans_ptr
      real    in_data(*), out_data(*), transform(3)
c
      trans_ptr = transform(TRANS_TRAN) + 0.5
      apply_transform = .FALSE.
c
c       no transcendental function
c
      if ( trans_ptr .eq. TRANS_IDENTITY ) then                           1b
          do 1 i = 1, n_data                                              2b
              out_data(i) = transform(TRANS_GAIN) * in_data(i) +
     1            transform(TRANS_OFFS)
1             continue                                                    2e
          apply_transform = .TRUE.
c
c       reciprocal
c
      else if ( trans_ptr .eq. TRANS_RECIPROC ) then                      1m
          do 2 i = 1, n_data                                              2b
              out_data(i) = 1.0 / ( transform(TRANS_GAIN) * in_data(i) +
     1            transform(TRANS_OFFS) )
2             continue                                                    2e
          apply_transform = .TRUE.
c
c       log_base_10
c
      else if ( trans_ptr .eq. TRANS_LOG_10 ) then                        1m
          do 3 i = 1, n_data                                              2b
              out_data(i) = log10( transform(TRANS_GAIN) * in_data(i) +
     1            transform(TRANS_OFFS) )
3             continue                                                    2e
          apply_transform = .TRUE.
c
c       unknown transcendental
c
      else
          do 4 i = 1, n_data                                              2b
              out_data(i) = 0.0
4             continue                                                    2e
          apply_transform = .FALSE.
          end if                                                          1e
      return
      end



      REAL FUNCTION REV_TRANSFORM( IN_DATA, TRANSFORM )
c
c Applies the reverse transformation to the value IN_DATA and returns the
c result as the value of REV_TRANSFORM .
c
c Argument
c  real in_data        # input array
c       transform(3)   # defines the transformation from DEFINE_TRANSFORM
c
c  transform(1) = 1.0  -- TRANSCENDENTAL = IDENTITY
c               = 2.0  --                = RECIPROCAL
c               = 3.0  --                = LOG_base_10
c  transform(2) = GAIN
c  transform(3) = OFFSET
c
c local variables
c  real
c    trans_ptr    # specifies type of transcendental function
c
      include 'stdio.def'
      include 'ttsqq3.def'
c
      integer trans_ptr
      real    in_data, transform(3)
c
      trans_ptr = transform(TRANS_TRAN) + 0.5
c
c       no transcendental function
c
      if ( trans_ptr .eq. TRANS_IDENTITY ) then                           1b
          rev_transform = ( in_data - transform(TRANS_OFFS) ) /
     1                    transform(TRANS_GAIN)
c
c       reciprocal
c
      else if ( trans_ptr .eq. TRANS_RECIPROC ) then                      1m
          rev_transform = ( 1.0 / in_data - transform(TRANS_OFFS) ) /
     1                    transform(TRANS_GAIN)
c
c       log_base_10
c
      else if ( trans_ptr .eq. TRANS_LOG_10 ) then                        1m
          rev_transform = ( 10.0 ** in_data - transform(TRANS_OFFS) ) /
     1                    transform(TRANS_GAIN)
c
c       unknown transcendental
c
      else
          rev_transform = 0.0
          end if                                                          1e
      return
      end

