c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

c ROUTINES FOR CALCULATING BOUNDARY-LAYER THICKNESS, DISPLACEMENT THICKNESS,
c MOMENTUM THICKNESS AND ENERGY THICKNESS FROM A MEAN VELOCITY DISTRIBUTION


      REAL FUNCTION QDELTA_NN( LOCATION, MEAN_SPEED, INCREM, N_POINTS,
     1                      EDGE_SPEED, WORK )
c
c         Calculate boundary layer thickness .
c Method : fit a polynomial over the outer part of the boundary layer profile
c          so that log_probe_location is a polynomial of mean_air_speed, and
c          and then evaluate the polynomial at the EDGE_SPEED .
c
c Arguments
c  real    location(*)  # array of distances from the wall
c          mean_speed(*)# array of mean air speed at each dist from wall
c  integer increm       # indexing increment
c          n_points     # number of data points in a velocity profile
c  real    edge_speed   # mean air speed at the edge of the boundary layer
c          work(*)      # work vector of length
c                       #   4 * n_points + 3 + 2 * DELTA_MAX_POLY
c
c local variables
c  logical
c   edelta_check_args        # function : check arguments
c  integer
c   dest_loc            # destination index for probe location
c   dest_vel            # destination index for mean air speed
c   source              # source index for probe location and air speed arrays
c  real
c   qcalc_nn            # function : calculates boundary layer thickness
c   qdelta_menu         #          : menu interface for thickness calculations
c  character
c   labels(DELTA_N_LABELS)*22 # routine name & graph labels
c
      include 'stdio.def'
      include 'bly_delt.def'
c
      logical edelta_check_args
      integer increm, n_points, dest_loc, dest_vel, source
      real location(*), mean_speed(*), edge_speed, work(*),
     1     qcalc_nn, qdelta_menu
      character labels(DELTA_N_LABELS)*22
c
      include 'bly_delt.cmn'
      save labels
      data labels /
     1    'QDELTA_NN',
     2    'boundary layer',
     3    '|~U (m/s)',
     4    'y (m)',
     5    'Data for B/L thickness' /
      external bly_delta_block, qcalc_nn
c
c
      qdelta_nn = -1.0
      if ( edelta_check_args( location, mean_speed, increm, n_points,
     1                      edge_speed, poly_order + 1,
     2                      labels(DELTA_CALLER ), 'EDGE_SPEED' ) ) then  1m
c
c       transfer data to work array,
c
          work( n_points * 2 + 1 ) = edge_speed
          dest_loc = 1
          dest_vel = n_points + 1
          do 2 source = 1, increm * ( n_points - 1 ) + 1, increm          2b
              work(dest_loc) = location(source)
              work(dest_vel) = mean_speed(source)
              dest_loc = dest_loc + 1
              dest_vel = dest_vel + 1
2             continue                                                    2e
c
c       edit the data if enabled; otherwise simply evaluate the result
c
          if ( edit_enable ) then                                         2b
              qdelta_nn = qdelta_menu( work, n_points,
     1                                 qcalc_nn, labels )
          else                                                            2m
              qdelta_nn = qcalc_nn( work, n_points, n_points )
              end if                                                      2e
          end if                                                          1e
c
      return
      end



      REAL FUNCTION QCALC_NN( DATA_ARRAY, N_DATA, N_ROWS )
c
c         Calculate boundary layer thickness .
c Method : fit a polynomial over the outer part of the boundary layer profile
c          so that log_probe_location is a polynomial of mean_air_speed, and
c          and then evaluate the polynomial at the EDGE_SPEED .
c
c Arguments
c  real data_array(*) # data array containing           # starting index
c            # INPUT  #  probe position data            #  1
c            # INPUT  #  mean air speed data            #  n_rows + 1
c            # INPUT  #  air speed at which the edge of #
c            #        #   the boundary layer is located #  2 * n_rows + 1
c            # OUTPUT #  number of fitted polynomial    #  2 * n_rows + 2
c            #        #   coordinates following         #
c               -- signals that further processing can be done by the caller
c            # OUTPUT #  fitted polynomial probe loc'ns #  2 * n_rows + 3
c            # OUTPUT #  fitted polynomial air speed    #  3 * n_rows + 3
c                     #   A total array size of
c                     #   4 * n_rows + 3 + 2 * DELTA_MAX_POLY is required .
c
c  integer n_data     # number of profile data coordinates actually used
c          n_rows     # number of data points available i.e. array size
c
c constants
c  LOC_INDEX=1        # start index of probe locations in DATA_ARRAY
c
c local variables
c  logical
c   got_crit          # flag for "selection criterion found or extracted"
c   got_error         # flag -- error has been detected
c  integer
c                     # data_array start indices for
c   vel_index         #   mean air speed
c   edge_index        #   EDGE_SPEED
c   nd_y_index        #   nondimensional probe location
c   nd_v_index        #   nondimensional air speed
c   weig_index        #   weights vector for polynomial fitting
c   work_index        #   work array for SPYORTH
c   count             # number of iterations
c   destin            # destination index for mean air speeds
c   source            # source index for probe locations or selection criteria
c   end_fit           # end of the profile data used for polynomial fitting
c   start_fit         # start of the profile data used for polynomial fitting
c   edge_search       # index of profile data interval containing the EDGE_SPEED
c   fit_max(DELTA_SEL_NNN)  # index of last fitted point for selection criteria
c   fit_min(DELTA_SEL_NNN)  # index of first fitted point for selection criteria
c   non_increasing    # number of non-increasing data points in the profile
c   n_fitted          # number of the profile data points used in polynomial fit
c   n_search          # number of data points remaining after skipping data
c                     #   effected by wall proximity
c   skip_search       # no. of data points ignored due to wall proximity effect
c   Jlsrch            # function : linear search
c  real
c                        # for iterative solution of polynomial :
c   delta_min, delta_max #   abscissa (probe location) interval bounds
c   speed_min, speed_max #   ordinate (air speed polynomial) interval bounds
c   denom                #   part of iteration calculation -- done separately
c                        #       to avoid divide-by-zero
c   fitted_poly(DELTA_MAX_POLY) # coefficients of fitted polynomial
c   location          # probe location values for evaluating fitted polynomial
c   loc_step          # increment in air speed value for fitted polynomial
c                     # references for non-dimensionizing :
c   nd_edge_speed     # non-dimensional speed defining edge of boundary layer
c   ref_locn          #   probe location
c   ref_speed         #   mean air speed
c   sum_sq_err        # sum square error of fitted polynomial
c   qpoly             # function : evaluate polynomial
c  subroutine
c   sortsh            # shell sort
c   spyorth           # fit polynomial to data
c   swap4             # swap two values
c
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer    LOC_INDEX
      parameter( LOC_INDEX=1 )
c
      logical got_crit, got_error
      integer n_data, n_rows, vel_index, edge_index, nd_y_index,
     1        nd_v_index, weig_index, work_index, count, destin, source,
     2        end_fit, start_fit, edge_search, fit_max(DELTA_SEL_NNN),
     3        fit_min(DELTA_SEL_NNN), non_increasing, n_fitted,
     4        n_search, skip_search, Jlsrch
      real    data_array(*), delta_min, delta_max, speed_min,
     1        speed_max, denom, fitted_poly(DELTA_MAX_POLY),
     2        location, loc_step, nd_edge_speed, ref_locn, ref_speed,
     3        sum_sq_err, qpoly
c
      include 'bly_delt.cmn'
c
c
c         START INDICES TO THE DATA ARRAY
c         for probe location, air speed, fitting weight and work array
c
      got_error = .FALSE.
      qcalc_nn   = -1.0
      vel_index  = LOC_INDEX + n_rows
      edge_index = vel_index  + n_rows
      nd_y_index = edge_index + 1
      nd_v_index = nd_y_index + n_data
      weig_index = nd_v_index + n_data
      work_index = weig_index + n_data
c
c
c         Sort the 2 columns of data containing probe location and air speed
c         into ascending order of probe location (1st column).
c
c          SORTSH( X, START_ROW, STOP_ROW, ROWS, SORT_COL, COLUMNS )
      call sortsh( data_array(LOC_INDEX), n_data, 1, n_rows, 1, 2 )
c
c         CHECK FOR WALL PROXIMITY EFFECT
c         If the air speed initially decreases as the probe is moved away from
c         the wall, it is assumed to be due to wall proximity effect, and
c         SKIP_SEARCH is adjusted so that the initially decreasing data is not
c         used for searches by JLSRCH .
c
      skip_search = 0
      if ( data_array(vel_index) .gt. data_array(vel_index+1) ) then      1b
          skip_search = 1
          source = vel_index + 2
c         start of loop                                                   2b
1         if ( source .lt. vel_index + n_data  .and.                      ..
     1            data_array(source-1) .gt. data_array(source) ) then
              skip_search = source - vel_index
              source = source + 1
              goto 1
              end if
c             end of loop                                                 2e
          end if                                                          1e
      if ( skip_search .gt. 0 ) then                                      1b
          write( STD_OUT, 2 ) skip_search
2         format( ' QCALC_NN : WARNING : Wall proximity effect ',
     1        'detected : ', i3, ' data points ignored' )
          got_error = .TRUE.
          end if                                                          1e
c
c         CHECK FOR NON-INCREASING DATA
c         Warn the user if velocity is non-increasing as the probe is moved
c         away from the wall . Do not include data influenced by wall
c         proximity.
c
      non_increasing = 0
      do 3 source = vel_index + skip_search + 1, vel_index + n_data - 1   1b
          if ( data_array(source-1) .gt. data_array(source) ) then        2b
              non_increasing = non_increasing + 1
              end if                                                      2e
3         continue                                                        1e
      if ( non_increasing .gt. 0 ) then                                   1b
          write( STD_OUT, 4 ) non_increasing
4         format( ' QCALC_NN : WARNING : Velocity profile is ',
     1        ' non-increasing at ', i4, ' points.' )
          got_error = .TRUE.
          end if                                                          1e
c
c         CHECK FOR INCONSISTENCIES IN THE DATA
c           i.e. edge speed must be within the range of profile air speeds
c                     -- extrapolation is dangerous & should be unnecessary
c                the number of data points must be greater than the order
c                     of the fitted polynomial by at least 2
c
      n_search = n_data - skip_search
      edge_search = Jlsrch( data_array( vel_index + skip_search ),
     1    1, n_search, data_array( edge_index ) ) + skip_search
      if ( edge_search .le. 0 .or. edge_search .ge. n_data ) then         1b
          write( STD_OUT, 5 ) edge_search,
     1        data_array(vel_index), data_array(vel_index-1+n_data)
5         format( ' QCALC_NN : ABORT : Edge velocity ', g12.5,
     2            ' is not within the profile' / 23x, 'velocity range ',
     3            g12.5, ' to ', g12.5 )
          got_error = .TRUE.
      else if ( n_data .lt. poly_order + 2 ) then                         1m
          write( STD_OUT, 6 ) n_data, poly_order
6         format( ' QCALC_NN : ABORT : ', i4, ' data points not ',
     1        'enough to fit order ', i2, ' polynomial' )
          got_error = .TRUE.
      else                                                                1m
c
c         Transform probe location and air_speed to nondimensional values
c         (1-Y/Y_ref) and ln(U_ref/U), and allocate the weights required
c         for fitting the polynomial
c
          ref_locn  = data_array(LOC_INDEX - 1 + edge_search)
          ref_speed = data_array(vel_index - 1 + edge_search )
          destin = nd_v_index
c         destin - n_data : for non-dimensional probe location
c         destin          : for non-dimensional mean air speed
c         destin + n_data : for work array
          do 7 source = 1, n_data                                         2b               1b
              data_array( destin - n_data) =
     1            2 - data_array(source) / ref_locn
              data_array( destin ) =
     1            log( ref_speed / data_array(source+n_rows) )
              data_array( destin + n_data ) = 1.0
              destin = destin + 1
7             continue                                                    2e
c
c       Select the region of the data to which the polynomial will be fitted
c       The selection criterion can be
c         a) limiting values of nondimensional probe location
c         b) limiting values of nondimensional air speed
c         c) number of selected points each side of the reference values
c
c       The first step is to determine the data array index for each limit
c       (minimum and maximum) for each criterion -- stored in FIT_MIN & FIT_MAX
c
c       location criterion
          fit_max( DELTA_SEL_LOC ) = skip_search +
     1        Jlsrch( data_array( nd_y_index + skip_search ),
     2                1, n_search, sel_min( DELTA_SEL_LOC ) )
          fit_min( DELTA_SEL_LOC ) = 1 + skip_search +
     1        Jlsrch( data_array( nd_y_index + skip_search ),
     2                1, n_search, sel_max( DELTA_SEL_LOC ) )
c       air speed criterion
          fit_max( DELTA_SEL_VEL ) = skip_search +
     1        Jlsrch( data_array( nd_v_index + skip_search ),
     2                1, n_search, sel_min( DELTA_SEL_VEL ) )
          fit_min( DELTA_SEL_VEL ) = 1 + skip_search +
     1        Jlsrch( data_array( nd_v_index + skip_search ),
     2                1, n_search, sel_max( DELTA_SEL_VEL ) )
c       number of selected points criterion
          fit_min( DELTA_SEL_NPT ) =
     1        edge_search - int( sel_min(DELTA_SEL_NPT) + 0.5 ) + 1
          fit_max( DELTA_SEL_NPT ) =
     1        edge_search + int( sel_max(DELTA_SEL_NPT) + 0.5 )
c
c       The next step is to check that at least one of the selection criteria
c       is set to "or" (DELTA_OR_CRIT) or "and" (DELTA_AND_CRIT). If no
c       selection criterion is found, the "select location" criterion is set
c       to "or" . Also ensure that the fitting limits are ordered correctly,
c       i.e. the minima are smaller than the maxima
c
          do 8 source = 1, DELTA_SEL_NNN                                  2b
              got_crit = got_crit .or.
     1            sel_crit(source) .eq. DELTA_OR_CRIT  .or.
     2            sel_crit(source) .eq. DELTA_AND_CRIT
              if ( fit_min(source) .gt. fit_max(source) ) then            3b
                  call swap4( fit_min(source), fit_max(source) )
                  end if                                                  3e
8             continue                                                    2e
          if ( .not. got_crit )
     1        sel_crit(DELTA_SEL_LOC) = DELTA_OR_CRIT
c
c       Now search through the selection criteria for "or"s, expanding the
c       range of the fitted polynomial with each one .
c
          start_fit = edge_search
          end_fit   = edge_search
          got_crit = .FALSE.
          do 9 source = 1, DELTA_SEL_NNN                                  2b
              if ( sel_crit(source) .eq. DELTA_OR_CRIT ) then             3b
                  start_fit = min( fit_min(source), start_fit )
                  end_fit   = max( fit_max(source), end_fit )
                  got_crit = .TRUE.
                  end if                                                  3e
9             continue                                                    2e
c
c       Now search through the selection criteria for "and"s, contracting the
c       range of the fitted polynomial with each one .
c
          do 10 source = 1, DELTA_SEL_NNN                                 2b
              if ( sel_crit(source) .eq. DELTA_AND_CRIT ) then            3b
                  if ( got_crit ) then                                    4b
                      start_fit = max( fit_min(source), start_fit )
                      end_fit   = min( fit_max(source), end_fit )
                  else                                                    4m
                      start_fit = min( fit_min(source), start_fit )
                      end_fit   = max( fit_max(source), end_fit )
                      end if                                              4e
                  got_crit = .TRUE.
                  end if                                                  3e
10            continue                                                    2e
c
c       Finally, ensure that the number of fitted data points is more
c       than the polynomial order by at least 1
c
          n_fitted = end_fit - start_fit + 1
          if ( n_fitted .lt. poly_order + 1 ) then                        2b
              end_fit = min( n_data,
     1            ( poly_order - n_fitted + 1 ) / 2 + end_fit )
              start_fit = end_fit - poly_order
              n_fitted = poly_order + 1
              end if                                                      2e
c
c       Fit a polynomial (least mean squared error) to the selected section
c       of data . Then convert the sum square error to rms error .
c
c              SPYORTH( X, Y, WEIGHT, NPTS, POLY, ORDER, ERROR, WORK )
          call spyorth( data_array( nd_y_index+start_fit-1 ),
     1                  data_array( nd_v_index+start_fit-1 ),
     2                  data_array( weig_index+start_fit-1 ), n_fitted,
     3                  fitted_poly, poly_order, sum_sq_err,
     4                  data_array( work_index ) )
          sum_sq_err = sqrt( max( 0.0, sum_sq_err / real( n_fitted ) ) )
c
c       Solve the polynomial implicitly at the EDGE_SPEED to obtain the
c       boundary layer thickness . Issue warning if the fitting error is large .
c
          if ( edge_search .ge. n_data ) then                             2b
              source = n_data
          else                                                            2m
              source = edge_search
              end if                                                      2e
          delta_min = data_array( nd_y_index + source - 1 )
          delta_max = data_array( nd_y_index + source )
          speed_min = data_array( nd_v_index + source - 1 )
          speed_max = data_array( nd_v_index + source )
          qcalc_nn  = -1.0
          nd_edge_speed = log( ref_speed / data_array(edge_index) )
          count = 1
c         start of iteration loop                                         2b
11        if ( abs( speed_min / nd_edge_speed - 1.0 ) .gt. iter_precis
     1          .and. abs( delta_max - delta_min ) .gt. iter_precis
     2          .and. count .le. max_iterations ) then
              denom = speed_max - speed_min
              if ( denom .ne. 0.0 ) then                                  3b
                  qcalc_nn = delta_min + ( delta_max - delta_min ) /
     1                denom * ( nd_edge_speed - speed_min )
                  delta_max = delta_min
                  speed_max = speed_min
                  delta_min = qcalc_nn
                  speed_min = qpoly( delta_min, fitted_poly, poly_order)
                  count = count + 1
                  goto 11
                  end if                                                  3e
              end if
c             end of iteration loop                                       2e
          if ( count .gt. max_iterations ) then                           2b
              write( STD_ERR, 12 ) max_iterations
12            format( ' QCALC_NN : WARNING : Maximum number of ',
     1            'iterations ', i7, ' exceeded' )
              got_error = .TRUE.
              end if
          if ( sum_sq_err .gt. max_fit_err *  qcalc_nn ) then             2b
              write( STD_ERR, 13 ) max_fit_err * 100.0, qcalc_nn
13            format( ' QCALC_NN : WARNING : RMS fitting error in ',
     1            'calculating boundary layer' / 25x,
     2            'thickness larger than ', f8.3, '% (of ', g12.5, ')' )
              got_error = .TRUE.
              end if                                                      2e
c
c       transform the result from the non-dimensional to the original form
c
          qcalc_nn = ( 2.0 - qcalc_nn ) * ref_locn
c
c       Calculate the fitting polynomial over the range of the fitted data
c       and place the results in the work array according to the following
c       format :
c           data_array(n_rows*2+2) = n_rows : number of points calculated
c               -- signals that further processing can be done by the caller
c           data_array(n_rows*2+3:n_rows*3+2) = probe location values
c           data_array(n_rows*3+3:n_rows*4+2) = air speed values
c           N.B. edge_index = 2 * n_rows + 1
c
          data_array(edge_index+1) = n_rows
          location = data_array(nd_y_index + start_fit - 1)
          loc_step = ( data_array(nd_y_index + end_fit - 1) - location )
     1               / ( n_rows - 1 )
          do 14 destin = edge_index + 2, edge_index + 1 + n_rows          2b
              data_array(destin+n_rows) = ref_speed / max( 1.0e-35, exp(
     1            qpoly( location, fitted_poly, poly_order ) ) )
              data_array(destin) = ( 2.0 - location ) * ref_locn
              location = location + loc_step
14            continue                                                    2e
          end if                                                          1e
      if ( got_error )
     1    call ilgwait()
      return
      end



      REAL FUNCTION QDELTA_STAR( LOCATION, MEAN_SPEED,  INCREM,
     1                           N_POINTS, FREE_STREAM, WORK )
c
c Calculate boundary layer displacement thickness from the velocity profile.
c
c Arguments
c  real    location(*)  # array of distances from the wall
c          mean_speed(*)# array of mean air speed at each dist from wall
c  integer increm       # indexing increment
c          n_points     # number of data points in a velocity profile
c  real    free_stream  # mean air speed at the edge of the boundary layer
c          work(*)      # work vector of length 2 * n_points
c
c constants
c  MIN_POINTS = 8       # minimum number of profile points
c
c local variables
c  logical
c   edelta_check_args   # function : check arguments
c  integer
c   dest_igd            # destination index for integrand
c   dest_loc            # destination index for probe location
c   source              # source index for probe location and air speed arrays
c  real
c   qcalc_delta         # function : calculates integral thickness
c   qdelta_menu         #          : menu interface for thickness calculations
c  character
c   labels(DELTA_N_LABELS)*32 # routine name & graph labels
c
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer MIN_POINTS
      parameter( MIN_POINTS=8 )
c
      logical edelta_check_args
      integer increm, n_points, dest_igd, dest_loc, source
      real    location(*), mean_speed(*), free_stream, work(*),
     1        qcalc_delta, qdelta_menu
      character labels(DELTA_N_LABELS)*32
      include 'bly_delt.cmn'
c
      save labels
      data labels /
     1    'QDELTA_STAR',
     2    'displacement',
     3    'y (m)',
     4    '1-|~U/U|v0',
     5    'Displacement thickness integrand' /
      external bly_delta_block, qcalc_delta
c
c
      qdelta_star = -1.0
      if ( edelta_check_args( location, mean_speed, increm, n_points,
     1                      free_stream, MIN_POINTS,
     2                      labels(DELTA_CALLER), 'FREE_STREAM' ) ) then  1b
c
c       Transfer the profile data to the work array . The velocity data
c       is converted into integrand for displacement thickness . The
c       first data point is (0.0,1.0) at the wall, so put it into the
c       work array first .
c
c         first data point
          dest_loc = 1
          dest_igd = n_points + 2
          work(dest_loc) = 0.0
          work(dest_igd) = 1.0
c         remainder of data
          dest_loc = dest_loc + 1
          dest_igd = dest_igd + 1
          do 2 source = 1, increm * ( n_points - 1 ) + 1, increm          2b
              work(dest_loc) = location(source)
              work(dest_igd) =
     1            ( 1.0 - mean_speed(source) / free_stream ) *
     2            ( 1.0 - location(source) / duct_radius )
              dest_loc = dest_loc + 1
              dest_igd = dest_igd + 1
2             continue                                                    2e
c
c       edit the data if enabled; otherwise simply evaluate the result
c
          if ( edit_enable ) then                                         2b
              qdelta_star = qdelta_menu( work, n_points + 1,
     1                                   qcalc_delta, labels )
          else                                                            2m
              qdelta_star = qcalc_delta( work, n_points + 1,
     1                                   n_points + 1 )
              end if                                                      2e
          end if                                                          1e
c
      return
      end



      REAL FUNCTION QDELTA_THETA( LOCATION, MEAN_SPEED,  INCREM,
     1                            N_POINTS, FREE_STREAM, WORK )
c
c Calculate boundary layer momentum thickness from the velocity profile.
c
c Arguments
c  real    location(*)  # array of distances from the wall
c          mean_speed(*)# array of mean air speed at each dist from wall
c  integer increm       # indexing increment
c          n_points     # number of data points in a velocity profile
c  real    free_stream  # mean air speed at the edge of the boundary layer
c          work(*)      # work vector of length 2 * n_points
c
c constants
c  MIN_POINTS = 8       # minimum number of profile points
c
c local variables
c  logical
c   edelta_check_args   # function : check arguments
c  integer
c   dest_igd            # destination index for integrand
c   dest_loc            # destination index for probe location
c   source              # source index for probe location and air speed arrays
c  real
c   qcalc_delta         # function : calculates the integral thickness
c   qdelta_menu         #          : menu interface for thickness calculations
c  character
c   labels(DELTA_N_LABELS)*28 # routine name & graph labels
c
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer MIN_POINTS
      parameter( MIN_POINTS=8 )
c
      logical edelta_check_args
      integer increm, n_points, dest_igd, dest_loc, source
      real    free_stream, work(*), location(*), mean_speed(*),
     1        qcalc_delta, qdelta_menu
      character labels(DELTA_N_LABELS)*28
      include 'bly_delt.cmn'
c
      save labels
      data labels /
     1    'QDELTA_THETA',
     2    'momentum',
     3    'y (m)',
     4    '(1-|~U/U|v0)(|~U/U|v0)',
     5    'Momentum thickness integrand' /
      external bly_delta_block, qcalc_delta
c
c
      qdelta_theta = -1.0
      if ( edelta_check_args( location, mean_speed, increm, n_points,
     1                      free_stream, MIN_POINTS,
     2                      labels(DELTA_CALLER), 'FREE_STREAM' ) ) then  1b
c
c       Transfer the profile data to the work array . The velocity data is
c       converted into integrand for momentum thickness . The first data
c       point is (0.0,0.0) at the wall, so put it into the work array first .
c
c         first data point
          dest_loc = 1
          dest_igd = n_points + 2
          work(dest_loc) = 0.0
          work(dest_igd) = 0.0
c         remainder of data
          dest_loc = dest_loc + 1
          dest_igd = dest_igd + 1
          do 2 source = 1, increm * ( n_points - 1 ) + 1, increm          2b
              work(dest_loc) = location(source)
              work(dest_igd) =
     1            ( 1.0 - mean_speed(source) / free_stream ) *
     2            ( 1.0 - location(source) / duct_radius ) *
     3            mean_speed(source) / free_stream
              dest_loc = dest_loc + 1
              dest_igd = dest_igd + 1
2             continue                                                    2e
c
c       edit the data if enabled; otherwise simply evaluate the result
c
          if ( edit_enable ) then                                         2b
              qdelta_theta = qdelta_menu( work, n_points + 1,
     1                                    qcalc_delta, labels )
          else                                                            2m
              qdelta_theta = qcalc_delta( work, n_points + 1,
     1                                    n_points + 1 )
              end if                                                      2e
          end if                                                          1e
c
      return
      end



      REAL FUNCTION QDELTA_ENERGY( LOCATION, MEAN_SPEED,  INCREM,
     1                             N_POINTS, FREE_STREAM, WORK )
c
c Calculate boundary layer energy thickness from the velocity profile.
c
c Arguments
c  real    location(*)  # array of distances from the wall
c          mean_speed(*)# array of mean air speed at each dist from wall
c  integer increm       # indexing increment
c          n_points     # number of data points in a velocity profile
c  real    free_stream  # mean air speed at the edge of the boundary layer
c          work(*)      # work vector of length 2 * n_points
c
c constants
c  MIN_POINTS = 8       # minimum number of profile points
c
c local variables
c  logical
c   edelta_check_args   # function : check arguments
c  integer
c   dest_igd            # destination index for integrand
c   dest_loc            # destination index for probe location
c   source              # source index for probe location and air speed arrays
c  real
c   qcalc_delta         # function : calculates the integral thickness
c   qdelta_menu         #          : menu interface for thickness calculations
c  character
c   labels(DELTA_N_LABELS)*27 # routine name & graph labels
c
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer MIN_POINTS
      parameter( MIN_POINTS=8 )
c
      logical edelta_check_args
      integer increm, n_points, dest_igd, dest_loc, source
      real    free_stream, work(*), location(*), mean_speed(*),
     1        qcalc_delta, qdelta_menu
      character labels(DELTA_N_LABELS)*27
      include 'bly_delt.cmn'
c
      save labels
      data labels /
     1    'QDELTA_ENERGY',
     2    'energy',
     3    'y (m)',
     4    '(1-(|~U/U|v0)|t2)(|~U/U|v0)',
     5    'Energy thickness integrand' /
      external bly_delta_block, qcalc_delta
c
c
      qdelta_energy = -1.0
      if ( edelta_check_args( location, mean_speed, increm, n_points,
     1                      free_stream, MIN_POINTS,
     2                      labels(DELTA_CALLER), 'FREE_STREAM' ) ) then  1b
c
c       Transfer the profile data to the work array . The velocity data is
c       converted into integrand for energy thickness . The first data
c       point is (0.0,0.0) at the wall, so put it into the work array first .
c
c         first data point
          dest_loc = 1
          dest_igd = n_points + 2
          work(dest_loc) = 0.0
          work(dest_igd) = 0.0
c         remainder of data
          dest_loc = dest_loc + 1
          dest_igd = dest_igd + 1
          do 2 source = 1, increm * ( n_points - 1 ) + 1, increm          2b
              work(dest_loc) = location(source)
              work(dest_igd) =
     1            ( 1.0 - ( mean_speed(source) / free_stream ) **2 ) *
     2            ( 1.0 - location(source) / duct_radius ) *
     3            mean_speed(source) / free_stream
              dest_loc = dest_loc + 1
              dest_igd = dest_igd + 1
2             continue                                                    2e
c
c       edit the data if enabled; otherwise simply evaluate the result
c
          if ( edit_enable ) then                                         2b
              qdelta_energy = qdelta_menu( work, n_points + 1,
     1                                     qcalc_delta, labels )
          else                                                            2m
              qdelta_energy = qcalc_delta( work, n_points + 1,
     1                                     n_points + 1 )
              end if                                                      2e
          end if                                                          1e
c
      return
      end



      REAL FUNCTION QCALC_DELTA( DATA_ARRAY, N_DATA, N_ROWS )
c
c Calculate the displacement thickness, momentum thickness or energy thickness
c of the boundary layer from the N_DATA probe locations and integrand values
c in DATA_ARRAY .
c
c Arguments
c  real data_array(*) # data array containing  # starting index
c            # INPUT  #   probe position data  # 1
c            # INPUT  #   thickness integrand  # n_rows + 1
c            # OUTPUT #   set to 0.0           # 2 * n_rows + 1
c            # OUTPUT #   set to 0.0           # 2 * n_rows + 2
c                     #   A total array size of 2 * n_points is required .
c
c  integer n_data     # number of profile data coordinates actually used
c          n_rows     # number of data points available i.e. array size
c
c constants
c  LOC_INDEX=1        # start array index for probe locations
c
c local variables
c  integer
c   integrand_index   # data_array start indices for integrand values
c                     # indices for deleting duplicate locations :
c   in_loc, in_itg    #   input location and integrand
c   out_loc, out_itg  #   output location and integrand
c  real
c   qqmind            # function : quadratic mean integration of discrete data
c   qtrap_vs          #          : trapezium rule of integration
c  subroutine
c   sortsh            # shell sort of data
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer LOC_INDEX
      parameter( LOC_INDEX=1 )
c
      integer n_data, n_rows, integrand_index,
     1        in_loc, in_itg, out_loc, out_itg
      real    data_array(*), qqmind, qtrap_vs
      include 'bly_delt.cmn'
c
c         start index to the data array for integrand
c
      integrand_index = LOC_INDEX + n_rows
c
c         Sort the 2 columns of data containing probe location and air speed
c         into ascending order of distance from the wall.
c
c          SORTSH( X, START_ROW, STOP_ROW, ROWS, SORT_COL, COLUMNS )
      call sortsh( data_array(LOC_INDEX), n_data, 1, n_rows, 1, 2 )
c
c         Then compress out duplicated probe locations. This is necessary
c         to avoid divide-by-zero errors in the integration routines.
c
      out_loc = LOC_INDEX
      out_itg = integrand_index
      in_itg  = integrand_index + 1
      do 1 in_loc = 2, n_data                                             1b
          if ( data_array(in_loc) .gt. data_array(out_loc) ) then         2b
              out_loc = out_loc + 1
              out_itg = out_itg + 1
              if ( out_loc .lt. in_loc ) then                             3b
                  data_array(out_loc) = data_array(in_loc)
                  data_array(out_itg) = data_array(in_itg)
                  end if                                                  3e
              end if                                                      2e
          in_itg = in_itg + 1
1         continue                                                        1e
      if ( out_loc .lt. n_data ) then                                     1b
          write( STD_ERR, 2 ) n_data - out_loc
2         format( ' QCALC_DELTA : WARNING : ', i6,
     1            ' duplicate locations deleted' )
          end if                                                          1e
c
c       do the integration
c
      if ( integ_method .eq. DELTA_TRAP_INT ) then                        1b
          qcalc_delta = qtrap_vs( data_array(LOC_INDEX),
     1                      data_array(integrand_index), 1, out_loc )
      else if ( integ_method .eq. DELTA_QUAD_INT ) then                   1m
          qcalc_delta = qqmind( data_array(LOC_INDEX),
     1                          data_array(integrand_index), 1, out_loc,
     2                          data_array(1), data_array(n_data) )
          end if                                                          1e
c
c       set the signals for no further processing by the caller
c
      data_array( 2 * n_rows + 1 ) = 0.0
      data_array( 2 * n_rows + 2 ) = 0.0
      return
      end



      BLOCK DATA BLY_DELTA_BLOCK
c
c Initialization for common block variables
c
      include 'stdio.def'
      include 'bly_delt.def'
c
      include 'bly_delt.cmn'
c
c
c editing
      data edit_enable / .FALSE. /
c
c
c integral thicknesses
      data integ_method/ DELTA_QUAD_INT /
      data duct_radius / DELTA_2_DIM_BL /
c
c
c boundary layer thickness
      data sel_crit    / DELTA_OR_CRIT, DELTA_UNUSED, DELTA_AND_CRIT /
      data max_iterations, poly_order  / 100, 4    /
      data max_fit_err, iter_precis / 0.02, 3.0e-7 /
c                        location   speed     n_points
      data sel_min     / 0.50,      -0.10,    3.0 /
      data sel_max     / 1.75,       0.30,    4.0 /
      end



      LOGICAL FUNCTION EDELTA_CHECK_ARGS( LOCATION, MEAN_SPEED,
     1                     INCREM, N_POINTS, FREE_STREAM,
     2                     MINIM_POINTS, CALLER, ID_FREE_STREAM )
c
c         Check arguments for calculating boundary layer thickness .
c
c Arguments
c  real    location(*)  # array of distances from the wall
c          mean_speed(*)# array of mean air speed at each dist from wall
c  integer increm       # indexing increment
c          n_points     # number of data points in a velocity profile
c  real    free_stream  # free_stream air speed
c  integer minim_points # minimum number of points required
c  character caller*(*) # name of calling routine
c          id_free_stream*(*) # name to be used if FREE_STREAM is illegal
c
c constants
c  ABORT        = ' : ABORT : Illegal argument '
c  MIN_INCREM   = 1     # minimum indexing increment
c  MAX_INCREM   = 1000  # maximum indexing increment
c  MAX_POINTS   = 1000  # maximum number of profile points
c  MIN_LOCATION = 1.0e-6# minimum probe location (mm)
c  MAX_LOCATION = 1.0e6 # maximum probe location (mm)
c  MIN_SPEED    = 0.001 # minimum air speed value
c  MAX_SPEED    = 500.0 # maximum air speed value
c
c local variables
c  integer
c   source              # index into probe location and air speed arrays
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer    MIN_INCREM,    MAX_INCREM,         MAX_POINTS
      real MIN_LOCATION,        MAX_LOCATION,
     1     MIN_SPEED,           MAX_SPEED
      character  ABORT*28
      parameter( MIN_INCREM=1,  MAX_INCREM=1000,    MAX_POINTS=1000,
     1     MIN_LOCATION=1.0e-6, MAX_LOCATION=1.0e6,
     2     MIN_SPEED=0.001,     MAX_SPEED=500.0,
     3     ABORT = ' : ABORT : Illegal argument ' )
c
      real location(*), mean_speed(*), free_stream
      integer increm, n_points, minim_points, source
      character caller*(*), id_free_stream*(*)
c
      edelta_check_args = .FALSE.
c
c       data indexing increment
c
      if ( increm .lt. MIN_INCREM .or. increm .gt. MAX_INCREM ) then      1b
          write( STD_ERR, 1 ) caller, ABORT, increm
1         format( 1x, a, a, 'INCREM = ', i12 )
          edelta_check_args = .FALSE.
c
c       number of data points
c
      else if ( n_points .lt. minim_points .or.
     1          n_points .gt. MAX_POINTS ) then                                 1m
          write( STD_ERR, 2 ) caller, ABORT, n_points                     1m
2         format( 1x, a, a, 'N_POINTS = ', i12 )
          edelta_check_args = .FALSE.
      else if ( free_stream .lt. MIN_SPEED .or.
     1          free_stream .gt. MAX_SPEED ) then                         1m
          write( STD_ERR, 3 ) caller, ABORT, id_free_stream, free_stream
3         format( 1x, a, a, a, ' = ', g12.5 )
      else                                                                1m
c
c         check that all the probe locations and air speeds are positive
c         and within reasonable limits
c
          do 4 source = 1, increm * ( n_points - 1 ) + 1, increm          2b
              if ( location(source) .le. MIN_LOCATION .or.
     1             location(source) .ge. MAX_LOCATION ) then              3b
                  write( STD_ERR, 3 ) caller, ABORT,
     1                'probe location', location(source)
                  return
              else if ( mean_speed(source) .le. MIN_SPEED .or.
     1                  mean_speed(source) .ge. MAX_SPEED ) then          3m
                  write( STD_ERR, 3 ) caller, ABORT,
     1                'mean air speed = ', location(source)
                  return
                  end if                                                  3e
4             continue                                                    2e
          edelta_check_args = .TRUE.
          end if                                                          1e
      return
      end




      REAL FUNCTION QDELTA_MENU( DATA_ARRAY, N_POINTS,
     1                         QDELTA_CALC, LABELS )
c
c This function is the menu interface for routines which calculates
c boundary layer thicknesses of various kinds .
c
c Arguments
c  real data_array(*)   # data array - first n_points values are probe locations
c                       #            - 2nd n_points values are mean air speeds
c  integer n_points     # number of data points
c  real    qdelta_calc  # function for calculating the thickness result
c  character labels(DELTA_N_LABELS)*(*)
c                       # array of strings containing various labels : the
c                       # array indices are
c                       #   DELTA_CALLER   - name of calling function
c                       #   DELTA_ABSCISSA - graph abscissa label
c                       #   DELTA_ORDINATE - graph ordinate label
c                       #   DELTA_GR_TITLE - graph title
c
c constants
c  MAX_TRIES     = 4    # maximum number of input attempts
c  DATA_Y_OFFSET = 1    # data array index for profile probe location data
c
c local variables
c  logical
c   got_result          # result has been calculated
c   Equery              # function : read logical
c  integer
c   data_v_offset       # data array index for profile velocity data
c   edge_v_offset       # data array index edge-of-boundary-layer velocity
c   curve_offset        # data array index fitted curve
c   capt_len            # length of capture file name
c   fit_index           # data_array index of fitted data for post-processing
c   item                # selected menu item
c   line_type           # dashed line style for the fitted polynomial
c   new_points          # number of data points after editing
c   result_len          # length of result name
c   Jresp               # function : read integer
c   str_length          #          : non-blank length of file name
c  character
c   capture_file*(32)   # capture file name
c  subroutine
c   cresp               # read string
c   sfudge              # edit data
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer    MAX_TRIES,   DATA_Y_OFFSET
      parameter( MAX_TRIES=4, DATA_Y_OFFSET=1 )
c
      logical got_result, plotted, Equery
      integer n_points, data_v_offset, edge_v_offset, curve_offset,
     1        capt_len, fit_index, item, line_type, new_points,
     2        result_len, Jresp, str_length
      real    data_array(*), qdelta_calc, qresp
      character labels(DELTA_N_LABELS)*(*), capture_file*(32)
      include 'bly_delt.cmn'
c
      external  qdelta_calc
      save capture_file
      data capture_file / ' ' /
c
c
      data_v_offset = DATA_Y_OFFSET + n_points
      edge_v_offset = data_v_offset + n_points
      curve_offset = edge_v_offset + 1
      plotted    = .FALSE.
      got_result = .FALSE.
      new_points = n_points
      result_len = max( 1, str_length( labels(DELTA_RESULT) ) )
c     start of menu loop                                                  1b
1         call newscr()
          if ( got_result ) then                                          2b
              write( STD_OUT, 2 ) labels(DELTA_RESULT)(:result_len),
     1                            qdelta_menu
2             format( 1x, a, ' thickness = ', g12.5 )
          else                                                            2m
              write( STD_OUT, 3 ) labels(DELTA_RESULT)(:result_len)
3             format( 1x, a, ' thickness not yet calculated' )
              end if                                                      2e
          capt_len = max( 1, str_length( capture_file ) )
          write( STD_OUT, 4 ) labels(DELTA_RESULT)(:result_len)
4         format( ' Select  -1) Abandon  0) Exit  1) Enter value'  /
     1        10x, '2) Restore  3) Edit' /
     2        10x, '4) Params   5) Help  6) Calculate ', a,
     3             ' thickness ', $ )
          item = Jresp( STD_IN, -1, 6, MAX_TRIES )
c
c       ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, ' Abandon : Are you sure ? ' ) ) then  3b
                  qdelta_menu = DELTA_ERROR
                  call ilgclos
                  return
                  end if                                                  3e
c
c       EXIT
c
          else if ( item .eq. 0 ) then                                    2m
              if ( got_result ) then                                      3b
                  call ilgclos
                  return
              else                                                        3m
                  write( STD_ERR, 5 ) labels(DELTA_RESULT)(:result_len)
5                 format( ' Must calculate or enter ', a,
     1                    ' thickness before normal Exit' )
                  call ilgwait()
                  end if                                                  3e
c
c       ENTER VALUE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 6 ) labels(DELTA_RESULT)(:result_len)
6             format( ' Enter the value of ', a )
              qdelta_menu = qresp( STD_IN, 1.0e-6, 2.0e3, MAX_TRIES )
              got_result = .TRUE.
c
c       RESTORE DELETED DATA
c
          else if ( item .eq. 2 ) then                                    2m
              new_points = n_points
c
c       EDIT DATA
c
          else if ( item .eq. 3 ) then                                    2m
              call ilgopen( capture_file )
              call ilgslaw( 'lin', 'lin' )
              call ilgxlab( labels( DELTA_ABSCISSA ) )
              call ilgylab( labels( DELTA_ORDINATE ) )
              call ilgtitl( labels( DELTA_GR_TITLE ) )
              call sfudge( data_array(DATA_Y_OFFSET),
     1                     data_array(data_v_offset),
     2                     1, new_points, new_points, n_points )
              line_type = 1
              plotted = .TRUE.
c
c       SET CALCULATION PARAMETERS
c
          else if ( item .eq. 4 ) then                                    2m
              call sdelta_set_menu()
              edit_enable = .TRUE.
c
c       HELP
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 7 ) DELTA_ERROR,
     1            labels(DELTA_RESULT)(:result_len)
7             format( ' Description of menu items' /
     1           '  -1) Abandon thickness calculation and return ',
     2                  'with error code ', g12.5 /
     3           '   0) Exit with the calculated thickness.' /
     4           '   1) Specify the name of the graphics capture file.'/
     5           '      The graphics are not captured if the file ',
     6                  'name is blank.' /
     7           '   2) Edit the profile data' /
     8           '   3) Calculate ', a, ' thickness' /
     9           '   4) Restore deleted and appended data. Shifted ',
     a                  'data cannot be restored.' )
              write( STD_OUT, 8 )
8             format( '   5) Change the calculation parameters' /
     1           '      N.B. In the cases of displacement, momentum ',
     2                  'and energy thickness,' /
     3           11x, 'changing the duct radius has no immediate ',
     4                  ' because the' /
     5           11x, 'integrand cannot be changed without a new call ',
     6                  'to QDELTA_STAR,' /
     7           11x, 'QDELTA_THETA or QDELTA_ENERGY' )
              call ilgwait()
c
c       CALCULATE RESULT
c
          else if ( item .eq. 6 ) then                                    2m
              qdelta_menu = qdelta_calc( data_array,
     1                                   new_points, n_points )
              got_result = qdelta_menu .ne. DELTA_ERROR
c
c           postprocessing - plot fitted curve
c             Any data returned from the calculation routine is appended
c             after the profile data . If any data is returned, the first
c             appended value is the number of profile points .
c
              if ( got_result .and. n_points .eq.
     1                int( data_array(curve_offset) + 0.5 ) ) then      3b
c                 first plot the profile data if not yet plotted
                  if ( .not. plotted ) then                               4b
                      call ilgopen( capture_file )
                      call ilgslaw( 'lin', 'lin' )
                      call ilgxlab( labels( DELTA_ABSCISSA ) )
                      call ilgylab( labels( DELTA_ORDINATE ) )
                      call ilgtitl( labels( DELTA_GR_TITLE ) )
                      call ilgline( 5, 0, 1 )
                      call ilgdata( data_array(DATA_Y_OFFSET),
     1                    data_array(data_v_offset), 1, new_points, 0 )
                      plotted = .TRUE.
                      line_type = 1
                      end if                                              4e
c                 plot the fitted polynomial
                  call ilgline( 0, line_type, 1 )
                  call ilgdata(
     1                data_array( curve_offset + 1 ),
     2                data_array( curve_offset + 1 + n_points ),
     3                1, n_points, 0 )
                  line_type = mod( line_type, 6 ) + 1
c                 plot the returned result
                  call ilgline( 1, 0, 1 )
                  call ilgdata( qdelta_menu, data_array(edge_v_offset),
     1                          1, 1, 0 )
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
c
      end


      SUBROUTINE SDELTA_SET_MENU()
c
c Menu interface for setting global parameters for boundary layer thickness
c determination .
c
c constant
c  MAX_TRIES = 4
c
c local variables
c  logical
c   eyes          # function : boolean to logical
c  integer
c   editing       # NO / YES enable editing
c   i             # loop/array index
c   item          # select menu item
c   Jresp         # function : read integer
c  real
c   Qresp         # function : read real
c  character
c    Cint_meth(2)*9 # 'trapezium', 'quadratic'
c    sel_meth(0:2)*6# string array of selection methods
c    Cyes*5       # function : boolean => 'no', 'yes' or 'query'
c  subroutine
c   newscr        # clear alphanumeric screen
c   sdelta_set_range  # set data selection range for fitting polynomial
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer    MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      logical eyes
      integer editing, i, item, Jresp
      real    Qresp
      character Cint_meth(2)*9, sel_meth(0:2)*6, Cyes*5
      include 'bly_delt.cmn'
c
      save sel_meth, Cint_meth
      data sel_meth / 'UNUSED', 'OR', 'AND' /
      data Cint_meth / 'trapezium', 'quadratic' /
c
c
      editing = NO
      if ( edit_enable )
     1    editing = YES
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes(editing), Cint_meth(integ_method),
     1        duct_radius, poly_order, max_fit_err, iter_precis,
     2        max_iterations, ( sel_meth(sel_crit(i)), sel_min(i),
     3          sel_max(i), i=1,DELTA_SEL_NNN )
2         format( ' Select' / '   0) Exit' /
     1        '   1) Enable editing', 20x, a /
     2        '   2) Integration method', 16x, a /
     3        '   3) Duct radius', 23x, g12.5 /
     4        '   4) Order of fitted polynomial', 9x, i2 /
     5        '   5) Minimum notifiable fitting error  ', g12.5 /
     6        '   6) Iteration precision', 15x, g10.3 /
     7        '   7) Maximum number of iterations    ', i5 /
     8        ' Data selection ranges for fitting polynomial' /
     9        '   8) Non-dimensional probe location     ',
     a          a, '  ', f6.3, ' to ', f6.3 /
     b        '   9) Non-dimensional air speed', 10x,
     c          a, '  ', f6.3, ' to ', f6.3 /
     d        '  10) Number of selected data points     ',
     e          a, '  ', f6.3, ' to ', f6.3, '  ', $ )
          item = Jresp( STD_IN, 0, 10, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              edit_enable = Eyes( editing, ' ' )
              return
          else if ( item .eq. 1 ) then                                    2m
              editing = mod( editing + 1, 2 )
c
c       INTEGRATION METHOD
c
          else if ( item .eq. 2 ) then                                    2m
              integ_method = mod( integ_method, 2 ) + 1
c
c       DUCT RADIUS
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Duct radius (for integral thicknesses) ? ', $ )
              duct_radius = Qresp( STD_IN, 0.0001, DELTA_2_DIM_BL,
     1                             MAX_TRIES )
c
c       ORDER OF FITTED POLYNOMIAL
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Order of fitted polynomial ? ', $ )
              poly_order = Jresp( STD_IN, 1, DELTA_MAX_POLY, MAX_TRIES )
c
c       MAXIMUM RECOMMENDED FITTING ERROR
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Minimum notifiable fitting error ? ', $ )
              max_fit_err = Qresp( STD_IN, 0.00001, 1.0, MAX_TRIES )
c
c       ITERATION PRECISION
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Iteration precision ? ', $ )
              iter_precis = Qresp( STD_IN, 1.0e-7, 1.0e-3, MAX_TRIES )
c
c       MAXIMUM NUMBER OF ITERATIONS
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 7 )
7             format( ' Maximum number of iterations ? ', $)
              max_iterations = Jresp( STD_IN,
     1            DELTA_ITER_MIN, DELTA_ITER_MAX, MAX_TRIES )
c
c       NON-DIMENSIONAL PROBE LOCATION
c
          else if ( item .eq. 8 ) then                                    2m
              call sdelta_set_range( DELTA_SEL_LOC, 0.0, 1.9 )
c
c       NON-DIMENSIONAL AIR SPEED
c
          else if ( item .eq. 9 ) then                                    2m
              call sdelta_set_range( DELTA_SEL_VEL, -0.5, 1.0 )
c
c       NUMBER OF SELECTED DATA POINTS
c
          else if ( item .eq. 10 ) then                                   2m
              call sdelta_set_range( DELTA_SEL_NPT, 1.0, 100.0 )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SDELTA_SET_RANGE( sel_var, minimum, maximum )
c
c Set data selection range for fitting polynomial .
c
c Arguments
c  integer
c   sel_var       # Specifies the selection range variable
c                 #   DELTA_SEL_LOC, DELTA_SEL_VEL or DELTA_SEL_NPT
c  real
c   minimum       # minimum value of selection range variable
c   maximum       # maximum value of selection range variable
c
c
c constants
c  MAX_TRIES   =4 # maximum number of input attempts
c
c local variables
c  integer
c   item          # selected menu item
c   DEFAULT_CRIT(DELTA_SEL_NNN) # array of default selection methods
c   Jresp         # Function : read integer
c  real
c   default_max(DELTA_SEL_NNN)  # array of default maxima
c   default_min(DELTA_SEL_NNN)  # array of default minima
c   Qresp         # function : read real
c  character
c   sel_meth(0:2)*6             # string array of selection methods
c   var_name(DELTA_SEL_NNN)*16  # string array of data selection variable names
c  subroutine
c   newscr        # clear alphanumeric screen
c
c
      include 'stdio.def'
      include 'bly_delt.def'
      integer    MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      integer sel_var, item, default_crit(DELTA_SEL_NNN), Jresp
      real    minimum, maximum,
     1        default_min(DELTA_SEL_NNN),  default_max(DELTA_SEL_NNN),
     2        Qresp
      character var_name(DELTA_SEL_NNN)*16, sel_meth(0:2)*6
      include 'bly_delt.cmn'
c
      save default_crit, default_min, default_max, var_name, sel_meth
      data default_crit / DELTA_OR_CRIT, DELTA_UNUSED, DELTA_AND_CRIT /
      data default_min / 0.50, -0.1, 4.0 /
      data default_max / 1.75,  0.3, 3.0 /
      data var_name /
     1     'probe location', 'mean air speed', 'number of points' /
      data sel_meth / 'UNUSED', 'OR', 'AND' /
c
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 )  var_name(sel_var),
     1        sel_meth( int( sel_crit(sel_var) + 0.5 ) ),
     2        sel_min(sel_var), sel_max(sel_var)
2         format( ' Change ', a, ' data selection range' /
     z            '   0) Exit' /
     1            '   1) Selection method ', a /
     2            '   2) Minimum value    ', f6.3 /
     3            '   3) Maximum value    ', f6.3 /
     4            '   4) Default values ', $ )
          item = Jresp( STD_IN, 0, 4, MAX_TRIES )
c
c       EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( sel_min(sel_var) .gt. sel_max(sel_var) )
     1            call swap4( sel_min(sel_var), sel_max(sel_var) )
              return
c
c       SELECTION METHOD
c
          else if ( item .eq. 1 ) then                                    2m
              sel_crit(sel_var) = mod( sel_crit(sel_var) + 1, 3 )
c
c       MINIMUM VALUE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Minimum value ? ', $ )
              sel_min(sel_var) = Qresp( STD_IN, minimum, maximum,
     1                                  MAX_TRIES )
c
c       MAXIMUM VALUE
c
          else if ( item .eq. 3 ) then                                    2e
              write( STD_OUT, 4 )
4             format( ' Maximum value ? ', $ )
              sel_max(sel_var) = Qresp( STD_IN, minimum, maximum,
     1                                  MAX_TRIES )
c
c       DEFAULTS
c
          else if ( item .eq. 4 ) then                                    2e
              sel_crit(sel_var) = DEFAULT_CRIT(sel_var)
              sel_min(sel_var) = DEFAULT_MIN(sel_var)
              sel_max(sel_var) = DEFAULT_MAX(sel_var)
              end if
          goto 1                                                          1e
c         end of menu loop
      end



      REAL FUNCTION QDELTA_SET_GET( WHAT, VALUE )
c
c Set or get global parameters for boundary layer thickness determination .
c The returned value is the actual value of the parameter (after setting) .
c
c Arguments
c  integer what   # specifies what to set if WHAT > 0 or
c                 #           what to get if WHAT < 0
c  real    value  # specifies the value to assign to the parameter if SETing
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Description             c Value of WHAT   c Enumerated/range of values  c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Enable editing data         c DELTA_EDIT_EN   c NO, YES                     c
c Integration method          c DELTA_INT_MET   c DELTA_TRAP_INT,             c
c                             c                 c   DELTA_QUAD_INT            c
c Duct radius                 c DELTA_DUC_RAD   c 0.0001 ... DELTA_2_DIM_BL   c
c Order of fitted polynomial  c DELTA_POL_ORD   c 1 ... DELTA_MAX_POLY        c
c Notifiable fitting error    c DELTA_MFIT_ER   c 0.00001 ... 1.0             c
c Iteration precision         c DELTA_IT_PREC   c 1.0e-7 ... 1.0e-3           c
c Maximum no. of iterations   c DELTA_IT_NMAX   c 10 ... 1000                 c
c                             c                 c                             c
c (Non-dimensional) data selection ranges for fitting polynomial              c
c Minimum probe location      c DELTA_SEL_LOC + c 0.0 ... 1.9                 c
c                             c DELTA_RAN_MIN   c                             c
c Maximum probe location      c DELTA_SEL_LOC + c 0.0 ... 1.9                 c
c                             c   DELTA_RAN_MAX c                             c
c Criterion for probe loc'n   c DELTA_SEL_LOC + c DELTA_UNUSED, DELTA_OR_CRIT c
c                             c   DELTA_RAN_CRT c   DELTA_AND_CRIT            c
c Minimum air speed           c DELTA_SEL_VEL + c -0.5 ... 1.0                c
c                             c   DELTA_RAN_MIN c                             c
c Maximum air speed           c DELTA_SEL_VEL + c -0.5 ... 1.0                c
c                             c   DELTA_RAN_MAX c                             c
c Criterion for air speed     c DELTA_SEL_VEL + c DELTA_UNUSED, DELTA_OR_CRIT c
c                             c   DELTA_RAN_CRT c   DELTA_AND_CRIT            c
c No. of data points on left  c DELTA_SEL_NPT + c 1.0 ... 100.0               c
c                             c   DELTA_RAN_MIN c                             c
c No. of data points on right c DELTA_SEL_NPT + c 1.0 ... 100.0               c
c                             c   DELTA_RAN_MAX c                             c
c Criterion for no. of points c DELTA_SEL_NPT + c DELTA_UNUSED, DELTA_OR_CRIT c
c                             c   DELTA_RAN_CRT c   DELTA_AND_CRIT            c
c                             c                 c                             c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c N.B. all enumerated constants are integers, and so must be converted to
c      REAL when used as values of the argument VALUE .
c
c constants
c  TINY=1.0e-5      # tolerance for equating integer & real values
c
c local variables
c  integer
c   abs_what        # abs( what )
c  real
c   sel_var         # selection range variable extracted from WHAT
c   minimum(DELTA_SEL_NNN) = { 0.0, -0.5,   1.0 }
c                   # minimum values for selection range variables
c   maximum(DELTA_SEL_NNN) = { 1.9,  1.0, 100.0 }
c                   # maximum values for selection range variables
c
c
      include 'stdio.def'
      include 'bly_delt.def'
      real    TINY
      parameter( TINY=1.0e-5 )
c
      integer what, abs_what
      real    value, sel_var,
     1        minimum(DELTA_SEL_NNN), maximum(DELTA_SEL_NNN)
      include 'bly_delt.cmn'
c
      save minimum, maximum
      data minimum / 0.0, -0.5,   1.0 /
      data maximum / 1.9,  1.0, 100.0 /
c
c
c
      abs_what = abs( what )
c
c       ENABLE EDITING DATA
c
      if ( abs_what .eq. DELTA_EDIT_EN ) then                             1b
          if ( what .gt. 0 ) then                                         2b
              if ( abs( value - NO ) .lt. TINY ) then                     3b
                  edit_enable = .FALSE.
              else if ( abs( value - YES ) .lt. TINY ) then               3m
                  edit_enable = .TRUE.
                  end if                                                  3e
              end if                                                      2e
          QDELTA_SET_GET = 0.0
          if ( edit_enable )
     1        QDELTA_SET_GET = 1.0
c
c       INTEGRATION METHOD
c
      else if ( abs_what .eq. DELTA_INT_MET ) then                        1m
          if ( what .gt. 0 ) then                                         2b
              if ( abs( value - DELTA_TRAP_INT ) .lt. TINY ) then         3b
                  integ_method = DELTA_TRAP_INT
              else if ( abs( value - DELTA_QUAD_INT ) .lt. TINY ) then    3m
                  integ_method = DELTA_QUAD_INT
                  end if                                                  3e
              end if                                                      2e
          QDELTA_SET_GET = integ_method
c
c       DUCT RADIUS
c
      else if ( abs_what .eq. DELTA_DUC_RAD ) then                        1m
          if ( what .gt. 0 ) then                                         2b
              if ( 0.0001 .le. value .and. value .le. DELTA_2_DIM_BL )
     1            duct_radius = value                                     2e
              end if
          QDELTA_SET_GET = duct_radius
c
c       ORDER OF FITTED POLYNOMIAL
c
      else if ( abs_what .eq. DELTA_POL_ORD ) then                        1m
          if ( what .gt. 0 ) then                                         2b
              if ( 0 .lt. int( value + 0.5 ) .and.
     1                    int( value + 0.5 ) .lt. DELTA_MAX_POLY )
     2            poly_order = int( value + 0.5 )
              end if                                                      2e
          QDELTA_SET_GET = poly_order
c
c       NOTIFIABLE FITTING ERROR
c
      else if ( abs_what .eq. DELTA_MFIT_ER ) then                        1m
          if ( what .gt. 0 ) then                                         2b
              if ( 0.00001 .lt. value .and. value .le. 1.0 )
     1            max_fit_err = value
              end if                                                      2e
          QDELTA_SET_GET = max_fit_err
c
c       ITERATION PRECISION
c
      else if ( abs_what .eq. DELTA_IT_PREC ) then                        1m
          if ( what .gt. 0 ) then                                         2b
              if ( 1.0e-7 .le. value .and. value .le. 1.0e-3 )
     1            iter_precis = value
              end if                                                      2e
          QDELTA_SET_GET = iter_precis
c
c       MAX NUMBER OF ITERATIONS
c
      else if ( abs_what .eq. DELTA_IT_NMAX ) then                        1m
          if ( what .gt. 0 ) then                                         2b
              if ( DELTA_ITER_MIN .le. int( value + 0.5 ) .and.
     1                int( value + 0.5 ) .le. DELTA_ITER_MAX )
     2            max_iterations = int( value + 0.5 )
              end if                                                      2e
          QDELTA_SET_GET = max_iterations
      else                                                                1m
c
c       (NON-DIMENSIONAL) DATA SELECTION RANGES FOR FITTING POLYNOMIAL
c
          sel_var  = mod( abs_what, DELTA_SET_GET )
          abs_what = abs_what - sel_var
c
c       MINIMUM SELECTION VARIABLE
c
          if ( abs_what .eq. DELTA_RAN_MIN ) then                         2b
              if ( what .gt. 0 ) then                                     3b
                  if ( minimum(sel_var) .le. value .and.
     1                            value .le. maximum(sel_var) )
     2                sel_min(sel_var) = value
                  end if                                                  3e
              QDELTA_SET_GET = sel_min(sel_var)
c
c       MAXIMUM SELECTION VARIABLE
c
          else if ( abs_what .eq. DELTA_RAN_MAX ) then                    2m
              if ( what .gt. 0 ) then                                     3b
                  if ( minimum(sel_var) .le. value .and.
     1                            value .le. maximum(sel_var) )
     2                sel_max(sel_var) = value
                  end if                                                  3e
              QDELTA_SET_GET = sel_max(sel_var)
c
c       SELECTION METHOD OF THE SELECTION VARIABLE
c
          else if ( abs_what .eq. DELTA_RAN_CRT ) then                    2m
              if ( what .gt. 0 ) then                                     3b
                  if ( abs( value - DELTA_UNUSED ) .lt. TINY ) then       4b
                      sel_crit(sel_var) = DELTA_UNUSED
                  else if ( abs(value - DELTA_OR_CRIT) .lt. TINY ) then   4m
                      sel_crit(sel_var) = DELTA_OR_CRIT
                  else if ( abs(value - DELTA_AND_CRIT) .lt. TINY ) then  4m
                      sel_crit(sel_var) = DELTA_AND_CRIT
                      end if                                              4e
                  end if                                                  3e
              QDELTA_SET_GET = sel_crit(sel_var)
c
c       ERROR - UNKNOWN "WHAT"
c
          else                                                            2m
              QDELTA_SET_GET = DELTA_ERROR
              end if                                                      2e
          end if                                                          1e
      return
      end



