c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      REAL FUNCTION QU_TAU( U_MEAN, Y, INCREM, N_POINTS, FREE_STREAM,
     1                      KINEM_VISC, WAKE_STRENGTH, DELTA_PLUS )
c
c THIS ROUTINE IS INTENDED TO BE THE **ONLY** ENTRY POINT IN THIS FILE.
c OTHER ROUTINES ARE NOT SUPPOSED TO CALLED FROM OUTSIDE THIS FILE.
c
c Calculates and returns the friction velocity of the experimental mean
c velocity profile contained in the arrays U_MEAN and Y according to the
c selected boundary layer model . The values of WAKE_STRENGTH and DELTA_PLUS
c are also returned if they are calulated . WAKE_STRENGTH is expressed as a
c multiple of the friction velocity, and DELTA_PLUS is expressed in wall units.
c
c N.B. All dimensioned quantities are in SI units except the argument Y
c      which is in millimetres.
c
c Arguments
c  real u_mean(*)     # IN  # array of mean velocity measurements in metres/sec
c       y(*)          # IN  # array of probe locations for mean velocity
c                           # measurements - distances from wall in millimetres
c  integer increm     # IN  # data array indexing increment
c          n_points   # IN  # number of available data points
c  real free_stream   # IN  # free stream speed
c       kinem_visc    # IN  # kinematic viscosity of fluid
c       wake_strength # OUT # amplitude of wake function -- Coles' PI
c       delta_plus    # OUT # boundary layer thickness   -- Coles' delta
c
c Local variables
c  integer src              # source index for copying data
c          dest             # destination index for copying data
c          item             # identifies item selected from menu
c          Jresp            # function : integer response
c          str_length       #          : length of character string
c  real fs_speed            # free stream speed retrieved from BLM
c       y_offset            # amount to add to probe location
c       get_BLM             # function : get boundary layer model parameter
c       Qresp               #          : real response
c
c  character name_BLM*24    # function : name of BLM parameter
c  subroutine
c      stau_sel_model       # select boundary layer model parameters
c      set_BLM              # set boundary layer parameters
c      sfudge               # edit data
c      stau_clauser         # find friction velocity graphically (Clauser chart)
c      stau_optimize        # find friction velocity etc by optimization
c      stau_profile         # plot non-dimensional velocity profile
c
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
c
      integer increm, n_points
      real u_mean(*), y(*), fs_speed, free_stream, kinem_visc,
     1     wake_strength, delta_plus
      character name_BLM*24
c
      integer src, dest, item, Jresp, str_length
      real y_offset, get_BLM, Qresp
      include 'qu_tau.cmn'
      external stau_block
c
c             Check arguments
c
      qu_tau = 0.0
      if ( increm .le. 0 .or. increm .gt. 100000 ) then
          write( STD_OUT, 1 ) increm
1         format( ' QU_TAU : ABORT : Illegal array increment ', i12 )
          return
      else if ( n_points .le. 0 ) then
          write( STD_OUT, 2 ) n_points
2         format( ' QU_TAU : ABORT : Illegal number of points ', i12 )
          return
      else if ( n_points .gt. MAX_DATA ) then
          write( STD_OUT, 3 ) n_points, MAX_DATA
3         format( ' QU_TAU : WARNING : ', i12, ' data points. Only ',
     1            i4, ' are used.' )
      else if ( free_stream .le. 1.0e-6 .or.
     1          free_stream .gt. 1000.0 ) then
          write( STD_OUT, 4 ) free_stream
4         format( ' QU_TAU : ABORT : Illegal free stream speed ',g12.5 )
          return
      else if ( kinem_visc .le. 1.0e-12 .or. kinem_visc .gt. 1.0 ) then
          write( STD_OUT, 5 ) kinem_visc
5         format( ' QU_TAU : ABORT : Illegal kinematic viscosity ',
     1            g12.5 )
          return
          end if
c
c           put free stream speed & kinematic viscosity into the model
c
      call set_BLM( BLM_fs_speed, free_stream )
      call set_BLM( BLM_kin_visc, kinem_visc )
c
c           copy data to working arrays
c
      src = 1
      n_data = min( MAX_DATA, n_points )
      do dest = 1, n_data
          u_data(dest) = u_mean(src)
          y_data(dest) = y(src) / 1000.0
          src = src + increm
          end do
c
c           main loop
c           *********
c
c     star of loop
10        write( STD_OUT, 11 )
11        format( ' Select action' / '    0) Exit' /
     1            '    1) Restore all data' /
     2            '    2) Select model' /
     3            '    3) Edit data' /
     4            '    4) Clauser chart => friction velocity' /
     5            '    5) Minimize error => friction velocity' /
     6            '    6) Adjust sensor position' /
     7            '    7) Plot non-dimensional profile' )
          item = Jresp( STD_IN, 0, 7, 4 )
c
c               EXIT
c               ****
c           Retrieve the friction velocity . For Coles (1968) and Bull (1969)
c           models, also retrieve the wake_strength and the boundary layer
c           thickness .
c
          if ( item .eq. 0 ) then
              call stau_blm_err( .FALSE. )
              call set_BLM( BLM_nondim, 0.0 )
              qu_tau = get_BLM( BLM_friction_vel )
              call stau_blm_err( .TRUE. )
              if ( get_BLM( BLM_68Coles ) .gt. 1.0 .or.
     1             get_BLM( BLM_MKBull  ) .gt. 1.0 .or.
     2             get_BLM( BLM_BG_68C  ) .gt. 1.0 .or.
     3             get_BLM( BLM_Spld_68C ) .gt. 1.0 ) then
                  wake_strength = get_BLM( BLM_PI_wake )
                  call stau_blm_err( .TRUE. )
                  delta_plus = get_BLM( BLM_delta )
                  call stau_blm_err( .TRUE. )
                  end if
              fs_speed = get_BLM( BLM_fs_speed )
              call stau_blm_err( .TRUE. )
              if ( fs_speed / 60.0 .gt. qu_tau .and.
     1             fs_speed / 15.0 .lt. qu_tau ) then
                  write( STD_OUT, 12 ) qu_tau,
     1                name_BLM( BLM_friction_vel )
12                format( ' QU_TAU : WARNING : Strange value of ', a,
     1                    g13.6 )
                  end if
              return
          else if ( item .eq. 1 ) then
c
c               RESTORE ALL DATA
c               ****************
c
              src = 1
              n_data = min( MAX_DATA, n_points )
              do dest = 1, n_data
                  u_data(dest) = u_mean(src)
                  y_data(dest) = y(src) / 1000.0
                  src = src + increm
                  end do
c
c               SELECT MODEL
c               ************
c
          else if ( item .eq. 2 ) then
              call stau_sel_model()
c
c               Edit data
c               *********
c
          else if ( item .eq. 3 ) then
              call stau_edit()
c
c               Friction velocity. Use Clauser chart
c               ************************************
c
          else if ( item .eq. 4 ) then
              call stau_clauser()
c
c               Friction velocity. Minimize error
c               *********************************
c
          else if ( item .eq. 5 ) then
              call stau_optimize()
c
c               Shift sensor location
c               *********************
c
          else if ( item .eq. 6 ) then
              write( STD_OUT, 17 )
17            format( ' How much to add to sensor location ? (mm) ', $ )
              y_offset = Qresp( STD_IN, -5.0, 5.0, 4 ) / 1000.0
              do 18 src = 1, n_data
                  y_data(src) = y_data(src) + y_offset
18                continue

          else if ( item .eq. 7 ) then
              call stau_profile()
              end if
          goto 10
c     end of loop
c
15    stop 'EOF'
16    stop 'error reading capture file'
      end




      SUBROUTINE STAU_BLM_ERR( PRINT )
c
c Prints any error message from the BLM package if PRINT is .TRUE. .
c If PRINT is .FALSE., error_BLM is called to reset .
c
c Argument
c  logical print        # .TRUE. = print error message / .FALSE. = reset
c
c logical ok            # result of error_BLM
c         error_BLM     # function : report BLM error message
c integer str_length    # function : non-blank string length
c character message*80  # error message printed
c
      include 'stdio.def'
c
      logical error_BLM, ok, print
      integer str_length
      character message*80
c
      if ( print ) then
          if ( error_BLM( message ) ) then
              write( STD_OUT, 1 ) message(1:str_length(message))
1             format( 1x, a )
              end if
      else
          ok = error_BLM( message )
          end if
      return
      end



      SUBROUTINE STAU_SEL_MODEL()
c
c Select the boundary layer model parameters
c
c Variables
c  integer imodel         # selected boundary-layer model
c          item           # item selected from model
c          pf_len         # length of print file name
c          pf_lu          # logical unit number of print file
c          Jresp          # function : integer response
c          new_lun        #          : get new logical unit number
c          open_file      #          : open file
c          str_length     #          : non blank length of string
c  real    get_BLM        # function : get parameter from boundary-layer model
c  character print_file*64    # name of print file
c            name_BLM*24      # function : name of BLM parameter
c  subroutine
c    set_BLM              # set boundary layer model parameter
c    prin_BLM             # print all boundary layer model parameters
c
c
      include 'stdio.def'
      include 'blm.def'
      integer MAX_TRIES, N_MODELS
      parameter( MAX_TRIES=4, N_MODELS=8 )
c
      integer imodel, item, pf_len, pf_lu, model_list(8),
     1        Jresp, new_lun, open_file, str_length
      real get_BLM
      character print_file*64, name_BLM*24
      data pf_len, pf_lu / 1, STD_OUT /
      data model_list /
     1       BLM_Clauser, BLM_56Coles, BLM_68Coles, BLM_MKBull,
     2       BLM_Beattie, BLM_BG_68C, BLM_Spalding, BLM_Spld_68C /
      data print_file / ' ' /
c
      pf_len = max( 1, str_length( print_file ) )
c     start loop
c
c           print the menu
c           **************
c
1         call newscr()
          call ref_BLM( STD_OUT )
          write( STD_OUT, 2 ) name_BLM( int( get_BLM( BLM_BLmodel ) ) ),
     1        print_file(1:pf_len)
2         format( ' Select model' / '   0) Exit' /
     1        '   1) Change type / formula: ', a /
     2        '   2) Change constants' /
     3        '   3) Change optimizing parameters' /
     4        '   4) Print file: ', a /
     5        '   5) Print model' )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
          if ( item .eq. 0 ) then
              call stau_blm_err( .FALSE. )
              return
c
c           Select the model type
c           *********************
c
          else if ( item .eq. 1 ) then
              write( STD_OUT, 3 )   ( imodel,
     1            name_BLM( model_list(imodel) ), imodel = 1, N_MODELS )
3             format( ' Select model' /
     1            ( '  ', i2, ' ) ', a ) )
              imodel = Jresp( STD_IN, 1, N_MODELS, MAX_TRIES )
              call set_BLM( model_list(imodel), 0.0 )
c
c           Change constants
c           ****************
c
          else if ( item .eq. 2 ) then
              call stau_sel_const()
c
c           Change optimizing parameters
c           ****************************
c
          else if ( item .eq. 3 ) then
              call stau_sel_opt()
c
c                 get name of print file
c                 **********************
c
          else if ( item .eq. 4 ) then
              if ( pf_lu .ne. 0 .and. pf_lu .ne. STD_OUT ) then
                  close( UNIT=pf_lu )
                  end if
              write( STD_OUT, 4 )
4             format( ' Name of print file ? ' )
              call Cresp( STD_IN, print_file, MAX_TRIES )
              if ( print_file .eq. ' ' .or. print_file .eq. 'vdu' ) then
                  pf_lu = STD_OUT
              else
                  pf_lu = open_file( new_lun(), print_file, 'new', 0 )
                  end if
              pf_len = max( 1, str_length( print_file ) )
c
c                 Print details of boundary layer model
c                 **************************************
c
          else if ( item .eq. 5 ) then
              if ( pf_lu .ne. 0 ) then
                  call prin_BLM( pf_lu )
                  call ilgwait()
                  end if
              end if
          goto 1
c         end of loop
c
      end



      SUBROUTINE STAU_SEL_CONST()
c
c Select the values of the constants for the boundary layer model
c
c Constants
c   N_PARAMS=7
c Variables
c  integer i,             # parameter list index
c          iconst,        # specifies the selected constant
c          param_list(7), # list of BLM parameters which can be changed
c          Jresp          # function : integer response
c  real get_BLM           # function : get boundary layer model parameter
c       Qresp             #          : real response
c  character Cdimens(0:1)*15  # 'non-dimensional' 'dimensioned'
c            name_BLM*24      # function : name of BLM parameter
c
      include 'stdio.def'
      include 'blm.def'
      integer N_PARAMS
      parameter( N_PARAMS=7 )
c
      integer i, iconst, param_list(N_PARAMS), Jresp
      real get_BLM, Qresp
      character Cdimens(0:1)*15, name_BLM*24
      data Cdimens / 'non-dimensional', 'dimensioned    ' /
c
      data param_list / BLM_Karmen, BLM_Constant, BLM_fs_speed,
     1    BLM_kin_visc, BLM_friction_vel, BLM_PI_wake, BLM_delta /
c
      call set_BLM( BLM_nondim, 0.0 )
c
c     start of loop
1        call newscr()
         write( STD_OUT, 2 )  ( i, name_BLM( param_list(i) ),
     1       get_BLM( param_list(i) ), i = 1, N_PARAMS ),
     2       N_PARAMS + 1, Cdimens( int( get_BLM( BLM_dimens ) ) )
2        format( ' Select constant' / '   0) Exit' /
     1       7( i4, ') ', a, g12.5 / )
     2          i4, ') ', a, ' input values    ', $ )
          iconst = Jresp( STD_IN, 0, N_PARAMS + 1, 4 )
c
c             return
c
          if ( iconst .eq. 0 ) then
              call stau_blm_err( .FALSE. )
              return
c
c           Karmen constant .... kinematic viscosity
c
          else if ( iconst .ge. 1 .and. iconst .le. N_PARAMS ) then
              write( STD_OUT, 3 ) name_BLM( param_list(iconst) )
3             format( 1x, a, '? ', $ )
              call set_BLM( param_list( iconst ),
     1                      Qresp( STD_IN, 0.0, 1000000.0, 4 ) )
c
c                 dimensioned / non-dimensional switch
c
          else if ( iconst .eq. N_PARAMS + 1 ) then
c                     to non-dimensional
              if ( get_BLM( BLM_dimens ) .gt. 0.5 ) then
                  call set_BLM( BLM_nondim, 0.0 )
c                     to dimensioned
              else if ( get_BLM( BLM_nondim ) .gt. 0.5 ) then
                  call set_BLM( BLM_dimens, 0.0 )
                  end if
              end if
          goto 1
c         end of loop
      end



      SUBROUTINE STAU_SEL_OPT()
c
c Select the values of the optimizing  parameters for the boundary layer model
c
c Constants
c   N_PARAMS=7
c Variables
c  integer ipar,              # parameter index
c          iopt,              # optimize variable index to BLM
c          n_opt,             # number of optimizing variables
c       opt_list(N_PARAMS),   # =1 if the corresponding BLM parameter in
c                             # param_list is an optimizing variable (else =0)
c     param_list(N_PARAMS),   # list of BLM indices to parameters
c          Jresp              # function : integer response
c  real get_BLM
c  character Cnoyes(0:1)*3
c            name_BLM*24      # function : name of BLM parameter
c  subroutine set_BLM
c
      include 'stdio.def'
      include 'qu_tau.def'
      include 'blm.def'
      integer N_PARAMS
      parameter( N_PARAMS=7 )
c
      integer ipar, iopt, n_opt, opt_list(N_PARAMS),
     1   param_list(N_PARAMS), Jresp
      real get_BLM
      character Cnoyes(0:1)*3, name_BLM*24
c
c
c         N.B. The order of parameters in this list is important . The two
c         which can depend upon the others are placed last so that the
c         correct values are assigned in QTAU_ERR .
c
      data param_list / BLM_Karmen, BLM_Constant, BLM_fs_speed,
     1    BLM_kin_visc, BLM_friction_vel, BLM_PI_wake, BLM_delta /
      data Cnoyes / 'no ', 'yes' /
c
      n_opt = int( get_BLM( BLM_n_opt ) )
      do 2 ipar = 1, N_PARAMS
          opt_list(ipar) = 0
          do 1 iopt = 1, n_opt
              if ( int( get_BLM( BLM_n_opt + iopt ) ) .eq.
     1                param_list(ipar) ) then
                  opt_list(ipar) = 1
                  end if
1             continue
2         continue

c     start of loop
3        call newscr()
         write( STD_OUT, 4 ) ( ipar, name_BLM( param_list(ipar) ),
     1       Cnoyes( opt_list(ipar) ), ipar = 1, N_PARAMS )
4        format( ' Select optimizing variables' / '   0) Exit'
     1       7( / i4, ') ', a, 1x, a ) '  ', $ )
          ipar = Jresp( STD_IN, 0, N_PARAMS, 4 )
c
c           Return; rebuild the optimizing variables in the BLM package .
c           ************************************************************
c           First count the optimizing variables .
c

          if ( ipar .eq. 0 ) then
              n_opt = 0
              do 5 ipar = 1, N_PARAMS
                  if ( opt_list(ipar) .eq. 1 )
     1                n_opt = min( n_opt + 1, MAX_OPTIM )
5                 continue
              iopt = 1
c
c           Set the number of optimizing variables each time a variables
c           is set -- because set_BLM can compress out duplicated variables
c
              do 6 ipar = 1, N_PARAMS
                  if ( opt_list(ipar) .eq. 1 .and.
     1                    iopt .le. MAX_OPTIM ) then
                      call set_BLM( BLM_n_opt, real( n_opt ) + 0.1 )
                      call set_BLM( BLM_n_opt + iopt,
     1                              real( param_list(ipar) ) + 0.1 )
                      iopt = iopt + 1
                      end if
6                 continue
              call stau_blm_err( .FALSE. )
              return
c
c           Karmen constant .... kinematic viscosity
c           ****************************************
c
          else if ( ipar .ge. 1 .and. ipar .le. N_PARAMS ) then
              opt_list(ipar) = 1 - opt_list(ipar)
              end if
          goto 3
c         end of loop
      end



      SUBROUTINE STAU_EDIT()
c
c       Edit the experimental velocity profile data .
c
c Constants
c  MODEL_POINTS=200     # number of points plotted for the model profile
c Variables
c  logical model_ok     # nondimensional model velocity can be calculated
c          error_BLM    # function : get error status of BLM
c  integer i            # index for data arrays
c  real y_min, y_max,   # limits of probe location
c       u_min, u_max,   # limits of mean velocity
c       y_model(MODEL_POINTS) # array of abscissa values
c       u_model(MODEL_POINTS) # array of ordinate values
c       y_step          # abscissa increment
c       Uplus_BLM       # function : calculate velocity according to
c                       #            boundary layer model
c       get_BLM         #          : get boundary layer model parameter
c  character message*80 # error message
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
      integer MODEL_POINTS
      parameter( MODEL_POINTS=100 )
c
      logical model_ok, error_BLM
      integer i
      real y_min, y_max, u_min, u_max, y_step,
     1     y_model(MODEL_POINTS), u_model(MODEL_POINTS),
     2     Uplus_BLM, get_BLM
c
      include 'qu_tau.cmn'
c
c           clear any BLM errors -- this routine tests for BLM errors,
c           and we don't want to catch any from outside
c
      call stau_blm_err( .FALSE. )
c
c           find limits of the experimental data
c
      y_min = max( y_data(1), 0.0 )
      y_max = y_min
      u_min = u_data(1)
      u_max = u_data(1)
      do  1 i = 2, n_data                                                 1b
          y_min = min( y_min, y_data(i) )
          y_max = max( y_max, y_data(i) )
          u_min = min( u_min, u_data(i) )
          u_max = max( u_max, u_data(i) )
1         continue                                                        1e
c
c           switch BLM package to DIMENSIONED
c           i.e. produce dimesioned velocity profiles
c
      call set_BLM( BLM_dimens, 0.0 )
c
c
c           only plot the model profile if the derived constants
c           have not been cleared (i.e. have been calculated)
c
      if ( get_BLM( BLM_clear ) .lt. 0.5 ) then                           1b
c
c           find model ordinate at y_min * 0.9 and y_max * 1.1
c           and adjust the limits accordingly
c
          y_min = 0.9 * y_min
          y_max = 1.1 * y_max
          u_min = min( Uplus_BLM( 0.9 * y_min ), u_min )
          u_max = max( Uplus_BLM( 1.1 * y_max ), u_max )
          model_ok = .not. error_BLM( message )
          if ( model_ok ) then                                            2b
c
c       if the model profile can be calculated
c       calculate the coordinates for logarithmic axis
c
              y_model(1) = y_min
              u_model(1) = Uplus_BLM( y_min )
              y_step = ( y_max / y_min )
     1            ** ( 1.0 / real( MODEL_POINTS - 1 ) )
              do 2 i = 2, MODEL_POINTS                                    3b
                  y_model(i) = y_model(i-1) * y_step
                  u_model(i) = Uplus_BLM( y_model(i) )
2                 continue                                                3e
          else                                                            2m
              write( STD_OUT, 3 ) message
3             format( 1x, a / 20x, ' Cannot plot model profile' )
              end if                                                      2e
      else
          write( STD_OUT, 4 )
4         format( ' Model is cleared : Cannot plot model profile' )
          end if                                                          1e
c
c       set up plotting parameters
c
      call ilgopen( ' ' )
      call ilgslaw( 'log', 'lin' )
      call ilgimit( y_min, u_min, y_max, u_max )
      call ilgxlab( 'distance from wall (mm)' )
      call ilgylab( 'air speed (m/s)' )
c
c       plot the model data as a solid line (label axes)
c
      if ( get_BLM( BLM_clear ) .lt. 0.5 .and. model_ok ) then            1b
          call ilgline( 0, 1, 1 )
          call ilgdata( y_model, u_model, 1, MODEL_POINTS, 0 )
          end if                                                          1e
c
c       edit the experimental data
c
      call sfudge( y_data, u_data, 1, n_data, n_data, MAX_DATA )
      call ilgclos()
      call stau_blm_err( .FALSE. )
      return
      end



      SUBROUTINE STAU_PROFILE()
c
c Plot the non-dimensional velocity profile according to the current boundary
c layer model, and on the same set of axes, plot the experimental data .
c
c logical error_BLM   # function : report BLM error
c integer i       # array index
c     str_length  # function : non-blank string length
c real frict_vel  # friction velocity
c      kin_visc   # kinematic viscosity
c      y_max      # abscissa maximum
c      y_min      # abscissa minimum
c      y_step     # abscissa increment
c      y_model(MAX_DATA)  # array of non-dimensional abscissa values
c      u_model(MAX_DATA)  # array of non-dimensional ordinate values
c      Uplus_BLM  # function : velocity profile from boundary layer model
c      get_BLM    #          : get BLM parameter
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
      logical error_BLM
      integer i, str_length
      real frict_vel, kin_visc, y_max, y_min, y_step, Uplus_BLM,
     1     get_BLM
      include 'qu_tau.cmn'
      real y_model(MAX_DATA), u_model(MAX_DATA)
c
      frict_vel = get_BLM( BLM_friction_vel )
      kin_visc  = get_BLM( BLM_kin_visc )
      if ( error_BLM( message ) ) then
          write( STD_OUT, 3 ) message(1:str_length(message))
3         format( 1x, a / '  Cannot plot non-dimensional profiles' )
          return
          end if
c
c       Plot the experimental data
c
      y_min =  1.0e30
      y_max = -1.0e30
      do 1 i = 1, n_data
          y_model(i) = y_data(i) * frict_vel / kin_visc
          u_model(i) = u_data(i) / frict_vel
          y_min = min( y_min, y_model(i) )
          y_max = max( y_max, y_model(i) )
1         continue
      call ilgopen( ' ' )
      call ilgslaw( 'log', 'lin' )
      call ilgline( 1, 0, 1 )
      call ilgdata( y_model, u_model, 1, n_data, 1 )
c
c       Plot the model profile
c
      call set_BLM( BLM_nondim, 0.0 )
      y_step = ( y_max / y_min ) ** ( 1.0 / real( MAX_DATA - 1 ) )
      y_model(1) = y_min
      u_model(1) = Uplus_BLM( y_min )
      do 2 i = 2, MAX_DATA
          y_model(i) = y_model(i-1) * y_step
          u_model(i) = Uplus_BLM( y_model(i) )
2         continue
      call ilgline( 0, 1, 0 )
      call ilgdata( y_model, u_model, 1, MAX_DATA, 0 )
c
c       Cosmetic details
c
      call ilgxlab( 'y|t+' )
      call ilgylab( 'U|t+' )
      call ilgnote( '|r |4-1 model|r   |s1  data' )
      call ilgwait()
      call ilgclos()
      return
      end



      SUBROUTINE STAU_CLAUSER()
c
c Find the friction velocity by the graphical Clauser chart method .
c
c constants
c  MODEL_POINTS=100       # number of points in a model profile
c  EXPER_POINTS=100       # maximum number of experimental data points
c
c variables
c  logical
c    got_prev             # previous value of friction velocity exists
c    drawn_model          # the boundary layer model is drawn
c    got_reqd_params      # params required to calculate B/L model are available
c    error_BLM            # function : get error status of BLM
c    etau_reqd_params     #          : are all required parameters available
c  integer
c    i                    # array index
c    i_var                # parameter ID of the optimizing variable
c    iselect              # selected menu item
c    Jresp                # function : interger response
c    str_length           #          : length of string
c  real
c    free_stream          # free stream velocity
c    kinem_visc           # kinematic viscosity
c    prev_frict_vel       # previous friction velocity
c    y_exper(EXPER_POINTS), u_exper(EXPER_POINTS)
c        # experimental velocity profile -- scaled for Clauser chart
c    y_model(MODEL_POINTS), u_model(MODEL_POINTS)
c        # velocity profile calculated according to the boundary layer model
c    y_min, y_max         # minimum and maximum abscissa values
c    u_min, u_max         # minimum and maximum ordinate values
c    get_BLM              # function : get boundary layer model parameter
c    Qresp                #          : real response
c character missing_param*24 # name of missing parameter
c           name_BLM*24   # function : name of BLM parameter
c
c  subroutines
c    ilgopen, ilgline, ilgdata, ilginvs   # ilglib : plotting
c    set_BLM              # set boundary layer model parameters
c    stau_Clauser_model   # calculate boundary layer profile from model in
c                         #     Clauser chart coordinates
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
      integer MODEL_POINTS, EXPER_POINTS
      parameter( MODEL_POINTS=100, EXPER_POINTS=100 )
c
      logical drawn_model, got_reqd_params, unused,
     1        error_BLM, etau_clauser_model, etau_reqd_params
      integer i, i_var, iselect, Jresp, str_length
      real free_stream, kinem_visc, prev_frict_vel,
     1     y_exper(EXPER_POINTS), u_exper(EXPER_POINTS),
     2     y_model(MODEL_POINTS), u_model(MODEL_POINTS),
     3     u_min, u_max, y_min, y_max, get_BLM, Qresp
      character missing_param*24, name_BLM*24
      include 'qu_tau.cmn'
      save i_var
      data i_var / 0 /
c
c       Check that sufficient information is available to draw the chart,
c       but first clear any pre-existing errors
c
      call stau_blm_err( .FALSE. )
      free_stream = get_BLM( BLM_fs_speed )
      kinem_visc  = get_BLM( BLM_kin_visc )
      if ( error_BLM( message ) )
     1    goto 6
      drawn_model = .FALSE.
c
c       plot the experimental data as
c       ( y_data * free_stream / kinem_visc,  u_data / free_stream )
c
      y_min = 1.0e36
      y_max = -1.0e36
      u_min = 1.0e36
      u_max = -1.0e36
      do 1 i = 1, n_data
          y_exper(i) = y_data(i) * free_stream / kinem_visc
          u_exper(i) = u_data(i) / free_stream
          y_min = min( y_exper(i), y_min )
          y_max = max( y_exper(i), y_max )
          u_min = min( u_exper(i), u_min )
          u_max = max( u_exper(i), u_max )
1         continue
      y_max = 1.1 * y_max
      u_max = 1.1 * u_max
      y_min = 0.9 * y_min
      u_min = 0.9 * u_min
c
      call ilgopen( ' ' )
      call ilgslaw( 'log', 'lin' )
      call ilgline( 1, 0, 1 )
      call ilgwild( 'UU', y_min, u_min, y_max, u_max )
      call ilgimit( y_min, u_min, y_max, u_max )
      call ilgdata( y_exper, u_exper, 1, n_data, 0 )
c
c     plot the model profile for the previous friction velocity value
c
      call ilgline( 0, 1, 1 )
      got_reqd_params = get_BLM( BLM_reqd_par ) .gt. 0.5
      if ( got_reqd_params ) then
          if ( .not. etau_Clauser_model( y_model, u_model, MODEL_POINTS,
     1                               y_min, y_max ) )
     2        goto 6
          call ilgdata( y_model, u_model, 1, MODEL_POINTS, 0 )
          drawn_model = .TRUE.
      else
          unused = error_BLM( missing_param )
          end if
c
c       How many variables are there ?
c
      call set_BLM( BLM_nondim, 0.0 )
      n_opt_pars = int( get_BLM( BLM_n_opt ) )
      if ( n_opt_pars .le. 0 ) then
          message = 'No optimizing parameters'
          goto 6
          end if
      if ( i_var .eq. 0 ) then
          i_var = int( get_BLM( BLM_n_opt + 1 ) )
          end if
      if ( error_BLM( message ) )
     1    goto 6
c
c       Main optimizing loop
c       ********************
c
c     start of loop
2         call newscr()
          call set_BLM( BLM_nondim, 0.0 )
          if ( .not. got_reqd_params ) then
              write( STD_OUT, 8 )
     1            missing_param(1:str_length(missing_param))
8             format( ' WARNING : Cannot calculate profile : ',
     1                a, ' unavailable' )
              end if
          write( STD_OUT, 3 ) name_BLM( i_var ), get_BLM( i_var )
3         format( ' 0) Exit' / ' 1) Value of ', a, ' (', g12.6e1, ')' /
     1        ' 2) Select variable ', $ )
          iselect = Jresp( STD_IN, 0, 2, 4 )
c
c           return -- clear the graph first
c
          if ( iselect .eq. 0 ) then
              call ilgclos()
              return
c
c           select the optimizing variable
c
          else if ( iselect .eq. 2 ) then
              write( STD_OUT, 4 ) ( i, name_BLM( int( get_BLM(
     1            BLM_n_opt + i ) ) ), i = 1, n_opt_pars )
4             format( ' Select' / '    0) Skip' / ( i5, ') ', a ) )
              i = Jresp( STD_IN, 0, n_opt_pars, 4 )
              if ( 0 .lt. i .and. i .le. n_opt_pars ) then
                  i_var = int( get_BLM( BLM_n_opt + i ) )
                  end if
c
c           get the value of the optimizing variable
c
          else if ( iselect .eq. 1 ) then
              write( STD_OUT, 5 ) name_BLM( i_var )
5             format( 1x, a, '?', $ )
              call set_BLM( i_var, Qresp( STD_IN, 0.0, 1.0e5, 4 ) )
              got_reqd_params = get_BLM( BLM_reqd_par ) .gt. 0.5
              if ( got_reqd_params ) then
c
c           erase the most recent model curve
c
                  if ( drawn_model ) then
                      call ilginvs()
                      call ilgdata( y_model, u_model, 1, MODEL_POINTS,
     1                              0 )
                      end if
c
c           replot experimental data
c
                  call ilgline( 1, 0, 1 )
                  call ilgdata( y_exper, u_exper, 1, n_data, 0 )
                  call ilgline( 0, 1, 1 )
c
c           plot new model curve
c
                  if ( .not. etau_Clauser_model( y_model, u_model,
     1                     MODEL_POINTS, y_min, y_max ) )
     2                goto 6
                  call ilgdata( y_model, u_model, 1, MODEL_POINTS, 0 )
                  drawn_model = .TRUE.
              else
                  unused = error_BLM( missing_param )
                  end if
              end if
          goto 2
c         end of loop
c
c       abort with error
c
6     call ilgclos()
      write( STD_ERR, 7 ) message
7     format( 1x, a / 10x, 'Cannot make Clauser chart' )
      call stau_blm_err( .FALSE. )
      return
      end




      LOGICAL FUNCTION ETAU_CLAUSER_MODEL( Y_MODEL, U_MODEL, N_POINTS,
     1                               Y_MIN, Y_MAX )
c
c Calculate the velocity profile according to the currently defined boundary
c layer model . Y_MIN and Y_MAX are the abscissa minimum and maximum
c respectively . N_POINTS data points, with abscissa values distributed
c uniformly on a logarithmic scale, are plotted .
c
c Arguments
c  real y_model(*), u_model(*)
c                     # OUT # velocity profiles from boundary layer model
c  integer n_points   # IN  # number of coordinates to be plotted
c  real y_min         # IN  # minimum abscissa value
c       y_max         # OUT # maximum abscissa value
c
c  All units are those shown on the Clauser chart .
c
c Variables
c  logical error_BLM  # function : report error status of BLM
c  integer i          # array index
c  real free_stream   # free stream air speed
c       kinem_visc    # kinematic viscosity
c       y_min, y_max  # abscissa limits for plotting
c       y_step        # abscissa increment
c       get_BLM       # function : get boundary layer model parameters
c       Uplus_BLM     #          : calculate mean velocity from boundary
c                     #            layer model
c  subroutine
c       set_BLM       # set a parameter for the boundary layer model
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
c
      logical error_BLM
      integer i, n_points
      real free_stream, kinem_visc, y_min, y_max, y_step,
     1     y_model(*), u_model(*), get_BLM, Uplus_BLM
      include 'qu_tau.cmn'
c
c       get required boundary layer parameters
c
      call stau_blm_err( .FALSE. )
      call set_BLM( BLM_dimens, 0.0 )
      free_stream = get_BLM( BLM_fs_speed )
      kinem_visc  = get_BLM( BLM_kin_visc )
c
c       calculate the velocity profile
c       N.B. arguments and return values are dimensioned (SI units)
c
      y_model(1) = y_min
      u_model(1) = Uplus_BLM( y_min * kinem_visc / free_stream ) /
     1             free_stream
      if ( error_BLM( message ) ) then
          etau_clauser_model = .FALSE.
          return
          end if
      y_step = ( y_max / y_min ) ** ( 1.0 / real( n_points - 1 ) )
      do 2 i = 2, n_points
          y_model(i) = y_model(i-1) * y_step
          u_model(i) = Uplus_BLM( y_model(i) * kinem_visc /
     1                            free_stream ) / free_stream
2         continue
c
      etau_clauser_model = .not. error_BLM( message )
      return
      end



      REAL FUNCTION QTAU_ERR( OPT_VAR )
c
c Calculate the sum square difference between the experimental data and
c the boundary layer model for the current model settings and those
c specified by the parameter values in  OPT_VAR .
c    The number of optimizing variables N_OPT_VARS and the identities
c (addresses) of the optimizing variables OPT_PAR are specified in
c the common block TAU_OPT_CMN .
c
c Argument
c  integer opt_var(*)
c
c Variables
c  integer i            # index for optimizing variables and experimental data
c  real Uplus_BLM       # function : boundary layer profile velocity profile
c
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
c
      integer i
      real opt_var(*), Uplus_BLM
      include 'qu_tau.cmn'
c
c           Set the optimize variables . Assume the optimize
c           variables have been set up correctly.
c
      do 1 i = 1, n_opt_pars
          call set_BLM( opt_par(i), opt_var(i) )
1         continue
c
c           Calculate the sum square difference between the
c           experimental data and the boundary layer model
c
      qtau_err = 0.0
      do 2 i = 1, n_data
          qtau_err = qtau_err +
     1        ( Uplus_BLM( y_data(i) ) - u_data(i) ) **2
2         continue
      return
      end



      SUBROUTINE STAU_OPTIMIZE()
c
c Calculate the missing boundary layer model parameters by optimization .
c
c Constants
c   MAX_ITER    = 1000    # Maximum number optimizing iterations
c   MAX_VERTICES= MAX_OPTIM+1  # maximum number of polyhedron vertices
c   WORK_SIZE   = MAX_OPTIM*8  # size of work array
c   X_TOLERANCE = 1.0e-5  # domain tolerance for optimization
c Variables
c  logical
c    check_opt_pars       # function : check optizing parameters
c  integer
c    i                    # indexing variable
c    id_var               # ID of optimizing variable
c    iterations           # actual number of iterations performed
c  real
c    fit_errs(BLM_max_opt)    # final fitting errors ( after optimization )
c    polyhed(BLM_max_opt,MAX_VERTICES)  # optimizing polyhedron :
c                         # Each column is one vertex
c    result               # solution of one-dimensional optimization
c    work(WORK_SIZE)      # work array for multi-dimensional optization
c    get_BLM              # function : get BLM parameter
c    qdscop               #          : one-dimensional optimization
c    qmean                #          : mean of an array
c    qtau_err             #          : calculates fitting error over the data
c  character
c    name_BLM             # function : name of BLM parameter
c  subroutine
c    amoeba               # multidimensional optimization
c    stau_init_pars       # initial optimizing parameters
c    set_BLM              # set BLM parameter
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
      include 'amoeba.def'
      integer MAX_ITER, MAX_VERTICES, WORK_SIZE
      real    X_TOLERANCE
      parameter( MAX_ITER=1000, MAX_VERTICES=MAX_OPTIM+1,
     1           WORK_SIZE=MAX_OPTIM*4, X_TOLERANCE=1.0e-5 )
c
      logical check_opt_pars
      integer i, id_var, iterations
      real fit_errs(BLM_max_opt), polyhed(BLM_max_opt,MAX_VERTICES),
     1     result, work(WORK_SIZE), get_BLM, qdscop, qmean, qtau_err
      character name_BLM*24
      external qtau_err
c
      include 'qu_tau.cmn'
c
      call set_BLM( BLM_dimens, 0.0 )
      n_opt_pars = int ( get_BLM( BLM_n_opt ) )
      call stau_blm_err( .FALSE. )
c
c       OPTIMIZE ONLY ONE VARIABLE
c       **************************
c
      if ( n_opt_pars .eq. 1 ) then                                       1b
          if ( check_opt_pars() ) then                                    2b
              call stau_init_pars( polyhed, BLM_max_opt )
              result = qdscop( polyhed(1,1), qtau_err,
     1            abs( polyhed(1,1) - polyhed(1,2) ),
     2            X_TOLERANCE, .FALSE. )
              id_var = int( get_BLM( BLM_n_opt + 1 ) )
              call set_BLM( id_var, result )
              result = sqrt( qtau_err( result ) / real( n_data ) ) /
     1                 get_BLM( BLM_friction_vel )
              call set_BLM( BLM_nondim, 0.0 )
              write( STD_OUT, 1 ) result, name_BLM( id_var ),
     1            get_BLM( id_var )
1             format( ' minimum RMS fitting error ', f9.6 /
     1                ' for ', a, ' = ', g13.6 )
              end if                                                      2e
c
c       OPTIMIZE MORE THAN ONE VARIABLE
c       *******************************
c
      else if ( 1 .lt. n_opt_pars .and. n_opt_pars .le. BLM_max_opt )
     1        then                                                        1m
          if ( check_opt_pars() ) then                                    2b
              call stau_init_pars( polyhed, BLM_max_opt )
              call amoeba( qtau_err, fit_errs, n_opt_pars,
     1            polyhed, BLM_max_opt, MAX_ITER, iterations,
     2            POLY_SIZE + ABS_FUNC,
     3            ( get_BLM( BLM_fs_speed) / 10000.0 ) **2,
     4            X_TOLERANCE, work )
c
c       the result is the centroid of the final polyhedron
c
              do 2 i = 1, n_opt_pars                                      3b
                  call set_BLM( int( get_BLM( BLM_n_opt + i ) ),
     1              qmean( polyhed(i,1), BLM_max_opt, n_opt_pars + 1 ) )
2                 continue                                                3e
              write( STD_OUT, 3 ) iterations,
     1            sqrt( qmean( fit_errs, 1, n_opt_pars ) /
     2                real( n_data ) ) / get_BLM( BLM_friction_vel )
3             format( i5, ' iterations' /
     1                ' minimum RMS fitting error ', f9.6 )
              call set_BLM( BLM_nondim, 0.0 )
              do 5 i = 1, n_opt_pars                                      3b
                  id_var = get_BLM( BLM_n_opt + i )
                  write( STD_OUT, 4 )
     1                name_BLM( id_var ), get_BLM( id_var )
4                 format( ' for ', a, ' = ', g13.6 )
5                 continue                                                3e
              end if                                                      2e
c
c       TRAPS
c
      else if ( n_opt_pars .le. 0 ) then                                  1m
          write( STD_ERR, 6 )
6         format( ' QU_TAU : WARNING : No variables to optimize' )
      else                                                                1m
          write( STD_ERR, 7 ) n_opt_pars
7         format( ' QU_TAU : WARNING : ', i12,
     1        ' is too many variables to optimize' )
          end if                                                          1e
      call stau_blm_err( .TRUE. )
      return
      end



      LOGICAL FUNCTION CHECK_OPT_PARS()
c
c Extract the information on the number and identity of optimizing variables
c from the BLM package and place it in the common block so that QTAU_ERR
c can get it quickly . Also check that all the required values are available
c and that the optimizing variables are OK .
c
c Variables
c  logical etau_check_par   # function : check one parameter
c  integer i                # index for list of optimizing variables
c  real get_BLM             # function : get boundary layer model parameter
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
c
      logical etau_check_par
      integer i
      real get_BLM
c
      include 'qu_tau.cmn'
c
c       Transfer IDs of optimizing variables into ready-use array
c
      call set_BLM( BLM_dimens, 0.0 )
      n_opt_pars = int( get_BLM( BLM_n_opt ) )
      do 1 i = 1, n_opt_pars
          opt_par(i) = int( get_BLM( BLM_n_opt + i ) )
1         continue
c
c       check that parameters which are not optimizing variables
c       are available
c
      check_opt_pars = etau_check_par( BLM_Constant )
      check_opt_pars = check_opt_pars .and.
     1    etau_check_par( BLM_Karmen )
      check_opt_pars = check_opt_pars .and.
     1    etau_check_par( BLM_fs_speed )
      check_opt_pars = check_opt_pars .and.
     1    etau_check_par( BLM_kin_visc )
      check_opt_pars = check_opt_pars .and.
     1    etau_check_par( BLM_friction_vel )
      if ( get_BLM( BLM_68Coles ) .gt. 0.5 .or.
     1     get_BLM( BLM_MKBull )  .gt. 0.5 ) then
          check_opt_pars = check_opt_pars .and.
     1        etau_check_par( BLM_PI_wake )
          check_opt_pars = check_opt_pars .and.
     1        etau_check_par( BLM_delta )
          end if
      return
      end



      LOGICAL FUNCTION ETAU_CHECK_PAR( IDENT_PARAM )
c
c If the BLM parameter IDENT_PARAM is not an optimizing variable, it must
c be available for use . If a variable is neither optimizing nor available,
c a .FALSE. value is returned, otherwise the returned value is .TRUE. .
c
c Argument
c  integer ident_param  # variable identifier to be checked
c
c Variables
c  logical error_BLM    # function : reports any BLM errors
c  integer i            # index for stepping through optimizing variables
c  real r_junk          # somewhere to put the returned value of GET_BLM
c       get_BLM         # function : get boundary layer model parameter
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
c
      logical error_BLM
      integer i, ident_param
      real r_junk, get_BLM
      include 'qu_tau.cmn'
c
c       Return .TRUE. if the variable is for optimizing
c
      do 1 i = 1, n_opt_pars
          if ( ident_param .eq. int( get_BLM( BLM_n_opt + i ) ) ) then
              etau_check_par = .TRUE.
              return
              end if
1         continue
c
c       Find out if the variable is available
c
      r_junk = get_BLM( BLM_Constant )
      etau_check_par = .not. error_BLM( message )
      if ( .not. etau_check_par ) then
          write( STD_ERR, 2 ) message
2         format( 1x, a, ' for optimization' )
          end if
      return
      end



      SUBROUTINE STAU_INIT_PARS( INITIAL_VALUES, MAX_OPT )
c
c Allocates the initial vertices of the polyhedron for the flexible polyhedron
c search algorithm . This routine needs to know roughly what the starting
c values and the characteristic step size for each BLM_* variable should be .
c
c Each starting N-dimensional vector is written into a column of the matrix
c INITIAL_VALUES .
c
c Arguments
c  real initial_values(max_opt,*)
c                  # OUT # initial polyhedron
c       max_opt    # IN  # number of delared rows in INITIAL_VALUES
c
c Variables
c  logical error_BLM  # function : gets & resets BLM error status
c  integer col        # column index
c          row        # row index
c  real start_value   # current value of the parameter is used if possible
c       def_BLM       # function : defaults values of BLM parameter
c       get_BLM       #          : get BLM parameter
c       step_BLM      #          : sensitivity or initial step of BLM parameter
c  character Cjunk*2  # unused message returned by error_BLM
c
      include 'stdio.def'
      include 'blm.def'
      include 'qu_tau.def'
c
      logical error_BLM
      integer col, row, max_opt
      real initial_values(max_opt,*), start_value,
     1     def_BLM, get_BLM, step_BLM
      character Cjunk*2
c
      include 'qu_tau.cmn'
c
c       Assign the starting values for the first vertex in the polyhedron .
c       If a parameter value is available, it is used, otherwise the
c       default value is used and use set_BLM to make it available .
c
      do 2 row = 1, n_opt_pars
          start_value = get_BLM( opt_par( row ) )
          if ( error_BLM( cjunk ) ) then
              initial_values(row,1) = def_BLM( opt_par( row ) )
              call set_BLM( opt_par(row), initial_values(row,1) )
          else
              initial_values(row,1) = start_value
              end if
c       Copy the first vertex to the other vertices
          do 1 col = 2, n_opt_pars + 1
              initial_values(row,col) = initial_values(row,1)
1             continue
2         continue
c
c       change all but the first vertex ( by the initial step size)
c
      do 3 row = 1, n_opt_pars
          col = row + 1
          initial_values(row,col) = initial_values(row,col) *
     1        ( 1.0 + step_BLM( opt_par( row ) ) )
3         continue
      return
      end



      BLOCK DATA STAU_BLOCK
      include 'stdio.def'
      include 'qu_tau.def'
      include 'qu_tau.cmn'
      data n_opt_pars, n_data / 0, 0 /
      end



c
c External functions
c
c set_BLM, def_BLM, get_BLM, step_BLM, Uplus_BLM, error_BLM, prin_BLM, name_BLM
c
c ilgopen, ilgslaw, ilgimit, ilgxlab, ilgylab, ilgdata, ilgclos, ilgline,
c  ilginvs
c
c amoeba, sfudge
c Jresp, open_file, str_length
c qdscop, qmean, qresp
c Equery

