c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      REAL FUNCTION
     1    QVKVS_RAW( HOT_WIRE_DATA, INCREM, N_POINTS,
     2            TIME_STEP, CYLIND_DIAM, KINEM_VISC,
     3            WORK, FORCE_PLOT )
c
c THIS ROUTINE IS INTENDED TO BE THE **ONLY** ENTRY POINT IN THIS FILE.
c OTHER ROUTINES ARE NOT SUPPOSED TO CALLED FROM OUTSIDE THIS FILE.
c
c Calculate the mean air speed from the raw data obtained by placing a
c hot-wire ``detector'' probe in the Von Karman vortex street shed from
c a long cylinder normal to the flow direction .
c
c
c Arguments
c  real hot_wire_data(*)  # INPUT  # raw data from hot-wire
c  integer increm         # INPUT  # data indexing increment
c          n_points       # INPUT  # number of data points to be used
c  real    time_step      # INPUT  # time between consecutive data samples (sec)
c          cylind_diam    # INPUT  # cylinder diameter (metres)
c          kinem_visc     # INPUT  # kinematic viscosity of air (SI units)
c          work(2*n_points+MAX_AR_COEFFS)
c                         # work array for calculating spectrum and plotting
c  logical force_plot     # .true. if force plotting and editing
c
c N.B. Input arguments are READ-ONLY -- in particular, cylind_diam is copied
c      to cyl_diameter which can be modified by QVKVS_RAW .
c
c constants
c  argument checking
c   ABORT_LABEL =' QVKVS_RAW : ABORT : '
c   MAX_POINTS  = 100000   # maximum number of data points of hot-wire signal
c   MIN_POINTS  = 20       # minimum number of data points of hot-wire signal
c   MAX_STEP    = 1.0      # maximum time between hot-wire data samples
c   MIN_STEP    = 1.0e-6   # minimum time between hot-wire data samples
c  Fixed analysis parameters
c   DSC_ACCURACY = 0.0002  # accuracy of iterative search by DSC algorithm
c   DSC_MAXIMUM  = .TRUE.  # DSC search for maximum
c   FIND_MAXIMA  = .TRUE.  # otherwise SPEAKS finds minima
c   AUTOMAGIC    = .TRUE.  # Select vortex shedding mode automagically
c   MANUAL       = .FALSE. # Select vortex shedding mode manually
c
c  Maximum array sizes
c   MAX_PEAKS     = 16     # maximum number of spectral peaks to be found
c
c  Work array and peak array indices
c   ABSCISSA = 1 # start of WORK array space for spectrum abscissa values
c   OLD_AR_COEFFS= MAX_AR_COEFFS + 1
c                # start of WORK array space for previous AR coefficients
c   FORW_RESIDUALS=OLD_AR_COEFFS + MAX_AR_COEFFS
c                # start of WORK array space for forward residuals
c   FREQ_COL = 1 # PEAK column containing spectral peak locations (frequency)
c   PSD_COL  = 2 # PEAK column containing spectral peak locations (frequency)
c
c local variables
c  logical
c   first_call      # first call to qvkvs_raw
c   problem(MAX_PROBLEM) # set of switches describing potential problems
c   select_enable   # allows routine to select start point for iterative search
c   Equery          # function : read logical
c   evkvs_check     #          : Are routine arguments OK ?
c   evkvs_is_prob   #          : Have any problems occurred ?
c   ilgread         #          : read crosshairs location
c  integer
c   actual_n_AR     # actual number of AR coefficients returned from SBURG
c   back_residuals  # start of WORK array space for backward residuals
c   destin          # destination index for shuffling spectral peaks
c   double_index    # index into list of PEAKs of the second of a double peak
c   formula         # Specifies vortex shedding formula, VK_ROSHKO, VK_WILLIAM..
c   index_closest   # function : index of closest array element
c   i_peak          # index into array of spectral PEAKs
c   n_peaks         # number of spectral peaks found by SPEAKS
c   ordinate        # start of WORK array space for spectrum ordinate values
c   search_index    # index into list of PEAKs of iteration starting frequency
c   source          # source index for shuffling spectral peaks
c   reqd_n_AR       # required number of AR coefficients
c   try_count       # number of analyses attempted
c   Jresp           #          : read integer
c real
c   AR_coeffs(MAX_AR_COEFFS)  # AR coefficients defining the signal spectrum
c   cyl_diameter    # cylinder diameter -- modifiable copy
c   freq_step       # frequency increment
c   hair_x, hair_y  # crosshair location
c   peak(MAX_PEAKS,PSD_COL) # list of spectral peaks
c   relative_height # height of a peak relative to the first one
c   Reynolds_no     # Reynolds number of cylinder vortex shedding
c   search_freq     # frequency at which iterative search for peak starts
c   search_height   # spectral density at start of iterative search
c   shedding_ang    # angle of vortex shedding in degrees
c   shedding_freq   # final estimate of the vortex shedding frequency
c   Strouhal_no     # Strouhal number of vortex shedding
c   qarpsd          # function : evaluate spectral density at a single frequency
c   qdscop          #          : DSC univariate optimization
c   qmean           #          : calculate mean
c   Qresp           #          : real real
c   qvkvs_eqn       #          : calculation for laminar vortex shedding
c   qvkvs_object    #          : objective function for optimization by QDSCOP
c  character
c   action*1        # primary user input : specifies action to take
c   input*1         # secondary user input : destination of keyboard input
c   shedding_mode*1 # vortex shedding mode P, I or X.
c   CVKVS_formula*10# function : returns character representation of "formula"
c  subroutine
c   ilgclos, ilgdata, ilgopen, ilgraph, ilgslaw,
c   ilgtitl, ilgwait, ilgxlab, ilgylab
c                   # ilglib graphics library routines
c   newscr          # new screen from hpprev graphics library
c   sarinz          # initialize parameters for calculating PSD values
c   sburg           # calculate AR coefficients of maximum entropy spectrum
c   sortsh          # shell sort of array columns
c   speaks          # make list of local extrema in a coordinate stream
c   svkvs_pr_prob   # print list of problems found
c   svkvs_set_par   # set parameters for vortex shedding analysis
c   svkvs_mode_set  # set the vortex shedding mode, formula and angle
c
c
      include 'stdio.def'
      include 'qvkvs.def'
      character ABORT_LABEL*21
      logical DSC_MAXIMUM, FIND_MAXIMA, AUTOMAGIC, MANUAL
      integer MAX_POINTS,    MIN_POINTS,     MAX_PEAKS,  ABSCISSA,
     1        OLD_AR_COEFFS, FORW_RESIDUALS, FREQ_COL,   PSD_COL
      real    MAX_STEP,      MIN_STEP,       DSC_ACCURACY
      parameter( ABORT_LABEL =' QVKVS_RAW : ABORT : ' )
      parameter( DSC_MAXIMUM = .TRUE.,  FIND_MAXIMA  = .TRUE.,
     1           AUTOMAGIC   = .TRUE.,  MANUAL       = .FALSE. )
      parameter( MAX_POINTS  = 100000,  MIN_POINTS   = 20,
     1           MAX_PEAKS   = 16,      ABSCISSA     = 1,
     2           OLD_AR_COEFFS= MAX_AR_COEFFS + 1,
     3           FORW_RESIDUALS=OLD_AR_COEFFS + MAX_AR_COEFFS,
     4           FREQ_COL    = 1,       PSD_COL      = 2 )
      parameter( MAX_STEP    = 1.0,     MIN_STEP     = 1.0e-6,
     1           DSC_ACCURACY = 0.0002 )
c
      logical force_plot, first_call, problem(MAX_PROBLEM),
     1        select_enable, Equery, evkvs_check, evkvs_is_prob, ilgread
      integer increm, n_points, actual_n_AR, back_residuals, destin,
     1        double_index, formula, index_closest, i_peak, n_peaks,
     2        ordinate, new_formula, search_index, source, reqd_n_AR,
     3        try_count, Jresp
      real    hot_wire_data(*), time_step, cylind_diam, kinem_visc,
     1        work(2*n_points+MAX_AR_COEFFS), AR_coeffs(MAX_AR_COEFFS),
     2        cyl_diameter, freq_step, hair_x, hair_y,
     3        peak(MAX_PEAKS,PSD_COL), relative_height, Reynolds_no,
     4        search_freq, search_height, shedding_ang, shedding_freq,
     5        Strouhal_no, new_angle, qarpsd, qdscop, qmean, Qresp,
     6        qvkvs_eqn, qvkvs_object
      character action*1, input*1, shedding_mode*1, new_mode*1,
     1        CVKVS_formula*10
c     subroutine ilgclos, ilgdata, ilgopen, ilgraph, ilgslaw, ilgtitl,
c             ilgwait, ilgxlab, ilgylab, newscr, sarinz, sburg, sortsh,
c             speaks, svkvs_pr_prob, svkvs_set_par, svkvs_mode_set
c
      include 'qvkvs.cmn'
      external qvkvs_object, qvkvs_block
      save first_call
      data first_call / .TRUE. /
c
c
c       CHECK ARGUMENTS
c       ***************
c
c
      qvkvs_raw = -1.0
      if ( .not. evkvs_check( ABORT_LABEL, VK_UNUSED, VK_UNUSED,
     1        VK_UNUSED, cylind_diam, kinem_visc ) ) then                 1b
          continue
      else if ( increm .le. 0 ) then                                      1m
          write( STD_ERR, 1 ) ABORT_LABEL, increm
1         format( a, 'Illegal data indexing increment INCREM = ', i12 )
      else if ( n_points .le. MIN_POINTS .or.
     1          n_points .gt. MAX_POINTS ) then                           1m
          write( STD_ERR, 2 ) ABORT_LABEL, n_points
2         format( a, 'Too few data points (or I don''t believe it) ',
     1            'N_POINTS = ', i12 )
          write( STD_ERR, 3 ) real( MIN_POINTS ), real( MAX_POINTS )
3         format( 21x, 'Limits are ', g12.5, ' and ', g12.5 )
      else if ( time_step .le. MIN_STEP .or.
     1          time_step .gt. MAX_STEP ) then                            1m
          write( STD_ERR, 4 ) ABORT_LABEL, time_step
4         format( a, 'Illegal raw data TIME_STEP = ', g12.5 )
          write( STD_ERR, 3 ) MIN_STEP, MAX_STEP
      else                                                                1m
c
c       Reset / initialize parameters
c
          cyl_diameter = cylind_diam
          max_n_plot = n_points
          back_residuals = FORW_RESIDUALS + n_points
          if ( first_call ) then                                          2b
              call svkvs_set_par( INITIAL_DEFAULTS )
              first_call = .FALSE.
              end if                                                      2e
          ordinate  = ABSCISSA + n_plot
          reqd_n_AR = min( MAX_AR_COEFFS, init_AR_coeffs )
          try_count = 0
          select_enable = .TRUE.
c
c         start of main loop                                              2b
10            continue
              try_count = try_count + 1
c
c       CALCULATE THE MAXIMUM ENTROPY POWER SPECTRUM
c       ********************************************
c       Calculate the autoregression coefficients using Burg's method --
c       which is a lattice method in which the reflection coefficients
c       are the harmonic mean of the forward and backward reflection
c       coefficients .
c       Not using 'fpe' or 'aic' as termination criterion avoids early exit.
c
              call sburg( hot_wire_data, increm, n_points,
     1            qmean( hot_wire_data, increm, n_points ),
     2            work(FORW_RESIDUALS), work(back_residuals),
     4            AR_coeffs, work(OLD_AR_COEFFS),
     5            actual_n_AR, reqd_n_AR, 'FIX' )
c
c       Calculate spectrum values -- first initialize with autoregression
c       coefficients, number of coefficients, time step and data variance .
c
              call sarinz( AR_coeffs, actual_n_AR, time_step, 1.0 )
              freq_step = 0.5 / float( n_plot ) / time_step
              work(ABSCISSA) = freq_step
              work(ordinate) = qarpsd( work(ABSCISSA) )
              do 11 destin = 1, n_plot - 1                                     3b             2b
                  work( ABSCISSA + destin ) =
     1                work(ABSCISSA + destin - 1 ) + freq_step
                  work( ordinate + destin ) =
     1                abs( qarpsd( work( ABSCISSA + destin ) ) )
11                continue                                                3e
c
c       find peaks ( and the number of peaks ) in spectrum
c
              n_peaks = 1
              call speaks( work(ABSCISSA), work(ordinate), 1, n_plot,
     2                     peak(1,FREQ_COL), peak(1,PSD_COL), n_peaks,
     3                     MAX_PEAKS, FIND_MAXIMA )
              if ( n_peaks .gt. 1 ) then                                  3b
c
c       if there are peaks at zero frequency, compress them out
c       don't forget to adjust the number of peaks when finished
c
                  destin = 0
                  do 12 source = 1, n_peaks                               4b
                      if ( peak(source,FREQ_COL).gt.freq_step ) then      5b
                          destin = destin + 1
                          peak(destin,FREQ_COL) = peak(source,FREQ_COL)
                          peak(destin,PSD_COL) = peak(source,PSD_COL)
                          end if                                          5e
12                    continue                                            4e
                  n_peaks = destin
                  end if                                                  3e
              if ( n_peaks .gt. 1 ) then                                  3b
c
c       sorts the peaks in order of descending height
c
c                 sort ARRAY from START_ROW to LAST_ROW out of
c                 TOTAL_ROWS, sorting COLUMN of TOTAL_COLUMNS
                  call sortsh( peak, 1, n_peaks,
     1                         MAX_PEAKS, PSD_COL, PSD_COL )
                  end if                                                  3e
c
c
c       EXAMINE THE PEAK VALUES IN THE SPECTRUM, LOOKING FOR POTENTIAL
c       PROBLEMS REQUIRING SPECIAL STRATEGIES AND/OR USER INTERVENTION
c       **************************************************************
c
c
c       DOUBLE PEAK
c       If the peak closest to the highest peak is closer to it than
c       CLOSENESS % / 100, and the heights of the second peak is not
c       smaller by a factor of more than RELATIVENESS ( which is < 1 ),
c       a DOUBLE PEAK has been found . Stop looking after the first
c       peak has been found .
c
c             clear the problem flag
              problem(DOUBLE_PEAK) = .FALSE.
              double_index = 0
c             don't search for a double peak if there is only one
              if ( n_peaks .gt. 1 ) then                                  3b
                  do 13 i_peak = 2, n_peaks                               4b
c                     search through array of peaks, use decreasing
c                         amplitude to make the search more efficient
                      relative_height = peak(i_peak,PSD_COL) /
     1                                  peak(1,PSD_COL)
                      if ( double_index .eq. 0 .and. 2.0 * abs(
     1                      peak(1,FREQ_COL) - peak(i_peak,FREQ_COL) ) /
     2                      ( peak(1,FREQ_COL) + peak(i_peak,FREQ_COL) )
     3                         .lt. closeness .and.
     4                        relative_height .gt. relativeness ) then    5b
c                         "double peak problem" has been detected
c                         set problem flag & remember which peak it was
                          problem(DOUBLE_PEAK) = .TRUE.
                          double_index = i_peak
                          end if                                          5e
13                    continue                                            4e
                  end if                                                  3e
c
c       SELECT STARTING POINT FOR ITERATIVE SEARCH FOR SPECTRAL PEAK
c       ************************************************************
c
c       Usually, the search is started at the highest peak from SPEAKS,
c       which after sorting is the first one, but if a double peak
c       is found, the one of higher frequency is selected.
c
              if ( select_enable .and. n_peaks .gt. 0 ) then              3b
c                 default place to start searching is the first peak
                  search_index = 1
c                 deal with the case of a double peak
                  if ( problem(DOUBLE_PEAK) .and.
     1                 peak(double_index,FREQ_COL).gt.peak(1,FREQ_COL) )
     3                search_index = double_index
c
                  search_height = peak(search_index,PSD_COL)
                  search_freq   = peak(search_index,FREQ_COL)
c
c       CHECK IF THE PEAK IS TOO WIDE
c       *****************************
c       A problem may exist if the selected peak is too wide . The peak
c       is too wide if the power spectral density is greater than the
c       fraction TALLNESS of the peak value at a (frequency) distance
c       WIDENESS from the peak .
c
                  problem(TOO_WIDE_PEAK) = tallness * search_height .lt.
     1              max( qarpsd( ( 1.0 - wideness/2.0 ) * search_freq ),
     2                   qarpsd( ( 1.0 + wideness/2.0 ) * search_freq ))
c
c       MULTIPLE HIGH PEAKS
c       *******************
c       A problem is recorded if there are other peaks which have a height
c       within a fraction SAMENESS of the height of the selected peak .
c       (ignore the selected peak and the double peak )
c
                  if ( n_peaks .gt. 1 ) then                              4b
                      problem(MULTIPLE_PEAK) = .FALSE.
                      do 14 i_peak = 1, n_peaks                           5b
                          if ( i_peak .ne. search_index .and.
     1                         i_peak .ne. double_index ) then            6b
                              if ( peak(i_peak,PSD_COL) .gt.
     1                                search_height * sameness )
     2                            problem(MULTIPLE_PEAK) = .TRUE.
                              end if                                      6e
14                        continue                                        5e
                      end if                                              4e
                  end if                                                  3e
c
c       THIRD OR LATER ATTEMPT
c       **********************
c       A problem is recorded if we are processing the data 3 or more times.
c
              problem(MULT_ATTEMPTS) = try_count .gt. 2
c
c
c       PLOT THE SPECTRUM IF A PROBLEM IS FOUND
c       ***************************************
c       (or forced plotting)
c
              if ( evkvs_is_prob(problem) .or. force_plot ) then          3b
                  call ilgopen( capture_file )
                  call ilgslaw( 'lin', 'log' )
                  call ilgraph( 1, 1, 26, 26, 1 )
                  call ilgdata( work(ABSCISSA), work(ordinate),
     1                          1, n_plot, 1 )
c                 call ilgxlab( 'Frequency (Hz)' )
                  call ilgylab( 'Hz|2t-1' )
                  call ilgtitl( 'Vortex shedding spectrum' )
                  plotted = .TRUE.
                  end if                                                  3e
c
c       ITERATIVE SEARCH FOR SPECTRAL PEAK AND CALCULATION OF AIR SPEED
c       ***************************************************************
c       by the DSC algorithm . The objective function optimized, QVKVS_OBJECT,
c       takes the log (base e) as its argument . The initial step size of the
c       search is FREQ_STEP / SEARCH_FREQ .
c
              if ( n_peaks .gt. 0 ) then                                  3b
                  shedding_freq = exp(
     1                qdscop( log( search_freq ), qvkvs_object,
     2                        freq_step / search_freq, DSC_ACCURACY,
     3                        DSC_MAXIMUM ) )
c
c       now calculate air speed and cylinder Reynolds number register a
c       problem if the Reynolds number is lower or higher than the laminar
c       vortex shedding limits
c
                  if ( try_count .eq. 1 ) then                            4b
                      call svkvs_mode_set( AUTOMAGIC,
     1       shedding_freq, VK_UNUSED, cyl_diameter, cyl_length,
     2       kinem_visc, formula, shedding_ang, shedding_mode )
                      end if                                              4e
                  qvkvs_raw = qvkvs_eqn( VK_VELOCITY + formula,
     1                shedding_freq, shedding_ang, VK_UNUSED,
     2                cyl_diameter, kinem_visc )
                  if ( qvkvs_raw .gt. 0.0 ) then                          4b
                      Reynolds_no = qvkvs_eqn( VK_REYNOLDS,
     1                    VK_UNUSED,    VK_UNUSED, qvkvs_raw,
     2                    cyl_diameter, kinem_visc )
                      Strouhal_no = qvkvs_eqn( VK_STROUHAL,
     1                    shedding_freq, VK_UNUSED, qvkvs_raw,
     2                    cyl_diameter,  kinem_visc )
                      call svkvs_mode_set( AUTOMAGIC, VK_UNUSED,
     1                    Reynolds_no, cyl_diameter, cyl_length,
     2                    kinem_visc, new_formula, new_angle,
     3                    new_mode )
                      problem(UNSTABLE_MODE) = new_formula .ne. formula
     1                    .or. new_mode  .ne. shedding_mode .or.
     2                  abs( new_angle - shedding_ang ) .gt. obliqueness
                  else                                                    4m
                      Reynolds_no = 0.0
                      Strouhal_no = 0.0
                      end if                                              4e
c
                  problem(REYNOLD_LIMITS) = 0 .ge.
     1                qvkvs_eqn( VK_IS_LAMIN, VK_UNUSED, VK_UNUSED,
     2                           qvkvs_raw, cyl_diameter, kinem_visc )
                  end if                                                  3e
c
c       CHECK IF THE PEAK FOUND IS CLOSE TO THE STARTING POINT
c       ******************************************************
c       a problem is flagged if the peak is "not found"
c
c
              problem(NOT_FOUND_PEAK) = n_peaks .le. 0 .or.
     1            select_enable .and.
     2            qarpsd( shedding_freq ) .gt. search_height .and.
     3            abs( shedding_freq -search_freq ) .gt. freq_step * 2.0
c
c       IF A PROBLEM EXISTS, OR PLOTTING IS FORCED, DISPLAY
c       AVAILABLE INFORMATION AND PROMPT FOR USER ACTION
c       ****************************************************
c
              if ( evkvs_is_prob(problem) .or. force_plot ) then          3b
c                 plot the spectrum if it is not already done
                  if ( .not. plotted ) then                               4b
                      call ilgopen( capture_file )
                      call ilgslaw( 'lin', 'log' )
                      call ilgraph( 1, 1, 26, 26, 1 )
                      call ilgdata( work(ABSCISSA), work(ordinate),
     1                              1, n_plot, 1 )
c                     call ilgxlab( 'Frequency (Hz)' )
                      call ilgylab( 'Hz|2t-1' )
                      call ilgtitl( 'Vortex shedding spectrum' )
                      plotted = .TRUE.
                      end if                                              4e
c                 display information, list of problems & prompt for user
c                 come back to here if user's response is illegal
20                call newscr()
                  write( STD_OUT, 21 ) shedding_mode,
     1                CVKVS_formula(formula), actual_n_AR, shedding_ang,
     2                Reynolds_no, cyl_diameter * 1000.0, shedding_freq,
     3                Strouhal_no, qvkvs_raw
21     format( ' Vortex shedding Mode  ', a1,
     1     '      Formula    ', a10, '   No. of AR coeffs ', i3 /
     2     ' Shedding angle ', f7.3, ' deg   Reynolds no. ', f7.3,
     3     '   Cylinder diam ', f7.3, ' mm' / ' Shedding freq ', f9.4,
     4     ' Hz   Strouhal no.  ', f7.4, '   Air speed    ', f8.4,
     5     ' m/s' )
            call svkvs_pr_prob( problem )
                  write( STD_OUT, 22 )
22                format( '  Enter command: A=Abandon    K=Key (+IF)',
     1                 '    N=No. AR coeffs    P=Parameters' /
     2                 17x, 'E=Exit OK    F=Final freq   S=Step AR ',
     3                 'coeffs   C=Cylind diam' /
     4                 '       H=Help    R=Read-CSP   I=Init freq    ',
     5                 'D=swap Double      M=Mode menu' )
c
c                 get user's response
                  if ( ilgread( hair_x, hair_y, action ) ) then           4b
c                     convert to upper case
                      if ( 'a' .le. action .and. action .le. 'z' )
     1                    action = char( ichar( action ) +
     2                             ichar( 'A' ) - ichar( 'a' ) )
c
c               Special case for keyboard input : read the crosshairs
c               again for 'I' or 'F' and get the value from the keyboard
c
                      if ( action .eq. 'K' ) then                         5b
                          write( STD_OUT, 23 )
23    format( / ' Press ''I'' then enter initial frequency for search,',
     1   ' or' / 6x, '''F'' then enter final vortex shedding frequency')
                          if ( ilgread( hair_x, hair_y, input ) ) then    6b
                              if ( index( 'FIfi', input ) .ne. 0 ) then                  7b
                                  write( STD_OUT, 24 )
24                                format( / ' Frequency (Hz) ', $ )
                                  hair_x = Qresp( STD_IN, 0.0,
     1                                0.5 / time_step, MAX_TRIES )
                                  end if                                  7e
                              end if                                      6e
                          end if                                          5e
c
c               ABANDON - return with error condition
c
                      if ( action .eq. 'A' ) then                         5b
                          if ( Equery( STD_IN,
     1                            ' Abandon : Are you sure ? ' ) ) then   6b
                              qvkvs_raw = -1.0
                              goto 34
                              end if                                      6e
c
c               EXIT - with displayed results
c
                      else if ( action .eq. 'E' ) then                    5m
                          goto 34
c
c               HELP
c
                      else if ( action .eq. 'H' ) then                    5m
                          write( STD_OUT, 25 ) increm_n_AR
25    format( / /
     1    ' A=Abandon        - Return with error status' /
     b    ' C=Cylind diam    - Specify cylinder diameter' /
     9    ' D=swap Double    - Restart search with other double peak' /
     2    ' E=Exit OK        - Return with the displayed results' /
     5    ' F=Final Freq     - Specify final shedding frequency' /
     3    ' H=Help           - This help text' /
     4    ' K=Key (+IF)      - Key in initial or final frequency' /
     6    ' I=Initial Freq   - Specify initial frequency for search' /
     7    ' M=shedding Mode  - Menu for selection of shedding mode' /
     8    ' N=No. AR coeffs  - Specify number of AR coefficients' /
     a    ' P=Parameters     - Menu of discrimination parameters' /
     8    ' S=Step AR coeffs - Increase number of AR coeffs by ', i3 /
     c    ' R=Read (+CSP)    - Read with crosshairs : C=Coordinate,' /
     d    '                    S=Spectrum value, P=spectral Peak' )
                          call ilgwait()
c
c               FREQUENCY - Specify shedding frequency
c
                      else if ( action .eq. 'F' .or. action .eq. 'K'
     1                        .and. index( 'Ff', input ) .ne. 0 ) then    5m
                          shedding_freq = hair_x
                          qvkvs_raw = qvkvs_eqn( VK_VELOCITY + formula,
     1                        shedding_freq, shedding_ang, VK_UNUSED,
     2                        cyl_diameter, kinem_visc )
                          Reynolds_no = qvkvs_eqn( VK_REYNOLDS,
     1                        VK_UNUSED,    VK_UNUSED, qvkvs_raw,
     2                        cyl_diameter, kinem_visc )
                          Strouhal_no = qvkvs_eqn( VK_STROUHAL,
     1                        shedding_freq, VK_UNUSED, qvkvs_raw,
     2                        cyl_diameter,  kinem_visc )
                          problem(REYNOLD_LIMITS) = 0 .ge.
     1                        qvkvs_eqn( VK_IS_LAMIN, VK_UNUSED,
     2                           VK_UNUSED, qvkvs_raw, cyl_diameter,
     3                           kinem_visc )
c
c               INITIAL - Specify initial point for search
c
                      else if ( action .eq. 'I' .or.
     1                      action .eq. 'K' .and. input .eq. 'I' ) then   5m
                          search_freq = hair_x
                          select_enable = .FALSE.
                          goto 10
c
c               N_COEFFS -  Specify number of AR coeffs
c
                      else if ( action .eq. 'N' ) then                    5m
                          write( STD_OUT, 26 )
26                        format( / ' Number of AR coefficients ? ', $ )
                          reqd_n_AR = Jresp( STD_IN, MIN_AR_COEFFS,
     1                                       MAX_AR_COEFFS, MAX_TRIES )
                          select_enable = .TRUE.
                          goto 10
c
c               STEP IN AR COEFFS -  increase number of AR coeffs
c
                      else if ( action .eq. 'S' ) then                    5m
                          select_enable = .TRUE.
                          reqd_n_AR = min( MAX_AR_COEFFS,
     1                                     reqd_n_AR + increm_n_AR )
                          goto 10
c
c               DOUBLE - restart search with the other double peak
c                      - only if a double peak was found
c
                      else if ( action .eq. 'D' ) then                    5m
                          if ( problem(DOUBLE_PEAK) ) then                6b
                              if ( search_freq .eq. peak(1,FREQ_COL ) )
     1                                then                                7b
                                  search_freq =
     1                                peak(double_index,FREQ_COL)
                                  search_height =
     1                                peak(double_index,PSD_COL)
                              else                                        7m
                                  search_freq = peak(1,FREQ_COL)
                                  search_height = peak(1,PSD_COL)
                                  end if                                  7e
                              select_enable = .FALSE.
                              goto 10
                          else                                            6m
                              write( STD_OUT, 27 )
27                            format( / ' QVKVS_RAW : WARNING : ',
     1                            'Illegal command : Double peak not ',
     2                            'found.' )
                              call ilgwait()
                              end if                                      6e
c
c               PARAMS - set problem discrimination parameters
c
                      else if ( action .eq. 'P' ) then                    5m
                          select_enable = .TRUE.
                          call svkvs_set_par( .not. INITIAL_DEFAULTS )
                          goto 10
c
c               CYL_DIAMETER - specify cylinder diameter
c
                      else if ( action .eq. 'C' ) then                    5m
                          write( STD_OUT, 28 )
28                        format( / ' Cylinder diameter (mm) ? ', $ )
                          cyl_diameter = Qresp( STD_IN,
     1                        1000.0 * MIN_DIAM, 1000.0 * MAX_DIAM,
     2                        MAX_TRIES ) / 1000.0
                          goto 10
c
c               READ - coordinate, spectral value or spectral peak
c
                      else if ( action .eq. 'R' ) then                    5m
                          write( STD_OUT, 29 )
29                        format( /
     1                    ' Press : C = Coordinate of crosshairs' /
     2                    9x, 'S = Spectrum value (at frequency)' /
     3                    9x, 'P = spectral Peak (nearest frequency)' )
                          if ( ilgread( hair_x, hair_y, input ) ) then    6b
c                             convert to upper case
                              if ( 'a' .le. input .and. input .le. 'z' )
     1                            input = char( ichar( input ) +
     2                                    ichar( 'A' )- ichar( 'a' ) )
c
c                             display crosshairs location
                              if ( input .eq. 'C' ) then                  7b
                                  write( STD_OUT, 30 ) hair_x, hair_y
30    format( / ' Crosshair location : frequency = ', f10.4, ' Hz' /
     1          '                      spectral density = ', g12.5 )
                                  call ilgwait()
c
c                             display spectral density at specified frequency
                              else if ( input .eq. 'S' ) then             7m
                                  write( STD_OUT, 31 ) hair_x,
     1                                abs( qarpsd( hair_x ) )
31    format( / ' Spectral Density at ', f10.4, ' Hz is ', g12.5,
     1        ' 1/Hz' )
                                  call ilgwait()
c
c                             display peak nearest in frequency to crosshairs
                              else if ( input .eq. 'P' ) then             7m
c                                 index_closest returns the index of the
c                                     array element (2nd arg) closest to
c                                     the value of the 1st arg
                                  i_peak = index_closest( hair_x,
     1                                peak(1,FREQ_COL), 1, n_peaks )
                                  if ( 1 .le. i_peak .and.
     1                                    i_peak .le. n_peaks ) then
                                      write( STD_OUT, 32 )
     1                                    peak(i_peak,FREQ_COL),
     2                                    peak(i_peak,PSD_COL)
32    format( / ' Spectral Peak at ', f10.4, ' Hz is ', g12.5, ' 1/Hz' )
                                      call ilgwait()
                                  else
                                      write( STD_OUT, 33 )
33    format( ' Could not find spectral peak' )
                                      end if
                                  end if                                  7e
                              end if                                      6e
c
c               MODE - select vortex shedding mode
c               svkvs_mode_select hides a menu for manual selection
c               and an evil empiricism for auto-selection of vortex shedding
c               mode parameters
c
                      else if ( action .eq. 'M' ) then                    5m
                          select_enable = .FALSE.
                          call svkvs_mode_set( MANUAL,
     1       VK_UNUSED, Reynolds_no, cyl_diameter, cyl_length,
     2       kinem_visc, formula, shedding_ang, shedding_mode )
                          goto 10
                          end if                                          5e
                      goto 20
                      end if                                              4b
                  end if                                                  3b
c             end of main loop                                            2e
          end if                                                          1e
34    if ( plotted ) then                                                 1b
          call ilgslaw( 'lin', 'lin' )
          call ilgclos()
          plotted = .FALSE.
          end if                                                          1e
      return
      end



      INTEGER FUNCTION INDEX_CLOSEST( TARGET, ARRAY, INCREM, N_POINTS )
c
c Returns the index of the element of the array ARRAY which is closest to
c the value of TARGET .
c
c Arguments
c  real    target     # target value -- value which we are seeking from ARRAY
c          array(*)   # array of values which are examined
c  integer increm     # indexing increment in ARRAY
c          n_points   # number of data points in ARRAY
c
c local variables
c  integer
c   index             # array index
c   n_last            # index of last useable array element
c  real
c   distance          # distance between array element and target value
c   min_distance      # minimum distance between array element and target
c                     # value found so far
c
      include 'stdio.def'
      integer increm, n_points, index, n_last
      real target, array(*), distance, min_distance
c
      n_last = ( n_points - 1 ) * increm + 1
      index_closest = 1
      min_distance = abs( target - array(1) )
      do 1 index = increm, n_last, increm
          distance = abs( target - array(index) )
          if ( min_distance .gt. distance ) then
              min_distance = distance
              index_closest = index
              end if
1         continue
      return
      end



      REAL FUNCTION QVKVS_OBJECT( LOG_FREQ )
c
c Same as QARPSD --  but the argument is log(FREQ) .
c Evaluates the autoregressive power spectrum density value at the
c log_frequency LOG_FREQ .
c
c Argument
c  real log_freq    # natural log of frequency
c
c constants
c   NO_SYMBOL  = 0  # plotting symbol
c   VERTICAL_STROKE = 14   # plotting symbol
c   NO_LINE    = 0  # dashed line type
c   SOLID_LINE = 1  # dashed line type
c   PEN_1      = 1  # plotting pen colour
c
c
      include 'stdio.def'
      integer    NO_SYMBOL,   VERTICAL_STROKE,
     1           NO_LINE,     SOLID_LINE,
     2           PEN_1
      parameter( NO_SYMBOL=0, VERTICAL_STROKE=14,
     1           NO_LINE=0,   SOLID_LINE=1,
     2           PEN_1=1 )
c
      real log_freq, qarpsd
      include 'qvkvs.cmn'
c
c
      qvkvs_object = qarpsd( exp( log_freq ) )
      if ( plotted ) then
          call ilgline( VERTICAL_STROKE, NO_LINE, PEN_1 )
          call ilgdata( exp( log_freq ), qvkvs_object, 1, 1, 0 )
          call ilgline( NO_SYMBOL, SOLID_LINE, PEN_1 )
          end if
c
      return
      end



      CHARACTER *10 FUNCTION CVKVS_FORMULA( FORMULA )
c
c Return a character string representation of the vortex shedding formula
c specified by FORMULA .
c
      include 'stdio.def'
      include 'qvkvs.def'
      integer formula
c
      if ( formula .eq. VK_ROSHKO ) then
          cvkvs_formula = 'Roshko    '
      else if ( formula .eq. VK_WILLIAM ) then
          cvkvs_formula = 'Williamson'
      else if ( formula .eq. VK_P_KOENIG ) then
          cvkvs_formula = 'Koenig -P-'
      else if ( formula .eq. VK_I_KOENIG ) then
          cvkvs_formula = 'Koenig -I-'
      else if ( formula .eq. VK_X_KOENIG ) then
          cvkvs_formula = 'Koenig -X-'
          end if
      return
      end



      REAL FUNCTION QVKVS_EQN( ACTION_EQN, SHEDDING_FREQ, SHEDDING_ANG,
     1       AIR_SPEED, CYLIND_DIAM, KINEM_VISC )
c
c          Von-Karmen Vortex Street
c Calculates the quantity requested by ACTION for vortex shedding frequency
c of Von-Karmen vortex street downstream of a cylinder . This includes a
c correction for test section blockage  *(width-diam)/width .
c
c Arguments
c  integer action_eqn   # INPUT  # Specifies the quantity to calculate and the
c                                  vortex shedding formula - value is formed by
c                                  adding one of
c       VK_IS_LAMIN     # Is the vortex shedding laminar (from Re) ?
c       VK_MAX_REYN     # Max. cylinder Reynolds no. for laminar vortex shedding
c       VK_MIN_REYN     # Min. cylinder Reynolds no. for laminar vortex shedding
c       VK_FREQUENC     # Laminar vortex shedding frequency (Hz)
c       VK_VELOCITY     # Air speed (metres/second)
c       VK_STROUHAL     # Strouhal number of vortex shedding
c       VK_REYNOLDS     # Reynolds number of the cylinder
c       VK_ROSHK_NO     # Roshko number of vortex shedding
c                                  and one of
c       VK_ROSHKO,   # S=0.212-4.5/Re : A. Roshko (1954) NACA Report 1191
c       VK_WILLIAM   # S=0.00016*Re+0.1816-3.3265/Re: Williamson (1989) JFM v206
c       VK_P_KOENIG  # S=0.0001448*Re+0.188-3.505/Re: P-mode Koenig et al (1990)
c       VK_I_KOENIG  # S=0.0001448*Re+0.188-3.505/Re: I-mode Koenig et al (1990)
c       VK_X_KOENIG  # S=-0.00001131*Re+0.201-4.669/Re: X-mode  Phys Fl A2
c                                  to select the formula
c  real shedding_freq   # INPUT  # (single sided) vortex shedding frequency (Hz)
c       shedding_ang    # INPUT  # vortex shedding angle
c       air_speed       # INPUT  # air speed normal to the cylinder (m/s)
c       cylind_diam     # INPUT  # cylinder diameter (metres)
c       kinem_visc      # INPUT  # kinematic viscosity of fluid (SI units)
c          any real argument of value VK_UNUSED is considered unavailable
c
c constants
c  ABORT_LABEL = ' QVKVS_EQN : ABORT'
c
c local variables
c  integer
c   equation        # specifies vorte shedding relationship
c                   # VK_ROSHKO, VK_WILLIAM, VK_P_KOENIG, VK_X_KOENIG
c   action          # specifies what to calculate
c  logical
c   evkvs_check     # function : check arguments
c  real
c   roshko_no       # nondimensional frequency - using frequency & viscosity
c   Reynolds_no     # Reynolds number of the cylinder
c   check_angle     # vortex shedding angle
c  inv_shedding_eqn # function : inverse vortex shedding relations
c   shedding_eqn    #          : vortex shedding relations
c
      include 'stdio.def'
      include 'qvkvs.def'
      character ABORT_LABEL*18
      parameter( ABORT_LABEL=' QVKVS_EQN : ABORT' )
c
      integer action_eqn, equation, action
      logical evkvs_check
      real    shedding_freq, shedding_ang, air_speed, cylind_diam,
     1        kinem_visc, roshko_no, Reynolds_no, check_angle,
     2        inv_shedding_eqn, shedding_eqn
c
c   resolve ACTION_EQ into its two components
      equation = action_eqn / 16 * 16
      action   = action_eqn - equation
c
c   VK_IS_LAMIN -- Is the vortex shedding laminar (from Re) ?
c   VK_REYNOLDS -- Reynolds number of the cylinder
c   VK_VELOCITY -- Air speed (metres/second)
c
c   The default action is to calculate the Reynolds number from
c   AIR_SPEED, CYLIND_DIAM and KINEM_VISC .
c   If the air_speed is available i.e. not VK_UNUSED and calculation of it is
c   not explicitly requested calculated, the air speed argument is used to
c   obtain the result is the most direct way. If air speed is not an input
c   parameter, non-dimensional frequency is calculated from SHEDDING_FREQ,
c   CYLIND_DIAM and KINEM_VISC, and the Reynolds number is calculated from
c   the selected vortex shedding relationship.
c
      if ( action .eq. VK_IS_LAMIN .or. action .eq. VK_REYNOLDS .or.
     1     action .eq. VK_VELOCITY ) then                                 1b
          if ( air_speed .ne. VK_UNUSED .and. action .ne. VK_VELOCITY )
     1            then                                                    2b
              if ( .not. evkvs_check( ABORT_LABEL, VK_UNUSED, VK_UNUSED,
     1                air_speed, cylind_diam, kinem_visc ) )
     2            return
              Reynolds_no = air_speed * cylind_diam / kinem_visc
          else                                                            2m
c   need to invert vortex shedding relationship
c   check arguments
              if ( equation .eq. VK_WILLIAM .or.
     1             equation .eq. VK_I_KOENIG ) then                       3b
                  check_angle = shedding_ang
              else if ( equation .eq. VK_P_KOENIG ) then                  3m
                  check_angle = 0.0
              else                                                        3m
                  check_angle = VK_UNUSED
                  end if                                                  3e
              if ( .not. evkvs_check( ABORT_LABEL, shedding_freq,
     1             check_angle, VK_UNUSED, cylind_diam, kinem_visc ) )
     2            return
c
c      calculate non-dimensional frequency
              roshko_no = shedding_freq * cylind_diam **2 / kinem_visc
c
c      calculate Reynolds number
              Reynolds_no = INV_SHEDDING_EQN( equation, roshko_no,
     1                                        check_angle )
              end if                                                      2e
c
c     got Reynolds number; now figure out the returned result
          if ( action .eq. VK_REYNOLDS ) then                             2b
              qvkvs_eqn = Reynolds_no
          else if ( action .eq. VK_VELOCITY ) then                        2m
              qvkvs_eqn = Reynolds_no * kinem_visc / cylind_diam
          else if ( action .eq. VK_IS_LAMIN ) then                        2m
              qvkvs_eqn = 0.0
              if ( MIN_LAMINAR_REYN .le. Reynolds_no .and.
     1             MAX_LAMINAR_REYN .ge. Reynolds_no )
     2            qvkvs_eqn = 1.0
              end if                                                      2e
c
c   VK_MAX_REYN -- Max. cylinder Reynolds no. for laminar vortex shedding
c
      else if ( action .eq. VK_MAX_REYN ) then                            1m
          qvkvs_eqn = MAX_LAMINAR_REYN
c
c   VK_MIN_REYN -- Min. cylinder Reynolds no. for laminar vortex shedding
c
      else if ( action .eq. VK_MIN_REYN ) then                            1m
          qvkvs_eqn = MIN_LAMINAR_REYN
c
c   VK_FREQUENC -- Laminar vortex shedding frequency (Hz)
c   VK_ROSHK_NO -- Roshko number (St.Re) number of vortex shedding
c
      else if ( action .eq. VK_FREQUENC .or. action .eq. VK_ROSHK_NO )
     1        then                                                        1m
          if ( action .eq. VK_ROSHK_NO .and.
     1         shedding_freq .ne. VK_UNUSED ) then                        2b
              if ( .not. evkvs_check( ABORT_LABEL, shedding_freq,
     1                VK_UNUSED, VK_UNUSED, cylind_diam, kinem_visc ) )
     2            return
              roshko_no = shedding_freq * cylind_diam **2 / kinem_visc
          else                                                            2m
c   check arguments
              if ( equation .eq. VK_WILLIAM .or.
     1             equation .eq. VK_I_KOENIG ) then                       3b
                  check_angle = shedding_ang
              else if ( equation .eq. VK_P_KOENIG ) then                  3m
                  check_angle = 0.0
              else                                                        3m
                  check_angle = VK_UNUSED
                  end if                                                  3e
              if ( .not. evkvs_check( ABORT_LABEL, VK_UNUSED,
     1              check_angle, air_speed, cylind_diam, kinem_visc ) )
     2            return
              Reynolds_no = air_speed * cylind_diam / kinem_visc
              roshko_no = SHEDDING_EQN( equation, Reynolds_no,
     1            check_angle )
              end if                                                      2e
          if ( action .eq. VK_ROSHK_NO ) then                             2b
              qvkvs_eqn = roshko_no
          else if ( action .eq. VK_FREQUENC ) then                        2m
              qvkvs_eqn = roshko_no * kinem_visc / cylind_diam **2
              end if                                                      2e
c
c   VK_STROUHAL -- Strouhal number of vortex shedding
c
      else if ( action .eq. VK_STROUHAL ) then                            1m
          if ( air_speed .ne. VK_UNUSED ) then                            2b
              if ( .not. evkvs_check( ABORT_LABEL, shedding_freq,
     1                 VK_UNUSED, VK_UNUSED, cylind_diam, VK_UNUSED ) )
     2            return
              qvkvs_eqn = shedding_freq * cylind_diam / air_speed
          else                                                            2m
c   check arguments
              if ( equation .eq. VK_WILLIAM .or.
     1             equation .eq. VK_I_KOENIG ) then                       3b
                  check_angle = shedding_ang
              else if ( equation .eq. VK_P_KOENIG ) then                  3m
                  check_angle = 0.0
              else                                                        3m
                  check_angle = VK_UNUSED
                  end if                                                  3e
              if ( .not. evkvs_check( ABORT_LABEL, shedding_freq,
     1              check_angle, VK_UNUSED, cylind_diam, kinem_visc ) )
     2            return
              roshko_no = shedding_freq * cylind_diam **2 / kinem_visc
              reynolds_no = inv_shedding_eqn( equation,
     1                                        roshko_no, check_angle )
              qvkvs_eqn = roshko_no / Reynolds_no
              end if                                                      2e
          end if                                                          1e
      return
      end



      REAL FUNCTION INV_SHEDDING_EQN( EQUATION, ROSHKO_NO, THETA )
c
c Inverts the vortex shedding equation so that the Reynolds number is obtained
c from the ROSHKO_NO number (St.Re) using the relationship specified by EQUATION
c for a vortex shedding angle THETA degrees.
c
c Arguments
c  integer equation    specifies vortex shedding relationship
c                VK_ROSHKO, VK_WILLIAM, VK_P_KOENIG, VK_I_KOENIG, VK_X_KOENIG
c  real    roshko_no
c          theta
c
c real
c   TINY = 1.0e-5            # zero tolerance for quadratic solver
c   PI   = 3.14159265359
c
c integer    status          # return status from quadratic solver
c complex    Re_1, Re_2      # Reynolds number solution from quadratic solver
c logical    ok_Re1, ok_Re2  # solutions are within laminar shedding range
c subroutine squadc          # solves quadratic equation
c
      include 'stdio.def'
      include 'qvkvs.def'
c
      real TINY, PI
      parameter( TINY = 1.0e-5, PI = 3.14159265359 )
c
      integer equation, status
      real    roshko_no, theta
      complex Re_1, Re_2
      logical ok_Re1, ok_Re2, err_printed
c
      err_printed = .FALSE.
      inv_shedding_eqn = 0
      if ( roshko_no .lt. MIN_ROSHKO_NO / 3.0 .or.
     1     roshko_no .gt. MAX_ROSHKO_NO * 3.0  ) then                     1b
          write( STD_ERR, 1 ) 'ROSHKO_NO', roshko_no,
     1                        MIN_ROSHKO_NO, MAX_ROSHKO_NO
1         format( ' INV_SHEDDING_EQN : FATAL : Illegal ', a, g12.5 /
     1            '   limits are ', g12.5, ' and ', g12.5 )
          err_printed = .TRUE.
      else if ( equation .ne. VK_ROSHKO   .and.
     1          equation .ne. VK_X_KOENIG .and.
     2        ( theta .lt. MIN_ANGLE .or. theta .gt. MAX_ANGLE ) ) then   1m
          write( STD_ERR, 1 ) 'THETA', theta, MIN_ANGLE, MAX_ANGLE
          err_printed = .TRUE.
      else                                                                1m
          if ( equation .eq. VK_ROSHKO ) then                             2b
              Re_1 = ( roshko_no + 4.5 ) / 0.212
              Re_2 = 0.0
              status = 1.0
          else if ( equation .eq. VK_WILLIAM ) then                       2m
              call squadc( Re_1, Re_2, 0.00016, 0.1816,
     1            ( -3.3265 - roshko_no / cos( PI / 180.0 * theta ) ),
     2            status, TINY )
          else if ( equation .eq. VK_P_KOENIG .or.
     1              equation .eq. VK_I_KOENIG ) then                      2m
              call squadc( Re_1, Re_2, 0.0001448, 0.188,
     1            ( -3.505 - roshko_no / cos( PI / 180.0 * theta ) ),
     2            status, TINY )
          else if ( equation .eq. VK_X_KOENIG ) then                      2m
              call squadc( Re_1, Re_2, -1.131e-5, 0.201,
     1            -4.669 - roshko_no, status, TINY )
              end if                                                      2e
c
c    warn about things which should not happen
          if ( status .eq. 0 ) then                                       2b
              write( STD_ERR, 2 ) 'Equal real roots', real( Re_1 )
2             format( ' INV_SHEDDING_EQN : WARNING : ', a, ' for Re; ',
     1            2g12.5 )
              err_printed = .TRUE.
          else if ( status .lt. 0 ) then                                  2m
              write( STD_ERR, 2 ) 'Complex roots (a+bi)', Re_1
              err_printed = .TRUE.
              end if                                                      2e
c
c    select the solution -- if possible
          ok_Re1 = MIN_LAMINAR_REYN .le. real(Re_1) .and.
     1             real(Re_1) .le. MAX_LAMINAR_REYN
          ok_Re2 = MIN_LAMINAR_REYN .le. real(Re_2) .and.
     1             real(Re_2) .le. MAX_LAMINAR_REYN
          if ( ok_Re1 .and. ok_Re2 ) then                                 2b
              inv_shedding_eqn = max( real( Re_1 ), real( Re_2 ) )
          else if ( ok_Re1 ) then                                         2m
              inv_shedding_eqn = real( Re_1 )
          else if ( ok_Re2 ) then                                         2m
              inv_shedding_eqn = real( Re_2 )
          else                                                            2m
              inv_shedding_eqn = 0.0
              write( STD_ERR, 2 )
     1            'Solution not in laminar vortex shedding range',
     2            real( Re_1 ) , real( Re_2 )
              err_printed = .TRUE.
              end if                                                      2e
          end if                                                          1e
      if ( err_printed )
     1    call ilgwait()
      return
      end



      REAL FUNCTION SHEDDING_EQN( EQUATION, REYNOLDS_NO, THETA )
c
c Applies the vortex shedding equation to calculate the Roshko number of
c vortex from the Reynolds number using the relationship specified by
c EQUATION for a vortex shedding angle THETA degrees.
c
c Arguments
c  integer equation    specifies vortex shedding relationship
c              VK_ROSHKO, VK_WILLIAM, VK_P_KOENIG, VK_I_KOENIG, VK_X_KOENIG
c  real    Reynolds_no
c          theta
c
c real
c   PI           =   3.14159265359
c
      include 'stdio.def'
      include 'qvkvs.def'
c
      logical err_printed
      real      PI
      parameter( PI=3.14159265359 )
      integer equation
      real    reynolds_no, theta
c
      err_printed = .FALSE.
      shedding_eqn = 0
      if ( Reynolds_no .lt. MIN_LAMINAR_REYN / 3.0 .or.
     1     Reynolds_no .gt. MAX_LAMINAR_REYN * 3.0 ) then                 1b
          write( STD_ERR, 1 ) 'REYNOLDS_NO', Reynolds_no,
     1                        MIN_LAMINAR_REYN, MAX_LAMINAR_REYN
1         format( ' SHEDDING_EQN : FATAL : Illegal ', a, g12.5 /
     1            '   limits are ', g12.5, ' and ', g12.5 )
          err_printed = .TRUE.
      else if ( equation .ne. VK_ROSHKO   .and.
     1          equation .ne. VK_X_KOENIG .and.
     2        ( theta .lt. MIN_ANGLE .or. theta .gt. MAX_ANGLE ) ) then   1m
          write( STD_ERR, 1 ) 'THETA', theta, MIN_ANGLE, MAX_ANGLE
          err_printed = .TRUE.
      else                                                                1m
          if ( equation .eq. VK_ROSHKO ) then                             2b
              shedding_eqn = 0.212 * Reynolds_no - 4.5
          else if ( equation .eq. VK_WILLIAM ) then                       2m
              shedding_eqn = ( ( 0.00016 * Reynolds_no + 0.1816 ) *
     1            Reynolds_no - 3.3265 ) * cos( PI / 180.0 * theta )
          else if ( equation .eq. VK_P_KOENIG .or.
     1              equation .eq. VK_I_KOENIG ) then                      2m
              shedding_eqn = ( ( 0.0001448 * Reynolds_no + 0.188 ) *
     1            Reynolds_no - 3.505 ) * cos( PI / 180.0 * theta )
          else if ( equation .eq. VK_X_KOENIG ) then                      2m
              shedding_eqn = ( -1.131e-5 * Reynolds_no + 0.201 ) *
     1            Reynolds_no - 4.669
              end if                                                      2e
          end if                                                          1e
          if ( err_printed ) call ilgwait()
      return
      end



      LOGICAL FUNCTION EVKVS_CHECK( NAME, SHEDDING_FREQ, SHEDDING_ANG,
     1            AIR_SPEED, CYLIND_DIAM, KINEM_VISC )
c
c  Check that each of the numeric arguments not equal to VK_UNUSED is
c  within reasonable limits for vortex shedding from a cylinder
c
c Arguments
c  character name*(*)   # INPUT # name of calling routine
c  real shedding_freq   # INPUT # (single sided) vortex shedding frequency (Hz)
c       shedding_ang    # INPUT # angle of vortex shedding
c       air_speed       # INPUT # air speed normal to the cylinder (m/s)
c       cylind_diam     # INPUT # cylinder diameter (metres)
c       kinem_visc      # INPUT # kinematic viscosity of fluid (SI units)
c
c constants -- limits of variables
c   MAX_BLOCKAGE= 0.1      # maximum blockage factor cylinder in the channel
c   MAX_DIAM, MIN_DIAM  -- refer to file QVKVS.DEF
c   MAX_FREQ    = 10.0e3   # maximum frequency of laminar vortex shedding
c   MIN_FREQ    = 1.0e-3   # minimum frequency of laminar vortex shedding
c   MAX_SPEED   = 100.0    # maximum speed of fluid
c   MIN_SPEED   = 1.0e-3   # minimum speed of fluid
c   MAX_VISC    = 1.0e3    # maximum fluid viscosity
c   MIN_VISC    = 1.0e-9   # maximum fluid viscosity
c
      include 'stdio.def'
      include 'qvkvs.def'
      real MAX_BLOCKAGE,
     1     MAX_FREQ, MIN_FREQ, MAX_SPEED, MIN_SPEED, MAX_VISC, MIN_VISC
      parameter( MAX_BLOCKAGE=0.1,
     1     MAX_FREQ=10.0e3,   MIN_FREQ=1.0e-3,
     4     MAX_SPEED=100.0,   MIN_SPEED=1.0e-3,
     3     MAX_VISC=1.0e3,    MIN_VISC=1.0e-9 )
c
      real shedding_freq, shedding_ang, air_speed, cylind_diam,
     1     kinem_visc
      character name*(*)
      evkvs_check = .TRUE.
c
c       SHEDDING_FREQ
c
      if ( shedding_freq .ne. VK_UNUSED ) then                            1b
          if ( shedding_freq .lt. MIN_FREQ .or.
     1         shedding_freq .gt. MAX_FREQ ) then                         2b
              write( STD_ERR, 1 ) name, shedding_freq
1             format( a, 'Illegal vortex shedding frequency ',
     1                'SHEDDING_FREQ = ', g12.5 )
              write( STD_ERR, 2 ) MIN_FREQ, MAX_FREQ
2             format( 21x, 'Limits are ', g12.5, ' and ', g12.5 )
              evkvs_check = .FALSE.
              end if                                                      2e
          end if                                                          1e
c
c       SHEDDING_ANG
c
      if ( shedding_ang .ne. VK_UNUSED ) then                             1b
          if ( shedding_ang .lt. MIN_ANGLE .or.
     1         shedding_ang .gt. MAX_ANGLE ) then                         2b
              write( STD_ERR, 3 ) name, shedding_ang
3             format( a, 'Illegal angle of vortex shedding ',
     1                'SHEDDING_ANG = ', g12.5 )
              write( STD_ERR, 2 ) MIN_ANGLE, MAX_ANGLE
              evkvs_check = .FALSE.
              end if                                                      2e
          end if                                                          1e
c
c       AIR_SPEED
c
      if ( air_speed .ne. VK_UNUSED ) then                                1b
          if ( air_speed .lt. MIN_SPEED .or.
     1         air_speed .gt. MAX_SPEED ) then                            2b
              write( STD_ERR, 4 ) name, air_speed
4             format( a, 'Illegal air_speed AIR_SPEED = ',
     1                g12.5 )
              write( STD_ERR, 2 ) MIN_SPEED, MAX_SPEED
              evkvs_check = .FALSE.
              end if                                                      2e
          end if                                                          1e
c
c       CYLIND_DIAM
c
      if ( cylind_diam .ne. VK_UNUSED ) then                              1b
          if ( cylind_diam .lt. MIN_DIAM .or.
     1         cylind_diam .gt. MAX_DIAM ) then                           2b
              write( STD_ERR, 5 ) name, cylind_diam
5             format( a, 'Illegal cylinder diameter CYLIND_DIAM = ',
     1                g12.5 )
              write( STD_ERR, 2 ) MIN_DIAM, MAX_DIAM
              evkvs_check = .FALSE.
              end if                                                      2e
          end if                                                          1e
c
c       KINEM_VISC
c
      if ( kinem_visc .ne. VK_UNUSED ) then                               1b
          if ( kinem_visc .lt. MIN_VISC .or.
     1         kinem_visc .gt. MAX_VISC ) then                            2b
              write( STD_ERR, 6 ) name, kinem_visc
6             format( a, 'Illegal kinematic viscosity KINEM_VISC = ',
     1                g12.5 )
              write( STD_ERR, 2 ) MIN_VISC, MAX_VISC
              evkvs_check = .FALSE.
              end if                                                      2e
          end if                                                          1e
      if ( .not. evkvs_check ) call ilgwait()
      return
      end



      SUBROUTINE SVKVS_MODE_SET( AUTOMAGIC,
     1    SHEDDING_FREQ, REYNOLDS_NO, CYLIND_DIAM,  CYLIND_LEN,
     2    KINEM_VISC,    FORMULA,     SHEDDING_ANG, SHEDDING_MODE )
c
c Manual or automatic selection of vortex shedding mode parameters
c The cylinder terminates in a boundary layer at each end.
c
c Argument list
c  logical automagic         # INPUT  #
c  real    shedding_freq,    # INPUT  # VK_UNUSED or shedding frequency
c                            #         used to estimate Reynolds number
c          reynolds_no,      # INPUT  # VK_UNUSED or Reynolds number
c          cylind_diam,      # INPUT  # diameter of cylinder
c          cylind_len,       # INPUT  # length of cylinder
c          kinem_visc,       # INPUT  # kinematic viscosity
c          shedding_ang      # OUTPUT # vortex shedding angle
c  integer formula           # OUTPUT # vortex shedding formula
c  character *1
c          shedding_mode     # OUTPUT # vortex shedding mode "P", "I" or "X"
c
c constants
c  DEFAULT_RE     # default Reynolds number - when no other is available
c local variables
c  integer
c   item          # selected menu item
c   Jresp         # function : read integer
c  real
c   Aspect_ratio  # cylinder aspect ratio
c   Re            # Reynolds number - can be changed
c   IP_mode_line  # function : minimum aspect ratio for P shedding mode
c   XI_mode_line  #          : minimum aspect ratio for I shedding mode
c   Qresp         #          : real real
c  character
c   Cvkvs_formula*10    # function : string representation of shedding formula
c  subroutine
c   newscr              # clear alphanumeric screen
c   set_angle_formula   # determine shedding angle and formula from mode
c
      include 'stdio.def'
      include 'qvkvs.def'
      real       DEFAULT_RE
      parameter( DEFAULT_RE=100 )
c
      integer i, formula, item, Jresp
      logical automagic, err_printed
      real    shedding_freq, reynolds_no, cylind_diam, cylind_len,
     1        kinem_visc, shedding_ang, Aspect_ratio, Re,
     2        IP_mode_line, XI_mode_line, Qresp
      character *1 shedding_mode, Cvkvs_formula*10
c
c
c Use shedding_frequency to estimate Reynolds number if Reynolds number
c is not supplied.
      if ( Reynolds_no .eq. VK_UNUSED ) then                              1b
          if ( shedding_freq .eq. VK_UNUSED .or.
     1         kinem_visc .eq. VK_UNUSED ) then                           2b
              Re = DEFAULT_RE
              write( STD_ERR, 1 ) Re
1             format( ' SVKVS_MODE_SET : WARNING : Assume Re= ',g12.5 )
              call ilgwait()
          else                                                            2m
              Re = ( shedding_freq * cylind_diam **2 / kinem_visc
     1               + 4.5 ) / 0.212
              end if                                                      2e
      else                                                                1m
          Re = Reynolds_no
          end if                                                          1e
c
      Aspect_ratio = cylind_len / cylind_diam / 2.0
c
c automagic selection logic
      if ( automagic ) then                                               1b
          if ( aspect_ratio .gt. IP_mode_line( Re ) ) then                2b
              shedding_mode = 'P'
          else if ( aspect_ratio .gt. XI_mode_line( Re ) ) then           2m
              shedding_mode = 'I'
          else                                                            2m
              shedding_mode = 'X'
              end if                                                      2e
          call set_angle_formula( Re, shedding_mode,
     1                            formula, shedding_ang )
      else                                                                1m
c
c manual selection
c     start of loop                                                       2b
2         call newscr()
          write( STD_OUT, 3 ) shedding_mode, shedding_ang,
     1        Cvkvs_formula( formula ), Re, Reynolds_no
3         format( ' 0) Exit with listed output values' /
     1        ' 1) Vortex shedding mode    (output) ', a /
     2        ' 2) Vortex shedding angle   (output) ', f6.2, ' deg' /
     3        ' 3) Vortex shedding formula (output) ', a /
     4        ' 4) Reynolds number      (temporary) ', f7.2 /
     5        ' 5) Reset Reynolds number to (input) ', f7.2 /
     6        ' 6) Select mode automagically' )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c     Exit
c
          if ( item .eq. 0 ) then                                         3b
              return
c
c     select vortex shedding mode
c
          else if ( item .eq. 1 ) then                                    3m
              if ( shedding_mode .eq. 'P' ) then                          4b
                  shedding_mode = 'I'
              else if ( shedding_mode .eq. 'I' ) then                     4m
                  shedding_mode = 'X'
              else                                                        4m
                  shedding_mode = 'P'
                  end if                                                  4e
              call set_angle_formula( Re, shedding_mode,
     1                                formula, shedding_ang )
c
c     select vortex shedding angle
c
          else if ( item .eq. 2 ) then                                    3m
              write( STD_OUT, 4 ) ' Shedding angle ? '
4             format( a )
              shedding_ang = Qresp( STD_IN, 0.0, 23.0, MAX_TRIES )
              if ( shedding_ang .ge. 23.0 ) then                          4b
                  shedding_ang = 23.0
                  shedding_mode = 'X'
                  formula = VK_X_KOENIG
              else if ( shedding_ang .le. 0.0 ) then                      4m
                  shedding_ang = 0.0
                  shedding_mode = 'P'
                  if ( formula .eq. VK_X_KOENIG .or.
     1                 formula .eq. VK_I_KOENIG )
     2                formula = VK_P_KOENIG
              else                                                        4m
                  if ( formula .eq. VK_P_KOENIG .or.
     1                 formula .eq. VK_X_KOENIG ) then                    5b
                      formula = VK_I_KOENIG
                  else if ( formula .eq. VK_ROSHKO ) then                 5m
                      formula = VK_WILLIAM
                      end if                                              5e
                  end if                                                  4e
c
c     select vortex shedding formula
c
          else if ( item .eq. 3 ) then                                    3m
              write( STD_OUT, 5 ) ( i, Cvkvs_formula(i*16), i = 0, 4 )
5             format( ' Select' / ( 2x, i1, ') ', a ) )
              formula = Jresp( STD_IN, 0, 4, MAX_TRIES ) * 16
              if ( formula .eq. VK_ROSHKO ) then                          4b
                  shedding_mode = 'P'
                  shedding_ang = 0.0
              else if ( formula .eq. VK_WILLIAM ) then                    4m
                  if ( shedding_mode .eq. 'X' ) shedding_mode = 'I'
              else if ( formula .eq. VK_P_KOENIG ) then                   4m
                  shedding_mode = 'P'
                  shedding_ang = 0.0
              else if ( formula .eq. VK_I_KOENIG ) then                   4m
                  shedding_mode = 'I'
              else if ( formula .eq. VK_X_KOENIG ) then                   4m
                  shedding_mode = 'X'
                  shedding_ang = 23.0
                  end if                                                  4e
c
c     Select Reynolds number
c
          else if ( item .eq. 4 ) then                                    3m
              write( STD_OUT, 4 ) ' Reynolds number ? '
              Re = Qresp( STD_IN, MIN_LAMINAR_REYN, MAX_LAMINAR_REYN,
     1                    MAX_TRIES )
c
c     Reset Reynolds number
          else if ( item .eq. 5 ) then                                    3m
              Re = Reynolds_no
c
c     Automagic mode selection
c
          else if ( item .eq. 6 ) then                                    3m
              if ( aspect_ratio .gt. IP_mode_line( Re ) ) then            4b
                  shedding_mode = 'P'
              else if ( aspect_ratio .gt. XI_mode_line( Re ) ) then       4m
                  shedding_mode = 'I'
              else                                                        4m
                  shedding_mode = 'X'
                  end if                                                  4e
              call set_angle_formula( Re, shedding_mode,
     1                                formula, shedding_ang )
              end if                                                      3e
          goto 2
c         end of loop                                                     2e
          end if                                                          1e
      return
      end



      SUBROUTINE SET_ANGLE_FORMULA( REYNOLDS_NO, MODE, FORMULA, ANGLE )
c
c Determines the vortex shedding FORMULA and shedding ANGLE from specified
c REYNOLDS_NO and MODE of vortex shedding.
c
c Argument list
c  real      Reynolds_no
c  character mode*1   # vortex shedding mode
c  integer   formula  # vortex shedding formula
c  real      angle    # angle of vortex shedding
c
c local variables
c  logical ok       # mode specification is OK
c  real    Rek      # Reynolds number transformed for fitted curve
c
      include 'stdio.def'
      include 'qvkvs.def'
c
      logical   ok
      character mode*1
      integer   formula
      real      angle, Reynolds_no, Rek
c
      ok = mode .eq. 'P' .or. mode .eq. 'I' .or. mode .eq. 'X'
      if ( .not. ok ) then                                                1b
          write( STD_ERR, 1 )
1         format( ' SET_ANGLE_FORMULA : WARNING : Illegal formula ',
     1        'specified. Assume I-mode, Koenig formula' )
          formula = VK_I_KOENIG
          call ilgwait()
          end if                                                          1e
      if ( mode .eq. 'P' ) then                                           1b
          if ( formula .eq. VK_I_KOENIG .or. formula .eq. VK_X_KOENIG )
     1        formula = VK_P_KOENIG
          angle = 0.0
      else if ( mode .eq. 'I' .or. .not. ok ) then                        1m
          if ( formula .eq. VK_P_KOENIG .or. formula .eq. VK_X_KOENIG )
     1            then                                                    2b
              formula = VK_I_KOENIG
          else if ( formula .eq. VK_ROSHKO ) then                         2m
              formula = VK_WILLIAM
              end if
          if ( Reynolds_no .le. 49.7527 .or.
     1         Reynolds_no .gt. 170.0 ) then                              2m
              angle = 0.0
          else if ( 49.76 .lt. Reynolds_no .and.
     1              Reynolds_no .lt. 65.0 ) then                          2m
              Rek = ( Reynolds_no - 65.0 ) **2
              angle = sqrt( max( 0.0,
     1          ( ( Rek * -5.0434937e-6 + 1.6299047e-4 ) * Rek
     2              - 0.23672760 ) * Rek + 109.59316 ) )
          else if ( 65.0 .le. Reynolds_no .and.
     1              Reynolds_no .le. 170.0 ) then                         2m
              Rek = alog10( Reynolds_no / 100.0 )
              angle = ( ( ( 517.061642 * Rek - 208.655108 ) * Rek +
     1            70.5516852 ) * Rek - 10.2161700 ) * Rek + 12.5659993
              end if                                                      2e
      else if ( mode .eq. 'X' ) then                                      1m
          formula = VK_X_KOENIG
          angle = 23.0
          end if                                                          1e
      return
      end



      REAL FUNCTION IP_MODE_LINE( REYNOLDS_NO )
c
c This is an ESTIMATE of the dividing line between the P-mode and I-mode
c of vortex shedding for a cylinder as a function of Reynolds number. The
c returned result is the minimum aspect ratio required for parallel vortex
c shedding at the middle of the span. This polynomial estimate was formulated
c from the data provided in Figs 9 and 10 of Koenig, Eisenlohr & Eckelmann
c (1990) Physics of Fluids, Vol A2(9) 1607-1614.
c
      include 'stdio.def'
      real Reynolds_no, Rek
      if ( Reynolds_no .le. 55.0 ) then
          IP_mode_line = 23.35202
      else if ( Reynolds_no .ge. 160.0 ) then
          IP_mode_line = 12344.74
      else
          Rek = Reynolds_no - 65.0
          IP_mode_line = ( ( 0.011658098 * Rek + 0.225348325 ) +
     1                   2.887080546 ) * Rek + 41.346093914
          end if
      return
      end



      REAL FUNCTION XI_MODE_LINE( REYNOLDS_NO )
c
c This is an ESTIMATE of the dividing line between the X-mode and I-mode
c of vortex shedding for a cylinder as a function of Reynolds number. The
c returned result is the minimum aspect ratio required for I-mode vortex
c shedding at the middle of the span. This polynomial estimate was formulated
c from the data provided in Figs 9 and 10 of Koenig, Eisenlohr & Eckelmann
c (1990) Physics of Fluids, Vol A2(9) 1607-1614.
c P.S. Sorry this is not a rational function, but I could not get a decent
c      fit to the data any other way.
c
      include 'stdio.def'
      real Reynolds_no, Rek
      if ( Reynolds_no .le. 60.0 ) then
          XI_mode_line = 22.4388538
      else if ( Reynolds_no .ge. 160.0 ) then
          XI_mode_line = 37.185188
      else
          Rek = alog10( Reynolds_no / 100 )
          XI_mode_line = Reynolds_no / ( ( ( ( ( 4130.1803189 * Rek +
     1        162.281737918 ) * Rek - 154.632363582 ) * Rek +
     2        48.50384903 ) * Rek + 3.350012169 ) * Rek + 1.167937982 )
          end if
      return
      end



      SUBROUTINE SVKVS_SET_PAR( SET_DEFAULTS )
c
c Set parameters for vortex shedding analysis
c
c Arguments
c  logical set_defaults    # .TRUE. set default parameters for the analysis
c                          # parameters if this has not been done yet
c
c constants
c   DEF_AR_COEFFS = 24     # default number of AutoRegression coefficients
c
c   MAX_AR_INCREM = -8     # maximum increment for number of AR coeffs
c   MIN_AR_INCREM = 8      # minimum increment for number of AR coeffs
c   DEF_AR_INCREM = 4      # default increment for number of AR coeffs
c
c   MIN_N_PLOT    = 25     # minimum number of spectral points to plot
c   DEF_N_PLOT    = 250    # default number of spectral points to plot
c
c   MAX_CLOSENESS = 0.70   # maximum || Maximum relative frequency of
c   MIN_CLOSENESS = 0.04   # minimum || one peak in a double peak
c   DEF_CLOSENESS = 0.30   # default || compared with the other
c
c   MIN_RELATNESS = 0.01   # maximum || Minimum relative height
c   MAX_RELATNESS = 0.95   # minimum || of the lower peak of a
c   DEF_RELATNESS = 0.10   # default || double peak
c
c   MIN_WIDENESS  = 0.003  # maximum || Maximum width of
c   MAX_WIDENESS  = 0.30   # minimum || an acceptably
c   DEF_WIDENESS  = 0.05   # default || sharp peak
c
c   MIN_TALLNESS  = 0.01   # maximum || Fraction of peak
c   MAX_TALLNESS  = 0.7071 # minimum || height where peak
c   DEF_TALLNESS  = 0.20   # default || width is measured
c
c   MIN_SAMENESS  = 0.01   # maximum || Fraction of selected peak
c   MAX_SAMENESS  = 1.00   # minimum || height for detecting
c   DEF_SAMENESS  = 0.30   # default || multiple major peaks
c
c   MIN_CYL_LEN   = 0.227  # minimum test section height
c   MAX_CYL_LEN   = 0.280  # maximum test section height
c   DEF_CYL_LEN   = 0.24016# default test section height
c
c   MAX_OBLIQNESS = 5.0    # maximum || detectable change in
c   MIN_OBLIQNESS = 0.001  # minimum || angle of oblique
c   DEF_OBLIQNESS = 0.05   # default || vortex shedding
c
c local variables
c  logical
c   first_pass  # .TRUE. before SVKVS_SET_PAR is called for the first time
c  integer
c   capt_len    # length of capture file name
c   item        # selected menu item
c   Jresp       # function : read integer
c   str_length  #          : non-blank length of string
c  real
c   Qresp       # function : read real
c  subroutine
c   newscr      # hpprev : clear alphanumeric screen
c
      include 'stdio.def'
      include 'qvkvs.def'
      integer                               DEF_AR_COEFFS,
     1        MIN_AR_INCREM, MAX_AR_INCREM, DEF_AR_INCREM,
     2        MIN_N_PLOT,                   DEF_N_PLOT
      real    MIN_CLOSENESS, MAX_CLOSENESS, DEF_CLOSENESS,
     1        MIN_RELATNESS, MAX_RELATNESS, DEF_RELATNESS,
     2        MIN_WIDENESS,  MAX_WIDENESS,  DEF_WIDENESS,
     3        MIN_TALLNESS,  MAX_TALLNESS,  DEF_TALLNESS,
     4        MIN_SAMENESS,  MAX_SAMENESS,  DEF_SAMENESS,
     5        MIN_CYL_LEN,   MAX_CYL_LEN,   DEF_CYL_LEN,
     6        MIN_OBLIQNESS, MAX_OBLIQNESS, DEF_OBLIQNESS
      parameter(                                  DEF_AR_COEFFS=48,
     2    MIN_AR_INCREM=8,    MAX_AR_INCREM=-8,   DEF_AR_INCREM=8,
     3    MIN_N_PLOT=25,                          DEF_N_PLOT=250,
     4    MIN_CLOSENESS=0.04, MAX_CLOSENESS=0.70, DEF_CLOSENESS=0.30,
     5    MIN_RELATNESS=0.01, MAX_RELATNESS=0.95, DEF_RELATNESS=0.10,
     6    MIN_WIDENESS=0.003, MAX_WIDENESS=0.30,  DEF_WIDENESS=0.05,
     7    MIN_TALLNESS=0.01,  MAX_TALLNESS=0.707, DEF_TALLNESS=0.20,
     8    MIN_SAMENESS=0.01,  MAX_SAMENESS=1.00,  DEF_SAMENESS=0.30,
     9    MIN_CYL_LEN=0.227,  MAX_CYL_LEN=0.280,  DEF_CYL_LEN=0.24016,
     a    MIN_OBLIQNESS=0.001,MAX_OBLIQNESS=5.0,  DEF_OBLIQNESS=0.05 )
c
      logical set_defaults, first_pass
      integer capt_len, item, Jresp, str_length
      real Qresp
      include 'qvkvs.cmn'
      save first_pass
      data first_pass / .TRUE. /
c
c       set default parameters for the analysis parameters if this
c       has not been done yet
c
      if ( set_defaults ) then                                            1b
          if ( first_pass ) then                                          2b
              init_AR_coeffs = DEF_AR_COEFFS
              increm_n_AR    = DEF_AR_INCREM
              n_plot         = DEF_N_PLOT
              closeness      = DEF_CLOSENESS
              relativeness   = DEF_RELATNESS
              wideness       = DEF_WIDENESS
              tallness       = DEF_TALLNESS
              sameness       = DEF_SAMENESS
              cyl_length     = DEF_CYL_LEN
              obliqueness    = DEF_OBLIQNESS
              capture_file   = ' '
              end if                                                      2e
          first_pass = .FALSE.
      else                                                                1m
c
c       allow user to set the values of analysis parameters
c
c         start of menu loop                                              2b
1             call newscr()
              capt_len = max( 1, str_length( capture_file ) )
              write( STD_OUT, 2 ) init_AR_coeffs, increm_n_AR, n_plot,
     1            closeness, relativeness, wideness, tallness, sameness,
     2            cyl_length, obliqueness, capture_file(:capt_len)
2         format( ' Select parameters for vortex shedding analysis' /
     1      '  0) Exit' /
     2      '  1) Initial number of AR coefficients', 16x, i2 /
     3      '  2) Increment for number of AR coefficients', 9x, i3 /
     4      '  3) Number of plotted spectrum points', 14x, i4 /
     5      '  4) Maximum relative frequency of one peak of a   ', f7.5/
     6           7x, 'double peak compared with the other' /
     7      '  5) Minimum relative height of the lower peak', 5x, f7.5 /
     8           7x, 'of a double peak' /
     9      '  6) Maximum width of an acceptably sharp peak', 5x, f7.5 /
     a      '  7) Fraction of peak height for width measurement ', f7.5/
     b      '  8) Fraction of selected peak height required for ', f7.5/
     c           7x, 'other peaks to detect multiple major peaks' /
     d      '  9) Length of vortex shedding cylinder            ', f7.5/
     e      ' 10) Smallest detectable change in shedding angle  ', f7.5/
     f      ' 11) Set all parameters to defaults' /
     g      ' 12) Capture file name', 29x, a, '  ', $ )
              item = Jresp( STD_IN, 0, 12, MAX_TRIES )
c
c       EXIT
c
              if ( item .eq. 0 ) then                                     3b
                  return
c
c       initial number of AR coefficients
c
              else if ( item .eq. 1 ) then                                3m
                  write( STD_OUT, 3 )
3                 format( ' Initial number of AR coefficients ? ', $ )
                  init_AR_coeffs = Jresp( STD_IN,
     1                MIN_AR_COEFFS, MAX_AR_COEFFS, MAX_TRIES )
c
c       increment for number of AR coeffs on 2nd etc analyses
c
              else if ( item .eq. 2 ) then                                3m
                  write( STD_OUT, 4 )
4                format( ' Increment for number of AR coefficients ? ' )
                  increm_n_AR = Jresp( STD_IN,
     1                MIN_AR_INCREM, MAX_AR_INCREM, MAX_TRIES )
c
c       number of plotted spectrum points
c
              else if ( item .eq. 3 ) then                                3m
                  write( STD_OUT, 5 )
5                 format( ' Number of plotted spectrum points ? ', $ )
                  n_plot = Jresp( STD_IN,
     1                MIN_N_PLOT, max_n_plot, MAX_TRIES )
c
c       maximum relative frequency of one peak in a double peak
c       compared with the other
c
              else if ( item .eq. 4 ) then                                3m
                  write( STD_OUT, 6 )
6                 format( ' Maximum relative frequency of one peak of ',
     1                'a double peak' /
     2                33x, 'compared with the other ? ', $ )
                  closeness = Qresp( STD_IN,
     1                MIN_CLOSENESS, MAX_CLOSENESS, MAX_TRIES )
c
c       minimum relative height of the lower peak of a double peak
c
              else if ( item .eq. 5 ) then                                3m
                  write( STD_OUT, 7 )
7                 format( '  Minimum relative height of the lower ',
     1                'peak of a double peak ? ', $ )
                  relativeness = Qresp( STD_IN,
     1                MIN_RELATNESS, MAX_RELATNESS, MAX_TRIES )
c
c       maximum width of a peak which is acceptably sharp
c
              else if ( item .eq. 6 ) then                                3m
                  write( STD_OUT, 8 )
8                 format( ' Maximum width of an acceptably sharp ',
     1                'peak ? ', $ )
                  wideness = Qresp( STD_IN,
     1                MIN_WIDENESS, MAX_WIDENESS, MAX_TRIES )
c
c       fraction of peak height at which width is measured
c
              else if ( item .eq. 7 ) then                                3m
                  write( STD_OUT, 9 )
9                 format( ' Fraction of peak height for width ',
     1                'measurement ? ', $ )
                  tallness = Qresp( STD_IN,
     1                MIN_TALLNESS, MAX_TALLNESS, MAX_TRIES )
c
c       fraction of selected peak height required for other
c       peaks to detect multiple major peaks
c
              else if ( item .eq. 8 ) then                                3m
                  write( STD_OUT, 10 )
10                format( ' Fraction of selected peak height ',
     1                'required for other peaks' /
     2                28x, 'to detect multiple major peaks ? ', $ )
                  sameness = Qresp( STD_IN,
     1                MIN_SAMENESS, MAX_SAMENESS, MAX_TRIES )
c
c       length of vortex shedding cylinder
c
              else if ( item .eq. 9 ) then                                3m
                  write( STD_OUT, 11 )
11                format( ' Length of vortex shedding cylinder ? ', $ )
                  cyl_length = Qresp( STD_IN,
     1                MIN_CYL_LEN, MAX_CYL_LEN, MAX_TRIES )
c
c        Smallest detectable change in shedding angle
c
              else if ( item .eq. 10 ) then                               3m
                  write( STD_OUT, 13 )
13      format( ' Smallest detectable change in shedding angle ? ', $ )
                  obliqueness = Qresp( STD_IN,
     1                MIN_OBLIQNESS, MAX_OBLIQNESS, MAX_TRIES )
c
c       Set all parameters to defaults
c
              else if ( item .eq. 11 ) then                               3m
                  init_AR_coeffs = DEF_AR_COEFFS
                  increm_n_AR    = DEF_AR_INCREM
                  n_plot         = DEF_N_PLOT
                  closeness      = DEF_CLOSENESS
                  relativeness   = DEF_RELATNESS
                  wideness       = DEF_WIDENESS
                  tallness       = DEF_TALLNESS
                  sameness       = DEF_SAMENESS
                  cyl_length     = DEF_CYL_LEN
                  obliqueness    = DEF_OBLIQNESS
c
c       Capture file name
c
              else if ( item .eq. 12 ) then                               3m
                  write( STD_OUT, 12 )
12                format( ' Capture file name ? ', $ )
                  call cresp( STD_IN, capture_file, MAX_TRIES )
                  end if                                                  3e
              goto 1
c             end of menu loop                                            2e
          end if                                                          1e
      end



      LOGICAL FUNCTION EVKVS_IS_PROB( PROBLEM )
c
c Returns .TRUE. if any of the PROBLEM flags are set .
c
c Argument
c   logical problem(MAX_PROBLEM)
c              # set of switches describing potential problems
c
c local variables
c  integer i   # index for problem switch arrays
c
c
      include 'stdio.def'
      include 'qvkvs.def'
c
      logical problem(MAX_PROBLEM)
      integer i
c
      evkvs_is_prob = .FALSE.
      do 1 i = 1, MAX_PROBLEM                                            1b
          evkvs_is_prob = evkvs_is_prob .or. problem(i)
1         continue                                                        1e
      return
      end



      SUBROUTINE SVKVS_PR_PROB( PROBLEM )
c
c Print list of problems found. Use 25 lines of display.
c
c Argument
c   logical problem(MAX_PROBLEM)
c                     # set of switches describing potential problems
c
c local variables
c  integer i          # index for problem switch and text arrays
c          text_len   # length of string to be printed
c          str_length # function : non-blank length of string
c
      include 'stdio.def'
      include 'qvkvs.def'
c
      logical problem(MAX_PROBLEM)
      integer i, line_count, text_len, str_length
      character prob_text(MAX_PROBLEM)*53
      save prob_text
      data prob_text /
     1    'Have made more than two attempts at analysis',
     2    'Found a double peak',
     3    'Found more than one peak of comparable height',
     4    'Spectral peak occurred at 0 Hz or peak not found',
     5    'Main spectral peak too wide for accurate measurement',
     6    'Vortex shedding mode, angle or formula changed',
     7    'Reynolds number beyond laminar vortex shedding limits' /
c
      line_count = 0
      do 2 i = 1, MAX_PROBLEM                                             1b
          if ( problem(i) ) then                                          2b
              text_len = max( 1, str_length( prob_text(i) ) )
              write( STD_OUT, 1 ) prob_text(i)(:text_len)
1             format( 1x, a )
              line_count = line_count + 1
              end if                                                      2e
2         continue                                                        1e
      do 4 i = 1, 25 - line_count                                         1b
          write( STD_OUT, 3 )
3         format()
4         continue                                                        1e
      return
      end



      BLOCK DATA QVKVS_BLOCK
c
c Initialization of global variables.
c Refer to SVKVS_SET_PAR for description of constants
c
      include 'stdio.def'
      integer DEF_AR_COEFFS,  DEF_AR_INCREM,      DEF_N_PLOT
      real    DEF_CLOSENESS,  DEF_RELATNESS,      DEF_WIDENESS,
     1        DEF_TALLNESS,   DEF_SAMENESS
      parameter(
     1    DEF_AR_COEFFS=48,   DEF_AR_INCREM=8,    DEF_N_PLOT=250,
     2    DEF_CLOSENESS=0.25, DEF_RELATNESS=0.10, DEF_WIDENESS=0.05,
     3    DEF_TALLNESS=0.20,  DEF_SAMENESS=0.30 )
c
      include 'qvkvs.cmn'
c
      data plotted / .FALSE. /
      data init_AR_coeffs, increm_n_AR,   max_n_plot, n_plot /
     1     DEF_AR_COEFFS,  DEF_AR_INCREM, DEF_N_PLOT, DEF_N_PLOT /
      data closeness,     relativeness,  wideness,     tallness,
     1     sameness /
     2     DEF_CLOSENESS, DEF_RELATNESS, DEF_WIDENESS, DEF_TALLNESS,
     3     DEF_SAMENESS /
      data capture_file / ' ' /
      end
