c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM TEST_QVKVS
c
c Program to test QVKVS_RAW, QVKVS_EQN .
c
c constants
c  MAX_DATA = 16384   # space available for hot-wire data
c  MAX_WORK = 4096    # size of WORK array
c  action_name(8)*18  # action description strings
c
c local variables
c  logical
c   Eyes              # function : Convert 'yes', 'no' to logical
c  integer
c   action            # Specifies the quantity to calculate
c   action_len        # length of action description string
c   column            # data column to be analysed
c   file_len          # length of hot-wire data file name
c   force_plot        # force plotting of shedding spectra
c   got_data          # hot-wire data is available
c   hot_wire_lu       # actual logical unit number for hot-wire data file
c   i                 # array index
c   item              # selected menu item
c   n_columns         # number of columns of hot-wire data
c   n_rows            # number of rows in hot-wire data file
c   reserved_lu       # reserved value of logical unit number for data file
c   Jresp             # function : read integer
c   new_lun           #          : allocate new logical unit number
c   open_file         #          : open file
c   str_length        #          : non-blank length of string
c  real
c   air_speed         # air speed normal to the cylinder (m/s)
c   cylind_diam       # cylinder diameter (metres)
c   eqn_result        # result returned from QVKVS_EQN or QVKVS_RAW
c   hot_wire_data(MAX_DATA) # data array containing voltage-time data from
c                           # the hot-wire(s) . At the top of each column
c                           # is the time step
c   kinem_visc        # kinematic viscosity of fluid (SI units)
c   shedding_freq     # (single sided) vortex shedding frequency (Hz)
c   test_sect_width   # wind tunnel test section width (metres)
c   work(MAX_WORK)    # work array for QVKVS_RAW
c   Qresp             # function : read real
c   qvkvs_eqn         #          : Calculator for ROSHKO'S formula
c   qvkvs_raw         #          : air speed from hot-wire in cylinder wake
c  character
c   action_name(8)*18 # action description strings
c   hot_wire_file*32  # name of file containing hot-wire data
c   Cyes*5            # function : 'yes', 'no', 'query'
c  subroutine
c   cresp             # read string
c   ilgwait           # ilglib : wait for <RETURN>
c   newscr            # hpprev : clear alphanumeric screen
c   read_data         # rows & columns of data from file
c
      include 'stdio.def'
      include 'qvkvs.def'
      character action_name(9)*18
      integer   MAX_DATA, MAX_WORK
      parameter( MAX_DATA=16384, MAX_WORK=4096 )
      data  action_name /
     1    'Laminar shedding ?', 'Max Reynolds no.  ',
     2    'Min Reynolds no.  ', 'Shedding frequency',
     3    'Air speed         ', 'Cylinder diameter ',
     4    'Strouhal number   ', 'Reynolds number   ',
     5    'Roshko number     ' /
c
      logical eyes
      integer action, action_len, column, file_len, force_plot,
     1        formula, got_data, hot_wire_lu, i, item, n_columns,
     2        n_rows, reserved_lu,
     3        Jresp, new_lun, open_file, str_length
      real    air_speed, cylind_diam, eqn_result,
     2        hot_wire_data(MAX_DATA), kinem_visc, shedding_freq,
     3        Reynolds_no, Roshko_no, Strouhal_no, shedding_ang,
     4        work(MAX_WORK), Qresp, qvkvs_eqn, qvkvs_raw
      character hot_wire_file*32, Cyes*5, Cvkvs_formula*10
c
      data action, column, force_plot, got_data /
     1     VK_VELOCITY, 1, YES, NO /
      data air_speed, cylind_diam, eqn_result, kinem_visc,
     1  shedding_ang, shedding_freq /
     2  1.0, 0.00159, 0.0, 1.5e-5, 10.0, 100.0 /
      data hot_wire_file / ' ' /
c
c
c     start of menu loop                                                  1b
1         call newscr()
          action_len = max( 1, str_length( action_name(action) ) )
          file_len   = max( 1, str_length( hot_wire_file ) )
          write( STD_OUT, 2 )
     1        action_name(action)(:action_len),
     2        cvkvs_formula( formula ),
     3        shedding_freq,
     4        air_speed,
     5        cylind_diam * 1000.0,
     6        kinem_visc,
     7        shedding_ang,
     8        action_name(action)(:action_len),
     9        hot_wire_file(:file_len),
     a        Cyes(got_data),
     b        column,
     c        Cyes(force_plot),
     d        action_name(action)(:action_len),
     e        eqn_result, Roshko_no, Reynolds_no, Strouhal_no
2         format( ' Select' / '   0) Exit' /
     1        '   1) What to calculate         ', a /
     2        '   2) Vortex shedding formula   ', a /
     3        '   3) Vortex shedding frequency ', g12.5, ' Hz' /
     4        '   4) Normal air speed          ', g12.5, ' m/s' /
     5        '   5) Cylinder diameter         ', g12.5, ' mm' /
     6        '   6) Fluid kinematic viscosity ', g12.5, ' SI units' /
     7        '   7) Vortex shedding angle     ', g12.5, ' degrees' /
     8        '   8) Calculate ', a, ' using QVKVS_EQN' /
     9        '   9) Hot-wire data file name   ', a /
     a        '  10) Read hot-wire data        ', a /
     b        '  11) Selected data column      ', i2 /
     c        '  12) Force plotting            ', a /
     d        '  13) Calculate air speed using QVKVS_RAW ' /
     e        '  14) Transfer previous result to ', a /
     f        '      Previous result = ', g12.5 /
     g        '   Roshko, Reynolds & Strouhal no = ', 3g12.5e1, ' ', $ )
          item = Jresp( STD_IN, 0, 14, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              stop
c
c           WHAT TO CALCULATE
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 ) ( i, action_name(i), i = 1, 9 )
3             format( ' Select' / ( i3, ') ', a ) )
              action = Jresp( STD_IN, 1, 9, MAX_TRIES )
c
c           VORTEX SHEDDING FORMULA
c
          else if ( item .eq. 2 ) then
              write( STD_OUT, 3 )
     1            ( i, cvkvs_formula(i*16), i = 0, 4 )
              formula = 16 * Jresp( STD_IN, 0, 4, MAX_TRIES )
c
c           VORTEX SHEDDING FREQUENCY
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' ? ', $ )
              shedding_freq = Qresp( STD_IN, VK_UNUSED, 1.0e9,
     1            MAX_TRIES )
c
c           NORMAL AIR SPEED
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 4 )
              air_speed = Qresp( STD_IN, VK_UNUSED, 100.0, MAX_TRIES )
c
c           CYLINDER DIAMETER
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 4 )
              cylind_diam = Qresp( STD_IN, VK_UNUSED, 100.0, MAX_TRIES )
     1            / 1000.0
c
c           FLUID KINEMATIC VISCOSITY
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 4 )
              kinem_visc = Qresp( STD_IN, VK_UNUSED, 1.0e6, MAX_TRIES )
c
c           VORTEX SHEDDING ANGLE
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 4 )
              shedding_ang =
     1            Qresp( STD_IN, VK_UNUSED, 30.0, MAX_TRIES )
c
c           CALCULATE USING QVKVS_EQN
c
          else if ( item .eq. 8 ) then                                    2m
              eqn_result = qvkvs_eqn( action + formula,
     1            shedding_freq, shedding_ang, air_speed, cylind_diam,
     2            kinem_visc )
c
c           HOT-WIRE DATA FILE NAME
c
          else if ( item .eq. 9 ) then                                    2m
              write( STD_OUT, 4 )
              call cresp( STD_IN, hot_wire_file, MAX_TRIES )
              got_data = NO
c
c           READ HOT-WIRE DATA
c
          else if ( item .eq. 10 ) then                                   2m
              if ( reserved_lu .eq. 0 )
     1            reserved_lu = new_lun()
              hot_wire_lu =
     1            open_file( reserved_lu, hot_wire_file, 'old', 0 )
              if ( hot_wire_lu .gt. 0 ) then                              3b
                  call read_data( hot_wire_lu, hot_wire_data, MAX_DATA,
     1                            n_rows, n_columns )
                  close( UNIT=hot_wire_lu )
                  got_data = NO
                  if ( n_rows .gt. 2 .and. n_columns .ge. 1 )
     1                got_data = YES
                  write( STD_OUT, 5 )
5     format( ' NOTE : Time step is at the top of each column' )
                  call ilgwait()
                  end if                                                  3e
c
c           SELECTED DATA COLUMN
c
          else if ( item .eq. 11 ) then                                   2m
              write( STD_OUT, 4 )
              column = Jresp( STD_IN, 1, n_columns, MAX_TRIES )
c
c           FORCE PLOTTING
c
          else if ( item .eq. 12 ) then                                   2m
              force_plot = mod( force_plot + 1, 2 )
c
c           CALCULATE AIR SPEED USING QVKVS_RAW
c
          else if ( item .eq. 13 ) then                                   2m
              eqn_result = qvkvs_raw( hot_wire_data(n_columns+column),
     1            n_columns, n_rows - 1, hot_wire_data(column),
     2            cylind_diam, kinem_visc,
     3            work, Eyes( force_plot, 'Force Plotting ?' ) )
c
c           Transfer previous result to variable
c
          else if ( item .eq. 14 ) then                                   2m
              if ( action .eq. VK_FREQUENC ) then                         3b
                  shedding_freq = eqn_result
              else if ( action .eq. VK_VELOCITY ) then                    3m
                  air_speed     = eqn_result
              else if ( action .eq. VK_DIAMETER ) then                    3m
                  cylind_diam   = eqn_result
              else if ( action .eq. VK_ROSHK_NO ) then
                  Roshko_no = eqn_result
              else if ( action .eq. VK_REYNOLDS ) then
                  Reynolds_no = eqn_result
              else if ( action .eq. VK_STROUHAL ) then
                  Strouhal_no = eqn_result
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end


