c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM EXPMODEL
c
c Calculate mean velocity profiles with model parameters from
c experimental data at free stream speed of 1.8 m/s.
c This program tests the code in BLM.FOR for the Spalding (1961) law of
c the wall.
c
      include 'stdio.def'
      include 'blm.def'
      integer REQD_COEFFS, COEFFS_SIZE, U_SIZE
      real FIRST_Y, LAST_Y, Y_STEP
      parameter( REQD_COEFFS=6, COEFFS_SIZE=144,  U_SIZE=2048,
     1           FIRST_Y=2.50, LAST_Y=500.0, Y_STEP=1.1220185 )
c
      integer coeffs_key(REQD_COEFFS), i, j, i_coeff, i_res,
     1    n_columns, n_profiles, n_res, res_lu, param_lu,
     2    iargc, new_lun, open_file
      logical error_blm
      real    coeffs(COEFFS_SIZE), u_plus(U_SIZE), y_plus,
     1        uplus_blm
      character err_msg*64, res_file*64, param_file*64
c
      data coeffs_key / BLM_fs_speed, BLM_friction_vel, BLM_Karmen,
     1                  BLM_Constant, BLM_PI_wake, BLM_delta /
c
      if ( iargc() .le. 0 ) then
          write( STD_ERR, 1 )
1         format( ' Usage : ' /
     1            '  EXPMODEL <file-of-B/L-model-specifications>' )
      else
c
c       read model parameters
c
          call getarg( 1, param_file )
          param_lu = open_file( new_lun(), param_file, 'old', 0 )
          if ( param_lu .gt. 0 ) then
              call read_data( param_lu, coeffs, COEFFS_SIZE,
     1                        n_profiles, n_columns )
              if ( n_columns .ne. REQD_COEFFS ) then
                  write( STD_ERR, 2 )
2                 format( ' EXPMODEL: ABORT: need exactly 6 numbers ',
     1                ' in each line of the spec file.' /
     2                ' i.e. U_0, U_tau, kappa, C, PI, delta^+' )
              else
c
c       set up the model
c
                  call set_blm( BLM_Spld_68C, 0.0 )
                  call set_blm( BLM_nondim, 0.0 )
                  i_res = 1
c
c       main loop -- once per profile
c
                  do 6 i = 1, n_profiles
                      i_coeff = REQD_COEFFS * ( i - 1 )
                      do 4 j = 1, REQD_COEFFS
                          call set_blm(coeffs_key(j),coeffs(i_coeff+j))
                          if ( error_blm( err_msg ) ) then
                              write( STD_ERR, 3 ) err_msg
3                             format( ' EXPMODEL: ABORT: ', a )
                              stop
                              end if
4                         continue
c
c       calculate the boundary layer profile
c
                      y_plus = FIRST_Y
5                     if ( y_plus .le. LAST_Y ) then
                          u_plus( i_res ) = uplus_blm( y_plus )
                          if ( error_blm( err_msg ) ) then
                              write( STD_ERR, 3 ) err_msg
                              stop
                              end if
                          y_plus = y_plus * Y_STEP
                          i_res  = i_res + 1
                          goto 5
                          end if
                      if ( n_res .le. 0 ) then
                          n_res = i_res - 1
                          end if
6                     continue
c
c       print results
c
                  res_lu = STD_OUT
                  if ( iargc() .gt. 1 ) then
                      call getarg( 2, res_file )
                      res_lu = open_file( new_lun(), res_file, 'new', 0)
                      if ( res_lu .le. 0 ) then
                          res_lu = STD_OUT
                          end if
                      end if
                  y_plus = FIRST_Y
                  do 8 i = 1, n_res
                      write( res_lu, 7 ) y_plus, ( u_plus(j+i),
     1                    j = 0, ( n_profiles -1 ) * n_res, n_res )
7                     format( 1x, f8.3, 12f8.4 : / ( 9x, 12f8.4 ) )
                      y_plus = y_plus * Y_STEP
8                     continue
                  end if
              end if
          end if
      stop
      end










