c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM TESTUTAU
c
c Tests the code in QU_TAU.FOR
c
c
c Constants
c  MAX_DATA = 250   # maximum number of values which can be read from file
c Variables
c  integer
c    i
c    columns        # number of columns of data in the data file
c    dat_lu         # logical unit number of the data file
c    first          # index of the first velocity datum in profile I_PROFILE
c    item           # item selected from menu
c    i_profile      # selected profile number
c    n_points       # number of data points in each profile
c    n_profiles     # total number of profiles
c    Jresp          # function : integer response
c    new_lun        #          : get new logical unit number
c    open_file      #          : open a file
c  real
c    array(MAX_DATA)  # data array containing velocity profiles
c    delta_plus     # Coles' delat scaled in wall units
c    free_stream    # free stream velocity in m/s
c    frict_vel      # boundary layer friction velocity in m/s
c    kinem_visc     # kinematic viscosity of the fluid in sq.m/s
c    wake_strength  # Coles' PI, wake strength in multiples of friction velocity
c    Qresp          # function : real response
c    Qu_tau         #          : find friction velocity etc
c
      include 'stdio.def'
      integer MAX_DATA
      parameter( MAX_DATA=250 )
c
      integer i, columns, dat_lu, first, item, i_profile, n_points,
     1        n_profiles, Jresp, new_lun, open_file
      real array(MAX_DATA), delta_plus, free_stream, frict_vel,
     1     kinem_visc, wake_strength, Qresp, Qu_tau
      data free_stream, frict_vel, kinem_visc /
     1     1.6698,      0.081033,  15.0e-6 /
      data i_profile / 1 /

c
c       read data
c
      dat_lu = new_lun()
      dat_lu = open_file( dat_lu, 'testutau.dat', 'old', 0 )
      if ( dat_lu .eq. 0 )
     1    stop
      call read_data( dat_lu, array, MAX_DATA, n_points, columns )
      n_profiles = columns / 2
      close( UNIT=dat_lu )
c
c       dimensionalise and print the data
c
      do 5 item = 2, n_points * columns, 2
          array(item)   = array(item)   * frict_vel
          array(item-1) = array(item-1) * kinem_visc / frict_vel * 1000.
5         continue
      do 7 item = 0, ( n_points - 1 ) * columns, columns
          write( STD_OUT, 6 )   ( array( item + i ),
     1        array( item + i + 1 ), i = 1, columns - 1, 2 )
6         format( 1x, 2f8.3, 3x, 2f8.3, 3x, 2f8.3, 3x, 2f8.3 )
7         continue
c
c     start of loop
1         write( STD_OUT, 2 ) i_profile, free_stream, kinem_visc
2         format( ' Select 0) Exit' /
     1        8x, '1) profile number       ', i2 /
     2        8x, '2) free stream velocity ', g12.5, ' m/s' /
     3        8x, '3) kinematic viscosity  ', g12.5, ' sq.m/s' /
     4        8x, '4) find friction velocity  ', $ )
          item = Jresp( STD_IN, 0, 4, 4 )
          if ( item .eq. 0 ) then
              stop
          else if ( item .eq. 1 ) then
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              i_profile = Jresp( STD_IN, 1, n_profiles, 4 )
          else if ( item .eq. 2 ) then
              write( STD_OUT, 3 )
              free_stream = Qresp( STD_IN, 0.1, 100.0, 4 )
          else if ( item .eq. 3 ) then
              write( STD_OUT, 3 )
              kinem_visc = Qresp( STD_IN, 1.0e-6, 1.0e-3, 4 )
          else if ( item .eq. 4 ) then
              first = i_profile * 2
              frict_vel = Qu_tau( array(first), array(first-1),
     1            columns, n_points, free_stream, kinem_visc,
     2            wake_strength, delta_plus )
              write( STD_OUT, 4 ) frict_vel, wake_strength, delta_plus
4             format( ' friction velocity   ', g12.5 /
     1                ' wake strength (PI)  ', g12.5 /
     2                ' delta_plus', 10x, g12.5 )
              end if
          goto 1
c         end of loop
      end

