      REAL FUNCTION PROX_LAW( Y_PLUS, CORR_LAW )
c
c   Evaluation of the correction law at distance Y_PLUS from the wall
c ( in wall units ) .
c
c Arguments
c  real y_plus        # distance from wall -- in wall units
c       corr_law(*)   # array describing the correction law - described above
c Constant
c  Hvdulu=6           # vdu
c Local variables
c  real qpoly         # function : evaluate polynomial
c
      include 'stdio.def/list'
      include 'wallprox.def'
      real EXP_UND_FLOW
      parameter( EXP_UND_FLOW=-85.0 )
c
      integer type, order
      real y_plus, yy_plus, corr_law(*), qpoly
c
      type  = int( corr_law(Htype) + 0.5 )
      order = int( corr_law(Horder) + 0.5 )
c
c               ensure that don't get too close to the wall
c
      if ( y_plus .lt. corr_law(Hdom_min) ) then
          yy_plus = corr_law(Hdom_min)
          write( STD_OUT, 2 )
2         format( ' PROX_LAW : Too close to wall' )
      else
          yy_plus = y_plus
          end if
      yy_plus = yy_plus / corr_law(Hy_fact)
c
c               return zero if exceed the domain maximum
c               ****************************************
      if ( y_plus .gt. corr_law(Hdom_max) ) then
          prox_law = 0.0
CC**********************************************************************
CC**
CC** c Correction law types 1, 2, 3 & 6 are removed
CC** c
CC** c               correction law TYPE 1
CC** c               *********************
CC** c velocity_error_plus = factor * y_plus ** exponent *
CC** c                       Qpoly( y_plus, poly_coeffs, order )
CC** c
CC**       else if ( type .eq. 1 ) then
CC**           prox_law = corr_law(Hv_fact) * yy_plus ** corr_law(Hexpon) *
CC**      1               qpoly( yy_plus, corr_law(Hpoly), order )
CC** c
CC** c               correction law TYPE 2
CC** c               *********************
CC** c velocity_error_plus = factor * y_plus ** exponent *
CC** c                       Qpoly( 1.0 / y_plus, poly_coeffs, order )
CC** c
CC**       else if ( type .eq. 2 ) then
CC**           prox_law = corr_law(Hv_fact) * yy_plus ** corr_law(Hexpon) *
CC**      1               qpoly( 1.0 / yy_plus, corr_law(Hpoly), order )
CC** c
CC** c               correction law TYPE 3
CC** c               *********************
CC** c velocity_error_plus = factor *
CC** c                       Qpoly( y_plus ** exponent, poly_coeffs, order )
CC** c
CC**       else if ( type .eq. 3 ) then
CC**           prox_law = corr_law(Hv_fact) *
CC**      1      qpoly( yy_plus ** corr_law(Hexpon), corr_law(Hpoly), order )
CC** c
CC** c               correction law TYPE 6
CC** c               *********************
CC** c velocity_error_plus = factor * y_plus ** exponent *
CC** c                       exp( - Qpoly( y_plus, poly_coeffs, order ) )
CC** c
CC**       else if ( type .eq. 6 ) then
CC**           prox_law = corr_law(Hv_fact) * yy_plus ** corr_law(Hexpon) *
CC**      1        exp( - qpoly( yy_plus, corr_law(Hpoly), order ) )
CC**
CC**********************************************************************
c
c               correction law TYPE 4
c               *********************
c               VELOCITY ERROR CORRECTION METHOD
c
c velocity_error_plus = factor * exponent / y_plus *
c                       exp( - Qpoly( y_plus**2, poly_coeffs, order ) )
c
      else if ( type .eq. 4 ) then
          prox_law = corr_law(Hv_fact) * corr_law(Hexpon) / yy_plus *
     1        exp( max( EXP_UND_FLOW,
     2             - qpoly( yy_plus **2, corr_law(Hpoly), order ) ) )
c
c               correction law TYPE 5
c               *********************
c               HEAT FLOW METHODS
c
c velocity_error_plus = factor * exponent *
c                       exp( - Qpoly( y_plus, poly_coeffs, order ) )
c
      else if ( type .eq. 5 ) then
          prox_law = corr_law(Hv_fact) * corr_law(Hexpon) *
     1        exp( max( EXP_UND_FLOW,
     2                  - qpoly( yy_plus, corr_law(Hpoly), order ) ) )
c
c               correction law TYPE 7
c               *********************
c               MODIFIED HEAT FLOW METHODS
c
c velocity_error_plus = factor *
c                       exp( - Qpoly( y_plus, poly_coeffs, order ) )
c
      else if ( type .eq. 7 ) then
          prox_law = corr_law(Hv_fact) *
     1        exp( max( EXP_UND_FLOW,
     2                  - qpoly( yy_plus, corr_law(Hpoly), order ) ) )
c
c               unknown correction law
c               **********************
c
      else
          write( STD_ERR, 1 )
1         format( ' ERROR_LAW : unknown correction law' )
          prox_law = 0.0
          end if
c
c               reduce probability of future underflow errors --
c               which are fatal on this crummy system
      if ( prox_law .lt. 1.0e-35 ) prox_law = 0.0
      return
      end



      SUBROUTINE SBHATIA( SPEED, INCREM, NPTS, Y, KINEM_VISC, CFFILE )
c
c     Correction of the wall proximity effect by the method velocity correction
c using Acharya's transformation and Bhatia's iteration procedure .
c
c Bhatia J.C., Durst F., Jovanovic J. (1982) :
c   Corrections of Hot-Wire Anemometer Measurements Near Walls,
c   {\em Journal Of Fluid Mechanics,} Vol.\ {\em 12}, pp 411--431 .
c
c Description of procedure .
c    When this routine is called for the first time, the correction law is
c read from file and a table of the form suggested by Acharya is created .
c This table is used to calculate the velocity error ( using linear
c interpolation ) . The interpolated value is then refined using Bhatia's
c iteration .
c    The Acharya's parametric equation for generating the table is
c \left( \frac{y U_m}{\nu_{\infty}}, \frac{\Delta U}{U_m} \right) =
c \left( y^+ \left[ f(y^+) +m y^+ \right], \frac{f(y^+)}{f(y^+) + m y^+} \right).
c where f(y^+) is the velocity error function, and m is the inferred slope
c of the sublayer mean velocity profile .
c
c    Bhatia's iteration may be described as
c 1) i = 1                        -- initialization
c 2) u_i = e_i = u_{\rm meas}     -- initialization
c 3) u_{i+1} = u_{\rm meas} - \sqrt{\nu u_i / y m} f( \sqrt{u_i y / \nu m} )
c 4) e_{i+1} = u_{i+1} - u_{i}
c 5) if ( e_{i+1} > {\rm Accuracy} u_{i+1} ) goto 2)
c
c Arguments
c  real    speed(*),   # instantaneous speed values to be corrected (metres/sec)
c  integer increm,     # sampling increment
c          npts        # number of data points
c  real    y,          # distance from the wall (metres)
c          kinem_visc  # kinematic viscosity    (sq.metres/sec)
c  character cffile*(*)# name of file specifying the wall proximity correction
c                      # law
c N.B. npts=0 is a signal to clear the remembered file name an correction law
c
c Constants
c  Hmax_terms=30          # maximum number of terms in a correction law
c  Hmax_loops=100         # maximum number of iterations
c  Hclose_enough=0.0001   # convergence limit
c  Htable_size=100        # size of table for Acharya's method
c Local variables
c  integer i              # array index
c          iter_count     # iteration counter
c          law_type       #
c          nlast          # last array index
c  logical prox_read      # read wall proximity correction law from file
c  real new_error, old_error      # old & new changes in y_plus
c       new_speed, old_speed      # current & previous iterates for speed
c       abs_speed         # abs( measured speed ) - idiot proofing
c       prev_y            # previous value of y_plus - must be smaller
c       Re_no             # Reynolds number abscissa for Acharya's table
c       vel_err           # velocity error calculated with prox_law
c       y_plus,           # used for preparing the Acharya table
c       y_step            # increment in y_plus for preparing the Acharya table
c       prox_law          # function : wall proximity error at y_plus
c       qlrpl1            # function : linear interpolation
c    temporary variables -- for optimizing calculation speed
c       coeff_y_plus      # factor for converting sqrt(velocity) to y_plus
c       coeff_u_tau       # factor for converting sqrt(velocity) to friction vel
c       clipped_min       # minimum permitted value of velocity iterate
c       sqrt_old_speed    # = sqrt(old_speed)
c Static variables
c  logical law_defined    # correction law is defined and available
c  real corr_law(Hmax_terms)      # the correction law
c       table_y(Htable_size)      # $y U_m / \nu$ data
c       table_u(Htable_size)      # $\Delta U / U_m$ data
c  character Cifile*64    # remembered file name
c
      include 'stdio.def'
      integer Hmax_terms, Hmax_loops, Htable_size
      real Hclose_enough
      parameter( Hmax_terms=30, Hmax_loops=100,
     1    Htable_size=100, Hclose_enough=0.00005 )
c
      logical law_defined, prox_read
      integer increm, law_type, npts, i, iter_count, nlast
      real speed(*), y, kinem_visc, corr_law(Hmax_terms),
     1    new_error, old_error, new_speed, old_speed, abs_speed,
     2    prev_y, Re_no, vel_err, y_plus, y_step,
     3    table_y(Htable_size), table_u(Htable_size),
     4    coeff_y_plus, coeff_u_tau, clipped_min, sqrt_old_speed,
     5    prox_law, qlrpl1
      character Cifile*64, cffile*(*)
c
      include 'wallprox.def'
      save law_defined, law_type, corr_law, Cifile, table_y, table_u
      data law_defined / .false. /
      data law_type / 0 /
      data Cifile / ' ' /
c
c               SPECIAL CALL SIGNALLED WITH NPTS=0 TO CLEAR
c                 REMEMBERED FILE NAME AND CORRECTION LAW
c               *******************************************
c
      if ( npts .eq. 0 ) then
          law_defined = .FALSE.
          Cifile = ' '
          write( STD_OUT, 1 )
1         format( ' SBHATIA : NOTE : Clearing correction law' )
          return
          end if
c
c               CHECK ARGUMENTS
c               ***************
c
      if ( increm .le. 0 .or. npts .lt. 0 ) then
          write( STD_ERR, 2 ) increm, npts
2         format( ' SBHATIA : ABORT : Illegal increment ', i12,
     1            ' or no. of points ', i12 )
          return
          end if
      if ( y .le. 0.0 .or. kinem_visc .le. 0.0 ) then
          write( STD_ERR, 3 ) y, kinem_visc
3         format( ' SBHATIA : ABORT : Non-positive y ', g12.5,
     1            ' or viscosity ', g12.5 )
          return
          end if
      nlast = ( npts - 1 ) * increm + 1
c
c               IF CALLED FOR THE FIRST TIME OR THE FILE IS CHANGED
c               ***************************************************
c
c               open correction law specification file
c
      if ( .not. law_defined .or. Cffile .ne. ' ' .and.
     1        Cffile .ne. Cifile ) then
          if ( .not. prox_read( corr_law, Hmax_terms, Cffile ) ) then
              write( STD_ERR, 4 )
4             format( ' SBHATIA : ABORT' )
              return
              end if
          law_type = corr_law(Htype) + 0.5
          if ( law_type .ne. 4 ) then
              write( STD_ERR, 5 ) law_type
5             format( ' SBHATIA : ABORT : Illegal correction law type ',
     1                i12, ' Need type 4' )
              end if
          Cifile = Cffile
          law_defined = .TRUE.
c
c               generate table for Acharya's transformation
c
          prev_y = 0.0
          y_plus = corr_law(Hdom_min)
          y_step = ( corr_law(Hdom_max) - corr_law(Hdom_min) ) /
     1        ( Htable_size - 1 )
          do 7 i = 1, Htable_size
              vel_err = prox_law( y_plus, corr_law )
              table_y(i) = y_plus * ( vel_err + corr_law(Hp_slope) *
     1            y_plus )
              table_u(i) = vel_err /
     1            ( vel_err + corr_law(Hp_slope) * y_plus )
              if ( table_y(i) .lt. prev_y ) then
                  write( STD_ERR, 6 )
6                 format( ' SBHATIA : ABORT : abscissa inversion ',
     1               'while generating Acharya''s table' )
                  return
                  end if
              prev_y = table_y(i)
              y_plus = y_plus + y_step
7             continue
          end if
c
c               PROCESS THE DATA ARRAY ELEMENT BY ELEMENT
c               *****************************************
c
      do  15 i = 1, nlast, increm
          abs_speed = abs( speed(i) )
c
c               IF THE DATA IS INFEASIBLE I.E. TOO SMALL, USE PREVIOUS VALUE
c
          Re_no = y * abs_speed / kinem_visc
          if ( Re_no .le. table_y(1) ) then
              write( STD_OUT, 8 ) i, speed(i), y
8             format( ' SBHATIA: Interpol''n, infeasible @',
     1                i7, 'th datum (u,y)=', 2g12.5 )
              if ( i .gt. 1 ) then
                  speed(i) = speed( i - increm )
                  write( STD_OUT, 9 )
9                 format( ' Using previous value' )
              else
                  speed(i) = abs_speed
                  write( STD_OUT, 10 )
10                format( ' Abort correction' )
                  end if
c
c               IF THE DATUM IS EFFECTED BY WALL PROXIMITY, CORRECT IT
c
          else if ( Re_no .le. table_y(Htable_size) ) then
c
c               get and initial correction from Acharya's table
c
              old_speed = abs_speed * ( 1.0 -  qlrpl1( table_y, table_u,
     1            1,  Htable_size, Re_no ) )
              old_error = old_speed
              clipped_min = abs_speed / 10000.0
              coeff_y_plus = sqrt( y / kinem_visc / corr_law(Hp_slope) )
              coeff_u_tau = sqrt( kinem_visc / y / corr_law(Hp_slope) )
c
c               iterative solution at each point
c
              do 12 iter_count = 1, Hmax_loops
                  sqrt_old_speed = sqrt( old_speed )
                  new_speed = max( clipped_min,
     1                abs_speed - sqrt_old_speed * coeff_u_tau *
     2                prox_law( sqrt_old_speed * coeff_y_plus,
     3                          corr_law ) )
                  new_error = abs( new_speed  - old_speed )
                  if ( new_error .gt. old_error
     1                    .and. iter_count .gt. 2 ) then
                      write( STD_OUT, 11 ) i, speed(i), y
11                    format( ' SBHATIA : Iteration diverging at ',
     1                  i7, 'th datum (u,y) = ', 2g12.5 )
                      if ( i .gt. 1 ) then
                          new_speed = speed( i - increm )
                          write( STD_OUT, 9 )
                      else
                          new_speed = abs_speed
                          write( STD_OUT, 10 )
                          end if
                      goto 14
                      end if
                  if ( new_error .le. Hclose_enough * new_speed )
     1                goto 14
                  old_speed = new_speed
                  old_error = new_error
12                continue
c
              write( STD_OUT, 13 ) i, speed(i), y
13            format( ' SBHATIA : Converged too slowly ', i6,
     1            'th  datum (u,y) = ', 2g13.5 )
14            continue
              speed(i) = new_speed
c
c               if the datum is outside proximity effect, do nothing
c
          else
              end if
15        continue
c
c               successful return
c
      return
      end



      SUBROUTINE PROX_NUSSELT( MADSIG, SPEED, INCREM, NPTS,
     1                         Y, KINEM_VISC,
     2                         QFUNC_E2toU, QFUNC_UtoE2, CFFILE )
c
c     Correction of the wall proximity effect by the method of Nusselt number
c correction using an iteration procedure similar to that of Bhatia et. al. .
c It is assumed that the hot-wire calibration has been selected using the
c routine CALHW1OPN .
c
c Bhatia J.C., Durst F., Jovanovic J. (1982) :
c   Corrections of Hot-Wire Anemometer Measurements Near Walls,
c   {\em Journal Of Fluid Mechanics,} Vol.\ {\em 12}, pp 411--431 .
c
c Description of procedure .
c    When this routine is called for the first time, the correction law is
c read from file . The correction procedure is an iteration which includes
c use of the hot-wire calibration .
c
c    The iteration may be described as
c 1) i \rightarrow 0                                      -- initialization
c 2) E^2_{\rm meas} \rightarrow u_{\rm 0,vel}  -- initialization
c 3) i+1 \rightarrow i
c 4) \sqrt{ u_{\rm i-1,vel} y / \nu} \rightarrow y^+
c 5) y^+ \rightarrow Nu_y
c 6) E^2_{\rm meas} - Nu_y / y \rightarrow E^2_{\rm i,vel}
c 7) E^2_{\rm i,vel} \rightarrow u_{\rm i,vel}
c 8) u_{\rm i,vel} - u_{\rm i-1,vel} \rightarrow e_i
c 9) if ( e_i > {\rm Accuracy} u_{i,vel} ) goto 2)
c
c Arguments
c  integer *2 Madsig(*)# A/D converter readings
c  real    speed(*),   # instantaneous speed values to be corrected (metres/sec)
c  integer increm,     # sampling increment for Madsig and speed
c          npts        # number of data points
c  real    y,          # distance from the wall (metres)
c          kinem_visc  # kinematic viscosity    (sq.metres/sec)
c          Qfunc_E2toU # function : convert voltage squared to air speed
c          Qfunc_UtoE2 # function : convert air speed to voltage squared
c  character cffile*(*)# name of file specifying the wall proximity correction
c                      # law
c N.B. npts=0  clears the remembered file name of the correction law and
c                  switches debuggin off
c      npts=-1 switches debugging on or off
c
c Constants
c  BACKWARD_ITERATION= 'back'   # backward iteration method
c  FORWARD_ITERATION = 'forw'   # forward iteration method
c  CLOSE_ENOUGH      = 0.00005  # max change in result for convergence
c  DIVERGE_LIMIT     = 3        # no of diverging iterat'ns to detect divergence
c  MAX_LOOPS         = 300      # maximum number of iterations
c  MAX_TERMS         = 30       # maximum number of terms in a correction law
c  Hclose_enough=0.00005  # convergence limit
c Local variables
c  integer direction      # direction of iteration
c          i              # array index
c          i_diverge      # number of successively divergent iterations
c          iter_count     # iteration counter
c          i_trace
c          law_type
c          n_diverge      # number of divergent iterations
c          nlast          # last array index
c          Nu_fl_order    # order of convection derating polynomial
c          Nu_fl_poly     # correction law index for convection derating polynom
c          Jresp          # function : read integer
c  logical prox_read      # read wall proximity correction law from file
c  real new_error, old_error      # old & new changes in y_plus
c       new_speed, old_speed      # current & previous iterates for speed
c       convergence       # convergence factor = 1.0 for perfect solution
c       E2_meas           # square of measured voltage
c       E2_vel            # approximation to squared voltage due to air speed
c       friction_vel      # friction velocity (obtained as part of iteration)
c       U_meas            # measured air speed
c       QHW1_ADtoE2       # function : convert A/D reading to voltage squared
c       prox_law          # function : wall proximity error at y_plus
c Static variables
c  logical debugging      # extra output required for debugging
c          law_defined    # correction law is defined and available
c  real corr_law(Hmax_terms)      # the correction law
c  character Cifile*64    # remembered file name
c
      include 'stdio.def'
      integer    BACKWARD_ITERATION, MAX_LOOPS,
     1           FORWARD_ITERATION,  MAX_TERMS,
     2           DIVERGE_LIMIT
      real       CLOSE_ENOUGH
      parameter( BACKWARD_ITERATION='back', MAX_LOOPS=300,
     1           FORWARD_ITERATION='forw',  MAX_TERMS=30,
     2           DIVERGE_LIMIT=3,
     3           CLOSE_ENOUGH=0.00005 )
      include 'wallprox.def'
c
      integer *2 Madsig(*)
      integer direction, i, i_diverge, iter_count, i_trace, law_type,
     1        n_diverge, nlast, increm, npts, Nu_fl_order, Nu_fl_poly,
     2        Jresp
      logical debugging, law_defined, prox_read
      real convergence, corr_func, corr_law(MAX_TERMS),
     1     E2_meas, E2_vel, friction_vel, kinem_visc, new_error,
     2     new_speed, old_error, old_speed, speed(*), U_meas, y, y_plus,
     3     prox_law, QHW1_ADtoE2, Qfunc_E2toU, Qfunc_UtoE2, Qpoly
      character Cifile*64, cffile*(*)
c
      save debugging, law_defined, law_type, Nu_fl_order, Nu_fl_poly,
     1     corr_law, Cifile
      data debugging, law_defined / .FALSE., .FALSE. /
      data law_type / 0 /
      data Cifile / ' ' /
c
c               SPECIAL CALL SIGNALLED WITH NPTS=0 TO CLEAR
c                 REMEMBERED FILE NAME AND CORRECTION LAW
c               *******************************************
c
      if ( npts .eq. 0 ) then                                             1b
          law_defined = .FALSE.
          debugging   = .FALSE.
          Cifile = ' '
          write( STD_ERR, 1 )
1         format( ' PROX_NUSSELT : NOTE : Clearing correction law. ',
     1        'Debugging OFF.' )
c
c               SPECIAL CALL SIGNALLED WITH NPTS=-1 TO CHANGE DEBUGGING MODE
c               ************************************************************
c
      else if ( npts .eq. -1 ) then                                       1m
          debugging = .not. debugging
          if ( debugging ) then                                           2b
              write( STD_ERR, 2 ) 'ON'
2             format( ' PROX_NUSSELT : NOTE : Debugging ', a )
          else                                                            2m
              write( STD_ERR, 2 ) 'OFF'
              end if                                                      2e
c
c               CHECK ARGUMENTS
c               ***************
c
      else if ( increm .le. 0 ) then                                      1m
          write( STD_ERR, 3 ) 'INCREM', real( increm )
3         format( ' PROX_NUSSELT : ABORT : Illegal argument', a, ' = ',
     1        g12.5 )
      else if ( npts .le. 0 ) then                                        1m
          write( STD_ERR, 3 ) 'NPTS', real( npts )
      else if ( y .le. 0.0 ) then                                         1m
          write( STD_ERR, 3 ) 'Y', y
      else if ( kinem_visc .le. 0.0 ) then                                1m
          write( STD_ERR, 3 ) 'KINEM_VISC', kinem_visc
      else                                                                1m
          n_diverge = 1
          nlast = ( npts - 1 ) * increm + 1
c
c               IF CALLED FOR THE FIRST TIME OR THE FILE IS CHANGED
c               ***************************************************
c
c               open correction law specification file
c
          if ( .not. law_defined .or. Cffile .ne. ' ' .and.
     1         Cffile .ne. Cifile ) then                                  2b
              if ( .not. prox_read( corr_law, MAX_TERMS, Cffile ) )
     1            return
              Cifile = Cffile
              law_defined = .TRUE.
              law_type = corr_law(Htype) + 0.5
              if ( law_type .ne. 5 .and. law_type .ne. 7 ) then           3b
                  write( STD_ERR, 4 ) law_type
4                 format( ' PROX_NUSSELT : ABORT : Illegal correction ',
     1                'law type ', i12, ' Need type 5 or 7' )
                  end if                                                  3e
              Nu_fl_order = corr_law(Hexpon) + 0.5
              Nu_fl_poly  = Hpoly + corr_law(Horder) + 1.5
              end if                                                      2e
          if ( debugging ) then                                           2b
              write( STD_ERR, 13 )
13            format( ' Next sample to trace ? ', $ )
              i_trace = Jresp( STD_IN, 1, nlast + 1, 4 )
              end if                                                      2e
c
c               PROCESS THE DATA ARRAY ELEMENT BY ELEMENT
c               *****************************************
c
          do  12 i = 1, nlast, increm                                     2b
c
c               initial guess;
c               use the final iterate for the previous sample * 1.05 as the
c               initial iterate for the current sample
c
              E2_meas = QHW1_ADtoE2( Madsig(i) )
              U_meas  = Qfunc_E2toU( E2_meas )
              if ( i .eq. 1 ) then                                        3b
                  old_speed = U_meas
              else                                                        3m
                  old_speed = old_speed * 1.05
                  end if                                                  3e
              old_error = old_speed
c
c               start with forward iteration method
c
              direction = FORWARD_ITERATION
c
c               table header for debugging & tracking the iteration
c
              if ( debugging .and. i .eq. i_trace ) then                  3b
                  call prox_plot( corr_law )
                  write( STD_ERR, 5 ) i, E2_meas, U_meas, old_speed
5                 format( ' Sample ', i7, ' E2_meas,U_meas,U_0= ',
     1                3f10.6 / '   i    y^+    corr''n func   e^2_i',
     2                '     u_tau      u_i      error_i     test' )
                  end if                                                  3e
c
c               iteration loop for each data sample
c
              do 9 iter_count = 1, MAX_LOOPS                             3b
c
c               new MODIFIED HEAT FLOW correction method
c               forward iteration only because the backward iteration
c               requires inversion of the correction function f(y^+)
c               --- only do backward iteration if REALLY required
c
                  if ( law_type .eq. 7 ) then                             4b
                      friction_vel = sqrt( kinem_visc * old_speed /
     1                                     y / corr_law(Hp_slope) )
                      y_plus = y * friction_vel / kinem_visc
                      corr_func = prox_law( y_plus, corr_law )
                      E2_vel = max( E2_meas / 10000.0, E2_meas -
     1                  corr_func * y ** ( HW_CALIB_EXPON_N - 1.0 ) *
     2                  friction_vel ** ( 2.0*HW_CALIB_EXPON_N - 1.0 ) )
                      new_speed = min( U_meas, Qfunc_E2toU( E2_vel ) )
CC**c
CC**c               code for old implementation of
CC**                  if ( law_type .eq. 7 ) then                             4b
CC**c
CC**c               MODIFIED HEAT FLOW correction method
CC**c               forward iteration method
CC**c               ************************
CC**                      if ( direction .eq. FORWARD_ITERATION ) then        5b
CC**                          friction_vel = sqrt( kinem_visc * old_speed /
CC**     1                                         y / corr_law(Hp_slope) )
CC**                          y_plus = y * friction_vel / kinem_visc
CC**                          corr_func = prox_law( y_plus, corr_law )
CC**                          E2_vel = max( E2_meas / 10000.0,
CC**     1                        E2_meas - corr_func / y *
CC**     2                        friction_vel ** ( HW_CALIB_EXPON_N-1.0 ) )
CC**                          new_speed = min( U_meas, Qfunc_E2toU(E2_vel) )
CC**c
CC**c               MODIFIED HEAT FLOW correction method
CC**c               backward iteration method
CC**c               *************************
CC**                      else                                                5m
CC**                          E2_vel = min( E2_meas,
CC**     1                                  Qfunc_UtoE2( old_speed ) )
CC**                          corr_func = prox_law( y_plus, corr_law )
CC**                          friction_vel = ( y * ( E2_meas - E2_vel ) /
CC**     1                        corr_func ) **
CC**     2                        ( 1.0 / ( HW_CALIB_EXPON_N - 1.0 ) )
CC**                          y_plus = y * friction_vel / kinem_visc
CC**                          new_speed = min( U_meas,
CC**     1                        friction_vel **2 * y * corr_law(Hp_slope)
CC**     2                        / kinem_visc )
CC**                          end if                                          5e
c
c               ORIGINAL HEAT FLOW METHOD
c               forward iteration method only
c               *****************************
c
                  else                                                    4m
                      friction_vel = sqrt( kinem_visc * old_speed /
     1                                     y / corr_law(Hp_slope) )
                      y_plus = y * friction_vel / kinem_visc
                      corr_func = prox_law( y_plus, corr_law )
                      E2_vel = max( E2_meas / 10000.0,
     1                    E2_meas - corr_func / y )
                      new_speed = min( U_meas, Qfunc_E2toU(E2_vel) )
                      end if                                              4e
                  new_error = abs( new_speed  - old_speed )
                  old_speed = new_speed
                  convergence = max( -9.9999, min( 99.9999, new_speed
     1                / y_plus / corr_law(Hp_slope) / friction_vel ) )
c
c               plot & table for debugging & tracking the iteration
c
                  if ( debugging .and. i .eq. i_trace ) then              4b
                      call ilgline( 14, 0, 1 )
                      call ilgdata( y_plus, corr_func, 1, 1, 0 )
                      write( STD_OUT, 6 ) direction, iter_count, y_plus,
     1                    corr_func, E2_vel, friction_vel, new_speed,
     2                    new_error, convergence
6                     format( 1x, a4, i4, f8.4, 1x, g11.4, f8.4,
     1                    2f11.6, f12.7, f9.6 )
                      end if
c
c               handle diverging iteration
c               if using FORWARD_ITERATION direction with MODIFIED HEAT FLOW
c               method, reverse the direction and keep trying, otherwise
c               bail out of the iteration
c
                  if ( new_error .gt. old_error ) then                    4b
                      if ( i_diverge .ge. DIVERGE_LIMIT ) then            5b
                          if ( 1 .eq. mod( n_diverge, 20 ) )
     1                        write( STD_ERR, 7 )
7                         format( ' PROX_NUSSELT: Diverging iteration' /
     1                        ' dir- sample  iterate  y     uncorr'''
     2                            '  corrected   normal' /
     3                        '  ''n', 17x, '(mm)   --- speed (m/s)--',
     4                            '-   error' )
                          write( STD_OUT, 8 ) direction, i, iter_count,
     1                        y * 1000.0, U_meas,
     2                        min( 999.9999, new_speed ), convergence
8                         format( 1x, a4, i7, 2x, i4, f8.4, 3f10.6 )
                          i_diverge = 0
                          n_diverge = n_diverge + 1
CC**                          if ( direction .eq. FORWARD_ITERATION .and.
CC**     1                         law_type .eq. 7 ) then                     6b
CC**                              direction = BACKWARD_ITERATION
CC**                          else                                            6m
CC**                              old_error = new_error
                              goto 11
CC**                             end if                                      6e
                      else                                                5m
                          i_diverge = i_diverge + 1
                          end if                                          5e
                  else                                                    4m
                      i_diverge = 0
                      end if                                              4e
c
c               iteration is complete if sufficiently converged
                  old_error = new_error
                  if ( new_error .le. CLOSE_ENOUGH * new_speed )
     1                goto 11
9                 continue                                                3e
c               end of iteration loop
c
              write( STD_OUT, 10 )
10            format( ' PROX_NUSSELT : Convergence too slow' )
              write( STD_OUT, 8 ) direction, i, iter_count, y * 1000.0,
     2            U_meas, min( 999.9999, new_speed ), convergence
c
c           Emerge from iteration to here on sufficiently small convergence
c           factor
c
11            continue
              speed(i) = new_speed
              if ( debugging .and. i .eq. i_trace ) then                  3b
                  write( STD_ERR, 13 )
                  i_trace = Jresp( STD_IN, i + increm, nlast + 1, 4 )
                  call ilgclos()
                  end if                                                  3e
12            continue                                                    2e
          end if                                                          1e
      return
      end



      SUBROUTINE PROX_PLOT( CORR_LAW )
c
c     Plot the correction function CORR_LAW as a line over the defined domain.
c Does not terminate graphics before returning.
c
c Arguments
c  real    corr_law(*)                # correction law
c Local variables
c  integer i                          # array index
c  real    step                       # abscissa increment
c          y_data(PLOTTED_POINTS)     # abscissa values
c          corr_data(PLOTTED_POINTS)  # values of correction function
c          prox_law                   # function : evaluates correction function
c
      include 'stdio.def'
      include 'wallprox.def'
      integer    PLOTTED_POINTS
      parameter( PLOTTED_POINTS=64 )
c
      integer i
      real    step, y_data(PLOTTED_POINTS), corr_data(PLOTTED_POINTS),
     1        corr_law(*), prox_law
c
c               calculate the data points
c
      step = ( corr_law(Hdom_max) - corr_law(Hdom_min) ) /
     1       real( PLOTTED_POINTS - 1 )
      do 1 i = 1, PLOTTED_POINTS                                          1b
          y_data(i)    = ( i - 1 ) * step + corr_law(Hdom_min)
          corr_data(i) = prox_law( y_data(i), corr_law )
1         continue                                                        1e
c
c               plot the data
c
      call ilgopen( ' ' )
      call ilgline( 0, 1, 1 )
      call ilgdata( y_data, corr_data, 1, PLOTTED_POINTS, 1 )
      return
      end



      LOGICAL FUNCTION PROX_READ( CORR_LAW, MAX_TERMS, FILE )
c
c    Reads the Wall Proximity Effect Correction Law from the file FILE
c into the array CORR_LAW . MAX_TERMS is the maximum number of values
c which can be read into CORR_LAW . If MAX_TERMS is not large enough,
c an error is reported and the returned value is .FALSE. . If the
c correction law is read properly, the returned value is .TRUE. . If the
c file FILE cannot be opened, or an error occurs on reading the file, or
c an EOF is found prematurely, an error is reported and the returned
c value is .FALSE. .
c
c Arguments
c  real corr_law(*)   # array describing the correction law - described above
c  integer max_terms  # maximum space available for storing the law
c  character file*(*) # correction law file
c Local variables
c  integer i          # array indexing
c          ispeclu    # logical unit number of input file
c          reqd_terms # required size of CORR_LAW array
c          new_lun    # function : get new logical unit number
c          open_file  #          : open file
c
      include 'stdio.def'
      include 'wallprox.def'
c
      real corr_law(*)
      integer i, ispeclu, max_terms, reqd_terms, new_lun, open_file
      character file*(*)
      save ispeclu
c
c             check for sufficient space to store the correction law
c
      prox_read = .FALSE.
      if ( max_terms .lt. Hpoly ) then
          write( STD_ERR, 1 ) max_terms, Hpoly
1         format( ' PROX_READ : ABORT : ', i12, ' terms is not enough ',
     1        'for correction law .' / '   Need at least ', i3 )
          return
          end if
c
c              open the file
c
      if ( ispeclu .eq. 0 ) then
          ispeclu = new_lun()
          end if
      if ( 0 .eq. open_file( ispeclu, file, 'old', 0 ) ) then
          write( STD_ERR, 2 ) file
2         format( ' PROX_READ : ABORT : Cannot open file ', a )
          return
          end if
c
c              read the correction law type and the order of the polynomial
c              (skip the first two lines )
c
      read( ispeclu, 3, ERR=5, END=7 )
3     format( / )
      read( ispeclu, *, ERR=5, END=7 )
     1    corr_law(Htype),   corr_law(Horder), corr_law(Hv_fact),
     2    corr_law(Hy_fact), corr_law(Hexpon)
c
c             check for sufficient space to store the correction law
c
      reqd_terms = Hpoly + int( corr_law(Horder) + 0.5 )
      if ( int( corr_law(Htype) + 0.5 ) .eq. 7 )
     1    reqd_terms =  reqd_terms + int( 1.5 + corr_law(Hexpon) )
      if ( max_terms .ge. reqd_terms ) then
c
c              read the remaining terms from the file
c
          read( ispeclu, *, ERR=5, END=7 )
     1       corr_law(Hp_slope), corr_law(Hdom_min), corr_law(Hdom_max),
     2       corr_law(Hpol_err), ( corr_law(i), i = Hpoly, reqd_terms )
          prox_read = .TRUE.
      else
          write( STD_ERR, 1 ) max_terms, reqd_terms
          end if
      close( UNIT=ispeclu )
      return
c
c ******************************************************************
c
c     Aborted returns
c
5     write( STD_ERR, 6 ) file
6     format( ' PROX_READ : ABORT : Cannot read from file ', a )
      goto 9
c
7     write( STD_ERR, 8 ) file
8     format( ' PROX_READ : ABORT : Premature EOF ', a )
9     close( UNIT=ispeclu )
      return
      end


      SUBROUTINE PROX_WRITE( CORR_LAW, FILE )
c
c    Writes the Wall Proximity Effect Correction Law to the file FILE into
c the array CORR_LAW . If the file FILE cannot be opened, or an error occurs
c on writing to the file, an error is reported .
c
c Arguments
c  real corr_law(*)   # array describing the correction law - described above
c  character file*(*) # correction law file
c
c Local variables
c  integer corr_lu    # logical unit no. of output file
c          i          # array index
c          ispeclu    # correction law specification file
c          last_coeff # last index of correction polynomials
c          order      # order of correction law polynomial
c          type       # correction law type
c          str_length # function : location of last non-blank in a string
c          new_lun    # function : get new logical unit number
c          open_file  #          : open file
c  character line*80  # comment line
c
      include 'stdio.def'
      include 'wallprox.def'
c
      integer corr_lu, i, ispeclu, last_coeff, order, type,
     1        str_length, new_lun, open_file
      real corr_law(*)
      character file*(*), line*80
      save ispeclu
      data ispeclu / 0 /
c
      if ( str_length( file ) .gt. 0 ) then
          if ( ispeclu .eq. 0 ) then
              ispeclu = new_lun()
              end if
          if ( 0 .eq. open_file( ispeclu, file, 'new', 0 ) ) then
              write( STD_ERR, 7 ) file
7             format( ' PROX_WRITE : ABORT : Cannot open file ', a )
              return
              end if
          corr_lu = ispeclu
      else
          corr_lu = STD_OUT
          end if
      type  = int( corr_law(Htype) + 0.5 )
      order = int( corr_law(Horder) + 0.5 )
      last_coeff = Hpoly + order
      if ( type .eq. 7 ) then
          last_coeff = last_coeff + int( corr_law(Hexpon) + 1.5 )
          end if
c
c       write label and comments
c
      if ( corr_lu .eq. ispeclu ) then
          write( corr_lu, 1, ERR=8 ) type
1         format( ' ; Wall proximity correction law : type ', i4 )
          if ( index( file, '.scr' ) .ne. 0 ) then
              write( corr_lu, 4 ) ' '
          else
              write( STD_OUT, 2 )
2             format( ' Enter one comment line please' )
              read( STD_IN, 3, ERR=10, END=12 ) line
3             format( a )
              write( corr_lu, 4, ERR=8 ) line
4             format( ' ; ', a )
              end if
c
c       write the correction law
c           type of law
c           order of polynomial
c           vertical scaling factor
c           horizontal scaling factor
c           y exponent or pre-multiply factor
c           inferred profile slope
c           domain_minimum  domain_maximum
c           mean square polynomial fitting error
c           polynomial_coefficients
c           convection derating polynomial coefficients (if any)
c
          write( corr_lu, 5, ERR=8 ) type, order,
     1        corr_law(Hv_fact),
     2        corr_law(Hy_fact),
     3        corr_law(Hexpon),
     4        corr_law(Hp_slope),
     5        corr_law(Hdom_min), corr_law(Hdom_max),
     6        corr_law(Hpol_err),
     7        ( corr_law(i), i = Hpoly, last_coeff )
5         format( 1x, i4 / 1x, i4 / 1x, g15.7 / 1x, g15.7 / 1x, g15.7 /
     1            1x, g15.7 / 1x, 2g15.7 / 1x, g15.7 / ( 1x, 5g15.7 ) )
          close( UNIT=ispeclu )
      else
          write( corr_lu, 6, ERR=8 ) type, order,
     1        corr_law(Hv_fact),
     2        corr_law(Hy_fact),
     3        corr_law(Hexpon),
     4        corr_law(Hp_slope),
     5        corr_law(Hdom_min), corr_law(Hdom_max),
     6        corr_law(Hpol_err),
     7        ( corr_law(i), i = Hpoly, last_coeff )
6        format( ' Wall proximity correction law' /
     1        '   Correction law type   ', i4 /
     2        '   Order of Polynomial   ', i4 /
     3        '   Velocity error factor ', g15.7 /
     4        '   Y (horizontal) factor ', g15.7 /
     5        '   Exponent              ', g15.7 /
     6        '   Profile slope         ', g15.7 /
     7        '   Domain max and min    ', 2g15.7 /
     8        '   Mean sq. polynomial fit error ', g15.7 /
     9        '   Polynomial coefficients ' / ( 1x, 5g15.7 ) )
          end if
      return
c
c               traps
c
8     if ( corr_lu .eq. ispeclu ) then
          close( UNIT=ispeclu )
          end if
      write( STD_ERR, 9 ) file
9     format( ' PROX_WRITE : ABORT : Cannot write ', a )
      return
c
10    if ( corr_lu .eq. ispeclu ) then
          close( UNIT=ispeclu )
          end if
      write( STD_ERR, 11 )
11    format( ' PROX_WRITE : ABORT : Cannot read' )
      return
c
12    stop ' Abort'
      end



