c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


c           *****************************************************
c           ***                                               ***
c           ***    P.V.Lanspeary private FORTRAN 77 library   ***
c           ***                                               ***
c           ***       f77_math : mathematical functions       ***
c           ***                                               ***
c           *****************************************************
c


      INTEGER *4 FUNCTION JCOMBN( N, M )
c                                                      N        N!
c    Function to find the combination of N & M , ie.  ( ) = -----------
c                                                      M     M!.(N-M)!
c
c integer *4 i,            # looping index
c            m,m1,         # sub argument :- number of items selected
c            n             # super argument :- set size
c
      implicit none
      integer *4 i, m, m1, n
c
      jcombn = 1
      if ( n .lt. 2 .or. n .le. m .or. m .le. 0 ) return
      m1 = m
      if ( m .gt. n-m ) m1 = n - m
      jcombn = n
      if ( m1 .gt. 1 ) then
        do i = 2, m1
          jcombn = jcombn * ( n-i+1 ) / i
          end do
        end if
      return
      end


      REAL FUNCTION ASINH( X )
c
c       Real inverse hyperbolic sine
c
      implicit none
      real x, alog, sqrt
c
      asinh = alog( x + sqrt( x **2 + 1.0 ) )
      return
      end



      REAL FUNCTION ACOSH( X )
c
c       Real inverse hyperbolic cosine
c
      implicit none
      real x, abs,alog,sqrt
c
      acosh = 1.0
      if ( x .lt. 1.0 ) then
        type 1, x
      else
        acosh = alog( abs( x ) + sqrt( x **2 - 1.0 ) )
        end if
      return
1     format( ' ACOSH : FATAL ; Invalid Argument . ', e14.6 )
      end



      REAL FUNCTION ATANH( X )
c
c       Real inverse hyperbolic tangent
c
      implicit none
      real x, abs, alog, sqrt
c
      atanh = 0.0
      if ( abs( x ) .ge. 1.0 ) then
        type 1, x
      else
        atanh = alog( sqrt( ( 1.0 + x ) / ( 1.0 - x ) ) )
        end if
      return
1     format( ' ATANH : FATAL ; Invalid Argument . ', e14.6 )
      end




      REAL FUNCTION QARC0( RANGLE )
c
c    Real function which receives the value RANGLE as an angle of arc in
c radians and returns the equivalent value between Pi and -Pi
c
      implicit none
      real pi
      parameter ( pi = 3.1415926)
      real Rangle, Amod, abs, sign
c
      Qarc0 = ( Amod( Abs( Rangle ) + pi , 2. * pi ) - pi ) *
     1          Sign( 1.0, Rangle )
      return
      end



      REAL FUNCTION QBINOM( A, K )
c
c    Function to find the Binomial coefficient of     A     A.(A-1)...(A-K+1)
c  A and K , ie                                      ( ) = -------------------
c                                                     K           K!
c    Arguments
c real a                # super argument
c integer *4 k,         # sub argument
c
c    Variables
c integer *4 i          # looping index
c
      implicit none
      real a, float
      integer *4 k, i
c*
      qbinom = 1.0
      if ( k .le. 0 ) return
      do i = 0, k-1
        qbinom = qbinom * ( a - float( i ) ) / ( i + 1 )
        end do
      return
      end



      REAL FUNCTION QDIST( X, Y, A, B, C )
c
c       DISTance from a point to a line
c   Function to calculate the distance from the line  A.x + B.y + C = 0
c to the point (X,Y) .
c
c   Arguments
c real  a,b,c,          # parameters for defining the line
c       x,y             # coordinates of the point
c
      implicit none
      real a, b, c, x, y, abs, sqrt
c
      QDIST = 0.0
      if ( a .ne. 0.0 .or. b .ne. 0.0 )
     1  QDIST = abs( a * x + b * y + c ) / sqrt( a **2 + b **2 )
      return
      end



      REAL FUNCTION QPOLY( X, B, K )
c
c       POLYnomial evaluation
c    Evaluates the Kth order polynomial :-
c P(x) = b(1) + b(2) * x + b(3) * x**2 + ... + b(k) * x**(k-1) + b(k+1) * x**k
c
c Arguments
c  real x,                     # dependent variable
c       b(k+1)                 # coefficients of the polynomial
c  integer *4 k                # order of the polynomial
c
c variable
c  integer *2 i                # looping index
c
      implicit none
      integer *2 i
      integer *4 k
      real x, b(*)
c
      Qpoly = 0.0
      if ( k .lt. 0 ) return
      Qpoly = B(k+1)
      do i = k, 1, -1
        Qpoly = Qpoly * x + b(i)
        end do
      return
      end



      DOUBLE PRECISION FUNCTION DPOLY( X, B, K )
c
c       POLYnomial evaluation
c    Evaluates the Kth order polynomial :-
c P(x) = b(1) + b(2) * x + b(3) * x**2 + ... + b(k) * x**(k-1) + b(k+1) * x**k
c
c Arguments
c  double precision x,   # dependent variable
c         b(k+1)         # coefficients of the polynomial
c  integer k             # order of the polynomial
c
c variable
c  integer i             # looping index
c
      include 'stdio.def'
      integer i, k
      double precision x, b(*)
c
      Dpoly = 0.0
      if ( k .lt. 0 ) return
      Dpoly = B(k+1)
      do 1 i = k, 1, -1
          Dpoly = Dpoly * x + b(i)
1         continue
      return
      end


