c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

c           *****************************************************
c           ***                                               ***
c           ***    P.V.Lanspeary private FORTRAN 77 library   ***
c           ***                                               ***
c           ***        f77_num : Numerical methods            ***
c           ***                                               ***
c           *****************************************************
c  
c    Common blocks : / Af77lib2 / for QZERO
c                    / Af77lib3 / for QLRPL1, QLRPLK
c                    / Af77lib5 / for QARPSD, SARINZ
c                    / AF77lib6 / for QQMIND
c                    / Af77lib7 / for SMAFRF


      INTEGER FUNCTION JBSRCH( X, NINCR, NPTS, X0 )
c
c       Binary SeaRCH
c  Function for locating the last element between the 1st and Nth elements
c  in the array X of N real numbers , which in the case of an ascending
c  array , is less than the value X0 . In the case of a descending array ,
c  it finds the last element which is greater than or equal to X0 . The
c  returned value of JBSRCH is the index of this element in the array X .
c    N.B. X must be an array of values arranged in STRICTLY ASCENDING or
c  STRICTLY DESCENDING order .
c
c     Arguments
c real  x(*),           # data array
c       x0              # search goal
c integer *4 nincr,     # data array indexing increment
c            npts       # number of data points used
c     Variables
c logical desc          # .true. if data array X is descending
c integer *4 n          # index of last point used
c           l,r,        # left & right hand indices of the search interval
c           c           # moving search index
c           JBSRCH      # index of last element in array less than X0
c
      real x(*), x0
      logical *2 desc
      integer *4 nincr, npts, n, l, r, c
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n = 1 + ( npts - 1 ) * nincr
      desc = x(1) .gt. x(n)
      if ( x0 .eq. x(1) ) then
        l = 1
      else if ( x0 .eq. x(n) ) then
        l = n - nincr
      else if ( x0 .lt. x(1) .xor. desc ) then
        l = 0
      else if ( x0 .gt. x(n) .xor. desc ) then
        l = n
      else
        l = 1
        r = n
        do while( r - l .gt. nincr )
          c = l + ( r - l ) / 2 / nincr * nincr
          if ( x0 .gt. x(c) .xor. desc ) then
            l = c
          else
            r = c
            end if
          end do
        end if
      JBSRCH = l
      return
      end



      INTEGER FUNCTION JLSRCH( X_DATA, INCREM, N_POINTS, X0 )
c
c       Linear SeaRCH
c  Function for locating the first element between the 1st and N_POINTSth
c  elements in the array X_DATA of N_POINTS real numbers, which in the case
c  of an ascending array, exceeds the value X0. In the case of a descending
c  array, it finds the first element which is less than or equal to X0. In
c  both cases, the returned value of JLSRCH is the index of the previous
c  element in the array X_DATA.
c      N.B. This routine is for use if the values in X_DATA should not be
c  arranged in STRICTLY ASCENDING or STRICTLY DESCENDING order. In this case
c  the value returned by JBSRCH is not defined, but if the data is strictly
c  monotonic JBSRCH is much quicker.
c
c     Arguments
c real    x_data(*) # data array
c integer increm    # data array indexing increment
c         n_points  # number of data points used
c real    x0        # search goal
c
c Local variables
c  logical
c   descend         # .true. if data array X is descending
c  integer
c   index           # data array index
c   n_last          # index of last point used
c
      include 'stdio.def'
      character ABORT_LABEL*18
      parameter( ABORT_LABEL=' JLSRCH : ABORT : ' )
c
      logical descend
      integer increm, index, n_points, n_last
      real    x_data(*), x0
c
      JLSRCH = 1
      if ( n_points .le. 0 .or. n_points .gt. 10000000 ) then             1b
          write( STD_ERR, 1 ) ABORT_LABEL, n_points
1         format( a, 'Illegal argument, N_POINTS = ', i12 )
      else if ( increm .le. 0 .or. increm .gt. 1000 ) then                1m
          write( STD_ERR, 2 ) ABORT_LABEL, increm
2         format( a, 'Illegal argument, INCREM = ', i12 )
      else                                                                1m
          n_last = 1 + ( n_points - 1 ) * increm
c
c       Decide if the data is increasing or decreasing from the first
c       change in array element value.
c
          index = 2
3         if ( index .le. n_last ) then                                   2b
              if ( x_data(index) .ne. x_data(1) ) then                    3b
                  descend = x_data( 1 ) .gt. x_data( index )
              else                                                        3m
                  index = index + increm
                  goto 3
                  end if                                                  3e
c
c       Linear search procedure
c
4             if ( JLSRCH .le. n_last ) then                              3b
                  if ( x0 .ge. x_data(JLSRCH) .neqv. descend ) then       4b
                      JLSRCH = JLSRCH + increm
                      goto 4
                      end if                                              4e
                  JLSRCH = max( 1, JLSRCH - increm )
                  end if                                                  3e       2e
              JLSRCH = min( JLSRCH, n_last )
          else                                                            2m
c
c       All data values are the same
c
              write( STD_ERR, 5 ) ABORT_LABEL
5             format( a, 'All data values are the same.' )
              end if                                                      2e
          end if                                                          1e
      return
      end



      REAL FUNCTION QDET( A, M, TOL )
c
c       Calculate the determinant of a matrix
c
c    Arguments
c real a(m,m),          # operand matrix
c       tol             # zero detection tolerance
c integer *4 m          # number of rows and number of columns in matrix A
c
      implicit none
      integer *4 i, m,j
      real a(m,m), tol
c
      call sgjredz( a, m, m, m, tol, .false. )
      qdet =1.0
      do i = 1, m
        qdet = qdet * a(i,i)
        end do
      return
      end


      REAL FUNCTION QDSCOP( X, F, STEP, TOL, MAX )
c
c       D.S.C. OPtimization
c   Function for minimizing or maximizing a function of one variable F(X) on
c an unrestricted domain to within a tolerance in X of   +/- tol  by the
c D.S.C. search algorithm . If MAX = .false. a minimization is done . If
c MAX = .true. a maximization is done .
c
c Arguments
c  real x,              # starting point for the search
c  external function f, # name of external real function to be optimized
c  real step,           # initial search step size
c       tol,            # resolution in X of the search
c logical *4 max        # minimize (.false.) / maximize (.true.) switch
c
c real  d,              # search increment
c       d1,             # expandable search increment
c       fp1,fp2,fp3,fx0,fx1,fx2,        #       f## = f( ## )
c       p1,p2,p3,       # lower,centre & upper pts for quadratic interpolation
c       r,              # search increment reduction ratio
c       s,              # half width of tolerance zone
c       atol,           # required tolerance ( +/- tol )
c       x0,             # central search point
c       x1,x2           # comparative search points
c
      real *4 d, d1, fp1, fp2, fp3, fx0, fx1, fx2, p1, p2, p3, r, s,
     1         atol, tol, x0, x1, x2, step, x
      logical *4 max
c
c       initialize
c
      d = step
      r = 1.9
      atol = abs( tol )
      s = atol * 2.0
      x0 = x
      fx0 = f( x0 )
c
c       main loop
c
      do while ( abs( s ) .gt. atol )
        x1 = x0 + d
        fx1 = f( x1 )
        if ( fx1 .gt. fx0 .xor. max ) then
          d = -d                            ! determine which search
          x1 = x0 + d                       ! direction improves f(x)
          fx1 = f( x1 )                     !
          if ( fx1 .gt. fx0 .xor. max ) then
            p1 = x0 - d           !
            fp1 = f( p1 )         ! if search => worse
            p2 = x0               !
            fp2 = fx0             ! values on both sides
            p3 = x1               !
            fp3 = fx1             ! set up quadratic
            s = d                 ! interpolation
            goto 1                !
            end if
          end if
        if ( abs( fx1 - fx0 ) .lt. amax1( abs( fx1 ), abs( fx0 ) ) *
     1        1.2e-7 ) then
          p1 = x0               !
          fp1 = fx0             ! equal values :
          p2 = x0 + d / 2.0     !
          fp2 = f( p2 )         ! set up quadratic
          p3 = x1               !
          fp3 = fx1             ! interpolation
          s = d / 2.0           !
          if ( abs( fp2 - fp1 ) .lt. abs( fx1 ) * 1.2e-7 ) goto 3
        else
c
c       increase search interval until a worse result appears
c
          d1 = d
c         repeat
2           x0 = x1
            fx0 = fx1
            d1 = d1 * 2.0
            x1  = x0 + d1
            fx1 = f( x1 )
          if ( .not. ( fx1 .gt. fx0 .xor. max ) ) goto 2
c         until( fx1 .gt. fx0 .xor. max )
          s = d1 / 2.0
          x2 = x0 + s
          fx2 = f( x2 )
c
c       set up quadratic interpolation
c
          if ( fx2 .gt. fx0 .xor. max ) then
            p1 = x0 - s
            fp1 = f( p1 )
            p2 = x0
            fp2 = fx0
            p3 = x2
            fp3 = fx2
          else
            p1 = x0
            fp1 = fx0
            p2 = x2
            fp2 = fx2
            p3 = x1
            fp3 = fx1
            end if
          end if
c
c       quadratic interpolation
c
1       x0 = p2
        if ( fp1 - fp2 - fp2 + fp3 .ne. 0 )
     1    x0 = p2 + s * ( fp1 - fp3 ) / ( fp1 - fp2 - fp2 + fp3 ) / 2.0
        fx0 = f(x0)
        d = d / r
        end do
3     QDSCOP = x0
      return
      end



      REAL FUNCTION QGSOPT( AA, BB, F, TOL )
c
c       Golden Section search OPTimizing
c   Function for minimizing  or maximizing a function of one variable F(X) on
c the domain  aa < x < bb to within an accuracy  (bb-aa)/2 <= tol by the
c golden section search algorithm . If aa > bb a maximization is done . If
c bb > aa a minimization is done . Correct results can only be guarranteed for
c unimodal functions .
c
c real  a,aa,           # lower bound of search interval
c       b,bb,           # upper bound of search interval
c       p,q,            # test points in the search interval
c       fp,fq,          # = f(p) & = f(q)
c       tol,atol        # tolerance on the result
c
c logical s             # maximize (.true.) / minimize (.false.) switch
c
      real a, aa, b, bb, p, q, fp, fq, tol, atol
      logical *2 s
c
c       Check parameters & initialize
c
      a = aa
      b = bb
      s = .false.
      atol = abs( tol )
      if ( a .gt. b ) then
        call swap4( a, b )
        s = .true.
        end if
      q = a + ( b - a ) * 0.6180340
      p = b - ( b - a ) * 0.6180340
      fp = f( p )
      fq = f( q )
      do while ( ( b - a ) / 2.0 .ge. atol )
        if ( fq .ge. fp .xor. s ) then
          b = q
          q = p
          fq = fp
          p = b - ( b - a ) * 0.6180340
          fp = f( p )
        else
          a = p
          p = q
          fp = fq
          q = a + ( b - a ) * 0.6180340
          fq = f( q )
          end if
        end do
      QGSOPT = ( a + b ) / 2.0
      return
      end



      REAL FUNCTION QLAGR_ITP( X, Y, NPTS, X_0 )
c
c       Lagrange formula for n point interpolation .
c  The arrays X and Y are NPTS data points . X_0 is the abscissa value
c  for which the the interpolated value Y_0 is to be found .
c  The returned value of QLAGR_ITP is the interpolated ordinate value Y_0 .
c
c  N.B. This is a straightforward application of Lagrange's formula .
c  The proper method would be to first evaluate the polynomial coefficients
c  from the data arrays, and then simply evaluate the polynomial value once
c  for each X_0 .
c
      implicit none
      real Li, x(*), y(*), x_0
      integer npts, i, j
c
      Qlagr_itp = 0.0
      do i = 1, npts
        Li = 1.0
        do j = 1, npts
          if ( i .ne. j ) then
            Li = Li * ( x_0 - x(j) ) / ( x(i) - x(j) )
            end if
          end do
        Qlagr_itp = Qlagr_itp + Li * y(i)
        end do
      return
      end


      REAL FUNCTION QLRPL1( X, Y, NINCR, NPTS, X0 )
c
c       LineaR inTerPoLation from tabulated data .
c   Function to linearly interpolate ordinate y0, given a table of (x,y)
c coordinates in the arrays X & Y and an abicissa value x0 . Starting from the
c first elments of X & Y , and with an indexing  increment of NINCR , NPTS
c coordinate pairs are used from X & Y .
c The array X is searched until adjacent values x(l) & x(r) straddle x0 .
c   N.B. ELEMENTS OF X MUST BE IN STRICTLY ASCENDING ORDER OR STRICTLY
c        DESCENDING ORDER
c
c       Arguments
c real  x(*),y(*),      # arrays of tabulated (x,y) coordinates
c       x0              # target for which a y0 value is required
c integer *4 nincr,     # data array indexing increment
c            npts       # number of data points used
c
c constant
c   Hvdulu              # v.d.u. logical unit number
c  Variables
c integer *4 n          # size of arrays
c            l          # left hand indices of the search interval
c logical pass1         # .true. until an attempt to extrapolate is detected
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real x(*), y(*), x0
      integer *4 nincr, npts, n, l, Jbsrch
      logical *2 pass11,pass1k
      common /af77lib3/ pass11, pass1k
      data pass11 / .true. /
c
c       Find the array interval
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n = 1 + ( npts - 1 ) * nincr
      l = Jbsrch( x, nincr, npts, x0 )
      if ( l .lt. 1 .or. l .ge. n ) then
        if ( pass11 ) write ( Hvdulu, 1 )
1                    format( ' QLRPL1 : Extrapolating' )
        pass11 = .false.
        l = max0( 1, min0( l, n - nincr ) )
        end if
c
c       Interpolate
c
      qlrpl1 = ( y(l+nincr) - y(l) ) / ( x(l+nincr) - x(l) ) *
     1           ( x0 - x(l) ) + y(l)
      return
      end



      REAL FUNCTION QLRPLK( X, Y, NINCR, NPTS, X0, K, MSERR )
c
c       LineaR inTerPoLation from tabulated data .
c   Function to linearly interpolate ordinate  y0, given a table of (x,y)
c coordinates in the arrays X & Y and an abicissa value x0 .   Starting from
c the first elments of X & Y , and with an indexing  increment of NINCR , NPTS
c coordinate pairs are used from X & Y .The array X is
c searched until adjacent values x(l) & x(r) straddle x0 . K is the number of
c points each side of x0 that are to be used in the interpolation . K > 1 .
c A linear regression is performed on the K points to the left of x0 , and the
c K points to the right of x0 , an y0 is calculated from the linear
c regression .
c   N.B. ELEMENTS OF X MUST BE IN ASCENDING ORDER
c
c   Arguments
c real  x(*),y(*),      # arrays of tabulated (x,y) coordinates
c       x0,             # target for which a y0 value is required
c       mserr           # mean square error from the linear regression .
c integer *4 nincr      # data array indexing increment
c            npts       # number of data points used
c            k,         # no. of points each side of x0 used in interpolation
c
c constant
c     Hvdulu            # v.d.u. logical unit number
c real  a,b,corr        # slope, intercept, correlation coeffient and mean
c                       # square error from the linear regression .
c integer *4 n          # index of last point used
c            l          # left hand indices of the search interval
c logical *2 pass1k     # .true. until an extrapolation is attempted
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu = 6 )
      integer *4 npts, nincr, n, k, l, ifirst, ilast, JBSRCH
      real x(*), y(*), x0, a, b, corr, mserr
      logical *2 pass11, pass1k
      common /af77lib3/ pass11, pass1k
c
      data pass1k / .true. /
c
c       Check for valid parameters first
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n = 1 + ( npts - 1 ) * nincr
      l = Jbsrch( x, nincr, npts, x0 )
      if ( l .le. 0 .or. l .ge. n ) then
        if ( pass1k ) write( Hvdulu, 1 )
1         format( ' QLRPLK : Extrapolating' )
        pass1k = .false.
        l = max0( 1, min0( l, n - nincr ) )
        endif
      ifirst = max0( l - ( k - 1) * nincr, 1 )
      ilast = min0( npts - ( ifirst - 1 ) / nincr ,
     1              ( l - ifirst ) / nincr + 1 + k )
      call slrreg( x(ifirst), y(ifirst), nincr, ilast, a, b,corr,mserr )
      qlrplk = a * x0 + b
      return
      end




      REAL FUNCTION QMEAN( X, NINCR, NPTS )
c
c       arithmetic MEAN of a sequence
c     Function to find the arithmetic mean of X, a sequence of NPTS real numbers,
c selected from X  with increments of NINCR
c
c     Arguments
c real x(*)            # operand array
c integer *4 nincr,    # index increment
c            npts      # number of data points to be averaged
c     Variables
c integer *4 i         # a working array index
c double precision rtotal
c
      implicit none
      integer *4 nincr, npts, i
      double precision rtotal
      real x(*)
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      rtotal = 0.0d0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        rtotal = rtotal + x(i)
        end do
      qmean = sngl( rtotal ) / float( npts )
      return
      end



      REAL FUNCTION QMEANG( X, NINCR, NPTS, K, C )
c
c       Generalized MEAN ( moment ) of a sequence
c     Function to evaluate the Kth moment of NPTS values spaced at intervals of
c NPTS in an array X real numbers , about the value C .
c     If  C = QMEAN( X, M1, M2, N ) , the central Kth moment is calculated .
c     QMEAN( X, M1, M2, N, 3, QMEAN( X, M1, M2, N ) is the skewness .
c
c     Arguments
c real x(*)            # operand array
c      c               # centre of moments
c integer *4 nincr,    # data array indexing incremant
c            npts,     # size of operand array
c            k         # order of the moment
c     Variables
c integer *4 i,        # a working array index
c double precision rtotal
c
      implicit none
      integer *4 npts, nincr, i, k
      double precision rtotal
      real x(*), c
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      rtotal = 0.0d0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        rtotal = rtotal + ( x(i) - c ) ** k
        end do
      QMEANG = sngl( rtotal ) / float( npts )
      return
      end



      REAL FUNCTION QQMIND( X, Y, NINCR, NPTS, XA, XB )
c
c       Quadratic Mean Integration of Discrete data .
c
c    This was developed for calculating the integral thicknesses of a boundary
c  layer
c  The procedure is to fit a parabola to 3 adjacent points , and integrate the
c  the parabola according to its coefficients . The index of the data points
c  is then incremented and the process is repeated .
c
c Arguments
c  real x(*)           # abicissa data array in strictly ascending order
c       y(*)           # ordinate data array
c  integer *4 nincr    # data array indexing increment
c             npts     # number of data points to be used
c  real xa             # lower abicissa limit of integration
c       xb             # upper abicissa limit of integration
c Variables
c  integer *4 i        # looping index
c             i1, i2, i3       # 1st, 2nd & 3rd pointsof the quadratic fit
c             Jbsrch   # function : binary index search
c  real a, b, c        # cubic coefficients of a.x**3+b.x**2+c.x
c       xaa, xbb       # corrected lower & upper limits of integration
c       qqmpol         # function : polynomial evaluation for QQMIND
c
      implicit none
      real a, b, c, x(*), y(*), xa, xb, xaa, xbb, qqmpol
      integer *4 i, i1, i2, i3, npts, nincr, nlast, Jbsrch
c
      common / Af77lib6 / a, b, c
c
c         check arguments
c
      qqmind = 0.0
      if ( nincr .lt. 1 .or. npts .lt. 3 .or. xa .eq. xb )
     1  return
      xaa = xa
      xbb = xb
      if ( xaa .gt. xbb )
     1  call swap4( xaa, xbb )
c
c         set lower limit
c
      xaa = max( x(1), xaa )
c
c         set upper limit
c
      i3 = nincr * ( npts -1 ) + 1
      xbb = min( x(i3), xbb )
c      WRITE(6,9999) XAA, XBB,(X(I),Y(I),I=1,NINCR*(NPTS-1)+1,NINCR)
c9999  FORMAT( ' XAA, XBB = ', 2G13.6 / ( 1X, 2G13.6 ) )
c
c         set last index
c
      nlast = Jbsrch( x, nincr, npts, xbb )
c      WRITE(6,9994) NLAST
c9994  FORMAT( ' NLAST = ', I5 )
      if ( x(nlast) .le. xbb )
     1  nlast = min( nlast+1, i3 )
c
c         integrate for 1st 3 data points
c
      i1 = Jbsrch( x, nincr, npts, xaa )
      i2 = nincr + i1
      i3 = nincr + i2
      a = ( (y(i3)-y(i2)) / (x(i3)-x(i2)) - (y(i1)-y(i2)) /
     1    (x(i1)-x(i2)) ) /(x(i3)-x(i1))
      b = ( (y(i2)-y(i1)) / (x(i2)-x(i1)) - (x(i2)+x(i1)) * a )
      c = y(i2) - ( a * x(i2) + b ) * x(i2)
      a = a / 3.0
      b = b / 2.0
      qqmind = qqmpol(x(i3)) + qqmpol(x(i2)) - 2 * qqmpol(xaa)
c      WRITE(6,9990) 1,QQMIND
c9990  FORMAT( ' QQMIND : ', I3, G13.6 )
c
c         integrate all but last point
c
      if ( nlast .ge. i3 ) then
        do i = i3 + nincr, nlast, nincr
          i1 = i2
          i2 = i3
          i3 = i
          a = ( (y(i3)-y(i2)) / (x(i3)-x(i2)) - (y(i1)-y(i2)) /
     1        (x(i1)-x(i2)) ) / (x(i3)-x(i1))
          b = ( (y(i2)-y(i1)) / (x(i2)-x(i1)) - (x(i2)+x(i1)) * a )
          c = y(i2) - ( a * x(i2) + b ) * x(i2)
          a = a / 3.0
          b = b / 2.0
c      WRITE(6,9997) A,B,C, X(I1), X(I2), X(I3)
c9997  FORMAT( ' NEW ABC ', 3G13.6 / '  FOR ', 3G13.6 )
          qqmind = qqmind + qqmpol(x(i3)) - qqmpol(x(i1))
c          xaa = xaa + qqmpol(x(i3)) - qqmpol(x(i1))
c          if ( xaa .gt. 0.0 )
c     1      qqmind = qqmind + xaa
c      WRITE(6,9990) 2,QQMIND
          end do
c
c         integrate last point
c
        qqmind = ( qqmind + 2.0 * qqmpol(xbb) - qqmpol(x(nlast)) -
     1    qqmpol(x(nlast-1)) ) / 2.0
c      WRITE(6,9990) 3,QQMIND
c
c         special case for xb between first 2 data points
c
      else
        qqmind = ( qqmind + 2.0 * qqmpol(xbb) - qqmpol(x(i2)) -
     1    qqmpol(x(i3)) ) / 2.0
        end if
      if ( xa .gt. xb )
     1  qqmind = -qqmind
      return
      end


      REAL FUNCTION QQMPOL( X )
c
c       POLynomial evaluation for QQMIND
c
      implicit none
      real x, a, b, c
c
      common / Af77lib6 / a, b, c
c
      qqmpol = ( ( a * x + b ) * x + c ) * x
c      WRITE(6,9999) QQMPOL,3.0*a*x*x+2.0*b*x+c, X, A, B, C
c9999  FORMAT( ' QXyABC', 6G12.6 )
      return
      end


      REAL FUNCTION QRKDE( X, Y, H, F )
c
c       Runge- Kutta solution of Differential Equation
c     Function for the Runge- Kutta ( numerical method ) solution of a fisrt
c order ordinary differential equation of the form :-
c       y' = dy/dx = f( x, y )
c
c                              input                           output
c real x,           # independent variable          # increased by h
c      y,              # initial value of y            # result
c      h,              # increment size
c      a,b,c,d,e               # intermediate values
c      QRKDE                                           # result
c
      implicit none
      real x, y, h, a, b, c, d, e, f
c
      e = x + h / 2.0
      a = h * f( x, y )
      b = h * f( e, y + a / 2.0 )
      c = h * f( e, y + b / 2.0 )
      x = x + h
      d = h * f( x, y + c )
      y = y + ( a + 2.0 * b + 2.0 * c + d ) / 6.0
      QRKDE = y
      return
      end



      REAL FUNCTION QRKNDE( X, Y, Z, H, F )
c
c       Runge- Kutta- Nystrom  solution of Differential Equation
c     Function for Runge- Kutta- Nystrom ( numerical method ) solution of a
c second order ordinary differential equation of the form :-
c       y" = d( dy/dx )/dx = f( x, y, y' ) ;   y' = dy/dx
c
c                              input                           output
c
c real x,              # independent variable          # increased by h
c      y,              # initial value of y            # result- value of y
c      z,              # initial value of y'           # final value of y'
c      h,              # increment size
c      a,b,c,d,e,g             # intermediate values
c      QRKNDE                                          # result
c
      implicit none
      real x, y, z, h, h2, a, b, c, d, e, g, f
c
      h2 = h / 2.0
      e = x + h2
      a = h2 * f( x, y, z )
      g = y + h2 * ( z + a / 2.0 )
      b = h2 * f( e, g, z + a )
      c = h2 * f( e, g, z + b )
      x = x + h
      d = h2 * f( x , y + h * ( z + c ), z + 2.0 * c )
      y = y + h * ( z + ( a + b + c ) / 3.0 )
      z = z + ( a + 2.0 * b + 2.0 * c + d ) / 3.0
      QRKNDE = y
      return
      end



      REAL FUNCTION QRKIN1( X, Y, H, F )
c
c       Runge-Kutta INtegration ( Single only )
c
c     Function for integrating the function F(x) with respect to the
c independent variable x to give the result :-
c       Y <= QRKIN1 <= { integral of F(x) from x to x+h }  +  y
c using the Runge-Kutta 4th order method . Repeated use of QRKIN1 produces an
c antiderivative function of F(x) , eg.
c       external f
c       x = ....
c       y = ....
c       do i = 1, n
c        $(
c         h = ....
c         if ( 1.00 <= QRKIN1( x, y, h, f ) ) break
c        $)
c integrates f(x) from x to x + { sum of h's }
c
c                              input                           output
c real x,              # independent variable          # increased by h
c      y,              # initial condition - constant  # integral
c      h               # step size
c      QRKIN1                                          # integral
c
      implicit none
      real x, y, h, f
c
      y = y + h / 6.0 * ( f( x ) + 4.0 * f( x + h / 2.0 ) + f( x + h ) )
      QRKIN1 = y
      x = x + h
      return
      end



      REAL FUNCTION QRKIN2( X, Y, Z, H, F )
c
c       Runge- Kutta nystrom INtegration ( Double )
c     Function for integrating the function F(x) twice with respect to the
c independent variable X to give the result :-
c       Z <= { integral of F(x) from x to x+h ) + z
c       Y <= QRKIN2 <= { double integral of F(x) from x to x+h } + h.z + y
c
c                              input                           output
c real x,              # independent variable          # increased by h
c      y,              # initial condition - constant  # integral
c      z,              # initial condition - slope     # slope of integral
c      h,              # step size
c      fx,fxh2         # f(x) & f(x+h/2)
c      QRKIN2                                          # integral
c
      implicit none
      real x, y, z, h, fx, fxh2, f
c
      fx = f(x)
      fxh2 = f( x + h / 2.0 ) * 2.0
      x = x + h
      y = y + h * z + h **2 / 6.0 * ( fx + fxh2 )
      z = z + h / 6.0 * ( fx + 2.0 * fxh2 + f(x) )
      QRKIN2 = y
      return
      end



      REAL FUNCTION QSRINC( A, B, N, F )
c
c       Simpson's Rule Integration ( Continuous )
c     Numerical integration of the function f(x) over the interval a < x < b ,
c by using Simpson's 4th order method with n subintervals ( n is even ) .
c
c     Arguments
c real a,              # lower limit of integration interval
c      b,              # upper limit of integration interval
c integer *4 n,        # number of subintervals - should be even
c
c     Variables
c real h, h2,          # length & twice the length of a subinterval
c      x,              # integration variable
c      tol             # tolerance on incrementing
c integer *4 n,n1      # number of subintervals - should be even
c
      implicit none
      real a, b, h, h2, x, tol, f
      integer *4 n, n1
c
      n1 = max0( n/2*2, 2 )
      h = ( b - a ) / n1
      h2 = 2.0 * h
      tol = h * 1.2e-7 * n1
      x = a
      QSRINC = f( a ) + 4.0 * f( a + h ) + f( b )
      do while ( x .lt. ( b - h2 - tol ) )
        x = x + h2
        QSRINC = QSRINC + 2.0 * f( x ) + 4.0 * f( x + h )
        end do
      QSRINC = QSRINC * h / 3.0
      return
      end



      REAL FUNCTION QSRIND( X, Y, NINCR, NPTS, A, B, N )
c
c       Simpson's Rule INtegration ( Discrete )
c   Numerical integration of the function defined by the table of values given
c by the coordinate pairs in arrays X & Y over the interval a < x < b , by
c using Simpson's 4th order method with n subintervals .
c
c    Arguments
c real  x(m),y(m),      # (x,y) data arrays
c       a,              # lower limit of integration interval
c       b,              # upper limit of integration interval
c integer *4 nincr      # data array indexing increment
c            npts,      # number of data points
c            n          # number of subintervals - should be even
c
c    Variables
c real  h, h2,          # length & twice the length of a subinterval
c       tol,            # tolerance on incrementing
c       z,              # integration variable
c       QLRPL1          # linear interpolation function
c integer *4 n1         # number of subintervals - should be even
c
      implicit none
      integer *4 nincr, npts,  n, n1
      real a, b, h, h2, tol, x(*), y(*), z, qlrpl1
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      n1 = max0( n/2*2, 2 )
      h = ( b - a ) / n1
      h2 = 2.0 * h
      tol = h * 1.2e-7 * n1
      z = a
      QSRIND = QLRPL1( x, y, nincr, npts, a ) + QLRPL1( x, y, nincr,
     1              npts, a+h )* 4.0 + QLRPL1( x, y, nincr, npts, b )
      tol = b - h2 - tol
      do while ( z .lt. tol )
        z = z + h2
        QSRIND = QSRIND + 2.0 * QLRPL1( x, y, nincr, npts, z ) + 4.0 *
     1             QLRPL1( x, y, nincr, npts, z+h )
        end do
      QSRIND = QSRIND * h / 3.0
      return
      end



      REAL FUNCTION QTRAP_VS( X, Y, INCREM, N_POINTS )
c
c       TRAPezoidal rule integration of discrete data array
c       ( Variable step size, Single result )
c
c  This routine integrates the function defined by the N_POINTS coordinate pairs
c  ( X(i), Y(i), i = 1, INCREM+1, INCREM+2 ... ) using the first X value as the
c  lower limit of integration and the last X value as the upper limit of
c  integration.
c  N.B. only a single value, an estimate of the integral over the full abscissa
c       range is returned. If progressive integral estimates at every X value
c       are required, STRAPZ should be used.
c
c Arguments
c  real x(*)          # abicissa data array in strictly ascending order
c       y(*)          # ordinate data array
c  integer increm     # data array indexing increment
c          n_points   # number of data points to integrate
c
c constants
c  MIN_INCREM=1       # minimum array index increment
c  MAX_INCREM=1000    # maximum array index increment
c  MIN_POINTS=2       # minimum number of points to integrate
c  MAX_POINTS=10000000# maximum number of points to integrate
c  ABORT_LABEL        # ABORT identification message
c
c local variables
c  integer
c   left              # array index for left side of subinterval
c   right             # array index for right side of subinterval
c
c
      include 'stdio.def'
      integer    MIN_INCREM, MAX_INCREM,
     1           MIN_POINTS, MAX_POINTS
      character  ABORT_LABEL*37
      parameter( MIN_INCREM=1, MAX_INCREM=1000,
     1           MIN_POINTS=2, MAX_POINTS=10000000,
     2           ABORT_LABEL=' QTRAP_VS : ABORT : Illegal argument ' )
c
      integer    increm, n_points, left, right
      real       x(*), y(*)
c
c         check arguments
c
      if ( increm .lt. MIN_INCREM .or. increm .gt. MAX_INCREM ) then     1b
          write( STD_ERR, 1 ) ABORT_LABEL, 'INCREM = ', increm
1         format( a, a, i12 )
      else if ( n_points .lt. MIN_POINTS .or.
     1          n_points .gt. MAX_POINTS ) then                          1m
          write( STD_ERR, 1 ) ABORT_LABEL, 'N_POINTS = ', increm
      else
          QTRAP_VS = 0.0
          left = 1
          do 2 right = 1 + increm, increm * ( n_points - 1 ) + 1, increm 2b
              QTRAP_VS = QTRAP_VS +
     1            ( x(right) - x(left) ) * ( y(right) + y(left) )
              left = right
2             continue                                                   2e
          QTRAP_VS = QTRAP_VS / 2.0
          end if                                                         1e
      return
      end




      REAL FUNCTION QZERO( F, XMIN, XMAX, N, ERROR, ZERTOL, OK )
c  Function subprogram to return the first encountered zero of a function.
c
c       The method is to split the domain Xmin<= x <= Xmax into N intervals
c The algorithm tests the endpoints of each interval and selects the interval
c (x1 <= x <= x2) which crosses the f=0 axis . The interval (x1 <= x <=x2) in
c which the zero occurs is narrowed by successive linear interpolation .
c
c    The parameters of ZERO are :
c real  f,       # = f(x) is the function for which the zero is to be found
c       Xmin,    # is the lower limit of the domain of f(x)
c       Xmax,    # is the upper limit of the domain of f(x)
c       zertol,  # is the tolerance for zero detection of f(x)
c       error    # = tolerance in the returned value of ZERO ,if error>0
c                # = tolerance on f(ZERO) , if error<0
c integer *4 N   # = number of intervals tested for zero crossing
c logical *4 ok  # = .True. if a zero crossing is found
c                # = .False. if a zero crossing is not found in
c                #              Xmin <= x <= Xmax
c The other variables are :
c real  x1,fx1,  # = (x1,f(x1)) = lower limit of interval
c       x2,fx2,  # = (x2,f(x2)) = upper limit of interval
c       x,fx,    # = (x,f(x)) = interpolated point
c       t        # = |zertol|
c logical eqsign # function subprogram : returns .True. if its 2 arguments are
c                #  the same.
c
      real f, xmin, xmax, zertol, error, x1, fx1, x2, fx2, x, fx, t
      logical *4 eqsign, ok
      integer *4 n
      common /af77lib2/x,x1,x2,fx,fx1,fx2,t,delx
      external f
c
      t = abs( zertol )
      x = ( xmax - xmin ) / n
      x1 = xmin
      fx1 = f( x1 )
      fx = fx1
      xmax1 = x * float( n ) * 0.0000003 + xmax
c
c       loop stepping thru' intervals
c
      do x2 = xmin + x, xmax, x
        fx2 = f(x2)
        if ( abs(fx1) .lt. t ) then
c
c               stop if interval limit is at a zero
c
          qzero = x1
          ok = .true.
          return
          end if

c
c               If the interval crosses the f(x)=0 axis , iterate linear
c               interpolation until the error is small enough
c
        if ( .not. eqsign( fx1, fx2 ) ) then
          delx = x2 - x1
          if ( error .gt.  0 ) then
            do while ( abs( delx ) .gt. error )
              call szero1( f )
              end do
          else
            do while ( abs( fx ) .gt. -error )
              call szero1( f )
              end do
            end if
          ok = .true.
          qzero = x
          return
          end if
        fx1 = fx2
        x1 = x2
        end do
c
c               signal "zero not found"
c
      ok = .false.
      qzero = x2
      return
      end

      SUBROUTINE SZERO1( f )
c Performs one iteration of the linear interpolation .
      implicit none
      real x, x1, x2, fx, fx1, fx2, t, delx, f
      common /af77lib2/x,x1,x2,fx,fx1,fx2,t,delx
c
      x = ( fx2 * x1 - fx1 * x2 ) / ( fx2 - fx1 )
      fx = f( x )
      if ( abs( fx1 ) .le. t ) then
        x2 = x
      else if ( abs( fx2 ) .le. t ) then
        x1 = x
      else
        call szero2
        end if
      if ( ( x2 - x1 ) / delx .gt. 0.6 ) then
        x = ( x2 + x1 ) / 2.0
        fx = f( x )
        call szero2
        end if
      delx = x2 - x1
      return
      end

      SUBROUTINE SZERO2
c Adjusts interval length
      implicit none
      real x, x1, x2, fx, fx1, fx2, t, delx
      logical *4 eqsign
      common /af77lib2/x,x1,x2,fx,fx1,fx2,t,delx
c
      if ( eqsign( fx, fx1 ) ) then
        x1 = x
        fx1 = fx
      else
        x2 = x
        fx2 = fx
        end if
      return
      end



      REAL FUNCTION QZERLI( A, B, F, TOL )
c
c       ZEROS BY LINEAR Interpolation
c   Function which uses linear interpolation to iterate toward the zero of the
c function F(X) . The function QZERLI requires 2 starting values  X = A and
c X = B . QZERLI is not guarranteed to converge to a zero of the function F(X)
c . Care should be taken in selecting the starting values  X = A and  X = B .
c Knowing the shape of F(X) can help with this initial selection .
c
c real a,b,             # starting values of x (1st values of search interval)
c      xa,xb,           # calculated values of search interval boundary
c      fa,fb,           # =f(xa) & =f(xb)
c      tol1,tol         # tolerance on the zero value of   F( QZERLI )
c
      implicit none
      integer Hvdulu
      parameter( Hvdulu=6 )
c
      real a, b, xa, xb, f, fa, fb, tol1, tol, denom
c
      tol1 = abs( tol )
      xa = a
      xb = b
      fa = f( xa )
      fb = f( xb )
      do while ( abs( fa ) .gt. tol1 )
        denom = fb / fa - 1.0
        if ( denom .eq. 0.0 ) then
          write( Hvdulu, 1 )
1         format( ' QZERLI : ABORT : Zero slope.' )
          return
          end if
        QZERLI = xa + ( xa - xb ) / denom
        xb = xa
        fb = fa
        xa = QZERLI
        fa = f( xa )
        end do
      return
      end



      DOUBLE PRECISION FUNCTION DZERLI( A, B, F, TOL )
c
c       ZEROS BY LINEAR Interpolation
c   Function which uses linear interpolation to iterate toward the zero of the
c function F(X) . The function DZERLI requires 2 starting values  X = A and
c X = B . DZERLI is not guarranteed to converge to a zero of the function F(X)
c . Care should be taken in selecting the starting values  X = A and  X = B .
c Knowing the shape of F(X) can help with this initial selection .
c
c double precision
c      a,b,             # starting values of x (1st values of search interval)
c      xa,xb,           # calculated values of search interval boundary
c      fa,fb,           # =f(xa) & =f(xb)
c      tol1,tol         # tolerance on the zero value of   F( DZERLI )
c
      include 'stdio.def'
      double precision a, b, xa, xb, f, fa, fb, tol1, tol, denom
c
      tol1 = abs( tol )
      xa = a
      xb = b
      fa = f( xa )
      fb = f( xb )
      DZERLI = xa
c     start of loop
1     if ( abs( fa ) .gt. tol1 ) then
          denom = fb / fa - 1.0
          if ( denom .eq. 0.0 ) then
              write( STD_OUT, 2 )
2             format( ' DZERLI : ABORT : Zero slope.' )
              return
              end if
          DZERLI = xa + ( xa - xb ) / denom
          xb = xa
          fb = fa
          xa = DZERLI
          fa = f( xa )
          goto 1
          end if
c         end of loop
      return
      end



      REAL FUNCTION QVAR( X, NINCR, NPTS )
c
c       VARiance of a sequence
c     Function to find the variance of NPTS numbers from the array X , taking
c  every NINCRth point .
c
c    Arguments
c real x(*),           # operand array
c integer *4 nincr     # array indexing increment
c            npts      # number of data points used
c
c    Variables
c double precision sx,sxx    # sums of x and x**2
c integer *4 i         # a working array index
c
      real x(*)
      double precision sx, sxx
      integer *4 nincr, npts, i
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      sx  = 0.0d0
      sxx = 0.0d0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = sx + x(i)
        sxx = sxx + x(i) **2
        end do
      qvar = sngl( sxx - sx **2 / dble( npts ) ) / float( npts - 1 )
      return
      end



      SUBROUTINE AMOEBA( FUNC, FUNC_VAL, N,
     1                   POLYHED, POLY_ROWS,
     2                   MAX_ITERATIONS, N_ITERATIONS,
     3                   TYPE_TOL, FUNC_TOL, X_TOL,
     4                   WORK )
c
c     Multidimensional minimization by the downhill
c     flexible polyhedral search method of Nelder and Mead.
c
c Nelder J.A. \& Mead R. (1964) :
c    Computer Journal, Vol. 7, p 308.
c
c This method uses only function evaluations (not derivative evaluations) .
c It does not use a one-dimensional minimization algorithm . In terms of the
c number of function evaluations required, it is not the most efficient
c algorithm available -- e.g. Powell's method is almost always faster, but
c it is relatively robust .
c
c The method minimizes a function of N independent variables by using function
c evaluations at the (N+1) vertices of a flexible polyhedron in N-dimensional
c space . At each stage of the iteration process, the vertex with the highest
c value of the function is projected through the centroid of the remaining
c vertices . Most of these projections are simple reflections so that the
c volume (hence nondegeneracy) of the polyhedron is preserved. Howevever, in
c certain cases, the shape of the polyhedron can be changed to fit the
c topography of the objective function,
c  a) elongating down long inclined planes,
c     -- reflection and expansion away from the high point if the reflected
c        point is lower than the previous lowest point .
c  b) changing direction in curving valleys
c     -- contraction along one dimension from the high point if the reflected
c        point is lower than the previous highest point but higher than the
c        previous second-highest point .
c  c) contracting in the neighbourhood of a minimum
c     -- contraction in all dimensions towards a low point if the reflected
c        point is lower than the previous highest point .
c The name of the subroutine AMOEBA describes the behaviour of the polyhedron .
c
c
c         DESCRIPTION OF ARGUMENTS
C         ************************
c
c The initial conditions are specified as the vertices POLYHED of an
c N-dimensional polyhedron .  POLYHED is an array of exactly POLY_ROWS
c rows and at least N+1 columns . POLY_ROWS is not smaller than N .
c The first N values in each of the first N+1 columns represents one of
c the N+1 vertices of the polyhedron . The minimum number of dimensions
c is N = 2 .
c
c Multidimensional minimization of the function FUNC(x) where x is a vector of
c N values, by the downhill simples method of Nelder and Mead. FUNC_VALUES
c contains the function values at the vertices of the polyhedron POLYHED .
c
c The maximum permitted number of iterations is MAX_ITERATIONS . The actual
c number of iterations carried out is N_ITERATIONS . A returned value of
c N_ITERATIONS=0 indicates failure due to incorrect arguments .
c
c The stopping criterion is selected by the value of TYPE_TOL :
c   POLY_SIZE   : absolute maximum length of an edges of the polyhedron is
c                 less than X_TOL .
c   ABS_FUNC    : absolute difference between highest and lowest function
c                 values at the vertices is less than FUNC_TOL .
c   REL_FUNC    : relative difference between highest and lowest function
c                 values at the vertices is less than FUNC_TOL .
c   POLY_SIZE + ABS_FUNC
c   POLY_SIZE + REL_FUNC
c The values of the constants POLY_SIZE, ABS_FUNC and REL_FUNC are
c defined in the include file AMOEBA.DEF
c
c
c Arguments
c  real    func                 # INPUT  # objective function -- declared as
c                                        #   EXTERNAL REAL FUNCTION
c          func_val(*)          # OUTPUT # function values at the vertices
c  integer n,                   # INPUT  # dimension of the problem
c  real    polyhed(poly_rows,*) # IN/OUT # initial and final polyhedron vertices
c  integer poly_rows            # INPUT  # declared number of rows in POLYHED
c          max_iterations       # INPUT  # maximum allowed number of iterations
c          n_iterations         # OUTPUT # actual number of iterations
c          type_tol             # INPUT  # specifies the stopping criterion
c  real    func_tol             # INPUT  # objective function tolerance
c          x_tol                # INPUT  # polyhedron size tolerance
c          work(*)              # INPUT  # work array of size 4*N
c
c
c Constants
c   DIMENS_LIMIT = 100      # maximum number of dimensions
c   ITERAT_LIMIT = 100000   # maximum number of iterations
c   MAX_REAL     = 1.0e37   # maximum real value
c   ALPHA = 1.0             # reflection coefficient
c   BETA  = 0.5             # contraction coefficient
c   GAMMA = 2.0             # expansion coefficient
c   DELTA = 0.5             # reduction coefficient
c
c ALPHA = 1.0 ensures that the size of the polyhedron remains unchanged until
c called for by a change in the topography . Fewer function evaluations are
c required than if ALPHA < 1 . Small ALPHA produces a more flexible polyhedron
c and allows more rapid reduction in size near a local minimum . With
c 0 < BETA < 0.4, flattening of the polyhedron may cause premature termination,
c and BETA > 0.6 increases the required number of interations .
c
c Pseudo constants
c   These are variables set once at the beginning to provide indexing
c   into the work array .
c integer
c   FACE_CENT       # centroid of the face opposite the highest vertex
c   CONT_EXP        # coordinate of the contracted or expanded point
c   POLY_CENT       # centroid of the all polyhedron vertices
c   REFLECTED       # coordinate of the initial reflected point
c
c Variables
c  logical          # stopping criterion is
c    f_abs_stop     #   absolute range of the objective function
c    f_rel_stop     #   relative range of the objective function
c    x_abs_stop     #   absolute maximum distance from a vertex to the centroid
c    f_stop         # value of objective function is OK ; stop
c    x_stop         # polyhedron is small enough ; stop
c  integer
c    lowest         # specifies which vertex has the lowest function value
c    highest        # specifies which vertex has the highest function value
c    sec_high       # specifies which vertex has second highest function value
c    i
c    j
c    n_vertices     # number of polyhedron vertices
c  real
c    abs_f_tol      # maximum absolute or relative function value for stopping
c    abs_x_tol      # square of maximum absolute or relative reduction in
c                   # distance from polyhedron centroid to a vertex for stopping
c    axis_len       # square of distance from a vertex to the centroid
c    con_exp_value  # value of objective function at contracted/expanded point
c    denom          # denominator of expressions for relative stopping criteria
c    max_axs_len    # maximum value of "axis_len"
c    prev_axs_len   # maximum value of "axis_len" in the previous iteration
c    refl_value     # value of objective function at the reflected point
c
      include 'stdio.def'
      include 'amoeba.def'
      integer DIMENS_LIMIT, ITERAT_LIMIT
      integer FACE_CENT, CONT_EXP, POLY_CENT, REFLECTED
      real    ALPHA, BETA, GAMMA, DELTA, MAX_REAL
      parameter( DIMENS_LIMIT=100, ITERAT_LIMIT=100000, MAX_REAL=1.0e37,
     1           ALPHA=1.0, BETA=0.5, GAMMA=2.0, DELTA=0.5 )
c
      logical f_abs_stop, f_rel_stop, x_abs_stop, f_stop, x_stop
      integer n, poly_rows, max_iterations, n_iterations, type_tol,
     1    lowest, highest, sec_high, i, j, n_vertices
      real func, func_val(*), polyhed(poly_rows,*), func_tol, x_tol,
     1    work(*), abs_f_tol, abs_x_tol, axis_len, con_exp_value,
     2    denom, max_axs_len, prev_axs_len, refl_value
c
c
c
      n_vertices = n + 1
      n_iterations = 0
      prev_axs_len = MAX_REAL
      max_axs_len  = MAX_REAL
c
c           check arguments
c           ***************
c

      if ( n .le. 1 .or. n .gt. DIMENS_LIMIT ) then                      1b
          write( STD_ERR, 1 )
1         format( ' AMOEBA : ABORT : ' )
          write( STD_ERR, 2 ) n
2         format( 11x, 'Illegal no. of dimensions ', i12 )
          return
      else if ( poly_rows .lt. n ) then                                  1m
          write( STD_ERR, 1 )
          write( STD_ERR, 3 ) poly_rows
3         format( 11x, 'Polyhedron size', i12, ' too small' )
          return
      else if ( max_iterations .le. 0 .or.
     1          max_iterations .gt. ITERAT_LIMIT ) then                  1m
          write( STD_ERR, 1 )
          write( STD_ERR, 4 ) max_iterations
4         format( 11x, 'Illegal iterations limit ', i12 )
          return
      else if ( type_tol .ne. POLY_SIZE .and.
     1        type_tol .ne. ABS_FUNC .and.
     2        type_tol .ne. REL_FUNC .and.
     3        type_tol .ne. POLY_SIZE + ABS_FUNC .and.
     4        type_tol .ne. POLY_SIZE + REL_FUNC ) then                  1m
          write( STD_ERR, 1 )
          write( STD_ERR, 5 ) type_tol
5         format( 11x, 'Illegal tolerance type ', i12 )
          return
      else if ( n .gt. DIMENS_LIMIT / 5 ) then                           1m
          write( STD_ERR, 6 ) n
6         format( ' AMOEBA : WARNING : Large no. of dimensions ', i12 )
          end if                                                         1e
c
c           Provide the correct indexing into the work array
c
      FACE_CENT = 0
      CONT_EXP  = n
      POLY_CENT = 2 * n
      REFLECTED = 3 * n
c
c           interpret the stopping criterion type
c           *************************************
c
      x_abs_stop = mod( type_tol / POLY_SIZE, 2 ) .gt. 0
      f_abs_stop = mod( type_tol / ABS_FUNC,  2 ) .gt. 0
      f_rel_stop = mod( type_tol / REL_FUNC,  2 ) .gt. 0
c
      abs_f_tol = abs( func_tol )
      abs_x_tol = x_tol ** 2
c
c           Evaluate the function for the initial polyhedron
c           ************************************************
c
      do 7 i = 1, n_vertices                                             1b
          func_val(i) = func( polyhed(1,i) )
7         continue                                                       1e
c
c     start of main loop
10    continue                                                           1b
c
c           Determine which vertex of the polyhedron has the highest
c           (worst), second highest and lowest (best) function values,
c           by looking at every vertex of the polyhedron
c           ********************************************************
c
          lowest = 1
          if ( func_val(1) .gt. func_val(2) ) then                       2b
              highest = 1
              sec_high = 2
          else                                                           2m
              highest = 2
              sec_high = 1
              end if                                                     2e
          do 11 i = 1, n_vertices
              if ( func_val(i) .lt. func_val(lowest) )
     1            lowest = i
              if ( func_val(i) .gt. func_val(highest) ) then             3b
                  sec_high = highest
                  highest = i
              else if ( func_val(i) .gt. func_val(sec_high) .and.        3m
     1                  i .ne. highest ) then
                  sec_high = i
                  end if                                                 3e
11            continue                                                   2e
c
c           Begin a new iteration . Compute the vector sum of all
c           points except the highest -- i.e. the centroid * N of
c           the face opposite the highest vertex .
c           *****************************************************

          do 12 j = 1, n                                                 2b
              work( FACE_CENT + j ) = 0.0
12            continue                                                   2e
          do 14 i = 1, n_vertices                                        2b
              if ( i .ne. highest ) then                                 3b
                  do 13 j = 1, n                                         4b
                      work( FACE_CENT + j ) = work( FACE_CENT + j )
     1                    + polyhed(j,i)
13                    continue                                           4e
                  end if                                                 3e
14            continue                                                   2e
c
c           Finish calculating the opposite face centroid .
c           Calculate the centroid of the whole polyhedron
c           Reflect the high point through the opposite face,
c               extrapolating by a factor ALPHA .
c           ************************************************
c
          do 15 j = 1, n                                                 2b
              work( POLY_CENT + j ) = ( work( FACE_CENT + j ) +
     1            polyhed(j,highest) ) / real( n_vertices )
              work( FACE_CENT + j ) = work( FACE_CENT + j ) / real( n )
              work( REFLECTED + j ) = ( 1.0 + ALPHA ) *
     1            work( FACE_CENT + j ) - ALPHA * polyhed(j,highest)
15            continue                                                   2e
c
c         calculate the square of the maximum distance from the centroid
c         of the polyhedron to a vertex -- used by stopping criterion
c         **************************************************************
c
          prev_axs_len = max_axs_len
          max_axs_len = 0.0
          do 17 i = 1, n_vertices                                        2b
              axis_len = 0
              do 16 j = 1, n                                             3b
                  axis_len = axis_len +
     1                ( polyhed(j,i) - work( POLY_CENT + j ) ) **2
16                continue                                               3e
              max_axs_len = max( max_axs_len, axis_len )
17            continue                                                   2e
c
c           Apply the stopping criterion
c           ****************************
c
          if ( x_abs_stop ) then                                         2b
              x_stop = max_axs_len .lt. abs_x_tol
          else                                                           2m
              x_stop = .TRUE.
              end if                                                     2e
          if ( f_abs_stop ) then                                         2b
              f_stop = abs( func_val(highest) - func_val(lowest) )
     1                .lt. abs_f_tol
          else if ( f_rel_stop ) then                                    2m
              denom = abs(func_val(highest)) + abs(func_val(lowest))
              if ( denom .le. 1.0 / MAX_REAL ) then                      3b
                  f_stop = .TRUE.
              else                                                       3m
                  f_stop = 2.0 * abs( func_val(highest) -
     1                func_val(lowest) ) / denom .lt. abs_f_tol
                  end if                                                 3e
          else                                                           2m
              f_stop = .TRUE.
              end if                                                     2e
          if ( x_stop .and. f_stop )
     1        return
c
          if ( n_iterations .ge. max_iterations ) then                   2b
              write( STD_ERR, 1 )
              write( STD_ERR, 18 ) n_iterations
18            format( 'Maximum number of iterations', i12,
     1            ' reached' )
              return
              end if                                                     2e
          n_iterations = n_iterations + 1
c
c           Evaluate the function at the reflected point
c
          refl_value = func( work( REFLECTED + 1 ) )
c
c           EXPANSION
c           *********
c           If the new point is better than the previous best, try
c           an additional extrapolation by a factor GAMMA .
c
          if ( refl_value .le. func_val(lowest) ) then                   2b
              do 20 j = 1, n                                             3b
                  work( CONT_EXP + j ) = GAMMA * work( REFLECTED + j ) +
     1                          ( 1.0 - GAMMA ) * work( FACE_CENT + j )
20                continue                                               3e
              con_exp_value = func( work( CONT_EXP + 1 ) )
c
c           If the expanded reflection is even lower, it replaces
c           the high point . If it is not, the original reflected
c           point replaces high point.
c
              if( con_exp_value .lt. func_val(lowest) ) then             3b
                  do 21 j = 1, n                                         4b
                      polyhed(j,highest) = work( CONT_EXP + j )
21                    continue                                           4e
                  func_val(highest) = con_exp_value
              else                                                       3m
                  do 22 j = 1, n                                         4b
                      polyhed(j,highest) = work( REFLECTED + j )
22                    continue                                           4e
                  func_val(highest) = refl_value
                  endif                                                  3e
c
c           CONTRACTION
c           ***********
c           If the reflected point is worse than the second-highest and
c           it is better than the highest, the reflected point replaces
c           the highest point . Of course, if it is worse, then it is
c           better not to do the replacement .
c
          else if ( refl_value .ge. func_val(sec_high) ) then            2m
              if ( refl_value .lt. func_val(highest) ) then              3b
                  do 30 j = 1, n                                         4b
                      polyhed(j,highest) = work( REFLECTED + j )
30                    continue                                           4e
                  func_val(highest) = refl_value
                  end if                                                 3e
c
c           Contract from the highest point toward the opposite face
c
              do 31 j = 1, n                                             3b
                  work( CONT_EXP + j ) = BETA * polyhed(j,highest) +
     1                          ( 1.0 - BETA ) * work( FACE_CENT + j )
31                continue                                               3e
              con_exp_value = func( work( CONT_EXP + 1 ) )
c
c           If the contraction is an improvement, use it to replace the
c           highest point .
c
              if ( con_exp_value .lt. func_val(highest) ) then           3b
                  do 32 j = 1, n                                         4b
                      polyhed(j,highest) = work( CONT_EXP + j )
32                    continue                                           4e
                  func_val(highest) = con_exp_value
c
c           REDUCTION
c           *********
c           Reflection and contraction do not lead to improvement, so
c           contract around the lowest point .
c
              else                                                       3m
                  do 34 i = 1, n_vertices                                4b
                      if ( i .ne. lowest ) then                          5b
                          do 33 j = 1, n                                 6b
                              polyhed(j,i) = DELTA *
     1                            ( polyhed(j,i) + polyhed(j,lowest) )
33                            continue                                   6e
                          func_val(i) = func( polyhed(1,i) )
                          end if                                         5e
34                    continue                                           4e
                  end if                                                 3e
c
c           If the original reflection is neither better than all
c           vertices, nor worse than all but the highest, just
c           replace the highest point with the reflected point .
c
          else                                                           2m
              do 35 j = 1, n                                             3b
                  polyhed(j,highest) = work( REFLECTED + j )
35                continue                                               3e
              func_val(highest) = refl_value
              end if                                                     2e
          goto 10
c         end of main loop                                               1b
      end



      SUBROUTINE SCUBIC( Z1, Z2, Z3, A, B, C, D, STATUS, TOL )
c
c Subroutine to find all the zeros of the cubic equation
c    A.z**3 + B.z**2 + C.z + D = 0
c The parameters are :
c complex z1,    # real zero of the cubic equation
c         z2,z3  # other zeros of the cubic, possibly complex conjugates
c real    A,     # coefficient of z cubed
c         B,     # coefficient of z squared
c         C,     # coefficient of z
c         D,     # constant coefficient
c         tol    # tolerance on zero detector for discriminator
c integer status # =1 , one real root & a pair of complex conjugate roots
c                # =0 , all roots real ,at least 2 of which are equal
c                # =-1, all roots are real
c The other variables are :
c double precision q,r,di,s1,s2,sum,diff,t,a0,a1,a2
c
c    We need some way of determining when the 1st coefficient A has a
c negligable effect on the zeros of the polynomial apart from introducing
c an extremely large valued zero .
c
      complex z1, z2, z3
      real a, b, c, d, tol
      double precision q,r,di,s1,s2,sum,diff,t,a0,a1,a2
      integer *4 status
c
      a0 = dble(d)/dble(a)
      a1 = dble(c)/dble(a)
      a2 = dble(b)/dble(a)
      t = dble(abs(tol))
      q = a1/3.0-a2**2/9.0
      r = a1*a2/6.0-a0/2.0-a2**3/27.0
      di = q**3+r**2
      if ( di .gt. t ) then
        di = dsqrt(di)
        s1 = (dabs(r+di))**0.333333333333333333d0
        if ( r+di .lt. 0.0d0 ) s1 = -s1
        s2 = (dabs(r-di))**0.333333333333333333d0
        if ( r-di .lt. 0.0d0 ) s2 = -s2
        sum = (s1+s2)/-2.0-a2/3.0
        diff = dsqrt(0.75d0)*(s1-s2)
        z2 = cmplx(sngl(sum),sngl(diff))
        z3 = conjg(z2)
        status = 1
      else if ( di .lt. -t ) then
        di = dsqrt(-di)
        s2 = datan2(di,r)/3.0d0
        sum = (r**2+di**2)**0.16666666666666667d0
        s1 = sum*dcos(s2)
        s2 = sum*dsin(s2)
        sum = -s1-a2/3.0
        diff = -dsqrt(3.0d0)*s2
        z2 = cmplx(sngl(sum+diff),0.0)
        z3 = cmplx(sngl(sum-diff),0.0)
        status = -1
      else
        s1 = dabs(r)**0.333333333333333333d0
        if ( r .lt. 0.0d0 ) s1 = -s1
        sum = -s1-a2/3.0
        z2 = cmplx(sngl(sum),0.0)
        z3 = z2
        status = 0
        end if
      z1 = cmplx(sngl(-2.0d0*sum-a2),0.0)
      return
      end


      SUBROUTINE SGJRED( A, M, N, K, TOL , NORMZP )
c
c       Gauss - Jordan REDuction of a matrix
c    Subroutine to use the Gauss - Jordan reduction method to reduce a matrix
c to reduced row echelon form . The matrix A has M rows and N columns .
c K columns are actively reduced ; ie. column K is the last one to be reduced.
c A pivot with absolute value less than the absolute value of TOL is taken as
c being zero .
c    SGJRED can be used for :-
c       1) solution of M simultaneous linear equations in N-1 unknowns .
c           NORMZP must be .true. if the solution is to be read read directly
c           from the constant coefficients column . The operand matrix for the
c           system of equations C.X=B is   A=[C|B], and the result is [I|X] .
c       2) inversion of MxM matrix   ( N = 2*N ) . NORMZP must be .true. if a
c           true inversion is to be obtained . The operand matrix is  A=[C|I]
c           and the result is  [I|inv(C)] . I is the MxM identity matrix .
c       3) evaluating determinant of MxM matrix ( M = N ) . NORMZP must be
c           .false. if a determinant of the MxM matrix C is required . The
c           determinant is the product of the leading diagonal elements of the
c           reduced matrix . A=C is the operand matrix and the result is
c           det(C) = a(1,1).a(2,2).a(3,3). ... .a(m,m)   .
c
c    Arguments
c real  a(m,n),         # operand matrix
c       tol             # zero detection tolerance
c integer *4 m,n,       # number of rows and number of columns in matrix A
c            k          # number of columns to be reduced
c logical *4 normzp     # .t. if the pivot row is normalized before reduction
c                       # .f. if pivot row is not normalized before reduction
c    Variable
c real  b,c             # temporary variables
c integer prowa,prowb,  # pivot row number after and before row swapping
c         pcol,         # pivot column number
c         i,j           # working row and column indices
c logical pexist,       # .true. if a pivot can be found in column pcol
c
      implicit none
      integer *4 m, n, k, prowa, prowb, pcol, i, j, k1
      real a(m,*), tol, tol1, b, c
      logical *4 normzp, pexist
c
c       check for valid parameters
c
      tol1 = abs( tol )
      k1 = min0( n, abs( k ) )
c
      pcol = 0
      do prowa = 1, m
c
c       repeat
1         if ( pcol .ge. k1 ) return        ! test if reduction is finished
          pcol = pcol + 1                   ! increment pivot column
          prowb = prowa                     ! initialize pivot column
          pexist = .false.
          do j = prowa, m
c
c           find the best pivot : test for non zero
c           & select largest value in the pivot column
c
            pexist = pexist .or. abs( a(j,pcol) ) .gt. tol1
            if ( abs( a(prowb,pcol) ) .lt. abs( a(j,pcol) ) )
     1        prowb = j
            end do
          if ( .not. pexist ) goto 1
c       until( pexist )
c
c           normalize the pivot row
c
        if ( normzp ) then
          c = a(prowb,pcol)
          do j = pcol, n
            a(prowb,j) = a(prowb,j)/c
            end do
          end if
c
c           swap pivot row into row prowa
c
        if ( prowb .ne. prowa ) then
          do j = pcol,n
            c = a(prowa,j)
            a(prowa,j) = a(prowb,j)
            a(prowb,j) = -c
            end do
          end if
        b = a(prowa,pcol)
        do i = 1, m
c
c           avoid reducing along pivot row
c
          if ( i .ne. prowa ) then
            c = a(i,pcol) / b
            do j = pcol, n                          ! reduce along a row
              a(i,j) = a(i,j) - a(prowa,j)*c
              end do
            end if
          end do
        end do
      return
      end


      SUBROUTINE SGJREDZ( A, M, N, K, TOL , NORMZP )
c
c       Gauss - Jordan REDuction of a COMPLEX matrix
c    Subroutine to use the Gauss - Jordan reduction method to reduce a matrix
c to reduced row echelon form . The matrix A has M rows and N columns .
c K columns are actively reduced ; ie. column K is the last one to be reduced.
c A pivot with absolute value less than the absolute value of TOL is taken as
c being zero .
c    SGJREDZ can be used for :-
c       1) solution of M simultaneous linear equations in N-1 unknowns .
c           NORMZP must be .true. if the solution is to be read read directly
c           from the constant coefficients column . The operand matrix for the
c           system of equations C.X=B is   A=[C|B], and the result is [I|X] .
c       2) inversion of MxM matrix   ( N = 2*N ) . NORMZP must be .true. if a
c           true inversion is to be obtained . The operand matrix is  A=[C|I]
c           and the result is  [I|inv(C)] . I is the MxM identity matrix .
c       3) evaluating determinant of MxM matrix ( M = N ) . NORMZP must be
c           .false. if a determinant of the MxM matrix C is required . The
c           determinant is the product of the leading diagonal elements of the
c           reduced matrix . A=C is the operand matrix and the result is
c           det(C) = a(1,1).a(2,2).a(3,3). ... .a(m,m)   .
c
c    Arguments
c complex  a(m,n),         # operand matrix
c real  tol             # zero detection tolerance
c integer *4 m,n,       # number of rows and number of columns in matrix A
c            k          # number of columns to be reduced
c logical *4 normzp     # .t. if the pivot row is normalized before reduction
c                       # .f. if pivot row is not normalized before reduction
c    Variable
c complex  b,c          # temporary variables
c integer prowa,prowb,  # pivot row number after and before row swapping
c         pcol,         # pivot column number
c         i,j           # working row and column indices
c logical pexist,       # .true. if a pivot can be found in column pcol
c
      implicit none
      integer *4 m, n, k, prowa, prowb, pcol, i, j, k1
      complex  a(m,*), b, c
      real tol, tol1
      logical *4 normzp, pexist
c
c       check for valid parameters
c
      tol1 = abs( tol )
      k1 = min0( n, abs( k ) )
c
      pcol = 0
      do prowa = 1, m
c
c       repeat
1         if ( pcol .ge. k1 ) return        ! test if reduction is finished
          pcol = pcol + 1                   ! increment pivot column
          prowb = prowa                     ! initialize pivot column
          pexist = .false.
          do j = prowa, m
c
c           find the best pivot : test for non zero
c           & select largest value in the pivot column
c
            pexist = pexist .or. abs( a(j,pcol) ) .gt. tol1
            if ( abs( a(prowb,pcol) ) .lt. abs( a(j,pcol) ) )
     1        prowb = j
            end do
          if ( .not. pexist ) goto 1
c       until( pexist )
c
c           normalize the pivot row
c
        if ( normzp ) then
          c = a(prowb,pcol)
          do j = pcol, n
            a(prowb,j) = a(prowb,j)/c
            end do
          end if
c
c           swap pivot row into row prowa
c
        if ( prowb .ne. prowa ) then
          do j = pcol,n
            c = a(prowa,j)
            a(prowa,j) = a(prowb,j)
            a(prowb,j) = - c
            end do
          end if
        b = a(prowa,pcol)
        do i = 1, m
c
c           avoid reducing along pivot row
c
          if ( i .ne. prowa ) then
            c = a(i,pcol) / b
            do j = pcol, n                          ! reduce along a row
              a(i,j) = a(i,j) - a(prowa,j)*c
              end do
            end if
          end do
        end do
      return
      end


      SUBROUTINE SLRREG( X, Y, NINCR, NPTS, A, B, CORR, MSERR )
c
c       LiNear REGression
c     Subroutine to do a linear regression ( least mean squares ) fit to NPTS
c  in the data arrays X & Y . Every NINCRth value is used . The line of best
c  fit is  y = A.x + B with correlation coefficient CORR and mean square error
c  MSERR .
c
c    Arguments                 input                           output
c real x(*),y(*),      # operand arrays
c      a,                                      # slope of line of best fit
c      b,                                      # y axis intercept of line
c      corr,                                   # correlation coefficient
c      mserr,                                   # mean square error
c integer *4 nincr,    # data array indexing increment
c            npts      # number of data points used
c
c    Variables
c real meanx,meany,             # mean of x and of y
c      varx,vary,               # variance of x and of y
c      covar                   # covariance of x & y
c
      implicit none
      real x(*), y(*), a, b, corr, mserr, meanx, meany, varx, vary,
     1  covar
      integer nincr, npts, i
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      call SMEANS( x, y,  nincr, npts, meanx, meany, varx, vary, covar )
      if ( varx .ne. 0.0 ) then
        a = covar / varx
        b = meany - a * meanx
      else
        a = -9.999e29
        b = -9.999e29
        end if
      corr = varx * vary
      if ( corr .ne. 0.0 ) then
        corr = covar / sqrt( corr )
      else
        corr = -9.999e29
        end if
      mserr = 0.0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        mserr = mserr + ( y(i) - a * x(i) - b ) **2
        end do
      mserr = mserr / npts
      return
      end



      SUBROUTINE SMEANG( X, NINCR, NPTS, M, K )
c
c   Calculates the 1st ( mean ) to kth moments of NPTS samples in the array X,
c  taking every NINCRth value. All but the 1st moment ( mean ) are centred
c   about the sample mean . The maximum and minimum values of K are 14 and 1 .
c  N.B. There are two one passes of the data
c
c    Arguments
c real x(*),            # data array
c      m(*),            # array of moment results
c integer *4 nincr,     # data array indexing increment
c            npts,      # number of data values used
c            k          # maximum moment required
c
c    Variables
c real r, rk            # temporay products
c      qmean            # function : calculates mean
c integer i,j           # looping indices
c double precision total(2:14)
c
      real x(*), m(*), r, rk, qmean
      double precision total(2:14)
      integer nincr, npts, j, k
c
c               Check parameters
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      if ( k .lt. 1 .or. k .gt. 14 ) return
c
c               Calculate mean
c
      m(1) = qmean( x, nincr, npts )
c
c               Calculate sums of (x-mean)**i : i=1..k
c
      if ( k .gt. 1 ) then
        do i = 2, k
          total(i) = 0.0
          end do
        do i = 1, ( npts - 1 ) * nincr + 1, nincr
          r = ( x(i) - m(1) )
          rk = r
          do j = 2, k
            rk = rk * r
            total(j) = total(j) + rk
            end do
          end do
c
c               Calculate moments
c
        do j = 2, k
          m(j) = sngl( total(j) ) / float( npts )
          end do
        m(2) = m(2) * float( npts ) / float( npts - 1 )
        end if
      return
      end



      SUBROUTINE SMEANM( X, NINCR, NPTS, M, K )
c
c   Calculates the 1st ( mean ) to kth moments of NPTS samples in the array X,
c  taking every NINCRth value. All but the 1st moment ( mean ) are centred
c   about the sample mean . The maximum and minimum values of K are 14 and 1 .
c  N.B. There is only one pass of the data
c       This method is prone to rounding error if the mean value is larger
c       than the standard deviation . In such a case you should use SMEANG
c
c             k     k  /  k     i         k-i  \
c  E( (X-E(X)) ) = Sum | ( ).E(X ).(-E(X))     |
c                  i=0 \  i                    /
c    Arguments
c real x(*),            # data array
c      m(*),            # array of moment results
c integer *4 nincr,     # data array indexing increment
c            npts,      # number of data values used
c            k          # maximum moment required
c
c    Variables
c real r                # temporay product
c integer i,j           # looping indices
c
      real x(*), m(*), r
      integer nincr, npts, j, k
c
c               Check parameters
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      if ( k .lt. 1 .or. k .gt. 14 ) return
c
c               Initialize sums
c
      do j = 1, k
        m(j) = 0.0
        end do
c
c               Evaluate sums of x,x**2,.. x**k
c
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        r = 1.0
        do j = 1, k
          r = r * x(i)
          m(j) = m(j) + r
          end do
        end do
c                  i
c               E(X ) : i=1,2..k
c
      do j = 1, k
        m(j) = m(j) / npts
        end do
c                          i
c               E( (X-E(X)) ) : i=1,2..k
c
      if ( k .gt. 1 ) then
        do i = k, 2, -1
          do j = i-1, 1, -1
            m(i) = m(i) + float(Jcombn(i,j)) * (-m(1))**(i-j) * m(j)
            end do
          m(i) = m(i) + (-m(1))**i
          end do
        end if
      return
      end



      SUBROUTINE SMEANI( MDATA, NINCR, NPTS, MEAN, VAR )
c
c       real MEAN of Integers
c   Calculates the mean and variance of an array of integers .
c
c    Arguments
c  integer *2 Mdata(N)                 # data array
c  integer *4 nincr,                   # data array indexing increment
c             npts                     # number of data points used
c  real Mean                           # mean of data
c       Var                            # variance of data
c variables
c  integer *4 i                        # looping index
c  double precision Sx                 # agregate of data
c                   dmean              # mean of data
c
      integer *4 nincr, npts, i
      integer *2 Mdata(*)
      real Mean, Var
      double precision Sx
c
      sx = 0.0d0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = sx + dble( Mdata(i) )
        end do
      dmean = sx / dble( npts )
      mean = sngl( dmean )
      sx = 0.0d0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = sx + ( dble( Mdata(i) ) - dmean ) **2
        end do
      var = sngl( sx ) / float( npts )
      return
      end



      SUBROUTINE SMEANS( X, Y, NINCR, NPTS, MEANX, MEANY, VARX, VARY,
     1  COVAR )
c
c    Subroutine to find the means , variances and covariance of two sequences
c  X and Y of N real numbers .
c
c    Arguments
c
c real covar,                  # covariance of x & y
c      meanx,meany,            # means of x & y
c      varx,vary,              # variances of x & y
c      x(*),y(*)               # operand sequences
c integer nincr,               # data array indexing increment
c         npts,                # number of data points used
c
c    Variables
c real sx,sy,sxx,syy,sxy,      # sums of x, y, x**2, y**2, x*y
c integer i                    # working array index
c
      implicit none
      integer *4 nincr, npts, i
      real x(*), y(*), meanx, meany, varx, vary, covar, sx, sy,
     1  sxx, syy, sxy
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
c
c           calculate means
c
      sx = 0.0
      sy = 0.0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = sx + x(i)
        sy = sy + y(i)
        end do
      meanx = sx / npts
      meany = sy / npts
c
c           calculate variances & covariances
c
      sxx = 0.0
      syy = 0.0
      sxy = 0.0
c
c   evaluate sums
c
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        sx = x(i) - meanx
        sy = y(i) - meany
        sxx = sxx + sx **2
        syy = syy + sy **2
        sxy = sxy + sx * sy
        end do
c
c    calculate means
c
      varx = sxx / npts
      vary = syy / npts
      covar = sxy / npts
      return
      end



      SUBROUTINE SORTSH( X, M1, M2, N, J1, M )
c
c               SORT ( SHell )
c       Subroutine for a shell sort into ascending or descending order .  The
c sorted array is an N x M matrix of N sets of Mtuples ( ie. N points in M
c dimensional space ) . Only the J1th column of the matrix is sorted into
c ascending or descending order , with the elements of the other columns
c undergoing the same exchanges . Sorting occurs between rows M1 and M2
c inclusive . If  M1 < M2 , the sort is descending . If M1 > M2 , the sort is
c ascending . Eg. for   SORTSH( X, 1, 4, 5, 1, 4 ) ,
c               2 4 3 1                 1 2 4 3
c               4 3 1 2                 2 4 3 1
c           X = 3 1 2 4     becomes X = 3 1 2 4
c               1 2 4 3                 4 3 1 2
c               3 1 2 4                 3 1 2 4
c
c real  x(n,m)          # operand array
c integer n,m,          # number of rows and columns in array
c         m1,m2,        # starting and finishing row indices
c         i1,i2,i3,j,   # working indices
c         j1,           # column to be sorted
c         id,           # sort segment size
c         m11,m12       # starting row & -1
c
      implicit none
      integer *4 n, nn, m, m1, m2, i1, i2, i3, j, j1, id, m11, m12
      real x(n,m)
c
      if ( m1 .eq. m2 .or. m1 .lt. 1 .or. m1 .gt. n .or. m2 .lt. 1 .or.
     1  m2 .gt. n .or.j1 .lt. 1 .or. j1 .gt. m ) return
      m11 = min0( m1, m2 )
      m12 = m11 - 1
      nn = iabs( m2 - m1 ) + 1
      id = 2 ** int( alog( float( nn ) ) / alog( 2.0 ) + 0.0001 ) - 1
c     repeat
1       do i1 = 1, nn-id
          do i2 = i1+m12, m11, -id
            i3 = i2 + id
            if ( x(i2,j1) .le. x(i3,j1) ) goto 2
            do j = 1, m
              call SWAP4( x(i2,j), x(i3,j) )
              end do
            end do
2         end do
          id = id / 2
        if ( id .gt. 0 ) goto 1
c     until ( id <= 0 )
      if ( m1 .lt.  m2 ) then
        i1 = m1
        i2 = m2
c       repeat
3         do j = 1, m
            call SWAP4( x(i1,j), x(i2,j) )
            end do
          i1 = i1 + 1
          i2 = i2 - 1
          if ( i1 .lt. i2 ) goto 3
c       until ( i1 >= i2 )
        end if
      return
      end


      SUBROUTINE SORTSH2( X, M1, M2, N, J1, M, INCCOL )
c
c               SORT ( SHell )
c       Subroutine for a shell sort into ascending or descending order .  The
c sorted array is an N x M matrix of N sets of Mtuples ( ie. N points in M
c dimensional space ) . The J1th column of the matrix is sorted into ascending
c or descending order , and all columns that are a multiple of INCCOL either
c side of the J1th column  undergo the same changes , while all other columns
c remain unchanged . Sorting occurs between rows M1 and M2 inclusive .
c If  M1 < M2 , the sort is descending . If M1 > M2 , the sort is ascending.
c  Eg. for   SORTSH( X, 1, 4, 5, 1, 4 , 2 ) ,
c               2 4 3 1                 1 4 4 1
c               4 3 1 2                 2 3 3 2
c           X = 3 1 2 4     becomes X = 3 1 2 4
c               1 2 4 3                 4 2 1 3
c               3 1 2 4                 3 1 2 4
c
c real  x(n,m)          # operand array
c integer n,m,          # number of rows and columns in array
c         m1,m2,        # starting and finishing row indices
c         i1,i2,i3,j,   # working indices
c         j1,           # column to be sorted
c         id,           # sort segment size
c         m11,m12       # starting row & -1
c         fstcol        # index of first column to be swapped
c         lastcol       # index of last column to be swapped
c         inccol        # increment length for column index
c
      implicit none
      integer *4 n, nn, m, m1, m2, i1, i2, i3, j, j1, id, m11, m12,
     1  fstcol, lastcol, inccol
      real x(n,m)
c
      if ( m1 .eq. m2 .or. m1 .lt. 1 .or. m1 .gt. n .or. m2 .lt. 1 .or.
     1  m2 .gt. n .or.j1 .lt. 1 .or. j1 .gt. m .or. inccol .lt. 0 .or.
     2  inccol .ge. m ) return
      m11 = min0( m1, m2 )
      m12 = m11 - 1
      nn = iabs( m2 - m1 ) + 1
      id = 2 ** int( alog( float( nn ) ) / alog( 2.0 ) + 0.0001 ) - 1
      fstcol = mod( J1 - 1, inccol ) + 1
      lastcol = fstcol + ( ( m - 1 ) / inccol ) * inccol
c     repeat
1       do i1 = 1, nn-id
          do i2 = i1+m12, m11, -id
            i3 = i2 + id
            if ( x(i2,j1) .le. x(i3,j1) ) goto 2
            do j = fstcol, lastcol, inccol
              call SWAP4( x(i2,j), x(i3,j) )
              end do
            end do
2         end do
          id = id / 2
        if ( id .gt. 0 ) goto 1
c     until ( id <= 0 )
      if ( m1 .lt.  m2 ) then
        i1 = m1
        i2 = m2
c       repeat
3         do j = 1, m
            call SWAP4( x(i1,j), x(i2,j) )
            end do
          i1 = i1 + 1
          i2 = i2 - 1
          if ( i1 .lt. i2 ) goto 3
c       until ( i1 >= i2 )
        end if
      return
      end


      SUBROUTINE SPEAKS( X, Y, NINCR, NPTS, XPEAK, YPEAK, NPEAK,
     1  MAXPEAK, FMAX )
c
c     Selects the local extrema from the (x,y) coordinates in arrays X
c and Y . There are NPTS with indexing increment of NINCR . The extrema
c coordinates are put into the arrays XPEAK and YPEAK as they are found .
c
c Arguments
c  real X(*),Y(*)              # data coordinate arrays                 INPUT
c  integer *4 nincr,           # data indexing increment                INPUT
c             npts             # number of data points                  INPUT
c  real xpeak(*), ypeak(*)     # extrema coordinates                    OUTPUT
c  integer *4 npeak,           # number of extreme points found         OUTPUT
c             maxpeak          # maximum number of extreme points       INPUT
c  logical *4 fmax             # .true. if finding maxima               INPUT
c                              # .false. if finding minima
c
c Constants
c  Hvdulu                      # vdu logical unit number
c Variables
c  integer *4 nlast            # index of last data point
c             i                # looping index
c  logical *2 newpeak          # looking for new peak
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      real x(*), y(*), xpeak(*), ypeak(*)
      integer *4 npeak, nincr, npts, maxpeak, nlast, i
      logical *4 fmax
      logical *2 newpeak
c
c       check arguments
c
      npeak = 0
      if ( nincr .le. 0 .or. npts .le. 0 .or. maxpeak .le. 0 ) then       1b
        write( Hvdulu, 1 )
1       format( ' SPEAKS : ABORT : Silly Arguments.' )
        return
        end if                                                            1e
      nlast = ( npts - 1 ) * nincr + 1
      npeak = 1
      newpeak = .false.
c
c         is the first point extreme ?
c
      if ( y(2) .ne. y(1) .and. ( ( y(2) .gt. y(1) ) .xor. fmax ) ) then  1b
        xpeak(1) = x(1)
        ypeak(1) = y(1)
        npeak = npeak + 1
        end if
c
c         find internal extrema
c
      do i = 1 + nincr, nlast, nincr                                      1b
        if ( y(i) .ne. y(i-nincr) .and. ( y(i) .lt. y(i-nincr) .xor.
     1      fmax ) ) then                                                 2b
          xpeak(npeak) = x(i)
          ypeak(npeak) = y(i)
          newpeak = .true.
        else                                                              2m
          if ( newpeak ) then                                             3b
            if ( npeak .ge. maxpeak )
     1        return
            npeak = npeak + 1
            newpeak = .false.
            end if                                                        3e
          end if                                                          2e
        end do                                                            1e
c
c         is the last point extreme ?
c
      if ( y(nlast) .ne. y(nlast-nincr) .and.
     1    ( y(nlast) .lt. y(nlast-nincr) .xor. fmax ) ) then              1b
        xpeak(npeak) = x(nlast)
        ypeak(npeak) = y(nlast)
        npeak = npeak + 1
        end if                                                            1e
      npeak = npeak - 1
      return
      end


      SUBROUTINE SPYORTH( X, Y, WEIGHT, NPTS, POLY, ORDER, ERROR, WORK )
c
c     Calculates a polynomial of order `order' which approximates the weighted
c (x,y) data . The method uses the minimum mean squared fitting error to select
c a series of orthogonal polynomials . The orthogonal polynomials are then
c combined to produce the result polynomial `poly' .
c
c This method is chosen because
c  1) It is not ill-conditioned .
c  2) It uses a very small amount of `work' array space
c  3) Calculation of the error is trivial
c  4) Time is proportional to ORDER**2
c  5) It is possible to return the list of orthogonal polynomials . From
c     this list, all `best' polynomials of order up to 'order' can be
c     extracted with very little effort .
c
c Arguments
c  real x(npts),       # IN  # abscissa values
c       y(npts),       # IN  # ordinate values
c       weight(npts)   # IN  # weights for ordinates
c  integer npts        # IN  # number of (x,y,weight) data points
c  real poly(order+1)  # OUT # coefficients of the approximating polynomial
c  integer order       # IN  # order of the approximating polynomial
c  real error,         # OUT # sum square error of approximating polynomial
c  real *8
c       work(2,order+1)# IN  # working array (for coeffs of orthog polonomials)
c
c Local variables
c  integer
c   i                  # index over the data points
c   j,                 # index over 1 to order+1
c   k                  # index over 1 to j
c
c       N.B. It has been found that within this routine use of double
c            precision is necessary to avoid loss of accuracy through
c            round off error . However, there is no advantage to be
c            gained by supplying the data in double precision .
c  real *8
c   alpha, beta        # recursion coeffients
c   new_d, old_d       # norms (Pj,Pj) of jth & (j-1)th orgogonal polynomials
c   o_coeff            # coefficients defining the approximating polynomial
c                      # in terms of the orthogonal polynomials Pj(x)
c   temp               # temporary storage for most recent coeff of orthog poly
c   poly_x             # Pj(Xi)
c   w_poly, w_poly2    # Wi.Pj(Xi), Wi.Pj(Xi).Pj(Xi)
c   int_error          # min mean square error -- internal use only
c
      implicit none
      integer npts, order
      real x(*), y(*), weight(*), poly(*), error
      real *8 work(2,*)
c
      integer i, j, k
      real *8 alpha, beta, new_d, old_d, o_coeff, temp,
     1        poly_x, w_poly, w_poly2, int_error
c
c             initialize coefficients of the recursion relationship
c
      alpha = 0.0
      beta  = 0.0
c
c             initialize the orthogonal polynomials and
c             result polynomial to zero
c
      do 1 j = 1, order + 1
          work(1,j) = 0.0
          work(2,j) = 0.0
          poly(j) = 0.0
1         continue
c
c             calculate the initial error
c
      int_error = 0.0
      do 2 i = 1, npts
          int_error = int_error + dble( weight(i) ) * dble( y(i) ) **2
2         continue
c
c             start of outer loop -- to calculate the jth orthogonal polynomial
c             #################################################################
c
      old_d = 1.0
      do j = 1, order + 1
c
c             calculate the coefficients of the jth orthogonal polynomial
c             ***********************************************************
c             using the recursion relationship ( i.e. from the previous 2
c             orthogonal polynomials
c
          work(2,j)=1.0
          if ( j .gt. 1) then
              do k = 2, j
                  temp = work(1,k-1) - alpha * work(2,k-1) -
     1                   beta * work(1,k)
                  work(1,k) = work(2,k-1)
                  work(2,k-1) = temp
                  end do
              end if
c
c             calculate new values for recursion coefficients
c             ***********************************************
c
          new_d = 0.0
          alpha = 0.0
          o_coeff = 0.0
c
          do i = 1, npts
c
c             evaluate the jth orthogonal polynomial
c
              poly_x  = 0.0
              do k = j, 1, -1
                  poly_x = poly_x * dble( x(i) ) + work(2,k)
                  end do
c
c             calculate the various sums
c
              w_poly  = dble( weight(i) ) * poly_x
              w_poly2 = w_poly * poly_x
              new_d   = new_d + w_poly2
              alpha   = alpha + w_poly2 * dble( x(i) )
              o_coeff = o_coeff + w_poly * dble( y(i) )
              end do
c
          alpha = alpha / new_d
          beta  = new_d / old_d
          o_coeff = o_coeff / new_d
          int_error = int_error - o_coeff **2 * new_d
          old_d = new_d
c
c             add the effect of the jth orthogonal polynomial to the result
c             *************************************************************
c             polynomial
c
          do k = 1, j
              poly(k) = poly(k) + o_coeff * work(2,k)
              end do
          end do
c
c             end of outer loop
c             #################
c
      error = int_error
      return
      end



      SUBROUTINE SPYORTHS( X, Y, NPTS, POLY, ORDER, ERROR, WORK )
c
c     Calculates a polynomial of order `order' which approximates the weighted
c (x,y) data . The method uses the minimum mean squared fitting error to select
c a series of orthogonal polynomials . The orthogonal polynomials are then
c combined to produce the result polynomial `poly' .
c
c This method is chosen because
c  1) It is not ill-conditioned .
c  2) It uses a very small amount of `work' array space
c  3) Calculation of the error is trivial
c  4) Time is proportional to ORDER**2
c  5) It is possible to return the list of orthogonal polynomials . From
c     this list, all `best' polynomials of order up to 'order' can be
c     extracted with very little effort .
c
c Arguments
c  real x(npts),       # IN  # abscissa values
c       y(npts),       # IN  # ordinate values
c  integer npts        # IN  # number of (x,y,weight) data points
c  real poly(order+1)  # OUT # coefficients of the approximating polynomial
c  integer order       # IN  # order of the approximating polynomial
c  real error,         # OUT # mean square error of approximating polynomial
c  real *8
c       work(2,order+1)# IN  # working array (for coeffs of orthog polonomials)
c
c Local variables
c  integer
c   i                  # index over the data points
c   j,                 # index over 1 to order+1
c   k                  # index over 1 to j
c
c       N.B. It has been found that within this routine use of double
c            precision is necessary to avoid loss of accuracy through
c            round off error . However, there is no advantage to be
c            gained by supplying the data in double precision .
c  real *8
c   alpha, beta        # recursion coeffients
c   new_d, old_d       # norms (Pj,Pj) of jth & (j-1)th orgogonal polynomials
c   o_coeff            # coefficients defining the approximating polynomial
c                      # in terms of the orthogonal polynomials Pj(x)
c   temp               # temporary storage for most recent coeff of orthog poly
c   poly_x             # Pj(Xi)
c   poly2              # Pj(Xi).Pj(Xi)
c   int_error          # min mean square error -- internal use only
c
      include 'stdio.def'
      integer npts, order
      real x(*), y(*), poly(*), error
      double precision work(2,*)
c
      integer i, j, k
      double precision alpha, beta, new_d, old_d, o_coeff, temp,
     1        poly_x, poly2, int_error
c
c             initialize coefficients of the recursion relationship
c
      alpha = 0.0
      beta  = 0.0
c
c             initialize the orthogonal polynomials and
c             result polynomial to zero
c
      do 1 j = 1, order + 1
          work(1,j) = 0.0
          work(2,j) = 0.0
          poly(j) = 0.0
1         continue
c
c             calculate the initial error
c
      int_error = 0.0
      do 2 i = 1, npts
          int_error = int_error + dble( y(i) ) **2
2         continue
c
c             start of outer loop -- to calculate the jth orthogonal polynomial
c             #################################################################
c
      old_d = 1.0
      do 7 j = 1, order + 1
c
c             calculate the coefficients of the jth orthogonal polynomial
c             ***********************************************************
c             using the recursion relationship ( i.e. from the previous 2
c             orthogonal polynomials
c
          work(2,j)=1.0
          if ( j .gt. 1) then
              do 3 k = 2, j
                  temp = work(1,k-1) - alpha * work(2,k-1) -
     1                   beta * work(1,k)
                  work(1,k) = work(2,k-1)
                  work(2,k-1) = temp
3                 continue
              end if
c
c             calculate new values for recursion coefficients
c             ***********************************************
c
          new_d = 0.0
          alpha = 0.0
          o_coeff = 0.0
c
          do 5 i = 1, npts
c
c             evaluate the jth orthogonal polynomial
c
              poly_x  = 0.0
              do 4 k = j, 1, -1
                  poly_x = poly_x * dble( x(i) ) + work(2,k)
4                 continue
c
c             calculate the various sums
c
              poly2   = poly_x **2
              new_d   = new_d + poly2
              alpha   = alpha + poly2 * dble( x(i) )
              o_coeff = o_coeff + poly_x * dble( y(i) )
5             continue
c
          alpha = alpha / new_d
          beta  = new_d / old_d
          o_coeff = o_coeff / new_d
          int_error = int_error - o_coeff **2 * new_d
          old_d = new_d
c
c             add the effect of the jth orthogonal polynomial to the result
c             *************************************************************
c             polynomial
c
          do 6 k = 1, j
              poly(k) = poly(k) + o_coeff * work(2,k)
6             continue
7         continue
c
c             end of outer loop
c             #################
c
      error = int_error
      return
      end



      SUBROUTINE SPYREG( X, Y, NINCR, NPTS, K, CORR, MSERR, A )
c
c       PolYnomial REGression
c   Subroutine to do a polynomial regression ( least mean squared error ) fit
c to the X, Y data arrays of N real numbers , from the M1th to the M2th
c elements ( inclusive ) . The line of best fit is
c   y = a(1) + a(2).x + ... + a(k-2).x**(k-3) + a(k-1).x**(k-2)
c where a(1)...a(k-1) is the 1st column of the KxK array A .
c with correlation coefficient CORR and mean squared error MSERR .
c
c     This routine has an important fault in that it frequently tries to
c solve ill-conditioned systems of equations, and so gets the wrong answer .
c The routine SPYORTH avoids this problem by generating a sequence of
c orthogonal polynomials, and is therefore preferred . It is also quicker
c and uses less array space . ( Translation to C is a waste of effort . )
c
c real  x(*),y(*),      # operand arrays : (x,y) coordinates
c       a(k,k),         # working array ; 1st column is coeffs of polynomial
c       corr,           # correlation coefficient of the data and the fit
c       mserr,          # mean square error of the fit
c integer nincr,        # data array indexing increment
c         npts,         # number of data points to be used
c         k,            # order of polynomial +2,
c
c    Variable
c real  spp,syp,syy,    # sums of polynomial**2, y*polynomial & y**2
c       s,              # sums for setting up matrix
c       tol             # zero tolerance for Gauss-Jordan reduction
c integer j,j1,j2,      # index & index +1 & +2
c         k2,           # = k-2 = order of polynomial
c         i             # index
c
      implicit none
      integer *4 nincr, npts, k, j, j1, j2, k2, i
      real x(*), y(*), a(k,k), corr, mserr, spp, syp, syy, s, tol
c
c         clear bottom row of matrix
c
      if( npts .le. 0 .or. nincr .le. 0 .or. npts .lt. k-1
     1  .or. k .lt. 3 ) return
      do j = 1, k
        a(k,j) = 0.0
        end do
c
c       evaluate matrix coefficients
c
      a(1,1) = npts
      do j = 1, 2 * ( k - 2 )
        s = 0.0
        j2 = j + 2
        do i = 1, ( npts - 1 ) * nincr + 1, nincr
          s = s + x(i) ** j
          end do
        do  i = max0( j-k+3 , 1 ) , min0( j+1 , k-1 )
          a(i,j2-i) = s
          end do
        end do
c
c       evaluate constant coefficients
c
      s = 0.0
      do i = 1, ( npts - 1 ) * nincr + 1, nincr
        s = s + y(i)
        end do
      k2 = k - 2
      a(1,k) = s
      do j = 1, k2
        j1 = j + 1
        s = 0.0
        do i = 1, ( npts - 1 ) * nincr + 1, nincr
          s = s + y(i) * x(i) ** j
          end do
        a(j1,k) = s
        end do
c
c       Solve system of equations
c
      tol = sqrt( a(2,2) ) * 1.2e-7
      call sgjred( a, k, k, k-1, tol, .true. )
      do j = 1, k-1
        a(j,1) = a(j,k)
        end do
c
c       do sums for corr. coeff. & m.s. error
c
      mserr = 0.0
      syy = 0.0
      syp = 0.0
      spp = 0.0
      do j = 1, ( npts - 1 ) * nincr + 1, nincr
        s = a(1,1)
        corr = 1.0
        do i = 2, k-1
          corr = corr * x(j)
          s = s + a(i,1) * corr
          end do
        syy = syy + y(j) **2
        syp = syp + y(j) * s
        spp = spp + s **2
        mserr = mserr + ( s - y(j) ) **2
        end do
c
c       calculate corr. coeff. & m.s.error
c
      if ( syy .gt. 0.0 .and. spp .gt. 0.0 ) then
        corr = syp / sqrt( syy * spp )
      else
        corr = 1.0
        end if
      mserr = mserr / npts
      return
      end



      SUBROUTINE SQMIND( X, Y, AREA, NINCR, NPTS )
c
c       Quadratic Mean Integration of Discrete data array
c
c  This routine integrates the function defined by the NPTS coordinate pairs
c  ( X(i), Y(i), i = 1, NINCR+1, NINCR+2 ... ) using the first X value as the
c  lower limit of integration . The result placed in AREA is the result of
c  progressively integrating the function with each successive X value as the
c  upper limit of integration . The first value in AREA is taken as the initial
c  value of the definite integral .
c
c  The procedure is to fit a parabola to 3 adjacent points , and integrate the
c  the parabola according to its coefficients . The index of the data points
c  is then incremented and the process is repeated .
c
c Arguments
c  real x(*)           # abicissa data array in strictly ascending order
c       y(*)           # ordinate data array
c       area(*)        # result array
c  integer *4 nincr    # data array indexing increment
c             npts     # number of data points to be used
c
c Variables
c  integer *4 i1,i2,i3 # indices of 1st, 2nd & 3rd points of the quadratic fit
c  real a, b, c        # cubic coefficients of a.x**3+b.x**2+c.x
c       qqmpol         # function : polynomial evaluation for QQMIND
c
      implicit none
      real a, b, c, x(*), y(*), area(*), qqmpol
      integer *4 i1, i2, i3, npts, nincr, nlast
c
      common / Af77lib6 / a, b, c
c
c         check arguments
c
      if ( nincr .ge. 1 .and. npts .ge. 3 ) then                          1b
c
c         set last index
c
        nlast = nincr * ( npts -1 ) + 1
c
c         INTEGRATE FOR 1ST 2.5 INTERVALS
c
c         set indices
        i1 = 1
        i2 = nincr + i1
        i3 = nincr + i2
c
c         quadratic coefficients
        a = ( (y(i3)-y(i2)) / (x(i3)-x(i2)) - (y(i1)-y(i2)) /
     1      (x(i1)-x(i2)) ) /(x(i3)-x(i1))
        b = ( (y(i2)-y(i1)) / (x(i2)-x(i1)) - (x(i2)+x(i1)) * a )
        c = y(i2) - ( a * x(i2) + b ) * x(i2)
c
c         integrated quadratic coefficients
        a = a / 3.0
        b = b / 2.0
c       c = c / 1.0
c
c         evaluate integral segments
        area(i2) = qqmpol(x(i2)) - qqmpol(x(i1)) + area(i1)
        area(i3) = 0.5 * ( qqmpol(x(i3)) - qqmpol(x(i2)) ) + area(i2)
c
c         INTEGRATE ALL BUT LAST POINT
c
        do while( i3 .le. nlast )                                           2b
c
c               set indices
          i1 = i2
          i2 = i3
          i3 = i3 + nincr
c
c               quadratic coefficients
          a = ( (y(i3)-y(i2)) / (x(i3)-x(i2)) - (y(i1)-y(i2)) /
     1        (x(i1)-x(i2)) ) / (x(i3)-x(i1))
          b = ( (y(i2)-y(i1)) / (x(i2)-x(i1)) - (x(i2)+x(i1)) * a )
          c = y(i2) - ( a * x(i2) + b ) * x(i2)
c
c               integrated quadratic coefficients
          a = a / 3.0
          b = b / 2.0
c         c = c / 1.0
c
c         evaluate integral segments
          area(i2) = 0.5 * ( qqmpol(x(i2)) - qqmpol(x(i1)) ) + area(i2)
          area(i3) = 0.5 * ( qqmpol(x(i3)) - qqmpol(x(i2)) ) + area(i2)
          end do                                                          2e
c
c         INTEGRATE LAST POINT
c
        area(i3) = 0.5 * ( qqmpol(x(i3)) - qqmpol(x(i2)) ) + area(i3)
        end if                                                            1e
      return
      end


      SUBROUTINE SQUADC( Z1, Z2, A, B, C, STATUS, TOL )
c
c Subroutine to find all the zeros of the quadratic equation
c     a.z**2 + b.z + c = 0
c The parameters are :
c complex z1,z2  # zeros of the quadratic equation, possibly complex conjugates
c real    a,     # coefficient of z squared
c         b,     # coefficient of z
c         c,     # constant coefficient
c         tol    # tolerance on zero detector for discriminator
c integer status # =1 , two distinct real roots
c                # =0 , two equal real roots
c                # =-1, pair of complex conjugate zeros
c
c local variables
c  double precision
c   q   # product of roots
c   r   # mean value of roots
c   s   # discriminant
c   t   # absolute tolerance
c
      implicit none
      complex z1, z2
      real a, b, c, tol
      double precision q, r, s, t
      integer status
c
      q = dble(c) / dble(a)
      r = dble(b) / dble(a) / -2.0d0
      s = r **2 - q
      t = dble( abs( tol ) )
c
c       real distinct roots -- avoid subtracting
      if ( s .gt. t ) then
        s = dsqrt( s )
        t = r + sign( s, r )
        z1 = cmplx( t )
        z2 = cmplx( q / t )
        status = 1
c
c       complex conjugate roots
      else if ( s .lt. -t ) then
        s = dsqrt( -s )
        z1 = cmplx( sngl( r ), sngl( s ) )
        z2 = conjg( z1 )
        status = -1
c
c       real, equal roots
      else
        z1 = cmplx( sngl( r ), 0.0 )
        z2 = z1
        status = 0
        end if
      return
      end



      SUBROUTINE SROTATE( XOUT, YOUT, XIN, YIN, THETA )
c
c Subroutine to rotate the coordinates ( Xin, Yin ) by the angle Theta to
c  produce the coordinates ( Xout, Yout )
c
      implicit none
      real Xin, Yin, Xout, Yout, Theta, s, c
c
      s = sin( Theta )
      c = cos( Theta )
      Xout = Xin * c - Yin * s
      Yout = Xin * s + Yin * c
      return
      end


      SUBROUTINE STRAPZ( X, Y, AREA, NINCR, NPTS )
c
c       Trapezoidal Rule Integration of Discrete data array
c
c  This routine integrates the function defined by the NPTS coordinate pairs
c  ( X(i), Y(i), i = 1, NINCR+1, NINCR+2 ... ) using the first X value as the
c  lower limit of integration . The result placed in AREA is the result of
c  progressively integrating the function with each successive X value as the
c  upper limit of integration . The first value in AREA is taken as the initial
c  value of the definite integral .
c
c Arguments
c  real x(*)           # abicissa data array in strictly ascending order
c       y(*)           # ordinate data array
c       area(*)        # result array
c  integer *4 nincr    # data array indexing increment
c             npts     # number of data points to be used
c
c Variables
c  integer *4 i1,i2    # indices of 1st, 2nd points of the sub-interval
c             nlast    # index of last data point
c
      implicit none
      real x(*), y(*), area(*)
      integer *4 i1, i2, npts, nincr, nlast
c
c         check arguments
c
      if ( nincr .ge. 1 .and. npts .ge. 3 ) then                          1b
        nlast = nincr * ( npts -1 ) + 1
        i2 = 1
c
        do while( i2 .le. nlast )                                         2b
          i1 = i2
          i2 = i2 + nincr
          area(i2) = ( x(i2) - x(i1) ) * ( y(i2) + y(i1) ) / 2.0 +
     1      area(i1)
          end do                                                          2e
        end if                                                            1e
      return
      end


      SUBROUTINE SWAP2( I, J )
c
c   Subroutine for swapping 2 2 byte quantities
c
      implicit none
      integer *2 i, j, k
c
      k = i
      i = j
      j = k
      return
      end


      SUBROUTINE SWAP4( A, B )
c
c   Subroutine for swapping 2 4 byte quantities
c
      implicit none
      integer *4 a, b, c
c
      c = a
      a = b
      b = c
      return
      end


      COMPLEX FUNCTION WDET( A, M, TOL )
c
c       Calculate the determinant of a complex matrix
c
c    Arguments
c complex a(m,m),       # operand matrix
c real  tol             # zero detection tolerance
c integer *4 m          # number of rows and number of columns in matrix A
c
      implicit none
      integer *4 i, m,j
      complex a(m,m)
      real tol
c
      call sgjredz( a, m, m, m, tol, .false. )
      wdet = (1.0,0.0)
      do i = 1, m
        wdet = wdet * a(i,i)
        end do
      return
      end


