c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


c           *****************************************************
c           ***                                               ***
c           ***    P.V.Lanspeary private FORTRAN 77 library   ***
c           ***                                               ***
c           ***      f77_ute : general purpose utilities      ***
c           ***                                               ***
c           *****************************************************
c
c    Common blocks : / Af77lib1 / for DAOPEN, DAGET, DAPUT, DAFLUSH, DAINIT
c                    / Af77lib4 / for QCPCLK, SCPCLK
c


      LOGICAL FUNCTION DAOPEN( IFILELU, CFILNAM, IRECLEN )
c
c        Direct Access file OPEN
c Routine to perform open a direct access file with name FILNAM and logical
c unit number FILELU . The record length is fixed at RECLEN bytes . There is
c no associated variable . If the file was opened successfully the returned
c value is DAOPEN=.true. ; otherwise DAOPEN=.false.
c N.B. 1) Maximum record length is 512 bytes
c
c    Arguments
c integer *4 ifilelu            # logical unit number of the file
c            ireclen            # length in  bytes of each file record
c            irecptr            # record pointer ( associated variable) of
c                      the file ( incremented +1 after each I/O operation )
c character*(*) cfilnam         # file name
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c    Variables
c byte       gopbuf(512),       # output buffer array
c            gipbuf(512)        # input buffer array
c integer *2 iopptr,iipptr,     # output and input buffer pointers
c            flu,               # logical unit number of the file
c            lenrec,            # length in  bytes of each file record
c            dlrec              # length long words of each file record
c integer open_file             # function : open file
c
      implicit none
      integer Hvdulu
      parameter( Hvdulu = 6 )
c
      byte gopbuf(512), gipbuf(512)
      integer *2 iopptr, iipptr, flu, lenrec, dlrec
      integer irecptr, ifilelu, ireclen, open_file
      character *(*) cfilnam
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, lenrec, dlrec,
     1  flu
c
      lenrec = ireclen
      flu    = ifilelu
      iopptr = 1
      iipptr = ireclen + 1
      dlrec  = ( ireclen + 3 ) / 4
      irecptr = 1
      if ( 0 .eq. open_file( ifilelu, cfilnam, 'unknown', ireclen ) )
     1        then
          write( Hvdulu, 1 )
1         format( ' DAOPEN : Cannot open file' )
          daopen = .FALSE.
      else
          daopen = .TRUE.
          end if
      return
      end


      LOGICAL FUNCTION DAPUT( NREC, DATA, LENGTH )
c
c       Appends the first LENGTH bytes of the data array DATA onto record NREC
c  of the direct access binary file with logical unit number FILELU .
c       Pointers IOPPTR points to the next byte in GOPBUF to be written .
c
c    Arguments
c byte       data(*)            # data array
c integer *4 nrec,              # record number to be written
c            length,            # length ( in bytes ) of the DATA array
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c byte       gopbuf(512),       # output buffer array
c            gipbuf(512),       # input buffer array
c integer *4 dopbuf(128)        # output buffer array
c            i                  # byte counter
c integer *2 j                  # byte counter
c            iopptr,iipptr,     # output and input buffer pointers
c            filelu,            # logical unit number of the file
c            reclen             # length in bytes of each record
c            dlrec              # length in long words of each record
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu = 6 )
c
      integer *2 j, iopptr, iipptr, filelu, reclen, dlrec
      integer *4 dopbuf(128), nrec, length, i
      byte gopbuf(512), gipbuf(512), data(*)
c
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, reclen, dlrec,
     1              filelu
      equivalence ( gopbuf(1), dopbuf(1) )
c
      do i = 1, length
        gopbuf( iopptr ) = data(i)
        iopptr = iopptr + 1
        if ( iopptr .gt. reclen ) then
          iopptr = 1
          write ( filelu, REC=nrec, ERR=3 ) ( dopbuf(j), j= 1, dlrec )
          nrec = nrec + 1
          end if
        end do
      daput = .true.
      return
c
c           error message
c
3     write ( Hvdulu, 4 )
4     format( ' DAPUT : Error' )
      daput = .false.
      return
      end



      LOGICAL FUNCTION DAGET( NREC, DATA, LENGTH )
c
c  Reads the next LENGTH  bytes from record NREC of the direct access
c  binary file with logical unit number FILELU into the array DATA .
c       Pointer IIPPTR points to the next byte to be read in GIPBUF .
c
c  N.B. If the read is successful the returned result is DAGET=.true.
c       If a read error occurs , the result is DAGET=.false. and an
c       error message is displayed .
c
c    Arguments
c byte       data(length)       # data array
c integer *4 nrec,              # record number to be read
c            length             # length ( in bytes ) of the data array
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c   Variables
c byte       gopbuf(512),       # output buffer array
c            gipbuf(512),       # input buffer array
c integer *4 dipbuf(128)        # input buffer array
c            i                  # byte counter
c integer *2 j                  # byte counter
c            iopptr,iipptr,     # output and input buffer pointers
c            filelu,            # logical unit number of the file
c            reclen,            # length in bytes of each record
c            dlrec              # length in long words of each record
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu = 6 )
c
      integer *2 j, iopptr, iipptr, filelu, reclen, dlrec
      integer *4 dipbuf(128), nrec, length, i
      byte gopbuf(512), gipbuf(512), data(*)
c
      common /Af77lib1/ gopbuf, gipbuf, iopptr, iipptr, reclen, dlrec,
     1              filelu
      equivalence ( gipbuf(1), dipbuf(1) )
c
      do i = 1,length
        if ( iipptr .gt. reclen ) then
          iipptr = 1
          read ( filelu, REC=nrec, ERR=3 )
     1         ( dipbuf(j), j = 1, dlrec )
          nrec = nrec + 1
          end if
        data(i) = gipbuf( iipptr )
        iipptr = iipptr + 1
        end do
      daget = .true.
      return
c
c         error message
c
3     write ( Hvdulu, 4 )
4     format( ' DAGET : Error' )
5     daget = .false.
      return
      end



      LOGICAL FUNCTION DAFLUSH( NREC )
c
c       Flushes the data remaining in the output buffer GOPBUF onto record
c  NREC of the direct access file with logical unit number FILELU .
c       Pointer IIPPTR points to the next byte in GOPBUF to be written
c
c    Argument
c integer *4 nrec               # record number to be written
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c Variables
c logical *1 gopbuf(512),       # output buffer array
c            gipbuf(512)        # input buffer array
c integer *4 dopbuf(128)        # output buffer array
c integer *2 i,j,               # byte counters
c            iopptr,iipptr,     # output and input buffer pointers
c            filelu,            # logical unit number of the file
c            reclen             # length in bytes of each file record
c            dlrec              # length in long words of each record
c
      implicit none
      integer Hvdulu
      parameter( Hvdulu = 6 )
      integer *2 i, j, iopptr, iipptr, filelu, reclen, dlrec
      integer *4 dopbuf(128), nrec
      byte gopbuf(512), gipbuf(512)
c
      common /af77lib1/ gopbuf, gipbuf, iopptr, iipptr, reclen, dlrec,
     1              filelu
      equivalence ( gopbuf(1), dopbuf(1) )
c
      if ( iopptr .gt. 1 ) then
        do i = iopptr , reclen
          gopbuf(i) = 0
          end do
        write ( filelu, REC=nrec, ERR = 2 ) ( dopbuf(j), j = 1, dlrec )
        nrec = nrec + 1
        iopptr = 1
        daflush = .true.
        end if
      return
c
c             error message
c
2     write ( Hvdulu, 3 )
3     format( ' DAFLUSH : Error' )
      daflush = .false.
      return
      end



      SUBROUTINE DAINIT
c
c       Initializes so that the next call to DAGET reads from the beginning of
c  the record NREC from direct access binary file with logical unit number
c  FILELU .  If the associated variable is NREC which is used to sequentially
c  access the file , this causes NREC to be the next record .
c
c byte       gopbuf(512),       # output buffer array
c            gipbuf(512)        # input buffer array
c integer *2 iopptr,iipptr,     # output and input buffer pointers
c            filelu,            # logical unit number of the file
c            reclen,            # length in bytes of each file record
c            dlrec              # length in long words of each record
c
      implicit none
      integer *2 iopptr, iipptr, filelu, reclen, dlrec
      byte gopbuf(512), gipbuf(512)
c
      common /af77lib1/ gopbuf, gipbuf, iopptr, iipptr, reclen, dlrec,
     1              filelu
c
      iipptr = reclen + 1
      return
      end



      LOGICAL FUNCTION EGETCL( CMDLINE )
c
c         Get the command line and put it into CMDLINE
c  The zeroth argument, which is the program name is not included in the
c  returned string . The result is left justified and extra blanks are
c  compressed out by the run-time library routine LIB$GET_FOREIGN .
c
c Argument
c  character cmdline*(*)       # command line arguments
c
      implicit none
      integer *4 lib$get_foreign
      character cmdline*(*)
c
      egetcl = iand( lib$get_foreign( cmdline ), 1 ) .gt. 0
      return
      end


      LOGICAL FUNCTION EIN( CSTR, CSET )
c
c     Tests if the character string CSTR is contained within the character
c string CSET .
c
c ARGUMENTS
c  character Cstr *(*)         # is this string
c            Cset *(*)         # contained in this string
c variables
c  integer *4 lenstr           # length of Cstr
c             lenset           # length of & finishing index for Cset
c             i                # starting index for Cset
c
      implicit none
      character Cstr *(*), Cset*(*)
      integer *4 lenstr, lenset, i
c
      ein = .false.
      lenstr = len( Cstr )
      lenset = len( Cset )
      if ( lenstr .gt. 0 .and. lenset .gt. 0 .and. lenstr .le. lenset )
     1    then
        do i = 1, lenset - lenstr + 1
          if ( Cstr .eq. Cset( i: lenstr ) ) then
            ein = .true.
            return
            end if
          lenstr = lenstr + 1
          end do
        end if
      return
      end


      LOGICAL FUNCTION EQABBR( CSTR, CTEST, MINLEN )
c
c       EQual ABBReviation
c     Tests if the character string CSTR is a legal abbreviation of the
c character string CTEST .
c
c Arguments
c  character cstr*(*),         # operand character string               INPUT
c            ctest*(*)         # test string                            INPUT
c  integer minlen              # minimum number of characters allowed   INPUT
c                              # in the abbreviation
c Variables
c  integer i,       # indexing counter
c          iend     # first letter in a word
c
      implicit none
      integer i, iend, minlen
      character cstr*(*), ctest*(*)
c
c         find end of the operand string
c
      iend = len( cstr )
      do while ( iend .gt. 1 .and. cstr(iend:iend) .eq. ' ' )
        iend = iend - 1
        end do
c
c         test
c
      if ( iend .gt. len( ctest ) ) then
        eqabbr = .false.
      else
        eqabbr = .true.
        i = 1
        do while( i .le. iend .and. eqabbr )
          eqabbr = eqabbr .and. cstr(i:i) .eq. ctest(i:i)
          i = i + 1
          end do
        eqabbr = eqabbr .and. i .gt. minlen
        end if
      return
      end


      LOGICAL FUNCTION EQSIGN( X, Y )
c
c Returns .True. if x and y have the same sign
c
      implicit none
      real *4 x,y
c
      eqsign = x * y .ge. 0.0
      return
      end



      LOGICAL FUNCTION EQUERY( IPLUN, MSG )
c
c       Displays a query on the vdu and receives a logical response with
c  logical unit number IPLUN .
c
c    Arguments
c integer *4 iplun              # input device logical unit number
c character msg*(*)             # message string
c
c constant
c     Hvdulu                    # v.d.u. logical unit number
c integer *2 iok,               # index of response in legal response list
c            index,             # lexical function
c character gquery*1            # received character
c           gok*8               # legal responses
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu = 6 )
c
      integer *2 iok, index
      integer *4  iplun
      character gquery*1, gok*10, msg*(*)
      data gok / 't1yY. f0nN' /
c
      write( Hvdulu, 1 ) msg
 1    format( 1x, a, ' ', $ )
c
      do while( .true. )
        read( iplun, 2, ERR=4 ) Gquery
 2      format( a1 )
        iok = index( gok, Gquery )
        if ( iok .gt. 0 ) then
          equery = iok .le. 5
          return
          end if
        write( Hvdulu, 3 )
 3      format( ' ? ', $ )
        end do
c
 4    stop 'ABORT'
      end



      LOGICAL FUNCTION ESPECF( CFILE )
c
c         is the File SPECification in CFILE valid ?
c
      implicit none
      integer *4 lib$find_file, zero
      character cfile*(*), cresult*128
      data  zero / 0 /
c
      especf = lib$find_file( cfile , cresult, zero ) .ne. 100052
      return
      end



      LOGICAL FUNCTION EXISTF( CFILE )
c
c         does the File CFILE    EXIST ?
c
      implicit none
      integer *4 lib$find_file, zero
      character cfile*(*), cresult*128
c
      zero = 0
      existf =  lib$find_file( cfile , cresult, zero ) .eq. 65537
      return
      end



      LOGICAL FUNCTION EYES( BOOLEAN, QUERY_MESSAGE )
c
c Converts the integer value BOOLEAN which must be NO, YES or QUERY to
c a logical value . If BOOLEAN is QUERY, the user is requested to make
c YES/NO decision .
c
c
      include 'stdio.def'
      integer boolean
      logical Equery
      character query_message*(*)
c
      if ( boolean .eq. NO ) then
          eyes = .FALSE.
      else if ( boolean .eq. YES ) then
          eyes = .TRUE.
      else if ( boolean .eq. QUERY ) then
          eyes = Equery( STD_IN, query_message )
      else
          write( STD_ERR, 1 ) boolean
1         format( ' EYES : WARNING : Illegal boolean ', i12 )
          end if
      return
      end



      INTEGER *4 FUNCTION IARGC()
c
c         Returns the number of arguments to the main program .
c
c
c    constants
c Hlen                   # Maximum argument list length
c    Variables
c character Cmd*Hlen     # program arguments
c integer *2 i,          # count up index
c logical *1 fspace      # found an argument separator
c logical *4 egetcl      # function get command line
c
      implicit none
      integer *2 Hlen
      parameter( Hlen=250)
c
      character  Cmd*250
      integer *2 i
      logical *1 fspace
      logical *4 egetcl
c
      iargc = 0
      if ( egetcl( Cmd ) ) then                                           1b
        fspace = .true.
        do i = 1, Hlen                                                    2b
          if ( cmd(i:i) .eq. ' ' ) then                                   3b
            fspace = .true.
          else if ( fspace ) then                                         3m
            iargc = iargc + 1
            fspace = .false.
            end if                                                        3e
          end do                                                          2e
        end if                                                            1e
      return
      end


      INTEGER FUNCTION JLJSTN( CSTR, NCOLS )
c
c         Left justify the character string CSTR ; ie eliminate leading blanks
c and return the number of characters in it after trailing blanks have also
c been removed . NCOLS is the returned number of tokens in Cstr to the left of
c any ';' . Spaces ' ' and commas ',' are token separators .
c
c    Argument
c character Cstr*(*)    # operand
c
c    Variables
c integer ncols   # number of columns of data
c integer length, # length of operand
c         i,      # count up index
c         j,      # count down index
c         n,      # justified result index
c logical fnocmt  # comment field not found
c         fspace  # found a numeric field separator
c
      implicit none
      character Cstr*(*)
      integer ncols, length, i, j, n
      logical fnocmt, fspace
c
      length = len( Cstr )
c
c       find first non blank char
c
      i = 1
      do while( Cstr(i:i) .eq. ' ' .and. i .lt. length )
          i = i + 1
          end do
c
c       find last non blank char
c
      j = length
      do while( Cstr(j:j) .eq. ' ' .and. j . gt. i )
          j = j - 1
          end do
      jljstn = j - i + 1
      if ( Cstr(j:j) .eq. ' ' ) then
          jljstn = 0
          ncols = 0
          return
          end if
c
c       left justify
c
      fspace = .false.
      fnocmt = .true.
      if ( Cstr(1:1) .eq. ';' ) then
          ncols = 0
      else
          ncols = 1
          do n = 1, jljstn
              Cstr(n:n) = Cstr(i:i)
              fnocmt = fnocmt .and. cstr(i:i) .ne. ';'
              if ( cstr(i:i) .eq. ' ' .or. cstr(i:i) .eq. ',' ) then
                  fspace = .true.
              else if ( fspace .and. fnocmt ) then
                  ncols = ncols + 1
                  fspace = .false.
                  end if
              i = i + 1
              end do
          end if
c
c       fill up with blanks
c
      do n = jljstn + 1, j
          Cstr(n:n) = ' '
          end do
      return
      end


      INTEGER *4 FUNCTION JLJSTR( CSTR )
c
c         Left justify the character string CSTR ; ie eliminate leading blanks
c and return the number of characters in it after trailing blanks have also
c been removed .
c
c    Argument
c character *(*) Cstr          # operand
c
c    Variables
c integer *2 length,           # length of operand
c            i,                # count up index
c            j,                # count down index
c            n,                # justified result index
c
      character *(*) Cstr
      integer *2 length, i, j, n
c
      length = len( Cstr )
c
c       find first non blank char
c
      i = 1
      do while( Cstr(i:i) .eq. ' ' .and. i .lt. length )
        i = i + 1
        end do
c
c       find last non blank char
c
      j = length
      do while( Cstr(j:j) .eq. ' ' .and. j . gt. i )
        j = j - 1
        end do
      Jljstr = j - i + 1
      if ( Cstr(j:j) .eq. ' ' ) Jljstr = 0
c
c       left justify
c
      do n = 1, jljstr
        Cstr(n:n) = Cstr(i:i)
        i = i + 1
        end do
c
c       fill up with blanks
c
      do n = jljstr + 1, j
        Cstr(n:n) = ' '
        end do
      return
      end


      INTEGER *4 FUNCTION JRESP( IPLU, IMIN, IMAX, NOREQS )
c               Integer RESPonse
c
c       The integer function JRESP returns an integer value between the
c  specified valid limits. This value is entered via the file with logical
c  unit number IPLU ( including possibly the keyboard ) , and is  checked
c  by JRESP, i.e. MIN < JRESP < MAX . The input file logical unit number
c  IPLU must be in the range 0 <= IPLU <= 99 , and must have been previously
c  defined if it is not a default device ( vdu, keyboard etc ). The maximum
c  number of input requests allowed NOREQS must be positive.
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer *4 Iplu              #  input file logical unit number
c            imax,             #  maximum valid response value
c            imin,             #  minimum valid response value
c            noreqs            #  maximum number of requests allowed
c
c constants
c     Hkbdlu                   # keyboard logical unit number
c     Hvdulu                   # v.d.u. logical unit number
c integer *4 cntr,             #  counter for the number of input attempts
c            Iplu1             #  input file logical unit number
c            nchars            # number of input characters
c            jljstr            # function : left justify string
c character Cstr*40            # input character string
c
      implicit none
      integer *2 Hkbdlu, Hvdulu
      parameter ( Hkbdlu=5, Hvdulu=6 )
c
      integer *4 iplu, imax, imin, noreqs, cntr, iplu1, nchars, jljstr
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. noreqs .le. 0 )
     1  write ( Hvdulu, 1 ) Iplu, Noreqs
1       format ( ' JRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = Hkbdlu
      do cntr = 1, noreqs
c
c           read data
c
        nchars = 0
        do while( nchars .le. 0 )
          read( Iplu1, 2, END=7 ) Cstr
2         format( a )
          nchars = jljstr( Cstr )
          end do
c
c           selected min or max
c
        if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
          jresp = imax
          return
        else if ( Cstr(1:3) .eq. 'min' .or. Cstr(1:3) .eq. 'MIN' ) then
          jresp = imin
          return
c
c           decode & check numeric value
c
        else
          read( Cstr, *, ERR=8 ) jresp
          if ( min(imin,imax) .le.jresp .and. jresp .le.max(imin,imax) )
     1      return
          write ( Hvdulu, 3 ) imin, imax
3         format(' JRESP : ERROR ; Integer outside limits ', i11, ' & ',
     1            i11 )
4         if ( Iplu1 .ne. Hkbdlu ) stop
          write ( Hvdulu, 5 )
5         format( ' Try again ', $ )
          end if
        end do
c
c           give up
c
        write ( Hvdulu, 6 )
6       format( ' Get help' )
7       stop
c
8     write( Hvdulu, 9 )
9     format( ' JRESP : ERROR : Can''t decode that ' )
      goto 4
      end



      INTEGER FUNCTION OPEN_FILE( LUN, FILE_NAME, NEW_OLD, TYPE )
c
c     Opens a data file for reading ( NEW_OLD = 'old' ) or writing
c ( NEW_OLD = 'new' ) . If FILE_NAME is '?', the name of the file is
c obtained from the user . If the file is for reading, the file name cannot be
c blank. If the file is for writing and TYPE=0 (ASCII), a blank filename is
c interpreted as the vdu screen, otherwise it is treated as '?'. The logical
c unit used for the file is LUN . If the file is successfully opened, LUN is
c the returned result . If a file could not be opened, the returned value is
c OPEN_FILE = 0 .
c
c The file type is specified by TYPE .
c   TYPE = -1  binary (unformatted), sequential access file
c   TYPE = 0   ASCII (formatted), sequential access file
c   TYPE > 0   binary (unformatted), direct access file with record length
c              of TYPE bytes .
c
c Arguments
c  integer   lun          # logical unit no. of file to be opened
c  character file_name*(*)# file name
c            new_old*(*)  # file status : 'old'=read, 'new'=write
c  integer   type         # file type -- see above
c Constants
c   Hkbdlu=5              # keyboard
c   Hvdulu=6              # vdu
c Local Variables
c  logical from_user      # user is required to supply file name
c          Equery         # function :
c  integer length         # length of file name string
c          str_length     # function : length of string
c character user_resp*64  # name of file - obtained from user
c
      implicit none
      integer Hkbdlu, Hvdulu
      parameter( Hkbdlu=5, Hvdulu=6 )
c
      logical Equery, from_user
      integer lun, type, length, str_length
      character file_name*(*), new_old*(*), user_resp*64
c
c             check argument
c
      open_file = 0
      if ( new_old .ne. 'old' .and. new_old .ne. 'new' .and.
     1     new_old .ne. 'unknown' ) then
          write( Hvdulu, 7 )
7         format( ' OPEN_FILE : ABORT : Illegal status' /
     1            21x, 'Need ''old'', ''new'' or ''unknown''' )
          return
      else if ( type .lt. -1 ) then
          write( Hvdulu, 8 )
8         format( ' OPEN_FILE : ABORT : Illegal file type' /
     1            21x, 'Need -1 (binary), 0 (ASCII), >1 (direct)' )
          return
          end if
      from_user = file_name .eq. '?' .or. file_name .eq. ' ' .and.
     1            ( new_old .eq. 'old' .or. type .ne. 0 )
c
c             get file name
c
c     repeat
1         if ( from_user ) then
              write( Hvdulu, 2 )
2             format( ' Name of file ? ', $ )
              read( Hkbdlu, 3, ERR=1, END=6 ) user_resp
3             format( a )
          else
              user_resp = file_name
              end if
          length = str_length( user_resp )
          if ( length .gt. 0 ) then
              if ( type .eq. -1 ) then
                  open( UNIT=lun, FILE=user_resp, STATUS=new_old,
     1                  ACCESS='sequential', FORM='unformatted', ERR=4 )
                  open_file = lun
              else if ( type .eq. 0 ) then
                  open( UNIT=lun, FILE=user_resp, STATUS=new_old,
     1                  ACCESS='sequential', FORM='formatted', ERR=4 )
                  open_file = lun
              else if ( type .gt. 0 ) then
                  open( UNIT=lun, FILE=user_resp, STATUS=new_old,
     1                  ACCESS='direct', FORM='unformatted',
     2                  RECL=(type+3)/4, ERR=4 )
                  open_file = lun
                  end if
          else if ( ( new_old .ne. 'new' .or. type .ne. 0 ) .and.
     1            from_user ) then
              goto 1
          else if ( Equery( Hkbdlu, 'To screen. Are you sure ?' ) ) then
              open_file = Hvdulu
          else
              goto 1
              end if
c         until( read non-blank file_name )
      return
c
c             TRAPS
c
4     write( Hvdulu, 5 )
5     format( ' OPEN_FILE : ABORT : Cannot open file' )
      open_file = 0
      return
c
6     stop ' ABORT'
      end


      INTEGER FUNCTION NEW_LUN()
c
c  Selects a new (unused) logical unit for a file .
c  The first logical unit is 10 .
c
      implicit none
      integer next_lun
      common /Af77lib9/ next_lun
      data next_lun / 10 /
c
      new_lun  = next_lun
      next_lun = next_lun + 1
      return
      end


      INTEGER FUNCTION STR_LENGTH( STRING )
c
c    Find the location of the last non-blank character in the character
c string STRING . If the whole string is blank, the returned value is 0 .
c
      implicit none
      character string*(*)
c
      str_length = len( string )
1     if ( str_length .ge. 1 ) then
          if ( string(str_length:str_length) .eq. ' ' ) then
              str_length = str_length - 1
          else
              return
              end if
          goto 1
          end if
      return
      end


      REAL FUNCTION QCPCLK
c
c       reads the CPu CLocK  ( in seconds )
c    For elapsed execution time, the clock must be "started" by a call to
c SCPCLK .
c
c variable
c  integer *4 itime            # CPU clock start time
c             icpclk           # elapsed CPU time in 100ths of seconds
c
      implicit none
      integer itime, icpclk
      common / Af77lib4 / itime
c
      call lib$stat_timer( 2, icpclk, itime )
      qcpclk = icpclk / 100.0
      return
      end



      REAL FUNCTION QLIMIT( R, RMIN, RMAX, ERRMSG )
c
c   Real function which ensures that the real variable R remains within the
c limits of Rmin and Rmax ; i.e. Rmin <= R <= Rmax
c
c    Arguments
c real  R,              # initial value
c       Rmin,           # minimum permissable value
c       Rmax            # maximum permissable value
c logical errmsg        # .false. if error message is disabled
c
      implicit none
      integer *2 Hvdulu
      parameter( Hvdulu=6 )
c
      logical *4 Errmsg
      real r, rmax, rmin, Amin1, Amax1
c
      Qlimit = Amin1( Rmax, Amax1( Rmin, R ) )
      if ( R .ne. Qlimit .and. Errmsg ) write( Hvdulu, 1 ) R
1       format( ' QLIMIT : ', e15.6, ' exceeds limits' )
      return
      end




      REAL FUNCTION QMUAIR( TEMP )
c
c       Dynamic Viscosity of dry AIR as a function of absolute TEMPerature .
c
c   Use Sutherland's constant  C = 120.187
c     & constant of proportionality K = 1.48875e-6
c
c    ARGUMENT
c real *4 temp    # absolute air temperature
c
      implicit none
      real temp, c, k
      parameter( c=120.187, k=1.48875e-6 )
c
      qmuair = k * sqrt( temp ) * temp / ( c + temp )
      return
      end



      REAL FUNCTION QRESP( IPLU, RMIN, RMAX, NOREQS )
c               Real RESPonse
c       The real funtion QRESP returns a real value between the specified
c valid limits . This value is entered via the file with logical unit number
c IPLU ( default value 0 is the keyboard ) , and is checked by QRESP
c ie RMIN <= QRESP <= RMAX . The input file logical unit number must be in the
c range 0 <= IPLU <= 99 , and must have been previously defined if it is not
c a default device ( keyboard etc ) . The maximum number of input requests
c allowed NOREQS must be positive .
c       If the words `MAX', `max', `MIN' or `min' are entered, the minimum or
c  or maximum allowed values are selected .
c
c    Arguments
c integer *4 Iplu       # input file logical unit number
c            Noreqs     # maximum number of requests allowed
c real  rMax,           # maximum valid response value
c       rMin            # minimum valid response value
c
c constants
c   Hkbdlu              # keyboard logical unit number
c   Hvdulu              # v.d.u. logical unit number
c integer *4 Cntr,      # counter for the number of input attempts
c            Iplu1      # input file logical unit number
c            nchars     # number of input characters
c            jljstr            # function : left justify string
c character Cstr*40     # input character string
c
      implicit none
      integer *2 Hvdulu, Hkbdlu
      parameter ( Hvdulu = 6, Hkbdlu = 5 )
c
      integer *4 iplu, noreqs, cntr, iplu1, nchars, jljstr
      real rMax, rMin
      character Cstr*40
c
c           check arguments
c
      if ( Iplu .le. 0 .or. Iplu .gt. 99 .or. Noreqs .le. 0 )
     1  write( Hvdulu, 1 ) Iplu, Noreqs
1       format( ' QRESP : Parameter error. Iplu = ',i7,', Noreqs = ',
     1          i7 )
      Iplu1 = Iplu
      if ( Iplu .le. 0 ) Iplu1 = Hkbdlu
      do Cntr = 1, noreqs
c
c           read data
c
        nchars = 0
        do while( nchars .le. 0 )
          read( Iplu1, 2, END=7 ) Cstr
2         format( a )
          nchars = jljstr( Cstr )
          end do
c
c           selected min or max
c
        if ( Cstr(1:3) .eq. 'max' .or. Cstr(1:3) .eq. 'MAX' ) then
          qresp = rmax
          return
        else if ( Cstr(1:3) .eq. 'min' .or. Cstr(1:3) .eq. 'MIN' ) then
          qresp = rmin
          return
c
c           decode & check numeric value
c
        else
          read ( Cstr, *, ERR=8 ) Qresp
          if ( min(rMin,rMax).le.Qresp .and. Qresp.le.max(rMin,rMax) )
     1      return
          write( Hvdulu, 3 ) rmin,rmax
3         format(' QRESP : ERROR ; Real outside limits ', g12.5, ' & ',
     1            g12.5 )
4         if ( Iplu1 .ne. Hkbdlu ) stop
          write ( Hvdulu, 5 )
5         format( ' Try again ', $ )
          end if
        end do
c
c           give up
c
      write ( Hvdulu, 6 )
6     format( ' Get help' )
7     stop
c
8     write( Hvdulu, 9 )
9     format( ' QRESP : ERROR : Can''t decode that ' )
      goto 4
      end


      SUBROUTINE GETARG( IARG, CARG )
c
c   Returns the IARGth argument of the main program to the character string
c CARG . The program name is the zeroth argument and cannot be returned .
c
c Arguments
c  integer *4 iarg       # Argument to be selected
c  character Carg*(*)    # Argument string returned
c
c    constants
c Hlen                   # Maximum argument list length
c    Variables
c character Cmd*Hlen     # program arguments
c integer *2 i,          # count up index
c            igot        # argument number found
c            jbegin      # where the IARGth argument starts
c logical *1 fspace      # found an argument separator
c logical *4 egetcl      # function get command line
c
      implicit none
      integer *2 Hlen
      parameter( Hlen=250)
c
      character  Cmd*250, Carg*(*)
      integer *4 iarg
      integer *2 i, igot, jbegin
      logical *1 fspace
      logical *4 egetcl
c
      if ( egetcl( Cmd ) ) then                                           1b
        igot = 0
        if ( cmd(1:1) .ne. ' ' )
     1    igot = 1
        fspace = .false.
        jbegin = 1
        do i = 1, Hlen                                                    2b
          if ( cmd(i:i) .eq. ' ' ) then                                   3b
            fspace = .true.
            if ( igot .eq. iarg ) then                                    4b
              Carg = Cmd(jbegin:i-1)
              return
              end if                                                      4e
          else if ( fspace ) then                                         3m
            igot = igot + 1
            jbegin = i
            fspace = .false.
            end if                                                        3e
          end do                                                          2e
        end if                                                            1e
      Carg = ' '
      return
      end


      SUBROUTINE READ_DATA( INPUT_LU, ARRAY, ARRAY_SIZE, ROWS, COLUMNS )
c
c  Reads the data from the file with logical unit number INPUT_LU file into
c  the array ARRAY . ARRAY_SIZE is the maximum possible number of data values .
c  The data file is an array with ROWS rows and COLUMNS columns . A ';' in the
c  data file begins a comment field . Comment fields are terminated by an end
c  of the line . Input format is list directed ( ie. free ) . This routine
c  reads any number ( > 0 ) of columns of data in the 1st row and the same
c  number of columns from succeeding rows .
c
c   Arguments
c integer input_lu      # data file logical unit number
c real array(*)         # data array
c integer array_size    # maximum number of data items
c         columns       # number of columns of data
c         rows          # number of rows of data
c
c constants
c  Hvdulu     # vdu logical unit number
c variables
c integer i             # data array index offset
c         index         # no. of data items read
c         nchars        # no. of characters in a row
c         nerrs,        # no. of error rows detected
c         tmp_cols      # no. of columns in 2nd & succeeding rows
c         jljstn        # function : left justify string
c character cline*64    # characters read from file record
c         casech*(*)    # function : change case
c
      implicit none
      integer Hvdulu
      parameter( Hvdulu=6 )
c
      integer input_lu, array_size, columns, rows, i, index, nchars,
     1        nerrs, tmp_cols, jljstn
      real array(*)
      character cline*128, casech*3
c
c         initialize
c
      nerrs = 0
      nchars = 1
      index = 0
c
c         read the first line of data
c
1     if ( index .le. 0 ) then                                           1b
          read( input_lu, 2, END=7 ) cline
2         format( a )
          nchars = jljstn( cline, columns )
          if ( nchars .gt. 0 .and. cline(1:1) .ne. ';' ) then            2b
              if ( 'end' .eq. casech( cline(1:3), 'l' ) )
     1            goto 7
              read( cline, *, ERR=3 ) ( array(i), i = 1, columns )
              index = index + columns
              nerrs = nerrs - 1
3             nerrs = nerrs + 1
              end if                                                     2e
          goto 1
          end if                                                         1e
c
c         read the remaining data
c
      if ( nerrs .le. 0 ) then                                           1b
4         if ( index .le. array_size - columns ) then                    2b        2b
              read( input_lu, 2, END=7 ) cline
              nchars = jljstn( cline, tmp_cols )
              if ( nchars .gt. 0 .and. cline(1:1) .ne. ';' ) then        3b
                  if ( tmp_cols .lt. columns ) then                      4b           4b
                      if ( nerrs .eq. 0 )
     1                write( Hvdulu, 5 )
5                     format( ' READ_DATA : Reduced number of columns' )
                  else                                                   4m
                      if ( 'end' .eq. casech( cline(1:3), 'l' ) )
     1                    goto 7
                      read( cline, *, ERR=6 )
     1                    ( array( index + i ), i = 1, columns )
                      index = index + columns
                      nerrs = nerrs - 1
                      end if                                             4e
6                 nerrs = nerrs + 1
                  end if                                                 3e
               goto 4
               end if                                                    2e
          end if                                                         1e
7     rows = index / columns
      write( Hvdulu, 8 ) rows, columns
8     format( ' READ_DATA  ', i6, ' rows, ', i6, ' columns read' )
      if ( nerrs .gt. 0 ) then                                           1b
          write( Hvdulu, 9 ) nerrs
9         format( ' READ_DATA : Unable to read ', i6, ' records' )
          end if                                                         1e
      return
      end


      SUBROUTINE SBYTCHR( BSTR, CSTR, LENSTR )
c
c         Transfer the character data in the BYTE array BSTR to a character
c  variable GSTR . LENSTR is the maximum number of characters which can be
c  transferred . BSTR and GSTR must be at least L:ENSTR bytes long . A zero
c  ( null ) in BSTR is understood to be a string terminator . The remainder
c  of GSTR is filled with blanks .
c
c     Arguments
c byte Bstr(*)         # byte string
c character Cstr(*)    # character string
c integer *4 lenstr    # maximum length of byte string
c
      implicit none
      integer * 4 lenstr, i, j, l
      byte Bstr(*)
      character *(*) Cstr
c
      i = 1
      l = len( Cstr )
      do while( i .le. min( lenstr, l ) .and. Bstr(i) .ne. 0 )
        Cstr(i:i) = char( Bstr(i) )
        i = i + 1
        end do
      if ( i .le. l ) then
        do j = i, l
          Cstr(j:j) = ' '
          end do
        end if
      return
      end


      SUBROUTINE SCASE( CSTRIN, CSTROUT, CASE )
c
c       CASE CHange
c     Change the case of character string CSTR to upper, lower of edit case .
c
c Arguments
c  character cstrin*(*),    # operand character string               INPUT
c            cstrout*(*),   # result character string                INPUT
c            case*1         # 'l' or 'L' - change to lower case      INPUT
c                           # 'u' or 'U' - change to upper case
c                           # 'e' or 'E' - change to edit case
c                           # anything else - no change
c Constant
c  Hshift=32                # +ve or -ve displacement required to change case
c Variables
c  integer *2 i,            # indexing counter
c             nchars        # number of characters to process
c  logical *1 first         # first letter in a word
c
      implicit none
      integer *4 Hshift
      parameter( Hshift=32 )
c
      integer *2 i, nchars
      logical *1 first
      character cstrin*(*), cstrout*(*), case*1
c
      cstrout = cstrin
      nchars = min( len( cstrout ), len( cstrin ) )
c
c         change to lower case
c
      if ( case .eq. 'l' .or. case .eq. 'L' ) then                         1b
        do 1 i = 1, nchars                                                 2b
          if ( 'A' .le. cstrout(i:i) .and. cstrout(i:i) .le. 'Z' )
     1      cstrout(i:i) = char( ichar( cstrout(i:i) ) + Hshift )
1         continue                                                         2e
c
c         change to upper case
c
      else if ( case .eq. 'u' .or. case .eq. 'U' ) then                    1m
        do 2 i = 1, nchars                                                 2b
          if ( 'a' .le. cstrout(i:i) .and. cstrout(i:i) .le. 'z' )
     1      cstrout(i:i) = char( ichar( cstrout(i:i) ) - Hshift )
2         continue                                                         2e
c
c         change to edit case
c
      else if ( case .eq. 'e' .or. case .eq. 'E' ) then                    1m
        first = .true.
        do 3 i = 1, nchars                                                 2b
          if ( 'A' .le. cstrout(i:i) .and. cstrout(i:i) .le. 'Z' ) then    3b
            if ( .not. first )
     1        cstrout(i:i) = char( ichar( cstrout(i:i) ) + Hshift )
            first = .false.
          else if ( 'a'.le.cstrout(i:i) .and. cstrout(i:i).le.'z' ) then   3m
            if ( first )
     1        cstrout(i:i) = char( ichar( cstrout(i:i) ) - Hshift )
            first = .false.
          else                                                             3m
            first = .true.
            end if                                                         3e
3         continue                                                         2e
        end if                                                             1e
      return
      end


      SUBROUTINE SCHECK( SSBR )
c
c Subroutine to allow execution of subroutine Ssbr to be repeated if a
c   previous execution is unsatisfactory .
c
c constant Hkbdlu       # keyboard logical unit number
c logical Equery        # Function; # Displays request & returns reply
c logical *1 msg(5)
c data msg / 'OK ?' /
c
      implicit none
      integer *2 Hkbdlu
      parameter ( Hkbdlu=5 )
      logical Equery
      character *5 msg
c
      data msg / 'OK ? ' /
c
1     call Ssbr
      if ( .not. Equery( Hkbdlu, msg ) ) goto 1
      return
      end



      SUBROUTINE SCPCLK
c
c     Start CPu CLocK
c   "Starts" the CPU clock so that a call to QCPCLK produces the CPU time
c used in the intervening time .
c
c variable
c  integer *4 itime            # CPU clock start time
c
      implicit none
      integer *4 itime
      common / Af77lib4 / itime
c
      call lib$init_timer( itime )
      return
      end


      CHARACTER *(*) FUNCTION CASECH( CSTR, CASE )
c
c       CASE CHange
c     Change the case of character string CSTR to upper, lower of edit case .
c
c Arguments
c  character cstr*(*),         # operand character string               INPUT
c            case*1            # 'l' or 'L' - change to lower case      INPUT
c                              # 'u' or 'U' - change to upper case
c                              # 'e' or 'E' - change to edit case
c                              # anyting else - no change
c Variables
c  integer i,       # indexing counter
c          nchars   # number of characters to process
c  logical first    # first letter in a word
c
      implicit none
      integer i, nchars
      logical first
      character cstr*(*), case*1
c
      casech = cstr
      nchars = min( len( casech ), len( cstr ) )
c
c         change to lower case
c
      if ( case .eq. 'l' .or. case .eq. 'L' ) then
        do i = 1, nchars
          if ( 'A' .le. casech(i:i) .and. casech(i:i) .le. 'Z' )
     1      casech(i:i) = char( ior( '20'X, ichar( casech(i:i) ) ) )
          end do
c
c         change to upper case
c
      else if ( case .eq. 'u' .or. case .eq. 'U' ) then
        do i = 1, nchars
          if ( 'a' .le. casech(i:i) .and. casech(i:i) .le. 'z' )
     1      casech(i:i) = char( iand( 'ffffffdf'X, ichar( casech(i:i))))
          end do
c
c         change to edit case
c
      else if ( case .eq. 'e' .or. case .eq. 'E' ) then
        first = .true.
        do i = 1, nchars
          if ( 'A' .le. casech(i:i) .and. casech(i:i) .le. 'Z' ) then
            if ( .not. first )
     1         casech(i:i) = char( ior( '20'X, ichar( casech(i:i) ) ) )
            first = .false.
          else if ( 'a'.le. casech(i:i) .and. casech(i:i) .le.'z' ) then
            if ( first )
     1        casech(i:i) = char( iand( 'ffffffdf'X, ichar( casech(i:i)
     2          ) ) )
            first = .false.
          else
            first = .true.
            end if
          end do
        end if
      return
      end



      CHARACTER *32 FUNCTION CBINARY( ITEM, NBITS )
c
c       Converts a 4 byte item into its its corresponding binary bit pattern
c
c Arguments
c  integer *4 item      # data item to be encoded
c  integer *2 nbits     # number of bits to be encoded
c
c Constant
c  Hnbits=32            # max no of bits in data item
c Variables
c  integer *2 i,        # looping index
c  integer *4 imask     # bit mask
c
      implicit none
      integer *2 Hnbits
      parameter( Hnbits=32 )
c
      integer *4 item, imask
      integer *2 nbits, i
c
      do 1 i = 1, nbits
1       Cbinary(i:i) = '0'
      do 2 i = nbits + 1, Hnbits
2       Cbinary(i:i) = ' '
      imask = 1
      do 3 i = nbits, 1, -1
        if ( iand( item, imask ) .ne. 0 )
     1    Cbinary(i:i) = '1'
3       imask = ishft( imask, 1 )
      return
      end



      CHARACTER *5 FUNCTION CYES( BOOLEAN )
c
c Returns a character string depending on the value of BOOLEAN :
c   'no   '  for  NO  or 0,
c   'yes  '       YES or 1,
c   'query'       QUERY or 2,
c   '????? '      anything else
c
      include 'stdio.def'
      integer boolean
c
      if ( boolean .eq. NO ) then                                         1b
          Cyes = 'no   '
      else if ( boolean .eq. YES ) then                                   1m
          Cyes = 'yes  '
      else if ( boolean .eq. QUERY ) then                                 1m
          Cyes = 'query'
      else                                                                1m
          Cyes = '?????'
          end if                                                          1e
      return
      end



      SUBROUTINE CRESP( LUN, STRING, N_TRIES )
c
c Get a character string from file with logical unit number LUN into STRING .
c
c Arguments
c  integer   lun          # IN  # logical unit number of input file
c  character string*(*)   # OUT # destination string
c  integer   n_tries      # IN  # maximum number of attempts
c
c Local variables
c  integer count          # attempts counter
c
      include 'stdio.def'
      integer lun, n_tries, count
      character string*(*)
c
      do 4 count = 1, n_tries
          read( lun, 1, ERR=2, END=5 ) string
1         format( a )
          return
2         write( STD_ERR, 3 )
3         format( ' CRESP : WARNING : Cannot read input' )
4         continue
c
5     write( STD_ERR, 6 )
6     format( ' Abort' )
      stop
      end

