c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

c
c               *************************************
c               ***                               ***
c               ***   GRAPHICAL EDITING OF DATA   ***
c               ***                               ***
c               *************************************
c

      SUBROUTINE SFUDGE( X, Y, NINCR, NPTSIN, NPTSOUT, MAX_POINTS )
c
c       Plots the NPTSIN (x,y) data contained in the ICOLX & ICOLY columns
c of the NROWS by NCOLS array X . NINCR is the indexing increment for the
c row number . You may move the crosshairs and then
c   a) Press `E' or `Q' to exit .
c   b) Place crosshairs near a data point and press `D' to delete, `S' to
c      shift or `R' to read it .
c   c) Append a data point located at the crosshair to the array by
c      pressing `A' .
c   d) Press `U' to undo a Delete, Shift or Append .
c   e) Press `W'' to locate one corner of a selection window. Move the
c      crosshairs to the opposite corner of the window and then press
c      `D', `S', or `R' to delete, shift or read every data point in
c      the selection window .
c
c Arguments
c    real    x(*), y(*)  # data arrays containing (x,y) points      INPUT
c    integer nincr   # data indexing increment                      INPUT
c            nptsin  # number of data points initially supplied     INPUT
c            nptsout # number of data points on exit from SFUDG2    OUTPUT
c         max_points # maximum number of data points for which      INPUT
c                    # array space is available
c
c Constants
c   MAX_LIST = 256   # Maximum number of points found in a window
c   MIN_REAL = 1.0e-36
c   MAX_REAL = 1.0e36
c   MAX_TRIES= 4
c
c Variables
c  logical
c    got_list        # a list of selected data points is available
c    got_message     # message is available for printing after menu
c    got_wind_corner # first corner of the selection window has been marked
c    Equery          # function : read logical
c    fudge_find      #          : find the data point closest to (x,y)
c    fudge_read      #          : read crosshairs location in window  coords
c    fudge_w2d       #          : convert from window to data coord system
c    fudge_wind      #          : find all data points inside window
c    fudge_wcorner   #          : draw or remove window corner
c  integer
c    found_list(MAX_LIST) # list of selected data point row numbers
c    index            # data array row number
c    isym            # plotting symbol
c    list_len        # number of selected data points
c    list_ptr        # index into list of selected point row numbers
c    nlast           # array index of last data point
c    ref_point       # row index of reference point for shifting data
c    str_length      # function : non-blank length of string
c  real
c    hair_x,   hair_y   # crosshairs location
c    shift_x,  shift_y  # coordinate translation for `shift'
c    tot_sh_x, tot_sh_y # total accumulated coordinate translation for `shift'
c    wind_x,   wind_y   # location of selection window corner
c    x_save,   y_save   # buffer for saving a row of the data array
c    x_offset, y_offset # typed translation for selected data
c    x_scale,  y_scale  # typed scale factor for selected data
c    Qresp              # function : read real
c  character
c    cjunk*1         # unused key pressed
c    command*1       # edit command -- key pressed
c    prev_cmd*1      # previous edit command -- key pressed
c    message*32      # error or warning message to user
c  subroutine
c    ilgdata,        # ILGLIB : plot data
c    ilgline,        # ILGLIB : set line styles
c    ilgwait,        # ILGLIB : wait for <RETURN>
c    anmode,         # HPPREV : alpha-numeric mode
c    bell,           # HPPREV : ring bell
c    invis,          # HPPREV : invisible graphics
c    newscr,         # HPPREV : new alpha-numeric screen
c    vecmod,         # HPPREV : vector graphics mode
c    visibl,         # HPPREV : visible graphics
c
      include 'stdio.def'
      integer    MAX_LIST,         MAX_TRIES
      real       MIN_REAL,         MAX_REAL
      parameter( MAX_LIST=256,     MAX_TRIES=4,
     1           MIN_REAL=1.0e-36, MAX_REAL=1.0e36 )
c
      logical got_list, got_message, got_wind_corner,
     1        Equery, fudge_find, fudge_read, fudge_w2d, fudge_wind,
     2        fudge_wcorner
      integer nincr, nptsin, nptsout, max_points, found_list(MAX_LIST),
     1        index, isym, list_len, list_ptr, nlast, ref_point,
     2        str_length
      real    x(*), y(*), hair_x, hair_y, shift_x, shift_y,
     1        tot_sh_x, tot_sh_y, wind_x, wind_y, x_save, y_save,
     2        x_offset, y_offset, x_scale, y_scale, Qresp
      character cjunk*1, command*1, prev_cmd*1, message*32
c
c
      if ( nincr      .le. 0  .or.  nptsin     .le. 0 .or.
     1     max_points .le. 0  .or.  max_points .lt. nptsin )
     2    return
      nptsout = nptsin
      nlast = ( nptsout - 1 ) * nincr + 1
      if ( nptsout .le. 10 ) then                                         1b
        isym = 1
      else if ( nptsout .le. 100 ) then                                   1m
        isym = 5
      else                                                                1m
        isym = 0
        end if                                                            1e
      call ilgline( isym, 0, 1 )
c
c         Plot the data & set up graphics constants
c
      call ilgdata( x, y, nincr, nptsout, 0 )
c
c  *****  main loop
c
      got_wind_corner = .FALSE.
      got_list = .TRUE.
      got_message = .FALSE.
      message  = ' '
      command  = ' '
      prev_cmd = ' '
c
c     start of loop                                                       1b
c
c       prompt for and get the edit command
c
1         call newscr()
          write( STD_OUT, 2 )
2         format( ' SFUDG2:  A=Append D=Delete E=Exit H=Help R=Read ',
     1        'S=Shift T=Type U=Undo W=Window' )
          if ( got_message ) then
              call bell()                                                 2b
              write( STD_OUT, 3 ) message(:max(1,str_length(message)))
3             format( ' SFUDGE : ', a )
              got_message = .FALSE.
              message = ' '
              end if                                                      2e
          if ( got_wind_corner ) then                                     2b
              write( STD_OUT, 4 )
4             format( ' SFUDGE: Place x-hairs at opposite ',
     1                'window corner; press command key D, R, S or T' )
              end if                                                      2e
          if ( .not. fudge_read( hair_x, hair_y, command ) ) then         2b
              write( STD_ERR, 6 )
6             format( ' SFUDGE : ABORT : I give up !' )
              return
              end if                                                      2e
c         change the character pressed to upper case
          if ( 'a' .le. command .and. command .le. 'z' )
     1        command = char( ichar( command ) +
     2                  ichar( 'A' ) - ichar( 'a' ) )
c
c       **** EXIT ****
c
          if ( command .eq. 'Q' .or. command .eq. 'E' ) then              2b
              call newscr()
              if ( Equery( STD_IN, ' SFUDGE : Exit : Are you sure ?' ) )
     1                then                                                3b
                  if ( got_wind_corner )
     1                got_wind_corner = fudge_wcorner( wind_x, wind_y )
                  return
                  end if                                                  3e
c
c       **** DELETE ****
c
          else if ( command .eq. 'D' ) then                               2m
c
c       identify the points to be deleted
c
              if ( got_wind_corner ) then                                 3b
                  got_list = fudge_wind( wind_x, wind_y, hair_x, hair_y,
     1                                  x, y, nincr, nptsout,
     2                                  found_list, list_len, MAX_LIST )
              else                                                        3m
                  got_list = fudge_find( hair_x, hair_y, x, y,
     1                                   nincr, nptsout, found_list(1) )
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
                  call vecmod
                  call invis
                  call anmode
                  do 15 list_ptr = 1, list_len                            4b
c
c                     save the coordinate to be deleted
                      x_save = x( found_list(list_ptr) )
                      y_save = y( found_list(list_ptr) )
c                     remove the data point from the screen
                      call ilgdata( x_save, y_save, 1, 1, 0 )
c                     bump all the following rows up one
                      do 12 index = found_list(list_ptr) + 1, nlast       5b
                          x(index-1) = x(index)
                          y(index-1) = y(index)
12                        continue                                       5e
c                     copy the deleted row to just beyond the table
                      x(nlast) = x_save
                      y(nlast) = y_save
                      nptsout = nptsout - 1
                      nlast = nlast - nincr
c                     adjust (decrement) the entries in the selection list
c                     which have not yet been deleted and are below the row
c                     just deleted
                      do 14 index = list_ptr + 1, list_len                 5b
                          if ( found_list(index) .gt.
     1                            found_list(list_ptr) )
     2                        found_list(index) = found_list(index) - 1
14                        continue                                        5e
15                    continue                                            4e
                  call vecmod
                  call visibl
                  call anmode
                  prev_cmd = 'D'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** Shift ****
c            - move a data point
c
          else if ( command .eq. 'S' ) then                               2m
c
c       identify the points to be shifted
c         if shifting a window, a reference point must be selected;
c         for shifting a single point, the shifted point IS the reference
c
              if ( got_wind_corner ) then                                 3b
                  got_list = .FALSE.
                  if ( fudge_wind( wind_x, wind_y, hair_x, hair_y,
     1                    x, y, nincr, nptsout,
     2                    found_list, list_len, MAX_LIST ) ) then         4b
                      write( STD_OUT, 20 )
20                    format( / ' SFUDGE : Locate reference point.' )
                      if ( fudge_read( hair_x, hair_y, cjunk ) ) then     5b
                          got_list = fudge_find( hair_x, hair_y,
     1                        x, y, nincr, nptsout, ref_point )
                      else                                                5m
                          write( STD_ERR, 3 )
                          return
                          end if                                          5e          2e
                      end if                                              4e
              else                                                        3m
                  got_list = fudge_find( hair_x, hair_y, x, y,
     1                                   nincr, nptsout, found_list(1) )
                  ref_point = found_list(1)
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
c
c       Locate the final position of the reference point
c         and convert it to data coordinate system
c
                  write( STD_OUT, 21 )
21                format( / ' SFUDGE : Locate final position of ',
     1                    'reference point : M=Move F=Finished ' )
                  tot_sh_x = 0
                  tot_sh_y = 0
c
c                 start of loop
22                if ( fudge_read( hair_x, hair_y, cjunk ) ) then         4b
                      if ( cjunk .eq. 'M' .or. cjunk .eq. 'm' ) then      5b
                          if ( fudge_w2d( hair_x, hair_y,
     1                                    shift_x, shift_y ) ) then       6b
c
c       calculate the amount of translation
c
                              shift_x = shift_x - x(ref_point)
                              tot_sh_x = tot_sh_x + shift_x
                              shift_y = shift_y - y(ref_point)
                              tot_sh_y = tot_sh_y + shift_y
c
c       remove the selected points from the screen
c
                              call vecmod
                              call invis
                              call anmode
                              do 23 list_ptr = 1, list_len                7b
                                  index = found_list(list_ptr)
                                  call ilgdata( x(index), y(index),
     1                                          1, 1, 0)
23                                continue                                7e
                              call vecmod
                              call visibl
                              call anmode
c
c       put the selected points in their new positions
c
                              do 24 list_ptr = 1, list_len                7b
                                  index = found_list(list_ptr)
                                  x(index) = x(index) + shift_x
                                  y(index) = y(index) + shift_y
                                  call ilgdata( x(index), y(index),
     1                                          1, 1, 0 )
24                                continue                                7e
                              end if                                      6e
                          goto 22
                      else if ( cjunk .ne. 'F' .and. cjunk .ne. 'f' )
     1                        then                                        5m
                          goto 22
                          end if                                          5e
                      end if                                              4e
                  prev_cmd = 'S'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** READ ****
c           Read & display the data point
c
          else if ( command .eq. 'R' ) then                               2m
c
c       identify the points to be read
c
              if ( got_wind_corner ) then                                 3b
                  got_list = fudge_wind( wind_x, wind_y, hair_x, hair_y,
     1                                  x, y, nincr, nptsout,
     2                                  found_list, list_len, MAX_LIST )
              else                                                        3m
                  got_list = fudge_find( hair_x, hair_y, x, y,
     1                                   nincr, nptsout, found_list(1) )
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
                  write( STD_OUT, 30 )
30                format( ' SFUDGE : Read' /
     1                    ' no. row     abscissa      ordinate' )
                  do 32 list_ptr = 1, list_len                            4b
                      index = found_list(list_ptr)
                      write( STD_OUT, 31 ) list_ptr, index,
     1                    x(index), y(index)
31                    format( 1x, i3, 1x, i3, 2g14.6 )
                      if ( mod( list_ptr, 16 ) .eq. 0 .or.
     1                     list_ptr .eq. list_len )
     2                    call ilgwait()
32                    continue                                            4e
                  prev_cmd = 'R'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** APPEND ****
c
          else if ( command .eq. 'A' ) then                               2m
c
c       cannot append if all available array space is already used
c
              if ( nptsout .ge. max_points ) then                         3b
                  got_message = .TRUE.
                  message = 'Cannot append. Array space full.'
                  prev_cmd = ' '
c
c       cannot append more then one point at a time, so no window
c
              else if ( .not. got_wind_corner ) then                      3m
c
c       convert the crosshair location from window to data coordinate
c       system, plot it and finally add it to the data array
c
                  got_list = fudge_w2d( hair_x, hair_y, hair_x, hair_y )
                  if ( got_list ) then                                    4b
                      call ilgdata( hair_x, hair_y, 1, 1, 0 )
                      nptsout = nptsout + 1
                      nlast = nlast + nincr
                      x(nlast) = hair_x
                      y(nlast) = hair_y
                      end if                                              4e      3e
                  prev_cmd = 'A'
              else
c                 remove the first window corner markers and post message
                  got_wind_corner = fudge_wcorner( wind_x, wind_y )
                  got_message = .TRUE.
                  message = 'Cannot append window.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       **** TYPE ****
c       multiply by scale factor or translate
c
          else if ( command .eq. 'T' ) then                               2m
c
c       identify the points to be scaled or shifted
c
              if ( got_wind_corner ) then                                 3b
                  got_list = fudge_wind( wind_x, wind_y, hair_x, hair_y,
     1                                  x, y, nincr, nptsout,
     2                                  found_list, list_len, MAX_LIST )
              else                                                        3m
                  got_list = fudge_find( hair_x, hair_y, x, y,
     1                                   nincr, nptsout, found_list(1) )
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
c
c       set default 'do nothing' scale factors & offsets
c
                  x_scale  = 1.0
                  y_scale  = 1.0
                  x_offset = 0.0
                  y_offset = 0.0
                  write( STD_OUT, 50 )
50                format( / ' SFUDGE : F=scale-factor O=offset')
                  if ( fudge_read( hair_x, hair_y, command ) ) then       4b
                      if ( command .eq. 'F' ) then                        5b
c
c       get scale factors from user
c
                          write( STD_OUT, 51 ) 'X'
51                        format( / ' Type ', a, ' scale factor ', $ )
                          x_scale = Qresp( STD_IN,
     1                         MIN_REAL, MAX_REAL, MAX_TRIES )
                          write( STD_OUT, 51 ) 'Y'
                          y_scale = Qresp( STD_IN,
     2                         MIN_REAL, MAX_REAL, MAX_TRIES )
                      else if ( command .eq. 'O' ) then
c                                                                         5m
c       get offsets from user
c
                          write( STD_OUT, 52 ) 'X'
52                        format( / ' Type ', a, ' offset ', $ )
                          x_offset = Qresp( STD_IN,
     1                        -MAX_REAL, MAX_REAL, MAX_TRIES )
                          write( STD_OUT, 52 ) 'Y'
                          y_offset = Qresp( STD_IN,
     1                        -MAX_REAL, MAX_REAL, MAX_TRIES )
                          end if                                          5e
c
c       remove the selected points from the screen
c
                      call vecmod
                      call invis
                      call anmode
                      do 53 list_ptr = 1, list_len                        5b
                          index = found_list(list_ptr)
                          call ilgdata( x(index), y(index), 1, 1, 0 )
53                        continue                                        5e
                      call vecmod
                      call visibl
                      call anmode
c
c       put the selected points in their new positions
c
                      do 54 list_ptr = 1, list_len                        5b
                          index = found_list(list_ptr)
                          x(index) = x(index) * x_scale + x_offset
                          y(index) = y(index) * y_scale + y_offset
                          call ilgdata( x(index), y(index), 1, 1, 0 )
54                        continue                                        5e
                      end if                                              4e
                  prev_cmd = 'T'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** UNDO ****
c
          else if ( command .eq. 'U' ) then                               2m
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** un - delete
c
              if ( prev_cmd .eq. 'D' ) then                               3b
                  do 114 list_ptr = list_len, 1, -1                       4b
c                     adjust (increment) the entries in the selection list
c                     which have been undeleted and are below the row
c                     just deleted

                      do 115 index = list_ptr + 1, list_len                5b
                          if ( found_list(index) .ge.
     1                            found_list(list_ptr) )
     2                        found_list(index) = found_list(index) + 1
115                       continue                                        5e
c
c                     save the row just beyond the table
                      x_save = x( nlast + nincr )
                      y_save = y( nlast + nincr )
c                     plot the data point
                      call ilgdata( x_save, y_save, 1, 1, 0 )
c                     bump all the following rows down one
                      do 112 index = nlast, found_list(list_ptr), -1       5b
                          x(index+1) = x(index)
                          y(index+1) = y(index)
112                       continue                                        5e
c                     copy the deleted row to its correct place
                      x(found_list(list_ptr)) = x_save
                      y(found_list(list_ptr)) = y_save
                      nptsout = nptsout + 1
                      nlast = nlast + nincr
114                   continue                                            4e
c
c       **** un - shift
c
              else if ( prev_cmd .eq. 'S' ) then                          3m
c
c         remove the selected points from their new positions on the screen
c
                  call vecmod
                  call invis
                  call anmode
                  do 120 list_ptr = 1, list_len                           4b
                      index = found_list(list_ptr)
                      call ilgdata( x(index), y(index), 1, 1, 0 )
120                   continue                                            4e
                  call vecmod
                  call visibl
                  call anmode
c
c       put the selected points back in their original positions
c
                  do 121 list_ptr = 1, list_len                           4b
                      index = found_list(list_ptr)
                      x(index) = x(index) - tot_sh_x
                      y(index) = y(index) - tot_sh_y
                      call ilgdata( x(index), y(index), 1, 1, 0 )
121                   continue                                            4e
c
c       **** un - read
c
              else if ( prev_cmd .eq. 'R' ) then                          3m
                  got_message = .TRUE.
                  message = 'Cannot undo "read".'
c
c       **** un - append
c
              else if ( prev_cmd .eq. 'A' ) then                          3m
                  call vecmod
                  call invis
                  call anmode
                  call ilgdata( x(nlast), y(nlast), 1, 1, 0)
                  call vecmod
                  call visibl
                  call anmode
                  nptsout = nptsout - 1
                  nlast = nlast - nincr
c
c       **** un - type
c
              else if ( prev_cmd .eq. 'T' ) then                          3m
c
c       remove the selected points from new locations on the screen
c
                  call vecmod
                  call invis
                  call anmode
                  do 151 list_ptr = 1, list_len                           4b
                      index = found_list(list_ptr)
                      call ilgdata( x(index), y(index), 1, 1, 0 )
151                   continue                                            4e
                  call vecmod
                  call visibl
                  call anmode
c
c       put the selected points in their old positions
c
                  do 152 list_ptr = 1, list_len                           4b
                      index = found_list(list_ptr)
                      x(index) = ( x(index) - x_offset ) / x_scale
                      y(index) = ( y(index) - y_offset ) / y_scale
                      call ilgdata( x(index), y(index), 1, 1, 0 )
152                   continue                                            4e
c
c       **** un - undo
c
              else if ( prev_cmd .eq. 'U' ) then                          3b
                  got_message = .TRUE.
                  message = 'Do it again. Can''t undo "undo".'
                  end if                                                  3e
              prev_cmd = 'U'
              got_list = .TRUE.
c
c       **** WINDOW ****
c
          else if ( command .eq. 'W' ) then                               2m
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c             draw the first window marker as another cross hair
              wind_x = hair_x
              wind_y = hair_y
c             tell the next command to operate on a window
              got_wind_corner = fudge_wcorner( wind_x, wind_y )
              got_list = .TRUE.
c
c       **** HELP ****
c
          else if ( command .eq. 'H' ) then                               2m
              call newscr()
              write( STD_OUT, 70 )
70            format( ' SFUDGE : Help : Data array editor.'
     1          ' You may move the crosshairs and then' /
     2          ' a) Press `E'' or `Q'' to exit.' /
     3          ' b) Place crosshairs near a data point and press' /
     4          '    `D'' to delete, `S'' to shift or `R'' to read it.'/
     5          ' c) Append a data point located at the crosshair to' /
     6          '    the array by pressing `A''.'
     7          ' d) Press `U'' to undo a Delete, Shift or Append.' /
     8          ' e) Press `W'' to locate one corner of a selection' /
     9          '    window. Move the crosshairs to the opposite' /
     a          '    corner of the window and then press `D'', `S''' /
     b          '    or `R'' to delete, shift or read every data' /
     c          '    point in the selection window.' )
              call ilgwait()
              got_list = .TRUE.
              end if                                                      2e
          goto 1
c         end of loop
      end



      LOGICAL FUNCTION FUDGE_FIND( X_HAIR, Y_HAIR,
     1                             X_ARRAY, Y_ARRAY, INCREM, N_POINTS,
     2                             FOUND_INDEX )
c
c   Find data point in X_ARRAY and Y_ARRAY nearest to the crosshairs location .
c The returned value in FOUND_INDEX is the row index of this point in ARRAY
c and the returned value of FUDGE_FIND is .TRUE. . If no data point is found
c closer than RESOLUTION * SCREEN_SIZE, the returned value of FUDGE_FIND is
c .FALSE. and FOUND_INDEX = NOT_FOUND (=-1) .
c
c Arguments
c  real x_hair          # abscissa of crosshairs location
c       y_hair          # ordinate of crosshairs location
c       x_array(*)      # data array
c  integer increm       # row increment
c          n_points     # number of data points
c          found_index  # location of the found data point
c
c constants
c  NOT_FOUND  = -1
c  MAX_REAL   = 1.0e37
c  RESOLUTION = 0.015
c
c local variables
c  integer
c    i          # loop index
c    n_last     # row number of the last data point
c  logical
c    fudge_d2w  # function : convert from data to window coordinate system
c  real
c    distance   # distance from data point to crosshairs squared
c    min_dist   # minimum (distance **2)
c    rpagsizya  # adjusted vertical page dimension (window units)
c    x_wind     # data point abscissa in window coordinates
c    y_wind     # data point ordinate in window coordinates
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
      integer NOT_FOUND
      real    MAX_REAL, RESOLUTION
      parameter( NOT_FOUND=-1, MAX_REAL=1.0e37, RESOLUTION=0.015 )
c
      integer increm, n_points, found_index, i, n_last
      logical fudge_d2w
      real    x_hair, y_hair, x_array(*), y_array(*), distance,
     1        min_dist, rpagsizya, x_wind, y_wind
c
      min_dist = MAX_REAL
      found_index = NOT_FOUND
      n_last = ( n_points - 1 ) * increm + 1
      rpagsizya = 780.0 / 1024.0 * rpagsizx
      do 1 i = 1, n_last, increm                                          1b
c
c           convert data to window coordinates
c
          if ( fudge_d2w( x_array(i), y_array(i), x_wind, y_wind )) then  2b
c
c            calculate distance
c
              distance = ( ( x_wind - x_hair ) / rpagsizx  ) **2 +
     1                   ( ( y_wind - y_hair ) / rpagsizya ) **2
              if ( distance .lt. min_dist ) then                          3b
                  min_dist = distance
                  found_index = i
                  end if                                                  3e
              end if                                                      2e
1         continue                                                        1e
      fudge_find = found_index .ne. NOT_FOUND .and.
     1             min_dist .le. RESOLUTION **2
      if ( .not. fudge_find )
     1    found_index = NOT_FOUND
      return
      end



      LOGICAL FUNCTION FUDGE_WIND( LL_X, LL_Y, UR_X, UR_Y,
     1                             X_ARRAY, Y_ARRAY, INCREM, N_POINTS,
     3                             FOUND_LIST, LIST_LEN, MAX_LIST )
c
c     List all data points in X_ARRAY and Y_ARRAY which lie within the window
c with lower left corner at (LL_X,LL_Y) and upper right corner at (UR_X, UR_Y).
c The row indices of these points in X_ARRAY and Y_ARRAY are returned in the
c array FOUND_LIST. If no data points are found within the window the returned
c value of FUDGE_WIND is .FALSE. ( otherwise it is .TRUE. ) .
c
c Arguments
c  real ll_x, ll_y      # INPUT  # lower left corner of the window
c       ur_x, ur_y      # INPUT  # upper right corner of the window
c       x_array(*),     # INPUT  # abscissa data array
c       y_array(*)      # INPUT  # ordinate data array
c  integer increm       # INPUT  # row increment
c          n_points     # INPUT  # number of data points
c          found_list(*)# OUTPUT # list of row indices of the point in window
c          list_len     # OUTPUT # number of data points in the window
c          max_list     # INPUT  # size of the array ``found_list''
c
c local variables
c  integer
c    i          # loop index
c    n_last     # row number of the last data point
c  logical
c    fudge_d2w  # function : convert from data to window coordinate system
c  real
c    x_wind     # data point abscissa in window coordinates
c    y_wind     # data point ordinate in window coordinates
c    left, right, top, bottom # edges of the selection window
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
      integer NOT_FOUND
      parameter( NOT_FOUND=-1 )
c
      integer increm, n_points, list_len, max_list, i, n_last,
     1        found_list(*)
      logical fudge_d2w
      real    ll_x, ll_y, ur_x, ur_y, x_array(*), y_array(*),
     1        x_wind, y_wind, left, right, top, bottom
c
      found_list(1) = NOT_FOUND
      list_len = 0
      n_last = ( n_points - 1 ) * increm + 1
c
      left   = min( ll_x, ur_x )
      right  = max( ll_x, ur_x )
      top    = max( ll_y, ur_y )
      bottom = min( ll_y, ur_y )
c
      do 1 i = 1, n_last, increm                                          1b
c
c           convert data to window coordinates
c
          if ( list_len .ge. max_list )
     1        goto 2
          if ( fudge_d2w( x_array(i), y_array(i), x_wind, y_wind )) then  2b
              if ( left   .le. x_wind .and. x_wind .le. right .and.
     1             bottom .le. y_wind .and. y_wind .le. top ) then        3b
                  list_len = list_len + 1
                  found_list(list_len) = i
                  end if                                                  3e
              end if                                                      2e
1         continue                                                        1e
2     fudge_wind = list_len .gt. 0
      return
      end



      SUBROUTINE SFUDG2( X, ICOLX, ICOLY, NROWS, NCOLS, NINCR, NPTSIN,
     1  NPTSOUT )
c
c       Plots the NPTSIN (x,y) data contained in the ICOLX & ICOLY columns
c of the NROWS by NCOLS array X . NINCR is the indexing increment for the
c row number . You may move the crosshairs and then
c   a) Press `E' or `Q' to exit .
c   b) Place crosshairs near a data point and press `D' to delete, `S' to
c      shift or `R' to read it .
c   c) Append a data point located at the crosshair to the array by
c      pressing `A' .
c   d) Press `U' to undo a Delete, Shift or Append .
c   e) Press `W'' to locate one corner of a selection window. Move the
c      crosshairs to the opposite corner of the window and then press
c      `D', `S', or `R' to delete, shift or read every data point in
c      the selection window .
c
c Arguments
c    real    x(nrows,ncols)  # data arrays containing (x,y) points  INPUT
c    integer icolx   # column for abscissa values                   INPUT
c            icoly   # column for ordinate values                   INPUT
c            nrows   # number of rows in data matrix                INPUT
c            ncols   # number of columns in data matrix             INPUT
c            nincr   # data indexing increment                      INPUT
c            nptsin  # number of data points initially supplied     INPUT
c            nptsout # number of data points on exit from SFUDG2    OUTPUT
c
c Constants
c   MAX_LIST = 256   # Maximum number of points found in a window
c   MIN_REAL = 1.0e-36
c   MAX_REAL = 1.0e36
c   MAX_TRIES= 4
c
c Variables
c  logical
c    got_list        # a list of selected data points is available
c    got_message     # message is available for printing after menu
c    got_wind_corner # first corner of the selection window has been marked
c    Equery          # function : read logical
c    fudg2_find      #          : find the data point closest to (x,y)
c    fudge_read      #          : read crosshairs location in window  coords
c    fudge_w2d       #          : convert from window to data coord system
c    fudg2_wind      #          : find all data points inside window
c    fudge_wcorner   #          : draw or remove window corner
c  integer
c    found_list(MAX_LIST) # list of selected data point row numbers
c    icol            # data array column number
c    irow            # data array row number
c    isym            # plotting symbol
c    list_len        # number of selected data points
c    list_ptr        # index into list of selected point row numbers
c    nlast           # array index of last data point
c    ref_point       # row index of reference point for shifting data
c    str_length      # function : non-blank string length
c  real
c    hair_x,   hair_y   # crosshairs location
c    shift_x,  shift_y  # coordinate translation for `shift'
c    tot_sh_x, tot_sh_y # total accumulated coordinate translation for `shift'
c    wind_x,   wind_y   # location of selection window corner
c    x_save(MAX_COLS)   # buffer for saving a row of the data array
c    x_offset, y_offset # typed translation for selected data
c    x_scale,  y_scale  # typed scale factor for selected data
c    Qresp              # function : read real
c  character
c    cjunk*1         # unused key pressed
c    command*1       # edit command -- key pressed
c    prev_cmd*1      # previous edit command -- key pressed
c    message*32      # error or warning message to user
c  subroutine
c    ilgdata,        # ILGLIB : plot data
c    ilgline,        # ILGLIB : set line styles
c    ilgwait,        # ILGLIB : wait for <RETURN>
c    anmode,         # HPPREV : alpha-numeric mode
c    invis,          # HPPREV : invisible graphics
c    newscr,         # HPPREV : new alpha-numeric screen
c    vecmod,         # HPPREV : vector graphics mode
c    visibl,         # HPPREV : visible graphics
c
      include 'stdio.def'
      integer    MAX_COLS,         MAX_LIST,        MAX_TRIES
      real       MIN_REAL,         MAX_REAL
      parameter( MAX_COLS=64,      MAX_LIST=256,    MAX_TRIES=4,
     1           MIN_REAL=1.0e-36, MAX_REAL=1.0e36 )
c
      logical got_list, got_message, got_wind_corner,
     1        Equery, fudg2_find, fudge_read, fudge_w2d, fudg2_wind,
     2        fudge_wcorner
      integer icolx, icoly, nrows, ncols, nincr, nptsin, nptsout,
     1        found_list(MAX_LIST), icol, irow, isym, list_len,
     2        list_ptr, nlast, ref_point, str_length
      real    x(nrows,ncols), hair_x, hair_y, shift_x, shift_y,
     1        tot_sh_x, tot_sh_y, wind_x, wind_y, x_save(MAX_COLS),
     2        x_offset, y_offset, x_scale, y_scale, Qresp
      character cjunk*1, command*1, prev_cmd*1, message*32
c
c
      if ( nincr .le. 0  .or.  nptsin .le. 0      .or.
     1     icolx .le. 0  .or.  icolx  .gt. ncols  .or.
     2     icoly .le. 0  .or.  icoly  .gt. ncols  .or.
     3     ncols .gt. MAX_COLS )
     4    return
      nptsout = nptsin
      nlast = ( nptsout - 1 ) * nincr + 1
      if ( nptsout .le. 10 ) then                                         1b
        isym = 1
      else if ( nptsout .le. 100 ) then                                   1m
        isym = 5
      else                                                                1m
        isym = 0
        end if                                                            1e
      call ilgline( isym, 0, 1 )
c
c         Plot the data & set up graphics constants
c
      call ilgdata( x(1,icolx), x(1,icoly), nincr, nptsout, 0 )
c
c  *****  main loop
c
      got_wind_corner = .FALSE.
      got_list = .TRUE.
      got_message = .FALSE.
      message  = ' '
      command  = ' '
      prev_cmd = ' '
c
c     start of loop                                                       1b
c
c       prompt for and get the edit command
c
1         call newscr()
          write( STD_OUT, 2 )
2         format( ' SFUDG2:  A=Append D=Delete E=Exit H=Help R=Read ',
     1        'S=Shift T=Type U=Undo W=Window' )
          if ( got_message ) then
              call bell()                                                 2b
              write( STD_OUT, 3 ) message(:max(1,str_length(message)))
3             format( ' SFUDGE : ', a )
              got_message = .FALSE.
              message = ' '
              end if                                                      2e
          if ( got_wind_corner ) then                                     2b
              write( STD_OUT, 4 )
4             format( ' SFUDG2: Place x-hairs at opposite ',
     1                'window corner; press command key D, R, S or T' )
              end if                                                      2e
          if ( .not. fudge_read( hair_x, hair_y, command ) ) then         2b
              write( STD_ERR, 6 )
6             format( ' SFUDG2 : ABORT : I give up !' )
              return
              end if                                                      2e
c         change the character pressed to upper case
          if ( 'a' .le. command .and. command .le. 'z' )
     1        command = char( ichar( command ) +
     2                  ichar( 'A' ) - ichar( 'a' ) )
c
c       **** EXIT ****
c
          if ( command .eq. 'Q' .or. command .eq. 'E' ) then              2b
              call newscr()
              if ( Equery( STD_IN, ' SFUDG2 : Exit : Are you sure ?' ) )
     1                then                                                3b
                  if ( got_wind_corner )
     1                got_wind_corner = fudge_wcorner( wind_x, wind_y )
                  return
                  end if                                                  3e
c
c       **** DELETE ****
c
          else if ( command .eq. 'D' ) then                               2m
c
c       identify the points to be deleted
c
              if ( got_wind_corner ) then                                 3b
                  got_list = fudg2_wind( wind_x, wind_y, hair_x, hair_y,
     1                x, icolx, icoly, nrows, nincr, nptsout,
     2                found_list, list_len, MAX_LIST )
              else                                                        3m
                  got_list = fudg2_find( hair_x, hair_y, x, icolx,
     1                icoly, nrows, nincr, nptsout, found_list(1) )
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
                  call vecmod
                  call invis
                  call anmode
                  do 15 list_ptr = 1, list_len                            4b
c
c                     save the row to be deleted
                      do 10 icol = 1, ncols                               5b
                          x_save(icol) = x( found_list(list_ptr), icol )
10                        continue                                        5e
c                     remove the data point from the screen
                      call ilgdata( x_save(icolx), x_save(icoly),
     1                              1, 1, 0 )
c                     bump all the following rows up one
                      do 12 irow = found_list(list_ptr) + 1, nlast        5b
                           do 11 icol = 1, ncols                          6b
                              x(irow-1,icol) = x(irow,icol)
11                            continue                                    6e
12                         continue                                       5e
c                     copy the deleted row to just beyond the table
                      do 13 icol = 1, ncols                               5b
                          x(nlast,icol) = x_save(icol)
13                        continue                                        5e
                      nptsout = nptsout - 1
                      nlast = nlast - nincr
c                     adjust (decrement) the entries in the selection list
c                     which have not yet been deleted and are below the row
c                     just deleted
                      do 14 irow = list_ptr + 1, list_len                 5b
                          if ( found_list(irow) .gt.
     1                            found_list(list_ptr) )
     2                        found_list(irow) = found_list(irow) - 1
14                        continue                                        5e
15                    continue                                            4e
                  call vecmod
                  call visibl
                  call anmode
                  prev_cmd = 'D'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** Shift ****
c            - move a data point
c
          else if ( command .eq. 'S' ) then                               2m
c
c       identify the points to be shifted
c         if shifting a window, a reference point must be selected;
c         for shifting a single point, the shifted point IS the reference
c
              if ( got_wind_corner ) then                                 3b
                  got_list = .FALSE.
                  if ( fudg2_wind( wind_x, wind_y, hair_x, hair_y,
     1                    x, icolx, icoly, nrows, nincr, nptsout,
     2                    found_list, list_len, MAX_LIST ) ) then         4b
                      write( STD_OUT, 20 )
20                    format( / ' SFUDG2 : Locate reference point.' )
                      if ( fudge_read( hair_x, hair_y, cjunk ) ) then     5b
                          got_list = fudg2_find( hair_x, hair_y,
     1                        x, icolx, icoly, nrows, nincr, nptsout,
     2                        ref_point )
                      else                                                5m
                          write( STD_ERR, 3 )
                          return
                          end if                                          5e          2e
                      end if                                              4e
              else                                                        3m
                  got_list = fudg2_find( hair_x, hair_y, x, icolx,
     1                icoly, nrows, nincr, nptsout, found_list(1) )
                  ref_point = found_list(1)
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
c
c       Locate the final position of the reference point
c         and convert it to data coordinate system
c
                  write( STD_OUT, 21 )
21                format( / ' SFUDG2 : Locate final position of ',
     1                    'reference point : M=Move F=Finished ' )
                  tot_sh_x = 0
                  tot_sh_y = 0
c
c                 start of loop
22                if ( fudge_read( hair_x, hair_y, cjunk ) ) then         4b
                      if ( cjunk .eq. 'M' .or. cjunk .eq. 'm' ) then      5b
                          if ( fudge_w2d( hair_x, hair_y,
     1                                    shift_x, shift_y ) ) then       6b
c
c       calculate the amount of translation
c
                              shift_x = shift_x - x(ref_point,icolx)
                              tot_sh_x = tot_sh_x + shift_x
                              shift_y = shift_y - x(ref_point,icoly)
                              tot_sh_y = tot_sh_y + shift_y
c
c       remove the selected points from the screen
c
                              call vecmod
                              call invis
                              call anmode
                              do 23 list_ptr = 1, list_len                7b
                                  irow = found_list(list_ptr)
                                  call ilgdata( x(irow,icolx),
     1                                          x(irow,icoly), 1, 1, 0)
23                                continue                                7e
                              call vecmod
                              call visibl
                              call anmode
c
c       put the selected points in their new positions
c
                              do 24 list_ptr = 1, list_len                7b
                                  irow = found_list(list_ptr)
                                  x(irow,icolx) = x(irow,icolx) +shift_x
                                  x(irow,icoly) = x(irow,icoly) +shift_y
                                  call ilgdata( x(irow,icolx),
     1                                          x(irow,icoly), 1, 1, 0 )
24                                continue                                7e
                              end if                                      6e
                          goto 22
                      else if ( cjunk .ne. 'F' .and. cjunk .ne. 'f' )
     1                        then                                        5m
                          goto 22
                          end if                                          5e
                      end if                                              4e
                  prev_cmd = 'S'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** READ ****
c           Read & display the data point
c
          else if ( command .eq. 'R' ) then                               2m
c
c       identify the points to be read
c
              if ( got_wind_corner ) then                                 3b
                  got_list = fudg2_wind( wind_x, wind_y, hair_x, hair_y,
     1                x, icolx, icoly, nrows, nincr, nptsout,
     2                found_list, list_len, MAX_LIST )
              else                                                        3m
                  got_list = fudg2_find( hair_x, hair_y, x, icolx,
     1                icoly, nrows, nincr, nptsout, found_list(1) )
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
                  write( STD_OUT, 30 )
30                format( ' SFUDG2 : Read' /
     1                    ' no. row     abscissa      ordinate' )
                  do 32 list_ptr = 1, list_len                            4b
                      irow = found_list(list_ptr)
                      write( STD_OUT, 31 ) list_ptr, irow,
     1                    x(irow,icolx), x(irow,icoly)
31                    format( 1x, i3, 1x, i3, 2g14.6 )
                      if ( mod( list_ptr, 16 ) .eq. 0 .or.
     1                     list_ptr .eq. list_len )
     2                    call ilgwait()
32                    continue                                            4e
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
              prev_cmd = 'R'

c
c       **** APPEND ****
c
          else if ( command .eq. 'A' ) then                               2m
c
c       cannot append if all available array space is already used
c
              if ( ( nptsout - 1 ) * nincr + 1 .ge. nrows ) then          3b
                  got_message = .TRUE.
                  message = 'Cannot append. Array space full.'
                  prev_cmd = ' '
c
c       connot append more then one point at a time, so no window
c
              else if ( .not. got_wind_corner ) then                      3m
c
c       convert the crosshair location from window to data coordinate
c       system, plot it and finally add it to the data array
c
                  got_list = fudge_w2d( hair_x, hair_y, hair_x, hair_y )
                  if ( got_list ) then                                    4b
                      call ilgdata( hair_x, hair_y, 1, 1, 0 )
                      nptsout = nptsout + 1
                      nlast = nlast + nincr
                      do 40 icol = 1, ncols                               5b      4b
                          x(nlast,icol) = 0.0
40                        continue                                        5e      4e
                      x(nlast,icolx) = hair_x
                      x(nlast,icoly) = hair_y
                      end if                                              4e      3e
                  prev_cmd = 'A'
              else                                                        3m
c                 remove the first window corner markers and post message
                  got_wind_corner = fudge_wcorner( wind_x, wind_y )
                  got_message = .TRUE.
                  message = 'Cannot append window.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       **** TYPE ****
c       multiply by scale factor or translate
c
          else if ( command .eq. 'T' ) then                               2m
c
c       identify the points to be scaled or shifted
c
              if ( got_wind_corner ) then                                 3b
                  got_list = fudg2_wind( wind_x, wind_y, hair_x, hair_y,
     1                x, icolx, icoly, nrows, nincr, nptsout,
     2                found_list, list_len, MAX_LIST )
              else                                                        3m
                  got_list = fudg2_find( hair_x, hair_y, x, icolx,
     1                icoly, nrows, nincr, nptsout, found_list(1) )
                  list_len = 1
                  end if                                                  3e
c
              if ( got_list ) then                                        3b
c
c       set default 'do nothing' scale factors & offsets
c
                  x_scale  = 1.0
                  y_scale  = 1.0
                  x_offset = 0.0
                  y_offset = 0.0
                  write( STD_OUT, 50 )
50                format( / ' SFUDG2 : F=scale-factor O=offset')
                  if ( fudge_read( hair_x, hair_y, command ) ) then       4b
                      if ( command .eq. 'F' ) then                        5b
c
c       get scale factors from user
c
                          write( STD_OUT, 51 ) 'X'
51                        format( / ' Type ', a, ' scale factor ', $ )
                          x_scale = Qresp( STD_IN,
     1                         MIN_REAL, MAX_REAL, MAX_TRIES )
                          write( STD_OUT, 51 ) 'Y'
                          y_scale = Qresp( STD_IN,
     2                         MIN_REAL, MAX_REAL, MAX_TRIES )
                      else if ( command .eq. 'O' ) then
c                                                                         5m
c       get offsets from user
c
                          write( STD_OUT, 52 ) 'X'
52                        format( / ' Type ', a, ' offset ', $ )
                          x_offset = Qresp( STD_IN,
     1                        -MAX_REAL, MAX_REAL, MAX_TRIES )
                          write( STD_OUT, 52 ) 'Y'
                          y_offset = Qresp( STD_IN,
     1                        -MAX_REAL, MAX_REAL, MAX_TRIES )
                          end if                                          5e
c
c       remove the selected points from the screen
c
                      call vecmod
                      call invis
                      call anmode
                      do 53 list_ptr = 1, list_len                        5b
                          irow = found_list(list_ptr)
                          call ilgdata( x(irow,icolx),
     1                                  x(irow,icoly), 1, 1, 0)
53                        continue                                        5e
                      call vecmod
                      call visibl
                      call anmode
c
c       put the selected points in their new positions
c
                      do 54 list_ptr = 1, list_len                        5b
                          irow = found_list(list_ptr)
                          x(irow,icolx) = x(irow,icolx) * x_scale +
     1                        x_offset
                          x(irow,icoly) = x(irow,icoly) * y_scale +
     1                        y_offset
                          call ilgdata( x(irow,icolx),
     1                                  x(irow,icoly), 1, 1, 0 )
54                        continue                                        5e
                      end if                                              4e
                  prev_cmd = 'T'
              else                                                        3m
                  got_message = .TRUE.
                  message = 'No data points found.'
                  prev_cmd = ' '
                  end if                                                  3e
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** UNDO ****
c
          else if ( command .eq. 'U' ) then                               2m
c
c       remove the first window corner markers
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c
c       **** un - delete
c
              if ( prev_cmd .eq. 'D' ) then                               3b
                  do 114 list_ptr = list_len, 1, -1                       4b
c                     adjust (increment) the entries in the selection list
c                     which have been undeleted and are below the row
c                     just deleted

                      do 115 irow = list_ptr + 1, list_len                5b
                          if ( found_list(irow) .ge.
     1                            found_list(list_ptr) )
     2                        found_list(irow) = found_list(irow) + 1
115                       continue                                        5e
c
c                     save the row just beyond the table
                      do 110 icol = 1, ncols                              5b
                          x_save(icol) = x( nlast + nincr, icol )
110                       continue                                        5e
c                     plot the data point
                      call ilgdata( x_save(icolx), x_save(icoly),
     1                              1, 1, 0 )
c                     bump all the following rows down one
                      do 112 irow = nlast, found_list(list_ptr), -1       5b
                          do 111 icol = 1, ncols                          6b
                              x(irow+1,icol) = x(irow,icol)
111                           continue                                    6e
112                       continue                                        5e
c                     copy the deleted row to its correct place
                      do 113 icol = 1, ncols                              5b
                          x(found_list(list_ptr),icol) = x_save(icol)
113                       continue                                        5e
                      nptsout = nptsout + 1
                      nlast = nlast + nincr
114                   continue                                            4e
c
c       **** un - shift
c
              else if ( prev_cmd .eq. 'S' ) then                          3m
c
c         remove the selected points from their new positions on the screen
c
                  call vecmod
                  call invis
                  call anmode
                  do 120 list_ptr = 1, list_len                           4b
                      irow = found_list(list_ptr)
                      call ilgdata( x(irow,icolx),
     1                              x(irow,icoly), 1, 1, 0 )
120                   continue                                            4e
                  call vecmod
                  call visibl
                  call anmode
c
c       put the selected points back in their original positions
c
                  do 121 list_ptr = 1, list_len                           4b
                      irow = found_list(list_ptr)
                      x(irow,icolx) = x(irow,icolx) - tot_sh_x
                      x(irow,icoly) = x(irow,icoly) - tot_sh_y
                      call ilgdata( x(irow,icolx),
     1                              x(irow,icoly), 1, 1, 0 )
121                   continue                                            4e
c
c       **** un - read
c
              else if ( prev_cmd .eq. 'R' ) then                          3m
                  got_message = .TRUE.
                  message = 'Cannot undo "read".'
c
c       **** un - append
c
              else if ( prev_cmd .eq. 'A' ) then                          3m
                  call vecmod
                  call invis
                  call anmode
                  call ilgdata( x(nlast,icolx), x(nlast,icoly), 1, 1, 0)
                  call vecmod
                  call visibl
                  call anmode
                  nptsout = nptsout - 1
                  nlast = nlast - nincr
c
c       **** un - type
c
              else if ( prev_cmd .eq. 'T' ) then                          3m
c
c       remove the selected points from new locations on the screen
c
                  call vecmod
                  call invis
                  call anmode
                  do 151 list_ptr = 1, list_len                           4b
                      irow = found_list(list_ptr)
                      call ilgdata( x(irow,icolx), x(irow,icoly), 1,1,0)
151                   continue                                            4e
                  call vecmod
                  call visibl
                  call anmode
c
c       put the selected points in their old positions
c
                  do 152 list_ptr = 1, list_len                           4b
                      irow = found_list(list_ptr)
                      x(irow,icolx) = ( x(irow,icolx) - x_offset ) /
     1                                x_scale
                      x(irow,icoly) = ( x(irow,icoly) - y_offset ) /
     1                                y_scale
                      call ilgdata( x(irow,icolx), x(irow,icoly), 1,1,0)
152                   continue                                            4e
c
c       **** un - undo
c
              else if ( prev_cmd .eq. 'U' ) then                          3b
                  got_message = .TRUE.
                  message = 'Do it again. Can''t undo "undo".'
                  end if                                                  3e
              prev_cmd = 'U'
              got_list = .TRUE.
c
c       **** WINDOW ****
c
          else if ( command .eq. 'W' ) then                               2m
              if ( got_wind_corner )
     1            got_wind_corner = fudge_wcorner( wind_x, wind_y )
c             draw the first window marker as another cross hair
              wind_x = hair_x
              wind_y = hair_y
c             tell the next command to operate on a window
              got_wind_corner = fudge_wcorner( wind_x, wind_y )
              got_list = .TRUE.
c
c       **** HELP ****
c
          else if ( command .eq. 'H' ) then                               2m
              call newscr()
              write( STD_OUT, 70 )
70            format( ' SFUDG2 : Help : Data array editor.'
     1          ' You may move the crosshairs and then' /
     2          ' a) Press `E'' or `Q'' to exit.' /
     3          ' b) Place crosshairs near a data point and press' /
     4          '    `D'' to delete, `S'' to shift or `R'' to read it.'/
     5          ' c) Append a data point located at the crosshair to' /
     6          '    the array by pressing `A''.'
     7          ' d) Press `U'' to undo a Delete, Shift or Append.' /
     8          ' e) Press `W'' to locate one corner of a selection' /
     9          '    window. Move the crosshairs to the opposite' /
     a          '    corner of the window and then press `D'', `S''' /
     b          '    or `R'' to delete, shift or read every data' /
     c          '    point in the selection window.' )
              call ilgwait()
              got_list = .TRUE.
              end if                                                      2e
          goto 1
c         end of loop
      end



      LOGICAL FUNCTION FUDGE_READ( X_WIND, Y_WIND, KEY_PRESSED )
c
c Reads the cross-hairs location into (X_WIND,Y_WIND) and records the character
c pressed in CHAR_PRESSED . If the crosshairs cannot be read or located within
c the ILGLIB graph in MAXTRIES attempts, FUDGE_READ returns with the result
c .FALSE. .
c
c Arguments
c  real x_wind              # crosshairs abscissa
c       y_wind              # crosshairs ordinate
c  character key_pressed*1  # key pressed
c
c constants
c  MAX_TRIES=4
c
c local variables
c  integer
c    isuccess   # =1 if LOCATE reads crosshairs successfully
c    itry       # number of attempts
c
c global variables
c  real rxllim, rxulim, ryllim, ryulim    # graph limits from ilglib
c
c subroutine
c  psecap, vecmod, locate, anmode, bell   # HPPREV
c
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
      integer MAX_TRIES
      parameter( MAX_TRIES=4 )
c
      integer isuccess, itry
      real x_wind, y_wind
      character key_pressed*1
c
      FUDGE_READ = .FALSE.
      itry = 0
      call psecap
c     start loop                                                          1b
1         itry = itry + 1
          call vecmod
          call locate( 1, x_wind, y_wind, key_pressed, isuccess )
          call anmode
          if ( isuccess .ne. 1 ) then                                     2b
              write( STD_ERR, 2 )
2             format( ' SFUDG2 : WARNING : Cannot read crosshair ',
     1                'location.' )
          else if ( rxllim .gt. x_wind .or. x_wind .gt. rxulim .or.
     1              ryllim .gt. y_wind .or. y_wind .gt. ryulim ) then     2m
              call fudge_OS_bound()
              if ( itry .le. MAX_TRIES )
     1            goto 1
          else
              FUDGE_READ = .TRUE.
              end if                                                      2e
c         end of loop
c
c       change the character pressed to upper case
c
      if ( 'a' .le. key_pressed .and. key_pressed .le. 'z' )
     1    key_pressed = char( ichar( key_pressed ) +
     2                        ichar( 'A' ) - ichar( 'a' ) )
      return
      end



      LOGICAL FUNCTION FUDGE_W2D( X_WIND, Y_WIND, X_DATA, Y_DATA )
c
c Converts the coordinate (X_WIND,Y_WIND) expressed in window coordinates to
c the data array coordinates system (X_DATA,Y_DATA) . The returned result
c is .TRUE. only if (X_DATA,Y_DATA) is within the ILGLIB graph .
c
c Arguments
c  real x_wind    # INPUT  # abscissa
c       y_wind    # INPUT  # ordinate
c       x_data    # OUTPUT # abscissa
c       y_data    # OUTPUT # ordinate
c
c  local variables
c    real x, y    # temporary coordinate for converting from rectangular
c                 # to polar coordinates
c         rxbase, rybase,  # base for coordinate conversion
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
c
      real x_wind, y_wind, x_data, y_data, x, y, rxbase, rybase
c
      fudge_w2d = rxllim .le. x_wind .and. x_wind .le. rxulim .and.
     1            ryllim .le. y_wind .and. y_wind .le. ryulim
      if ( fudge_w2d ) then                                               1b
          if ( fpolar ) then                                              2b
              x = x_wind - rxpole
              y = y_wind - rypole
              x_data = atan2d( y, x )
              y_data = sqrt( x **2 + y **2 )
          else                                                            2m
              x_data = x_wind
              y_data = y_wind
              end if                                                      2e
c
          rxbase = rgroffx + ( rxofset - rxmin ) * rdatfacx
          x_data = ( x_data - rxbase ) / rdatfacx
          if ( flogx )
     1        x_data = 10.0 ** x_data
c
          rybase = rgroffy + ( ryofset - rymin ) * rdatfacy
          y_data = ( y_data - rybase ) / rdatfacy
          if ( flogy )
     1        y_data = 10.0 ** y_data
c
      else                                                                1m
          call fudge_OS_bound()
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION FUDGE_D2W( X_DATA, Y_DATA, X_WIND, Y_WIND )
c
c Converts the coordinate (X_DATA,Y_DATA) expressed in data array coordinates to
c the window coordinate system (X_WIND,Y_WIND) .
c
c Arguments
c  real x_data    # OUTPUT # abscissa
c       y_data    # OUTPUT # ordinate
c       x_wind    # INPUT  # abscissa
c       y_wind    # INPUT  # ordinate
c
c  local variables
c    real rtemp   # temporary x value to avoid over-writing polar angle
c         rxbase, rybase,  # base for coordinate conversion
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
c
      real x_wind, y_wind, x_data, y_data, rtemp, rxbase, rybase
c
      rxbase = rgroffx + ( rxofset - rxmin ) * rdatfacx
      x_wind = x_data
      if ( flogx )
     1    x_wind = alog10( x_wind )
      x_wind = x_wind * rdatfacx + rxbase
c
      rybase = rgroffy + ( ryofset - rymin ) * rdatfacy
      y_wind = y_data
      if ( flogy )
     1    y_wind = alog10( y_wind )
      y_wind = y_wind * rdatfacy + rybase
c
      if ( fpolar ) then                                                  1b
          rtemp = rxpole + y_wind * cosd( x_wind )
          y_wind = rypole + y_wind * sind( x_wind )
          x_wind = rtemp
          end if                                                          1e
      fudge_d2w = rxllim .le. x_wind .and. x_wind .le. rxulim .and.
     1            ryllim .le. y_wind .and. y_wind .le. ryulim

      if ( .not. fudge_d2w )
     1    call fudge_OS_bound()
c
      return
      end



      LOGICAL FUNCTION FUDG2_FIND( X_HAIR, Y_HAIR,
     1                             ARRAY, ABSCISSA, ORDINATE,
     2                             N_ROWS, INCREM, N_POINTS,
     3                             FOUND_INDEX )
c
c     Find data point in ARRAY nearest to the crosshairs location . The
c returned value in FOUND_INDEX is the row index of this point in ARRAY
c and the returned value of FUDG2_FIND is .TRUE. . If no data point is
c found closer than RESOLUTION * SCREEN_SIZE, the returned value of
c FUDG2_FIND is .FALSE. and FOUND_INDEX = NOT_FOUND (=-1) .
c
c Arguments
c  real x_hair          # abscissa of crosshairs location
c       y_hair          # ordinate of crosshairs location
c       array(n_rows,*) # data array
c  integer abscissa     # array column containing the data abscissa
c          ordinate     # array column containing the data ordinate
c          n_rows       # number of rows in the data array
c          increm       # row increment
c          n_points     # number of data points
c          found_index  # location of the found data point
c
c constants
c  NOT_FOUND  = -1
c  MAX_REAL   = 1.0e37
c  RESOLUTION = 0.015
c
c local variables
c  integer
c    i          # loop index
c    n_last     # row number of the last data point
c  logical
c    fudge_d2w  # function : convert from data to window coordinate system
c  real
c    distance   # distance from data point to crosshairs squared
c    min_dist   # minimum (distance **2)
c    rpagsizya  # adjusted vertical page dimension (window units)
c    x_wind     # data point abscissa in window coordinates
c    y_wind     # data point ordinate in window coordinates
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
      integer NOT_FOUND
      real    MAX_REAL, RESOLUTION
      parameter( NOT_FOUND=-1, MAX_REAL=1.0e37, RESOLUTION=0.015 )
c
      integer abscissa, ordinate, n_rows, increm, n_points, found_index,
     1        i, n_last
      logical fudge_d2w
      real    x_hair, y_hair, array(n_rows,*), distance, min_dist,
     1        rpagsizya, x_wind, y_wind
c
      min_dist = MAX_REAL
      found_index = NOT_FOUND
      n_last = min( ( n_points - 1 ) * increm + 1, n_rows )
      rpagsizya = 780.0 / 1024.0 * rpagsizx
      do 1 i = 1, n_last, increm                                          1b
c
c           convert data to window coordinates
c
          if ( fudge_d2w( array(i,abscissa), array(i,ordinate),
     1                    x_wind, y_wind ) ) then                         2b
c
c            calculate distance
c
              distance = ( ( x_wind - x_hair ) / rpagsizx  ) **2 +
     1                   ( ( y_wind - y_hair ) / rpagsizya ) **2
              if ( distance .lt. min_dist ) then                          3b
                  min_dist = distance
                  found_index = i
                  end if                                                  3e
              end if                                                      2e
1         continue                                                        1e
      fudg2_find = found_index .ne. NOT_FOUND .and.
     1             min_dist .le. RESOLUTION **2
      if ( .not. fudg2_find )
     1    found_index = NOT_FOUND
      return
      end



      LOGICAL FUNCTION FUDG2_WIND( LL_X, LL_Y, UR_X, UR_Y,
     1                             ARRAY, ABSCISSA, ORDINATE,
     2                             N_ROWS, INCREM, N_POINTS,
     3                             FOUND_LIST, LIST_LEN, MAX_LIST )
c
c     List all data points in ARRAY which lie within the window with
c lower left corner at (LL_X,LL_Y) and upper right corner at (UR_X, UR_Y) .
c The row indices these points in ARRAY are returned in the array FOUND_LIST.
c If no data points are found within the window the returned value of
c fudg2_wind is .FALSE. ( otherwise it is .TRUE. ) .
c
c Arguments
c  real ll_x, ll_y      # INPUT  # lower left corner of the window
c       ur_x, ur_y      # INPUT  # upper right corner of the window
c       array(n_rows,*) # INPUT  # data array
c  integer abscissa     # INPUT  # array column containing the data abscissa
c          ordinate     # INPUT  # array column containing the data ordinate
c          n_rows       # INPUT  # number of rows in the data array
c          increm       # INPUT  # row increment
c          n_points     # INPUT  # number of data points
c          found_list(*)# OUTPUT # list of row indices of the point in window
c          list_len     # OUTPUT # number of data points in the window
c          max_list     # INPUT  # size of the array ``found_list''
c
c local variables
c  integer
c    i          # loop index
c    n_last     # row number of the last data point
c  logical
c    fudge_d2w  # function : convert from data to window coordinate system
c  real
c    x_wind     # data point abscissa in window coordinates
c    y_wind     # data point ordinate in window coordinates
c    left, right, top, bottom # edges of the selection window
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
      integer NOT_FOUND
      parameter( NOT_FOUND=-1 )
c
      integer abscissa, ordinate, n_rows, increm, n_points, list_len,
     1        max_list, i, n_last, found_list(*)
      logical fudge_d2w
      real    ll_x, ll_y, ur_x, ur_y, array(n_rows,*),
     1        x_wind, y_wind, left, right, top, bottom
c
      found_list(1) = NOT_FOUND
      list_len = 0
      n_last = min( ( n_points - 1 ) * increm + 1, n_rows )
c
      left   = min( ll_x, ur_x )
      right  = max( ll_x, ur_x )
      top    = max( ll_y, ur_y )
      bottom = min( ll_y, ur_y )
c
      do 1 i = 1, n_last, increm                                          1b
c
c           convert data to window coordinates
c
          if ( list_len .ge. max_list )
     1        goto 2
          if ( fudge_d2w( array(i,abscissa), array(i,ordinate),
     1                    x_wind, y_wind ) ) then                         2b
              if ( left   .le. x_wind .and. x_wind .le. right .and.
     1             bottom .le. y_wind .and. y_wind .le. top ) then        3b
                  list_len = list_len + 1
                  found_list(list_len) = i
                  end if                                                  3e
              end if                                                      2e
1         continue                                                        1e
2     fudg2_wind = list_len .gt. 0
      return
      end



      LOGICAL FUNCTION FUDGE_WCORNER( HAIR_X, HAIR_Y )
c
c Write or remove a crosshair to mark the first corner of the selection window .
c The returned result is .TRUE. if the crosshairs are drawn, .FALSE. if they
c are removed .
c
c Arguments
c  real hair_x  # abscissa of crosshair location
c       hair_y  # ordinate of crosshair location
c
c local variables
c  logical
c    remove     # toggle : TRUE = remove crosshairs : FALSE = draw crosshairs
c
      include 'stdio.def'
      include '[planspear.for.plt.v1]ilglib.cmn'
c
      logical remove
      real hair_x, hair_y
      save remove
      data remove / .FALSE. /
c
      call vecmod()
      if ( remove )
     1    call invis()
      call move( rpagzerx + rxspoff, hair_y )
      call draw( rpagzerx + rpagsizx + rxspoff, hair_y )
      call move( hair_x, rpagzery + ryspoff )
      call draw( hair_x, rpagzery + rpagsizy + ryspoff )
      if ( remove )
     1    call visibl()
      remove = .not. remove
      call anmode()
      fudge_wcorner = remove
      return
      end



      SUBROUTINE FUDGE_OS_BOUND()
c
c Report crosshairs outside the graph boundaries
c
      include 'stdio.def'
c
      call bell()
      write( STD_ERR, 1 )
1     format( / ' SFUDG2 : WARNING : Place crosshairs inside the',
     1        ' graph boundaries.' )
      return
      end


      SUBROUTINE SPLTPUT( X, Y )
c
c       PUT a single point (X,Y) on a PLoT
c
      implicit none
      integer Hsize
      parameter( Hsize=256 )
c
      real uxp(Hsize), uyp(Hsize), x, y
      integer iptr
      common / Af77lib8 / uxp, uyp, iptr
      data iptr / 1 /
c
      uxp(iptr) = x
      uyp(iptr) = y
      iptr = iptr + 1
      if ( iptr .gt. Hsize ) then
        iptr = 1
        call ilgdata( uxp, uyp, 1, Hsize, 0 )
        end if
      return
      end


      SUBROUTINE SPLTFLS
c
c       FLuSh the PLoT buffer used by SPLTPUT
c
      implicit none
      integer Hsize
      parameter( Hsize=256 )
c
      real uxp(Hsize), uyp(Hsize), rx, ry
      integer iptr
      common / Af77lib8 / uxp, uyp, iptr
      data iptr / 1 /
c
      if ( iptr .gt. 1 )
     1  call ilgdata( uxp, uyp, 1, iptr, 0 )
      iptr = 1
      return
      end


