c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


c               ####################################
c               ###                              ###
c               ###            BARLIB            ###
c               ###     Baratron user library    ###
c               ###                              ###
c               ####################################
c
c Converts voltage output from an electronic manometer into a pressure
c measurement.
c
c LOGICAL FUNCTION EBAR_OPEN( CAL_REC, CALIB_FILE, AUX_LUN )
c LOGICAL FUNCTION EBAR_ADtoP( AD_SIG, PRESS, INCREM, NPTS )
c REAL FUNCTION QBAR_ADtoE( IADSIG )
c REAL FUNCTION QBAR_EtoP( VOLTS )
c REAL FUNCTION QTSDYN( RDYNQ, NPLUG )
c REAL FUNCTION QBAR_DISPLtoP( READING )
c SUBROUTINE SBAR_CLOSE
c SUBROUTINE SBAR_EXAM()
c
c



      LOGICAL FUNCTION EBAR_OPEN( CAL_REC, CALIB_FILE, AUX_LUN )
c
c       BARatron (pressure sensor) CALibration file OPEN
c Reads the CAL_RECth baratron / barocel / pressure-sensor calibration
c record from the calibration file CALIB_FILE .
c
c Arguments
c  integer   cal_rec          # record to be read from the calibration file
c                             #   0 = ask for record
c  character calib_file*(*)   # name of the calibration file
c                             #  ' ' = ask for file name
c  integer   aux_lun          # logical unit number of auxillary file (0=none)
c                             # The file is only written if 0 < aux_lun < 100
c
c
c constants
c  MAGIC='Cb'
c
c local variables
c  integer
c    i            # looping index
c    name_len     # length of file name
c    record_no    # required record number
c    reserved_lu  # value of logical unit number reserved for calibration file
c    jljstr       # function : left justify string
c    new_lun      #          : new logical unit number
c  logical
c    equery       # function : read boolean
c    existf       #          : file exists ?
c  character
c    Cmagic*2     # magic number read from file
c    new_file*32  # new file name
c    prev_file*32 # previous file name
c  subroutine
c    Cresp        # read string
c    sbar_exam    # examine baratron calibration file
c
c
      include 'stdio.def'
      character MAGIC*2
      parameter( MAGIC='Cb' )
c
      integer cal_rec, aux_lun, i, name_len, record_no, reserved_lu,
     1    jljstr, new_lun
      logical equery, existf
      character calib_file*(*), Cmagic*2, new_file*32, prev_file*32
      include 'barlib.cmn'
      external barlib_block
      save name_len, reserved_lu, new_file, prev_file
      data name_len, reserved_lu / 1, 0 /
      data new_file, prev_file / ' ', ' ' /
c
c           if file name & record number are unchanged , return
c
      ebar_open  = .TRUE.
      aux_lu     = 0
      if ( aux_lun .gt. 0 .and. aux_lun .lt. 100 )
     1    aux_lu = aux_lun
      new_file   = calib_file
      if ( prev_file .eq. new_file .and. cal_rec .eq. curr_rec
     1        .and. calib_lu .ne. 0 )
     2    return
      ebar_open  = .FALSE.
      record_no  = max( 0, cal_rec )
c
c           if new file or none is open, open a calibration file
c
      if ( prev_file .ne. new_file .or. calib_lu .eq. 0 ) then            1b
          if ( reserved_lu .eq. 0 )
     1        reserved_lu = new_lun()
          if ( calib_lu .ne. 0 )
     1        call close( calib_lu )
          name_len = max( 1, jljstr( new_file ) )
          if ( new_file .eq. ' ' ) then                                   2b
              write( STD_OUT, 1 )
1             format( ' Baratron calibration file name ? ', $ )
              call Cresp( STD_IN, new_file, 4 )
2             format( a )
              name_len = max( 1, jljstr( new_file ) )
              end if                                                      2e
          if ( .not. Existf( new_file ) ) then                            2b
3             write( STD_OUT, 4 ) new_file(:name_len)
4             format( ' CALBAROPN : cannot find file ', a )
              return
              end if                                                      2e
          open( UNIT=reserved_lu, FILE=new_file, STATUS='old',
     1        ACCESS='sequential', FORM='unformatted', ERR=11,
     2        RECORDTYPE='variable', BLOCKSIZE=8192 )
          calib_lu = reserved_lu
          prev_file = new_file
          curr_rec = 0
          if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1        write( aux_lu, 5 ) new_file(:jljstr(new_file) )
5             format( ' EBAR_OPEN : file name = ', a )
c
c           old file - rewind if new record precedes old one
c
      else if ( record_no .eq. 0 .or. record_no .lt. curr_rec ) then      1m
          rewind( UNIT=calib_lu )
          curr_rec = 0
          end if                                                          1e
c
c           skip required number of records
c
      if ( record_no .gt. curr_rec + 1 ) then                             1b
          do 6 i = 1, record_no - curr_rec - 1                            2b
              read( calib_lu, END=13 )
              curr_rec = curr_rec + 1
6             continue                                                    2e
          end if                                                          1e
c
c
c       READ CALIBRATION FILE RECORD UNTIL THE RIGHT ONE
c
c     start of loop
7     if ( record_no .ne. curr_rec .or. record_no .eq. 0 ) then
          read( calib_lu, ERR=14, END=13 ) Cmagic,
     1        calib_no, Ctype, plug_no, Cbar_ad, bar_chan,
     2        n_points, calib_order, calib_error,
     3        ( calib_poly(i), i = 1, calib_order + 1 ),
     4        ( pressure(i), mean_volts(i), rms_volts(i),
     5            i = 1, n_points ),
     6        lin_order, lin_error, lin_maxim, lin_minim, lin_offset,
     7            lin_slope, ( lin_poly(i), i=1,lin_order + 1 )
          if ( Cmagic .ne. MAGIC )
     1        goto 16
          if ( record_no .eq. 0 ) then
              call sbar_exam
              if ( .not. equery( STD_IN, 'Select this one ? ' ) )
     1            goto 7
              record_no = curr_rec
          else
              curr_rec = curr_rec + 1
              goto 7
              end if
          end if
c         end of loop
c
c       assign the A/D converter calibration constant
c
      if ( Cbar_ad .eq. 'DAS 1128 slo' ) then
          volts_to_AD = 400.0
      else if ( Cbar_ad .eq. 'DAS 250A ran' .or.
     1       Cbar_ad .eq. 'DAS 250A seq' ) then
          volts_to_AD = -409.6
      else
          write( STD_ERR, 8 )
8         format( ' EBAR_OPEN : ABORT : Unknown A/D converter' )
          return
          end if
c
      if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1    write( aux_lu, 10 ) curr_rec
10     format( ' EBAR_OPEN : record number = ', i4 )
      ebar_open = .TRUE.
      return
c
c       ERROR TRAPS
c
11    write( STD_ERR, 12 ) new_file(:name_len)
12    format( ' EBAR_OPEN : Cannot open file ', a )
      return
c
c           ask for rewind if End Of File
c
13    if ( .not. Equery( STD_IN,
     1                   'End of file ; rewind & continue ?' ) ) then
          if ( Equery( STD_IN, 'Will Abort : Are you sure ?' ) )
     1        return
          end if
      curr_rec = 0
      rewind( UNIT=calib_lu )
      goto 7
c
14    write( STD_ERR, 15 )
15    format( ' EBAR_OPEN : Cannot read from file ', a )
      return
c
16    write( STD_ERR, 17 ) new_file(:name_len)
17    format( ' EBAR_OPEN : Not a baratron calibration file .' /
     1    'Wrong magic number in ', a )
      return
      end



      SUBROUTINE SBAR_CLOSE
c
c       Close the Baratron calibration file .
c
      include 'stdio.def'
      include 'barlib.cmn'
      external barlib_block
c
      if ( calib_lu .ne. 0 ) then
          close( UNIT=calib_lu )
          calib_lu = 0
          end if
      return
      end



      REAL FUNCTION QBAR_ADtoE( AD_SIG )
c
c Convert a single pressure transducer A/D reading into an A/D voltage value .
c The result value can be converted into a pressure using QBAR_EtoP .
c This function requires information from the baratron calibration file .
c
c Argument
c  integer *2 ad_sig    # A/D converter reading
c
c local variables
c  logical
c    ebar_open  # function : open calibration file
c
      include 'stdio.def'
      logical ebar_open
      integer *2 ad_sig
      include 'barlib.cmn'
      external barlib_block
c
c       open calibration file if necessary
c
      if ( calib_lu .eq. 0 ) then
          write( STD_ERR, 1 )
          if ( .not. ebar_open( 0, ' ', 0 ) )
     1        return
          end if
c
c       do the conversion
c
      QBAR_ADtoE = real( ad_sig ) / volts_to_AD
      return
1     format( ' QBAR_ADtoE : WARNING : Calibration file not opened' )
      end



      REAL FUNCTION QBAR_EtoP( VOLTS )
c
c Convert the voltage value at the A/D converter into a pressure value .
c This function requires information from the baratron calibration file .
c
c Argument
c  real volts   # voltage at the A/D converter
c
c local variables
c  logical
c    ebar_open  # function : open calibration file
c  real
c    nom_press  # nominal pressure reading (before linearisation)
c    qpoly      # function : evaluate polynomial
c
      include 'stdio.def'
      logical ebar_open
      real volts, nom_press, qpoly
      external barlib_block
      include 'barlib.cmn'
c
c       open calibration file if necessary
c
      if ( calib_lu .eq. 0 ) then                                         1b
          write( STD_ERR, 1 )
1         format( ' QBAR_EtoP : WARNING : Calibration file not opened' )
          if ( .not. ebar_open( 0, ' ', 0 ) )
     1        return
          end if                                                          1e
c
c       Do the conversion, first convert the A/D voltage into a nominal
c       pressure value, then apply the linearisation polynomial
c
      nom_press = qpoly( volts, calib_poly, calib_order )
      if ( lin_minim .le. nom_press .and.
     1     nom_press .le. lin_maxim ) then                                1b
          QBAR_EtoP = Qpoly( nom_press, lin_poly, lin_order )
      else                                                                1m
          QBAR_EtoP = nom_press * lin_slope + lin_offset
          end if                                                          1e
      return
      end




      LOGICAL FUNCTION EBAR_ADtoP( AD_SIG, PRESS, INCREM, NPTS )
c
c       calculate pressure from A/D reading
c
c Convert NPTS A/D converter readings in array AD_SIG at increments of INCREM
c into pressure values PRESS using the pressure sensor calibration . The same
c data increment INCREM is used in PRESS .
c
c Arguments
c  integer *2 ad_sig(*) # A/D converter readings
c  real    press(*)     # pressure values
c  integer increm       # data array indexing increments
c          npts         # number of data points
c
c local variables
c  logical
c    ebar_open  # function : open calibration file
c  integer
c    i          # array index
c  real
c    nom_press  # nominal pressure reading (before linearisation)
c    qpoly      # function : evaluate polynomial
c
      include 'stdio.def'
      logical ebar_open
      integer *2 ad_sig(*)
      integer i, increm, npts
      real nom_press, press(*), qpoly
      include 'barlib.cmn'
      external barlib_block
c
c       open calibration file if necessary
c
      EBAR_ADtoP = .FALSE.
      if ( npts .le. 0 .or. increm .le. 0 ) then                          1b
          write( STD_ERR, 1 )
1         format( ' EBAR_ADtoP : ABORT : Invalid argument(s)')
          return
          end if                                                          1e
      if ( calib_lu .eq. 0 ) then                                         1b
          write( STD_ERR, 2 )
2         format( ' EBAR_ADtoP : WARNING : Calibration file not opened')
          if ( .not. ebar_open( 0, ' ', 0 ) )
     1        return
          end if                                                          1e
c
c       Do the conversion, first convert the A/D voltage into a nominal
c       pressure value, then apply the linearisation polynomial
c

      do 3 i = 1, ( npts - 1 ) * increm + 1, increm                       1b
          nom_press = qpoly( real( ad_sig(i) ) / volts_to_AD,
     1                       calib_poly, calib_order )
          if ( lin_minim .le. nom_press .and.
     1         nom_press .le. lin_maxim ) then                            2b
              press(i) = Qpoly( nom_press, lin_poly, lin_order )
          else                                                            2m
              press(i) = nom_press * lin_slope + lin_offset
              end if                                                      2e
3         continue                                                        1e
      EBAR_ADtoP = .TRUE.
      return
      end



      REAL FUNCTION QBAR_DISPLtoP( READING )
c
c Convert the value displayed on the pressure transducer readout unit into
c a pressure value in Torr using the linearization polynomial.
c This function uses the linearization polynomial defined in the transducer
c calibration file if this file has been read. If the calibration file has
c not been read, the linearization polynomial is taken from the default
c linearization file '[planspear.com]press_trans.lin' .
c
c Argument
c  real reading # from the transducer readout unit
c
c constants
c    DEF_LINEAR_FILE = '[planspear.com]press_trans.lin'
c local variables
c  logical
c    got_lin_poly# linearization polynomial has been read from default
c                # linearization file
c  integer
c    i           # polynomial array index
c    linear_lu   # logical unit number of linearisation file (0=closed)
c    res_linear  # reserved value of linear_lu
c    new_lun     # function : get new logical unit number
c    open_file   #          : open file
c  real
c    qpoly       # function : evaluate polynomial
c  subroutine
c   ilgwait
c
      include 'stdio.def'
      character DEF_LINEAR_FILE*32
      parameter( DEF_LINEAR_FILE='[planspear.com]press_trans.lin' )
c
      logical got_lin_poly
      integer i, linear_lu, res_linear, new_lun, open_file
      real    reading, qpoly
c
      external barlib_block
      include 'barlib.cmn'
      save got_lin_poly, res_linear
      data got_lin_poly / .FALSE. /
      data res_linear / 0 /
c
c       open and read linearization file if necessary
c
      if ( calib_lu .eq. 0 .and. .not. got_lin_poly ) then                1b
          write( STD_ERR, 1 )
1         format( ' QBAR_DISPtoP : WARNING : Calibration file not ',
     1        'opened.' / ' Using default file  ', a )
          if ( res_linear .eq. 0 )
     1        res_linear = new_lun()
c
c           open, read and close default linearization file
c
          linear_lu = open_file( res_linear, DEF_LINEAR_FILE, 'old', -1)
          if ( linear_lu .ne. 0 ) then                                    2b
              read( linear_lu, ERR=3 ) lin_error, lin_minim, lin_maxim,
     1            lin_slope, lin_offset, lin_order,
     2            ( lin_poly(i), i = 1, lin_order + 1 )
              close( UNIT=linear_lu )
              got_lin_poly = .TRUE.
          else                                                            2m
              write( STD_ERR, 2 ) DEF_LINEAR_FILE
2             format( ' QBAR_DISPLtoP : ABORT :',
     1                '   Cannot open linearization file ', a )
              end if                                                      2e
          end if                                                          1e
c
c           Apply the linearisation polynomial
c
      if ( lin_minim .le. reading .and. reading .le. lin_maxim ) then     1b
          QBAR_DISPLtoP = Qpoly( reading, lin_poly, lin_order )
      else                                                                1m
          QBAR_DISPLtoP = reading * lin_slope + lin_offset
          end if                                                          1e
      return
c
c           Error trap
c
3     write( STD_ERR, 4 ) DEF_LINEAR_FILE
4     format( ' QBAR_DISPLtoP : ABORT :',
     1        '   Cannot read linearization file ', a )
      call ilgwait()
      return
      end



      SUBROUTINE SBAR_EXAM()
c
c         EXAMine the BARatron CALibration
c    Displays all of the Baratron calibration information
c
c local variables
c  logical
c    ebar_open  # function : open calibration file
c    equery     #          : read logical
c  integer
c    i          # array index
c
      include 'stdio.def'
      logical ebar_open, equery
      integer i
      external barlib_block
      include 'barlib.cmn'
c
c       open calibration file if necessary
c
      if ( calib_lu .eq. 0 ) then                                         1b
          write( STD_ERR, 1 )
1         format( ' SBAR_EXAM : WARNING : Calibration file not opened')
          if ( .not. ebar_open( 0, ' ', 0 ) )
     1        return
          end if                                                          1e
c
      write( STD_OUT, 2 ) calib_no, Ctype, plug_no, Cbar_ad, bar_chan,
     1    n_points, calib_order, calib_error,
     2    ( i - 1, calib_poly(i), i = 1, calib_order + 1 )
2     format( ' PRESSURE SENSOR CALIBRATION' /
     1    ' Calibration number              ', i3 /
     2    ' Type of pressure measurement    ', a /
     3    ' W/T test section plug number    ', i3 /
     4    ' A/D converter                   ', a /
     5    ' A/D channel                     ', i2 /
     6    ' Number of calibration points    ', i3 /
     7    ' Order of calibration polynomial ', i2 /
     8    ' RMS error of calibration polynomial ', g13.6 /
     9    ' Coeffs of calibration polynomial' /
     a    ( 11x, i2, 2x, g13.6 ) )
c
      write( STD_OUT, 3 ) lin_order, lin_error, lin_minim, lin_maxim,
     1    lin_slope, lin_offset,
     2    ( i - 1, lin_poly(i), i = 1, lin_order + 1 )
3     format( ' Order of linearisation polynomial        ', i2 /
     1    ' RMS error of linearisation polynomial    ', g13.6 /
     2    ' Linearisation minimum and maximum        ', 2g14.6 /
     3    ' Slope & offset outside linearis''n range  ', 2g14.6 /
     4    ' Coeffs of linearisation polynomial' /
     5    ( 11x, i2, 2x, g13.6 ) )
      if ( 0 .lt. aux_lu .and. aux_lu .lt. 100 ) then                     1b
          write( aux_lu, 2 ) calib_no, Ctype, plug_no, Cbar_ad,
     1        bar_chan, n_points, calib_order, calib_error,
     2        ( i - 1, calib_poly(i), i = 1, calib_order + 1 )
          write( aux_lu, 3 ) lin_order, lin_error, lin_minim,
     1        lin_maxim, lin_slope, lin_offset,
     2        ( i - 1, lin_poly(i), i = 1, lin_order + 1 )
          end if                                                          1e
c
      if ( Equery( STD_IN, ' Display data ?' ) ) then                     1b
          write( STD_OUT, 4 ) ( pressure(i), mean_volts(i),
     1                          rms_volts(i), i = 1, n_points )
4         format( '   Pressure   Mean voltage  RMS voltage' /
     1        ( 3g13.5 ) )
          if ( 0 .lt. aux_lu .and. aux_lu .lt. 100 )
     1        write( aux_lu, 4 ) ( pressure(i), mean_volts(i),
     2                             rms_volts(i), i = 1, n_points )
          end if                                                          1e
      return
      end



      BLOCK DATA BARLIB_BLOCK
c
c         initialize the common block for BARLIB.FOR
c
      include 'stdio.def'
      include 'barlib.cmn'
c
c   MISCELLANEOUS PARAMETERS
      data aux_lu, calib_lu, curr_rec / 0, 0, 0 /
c
c     CALIBRATION DATA
      data n_points / 0 /
      data volts_to_AD / 400.0 /
      end



      REAL FUNCTION QTSDYN( RDYNQ, NPLUG )
c
c           Converts dynamic pressure just upstream of the sonic choke
c RDYNQ into free stream dynamic pressure in the test section at plug
c number NPLUG . The free stream dynamic pressure is assumed to be a
c function of location along the test section and pre-centrebody dynamic
c pressure only . This function is described as a series of 15
c polynomials (one for each test section port) contained in the
c unformatted file '[planspear.com]tsqq.cal' .
c
c This function has been superseded by QTTS_DYN which takes account of
c the trip diameter .
c
c         Arguments
c real rdynq          # dynamic pressure upstream of sonic choke
c integer nplug       # Wind Tunnel plug number for which the free
c                     # stream dynamic pressure is required
c variable
c  logical fpass1     # first pass flag
c  integer nprevplug  # wind tunnel plug used previously
c          iplug      # plug as recorded on current file record
c          itsqlu     # logical unit no. of test section q profiles
c          norder     # order of the fitted polynomial
c          i          # looping index
c          new_lun    # function : get new logical unit number
c  real b(9)          # coefficients of the fitted poynomial
c       qpoly         # function : evaluate polynomial
c
      include 'stdio.def'
      logical fpass1
      integer nplug, nprevplug, iplug, itsqlu, norder, i,
     1    new_lun
      real b(9), rdynq, qpoly
c
      save fpass1, itsqlu, nprevplug, iplug, norder, b
      data fpass1 / .TRUE. /
      data nprevplug, itsqlu / 0, 0 /
c
c         Check arguments
c
      if ( nplug .le. 0 .or. nplug .gt. 15 ) then
          write( STD_OUT, 1 ) nplug
1         format( ' QTSDYN : FATAL : Illegal W/T plug number ', i6 )
          stop
      else if ( rdynq .lt. 23.0 .or. rdynq .gt. 11000.0 ) then
          write( STD_OUT,  2 ) Rdynq
2         format( ' QTSDYN : WARNING : Dynamic pressure ', g13.6,
     1            ' Pa  out of range .' )
          qtsdyn = 1.0e-6
      else
          if ( itsqlu .eq. 0 ) then
              itsqlu = new_lun()
              end if
          if ( fpass1 )
     1        open( UNIT=itsqlu, FILE='[planspear.com]tsqq.cal',
     2            ACCESS='sequential', FORM='unformatted',
     3            RECORDTYPE='variable', STATUS='old' )
          if ( nplug .ne. nprevplug ) then
              if ( nplug .lt. nprevplug ) then
                  rewind( itsqlu )
                  nprevplug = 0
                  end if
              if ( nplug .gt. nprevplug+1 ) then
                  do i = 1, nplug - nprevplug - 1
                      read( itsqlu )
                      end do
                  end if
              read( itsqlu ) iplug, norder, ( b(i), i = 1, norder+1 )
              if ( nplug .ne. iplug .or. norder .lt. 2 .or.
     1                norder .gt. 7 ) then
                  write( STD_OUT, 3 )
3                 format( ' QTSDYN : PANIC : Calibration file screwed' )
                  stop
                  end if
              nprevplug = nplug
              fpass1 = .false.
              end if
          qtsdyn = qpoly( log10( Rdynq ), b, norder ) * Rdynq
          end if
      return
      end



      REAL FUNCTION QTTS_DYN( CB_DYN_PRESS, PLUG_NUMBER, TRIP_DIAM )
c
c   Converts dynamic  pressure just upstream of the sonic choke CB_DYN_PRESS
c into  free  stream  dynamic  pressure  in  the test section at plug number
c PLUG_NUMBER for a ball-trip diameter of TRIP_DIAM mm. The free stream dyn-
c amic  pressure in the test section is assumed to be a function of location
c along the test section, pre-centrebody dynamic pressure and trip diameter.
c This function is described as a series of polynomials (one for each  test-
c section-port/trip-diameter  combination)  contained in the formatted  file
c '[planspear.com]tts_dyn.cal' . Each  record in this  file  represents  one
c polynomial and has the format
c
c $ <plug_number> <trip_diameter> <min_cb_dyn_press> <max_cb_dyn_press> <n>
c <polynomial_coeff(0)> ... <polynomial_coeff(n)>
c
c where <n> is the polynomial order and the <plug_number>s must be listed in
c ascending  order . For each <plug_number>, the <trip_diameter>s must be in
c ascending  order,  and for each  <plug_number> - <trip_diameter> pair, the
c centrebody dynamic pressure ranges (<min_cb_dyn_press> to <max_cb_dyn_press>)
c must also ascend . Each  record may consist of 2 or more lines . The first
c line begins with an identifying '$' and contains plug_number, trip_diameter,
c min_cb_dyn_press, max_cb_dyn_press and polynomial_order in that order. The
c succeeding  lines  do  not  begin with a special identifying character and
c contain only the coefficients of the fitted polynomial . Lines in the file
c which begin with a ';' are comments .
c
c
c This function supersedes QTSDYN .
c
c Arguments
c  real    cb_dyn_press   # dynamic pressure just upstream of sonic choke
c  integer plug_number    # Wind Tunnel plug number for which the free
c                         #     stream dynamic pressure is required
c  real    trip_diam      # diameter of the trip balls in mm
c
c
c local variables
c real
c   current(TSC_POLY+MAX_POLY) # currently available test section calibration
c   dummy                      # dummy argument for TSC_FILE handler
c   poly_arg                   # point at which calibration poly'l is evaluated
c   previous(TSC_POLY+MAX_POLY)# previous test section calibration
c   qpoly                      # function : evaluate polynomial
c logical
c   first_PT_match        # obtained first plug and trip_diameter match
c   got_plug_trip         # calibration contains correct plug no. & trip diam
c   TSC_FILE              # function : Test Section Calibration FILE handler
c character
c   debug_label*20        # ' QTTS_DYN : DEBUG : '
c   abort_label*20        # ' QTTS_DYN : ABORT : '
c subroutine
c   TSC_COPY              # copy calibration
c
      include 'stdio.def'
      include 'qtts_dyn.def'
c
      logical first_PT_match, got_plug_trip, TSC_FILE
      integer plug_number
      real    cb_dyn_press, trip_diam, current(TSC_POLY+MAX_POLY),
     1        dummy, poly_arg, previous(TSC_POLY+MAX_POLY), qpoly
      character debug_label*20, abort_label*20
c
      save first_PT_match, got_plug_trip, current, debug_label
      data first_PT_match, got_plug_trip / .TRUE., .FALSE. /
      data current / TSC_POLY * TSC_UNKNOWN, MAX_POLY * TSC_UNKNOWN /
      data debug_label, abort_label /
     1     ' QTTS_DYN : DEBUG : ', ' QTTS_DYN : ABORT : ' /
c
c           Check arguments
c
      if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1    write( STD_ERR, 1 ) debug_label, cb_dyn_press, plug_number,
     2        trip_diam
1         format( a, 'Called with arguments' /
     1        '   cb_dyn_press = ', g12.5, ' Pascals' /
     2        '   plug_number  =   ', i3 /
     3        '   trip_diam    = ', g12.5, ' mm' )
      qtts_dyn = -1.0e9
      if ( cb_dyn_press .lt. MIN_CB_DYN .or.
     1     cb_dyn_press .gt. MAX_CB_DYN ) then                            1b
          write( STD_ERR,  2 ) abort_label,
     1        cb_dyn_press, MIN_CB_DYN, MAX_CB_DYN
2         format( a, 'Illegal pre-centrebody dynamic pressure ', g12.5,
     1        ' Pa' /
     2        20x, ' is outside the limits ', g12.5, ' and ', g12.5 )
      else if ( plug_number .lt. MIN_PLUG .or.
     1          plug_number .gt. MAX_PLUG ) then                          1m
          write( STD_ERR, 3 ) abort_label,
     1        plug_number, MIN_PLUG, MAX_PLUG
3         format( a, 'Illegal W/T plug number ', i6 /
     1        20x, ' is outside the limits ', i12, ' and ', i12 )
      else if ( trip_diam .lt. MIN_TRIP .or.
     1          trip_diam .gt. MAX_TRIP ) then                            1m
          write( STD_ERR,  4 ) abort_label,
     1        trip_diam, MIN_TRIP, MAX_TRIP
4         format( a, 'Illegal trip diameter ', g13.6,
     1        ' mm' /
     2        20x, ' is outside the limits ', g12.5, ' and ', g12.5 )
      else                                                                1m
c
c           open the calibration file if necessary
c
          if ( .not. tsc_file( TSC_ISOPEN, dummy ) ) then                 2b
              if ( .not. tsc_file( TSC_OPEN, dummy ) ) then               3b
                  write( STD_ERR, 5 ) abort_label, TTSQ_FILE
5                 format( a, 'Cannot open test ',
     1                'section calibration database file.' )
                  goto 20
c                     -- i.e. error return
                  end if                                                  3e
              end if                                                      2e
c
c           rewind the file if at END-OF-FILE
c
          if ( tsc_file( TSC_IS_EOF, dummy ) ) then                       2b
              if ( .not. tsc_file( TSC_REWIND, current ) )
     1            goto 20
c                     -- i.e. error return
              end if                                                      2e
c
c
c           if the required plug number or trip diameter have changed
c           or the centrebody dynamic pressure does not fall within the
c           calibration range, the correct polynomial must be read from
c           the file.
c
          got_plug_trip =
     1        plug_number .eq. int( current(TSC_PLUG) + 0.5 ) .and.
     2        abs( trip_diam / current(TSC_TRIP) - 1.0 ) .le. TINY
          if ( .not. got_plug_trip )
     1         first_PT_match = .TRUE.
          if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1        write( STD_ERR, 6 ) debug_label, got_plug_trip,
     2            first_PT_match
6             format( a, 'Matched plug number & trip diam = ', l1 /
     1            20x, 'First plug no. & trip diam match for speed ',
     2            'range = ', l1 )
          if ( .not. got_plug_trip .or.
     1         cb_dyn_press / current(TSC_MIN_CB) .le. 1.0 - TINY   .or.
     2         cb_dyn_press / current(TSC_MAX_CB) .gt. 1.0 + TINY ) then  2b        2b
c
c           rewind to the beginning of the file if we might have
c           passed the required record.
c
c           assume the plug numbers, trip diameters and the
c           centrebody dynamic pressure ranges are in ascending order
c
              if ( plug_number .lt. int( current(TSC_PLUG) + 0.5 ) .or.
     1             plug_number .eq. int( current(TSC_PLUG) + 0.5 ) .and.
     2             ( trip_diam / current(TSC_TRIP) .le. 1.0 - TINY .or.
     3               abs( trip_diam / current(TSC_TRIP) - 1.0 ) .le.
     4                    TINY .and. .not. first_PT_match .and.
     5               cb_dyn_press / current(TSC_MIN_CB) .le. 1.0-TINY )
     6                 ) then                                             3b
                  if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1                write( STD_ERR, 7 )  debug_label, plug_number,
     2                    trip_diam
7                     format( a, 'Have read past the required plug ',
     1                    i3, ' & trip diam. ', f7.3, ' mm' )
                  if ( .not. TSC_FILE( TSC_REWIND, current ) )
     1                goto 20
c                     -- i.e. error return
                  got_plug_trip = .FALSE.
                  first_PT_match = .TRUE.
                  end if                                                  3e
c
c           Read records from the file until the required plug number
c           and trip diameter are found.
c
              if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1            write( STD_ERR, 8 ) debug_label, plug_number,
     2                                trip_diam
8                 format( a, 'Searching for plug ', i3,
     1                ' and trip diam. ', f7.3, ' mm' )
c             start of loop                                               3b
9             if ( .not. got_plug_trip ) then
                  if ( .not. TSC_FILE( TSC_READ, current ) ) then         4b
                      write( STD_ERR, 10 ) abort_label,
     1                    plug_number, trip_diam
10                    format( a, 'Cannot find plug ',
     1                    i3, ' & trip diam. ', f7.3, ' mm' )
                      goto 20
c                     -- i.e. error return
                      end if                                              4e
                  if ( int( current(TSC_PLUG) + 0.5 ) .gt. plug_number
     1                     .or.
     2                 int( current(TSC_PLUG) + 0.5 ) .eq. plug_number
     3                     .and.
     4                 trip_diam / current(TSC_TRIP) .lt. 1.0 - TINY )
     5                    then                                            4b
                      write( STD_ERR, 10 ) abort_label,
     1                    plug_number, trip_diam
                      goto 20
c                     -- i.e. error return
                      end if                                              4e
                  got_plug_trip =
     1                plug_number .eq. int( current(TSC_PLUG) + 0.5 )
     2                .and. abs( trip_diam / current(TSC_TRIP) - 1.0 )
     3                      .le. TINY
                  goto 9
                  end if                                                  3e
c
c           now that the required plug number and trip diameter are
c           found, check that the pre-centrebody dynamic pressure is
c           within the calibrated range
c
c           If we have just found the first plug-number/trip-diameter
c           match and the pre-centrebody dynamic pressure is below the
c           calibrated minimum, use this minimum value.
c
              if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1            write( STD_ERR, 11 )  debug_label, plug_number,
     2                trip_diam
11                format( a, 'Found plug ', i3, ' and trip diam. ',
     1                f7.3, ' mm' )
              if ( cb_dyn_press / current(TSC_MIN_CB) .le. 1.0 - TINY )
     1                then                                                3b
                  if ( first_PT_match ) then                              4b
                      write( STD_ERR, 12 ) cb_dyn_press, 'lower',
     1                    'minimum', current(TSC_MIN_CB)
12                    format( ' QTTS_DYN : WARNING : Pre-centrebody '
     1                   'dynamic pressure ', g12.5, 1x, a /
     2                   22x, 'than the calibration ', a, 1x, g12.5 )
                      poly_arg = current(TSC_MIN_CB)
                  else                                                    4m
                      write( STD_ERR, 13 )
13                    format( ' QTTS_DYN : SERIOUS BUG : Required ',
     1                    'c/body dyn press is than lower calibrated ',
     2                    'minimum, but not the first plug/trip match' )
                      goto 20
c                     -- i.e. error return
                      end if                                              4e
c
c           If the pre-centrebody dynamic pressure is below the
c           calibrated maximum, we don't need to look any further
c
              else if ( cb_dyn_press / current(TSC_MAX_CB) .lt.
     1                  1.0 + TINY ) then                                 3m
                  poly_arg = cb_dyn_press
                  if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1                write( STD_ERR, 14 ) debug_label
14                    format( a, 'Found required calibration record. ' /
     1                    20x, 'C''body dyn. press is within ',
     2                    'calibration range' )
c
c           If the pre-centrebody dynamic pressure is above the
c           calibrated maximum, read the next file record until it is
c           not -- but only while the plug and trip are correct
c
              else                                                        3m
c                 start of loop
15                if ( got_plug_trip .and.
     1                  .not. TSC_FILE( TSC_IS_EOF, dummy ) .and.
     1                  cb_dyn_press / current(TSC_MAX_CB) .gt.
     2                  1.0 + TINY ) then                                  4b
                      call TSC_COPY( current, previous )
c                     read the file and abort if cannot read
                      if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1                    write( STD_ERR, 16 ) debug_label
16                        format( a, 'Reading file record; searching ',
     1                        'for higher speed range' )
                      if ( .not. TSC_FILE( TSC_READ, current ) ) then     5b
                          if ( TSC_FILE( TSC_IS_ERR, dummy ) )
     1                        goto 20
c                             -- i.e. return
                          end if                                          5e
                      got_plug_trip = plug_number .eq.
     1                    int( current(TSC_PLUG) +0.5 ) .and.
     2                    abs( trip_diam / current(TSC_TRIP) - 1.0 )
     3                        .le. TINY
                      first_PT_match = .not. got_plug_trip
                      if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1                    write( STD_ERR, 6 ) debug_label,
     2                        got_plug_trip, first_PT_match
                      goto 15
c                     end of loop
                      end if                                              4e
                  if ( .not. got_plug_trip ) then                         4b
                      write( STD_ERR, 12 ) cb_dyn_press, 'higher',
     1                    'maximum', previous(TSC_MAX_CB)
c                     The next 2 lines are due to the absence of pointers
c                     in standard fortran . With pointers the next 2 lines
c                     would be "calib_ptr = previous"
                      qtts_dyn = qpoly( log10( previous(TSC_MIN_CB) ),
     1                    previous(TSC_POLY),
     2                    int( previous(TSC_ORDER) + 0.5 ) ) *
     3                    cb_dyn_press
                      goto 20
c                     -- i.e. return
                  else if ( cb_dyn_press / current(TSC_MAX_CB) .le.
     1                 1.0 + TINY ) then                                  4m
                      if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1                    write( STD_ERR, 14 ) debug_label
                      poly_arg = cb_dyn_press
                  else
                      write( STD_ERR, 17 )
17                    format( ' QTTS_DYN : SERIOUS BUG : Impossible ',
     1                    'break-out from file reading loop ' )
                      goto 20
c                     -- i.e. return
                      end if                                              4e
                  end if                                                  3e
          else                                                            2m
              if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1            write( STD_ERR, 18 ) debug_label
18                format( a, 'Did not need to read ',
     1                'calibration file' )
              poly_arg = cb_dyn_press
              end if                                                      2e
c
c           now that the required plug number, trip diameter and pre-
c           centrebody dynamic pressure are found evaluate the free
c           dynamic stream dynamic pressure in the test section.
c
          qtts_dyn = qpoly( log10( poly_arg ), current(TSC_POLY),
     1        int( current(TSC_ORDER) + 0.5 ) ) * cb_dyn_press
          if ( TSC_FILE( TSC_ISBUG, dummy ) )
     1        write( STD_ERR, 19 ) debug_label, qtts_dyn / cb_dyn_press
19            format( a, 'Calculated polynomial = ',
     2            g13.6 )
          end if                                                          1e
20    continue
c
      if ( TSC_FILE( TSC_ISBUG, dummy ) ) then                            1b
          write( STD_ERR, 21 ) debug_label, qtts_dyn
21        format( a, 'Returned value = ', g13.6 /
     1            '    Press <RETURN> to continue', $ )
          read( STD_IN, 22 )
22        format()
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION TSC_FILE( ACTION, OUT_PARAMETERS )
c
c Handler routine for Test Section Calibration FILE .
c
c Arguments
c  integer action           # what to do
c  real    out_parameters   # parameters returned to calling routine
c
c ACTION for function TSC_FILE must be one of
c      --- condition monitors ---- the argument OUT_PARAMETERS is not used .
c   TSC_ISOPEN  -- returns .TRUE. only if the file is open
c   TSC_IS_ERR  -- returns .TRUE. only if an error has occurred reading or
c                  decoding a file record and clears the error
c   TSC_IS_EOF  -- returns .TRUE. only if an END-OF-FILE is encountered
c      ---file operations----
c   For the file operations, .TRUE. is returned if successful, .FALSE. if
c      unsuccessful .
c   TSC_OPEN    -- open the file, OUT_PARAMETERS are set to TSC_UNKNOWN .
c   TSC_CLOSE   -- close the file, OUT_PARAMETERS not used
c   TSC_REWIND  -- rewind the file, OUT_PARAMETERS are set to TSC_UNKNOWN .
c   TSC_READ    -- read the next calibration from the file and place the result
c                  in OUT_PARAMETERS
c      --- debugging ---- the argument OUT_PARAMETERS is not used .
c   TSC_PRINT   -- print ASCII representation of the calibration
c                  OUT_PARAMETERS; returns .TRUE. .
c   TSC_ISBUG   -- returns .TRUE. if TSC_FILE prints debug messages
c   TSC_DEBUG   -- returns .TRUE. and forces TSC_FILE to print debug messages
c   TSC_NOBUG   -- returns .FALSE. and disables printing of debug messages
c
c OUT_PARAMETERS consists of
c  1 = TSC_PLUG    test section plug number
c  2 = TSC_TRIP    diameter of trip balls (in mm)
c  3 = TSC_MIN_CB  minimum pre-centrebody dynamic pressure
c  4 = TSC_MAX_CB  maximum pre-centrebody dynamic pressure
c  5 = TSC_ORDER   order of the following polynomial
c  6 = TSC_POLY    first of a list of polynomial coefficients
c
c
c local variables
c  integer
c    error_index      # index into array of error messages
c    error_len        # length of error message
c    i                # index into array of polynomial coefficients
c    line_len         # length of line read from file
c    n_values         # number of items on line read from file
c    plug_no          # test secion plug number -- temporary
c    poly_order       # polynomial order -- temporary
c    record_number    # record number being read from file
c    res_ttsq_lu      # logical unit number reserved for the file
c    ttsq_lu          # actual logical unit number of the file
c    jljstn           # function : left justify string and count tokens
c    new_lun          #          : get new logical unit number
c    open_file        #          : open file
c    str_length       #          : non-blank length of string
c  logical
c   eof_error         # EOF encountered flag
c   debug             # enable printing debug messages
c  character
c    line*128         # line read from file
c    error_message(0:7)*30    # error message texts
c    TSC_NAME*10      # function : returns ASCII form of ACTION
c
c
      include 'stdio.def'
      include 'qtts_dyn.def'
c
      integer action, error_index, error_len, i, line_len, n_values,
     1        plug_no, poly_order, record_number, res_ttsq_lu, ttsq_lu,
     2       jljstn, new_lun, open_file, str_length
      real    out_parameters(*)
      logical eof_error, debug
      character line*128, error_message(0:7)*34, TSC_NAME*10
c
      save error_index, record_number, res_ttsq_lu, ttsq_lu,
     1     eof_error, debug, error_message
      data error_index, record_number, res_ttsq_lu, ttsq_lu /
     1     4 * 0 /
      data eof_error, debug / 2*.FALSE. /
      data error_message /
     z    'No error                          ',
     1    'Cannot read line from             ',
     2    'Too few values in initial line    ',
     3    'Cannot decode initial line of     ',
     4    'Invalid parameter value in        ',
     5    'Cannot decode poly''l coeffs in    ',
     6    'Reached end of                    ',
     7    'Unexpected EOF; missing poly''l in ' /
c
c
      if ( debug .and. action .ne. TSC_ISBUG )
     1    write( STD_ERR, 1 ) TSC_NAME( action )
1         format( ' TSC_FILE : DEBUG : Called with 1st argument = ', a )
c
c       test if the file is open -- ttsq_lu > 0 if it is
c       ************************
c
      if ( action .eq. TSC_ISOPEN ) then                                  1b
          TSC_FILE = ttsq_lu .gt. 0

c
c       tell the caller if an error has occurred
c       ****************************************
c
      else if ( action .eq. TSC_IS_ERR ) then                             1m
          TSC_FILE = error_index .gt. 0
          error_index = 0
c
c       tell the caller if EOF has been encountered
c       *******************************************
c
      else if ( action .eq. TSC_IS_EOF ) then                             1m
          TSC_FILE = eof_error
c
c       open the Test Section Calibration file
c       **************************************
c
      else if ( action .eq. TSC_OPEN ) then                               1m
          if ( res_ttsq_lu .eq. 0 ) then                                  2b
              res_ttsq_lu = new_lun()
              ttsq_lu = 0
              end if                                                      2e
          if ( ttsq_lu .eq. 0 ) then                                      2b
              ttsq_lu = open_file( res_ttsq_lu, TTSQ_FILE, 'old', 0 )
              record_number = 1
              error_index   = 0
              eof_error = .FALSE.
              if ( debug )
     1            write( STD_ERR, 2 ) TTSQ_FILE, ttsq_lu
2                 format( ' TSC_FILE : DEBUG : Opened calibration ',
     1                'file ', a, ' on logical unit ', i3 )
          else                                                            2m
              write( STD_ERR, 3 ) TTSQ_FILE
3             format( ' TSC_FILE : OPEN : WARNING : File ', a,
     1                ' already open.' )
              end if                                                      2e
          TSC_FILE = ttsq_lu .gt. 0
c
c       close the Test Section Calibration file
c       ***************************************
c
      else if ( action .eq. TSC_CLOSE ) then                              1m
          if ( ttsq_lu .gt. 0 ) then                                      2b
              close( UNIT=ttsq_lu )
              record_number = TSC_UNKNOWN
              if ( debug )
     1            write( STD_ERR, 4 ) ttsq_lu
4                 format( ' TSC_FILE : DEBUG : Closed calibration ',
     1                'file on logical unit ', i3 )
          else                                                            2m
              write( STD_ERR, 5 ) 'CLOSE', TTSQ_FILE
5             format( ' TSC_FILE : ', a, ' : ABORT : File ', a,
     1                ' not open' )
              end if                                                      2e
          TSC_FILE = ttsq_lu .gt. 0
          ttsq_lu = 0
c
c       rewind the Test Section Calibration file
c       ****************************************
c
      else if ( action .eq. TSC_REWIND ) then                             1m
          if ( ttsq_lu .gt. 0 ) then                                      2b
              rewind( UNIT=ttsq_lu )
              do 6 i = TSC_PLUG, TSC_POLY                                 3b
                  out_parameters(i) = TSC_UNKNOWN
6                 continue                                                3e
              eof_error = .FALSE.
              record_number = 1
              if ( debug )
     1            write( STD_ERR, 7 ) ttsq_lu
7                 format( ' TSC_FILE : DEBUG : Rewound calibration ',
     1                'file on logical unit ', i3 )
          else                                                            2m
              write( STD_ERR, 5 ) 'REWIND', TTSQ_FILE
              end if                                                      2e
          TSC_FILE = ttsq_lu .gt. 0
c
c       read the Test Section Calibration file
c       **************************************
c
      else if ( action .eq. TSC_READ ) then                               1m
          if ( ttsq_lu .gt. 0 ) then                                      2b
c
c           read the first line in a record
c           --- reads lines until one beginning with a '$' is found
c
              error_index = 1
c             repeat                                                      3b
8                 read( ttsq_lu, 9, END=20, ERR=17 ) line
9                 format( a )
                  line_len = jljstn( line, n_values )
                  if ( debug )
     1                write( STD_ERR, 10 ) ttsq_lu, line(:line_len)
10                    format( ' TSC_FILE : DEBUG : Read line from file',
     1                    ' on logical unit ', i3 / ' <<', a, '>>' )
                  if ( line(1:1) .ne. '$' ) goto 8
c                 until ( line(1:1) == '$' )                              3e
              error_index = 2
              if ( n_values .lt. 6 )
     1            goto 17
c
c           decode the values in the first line
c
              error_index = 3
              read( line(2:), *, ERR=17 )
     1            ( out_parameters(i), i = TSC_PLUG, TSC_ORDER )
              if ( debug )
     1            write( STD_ERR, 11 )
11                format( ' TSC_FILE : DEBUG : Calibration parameters ',
     1                'decoded' )
c
c           check the values read against reasonable limits
c
              error_index = 4
              plug_no = out_parameters(TSC_PLUG) + 0.5
              poly_order = out_parameters(TSC_ORDER) + 0.5
              if ( poly_order .lt. 0 .or.
     1             poly_order .gt. MAX_POLY .or.
     2             plug_no .lt. MIN_PLUG .or.
     3             plug_no .gt. MAX_PLUG .or.
     4             out_parameters(TSC_TRIP) .lt. MIN_TRIP .or.
     5             out_parameters(TSC_TRIP) .gt. MAX_TRIP .or.
     6             out_parameters(TSC_MIN_CB) .lt. MIN_CB_DYN .or.
     7             out_parameters(TSC_MIN_CB) .ge.
     8                 out_parameters(TSC_MAX_CB) .or.
     9             out_parameters(TSC_MAX_CB) .gt. MAX_CB_DYN )
     a            goto 17
c
c           read the continuation lines containing the polynomial
c
              error_index = 5
              read( ttsq_lu, *, END=21, ERR=17 ) ( out_parameters(i),
     1            i = TSC_POLY, TSC_POLY + poly_order )
              eof_error = .FALSE.
              error_index = 0
              record_number = record_number + 1
              TSC_FILE = .TRUE.
              if ( debug )
     1            write( STD_ERR, 12 )
12                format( ' TSC_FILE : DEBUG : Polynomial read and ',
     1                'decoded' )
          else                                                            2m
              write( STD_ERR, 5 ) 'READ', TTSQ_FILE
              TSC_FILE = .FALSE.
              end if                                                      2e
c
c       print calibration OUT_PARAMETERS
c       ********************************
c
      else if ( action .eq. TSC_PRINT ) then                              1m
          poly_order = max( 0.0, out_parameters(TSC_ORDER) + 0.5 )
          write( STD_OUT, 13 ) int( out_parameters(TSC_PLUG) + 0.5 ),
     1        out_parameters(TSC_TRIP), out_parameters(TSC_MIN_CB),
     2        out_parameters(TSC_MAX_CB), poly_order,
     3        ( out_parameters(i), i = TSC_POLY, TSC_POLY + poly_order)
13            format( ' TSC_FILE : PRINT : Calibration is' /
     1            '   test section plug number    ', i4 /
     2            '   diameter of trip balls (mm) ', f8.3 /
     3            '   pre-c''body dyn press - min  ', g12.5 /
     4            '   pre-c''body dyn press - max  ', g12.5 /
     5            '   order of polynomial         ', i4 /
     6            '   polynomial coefficients' / ( 1x, 5g15.7 ) )
          TSC_FILE = .TRUE.
c
c       Enquire, set or reset the debug flag
c       ************************************
c
      else if ( action .eq. TSC_ISBUG ) then                              1m
          TSC_FILE = debug
      else if ( action .eq. TSC_DEBUG ) then                              1m
          debug = .TRUE.
          TSC_FILE = .TRUE.
          write( STD_ERR, 14 ) 'Switched on'
14        format( ' TSC_FILE : DEBUG : ', a )
      else if ( action .eq. TSC_NOBUG ) then                              1m
          if ( debug )
     1        write( STD_ERR, 14 ) 'Switched off'
          debug = .FALSE.
          TSC_FILE = .FALSE.
          end if                                                          1e
c
15    if ( debug .and. action .ne. TSC_ISBUG )
     1    write( STD_ERR, 16 ) TSC_FILE, record_number
16        format( ' TSC_FILE : DEBUG : Returning with value = ', l1 /
     1            20x, 'Next database file record number = ', i4 )
      return
c
c
c           ERROR messages for READ : return with FALSE
c
17    error_len = str_length( error_message(error_index) )
      write( STD_ERR, 18 ) error_message(error_index)(1:error_len),
     1                     TTSQ_FILE, record_number
18    format( ' TSC_FILE : ABORT : ', a, ' file' /
     1    1x, a, ' at record number ', i4 )
      do 19 i = TSC_PLUG, TSC_POLY                                        1b
          out_parameters(i) = TSC_UNKNOWN
19        continue                                                        1e
      eof_error = .FALSE.
      TSC_FILE = .FALSE.
      goto 17
c
c
20    error_index = 6
      eof_error = .TRUE.
      goto 17
c
21    error_index = 7
      eof_error = .TRUE.
      goto 17
      end



      SUBROUTINE TSC_COPY( SOURCE, DESTIN )
c
c    Copy the SOURCE test section calibration to DESTINATION .
c
c Arguments
c   real source(TSC_POLY+MAX_POLY)  # source test section calibration
c        destin(TSC_POLY+MAX_POLY)  # destination test section calibration
c
      include 'stdio.def'
      include 'qtts_dyn.def'
c
      integer i
      real source(TSC_POLY+MAX_POLY), destin(TSC_POLY+MAX_POLY)
c
      do 1 i = TSC_PLUG, TSC_ORDER
          destin(i) = source(i)
1         continue
      do 2 i = TSC_POLY, TSC_POLY + int( source(TSC_ORDER) + 0.5 )
          destin(i) = source(i)
2         continue
      return
      end



      CHARACTER *10 FUNCTION TSC_NAME( ACTION )
c
c Returns an ASCII representation of the TSC_FILE action argument
c
      include 'stdio.def'
      include 'qtts_dyn.def'
      integer action
c
      if ( action .eq. TSC_ISOPEN ) then
          tsc_name = 'TSC_ISOPEN'
      else if ( action .eq. TSC_IS_ERR ) then
          tsc_name = 'TSC_IS_ERR'
      else if ( action .eq. TSC_IS_EOF ) then
          tsc_name = 'TSC_IS_EOF'
      else if ( action .eq. TSC_OPEN ) then
          tsc_name = 'TSC_OPEN  '
      else if ( action .eq. TSC_CLOSE ) then
          tsc_name = 'TSC_CLOSE '
      else if ( action .eq. TSC_REWIND ) then
          tsc_name = 'TSC_REWIND'
      else if ( action .eq. TSC_READ ) then
          tsc_name = 'TSC_READ  '
      else if ( action .eq. TSC_PRINT ) then
          tsc_name = 'TSC_PRINT'
      else if ( action .eq. TSC_ISBUG ) then
          tsc_name = 'TSC_ISBUG '
      else if ( action .eq. TSC_DEBUG ) then
          tsc_name = 'TSC_DEBUG '
      else if ( action .eq. TSC_NOBUG ) then
          tsc_name = 'TSC_NOBUG '
      else
          tsc_name = 'TSCunknown'
          end if
      return
      end


