c            ###########################################
c            ###                                     ###
c            ###            HW1LIB                   ###
c            ###     Single Hot Wire user library    ###
c            ###                                     ###
c            ###########################################
c
c CHARACTER *21 FUNCTION   GHW1_METHOD( )
c INTEGER *2    FUNCTION   JHW1_E2toAD( CTA_SQ )
c INTEGER       FUNCTION   JHW1_LOWER ( SPEEDS, INCREM, NPTS )
c INTEGER       FUNCTION   JHW1_METHOD( HOW )
c INTEGER       FUNCTION   JHW1_UPPER ( SPEEDS, INCREM, NPTS )
c LOGICAL       FUNCTION   EHW1_ADtoU ( AD_SIG, SPEED, INCREM, NPTS )
c LOGICAL       FUNCTION   EHW1_OPEN  ( CAL_REC, CALIB_FILE, AUX_LUN )
c REAL          FUNCTION   QHW1_ADtoE2( IADSIG )
c REAL          FUNCTION   QHW1_E2toU ( CTA_SQ )
c REAL          FUNCTION   QHW1_UtoE2 ( SPEED )
c               SUBROUTINE SHW1_EXAM  ( )
c               SUBROUTINE SHW1_CLOSE ( )
c               SUBROUTINE SHW1_LIMIT ( OUT_LU )
c

      LOGICAL FUNCTION EHW1_OPEN( CAL_REC, CALIB_FILE, AUX_LUN )
c
c       Hot Wire CALibration file OPeN
c Reads the CAL_RECth single Hot Wire calibration record from the calibration
c file CALIB_FILE .
c
c Arguments
c  integer   cal_rec          # record to be read from the calibration file
c                             #   0 = ask for record
c  character calib_file*(*)   # name of the calibration file
c                             #  ' ' = ask for file name
c  integer   aux_lun          # logical unit number of auxillary file (0=none)
c                             # The file is only written if 0 < aux_lun < 100
c
c
c constant
c  MAGIC='Kh'     # Magic number beginning each file record
c local variables
c  logical
c    equery       # function : read boolean
c    existf       #          : file exists ?
c  integer
c    i            # looping index
c    name_len     # length of file name
c    record_no    # required record number
c    reserved_lu  # value of logical unit number reserved for calibration file
c    unused       # unused return value
c    jhw1_limit   # function : control calibration clipping limits
c    jljstr       #          : left justify string
c    new_lun      #          : new logical unit number
c  real
c    expon        # 1 / ( n_points - 1 )
c    min_speed, max_speed   # minimum & maximum calibration data speeds
c    min_wire, max_wire     # minimum & maximum calibration data mean CTA O/P
c    speed_increm # speed increment for constructing interpolation table
c    trip_diam       # boundary layer trip diameter (mm)
c    wire_increm  # CTA O/P increment for constructing interpolation table
c    qhw1_poly    # function : evaluate polynomial
c  character
c    Cmagic*2     # magic number read from file
c    new_file*32  # new file name
c    prev_file*32 # previous file name
c  subroutine
c    Cresp        # read string
c    shw1_exam    # examine hot wire calibration file
c
      include 'stdio.def/list'
      include 'hw1lib.def'
      character MAGIC*2, id_abort*21
      parameter( MAGIC='Kh', id_abort = ' EHW1_OPEN : ABORT : ' )
c
      logical equery, existf
      integer aux_lun, cal_rec, i, name_len, record_no, reserved_lu,
     1    unused, jhw1_limit, jljstr, new_lun
      real expon, min_speed, max_speed, min_wire, max_wire,
     1    speed_increm, trip_diam, wire_increm, qhw1_poly
      character Cmagic*2, new_file*32, prev_file*32, calib_file*(*)
c
      include 'hw1lib.cmn/list'
      external hw1_block
      save name_len, prev_file
      data name_len / 1 /
      data prev_file / ' ' /
c
c       initialization of global variables
c
      data aux_lu, calib_lu, curr_rec, n_points, wire_chan / 5*0 /
c
c           if file name & record number are unchanged , return
c
      ehw1_open  = .TRUE.
      aux_lu     = 0
      if ( aux_lun .gt. 0 .and. aux_lun .lt. 100 )
     1    aux_lu = aux_lun
      new_file   = calib_file
      if ( prev_file .eq. new_file .and. cal_rec .eq. curr_rec
     1        .and. calib_lu .ne. 0 )
     2    return
      ehw1_open  = .FALSE.
      record_no  = max( 0, cal_rec )
c
c           if new file or none is open, open a calibration file
c
      if ( prev_file .ne. new_file .or. calib_lu .eq. 0 ) then
          if ( reserved_lu .eq. 0 ) then
              reserved_lu = new_lun()
              end if
          if ( calib_lu .ne. 0 )
     1        call close( calib_lu )
          name_len = max( 1, jljstr( new_file ) )
          if ( new_file .eq. ' ' ) then
              write( STD_OUT, 1 )
1             format( ' Single hot wire calibration file name ? ', $ )
              call Cresp( STD_IN, new_file, 4 )
2             format( a )
              name_len = max( 1, jljstr( new_file ) )
              end if
          if ( .not. Existf( new_file ) ) then
3             write( STD_OUT, 4 ) id_abort, new_file(:name_len)
4             format( a, 'cannot find file ', a )
              return
              end if
          open( UNIT=reserved_lu, FILE=new_file, STATUS='old',
     1        ACCESS='sequential', FORM='unformatted', ERR=13,
     2        RECORDTYPE='variable', BLOCKSIZE=8192 )
          calib_lu = reserved_lu
          prev_file = new_file
          curr_rec = 0
          if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1        write( aux_lu, 5 ) new_file
5             format( ' EHW1_OPEN : file name = ', a )
c
c           old file - rewind if new record precedes old one
c
      else if ( record_no .eq. 0 .or. record_no .lt. curr_rec ) then
          rewind( UNIT=calib_lu )
          curr_rec = 0
          end if
c
c           skip required number of records
c
      if ( record_no .gt. curr_rec + 1 ) then
          do 6 i = 1, record_no - curr_rec - 1
              read( calib_lu, END=15 )
              curr_rec = curr_rec + 1
6             continue
          end if
c
c
c       READ CALIBRATION FILE RECORD UNTIL THE RIGHT ONE
c
c     start of loop
7     if ( record_no .ne. curr_rec .or. record_no .eq. 0 ) then
          read( calib_lu, ERR=16, END=15 ) Cmagic,
     1        calib_no, Ctype, iplug, trip_diam, Cwire_ad, wire_chan,
     2        n_points, Amp_gain, Amp_offset,
     3        done_siddal, siddal_expon, siddal_order, siddal_error,
     4        ( siddal_poly(i), i=1,siddal_order+1 ),
     5        done_poly, poly_order, poly_error, poly_offset,
     6        ( poly_poly(i+1), i=0,poly_order ),
     7        ( air_speed(i), mean_wire(i), rms_wire(i), i= 1,n_points )
          if ( Cmagic .ne. MAGIC )
     1        goto 18
          if ( record_no .eq. 0 ) then
              call shw1_exam
              if ( .not. equery( STD_IN, 'Select this one ? ' ) )
     1            goto 7
              record_no = curr_rec
          else
              curr_rec = curr_rec + 1
              goto 7
              end if
          end if
c         end of loop
c
c       assign the A/D converter calibration constant
c
      if ( Cwire_ad .eq. 'DAS 1128 slo' ) then
          volts_to_AD = 400.0
      else if ( Cwire_ad .eq. 'DAS 250A ran' .or.
     1       Cwire_ad .eq. 'DAS 250A seq' ) then
          volts_to_AD = -409.6
      else
          write( STD_ERR, 8 ) id_abort
8         format( a, 'Unknown A/D converter' )
          return
          end if
c
      if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1    write( aux_lu, 10 ) curr_rec
10     format( ' EHW1_OPEN : record number = ', i4 )
c
c       Construct data tables for polynomial inversion by iterative
c       interpolation . First determine the limits of the data
c
      min_speed = air_speed(1)
      max_speed = air_speed(1)
      min_wire  = mean_wire(1)
      max_wire  = mean_wire(1)
      do 11 i = 2, n_points
          min_speed = min( min_speed, air_speed(i) )
          max_speed = max( max_speed, air_speed(i) )
          min_wire  = min( min_wire, mean_wire(i) )
          max_wire  = max( max_wire, mean_wire(i) )
11        continue
c
      expon = 1.0 / ( TABLE_ROWS - 1 )
c
c       table for polynomial method
c
      if ( done_poly ) then                                               1b
          wire_increm = ( max_wire / min_wire ) ** expon
          table(1,POLY_WIRE) = min_wire
          table(1,POLY_AIR)  = qhw1_poly(
     1        min_wire - poly_offset, poly_poly, poly_order )
          do 12 i = 2, TABLE_ROWS                                         2b
              table(i,POLY_WIRE) = table(i-1,POLY_WIRE) * wire_increm
              table(i,POLY_AIR)  = qhw1_poly( table(i-1,POLY_WIRE) -
     1            poly_offset, poly_poly, poly_order )
12            continue                                                    2e
c
c       Reset the "clipped by limits" counters
c       and calculate extrapolation limits for polynomial method
c
          method = POLY_METHOD
          unused = JHW1_LIMIT( HW1_LIM_EV_0, 0.0 )
          unused = JHW1_LIMIT( HW1_LIM_CALC, 0.0 )
          end if                                                          1e
c
c       table for Siddal-Davies method
c
      if ( done_siddal ) then                                             1b
          speed_increm = ( max_speed / min_speed ) **
     1        ( siddal_expon * expon )
          table(1,SIDD_AIR)  = min_speed ** siddal_expon
          table(1,SIDD_WIRE) = qhw1_poly( table(1,SIDD_AIR),
     1        siddal_poly, siddal_order )
          do 20 i = 2, TABLE_ROWS                                         2b
              table(i,SIDD_AIR)  = table(i-1,SIDD_AIR) * speed_increm
              table(i,SIDD_WIRE) = qhw1_poly( table(i-1,SIDD_AIR),
     1            siddal_poly, siddal_order )
20            continue                                                    2e
c
c       Reset the "clipped by limits" counters
c       and calculate extrapolation limits for polynomial method
c
          method = SIDD_METHOD
          unused = JHW1_LIMIT( HW1_LIM_EV_0, 0.0 )
          unused = JHW1_LIMIT( HW1_LIM_CALC, 0.0 )
          end if                                                          1e
c
c       allocate a permissable default method
c
      if ( .not. done_siddal .and. .not. done_poly ) then
          write( STD_ERR, 9 ) id_abort
9         format( a, 'Record contains no calibration relations.' )
          method = 0
          return
      else if ( .not. done_siddal ) then
          method = POLY_METHOD
      else if ( .not. done_poly .or. method .eq. 0 ) then
          method = SIDD_METHOD
          end if
c
      ehw1_open = .TRUE.
      return
c
c       ERROR TRAPS
c
13    write( STD_ERR, 14 ) id_abort, new_file(:name_len)
14    format( a, 'Cannot open file ', a )
      return
c
c           ask for rewind if End Of File
c
15    if ( .not. Equery( STD_IN,
     1                   'End of file ; rewind & continue ?' ) ) then
          if ( Equery( STD_IN, 'Will Abort : Are you sure ?' ) )
     1        return
          end if
      curr_rec = 0
      rewind( UNIT=calib_lu )
      goto 7
c
16    write( STD_ERR, 17 ) id_abort
17    format( a, 'Cannot read from file ', a )
      return
c
18    write( STD_ERR, 19 ) id_abort
19    format( a, 'Not a hot wire calibration file .' /
     1    'Wrong magic number in ', a )
      return
      end





      SUBROUTINE SHW1_CLOSE
c
c       Close the Single hot wire calibration file .
c
      include 'stdio.def/list'
      include 'hw1lib.def/list'
      include 'hw1lib.cmn/list'
c
      if ( calib_lu .ne. 0 ) then
          close( UNIT=calib_lu )
          calib_lu = 0
          end if
      return
      end



      REAL FUNCTION QHW1_ADtoE2( IADSIG )
c
c Convert a single hot wire A/D reading into a CTA output voltage squared
c value . The result value can be converted into an air speed value using
c QHW1_E2toU . This function requires information from the hot-wire
c calibration file .
c
c Argument
c  integer *2 iadsig    # A/D converter reading
c
c local variables
c  logical
c    ehw1_open  # function : open calibration file
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      integer *2 iadsig
      include 'hw1lib.cmn/list'
c
c       open calibration file if necessary
c
      if ( calib_lu .eq. 0 ) then
          write( STD_ERR, 1 )
          if ( .not. ehw1_open( 0, ' ', 0 ) )
     1        return
          end if
c
c       do the conversion
c
      QHW1_ADtoE2 = ( ( real( max( 0, min( iadsig, 4095 ) ) ) /
     1              volts_to_AD - amp_offset ) / amp_gain ) **2
      return
1     format( ' QHW1_ADtoE2 : WARNING : Calibration file not opened' )
      end



      INTEGER *2 FUNCTION JHW1_E2toAD( CTA_SQ )
c
c Convert a the CTA output voltage squared value into a single hot wire A/D
c reading using the amplifier settings & A/D converter identifier from
c the hot-wire calibration file .
c
c Argument
c  real CTA_sq  # hot-wire voltage squared
c
c local variables
c  logical
c    ehw1_open  # function : open calibration file
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      real CTA_sq
      include 'hw1lib.cmn/list'
c
c
c       open calibration file if necessary
c
      if ( calib_lu .eq. 0 ) then
          write( STD_ERR, 1 )
          if ( .not. ehw1_open( 0, ' ', 0 ) )
     1        return
          end if
c
c       do the conversion
c
      JHW1_E2toAD = min( int( ( sqrt( CTA_sq ) * amp_gain + amp_offset )
     1                   * volts_to_AD + 0.5 ), 4095 )
      return
1     format( ' JHW1_E2toAD : WARNING : Calibration file not opened' )
      end




      REAL FUNCTION QHW1_UtoE2( SPEED )
c
c Calculate CTA output voltage squared (a single hot-wire A/D sample) from
c an air speed value using the specified method .
c
c Argument
c  real speed   # air speed
c
c local variables
c  logical
c    ehw1_open  # function : open calibration file
c  integer
c    straddle   # index for straddling target value in interpolation table
c    Jbsrch     # function : binary search
c  real
c    U          # air speed or air speed **n
c    QHW1_implic# function : implicit objective function
c    qhw1_poly  #          : evaluate polynomial
c    qzerli     #          : find zero by linear interpolation
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      integer straddle, Jbsrch
      real speed, U, QHW1_implic, qhw1_poly, qzerli
      include 'hw1lib.cmn/list'
      external QHW1_implic
c
      if ( calib_lu .eq. 0 ) then                                         1b
          write( STD_ERR, 1 )
1         format( ' QHW1_UtoE2 : WARNING : Calibration file not opened')
          if ( .not. ehw1_open( 0, ' ', 0 ) )
     1        return
          end if                                                          1e
c
c     Siddal-Davies method
c     CTA_output  **2 = polynomial( air_speed ** siddal_expon )
c     -- apply the polynomial explicitly
c
      if ( method .eq. SIDD_METHOD .and. done_siddal ) then               1b
          u = speed ** siddal_expon
          if ( u .lt. limits(HW1_MIN,SIDD_AIR) ) then                     2b
              u = limits(HW1_MIN,SIDD_AIR)
              n_clip(HW1_MIN,SIDD_AIR) = n_clip(HW1_MIN,SIDD_AIR) + 1
          else if ( u .gt. limits(HW1_MAX,SIDD_AIR) ) then                2m
              u = limits(HW1_MAX,SIDD_AIR)
              n_clip(HW1_MAX,SIDD_AIR) = n_clip(HW1_MAX,SIDD_AIR) + 1
              end if                                                      2e
          QHW1_UtoE2 = qhw1_poly( u, siddal_poly, siddal_order )
c
c     Polynomial method
c     air_speed = polynomial( CTA_output )
c     -- solution of implicit equation by interpolative iteration
c        don't bother with explicit solutions -- all polynomials will
c        be of order 3 or higher
c     ideally, extrapolation from the calibration data should be disallowed
c        but clipping is disabled because, in reality, it is a nuisance
c
      else if ( method .eq. POLY_METHOD .and. done_poly ) then            1m
          u = speed
          if ( u .lt. limits(HW1_MIN,POLY_AIR) ) then                     2b
c             u = limits(HW1_MIN,POLY_AIR)
              n_clip(HW1_MIN,POLY_AIR) = n_clip(HW1_MIN,POLY_AIR) + 1
          else if ( u .gt. limits(HW1_MAX,POLY_AIR) ) then                2m
c             u = limits(HW1_MAX,POLY_AIR)
              n_clip(HW1_MAX,POLY_AIR) = n_clip(HW1_MAX,POLY_AIR) + 1
              end if                                                      2e
          straddle = max( 1, min( TABLE_ROWS - 1,
     1        Jbsrch( table(1,POLY_AIR), 1, TABLE_ROWS, u ) ) )
          implic_offset = u
          QHW1_UtoE2 = qzerli( table(straddle,POLY_WIRE),
     1        table(straddle+1,POLY_WIRE), QHW1_implic, TOLERANCE ) **2
      else                                                                1e
          write( STD_ERR, 2 )
2         format( ' QHW1_UtoE2 : ABORT : Unknown or unavailable method')
          end if
      return
      end




      REAL FUNCTION QHW1_IMPLIC( ABSCISSA )
c
c This is the implicit function to be solved iteratively by linear
c interpolation .
c
c Argument
c  real abscissa  # SIDDAL_METHOD : air_speed ** siddal_expon
c                 # POLY_METHOD   : CTA voltage
c
c local variables
c  real qhw1_poly # function : evaluate polynomial
c
      include 'stdio.def'
      include 'hw1lib.def'
      real abscissa, qhw1_poly
      include 'hw1lib.cmn'
c
c     Siddal-Davies method - conversion of CTA output to air speed
c     CTA_output  **2 = polynomial( air_speed ** siddal_expon )
c     -- evaluate polynomial and subtract offset
c
      if ( method .eq. SIDD_METHOD .and. done_siddal ) then
          QHW1_implic = qhw1_poly( abscissa, siddal_poly, siddal_order )
     1        - implic_offset
c
c     Polynomial method - conversion of air speed to CTA output
c     air_speed = polynomial( CTA_output )
c     -- evaluate polynomial and subtract offset
c
      else if ( method .eq. POLY_METHOD .and. done_poly ) then
          QHW1_implic = qhw1_poly( abscissa - poly_offset, poly_poly,
     1        poly_order ) - implic_offset
      else
          QHW1_implic = 0.0
          end if
      return
      end



      REAL FUNCTION QHW1_E2toU( CTA_SQ )
c
c Convert CTA output voltage squared value ( calculated from a single
c hot-wire A/D reading ) into an air speed value using the specified method .
c
c Argument
c  real CTA_sq  # CTA output voltage squared
c
c local variables
c  logical
c    ehw1_open  # function : open calibration file
c  integer
c    status     # status flag returned from quadratic solver
c    straddle   # index for straddling target value in interpolation table
c    Jbsrch     # function : binary search
c  real
c    CTA_2      # CTA output or output squared
c    QHW1_implic# function : implicit objective function
c    qhw1_poly  #          : evaluate polynomial
c    qzerli     #          : find zero by linear interpolation
c  complex
c    U1, U2     # roots of quadratic
c  subroutine
c    squadc     # solve quadratic equation
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      integer status, straddle, Jbsrch
      real CTA_sq, CTA_2, QHW1_implic, qhw1_poly, qzerli
      complex U1, U2
      include 'hw1lib.cmn/list'
      external QHW1_implic
c
c
      if ( calib_lu .eq. 0 ) then                                         1b
          write( STD_ERR, 1 )
1       format( ' QHW1_E2toU : WARNING : Calibration file not opened' )
        if ( .not. ehw1_open( 0, ' ', 0 ) )
     1      return
        end if                                                            1e
c
c     Siddal-Davies method
c     CTA_output  **2 = polynomial( air_speed ** siddal_expon )
c     -- for order
c         =1 : linear    -- easy to solve explicitly
c         =2 : quadratic -- explicit solution for speed (more tricky)
c         >2 : solution of implicit equation by interpolative iteration
c
      if ( method .eq. SIDD_METHOD .and. done_siddal ) then               1b
          CTA_2 = CTA_sq
c
c         input clipping
c            detection remains; actual clipping disabled
          if ( CTA_2 .lt. limits(HW1_MIN,SIDD_WIRE) ) then                2b
c             CTA_2 = limits(HW1_MIN,SIDD_WIRE)
              n_clip(HW1_MIN,SIDD_WIRE) = n_clip(HW1_MIN,SIDD_WIRE) + 1
          else if ( CTA_2 .gt. limits(HW1_MAX,SIDD_WIRE) ) then           2m
c             CTA_2 = limits(HW1_MAX,SIDD_WIRE)
              n_clip(HW1_MAX,SIDD_WIRE) = n_clip(HW1_MAX,SIDD_WIRE) + 1
              end if                                                      2e
c
c         King's law -- linear
          if ( siddal_order .eq. 1 ) then                                 2b
              QHW1_E2toU = ( CTA_2 - siddal_poly(1) ) / siddal_poly(2)
c
c         Siddal & Davies -- quadratic
          else if ( siddal_order .eq. 2 ) then                            2m
              call squadc( U1, U2, siddal_poly(3), siddal_poly(2),
     1                siddal_poly(1) - CTA_2, status, TOLERANCE )
c
c             select & check the required root:
c             both roots should be real & distinct
              if ( status .ne. 1 ) then                                   3b
                  write( STD_OUT, 2 )
2                 format( ' QHW1_E2toU : WARNING : Complex or equal ',
     1                    'roots .' )
                  QHW1_E2toU = real( U1 )
              else if ( siddal_poly(3) .lt. 0.0 .and.
     1                  siddal_poly(2) .gt. 0.0 ) then                    3m
                  QHW1_E2toU = min( real( U1 ), real( U2 ) )
              else if ( siddal_poly(3) .gt. 0.0 ) then                    3m
                  QHW1_E2toU = max( real( U1 ), real( U2 ) )
                  end if                                                  3e
c
c         higher order polynomial
          else if ( siddal_order .gt. 2 ) then                            2m
              straddle = max( 1, min( TABLE_ROWS - 1,
     1          Jbsrch( table(1,SIDD_WIRE), 1, TABLE_ROWS, CTA_2 ) ) )
              implic_offset = CTA_2
              QHW1_E2toU = qzerli( table(straddle,SIDD_AIR),
     1            table(straddle+1,SIDD_AIR), QHW1_implic, TOLERANCE )
              end if                                                      2e
c
c         output clipping
          if ( QHW1_E2toU .lt. limits(HW1_MIN,SIDD_AIR) ) then            2b
              QHW1_E2toU = limits(HW1_MIN,SIDD_AIR)
              n_clip(HW1_MIN,SIDD_AIR) = n_clip(HW1_MIN,SIDD_AIR) + 1
          else if ( QHW1_E2toU .gt. limits(HW1_MAX,SIDD_AIR) ) then       2m
              QHW1_E2toU = limits(HW1_MAX,SIDD_AIR)
              n_clip(HW1_MAX,SIDD_AIR) = n_clip(HW1_MAX,SIDD_AIR) + 1
              end if                                                      2e
          QHW1_E2toU = QHW1_E2toU ** ( 1.0 / siddal_expon )
c
c     Polynomial method
c     air_speed = polynomial( CTA_output )
c     -- apply the polynomial explicitly
c
      else if ( method .eq. POLY_METHOD .and. done_poly ) then            1m
          CTA_2 = sqrt( CTA_sq )
c
c         input clipping
          if ( CTA_2 .lt. limits(HW1_MIN,POLY_WIRE ) ) then               2b
              CTA_2 = limits(HW1_MIN,POLY_WIRE)
              n_clip(HW1_MIN,POLY_WIRE) = n_clip(HW1_MIN,POLY_WIRE) + 1
          else if ( CTA_2 .gt. limits(HW1_MAX,POLY_WIRE) ) then           2m
              CTA_2 = limits(HW1_MAX,POLY_WIRE)
              n_clip(HW1_MAX,POLY_WIRE) = n_clip(HW1_MAX,POLY_WIRE) + 1
              end if                                                      2e
c         evaluation of polynomial
          QHW1_E2toU = qhw1_poly( CTA_2 - poly_offset,
     1        poly_poly, poly_order )
      else                                                                1m
          write( STD_ERR, 5 )
5         format( ' QHW1_E2toU : ABORT : Unknown or unavailable method')
          end if
      return
      end



      LOGICAL FUNCTION EHW1_ADtoU( AD_SIG, SPEED, INCREM, NPTS )
c
c       calculate air speed - Calibration Law Assumption
c
c Convert NPTS A/D converter readings in array AD_SIG at increments of INCREM
c into air speed values SPEED using the specified method . The same data
c increment INCREM is used in SPEED.
c
c Arguments
c  integer *2 ad_sig(*) # A/D converter readings
c  real    speed(*)     # air speed values
c  integer increm       # data array indexing increments
c          npts         # number of data points
c
c local variable
c  logical
c    ehw1_open  # function : open hot-wire calibration file
c  integer
c    i          # looping index
c    nlast      # index of last data item
c  real
c    CTA        # CTA output voltage
c    QHW1_E2toU # function : convert CTA voltage squared to air speed
c    qhw1_poly  #          : evaluate polynomial
c
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      integer *2 ad_sig(*)
      integer i, increm, nlast, npts
      real CTA, speed(*), qhw1_E2toU, qhw1_poly
      include 'hw1lib.cmn'
c
      EHW1_ADtoU = .FALSE.
      if ( calib_lu .eq. 0 ) then                                         1b
          write( STD_ERR, 1 )
1         format( ' EHW1_ADtoU : WARNING: Calibration file not opened' )
          if ( .not. ehw1_open( 0, ' ', 0 ) )
     1        return
        end if                                                            1e
c
c     Siddal-Davies method
c     CTA_output  **2 = polynomial( air_speed ** siddal_expon )
c     -- for order
c         =1 : linear -- easy to solve explicitly
c         =2 : quadratic -- explicit solution for speed (more tricky)
c         >2 : solution of implicit equation by interpolative iteration
c     Use the routine QHW1_E2toU to do the work because inversion of the
c     calibration formula is quite complicated and the overhead/duplication
c     associated with the function call is relatively small.
c
      nlast = ( npts - 1 ) * increm + 1
      if ( method .eq. SIDD_METHOD .and. done_siddal ) then               1b
          do 2 i = 1, nlast, increm                                       2b
              CTA = ( ( real( ad_sig(i) ) / volts_to_AD - amp_offset ) /
     1                amp_gain ) **2
              speed(i) = QHW1_E2toU( CTA )
2             continue                                                    2e
          EHW1_ADtoU = .TRUE.
c
c     Polynomial method
c     air_speed = polynomial( CTA_output )
c     -- apply the polynomial explicitly
c     Calling QHW1_E2toU would require an additional multiply and square root
c     so we do all the work here
c
      else if ( method .eq. POLY_METHOD .and. done_poly ) then            1m
          do 3 i = 1, nlast, increm                                       2b
              CTA = ( real(ad_sig(i)) / volts_to_AD - amp_offset )
     1              / amp_gain
c
c         input clipping -- count only; actual clipping disabled
              if ( CTA .lt. limits(HW1_MIN,POLY_WIRE ) ) then             3b
c                 CTA = limits(HW1_MIN,POLY_WIRE)
                  n_clip(HW1_MIN,POLY_WIRE) =
     1                n_clip(HW1_MIN,POLY_WIRE) + 1
              else if ( CTA .gt. limits(HW1_MAX,POLY_WIRE) ) then         3m
c                 CTA = limits(HW1_MAX,POLY_WIRE)
                  n_clip(HW1_MAX,POLY_WIRE) =
     1                n_clip(HW1_MAX,POLY_WIRE) + 1
                  end if                                                  3e
c         evaluation of polynomial
              speed(i) = qhw1_poly( CTA - poly_offset, poly_poly,
     1                              poly_order )
3             continue
          EHW1_ADtoU = .TRUE.
      else                                                                2m
          write( STD_ERR, 4 )
4         format( ' EHW1_ADtoU : ABORT : Unknown or unavailable method')
          end if                                                          2e
      return
      end



      SUBROUTINE SHW1_EXAM
c
c         EXAMine the Single Hot Wire CALibration
c    Displays all of the single hot wire calibration information
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical Equery
      integer i
      include 'hw1lib.cmn'
c
      if ( calib_lu .eq. 0 ) then
          write( STD_OUT, 1 )
1         format( ' SHW1_EXAM : calibration not available' )
          return
          end if
c
c       calibration incidentals
c
      write( STD_OUT, 2, ERR=6 )
     1    calib_no, Ctype, iplug, Cwire_ad, wire_chan, n_points,
     2    Amp_gain, Amp_offset
      if ( 0 .gt. aux_lu .and. aux_lu .lt. 100 )
     1    write( aux_lu, 2, ERR=7 )
     2        calib_no, Ctype, iplug, Cwire_ad, wire_chan, n_points,
     3        Amp_gain, Amp_offset
2     format( '******** CALIBRATION INCIDENTALS ********' /
     1        6x, 'Calibration number', 9x, i3 /
     2        6x, 'Type of hot wire used  ', a32 /
     3        6x, 'Wind tunnel plug number    ', i4 /
     4        6x, 'A/D converter used', 9x, a12 /
     5        6x, 'A/D converter channel', 6x, i3 /
     6        6x, 'No. of calibration points  ', i3 /
     7        6x, 'Amplifier gain', 13x, f9.5 /
     8        16x, 'offset', 11x, f9.5 )
c
c       Siddal-Davies calibration
c
      if ( done_siddal ) then
          write( STD_OUT, 3, ERR=6 )
     1        siddal_expon, siddal_order, siddal_error,
     2        ( i, siddal_poly(i+1), i = 0, siddal_order )
3         format( / '   ***** Siddal-Davies calibration *****' /
     1        6x, 'Air speed exponent', 9x, f10.7 /
     2        6x, 'Order of fitted polynomial ', i2 /
     3        6x, 'RMS error of regression    ', g13.6 /
     4        6x, 'Calibration polynomial ' /
     5        ( 11x, i2, g13.6 ) )
          if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1        write( aux_lu, 3, ERR=7 )
     2            siddal_expon, siddal_order, siddal_error,
     3            ( i, siddal_poly(i+1), i = 0, siddal_order )
          end if
c
c       Polynomial-of-voltage calibration
c
      if ( done_poly ) then
          write( STD_OUT, 4, ERR=6 ) poly_order, poly_error,
     1        ( i, poly_poly(i+1), i = 0, poly_order )
4         format( / '    ***** Polynomial calibration *****' /
     1        6x, 'Order of fitted polynomial ', i2 /
     2        6x, 'RMS error of regression    ', g13.6 /
     3        6x, 'Calibration polynomial ' /
     4        ( 11x, i2, g13.6 ) )
          if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1        write( aux_lu, 4, ERR=7 ) poly_order, poly_error,
     2            ( i, poly_poly(i+1), i = 0, poly_order )
          end if
c
c       calibration data
c
      if ( Equery( STD_IN, ' SHW1_EXAM : Display data ?' ) ) then
          write( STD_OUT, 5, ERR=6 )   ( i, air_speed(i), mean_wire(i),
     1        rms_wire(i), i= 1,n_points )
5         format( / 16('*'), ' Calibration data ', 16('*') /
     1        ' No. Air-speed Mean-volts RMS-volts ' /
     2        ( 1x, i3, 3f10.4 ) )
          if ( aux_lu .gt. 0 .and. aux_lu .lt. 100 )
     1        write( aux_lu, 5, ERR=7 )  ( i, air_speed(i),
     2            mean_wire(i), rms_wire(i), i= 1,n_points )
          end if
      return
c
c       error traps
c
6     write( STD_ERR, 8 ) 'screen'
      return
7     write( STD_ERR, 8 ) 'auxillary file'
8     format( ' SHW1_EXAM : Cannot write calibration to ', a )
      return
      end



      INTEGER FUNCTION JHW1_LIMIT( ACTION, VALUE )
c
c Controls the clipping limits for the selected calibration method
c
c Arguments
c  integer action   # >=0 # Write clipping limits for the selected calibration
c                   #     # method to logical unit ACTION
c                   #  -1 # Clear the clipping event counters for the selected
c                   #     # calibration method.
c                   #  -2 # Print menu of ACTION values & return ACTION value
c                   #     #     from user
c                   #  -3 # Print menu of ACTION values, return ACTION value
c                   #     #     from user and perform the indicated ACTION
c                   #  -4 # Re-calculate clipping limits
c                   #  -5 # Set factor bases for extrapolation to default
c                   #     #     values & re-calculate clipping limits
c                   #  -6 # Set factor base for extrapolating below minimum
c                   #     #     data to VALUE & re-calculate clipping limits
c                   #  -7 # Set factor base for extrapolating above maximum
c                   #     #     data to VALUE & re-calculate clipping limits
c                  #################################################
c                  ##     #  min/max #  variable  #  method       ##
c                  ##  -8 #  minimum #  air speed # polynomial    ##
c              Set ##  -9 #  minimum # CTA output # polynomial    ##
c             clip ## -10 #  minimum #  air speed # Siddal-Davies ##
c           limits ## -11 #  minimum # CTA output # Siddal-Davies ##
c               to ## -12 #  maximum #  air speed # polynomial    ##
c            VALUE ## -13 #  maximum # CTA output # polynomial    ##
c                  ## -14 #  maximum #  air speed # Siddal-Davies ##
c                  ## -15 #  maximum # CTA output # Siddal-Davies ##
c                  #################################################
c
c  real    value   # value assigned to limit or extrapolation factor base
c
c
      include 'stdio.def'
      include 'hw1lib.def'
      character  INV_METHOD*41
      parameter(
     1    INV_METHOD='Unknown or unavailable calibration method' )
c
c
      integer action, i, item, men_lim_prt(4), out_lu,
     1        Jresp
      character Ghw1_method*21
      real    max_speed, max_wire, min_speed, min_wire, rvalue, value,
     1        qhw1_poly, Qresp
c
      save men_lim_prt
      data men_lim_prt / POLY_AIR, SIDD_AIR, POLY_WIRE, SIDD_WIRE /
      include 'hw1lib.cmn'
c
      if ( action .eq. HW1_LIM_MENU .or. action .eq. HW1_LIM_ENQU ) then  1b
          write( STD_OUT, 1 )
     1        Ghw1_method(), min_extr_base, max_extr_base
1         format( '  JHW1_LIMIT : Selected calibration method : ', a /
     1            '   0) Return ' /
     2            '   1) Clear the clipping event counters' /
     3            '   2) Not used' /
     4            '   3) Display clipping limits & events' /
     5            '   4) Re-calculate clipping limits' /
     6            '      Set factor bases for extrapolating' /
     7            '   5)   to defaults & recalculate' /
     8            '   6)   below minimum & recalculate  ', f8.6 /
     9            '   7)   above maximum & recalculate  ', f8.3 )
          write( STD_OUT, 2 )
     1        ( limits( HW1_MIN, men_lim_prt(i) ), i = 1, 4 ),
     2        ( limits( HW1_MAX, men_lim_prt(i) ), i = 1, 4 )
2         format( '      Set minimum limits' /
     1            18x, '--polynomial--   -Siddal-Davies-' /
     2            '      air speed   8) ', g12.6e1, '  10) ', g12.6e1 /
     3            '      CTA output  9) ', g12.6e1, '  11) ', g12.6e1 /
     4            '      Set maximum limits' /
     5            18x, ' --polynomial--   -Siddal-Davies-' /
     6            '      air speed  12) ', g12.6e1, '  14) ', g12.6e1 /
     7            '      CTA output 13) ', g12.6e1, '  15) ', g12.6e1 )
          item = Jresp( STD_IN, 0, 15, 4 )
          if ( item .eq. - HW1_LIM_ENQU )
     1        out_lu = STD_OUT
          if ( item .eq. - HW1_LIM_MINB ) then                            2b
              write( STD_OUT, 3 )
3             format( ' JHW1_LIMIT : What value ? ' )
              rvalue = Qresp( STD_IN, 0.0001, 1.0, 4 )
          else if ( item .eq. - HW1_LIM_MAXB ) then                       2m
              write( STD_OUT, 3 )
              rvalue = Qresp( STD_IN, 1.0, 1000.0, 4 )
          else if ( - HW1_LIM_MIN - HW1_LIM_POLY - HW1_LIM_AIR
     1                       .le. item .and. item .le.
     2              - HW1_LIM_MAX - HW1_LIM_SIDD - HW1_LIM_CTA ) then     2m
              write( STD_OUT, 3 )
              rvalue = Qresp( STD_IN, 0.0, 1000.0, 4 )
              end if                                                      2e
      else if ( action .ge. 0 ) then                                      1m
          out_lu = action
          item   = - HW1_LIM_ENQU
      else
          item   = -action
          rvalue = value
          end if                                                          1e
      jhw1_limit = -item
c
c  ACTION = HW1_LIM_EV_0 = - ITEM = -1
c    Clear the clipping event counters for the selected calibration method.
c
      if ( item .eq. - HW1_LIM_EV_0 ) then                                1b
          if ( method .eq. SIDD_METHOD .and. done_siddal ) then           2b
              n_clip(HW1_MIN,SIDD_AIR)  = 0
              n_clip(HW1_MIN,SIDD_WIRE) = 0
              n_clip(HW1_MAX,SIDD_AIR)  = 0
              n_clip(HW1_MAX,SIDD_WIRE) = 0
          else if ( method .eq. POLY_METHOD .and. done_poly ) then        2m
              n_clip(HW1_MIN,POLY_AIR)  = 0
              n_clip(HW1_MIN,POLY_WIRE) = 0
              n_clip(HW1_MAX,POLY_AIR)  = 0
              n_clip(HW1_MAX,POLY_WIRE) = 0
          else                                                            2m
              write( STD_ERR, 4 ) INV_METHOD
4             format( ' JHW1_LIMIT : ABORT : ', a )
              end if                                                      2e
c
c  ACTION >= HW1_LIM_ENQU = - ITEM
c    Write clipping limits for the selected
c    calibration method to logical unit ACTION
c
      else if ( item .eq. - HW1_LIM_ENQU ) then                           1m
          if ( method .eq. SIDD_METHOD .and. done_siddal ) then           2b
              write( out_lu, 5, ERR=11 )
5             format( ' Siddal Davies method' / '  Limit   ',
     1            '-----air-speed**n----   --CTA-output-squared-' )
              write( out_lu, 6, ERR=11 )
     1            limits(HW1_MIN,SIDD_AIR),  n_clip(HW1_MIN,SIDD_AIR),
     2            limits(HW1_MIN,SIDD_WIRE), n_clip(HW1_MIN,SIDD_WIRE),
     3            limits(HW1_MAX,SIDD_AIR),  n_clip(HW1_MAX,SIDD_AIR),
     4            limits(HW1_MAX,SIDD_WIRE), n_clip(HW1_MAX,SIDD_WIRE)
6             format(12x,'value    clip events    value    clip events'/
     1            ' minimum  ', g12.6e1, 2x, i7, 3x, g12.6e1, 2x, i7 /
     2            ' maximum  ', g12.6e1, 2x, i7, 3x, g12.6e1, 2x, i7 )
          else if ( method .eq. POLY_METHOD .and. done_poly ) then        2m
              write( out_lu, 7, ERR=11 )
7             format( ' Polynomial method' / '  Limit   ',
     1            '------air-speed------   -----CTA-output------' )
              write( out_lu, 5, ERR=11 )
     1            limits(HW1_MIN,POLY_AIR),  n_clip(HW1_MIN,POLY_AIR),
     2            limits(HW1_MIN,POLY_WIRE), n_clip(HW1_MIN,POLY_WIRE),
     3            limits(HW1_MAX,POLY_AIR),  n_clip(HW1_MAX,POLY_AIR),
     4            limits(HW1_MAX,POLY_WIRE), n_clip(HW1_MAX,POLY_WIRE)
          else                                                            2m
              write( STD_ERR, 4 ) INV_METHOD
              end if                                                      2e
      else if ( item .eq. - HW1_LIM_CALC .or.
     1          item .eq. - HW1_LIM_DEFB .or.
     2          item .eq. - HW1_LIM_MINB .or.
     3          item .eq. - HW1_LIM_MAXB ) then                           1m
c
c    ACTION = HW1_LIM_DEFB = - ITEM
c    Set extrapol'n factor bases to defaults & recalculate
c
          if ( item .eq. - HW1_LIM_DEFB ) then                            2b
              min_extr_base = D_MIN_EXTR_BASE
              max_extr_base = D_MAX_EXTR_BASE
c
c    ACTION = HW1_LIM_MINB = - ITEM
c    Set factor base for extrapolating below minimum
c
          else if ( item .eq. - HW1_LIM_MINB ) then                       2m
              min_extr_base = rvalue
c
c    ACTION = HW1_LIM_MAXB = - ITEM
c    Set factor base for extrapolating above maximum
c
          else if ( item .eq. - HW1_LIM_MAXB ) then                       2m
              max_extr_base = rvalue
              end if                                                      2e
c
c    ACTION = HW1_LIM_CALC = - ITEM  Re-calculate clipping limits
c       First determine the limits of the data
c
          min_speed = air_speed(1)
          max_speed = air_speed(1)
          min_wire  = mean_wire(1)
          max_wire  = mean_wire(1)
          do 8 i = 2, n_points                                            2b
              min_speed = min( min_speed, air_speed(i) )
              max_speed = max( max_speed, air_speed(i) )
              min_wire  = min( min_wire, mean_wire(i) )
              max_wire  = max( max_wire, mean_wire(i) )
8             continue                                                    2e
          if ( method .eq. SIDD_METHOD .and. done_siddal ) then           2b
c
c       extrapolation limits for Siddal-Davies method
c
              limits(HW1_MIN,SIDD_AIR)  = min_speed ** siddal_expon *
     1            min_extr_base ** ( 1.0 / real( siddal_order ) )
              limits(HW1_MAX,SIDD_AIR)  = max_speed ** siddal_expon *
     1            max_extr_base ** ( 1.0 / real( siddal_order ) )
              limits(HW1_MIN,SIDD_WIRE) = qhw1_poly(
     1            limits(HW1_MIN,SIDD_AIR), siddal_poly, siddal_order )
              limits(HW1_MAX,SIDD_WIRE) = qhw1_poly(
     1            limits(HW1_MAX,SIDD_AIR), siddal_poly, siddal_order )
c
c           special limits for quadratic Siddal-Davies
              if ( siddal_order .eq. 2 ) then                             3b
c
c               a>0 & b>0
c                     E2min=c, E2max=Inf, Umin=0, Umax=Inf
                  if ( siddal_poly(3) .gt. 0.0 .and.
     1                 siddal_poly(2) .gt. 0.0 ) then                     4b
                      limits(HW1_MIN,SIDD_WIRE) = max(
     1                    limits(HW1_MIN,SIDD_WIRE), siddal_poly(1) )
c
c               a>0 & b<0
c                     E2min=c-bb/4a, E2max=Inf, Umin=-b/2a, Umax=Inf
                  else if ( siddal_poly(3) .gt. 0.0 .and.
     1                      siddal_poly(2) .lt. 0.0 ) then                4m
                      limits(HW1_MIN,SIDD_WIRE) = max(
     1                    limits(HW1_MIN,SIDD_WIRE),
     2                    - siddal_poly(2) / 2.0 / siddal_poly(3) )
                      limits(HW1_MIN,SIDD_AIR)  = max(
     1                    limits(HW1_MIN,SIDD_AIR), siddal_poly(1) -
     2                    siddal_poly(2) **2 / 4.0 / siddal_poly(3) )
c
c               a<0 & b>0
c                     E2min=c, E2max=c-bb/4a, Umin=0, Umax=-b/2a
                  else if ( siddal_poly(3) .lt. 0.0 .and.
     1                      siddal_poly(2) .gt. 0.0  ) then               4m
                      limits(HW1_MIN,SIDD_WIRE) = max(
     1                    limits(HW1_MIN,SIDD_WIRE), siddal_poly(1) )
                      limits(HW1_MAX,SIDD_WIRE) = min(
     1                    limits(HW1_MAX,SIDD_WIRE), siddal_poly(1) -
     2                    siddal_poly(2) **2 / 4.0 / siddal_poly(3) )
                      limits(HW1_MAX,SIDD_AIR)  = min(
     1                    limits(HW1_MAX,SIDD_AIR),
     2                    - siddal_poly(2) / 2.0 / siddal_poly(3) )
c               a<0 & b<0
                  else if ( siddal_poly(3) .lt. 0.0 .and.
     1                      siddal_poly(2) .lt. 0.0  ) then               4m
                      write( STD_ERR, 3 )
     1                'Illegal calibration coefficients; negative slope'
                      end if                                              4e
                  end if                                                  3e
          else if ( method .eq. POLY_METHOD .and. done_poly ) then        2m
              limits(HW1_MIN,POLY_WIRE) = min_wire *
     1            min_extr_base ** ( 1.0 / real( poly_order ) )
              limits(HW1_MAX,POLY_WIRE) = max_wire *
     1            max_extr_base ** ( 1.0 / real( poly_order ) )
              limits(HW1_MIN,POLY_AIR)  =
     1            qhw1_poly( limits(HW1_MIN,POLY_WIRE) -
     2                       poly_offset, poly_poly, poly_order )
              limits(HW1_MAX,POLY_AIR)  =
     1            qhw1_poly( limits(HW1_MAX,POLY_WIRE) -
     2                       poly_offset, poly_poly, poly_order )
          else                                                            2m
              write( STD_ERR, 4 ) INV_METHOD
              end if                                                      2e
c
c    ACTION = HW1_LIM_MIN + HW1_LIM_POLY + HW1_LIM_AIR =  -8
c           = HW1_LIM_MIN + HW1_LIM_POLY + HW1_LIM_CTA =  -9
c           = HW1_LIM_MIN + HW1_LIM_SIDD + HW1_LIM_AIR = -10
c           = HW1_LIM_MIN + HW1_LIM_SIDD + HW1_LIM_CTA = -11
c    Set minimum limit
c
      else if ( - HW1_LIM_MIN - HW1_LIM_POLY - HW1_LIM_AIR
     1                 .le. item .and. item .le.
     2          - HW1_LIM_MIN - HW1_LIM_SIDD - HW1_LIM_CTA ) then         1m
          limits( HW1_MIN, item + 1 + HW1_LIM_MIN ) = rvalue
          write( STD_ERR, 9 )
9         format( ' JHW1_LIMIT : WARNING : Limit values not checked' )
c
c    ACTION = HW1_LIM_MAX + HW1_LIM_POLY + HW1_LIM_AIR = -12
c           = HW1_LIM_MAX + HW1_LIM_POLY + HW1_LIM_CTA = -13
c           = HW1_LIM_MAX + HW1_LIM_SIDD + HW1_LIM_AIR = -14
c           = HW1_LIM_MAX + HW1_LIM_SIDD + HW1_LIM_CTA = -15
c    Set maximum limit
c
      else if ( - HW1_LIM_MAX - HW1_LIM_POLY - HW1_LIM_AIR
     1                 .le. item .and. item .le.
     2          - HW1_LIM_MAX - HW1_LIM_SIDD - HW1_LIM_CTA ) then         1m
          limits( HW1_MAX, item + 1 + HW1_LIM_MAX ) = rvalue
          write( STD_ERR, 9 )
          end if                                                          1e
      return
c
11    write( STD_ERR, 4 )  'Cannot write'
      return
      end



      INTEGER FUNCTION JHW1_UPPER( SPEEDS, INCREM, NPTS )
c
c         exceed Upper bound Count
c    Counts the number of values of air speed in SPEEDS which are greater than
c the maximum air speed value in the calibration table .
c
c Arguments
c  real speeds      # array of air speed values
c  integer increm   # indexing increment for UHW1
c          npts     # number of air speed values tested
c
c local variables
c  logical EHW1_OPEN# function open hot-wire calibration file
c  integer i        # looping index
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      integer i, increm, npts
      real speeds(*)
      include 'hw1lib.cmn'
c
      if ( calib_lu .eq. 0 ) then
          write( STD_ERR, 1 )
1         format( ' JHW1_UPPER : WARNING: Calibration file not opened' )
          if ( .not. ehw1_open( 0, ' ', 0 ) )
     1        return
          end if
c
      Jhw1_upper = 0
      do 2 i = 1, increm * ( npts - 1 ) + 1, increm
          if ( speeds(i) .gt. table(TABLE_ROWS,POLY_AIR) )
     1        Jhw1_upper = Jhw1_upper + 1
2         continue
      return
      end



      INTEGER FUNCTION JHW1_LOWER( SPEEDS, INCREM, NPTS )
c
c         exceed Lower bound Count
c Counts the number of values of air speed in SPEEDS which are less than
c the minimum air speed value in the calibration table .
c
c Arguments
c  real    speeds   # array of air speed values
c  integer increm   # indexing increment for UHW1
c          npts     # number of air speed values tested
c
c local variables
c  logical EHW1_OPEN# function open hot-wire calibration file
c  integer i
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      logical ehw1_open
      integer i, increm, npts
      real speeds(*)
      include 'hw1lib.cmn'
c
      if ( calib_lu .eq. 0 ) then
          write( STD_ERR, 1 )
1         format( ' JHW1_LOWER : WARNING: Calibration file not opened' )
          if ( .not. ehw1_open( 0, ' ', 0 ) )
     1        return
          end if
c
      Jhw1_lower = 0
      do 2 i = 1, increm * ( npts - 1 ) + 1, increm
          if ( speeds(i) .lt. table(1,POLY_AIR) )
     1        Jhw1_lower = Jhw1_lower + 1
2         continue
      return
      end



      INTEGER FUNCTION JHW1_METHOD( HOW )
c
c Sets and reads the linearization relation used .
c  HOW = 0              ask the user
c  HOW = 1              polynomial of voltage .
c  HOW = 2              Siddal-Davies formula
c  HOW = anything-else  no change
c The returned value is the selected method .
c
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      integer how, Jresp
      include 'hw1lib.cmn'
c
      if ( how .eq. POLY_METHOD .or. how .eq. SIDD_METHOD ) then
          method = how
      else if ( how .eq. 0 ) then
          write( STD_OUT, 1 )
1         format(
     1        ' What method of hot-wire linearisation are you using ?' /
     2        '      1 = Polynomial of voltage' /
     3        '      2 = Siddal-Davies method   ', $ )
          method = Jresp( STD_IN, 1, 2, 4 )
          end if
      JHW1_METHOD = method
      return
      end



      CHARACTER *21 FUNCTION GHW1_METHOD()
c
c Returns the currently selected method .
c
      include 'stdio.def'
      include 'hw1lib.def'
c
      include 'hw1lib.cmn'
c
      if ( method .eq. POLY_METHOD ) then
          ghw1_method = 'Polynomial of voltage'
      else if ( method .eq. SIDD_METHOD ) then
          ghw1_method = 'Siddal-Davies formula'
      else
          ghw1_method = 'Bug unknown method   '
          end if
      return
      end



      REAL FUNCTION QHW1_POLY( X, B, K )
c
c       POLYnomial evaluation
c    Evaluates the Kth order polynomial :-
c P(x) = b(1) + b(2) * x + b(3) * x**2 + ... + b(k) * x**(k-1) + b(k+1) * x**k
c Arguments are single precision, but the arithmetic is double precision
c
c Arguments
c  real x,      # dependent variable
c       b(k+1)  # coefficients of the polynomial
c  integer  k   # order of the polynomial
c
c variable
c  integer i    # looping index
c  double precision
c    double_x
c    sum
c
      include 'stdio.def'
      integer i, k
      real x, b(*)
      double precision double_x, sum
c
      Qhw1_poly = 0.0
      if ( k .lt. 0 ) return
      sum = dble( B(k+1) )
      double_x = dble( x )
      do 1 i = k, 1, -1
          sum = sum * double_x + dble( b(i) )
1         continue
      Qhw1_poly = real( sum )
      return
      end



      BLOCK DATA HW1_BLOCK
      include 'stdio.def'
      include 'hw1lib.def'
c
      include 'hw1lib.cmn'
      data aux_lu, calib_lu, curr_rec, method / 4 * 0 /
      data n_clip / TABLE_COLS*0, TABLE_COLS*0 /
      data min_extr_base,   max_extr_base   /
     1     D_MIN_EXTR_BASE, D_MAX_EXTR_BASE /
      end
