c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


      PROGRAM BOUNDARY_LAYER_PROFILE
c
c Analysis of raw data files which have been created by GENACQ and contain
c boundary layer profile data .
c
c Invocation :
c    PRF [ <data_file> ]
c
c PRF Main menu
c   0) Exit
c   1) Name of raw data file  data_file.dat
c   2) Setup menu
c   3) Baratron calibration  baratron.bar
c   4) Hot-wire calibration  hot_wire.hw1
c   5) Examine data file
c
c
c local variables
c  integer
c    item               # selected menu item
c    len_df_name        # length of data file name
c    get_baratron       # function : get details of baratron calibration
c    get_hot_wire       #          : get details of hot-wire calibration
c    iargc              #          : number of command line arguments
c    jljstr             #          : left justify string
c    Jresp              #          : read integer
c  real
c    qcpclk             # function : read CPU time clock
c  character
c    Cdata_file*64      # name of data file
c    Cyes*5             # function : no / yes / query
c  subroutine
c    Cresp              # f77_ute : read char string
c    getarg             # f77_ute : get program argument
c    ilgwait            # ilglib  : wait for CR
c    newscr             # hpprev  : clear alphanumeric screen
c    prf_examine_file   # contains all raw data file handling
c    prf_glob_setup     # setup menu for global parameters
c    scpclk             # f77_ute : start CPU clock
c
c global variables
c   dmp_units, n_profiles
c
      include 'stdio.def/list'
      include 'prf.def/list'
c
      integer item, len_df_name, get_baratron, get_hot_wire, iargc,
     1    jljstr, Jresp
      real qcpclk
      character Cdata_file*64, Cyes*5
      include 'prf.cmn/list'
c
      data got_bar_cal, got_hw1_cal, len_df_name / NO, NO, 1 /
      data Cdata_file / ' ' /
c
c               Print Label
c
      call scpclk()
      write( STD_OUT, 1 )
1     format( 21x, 'PRF : Boundary layer profile analysis' /
     1        21x, 37( '=' ) / )
c
c         Get program arguments -- if any
c             first argument  = data file name
c
      if ( iargc() .gt. 0 )
     1    call getarg( 1, Cdata_file )
      len_df_name = max( jljstr( Cdata_file ), 1 )
      n_profiles = 1
c
c     start of loop                                                       1b
c
c           PRINT MENU
c
2         call newscr()
          write( STD_OUT, 3 ) Cdata_file(1:len_df_name),
     1        Cyes( got_bar_cal ), Cyes( got_hw1_cal )
3         format( ' PRF Main menu' / '   0) Exit' /
     1        '   1) Name of raw data file ', a /
     2        '   2) Setup menu' /
     3        '   3) Baratron calibration  ', a /
     4        '   4) Hot-wire calibration  ', a /
     5        '   5) Examine data file' /
     6        '   6) Print table of profile properties' /
     7        '   7) Clear table of profile properties ', $ )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              write( STD_OUT, 4 ) qcpclk()
4             format( ' Total CPU time = ', f8.2, ' seconds' )
              stop
c
c           DATA FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' Enter name of data file ', $ )
              call Cresp( STD_IN, Cdata_file, MAX_TRIES )
              len_df_name = max( jljstr( Cdata_file ), 1 )
c
c           MAIN SETUP
c
          else if ( item .eq. 2 ) then                                    2m
              call prf_glob_setup()
c
c           SPECIFY BARATRON CALIBRATION
c
          else if ( item .eq. 3 ) then                                    2m
              got_bar_cal = get_baratron()
c
c           SPECIFY HOT-WIRE CALIBRATION
c
          else if ( item .eq. 4 ) then                                    2m
              got_hw1_cal = get_hot_wire()
c
c           EXAMINE DATA FILE
c
          else if ( item .eq. 5 ) then                                    2m
              if ( Cdata_file .ne. ' ' ) then                             3b
                  call prf_examine_file( Cdata_file )
              else                                                        3m
                  write( STD_OUT, 6 )
6                 format( ' PRF : WARNING : Data file not specified' )
                  call ilgwait()
                  end if                                                  3e
c
c           PRINT TABLE OF PROFILE PROPERTIES
c
          else if ( item .eq. 6 ) then                                    2m
              call print_properties( STD_OUT, 1, n_profiles - 1 )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            call print_properties( dmp_units(DUMP_AUXL),
     2                                   1, n_profiles - 1 )
c
c           CLEAR TABLE OF PROFILE PROPERTIES
c
          else if ( item .eq. 7 ) then                                    2m
              n_profiles = 1
              end if                                                      2e
          goto 2
c         end of loop                                                     1e
      end



      INTEGER FUNCTION GET_BARATRON()
c
c Open the baratron calibration file or change the baratron calibration
c record .
c
c Baratron calibration menu
c   0) Exit
c   1) file name     baratron.bar
c   2) record number 0
c
c local variables
c  logical
c    EBAR_OPEN      # function : open pressure sensor calibration file
c  integer
c    name_len       # length of file name
c    record         # record number of baratron calibration file
c    item           # selected menu item
c    Jresp          # function : integer response
c    jljstr         #          : left justify string
c  character
c    Cbar_file*64   # name of baratron calibration file
c  subroutine
c    Cresp          # read character string
c
c global variables
c  dmp_units
c
      include 'stdio.def'
      include 'prf.def'
c
      logical EBAR_OPEN
      integer name_len, record, item, Jresp, jljstr
      character Cbar_file*64
      include 'prf.cmn'
      save Cbar_file, name_len, record
      data Cbar_file / ' ' /
      data name_len, record / 1, 0 /
c
      name_len = max( 1, jljstr( Cbar_file ) )
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) Cbar_file(1:name_len), record
2         format( ' Baratron calibration menu' / '   0) Exit' /
     1        '   1) file name     ', a /
     2        '   2) record number ', i3, ' ', $ )
          item = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              get_baratron = NO
              if ( Cbar_file .ne. ' ' ) then                              3b
                  if ( EBAR_OPEN( record, Cbar_file,
     1                            dmp_units(DUMP_AUXL) ) )
     2                get_baratron = YES
                  end if                                                  3e
              return
c
c           BARATRON FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Baratron calibration file name ? ', $ )
              call Cresp( STD_IN, Cbar_file, MAX_TRIES )
              name_len = max( 1, jljstr( Cbar_file ) )
c
c           RECORD NUMBER
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Baratron calibration record number ? ' /
     1            '  ( 0 = view calibrations before selecting ) ', $ )
              record = Jresp( STD_IN, 0, 50, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      INTEGER FUNCTION GET_HOT_WIRE()
c
c Open the hot-wire calibration file or change the hot-wire calibration
c record .
c  Hot-wire calibration menu
c    0) Exit
c    1) file name     hot_wire.hw1
c    2) record number 0
c    3) select method
c
c local variables
c  logical
c    ehw1_open      # function : open hot-wire calibration file
c  integer
c    item           # selected menu item
c    name_len       # length of file name
c    record         # record number of baratron calibration file
c    Jhw1_method    # function : select conversion method
c    Jresp          #          : integer response
c    jljstr         #          : left justify string
c  character
c    Chw1_file*64   # name of baratron calibration file
c    Ghw1_method*21 # function : methods of conversion
c  subroutine
c    Cresp          # read character string
c
c global variables
c  dmp_units
c
      include 'stdio.def'
      include 'prf.def'
c
      logical ehw1_open
      integer item, name_len, record, unused,
     1        Jhw1_limit, Jhw1_method, Jresp, jljstr
      character Chw1_file*64, Ghw1_method*21
      include 'prf.cmn'
c
      save Chw1_file, name_len, record
      data Chw1_file / ' ' /
      data name_len, record / 1, 0 /
c
      get_hot_wire = NO
      if ( hw1_method .eq. 0 )
     1    hw1_method = max( 1, min( 2, Jhw1_method( 0 ) ) )
      name_len = max( 1, jljstr( Chw1_file ) )
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) Chw1_file(1:name_len), record,
     1        Ghw1_method()
2         format( ' Hot-wire calibration menu' / '   0) Exit' /
     1        '   1) file name       ', a /
     2        '   2) record number ', i3 /
     3        '   3) select method   ', a /
     4        '   4) clipping limit control ', $ )
          item = Jresp( STD_IN, 0, 4, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( get_hot_wire .eq. NO .and. Chw1_file .ne. ' ' ) then   3b
                  if ( ehw1_open( record, Chw1_file,
     1                            dmp_units(DUMP_AUXL) ) )
     2                get_hot_wire = YES
                  end if                                                  3e
              return
c
c           HOT-WIRE FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Hot-wire calibration file name ? ', $ )
              call Cresp( STD_IN, Chw1_file, MAX_TRIES )
              name_len = max( 1, jljstr( Chw1_file ) )
              get_hot_wire = NO
c
c           RECORD NUMBER
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Hot-wire calibration record number ? ' /
     1            '  ( 0 = view calibrations before selecting ) ', $ )
              record = Jresp( STD_IN, 0, 50, MAX_TRIES )
              get_hot_wire = NO
c
c           SELECT METHOD
c
          else if ( item .eq. 3 ) then                                    2m
              hw1_method = max( 1, min( 2, Jhw1_method( 0 ) ) )
c
c           CLIPPING LIMIT CONTROL
c
          else if ( item .eq. 4 ) then                                    2m
              if ( get_hot_wire .eq. NO .and. Chw1_file .ne. ' ' ) then   3b
                  if ( ehw1_open( record, Chw1_file,
     1                            dmp_units(DUMP_AUXL) ) )
     2                get_hot_wire = YES
                  end if                                                  3e
              unused = Jhw1_limit( -3, 0.0 )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE PRF_GLOB_SETUP()
c
c    Menu for setting global setup parameters
c PRF : Global setup menu
c   0) Exit
c   1) Auxillary file name    prf.aux
c   2) Plot file name    prf.plt
c      Single point analysis -
c   3)   repeat enabled                             no
c   4)   change parameters for each profile         no
c   5)   change parameters for each probe position  no
c
c local variables
c  integer
c   aux_len       # length of auxillary file name
c   item          # selected menu item
c   plot_len      # length of plot file name
c   jljstr        # function : left justify string
c   Jresp         #          : read integer
c   new_lun       #          : get new logical unit number
c   open_file     #          : open file
c  character      #
c   Cold_aux*32   # previous name of auxillary file
c   Cold_plot*32  # previous name of plot file
c   Chost(0:2)*8  # host machine descriptions
c   Cyes*5        # function : no / yes / query
c  subroutine
c   newscr        # hpprev : new screen
c   Cresp         # f77_ute: read string
c   open_dump     # open dump file
c
c global variables
c  dmp_units, dmp_names, host_type, setdo_posn, sp_rep_enab, verbosity
c
      include 'stdio.def'
      include 'prf.def'
c
      integer aux_len, item, plot_len,
     1    jljstr, Jresp, new_lun, open_file
      character Cold_aux*32, Cold_plot*32, Chost(0:2)*8, Cyes*5
c
      include 'prf.cmn'
      save aux_len, plot_len, Cold_aux, Chost
      data aux_len, plot_len / 1, 1 /
      data Chost, Cold_aux, Cold_plot /
     1    'VAX-VMS ', 'PDP11/34', 'IBM XT  ', ' ', ' ' /
c
      aux_len  = max( 1, jljstr( dmp_names(DUMP_AUXL) ) )
      plot_len = max( 1, jljstr( dmp_names(DUMP_PLOT) ) )
c     start of loop                                                       1b
c
c           PRINT MENU
c
1         call newscr()
          write( STD_OUT, 2 ) dmp_names(DUMP_AUXL)(1:aux_len),
     1        dmp_names(DUMP_PLOT)(1:plot_len), verbosity,
     2        Chost(host_type), Cyes(sp_rep_enab), Cyes(setdo_posn)
2         format( ' PRF : Global setup menu' / '   0) Exit' /
     1        '   1) Auxillary file name    ', a /
     2        '   2) Plot file name         ', a /
     3        '   3) Verbosity', 14x, i2 /
     4        '   4) Host type', 14x, a /
     5        '      Single point analysis -' /
     6        '   5)   repeat enabled', 29x, a /
     7        '   6)   change parameters for each probe position  ', a ,
     8        '   ', $ )
          item = Jresp( STD_IN, 0, 6, MAX_TRIES )
c
c           EXIT
c           open/close the auxillary files according to the changes in
c           the file names
c
          if ( item .eq. 0 ) then                                         2b
c
c             auxillary file handling
              if ( dmp_names(DUMP_AUXL) .ne. Cold_aux )
     1            call close_dump( DUMP_AUXL )
              if ( dmp_names(DUMP_AUXL) .ne. ' ' .and.
     1                dmp_units(DUMP_AUXL) .eq. 0 )
     2            call open_dump( DUMP_AUXL )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            Cold_aux = dmp_names(DUMP_AUXL)
c
c             plot file handling
              if ( dmp_names(DUMP_PLOT) .ne. Cold_plot )
     1            call close_dump( DUMP_PLOT )
              if ( dmp_names(DUMP_PLOT) .ne. ' ' .and.
     1                dmp_units(DUMP_PLOT) .eq. 0 )
     2            call open_dump( DUMP_PLOT )
              if ( dmp_units(DUMP_PLOT) .ne. 0 )
     1            Cold_plot = dmp_names(DUMP_PLOT)
              return
c
c           AUXILLARY FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Name of file ? ', $ )
              call Cresp( STD_IN, dmp_names(DUMP_AUXL), MAX_TRIES )
              aux_len = max( 1, jljstr( dmp_names(DUMP_AUXL) ) )
c
c           PLOT FILE NAME
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, dmp_names(DUMP_PLOT), MAX_TRIES )
              plot_len = max( 1, jljstr( dmp_names(DUMP_PLOT) ) )
c
c           VERBOSITY
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' What is verbosity ? ', $ )
              verbosity = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c             HOST MACHINE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' What is the host machine ? 0) VAX - VMS' /
     1            28x, '1) PDP11/34 - unix' /
     2            28x, '2) IBM XT/AT - MS-DOS' )
              host_type = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c           ENABLE REPEAT OF SINGLE POINT ANALYSIS
c
          else if ( item .eq. 5 ) then                                    2m
              sp_rep_enab = mod( sp_rep_enab + 1, 3 )
c
c           CHANGE PARAMETERS FOR EACH PROBE POSITION
c
          else if ( item .eq. 6 ) then                                    2m
              setdo_posn = mod( setdo_posn + 1, 3 )
              end if                                                      2e
          goto 1
c         end of loop                                                     1e
      end



      SUBROUTINE PRINT_PROPERTIES( OUT_LU, FIRST, LAST )
c
c Print summary of boundary layer profile properties from FIRST to LAST .
c
c Arguments
c  integer out_lu   # logical unit number of output file
c          first    # first profile to be printed
c          last     # last profile to be printed
c
c local variables
c  logical
c   ein       # function : is 1st-arg-char in 2nd-arg-string ?
c  integer
c   i, j      # row and column indices for profile array
c  subroutine
c   ilgwait   # ilglib :
c
c global variables
c   n_profiles, profile
c
      include 'stdio.def'
      include 'prf.def'
      logical ein
      integer i, j, out_lu, first, last
      include 'prf.cmn'
c
c       Cannot print empty table
c
      if ( n_profiles .le. 1 )
     1    return
      if ( first .gt. last )
     1    call swap4( first, last )
c
c  profile   free   friction    B/L     disp-   momentum  energy   Coles
c  number   stream  velocity thickness lacement  thick-   thick    thick
c           speed                     thickness   ness     ness     ness
c          (m/s)     (m/s)     (mm)     (mm)      (mm)     (mm)     (mm)
c
      write( out_lu, 1 ) ( i, ( profile(i,j),
     1    j = FREESTR_SPEED, COLES_THICK ),
     2    i = max( first, 1 ), min( last, n_profiles - 1 ) )
1     format( / 21x, 'DIMENSIONED PROFILE CONSTANTS' / 21x, 29( '=' ) /
     1    '  profile   free   friction    B/L     disp-   ',
     2         'momentum  energy   Coles' /
     3    '  number   stream  velocity thickness lacement  ',
     4         'thick-   thick-    thick-' /
     5    '           speed', 21x, 'thickness', 3( '   ness  ' ) /
     6    10x, 2( '(m/s)     ' ), 5( '(mm)     ' ) /
     7    ( 3x, i3, 3x, f9.5, g11.5e2, 5f9.5 ) )
      if ( out_lu .eq. STD_OUT )
     1    call ilgwait
c
c  profile    H       H32      H13      Cf     Coles PI
c
      write( out_lu, 2 ) ( i, ( profile(i,j),
     1    j = SHAPE_FACTOR, COLES_PI_WAKE ),
     2    i = max( first, 1 ), min( last, n_profiles - 1 ) )
2     format( / 21x, 'SHAPE & FRICTION FACTORS' / 21x, 24('=') /
     1  '  profile    H       H32      H13      Cf     Coles PI' /
     3  ( 3x, i3, 3x, 5f9.6 ) )
      if ( out_lu .eq. STD_OUT )
     1    call ilgwait
c
c  profile   Re          Re          Re        delta
c  number  delta    displacement  momentum     plus
c
      write( out_lu, 3 ) ( i, ( profile(i,j),
     1    j = REYN_DELTA, DELTA_PLUS ),
     2    i = max( first, 1 ), min( last, n_profiles - 1 ) )
3     format( / 18x, 'REYNOLDS NUMBERS' / 21x, 16('=') /
     1    '  profile   Re          Re          Re        delta' /
     2    '  number  delta    displacement  momentum     plus' /
     3    ( 3x, i3, 3x, 4g12.5e1 ) )
      if ( out_lu .eq. STD_OUT )
     1    call ilgwait
      return
      end



      SUBROUTINE PRF_EXAMINE_FILE( DATA_FILE )
c
c This routine contains all the raw data file handling except the simple
c enquiries .
c
c Argument
c  character data_file*(*)
c
c local variables
c  logical
c    first_hw_data      # first hot-wire data
c    got_data           # the data for a profile has been read
c    got_traverse       # traverse reading is available
c    new_profile        # new profile is available for processing
c    equery             # function : f77_ute: read logical
c    erdfacq            #          : rdflib : read data
c    erdfmhd            #          : rdflib : read module header
c    erdfopn            #          : rdflib : open raw data file
c    esetdo             #          : set single point analysis parameters
c  integer
c    advance_table      # YES/NO advance to next row of profile properties table
c    i                  # looping index
c    item               # selected menu item
c    rdf_rec            # raw data file record number
c    get_baratron       # function : get baratron calibration
c    get_hot_wire       #          : get hot-wire calibration
c    Jresp              #          : read integer
c  integer *2
c    Mdata(MAX_SAMPLES) # data from A/D data
c  real
c    CPU_clock          # CPU time elapsed
c    y_actual           # actual probe position
c    y_required         # required probe position
c    qcpclk             # function : f77_ute : read CPU clock
c    qmuair             #          : f77_ute : dynamic viscosity
c    Qresp              #          : f77_ute : read real
c    qextvreq           #          : rdflib : required probe position
c    qextvact           #          : rdflib : actual probe position
c  character
c    Csublab*8          # sub-block label
c    Cyes*5             # function : 'no', 'yes', 'query'
c  subroutine
c    baratron_speed     # calculate free stream speed from pitot-static tube
c    print_properties   # print summary of profile properties
c    process_hot_wire   # process hot-wire data for single probe location
c    spfanal            # profile analysis
c    newscr             # hpprev : new screen
c    sextall            # rdflib : display header parameters
c    srdfcls            # rdflib : close raw data file
c    srdfhost           # rdflib : specify host type
c    srdfinit           # rdflib : read initialise header
c    srdfmute           # rdflib : specify verbosity
c
c global variables
c   air_temp, air_pressure, air_density, air_viscosity, bar_pts, dmp_units,
c   first_hw_data, got_bar_cal, got_hw1_cal, got_do, host_type, n_points,
c   n_profiles, n_samples, n_segs, plug_no, result, trip_diam, verbosity
c
c
      include 'stdio.def'
      include 'prf.def'
      logical first_hw_data, got_data, got_traverse, new_profile,
     1    equery, erdfacq, erdfmhd, erdfopn, esetdo
      integer advance_table, i, item, rdf_rec,
     1    get_baratron, get_hot_wire, Jresp
      integer *2 Mdata(MAX_SAMPLES)
      real CPU_clock, y_actual, y_required,
     1    qcpclk, qextvreq, qextvact, qmuair, Qresp
      character data_file*(*), Csublab*8, Cyes*5
      include 'prf.cmn'
      save advance_table
      data advance_table / YES /
c
      if ( .not. erdfopn( data_file, rdf_rec, dmp_units(DUMP_AUXL) )
     1       ) then                                                       1b
         write( STD_OUT, 1 )
1        format( ' PRF : Cannot open data file ' )
      else                                                                1m
          call srdfmute( verbosity )
          call srdfhost( host_type )
c
c             get ambient conditions
c
          if ( erdfmhd( '#initial', rdf_rec ) ) then                      2b
              call srdfinit( air_pressure, air_temp, rdf_rec )
          else                                                            2m
              write( STD_OUT, 2 )
2             format( ' Initialize module not found' /
     1            ' What is the barometric pressure ? (Pascals) ', $ )
              air_pressure = Qresp( STD_IN, 90.0e3, 110.0e3, 4 )
              write( STD_OUT, 3 )
3             format( ' What is the ambient temperature ? (Celsius) ',
     1            $ )
              air_temp = 273.15 + Qresp( STD_IN, 0.0, 40.0, 4 )
              end if                                                      2e
          air_density = air_pressure / air_temp / AIR_GAS_CONST
          air_viscosity = qmuair( air_temp ) / air_density
          write( STD_OUT, 4 ) air_pressure, air_temp, air_density,
     1        qmuair( air_temp ), air_viscosity
4         format( / 21x,'AMBIENT AIR CONDITION' / 21x, 21( '=' ) /
     1      ' pressure', 12x, g12.5 / ' temperature', 9x, g12.5 /
     2      ' density', 13x, g12.5 / ' dynamic viscosity   ', g12.5 /
     3      ' kinematic viscosity ', g12.5 / )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1        write( dmp_units(DUMP_AUXL), 4 ) air_pressure, air_temp,
     2            air_density, qmuair( air_temp ), air_viscosity
c
c           read #acquire header block - one for each profile
c
c         start of profile loop
          new_profile = .TRUE.
5         if ( erdfmhd( '#acquire', rdf_rec ) ) then                      2b
              got_data = .FALSE.
c
c           print menu for processing profile
c
c             start of menu loop                                          3b
6                 call newscr()
                  write( STD_OUT, 7 ) n_profiles, plug_no, trip_diam,
     1                Cyes( got_bar_cal ), Cyes( got_hw1_cal )
7                 format( ' PRF profile menu for profile number ', i3 /
     Z                '   0) Exit - Finished with data file' /
     1                '   1) Go to next profile' /
     2                '   2) Wind tunnel plug number  ', i3 /
     3                '   3) Boundary layer trip diam. ', f7.3, ' mm' /
     4                '   4) Baratron calibration     ', a /
     5                '   5) Hot-wire calibration     ', a /
     6                '   6) Set single point analysis parameters' /
     7            '   7) Read profile data & do single point analyses' /
     8                '   8) Do profile analysis' /
     9                '   9) Print table of profile properties ', $ )
                  item = Jresp( STD_IN, 0, 9, MAX_TRIES )
c
c           EXIT - print total CPU time, close data file
c
                  if ( item .eq. 0 ) then                                 4b
                      write( STD_OUT, 8 ) qcpclk() - CPU_clock
8                     format( ' CPU time on data file   ', f8.2,
     1                    'seconds' )
                      CPU_clock = qcpclk()
                      if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1                    write( dmp_units(DUMP_AUXL), 8 ) CPU_clock
                      call srdfcls()
                      return
c
c           GO TO NEXT PROFILE
c
                  else if ( item .eq. 1 ) then                            4m
                      new_profile = .TRUE.
                      if ( n_profiles .ge. MAX_PROFILES ) then            5b
                          if ( Equery( STD_IN,
     1                        'No more space for profiles . Clear all ?'
     2                        ) ) n_profiles = 1
                          advance_table = NO
                      else                                                5m
                          advance_table = YES
                          end if                                          5e
                  goto 5
c
c           WIND TUNNEL PLUG NUMBER
c
                  else if ( item .eq. 2 ) then                            4m
                      write( STD_OUT, 9 )
9                     format( ' Enter the wind tunnel plug number ', $ )
                      plug_no = Jresp( STD_IN, 1, 13, MAX_TRIES )
c
c           BOUNDARY LAYER TRIP DIAM
c
                  else if ( item .eq. 3 ) then                            4m
                      write( STD_OUT, 10 )
10                    format( ' Enter the boundary layer trip ',
     1                    'diameter (mm) ', $ )
                      trip_diam = Qresp( STD_IN, 1.0, 10.0, MAX_TRIES )
c
c           BARATRON CALIBRATION
c
                  else if ( item .eq. 4 ) then                            4m
                      got_bar_cal  = get_baratron()
c
c           HOT-WIRE CALIBRATION
c
                  else if ( item .eq. 5 ) then                            4m
                      got_hw1_cal = get_hot_wire()
c
c           SET SINGLE POINT ANALYSIS PARAMETERS
c
                  else if ( item .eq. 6 ) then                            4m
                      got_do = esetdo()
c
c           READ PROFILE DATA & DO SINGLE POINT ANALYSES
c
                  else if ( item .eq. 7 ) then                            4m
                      if ( plug_no .le. 0 ) then                          5b
                          write( STD_OUT, 11 ) 'plug number'
11                        format( ' PRF : ABORT : Undefined ', a,
     1                        '. It must be defined BEFORE analysis.' )
                          call ilgwait()
                      else if ( trip_diam .le. 0 ) then                   5m
                          write( STD_OUT, 11 ) 'trip diameter'
                          call ilgwait()
                      else if ( got_data ) then                           5m
                          write( STD_OUT, 12 )
12                        format( ' PRF : ABORT : Profile data has ',
     1                        'been read and single point analyses ' /
     2                        17x, 'performed. Get the next profile ',
     3                        'for more data.' )
                          call ilgwait()
                      else                                                5m
c
c           initialise flags & counters for profile
c
                          got_data = .TRUE.
                          bar_pts = 0
                          n_points = 1
                          irow = 1
                          first_hw_data = .TRUE.
                          n_profiles = n_profiles + advance_table - 1
c
c           loop for reading the data
c           read data, then decide what we have
c
13                        if ( erdfacq( Csublab, n_samples, Mdata,
     1                          MAX_SAMPLES, rdf_rec, .FALSE. ) ) then    6b
                              call sextall()
c
c           pause sub-block is ignored
c
                              if ( Csublab .eq. '###pause' ) then         7b
                                  continue
c
c           traverse sub-block -- remember required and actual positions
c
                              else if ( Csublab .eq. '###trav ' ) then    7m
                                  got_traverse = .TRUE.
                                  y_required = qextvreq( 2 )
                                  y_actual = qextvact( 2 )
c
c           secondary data sub-block
c           -- assumed to be measurement of dynamic pressure with pitot tube
c           -- remember the air speed
c
                              else if ( Csublab .eq. '###2dat2' ) then    7m
                                  call baratron_speed( Mdata )
c
c           primary data sub-block -- for hot-wire data
c
                              else if ( Csublab .eq. '###1dat1' ) then    7m
                                  if ( got_traverse ) then                8b
                                      do 14 i = irow, irow + n_segs - 1   9b
                                          result(i,ACTUAL_POS) =y_actual
                                          result(i,REQUIRED_POS) =
     1                                        y_required
14                                        continue                        9e
                                      end if                              8e
                                  call process_hot_wire( Mdata,
     1                                first_hw_data )
                                  irow = irow + n_segs
                                  n_points = n_points + 1
                              else                                        7m
                                  write( STD_ERR, 15 )
15                                format( ' PRF : ABORT : Unknown ',
     1                                'sub-block label' )
                                  n_profiles = n_profiles -
     1                                 advance_table + 1
                                  goto 6
                                  end if                                  7e
                              goto 13
                              end if
c                             end of reading data loop                    6e
                          n_profiles = n_profiles - advance_table + 1
                          n_points = n_points - 1
                          call close_dump( DUMP_HIST )
                          call close_dump( DUMP_PROB )
                          call close_dump( DUMP_AUTO )
                          call close_dump( DUMP_SPEC )
                          end if                                          5e
c
c           DO PROFILE ANALYSIS
c
                  else if ( item .eq. 8 ) then                            4m
                      n_profiles = n_profiles + advance_table - 1
                      call spfanal( new_profile )
                      n_profiles = n_profiles + 1
                      advance_table = NO
c
c           PRINT TABLE OF PROFILE PROPERTIES
c
                  else if ( item .eq. 9 ) then                            4m
                      call print_properties( STD_OUT,
     1                    n_profiles - 1, n_profiles - 1 )
                      if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1                    call print_properties( dmp_units(DUMP_AUXL),
     2                        n_profiles - 1, n_profiles - 1 )
                      end if                                              4e
                  goto 6
c                 end of menu loop                                        3e
              end if
c         end of profile loop                                             2e
c
          call srdfcls()
          end if                                                          1e
      return
      end



      SUBROUTINE PROCESS_HOT_WIRE( MDATA, FIRST_HW_DATA )
c
c Process the CTA output (hot-wire) data from the A/D converter for a single
c probe location .
c
c Arguments
c  integer *2 Mdata(*)    # raw data from A/D converter
c  logical first_hw_data  # hot-wire data is the first for the profile
c
c local variables
c  integer
c    item           # selected menu item
c    saved_seg_len  # saved segment length
c    Jresp          # function : read integer
c    get_hot_wire   #          : specify hot-wire calibration
c  logical
c    esetdo         # specify single point analysis parameters
c    eyes           # function : no/yes/query => logical
c  subroutine
c    newscr         # hpprev : clear screen
c    sascvt         # convert A/D hot-wire data to air speed
c    sspanal        # do single point analysis
c
c global variables
c   got_do, got_hw1_cal, n_points, setdo_posn, sp_rep_enab
c

      include 'stdio.def'
      include 'prf.def'
c
      logical first_hw_data, esetdo, eyes
      integer *2 Mdata(*)
      integer item, saved_seg_len, Jresp, get_hot_wire
c
      include 'prf.cmn'
c
c           first check that calibrations and analysis parameters are available
c
      if ( .not. got_hw1_cal ) then                                       1b
          write( STD_ERR, 1 )
1         format( ' PRF : WARNING : Hot-wire calibration not available')
          got_hw1_cal = get_hot_wire()
          end if                                                          1e
      if ( .not. got_do ) then                                            1b
          write( STD_ERR, 2 )
2         format( ' PRF : WARNING : Single point analysis parameters ',
     1        'not defined' )
          got_do = esetdo()
          end if                                                          1e
      if ( .not. got_hw1_cal .or. .not. got_do ) then                     1b
          write( STD_ERR, 3 )
3         format( ' PRF : ABORT : Hot-wire calibration or analysis ',
     1        'parameters still not available.' /
     2    '   Cannot process hot-wire data.' )
      else                                                                1m
c
c           set single point analysis parameters at each probe position
c
          if ( .not. first_hw_data ) then                                 2b
              if ( eyes( setdo_posn,
     1                   'Set single point analysis parameters ? ' ) )
     2            got_do = esetdo()
              end if                                                      2e
          first_hw_data = .FALSE.
c
c           convert A/D readings to air speed and analysis it
c           sascvt can change seg_len, so save it so it can be restored at end
c
          saved_seg_len = seg_len
          call sascvt( Mdata )
          call sspanal()
c
c           Repeat single point analysis (if enabled)
c
          if ( Eyes( sp_rep_enab, 'Repeat point analysis ? ' ) ) then     2b
c             start of menu loop                                          3b
c
c           PRINT MENU
c
4                 call newscr()
                  write( STD_OUT, 5 )
5                 format( ' PRF Repeat single point analysis menu' /
     1                '   0) Exit' /
     2                '   1) Repeat analysis' /
     3                '   2) Respecify parameters ', $ )
                  item = Jresp( STD_IN, 0, 2, MAX_TRIES )
c
c           EXIT
                  if ( item .eq. 0 ) then                                 4b
                      return
c
c           REPEAT ANALYSIS
                  else if ( item .eq. 1 ) then                            4m
                      call sspanal()
c
c           RESPECIFY PARAMETERS
                  else if ( item .eq. 2) then                             4m
                      got_do = esetdo()
                      end if                                              4e
                  goto 4
c                 end of menu loop                                        3e
              end if                                                      2e
          seg_len = saved_seg_len
          end if                                                          1e
      return
      end



      SUBROUTINE BARATRON_SPEED( MDATA )
c
c Calculate the free stream air speed at the specified test section port
c from the measurement of dynamic pressure just upstream of the sonic
c choke centre-body .
c
c Arguments
c   integer *2 Mdata(*)   # array of integer data from A/D converter
c
c local variables
c  logical
c    EBAR_ADtoP     # function : converts A/D readings into pressure
c  integer
c    cycle_len      # number of A/D channels used
c    get_baratron   # function : get baratron calibration
c    Jextcyln       #          : rdflib : A/D cycle length
c    Jextchls       #          : rdflib : list A/D channels
c  real
c    ts_dyn_press   # free stream dynamic pressure in the test section
c    qtts_dyn       # function : calculate dynamic pressure in test section
c    qmean          #          : calculate mean
c
c global variables
c   air_density, dmp_units, bar_pts, free_stream, got_bar_cal, n_samples,
c   plug_no, trip_diam, udata
c
c
      include 'stdio.def'
      include 'prf.def'
      logical EBAR_ADtoP
      integer cycle_len, get_baratron, Jextcyln, Jextchls
      integer *2 Mdata(*)
      real ts_dyn_press, qtts_dyn, qmean
      include 'prf.cmn'
c
c           first check that baratron calibration is available
c
      if ( .not. got_bar_cal ) then                                       1b
          write( STD_ERR, 1 )
1        format( ' PRF : WARNING : Baratron calibration not available' )
          got_bar_cal = get_baratron()
          end if                                                          1e
      if ( .not. got_bar_cal ) then                                       1b
          write( STD_ERR, 2)
2         format( ' PRF : ABORT : Baratron calibration still not ',
     1        'available.' / '   Cannot calculate free stream speed.' )
      else                                                                1m
c
c           Only use the last A/D channel if more than one is used
c
          cycle_len = Jextcyln()
          if ( cycle_len .gt. 1 )
     1        write( STD_ERR, 3 ) cycle_len, Jextchls(cycle_len)
3             format( 'PRF : WARNING : ', i3, ' baratron channels . ',
     1            'Using only the last channel : ', i3 )
c
c           Convert A/D readings into Pressure
c
          if ( EBAR_ADtoP( Mdata(cycle_len), udata(cycle_len),
     1                     cycle_len, n_samples / cycle_len ) ) then      2b
c
c           calculate air speed and print result
c
              bar_pts = bar_pts + 1
c             --old-code-using-qtsdyn
c              free_stream(bar_pts) = sqrt( 2.0 / air_density *
c     1            qtsdyn( qmean( udata(cycle_len), cycle_len,
c     2                           n_samples / cycle_len ), plug_no ) )
c             --start-of-new-code-using-qtts_dyn
              ts_dyn_press =
     1            qtts_dyn( qmean( udata(cycle_len), cycle_len,
     2                             n_samples / cycle_len ),
     3                      plug_no, trip_diam )
              if ( ts_dyn_press .gt. 0 ) then                             3b
                  free_stream(bar_pts) =
     1                sqrt( 2.0 / air_density * ts_dyn_press )
              else                                                        3m
                  call ilgwait()
                  free_stream(bar_pts) = 0.0
                  end if                                                  3e
c             --end-of-new-code-using-qtts_dyn
              write( STD_OUT, 4 ) free_stream(bar_pts)
4             format( ' mean air speed (baratron) = ', f8.3, ' m/s' )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            write( dmp_units(DUMP_AUXL), 4 ) free_stream(bar_pts)
          else                                                            2m
              write( STD_ERR, 5 )
5             format( ' PRF : ABORT : Cannot convert A/D to pressure' )
              end if                                                      2e
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION ESETDO
c
c       Determine bounds of data segments for a single traverse position,
c what single point analysis to do on this data and set all of the required
c parameters for this analysis .
c
c Local variables
c  integer
c   i               # looping index
c   item            # selected menu item
c   Jresp           # function : integer response
c   Jhw1_method     #          : select A/D to air speed conversion method
c  logical
c   Equery          # function : read logical
c  character
c   Ghw1_method*21  # function : show A/D to air speed conversion method
c   Cyes*5          #          : 'no', 'yes', 'query'
c  subroutine
c   newscr()        # clear screen
c   set_segment()   # set data segmentation parameters
c   set_wall_prox() # set wall proximity correction parameters
c   set_filter()    # set FIR filter parameters
c   set_thistory()  # set time history parameters
c   set_means()     # set parameters for means and statistical moments
c   set_prob_dens() # set parameters for probability density
c   set_autocorel() # set parameters for autocorrelation
c   set_spectrum()  # set parameters for power spectral density
c
c global variables
c   auto_do, filter_do, got_do, hist_do, hw1_method, means_do, n_segs,
c   prob_do, seg_increm, seg_len, seg_start, spec_do, wall_do
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer i, item, Jresp, Jhw1_method
      logical Equery
      character Ghw1_method*21, Cyes*5
      include 'prf.cmn'
c
c           Make sure everything is cleared if nothing is set
c
      if ( .not. got_do ) then                                            1b
          esetdo   = .FALSE.
          seg_start= 1
          seg_len  = MIN_SEG_LEN
          seg_increm = 1
          n_segs   = 1
          filter_do= NO
          wall_do  = NO
          hist_do  = NO
          means_do = NO
          prob_do  = NO
          auto_do  = NO
          spec_do  = NO
          end if                                                          1e
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) seg_start,
     1        seg_start + ( seg_len - 1 ) * seg_increm, seg_increm,
     2        n_segs, Ghw1_method(), Cyes(wall_do), Cyes(filter_do),
     3        Cyes(hist_do), Cyes(means_do), Cyes(prob_do),
     4        Cyes(auto_do), Cyes(spec_do)
2         format( ' PRF Single point analysis parameters' /
     N        '  -1) Clear all settings' /
     Z        '   0) Exit' /
     1        '   1) Data segment ',i6,' to ',i6,', step ',i6,' x', i6 /
     2        '   2) Select hot-wire calibration method ', a /
     3        '   3) Wall proximity correction     ', a /
     4        '   4) Pre-filter                    ', a /
     5        '   5) Plot air speed histories      ', a /
     6        '   6) Means (& statistical moments) ', a /
     7        '   7) Probability density           ', a /
     8        '   8) Autocorrelation               ', a /
     9        '   9) Power spectral density        ', a, '  ', $ )

          item = Jresp( STD_IN, 0, 9, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              esetdo = .TRUE.
              if ( Equery( STD_IN, ' Are you sure ? ' ) )
     1            return
c
c           DATA SEGMENT
c
          else if ( item .eq. 1 ) then                                    2m
              call set_segment()
c
c           SELECT HOT-WIRE CALIBRATION METHOD
c
          else if ( item .eq. 2 ) then                                    2m
              hw1_method = Jhw1_method( 0 )
c
c           WALL PROXIMITY CORRECTION
c
          else if ( item .eq. 3 ) then                                    2m
              call set_wall_prox()
c
c           PRE-FILTER
c
          else if ( item .eq. 4 ) then                                    2m
              call set_filter()
c
c           PLOT AIR SPEED HISTORIES
c
          else if ( item .eq. 5 ) then                                    2m
              call set_thistory()
c
c           MEANS (& STATISTICAL MOMENTS)
c
          else if ( item .eq. 6 ) then                                    2m
              call set_means()
c
c           PROBABILITY DENSITY
c
          else if ( item .eq. 7 ) then                                    2m
              call set_prob_dens()
c
c           AUTOCORRELATION
c
          else if ( item .eq. 8 ) then                                    2m
              call set_autocorel()
c
c           POWER SPECTRAL DENSITY
c
          else if ( item .eq. 9 ) then                                    2m
              call set_spectrum()
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_SEGMENT()
c
c local variables
c  logical
c   Equery      # function : read logical
c  integer
c   item        # selected menu item
c   set_samples # number of data samples -- for setup only -- not actual
c   Jresp       # function : read integer
c  subroutine
c   newscr      # clear screen
c
c global variables
c   n_samples, n_segs, seg_increm, seg_len, seg_start
c
c
      include 'stdio.def'
      include 'prf.def'
      logical Equery
      integer item, set_samples, Jresp
      include 'prf.cmn'
c
      set_samples = max( 1, min( n_samples, MAX_SAMPLES ) )
c
c         determine bounds of data segments
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) seg_start, seg_increm, seg_len, n_segs,
     1        set_samples
2         format( ' PRF Specify data segment/window' / '   0) Exit' /
     1        '   1) First data segment start index  ', i6 /
     2        '   2) Data sampling increment  ', i6 /
     3        '   3) Data window length       ', i6 /
     4        '   4) Number of windows        ', i6 /
     5        '   5) No. of data samples (set-up only) ', i6, '  ', $ )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( n_segs * seg_len * seg_increm + seg_start - 1 .le.
     1                set_samples )
     2            return
              if ( Equery( STD_IN,
     1  'Data segments exceed total record length. Are you sure ? ' ) )
     2            return
c
c           FIRST DATA SEGMENT START INDEX
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              seg_start = Jresp( STD_IN, 1, max( 1,
     1            set_samples - n_segs * seg_increm * MIN_SEG_LEN ),
     2            MAX_TRIES )
c
c           DATA SAMPLING INCREMENT
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
              seg_increm = Jresp( STD_IN, 1, max( 1,
     1            ( set_samples - seg_start + 1 ) / MIN_SEG_LEN ),
     2            MAX_TRIES )
c
c           DATA WINDOW LENGTH
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
              seg_len = Jresp( STD_IN, MIN_SEG_LEN, max( MIN_SEG_LEN,
     1            ( set_samples - seg_start + 1 ) /
     2            n_segs / seg_increm ), MAX_TRIES )
c
c           NUMBER OF WINDOWS
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              n_segs = Jresp( STD_IN, 1,
     1            max( 1, ( set_samples - seg_start + 1 ) /
     1            ( ( seg_len - 1 ) * seg_increm + 1 ) ), MAX_TRIES )
c
c           NUMBER OF SAMPLES (SETUP ONLY)
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 3 )
              set_samples = Jresp( STD_IN, 1, MAX_SAMPLES, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_FILTER()
c
c Set pre - filtering parameters . A FIR filter may be applied to the air
c speed data stream before any other analysis . The filter specification
c is contained in an ASCII file produced from MATRIXX .
c
c local variables
c  logical
c    Existf   # function : file exists ?
c  integer
c    done     # index of filter for which a file name is available
c    ifilter  # filter number
c    item     # selected menu item
c    len_file # non-blank length of file name
c    jljstr   # function : left justify string
c    Jresp    #          : read integer
c  character
c    Cyes*5   # function : 'no', 'yes', 'query'
c  subroutine
c    Cresp    # f77_ute : read string
c    newscr   # hpprev  : clear screen
c    swap4    # f77_ute : swap two 4-byte values
c
c global variables
c   filter_do, filter_file, filter_max, filter_min, filter_skip, n_filters
c
      include 'stdio.def'
      include 'prf.def'
c
      logical Existf
      integer done, ifilter, item, len_file, jljstr, Jresp
      character Cyes*5
      character Ctransient(0:2)*17
      include 'prf.cmn'
      save Ctransient
      data Ctransient / 'assume zero data ', 'assume const data',
     1                  'discard transient' /
c
      ifilter = 1
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) n_filters
2         format( ' PRF Pre-filter data' /
     H        ' FIR filtering of air speed before further analysis' /
     Z        '   0) Exit' /
     1        '   1) Number of filters ', i2 )
          if ( n_filters .gt. 0 ) then                                    2b
              len_file = max( 1, jljstr( filter_file(ifilter) ) )
              write( STD_OUT, 3 ) ifilter,
     4            filter_file(ifilter)(:len_file),
     5            Cyes( filter_desc( ifilter, FILTER_LIN ) ),
     6            Ctransient( filter_desc( ifilter, FILTER_TRN) ),
     7            Cyes( filter_desc( ifilter, FILTER_ATD ) ),
     8            Cyes( filter_desc( ifilter, FILTER_VEL ) ),
     9            filter_desc( ifilter, FILTER_MIN ),
     a            filter_desc( ifilter, FILTER_MAX )
3             format(
     2            '   2) filter number        ', i2 /
     3            '   3) advance to next filter' /
     4            '   4) filter file name      ', a /
     5            '   5) linear filter ?       ', a /
     6            '   6) treatment of transient', a /
     7            '   7) filter A/D samples    ', a /
     8            '   8) filter velocity data  ', a /
     9            '   9) usage range - min    ', i3 /
     a            '  10)             - max    ', i3 /
     b            '  11) Delete filter  ', $ )
              end if                                                      2e
          item = Jresp( STD_IN, 0, 11, MAX_TRIES )
c
c             EXIT
c         check that each filter has a file name and compress out those
c         which don't
c
          if ( item .eq. 0 ) then                                         2b
              done = 0
              do 5 ifilter = 1, n_filters
                  if ( filter_desc(ifilter,FILTER_MAX) .lt.
     1                 filter_desc(ifilter,FILTER_MIN) )                  3b
     2                call swap4( filter_desc(ifilter,FILTER_MAX),
     3                            filter_desc(ifilter,FILTER_MIN) )
                  if ( filter_file(ifilter) .ne. ' ' )
     1              done = done + 1
                  if ( ifilter .gt. done .and. done .gt. 0 ) then         3b
                      do 4 item = 1, FILTER_MAX                           4b
                          filter_desc(done,item) =
     1                        filter_desc(ifilter,item)
4                         continue                                        4e
                      filter_file(done)  = filter_file(ifilter)
                      end if                                              3e
5                 continue                                                2e
              n_filters = done
              filter_do = NO
              if ( n_filters .gt. 0 )
     1            filter_do = YES
              return
c
c             NUMBER OF FILTERS
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 6 )
6             format( ' Value ? ', $ )
              n_filters = Jresp( STD_IN, 0, MAX_FILTERS, MAX_TRIES )
          else if ( n_filters .le. 0 ) then                               2m
              goto 1
c
c             FILTER NUMBER
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 6 )
              ifilter = Jresp( STD_IN, 1, n_filters, MAX_TRIES )
c
c             NEXT FILTER
c
          else if ( item .eq. 3 ) then                                    2m
              ifilter = mod( ifilter, n_filters ) + 1
c
c             FILTER FILE NAME
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 6 )
              call Cresp( STD_IN, filter_file(ifilter), MAX_TRIES )
              if ( .not. Existf( filter_file(ifilter) ) ) then            3b
                  write( STD_ERR, 7 ) filter_file(ifilter)
7                 format( ' PRF : WARNING : Cannot find file ', a )
                  filter_file(ifilter) = ' '
                  call ilgwait()
                  end if                                                  3e
c
c             LINEAR FILTER ?
c
          else if ( item .eq. 5 ) then                                    2m
              filter_desc(ifilter,FILTER_LIN) =
     1            mod( filter_desc(ifilter,FILTER_LIN) + 1, 3 )
c
c             TREATMENT OF TRANSIENT (zero,constant,discard)
c
          else if ( item .eq. 6 ) then                                    2m
              filter_desc(ifilter,FILTER_TRN) =
     1            mod( filter_desc(ifilter,FILTER_TRN) + 1, 3 )
c
c             FILTER A/D SAMPLES
c
          else if ( item .eq. 7 ) then                                    2m
              filter_desc(ifilter,FILTER_ATD) =
     1            mod( filter_desc(ifilter,FILTER_ATD) + 1, 2 )
c
c             FILTER VELOCITY DATA
c
          else if ( item .eq. 8 ) then                                    2m
              filter_desc(ifilter,FILTER_VEL) =
     1            mod( filter_desc(ifilter,FILTER_VEL) + 1, 2 )
c
c             USAGE RANGE IN PROFILE - MIN
c
          else if ( item .eq. 9 ) then                                    2m
              write( STD_OUT, 6 )
              filter_desc(ifilter,FILTER_MIN) =
     1            Jresp( STD_IN, 1, MAX_PROFILES, MAX_TRIES )
c
c             USAGE RANGE IN PROFILE - MAX
c
          else if ( item .eq. 10 ) then                                   2m
              write( STD_OUT, 6 )
              filter_desc(ifilter,FILTER_MAX) =
     1            Jresp( STD_IN, 1, MAX_PROFILES, MAX_TRIES )
c
c             DELETE FILTER
c
          else if ( item .eq. 11 ) then                                   2m
              n_filters = max( 0, n_filters - 1 )
              if ( ifilter .le. n_filters ) then                          3b
                  do 9 done = ifilter + 1, n_filters + 1                  4b
                      filter_file(done-1) = filter_file(done)
                      do 8 item = 1, FILTER_MAX                           5b
                          filter_desc(done-1,item) =
     1                        filter_desc(done,item)
8                         continue                                        5e
9                     continue                                            4e
              else                                                        3m
                  ifilter = max( 1, n_filters )
                  end if                                                  3e
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_WALL_PROX()
c
c Specify parameters for wall proximity correction .
c
c local variables
c  integer
c    item       # selected menu item
c    len_file   # length of file name
c    jljstr     # function : left justify string
c    Jresp      #          : read integer
c  logical
c    Existf     # function : file exists ?
c  character
c    Cmethod(2)*14  # available methods
c  subroutine
c    sbhatia    # blylib : to force initialization when used
c    Cresp      # read string
c    swap4      # swap 2 values
c
c global variables
c   wall_do, Wall_file, wall_max, wall_method, wall_min
c
c
      include 'stdio.def'
      include 'prf.def'
      integer *2 AD_dummy
      integer item, len_file, jljstr, Jresp
      logical debug_nuss, Existf
      real    vel_dummy, QHW1_E2toU, QHW1_UtoE2
      character Cmethod(2)*14, Cyes*5
      include 'prf.cmn'
c
      external QHW1_E2toU, QHW1_UtoE2
      save len_file
      data len_file / 1 /
      data Cmethod / 'Velocity error', 'Heat flow' /
c
      debug_nuss = NO
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) Wall_file(:len_file),
     1        Cmethod(wall_method), wall_min, wall_max,
     2        Cyes( debug_nuss )
2         format( ' PRF Wall proximity correction parameters' /
     Z        '   0) exit' /
     1        '   1) Correction law file name ', a /
     2        '   2) Correction method        ', a  /
     3        '   3) Useage range in profile : min ', i3 /
     4        '   4)                         : max ', i3 /
     5        '   5) Switch debugging on/off  ', a, ' ', $ )
          item = Jresp( STD_IN, 0, 5, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              wall_do = NO
              if ( Wall_file .ne. ' ' ) then                              3b
                  wall_do = YES
                  call sbhatia( vel_dummy, 1, 0, 0.0, 0.0, ' ' )
                  call prox_nusselt( AD_dummy, vel_dummy, 1, 0,
     1                     0.0, 0.0, QHW1_E2toU, QHW1_UtoE2, ' ' )
                  if ( debug_nuss .eq. YES ) then                         4b
                      call prox_nusselt( AD_dummy, vel_dummy, 1, -1,
     1                         0.0, 0.0, QHW1_E2toU, QHW1_UtoE2, ' ' )
                      end if                                              4e
                  end if                                                  3e
              if ( wall_min .gt. wall_max )
     1            call swap4( wall_min, wall_max )
              return
c
c           CORRECTION LAW FILE NAME
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' What value ? ', $ )
              call Cresp( STD_IN, Wall_file, MAX_TRIES )
              len_file = max( 1, jljstr( Wall_file ) )
              if ( .not. Existf( Wall_file ) ) then                       3b
                  write( STD_ERR, 4 ) Wall_file(:len_file)
4                 format( ' PRF : WARNING : Cannot find file ', a )
                  call ilgwait()
                  Wall_file = ' '
                  end if
c
c           CORRECTION METHOD
c
          else if ( item .eq. 2 ) then                                    2m
              wall_method = mod( wall_method, 2 ) + 1
c
c           USEAGE RANGE IN PROFILE : MIN
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
              wall_min = Jresp( STD_IN, 1, MAX_POINTS, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MAX
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              wall_max = Jresp( STD_IN, 1, MAX_POINTS, MAX_TRIES )
c
c           SWITCH DEBUGGING ON/OFF
c
          else if ( item .eq. 5 ) then                                    2m
              debug_nuss = mod( debug_nuss + 1, 2 )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_THISTORY()
c
c Set parameters for air speed time histories .
c
c Local variables
c  integer
c   capt_len    # length of capture file name
c   dump_len    # length of dump file name
c   item        # selected menu item
c   jljstr      # function : left justify string
c   Jresp       #          : read integer
c  character
c   Cyes*5      # function : 'no' / 'yes' / 'query'
c  subroutine
c   newscr      # hpprev  : clear screen
c   Cresp       # f77_ute : read string
c   swap4       # f77_ute : swap 2 values
c
c global variables
c   dmp_names, hist_do, hist_plot, hist_wait, hist_capt, hist_min, hist_max
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer capt_len, dump_len, item, jljstr, Jresp
      character Cyes*5
c
      include 'prf.cmn'
      save capt_len, dump_len
      data capt_len, dump_len / 1, 1 /
c
      dump_len = max( 1, jljstr( dmp_names(DUMP_HIST) ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes( hist_do ),
     1         dmp_names(DUMP_HIST)(:dump_len), Cyes( hist_plot ),
     2         Cyes( hist_wait ), hist_capt(:capt_len),
     3         hist_min, hist_max
2         format( ' PRF Set time series air speed history parameters' /
     Z        '   0) Exit' /
     1        '   1) Do histories ', a /
     2        '   2) Dump file    ', a /
     3        '   3) Plot         ', a /
     4        '   4) Wait after plotting ', a /
     5        '   5) Capture file ', a /
     6        '   6) Useage range in profile : min ', i3 /
     7        '   7)                         : max ', i3, '  ', $ )
          item = Jresp( STD_IN, 0, 7, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( hist_min .gt. hist_max )
     1            call swap4( hist_min, hist_max )
              return
c
c           DO HISTORIES
c
          else if ( item .eq. 1 ) then                                    2m
              hist_do = mod( hist_do + 1, 3 )
c
c           DUMP FILE
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              call Cresp( STD_IN, dmp_names(DUMP_HIST), MAX_TRIES )
              dump_len = max( 1, jljstr( dmp_names(DUMP_HIST) ) )
c
c           PLOT
c
          else if ( item .eq. 3 ) then                                    2m
              hist_plot = mod( hist_plot + 1, 3 )
c
c           WAIT AFTER PLOTTING
c
          else if ( item .eq. 4 ) then                                    2m
              hist_wait = mod( hist_wait + 1, 2 )
c
c           CAPTURE FILE
c
          else if ( item .eq. 5 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, hist_capt, MAX_TRIES )
              capt_len = max( 1, jljstr( hist_capt ) )
c
c           USEAGE RANGE IN PROFILE : MIN
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 3 )
              hist_min = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MAX
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 3 )
              hist_max = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_MEANS()
c
c set parameters for means and statistical moments
c
c local variables
c  integer
c   item        # selected menu item
c   Jresp       # function : read integer
c   jljstr      #          : left justify string
c  character
c   Cyes*5      # function : 'no', 'yes', 'query'
c  subroutine
c   newscr      # hpprev  : clear screen
c   Cresp       # f77_ute : read string
c   swap4       # swap 2 values
c
c global variables
c   means_do, means_moment, means_min, means_max
c
      include 'stdio.def'
      include 'prf.def'
c
      integer item, Jresp, jljstr
      character Cyes*5
c
      include 'prf.cmn'
c
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes( means_do ), means_moment,
     1        means_min, means_max
2         format( ' PRF Set mean and statistical moments parameters' /
     Z        '   0) Exit' /
     1        '   1) Do means & moments   ', a /
     2        '   2) Maximum moment order ', i3 /
     3        '   3) Useage range in profile : min ', i3 /
     4        '   4)                         : max ', i3, '  ', $ )
          item = Jresp( STD_IN, 0, 4, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( means_min .gt. means_max )
     1            call swap4( means_min, means_max )
              return
c
c           DO MEANS & MOMENTS
c
          else if ( item .eq. 1 ) then                                    2m
              means_do = mod( means_do + 1, 3 )
c
c           MAXIMUM MOMENT ORDER
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              means_moment = Jresp( STD_IN, 1, MAX_MOMENT, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MIN
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
              means_min = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MAX
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              means_max = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_PROB_DENS()
c
c set parameters for probability density
c
c local variables
c  integer
c   capt_len    # length of capture file name
c   default_bins# default number of PDF bins
c   dump_len    # length of dump file name
c   item        # selected menu item
c   Jresp       # function : read integer
c   jljstr      #          : left justify string
c  character
c   Cyes        # function : 'no' / 'yes' / 'query'
c  subroutine
c   newscr      # clear screen
c   Cresp       # read string
c   swap4       # swap 2 values
c
c global variables
c   dmp_names, prob_bins, prob_capt, prob_do, prob_max, prob_min,
c   prob_moment, prob_plot, prob_test, prob_wait
c
      include 'stdio.def'
      include 'prf.def'
c
      integer capt_len, default_bins, dump_len, item, Jresp, jljstr
      character Cyes*5
      include 'prf.cmn'
      save capt_len, dump_len
      data capt_len, dump_len / 1, 1 /
c
c
      default_bins = int( sqrt( real( seg_len ) ) )
      if ( prob_bins .le. 0 )
     1    prob_bins = default_bins
      dump_len = max( 1, jljstr( dmp_names(DUMP_PROB) ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes( prob_do ), default_bins,
     1        prob_bins, dmp_names(DUMP_PROB)(:dump_len),
     2        Cyes( prob_plot ), Cyes(prob_wait), prob_capt(:capt_len),
     3        prob_min, prob_max, Cyes(prob_test), prob_moment
2         format( ' PRF Set probability density parameters' /
     Z        '   0) Exit' /
     1        '   1) Do probability density ', a /
     2        '   2) Default number of PDF values ', i5 /
     3        '   3) Number of PDF values ', i5 /
     4        '   4) Dump file            ', a /
     5        '   5) Plot                 ', a /
     6        '   6) Wait after plotting  ', a /
     7        '   7) Capture file         ', a /
     8        '   8) Useage range in profile : min ', i3 /
     9        '   9)                         : max ', i3, /
     a        '  10) Test convergence of moments   ', a /
     b        '  11) Maximum moment integrand      ', i2, '  ', $ )
          item = Jresp( STD_IN, 0, 11, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( prob_min .gt. prob_max )
     1            call swap4( prob_min, prob_max )
              return
c
c           DO PROBABILITY DENSITY
c
          else if ( item .eq. 1 ) then                                    2m
              prob_do = mod( prob_do + 1, 3 )
c
c           DEFAULT NUMBER OF PDF VALUES
c
          else if ( item .eq. 2 ) then                                    2m
              prob_bins = default_bins
c
c           NUMBER OF PDF VALUES
c
          else if ( item .eq. 3 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              prob_bins = Jresp( STD_IN, 2,
     1            min( MAX_SAMPLES, seg_len ), MAX_TRIES )
c
c           DUMP FILE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, dmp_names(DUMP_PROB), MAX_TRIES )
              dump_len = max( 1, jljstr( dmp_names(DUMP_PROB) ) )
c
c           PLOT
c
          else if ( item .eq. 5 ) then                                    2m
              prob_plot = mod( prob_plot + 1, 3 )
c
c           WAIT AFTER PLOTTING
c
          else if ( item .eq. 6 ) then                                    2m
              prob_wait = mod( prob_wait + 1, 2 )
c
c           CAPTURE FILE
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, prob_capt, MAX_TRIES )
              capt_len = max( 1, jljstr( prob_capt ) )
c
c           USEAGE RANGE IN PROFILE : MIN
c
          else if ( item .eq. 8 ) then                                    2m
              write( STD_OUT, 3 )
              prob_min = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MAX
c
          else if ( item .eq. 9 ) then                                    2m
              write( STD_OUT, 3 )
              prob_max = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
c
c           TEST CONVERGENCE OF MOMENTS
c
          else if ( item .eq. 10 ) then                                    2m
              prob_test = mod( prob_test + 1, 3 )
c
c           MAXIMUM MOMENT INTEGRAND
c
          else if ( item .eq. 11 ) then                                   2m
              write( STD_OUT, 3 )
              prob_moment = Jresp( STD_IN, 1, MAX_MOMENT, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



ccccccccccccccccccccccccccc
c
c intermittency parameters
c   discrimination level
c   selection time
c   dump file
c   plot
c   capture file
c
ccccccccccccccccccccccccccc



      SUBROUTINE SET_AUTOCOREL()
c
c set parameters for autocorrelation
c
c local variables
c  integer
c   capt_len    # length of capture file name
c   dump_len    # length of dump file name
c   item        # selected menu item
c   jljstr      # function : left justify string
c   Jresp       #          : read integer
c  character
c   Cmethod(2)*4# 'time' / 'FFT '  method of calculating autocorrelation
c   Cyes*5      # function : 'no' / 'yes' / 'query'
c  subroutine
c   newscr      # hpprev  : clear screen
c   Cresp       # f77_ute : read string
c   swap4       # f77_ute : swap 2 values
c
c global variables
c   dmp_names, auto_capt, auto_do, auto_lags, auto_max, auto_min,
c   auto_method, auto_plot, auto_wait, seg_len
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer capt_len, dump_len, item, jljstr, Jresp
      character Cmethod(2)*4, Cyes*5
      include 'prf.cmn'
      save capt_len, dump_len, Cmethod
      data capt_len, dump_len / 1, 1 /
      data Cmethod / 'time', 'FFT ' /
c
c
      dump_len = max( 1, jljstr( dmp_names(DUMP_AUTO) ) )
c     start of menu loop                                                  1b
1         call newscr()
          write( STD_OUT, 2 ) Cyes( auto_do ), auto_lags,
     1        Cmethod(auto_method), dmp_names(DUMP_AUTO)(:dump_len),
     2        Cyes( auto_plot ), Cyes(auto_wait), auto_capt(:capt_len),
     3        auto_min, auto_max
2         format( ' PRF Set autocorrelation parameters' /
     Z        '   0) Exit' /
     1        '   1) Do autocorrelation ', a /
     2        '   2) Number of lags     ', i5 /
     3        '   3) method             ', a /
     4        '   4) Dump file          ', a /
     5        '   5) Plot               ', a /
     6        '   6) Wait after plot    ', a /
     7        '   7) Capture file       ', a /
     8        '   8) Useage range in profile : min ', i3 /
     9        '   9)                         : max ', i3, '  ', $ )
          item = Jresp( STD_IN, 0, 9, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( auto_min .gt. auto_max )
     1            call swap4( auto_min, auto_max )
              return
c
c           DO AUTOCORRELATION
c
          else if ( item .eq. 1 ) then                                    2m
              auto_do = mod( auto_do + 1, 3 )
c
c           NUMBER OF LAGS
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              auto_lags = Jresp( STD_IN, 1,
     1            min( MAX_SAMPLES, seg_len ), MAX_TRIES )
c
c           METHOD
c
          else if ( item .eq. 3 ) then                                   2m
              auto_method = mod( auto_method, 2 ) + 1
c
c           DUMP FILE
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, dmp_names(DUMP_AUTO), MAX_TRIES )
              dump_len = max( 1, jljstr( dmp_names(DUMP_AUTO) ) )
c
c           PLOT
c
          else if ( item .eq. 5 ) then                                    2m
              auto_plot = mod( auto_plot + 1, 3 )
c
c           WAIT AFTER PLOTTING
c
          else if ( item .eq. 6 ) then                                    2m
              auto_wait = mod( auto_wait + 1, 2 )
c
c           CAPTURE FILE
c
          else if ( item .eq. 7 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, auto_capt, MAX_TRIES )
              capt_len = max( 1, jljstr( auto_capt ) )
c
c           USEAGE RANGE IN PROFILE : MIN
c
          else if ( item .eq. 8 ) then                                    2m
              write( STD_OUT, 3 )
              auto_min = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MAX
c
          else if ( item .eq. 9  ) then                                    2m
              write( STD_OUT, 3 )
              auto_max = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SET_SPECTRUM()
c
c set parameters for power spectral density
c
c Local variables
c  integer
c    capt_len         # length of capture file name
c    dump_len         # length of dump file name
c    half_freq        # spectrum with half integer frequencies
c    hanning_wind     # hanning time window on time data
c    item             # selected menu item
c    max_len_fft      # maximum allowed length of FFT - depends on half_freq
c    tukey_wind       # Tukey smoothing of spectrum
c    x_scale          # x scaling : 1=linear, 2=log
c    y_scale          # y scaling : 1=linear, 2=log
c    jljstr           # function : left justify string
c    Jresp            #          : integer response
c  character
c    Cscale(2)*3      # 'lin', 'log' for scaling laws
c    Cyes*5           # function : 'yes' or 'no'
c  subroutine
c    Cresp            # f77_ute : read string
c    newscr           # hpprev  : clear screen
c    swap4            # f77_ute : swap 2 values
c
c global variables
c  dmp_names, spec_do, spec_capt, spec_max, spec_min, spec_nfft, spec_plot,
c  spec_wait, spec_window, spec_xlaw, spec_ylaw
c
c
      include 'stdio.def'
      include 'prf.def'
      integer capt_len, dump_len, half_freq, hanning_wind,
     1    item, max_len_fft, tukey_wind, x_scale, y_scale,
     2    jljstr, Jresp
      character Cscale(2)*3, Cyes*5
      include 'prf.cmn'
c
      data Cscale / 'lin', 'log' /
      data dump_len, capt_len, x_scale, y_scale / 1, 1, 1, 1 /
      data half_freq, tukey_wind, hanning_wind /  NO, NO, YES /
      save dump_len, capt_len, x_scale, y_scale,
     1     half_freq, tukey_wind, hanning_wind, Cscale
c
c
      dump_len = max( 1, jljstr( dmp_names(DUMP_SPEC) ) )
c     start of loop                                                       1b
1         if ( half_freq ) then                                           2b
              max_len_fft = MAX_FFT - 1
          else                                                            2m
              max_len_fft = MAX_FFT
              end if                                                      2e
          spec_nfft = min( spec_nfft, max_len_fft )
          call newscr()
          write( STD_OUT, 2 ) Cyes(spec_do), 2**spec_nfft,
     1        Cyes(Hanning_wind), Cyes(Tukey_wind), Cyes(half_freq),
     2        dmp_names(DUMP_SPEC)(:dump_len), Cyes(spec_plot),
     3        Cyes(spec_wait), spec_capt(:capt_len), Cscale(x_scale),
     4        Cscale(y_scale), spec_min, spec_max
2         format( / ' Select spectral parameter' / '  0) exit' /
     1        '  1) Do power spectrum        ', a /
     2        '  2) length of FFT          ', i6 /
     3        '  3) Hanning (time) window    ', a /
     4        '  4) Tukey (smoothing) window ', a /
     5        '  5) Half integer frequencies ', a /
     6        '  6) Dump file       ', a /
     7        '  7) Plot            ', a /
     8        '  8) Wait after plot ', a /
     9        '  9) Capture file    ', a /
     a        ' 10) X-axis scaling  ', a /
     b        ' 11) Y-axis scaling  ', a /
     c        ' 12) Useage range in profile : min ', i3 /
     d        ' 13)                         : max ', i3, '  ', $ )
          item = Jresp( STD_IN, 0, 13, MAX_TRIES )
c
c           EXIT
c           close/open dump file as necessary
c
          if ( item .eq. 0 ) then                                         2b
              if ( spec_min .gt. spec_max )
     1            call swap4( spec_min, spec_max )
c
c           set the window types
              item = 1
              spec_window = '   '
              if ( Hanning_wind .eq. YES ) then                           3b
                  spec_window(item:item) = 'h'
                  item = item + 1
                  end if                                                  3e
              if ( Tukey_wind .eq. YES ) then                             3b
                  spec_window(item:item) = 'a'
                  item = item + 1
                  end if                                                  3e
              if ( half_freq .eq. YES ) then                              3b
                  spec_window(item:item) = 'z'
                  end if                                                  3e
c
c           x & y scaling laws
c
              spec_xlaw = Cscale(x_scale)
              spec_ylaw = Cscale(y_scale)
              return
c
c           DO SPECTRUM
c
          else if ( item .eq. 1 ) then                                    2m
              spec_do = mod( spec_do + 1, 3 )
c
c           LENGTH OF FFT
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Value ? ', $ )
              spec_nfft = Jresp( STD_IN, 16, 2**max_len_fft, MAX_TRIES )
c                      ensure spec_nfft is a power of 2
              spec_nfft =
     1            int( log( real( spec_nfft ) ) / log( 2.0 ) + 0.00001 )
c
c           HANNING (TIME) WINDOW
c
          else if ( item .eq. 3 ) then                                    2m
              hanning_wind = mod( hanning_wind + 1, 2 )
c
c           TUKEY (SMOOTHING) WINDOW
c
          else if ( item .eq. 4 ) then                                    2m
              tukey_wind = mod( tukey_wind + 1, 2 )
c
c           HALF INTEGER FREQUENCIES
c
          else if ( item .eq. 5 ) then                                    2m
              half_freq = mod( half_freq + 1, 2 )
c
c           DUMP FILE
c
          else if ( item .eq. 6 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, dmp_names(DUMP_SPEC), MAX_TRIES )
              dump_len = max( 1, jljstr( dmp_names(DUMP_SPEC) ) )
c
c           PLOT SPECTRUM
c
          else if ( item .eq. 7 ) then                                    2m
              spec_plot = mod( spec_plot + 1, 3 )
c
c           WAIT AFTER PLOTTING
c
          else if ( item .eq. 8 ) then                                    2m
              spec_wait = mod( spec_wait + 1, 2 )
c
c           NAME OF CAPTURE FILE
c
          else if ( item .eq. 9 ) then                                    2m
              write( STD_OUT, 3 )
              call Cresp( STD_IN, spec_capt, MAX_TRIES )
              capt_len = max( 1, jljstr( spec_capt ) )
c
c           X-AXIS SCALING
c
          else if ( item .eq. 10 ) then                                   2m
              x_scale = mod( x_scale, 2 ) + 1
c
c           Y-AXIS SCALING
c
          else if ( item .eq. 11 ) then                                   2m
              y_scale = mod( y_scale, 2 ) + 1
c
c           USEAGE RANGE IN PROFILE : MIN
c
          else if ( item .eq. 12 ) then                                   2m
              write( STD_OUT, 3 )
              spec_min = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
c
c           USEAGE RANGE IN PROFILE : MAX
c
          else if ( item .eq. 13 ) then                                   2m
              write( STD_OUT, 3 )
              spec_max = Jresp( STD_IN, 0, MAX_POINTS, MAX_TRIES )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      SUBROUTINE SP_CALIB_LIMIT()
c
c Test if calibration limits are exceeded .
c
c local variables
c  integer
c   actual_len  # actual data segment length after checking availability of data
c   exceed_max  # number of data points above calibration maximum
c   exceed_min  # number of data points below calibration minimum
c   iseg        # segment number
c   seg_begin   # index for start of segment iseg
c   check_seg_len # function : check available data
c   Jhw1_upper  #          : count data points above calibration maximum
c   Jhw1_lower  #          : count data points below calibration minimum
c
c global variables
c   dmp_units, n_segs, seg_increm, seg_len, seg_start, Udata
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer actual_len, exceed_max, exceed_min, iseg, seg_begin,
     1    unused, check_seg_len, Jhw1_limit, Jhw1_upper, Jhw1_lower
      include 'prf.cmn'
c
c
      seg_begin = seg_start
      do 3 iseg = 1, n_segs                                                1b
          actual_len = check_seg_len( iseg, seg_begin,
     1        'SP_CALIB_LIMIT' )
          if ( actual_len .le. 0 )
     1        return
          exceed_max = Jhw1_upper( Udata(seg_begin), seg_increm,
     1        actual_len )
          if ( exceed_max .gt. 0 ) then                                   2b
              write( STD_OUT, 1 ) exceed_max, iseg
1             format( ' PRF: WARNING: Calibration maximum exceeded ',
     1            'by ', i6, ' data points in segment ', i3 )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            write( dmp_units(DUMP_AUXL), 1 ) exceed_max, iseg
              end if                                                      2e
          exceed_min = Jhw1_lower( Udata(seg_begin), seg_increm,
     1        actual_len )
          if ( exceed_min .gt. 0 ) then                                   2b
              write( STD_OUT, 2 ) exceed_min, iseg
2             format( ' PRF: WARNING: ', i6, ' data points in ',
     1            'segment ', i3, ' below calibration minimum' )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            write( dmp_units(DUMP_AUXL), 1 ) exceed_min, iseg
              end if                                                      2e
          seg_begin = seg_begin + seg_len * seg_increm
3         continue                                                        1e
      unused = jhw1_limit( STD_OUT, 0.0 )
      unused = jhw1_limit( -1, 0.0 )
      return
      end



      SUBROUTINE SP_FILTER( REAL_DATA, MDATA )
c
c Filter the data stream . This routine changes the air speed data and also
c may change the segment length 'seg_len' .
c
c Argument
c  logical real_data    # .TRUE.  filter data array UDATA : velocity signal
c                       # .FALSE. filter data array ????? : A/D signal
c  integer *2 mdata(*)  # A/D signal
c
c local variables
c  logical
c   ad_filter   # filter A/D signal
c   vel_filter  # filter velocity signal
c   Efir_read   # function : read FIR filter spec file
c  integer
c   ifilter     # index into filter specifications
c   n_coeffs    # number of FIR filter coefficients
c   stream_len  # data stream length for filtering
c   Ifir_filt   # function # apply FIR filter, integer *2 data
c   Jfir_filt   # function # apply FIR filter, real data
c  double precision
c   filter_coeffs(MAX_FIRLEN) # coefficients of the FIR filter
c
c global variables
c   dmp_units, filter_desc, n_filters, n_points, n_samples, n_segs,
c   seg_increm, seg_len, seg_start, udata, filter_file
c
      include 'stdio.def'
      include 'prf.def'
c
      logical real_data, ad_filter, vel_filter, Efir_read
      integer ifilter,  n_coeffs, stream_len, Ifir_filt, Jfir_filt
      integer *2 Mdata(*)
      double precision filter_coeffs(MAX_FIRLEN)
c
      include 'prf.cmn'
c
c
c           search through the list of filters until one is found which
c           has useage limits straddling the `n_points' value .
c
      do 4 ifilter = 1, n_filters                                         1b
          vel_filter = filter_desc(ifilter,FILTER_VEL) .eq. YES .and.
     1                 real_data
          ad_filter  = filter_desc(ifilter,FILTER_ATD) .eq. YES .and.
     2                 .not. real_data
          if ( ( vel_filter .or. ad_filter ) .and.
     1         filter_desc(ifilter,FILTER_MIN) .le. n_points .and.
     2         filter_desc(ifilter,FILTER_MAX) .ge. n_points ) then       2b
              write( STD_OUT, 1 )
1             format( ' PRF : FIR filtering' )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            write( dmp_units(DUMP_AUXL), 1 )
c
c           read the filter specification from file
c
              if ( Efir_read( filter_file(ifilter), MAX_FIRLEN,
     1                        filter_coeffs, n_coeffs ) ) then            3b
c
c           now actually do the filtering
c           and then return (filter the signal only once)
c
                  stream_len = min( seg_len * n_segs,
     1                              n_samples / seg_increm + 1 )
                  if ( vel_filter ) then                                  4b
                      stream_len = Jfir_filt( udata(seg_start),
     1                    udata(seg_start), seg_increm, stream_len,
     2                    filter_coeffs, n_coeffs,
     3                    filter_desc(ifilter,FILTER_LIN) +
     4                        filter_desc(ifilter,FILTER_TRN) * 4 )
                  else if ( ad_filter ) then                              4m
                      stream_len = Ifir_filt( Mdata(seg_start),
     1                    Mdata(seg_start), seg_increm, stream_len,
     2                    filter_coeffs, n_coeffs,
     3                    filter_desc(ifilter,FILTER_LIN) +
     4                        filter_desc(ifilter,FILTER_TRN) * 4 )
                      end if                                              4e
                  if ( stream_len .gt. 0 ) then                           4b
                      seg_len = stream_len / n_segs
                  else                                                    4m
                      write( STD_ERR, 2 )
2                     format( ' PRF : ABORT : Filter failed' )
                      if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1                    write( dmp_units(DUMP_AUXL), 2 )
                      end if                                              4e
              else                                                        3m
                  write( STD_ERR, 3 ) filter_file(ifilter)
3                 format( ' PRF : ABORT : Cannot read filter spec ',
     1                'file ', a )
                  if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1                write( dmp_units(DUMP_AUXL), 2 )
     2                    filter_file(ifilter)
                  end if                                                  3e
              return
              end if                                                      2e
4         continue                                                        1e
      return
      end




      SUBROUTINE SP_WALL_PROX()
c
c Velocity error method of correcting for wall proximity effect
c
c global variables
c   dmp_units, air_viscosity, irow, n_points, n_segs, result, seg_increm,
c   seg_len, seg_start, udata, wall_file, wall_max, wall_min
c
c
      include 'stdio.def'
      include 'prf.def'
      include 'prf.cmn'
c
      if ( wall_min .le. n_points .and. n_points .le. wall_max ) then     1b
          write( STD_OUT, 1 )
1         format( ' Applying velocity error wall proximity correction' )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1        write( dmp_units(DUMP_AUXL), 1 )
          call sbhatia( udata(seg_start), seg_increm,
     1        min( seg_len * n_segs, n_samples / seg_increm + 1 ),
     1        result(irow,ACTUAL_POS) / 1000.0 , air_viscosity,
     2        wall_file )
          end if                                                          1e
      return
      end



      SUBROUTINE SP_TIME_HISTORY( TIME )
c
c Display time series air speed data
c
c Argument
c   integer time(*)   # used to contain time-abscissa values
c
c local variables
c  logical
c   Eyes        # function : n/y/q to logical
c  integer
c   actual_len  # actual data segment length after checking availability of data
c   i           # looping index
c   ilast       # index of last sample in segment
c   iseg        # segment number
c   nyplts      # number of segments plotted on the screen
c   seg_begin   # stard index for segment iseg
c   check_seg_len # function : check available data
c  character
c   Clabel*42   # title or x axis label
c  real
c   cycle_time  # A/D cycle time
c   Qextcytm    # function : rdflib : A/D cycle time
c  subroutine
c   ilgclos, ilgdata, ilgopen, ilgpage, ilgtitl, ilgxlab, ilgylab, ilgwait
c   open_dump   # open dump file
c
c global variables
c   dmp_names, dmp_units, hist_capt, hist_max, hist_min, hist_plot,
c   hist_wait, irow, n_points, n_profiles, n_segs, result,
c   seg_increm, seg_len, seg_start, udata,
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical Eyes
      integer actual_len, i, ilast, iseg, nyplts, seg_begin,
     1    check_seg_len
      character Clabel*42
      real cycle_time, time(*), Qextcytm
      include 'prf.cmn'
c
c
      if ( hist_min .le. n_points .and. n_points .le. hist_max ) then     1b
          cycle_time = Qextcytm()
c
c             plot
c
          if ( Eyes( hist_plot, 'Plot time series data ? ' ) ) then       2b
              write( STD_OUT, 1 )
1             format( ' PRF : Display time series data' )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            write( dmp_units(DUMP_AUXL), 1 )
              call ilgopen( hist_capt )
c
              nyplts = min( MAX_BOXES, max( 2, n_segs ) )
              call ilgpage( 'a4', 'wide', 1, nyplts )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgslaw( 'lin', 'lin' )
c
              write( Clabel, 2 ) result(irow,ACTUAL_POS), n_profiles
2             format( 'Air speed at y = ', f6.2, ' mm for profile ', i2)
              call ilgtitl( Clabel )
c
              seg_begin = seg_start - 1
              do 5 iseg = 1, n_segs                                       3b
                  actual_len = check_seg_len( iseg, seg_begin + 1,
     1                'SP_TIME_HISTORY' )
                  if ( actual_len .le. 0 )
     1                goto 6
                  do 3 i = 1, ( actual_len - 1 ) * seg_increm + 1,        4b
     1                    seg_increm
                      time(i) = ( seg_begin + i ) * cycle_time
3                     continue                                            4e
                  call ilgdata( time, udata(seg_begin+1), seg_increm,
     1                actual_len, 1 )
                  seg_begin = seg_begin + seg_len * seg_increm
c
                  write( Clabel, 4 ) iseg
4                 format( 'Segment', i4, ' : time (seconds)' )
                  call ilgxlab( Clabel )
                  call ilgylab( 'm/s' )
                  if ( hist_wait .eq. YES ) then                          4b
                      if ( iseg .eq. n_segs .or.
     1                        mod( iseg, MAX_BOXES ) .eq. 0 )
     2                    call ilgwait()
                      end if                                              4e
5                 continue                                                3e
6             call ilgclos
              end if                                                      2e
c
c           tabulate/dump
c
          if ( dmp_names(DUMP_HIST) .ne. ' ' ) then                       2b
              if ( dmp_units(DUMP_HIST) .eq. 0 )
     1            call open_dump( DUMP_HIST )
              if ( dmp_units(DUMP_HIST) .ne. 0 ) then                     3b
                  seg_begin = seg_start
                  do 8 iseg = 1, n_segs                                   4b
                      actual_len = check_seg_len( iseg, seg_begin,
     1                     'SP_TIME_HISTORY' )
                      if ( actual_len .le. 0 )
     1                    goto 9
                      ilast = ( actual_len - 1 ) * seg_increm + 1
                      call dump_header( dmp_units(DUMP_HIST), iseg,
     1                  'Air speed time series data' )
                      write( dmp_units(DUMP_HIST), 7 )
     1                    ( i * cycle_time, udata(i),
     2                      i = seg_begin, ( actual_len - 1 ) *
     3                      seg_increm + seg_begin, seg_increm )
7                     format( ' ;   time     air speed' /
     7                    ' ;  seconds     m/s' /
     8                    / ( 1x, g12.5, g13.5 ) )
8                     continue                                            4e
9                 end if                                                  3e
              end if                                                      2e
          end if                                                          1e
      return
      end



      SUBROUTINE SP_MEANS()
c
c Means and statistical moments of the data stream
c
c Local variables
c  integer
c   actual_len  # actual data segment length after checking availability of data
c   i           # looping index for moments
c   iseg        # segment number
c   seg_begin   # start index of segment 'iseg'
c   total_seg   # total number of segments processed
c   check_seg_len # function : check available data
c  real
c   denom       # rms**i denominator for calculating skewness etc
c   stats(MAX_MOMENT)  # statistical moments
c  subroutine
c   smeang      # f77_num : calculate statistical moments
c
c global variables
c   dmp_units, irow, means_max, means_min, means_moment, n_points, n_segs,
c   result, seg_increm, seg_len, seg_start, udata
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer actual_len, i, iseg, seg_begin, total_seg, check_seg_len
      real denom, stats(MAX_MOMENT)
      include 'prf.cmn'
c
c
      if ( means_min .le. n_points .and. n_points .le. means_max ) then   1b
          write( STD_OUT, 1 )
1         format( ' Calculating means & statistical moments' )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1        write( dmp_units(DUMP_AUXL), 1 )
c
          seg_begin = seg_start
          total_seg = irow
          do 4 iseg = 1, n_segs                                           2b
              actual_len = check_seg_len( iseg, seg_begin, 'SP_MEANS' )
              if ( actual_len .le. 0 )
     1            return
c
c           calculate statistical moments and store them in results array
c
              call smeang( udata(seg_begin), seg_increm, actual_len,
     1            stats, means_moment )
              do 2 i = 1, means_moment                                    3b
                  result(total_seg,MEAN_AVERAGE-1 + i ) = stats(i)
2                 continue                                                3e
c
c           convert 2nd moment into rms & turbulence level
c
              if ( means_moment .gt. 1 ) then                             3b
                  result(total_seg,MEAN_RMS) = sqrt( stats(2) )
                  if ( result(total_seg,MEAN_AVERAGE) .eq. 0.0 ) then     4b
                      result(total_seg,MEAN_TURBUL) = 0.0
                  else                                                    4m
                      result(total_seg,MEAN_TURBUL) =
     1                    result(total_seg,MEAN_RMS) /
     2                    result(total_seg,MEAN_AVERAGE)
                      end if                                              4e
                  end if                                                  3e
              if ( means_moment .gt. 2 ) then                             3b
                  denom = stats(2)
                  do 3 i = 3, means_moment                                4b
                      denom = denom * result(total_seg,MEAN_RMS)
                      if ( denom .eq. 0.0 ) then                          5b
                          result(total_seg,MEAN_SKEW-3+i) = 0.0
                      else                                                5m
                          result(total_seg,MEAN_SKEW-3+i) =
     1                        result(total_seg,MEAN_VARIANCE-2+i) /denom
                          end if                                          5e
3                     continue                                            4e
                  end if                                                  3e
              seg_begin = seg_begin + seg_len * seg_increm
              total_seg = total_seg + 1
4             continue                                                    2e
          end if                                                          1e
      return
      end



      SUBROUTINE SP_PROB_DENSITY( air_speed, pdf )
c
c Calculate, plot and dump the probabilty density distribution of the
c air speed data stream .
c
c Arguments
c  real air_speed(*)    # storage for air speed values from SPDF1
c       pdf(*)          # storage for PDF values
c
c local variables
c  logical
c   dumping     # dumping PDFs to file
c   plotting    # plot PDFs
c   test_conv   # test for convergence of moment integrands
c   Eyes        # function : n/y/q to logical
c  integer
c   actual_len  # actual data segment length after checking availability of data
c   i           # looping index for the data stream
c   imoment     # looping index for moment order
c   iseg        # data segment number
c   nxplts      # number of plot boxes across the screen
c   nyplts      # number of plot boxes down the screen
c   n_bins      # number of PDF values
c   seg_begin   # start index for segment iseg
c   check_seg_len # function : check available data
c  real
c   del_speed   # air speed increment from PDF
c   mean_speed  # mean air speed
c   qmean       # function : mean air speed
c  character
c   Clabel*38   # graph title of axis label
c  subroutine
c   ilgclos, ilgdata, ilgopen, ilgpage
c   ilgtitl, ilgwait, ilgxlab, ilgylab
c   spdf1       # f77_sig : calculate probability density function
c   open_dump   # open dump file
c
c global variables
c   dmp_units, dmp_names, n_points, n_points, n_profiles, n_segs, prob_bins,
c   prob_capt, prob_max, prob_min, prob_moment, prob_plot,
c   prob_test, prob_wait, seg_increm, seg_len, seg_start, udata
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical dumping, plotting, test_conv, Eyes
      integer actual_len, i, imoment, iseg, nxplts, nyplts,
     1    n_bins, seg_begin, check_seg_len
      real del_speed, mean_speed, air_speed(*), pdf(*), qmean
      character Clabel*38
c
      include 'prf.cmn'
c
c
      if ( prob_min .le. n_points .and. n_points .le. prob_max ) then     1b
          write( STD_OUT, 1 )
1         format( ' Calculating probability density' )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1        write( dmp_units(DUMP_AUXL), 1 )
          test_conv = Eyes( prob_test, 'Evaluate moment integrands ? ' )
     1        .and. prob_moment .gt. 0
          plotting = Eyes( prob_plot, 'Plot probability densities ? ' )
          dumping = dmp_names(DUMP_PROB) .ne. ' '
          if ( dmp_units(DUMP_PROB) .eq. 0 .and. dumping )
     1        call open_dump( DUMP_PROB )
c
c           initialize plotting for pdf only
c
          if ( .not. test_conv .and. plotting ) then                      2b
              call ilgopen( prob_capt )
              nxplts = min( int( sqrt( real( n_segs - 1 ) ) ) + 1,
     1            MAX_BOXES )
              nyplts = min( ( n_segs + nxplts - 1 ) / nxplts,
     1            MAX_BOXES )
c             this is to get the lettering size right
              call ilgpage( 'a4', 'wide', 1, nxplts * nyplts )
              write( Clabel, 2 ) n_points, n_profiles
2             format( 'PDFs for point ', i2, ' in profile ',i2 )
              call ilgtitl( Clabel )
              call ilgpage( 'a4', 'WIDE', nxplts, nyplts )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgslaw( 'lin', 'lin' )
              end if                                                      2e
c
          seg_begin = seg_start
          do 12 iseg = 1, n_segs                                          2b
              actual_len = check_seg_len( iseg, seg_begin,
     1                'SP_PROB_DENSITY' )
              if ( actual_len .le. 0 )
     1            goto 13
c
c           calculate pdfs, and abscissa values
c
              call spdf1( udata(seg_begin), seg_increm, actual_len,
     1            pdf, 1, prob_bins, air_speed(1), del_speed )
              air_speed(1) = air_speed(1) + del_speed / 2.0
              do 3 i = 2, prob_bins                                       3b
                  air_speed(i) = air_speed(i-1) + del_speed
3                 continue                                                3e
c
c           initialize plotting for moments of distribution
c
              if ( test_conv .and. plotting ) then                        3b
                  call ilgopen( prob_capt )
                  nxplts = min( int( sqrt( real( prob_moment ) ) ) + 1,
     1                MAX_BOXES )
                  nyplts = min( ( prob_moment + nxplts ) / nxplts,
     1                MAX_BOXES )
c                 this is to get the lettering size right
                  call ilgpage( 'a4', 'wide', 1, nxplts * nyplts )
                  write( Clabel, 4 ) iseg, n_points, n_profiles
4                 format( 'Segment',i3, ' for point ', i2,
     1                ' in profile ', i2 )
                  call ilgtitl( Clabel )
                  call ilgpage( 'a4', 'WIDE', nxplts, nyplts )
                  call ilgraph( 1, 1, 11, 11, 1 )
                  call ilgslaw( 'lin', 'lin' )
                  end if                                                  3e
c
c           if testing convergence of moment integrands, first calculate the
c           mean of the distribution
c
              if ( test_conv )
     1            mean_speed = qmean( udata(seg_begin), seg_increm,
     2                actual_len )
c
c           plot pdf
c
              if ( plotting ) then                                        3b
                  call ilgdata( air_speed, pdf, 1, prob_bins, 1 )
                  call ilgxlab( 'air speed (m/s)' )
                  call ilgylab( 'PDF' )
c
c           if testing convergence of moment integrands, calculate each one
c           and then plot it .
c
                  if ( test_conv ) then                                   4b
                      do 7 imoment = 1, prob_moment                       5b
                          do 5 i = 1, prob_bins                           6b
                              pdf(i) = pdf(i) *
     1                            ( air_speed(i) - mean_speed )
5                             continue                                    6e
                          call ilgdata( air_speed, pdf, 1, prob_bins, 1)
                          call ilgxlab( 'air speed (m/s)' )
                          write( Clabel, 6 ) imoment
6                         format( 'Mom ',i1, ' integrand' )
                          call ilgylab( Clabel )
7                         continue                                        5e
c
c           wait for <RETURN>
c
                      if ( prob_wait .eq. YES )
     1                    call ilgwait()
                  else                                                    4m
                      if ( prob_wait .eq. YES ) then                      5b
                          if ( iseg .eq. n_segs .or.
     1                            mod( iseg, nxplts * nyplts ) .eq. 0 )
     2                        call ilgwait()
                          end if                                          5e
                      end if                                              4e
                  end if                                                  3e
c
c             tabulate pdf
c
              if ( dumping .and. dmp_units(DUMP_PROB) .ne. 0 ) then                    3b
c
c           if moment integrands were plotted, recalculate PDF, because
c           the PDF values have been destroyed
c
                  if ( plotting .and. test_conv ) then                    4b
                      call spdf1( udata(seg_begin), seg_increm,
     1                    actual_len, pdf, 1, prob_bins, air_speed(1),
     2                    del_speed )
                      air_speed(1) = air_speed(1) + del_speed / 2.0
                      end if                                              4e
c
c             header for dump file
c
                  call dump_header( dmp_units(DUMP_PROB), iseg,
     1                'Probability density data' )
                  write( dmp_units(DUMP_PROB), 8 )
8                 format( ' ; air speed   PDF' / ' ;    m/s' )
c
c             calculate and tabulate moment integrands (& PDF)
c
                  if ( test_conv ) then                                   4b
                      do 10 i = 1, prob_bins                              5b
                          write( dmp_units(DUMP_PROB), 9 )
     1                        air_speed(i), pdf(i),
     2                        ( pdf(i) * ( air_speed(i) - mean_speed )
     3                          ** imoment, imoment = 1, prob_moment )
9                         format( 1x, f7.4, 7g13.5 )
10                        continue                                        5e
                  else                                                    4m
c
c             tabulate PDF only
c
                      write( dmp_units(DUMP_PROB), 11 )
     1                    ( air_speed(i), pdf(i), i = 1, prob_bins )
11                    format( 1x, f7.4, g13.5 )
                      end if                                              4e
                  end if                                                  3e
              seg_begin = seg_begin + seg_len * seg_increm
12            continue                                                    2e
13        if ( plotting )
     1        call ilgclos()
          end if                                                          1e
      return
      end



      SUBROUTINE SP_AUTOCORREL( TIME, AUTOCORR )
c
c Calculate, plot and tabulate the autocorrelation of the air speed data stream.
c
c real time(*)      # storage for time lag values
c      autocorr(*)  # storage for autocorrelation values
c
c local variables
c  integer
c   actual_len  # actual data segment length after checking availability of data
c   i           # index for lag time array
c   ierror      # non-zero indicates error from SAUTO
c   iseg        # data segment number
c   iwork(MAX_FFT)  # work array required for FFT
c   nxplts      # number of plot boxes across screen
c   nyplts      # number of plot boxes down screen
c   seg_begin   # start index for data segment iseg
c   check_seg_len # function : check available data
c  logical
c   dumping     # dumping autocorrelation results to file
c   plotting    # plotting autocorrelation results
c   Eyes        # function : converts n/y/q to logical
c  character
c   Clabel*42   # graph title
c  real
c   cycle_time  # A/D cycle time
c   Qextcytm    # rdflib : get A/D cycle time
c  subroutine
c   ilgclos, ilgdata, ilgopen, ilgpage
c   ilgtitl, ilgwait, ilgxlab, ilgylab
c   dump_header
c   open_dump   # open dump file
c   sauto       # autocorrelation by FFT method
c   sacorr      # autocorrelation by time domain method
c
c global variables
c   dmp_names, dmp_units, auto_capt, auto_lags, auto_max, auto_min,
c   auto_method, auto_plot, dmp_units, n_points, n_profiles, n_segs,
c   seg_increm, seg_len, seg_start, udata, zwork
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical dumping, plotting, Eyes
      integer actual_len, i, ierror, iseg, iwork(MAX_FFT),
     1    nxplts, nyplts, seg_begin, check_seg_len
      real time(*), autocorr(*), cycle_time, Qextcytm
      character Clabel*42
      include 'prf.cmn'
c
c
      if ( auto_min .le. n_points .and. n_points .le. auto_max ) then     1b
          write( STD_OUT, 1 )
1         format( ' Calculating autocorrelation' )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1        write( dmp_units(DUMP_AUXL), 1 )
          plotting = Eyes( auto_plot, 'Plotting autocorrelations ? ' )

          plotting = Eyes( prob_plot, 'Plot probability densities ? ' )
          dumping = dmp_names(DUMP_AUTO) .ne. ' '
          if ( dmp_units(DUMP_AUTO) .eq. 0 .and. dumping )
     1        call open_dump( DUMP_AUTO )
          cycle_time = Qextcytm()
c
c           initialize plot
c
          if ( plotting ) then                                            2b
              call ilgopen( auto_capt )
              nxplts = min( int( sqrt( real( n_segs - 1 ) ) ) + 1,
     1            MAX_BOXES )
              nyplts = min( ( n_segs + nxplts - 1 ) / nxplts,
     1            MAX_BOXES )
              call ilgpage( 'a4', 'wide', 1, nxplts * nyplts )
              write( Clabel, 2 ) n_points, n_profiles
2             format( 'Autocorrelation for point ', i2, ' in profile ',
     1            i2 )
              call ilgtitl( Clabel )
              call ilgpage( 'a4', 'WIDE', nxplts, nyplts )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgslaw( 'lin', 'lin' )
              end if                                                      2e
c
c           set the lag time (abscissa array)
c
          time(1) = 0.0
          do 3 i = 2, auto_lags + 1                                       2b
              time(i) = time(i-1) + seg_increm * cycle_time
3             continue                                                    2e
c
c           calculate autocorrelation
c
          seg_begin = seg_start
          do 6 iseg = 1, n_segs                                           2b
              actual_len = check_seg_len( iseg, seg_begin + 1,
     1            'SP_AUTOCORREL' )
              if ( actual_len .le. 0 )
     1            goto 7
              if ( auto_method .eq. 1 ) then                              3b
                  call sauto( udata(seg_begin), seg_increm, actual_len,
     1                autocorr, 1, auto_lags + 1, ierror, iwork, zwork )
                  if ( ierror .ne. 0 )
     1                write( STD_OUT, 4 ) ierror
4                     format( ' PRF : Error calculating ',
     1                    'autocorrelation : ', i6 )
              else                                                        3m
                  autocorr(1) = 1.0
                  call sacorr( udata(seg_begin), seg_increm, actual_len,
     1                autocorr(2), 1, auto_lags )
                  end if                                                  3e
c
c               plot
c
              if ( plotting ) then                                        3b
                  call ilgdata( time, autocorr, 1, auto_lags, 1 )
                  call ilgxlab( 'lag time (sec)' )
                  call ilgylab( 'Autocorr''n' )
                  if ( auto_wait .eq. YES ) then                          4b
                      if ( iseg .eq. n_segs .or.
     1                     mod( iseg, nxplts * nyplts ) .eq. 0 )
     2                    call ilgwait()
                      end if                                              4e
                  end if                                                  3e
c
c               tabulate
c
              if ( dumping .and. dmp_units(DUMP_AUTO) .ne. 0 ) then       3b
                  call dump_header( dmp_units(DUMP_AUTO), iseg,
     1                'Autocorrelation of air speed stream' )
                  write( dmp_units(DUMP_AUTO), 5 )
     1                ( time(i), autocorr(i), i = 1, auto_lags )
5                 format( ' ;      time   autocorrelation' /
     1                ' ;    seconds' / ( 1x, 2g13.5e2 ) )
                  end if                                                  3e
              seg_begin = seg_begin + seg_len * seg_increm
6             continue                                                    2e
7         if ( plotting )
     1        call ilgclos()
          end if                                                          1e
      return
      end



      SUBROUTINE SP_POWER_SPEC( FREQUENCY, PSD )
c
c Calculate, plot & tabulate the power spectrum density of the air speed
c signal . The RMS average of periodograms method is used .
c
c Arguments
c  real frequency(*)  # storage for frequency array
c       psd(*)        # storage for power spectral density
c
c constant
c  TINY_REAL = 1.0e-14
c
c local variables
c  integer
c   actual_len  # actual data segment length after checking availability of data
c   i           # index for frequency array
c   ierror      # non-zero indicates error from SAUTO
c   iseg        # data segment number
c   len_fft     # number of frequency values
c   iwork(MAX_FFT)  # work array required for FFT
c   nxplts      # number of plot boxes across screen
c   nyplts      # number of plot boxes down screen
c   seg_begin   # start index for data segment iseg
c   check_seg_len # function : check available data
c  logical
c   dumping     # dumping autocorrelation results to file
c   plotting    # plotting autocorrelation results
c   ein         # function : 1st arg char is in 2nd arg string
c   Eyes        #          : converts n/y/q to logical
c  character
c   Clabel*42   # graph title and axis title
c  real
c   cycle_time  # A/D cycle time
c   delta_freq  # frequency step
c   Qextcytm    # rdflib : get A/D cycle time
c  subroutine
c   ilgclos, ilgdata, ilgpage, ilgopen, ilgraph,
c   ilgslaw, ilgtitl, ilgwait, ilgxlab, ilgylab
c   dump_header # write header to dump file
c   open_dump   # open dump file
c   sffpsd      # calculate power spectral density by FFT method
c
c global variables
c   dmp_units, n_points, n_profiles, n_segs, seg_increm, seg_len, seg_start,
c   spec_capt, spec_dump, spec_max, spec_min, spec_nfft, spec_plot,
c   spec_wait, spec_window, spec_xlaw, spec_ylaw, udata, zwork
c
c
      include 'stdio.def'
      include 'prf.def'
      real TINY_REAL
      parameter( TINY_REAL=1.0e-14 )
c
      integer actual_len, i, ierror, iseg, len_fft, iwork(MAX_FFT),
     1    nxplts, nyplts, seg_begin, check_seg_len
      logical dumping, plotting, ein, Eyes
      real frequency(*), psd(*), cycle_time, delta_freq, Qextcytm
      character Clabel*42
      include 'prf.cmn'
c
c
      if ( spec_min .le. n_points .and. n_points .le. spec_max ) then     1b
          write( STD_OUT, 1 )
1         format( ' Calculating power spectral density' )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1         write( dmp_units(DUMP_AUXL), 1 )
          plotting = Eyes( spec_plot, 'Plot power spectra ? ' )
          dumping = dmp_names(DUMP_SPEC) .ne. ' '
          if ( dmp_units(DUMP_SPEC) .ne. 0 .and. dumping )
     1        call open_dump( DUMP_SPEC )
          cycle_time = Qextcytm()
          seg_begin = seg_start
c
c           initialize plot
c
          if ( plotting ) then                                            2b
              call ilgopen( spec_capt )
              nxplts = min( int( sqrt( real( n_segs - 1 ) ) ) + 1,
     1            MAX_BOXES )
              nyplts = min( ( n_segs + nxplts - 1 ) / nxplts,
     1            MAX_BOXES )
              call ilgpage( 'a4', 'wide', 1, nxplts * nyplts )
              write( Clabel, 2 ) n_points, n_profiles
2             format( 'Power spectrum for point ', i2, ' in profile ',
     1            i2 )
              call ilgtitl( Clabel )
              call ilgpage( 'a4', 'WIDE', nxplts, nyplts )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgslaw( spec_xlaw, spec_ylaw )
              end if                                                      2e
c
          len_fft = 2 ** spec_nfft
          if ( .not. ein( 'z', spec_window ) )
     1        len_fft = len_fft / 2
c
c               calculate psd
c
          do 7 iseg = 1, n_segs                                           2b
              actual_len = check_seg_len( iseg, seg_begin + 1,
     1           'SP_POWER_SPEC' )
              if ( actual_len .le. 0 )
     1            goto 8
              call sffpsd( udata(seg_begin), seg_increm, actual_len,
     1            cycle_time, spec_nfft, spec_window,
     2            psd, 1, delta_freq, zwork, iwork, ierror )
              if ( ierror .ne. 0 )
     1            write( STD_ERR, 3 ) ierror
3                 format( ' PRF : WARNING : error calculating power ',
     1                'spectral density : ', i4 )
              if ( iseg .eq. 1 ) then                                     3b
                  frequency(1) = 0.0
                  do 4 i = 2, len_fft                                     4b
                      frequency(i) = frequency(i-1) + delta_freq
4                     continue                                            4e
                  end if                                                  3e
c
c               plot
c
              if ( plotting ) then                                        3b
                  if ( spec_xlaw .eq. 'log') then                         4b
                      do 5 i = 1, len_fft                                 5b
                          psd(i) = max( psd(i), TINY_REAL )
5                         continue                                        5e
                      call ilgdata( frequency(2), psd(2), 1,
     1                    len_fft - 1, 1 )
                  else                                                    4m
                      call ilgdata( frequency, psd, 1, len_fft, 1 )
                      end if                                              4e
                  call ilgxlab( 'frequency (Hz)' )
                  call ilgylab( 'PSD' )
                  if ( spec_wait .eq. YES ) then                          4b
                      if ( iseg .eq. n_segs .or.
     1                     mod( iseg, nxplts * nyplts ) .eq. 0 )
     2                    call ilgwait()
                      end if                                              4e
                  end if                                                  3e
c
c               tabulate
c
              if ( dumping .and. dmp_units(DUMP_SPEC) .ne. 0 ) then       3b
                  call dump_header( dmp_units(DUMP_SPEC), iseg,
     1                'Power spectral density' )
                  write( dmp_units(DUMP_SPEC), 6 )
     1                ( frequency(i), psd(i), i = 1, len_fft )
6                 format( ' ;  frequency    PSD' /
     1                ' ;  Hz     (m/s)**2/Hz' /
     2                ( 1x, g12.5, g13.5 ) )
                  end if                                                  3e
              seg_begin = seg_begin + seg_len * seg_increm
7             continue                                                    2e
8         if ( plotting )
     1        call ilgclos()
          end if                                                          1e
      return
      end



      SUBROUTINE SSPANAL
c
c       Single Point ANALysis
c
c    The elements of single point analysis are :
c (0) filtering data
c (1) wall proximity correction
c (2) display time series air speed data
c (3) means and moments
c (4) probability density function
c (5) intermittency function
c (6) power spectral density
c (7) autocorrelation function
c
c local variables
c  logical
c   Eyes            # function : convert n/y/q to logical
c  integer
c   iseg            # data segment number
c   save_len        # saved value of segment length (restored at end)
c   total_segs      # total number of data segments
c  real
c   abscissa(MAX_SAMPLES) # abscissa storage for individual process routines
c   ordinate(MAX_SAMPLES) # ordinate storage for individual process routines
c  subroutine
c   sp_calib_limit  # count data points exceeding calibration limits
c   sp_filter       # pre-filter signal (to remove noise)
c   sp_wall_prox    # apply wall proximity correction - velocity error method
c   sp_time_history # display time series data
c   sp_means        # calculate means and statistical moments
c   sp_prob_density # calculate probability density function
c   sp_autocorrel   # calculate autocorrelation
c   sp_power_spec   # calculate power sspectral density
c
c global variables
c   auto_do, filter_do, hist_do, irow, means_do, n_points, n_segs, prob_do,
c   profile_pt, profile_seg, seg_len, seg_start, spec_do, wall_do, wall_method
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical Eyes
      integer iseg, save_len, save_start, total_segs
      integer *2 ad_dummy
      real abscissa(MAX_SAMPLES), ordinate(MAX_SAMPLES)
c
      include 'prf.cmn'
c
c
c           assign profile point number and segment numbers to each segment
c
      total_segs = irow
      do 1 iseg = 1, n_segs
          profile_pt(total_segs) = n_points
          profile_seg(total_segs) = iseg
          total_segs = total_segs + 1
1         continue
c
c           save the segment length (to be restored later )
c
      save_len = seg_len
c
c
c            count data points exceeding calibration limits
      call sp_calib_limit()
c
c             pre-filter the velocity signal
c             this may change segment length and start index
      if ( Eyes( filter_do, 'Pre-filter velocity data stream ? ' ) )
     1    call sp_filter( .TRUE., AD_dummy )
c
c             apply wall proximity correction - velocity error method
      if ( wall_method .eq. VELOCITY_ERROR ) then
          if ( Eyes( wall_do, 'Apply wall proximity correction ? ' ) )
     1        call sp_wall_prox()
          end if
c
c             display time series data
      if ( Eyes( hist_do, 'Display time series data ? ' ) )
     1    call sp_time_history( abscissa )
c
c             calculate means and statistical moments
      if ( Eyes( means_do, 'Calculate means & moments ? ' ) )
     1    call sp_means()
c
c             calculate probability density function
      if ( Eyes( prob_do, 'Calculate probability density ? ' ) )
     1    call sp_prob_density( abscissa, ordinate )
c
c             calculate autocorrelation
      if ( Eyes( auto_do, 'Calculate autocorrelation ? ' ) )
     1    call sp_autocorrel( abscissa, ordinate )
c
c             calculate power spectral density
      if ( Eyes( spec_do, 'Calculate power spectrum ? ' ) )
     1    call sp_power_spec( abscissa, ordinate)
c
c           restore segment length and start index
c
      seg_len = save_len
      return
      end



      SUBROUTINE SASCVT( Mdata )
c
c     ConVerT raw data A/D samples into Air Speed values by the method
c previously specified .
c
c Argument
c  integer *2 Mdata(*)  # A/D samples from hot-wire
c
c local variables
c  logical
c   ok            # success on converting A/D samples to air speed
c   Eyes          # function : convert n/y/q to logical
c   Ehw1_ADtoU    #          : convert A/D samples to air speed
c  real
c   QHW1_E2toU    # function : hw1lib : convert CTA volts squared to air speed
c  subroutine
c   prox_nusselt  # apply heat flow wall proximity correction
c
c global variables
c   air_viscosity, dmp_units, irow, n_points, n_samples, result, Udata,
c   wall_do, wall_file, wall_max, wall_method, wall_min
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical ok, Eyes, Ehw1_ADtoU
      integer *2 Mdata(*)
      real QHW1_E2toU, QHW1_UtoE2
      include 'prf.cmn'
      external QHW1_E2toU, QHW1_UtoE2
c
c
c       filter the A/D signal
c             this may change segment length and start index
      if ( Eyes( filter_do, 'Pre-filter A/D data stream ? ' ) )
     1    call sp_filter( .FALSE., Mdata )
c
c       Correction for wall proximity effect by the heat flow method
c       This incorporates conversion of A/D samples to air speed
c
      if ( wall_method .eq. 2 .and.
     1        wall_min .le. n_points .and. n_points .le. wall_max ) then  1b
          if ( Eyes( wall_do,
     1               'Apply wall proximity correction ? ' ) ) then        2b
              write( STD_OUT, 1 )
1             format( ' PRF Applying heat flow wall proximity ',
     1            'correction' )
              if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1            write( dmp_units(DUMP_AUXL), 1 )
              call prox_nusselt( Mdata, Udata, 1, n_samples,
     1            result(irow,ACTUAL_POS) / 1000.0, air_viscosity,
     2            QHW1_E2toU, QHW1_UtoE2,  wall_file )
              ok = .TRUE.
c
c       Convert A/D samples to air speed
c
          else                                                            2m
              ok = Ehw1_ADtoU( Mdata, Udata, 1, n_samples )
              end if                                                      2e
      else                                                                1m
          ok = Ehw1_ADtoU( Mdata, Udata, 1, n_samples )
          end if                                                          1e
c
c       Report failure
c
      if ( .not. ok ) then                                                1b
          write( STD_OUT, 2 )
2         format( ' PRF : WARNING : Could not convert A/D samples to ',
     1        'air speed' )
          if ( dmp_units(DUMP_AUXL) .ne. 0 )
     1        write( dmp_units(DUMP_AUXL), 2 )
          end if                                                          1e
      return
      end



      SUBROUTINE REMOVE_FAULTY_DATA( TOTAL_SEGS )
c
c Remove faulty data points from the velocity profile .
c
c Arguments
c  integer total_segs  # total number od data segments
c
c local variables
c  integer
c   col           # column index for sweeping through profile variables
c   i             # index for profile points
c   Jresp         # function : read integer
c  logical *1
c   del_flags(MAX_POINTS) # flags points for deletion
c  logical
c   Equery        # function : read logical
c  subroutine
c   rotate_vector # rotate element of an array to the end
c
c global variables
c   profile_pt, profile_seg, result
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer total_segs, col, i, Jresp
      logical *1 del_flags(MAX_POINTS)
      logical Equery
      include 'prf.cmn'
c
c       display mean & rms air speed
c
c     repeat                                                              1b
1         write( STD_OUT, 2 )
2         format( ' Removing faulty data ' /
     1        '    List of traverse postions and air speed statistics' /
     2        '  pos"n seg  actual' /
     3        '    no. no. position    mean        rms       skewness',
     4            '    flatness' )
          do 4 i = 1, total_segs                                          2b
              del_flags(i) = .FALSE.
              write( STD_OUT, 3 ) i, profile_pt(i), profile_seg(i),
     1            result(i,ACTUAL_POS), result(i,MEAN_AVERAGE),
     2            result(i,MEAN_RMS),   result(i,MEAN_SKEW),
     3            result(i,MEAN_FLAT)
3             format( 1x, i3, 1x, i3, 1x, i3, f8.3, 4g12.5e1 )
4             continue                                                    2e
c
c       get the user to specify which values to delete
c
          write( STD_OUT, 5 )
5         format( ' List positions to be deleted' /
     1        ' (use left hand column, finish with 0)' )
c         repeat                                                          2b
6             write( STD_OUT, 7 )
7             format( ' Delete ? ', $ )
              i = Jresp( STD_IN, 0, total_segs, MAX_TRIES )
              if ( 0 .lt. i .and. i .le. total_segs ) then                3b
                  del_flags(i) = .TRUE.
                  goto 6
                  end if                                                  3e
c             until( i .eq. 0 )                                           2e
c
c       find any duplicate probe locations and flag for deletion
c
          do 9 i = 2, total_segs                                          2b
              if ( abs( result(i,ACTUAL_POS) - result(i-1,ACTUAL_POS) )
     1                .le. 0.0005 .and.
     2                profile_pt(i) .ne. profile_pt(i-1) .and.
     3                profile_seg(i) .ne. profile_seg(i-1) ) then
                  write( STD_OUT, 8 ) i, result(i,ACTUAL_POS)
8                 format( ' Duplicate position ', i3, ' at ', f8.3, $ )
                  if ( Equery( STD_IN, ' : remove it ? ' ) )
     1                del_flags(i) = .TRUE.                               3e
                  end if                                                  2e
9             continue
c
c          delete data by swapping the flagged rows to the bottom of the table
c
          do 11 i = total_segs, 1, -1                                     2b
              if ( del_flags(i) ) then                                    3b
                  do 10 col = 1, MAX_RESULT                               4b
                      call rotate_vector( result(1,col), i, total_segs )
10                    continue                                            4e
                  call rotate_vector( profile_pt, i, total_segs )
                  call rotate_vector( profile_seg, i, total_segs )
                  total_segs = total_segs - 1
                  end if                                                  3e
11            continue                                                    2e
c
          if ( .not. Equery( STD_IN, 'Finished deletions ? ' ) )
     1        goto 1
c       until ( finished deletions )                                      1e
      return
      end



      LOGICAL FUNCTION ESEL_FREE_STREAM( TOTAL_SEGS, SPEED_FS )
c
c choose free stream air speed
c
c Arguments
c   integer total_segs  # total number of data segments
c   real    speed_fs    # selected free stream speed
c
c local variables
c  integer
c   i           # selection list looping index
c   item        # selected menu item
c   n_averaged  # number of values averaged to calculate free stream air-speed
c   select_len  # length of selection list
c   select_list(MAX_POINTS) # flags which elements in the list are used
c   Jresp       # function : read integer
c  real
c   clip_speed  # lowest permissible speed for including data in the list
c   ratio       # clip_speed = ratio * mean baratron speed
c   select_fs(MAX_POINTS) # air speeds in selection list
c   select_mean # selected mean free stream air speed
c   select_y(MAX_POINTS)  # probe locations in selection list
c   qmean       # function : calculate mean
c   Qresp       #          : read real
c  logical
c   Equery      # function : read logical
c  subroutine
c   newscr      # clear screen
c   rotate_vector # rotate elements forward in a vector
c
c global variables
c   bar_pts, free_stream, result
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer total_segs, i, item, n_averaged, select_len,
     1    select_list(MAX_POINTS), Jresp
      real speed_fs, clip_speed, ratio, select_fs(MAX_POINTS),
     1    select_mean, select_y(MAX_POINTS), qmean, Qresp
      logical Equery
      include 'prf.cmn'
      save clip_speed, ratio, select_mean
      data ratio / 0.9 /
c
c
      clip_speed = ratio * qmean( free_stream, 1, bar_pts )
c     start outer loop                                                    1b
1         select_len = 0
          do 2 i = 1, bar_pts                                             2b
              select_len = select_len + 1
              select_fs(select_len) = free_stream(i)
              select_y(select_len)  = 0.0
              select_list(select_len) = .FALSE.
2             continue                                                    2e
          do 3 i = 1, total_segs                                          2b
              if ( result(i,MEAN_AVERAGE) .ge. clip_speed ) then          3b
                  select_len = select_len + 1
                  select_fs(select_len) = result(i,MEAN_AVERAGE)
                  select_y(select_len)  = result(i,ACTUAL_POS)
                  select_list(select_len) = .FALSE.
                  end if                                                  3e
3             continue                                                    2e
          select_mean = 0.0
c
c         start of menu loop                                              2b
4             select_mean = 0.0
              n_averaged = 0
              do 5 i = 1, select_len                                      3b
                  if ( select_list(i) ) then                              4b
                      select_mean = select_mean + select_fs(i)
                      n_averaged = n_averaged + 1
                      end if                                              4e
5                 continue                                                3e
              if ( n_averaged .ne. 0 )
     1            select_mean = select_mean / n_averaged
              call newscr()
              write( STD_OUT, 6 ) clip_speed, ratio, select_mean,
     1            ( i, select_list(i), select_y(i), select_fs(i),
     2              i = 1, select_len )
6             format( ' PRF Free stream selection menu' /
     N            '  -1) Abandon' /
     Z            '   0) Exit (ok)' /
     1            '   1) Minimum for initial selection         ', f8.4 /
     2            '   2) Reduction ratio for initial selection ', f8.4 /
     3            '   3) Initialize selection list' /
     4            '   4) Delete item from selection list' /
     5            '   5) Select item from list' /
     6            '      Selected free stream speed            ', f8.4 /
     7            '        distance   air' /
     8            ' No.    from wall  speed' /
     9            '    sel   (mm)     (m/s)' /
     a            ( 1x, i2, 2x, l1, 2x, f8.4, 1x, f8.4 ) )
              item = Jresp( STD_IN, -1, 5, MAX_TRIES )
c
c           ABANDON
c
              if ( item .eq. -1 ) then                                    3b
                  if ( Equery( STD_IN, 'Are you sure ? ' ) ) then         4b
                      esel_free_stream = .FALSE.
                      return
                      end if                                              4e
c
c           EXIT (OK)
c
              else if ( item .eq. 0 ) then                                3m
                  esel_free_stream = select_mean .ne. 0.0
                  speed_fs = select_mean
                  return
c
c           MINIMUM FOR INITIAL SELECTION
c
              else if ( item .eq. 1 ) then                                3m
                  write( STD_OUT, 7 )
7                 format( ' Minimum ? ', $ )
                  clip_speed = Qresp( STD_IN, 0.5, 40.0, MAX_TRIES )
c
c           REDUCTION RATIO FOR INITIAL SELECTION
c
              else if ( item .eq. 2 ) then                                3m
                  write( STD_OUT, 8 )
8                 format( ' Ratio ? ', $ )
                  ratio = Qresp( STD_IN, 0.3, 1.5, MAX_TRIES )
                  clip_speed = ratio * qmean( free_stream, 1, bar_pts )
c
c           INITIALIZE SELECTION LIST
c
              else if ( item .eq. 3 ) then                                3m
                  goto 1
c
c           DELETE ITEM FROM SELECT LIST
c
              else if ( item .eq. 4 .or. item .eq. 5 ) then               3m
                  if ( item .eq. 4 ) then                                 4b
                      write( STD_OUT, 9 )
9                     format( ' Delete ? ', $ )
                      i = Jresp( STD_IN, 1, select_len, MAX_TRIES )
                      call rotate_vector( select_y, i, select_len )
                      call rotate_vector( select_fs, i, select_len )
                      call rotate_vector( select_list, i, select_len )
                      select_len = select_len - 1
c
c           SELECT ITEM FROM LIST
c
                  else                                                    4m
                      write( STD_OUT, 10 )
10                    format( ' Select ? ', $ )
                      i = Jresp( STD_IN, 1, select_len, MAX_TRIES )
                      select_list(i) = .TRUE.
                      end if                                              4e
                  end if                                                  3m
              goto 4
c             end of menu loop                                            2e
c         end of loop                                                     1e
      end



      SUBROUTINE PROFILE_PROPERTIES( NEW_PROFILE, TOTAL_SEGS )
c
c Calculate boundary layer profile properties .
c
c Argument
c  logical new_profile  # IN # new_profile - clear properties before starting
c  integer total_segs   # IN # total number of data segments
c
c constants
c  BLM_dimens  = select dimensioned variable for boundary layer model
c  BLM_PI_wake = select variable Coles' PI for boundary layer model
c  TEST_SECTION_RADIUS = 114.43
c
c local variables
c  integer
c   BL_type       # boundary layer type,
c                 #   1='2 Dimensional', 2='Uniform Duct ', 3='Variable Duct',
c   dump_len      # length of dump file name
c   edit_enabled  # NO / YES enable edit of thickness integrands
c   i             # looping index
c   item          # selected menu item
c   jljstr        # function : left justify string
c   Jresp         #          : read integer
c  logical
c   got_displ     # displacement thickness is available
c   got_energy    # energy thickness is available
c   got_free_str  # free stream speed is available
c   got_frict_vel # friction velocity is available
c   got_moment    # momentum thickness is available
c   got_thick     # boundary layer thickness is available
c   got_ts_dimens # test section dimensions have been read from file
c   esel_free_stream  # function : select free stream speed
c   eread_ts_dimens   #          : read test section dimensions from file
c  real
c   ts_width(MAX_PLUG)  # width of test section at each port
c   ts_height(MAX_PLUG) # height of test section at each port
c   ts_radius           # test section radius
c   qdelta_energy # function : energy thickness
c   qdelta_nn     #          : boundary layer thickness
c   qdelta_star   #          : displacement thickness
c   qdelta_set_get#          : set / get thickness calcultaion parameters
c   qdelta_theta  #          : momentum thickness
c   qu_tau        #          : friction velocity
c   Uplus_BLM     #          : mean velocity profile model
c  character
c   Cedit_enabled(0:1)*8 # 'disabled'/'enabled ' editing of thickness integrands
c   CBL_type(3)*13       # boundary layer type
c                        #   'Uniform Duct ', 'Variable Duct', '2 Dimensional'
c  subroutine
c   close_dump    # close dump file
c   Cresp         # f77_ute : read string
c   ilgwait       # ilglib  : wait for CR
c   newscr        # hpprev  : clear screen
c   open_dump     # open dump file
c   print_properties # print profile properties
c   sdelta_set_menu  # menu for setting thickness calculation parameters
c   set_BLM       # set values of variables for boundary layer model
c
c global variables
c   air_viscosity, dmp_names, dmp_units, n_points, n_profiles, plug_no,
c   profile, result, zwork
c
c
      include 'stdio.def'
      include 'prf.def'
      include '[planspear.for.lib.bly]blm.def'
      include '[planspear.for.lib.bly]bly_delta.def'
      real TEST_SECTION_RADIUS
      parameter( TEST_SECTION_RADIUS=114.43 )
c
      integer total_segs, BL_type, dump_len, edit_enabled, i, item,
     1        jljstr, Jresp
      logical new_profile, got_displ, got_energy, got_free_str,
     1        got_frict_vel, got_moment, got_thick, got_ts_dimens,
     2        esel_free_stream, eread_ts_dimens
      real    ts_width(MAX_PLUG), ts_height(MAX_PLUG), ts_radius,
     1        qdelta_energy, qdelta_nn, qdelta_star, qdelta_set_get,
     2        qdelta_theta, qu_tau, Uplus_BLM
      character CBL_type(3)*13, Cedit_enabled(0:1)*8
c
      include 'prf.cmn'
      save BL_type, dump_len, edit_enabled,
     1    got_displ, got_energy, got_free_str, got_frict_vel,
     2    got_moment, got_thick,
     3    CBL_type, Cedit_enabled
      data BL_type, dump_len, edit_enabled / 2, 1, YES /
      data got_displ, got_energy, got_free_str, got_frict_vel,
     1    got_moment, got_thick, got_ts_dimens / 7*.FALSE. /
      data Cedit_enabled / 'disabled', 'enabled ' /
      data CBL_type /
     1     '2 Dimensional', 'Uniform Duct ', 'Variable Duct' /
c
c
c           read test section dimensions from file
c
      if ( .not. got_ts_dimens ) then                                     1b
          got_ts_dimens = eread_ts_dimens( ts_width, ts_height )
          end if                                                          1e
c
c           clear properties if new profile
c
      if ( new_profile ) then                                             1b
          got_free_str  = .FALSE.
          got_frict_vel = .FALSE.
          got_thick  = .FALSE.
          got_displ  = .FALSE.
          got_moment = .FALSE.
          got_energy = .FALSE.
          end if                                                          1e
c
c           print menu
c
c     start of menu loop                                                  1b
1         call newscr()
c
c           tell boundary layer thickness package about the channel radius
c
          if ( BL_type .eq. 1 ) then                                      2b
              ts_radius = DELTA_2_DIM_BL
          else if ( BL_type .eq. 3 ) then                                 2m
              if ( got_ts_dimens .and.
     1                1 .le. plug_no .and. plug_no .le. 15 ) then         3b
                  ts_radius =
     1                ( ts_width(plug_no) + ts_height(plug_no) ) / 4.0
              else                                                        3m
                  ts_radius = TEST_SECTION_RADIUS
                  write( STD_ERR, 2 ) ts_radius
2                 format( ' WARNING : Test section dimensions are ',
     1                'unavailable or illegal plug number' /
     2                    '         : Using default radius ', f8.3 )
                  end if                                                  3e
          else                                                            2m
              BL_type = 2
              ts_radius = TEST_SECTION_RADIUS
              end if                                                      2e
          if ( qdelta_set_get(DELTA_DUC_RAD, ts_radius) .ne. ts_radius )
     1        write( STD_ERR, 3 )
3             format( ' QDELTA_SET_GET Cannot set duct radius' )
c
c
          edit_enabled = min( 1, max( 0, int(
     1        qdelta_set_get( -DELTA_EDIT_EN, 0.0 ) + 0.5 ) ) )
          dump_len = max( 1, jljstr( dmp_names(DUMP_GENL) ) )
          write( STD_OUT, 4 ) profile(n_profiles,FREESTR_SPEED),
     1        profile(n_profiles,FRICT_VELOCITY),
     2        profile(n_profiles,COLES_PI_WAKE),
     3        profile(n_profiles,COLES_THICK),
     4        Cedit_enabled( edit_enabled ), CBL_type( BL_type ),
     5        profile(n_profiles,BOUND_L_THICK),
     6        profile(n_profiles,DISPLAC_THICK),
     7        profile(n_profiles,MOMENTUM_THICK),
     8        profile(n_profiles,ENERGY_THICK),
     9        dmp_names(DUMP_GENL)(:dump_len)
4         format( ' PRF Calculate profile properties menu' /
     Z        '   0) Exit' /
     1        '   1) Free stream speed        ', f9.5 /
     2        '   2) Friction velocity        ', f9.5 /
     3        '      Coles wake strength      ', f9.5 /
     4        '      Coles B/L thickness      ', f9.5 /
     5        '   3) Wake (profile) function' /
     6        '   4) Thickness calculation parameters menu' /
     7        19x, 'Editing ', a /
     8        '   5) ', a, ' boundary layer' /
     9        '   6) Boundary layer thickness ', f9.5 /
     a        '   7) Displacement thickness   ', f9.5 /
     b        '   8) Momentum thickness       ', f9.5 /
     c        '   9) Energy  thickness        ', f9.5 /
     d        '  10) Dump_file                ', a /
     e        '  11) Dump all properties' /
     f        '  12) Clear all properties ', $ )
          item = Jresp( STD_IN, 0, 12, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              return
c
c           SELECT FREE STREAM SPEED
c
          else if ( item .eq. 1 ) then                                    2m

              got_free_str = esel_free_stream( total_segs,
     1            profile(n_profiles,FREESTR_SPEED) )
c
c           CALCULATE FRICTION VELOCITY & COLES WAKE & DELTA
c
          else if ( item .eq. 2 ) then                                    2m
              if ( got_free_str ) then                                    3b
                  profile(n_profiles,FRICT_VELOCITY) =
     1            qu_tau( result(1,MEAN_AVERAGE), result(1,ACTUAL_POS),
     2                1, total_segs, profile(n_profiles,FREESTR_SPEED),
     3                air_viscosity, profile(n_profiles,COLES_PI_WAKE),
     4                profile(n_profiles,COLES_THICK) )
                  profile(n_profiles,FRICT_COEFF)  = 2.0 *
     1                ( profile(n_profiles,FRICT_VELOCITY) /
     2                  profile(n_profiles,FREESTR_SPEED) ) **2
                  got_frict_vel =
     1                0.0012 .lt. profile(n_profiles,FRICT_COEFF) .and.
     2                profile(n_profiles,FRICT_COEFF) .lt. 0.008
              else                                                        3m
                  write( STD_OUT, 5 )
5                 format( ' PRF : WARNING : Free stream speed not ',
     1                'available . Cannot calculate friction velocity' )
                  call ilgwait()
                  end if                                                  3e
c
c           CALCULATE WAKE VELOCITY COMPONENTS
c
          else if ( item .eq. 3 ) then                                    2m
              if ( got_frict_vel .and.                                    3b
     1                profile(n_profiles,COLES_PI_WAKE) .gt. 0 ) then
                  call set_BLM( BLM_PI_wake, 0.0 )
                  call set_BLM( BLM_dimens, 0.0 )
                  do 6 i = 1, total_segs                                  4b
                      result(i,MEAN_WAKE) = result(i,MEAN_AVERAGE)
     1                    - Uplus_BLM( result(i,ACTUAL_POS) * 0.001 )
6                     continue                                            4e
                  call set_BLM( BLM_nondim, 0.0 )
                  call set_BLM( BLM_PI_wake,
     1                          profile(n_profiles,COLES_PI_WAKE) )
              else                                                        3m
                  write( STD_OUT, 7 )
7                 format( ' PRF : WARNING : Friction velocity not ',
     1                'available . Cannot calculate wake function' )
                  call ilgwait()
                  end if                                                  3e
c
c           THICKNESS CALCULATION PARAMETERS MENU
c
          else if ( item .eq. 4 ) then                                    2m
              call sdelta_set_menu()
c
c           DUCT FLOW / TWO DIMENSIONAL FLOW FOR INTEGRAL THICKNESSES
c
          else if ( item .eq. 5 ) then                                    2m

              BL_type = mod( BL_type, 3 ) + 1
c
c           BOUNDARY LAYER THICKNESS
c
          else if ( item .eq. 6 ) then                                    2m
              if ( got_free_str ) then                                    3b
c                 QDELTA_NN( LOCATION, MEAN_SPEED, INCREM, N_POINTS,
c                            EDGE_SPEED, WORK )
                  profile(n_profiles,BOUND_L_THICK) = qdelta_nn(
     1                result(1,ACTUAL_POS), result(1,MEAN_AVERAGE),
     2                1, total_segs,
     3                0.99 * profile(n_profiles,FREESTR_SPEED), zwork )
                  got_thick = .TRUE.
              else                                                        3m
                  write( STD_OUT, 8 ) 'boundary layer'
8                 format( ' PRF : WARNING : Free stream air speed not ',
     1                'available . Cannot calculate ', a, 'thickness.' )
                  call ilgwait()
                  end if                                                  3e
c
c           DISPLACEMENT THICKNESS
c
          else if ( item .eq. 7 ) then                                    2m
              if ( got_free_str ) then                                    3b
c                 QDELTA_STAR( LOCATION, MEAN_SPEED, INCREM, N_POINTS,
c                              FREE_STREAM, WORK )
                  profile(n_profiles,DISPLAC_THICK) = qdelta_star(
     1                result(1,ACTUAL_POS), result(1,MEAN_AVERAGE),
     2                1, total_segs,
     3                profile(n_profiles,FREESTR_SPEED), zwork )
                  got_displ = .TRUE.
              else                                                        3m
                  write( STD_OUT, 8 ) 'displacement'
                  call ilgwait()
                  end if                                                  3e
c
c           MOMENTUM THICKNESS
c
          else if ( item .eq. 8 ) then                                    2m
              if ( got_free_str ) then                                    3b
c                 QDELTA_THETA( LOCATION, MEAN_SPEED, INCREM, N_POINTS,
c                               FREE_STREAM, WORK )
                  profile(n_profiles,MOMENTUM_THICK) = qdelta_theta(
     1                result(1,ACTUAL_POS), result(1,MEAN_AVERAGE),
     2                1, total_segs,
     3                profile(n_profiles,FREESTR_SPEED), zwork )
                  got_moment = .TRUE.
              else                                                        3m
                  write( STD_OUT, 8 ) 'momentum'
                  call ilgwait()
                  end if                                                  3e
c
c           ENERGY  THICKNESS
c
          else if ( item .eq. 9 ) then                                    2m
              if ( got_free_str ) then                                    3b
c                 QDELTA_ENERGY( LOCATION, MEAN_SPEED, INCREM, N_POINTS,
c                                FREE_STREAM, WORK )
                  profile(n_profiles,ENERGY_THICK) = qdelta_energy(
     1                result(1,ACTUAL_POS), result(1,MEAN_AVERAGE),
     2                1, total_segs,
     3                profile(n_profiles,FREESTR_SPEED), zwork )
                  got_energy = .TRUE.
              else                                                        3m
                  write( STD_OUT, 8 ) 'energy'
                  call ilgwait()
                  end if                                                  3e
c
c           DUMP FILE NAME
c
          else if ( item .eq. 10 ) then                                   2m
              write( STD_OUT, 9 )
9             format( ' Dump file name ? ', $ )
              call Cresp( STD_IN, dmp_names(DUMP_GENL), MAX_TRIES )
              dump_len = max( 1, jljstr( dmp_names(DUMP_GENL) ) )
c
c           DUMP ALL PROPERTIES
c
          else if ( item .eq. 11 ) then                                   2m
              call open_dump( DUMP_GENL )
              if ( dmp_units(DUMP_GENL) .ne. 0 ) then                     3b
c             the fiddle in the next 3 lines is required because
c             print_properties expects n_profiles to point to the
c             first unused row in the table of profile properties
                  n_profiles = n_profiles + 1
                  call print_properties( dmp_units(DUMP_GENL),
     1                n_profiles - 1, n_profiles - 1 )
                  n_profiles = n_profiles - 1
                  call close_dump( DUMP_GENL )
                  end if                                                  3e
c
c           CLEAR ALL PROPERTIES
c
          else if ( item .eq. 12 ) then                                   2m
              got_free_str  = .FALSE.
              got_frict_vel = .FALSE.
              got_thick  = .FALSE.
              got_displ  = .FALSE.
              got_moment = .FALSE.
              got_energy = .FALSE.
              end if                                                      2e
c
c         calculate Shape factors & Reynolds numbers
c
          if ( got_displ .and. got_moment )
     1        profile(n_profiles,SHAPE_FACTOR) =
     2              profile(n_profiles,DISPLAC_THICK) /
     3              profile(n_profiles,MOMENTUM_THICK)
          if ( got_energy .and. got_moment )
     1        profile(n_profiles,SHAPE_32) =
     2            profile(n_profiles,ENERGY_THICK) /
     3            profile(n_profiles,MOMENTUM_THICK)
          if ( got_energy .and. got_displ )
     1        profile(n_profiles,SHAPE_31) =
     2            profile(n_profiles,ENERGY_THICK) /
     3            profile(n_profiles,DISPLAC_THICK)
          if ( got_free_str .and. got_thick )
     1        profile(n_profiles,REYN_DELTA) =
     2            profile(n_profiles,FREESTR_SPEED) *
     3            profile(n_profiles,BOUND_L_THICK) /
     4            air_viscosity / 1000.0
          if ( got_free_str .and. got_displ )
     1        profile(n_profiles,REYN_DISPL) =
     2            profile(n_profiles,FREESTR_SPEED) *
     3            profile(n_profiles,DISPLAC_THICK) /
     4            air_viscosity / 1000.0
          if ( got_free_str .and. got_moment )
     1        profile(n_profiles,REYN_MOMENT) =
     2            profile(n_profiles,FREESTR_SPEED) *
     3            profile(n_profiles,MOMENTUM_THICK) /
     4            air_viscosity / 1000.0
          if ( got_frict_vel .and. got_thick )
     1        profile(n_profiles,DELTA_PLUS) =
     2            profile(n_profiles,FRICT_VELOCITY) *
     3            profile(n_profiles,BOUND_L_THICK) /
     4            air_viscosity / 1000.0
          goto 1
c         end of menu loop                                                 1e
      end



      LOGICAL FUNCTION EREAD_TS_DIMENS( WIDTH, HEIGHT )
c
c
c Read test section internal cross section dimensions (width and height) at
c each port from file WTTS_DIMENS.
c  The format of each line of data in the file is
c   <plug_number> <channel_width> <channel_height>
c
c Argument
c  real width(*)    # OUTPUT # width of test section at each port
c       height(*)   # OUTPUT # height of test section at each port
c
c constants
c  WTTS_DIMENS = 'wtts_dimens.dat'
c
c local variables
c  integer
c   dimen_lu        # actual logical unit number of the file
c   line_count      # counts the number of lines read from file
c   n_chars         # number of characters on line after left justification
c   n_tokens        # number of tokens on line
c   plug_no         # plug number
c   res_dim_lu      # actual logical unit number of the file
c   jljstn          # function : left justify & count tokens
c   new_lun         #          : get new logical unit number
c   open_file       #          : open file
c  character
c   line*80         # line read from file
c
c
      include 'stdio.def'
      include 'prf.def'
      character WTTS_DIMENS*15
      parameter( WTTS_DIMENS='wtts_dimens.dat' )
c
      integer dimen_lu, line_count, n_chars, n_tokens, plug_no,
     1        res_dim_lu, jljstn, new_lun, open_file
      real    width(*), height(*)
      character line*80
c
c       open file containing wind tunnel test section dimensions
c       try the the current directory first, then try [planspear.com]
c
      if ( res_dim_lu .eq. 0 ) then
          res_dim_lu = new_lun()
          end if
      dimen_lu = open_file( res_dim_lu, WTTS_DIMENS, 'old', 0 )
      if ( dimen_lu .eq. 0 ) then
          dimen_lu = open_file( res_dim_lu,
     1        '[planspear.com]' // WTTS_DIMENS, 'old', 0 )
          end if
      if ( dimen_lu .ne. 0 ) then
          line_count = 0
c
c         start of loop - once for each line in the file
c
1             line_count = line_count + 1
c
c       read line from the file, check that is not a comment and is long enough
c
              read( dimen_lu, 2, END=3, ERR=5 ) line
2             format( a )
              n_chars = jljstn( line, n_tokens )
              if ( line(1:1) .ne. ';' ) then
                  if ( n_chars .lt. 5 .or. n_tokens .lt. 3 )
     1                goto 8
c
c       decode plug number, channel width and channel height
c
                  read( line, *, ERR=6 ) plug_no
                  if ( plug_no .le. 0 .or. plug_no .gt. MAX_PLUG )
     1                goto 7
                  read( line, *, ERR=6 )
     1                plug_no, width(plug_no), height(plug_no)
                  end if
              goto 1
c             end of loop
c
c           normal termination -- end of file
c
3         close( UNIT=dimen_lu )
          eread_ts_dimens = .TRUE.
      else
c
c       report errors from here
c
          write( STD_ERR, 4 ) ' Cannot open file ' // WTTS_DIMENS
          end if
      return
4     format( ' EREAD_TS_DIMENS : ABORT : ', a, ' at line ', i4 )
5     write( STD_ERR, 4 ) 'Cannot read file', line_count
      return
6     write( STD_ERR, 4 ) 'Cannot decode value', line_count
      return
7     write( STD_ERR, 4 ) 'Illegal plug number', line_count
      return
8     write( STD_ERR, 4 ) 'Insufficient data', line_count
      return
      end



      SUBROUTINE SPFANAL( NEW_PROFILE )
c
c               PROFILE ANALYSIS
c
c Argument
c   logical new_profile   # properties of this profile have not been
c                         # processed before
c
c local variables
c  integer
c   icol            # column number for profile listing table
c   item            # selected menu item
c   name_len        # length of profile list file name
c   plt_defined     # profile listing defined
c   prf_defined     # profile plots defined
c   total_segs      # total number of data segments in profile
c   def_prf_list    # function : Define profile listing
c   def_prf_plot    #          : Define profile plot
c   jljstr          #          : left justify string
c   jresp           #          : read integer
c  real
c   make_norm_factor  # function : calculate normalising factor from the code
c  character
c   Cyes*5          # 'no', 'yes', 'query'
c  subroutine
c   Cresp           # read string
c   gen_prf_list    # Generate profile list
c   gen_prf_plot    # Generate profile plots' /
c   newscr          # clear screen
c   profile_properties  # Calculate profile properties' /
c   remove_faulty_data  # Remove faulty data
c   sortsh          # shell sort of array
c
c global variables
c   n_points, n_segs, result
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical new_profile
      integer icol, item, name_len, prf_defined, plt_defined,
     1    total_segs, def_prf_list, def_prf_plot, jljstr, jresp
      real make_norm_factor
      character Cyes*5
c
      include 'prf.cmn'
      save name_len, plt_defined, prf_defined
      data name_len, plt_defined, prf_defined / 1, NO, NO /
c
c           sort data into ascending distance from wall
c
      total_segs = n_points * n_segs
      call sortsh( result, total_segs, 1, MAX_POINTS, ACTUAL_POS,
     1    MAX_RESULT )
      if ( total_segs .le. 0 .or.
     1     n_profiles .lt. 0 .or. n_profiles .gt. MAX_PROFILES )
     2    return
c
c     start of menu loop                                                  1b
1         call newscr()
          name_len = max( 1, jljstr( dmp_names(DUMP_PROF) ) )
          write( STD_OUT, 2 )  Cyes( prf_defined ),
     1        dmp_names(DUMP_PROF)(:name_len), Cyes( plt_defined )
2         format( ' PRF Profile analysis menu' /
     Z        '   0) Exit' /
     1        '   1) Remove faulty data' /
     2        '   2) Calculate profile properties' /
     3        '   3) Define profile listing ', a /
     4        '   4) Profile list file name ', a /
     5        '   5) Generate profile list' /
     6        '   6) Recalculate normalising factors' /
     7        '   7) Define profile plot    ', a /
     8        '   8) Generate profile plots' /
     9        '   9) Restore deleted data' )
          item = Jresp( STD_IN, 0, 9, MAX_TRIES )
c
c           EXIT
c
          if ( item .eq. 0 ) then                                         2b
              if ( dmp_units(DUMP_PROF) .ne. 0 )
     1            call close_dump( DUMP_PROF )
              return
c
c           REMOVE FAULTY DATA
c
          else if ( item .eq. 1 ) then                                    2m
              call remove_faulty_data( total_segs )
c
c           CALCULATE PROFILE PROPERTIES
c
          else if ( item .eq. 2 ) then                                    2m
              call profile_properties( new_profile, total_segs )
              new_profile = .FALSE.
c
c           DEFINE PROFILE LISTING
c
          else if ( item .eq. 3 ) then                                    2m
              if ( new_profile ) then                                     3b
                  write( STD_ERR, 3 )
3                 format( ' PRF : WARNING : Profile properties not ',
     1                'defined .' )
                  call ilgwait()
              else                                                        3m
                  prf_defined = def_prf_list()
                  end if                                                  3e
c
c           PROFILE LIST FILE NAME
c
          else if ( item .eq. 4 ) then                                    2m
              write( STD_OUT, 4 )
4             format( ' Profile list file name ? ', $ )
              call Cresp( STD_IN, dmp_names(DUMP_PROF), MAX_TRIES )
c
c           GENERATE PROFILE LIST
c
          else if ( item .eq. 5 ) then                                    2m
              if ( prf_defined ) then                                     3b
                  call gen_prf_list( total_segs )
              else                                                        3m
                  write( STD_ERR, 5 ) 'make list.'
5                 format( ' PRF : WARNING : Profile list not defined. ',
     1                'Cannot ', a )
                  call ilgwait()
                  end if                                                  3e
c
c           RECALCULATE NORMALISING FACTORS
c
          else if ( item .eq. 6 ) then                                    2m
              if ( .not. prf_defined ) then                               3b
                  write( STD_ERR, 5 ) 'calculate factors.'
                  call ilgwait()
              else if ( new_profile ) then                                3m
                  write( STD_ERR, 3 )
                  call ilgwait()
              else                                                        3m
                  do 6 icol = 1, n_list                                   4b
                      norm_factor(icol) = make_norm_factor( icol )
6                     continue                                            4e
                  end if                                                  3e
c
c           DEFINE PROFILE PLOT
c
          else if ( item .eq. 7 ) then                                    2m
              if ( new_profile )
     1            write( STD_ERR, 3 )
              if ( prf_defined ) then                                     3b
                  plt_defined = def_prf_plot()
              else                                                        3m
                  write( STD_ERR, 5 ) 'define plots'
                  call ilgwait()
                  end if                                                  3e
c
c           GENERATE PROFILE PLOTS
c
          else if ( item .eq. 8 ) then                                    2m
              if ( plt_defined ) then                                     3b
                  call gen_prf_plot( total_segs )
              else                                                        3m
                  write( STD_ERR, 7 )
7                 format( ' PRF : WARNING : Profile plots not defined. ',
     1                ' Cannot plot.' )
                  call ilgwait()
                  end if                                                  3e
c
c           RESTORE DELETED DATA
c
          else if ( item .eq. 9 ) then                                    2m
              total_segs = n_points * n_segs
              call sortsh( result, total_segs, 1, MAX_POINTS,
     1            ACTUAL_POS, MAX_RESULT )
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      INTEGER FUNCTION DEF_PRF_PLOT()
c
c Define profile plot
c
c local variables
c  logical
c   ok            # analogue of returned result
c   Equery        # function : read logical
c  integer
c   abscis_ptr    # pointer to abscissa variable id in prf_def
c   abscis_len    # length of abscissa description
c   i             # plot table index
c   iplot         # plot number
c   item          # selected menu item
c   name_len      # length of description file name
c   ordin_ptr     # pointer to ordinate variable id in prf_def
c   ordin_len     # length of ordinate description
c   jljstr        # function : left justify string
c   Jresp         #          : read integer
c   str_length    #          : non-blank length of string
c  character
c   Cscale(2)*3   # 'lin', 'log'
c   Cline*80      # line read from description file
c   Cyes*5        # function : 'no', 'yes', 'query'
c  subroutine
c   newscr        # clear screen
c   open_dump     # open description file
c   Cresp         # read string
c
c global variables
c  dmp_names, dmp_units
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical ok, Equery
      integer abscis_ptr, abscis_len, i, iplot, item, name_len,
     1    ordin_ptr, ordin_len, jljstr, Jresp, str_length
      character Cscale(2)*3, Cline*80, Cyes*5
c
      include 'prf.cmn'
      save iplot, name_len, Cscale
      data iplot, name_len / 1, 1 /
      data Cscale / 'lin', 'log' /
c
      name_len = max( 1, jljstr( dmp_names(DUMP_PDEF) ) )
c     start of menu loop                                                  1b
1         call newscr()
          abscis_ptr = max( 1, min( MAX_LIST,
     1        prf_def(iplot,PP_ABSCISSA) ) )
          abscis_len = PL_FMT_LEN + max( 1, str_length(
     1        cprf_def(abscis_ptr,PL_VAR_DES)(PL_FMT_LEN+1:) ) )
          prf_def(iplot,PP_X_SCALE) = max( 1, min( 2,
     1        prf_def(iplot,PP_X_SCALE) ) )
          ordin_ptr = max( 1, min( MAX_LIST,
     1        prf_def(iplot,PP_ORDINATE) ) )
          ordin_len = PL_FMT_LEN + max( 1, str_length(
     1        cprf_def(ordin_ptr,PL_VAR_DES)(PL_FMT_LEN+1:) ) )
          prf_def(iplot,PP_Y_SCALE) = max( 1, min( 2,
     1        prf_def(iplot,PP_Y_SCALE) ) )
          write( STD_OUT, 2 ) n_plots, iplot,
     1        mod( iplot, max( n_plots, 1 ) ) + 1,
     2        prf_def(iplot,PP_ABSCISSA),
     3        cprf_def(abscis_ptr,PL_VAR_DES)(PL_FMT_LEN+1:abscis_len),
     4        prf_def(iplot,PP_ORDINATE),
     5        cprf_def(ordin_ptr,PL_VAR_DES)(PL_FMT_LEN+1:ordin_len),
     6        cprf_def(iplot,PP_CAPTURE),
     7        Cscale(prf_def(iplot,PP_X_SCALE)),
     8        Cscale(prf_def(iplot,PP_Y_SCALE)),
     9        Cyes(prf_def(iplot,PP_WAIT)),
     a        dmp_names(DUMP_PDEF)(:name_len)
2         format( ' PRF Define profile plots' / '  -1) Abandon' /
     Z        '   0) Exit' /
     1        '   1) Number of plots     ', i2 /
     2        '   2) Current plot number ', i2 /
     3        '   3) Next plot           ', i2 /
     4      '   4) Abscissa (column from profile listing) ', i3, 2x, a /
     5      '   5) Ordinate (column from profile listing) ', i3, 2x, a /
     6        '   6) Capture file name   ', a /
     7        '   7) X axis scaling      ', a /
     8        '   8) Y axis Scaling      ', a /
     9        '   9) Wait after plotting ', a /
     a        '  10) Delete plot' /
     b        '  11) Clear all plots' /
     c        '  12) Definition file name ', a /
     d        '  13) Read definition file' /
     e        '  14) Write definition file  ', $ )
          item = Jresp( STD_IN, -1, 14, MAX_TRIES )
c
c           ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, ' Are you sure ? ' ) ) then            3b
                  def_prf_plot = NO
                  return
                  end if                                                  3e
c
c           EXIT
c
          else if ( item .eq. 0 ) then                                    2m
              ok = n_plots .gt. 0
              do 3 i = 1, n_plots                                         3b
                  ok = ok .and.
     1                0 .lt. prf_def(i,PP_ABSCISSA) .and.
     2                       prf_def(i,PP_ABSCISSA) .le. n_list .and.
     3                0 .lt. prf_def(i,PP_ORDINATE) .and.
     4                       prf_def(i,PP_ORDINATE) .le. n_list
3                 continue                                                3e
              if ( ok ) then                                              3b
                  def_prf_plot = YES
              else                                                        3m
                  def_prf_plot = NO
                  write( STD_ERR, 4 )
4                 format( ' PRF : ABORT : Incomplete or incomplete ',
     1                'profile plot definition(s)' )
                  call ilgwait()
                  end if                                                  3e
              return
c
c           NUMBER OF PLOTS
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' What value ? ', $ )
              n_plots = Jresp( STD_IN, 0, MAX_LIST, MAX_TRIES )
              iplot = min( iplot, n_plots )
c
c           CLEAR ALL PLOTS
c
          else if ( item .eq. 11 ) then                                   2m
              n_plots = 0
c
c           DEFINITION FILE NAME
c
          else if ( item .eq. 12 ) then                                   2m
              write( STD_OUT, 5 )
              call Cresp( STD_IN, dmp_names(DUMP_PDEF), MAX_TRIES )
              name_len = max( 1, jljstr( dmp_names(DUMP_PDEF) ) )
c
c           READ DEFINITION FILE
c
          else if ( item .eq. 13 ) then                                   2m
              call open_dump( DUMP_PDEF )
              if ( dmp_units(DUMP_PDEF) .ne. 0 ) then                     3b
                  n_plots = 1
c                 start of loop                                           4b
6                     read( dmp_units(DUMP_PDEF), 7, ERR=13, END=9 )
     1                    Cline
7                     format( a )
                      if ( Cline(1:1) .eq. ';' .or.
     1                     Cline(1:2) .eq. ' ;' )
     2                    goto 6
                      read( Cline, 8, ERR=15 )
     1                    prf_def(n_plots,PP_ABSCISSA),
     2                    prf_def(n_plots,PP_ORDINATE),
     3                    prf_def(n_plots,PP_X_SCALE),
     4                    prf_def(n_plots,PP_Y_SCALE),
     5                    prf_def(n_plots,PP_WAIT),
     6                    cprf_def(n_plots,PP_CAPTURE)
8                     format( 2x, i3, 3x, i3, 3x, i3, 2x, i3, 2x, i3,
     1                        2x, a )
                      n_plots = n_plots + 1
                      goto 6
c                     end of loop                                         4e
9                 call close_dump( DUMP_PDEF )
                  n_plots = n_plots - 1
                  end if                                                  3e
c
c           CURRENT PLOT NUMBER
c
          else if ( n_plots .gt. 0 ) then                                 2m
              if ( item .eq. 2 ) then                                     3m
                  write( STD_OUT, 5 )
                  iplot = Jresp( STD_IN, 1, n_plots, MAX_TRIES )
c
c           NEXT PLOT
c
              else if ( item .eq. 3 ) then                                3m
                  iplot = mod( iplot, n_plots ) + 1
c
c           ABSCISSA (COLUMN FROM PROFILE LISTING)
c
              else if ( item .eq. 4 ) then                                3m
                  write( STD_OUT, 10 )
10                format( ' Enter column number from profile listing ',
     1                $ )
                  prf_def(iplot,PP_ABSCISSA) =
     1                Jresp( STD_IN, 1, n_list, MAX_TRIES )
c
c           ORDINATE (COLUMN FROM PROFILE LISTING)
c
              else if ( item .eq. 5 ) then                                3m
                  write( STD_OUT, 10 )
                  prf_def(iplot,PP_ORDINATE) =
     1                Jresp( STD_IN, 1, n_list, MAX_TRIES )
c
c           CAPTURE FILE NAME
c
              else if ( item .eq. 6 ) then                                3m
                  write( STD_OUT, 5 )
                  call Cresp( STD_IN, cprf_def(iplot,PP_CAPTURE),
     1                MAX_TRIES )
c
c           X AXIS SCALING
c
              else if ( item .eq. 7 ) then                                3m
                  prf_def(iplot,PP_X_SCALE) =
     1                mod( prf_def(iplot,PP_X_SCALE), 2 ) + 1
c
c           Y AXIS SCALING
c
              else if ( item .eq. 8 ) then                                3m
                  prf_def(iplot,PP_Y_SCALE) =
     1                mod( prf_def(iplot,PP_Y_SCALE), 2 ) + 1
c
c           WAIT AFTER PLOTTING
c
              else if ( item .eq. 9 ) then                                3m
                  prf_def(iplot,PP_WAIT) =
     1                mod( prf_def(iplot,PP_WAIT), 2 ) + 1
c
c           DELETE PLOT
c
              else if ( item .eq. 10 ) then                               3m
                  do 11 i = iplot, n_plots                                4b
                      prf_def(i,PP_ABSCISSA) = prf_def(i+1,PP_ABSCISSA)
                      prf_def(i,PP_ORDINATE) = prf_def(i+1,PP_ORDINATE)
                      prf_def(i,PP_WAIT)     = prf_def(i+1,PP_WAIT)
                      prf_def(i,PP_X_SCALE)  = prf_def(i+1,PP_X_SCALE)
                      prf_def(i,PP_Y_SCALE)  = prf_def(i+1,PP_Y_SCALE)
                      cprf_def(i,PP_CAPTURE) = cprf_def(i+1,PP_CAPTURE)
11                    continue                                            4e
                  n_plots = n_plots - 1
c
c           WRITE DEFINITION FILE
c
              else if ( item .eq. 14 ) then                               3m
                  call open_dump( DUMP_PDEF )
                  if ( dmp_units(DUMP_PDEF) .ne. 0 ) then                 4b
                      write( dmp_units(DUMP_PDEF), 12, ERR=13 )
12                    format( '; Profile plot definition file' /
     1                    ';abscis ordin x_sc y_sc wait capture_file' )
                      write( dmp_units(DUMP_PDEF), 8, ERR=13 )  (
     1                    prf_def(i,PP_ABSCISSA),prf_def(i,PP_ORDINATE),
     2                    prf_def(i,PP_X_SCALE), prf_def(i,PP_Y_SCALE),
     3                    prf_def(i,PP_WAIT),    cprf_def(i,PP_CAPTURE),
     4                    i = 1, n_plots )
                      call close_dump( DUMP_PDEF )
                      end if                                              4e
                  end if                                                  3e
              end if                                                      2e
          goto 1
c
c           ERROR MESSAGES
c
13        write( STD_ERR, 14 ) dmp_names(DUMP_PDEF)
14        format( ' PRF: ABORT: Cannot read/write definition file ', a )
          goto 17
c
15        write( STD_ERR, 16 ) dmp_names(DUMP_PDEF)
16        format( ' PRF : ABORT : Cannot decode definition file ', a )
17        call close_dump( DUMP_PDEF )
          goto 1
c         end of menu loop                                                1e
      end




      SUBROUTINE GEN_PRF_PLOT( TOTAL_SEGS )
c
c Generate profile plots
c
c Arguments
c  integer total_segs # total number of data segments in profile
c
c
c local variables
c  integer
c   abscis_var    # column of result array from which the abscissa is taken
c   abscis_ptr    # column of profile list which defines the abscissa
c   i             # looping index for data points
c   iplot         # plot number
c   ordin_var     # column of result array from which the ordinate is taken
c   ordin_ptr     # column of profile list which defines the ordinate
c  real
c   abscissa(MAX_POINTS)  # array of abscissa values
c   ordinate(MAX_POINTS)  # array of ordinate values
c  character
c   Cscale(2)*3   # 'lin', 'log'
c  subroutine
c   ilgclos, ilgdata, ilgline, ilgopen, ilgpage
c   ilgraph, ilgslaw, ilgwait, ilgxlab, ilgylab
c
c global variables
c   n_points, result
c
      include 'stdio.def'
      include 'prf.def'
c
      integer total_segs, abscis_var, abscis_ptr, i, iplot, ordin_var,
     1    ordin_ptr
      real abscissa(MAX_POINTS), ordinate(MAX_POINTS)
      character Cscale(2)*3
c
      include 'prf.cmn'
      save Cscale
      data Cscale / 'lin', 'log' /
c
c
      do 2 iplot = 1, n_plots                                             1b
          abscis_ptr = prf_def(iplot,PP_ABSCISSA)
          abscis_var = prf_def(abscis_ptr,PL_VAR_ID)
          ordin_ptr = prf_def(iplot,PP_ORDINATE)
          ordin_var = prf_def(ordin_ptr,PL_VAR_ID)
          do 1 i = 1, total_segs                                          2b
              abscissa(i) = norm_factor(abscis_ptr) *
     1            result(i,abscis_var)
              ordinate(i) = norm_factor(ordin_ptr) *
     1            result(i,ordin_var)
1             continue                                                    2e
c
c           initialize plotting
c
          if ( cprf_def(iplot,PP_CAPTURE) .ne. ' ' .or.
     1            iplot .le. 1 ) then                                     2b
              call ilgopen( cprf_def(iplot,PP_CAPTURE) )
              call ilgpage( 'a4', 'wide', 1, 1 )
              call ilgraph( 1, 1, 11, 11, 1 )
              call ilgline( 5, 0, 1 )
              end if                                                      2e
c
c           set the scaling, draw the data & the labels
c
          call ilgslaw( Cscale( prf_def(iplot,PP_X_SCALE) ),
     1                  Cscale( prf_def(iplot,PP_Y_SCALE) ) )
          call ilgdata( abscissa, ordinate, 1, total_segs, 1 )
          call ilgxlab( cprf_def(abscis_ptr,PL_VAR_DES)(PL_FMT_LEN+1:) )
          call ilgylab( cprf_def(ordin_ptr,PL_VAR_DES)(PL_FMT_LEN+1:) )
          if ( prf_def(iplot,PP_WAIT) .eq. YES )
     1        call ilgwait()
2         continue                                                        1e
      call ilgclos()
      return
      end



      SUBROUTINE GEN_PRF_LIST( TOTAL_SEGS )
c
c Generate profile list
c
c Arguments
c  integer   total_segs     # total number of data segments in profile
c
c local variables
c  integer
c   col         # column number in the table
c   fmt_end     # index of end of format descriptor in the format string
c   fmt_len     # length of format descriptor (up to 5)
c   fmt_ptr     # index of start of format descriptor in the format string
c   row         # rown number in the table
c   jljstr      # function : left justify string
c  character
c   fmt_str*120 # format string for all columns
c  subroutine
c   open_dump   # open dump file
c
c  global variables
c   dmp_names, dmp_units, n_profiles, result
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer total_segs, col, fmt_end, fmt_len, fmt_ptr, row, jljstr
      character fmt_str*120
c
      include 'prf.cmn'
c
c
      call open_dump( DUMP_PROF )
      if ( dmp_units(DUMP_PROF) .ne. 0 ) then                             1b
c
c       make format string
c       append format descriptors to the format string
c
          fmt_str = '('
          fmt_ptr = 2
          do 1 col = 1, n_list                                            2b
              fmt_len = jljstr( cprf_def(col,PL_VAR_DES)(1:PL_FMT_LEN) )
              fmt_end = fmt_ptr + fmt_len
              fmt_str(fmt_ptr:fmt_end-1) =
     1              cprf_def(col,PL_VAR_DES)(1:fmt_len)
              fmt_str(fmt_end:fmt_end) = ','
              fmt_ptr = fmt_end + 1
1             continue                                                    2e
          fmt_str(fmt_end:fmt_end) = ')'
c
c       print header
c
          write( dmp_units(DUMP_PROF), 2 ) n_profiles, ( col,
     1        cprf_def(col,PL_VAR_DES)(PL_FMT_LEN+1:), col = 1, n_list )
2         format( ' ; ' /
     1        ' ; PRF : listing of profile ', i3, ' containing' /
     2        ' ; col    contents' /
     3        ( ' ; ', i3, 3x, a ) )
c
c       print each line of the table
c
          do 3 row = 1, total_segs                                        2b
              write( dmp_units(DUMP_PROF), fmt_str, ERR=4 )
     1            ( norm_factor(col) *
     2              result( row, prf_def( col, PL_VAR_ID) ),
     3              col = 1, n_list )
3             continue                                                    2e
c
c       close the dump file
c
          call close_dump( DUMP_PROF )
          end if                                                          1e
      return
c
c       error writing to file
c
4     write( STD_ERR, 5 ) dmp_names(DUMP_PROF)
5     format( ' PRF : ABORT : Cannot write profile list to file ', a )
      call close_dump( DUMP_PROF )
      return
      end



      INTEGER FUNCTION DEF_PRF_LIST()
c
c  Define profile listing
c
c local variables
c  integer
c   descr_len     # length of normalised variable description
c   i             # looping index for column number
c   icol          # column number for profile listing
c   item          # selected menu item
c   name_len      # length of file name
c   var_ptr       # pointer to variable in result
c   jljstr        # function : left justify string
c   Jresp         #          : read integer
c   sel_norm_factor         #: select normalizing variables
c   str_length    #          : non-blank length of string
c  character
c   Cline         # character string read from STD_IN or definition file
c   Cvar_frmt(MAX_RESULT)*7 # default format descriptors for result variables
c   Cvar_name(MAX_RESULT)*5 # names of result variables
c  logical
c   ok            # analogue of returned result
c   Equery        # function : read logical
c  real
c   make_norm_factor  # function : calculate normalising factor from the code
c  subroutine
c   Cresp       # f77_ute : read string
c   ilgwait     # ilglib  : wait for CR
c   open_dump   # open dump file
c   newscr      # hpprev  : clear screen
c
c global variabes
c   dmp_units, dmp_names
c
c
      include 'stdio.def'
      include 'prf.def'
      include 'prf.cmn'
c
      logical ok, Equery
      integer descr_len, i, icol, item, name_len, var_ptr,
     1    jljstr, Jresp, sel_norm_factor, str_length
      real make_norm_factor
      character Cline*80, Cvar_frmt(MAX_RESULT)*7,
     1    Cvar_name(MAX_RESULT)*5
c
      save icol, name_len, Cvar_name, Cvar_frmt
      data icol, name_len / 1, 1 /
      data Cvar_name /
     1    'Y_req', 'Y_act', 'mean ', 'vari ', 'mom 3',
     2    'mom 4', 'mom 5', 'mom 6', 'wake ', 'r.m.s',
     3   'u''/U ', 'skew ', 'flat ', 'sskew', 'sflat' /
      data Cvar_frmt /
     1    'f11.6  ', 'f11.6  ', 'f10.4  ', 'g12.5e1', 'g12.5e1',
     2    'g12.4  ', 'g12.4  ', 'g12.4  ', 'f10.5  ', 'f9.5   ',
     3    'f8.5   ', 'f8.4   ', 'f7.4   ', 'f7.2   ', 'f7.2   ' /
c
c
      name_len = max( 1, jljstr( dmp_names(DUMP_LDEF) ) )
c     start of menu loop                                                  1b
1         call newscr()
          var_ptr = prf_def(icol,PL_VAR_ID)
          if ( var_ptr .le. 0 .or. var_ptr .gt. MAX_RESULT )
     1        var_ptr = 1
          descr_len = PL_FMT_LEN + max( 1, str_length(
     1        Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+1:) ) )
          write( STD_OUT, 2 ) n_list, icol,
     1        mod( icol, max( n_list, 1 ) ) + 1,
     2        prf_def(icol,PL_VAR_ID), Cvar_name(var_ptr),
     3        Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+1:descr_len),
     4        norm_factor(icol),
     5        Cprf_def(icol,PL_VAR_DES)(1:PL_FMT_LEN),
     6        dmp_names(DUMP_LDEF)(:name_len)
2         format( ' PRF Define profile listing' / '  -1) Abandon' /
     Z        '   0) Exit' /
     1        '   1) Number of table columns  ', i2 /
     2        '   2) Current column number    ', i2 /
     3        '   3) Next column              ', i2 /
     4        '   4) Select variable          ', i3, 2x, a /
     5        '                  description  ', a /
     6        '   5) Select normalizing variables ' /
     7        '                        value  ', g12.5 /
     8        '   6) Change format descriptor ', a /
     9        '   7) Delete column' /
     a        '   8) Clear all columns' /
     b        '  10) Definition file name ', a /
     c        '  11) Read definition file' /
     d        '  12) Write definition file ', $ )
          item = Jresp( STD_IN, -1, 12, MAX_TRIES )
c
c           ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, ' Are you sure ? ' ) ) then            3b
                  def_prf_list = NO
                  return
                  end if                                                  3e
c
c           EXIT
c
          else if ( item .eq. 0 ) then                                    2m
              ok = n_list .gt. 0
              do 3 i = 1, n_list                                          3b
                  ok = ok .and.
     1                0 .lt. prf_def(i,PL_VAR_ID) .and.
     2                0 .lt. norm_factor(i) .and.
     3                Cprf_def(i,PL_VAR_DES)(1:5) .ne. ' '
3                 continue                                                3e
              if ( ok ) then                                              3b
                  def_prf_list = YES
              else                                                        3m
                  def_prf_list = NO
                  write( STD_ERR, 4 )
4                 format( ' PRF : ABORT : Incomplete or incomplete ',
     1                'profile plot definition(s)' )
                  call ilgwait()
                  end if                                                  3e
              return
c
c           NUMBER OF TABLE COLUMNS
c
          else if ( item .eq. 1 ) then                                    2m
              write( STD_OUT, 5 )
5             format( ' What value ? ', $ )
              n_list = Jresp( STD_IN, 0, MAX_LIST, MAX_TRIES )
              icol = min( icol, n_list )
c
c           CLEAR ALL COLUMNS
c
          else if ( item .eq. 8 ) then                                    2m
              n_list = 0
c
c           DEFINITION FILE NAME
c
          else if ( item .eq. 10 ) then                                   2m
              write( STD_OUT, 5 )
              call Cresp( STD_IN, dmp_names(DUMP_LDEF), MAX_TRIES )
              name_len = max( 1, jljstr( dmp_names(DUMP_LDEF) ) )
c
c           READ DEFINITION FILE
c
          else if ( item .eq. 11 ) then                                   2m
              call open_dump( DUMP_LDEF )
              if ( dmp_units(DUMP_LDEF) .ne. 0 ) then                     3b
                  n_list = 1
c                 start of loop                                           4b
6                     read( dmp_units(DUMP_LDEF), 7, ERR=14, END=9 )
     1                    Cline
7                     format( a )
                      if ( Cline(1:1) .eq. ';' .or.
     1                     Cline(1:2) .eq. ' ;' )
     1                    goto 6
                      read( Cline, 8, ERR=16 )
     1                    prf_def(n_list,PL_VAR_ID),
     2                    prf_def(n_list,PL_NORMCODE),
     3                    Cprf_def(n_list,PL_VAR_DES)
                      norm_factor(n_list) = make_norm_factor( n_list )
8                     format( 2x, i3, 3x, i11, 3x, a )
                      n_list = n_list + 1
                      goto 6
c                     end of loop                                         4e
9                 call close_dump( DUMP_LDEF )
                  n_list = n_list - 1
                  end if                                                  3e
c
c           CURRENT COLUMN NUMBER
c
c
          else if ( n_list .gt. 0 ) then                                  2m
              if ( item .eq. 2 ) then                                     3m
                  write( STD_OUT, 5 )
                  icol = Jresp( STD_IN, 1, n_list, MAX_TRIES )
c
c           NEXT COLUMN
c
              else if ( item .eq. 3 ) then                                3m
                  icol = mod( icol, n_list ) + 1
c
c           SELECT VARIABLE
c
              else if ( item .eq. 4 ) then                                3m
                  write( STD_OUT, 10 )
10                format( ' Which variable ?' /
     1                '  1) Required probe postion    6) 4th moment',
     2                     '       11) Turbulence level' /
     3                '  2) Actual probe position     7) 4th moment',
     4                     '       12) Skewness' /
     5                '  3) Mean air speed            8) 4th moment',
     6                     '       13) Flatness' /
     7                '  4) Mean square air speed     9) wake function',
     8                     '    14) Super skewness' /
     9                '  5) 3rd moment of air speed  10) Root mean ',
     a                     'square 15) Super flatness ', $ )
                  prf_def(icol,PL_VAR_ID) =
     1                Jresp( STD_IN, 1, MAX_RESULT, MAX_TRIES )
                  Cprf_def(icol,PL_VAR_DES)(1:PL_FMT_LEN) =
     1                Cvar_frmt(prf_def(icol,PL_VAR_ID))
                  Cprf_def(icol,PL_VAR_DES)
     1                (PL_FMT_LEN+1:PL_FMT_LEN+PL_DES_LEN) =
     2                Cvar_name(prf_def(icol,PL_VAR_ID))
c
c           SELECT NORMALIZING VARIABLES => value
c
              else if ( item .eq. 5 ) then                                3m
                  prf_def(icol,PL_NORMCODE) = sel_norm_factor( icol )
                  norm_factor(icol) = make_norm_factor( icol )
c
c           CHANGE FORMAT DESCRIPTOR
c
              else if ( item .eq. 6 ) then                                3m
                  write( STD_OUT, 11 ) PL_FMT_LEN
11                format( ' Enter new (FORTRAN) format descriptor ' /
     1                '  (e, f or g descriptor, maximum of ', i2,
     2                ' chars, blank=no change ) ',  $ )
                  call Cresp( STD_IN, Cline, MAX_TRIES )
                  if ( Cline(1:PL_FMT_LEN) .ne. ' ' )
     1                Cprf_def(icol,PL_VAR_DES)(1:PL_FMT_LEN) =
     2                    Cline(1:PL_FMT_LEN)
c
c           DELETE COLUMN
c
              else if ( item .eq. 7 ) then                                3m
                  do 12 i = icol, n_list                                  4b
                      prf_def(i,PL_VAR_ID)   = prf_def(i+1,PL_VAR_ID)
                      prf_def(i,PL_NORMCODE) = prf_def(i+1,PL_NORMCODE)
                      Cprf_def(i,PL_VAR_DES) = Cprf_def(i+1,PL_VAR_DES)
                      norm_factor(i) = norm_factor(i+1)
12                    continue                                            4e
                  n_list = n_list - 1
c
c           WRITE DEFINITION FILE
c
              else if ( item .eq. 12 ) then                               3m
                  call open_dump( DUMP_LDEF )
                  if ( dmp_units(DUMP_LDEF) .ne. 0 ) then                 4b
                      write( dmp_units(DUMP_LDEF), 13, ERR=14 )
13                    format( ' ; Profile list definition file' /
     1                    ' ;var-id   norm-code format & variable ',
     2                    'description' )
                      write( dmp_units(DUMP_LDEF), 8, ERR=14 )  (
     1                    prf_def(i,PL_VAR_ID), prf_def(i,PL_NORMCODE),
     2                    Cprf_def(i,PL_VAR_DES), i = 1, n_list )
                      call close_dump( DUMP_LDEF )
                      end if                                              4e
                  end if                                                  3e
              end if                                                      2e
          goto 1
c
c           ERROR MESSAGES
c
14        write( STD_ERR, 15 ) dmp_names(DUMP_LDEF)
15        format( ' PRF: ABORT: Cannot read/write definition file ', a )
          goto 18
c
16        write( STD_ERR, 17 ) dmp_names(DUMP_LDEF)
17        format( ' PRF : ABORT : Cannot decode definition file ', a )
18        call close_dump( DUMP_LDEF )
          goto 1
c         end of menu loop                                                1e
      end



      INTEGER FUNCTION SEL_NORM_FACTOR( ICOL )
c
c Selects the normalisation factor for column ICOL of the profile listing .
c The factor is respecified from the beginning rather than edited .
c
c Arguments
c  integer   icol                 # column number
c
c constants
c  MULTIPLY=1
c  DIVIDE  =2
c
c local variables
c  integer
c   action        # 1=MULTIPLY, 2=DIVIDE
c   code_digit    # code digit to indicate selected normalising property
c   descr_len     # length of description of normalising variable
c   fmt_ptr       # points to next free space in description string
c   item          # selected menu item
c   Jresp         # function : read integer
c   str_length    #          : non-blank length of string
c  logical
c   Equery        # function : read logical
c  character
c   Caction(2)*8  # 'multiply', 'divide'
c   Cfactor_names(MAX_NORM)*5 # list of normalising factor names
c
c
      include 'stdio.def'
      include 'prf.def'
      integer MULTIPLY, DIVIDE
      parameter( MULTIPLY=1, DIVIDE=2 )
c
      integer action, code_digit, descr_len, fmt_ptr, icol, item,
     1    Jresp, str_length
      logical Equery
      character Caction(2)*8, Cfactor_names(MAX_NORM)*5
c
      include 'prf.cmn'
      save action, Cfactor_names, Caction
      data action / 1 /
      data Cfactor_names /
     1  'U_inf', 'U_tau', 'delta', 'displ', 'theta', 'energ',
     2  'Coled', 'H_12 ', 'H_23 ', 'H_13 ', 'C_f  ', 'PI   ',
     3  'Re_bl', 'Re_ds', 'Re_th', 'delt+', 'visc ' /
      data Caction / 'multiply', 'divide  ' /
c
c
      Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+PL_DES_LEN+1:) = ' '
      fmt_ptr = PL_FMT_LEN + PL_DES_LEN + 1
      sel_norm_factor = 0
c
c     start of loop                                                       1b
1         call newscr()
          write( STD_OUT, 2 ) icol, Caction(action),
     1        Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+PL_DES_LEN+1:)
2         format( ' PRF Select normalising factors for list column ',
     N        i2 / '   -1) Abandon' /
     Z        '   0) Exit' /
     1        '   1) ', a /
     2        '   2) Add factor ', a /
     3        '   3) Clear factors ', $ )
          item = Jresp( STD_IN, -1, 3, MAX_TRIES )
c
c           ABANDON
c
          if ( item .eq. -1 ) then                                        2b
              if ( Equery( STD_IN, 'Are you sure ? ' ) ) then             3b
                  sel_norm_factor = -1
                  Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+PL_DES_LEN+1:) =
     1                ' '
                  return
                  end if
c
c           EXIT
c
          else if ( item .eq. 0 ) then                                    2m
              if ( sel_norm_factor .le. 0 ) then
                  sel_norm_factor = 0
                  Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+PL_DES_LEN+1:) =
     1                ' '
                  end if
              return
c
c           MULTIPLY / DIVIDE
c
          else if ( item .eq. 1 ) then                                    2m
              action = mod( action, 2 ) + 1
c
c           ADD FACTOR
c
          else if ( item .eq. 2 ) then                                    2m
              write( STD_OUT, 3 )
3             format( ' Select factor to add' /
     1            ' 1) Free stream speed   7) Coles thickness       ',
     2                 '13) Reynolds_delta' /
     3            ' 2) Friction velocity   8) Shape factor          ',
     4                 '14) Reyn_displacement' /
     5            ' 3) Bound layer thick   9) Energy/momentum thick ',
     6                 '15) Reyn_momentum' /
     7            ' 4) Displacement thick 10) Energy/displacement   ',
     8                 '16) Delta^plus' /
     9            ' 5) Momentum thickness 11) Frict coefficient     ',
     a                 '17) Kinematic viscosity' /
     b            ' 6) Energy thickness   12) Coles wake strength PI ',
     c                 $ )
              code_digit = Jresp( STD_IN, 1, MAX_NORM, MAX_TRIES )
              descr_len = str_length( Cfactor_names(code_digit) )
              Cprf_def(icol,PL_VAR_DES)(fmt_ptr+1:fmt_ptr+descr_len) =
     1            Cfactor_names(code_digit)
              if ( action .eq. MULTIPLY ) then
                  Cprf_def(icol,PL_VAR_DES)(fmt_ptr:fmt_ptr) = '*'
              else
                  Cprf_def(icol,PL_VAR_DES)(fmt_ptr:fmt_ptr) = '/'
                  code_digit = code_digit + MAX_NORM
                  end if
              sel_norm_factor = sel_norm_factor * (2*MAX_NORM+1) +
     1            code_digit
              fmt_ptr = fmt_ptr + descr_len + 1
c
c           CLEAR ALL FACTORS
c
          else if ( item .eq. 3 ) then                                    2m
              sel_norm_factor = 0
              Cprf_def(icol,PL_VAR_DES)(PL_FMT_LEN+PL_DES_LEN+1:) = ' '
              fmt_ptr = PL_FMT_LEN+PL_DES_LEN+1
              end if                                                      2e
          goto 1
c         end of menu loop                                                1e
      end



      REAL FUNCTION MAKE_NORM_FACTOR( ICOL )
c
c Evaluates the normalizing factor from the normalizing code .
c An error condition ( zero result) is produced if
c PRF_DEF(ICOL,PL_NORMCODE)<0 .
c
c Argument
c  integer icol   # profile listing coumn number
c
c local variables
c  logical
c   divide        # .TRUE.=divide, .FALSE.=multiply
c  integer
c   code_digit    # single code `digit'
c   code_number   # remainder of normalising code after extraction of a digit
c  real
c   new_factor    # new component of the normalising factor
c  character
c   var_type(MAX_RESULT)*2  # dimension codes for the result variables .
c       # One array element is used for each result variable according to
c       # REQUIRED_POS=1, ACTUAL_POS=2, MEAN_AVERAGE=3,  MEAN_VARIANCE=4,
c       # MEAN_WAKE=9,    MEAN_RMS=10,  MEAN_TURBUL=11,  MEAN_SKEW=12,
c       # MEAN_FLAT=13 etc.
c   norm_type(MAX_NORM)*2   # dimension codes for normalising quantities
c       # One array element is used for each result variable according to
c       # FREESTR_SPEED=1,  FRICT_VELOCITY=2, BOUND_L_THICK=3, DISPLAC_THICK=4,
c       # MOMENTUM_THICK=5, ENERGY_THICK=6,   COLES_THICK=7,   SHAPE_FACTOR=8,
c       # SHAPE_32=9,       SHAPE_31=10,      FRICT_COEFF=11,  COLES_PI_WAKE=12,
c       # REYN_DELTA=13,    REYN_DISPL=14,    REYN_MOMENT=15,  DELTA_PLUS=16
c   The dimension codes are
c     1) 'L' = length
c     2) 'V' = velocity
c     3) digit = exponent; e.g. 'V3' is velocity cubed
c
c global variables
c   n_profiles, profile
c
c
      include 'stdio.def'
      include 'prf.def'
c
      logical divide
      integer code_digit, code_number, icol
      real new_factor
      character var_type(MAX_RESULT)*2, norm_type(MAX_NORM)*2
c
      include 'prf.cmn'
      save var_type, norm_type
      data var_type  / 'L ', 'L ', 'V ', 'V2', 'V3', 'V4', 'V5', 'V6',
     1                 'V ', 'V ', '  ', '  ', '  ', '  ', '  ' /
      data norm_type / 'V ', 'V ', 'L ', 'L ', 'L ', 'L ', 'L ', '  ',
     1           '  ', '  ', '  ', '  ', '  ', '  ', '  ', '  ', 'VL' /
c
c
      if ( prf_def(icol,PL_NORMCODE) .lt. 0 ) then                                        1b
          make_norm_factor = 0.0
      else                                                                1m
          code_number = prf_def(icol,PL_NORMCODE)
c
c       if the selected variable is a linear measurement (distance from wall),
c       and it is normalised (by anything) i.e. code_number .ne. 0 then
c       it should be divided by 1000 .
c
          if ( code_number .ne. 0 .and.
     1         var_type(prf_def(icol,PL_VAR_ID)) .eq. 'L ' ) then         2b
              make_norm_factor = 0.001
          else                                                            2m
              make_norm_factor = 1.0
              end if                                                      2e
c
c       Extract the next code digit using modulo-remainder and dividing
c       by the base until nothing is left .
c
c         start of loop                                                   2b
1         if ( code_number .gt. 0 ) then
              code_digit = mod( code_number, 2*MAX_NORM+1 )
              divide = code_digit .gt. MAX_NORM
              if ( divide )
     1            code_digit = code_digit - MAX_NORM
c
c       calculate new component of normalizing factor
c       not forgetting that dimensioned lengths are in mm
c
              if ( 1 .le. code_digit .and.
     1                    code_digit .le. MAX_PROPERTIES ) then           3b
                  new_factor = profile( n_profiles, code_digit )
                  if ( norm_type(code_digit) .eq. 'L ' )
     1                new_factor = new_factor * 0.001
              else if ( code_digit .eq. MAX_PROPERTIES + 1 ) then         3m
                  new_factor = air_viscosity
              else                                                        3m
                  write( STD_ERR, 2 )
2                 format( ' MAKE_NORM : BUG : Cannot decipher ',
     1                'normalising code' )
                  new_factor = 1.0
                  end if                                                  3e
c
c       bale out if the normalising factor is zero
c
              if ( new_factor .eq. 0.0 ) then
                  make_norm_factor = 1.0
                  write( STD_ERR, 3 ) cprf_def( icol, PL_VAR_DES )(6:)
3                 format( ' MAKE_NORM : ABORT : Zero normalising ',
     1                'factor for ', a )
                  return
              else
c
c       divide by profile property
c
                  if ( divide ) then                                          3b
                      make_norm_factor = make_norm_factor / new_factor
c
c       multiply by profile property
c
                  else                                                        3m
                      make_norm_factor = make_norm_factor * new_factor
                      end if                                                  3e
                  end if
c
c       expose the next code digit
c
              code_number = code_number / ( 2*MAX_NORM+1)
              goto 1
              end if                                                      2e
c             end of loop
          end if                                                          1e
      return
      end



      SUBROUTINE OPEN_DUMP( DUMP_NO )
c
c Open the dump file with index DUMP_NO into the dump table arrays .
c The returned value is the logical unit number of the dump file .
c
c Argument
c  integer dump_no    # index into the dump table arrays
c
c The dump table arrays are
c  dmp_reserv(MAX_DUMPS)    # list of reserved logical unit numbers
c  dmp_units(MAX_DUMPS)     # actual logical unit numbers of dump files
c  dmp_names(MAX_DUMPS)*32  # names of dump_files
c The permitted indices into these arrays are (see prf.def for details)
c  DUMP_AUXL, DUMP_PLOT, DUMP_HIST, DUMP_PROB, DUMP_AUTO, DUMP_SPEC
c
c If the file name is 'aux' the dump is sent to the auxillary file . If the file
c name is 'vdu' the dump is sent to the screen . If the file name is 'plt' the
c dump is sent to the plot dump file .
c
c local variables
c  integer
c   i             # looping index
c   new_lun       # function : get new logical unit number
c   open_file     #          : open file
c  character
c   lower_case*3  # first 3 chars of file name converted lower case
c
c global variables
c   res_dump_lu
c
c
      include 'stdio.def'
      include 'prf.def'
c
      integer dump_no, i, new_lun, open_file
      character lower_case*3
      include 'prf.cmn'
c
c       Check for illegal argument
c
      if ( dump_no .ne. DUMP_AUXL .and. dump_no .ne. DUMP_PLOT .and.
     1     dump_no .ne. DUMP_HIST .and. dump_no .ne. DUMP_PROB .and.
     2     dump_no .ne. DUMP_AUTO .and. dump_no .ne. DUMP_SPEC .and.
     3     dump_no .ne. DUMP_GENL .and. dump_no .ne. DUMP_PDEF .and.
     4     dump_no .ne. DUMP_LDEF .and. dump_no .ne. DUMP_PROF ) then     1b
          write( STD_ERR, 1 )
1         format( ' OPEN_DUMP : BUG : Illegal index to dump array.' )
      else                                                                1m
c
c       Auxillary file
c
          call scase( dmp_names(dump_no)(1:3), lower_case, 'l' )
          if ( lower_case .eq. 'aux' .and. dump_no .ne. DUMP_AUXL ) then  2b
              dmp_units(dump_no) = dmp_units(DUMP_AUXL)
              if ( dmp_units(DUMP_AUXL) .ne. 0 ) then                     3b
                  dmp_names(dump_no) = dmp_names(DUMP_AUXL)
              else                                                        3m
                  write( STD_ERR, 2 )
2                 format( ' PRF : Cannot dump to non-existent ',
     1                    'auxillary file.' )
                  end if                                                  3e
c
c       Plot file
c
          else if ( lower_case .eq. 'plt' .and. dump_no .ne. DUMP_PLOT )
     1            then                                                    2m
              dmp_units(dump_no) = dmp_units(DUMP_PLOT)
              if ( dmp_units(dump_no) .ne. 0 ) then                       3b
                  dmp_names(dump_no) = dmp_names(DUMP_PLOT)
              else                                                        3m
                  write( STD_ERR, 3 )
3                 format( ' PRF : Cannot dump to non-existent ',
     1                    'plot file.' )
                  end if                                                  3e
c
c       Screen
c
          else if ( lower_case .eq. 'vdu' ) then                          2m
              dmp_units(dump_no) = STD_OUT
          else                                                            2m
c
c       Check if the dump file has already been opened . If so, return
c       the logical unit of the already opened file .
c
              do 4 i = 1, MAX_DUMPS                                       3b
                  if ( i .ne. dump_no .and. dmp_units(i) .ne. 0 ) then    4b
                      if ( dmp_names(dump_no) .eq. dmp_names(i) ) then    5b
                          dmp_units(dump_no) = dmp_units(i)
                          return
                          end if                                          5e
                      end if                                              4e
4                 continue                                                3e
c
c       If we get this far, the file is new
c
              if ( dmp_reserv(dump_no) .eq. 0 )
     1            dmp_reserv(dump_no) = new_lun()
              dmp_units(dump_no) = open_file( dmp_reserv(dump_no),
     1            dmp_names(dump_no), 'unknown', 0 )
              if ( dmp_units(dump_no) .ne. 0 )
     1            write( STD_OUT, 5 ) dmp_names(dump_no)
5                 format( ' PRF : NOTE : File opened : ', a )
              end if                                                      2e
          end if                                                          1e
      return
      end



      SUBROUTINE CLOSE_DUMP( DUMP_NO )
c
c Close dump file number DUMP_NO .
c
      include 'stdio.def'
      include 'prf.def'
c
      integer dump_no, i
      include 'prf.cmn'
c
c
c       Check for illegal argument
c
      if ( dump_no .ne. DUMP_AUXL .and. dump_no .ne. DUMP_PLOT .and.
     1     dump_no .ne. DUMP_HIST .and. dump_no .ne. DUMP_PROB .and.
     2     dump_no .ne. DUMP_AUTO .and. dump_no .ne. DUMP_SPEC .and.
     3     dump_no .ne. DUMP_GENL .and. dump_no .ne. DUMP_PDEF .and.
     4     dump_no .ne. DUMP_LDEF .and. dump_no .ne. DUMP_PROF ) then     1b
          write( STD_ERR, 1 )
1         format( ' CLOSE_DUMP : BUG : Illegal index to dump array.' )
c
c       Only proceed if the file is opened
c
      else if ( dmp_units(dump_no) .ne. 0 ) then                          1m
c
c       Check if the dump file has already been opened . If so, do a
c       dummy close .
c
          do 4 i = 1, MAX_DUMPS                                           2b
              if ( i .ne. dump_no .and. dmp_units(i) .ne. 0 ) then        3b
                  if ( dmp_names(dump_no) .eq. dmp_names(i) ) then        4b
                      dmp_units(dump_no) = 0
                      return
                      end if                                              4e
                  end if                                                  3e
4             continue                                                    2e
c
c       If we get this far, we must really close the file
c
          if ( dmp_units(dump_no) .ne. 0 ) then                           2b
              if ( dmp_units(dump_no) .eq. STD_OUT ) then                 3b
                  call ilgwait()
              else                                                        3m
                  close( UNIT=dmp_units(dump_no) )
                  write( STD_OUT, 5 ) dmp_names(dump_no)
5                 format( ' PRF : NOTE : File closed : ', a )
                  end if                                                  3e
              dmp_units(dump_no) = 0
              end if                                                      2e
          end if                                                          1e
      return
      end



      SUBROUTINE DUMP_HEADER( DUMP_LU, SEG_NO, LABEL )
c
c Writes a header onto the dump file .
c
c Arguments
c  integer dump_lu,     # logical unit number of the dump file
c          seg_no       # data segment number
c  character label*(*)  # label/title
c
c global variables
c   n_points, n_profiles, result, seg_no

c
      include 'stdio.def'
      include 'prf.def'
c
      integer dump_lu, seg_no
      character label*(*)
      include 'prf.cmn'
c
      write( dump_lu, 1 ) label, n_profiles, n_points,
     1    result(irow,ACTUAL_POS), seg_no
1     format( ' ;   ', a /
     1    ' ; ============================' /
     2    ' ;  Profile number        ', i3 /
     3    ' ;  Probe position number ', i3 /
     4    ' ;  Distance from wall    ', f7.3 /
     5    ' ;  Segment number        ', i3 )
      return
      end



      SUBROUTINE ROTATE_VECTOR( ARRAY, START, LENGTH )
c
c Rotate the (START+1)th to LENGTHth elements of the array ARRAY toward
c the start of the array by one element . The STARTth element is placed
c at the end of the array (at the LENGTHth element) .
c
c Arguments
c  integer array(*),  # operand array
c          start,     # index for start of rotation
c          length     # length of array
c
c local variables
c  integer
c   i     # looping index for stepping through the array
c   temp  # temporary storage
c
c
      include 'stdio.def'
      integer array(*), start, length, i, temp
c
      if ( start .lt. length ) then                                       1b
          temp = array(start)
          do 1 i = start+1, length                                        2b
              array(i-1) = array(i)
1             continue                                                    2e
          array(length) = temp
          end if                                                          1e
      return
      end



      INTEGER FUNCTION CHECK_SEG_LEN( SEGMENT_NO, START_INDEX,
     1                                FUNCT_NAME )
c
c Checks that the segment number SEGMENT_NO starting at data stream index
c START_INDEX does not extend beyond the available data . The returned value
c is a data segment length which does not exceed the bounds of the data
c stream . If the number of available data samples is not at least MIN_SEG_LEN
c the returned value is 0 . If the available data is less than ``seg_len'',
c the returned value is clipped .
c
c Arguments
c  integer segment_no       # data segment number
c          start_index      # start index for the data stream
c  character funct_name*(*) # name of calling function
c
c local variables
c  integer
c   actual_len    # maximum length of data segment for the available data
c
c  global variables
c   n_samples, seg_len, seg_increm
c
c
      include 'stdio.def/list'
      include 'prf.def/list'
c
      integer segment_no, start_index, actual_len
      character funct_name*(*)
      include 'prf.cmn/list'
c
      actual_len = ( n_samples - start_index + 1 ) / seg_increm
      if ( actual_len .lt. MIN_SEG_LEN ) then
          check_seg_len = 0
          write( STD_OUT, 1 )  segment_no, funct_name
1         format( ' PRF : ABORT : Cannot process segment ', i4,
     1        ' in routine ', a, '. Ran out of data.' )
      else if ( actual_len .lt. seg_len ) then
          check_seg_len = actual_len
          write( STD_OUT, 2 )  segment_no, actual_len, funct_name
2         format( ' PRF : WARNING : Data segment ', i4, ' truncated ',
     1        'to length ', i6, ' insufficient data for ', a )
      else
          check_seg_len = seg_len
          end if
      return
      end



      BLOCK DATA PRF_BLOCK
c
c       initialise common block data
c
      include 'stdio.def'
      include 'prf.def'
      include 'prf.cmn'
c
c  ADMINISTRATION
      data bar_pts, got_bar_cal, got_do, got_hw1_cal, host_type,
     1    hw1_method, plug_no, setdo_posn, sp_rep_enab, verbosity /
     2    0, NO, NO, NO, 2, 0, -1, NO, NO, 1 /
      data trip_diam / -1.0 /
c
c  DUMP FILE HANDLING
      data dmp_reserv, dmp_units / MAX_DUMPS*0, MAX_DUMPS*0 /
      data dmp_names / MAX_DUMPS*' ' /
c
c  INDICES FOR RESULTS
      data irow, n_points, n_profiles, n_samples /
     1    1, 1, 1, 0 /
c
c  DATA STREAM SEGMENTATION PARAMETERS
      data n_segs, seg_increm, seg_len, seg_start, profile_pt,
     1    profile_seg /
     2    1, 1, MIN_SEG_LEN, 1, MAX_POINTS*0, MAX_POINTS*0 /
c
c  FIR FILTER PARAMETERS
      data filter_do, filter_desc, n_filters /
     2    NO, MAX_FILTERS*QUERY, MAX_FILTERS*2, MAX_FILTERS*NO,
     3        MAX_FILTERS*YES,   MAX_FILTERS*0, MAX_FILTERS*0,  0 /
      data filter_file / MAX_FILTERS*' ' /
c
c  WALL PROXIMITY CORRECTION PARAMETERS
      data wall_do, wall_method, wall_max, wall_min /
     1    NO, VELOCITY_ERROR, 0, 0 /
      data Wall_file / ' ' /
c
c  TIME HISTORY PARAMETERS
      data hist_do, hist_max, hist_min, hist_plot, hist_wait /
     1    NO, 0, 0, YES, YES /
      data hist_capt / ' ' /
c
c  MEANS & STATISTICAL MOMENTS PARAMETERS
      data means_do, means_max, means_min, means_moment /
     1    NO, 0, 0, 4 /
c
c  PROBABILITY DENSITY FUNCTION PARAMETERS
      data prob_do, prob_bins, prob_max, prob_min, prob_plot,
     1    prob_wait, prob_test, prob_moment /
     2    NO, 0, 0, 0, YES, YES, NO, 0 /
      data prob_capt / ' ' /
c
c  AUTOCORRELATION PARAMETERS
      data auto_do, auto_lags, auto_method, auto_max, auto_min,
     1    auto_plot, auto_wait /
     2    NO, 0, 1, 0, 0, YES, YES /
      data auto_capt / ' ' /
c
c  POWER SPECTRAL DENSITY PARAMETERS
      data spec_do, spec_max, spec_min, spec_nfft, spec_plot, spec_wait
     1    / NO, 0, 0, 8, YES, YES /
      data  spec_capt, spec_xlaw, spec_ylaw, spec_window /
     1    ' ', 'lin', 'lin', 'h  ' /
c
c  DESCRIPTION OF PROFILE LISTING AND PROFILE PLOTS
      data n_list, n_plots / 0, 0 /
      data norm_factor / MAX_LIST*0.0 /
c
c  EXPERIMENTAL DATA
      data air_density, air_pressure, air_temp, air_viscosity /
     1    1.204, 101325.0, 293.0, 1.5e-5 /
c
c  PROFILE LISTING AND PLOT DEFINITIONS
      data cprf_def / MAX_LIST*' ', MAX_LIST*' ' /
      end

