c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c

c               BOUNDARY-LAYER MODEL
c    Modelling the mean velocity distribution in a turbulent boundary layer.
c    These routines are used by QU_TAU, which find the friction velocity
c    and other model parameters by a graphical/manual/automatic optimisation.
c

      REAL FUNCTION GET_BLM( WHAT )
c
c Get boundary layer model parameter WHAT which is a value selected from
c the file BLM.def . Any value in BLM.def can be retrieved except "BLM_reset"
c which is a no-op . All error conditions are indicated by a zero return
c value . These may arise because the the quantity is unavailable or because
c the friction velocity or kinematic viscosity which is required to convert a
c derived quantity to a dimensioned value is unavailable.
c
c Argument
c  integer what             # specify the value to get
c
c Variables
c  real logical_to_real     # function : convert logical to real
c  logical calc_wake        # wake component of boundary layer is calculated
c  character *24 name_BLM   # function : name of BLM parameter
c            fr_vel_visc_str*45       # function :
c
c
      include 'stdio.def'
      include 'blm.def'
      integer what
      logical calc_wake
      real logical_to_real
      character Cabort*31, name_BLM*24, fr_vel_visc_str*45
      include 'blm.cmn'
c
c       external forces block data to be included at link time
      external block_blm
      data Cabort / ' GET_BLM : ABORT : Unavailable ' /
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       Von Karmen constant, Kappa
c       **************************
c         -- slope of the log-law velocity profile
c         -- is the same for both dimensioned & nondimensional cases
c
      if ( what .eq. BLM_Karmen ) then                                    1b
          get_BLM = Kappa
c
c       vertical offset C of the log-law velocity profile
c       *************************************************
c         -- is the same for both dimensioned & nondimensional cases
c
      else if ( what .eq. BLM_Constant ) then                             1m
          get_BLM = Constant
c
c       kinematic viscosity
c       *******************
c         -- dimensioned value only ( metres squared / sec )
c
      else if ( what .eq. BLM_kin_visc ) then                             1m
          if ( got_visc ) then                                            2b
              get_BLM = kinem_visc
          else                                                            2m
              get_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_kin_visc ),
     1            error_str )
              end if                                                      2e
c
c       friction velocity
c       *****************
c         -- dimensioned value only ( metres / second )
c
      else if ( what .eq. BLM_friction_vel ) then                         1m
          if ( got_frict ) then                                           2b
              get_BLM = frict_vel
          else                                                            2m
              get_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_friction_vel ),
     1            error_str )
              end if                                                      2e
c
c       free stream speed
c       *****************
c         -- dimensioned value only ( metres / sec )
c
      else if ( what .eq. BLM_fs_speed ) then                             1m
          if ( got_fsv ) then                                             2b
              get_BLM = free_stream
          else                                                            2m
              get_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_fs_speed ),
     1            error_str )
              end if                                                      2e
c
c       Coles' PI wake strength
c       ***********************
c         -- dimensioned     -- metres / second
c         -- non-dimensional -- multiples of friction velocity
c
      else if ( what .eq. BLM_PI_wake ) then                              1m
          if ( got_PI ) then                                              2b
              if ( .not. dimensioned ) then                               3b
                  get_BLM = PI_wake
              else if ( got_frict ) then                                  3m
                  get_BLM = PI_wake * frict_vel
              else                                                        3m
                  get_BLM = 0.0
                  got_error = .TRUE.
                  call str_cat( Cabort, name_BLM( BLM_friction_vel ),
     1                error_str )
                  end if                                                  3e
          else                                                            2m
              get_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_PI_wake ), error_str )
              end if                                                      2e
c
c       Coles' delta "boundary layer thickness"
c       ***************************************
c         -- dimensioned     -- metres
c         -- non-dimensional -- multiples of Y_plus
c
      else if ( what .eq. BLM_delta ) then                                1m
          if ( got_delta ) then                                           2b
              if ( .not. dimensioned ) then                               3b
                  get_BLM = delta_plus
              else if ( got_frict .and. got_visc ) then                   3m
                  get_BLM = delta_plus * kinem_visc / frict_vel
              else                                                        3m
                  get_BLM = 0.0
                  got_error = .TRUE.
                  call str_cat( Cabort, fr_vel_visc_str(), error_str )
                  end if                                                  3e
          else                                                            2m
              get_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_delta ), error_str )
              end if                                                      2e
c
c       Parameters are dimensioned ?
c       ****************************
c
      else if ( what .eq. BLM_dimens ) then                               1m
          get_BLM = logical_to_real( dimensioned )
c
c       Parameters are non-dimensional ?
c       ********************************
c
      else if ( what .eq. BLM_nondim ) then                               1m
          get_BLM = logical_to_real( .not. dimensioned )
c
c       Are all the required parameters available ?
c       *******************************************
c       for calculating the mean velocity model
c
      else if ( what .eq. BLM_reqd_par ) then                             1m
          calc_wake = model .eq. BLM_68Coles .or. model .eq. BLM_MKBull
     1          .or. model .eq. BLM_BG_68C .or. model .eq. BLM_Spld_68C
          if ( .not. got_frict ) then
              get_BLM = 0.0
              got_error = .TRUE.
              error_str = name_BLM( BLM_friction_vel )
          else if ( calc_wake .and. .not. got_PI ) then
              get_BLM = 0.0
              got_error = .TRUE.
              error_str = name_BLM( BLM_PI_wake )
          else if ( calc_wake .and. .not. got_delta ) then
              get_BLM = 0.0
              got_error = .TRUE.
              error_str = name_BLM( BLM_delta )
          else if ( dimensioned .and. .not. got_frict ) then
              get_BLM = 0.0
              got_error = .TRUE.
              error_str = name_BLM( BLM_kin_visc )
          else
              got_error = .FALSE.
              get_BLM = 1.5
              end if
c
c       Returns the current BOUNDARY-LAYER MODEL
c       ****************************************
c
      else if ( what .eq. BLM_BLmodel ) then                              1m
          get_BLM = real( model ) + 0.5
c
c       Returns a "true" or "false" to verify or deny that WHAT defines
c       the selected model .
c       ********************
c
c BLM_Clauser           Clauser F.H (1954)
c BLM_56Coles           Coles D.E. (1956)
c BLM_68Coles           Coles D.E. (1968)
c BLM_MKBull            Bull M.K. (1969)
c BLM_Beattie           Beattie & Green (1983)
c BLM_BG_68C            Beattie & Green (1983) + Coles 1968 wake function
c BLM_Spalding          Spalding D.B. (1961)
c BLM_Spld_68C          Spalding D.B. (1961) + Coles 1968 wake function
c
      else if ( what .eq. BLM_Clauser .or. what .eq. BLM_56Coles .or.
     1      what .eq. BLM_68Coles .or. what .eq. BLM_MKBull .or.
     2      what .eq. BLM_Beattie .or. what .eq. BLM_BG_68C .or.
     3      what .eq. BLM_Spalding .or. what .eq. BLM_Spld_68C ) then     1m
          get_BLM = logical_to_real( what .eq. model )
c
c       I.e. is the boundary layer model cleared ?
c       ******************************************
c
      else if ( what .eq. BLM_clear ) then                                1m
          get_BLM = logical_to_real( cleared )
c
c       Cannot get reset
c       ****************
c
      else if ( what .eq. BLM_reset ) then                                1m
          get_BLM = 0.0
          got_error = .TRUE.
          error_str = ' GET_BLM : ABORT : Cannot reset'
c
c       Number of optimize variables
c       ****************************
c
      else if ( what .eq. BLM_n_opt ) then                                1m
          get_BLM = real( n_optim ) + 0.5
c
c       Identifier for an optimize variable
c       ***********************************
c
      else if ( BLM_n_opt .lt. what .and.
     1               what .le. BLM_n_opt + n_optim ) then                 1m
          get_BLM = real( opt_par( what - BLM_n_opt ) ) + 0.5
c
c       Illegal argument
c       ****************
c
      else                                                                1m
          get_BLM = 0.0
          got_error = .TRUE.
          error_str = ' GET_BLM : ABORT : Illegal argument'
          end if                                                          1e
      return
      end



      SUBROUTINE SET_BLM( WHAT, VALUE )
c
c Set the boundary layer model parameter WHAT ( which is a value selected
c from the file BLM.def ) to the value VALUE .
c
c Arguments
c  integer what         # identifies selected parameter -- from BLM.def
c  real value           # value of selected parameter
c
c Variables
c  logical ok           # unused returned value of is_between
c          is_between   # function : assigns the parameter if it is
c                       #            between permitted limits
c  integer i_var        # defines which optimizing variable 1st, 2nd, 3rd etc
c          i_val        # -- which parameter it is
c          src          # source index for removing duplicate      | optimizing
c          dest         # destination index for removing duplicate | variables
c          prev         # indices of previously filtered           |
c  character *24 name_BLM   # function : name of BLM parameter
c            fr_vel_visc_str*45       # function :
c
      include 'stdio.def'
      include 'blm.def'
c
      logical ok, is_between
      integer what, i_var, i_val, src, dest, prev
      real    value
      character Cabort*31, name_BLM*24, fr_vel_visc_str*45
      include 'blm.cmn'
      data Cabort / ' SET_BLM : ABORT : Unavailable ' /
c
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       Von Karmen constant, Kappa
c       **************************
c         -- slope of the log-law velocity profile
c         -- is the same for both dimensioned & nondimensional cases
c
      if ( what .eq. BLM_Karmen ) then                                    1b
          ok = is_between( value, Kappa, 0.2, 1.0 )
c
c       vertical offset C of the log-law velocity profile
c       *************************************************
c         -- is the same for both dimensioned & nondimensional cases
c
      else if ( what .eq. BLM_Constant ) then                             1m
          ok = is_between( value, Constant, 1.0, 20.0 )
c
c       kinematic viscosity
c       *******************
c         -- dimensioned value only ( metres squared / sec )
c
      else if ( what .eq. BLM_kin_visc ) then                             1m
          got_visc = is_between( value, kinem_visc, 1.0e-9, 1.0 )
c
c       free stream speed
c       *****************
c         -- dimensioned value only ( metres / sec )
c
      else if ( what .eq. BLM_fs_speed ) then                             1m
          got_fsv = is_between( value, free_stream, 0.001, 1000.0 )
c
c       friction velocity
c       *****************
c          -- dimensioned value only ( metres / second )
c
      else if ( what .eq. BLM_friction_vel ) then                         1m
          got_frict = is_between( value, frict_vel, 0.0001, 100.0 )
c
c       Coles' PI wake strength
c       ***********************
c         -- dimensioned     -- metres / second
c         -- non-dimensional -- multiples of friction velocity
c
      else if ( what .eq. BLM_PI_wake ) then                              1m
          if ( dimensioned ) then                                         2b
              if ( got_frict ) then                                       3b
                  got_PI = is_between( value / frict_vel, PI_wake,
     1                                   0.0, 100.0 )
              else                                                        3m
                  call str_cat( Cabort, name_BLM( BLM_friction_vel ),
     1                error_str )
                  got_error = .TRUE.
                  write ( STD_ERR, 1 ) error_str(1:len(error_str))
1                 format( a )
                  end if                                                  3e
          else                                                            2m
              got_PI = is_between( value, PI_wake, 0.0, 20.0 )
              end if                                                      2e
c
c       Coles' delta "boundary layer thickness"
c       ***************************************
c         -- dimensioned     -- metres
c         -- non-dimensional -- multiples of Y_plus
c
      else if ( what .eq. BLM_delta ) then                                1m
          if ( dimensioned ) then                                         2b
              if ( got_frict .and. got_visc ) then                        3b
                  got_delta = is_between( value * frict_vel /
     1                kinem_visc, delta_plus, 10.0, 1000000.0 )
              else                                                        3m
                  call str_cat( Cabort, fr_vel_visc_str(), error_str )
                  write( STD_ERR, 1 ) error_str(1:len(error_str))
                  end if                                                  3e
          else                                                            2m
              got_delta = is_between( value, delta_plus, 10.0, 1.0e6 )
              end if                                                      2e
c
c       Parameters are dimensioned
c       **************************
c
      else if ( what .eq. BLM_dimens ) then                               1m
          dimensioned  = .TRUE.
c
c       Parameters are non-dimensional
c       ******************************
c
      else if ( what .eq. BLM_nondim ) then                               1m
          dimensioned = .FALSE.
c
c       Select the model
c       ****************
c
c BLM_Clauser           Clauser F.H (1954)
c BLM_56Coles           Coles D.E. (1956)
c BLM_68Coles           Coles D.E. (1968)
c BLM_MKBull            Bull M.K. (1969)
c BLM_Beattie           Beattie & Green (1983)
c BLM_BG_68C            Beattie & Green (1983) + Coles 1968 wake function
c BLM_Spalding          Spalding d.b. (1961)
c BLM_Spld_68C          Spalding d.b. (1961) + Coles 1968 wake function
c
      else if ( what .eq. BLM_Clauser ) then                              1m
          model = BLM_Clauser
          Kappa = 0.41
          Constant = 4.9
          n_optim = 1
          opt_par(1) = BLM_friction_vel
      else if ( what .eq. BLM_56Coles ) then                              1m
          model = BLM_56Coles
          Kappa = 0.40
          Constant = 5.10
          n_optim = 1
          opt_par(1) = BLM_friction_vel
      else if ( what .eq. BLM_68Coles ) then                              1m
          model = BLM_68Coles
          Kappa = 0.41
          Constant = 5.0
          n_optim = 3
          opt_par(1) = BLM_friction_vel
          opt_par(2) = BLM_PI_wake
          opt_par(3) = BLM_delta
      else if ( what .eq. BLM_MKBull  ) then                              1m
          model = BLM_MKBull
          Kappa = 0.41
          Constant = 5.0
          n_optim = 3
          opt_par(1) = BLM_friction_vel
          opt_par(2) = BLM_PI_wake
          opt_par(3) = BLM_delta
      else if ( what .eq. BLM_Beattie ) then                              1m
          model = BLM_Beattie
          Kappa = 0.407
          Constant = 5.66
          n_optim = 1
          opt_par(1) = BLM_friction_vel
      else if ( what .eq. BLM_BG_68C ) then                               1m
          model = BLM_BG_68C
          Kappa = 0.407
          Constant = 5.66
          n_optim = 3
          opt_par(1) = BLM_friction_vel
          opt_par(2) = BLM_PI_wake
          opt_par(3) = BLM_delta
      else if ( what .eq. BLM_Spalding ) then                             1m
          model = BLM_Spalding
          Kappa = 0.40
          Constant = 5.5
          n_optim = 1
          opt_par(1) = BLM_friction_vel
      else if ( what .eq. BLM_Spld_68C ) then                             1m
          model = BLM_Spld_68C
          Kappa = 0.41
          Constant = 5.0
          n_optim = 3
          opt_par(1) = BLM_friction_vel
          opt_par(2) = BLM_PI_wake
          opt_par(3) = BLM_delta
c
c       Clear the boundary layer model
c       ******************************
c
      else if ( what .eq. BLM_clear ) then                                1m
          cleared = .TRUE.
          got_frict = .FALSE.
          got_PI    = .FALSE.
          got_delta = .FALSE.
c
c       Reset to default values
c       ***********************
c
      else if ( what .eq. BLM_reset ) then                                1m
          cleared   = .TRUE.
          dimensioned = .FALSE.
          got_frict = .FALSE.
          got_PI    = .FALSE.
          got_delta = .FALSE.
          got_visc  = .FALSE.
          got_fsv   = .FALSE.
          got_error = .FALSE.
          model     = BLM_56Coles
          Kappa     = 0.40
          Constant  = 5.1
c
c       Number of optimize variables
c       ****************************
c
      else if ( what .eq. BLM_n_opt ) then                                1m
          if ( is_between( value, value, 0.0, BLM_max_opt + 0.4999 ) )
     1            then                                                    2b
              i_val = int( value + 0.5 )
              if ( i_val .gt. n_optim ) then                              3b
                  do 2 src = n_optim + 1, i_val, 1                        4b
                      opt_par(src) = 0
2                     continue                                            4e
              else if ( i_val .lt. n_optim ) then                         3m
                  do 3 src = i_val + 1, n_optim                           4b
                      opt_par(src) = 0
3                     continue                                            4e
                  end if                                                  3e
              n_optim  = i_val
              end if                                                      2e
c
c       Identifier for an optimize variable
c       ***********************************
c
      else if ( BLM_n_opt .lt. what .and.
     1               what .le. BLM_n_opt + n_optim ) then                 1m
c
c           i_var defines which optimizing variable 1st, 2nd, 3rd etc
c           i_val which parameter it is
          i_var = what - BLM_n_opt
          i_val = int( value + 0.5 )
c
c           any of BLM_Constant, BLM_Karmen, BLM_fs_speed, BLM_kin_visc,
c           BLM_friction_vel can be optimizing variables
c
          if ( i_val .eq. BLM_Constant .or. i_val .eq. BLM_Karmen .or.
     1         i_val .eq. BLM_fs_speed .or. i_val .eq. BLM_kin_visc .or.
     2         i_val .eq. BLM_friction_vel ) then                         2b
              opt_par(i_var) = i_val
c
c           BLM_delta, BLM_PI_wake can only be used as optimizing variables
c           with the BLM_68Coles, BLM_MKBull, BLM_BG_68C and BLM_Spld_68C
c           models
c
          else if ( ( model .eq. BLM_68Coles  .or.
     1                model .eq. BLM_MKBull   .or.
     2                model .eq. BLM_BG_68C   .or.
     3                model .eq. BLM_Spld_68C ) .and.
     2         ( i_val .eq. BLM_delta .or. i_val .eq. BLM_PI_wake ) )
     3           then                                                     2m
              opt_par(i_var) = i_val
          else                                                            2m
              error_str =
     1            ' SET_BLM : ABORT : Illegal optimizing variable'
              got_error = .TRUE.
              write( STD_ERR, 1 ) error_str(1:len(error_str))
              end if                                                      2e
c
c           compress out any duplicated optimizing variables
c           but don't bother compressing out zeros
c
          if ( n_optim .gt. 1 ) then                                      2b
              dest = 2
              do 5 src = 2, n_optim                                       3b
                  do 4 prev = 1, dest - 1                                 4b
                      if ( opt_par(src) .ne. 0 .and.
     1                     opt_par(src) .eq. opt_par(prev) ) then         5b
                          n_optim = n_optim - 1
                          goto 5
                          end if                                          5e
4                     continue                                            4e
c           the next 2 lines are executed only if opt_par(src) is
c           not dupicated in opt_par(1) to opt_par(dest-1)
                  opt_par(dest) = opt_par(src)
                  dest = dest + 1
5                 continue                                                3e
              end if                                                      2e
c
c           Illegal argument
c           ****************
c
      else                                                                1m
          error_str = ' SET_BLM : ABORT : Illegal argument'
          got_error = .TRUE.
          write( STD_ERR, 1 ) error_str(1:len(error_str))
          end if                                                          1e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       Set the "cleared" status according to the "got_*" flags and "model"
c
      if ( model .eq. BLM_68Coles .or. model .eq. BLM_MKBull .or.
     1     model .eq. BLM_BG_68C .or. model .eq. BLM_Spld_68C ) then      1b
          cleared = .not. ( got_frict .and. got_PI . and. got_delta )
      else if ( model .eq. BLM_56Coles .or. model .eq. BLM_Clauser .or.
     1          model .eq. BLM_Beattie .or. model .eq. BLM_Spalding
     2        ) then                                                      1m
          cleared = .not. got_frict
      else                                                                1m
          error_str = ' SET_BLM : ABORT : Illegal model'
          got_error = .TRUE.
          write( STD_ERR, 1 ) error_str(1:len(error_str))
          end if                                                          1e
      return
      end



      SUBROUTINE PRIN_BLM( OUT_LU )
c
c Print the status of the boundary layer model to the file with logical
c unit number OUT_LU .
c
c Arguments
c  integer out_lu   # logical unit number of the output file
c
c Variables
c  integer i        # looping index
c  character *24 name_BLM # function : name of BLM parameter
c
      include 'stdio.def'
      include 'blm.def'
c
      integer out_lu, i
      character *24 name_BLM
      include 'blm.cmn'
c
c       --cleared
c
      write( out_lu, 1 )
1     format( 10x, 'BOUNDARY LAYER MODEL' /
     1        10x, '********************' )
      if ( cleared ) then
          write( out_lu, 2 )
2         format( ' Results are cleared' )
      else
          write( out_lu, 3 )
3         format( ' Results are available' )
          end if
c
c       --dimensioned
c
      if ( dimensioned ) then
          write( out_lu, 4 )
4         format( ' Input and output are dimensioned' )
      else
          write( out_lu, 5 )
5         format( ' Input and output are non-dimensional' )
          end if
c
c       --model
c
      write( out_lu, 6 )
6     format( / ' Boundary layer model :' )
      call Ref_BLM( out_lu )
c
c       --Kappa, Constant
c
      write( out_lu, 7 ) name_BLM( BLM_Karmen ), Kappa
      write( out_lu, 7 ) name_BLM( BLM_Constant ), Constant
7     format( 1x, a, ' :  ', g12.5, a )
c
c       --frict_vel
c
      if ( got_frict ) then
          write( out_lu, 7 )
     1       name_BLM( BLM_friction_vel ), frict_vel, ' m/s'
      else
          write( out_lu, 8 ) name_BLM( BLM_friction_vel )
8         format( 1x, a, ' :  unavailable' )
          end if
c
c       --PI_wake
c
      if ( got_PI ) then
          write( out_lu, 7 )
     1        name_BLM( BLM_PI_wake ), PI_wake, ' * U_tau'
      else
          write( out_lu, 8 ) name_BLM( BLM_PI_wake )
          end if
c
c       --delta
c
      if ( got_delta ) then
          write( out_lu, 7 )
     1        name_BLM( BLM_delta ), delta_plus, ' * Y_plus'
      else
          write( out_lu, 8 ) name_BLM( BLM_delta )
          end if
c
c       --free_stream
c
      if ( got_fsv ) then
          write( out_lu, 7 )
     1        name_BLM( BLM_fs_speed ), free_stream, ' m/s'
      else
          write( out_lu, 8 ) name_BLM( BLM_fs_speed )
          end if
c
c       --kinem_visc
c
      if ( got_visc ) then
          write( out_lu, 7 )
     1        name_BLM( BLM_kin_visc ), kinem_visc, ' sq.m/s'
      else
          write( out_lu, 8 ) name_BLM( BLM_kin_visc )
          end if
c
c       --n_optim     number of optimizing variables
c
      write( out_lu, 9 ) n_optim
9     format( ' Number of optimize parameters ', i2 )
c
c       --opt_par     optimize parameters
c
      write( out_lu, 10 )
10    format( ' Optimize parameters : ' )
      do 12 i = 1, min( n_optim, BLM_max_opt )
          write( out_lu, 11 ) i, Name_BLM( opt_par(i) )
11        format( 2x, i2, ') ', a )
12        continue
c
      return
      end



      CHARACTER *24 FUNCTION name_BLM( WHAT )
c
c  Returns a character string describing the model parameter WHAT .
c
      include 'stdio.def'
      include 'blm.def'
      integer what
      include 'blm.cmn'
c
c       models
c
      if ( what .eq. BLM_BLmodel ) then
          name_BLM = 'model specifier'
      else if ( what .eq. BLM_Clauser ) then
          name_BLM = 'Clauser F.H (1954)'
      else if ( what .eq. BLM_56Coles ) then
          name_BLM = 'Coles D.E. (1956)'
      else if ( what .eq. BLM_68Coles ) then
          name_BLM = 'Coles D.E. (1968)'
      else if ( what .eq. BLM_MKBull ) then
          name_BLM = 'Bull M.K. (1969)'
      else if ( what .eq. BLM_Beattie ) then
          name_BLM = 'Beattie & Green (1983)'
      else if ( what .eq. BLM_BG_68C ) then
          name_BLM = 'Beattie + Coles ''68 wake'
      else if ( what .eq. BLM_Spalding ) then
          name_BLM = 'Spalding (1961) '
      else if ( what .eq. BLM_Spld_68C ) then
          name_BLM = 'Spalding +Coles ''68 wake'
c
c       parameters
c
      else if ( what .eq. BLM_Constant ) then
          name_BLM = 'C - log-law offset'
      else if ( what .eq. BLM_Karmen   ) then
          name_BLM = 'Kappa - log-law slope'
      else if ( what .eq. BLM_fs_speed ) then
          name_BLM = 'free stream speed'
      else if ( what .eq. BLM_kin_visc ) then
          name_BLM = 'kinematic viscosity'
      else if ( what .eq. BLM_friction_vel ) then
          name_BLM = 'friction velocity'
      else if ( what .eq. BLM_PI_wake ) then
          name_BLM = 'PI - Coles wake strength'
      else if ( what .eq. BLM_delta ) then
          name_BLM = 'boundary layer thickness'
c
c       Optimization
c
      else if ( what .eq. BLM_max_opt ) then
          name_BLM = 'Maximum optimize params'
      else if ( what .eq. BLM_n_opt ) then
          name_BLM = 'No. of optimize params'
      else if ( what .gt. BLM_n_opt .and.
     1          what .le. BLM_n_opt + n_optim ) then
          write( name_BLM, 1 ) what - BLM_n_opt
1         format( 'optimize parameter ID ', i2 )
c
c       switches
c
      else if ( what .eq. BLM_reset ) then
          name_BLM = 'reset to default values'
      else if ( what .eq. BLM_clear ) then
          name_BLM = 'clear result parameters'
      else if ( what .eq. BLM_dimens ) then
          name_BLM = 'dimensioned I/O'
      else if ( what .eq. BLM_nondim ) then
          name_BLM = 'non-dimensional I/O'
      else
          name_BLM = 'unknown'
          got_error = .TRUE.
          error_str = ' NAME_BLM : ABORT : unknown parameter'
          end if
      return
      end


      SUBROUTINE REF_BLM( OUT_LU )
c
c Write the literature reference of the currently selected boundary layer
c model to the file with logical unit number OUT_LU .
c
c Argument
c  integer out_lu     # logical unit number of the output file
c
      include 'stdio.def'
      include 'blm.def'
c
      integer out_lu
      include 'blm.cmn'
c
      if ( model .eq. BLM_Clauser ) then
          write( out_lu, 1 )
1         format( ' Clauser F.H. (1954) :' /
     1  9x, 'Turbulent boundary layers in adverse pressure gradients,'/
     2  9x, 'Journal of the Aeronautical Sciences, Vol. 21(2), ',
     3  'pp 91--108' )
c Kappa = 0.41, Constant=4.9
c
c
      else if ( model .eq. BLM_56Coles ) then
          write( out_lu, 2 )
2         format( ' Coles D.E. (1956) :' /
     1  9x, 'The law of the wake in the turbulent boundary layer,' /
     2  9x, 'Journal of Fluid Mechanics, Vol. 1, pp 191--226' )
c Kappa=0.40, Constant=5.10
c
c
      else if ( model .eq. BLM_68Coles ) then
          write( out_lu, 3 )
3         format( ' Coles D.E. (1968) :' /
     1  9x, 'The Young Person''s Guide to the Data,' /
     2  9x, '1968 AFOSR-IFP-Stanford Conference on Computation of ' /
     3  13x, 'Turbulent Boundary Layers, p1' )
c Kappa=0.41, Constant=5.0
c
c
      else if ( model .eq. BLM_MKBull ) then
          write( out_lu, 4 )
4         format( ' Bull M.K. (1969) :' /
     1  9x, 'Velocity profiles of turbulent boundary layers,' /
     2  9x, 'Aeronutical Journal of the Royal Aeronautical Society, ' /
     3  13x, 'Vol 73, p 143' )
c
c
      else if ( model .eq. BLM_Beattie ) then
          write( out_lu, 5 )
5         format( ' Beattie D.R.H. & Green W.J. (1983) :' /
     1  9x, 'A Simplified and Accurate Method of Describing Non-',
     2      'Turbulent Transport' /
     3 13x, 'of Momentum in Turbulent Flow Theory,' /
     4  9x, 'Proceedings, Eighth Australasian Fluid Mechanics ',
     5      'Conference,} Newcastle' / 13x, 'N.S.W. pp 10A.5--9' )
c Kappa=0.407, Constant=5.66
c
      else if ( model .eq. BLM_BG_68C ) then
          write( out_lu, 7 )
7         format( ' Law of the wall from' )
          write( out_lu, 5 )
          write( out_lu, 8 )
8         format( ' and law of the wake from' )
          write( out_lu, 3 )
c Kappa=0.407, Constant=5.66
      else if ( model .eq. BLM_Spalding ) then
          write( out_lu, 9 )
9         format( ' Spalding D.B. (1961) :' /
     1         9x, 'A single formula for the "law of the wall",' /
     2         9x, 'Journal of Applied Mechanics, Trans. A.S.M.E.,' /
     3         13x, 'Vol 28, No. 3 (Sept), pp 455-458' )
c Kappa=0.40, Constant=5.5
      else if ( model .eq. BLM_Spld_68C ) then
          write( out_lu, 7 )
          write( out_lu, 9 )
          write( out_lu, 8 )
          write( out_lu, 3 )
c Kappa=0.41, Constant=5.0
      else
          error_str = ' REF_BLM : WARNING : No boundary layer model'
          got_error = .TRUE.
          write( out_lu, 6 ) error_str
6         format( a )
          end if
      return
      end



      REAL FUNCTION UPLUS_BLM( Y_ARG )
c
c Calculates the non-dimensional mean "law of the wall"  streamwise velocity
c U_plus in a two dimensional boundary layer as a function of Y_PLUS, which
c is the distance from the surface in "wall units" for the currently selected
c boundary layer model .
c
c Argument
c  real y_arg           # distance from the wall in wall units or in metres
c
c Constants
c  MKBull_A=0.8         # for Bull (1969) model
c  PI=3.14159265359
c
c Variables
c  real y_plus          # distance from wall in wall units
c       y_zero_plus     # edge of linear sublayer- Beattie & Green (1983) model
c       Spald_K         # = kappa
c       Spald_C         # = exp( kappa * constant )
c       Spald_y         # = y_plus
c       Spald_w         # = kappa * wake function
c       Spald_u         # = initial estimate of U_plus for Spalding's formula
c  double precision dzerli # function : find zero by linear interpolation
c  character Cabort*33  # error message components
c            fr_vel_visc_str*45       # function :
c
      include 'stdio.def'
      include 'blm.def'
c
      real MKBull_A, PI
      parameter( MKBull_A=0.8, PI=3.15159265359 )
c
      real y_arg, y_plus, y_zero_plus
      character Cabort*33, name_BLM*24, fr_vel_visc_str*45
c
c     Transfer variables to Spalding 'law of the wall' via common block
      double precision Spald_K, Spald_C, Spald_y, Spald_w, Spald_u
      double precision dzerli
      common /Spalding/ Spald_K, Spald_C, Spald_y, Spald_w
      include 'blm.cmn'
c
      external Spald_law
      data Cabort / ' UPLUS_BLM : ABORT : Unavailable ' /
c
c           Negative Y_PLUS
c
      if ( y_arg .le. 0.0 ) then                                          1b
          Uplus_BLM  = 0.0
          got_error  = .TRUE.
          error_str = ' UPLUS_BLM : ABORT : Negative distance from wall'
      else                                                                1m
          if ( dimensioned ) then                                         2b
              if ( got_visc .and. got_frict ) then                        3b
                  y_plus = y_arg * frict_vel / kinem_visc
              else                                                        3m
                  got_error = .TRUE.
                  call str_cat( Cabort, fr_vel_visc_str(), error_str )
                  Uplus_BLM = 0.0
                  return
                  end if                                                  3e
          else
              y_plus = y_arg
              end if                                                      2e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Clauser (1954)
c    Without calculating wake strength . The velocity profile is
c        U^+ = \frac{1}{\kappa} \ln y^+ + c
c
          if ( model .eq. BLM_Clauser ) then                              2b
              Uplus_BLM = min( y_plus, max( 0.0,
     1                         alog( y_plus ) ) / Kappa + Constant )
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Coles (1956)
c    Calculate wake strength after calculating the friction velocity. The
c    wake function is not specified, and so it is not included in the
c    velocity profile . The velocity profile is
c        U^+ = \frac{1}{\kappa} \ln y^+ + c +
c              \frac{\PI}{\kappa} w \left( \frac{y}{\delta} \right)
c    The wake strength is  calculated iteratively from the transcendental
c    equation
c         2 \Pi = \ln ( 1 + \PI ) + \kappa \frac{U_1}{u_{\tau}} -
c                 \ln \frac{\delta^* U_1}{\nu} - \kappa c - \ln \kappa
c    The value of displacement thickness $\delta^*$ must be supplied .
c    Would suggest using LHS as the next iterate with an initial $\Pi=0$.
c    Watch out for non-convergence, oscillations, infeasible values etc.
c
          else if ( model .eq. BLM_56Coles ) then                         2m
              Uplus_BLM = min( y_plus, max( 0.0,
     1                         alog( y_plus ) ) / Kappa + Constant )
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Coles (1968)
c    The friction velocity, wake strength and boundary layer thickness
c    are calculated simultaneously by minimizing the mean square difference
c    between the model profile and the experimental data. The velocity
c    profile is
c        U^+ = \frac{1}{\kappa} \ln y^+ + c +
c              \frac{\PI}{\kappa} w \left( \frac{y}{\delta} \right)
c    where the wake function is
c       w \left( \frac{y^+}{\delta^+} \right) =
c               1 - \cos \left( \pi \frac{y^+}{\delta^+} \right)
c
          else if ( model .eq. BLM_68Coles ) then                         2m
              if ( got_delta ) then                                       3b
                  Uplus_BLM = min( y_plus, max( 0.0,
     1                alog( y_plus ) ) / Kappa + constant +
     2                PI_wake / Kappa *
     3                    ( 1.0 - cos( PI  * y_plus / delta_plus ) ) )
              else                                                        3m
                  got_error = .TRUE.
                  call str_cat( Cabort, name_BLM( BLM_delta ),
     1                error_str )
                  Uplus_BLM = 0.0
                  end if                                                  3e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Bull (1969)
c    The friction velocity, wake strength and boundary layer thickness
c    are calculated simultaneously by minimizing the mean square difference
c    between the model profile and the experimental data. The velocity
c    profile is the same as Coles (1968) :
c        U^+ = \frac{1}{\kappa} \ln y^+ + c +
c              \frac{\PI}{\kappa} w \left( \frac{y}{\delta} \right)
c    but the wake function is adjusted
c       w \left( \frac{y^+}{\delta^+} \right) =
c               1 - \cos \left( \pi \frac{y^+/\delta^+-a}{1-a} \right)
c
          else if ( model .eq. BLM_MKBull  ) then                         2m
              if ( got_delta ) then                                       3b
                  Uplus_BLM = min( y_plus, max( 0.0,
     1      alog( y_plus ) ) / Kappa + constant + PI_wake / Kappa *
     2        ( 1.0 - cos( PI * ( y_plus / delta_plus - MKBull_A ) /
     3                            ( 1.0 -  MKBull_A ) ) ) )
              else                                                        3m
                  got_error = .TRUE.
                  call str_cat( Cabort, name_BLM( BLM_delta ),
     1                error_str )
                  Uplus_BLM = 0.0
                  end if                                                  3e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Beattie & Green (1983)
c    The wake strength is not calculated
c  U^+ = y^+    for  0 < y^+ < y^+_0
c      = y^+_0 + \frac{1}{k} \ln ( \kappa [ y^+ - y^+_0 ] + 1 )
c  where  y^+_0 = C - \frac{1}{\kappa} \ln \kappa
c
          else if ( model .eq. BLM_Beattie ) then                         2m
              y_zero_plus = constant - alog( Kappa ) / Kappa
              if ( y_plus .le. y_zero_plus ) then                         3b
                  Uplus_BLM = y_plus
              else                                                        3m
                  Uplus_BLM = y_zero_plus +
     1              alog( Kappa * ( y_plus - y_zero_plus ) + 1 ) / Kappa
                  end if                                                  3e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Beattie & Green (1983), Coles (1968)
c  The law of the wall is taken from Beattie & Green (1983) . The law of
c  the wake is taken from Coles (1968) .
c  U^+ = y^+    for  0 < y^+ < y^+_0
c      = y^+_0 + \frac{1}{k} \ln ( \kappa [ y^+ - y^+_0 ] + 1 ) +
c              \frac{\PI}{\kappa} w \left( \frac{y}{\delta} \right)
c    where the wake function is
c       w \left( \frac{y^+}{\delta^+} \right) =
c               1 - \cos \left( \pi \frac{y^+}{\delta^+} \right)
c    and    y^+_0 = C - \frac{1}{\kappa} \ln \kappa
c
          else if ( model .eq. BLM_BG_68C ) then                          2m
              if ( got_delta ) then                                       3b
                  y_zero_plus = constant - alog( Kappa ) / Kappa
                  if ( y_plus .le. y_zero_plus ) then                     4b
                      Uplus_BLM = y_plus
                  else                                                    4m
                      Uplus_BLM = y_zero_plus +
     1                    ( alog( Kappa * ( y_plus - y_zero_plus ) + 1 )
     2                      + PI_wake *
     3                        ( 1.0 - cos( PI  * y_plus / delta_plus ) )
     4                    ) / Kappa
                      end if                                              4e
              else                                                        3m
                  got_error = .TRUE.
                  call str_cat( Cabort, name_BLM( BLM_delta ),
     1                error_str )
                  Uplus_BLM = 0.0
                  end if                                                  3e
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c           Spalding (1961)
c  The law of the wall is calculated by inverting
c    y^+ = U^+ + e^{-\kappa C} { e^{\kappa U^+ - w} - P( \kappa U^+ - w ) }
c  where P is the polynomial
c    P(x) = 1 + x + x^2/2! + x^3/3! + x^4/4!
c  and where  w  is the wake function
c       w \left( \frac{y^+}{\delta^+} \right) =
c               1 - \cos \left( \pi \frac{y^+}{\delta^+} \right)
c
          else if ( model .eq. BLM_Spalding .or.
     1              model .eq. BLM_Spld_68C ) then                        2m
              if ( model .eq. BLM_Spalding .or. got_delta ) then          3b
                  Spald_K = kappa
                  Spald_C = exp( - kappa * constant )
                  Spald_y = y_plus
                  if ( model .eq. BLM_Spld_68C ) then                     3b
                      Spald_w = PI_wake *
     1                      ( 1.0 - cos( PI  * y_plus / delta_plus ) )
                  else                                                    3m
                      Spald_w = 0.0
                      end if                                              3e
                  Spald_u  = min( y_plus,
     1                           alog( y_plus ) / kappa + constant )
c      WRITE(6,9999) Spald_K, Spald_C, Spald_y, Spald_w, Spald_u
c9999  FORMAT( ' SPALD:KCywu ', 5G13.5 )
                  Uplus_BLM = dzerli( Spald_u, 0.8 * Spald_u,
     1                                Spald_law, 1.0e-5 )
              else                                                        3m
                  got_error = .TRUE.
                  call str_cat( Cabort, name_BLM( BLM_delta ),
     1                error_str )
                  Uplus_BLM = 0.0
                  end if                                                  3e
              end if                                                      2e
c
c       for dimensioned I/O
c
          if ( dimensioned ) then                                         2b
              Uplus_BLM = Uplus_BLM * frict_vel
              end if                                                      2e
          end if                                                          1e
c
      return
      end



      DOUBLE PRECISION FUNCTION SPALD_LAW( U_PLUS )
c
c  Evaluate Spalding's law of the wall ... then subtract the original y_plus
c  to obtain a remainder or error.  Before this, the wake component is removed
c  from the given velocity U_PLUS.  Double precision is to avoid various
c  undesirable consequences of round-off error.
c
c  Argument
c   double precision U_plus    # dependent variable
c
c Local Variables
c  double precision U_wall     # law-of-wall component of flow speed
c       Spald_poly(5)          # polynomial coefficients -- 1st 5 from exp(x)
c       Dpoly                  # function : evaluate polynomial
c
      include 'stdio.def'
      double precision U_plus, U_wall, P, Spald_poly(5), Dpoly
      double precision Spald_K, Spald_C, Spald_y, Spald_w
      common /Spalding/ Spald_K, Spald_C, Spald_y, Spald_w
      data Spald_poly / 1.0, 1.0, 0.5, 0.16666666666666667,
     1                  0.041666666666666667 /
c
      U_wall = Spald_K * U_plus - Spald_w
      P = Dpoly( U_wall, Spald_poly, 4 )
      Spald_law = U_plus - Spald_y + Spald_C *
     1            ( exp( U_wall ) - P )
c      WRITE(6,9999) real(U_plus), real(U_wall), real(Spald_law)
c9999  FORMAT( ' (U_PLUS,U_WALL)=>SPALD_LAW ', 3g15.7 )
      return
      end




      LOGICAL FUNCTION ERROR_BLM( MESSAGE )
c
c Return .TRUE. if an error has occurred in a *_BLM routine and the error
c message is placed into MESSAGE. .FALSE. is returned and MESSAGE is not
c changed if no error has occurred .
c
      include 'stdio.def'
      include 'blm.def'
      character message*(*)
      include 'blm.cmn'
c
      error_blm = got_error
      if ( got_error ) then
          got_error = .FALSE.
          message   = error_str
          end if
      return
      end



      REAL FUNCTION DEF_BLM( WHAT )
c
c Returns a default value for the boundary layer model parameter WHAT .
c If WHAT is not a recognized parameter, an error message is generated .
c
c Argument
c  integer what     # specifies which boundary layer model parameter
c
c Variables
c  character *24 name_BLM   # function : name of BLM parameter
c            fr_vel_visc_str*45       # function :
c
      include 'stdio.def'
      include 'blm.def'
c
      integer what
      character Cabort*31, name_BLM*24, fr_vel_visc_str*45
      include 'blm.cmn'
c
      data Cabort / ' DEF_BLM : ABORT : Unavailable ' /
c
      if ( what .eq. BLM_Constant ) then                                  1b
          def_BLM = 5.0
      else if ( what .eq. BLM_Karmen ) then                               1m
          def_BLM = 0.41
      else if ( what .eq. BLM_fs_speed ) then                             1m
          def_BLM = max( min( 100.0, free_stream ), 1.0 )
      else if ( what .eq. BLM_kin_visc ) then                             1m
          def_BLM = 15.0e-6
      else if ( what .eq. BLM_friction_vel ) then                         1m
          if ( got_fsv ) then                                             2b
              def_BLM = max( min( 100.0, free_stream ), 1.0 ) / 25.0
          else                                                            2m
              def_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_fs_speed ),
     1            error_str )
              end if                                                      2e
      else if ( what .eq. BLM_PI_wake  ) then                             1m
          if ( .not. dimensioned ) then                                   2b
              def_BLM = 1.2
          else if ( got_frict ) then                                      2m
              def_BLM = 1.2 * frict_vel
          else                                                            2m
              def_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, name_BLM( BLM_friction_vel ),
     1            error_str )
              end if                                                      2e
      else if ( what .eq. BLM_delta    ) then                             1m
          if ( .not. dimensioned ) then                                   2b
              def_BLM = 3000.0
          else if ( got_frict .and. got_visc ) then                       2m
              def_BLM = 3000.0 * kinem_visc / frict_vel
          else                                                            2m
              def_BLM = 0.0
              got_error = .TRUE.
              call str_cat( Cabort, fr_vel_visc_str(), error_str )
              end if                                                      2e
      else                                                                1m
          def_BLM = 0.0
          error_str = ' DEF_BLM : ABORT : Argument not a parameter'
          got_error = .TRUE.
          end if                                                          1e
      return
      end



      REAL FUNCTION STEP_BLM( WHAT )
c
c Returns what is thought to be a suitable sensitivity or initial step
c size value for the boundary layer model parameter WHAT . This is
c expressed as a fractional value ( percentage / 100 )
c If WHAT is not a recognized parameter, an error message is generated .
c
c Argument
c  integer what     # specifies which boundary layer model parameter
c
      include 'stdio.def'
      include 'blm.def'
c
      integer what
      include 'blm.cmn'
c
      if ( what .eq. BLM_Constant ) then
          step_BLM = 0.0025
      else if ( what .eq. BLM_Karmen   ) then
          step_BLM = 0.0025
      else if ( what .eq. BLM_fs_speed ) then
          step_BLM = 0.01
      else if ( what .eq. BLM_kin_visc ) then
          step_BLM = 0.01
      else if ( what .eq. BLM_friction_vel ) then
          step_BLM = 0.01
      else if ( what .eq. BLM_PI_wake  ) then
          step_BLM = 0.02
      else if ( what .eq. BLM_delta    ) then
          step_BLM = 0.02
      else
          step_BLM = 0.0
          error_str = ' STEP_BLM : ABORT : Argument not a parameter'
          got_error = .TRUE.
          end if
      return
      end



      BLOCK DATA BLOCK_BLM
      include 'stdio.def'
      include 'blm.def'
      include 'blm.cmn'
      data
     1  cleared, dimensioned, got_frict, got_PI, got_delta, got_error /
     2  .TRUE.,  .FALSE.,     .FALSE.,  .FALSE., .FALSE.,   .FALSE.   /
      data model, n_optim, opt_par /
     1     BLM_56Coles, 1, BLM_friction_vel, 6*0 /
      data Kappa, Constant, kinem_visc /
     1     0.40,  5.1,      15.0e-6    /
      end



      REAL FUNCTION LOGICAL_TO_REAL( BOOLEAN )
c
c Converts the logical argument BOOLEAN into a real value :
c       .FALSE. => 0.0
c       .TRUE.  => 1.5
c
      include 'stdio.def'
      logical boolean
c
      if ( boolean ) then
          logical_to_real = 1.5
      else
          logical_to_real = 0.0
          end if
      return
      end



      LOGICAL FUNCTION IS_BETWEEN( SOURCE, DESTINATION, MINIM, MAXIM )
c
c Determines if the value of SOURCE is between the values of MINIM and MAXIM
c and returns the corresponding logical value. If the returned value is
c .TRUE. the value of SOURCE is placed in DESTINATION . If the returned
c value is .FALSE., SOURCE is not transferred to DESTINATION and an error
c is report .
c
c Arguments
c  real source,       # source value
c       destination   # destination address
c       minim         # minimum allowed value of DESTINATION
c       maxim         # maximum allowed value of DESTINATION
c
      include 'stdio.def'
      include 'blm.def'
      real source, destination, minim, maxim
      include 'blm.cmn'
c
      is_between =  minim .le. source .and. source .le. maxim
      if ( is_between ) then
          destination = source
      else
          got_error = .TRUE.
          write( error_str, 1 ) source, minim, maxim
1         format( ' SET_BLM : ABORT : ', g11.4, ' not between limits ',
     1            g11.4, ' and ', g11.4 )
          write( STD_ERR, 2 ) error_str
2         format( a )
          end if
      return
      end



      CHARACTER *45 FUNCTION FR_VEL_VISC_STR()
c
c Concatenates the names of the BLM_friction_vel and BLM_kin_visc parameters
c without exceeding a total length of 45 characters . This is required
c because the concatenation operator // stuffs up if the source string is
c longer than the destination string .
c
c Variables
c  integer len_1          # non-blank length of the first component string
c          str_length     # function : non-blank string length
c  character str_1*24     # first component string
c            name_BLM*24  # function : name of BLM parameter
c
      include 'stdio.def'
      include 'blm.def'
c
      integer len_1, str_length
      character str_1*24, name_BLM*24
c
      str_1 = name_BLM( BLM_friction_vel )
      len_1 = str_length( str_1 )
      fr_vel_visc_str = str_1( 1 : len_1 )
      if ( len_1 + 1 .le. 45 ) then
          fr_vel_visc_str( len_1+1 : ) = ' or '
          if ( len_1 + 5 .le. 45 ) then
              fr_vel_visc_str( len_1+5 : ) = name_BLM( BLM_kin_visc )
              end if
          end if
      return
      end



      SUBROUTINE STR_CAT( FIRST, APPEND, OUT )
c
c Concatenate the character string APPEND to the end of the string FIRST and
c places the result into OUT .
c
      include 'stdio.def'
      integer len_first
      character first*(*), append*(*), out*(*)
c
      out = first
      len_first = len( first )
      if ( len_first .lt. len( out ) ) then
          out( len_first + 1 : ) = append
          end if
      return
      end


