c
c    Copyright (C) 1997  Peter Vernon Lanspeary
c
c    This program is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program; if not, write to the Free Software
c    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccc                                                                cccc
cccc                 NUMERICAL INTEGRATION LIBRARY                  cccc
cccc                                                                cccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c
c
c  INTEG_NAM()
c  INTEG_GET( PROPERTY )
c  INTEG_SET( PROPERTY, VALUE )
c  INTEGRATE( INTEGRAND_FUNCTION, LOWER_LIMIT, UPPER_LIMIT )
c
c



      CHARACTER *16 FUNCTION INTEG_NAM()
c
c Returns the name the currently selected integration method .
c
      include 'stdio.def'
      include 'integ.def'
c
      include 'integ.cmn'
c
      if ( method .eq. INTEG_TRAPEZ +  INTEG_CLOSED ) then
          integ_nam = 'closed trapezium'
      else if ( method .eq. INTEG_SIMPSON +  INTEG_CLOSED ) then
          integ_nam = 'closed simpson'
      else if ( method .eq. INTEG_RHOMBERG +  INTEG_CLOSED ) then
          integ_nam = 'closed rhomberg'
      else if ( method .eq. INTEG_TRAPEZ + INTEG_OPEN ) then
          integ_nam = 'open trapezium'
      else if ( method .eq. INTEG_SIMPSON + INTEG_OPEN ) then
          integ_nam = 'open simpson'
      else if ( method .eq. INTEG_RHOMBERG + INTEG_OPEN ) then
          integ_nam = 'open rhomberg'
      else
          integ_nam = 'unknown method'
          end if
      return
      end



      REAL FUNCTION INTEG_GET( PROPERTY )
c
c Read the integration properties .
c
c Arguments
c  integer property   # Specifies the property to be set . Must be one of
c   INTEG_MAX_EVAL -- max number of integrand evaluations
c   INTEG_METHOD   -- integration method; value = { INTEG_CLOSED, INTEG_OPEN } +
c                     { INTEG_TRAPEZ, INTEG_SIMPSON, INTEG_RHOMBERG }
c   INTEG_PRECIS   -- integration precision
c   INTEG_ZER_TOL  -- zero detection tolerance
c   INTEG_EVALNS   -- number of function evaluations
c   INTEG_UPPER    -- upper limit of integration
c   INTEG_LOWER    -- lower limit of integration
c
      include 'stdio.def'
      include 'integ.def'
c
      integer property
      include 'integ.cmn'
c
      if ( property .eq. INTEG_MAX_EVAL ) then                            1b
          integ_get = max_evalns
c
c       integration method
c
      else if ( property .eq. INTEG_METHOD ) then                         1m
          integ_get = method
c
c       integration precision
c
      else if ( property .eq. INTEG_PRECIS ) then                         1m
          integ_get = precision
c
c       zero detection tolerance
c
      else if ( property .eq. INTEG_ZER_TOL ) then                        1m
          integ_get = tolerance
c
c        number of integrand function evaluations (get only)
c
      else if ( property .eq. INTEG_EVALNS ) then                         1m
          integ_get = n_evalns
c
c       upper limit of integration
c
      else if ( property .eq. INTEG_UPPER ) then                          1m
          integ_get = upper
c
c       lower limit of integration
c
      else if ( property .eq. INTEG_LOWER ) then                          1m
          integ_get = lower
      else
          write( STD_ERR, 1 ) property
1         format( ' INTEG_GET : ABORT : Unknown property argument ',
     1            i12 )
          end if                                                          1e
      return
      end



      LOGICAL FUNCTION INTEG_SET( PROPERTY, VALUE )
c
c Set integration properties .
c
c Arguments
c  integer property   # Specifies the property to be set . Must be one of
c                     #  INTEG_MAX_EVAL -- max number of integrand evaluations
c                     #  INTEG_METHOD   -- integration method;
c                     #    value = { INTEG_CLOSED, INTEG_OPEN } +
c                     #       { INTEG_TRAPEZ, INTEG_SIMPSON, INTEG_RHOMBERG }
c                     #  INTEG_PRECIS   -- integration precision
c                     #  INTEG_ZER_TOL  -- zero detection tolerance
c                     #  INTEG_UPPER    -- upper limit of integration
c                     #  INTEG_LOWER    -- lower limit of integration
c  real    value      # Property value .
c
c Local variables
c  integer
c   int_value         # int( abs( value ) )
c  real
c   abs_value         # abs( value )
c  character
c   propert_str*15    # string representation of the selected property
c
c
      include 'stdio.def'
      include 'integ.def'
c
      integer   property, int_value
      real      value, abs_value
      character propert_str*15
c
      include 'integ.cmn'
c
      abs_value = abs( value )
c
c       maximum permitted number of integrand evaluations
c
      if ( property .eq. INTEG_MAX_EVAL ) then                            1b
          integ_set = 1.5 .lt. abs_value .and. abs_value .le. 100000.0
          propert_str = 'Max evaluations'
          if ( integ_set )
     1        max_evalns = int( abs_value + 0.5 )
c
c       integration method
c
      else if ( property .eq. INTEG_METHOD ) then                         1m
          int_value = int( abs( value ) + 0.5 )
          integ_set =
     1        int_value .eq. INTEG_CLOSED + INTEG_TRAPEZ   .or.
     2        int_value .eq. INTEG_CLOSED + INTEG_SIMPSON  .or.
     3        int_value .eq. INTEG_CLOSED + INTEG_RHOMBERG .or.
     4        int_value .eq. INTEG_OPEN   + INTEG_TRAPEZ   .or.
     5        int_value .eq. INTEG_OPEN   + INTEG_SIMPSON  .or.
     6        int_value .eq. INTEG_OPEN   + INTEG_RHOMBERG
          propert_str = 'Integr''n method'
          if ( integ_set )
     1        method = int_value
c
c       integration precision
c
      else if ( property .eq. INTEG_PRECIS ) then                         1m
          integ_set = 0.0 .le. abs_value .and. abs_value .lt. 1.0
          propert_str = 'Precision'
          if ( integ_set )
     1        precision = abs_value
c
c       zero detection tolerance
c
      else if ( property .eq. INTEG_ZER_TOL ) then                        1m
          integ_set = 0.0 .le. abs_value .and. abs_value .lt. 1.0
          propert_str = 'Zero tolerance'
          if ( integ_set )
     1        tolerance = abs_value
c
c       upper limit of integration
c
      else if ( property .eq. INTEG_UPPER ) then                          1m
          upper = value
          propert_str = 'Lower limit'
c
c       lower limit of integration
c
      else if ( property .eq. INTEG_LOWER ) then                          1m
          lower = value
          propert_str = 'Upper limit'
      else
          integ_set = .FALSE.
          write( STD_ERR, 1 ) property
1         format( ' INTEG_SET : ABORT : Unknown property argument ',
     1            i12 )
          return
          end if                                                          1e
      if ( .not. integ_set )
     1    write( STD_ERR, 2 ) propert_str, value
2         format( ' INTEG_SET : ABORT : ', a, ' is invalid : ', g15.7 )
      return
      end



      REAL FUNCTION INTEGRATE( INTEGRAND, LOWER_LIMIT, UPPER_LIMIT )
c
c Calculates the integral the function INTEGRAND between the domain limits
c LOWER_LIMIT and UPPER_LIMIT to an accuracy which can be specified with
c INTEG_SET . The integration method, integration limits, zero tolerance
c and maximum number of function evaluations can also be specified with
c INTEG_SET .
c
c Arguments
c  real integrand     # function to be integrated
c       lower_limit   # lower limit of the integration interval
c       upper_limit   # upper limit of the integration interval
c
c Local variables
c  integer
c   integ_bas_midpt   # function : basic integration; single interation
c                     #            of the (open) midpoint rule
c   integ_bas_trapz   #          : basic integration; single interation
c                     #            of the (closed) trapezoidal rule
c
c  real
c   temp              # temporary variable for swapping the limits
c   integ_gen_rhomb   # function : generalised Rhomberg method
c   integ_gen_simps   #          : generalised Simpson's rule
c   integ_gen_trapz   #          : generalised Trapezoidal rule
c
c global variables
c  integer max_evalns, method
c  real    lower, upper, tolerance
c
      include 'stdio.def'
      include 'integ.def'
c
      integer integ_bas_midpt, integ_bas_trapz
      real    integrand, lower_limit, upper_limit, temp,
     1        integ_gen_rhomb, integ_gen_simps, integ_gen_trapz
c
      include 'integ.cmn'
      external integ_bas_midpt, integ_bas_trapz, integ_block
c
c
      integrate = INTEG_ERROR
c
c       Only use the limits-of-integration arguments if they are not INTEG_ERROR
c
      if ( lower_limit .ne. INTEG_ERROR )
     1    lower = lower_limit
      if ( upper_limit .ne. INTEG_ERROR )
     1    upper = upper_limit
c
c       Ensure that the lower limit is smaller than the upper limit
c
      if ( lower .gt. upper ) then                                        1b
          temp  = lower
          lower = upper
          upper = temp
          end if                                                          1e
c
c       Abort with error if the maximum number of evaluations is too
c       low . Strictly, the minimum is one evaluation for open formulas,
c       but who cares ?
c
      if ( max_evalns .le. 2 ) then                                       1b
          write( STD_ERR, 1 )
1         format( ' INTEGRATE : ABORT : Maximum permitted evaluations ',
     1            'is too low' )
c
c       The result is zero if the limits are equal
c
      else if ( abs( upper - lower ) .le.
     1        tolerance * max( abs( upper ), abs( lower ) ) ) then        1b
          integrate = 0.0
c
c       CLOSED FORMULA TRAPEZOIDAL RULE
c
      else if ( method .eq. INTEG_CLOSED + INTEG_TRAPEZ ) then            1m
          integrate = integ_gen_trapz( integ_bas_trapz, integrand )
c
c       CLOSED FORMULA SIMPSON'S RULE
c
      else if ( method .eq. INTEG_CLOSED + INTEG_SIMPSON ) then           1m
          integrate = integ_gen_simps( integ_bas_trapz, 2, integrand )
c
c       CLOSED FORMULA RHOMBERG METHOD
c
      else if ( method .eq. INTEG_CLOSED + INTEG_RHOMBERG ) then          1m
          integrate = integ_gen_rhomb( integ_bas_trapz, 2, integrand )
c
c       OPEN FORMULA TRAPEZOIDAL RULE
c
      else if ( method .eq. INTEG_OPEN + INTEG_TRAPEZ ) then              1m
          integrate = integ_gen_trapz( integ_bas_midpt, integrand )
c
c       OPEN FORMULA SIMPSON'S RULE
c
      else if ( method .eq. INTEG_OPEN + INTEG_SIMPSON ) then             1m
          integrate = integ_gen_simps( integ_bas_midpt, 3, integrand )
c
c       OPEN FORMULA RHOMBERG METHOD
c
      else if ( method .eq. INTEG_OPEN + INTEG_RHOMBERG ) then            1m
          integrate = integ_gen_rhomb( integ_bas_midpt, 3, integrand )
      else                                                                1m
          write( STD_ERR, 2 )
2         format( ' INTEGRATE : ABORT : Unknown integration method' )
          end if                                                          1e
      return
      end



      REAL FUNCTION INTEG_GEN_RHOMB( INTEG_BASE, STEP_RATIO, INTEGRAND )
c
c General Rhomberg method integration.
c Integrates the function INTEGRAND between the integration limits LOWER
c and UPPER to a specified accuracy using the base integration routine
c INTEG_BASE -- which may be an open, semi-open, closed, transformed or
c improper integration formula .
c
c Arguments
c  integer integ_base # base integration routine
c                     # INTEG_BAS_TRAPZ, or INTEG_BAS_MIDPT
c          step_ratio # step size reduction factor from one iteration to
c                     # the next -- depends on the basic integration routine,
c                     # 2 for INTEG_BAS_TRAPZ, 3 for INTEG_BAS_MIDPT
c  real integrand     # INPUT  # function (of one variable) defining integrand
c
c local variables
c  integer
c   i                 # array index
c   n_iterations      # number of iterations performed
c   n_points          # number of internal evaluation points
c  real
c   approxim(EXTRAPOLATION_ORDER+1)   # list of most recent integral estimates
c   step_size(EXTRAPOLATION_ORDER+1)  # list of most recent step sizes
c   extrap_coeff      #
c   extrap_error      # Estimated polynomial extrapolation error
c   pol_inter         # function : polynomial extrapolation
c
c global variables
c  integer max_evalns, n_evalns
c  real    precision
c
      include 'stdio.def'
      integer    BISECT_CLOSEST,   MAX_EXTRAP_ORDER
      parameter( BISECT_CLOSEST=2, MAX_EXTRAP_ORDER=4 )
c
      integer integ_base, step_ratio,
     1        extrapoln_order, i, n_iterations, n_points
      real integrand, extrap_coeff, extrap_error, pol_inter,
     1     approxim(MAX_EXTRAP_ORDER+1), step_size(MAX_EXTRAP_ORDER+1)
c
      include 'integ.cmn'
      external integ_base, integrand
c
      extrap_coeff = step_ratio **2
c
      step_size(1) = 1.0
      n_points = 0
      n_evalns = integ_base( integrand, lower, upper, n_points,
     1                       approxim(1) )
      integ_gen_rhomb = approxim(1)
      n_iterations = 1
      extrapoln_order = 0
c     start of loop                                                       1b
1     if ( n_points + n_evalns .lt. max_evalns ) then
c
c       move the values in STEP_SIZE & APPROXIM down by one to make
c       room for the new values
c
          if ( extrapoln_order .lt. MAX_EXTRAP_ORDER )
     1        extrapoln_order = n_iterations
          do 2 i = extrapoln_order, 1, -1                                 2b
              step_size(i+1) = step_size(i)
              approxim(i+1)  = approxim(i)
2             continue                                                    2e
c
c       Calculate new STEP_SIZE and APPROXIMation coordinate value
c
          step_size(1) = step_size(2) / extrap_coeff
          n_evalns = n_evalns + integ_base(
     1        integrand, lower, upper, n_points, approxim(1) )
c
c       Extrapolate APPROXIMation to zero STEP_SIZE
c
          integ_gen_rhomb = pol_inter( step_size, approxim, 1,
     1        extrapoln_order + 1, extrapoln_order,
     2        BISECT_CLOSEST, 0.0, extrap_error )
          if ( abs( extrap_error ) .lt.
     1         precision * abs( integ_gen_rhomb ) )
     2        return
          n_iterations = n_iterations + 1
          goto 1
          end if
c         end of loop                                                     1e
      return
      end



      REAL FUNCTION INTEG_GEN_SIMPS( INTEG_BASE, STEP_RATIO, INTEGRAND )
c
c General Simpson's rule integration.
c Integrates the function INTEGRAND between the integration limits LOWER
c and UPPER to a specified accuracy using the base integration routine
c INTEG_BASE -- which may be an open, semi-open, closed, transformed or
c improper integration formula .
c
c Arguments
c  integer integ_base # base integration routine
c                     # INTEG_BAS_TRAPZ, or INTEG_BAS_MIDPT
c          step_ratio # step size reduction factor from one iteration to
c                     # the next -- depends on the basic integration routine,
c                     # 2 for INTEG_BAS_TRAPZ, 3 for INTEG_BAS_MIDPT
c  real    integrand  # INPUT  # function (of one variable) defining integrand
c
c local variables
c  integer
c   n_points          # number of internal evaluation points
c  real
c   previous          # previous estimate of integral
c   trap_n            # result of previous call to basic integration routine
c   trap_2n           # result of current call to basic integration routine
c
c global variables
c  integer max_evalns, n_evalns
c  real    precision
c
      include 'stdio.def'
c
      integer integ_base, step_ratio, n_points
      real integrand, extrap_coeff, previous, trap_n, trap_2n
c
      include 'integ.cmn'
      external integ_base, integrand
c
      extrap_coeff = step_ratio **2 - 1
c
      n_points = 0
      n_evalns = integ_base( integrand, lower, upper, n_points, trap_2n)
      integ_gen_simps = trap_2n
c     start of loop                                                       1b
1     if ( n_points + n_evalns .lt. max_evalns ) then
          previous = integ_gen_simps
          trap_n = trap_2n
          n_evalns = n_evalns +
     1        integ_base( integrand, lower, upper, n_points, trap_2n )
          integ_gen_simps =  trap_2n + ( trap_2n - trap_n ) /
     1                                 extrap_coeff
          if ( abs( integ_gen_simps - previous ) .lt. precision *
     1            max( abs( integ_gen_simps ), abs( previous ) ) )
     2        return
          goto 1
          end if
c         end of loop                                                     1e
      return
      end



      REAL FUNCTION INTEG_GEN_TRAPZ( INTEG_BASE, INTEGRAND )
c
c General trapezoidal rule integration.
c Integrates the function INTEGRAND between the integration limits LOWER
c and UPPER to a specified accuracy using the base integration routine
c INTEG_BASE -- which may be an open, semi-open, closed, transformed or
c improper integration formula .
c
c Arguments
c  integer integ_base # base integration routine
c                     # INTEG_BAS_TRAPZ, or INTEG_BAS_MIDPT
c  real    integrand  # INPUT  # function (of one variable) defining integrand
c
c local variables
c  integer
c   n_points          # number of internal evaluation points
c  real
c   previous          # previous estimate of integral
c
c global variables
c  integer max_evalns, n_evalns, n_points,
c  real    precision
c
      include 'stdio.def'
c
      integer integ_base, n_points
      real integrand, previous
c
      include 'integ.cmn'
      external integ_base, integrand
c
      n_points = 0
      n_evalns = integ_base( integrand, lower, upper, n_points,
     1                       integ_gen_trapz )
c     start of loop
1     if ( n_points + n_evalns .lt. max_evalns ) then                     3b
          previous = integ_gen_trapz
          n_evalns = n_evalns +
     1        integ_base( integrand, lower, upper, n_points,
     2                    integ_gen_trapz )
          if ( abs( integ_gen_trapz - previous ) .lt. precision *
     1            max( abs( integ_gen_trapz ), abs( previous ) ) )
     2        return
          goto 1
          end if                                                          3e
      return
      end



      INTEGER FUNCTION INTEG_BAS_TRAPZ( INTEGRAND, LOWER, UPPER,
     1                                  N_POINTS, INTEGRAL )
c
c Trapezoidal Rule.
c Base routine for evaluating the integral of the integrand function INTEGRAND
c between the integration limits LOWER and UPPER using N_POINTS function
c evaluations at points internal to the integration interval [LOWER,UPPER] .
c This routine is intended for calling from INTEG_GEN_TRAPZ which calculates
c the integral to a specified accuracy, and cannot be called directly by a
c user-written routine . INTEG_GEN_TRAPZ calls this routine repeatedly to
c achieve the required accuracy. The first call is with N_POINTS=0, and
c INTEGRAND is evaluated only at the endpoints LOWER and UPPER . This routine
c changes N_POINTS to 1, then 2, then 4 etc which is the number of evaluations
c required for the next call . Excepting the first call, N_POINTS must not be
c changed between calls. The estimate for integral from the previous call to
c this routine INTEGRAL is modified and returned . The returned result of
c this routine is the number of INTEGRAND function evaluations performed .
c
c Arguments
c  real    integrand,   # INPUT  # function (of one variable) defining integrand
c          lower,       # INPUT  # lower limit of integration
c          upper        # INPUT  # upper limit of integration
c  integer n_points     # IN/OUT # number of internal evaluations
c  real    integral     # IN/OUT # previous and returned integral
c
c local variables
c  real
c   spacing   # space between evaluation points
c   sum       #
c   x         # abscissa - where integrand is evaluated
c
      include 'stdio.def'
c
      integer n_points
      real integrand, lower, upper, integral, spacing, sum, x
      external integrand
c
      if ( n_points .le. 0 ) then                                         1b
          integral = 0.5 * ( upper - lower ) *
     1             ( integrand( lower ) + integrand( upper ) )
          n_points = 1
          integ_bas_trapz = 2
      else                                                                1m
          spacing = ( upper - lower ) / n_points
          sum = 0.0
          do 1 x = lower + 0.5 * spacing, upper, spacing                  2b
              sum = sum + integrand( x )
1             continue                                                    2e
          integral = 0.5 * ( integral +
     1                       ( upper - lower ) * sum / n_points )
          integ_bas_trapz = n_points
          n_points = n_points * 2
          end if                                                          1e
      return
      end



      INTEGER FUNCTION INTEG_BAS_MIDPT( INTEGRAND, LOWER, UPPER,
     1                                  N_POINTS, INTEGRAL )
c
c Midpoint Rule.
c Base routine for evaluating the integral of the integrand function INTEGRAND
c between the integration limits LOWER and UPPER using N_POINTS function
c evaluations at points internal to the integration interval [LOWER,UPPER] .
c This routine is intended for calling from INTEG_GEN_TRAPZ which calculates
c the integral to a specified accuracy, and cannot be called directly by a
c user-written routine . INTEG_GEN_TRAPZ calls this routine repeatedly to
c achieve the required accuracy. The first call is with N_POINTS=0, and
c INTEGRAND is evaluated only at the endpoints LOWER and UPPER . This routine
c changes N_POINTS to 1, then 2, then 4 etc which is the number of evaluations
c required for the next call . Excepting the first call, N_POINTS must not be
c changed between calls. The estimate for integral from the previous call to
c this routine INTEGRAL is modified and returned . The returned result of
c this routine is the number of INTEGRAND function evaluations performed .
c
c Arguments
c  real    integrand,   # INPUT  # function (of one variable) defining integrand
c          lower,       # INPUT  # lower limit of integration
c          upper        # INPUT  # upper limit of integration
c  integer n_points     # IN/OUT # number of internal evaluations
c  real    integral     # IN/OUT # previous and returned integral
c
c local variables
c  real
c   doub_space    # twice the space between evaluation points
c   n_loops       # number of loops performed = number of evaluations / 2
c   sum           #
c   x             # abscissa - where integrand is evaluated
c
c
      include 'stdio.def'
c
      integer n_points
      real    integrand, lower, upper, integral,
     1        doub_space, n_loops, sum, x
      external integrand
c
      if ( n_points .le. 0 ) then                                         1b
          integral = ( upper - lower ) *
     1               integrand( 0.5 * ( lower + upper ) )
          n_points = 2
          integ_bas_midpt = 1
      else                                                                1m
          n_loops = n_points / 2
          doub_space = 2.0 / 3.0 * ( upper - lower ) / n_loops
          sum = 0.0
          do 1 x = lower + 0.25 * doub_space, upper, 1.5 * doub_space     2b
              sum = sum + integrand( x ) + integrand( x + doub_space )
1             continue                                                    2e
          integral = ( integral +
     1                 ( upper - lower ) * sum / n_loops ) / 3.0
          integ_bas_midpt = n_points
          n_points = n_points * 3
          end if                                                          1e
      return
      end



      BLOCK DATA INTEG_BLOCK
c
      include 'stdio.def'
      include 'integ.def'
c
      include 'integ.cmn'
c
      data max_evalns,   method,        n_evalns   /
     1     1000,         INTEG_SIMPSON, 0          /
      data lower, upper, precision,     tolerance  /
     1     0.0,   1.0,   INTEG_TINY,    INTEG_TINY /
      end







      REAL FUNCTION POL_INTER( X_DATA, Y_DATA, INCREM, N_POINTS,
     1                          ORDER, FLAGS, X0, ERROR )
c
c For the data arrays X_DATA and Y_DATA, each of length N_POINTS, and given
c a value X0, this routine returns an interpolated value POLY_INT, and an
c error estimate ERROR . If P(x) is the polynomial of degree ORDER-1 such
c that P(X_DATA(I))=Y_DATA(I), I=1,2 .. N_POINTS, then the returned value
c POLY_INT=P(X0) .
c
c Arguments
c  real x_data        # abscissa data
c       y_data        # ordinate data
c  integer increm,    # indexing increment for the data arrays
c          n_points,  # number of data points used
c          order,     # order of the interpolating polynomial
c          flags      # specifies method for selecting interpolating data points
c  real x0            # target value
c       error         # error estimate
c
c The modulo 4 value of FLAGS can be
c  0 --- use bisecting search algorithm for finding the values in X_DATA which
c        straddle X0 and select the same number of data points on each side of
c        X0 .
c  1 --- use interval doubling hunt algorithm for finding the values in X_DATA
c        which straddle X0 and select the same number of data points on each
c        side of X0 .
c  2 --- use bisecting search algorithm for finding the values in X_DATA which
c        straddle X0 and select the data points with abscissae closest to X0 .
c  3 --- use interval doubling hunt algorithm for finding the values in X_DATA
c        which straddle X0 and select the data points with abscissae closest
c        to X0
c
c This function uses the Neville algorithm . The algorithm uses a recursion
c relation to calculate successively smaller error terms in a tableau .
c The code for this function was adapted from
c Press W.H., Flannery B.P., Teukolsky S.A. \& Vetterling W.T. (1986) :
c   Numerical Recipes (The Art of Scientific Computing),
c   {\em Cambridge University Press, New York.}
c The algorithm is detailed in
c Stoer J. \& Bulirsch R. (1980) :
c   Introduction to numerical analysis,
c   {\em Springer-Verlag, New York}, Chapter 2.1
c
c Constants
c    MAX_ORDER1 = 20  # maximum order of interpolating polynomial + 1
c    ABORT_LABEL=' POL_INTER : ABORT : '
c Variables
c  integer
c    actual_order     # actual (clipped) order of interpolating polynomial + 1
c    column           # column number for the calculation tableau
c    data_step        # distance between ends of current tableau column
c    data_src         # data array index to the start of the tableau column
c    dest, src        # source and destination array indices
c    err_index        # index for forward and backward error arrays
c    mod_flag         # absolute modulo 4 value of flags
c    n_closest        # index of value in X_DATA closest to X0
c    n_last           # index of last useable value in X_DATA
c    n_start          # index of 1st value in X_DATA used for interpolation
c    JBSRCH           # function : bisecting search
c    JCLOSEST         # function : find the closest items in data array
c    JDHUNT           # function : interval doubling hunt search
c  real
c    back_err(MAX_ORDER1)    # successive backward errors
c    forw_err(MAX_ORDER1)    # successive forward errors
c    back_numer       # numerator for calculation of backward error
c    forw_numer       # numerator for calculation of forward error
c    denom            # denominator for calculation of errors
c    err_diff         # difference between previous forward & backward errors
c
c
      include 'stdio.def'
      integer MAX_ORDER1
      character ABORT_LABEL*21
      parameter( MAX_ORDER1=20, ABORT_LABEL=' POL_INTER : ABORT : ' )
c
      integer actual_order, column, data_step, data_src, dest, src,
     1        err_index, flags, increm, mod_flag, n_closest, n_last,
     2        n_points, n_start, order, JBSRCH, JDHUNT, JCLOSEST
      real back_err(MAX_ORDER1), forw_err(MAX_ORDER1),
     1     x_data(*), y_data(*), x0, error,
     2     back_numer, forw_numer, err_diff, denom
c
c           check arguments
c
      pol_inter = 0.0
      if ( n_points .le. 0 ) then
          write( STD_ERR, 1 ) ABORT_LABEL, n_points
1         format( a, 'Illegal N_POINTS ', i12, ' <= 0' )
          return
      else if ( order .ge. MAX_ORDER1 .and.
     1          n_points .gt. MAX_ORDER1 ) then
          write( STD_ERR, 2 ) ABORT_LABEL, order, MAX_ORDER1
2         format( a, 'ORDER ', i12, ' > ', i12 )
          return
      else if ( order .lt. 0 ) then
          write( STD_ERR, 3 ) ABORT_LABEL, order, MAX_ORDER1
3         format( a, 'ORDER ', i12, ' < 0' )
          return
      else if ( increm .le. 0 ) then
          write( STD_ERR, 4 ) ABORT_LABEL, increm
4         format( a, 'Illegal INCREM ', i12,' <= 0' )
          return
          end if
      n_last = 1 + ( n_points - 1 ) * increm
      actual_order = min( n_points, order + 1 )
c
c           find the index N_CLOSEST of the closest table entry
c
      mod_flag = mod( abs( flags ), 4 )
      if ( mod( mod_flag, 2 ) .eq. 0 ) then
          n_closest = JBSRCH( x_data, increm, n_points, x0 )
      else
          n_closest = JDHUNT( x_data, increm, n_points, -999, x0 )
          end if
c
c           find the index N_START of the first data item to be used
c
      if ( mod_flag .lt. 2 ) then
          n_start = n_closest - ( actual_order - 1 ) / 2 * increm
          n_start = min( max( n_start, 1 ),
     1                   n_last - ( actual_order - 1 ) * increm )
      else
          n_start = JCLOSEST( x_data, increm, n_points, n_closest,
     1                        actual_order, x0 )
          end if
c
c           initialize the forward and backward errors
c
      src = n_start
      do 5 dest = 1, actual_order
          forw_err(dest) = y_data( src )
          back_err(dest) = y_data( src )
          src = src + increm
5         continue
c
c         initial approximation to the interpolated value
c
      pol_inter = y_data(n_closest)
      error = pol_inter
      n_closest = ( n_closest - n_start ) / increm
c
c         for each column of the tableau, loop over the current FORW_ERRs
c         and BACK_ERR's, and update them
c
      data_step = increm
      do 8 column = 1, actual_order - 1
          data_src = n_start
          do 7 err_index = 1, actual_order - column
              forw_numer = x_data( data_src ) - x0
              back_numer = x_data( data_src + data_step ) - x0
              err_diff = forw_err( err_index + 1 ) - back_err(err_index)
              denom = x_data( data_src ) - x_data( data_src + data_step)
c
c         This error can occur only if two inputs X_DATA are ( to within
c         roundoff error) the same .
c
              if ( denom .eq. 0.0 ) then
                  write( STD_ERR, 6 ) ABORT_LABEL,
     1                x_data( data_src ), data_src
6                 format( a, 'Identical abscissae ', g14.7, ' at ',
     1                    i7, ' and ', i7 / ' Divide by zero .' )
                  return
                  end if
c
c         Updata the forward and backward errors here
c
              back_err( err_index ) = back_numer * err_diff / denom
              forw_err( err_index ) = forw_numer * err_diff / denom
              data_src = data_src + increm
7             continue
c
c After each column in the tableau is completed, we decide which correction,
c FORW_ERR or BACK_ERR, we want to add to our accumulating interpolated value .
c I.e. which path to take for the tableau -- forking up or down . We do this in
c such a way to take the most "straight line" route through the tableau to its
c apex, updating N_CLOSEST accordingly to keep track of where we are . This
c route keeps the partial approximations centred (insofar as possible) on the
c target X0 . The last FORW_ERR or BACK_ERR added is thus the error indication .
c
          if ( 2 * n_closest .lt. actual_order - column ) then
              error = forw_err( n_closest + 1 )
          else
              error = back_err( n_closest )
              n_closest = n_closest - 1
              end if
          pol_inter = pol_inter + error
          data_step = data_step + increm
8         continue
      return
      end



      INTEGER FUNCTION JCLOSEST( X_DATA, INCREM, N_POINTS, START,
     1                           N_VALUES, X0 )
c          CLOSEST values
c
c Find the N_VALUES closest values in the array X_DATA to the value X0, given
c the index START for which X_DATA(START) <= X0 < X_DATA(START+INCREM) .
c If N_VALUES > N_POINTS-1, the value of N_VALUES is reduced to N_POINTS-1 .
c The returned value of JCLOSEST is the index of of the first N_VALUES values
c in X_DATA closest to X0 .
c    N.B. X must be an array of values arranged in STRICTLY ASCENDING or
c  STRICTLY DESCENDING order .
c
c Arguments
c  real x_data        # abscissa data
c  integer increm,    # indexing increment for the data array
c          n_points,  # number of data points available
c          start,     # value obtained from JBSRCH or JDHUNT
c          n_values   # number of closest values to find
c  real x0            # target value
c
c Variables
c   integer
c     left_in,        # index of the first selected data value
c     right_in,       # index of the last selected data value
c     n_out,          # number of values selected
c     n_last          # index of last available data value
c   logical
c     within_left,    # have not reached the beginning of the data array
c     within_right    # have not reached the end the data array
c
      include 'stdio.def'
      real x_data(*), x0
      integer increm, n_points, start, n_values
c
      integer left_in, right_in, n_out, n_last
      logical within_left, within_right
c
c           initialize, ensure a valid starting point
c
      if ( n_points .le. 0 .or. increm .le. 0 .or. n_values .le. 0 )
     1     return
      n_last = ( n_points - 1 ) * increm + 1
      left_in = max( 1, min( n_last,
     1         ( ( start - 1 ) /  increm ) * increm + 1 ) )
      right_in = left_in + increm
      jclosest = right_in
      n_out = 1
c
c           if have reached both the beginning and end of the array, or the
c           required number of values has been examined, terminate the search
c
c     start of loop
1     within_left = left_in .ge. 1
      within_right = right_in .le. n_last
      if ( n_out .le. n_values .and. ( within_left .or. within_right ) )
     1        then
c
c           if the right hand value is not inside data array, move to the left
c           if the left hand value is not inside data array, move to the right
c
          if ( .not. within_right ) then
              jclosest = left_in
              left_in = left_in - increm
          else if ( .not. within_left ) then
              right_in = right_in + increm
c
c           it is now OK to look at the left and right hand values ( without
c           exceeding array bounds ) include whichever one is closest to X0 .
c
          else if ( abs( x_data(left_in) - x0 ) .gt.
     1              abs( x_data(right_in) - x0 )  ) then
              right_in = right_in + increm
          else
              jclosest = left_in
              left_in = left_in - increm
              end if
          n_out = n_out + 1
          goto 1
          end if
c         end of loop
c
c           Adjust the number of values if there is insufficient data
c
      n_values = n_out - 1
      return
      end



      INTEGER FUNCTION JBSRCH( X, NINCR, NPTS, X0 )
c
c       Binary SeaRCH
c  Function for locating the last element between the 1st and Nth elements
c  in the array X of N real numbers , which in the case of an ascending
c  array , is less than the value X0 . In the case of a descending array ,
c  it finds the last element which is greater than or equal to X0 . The
c  returned value of JBSRCH is the index of this element in the array X .
c    N.B. X must be an array of values arranged in STRICTLY ASCENDING or
c  STRICTLY DESCENDING order .
c
c     Arguments
c real  x(*),           # data array
c       x0              # search goal
c integer *4 nincr,     # data array indexing increment
c            npts       # number of data points used
c     Variables
c logical desc          # .true. if data array X is descending
c integer nlast         # index of last point used
c         left, right,  # left & right hand indices of the search interval
c         centre        # moving search index
c         JBSRCH        # index of last element in array less than X0
c
      include 'stdio.def'
      real x(*), x0
      logical desc
      integer nincr, npts, nlast, left, right, centre
c
      if( npts .le. 0 .or. nincr .le. 0 ) return
      nlast = 1 + ( npts - 1 ) * nincr
      desc = x(1) .gt. x(nlast)
c
c             deal with the boundary conditions
c ( the method of left=0, right=nlast+1 from Numerical Recipes does not
c   work for nincr>1 . )
c
      if ( x0 .lt. x(1) .neqv. desc ) then
          left = 1
      else if ( x0 .gt. x(nlast) .neqv. desc ) then
          left = nlast
      else if ( x0 .eq. x(1) ) then
          left = 1
      else if ( x0 .eq. x(nlast) ) then
          left = nlast - nincr
      else
c
c             bisection procedure
c ( the simple averaging of left & right does not find a valid array index
c   for centre )
c
          left = 1
          right = nlast
c         start of loop
1         if ( right - left .gt. nincr ) then
              centre = left + ( right - left ) / 2 / nincr * nincr
              if ( x0 .gt. x(centre) .neqv. desc ) then
                  left = centre
               else
                  right = centre
                  end if
              goto 1
              end if
c             end of loop
          end if
      JBSRCH = left
      return
      end


      INTEGER FUNCTION JDHUNT( X_DATA, INCREM, N_POINTS, INIT_GUESS,
     1                         X0 )
c
c       inteval Doubling HUNT
c Function for locating the last element between the 1st and Nth elements in
c the array X of N real numbers , which in the case of an ascending array, is
c less than the value X0 . In the case of a descending array, it finds the
c last element which is greater than or equal to X0 . The returned value of
c JDHUNT is the index of this element in the array X . The search begins with
c an interval-doubling hunt until the target value X0 is straddled, and then
c the search interval is reduced by bisection search . INITIAL_GUESS is an
c initial guess for the returned result supplied by the calling program . If
c INITIAL_GUESS=-999, and JDHUNT has been used before, the previous returned
c result is used as the starting value . If INITIAL_GUESS is not within the
c bounds of the data array (e.g. 0), the search is by bisection only .
c    N.B. X must be an array of values arranged in STRICTLY ASCENDING or
c  STRICTLY DESCENDING order .
c
c     Arguments
c real x_data(*)        # data array
c integer increm,       # data array indexing increment
c         n_points      # number of data points used
c         init_guess    # starting point for search
c real x0               # search target value
c
c     Variables
c logical desc          # .true. if data array X is descending
c         not_pass1     # second or later call to this function
c integer nlast         # index of last point used
c         left, right,  # left & right hand indices of the search interval
c         centre        # moving search index
c         saved_result  # result of the previous call
c         step          # hunt interval size
c
      include 'stdio.def'
      real x_data(*), x0
      logical desc, not_pass1
      integer increm, n_points, init_guess,
     1        nlast, left, right, centre, saved_result, step
c
      save not_pass1, saved_result
      data not_pass1 / .FALSE. /
      data saved_result / 0 /
c
      if( n_points .le. 0 .or. increm .le. 0 ) return
      nlast = 1 + ( n_points - 1 ) * increm
      desc = x_data(1) .gt. x_data(nlast)
c
c             deal with the boundary conditions
c ( the method of left=0, right=nlast+1 from Numerical Recipes does not
c   work for nincr>1 . )
c
      if ( x0 .lt. x_data(1) .neqv. desc ) then                          1b
          left = 1
      else if ( x0 .gt. x_data(nlast) .neqv. desc ) then                 1m
          left = nlast
      else if ( x0 .eq. x_data(1) ) then                                 1m
          left = 1
      else if ( x0 .eq. x_data(nlast) ) then                             1m
          left = nlast - increm
      else                                                               1m
c
c             decide if a hunt is required
c
          if ( not_pass1 .and. init_guess .eq. -999 .or.
     1            init_guess .ge. 1 .and. init_guess .le. nlast ) then   2b
              if ( init_guess .eq. -999 ) then                           3b
                  left = saved_result
              else                                                       3m
                  left = init_guess
                  end if                                                 3e
              left = max( 1, min( nlast,
     1            ( ( left - 1 ) /  increm ) * increm + 1 ) )
c
c             doubling hunt procedure
c
              step = increm
              if ( x0 .ge. x_data(left) .eqv. desc ) then                3b
c                 start loop                                             4b
1                     right = left
                      left = right - step
                      step = 2 * step
                      if ( left .lt. 1 ) then                            5b
                          left = 1
                      else if ( x0 .lt. x_data(left) .neqv. desc ) then  5m
                          goto 1
                          end if                                         5e
c                     end loop                                           4e
              else                                                       3m
                  right = left
c                 start of loop                                          4b
2                     left = right
                      right = left + step
                      step = 2 * step
                      if ( right .gt. nlast ) then                       5b
                          right = nlast
                      else if ( x0 .ge. x_data(right) .neqv. desc ) then 5m
                          goto 2
                          end if                                         5e
c                     end of loop                                        4e
                  end if                                                 3e
          else                                                           2m
              left = 1
              right = nlast
              end if                                                     2e
c
c             bisection procedure
c ( the simple averaging of left & right does not find a valid array index
c   for centre )
c
3         if ( right - left .gt. increm ) then                           2b
              centre = left + ( right - left ) / 2 / increm * increm
              if ( x0 .gt. x_data(centre) .neqv. desc ) then             3b
                  left = centre
              else                                                       3m
                  right = centre
                  end if                                                 3e
              goto 3
              end if                                                     2e
          end if                                                         1e
      not_pass1 = .TRUE.
      saved_result = left
      JDHUNT = left
      return
      end


